/******************************************************************************
*  Copyright (c) 2002 Linux UPnP Internet Gateway Device Project              *    
*  All rights reserved.                                                       *
*                                                                             *   
*  This file is part of The Linux UPnP Internet Gateway Device (IGD).         *
*                                                                             *
*  The Linux UPnP IGD is free software; you can redistribute it and/or modify *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *    
*  The Linux UPnP IGD is distributed in the hope that it will be useful,      *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *   
*  You should have received a copy of the GNU General Public License          * 
*  along with Foobar; if not, write to the Free Software                      *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
*                                                                             *  
*                                                                             *  
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <sys/sysinfo.h>
#include <string.h>
#include <time.h>
#include "tool.h"
#include "upnpd.h"

extern struct GateEnv gate;

int get_sockfd()
{

	 int sockfd = -1;
	
		//if ((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) == -1)
		if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
		{
			perror("user: socket creating failed");
			return (-1);
		}
	return sockfd;
}

char *getIPAddress(char *ifname)
{
	struct ifreq ifr;
	struct sockaddr_in *saddr;
	int fd;
	
	fd = get_sockfd();
	if (fd >= 0 )
	{
		memset(&ifr,0,sizeof(ifr));
		strcpy(ifr.ifr_name, ifname);
		ifr.ifr_addr.sa_family = AF_INET;
		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0)
		{
			saddr = (struct sockaddr_in *)&ifr.ifr_addr;
			close(fd);
			return inet_ntoa(saddr->sin_addr);
		}
		else
		{
			perror("get IP address");
			close(fd);
			return NULL;
		}
	}
	return NULL;
}


/* Check the interface status to see if the interface is up or down */
int getConnectionStatus(void)
{

	FILE *fp;
	char tmp[128];

	if((fp=fopen("/proc/avalanche/avsar_modem_training","r"))==NULL)
		return -1;
	fgets(tmp,128,fp);
	fclose(fp);

	if(strcmp(tmp,"SHOWTIME")==0)
		strcpy(gate.ConnectionStatus,"Connected");
	else
		strcpy(gate.ConnectionStatus,"Disconnected");
	
	return 1;
}

int getUptime(void)
{
	struct sysinfo info;
	sysinfo(&info);
	return (info.uptime - gate.startup_time);
}
#if 0
char * getDefaultConnectionService(void)
{
   char *t = malloc(strlen("NULL")+1);
   strcpy(t,"NULL");
   return t;
}

char * getConnectionType(void)
{
   char *t = malloc(strlen("IP_Routed")+1);
   strcpy(t,"IP_Routed");
   return t;
}
#endif
char * getPossibleConnectionTypes(void)
{
   return NULL;
}

char * getWANAccessType(void)
{
   return NULL;
}

int getLayer1MaxBitRate(void)
{
	FILE *fp;
	char tmp[1024];
	char *pt;
	if((fp=fopen("/proc/avalanche/avsar_modem_stats","r"))==NULL)
		return -1;
	fread(tmp,1024,1,fp);
	if((pt=strstr(tmp,"US Connection Rate:"))==NULL)
		return -1;
	sscanf(pt,"%*[^:]:%d%*[^:]:%d",&gate.UpstreamSpeed,&gate.DownstreamSpeed);
	fclose(fp);
   return 1;
}

#if 0
char * getPhysicalLinkStatus(void)
{
   char *t;

   t = malloc(strlen("Up")+1);
   strcpy(t,"Up");

   return t;
}
char * getWANAccessProvider(void)
{
   return NULL;
}

char * getMaximumActiveConnections(void)
{
   char *t = malloc(2);
   t[0] = '0';
   return t;
}

char * getNumberOfActiveConnections(void)
{
   // don't know how to get this
   char *t = malloc(2);
   t[0] = '0';
   return t;
}

#endif
char * getActiveConnectionDeviceContainer(void)
{
   return NULL;
}

char * getActiveConnectionServiceID(void)
{
   return NULL;
}

/* it seems that getTotalBytesSent(void) & getTotalBytesReceived(void) 
   are not being used */
   
unsigned long getTotalBytesSent(void)
{
	char dev[15];
	FILE *stream;
        unsigned long bytes=0, total=0;
        
	/* Read sent from /proc */
	stream = fopen ( "/proc/net/dev", "r" );
	if ( stream != NULL )
	{
		while ( getc ( stream ) != '\n' );
		while ( getc ( stream ) != '\n' );

		while ( !feof( stream ) )
		{
			fscanf ( stream, "%[^:]:%*u %*u %*u %*u %*u %*u %*u %*u %lu %*u %*u %*u %*u %*u %*u %*u\n", dev, &bytes ); 
                        if ( strcmp ( dev, gate.ext_ifname)==0 )
                                total += bytes;	
		}
		fclose ( stream );
	}
   return total;   


}

unsigned long getTotalBytesReceived(void)
{
	char dev[15];
	FILE *stream;
        unsigned long bytes=0, total=0;
        
	/* Read sent from /proc */
	stream = fopen ( "/proc/net/dev", "r" );
	if ( stream != NULL )
	{
		while ( getc ( stream ) != '\n' );
		while ( getc ( stream ) != '\n' );

		while ( !feof( stream ) )
		{
                        fscanf ( stream, "%[^:]:%lu %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u\n", dev, &bytes );
                        if ( strcmp ( dev, gate.ext_ifname)==0 )
                                total += bytes;	
		}
		fclose ( stream );
	}
   return total;   

}

int getNATEnabled(void)
{
#ifdef NVRAM
	char *fw_nat;

	fw_nat=nvram_safe_get("fw_nat");
	if(*fw_nat=='0')
		return 0;
	if(*fw_nat=='1')
		return 1;
	else 
		return -1;
#else

	return 1;
#endif

}



unsigned long getTotalPacketsSent(void)
{

	char dev[15];
	FILE *stream;
        unsigned long pkt=0, total=0;
        
	/* Read sent from /proc */
	stream = fopen ( "/proc/net/dev", "r" );
	if ( stream != NULL )
	{
		while ( getc ( stream ) != '\n' );
		while ( getc ( stream ) != '\n' );

		while ( !feof( stream ) )
		{
                        fscanf ( stream, "%[^:]:%*u %*u %*u %*u %*u %*u %*u %*u %*u %lu %*u %*u %*u %*u %*u %*u\n", dev, &pkt );
                        if ( strcmp ( dev, gate.ext_ifname)==0 )
                                total += pkt;	
		}
		fclose ( stream );
	}
   return total;   
}

unsigned long getTotalPacketsReceived(void)
{

	char dev[15];
        FILE *stream;
        unsigned long pkt=0, total=0;

        /* Read sent from /proc */
        stream = fopen ( "/proc/net/dev", "r" );
        if ( stream != NULL )
        {
                while ( getc ( stream ) != '\n' );
                while ( getc ( stream ) != '\n' );

                while ( !feof( stream ) )
                {
                        fscanf ( stream, "%[^:]:%*u %lu %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u\n", dev, &pkt );
                        if ( strcmp ( dev, gate.ext_ifname )==0 )
                                total += pkt;
                }
                fclose ( stream );
        }

	//printf(" received  %lu\n",total);
	return total;

}

int getEnabledForInternet(void)
{
      return 1;
}

int getDHCPServerConfigurable(void)
{
   return 1; // yes
}

int getDHCPRelay(void)
{
   return 1; // yes
}

char * getSubnetMask(void)
{
    char *netmask="255.255.255.0";
    return netmask;
}

char * getIPRouters(void)
{
   return NULL;   
}

char * getDNSServers(void)
{
      return NULL;
}

char * getDomainName(void)
{
   return NULL;
}

char * getMinAddress(void)
{
   return NULL;
}

char * getMaxAddress(void)
{
   return NULL;
}

char * getReservedAddress(void)
{
   return NULL; 
}

char * getHostName(void)
{
	char *host="UPNP_HOST";
	return  host;
}

int substr(char *infile, char *outfile, char *str_from, char *str_to)
{
        FILE *fpi, *fpo;
        char pathi[256], patho[256];
        char buffi[4096], buffo[4096];
        int len_buff, len_from, len_to;
        int i, j;

        sprintf(pathi, "%s", infile);
        if ((fpi = fopen(pathi,"r")) == NULL)
        {
//                printf("input file can not open\n");
                return (-1);
        }

        sprintf(patho, "%s",outfile);
        if ((fpo = fopen(patho,"w")) == NULL) {
//                printf("output file can not open\n");
                fclose(fpi);
                return (-1);
        }

        len_from = strlen(str_from);
        len_to   = strlen(str_to);

        while (fgets(buffi, 4096, fpi) != NULL)
        {
                len_buff = strlen(buffi);
                for (i=0, j=0; i <= len_buff-len_from; i++, j++)
                {
                        if (strncmp(buffi+i, str_from, len_from)==0)
                        {
                                strcpy (buffo+j, str_to);
                                i += len_from - 1;
                                j += len_to - 1;
                        } else
                        *(buffo + j) = *(buffi + i);
                }
                strcpy(buffo + j, buffi + i);
                fputs(buffo, fpo);
        }

        fclose(fpo);
        fclose(fpi);
        return (0);
}
