#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <sys/sysinfo.h>
#include "upnpd.h"
#include "tool.h"
/* Ron */
#include "dll.h"
extern int IGDDeviceStateTableInit(char*,char*);
extern int IGDdelAllPortForwarding(void);
extern int IGDconfigure(void);
extern int substr(char *infile, char *outfile, char *str_from, char *str_to);

struct GateEnv gate;

int custom_ttl;

extern int IGDDeviceCallbackEventHandler(Upnp_EventType, void*, void*);

int main(int argc, char** argv)
{

    int ret=1;
    int port=0;
    char *desc_doc_name=NULL, *web_dir_path=NULL;
    int sig;
    char fullurl[200];
    char desc_doc_url[200];
    struct sysinfo info;
    sigset_t sigs_to_catch;
    
    bzero(&gate,sizeof(struct GateEnv));
    

	/* Variable initialize */
    IGDconfigure();

    getConnectionStatus();
    getLayer1MaxBitRate();    
  
    sysinfo(&info);
    gate.startup_time = info.uptime;
    custom_ttl=gate.advr_ttl;

    
    desc_doc_name=DEF_DOCUMENT;  
    web_dir_path=DEF_DIR_PATH;  

    substr("/usr/upnp/gateway.mod", "/etc/gateway.xml", "IPADDR",gate.InternalIPAddress);

    if ((ret = UpnpInit(gate.InternalIPAddress, port)) != UPNP_E_SUCCESS) {
	printf("Error with UpnpInit -- %d\n", ret);
	UpnpFinish();
	exit(1);
    }
    printf("UPnP Initialized\n");
   
    if (port==0)
        port = UpnpGetServerPort();

     sprintf(fullurl, "http://%s:%d/%s", gate.InternalIPAddress , port, desc_doc_name);
     sprintf(desc_doc_url, "http://%s:%d", gate.InternalIPAddress , port);
    
    printf("Intialized UPnP \n\twith fullurl=%s\n\t", fullurl);
    printf("\t     ipaddress=%s port=%d\n", gate.InternalIPAddress, port);
    printf("\t     web_dir_path=%s\n", web_dir_path);
    printf("\t     desc_doc_url=%s\n", desc_doc_url);

	printf("Specifying the webserver root directory -- %s\n", web_dir_path);
    if ((ret = UpnpSetWebServerRootDir(web_dir_path)) != UPNP_E_SUCCESS) {
	printf("Error specifying webserver root directory -- %s: %d\n", web_dir_path, ret);
	UpnpFinish();
	exit(1);
    }

    printf("Registering the RootDevice\n");
    if ((ret = UpnpRegisterRootDevice(fullurl, IGDDeviceCallbackEventHandler, &gate.device_handle, &gate.device_handle)) != UPNP_E_SUCCESS) {
	printf("Error registering the rootdevice : %d\n", ret);
	UpnpFinish();
	exit(1);
    } else {
	printf("RootDevice Registered\n");
     
	printf("Initializing State Table\n");
	IGDDeviceStateTableInit(desc_doc_url,desc_doc_name);
	if ((ret = UpnpSendAdvertisement(gate.device_handle, gate.advr_expire)) != UPNP_E_SUCCESS) {
	    printf("Error sending advertisements : %d\n", ret);
	    UpnpFinish();
	    exit(1);
	}
	printf("Advertisements Sent\n");
    }

    /* Catch Ctrl-C and properly shutdown */
    sigemptyset(&sigs_to_catch);
    sigaddset(&sigs_to_catch, SIGINT);
    sigaddset(&sigs_to_catch, SIGTERM);
    sigwait(&sigs_to_catch, &sig);
    printf("Shutting down on signal %d...\n", sig);
    UpnpUnRegisterRootDevice(gate.device_handle);
    UpnpFinish();

    IGDdelAllPortForwarding();
    exit(0);
}


