/*****************************************************************************/

/*
 *	cpu.c -- simple CPU usage reporting tool.
 *
 *	(C) Copyright 2000, Greg Ungerer (gerg@lineo.com)
 */

/*****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/termios.h>
#include <sys/time.h>
#include <string.h>

/*****************************************************************************/

char *version = "1.0.0";

/*****************************************************************************/

struct stats {
	unsigned int	user;
	unsigned int	nice;
	unsigned int	system;
	unsigned int	idle;
	unsigned int	total;
};

/*****************************************************************************/

int getdata(struct stats *st)
{
	unsigned char	buf[80];
	FILE *fp;

	if ((fp = fopen("/proc/stat", "r")) == NULL) {
		exit(-1);
	}
	if (fseek(fp, 0, SEEK_SET) < 0)
		return(-1);
	fscanf(fp, "%s %d %d %d %d", &buf[0], &st->user, &st->nice,
		&st->system, &st->idle);

	st->total = st->user + st->nice + st->system + st->idle;
	//printf("%d %d %d %d\n", st->user, st->nice, st->system, st->idle);
	fclose(fp);
	return(0);
}


int main(int argc, char *argv[])
{
	struct stats	st, stold;
	unsigned int	curtotal;
	int		c, cnt, repeat, delay, average;
	char		*procdevice;
	FILE		*fp;
	char 		*filename=NULL;
	repeat = 0;
	delay = 1;
	procdevice = "/proc/stat";
	cnt = 1;
	average = 0;

	while ((c = getopt(argc, argv, "ravh?s:d:c:f:")) > 0) {
		switch (c) {
		case 'v':
			printf("%s: version %s\n", argv[0], version);
			exit(0);
		case 'a':
			average++;
			break;
		case 'r':
			repeat++;
			break;
		case 's':
			delay = atoi(optarg);
			break;
		case 'd':
			procdevice = optarg;
			break;
		case 'c':
			cnt = atoi(optarg);
			break;
		case 'f':
			filename = optarg;
			break;
		default:
			fprintf(stderr, "ERROR: unkown option '%c'\n", c);
			break;
		}
	}

	/*
	 *	Check device is real, and open it.
	 */
	getdata(&st);
	
	/* default value */
	if(filename==NULL){
		filename=strdup("/tmp/cpu_avg");
	}

	for (c = 0; ((c < cnt) || repeat); c++) {
		sleep(delay);
		stold = st;
		getdata(&st);

		curtotal = st.total - stold.total;
		fp=fopen(filename,"w");
		if(fp){
			fprintf(fp,"%d\n",((st.system + st.user + st.nice) -
			 (stold.system + stold.user + stold.nice)) *
			 100 / curtotal);
			fclose(fp);
		}
	}

	exit(0);
}

/*****************************************************************************/
