#ifndef __SNMP_ASN1_H__
#define __SNMP_ASN1_H__

#include "snmp_types.h"

#define OID_LENGTH(x)  (sizeof(x)/sizeof(oid))


#define ASN_BOOLEAN	    ((u_char)0x01)
#define ASN_INTEGER	    ((u_char)0x02)
#define ASN_BIT_STR	    ((u_char)0x03)
#define ASN_OCTET_STR	    ((u_char)0x04)
#define ASN_NULL	    ((u_char)0x05)
#define ASN_OBJECT_ID	    ((u_char)0x06)
#define ASN_SEQUENCE	    ((u_char)0x10)
#define ASN_SET		    ((u_char)0x11)

#define ASN_UNIVERSAL	    ((u_char)0x00)
#define ASN_APPLICATION     ((u_char)0x40)
#define ASN_CONTEXT	    ((u_char)0x80)
#define ASN_PRIVATE	    ((u_char)0xC0)

#define ASN_PRIMITIVE	    ((u_char)0x00)
#define ASN_CONSTRUCTOR	    ((u_char)0x20)

#define ASN_LONG_LEN	    (0x80)
#define ASN_EXTENSION_ID    (0x1F)
#define ASN_BIT8	    (0x80)

#define IS_EXTENSION_ID(byte)	(((byte) & ASN_EXTENSION_ID) == ASN_EXTENSION_ID)

    /*
     * defined types (from the SMI, RFC 1157) 
     */
#define ASN_IPADDRESS   (ASN_APPLICATION | 0)
#define ASN_COUNTER	(ASN_APPLICATION | 1)
#define ASN_GAUGE	(ASN_APPLICATION | 2)
#define ASN_UNSIGNED    (ASN_APPLICATION | 2)   /* RFC 1902 - same as GAUGE */
#define ASN_TIMETICKS   (ASN_APPLICATION | 3)
#define ASN_OPAQUE	(ASN_APPLICATION | 4)   /* changed so no conflict with other includes */
#define ASN_PHYSADDR    (ASN_PRIVATE | ASN_OCTET_STR) 


    /*
     * defined types (from the SMI, RFC 1442) 
     */
#define ASN_NSAP	(ASN_APPLICATION | 5)   /* historic - don't use */
#define ASN_COUNTER64   (ASN_APPLICATION | 6)
#define ASN_UINTEGER    (ASN_APPLICATION | 7)   /* historic - don't use */

struct counter64 {
    u_long          high;
    u_long          low;
};

int asn_realloc(u_char ** pkt, size_t * pkt_len);
int asn_realloc_rbuild_header(u_char ** pkt, size_t * pkt_len,
                              size_t * offset, int r,
                              u_char type, size_t length);

u_char *asn_parse_length(u_char * data, u_long * length);

u_char *asn_parse_header(u_char *, size_t *, u_char *);
u_char *asn_parse_int(u_char * data, size_t * datalength,
                      u_char * type, long *intp, size_t intsize);
u_char *asn_parse_sequence(u_char * data, size_t * datalength,
                           u_char * type, u_char expected_type,
                           const char *estr);
u_char *asn_parse_bitstring(u_char * data,
                            size_t * datalength,
                            u_char * type, u_char * string,
                            size_t * strlength);

u_char *asn_parse_unsigned_int(u_char * data, size_t * datalength,
                               u_char * type, u_long * intp, size_t intsize);

u_char *asn_parse_unsigned_int64(u_char * data, size_t * datalength,
                                 u_char * type, struct counter64 * cp,
                                 size_t countersize);
u_char *asn_parse_string(u_char * data, size_t * datalength,
                         u_char * type, u_char * string, size_t * strlength);
u_char *asn_parse_objid(u_char * data, size_t * datalength,
                        u_char * type, oid * objid, size_t * objidlength);
int asn_realloc_rbuild_int(u_char ** pkt, size_t * pkt_len,
                           size_t * offset, int r,
                           u_char type, long *intp, size_t intsize);
int asn_realloc_rbuild_string(u_char ** pkt, size_t * pkt_len,
                              size_t * offset, int r,
                              u_char type,
                              const u_char * string, size_t strlength);
int asn_realloc_rbuild_unsigned_int(u_char ** pkt, size_t * pkt_len,
                                    size_t * offset, int r,
                                    u_char type, u_long * intp, size_t intsize);
int asn_realloc_rbuild_unsigned_int64(u_char ** pkt, size_t * pkt_len,
                                      size_t * offset, int r,
                                      u_char type,
                                      struct counter64 *cp, size_t countersize);
int asn_realloc_rbuild_sequence(u_char ** pkt, size_t * pkt_len,
                                size_t * offset, int r,
                                u_char type, size_t length);
int asn_realloc_rbuild_objid(u_char ** pkt, size_t * pkt_len,
                             size_t * offset, int r, u_char type,
                             const oid * objid, size_t objidlength);
int asn_realloc_rbuild_null(u_char ** pkt, size_t * pkt_len,
                            size_t * offset, int r, u_char type);
int asn_realloc_rbuild_bitstring(u_char ** pkt, size_t * pkt_len,
                                 size_t * offset, int r,
                                 u_char type,
                                 u_char * string, size_t strlength);
int _asn_realloc_build_header_check(const char *str,
                                    u_char ** pkt, size_t * pkt_len,
                                    size_t typedlen);
u_char *asn_build_header(u_char * data,
                         size_t * datalength, u_char type, size_t length);
u_char *asn_build_length(u_char * data, size_t * datalength, size_t length);


#endif
