#ifndef __SNMP_LIB_H__
#define __SNMP_LIB_H__

#include <sys/types.h>
#include "snmp_types.h"
#include "snmp_asn1.h"

#define MAX_OID_LEN 128

#define SNMP_ERR_NOERROR                (0)     /* XXX  Used only for PDUs? */
#define SNMP_ERR_TOOBIG	                (1)
#define SNMP_ERR_NOSUCHNAME             (2)
#define SNMP_ERR_BADVALUE               (3)
#define SNMP_ERR_READONLY               (4)
#define SNMP_ERR_GENERR	                (5)

#define SNMP_ERR_WRONGTYPE		(7)

#define SNMP_NOSUCHOBJECT    (ASN_CONTEXT | ASN_PRIMITIVE | 0x0)        /* 80 = 128 */
#define SNMP_NOSUCHINSTANCE  (ASN_CONTEXT | ASN_PRIMITIVE | 0x1)        /* 81 = 129 */
#define SNMP_ENDOFMIBVIEW    (ASN_CONTEXT | ASN_PRIMITIVE | 0x2)        /* 82 = 130 */

#define SNMP_MIN(a,b) ((a) > (b) ? (b) : (a))
#define SNMP_MALLOC_STRUCT(s)   (struct s *) calloc(1, sizeof(struct s))
#define SNMP_FREE(s)		if (s) { free((void *)s); s=NULL; }

struct variable_list {
	struct variable_list *next_variable;    /* NULL for last variable */
	oid            *name;   /* Object identifier of variable */
	size_t          name_length;    /* number of subid's in name */
	u_char          type;   /* ASN type of variable */
	union {                 /* value of variable */
		long           *integer;
		u_char         *string;
		oid            *objid;
		u_char         *bitstring;
		struct counter64 *counter64;
	} val;
	size_t          val_len;
	oid             name_loc[MAX_OID_LEN];  /* 90 percentile < 24. */
	u_char          buf[40];        /* 90 percentile < 40. */
	int             index;
//	struct map_entry	*entry;		/* MAP entry */
};

typedef struct variable_list  netsnmp_variable_list;

int snmp_parse_version(u_char *, size_t);
u_char *snmp_comstr_parse(u_char * data,  size_t * length,
                          u_char * psid, size_t * slen, long *version);
u_char *snmp_parse_var_op(u_char * data, oid * var_name, size_t * var_name_len,
                          u_char * var_val_type, size_t * var_val_len,
                          u_char ** var_val, size_t * listlength);
int snmp_set_var_objid(netsnmp_variable_list * vp,
                       const oid * objid, size_t name_length);
int snmp_realloc(u_char ** buf, size_t * buf_len);
int snmp_realloc_rbuild_var_op(u_char ** pkt, size_t * pkt_len,
                               size_t * offset, int allow_realloc,
                               const oid * var_name, size_t * var_name_len,
                               u_char var_val_type,
                               u_char * var_val, size_t var_val_len);
int snmp_set_var_typed_value(netsnmp_variable_list * newvar, u_char type,
                             const u_char * val_str, size_t val_len);
int snmp_set_var_value(netsnmp_variable_list * newvar,
                       const u_char * val_str, size_t val_len);
int count_varbinds(netsnmp_variable_list * var_ptr);
void snmp_free_var(netsnmp_variable_list * var);
void snmp_free_varbind(netsnmp_variable_list * var);


#endif
