#include <sys/sysinfo.h>
#include <sys/utsname.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "include/snmp_asn1.h"
#include "include/snmp.h"
#include "my_snmp_lib.h"
#include "socket_tools.h"
#include "utility.h"

#define SNMP_DEF_SYSNAME ""
#define SNMP_DEF_LOCATION	""
#define SNMP_DEF_CONTACT	""
#define SNMP_DEF_OID	 1,3,6,1,4,1,10456,4,23
#define SNMP_DEF_EMPTY ""

/* system */
inline int get_sysDescr(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{	
#if 0
	FILE *fp;
	char tmp[40]="";
	
	fp=fopen("/etc/version","r");	
	if(fp==NULL)
		return -1;
	fread(tmp,40,1,fp);
	fclose(fp);
	sprintf(vp->buf,"NETGEAR DG834 FW %s",tmp);
#endif
	strcpy(vp->buf,"ADSL Router");
	return 0;
}
inline int get_ObjectID(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	oid name[10]={SNMP_DEF_OID};
	memcpy(vp->buf,name,9*sizeof(oid));
	vp->val_len = 9*sizeof(oid);
	return 0;
}

inline int get_sysUpTime(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;	
	int i=0;
	char j[10]="";;

	fp=fopen("/proc/uptime","r");
	if(fp){
		fscanf(fp,"%[^.].%d",j,&i);
		*(long *)(vp->buf)=atoi(j)*100+i;
		fclose(fp);
	}
	
	return 0;
}

inline int get_sysContact(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("snmp_sys_contact",&pt);
	strcpy(vp->buf,pt?:SNMP_DEF_CONTACT);
	if(pt)	free(pt);
	return 0;
}

inline int get_sysName(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("snmp_sys_name",&pt);
	strcpy(vp->buf,pt?:SNMP_DEF_SYSNAME);
	if(pt)	free(pt);
	return 0;
}
inline int get_sysLocation(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("snmp_sys_location",&pt);
	strcpy(vp->buf,pt?:SNMP_DEF_LOCATION);
	if(pt)	free(pt);
	return 0;
}

inline int get_sysServices(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	*(long *)(vp->buf) = 4;
	return 0;
}
/* interfaces */
#define IF_ADSL	1
#define IF_LAN	2
#define IF_WLAN	3
#define IF_NUM  3


inline int get_ifNumber(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	*(long *)(vp->buf) = IF_NUM;
	return 0;
}

inline int get_ifIndex(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item > IF_NUM || item==0)
		return -1;
	*(long *)(vp->buf) = item;
	vp->val.integer = (long *)(vp->buf);   
        *vp->val.integer = item; 
        vp->val_len = sizeof(long);

	return 0;
}

inline int get_ifDescr(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{

	switch(item){
		case IF_LAN:
			strcpy(vp->buf,"LAN");
			break;
		case IF_WLAN:
			strcpy(vp->buf,"Wireless");
			break;
		case IF_ADSL:
			strcpy(vp->buf,"ADSL");
			break;
		default:
			return -1;
	}
	return 0;
}

inline int get_ifType(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	
	switch(item){
		case IF_LAN:
        		*(long *)(vp->buf) = 6; 
			break;
		case IF_WLAN:
		        *(long *)(vp->buf) = 1; 
			break;
		case IF_ADSL:
        		*(long *)(vp->buf)= 18; 
			break;
		default:
			return -1;
	}

	return 0;
}
inline int get_ifMtu(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{

	switch(item){
		case IF_LAN:
			getIfinfo("eth0",vp->buf,IF_MTU);
			break;
		case IF_WLAN:
			getIfinfo("wlan0",vp->buf,IF_MTU);
			break;
		case IF_ADSL:{
				char *pt=NULL;
				scfgmgr_get("wan_ifname",&pt);
				getIfinfo(pt?:"nas0",vp->buf,IF_MTU);
				if(pt) free(pt);
				break;
			     }
		default:
			return -1;
	}

	return 0;
}

inline int get_ifSpeed(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	switch(item){
		case IF_LAN:
			*(long *)(vp->buf)=100000000;
			break;
		case IF_WLAN:{
				     
				char *pt=NULL;
				scfgmgr_get("wifi_dot11_mode",&pt);
				if(!pt)
					return -1;
				/* 1 is b only */
			  	if(*pt=='1') 		
					*(long *)(vp->buf)=11000000;
				else
					*(long *)(vp->buf)=54000000;
				free(pt);		
				break;
			     }
		case IF_ADSL:{
				FILE *fp;
				char tmp[1024];
				int down_stream=0;
				char *pt;
				if((fp=fopen("/proc/avalanche/avsar_modem_stats","r"))==NULL)
					                return -1;
				fread(tmp,1024,1,fp);
			        fclose(fp);
				
				if((pt=strstr(tmp,"US Connection Rate:"))==NULL)
				                return -1;
			        sscanf(pt,"%*[^:]:%*d%*[^:]:%d",&down_stream);	
				*(long *)(vp->buf)=down_stream*1000;
				break;
			     }
		default:
			return -1;
	}

	return 0;
}

inline int get_ifPhysAddress(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char tmp[40]="";
	
	switch(item){
		case IF_LAN:
			getIfinfo("eth0",tmp,IF_MAC);
			break;
		case IF_WLAN:
			getIfinfo("wlan0",tmp,IF_MAC);
			break;
		case IF_ADSL:
			getIfinfo("nas0",tmp,IF_MAC);
			break;
		default:
			return -1;
	}
	vp->val_len=convertStr2PhysAddr(vp->buf,tmp);
	return 0;
}

inline int get_ifAdminStatus(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	switch(item){
		case IF_LAN:
				*(long *)(vp->buf) = 1;
				break;
		case IF_WLAN:
			{
				char *pt;
				scfgmgr_get("wifi_if_on",&pt);
				if(!pt)
					*(long *)(vp->buf) = 2;
				else if(*pt=='1')
					*(long *)(vp->buf) = 1;
				else
					*(long *)(vp->buf) = 2;
				free(pt);
				break;
			}
		case IF_ADSL:
				*(long *)(vp->buf) = 1;
				break;
		default:
			return -1;
	}

	return 0;
}
inline int get_ifOperStatus(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	char tmp[1024];
	
	switch(item){
		case IF_LAN:
			{
				fp=fopen("/proc/avalanche/cpmac_link","r");
				if(!fp)
					return -1;
				fread(tmp,1024,1,fp);
				fclose(fp);
				if(strstr(tmp,"UP"))
					*(long *)(vp->buf) = 1;
				else
					*(long *)(vp->buf) = 2;
				break;
			}
		case IF_WLAN:
			{
				if(access("/tmp/wlan_uptime",F_OK)==0)
					*(long *)(vp->buf) = 1;
				else
					*(long *)(vp->buf) = 2;
				break;
			}
		case IF_ADSL:
			{
				fp=fopen("/proc/avalanche/avsar_modem_training","r");
				if(!fp)
					return -1;
				fread(tmp,1024,1,fp);
				fclose(fp);
				if(strstr(tmp,"IDLE"))
					*(long *)(vp->buf) = 2;
				else if(strstr(tmp,"SHOWTIME"))
					*(long *)(vp->buf) = 1;
				else
					*(long *)(vp->buf) = 3;
				break;
			}
		default:
			return -1;
	}

	return 0;
}

inline int get_ifLastChange(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp=NULL;
	
	switch(item){
		case IF_LAN:
			fp=fopen("/tmp/lan_uptime","r");
			break;
		case IF_WLAN:
			fp=fopen("/tmp/wlan_uptime","r");
			break;
		case IF_ADSL:
			fp=fopen("/tmp/wan_uptime","r");
			break;
		default:
			return -1;
	}
	if(fp){
		fscanf(fp,"%ld %*d",(long *)(vp->buf));
		fclose(fp);
	}else
		*(long *)(vp->buf)=0;
	
	return 0;
}
inline int get_ifInOctets(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.in_bytes;
	return 0;
}
inline int get_ifInUcastPkts(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.in_packets-info.in_mcast;
	return 0;
}
inline int get_ifInNUcastPkts(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.in_mcast;
	return 0;
}
inline int get_ifInDiscards(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item > IF_NUM)
		return -1;
	*(long *)(vp->buf)=0;
	return 0;
}
inline int get_ifInErrors(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.in_err;
	return 0;
}
inline int get_ifInUnknownProtos(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item > IF_NUM)
		return -1;
	*(long *)(vp->buf)=0;
	return 0;
}

inline int get_ifOutOctets(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.out_bytes;
	return 0;
}
inline int get_ifOutUcastPkts(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.out_packets-info.out_mcast;
	return 0;
}
inline int get_ifOutNUcastPkts(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.out_mcast;
	return 0;
}
inline int get_ifOutDiscards(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item > IF_NUM)
		return -1;
	
	*(long *)(vp->buf)=0;
	return 0;
}
inline int get_ifOutErrors(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	dev_info_t info;
	memset(&info,0,sizeof(dev_info_t));	
	switch(item){
		case IF_LAN:
			get_dev_info("eth0",&info);
			break;
		case IF_WLAN:
			get_dev_info("wlan0",&info);
			break;
		case IF_ADSL:
			get_dev_info("nas0",&info);
			break;
		default:
			return -1;
	}
	*(long *)(vp->buf)=info.out_err;
	return 0;
}
inline int get_ifOutQLen(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	switch(item){
		case IF_LAN:
			getIfinfo("eth0",vp->buf,IF_QLEN);
			break;
		case IF_WLAN:
			getIfinfo("wlan0",vp->buf,IF_QLEN);
			break;
		case IF_ADSL:
			getIfinfo("nas0",vp->buf,IF_QLEN);
			break;
		default:
			return -1;
	}
	return 0;
}
inline int get_ifSpecific(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	oid name[10]={0,0};

	if(item > IF_NUM)
		return -1;

	memcpy(vp->buf,name,2*sizeof(oid));
	vp->val_len = 2*sizeof(oid);
	return 0;
}

/* ip icmp tcp udp */

#define SNMP_IP		4
#define SNMP_ICMP	5 
#define SNMP_TCP	6
#define SNMP_UDP	7

static snmp_info_t info;
inline int get_SNMP_info(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	int table=vp->name[SNMP_MIB2_TABLE];
	int group=vp->name[SNMP_MIB2_GROUP]; 
	
	memset(&info,0,sizeof(snmp_info_t));
	get_snmp_info(&info);
	switch(group){
		case SNMP_IP:
			if(table>19)
				return -1;
			*(long *)(vp->buf)=info.ip[table-1];
			break;
		case SNMP_ICMP:
			if(table>26)
				return -1;
			*(long *)(vp->buf)=info.icmp[table-1];

			break;
		case SNMP_TCP:
			if(table>14)
				return -1;
			*(long *)(vp->buf)=info.tcp[table-1];

			break;
		case SNMP_UDP:
			if(table>4)
				return -1;
			*(long *)(vp->buf)=info.udp[table-1];

			break;
	}	
	
	return 0;
}


/* ip AddrTable */
void get_IpAddress(u_char *IpAddr1,u_char *IpAddr2,u_char *If1,u_char *If2)
{
	char *pt=NULL;
	scfgmgr_get("wan_ifname",&pt);
	getIfinfo(pt?:"nas0",IpAddr1,IF_IPADDR);
	
	getIfinfo("br0",IpAddr2,IF_IPADDR);
#if 1	
	if(ntohl(*(uint32_t *)IpAddr1) > ntohl(*(uint32_t *)IpAddr2)){
		uint32_t  tmp=0;
		tmp=*(uint32_t *)IpAddr1;
		*(uint32_t *)IpAddr1=*(uint32_t *)IpAddr2;
		*(uint32_t *)IpAddr2=tmp;
		strcpy(If2,pt?:"nas0");
		strcpy(If1,"br0");
	}else{
		strcpy(If1,pt?:"nas0");
		strcpy(If2,"br0");
	}
//	printf("%s IP1==%08x %s IP2==%08x\n",If1,*(uint32_t *)IpAddr1
//			,If2,*(uint32_t *)IpAddr2);
	if(pt) 
		free(pt);
#endif		

}

void oidcpy(oid *name ,u_char *src,int size)
{
	int i=0;
	while(i<size){
		*(name+i)=*(src+i);
		i++;
	}
}
int oidcmp(u_char *src, oid *name,int size)
{
	int i=0;
	while(i<size){
		if((u_char)*(name+i)!=(u_char)*(src+i)){
			return -1;
		}
		i++;
	}
	return 0;
}

inline int get_ipAdEntAddr(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	u_char	IpAddr1[4];	
	u_char	IpAddr2[4];	
	u_char  If1[5]="";
	u_char  If2[5]="";
	
	get_IpAddress(IpAddr1,IpAddr2,If1,If2);
	
	/* GET */
	if(pdu->command == SNMP_MSG_GET){
		if(vp->name_length==11)
			return-1;
		else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
			memcpy(vp->buf,IpAddr1,4);
			vp->name_length=14;
			return 0;
		}else if(oidcmp(IpAddr2,&vp->name[10],4)==0){
			memcpy(vp->buf,IpAddr2,4);
			vp->name_length=14;
			return 0;
		}else
			return -1;
	}
	/* GET-NEXT*/	
	if(vp->name_length==11){
		memcpy(vp->buf,IpAddr1,4);
		oidcpy(&vp->name[10],IpAddr1,4);
		vp->name_length=14;
	}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
		memcpy(vp->buf,IpAddr2,4);
		oidcpy(&vp->name[10],IpAddr2,4);
		vp->name_length=14;
	}else
		return -1;
	
	return 0;
}

inline int get_ipAdEntIfIndex(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	u_char	IpAddr1[4];	
	u_char	IpAddr2[4];	
	u_char  If1[5]="";
	u_char  If2[5]="";
	
	get_IpAddress(IpAddr1,IpAddr2,If1,If2);

	vp->val.integer = (long *)(vp->buf);   
	vp->val_len = sizeof(long);
	
	/* GET */
	if(pdu->command == SNMP_MSG_GET){
		if(vp->name_length==11)
			return-1;
		else if(oidcmp(IpAddr2,&vp->name[10],4)==0){
			*vp->val.integer=(strcmp(If2,"br0")==0)?IF_WLAN:IF_ADSL; 
			return 0;
		}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
			*vp->val.integer=(strcmp(If1,"br0")==0)?IF_WLAN:IF_ADSL; 
			return 0;
		}else
			return -1;
	}
	/* GET-NEXT*/	
	if(vp->name_length==11){
		*vp->val.integer = (strcmp(If1,"br0")==0)?IF_WLAN:IF_ADSL; 
		oidcpy(&vp->name[10],IpAddr1,4);
		vp->name_length=14;
	}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
		*vp->val.integer = (strcmp(If2,"br0")==0)?IF_WLAN:IF_ADSL; 
		oidcpy(&vp->name[10],IpAddr2,4);
		vp->name_length=14;
	}else
		return -1;
	return 0;
}

inline int get_ipAdEntNetMask(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	u_char	IpAddr1[4];	
	u_char	IpAddr2[4];	
	u_char  If1[5]="";
	u_char  If2[5]="";
	
	get_IpAddress(IpAddr1,IpAddr2,If1,If2);

	
	/* GET */
	if(pdu->command == SNMP_MSG_GET){
		if(vp->name_length==11)
			return-1;
		else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
			getIfinfo(If1,vp->buf,IF_NETMASK);
			vp->name_length=14;
			return 0;
		}else if(oidcmp(IpAddr2,&vp->name[10],4)==0){
			getIfinfo(If2,vp->buf,IF_NETMASK);
			vp->name_length=14;
			return 0;
		}else
			return -1;
	}
	/* GET-NEXT*/	
	if(vp->name_length==11){
		getIfinfo(If1,vp->buf,IF_NETMASK);
		oidcpy(&vp->name[10],IpAddr1,4);
		vp->name_length=14;
	}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
		getIfinfo(If2,vp->buf,IF_NETMASK);
		oidcpy(&vp->name[10],IpAddr2,4);
		vp->name_length=14;
	}else
		return -1;
	return 0;
}

inline int get_ipAdEntBcastAddr(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	unsigned long int saddr,mask=1;
	int i=1,ret=0;
	u_char	IpAddr1[4];	
	u_char	IpAddr2[4];	
	u_char  If1[5]="";
	u_char  If2[5]="";
	
	get_IpAddress(IpAddr1,IpAddr2,If1,If2);

	/* GET */
	if(pdu->command == SNMP_MSG_GET){
		if(vp->name_length==11)
			ret=-1;
		else if(oidcmp(IpAddr2,&vp->name[10],4)==0){
			getIfinfo(If2,(char *)&saddr,IF_BRDADDR);
			vp->name_length=14;
		}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
			getIfinfo(If1,(char *)&saddr,IF_BRDADDR);
			vp->name_length=14;
		}
	}
	/* GET-NEXT*/
	else if(vp->name_length==11){
		getIfinfo(If1,(char *)&saddr,IF_BRDADDR);
		oidcpy(&vp->name[10],IpAddr1,4);
		vp->name_length=14;
	}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
		getIfinfo(If2,(char *)&saddr,IF_BRDADDR);
		oidcpy(&vp->name[10],IpAddr2,4);
		vp->name_length=14;
	}else
		ret = -1;
	
	if(ret==0){
		while(i < 32 && !(mask & ntohl(saddr))){
			mask=1<<i;
			i++;
		}
		*(long *)(vp->buf)=(i>32)?1:i;
	}
	return ret;
}

inline int get_ipAdEntReasmMaxSize(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	u_char	IpAddr1[4];	
	u_char	IpAddr2[4];	
	u_char  If1[5]="";
	u_char  If2[5]="";
	
	get_IpAddress(IpAddr1,IpAddr2,If1,If2);

	/* GET */
	if(pdu->command == SNMP_MSG_GET){
		if(vp->name_length==11)
			return-1;
		else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
			getIfinfo(If1,vp->buf,IF_MTU);
			vp->name_length=14;
			return 0;
		}else if(oidcmp(IpAddr2,&vp->name[10],4)==0){
			getIfinfo(If2,vp->buf,IF_MTU);
			vp->name_length=14;
			return 0;
		}else
			return -1;
	}
	/* GET-NEXT*/	
	if(vp->name_length==11){
		getIfinfo(If1,vp->buf,IF_MTU);
		oidcpy(&vp->name[10],IpAddr1,4);
		vp->name_length=14;
	}else if(oidcmp(IpAddr1,&vp->name[10],4)==0){
		getIfinfo(If2,vp->buf,IF_MTU);
		oidcpy(&vp->name[10],IpAddr2,4);
		vp->name_length=14;
	}else
		return -1;

	return 0;
}

route_table_t rt;
int get_rt(oid *name)
{
	int ret=0,item=1;
	char dest[16]="";
	while(ret>=0){
		ret=get_route_info(&rt,item++);
		if(ret < 0)
			return -1;
		sprintf(dest,"%d.%d.%d.%d"
			,(u_char)*name,(u_char)*(name+1)
			,(u_char)*(name+2),(u_char)*(name+3));
		if(strcmp(dest,inet_ntoa(rt.dest.sin_addr))==0)
			return 0;
	}
	return -1;

}
int Search_RouteDest(oid *name,int get_next)
{
	int item=1,ret=0;
	char dest[16]="";
	if(get_next==0){
		return get_rt(name);
	}else{
		uint32_t min=0,max=0;
		sprintf(dest,"%d.%d.%d.%d"
				,(u_char)*name,(u_char)*(name+1)
				,(u_char)*(name+2),(u_char)*(name+3));

		min=(uint32_t)inet_addr(dest);
		max=0xFFFFFFFF;
		while(ret>=0){
			ret=get_route_info(&rt,item++);
			if(ret < 0)
				break;
			if(ntohl(rt.dest.sin_addr.s_addr) > ntohl(min) 
					&& ntohl(rt.dest.sin_addr.s_addr) < ntohl(max))
				max=rt.dest.sin_addr.s_addr;	
		}
//		printf("min==%lx max=%lx\n",min,max);
		if(max==0xFFFFFFFF)
			return -1;
		
		*name=(max) & 0x000000FF;
		*(name+1)=(max>>8) & 0x000000FF;
		*(name+2)=(max>>16) & 0x000000FF;
		*(name+3)=(max>>24) & 0x000000FF;

		return get_rt(name);	
	}
	return -1;
}
int get_ipRoute(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	int get_next=(pdu->command == SNMP_MSG_GET)?0:1;
	if(get_next==0 && vp->name_length==11)
		return -1;	
		
	if(vp->name_length==11){
		int ret=0,item=1;
		uint32_t min=0xFFFFFFFF;
		while(ret>=0){
			ret=get_route_info(&rt,item++);
			if(ret >= 0 
				&& (ntohl(min)>ntohl(rt.dest.sin_addr.s_addr)))
			min=rt.dest.sin_addr.s_addr;
		}	
		vp->name[10]=(min) & 0x000000FF;
		vp->name[11]=(min>>8) & 0x000000FF;
		vp->name[12]=(min>>16) & 0x000000FF;
		vp->name[13]=(min>>24) & 0x000000FF;
		get_next=0;
	}
	if(Search_RouteDest(&vp->name[10],get_next)<0)
		return -1;
		
	vp->name_length=14;
	return 0;

}

inline int get_ipRouteDest(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=(unsigned long)rt.dest.sin_addr.s_addr;
		
	return 0;
}

inline int get_ipRouteIfIndex(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=(strncmp(rt.name,"br0",3)==0)?3:1;
		
	return 0;
}
inline int get_ipRouteMetric1(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=rt.metric;
		
	return 0;
}

inline int get_ipRouteMetric2(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=-1;
		
	return 0;
	
}
inline int get_ipRouteNextHop(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=(unsigned long)rt.gateway.sin_addr.s_addr;			
	return 0;

}
inline int get_ipRouteType(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=1;
	return 0;

}
inline int get_ipRouteProto(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=1;
	return 0;

}
inline int get_ipRouteAge(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;
	
	*(int *)vp->buf=1;
	return 0;
}
inline int get_ipRouteMask(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;

	*(int *)vp->buf=(unsigned long)rt.mask.sin_addr.s_addr;			
	return 0;
}
inline int get_ipRouteInfo(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	oid name[10]={0,0};
	if(get_ipRoute(pdu,vp,item)!=0)
		return -1;

	memcpy(vp->buf,name,2*sizeof(oid));
	vp->val_len = 2*sizeof(oid);
	return 0;
}

/* ipNetToMediaTable */
struct arp_info arpinfo;

inline int get_ipNetToMediaIfIndex(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	
	int ret=0;
	if(item==0)item=1;
	ret=get_arp_info(&arpinfo,item);

	if(ret <0)
		return -1;
	
	*(int *)vp->buf=(strncmp(arpinfo.ifname,"br0",3)==0)?2:1;

	return 0;
}
inline int get_ipNetToMediaPhysAddress(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	int ret=0;
	ret=get_arp_info(&arpinfo,item);
	
	if(ret <0)
		return -1;

	vp->val_len=convertStr2PhysAddr(vp->buf,arpinfo.mac);
	return 0;
}
inline int get_ipNetToMediaNetAddress(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	int ret=0;
	ret=get_arp_info(&arpinfo,item);
	
	if(ret <0)
		return -1;

	
	*(int *)vp->buf=(unsigned long)inet_addr(arpinfo.ipaddr);			
	return 0;
}
inline int get_ipNetToMediaType(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	int ret=0;
	ret=get_arp_info(&arpinfo,item);
	
	if(ret <0)
		return -1;

	*(int *)vp->buf=3;//arpinfo.type;
	return 0;
}

static map_entry_t MIB2Tab[] = {
    /* OID                          LENGTH  TYPE	  GET */
    { {SNMP_OID_MIB2, 1, 1},		 8, ASN_STR 	, get_sysDescr},
    { {SNMP_OID_MIB2, 1, 2},		 8, ASN_OID 	, get_ObjectID},
    { {SNMP_OID_MIB2, 1, 3},		 8, ASN_TIME	, get_sysUpTime},
    { {SNMP_OID_MIB2, 1, 4},		 8, ASN_STR	, get_sysContact},
    { {SNMP_OID_MIB2, 1, 5},		 8, ASN_STR 	, get_sysName},
    { {SNMP_OID_MIB2, 1, 6},		 8, ASN_STR 	, get_sysLocation},
    { {SNMP_OID_MIB2, 1, 7},		 8, ASN_INT 	, get_sysServices},
    /* interface */
    { {SNMP_OID_MIB2, 2, 1},		 8, ASN_INT 	, get_ifNumber},
    { {SNMP_OID_MIB2, 2, 2, 1, 1},	10, ASN_INT 	, get_ifIndex},
    { {SNMP_OID_MIB2, 2, 2, 1, 2},	10, ASN_STR 	, get_ifDescr},
    { {SNMP_OID_MIB2, 2, 2, 1, 3},	10, ASN_INT 	, get_ifType},
    { {SNMP_OID_MIB2, 2, 2, 1, 4},	10, ASN_INT 	, get_ifMtu},
    { {SNMP_OID_MIB2, 2, 2, 1, 5},	10, ASN_GAUGE 	, get_ifSpeed},
    { {SNMP_OID_MIB2, 2, 2, 1, 6},	10, ASN_PHY 	, get_ifPhysAddress},
    { {SNMP_OID_MIB2, 2, 2, 1, 7},	10, ASN_INT 	, get_ifAdminStatus},
    { {SNMP_OID_MIB2, 2, 2, 1, 8},	10, ASN_INT 	, get_ifOperStatus},
    { {SNMP_OID_MIB2, 2, 2, 1, 9},	10, ASN_TIME 	, get_ifLastChange},
    { {SNMP_OID_MIB2, 2, 2, 1, 10},	10, ASN_CNT 	, get_ifInOctets},
    { {SNMP_OID_MIB2, 2, 2, 1, 11},	10, ASN_CNT 	, get_ifInUcastPkts},
    { {SNMP_OID_MIB2, 2, 2, 1, 12},	10, ASN_CNT 	, get_ifInNUcastPkts},
    { {SNMP_OID_MIB2, 2, 2, 1, 13},	10, ASN_CNT 	, get_ifInDiscards},
    { {SNMP_OID_MIB2, 2, 2, 1, 14},	10, ASN_CNT 	, get_ifInErrors},
    { {SNMP_OID_MIB2, 2, 2, 1, 15},	10, ASN_CNT 	, get_ifInUnknownProtos},
    { {SNMP_OID_MIB2, 2, 2, 1, 16},	10, ASN_CNT 	, get_ifOutOctets},
    { {SNMP_OID_MIB2, 2, 2, 1, 17},	10, ASN_CNT 	, get_ifOutUcastPkts},
    { {SNMP_OID_MIB2, 2, 2, 1, 18},	10, ASN_CNT 	, get_ifOutNUcastPkts},
    { {SNMP_OID_MIB2, 2, 2, 1, 19},	10, ASN_CNT 	, get_ifOutDiscards},
    { {SNMP_OID_MIB2, 2, 2, 1, 20},	10, ASN_CNT 	, get_ifOutErrors},
    { {SNMP_OID_MIB2, 2, 2, 1, 21},	10, ASN_GAUGE 	, get_ifOutQLen},
    { {SNMP_OID_MIB2, 2, 2, 1, 22},	10, ASN_OID 	, get_ifSpecific},
    /* ip */
    { {SNMP_OID_MIB2, 4, 1},		 8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 2},		 8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 3},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 4},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 5},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 6},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 7},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 8},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4, 9},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,10},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,11},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,12},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,13},		 8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,14},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,15},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,16},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,17},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,18},		 8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 4,19},		 8,  ASN_CNT 	, get_SNMP_info},
    /* ip  ipAddrTable */
    { {SNMP_OID_MIB2, 4,20, 1, 1},	10,  ASN_IP 	, get_ipAdEntAddr},
    { {SNMP_OID_MIB2, 4,20, 1, 2},	10,  ASN_INT 	, get_ipAdEntIfIndex}, /* see MIBII */
    { {SNMP_OID_MIB2, 4,20, 1, 3},	10,  ASN_IP 	, get_ipAdEntNetMask},
    { {SNMP_OID_MIB2, 4,20, 1, 4},	10,  ASN_INT 	, get_ipAdEntBcastAddr},
    { {SNMP_OID_MIB2, 4,20, 1, 5},	10,  ASN_INT 	, get_ipAdEntReasmMaxSize},  /* MRU = MTU ?? */
    /* ip RouteTable */ 
    { {SNMP_OID_MIB2, 4,21, 1, 1},	10,  ASN_IP 	, get_ipRouteDest},
    { {SNMP_OID_MIB2, 4,21, 1, 2},	10,  ASN_INT 	, get_ipRouteIfIndex},
    { {SNMP_OID_MIB2, 4,21, 1, 3},	10,  ASN_INT 	, get_ipRouteMetric1},
    { {SNMP_OID_MIB2, 4,21, 1, 4},	10,  ASN_INT 	, get_ipRouteMetric2},
    { {SNMP_OID_MIB2, 4,21, 1, 5},	10,  ASN_INT 	, get_ipRouteMetric2},
    { {SNMP_OID_MIB2, 4,21, 1, 6},	10,  ASN_INT 	, get_ipRouteMetric2},
    { {SNMP_OID_MIB2, 4,21, 1, 7},	10,  ASN_IP 	, get_ipRouteNextHop},
    { {SNMP_OID_MIB2, 4,21, 1, 8},	10,  ASN_INT 	, get_ipRouteType},
    { {SNMP_OID_MIB2, 4,21, 1, 9},	10,  ASN_INT 	, get_ipRouteProto},
    { {SNMP_OID_MIB2, 4,21, 1, 10},	10,  ASN_INT 	, get_ipRouteAge},
    { {SNMP_OID_MIB2, 4,21, 1, 11},	10,  ASN_IP 	, get_ipRouteMask},
    { {SNMP_OID_MIB2, 4,21, 1, 12},	10,  ASN_INT 	, get_ipRouteMetric2},
    { {SNMP_OID_MIB2, 4,21, 1, 13},	10,  ASN_OID 	, get_ipRouteInfo},
    { {SNMP_OID_MIB2, 4,22, 1, 1},	10,  ASN_INT 	, get_ipNetToMediaIfIndex},
    { {SNMP_OID_MIB2, 4,22, 1, 2},	10,  ASN_PHY 	, get_ipNetToMediaPhysAddress},
    { {SNMP_OID_MIB2, 4,22, 1, 3},	10,  ASN_IP 	, get_ipNetToMediaNetAddress},
    { {SNMP_OID_MIB2, 4,22, 1, 4},	10,  ASN_INT 	, get_ipNetToMediaType},

    /* icmp */
    { {SNMP_OID_MIB2, 5, 1},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 2},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 3},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 4},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 5},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 6},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 7},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 8},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5, 9},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,10},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,11},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,12},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,13},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,14},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,15},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,16},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,17},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,18},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,19},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,20},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,21},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,22},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,23},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,24},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,25},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 5,26},		8,  ASN_CNT 	, get_SNMP_info},
    /* tcp */
    { {SNMP_OID_MIB2, 6, 1},		8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 2},		8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 3},		8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 4},		8,  ASN_INT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 5},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 6},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 7},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 8},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6, 9},		8,  ASN_GAUGE 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6,10},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6,11},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6,12},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6,13},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 6,14},		8,  ASN_CNT 	, get_SNMP_info},
    /* udp */
    { {SNMP_OID_MIB2, 7, 1},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 7, 2},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 7, 3},		8,  ASN_CNT 	, get_SNMP_info},
    { {SNMP_OID_MIB2, 7, 4},		8,  ASN_CNT 	, get_SNMP_info},

    
    { {SNMP_OID_MIB2, 0, 0}, 		0,     	  0 	, NULL}

};
int get_mib2_entry(snmp_pdu *pdu,netsnmp_variable_list *vp)
{
	map_entry_t *tb;
	int ret=-1;
	int result=0;
	int item=0;
	int group=0;
	int table=0;
	if(vp->name_length > 7){
		group=vp->name[6];
		table=vp->name[7];
	}
	
	for(tb=MIB2Tab;tb->nameLen >0 ;tb++){

		/* GET */
		if(pdu->command == SNMP_MSG_GET){
			result=memcmp(vp->name,tb->name,tb->nameLen * sizeof(oid));
			if(result!=0)
				continue;
			vp->type=tb->type;
			item=vp->name[vp->name_length-1];
			return (*(tb->getfun))(pdu,vp,item);
		}
		
		/* GET NEXT */
		if( vp->name_length == 6){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}
		if( vp->name_length == 7){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}
		if( vp->name_length == 8){
			if((group==2 && table==2)
				|| (group==4 && (table==20 || table==21 || table==22 ))){
				vp->name[vp->name_length]=1;
				vp->name_length++;
			}
		}
		if( vp->name_length == 9 &&  vp->name[vp->name_length-1]!=0 ){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}
		result=memcmp(vp->name,tb->name,tb->nameLen * sizeof(oid));
		
		if(result != 0)
			continue;
		
		if(vp->name_length > tb->nameLen){
			item=vp->name[vp->name_length-1];
			
			if(item==0){
				if((group==4 && (table==20 || table==21)))
					;/* nothing */
				/* get next entry in table */
				else if((tb+1)->nameLen!=0){
					tb++;
					memcpy(vp->name,tb->name,(tb->nameLen)*sizeof(oid));
					vp->name_length=tb->nameLen;
					if((group==2 && table==1) ||
							(group==4 && table==19)){
						vp->name[tb->nameLen] = 1;
					}else
						vp->name[tb->nameLen] = 0;
					vp->name_length++;
					
				}else 
					return -1;
			}else{
				if(!(group==4 && (table==20 || table==21)))
				vp->name[vp->name_length-1]+=1;
			}
			
		}else{
			/* those entry have instance */
			if((group==2 && table==2)
				|| (group==4 && (table==20 || table==21 || table==22 ))){
				vp->name[tb->nameLen] = 1;
				vp->name_length++;
			}else{
				vp->name[tb->nameLen] = 0;
				vp->name_length++;
			}
		}
			
		if(!tb->getfun)
			ret=-1;	
		else{
			item=vp->name[vp->name_length-1];	
			vp->type=tb->type;
			ret=(*(tb->getfun))(pdu,vp,item);
		}
		/* find nearest */
		while(ret==-1){
			if((tb+1)->nameLen!=0){
				tb++;
				memcpy(vp->name,tb->name,(tb->nameLen)*sizeof(oid));
				vp->name_length=tb->nameLen;
				
				if(vp->name_length > 7){
					group=vp->name[6];
					table=vp->name[7];
				}else{
					group=0;
					table=0;
				}
			
				if((group==2 && table==2)
					|| (group==4 && table==22)){
					vp->name[tb->nameLen] = 1;
					vp->name_length++;
					item=1;
				}else{
					vp->name[tb->nameLen] = 0;
					vp->name_length++;
					item=0;
				}

			}else
				return -1;
			
			vp->type=tb->type;
			ret=(*(tb->getfun))(pdu,vp,item);
		}
		return ret;
	}
	return ret;
}

