#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <unistd.h>
#include <syslog.h>
#include <arpa/inet.h>
#include "my_snmp_lib.h"
#include "socket_tools.h"
#include "snmp.h"

#include "snmp_lib.h"
#include "snmp_asn1.h"

#define MAX_PACKET_LENGTH   (0x7fffffff)
#define COMMUNITY_MAX_LEN 16
#define SET_N_TEXT(var,value,len) {var=malloc(len+1); bzero(var,len+1); memcpy(var,value,len);} 
#define CLEAR_TEXT(var) {if(var) free(var);}

#define dprintf(...) 

int init_snmp(char *ipaddr)
{
	int server_fd;
    	struct sockaddr_in  server_sock;
	int retval=-1;
	int so_reuseaddr = 1;
	
	server_fd=socket(AF_INET, SOCK_DGRAM, 0);
	if(server_fd<0){
        	printf("Couldn't create socket\n");
	        exit(1);
    	}
	setsockopt(server_fd,SOL_SOCKET,SO_REUSEADDR,&so_reuseaddr,sizeof(so_reuseaddr));
	
	server_sock.sin_family=AF_INET;
	//server_sock.sin_addr.s_addr = htonl(INADDR_ANY);
	server_sock.sin_addr.s_addr = inet_addr(ipaddr);

	server_sock.sin_port = htons(SNMP_PORT);
	retval = bind(server_fd, (struct sockaddr *) &server_sock, sizeof(server_sock));
	
	if(retval < 0){
	        printf("can't bind socket\n");
        	exit(1);
	}
	return server_fd;

}

/* 
   This takes a PDU, and parses its data.
   Returns:    0 for success    negitive for error
*/
static int parse_pdu_data (snmp_pdu *pdu, u_char *pData, int *recvlen)
{
	u_char          type;
	u_char          *var_val;
	netsnmp_variable_list   *vp = NULL;
	oid         objid[MAX_OID_LEN];
	size_t          len;
	int         badtype = 0;

	/* retrieve request ID */
	pData = asn_parse_int(pData, recvlen, &type, &pdu->req_id,sizeof(pdu->req_id));
    
	if (!pData){
        	syslog(LOG_INFO, "Request ID parsing failed\n");
		return -1;
	}

	/* retrieve error status (non-repeaters for getbulk) */    
	pData = asn_parse_int(pData, recvlen, &type, &pdu->errstat,sizeof(pdu->errstat));
	
	if (!pData){
        	syslog(LOG_INFO,"Error status (max-repeaters) parsing failed\n");
		return -1;
	}

	/* retrieve error index (max-repetitions for getbulk) */
	pData = asn_parse_int(pData, recvlen, &type, &pdu->errindex,sizeof(pdu->errindex));
	
	if (!pData){
        	syslog(LOG_INFO,"Error index (max-repetitions) parsing failed\n");
		return -1;
	}

	/* get header for variable-bindings sequence */
	pData = asn_parse_sequence(pData, recvlen, &type,(ASN_SEQUENCE | ASN_CONSTRUCTOR),"varbinds");
	if (!pData){
        	syslog(LOG_INFO, "Var-Bind header parsing failed\n");
		return -1;
	}

	/* get each varBind sequence */
	while (*recvlen > 0){
        	netsnmp_variable_list *vptemp;
		vptemp = (netsnmp_variable_list *)malloc(sizeof(*vptemp));
        	if (!vptemp)
	    		return -1;

        	if (!vp){
		        pdu->pVarBind = vptemp;
		} else {
            		vp->next_variable = vptemp;
        	}
        	
		vp = vptemp;
	        vp->next_variable = NULL;
        	vp->val.string = NULL;
	        vp->name_length = MAX_OID_LEN;
        	vp->name = 0;
	        vp->index = 0;
        	pData = snmp_parse_var_op(pData, objid, &vp->name_length, &vp->type,&vp->val_len
				, &var_val, recvlen);
	        if (!pData)
        		return -1;

		if (snmp_set_var_objid(vp, objid, vp->name_length))
            		return -1;
#if 0 
		printf("vp->nlen=%d vp->type=%d",vp->name_length,vp->type);
		if(1){
			int i=0;
			for(i=0;i<(vp->name_length);i++)
				printf("%lx",vp->name[i]);
			printf("\n");	
		}
#endif
		len = MAX_PACKET_LENGTH;

		switch ((short) vp->type){
		        case ASN_INTEGER:
                		vp->val.integer = (long *) vp->buf;
		                vp->val_len = sizeof(long);
                		asn_parse_int(var_val, &len, &vp->type,
	                        (long *) vp->val.integer,sizeof(vp->val.integer));
                	break;
		        case ASN_COUNTER:
		        case ASN_GAUGE:
		        case ASN_TIMETICKS:
		        case ASN_UINTEGER:
                		vp->val.integer = (long *) vp->buf;
                		vp->val_len = sizeof(u_long);
		                asn_parse_unsigned_int(var_val, &len, &vp->type
						,(u_long *) vp->val.integer,vp->val_len);
		        	break;
			case ASN_COUNTER64:
	                	vp->val.counter64 = (struct counter64 *) vp->buf;
		                vp->val_len = sizeof(struct counter64);
        		        asn_parse_unsigned_int64(var_val, &len, &vp->type
						,(struct counter64 *) vp->val.counter64
						, vp->val_len);
		                break;
                	case ASN_OCTET_STR:
		        case ASN_IPADDRESS:
			case ASN_OPAQUE:
			case ASN_NSAP:
				if (vp->val_len < sizeof(vp->buf)){
					vp->val.string = (u_char *) vp->buf;
				} else {
					vp->val.string = (u_char *) malloc(vp->val_len);
				}
				if (!vp->val.string)
					return -1;
				asn_parse_string(var_val, &len, &vp->type, vp->val.string
						,&vp->val_len);
				break;
		        case ASN_OBJECT_ID:
				vp->val_len = MAX_OID_LEN;
				asn_parse_objid(var_val, &len, &vp->type, objid, &vp->val_len);
				vp->val_len *= sizeof(oid);
				vp->val.objid = (oid *) malloc(vp->val_len);
				if (!vp->val.objid)
					return -1;
				memmove(vp->val.objid, objid, vp->val_len);
				break;
		        case SNMP_NOSUCHOBJECT:
			case SNMP_NOSUCHINSTANCE:
			case SNMP_ENDOFMIBVIEW:
			case ASN_NULL:
				break;
			case ASN_BIT_STR:
				vp->val.bitstring = (u_char *) malloc(vp->val_len);
		                if (!vp->val.bitstring)
                			return -1;
		                asn_parse_bitstring(var_val, &len, &vp->type
						,vp->val.bitstring, &vp->val_len);
		                break;
		        default:
		                syslog(LOG_INFO, "bad type returned (%x)\n", vp->type);
                		badtype = -1;
		                break;
		}
    }

    return (badtype);
}

static int parse_pdu(snmp_pdu *pdu, char *buffer, int recvlen)
{
    u_char   *pData;
    u_char   community[COMMUNITY_MAX_LEN];
    size_t   community_length = COMMUNITY_MAX_LEN;
    u_char   ucOperation;

    /* Assign our working pointer to the recv buffer */
    pData = (u_char *)buffer;

    /* retrieve version from SNMP PDU */
    pdu->version = snmp_parse_version(pData, recvlen);

    /* retrieve community name from SNMP PDU */
    pData = snmp_comstr_parse (pData, &recvlen, community,&community_length, &pdu->version);
    
    if (!pData) {
        syslog(LOG_INFO, "Community parsing failed\n");
        return -1;
    }
    
    /* Save off the community name */
    SET_N_TEXT(pdu->community, community, community_length);
    
    /* get the PDU type (operation) */
    pData = asn_parse_header(pData, &recvlen, &ucOperation);
    if (!pData) {
        syslog(LOG_INFO, "Operation parsing failed\n");
        return -1;
    }
    pdu->command = ucOperation;

    /* Now, just parse the rest of the pdu data */
    return parse_pdu_data(pdu, pData, &recvlen);
}

static void flush_pdu(snmp_pdu *pdu)
{
    netsnmp_variable_list *vp, *vpTemp;

    if(!pdu)
        return;

    /* Loose the strings */
    CLEAR_TEXT(pdu->community);

    /* Clean out the varbinds */
    for (vp = pdu->pVarBind; vp; vp = vpTemp){
        vpTemp = vp->next_variable;
        free(vp);
    }
    pdu->pVarBind = NULL;
}

void snmp_process_getBulk(snmp_pdu *pdu)
{
    netsnmp_variable_list       *varbind_ptr,
                    *vbsave,
                    *vbptr,
                    **prevNext;
    int bulkcount = 0, bulkrep = 0;
    int i = 0,
        n = 0,
        r = 0;
    int count = count_varbinds(pdu->pVarBind);

    if(!pdu)
        return;

    if (pdu->errstat < 0)
        pdu->errstat = 0;

    if (pdu->errindex < 0)
        pdu->errindex = 0;

    if (pdu->errstat < count)
        n = pdu->errstat;
    else
        n = count;

    if ((r = count - n) < 0){
        r = 0;
        pdu->pBulk = NULL;
    } else {
        pdu->pBulk = (netsnmp_variable_list **) malloc(
                pdu->errindex * r * sizeof(struct varbind_list *));
    }

    prevNext = &(pdu->pVarBind);
    for (varbind_ptr = pdu->pVarBind; varbind_ptr; varbind_ptr = vbsave){
        /*
         * getbulk mess with this pointer, so save it 
         */
        vbsave = varbind_ptr->next_variable;

        if (n > 0) {
            n--;
        } else {
            /*
             * repeate request varbinds on GETBULK.  These will
             * have to be properly rearranged later though as
             * responses are supposed to actually be interlaced
             * with each other.  This is done with the asp->bulkcache. 
             */
            bulkrep = pdu->errindex - 1;
            if (pdu->errindex > 0) {
                vbptr = varbind_ptr;
                pdu->pBulk[bulkcount++] = vbptr;

                for (i = 1; i < pdu->errindex; i++) {
                    vbptr->next_variable =
                        SNMP_MALLOC_STRUCT(variable_list);
                    /*
                     * don't clone the oid as it's got to be
                     * overwwritten anyway 
                     */
                    if (!vbptr->next_variable) {
                        /*
                         * XXXWWW: ack!!! 
                         */
                    } else {
                        vbptr = vbptr->next_variable;
                        vbptr->name_length = 0;
                        vbptr->type = ASN_NULL;
                        pdu->pBulk[bulkcount++] = vbptr;
                    }
                }
                vbptr->next_variable = vbsave;
            } else {
                /*
                 * 0 repeats requested for this varbind, so take it off
                 * the list.  
                 */
                vbptr = varbind_ptr;
                *prevNext = vbptr->next_variable;
                vbptr->next_variable = NULL;
                snmp_free_varbind(vbptr);
                pdu->vbcount--;
                continue;
            }
        }

        /*
         * count the varbinds 
         */
        prevNext = &(varbind_ptr->next_variable);
    }
}

#if 0
static void modify_requestOID(netsnmp_variable_list *vp,
                              int loc,
                              oid *name, int nameLen,
                              int indexLen)
{
    int i;
    int len = nameLen + indexLen;
    
    if ((vp->name != vp->name_loc) &&
        (vp->name != NULL) &&
        ((len + 6) > vp->name_length))
    {
        if (vp->name)
            free(vp->name);

        vp->name = (oid *) malloc(sizeof(oid) * (len + 6));
    }
    
    // make sure we have OID up to mib2
    vp->name[0] = 1;
    vp->name[1] = 3;
    vp->name[2] = 6;
    vp->name[3] = 1;
    vp->name[4] = 2;
    vp->name[5] = 1;
    
    for (i = 0; i < len; i++)
    {
        vp->name[loc+i] = name[i];
    }
    
    vp->name_length = len + 6;
}
static void fix_GetNextOid(netsnmp_variable_list *vp)
{
    oid mgmt = vp->name[SNMP_MIB2_MGMT];
    oid group = vp->name[SNMP_MIB2_GROUP];
    oid table = vp->name[SNMP_MIB2_TABLE];

    if (mgmt > 2)
        return;
    
    // fill up the mib2 part if not long enough
    if (vp->name_length < SNMP_MIB2_GROUP)
        modify_requestOID(vp, 0, NULL, 0, 0);

    // missing group. fill it up
    // We must doing GetNext to reach this point
    if (vp->name_length < SNMP_MIB2_LEAF)
    {
        vp->name[SNMP_MIB2_LEAF] = 1;
        return;
    }
    
    if (group == 1)  // sysORtable
    {
        if (table == 9)
        {
            if (vp->name_length == 11)
                return;
            
            if (vp->name_length > 11)
            {
                // vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMP_MIB2_INDEX+1;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMP_MIB2_TBLENTRY] = 1;
                vp->name[SNMP_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMP_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }
    else if (group == 2)
    {
        if (table == 2)
        {
            if (vp->name_length == 11)
                return;
            
            if (vp->name_length > 11)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMP_MIB2_INDEX+1;
                return;
            }
            
            if (vp->name_length < 10)
            {
                vp->name[SNMP_MIB2_TBLENTRY] = 1;
                vp->name[SNMP_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMP_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }
    else if (group == 4)
    {
        if ((table == 20) || (table == 21))
        {
            if (vp->name_length == 14)
                return;
            if (vp->name_length > 14)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMP_MIB2_INDEX+4;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMP_MIB2_TBLENTRY] = 1;
                vp->name[SNMP_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMP_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
        else if (table == 22)
        {
            if (vp->name_length == 15)
                return;
            
            if (vp->name_length > 15)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMP_MIB2_INDEX+5;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMP_MIB2_TBLENTRY] = 1;
                vp->name[SNMP_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMP_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }

#if 0
    if (vp->name_length > 8)
    {                    
        vp->name_length = SNMPL_MIB2_LEAF+1;
    }
#endif
}

#endif
extern int get_map_entry(snmp_pdu *pdu,netsnmp_variable_list *vp);

static int snmp_process_entry(snmp_pdu *pdu,netsnmp_variable_list *vp)
{
	int ret=-1;

//        fix_GetNextOid(vp);
	
	ret=get_map_entry(pdu,vp);
	
	if(ret < 0){

        	pdu->errstat = SNMP_ERR_NOSUCHNAME;
	        pdu->errindex = pdu->handled_index;
		return -1;
	}
	return 0;
}

/* returns 0 if the PDU should be sent back or -1 if the PDU response should be dropped */
static int handle_pdu_commands(snmp_pdu *pdu)
{
	int retval, saved_command;
	netsnmp_variable_list   *prevVp = NULL, *vp = NULL;

	if(!pdu)
		return -1;
	
	 switch (pdu->command){
        	case SNMP_MSG_GET:
	        case SNMP_MSG_GETNEXT:
        	case SNMP_MSG_GETBULK:
			for (vp = pdu->pVarBind; vp; vp = vp->next_variable){
                		snmp_set_var_typed_value(vp, ASN_NULL, NULL, 0);
			}
            		pdu->vbcount = count_varbinds(pdu->pVarBind);
			dprintf("pdu->vbcount=%d\n",pdu->vbcount);
	        	break;
	        case SNMP_MSG_SET:
        	    /* Check Read-Write access here when supported */
	            /* Fall through in phase I */
        	default:
	            printf("Not supported SNMP operation(%x)\n",pdu->command);
            	return -1;
	}


	if (pdu->command == SNMP_MSG_GETBULK)
        	snmp_process_getBulk(pdu);


	/* Now, work through the entries, processing each entry. */
	for (vp = pdu->pVarBind; vp; vp = vp->next_variable)
	{
        	pdu->handled_index++;  // This index has to be 1-base
		saved_command = pdu->command;
	        if (pdu->command == SNMP_MSG_GETBULK){
			if (prevVp)
		                snmp_set_var_objid(vp, prevVp->name,prevVp->name_length);
            
		        pdu->command = SNMP_MSG_GETNEXT;
	        }
		
       		do{
        		retval=snmp_process_entry(pdu, vp);
		}while(retval > 0);	

	        if(retval<0){
        		/* Error with this entry. Check the return code */
	        	if(retval < -1){
                		/* Do not send error packet */
	                	return -1;
        		}
	        	return 0;   /* Send error packet */
        	}

        	pdu->handled_var = vp;
	        pdu->command = saved_command;
        	prevVp = vp;
    	}

	return 0;
}

int snmp_rebuild_varBind(snmp_pdu *pdu, u_char **pkt,size_t *pkt_len, size_t *offset)
{
#ifndef VPCACHE_SIZE
#define VPCACHE_SIZE 50
#endif
    netsnmp_variable_list *vpcache[VPCACHE_SIZE];
    netsnmp_variable_list *vp, *tmpvp;
    size_t          start_offset = *offset;
    int             i,
            wrapped = 0,
            notdone, final,
            rc = 0;

    if(!pdu)
        return 0;

    for (vp = pdu->pVarBind, i = VPCACHE_SIZE - 1; vp;
                vp = vp->next_variable, i--) {
        if (i < 0) {
            wrapped = notdone = 1;
            i = VPCACHE_SIZE - 1;
        }
        vpcache[i] = vp;
    }
    final = i + 1;

    do {
        for (i = final; i < VPCACHE_SIZE; i++) {
            vp = vpcache[i];
            rc = snmp_realloc_rbuild_var_op(pkt, pkt_len, offset, 1,
                    vp->name, &vp->name_length,
                    vp->type,
                    (u_char *) vp->val.string,
                    vp->val_len);

	    //printf("rc=%d vp->type=%d vp->val.string=%s,vp->val_len=%d\n",rc,vp->type,vp->val.string,vp->val_len);
            if (!rc)
                return 0;
        }

        if (wrapped) {
            notdone = 1;
            for (i = 0; i < final; i++) {
                vp = vpcache[i];
                rc = snmp_realloc_rbuild_var_op(pkt, pkt_len, offset, 1,
                        vp->name, &vp->name_length,
                        vp->type,
                        (u_char *) vp->val.string,
                        vp->val_len);
                if (!rc)
                    return 0;
            }

            if (!final) {
                tmpvp = vpcache[VPCACHE_SIZE - 1];
            } else {
                tmpvp = vpcache[final - 1];
            }
            wrapped = 0;

            for (vp = pdu->pVarBind, i = VPCACHE_SIZE - 1;
                        vp && vp != tmpvp;
                        vp = vp->next_variable, i--) {
                if (i < 0) {
                    wrapped = 1;
                    i = VPCACHE_SIZE - 1;
                }
                vpcache[i] = vp;
            }
            final = i + 1;
        } else {
            notdone = 0;
        }
    } while (notdone);

    /*
     * Save current location and build SEQUENCE tag and length placeholder for
     * variable-bindings sequence (actual length will be inserted later).  
     */
    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) (ASN_SEQUENCE |
                  ASN_CONSTRUCTOR),
            *offset - start_offset);
    /*
     * Store fields in the PDU preceeding the variable-bindings sequence.  
     */
    /*
     * Error index (getbulk max-repetitions).  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->errindex,
            sizeof(pdu->errindex));
    if (rc == 0) {
        return 0;
    }

    /*
     * Error status (getbulk non-repeaters).  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->errstat,
            sizeof(pdu->errstat));
    if (rc == 0) {
        return 0;
    }

    /*
     * Request ID.  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->req_id,
            sizeof(pdu->req_id));
    if (rc == 0) {
        return 0;
    }

    /*
     * Build the PDU sequence.  
     */
    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) pdu->command,
            *offset - start_offset);
    
    return rc;
}

int snmp_build(snmp_pdu *pdu, u_char **pkt,size_t *pkt_len, size_t *offset)
{
    int    rc;
    long   version;
    size_t length, start_offset = *offset;

    if(!pdu)
        return -1;

    length = *pkt_len;

    /* use reverse asn encoding */
    /* rebuild the var-bind */    
    rc = snmp_rebuild_varBind(pdu, pkt, pkt_len, offset);
    if (!rc)
        return -1;

    /* rebuild community string */
    rc = asn_realloc_rbuild_string(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_OCTET_STR),
            pdu->community,
            strlen(pdu->community));
    if (!rc)
        return -1;

    /* rebuild version */
    version = pdu->version;
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            (long *) &version,
            sizeof(version));
    if (!rc)
        return -1;

    /* rebuild the final sequence */
    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) (ASN_SEQUENCE |
                  ASN_CONSTRUCTOR),
            *offset - start_offset);

    if (!rc)
        return -1;

    return 0;
}
/* TODO: won't work. We have to sent the resposne to a known ip */
void snmp_send_response(snmp_pdu *pdu)
{
#define SNMP_RESP_LEN 2048
    u_char *pktbuf = NULL, *packet = NULL;
    size_t  pktbuf_len = 0, offset = 0, length = 0;
    int     result;

    if(!pdu)
        return;

    pdu->command = SNMP_MSG_RESPONSE;

    if ( (pktbuf = malloc(SNMP_RESP_LEN)) == NULL){
        syslog(LOG_INFO, "Can not allocate packet buffer for SNMP response\n");
        return;
    }

    length = pktbuf_len = SNMP_RESP_LEN;
    packet = pktbuf;

    result = snmp_build(pdu, &pktbuf, &pktbuf_len, &offset);

    if (result < 0){
        free (packet);
        syslog (LOG_INFO, "SNMP response PDU encoding failure\n");
        return;
    }
    packet = pktbuf + pktbuf_len - offset;
    length = offset;

    /* TODO: change to STDOUT, but first have to setup the fd in main */
    sendto(STDIN_FILENO, packet,length,0,(struct sockaddr *)pdu->from, sizeof(struct sockaddr_in));
    free (pktbuf);
}

int check_community(snmp_pdu *pdu)
{
	char *pt=NULL;
	int ret=-1;
	/* only support read function*/
	if(pdu->command == SNMP_MSG_GETBULK
			|| pdu->command == SNMP_MSG_GET
			|| pdu->command == SNMP_MSG_GETNEXT){
		scfgmgr_get("snmp_read_community",&pt);	
		if(pt==NULL)
			ret=-1;
		else if(strlen(pt)==0)
			ret=-1;
		else if(strcmp(pdu->community,pt)==0)
			ret=0;
		else 
			ret=-1;
	}
		
	if(pt) free(pt);
	
	return ret;
}

int process_snmpPDU(char *rcvBuffer,int recvlen, struct sockaddr_in *from)
{
#define COMMUNITY_MAX_LEN 16
	snmp_pdu 	pdu;
	
    	memset(&pdu, 0, sizeof(snmp_pdu));
	
	pdu.from = from;
	
	if(parse_pdu(&pdu, rcvBuffer, recvlen) <0){
	        /* Error parsing the pdu. Flush the leftover, and leave */
        	flush_pdu(&pdu);
	        return -1;
    	}
	/* check version */
	if ((pdu.version != SNMP_VERSION_1) && (pdu.version != SNMP_VERSION_2c)) {
		printf( "Bad version(%ld) in SNMP PDU\n",pdu.version);
        	flush_pdu(&pdu);
	        return -1;
	}
	
	/* check community */
	if(check_community(&pdu)<0){
        	flush_pdu(&pdu);
	        return -1;
	}
	
	if(!handle_pdu_commands(&pdu)){
        	/* The PDU was handled correctly */
	        snmp_send_response(&pdu);
	}

	flush_pdu(&pdu);

	return 0;
}

int snmp_oid_cmp(oid *in_name1, oid *in_name2, int len1, int len2)
{
    register int    len;
    register const oid *name1 = in_name1;
    register const oid *name2 = in_name2;

    /*
     * len = minimum of len1 and len2 
     */
    if (len1 < len2)
        len = len1;
    else
        len = len2;
    /*
     * find first non-matching OID 
     */
    while (len-- > 0) {
        /*
         * these must be done in seperate comparisons, since
         * subtracting them and using that result has problems with
         * subids > 2^31. 
         */
        if (*(name1) != *(name2)) {
            if (*(name1) < *(name2))
                return -1;
            return 1;
        }
        name1++;
        name2++;
    }
    /*
     * both OIDs equal up to length of shorter OID 
     */
    if (len2 > len1)
        return -1;
    if (len1 > len2)
        return 1;
    return 0;
}

