#include <sys/sysinfo.h>
#include <sys/utsname.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "include/snmp_asn1.h"
#include "include/snmp.h"
#include "my_snmp_lib.h"
#include "socket_tools.h"
#include "utility.h"

/* length 7 */
#define SNMP_OID_NETGEAR		1, 3, 6, 1, 4, 1, 4526

inline int get_dgNMSAccessIndex(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	
	*(long *)(vp->buf)=1;
	
	return 0;
}
inline int get_dgNMSAccessIp(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	
	*(long *)(vp->buf)=0xffffffff;

	return 0;
}
inline int get_dgNMSAccessIpMask(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	
	*(long *)(vp->buf)=0xffffffff;
	return 0;
}
inline int get_dgNMSAccessCommunity(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;

	if(item!=1)
		return -1;
	
	scfgmgr_get("snmp_read_community",&pt);
	if(!pt)
		return -1;
	strcpy(vp->buf,pt);
       	free(pt);
	
	return 0;
}
inline int get_dgNMSAccessType(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=1;
	return 0;
}
inline int get_dgNMSAccessRowStatus(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{	
	if(item!=1)
		return -1;
	
	*(long *)(vp->buf)=1;

	return 0;
}

inline int get_dgCPUUsage1min(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	int i=0;
	
	fp=fopen("/tmp/cpu_avg1","r");
	if(fp){
		fscanf(fp,"%d",&i);
		*(long *)(vp->buf)=i;
		fclose(fp);
		return 0;	
	}else{
		*(long *)(vp->buf)=i;
		return 0;	
	}
	return  -1;	
}
inline int get_dgCPUUsage5min(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	int i=0;
	
	fp=fopen("/tmp/cpu_avg5","r");
	if(fp){
		fscanf(fp,"%d",&i);
		*(long *)(vp->buf)=i;
		fclose(fp);
		return 0;	
	}else{
		*(long *)(vp->buf)=i;
		return 0;	
	}
	return  -1;	
}

inline int get_dgMEMUsage(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	int  mem=0;
	fp=fopen("/proc/meminfo","r");
	if(!fp)
		return -1;
	
	while ( getc (fp) != '\n' && feof(fp)==0);
	
	if(feof(fp)){
		fclose(fp);
		return -1;
	}

	fscanf(fp,"%*s  %*d %d",&mem);
	fclose(fp);
	*(long *)(vp->buf)=mem;
	return 0;
}
inline int get_dgFreeMEM(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	int  total_mem=0;
	int  use_mem=0;
	fp=fopen("/proc/meminfo","r");
	if(!fp)
		return -1;
	while ( getc (fp) != '\n' && feof(fp)==0);
	
	if(feof(fp)){
		fclose(fp);
		return -1;
	}

	fscanf(fp,"%*s  %d %d",&total_mem,&use_mem);
	fclose(fp);
	*(long *)(vp->buf)=total_mem-use_mem;
	return 0;
}

/* wireless */
inline int get_dgdot11OperationalStatus(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	*(long *)(vp->buf)=(access("/tmp/wlan_uptime",F_OK)==0)?1:2;
		
	return 0;
}
inline int get_dgdot11DesiredSSID(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("wifi_ssid",&pt);
	if(!pt)
		return -1;
	strcpy(vp->buf,pt);
       	free(pt);
	
	return 0;
}
inline int get_dgdot11SSIDBroadcastOrNot(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("wifi_broadcast_ssid",&pt);
	if(!pt)
		return -1;
	*(long *)(vp->buf)=(*pt=='1')?1:2;
	free(pt);
	
	return 0;
}

inline int get_dgdot11Mode(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("wifi_dot11_mode",&pt);
	if(!pt)
		return -1;
	
	if(*pt=='0') /* gb */
		*(long *)(vp->buf)=3;
	else if(*pt=='1') /* b */
		*(long *)(vp->buf)=1;
	else /* g */
		*(long *)(vp->buf)=2;
			
	free(pt);	
	return 0;
}
inline int get_dgdot11ChannelSelection(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	char *pt=NULL;
	scfgmgr_get("wifi_channel",&pt);
	if(!pt)
		return -1;
	
	*(long *)(vp->buf)=atoi(pt);
	free(pt);
	return 0;
}

inline int get_dgdot11NumberSupportedPowerLevels(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=5;
	return 0;
}
inline int get_dgdot11TXPowerLevel1(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=79;
	return 0;
}
inline int get_dgdot11TXPowerLevel2(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=63;
	return 0;
}
inline int get_dgdot11TXPowerLevel3(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=40;
	return 0;
}
inline int get_dgdot11TXPowerLevel4(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=32;
	return 0;
}
inline int get_dgdot11TXPowerLevel5(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=25;
	return 0;
}
inline int get_dgdot11TXPowerLevel6(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=0;
	return 0;
}
inline int get_dgdot11CurrentTXPowerLevel(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=1)
		return -1;
	*(long *)(vp->buf)=79;
	return 0;
}
inline int get_dgdot11Noise(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	if(item!=0)
		return -1;
	*(long *)(vp->buf)=0;
	return 0;
}
inline int get_dgdot11NumberOfAssociatedDevices(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	int len=0;
	char *pt=NULL;
	scfgmgr_get("wifi_present",&pt);
	
	if(!pt || *pt=='0'){
		if(pt) free(pt);	
		return -1;
	}
	
	free(pt);	
	
	system("/usr/sbin/rc list > /tmp/list");
	fp=fopen("/tmp/list","r");
	if(!fp)
		return -1;
	fseek(fp,0,SEEK_END);
	len=ftell(fp);
	fclose(fp);
	*(long *)(vp->buf)=len/18;
	return 0;
}
/* lan */
inline int get_dgNumberOfDHCPLeases(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	long len=0;
	fp=fopen("/var/udhcpd.leases","r");
	if(!fp){
		*(long *)(vp->buf)=0;	
		return 0;
	}
	fseek(fp,0,SEEK_END);
	len=ftell(fp);
	fclose(fp);
	*(long *)(vp->buf)=len/sizeof(struct leases_t);	
	return 0;
}

inline int get_dgLanDeviceMacAddr(snmp_pdu *pdu,netsnmp_variable_list *vp,int item)
{
	FILE *fp;
	struct leases_t lease;
	int len=0;
	char tmp[40]="";
	
	fp=fopen("/var/udhcpd.leases","r");
	if(!fp)
		return -1;
	if(item==0)
		return -1;	
	fseek(fp,0,SEEK_END);
	len=ftell(fp);
	
	if(item > len/sizeof(struct leases_t)){
		fclose(fp);
		return -1;
	}
	fseek(fp,(item-1)*sizeof(lease),0);
	fread(&lease,sizeof(lease),1,fp);
	fclose(fp);
	sprintf(tmp,"%02X:%02X:%02X:%02X:%02X:%02X"
			,lease.chaddr[0]
			,lease.chaddr[1]
			,lease.chaddr[2]
			,lease.chaddr[3]
			,lease.chaddr[4]
			,lease.chaddr[5]);
	vp->val_len=convertStr2PhysAddr(vp->buf,tmp);
	return 0;
}

static map_entry_t PrivteTab[] = {
    /* OID                                  LENGTH    	TYPE	  GET */
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 1},	13, ASN_INT, get_dgNMSAccessIndex},
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 2},	13, ASN_IP, get_dgNMSAccessIp},
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 3},	13, ASN_IP, get_dgNMSAccessIpMask},
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 4},	13, ASN_STR, get_dgNMSAccessCommunity},
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 5},	13, ASN_INT, get_dgNMSAccessType},
    { {SNMP_OID_NETGEAR, 3, 2, 1, 1, 1, 6},	13, ASN_INT, get_dgNMSAccessRowStatus},
    /* system */
    { {SNMP_OID_NETGEAR, 3, 2, 2, 1},		11, ASN_GAUGE, get_dgCPUUsage1min},
    { {SNMP_OID_NETGEAR, 3, 2, 2, 2},		11, ASN_GAUGE, get_dgCPUUsage5min},
    { {SNMP_OID_NETGEAR, 3, 2, 2, 3},		11, ASN_GAUGE, get_dgMEMUsage},
    { {SNMP_OID_NETGEAR, 3, 2, 2, 4},		11, ASN_GAUGE, get_dgFreeMEM},
    /* wireless */
    { {SNMP_OID_NETGEAR, 3, 2, 3, 1},		11, ASN_INT, get_dgdot11OperationalStatus},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 2},		11, ASN_STR, get_dgdot11DesiredSSID},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 3},		11, ASN_INT, get_dgdot11SSIDBroadcastOrNot},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 4},		11, ASN_INT, get_dgdot11Mode},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 5},		11, ASN_INT, get_dgdot11ChannelSelection},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 1},	13, ASN_INT, get_dgdot11NumberSupportedPowerLevels},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 2},	13, ASN_INT, get_dgdot11TXPowerLevel1},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 3},	13, ASN_INT, get_dgdot11TXPowerLevel2},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 4},	13, ASN_INT, get_dgdot11TXPowerLevel3},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 5},	13, ASN_INT, get_dgdot11TXPowerLevel4},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 6},	13, ASN_INT, get_dgdot11TXPowerLevel5},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 7},	13, ASN_INT, get_dgdot11TXPowerLevel6},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 8},	13, ASN_INT, get_dgdot11TXPowerLevel6},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1, 9},	13, ASN_INT, get_dgdot11TXPowerLevel6},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 6, 1,10},	13, ASN_INT, get_dgdot11CurrentTXPowerLevel},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 7},		11, ASN_INT, get_dgdot11Noise},
    { {SNMP_OID_NETGEAR, 3, 2, 3, 8},		11, ASN_INT, get_dgdot11NumberOfAssociatedDevices},
    /* lan */
    { {SNMP_OID_NETGEAR, 3, 2, 4, 1},		11, ASN_INT, get_dgNumberOfDHCPLeases},
    { {SNMP_OID_NETGEAR, 3, 2, 4, 2, 1, 1},	13, ASN_PHY, get_dgLanDeviceMacAddr},

    { {SNMP_OID_NETGEAR, 0, 0},                  0,       0, NULL},
};

int get_private_entry(snmp_pdu *pdu,netsnmp_variable_list *vp)
{
	map_entry_t *tb;
	int ret=-1;
	int result=0;
	int item=0;
	
	for(tb=PrivteTab;tb->nameLen >0 ;tb++){

		/* GET */
		if(pdu->command == SNMP_MSG_GET){
			result=memcmp(vp->name,tb->name,tb->nameLen * sizeof(oid));
			if(result!=0)
				continue;
			vp->type=tb->type;
			item=vp->name[vp->name_length-1];
			return (*(tb->getfun))(pdu,vp,item);
		}
		
		/* GET NEXT */
		if( vp->name_length == 7 ){
			vp->name[vp->name_length]=3;
			vp->name_length++;
		}

		if( vp->name_length == 8 ){
			vp->name[vp->name_length]=2;
			vp->name_length++;
		}
		if( vp->name_length == 9 ){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}

		if( vp->name_length == 10 ){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}
		if(vp->name_length == 11 && ((vp->name[9]==4 && vp->name[10]==2)
			       	|| (vp->name[9]==3 && vp->name[10]==6)
				|| vp->name[9]==1 )){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}	
		if(vp->name_length == 12 && ((vp->name[9]==4 && vp->name[10]==2)
			       	|| (vp->name[9]==3 && vp->name[10]==6)
				|| vp->name[9]==1 )){
			vp->name[vp->name_length]=1;
			vp->name_length++;
		}	

		result=memcmp(vp->name,tb->name,tb->nameLen * sizeof(oid));
		
		if(result != 0)
			continue;
			
		if(vp->name_length > tb->nameLen){
			item=vp->name[vp->name_length-1];
			
			if(item==0){
				/* get next entry in table */
				if((tb+1)->nameLen!=0){
					tb++;
					memcpy(vp->name,tb->name,(tb->nameLen)*sizeof(oid));
					vp->name_length=tb->nameLen;

					if((vp->name[9]==4 && vp->name[10]==2)
							||(vp->name[9]==3 && vp->name[10]==6) || vp->name[9]==1)
						vp->name[tb->nameLen] = 1;
					else
						vp->name[tb->nameLen] = 0;
					vp->name_length++;
					
				}else{
					return -1;
				}
			}else{
				vp->name[vp->name_length-1]+=1;
			}
			
		}else{
			/* those entry have instance */
			if((vp->name[9]==4 && vp->name[10]==2)
					||(vp->name[9]==3 && vp->name[10]==6) 
					|| vp->name[9]==1 ){
				vp->name[tb->nameLen] = 1;
			}else{
				vp->name[tb->nameLen] = 0;
			}
			vp->name_length++;
		}
			
			
		if(!tb->getfun)
			ret=-1;	
		else{
			item=vp->name[vp->name_length-1];
			vp->type=tb->type;
			ret=(*(tb->getfun))(pdu,vp,item);
		}
//		return ret;
		/* find nearest */
		while(ret==-1){
			if((tb+1)->nameLen!=0){
				tb++;
				memcpy(vp->name,tb->name,(tb->nameLen)*sizeof(oid));
				vp->name_length=tb->nameLen;
				if((vp->name[9]==4 && vp->name[10]==2) 
					||(vp->name[9]==3 && vp->name[10]==6) 
					|| vp->name[9]==1)
					item=1;
				else
					item=0;
				vp->name[tb->nameLen] = item;
				vp->name_length++;
				
			}else
				return -1;
			vp->type=tb->type;
			ret=(*(tb->getfun))(pdu,vp,item);
		}

		return ret;
		
	}
	return ret;
}

