#include <sys/types.h>

#include "include/snmp_asn1.h"
#include "snmpl_lib.h"

#define ASN_STR ASN_OCTET_STR
#define ASN_OID ASN_OBJECT_ID
#define ASN_INT ASN_INTEGER
#define ASN_TIME ASN_TIMETICKS
#define ASN_CNT ASN_COUNTER
#define ASN_IP ASN_IPADDRESS
#define ASN_PHY ASN_PHYSADDR

struct map_entry OidToKeyTbl[] = {
    { "snmpcm", "settings/system",      0, ASN_STR, 2, {1, 1} },
    { "snmpcm", "settings/system",      1, ASN_OID, 2, {1, 2} },
    { "snmpcm", "settings/system",      2, ASN_TIME, 2, {1, 3} },
    { "snmpcm", "settings/system",      3, ASN_STR, 2, {1, 4} },
    { "snmpcm", "settings/system",      4, ASN_STR, 2, {1, 5} },
    { "snmpcm", "settings/system",      5, ASN_STR, 2, {1, 6} },
    { "snmpcm", "settings/system",      6, ASN_INT, 2, {1, 7} },
    { "snmpcm", "status/sysorchange",   0, ASN_TIME, 2, {1, 8} },
    
    { "snmpcm", "status/sysortbl",      1, ASN_OID, 4, {1, 9, 1, 2} },
    { "snmpcm", "status/sysortbl",      2, ASN_STR, 4, {1, 9, 1, 3} },
    { "snmpcm", "status/sysortbl",      3, ASN_TIME, 4, {1, 9, 1, 4} },
    
    { "snmpcm", "status/ifnum",         0, ASN_INT, 2, {2, 1} },
    
    { "snmpcm", "status/ifgeneral1",    0, ASN_INT, 4, {2, 2, 1, 1} },
    { "snmpcm", "status/ifgeneral1",    1, ASN_STR, 4, {2, 2, 1, 2} },
    { "snmpcm", "status/ifgeneral1",    2, ASN_INT, 4, {2, 2, 1, 3} },
    { "snmpcm", "status/ifgeneral1",    3, ASN_INT, 4, {2, 2, 1, 4} },
    { "snmpcm", "status/ifgeneral1",    4, ASN_GAUGE, 4, {2, 2, 1, 5} },
    { "snmpcm", "status/ifgeneral1",    5, ASN_PHY, 4, {2, 2, 1, 6} },
    { "snmpcm", "status/ifgeneral2",    1, ASN_INT, 4, {2, 2, 1, 7} },
    { "snmpcm", "status/ifgeneral2",    2, ASN_INT, 4, {2, 2, 1, 8} },
    { "snmpcm", "status/ifinstats",     1, ASN_CNT, 4, {2, 2, 1, 10} },
    { "snmpcm", "status/ifinstats",     2, ASN_CNT, 4, {2, 2, 1, 11} },
    { "snmpcm", "status/ifinstats",     3, ASN_CNT, 4, {2, 2, 1, 13} },
    { "snmpcm", "status/ifinstats",     4, ASN_CNT, 4, {2, 2, 1, 14} },
    { "snmpcm", "status/ifoutstats",    1, ASN_CNT, 4, {2, 2, 1, 16} },
    { "snmpcm", "status/ifoutstats",    2, ASN_CNT, 4, {2, 2, 1, 17} },
    { "snmpcm", "status/ifoutstats",    3, ASN_CNT, 4, {2, 2, 1, 19} },
    { "snmpcm", "status/ifoutstats",    4, ASN_CNT, 4, {2, 2, 1, 20} },
    { "snmpcm", "status/ifoutstats",    5, ASN_GAUGE, 4, {2, 2, 1, 21} },
    { "snmpcm", "status/ifgeneral2",    3, ASN_OID, 4, {2, 2, 1, 22} },
		  							  	 			 				    					      						     	 
    { "snmpcm", "status/ipgeneral",     0, ASN_INT, 2, {4, 1} },
    { "snmpcm", "status/ipgeneral",     1, ASN_INT, 2, {4, 2} },
    { "snmpcm", "status/ipinstats",     0, ASN_CNT, 2, {4, 3} },
    { "snmpcm", "status/ipinstats",     1, ASN_CNT, 2, {4, 4} },
    { "snmpcm", "status/ipinstats",     2, ASN_CNT, 2, {4, 5} },
    { "snmpcm", "status/ipinstats",     3, ASN_CNT, 2, {4, 6} },
    { "snmpcm", "status/ipinstats",     4, ASN_CNT, 2, {4, 7} },
    { "snmpcm", "status/ipinstats",     5, ASN_CNT, 2, {4, 8} },
    { "snmpcm", "status/ipinstats",     6, ASN_CNT, 2, {4, 9} },
    { "snmpcm", "status/ipoutstats",    0, ASN_CNT, 2, {4, 10} },
    { "snmpcm", "status/ipoutstats",    1, ASN_CNT, 2, {4, 11} },
    { "snmpcm", "status/ipoutstats",    2, ASN_CNT, 2, {4, 12} },
    { "snmpcm", "status/ipreasmstats",  0, ASN_INT, 2, {4, 13} },
    { "snmpcm", "status/ipreasmstats",  1, ASN_CNT, 2, {4, 14} },
    { "snmpcm", "status/ipreasmstats",  2, ASN_CNT, 2, {4, 15} },
    { "snmpcm", "status/ipreasmstats",  3, ASN_CNT, 2, {4, 16} },
    { "snmpcm", "status/ipreasmstats",  4, ASN_CNT, 2, {4, 17} },
    { "snmpcm", "status/ipreasmstats",  5, ASN_CNT, 2, {4, 18} },
    { "snmpcm", "status/ipreasmstats",  6, ASN_CNT, 2, {4, 19} },
                                         	                  
    { "snmpcm", "status/ipaddrtbl",     0, ASN_IP, 4, {4, 20, 1, 1} },
    { "snmpcm", "status/ipaddrtbl",     1, ASN_INT, 4, {4, 20, 1, 2} },
    { "snmpcm", "status/ipaddrtbl",     2, ASN_IP, 4, {4, 20, 1, 3} },
    { "snmpcm", "status/ipaddrtbl",     3, ASN_INT, 4, {4, 20, 1, 4} },
		  							  	 			 				    					      						     	 
    { "snmpcm", "status/iprttbl1",       0, ASN_IP, 4, {4, 21, 1, 1} },
    { "snmpcm", "status/iprttbl1",       1, ASN_INT, 4, {4, 21, 1, 2} },
    { "snmpcm", "status/iprttbl1",       2, ASN_INT, 4, {4, 21, 1, 3} },
    { "snmpcm", "status/iprttbl1",       3, ASN_IP, 4, {4, 21, 1, 7} },
    { "snmpcm", "status/iprttbl1",       4, ASN_INT, 4, {4, 21, 1, 8} },
    { "snmpcm", "status/iprttbl2",       1, ASN_INT, 4, {4, 21, 1, 9} },
    { "snmpcm", "status/iprttbl2",       2, ASN_INT, 4, {4, 21, 1, 10} },
    { "snmpcm", "status/iprttbl2",       3, ASN_IP, 4, {4, 21, 1, 11} },
    { "snmpcm", "status/iprttbl2",       4, ASN_OID, 4, {4, 21, 1, 13} },
		  							  	 			 				    					      						     	 
    { "snmpcm", "status/ipn2mtbl",      0, ASN_INT, 4, {4, 22, 1, 1} },
    { "snmpcm", "status/ipn2mtbl",      2, ASN_PHY, 4, {4, 22, 1, 2} },
    { "snmpcm", "status/ipn2mtbl",      1, ASN_IP, 4, {4, 22, 1, 3} },
    { "snmpcm", "status/ipn2mtbl",      3, ASN_INT, 4, {4, 22, 1, 4} },
                                         	                  
    { "snmpcm", "status/ipgeneral",     2, ASN_CNT, 2, {4, 23} },
		  							  	 			 				    					      						     	 
    { "snmpcm", "status/icmpinstats1",  0, ASN_CNT, 2, {5, 1} },
    { "snmpcm", "status/icmpinstats1",  1, ASN_CNT, 2, {5, 2} },
    { "snmpcm", "status/icmpinstats1",  2, ASN_CNT, 2, {5, 3} },
    { "snmpcm", "status/icmpinstats1",  3, ASN_CNT, 2, {5, 4} },
    { "snmpcm", "status/icmpinstats1",  4, ASN_CNT, 2, {5, 5} },
    { "snmpcm", "status/icmpinstats1",  5, ASN_CNT, 2, {5, 6} },
    { "snmpcm", "status/icmpinstats1",  6, ASN_CNT, 2, {5, 7} },
    { "snmpcm", "status/icmpinstats2",  0, ASN_CNT, 2, {5, 8} },
    { "snmpcm", "status/icmpinstats2",  1, ASN_CNT, 2, {5, 9} },
    { "snmpcm", "status/icmpinstats2",  2, ASN_CNT, 2, {5, 10} },
    { "snmpcm", "status/icmpinstats2",  3, ASN_CNT, 2, {5, 11} },
    { "snmpcm", "status/icmpinstats2",  4, ASN_CNT, 2, {5, 12} },
    { "snmpcm", "status/icmpinstats2",  5, ASN_CNT, 2, {5, 13} },
    { "snmpcm", "status/icmpoutstats1", 0, ASN_CNT, 2, {5, 14} },
    { "snmpcm", "status/icmpoutstats1", 1, ASN_CNT, 2, {5, 15} },
    { "snmpcm", "status/icmpoutstats1", 2, ASN_CNT, 2, {5, 16} },
    { "snmpcm", "status/icmpoutstats1", 3, ASN_CNT, 2, {5, 17} },
    { "snmpcm", "status/icmpoutstats1", 4, ASN_CNT, 2, {5, 18} },
    { "snmpcm", "status/icmpoutstats1", 5, ASN_CNT, 2, {5, 19} },
    { "snmpcm", "status/icmpoutstats1", 6, ASN_CNT, 2, {5, 20} },
    { "snmpcm", "status/icmpoutstats2", 0, ASN_CNT, 2, {5, 21} },
    { "snmpcm", "status/icmpoutstats2", 1, ASN_CNT, 2, {5, 22} },
    { "snmpcm", "status/icmpoutstats2", 2, ASN_CNT, 2, {5, 23} },
    { "snmpcm", "status/icmpoutstats2", 3, ASN_CNT, 2, {5, 24} },
    { "snmpcm", "status/icmpoutstats2", 4, ASN_CNT, 2, {5, 25} },
    { "snmpcm", "status/icmpoutstats2", 5, ASN_CNT, 2, {5, 26} },
    { NULL,     NULL,                   0,       0, 0, {0,  0} }
};


struct map_entry  *pKeyTbl = OidToKeyTbl;
