

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <arpa/inet.h>
#include <sys/utsname.h>
#include <time.h>
#include <net/route.h>
#include <stdlib.h>

#include "cm_module.h"
#include "cm_event.h"
#include "cm_message.h"
#include "cm_logger.h"

#include "snmp.h"
#include "snmp_mib.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif /* DMALLOC */

#ifndef MAX_SUBID               /* temporary - duplicate definition protection */
#ifndef EIGHTBIT_SUBIDS
typedef u_long  oid;
#define MAX_SUBID   0xFFFFFFFF
#else
typedef u_char  oid;
#define MAX_SUBID   0xFF
#endif
#endif

#define SNMP_ERR_NOSUCHNAME             (2)

#define MAX_COMMUNITY_NUM   3
#define SNMP_LISTEN_INET_TIMEOUT "30"
#define SNMP_DEF_NAME	"sptcrouter"
#define SNMP_DEF_LOCATION	"germantown,md,usa"
#define SNMP_DEF_CONTACT	"support@telogy.com"
#define SNMP_DEF_OID	"1.3.6.1.4.1.294"
#define SNMP_DEF_EMPTY ""

#define IP_STATS_LINE	"Ip: %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu"
#define ICMP_STATS_LINE	"Icmp: %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu"

#define IP_STATS_PREFIX_LEN	  4
#define ICMP_STATS_PREFIX_LEN 6

int Interface_Scan_Get_Count(void);

typedef struct community_s
{
    char *name;
    char *acsRight;
} COMMUNITY;

char *default_community = "public";
char *default_acsRight = "ro";

typedef struct snmp_priv_s
{
    COMMUNITY  community[MAX_COMMUNITY_NUM];
    char       *idleTimeout;
    char 			 *sysName;
    char 			 *sysLocation;
    char		   *sysContact;
    char 		   *sysOID;
}SNMP_PRIV;

static int read_ip_stat(void *mibPtr, int type)
{
    FILE           *in = fopen("/proc/net/snmp", "r");
    char            line[1024];
    struct ip_mib *ip_mib;
    
    if (!in)
        return -1;

    if (type == IP_STATS)
    {
         ip_mib = (struct ip_mib *) mibPtr;
        while (line == fgets(line, sizeof(line), in)) {
            if (!strncmp(line, IP_STATS_LINE,
                         IP_STATS_PREFIX_LEN)) {
                sscanf(line, IP_STATS_LINE,
                       &(ip_mib->ipForwarding),
                       &(ip_mib->ipDefaultTTL),
                       &(ip_mib->ipInReceives),
                       &(ip_mib->ipInHdrErrors),
                       &(ip_mib->ipInAddrErrors),
                       &(ip_mib->ipForwDatagrams),
                       &(ip_mib->ipInUnknownProtos),
                       &(ip_mib->ipInDiscards),
                       &(ip_mib->ipInDelivers),
                       &(ip_mib->ipOutRequests),
                       &(ip_mib->ipOutDiscards),
                       &(ip_mib->ipOutNoRoutes),
                       &(ip_mib->ipReasmTimeout),
                       &(ip_mib->ipReasmReqds),
                       &(ip_mib->ipReasmOKs),
                       &(ip_mib->ipReasmFails),
                       &(ip_mib->ipFragOKs),
                       &(ip_mib->ipFragFails),
                       &(ip_mib->ipFragCreates));
                       ip_mib->ipRoutingDiscards = 0;        /* XXX */
            }
        }
        /*
         * Tweak illegal values:
         *
         * valid values for ipForwarding are 1 == yup, 2 == nope
         * a 0 is forbidden, so patch:
         */
        if (!ip_mib->ipForwarding)
            ip_mib->ipForwarding = 2;   
    }
    
    else if (type == ICMP_STATS)
    {
        struct icmp_mib *icmpstat = (struct icmp_mib *) mibPtr;
        
        while (line == fgets(line, sizeof(line), in))
        {
            if (!strncmp(line, ICMP_STATS_LINE,
                         ICMP_STATS_PREFIX_LEN))
            {
                sscanf(line, ICMP_STATS_LINE,
                       &icmpstat->icmpInMsgs,
                       &icmpstat->icmpInErrors,
                       &icmpstat->icmpInDestUnreachs,
                       &icmpstat->icmpInTimeExcds,
                       &icmpstat->icmpInParmProbs,
                       &icmpstat->icmpInSrcQuenchs,
                       &icmpstat->icmpInRedirects,
                       &icmpstat->icmpInEchos,
                       &icmpstat->icmpInEchoReps,
                       &icmpstat->icmpInTimestamps,
                       &icmpstat->icmpInTimestampReps,
                       &icmpstat->icmpInAddrMasks,
                       &icmpstat->icmpInAddrMaskReps,
                       &icmpstat->icmpOutMsgs,
                       &icmpstat->icmpOutErrors,
                       &icmpstat->icmpOutDestUnreachs,
                       &icmpstat->icmpOutTimeExcds,
                       &icmpstat->icmpOutParmProbs,
                       &icmpstat->icmpOutSrcQuenchs,
                       &icmpstat->icmpOutRedirects,
                       &icmpstat->icmpOutEchos,
                       &icmpstat->icmpOutEchoReps,
                       &icmpstat->icmpOutTimestamps,
                       &icmpstat->icmpOutTimestampReps,
                       &icmpstat->icmpOutAddrMasks,
                       &icmpstat->icmpOutAddrMaskReps);
            }
        }
    }

    fclose(in);
    return 0;
}
/*
static int snmp_load_community(struct cm_module *module)
{    
    SNMP_PRIV *priv = module->priv;
    
    // update the local memory with new values (all 3)
    // ?? any faster way of getting them
    priv->community[0].name =
        tree_getKeyContent(module->config,
                           "community/entry1/name");
    priv->community[0].acsRight =
        tree_getKeyContent(module->config,
                           "community/entry1/access");
    priv->community[1].name =
        tree_getKeyContent(module->config,
                           "community/entry2/name");
    priv->community[1].acsRight =
        tree_getKeyContent(module->config,
                           "community/entry2/access");
    priv->community[2].name =
        tree_getKeyContent(module->config,
                           "community/entry3/name");
    priv->community[2].acsRight =
        tree_getKeyContent(module->config,
                           "community/entry3/access");
    priv->idleTimeout = 
        tree_getKeyContent(module->config,
                           "system/idletime");
    return 0;
}  */

static int snmp_load_community(struct cm_module *module)
{
    SNMP_PRIV *priv = module->priv;
    char *ptr,qrystr[30];
    int i = 0;

    for( i = 0; i < MAX_COMMUNITY_NUM; i++ )
    {
			sprintf(qrystr,"community/entry%d/name",i);
      ptr = tree_getKeyContent(module->config,qrystr);
    	priv->community[i].name = ptr ? ptr:SNMP_DEF_EMPTY;
			sprintf(qrystr,"community/entry%d/access",i);
      ptr = tree_getKeyContent(module->config,qrystr);
    	priv->community[i].acsRight = ptr ? ptr:SNMP_DEF_EMPTY;
    }

		sprintf(qrystr,"system/idletime");
    ptr = tree_getKeyContent(module->config,qrystr);
    priv->idleTimeout = ptr ? ptr:SNMP_LISTEN_INET_TIMEOUT;

		sprintf(qrystr,"system/sysname");
    ptr = tree_getKeyContent(module->config,qrystr);
    priv->sysName = ptr ? ptr:SNMP_DEF_NAME;

		sprintf(qrystr,"system/syslocation");
    ptr = tree_getKeyContent(module->config,qrystr);
    priv->sysLocation = ptr ? ptr:SNMP_DEF_LOCATION;

		sprintf(qrystr,"system/syscontact");
    ptr = tree_getKeyContent(module->config,qrystr);
    priv->sysContact = ptr ? ptr:SNMP_DEF_CONTACT;

		sprintf(qrystr,"system/sysoid");
    ptr = tree_getKeyContent(module->config,qrystr);
    priv->sysOID = ptr ? ptr:SNMP_DEF_OID;

    return 0;
}

static int snmp_reject(struct cm_module *module)
{
    tree_flush(module->temp);

    return RET_MSG_OK;
}

static int snmp_apply(struct cm_module *module)
{
    tree_move(module->config, module->temp);
    snmp_load_community(module);

    // Notify SNMP agent for community change.
    msgr_sendReport("snmpcm", "snmpd", "ReadInfo", module->id,
                    "CommunityChang", strlen("CommunityChang"));
    return RET_MSG_OK;
}


static int snmp_set(struct cm_module *module,
                    char             *key,
                    char             *value)
{
    struct key_parts kp;
#if 0
    char *curval;
#endif

    if(key_split(key, &kp) < 0)
        return RET_MSG_BAD_KEY; // key malformed

    while (kp.keytype != KEY_SETTINGS)
    {
        if (key_split(key, &kp) < 0)
            return RET_MSG_BAD_KEY;
    }

    if((kp.keytype != KEY_SETTINGS) &&
       (kp.keytype != KEY_UNKNOWN))
        return RET_MSG_BAD_KEY; // no such key

#if 0
    if(!value)
        return RET_MSG_OK; // IGNORE FOR NOW
#endif
#if 0
    /* If lan or wan access is going to change broadcast an event */
    if(!strcmp(kp.key,"system/lanaccess"))
		{
			 curval = tree_getKeyContent(module->config, kp.key);
       if( (curval != NULL) && (strcmp(value,curval)))
    		event_broadcast(module, "flan", EVT_REPORT_FW_SNMP,
                        (void *)(atoi(value)));
		}
    else if(!strcmp(kp.key,"system/wanaccess"))
		{
			 curval = tree_getKeyContent(module->config, kp.key);
       if( (curval != NULL) && (strcmp(value,curval)))
    		event_broadcast(module, "fwan",EVT_REPORT_FW_SNMP,
                        (void *)(atoi(value)));
		}
#endif

    tree_setKeyContent(module->temp, kp.key, value);

    return RET_MSG_OK;
}

// return all 3 community info
// (Row 1)   col0  col1    col2  col3    col4  col5    col6
// Community name1 access1 name2 access2 name3 access3 IdleTime

static int snmp_sendCommunityInfo(struct cm_module *module,
                                  struct msg_base  *msg)
{
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    SNMP_PRIV *priv = module->priv;
    
	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 7,
                        priv->community[0].name,
                        priv->community[0].acsRight,
                        priv->community[1].name,
                        priv->community[1].acsRight,
                        priv->community[2].name,
                        priv->community[2].acsRight,
                        priv->idleTimeout);
    msg_send(respMsg);
    
    return 0;
}

static int snmp_sendCommunityEntryInfo(struct cm_module *module,
                                  struct msg_base  *msg)
{
		int i = 0;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    SNMP_PRIV *priv = module->priv;

		respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    for( i = 0; i < MAX_COMMUNITY_NUM; i++)
		{
    	msgr_addTransValues(respMsg, 2,
                        priv->community[i].name,
                        priv->community[i].acsRight);
		}
    msg_send(respMsg);

    return 0;
}

static int snmp_sendSystemIdletime(struct cm_module *module,
                                  struct msg_base  *msg)
{
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    SNMP_PRIV *priv = module->priv;

		respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 1,priv->idleTimeout);
    msg_send(respMsg);
    return 0;
}

static int snmp_sendSystemElements(struct cm_module *module,
                               struct msg_base  * msg)
{
    SNMP_PRIV *priv;
    char *value;
    struct key_parts kp;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);

    priv = module->priv;

	  respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    if(key_split(key, &kp) < 0)
        return RET_MSG_BAD_KEY; // key malformed

    value = tree_getKeyContent(module->config,kp.key);
    value = value ? value:SNMP_DEF_EMPTY;
    msgr_addTransValues(respMsg,1,value);
    msg_send(respMsg);
    return 0;
}

// return all 7 system info
// (Row 1) col1     col2   col3      col4       col5    col6   col7
//         sysDescr:sysOID:sysUpTime:sysContact:sysName:sysLoc:sysService

#define SYS_STRING_LEN	256
#define MAX_OID_LEN	    128 /* max subid's in an oid */
extern long startup_time;

static int snmp_sendSystemInfo(struct cm_module *module,
                               struct msg_base  * msg)
{
    struct utsname  utsName;
    char version_descr[SYS_STRING_LEN] = "unknown";
    long  uptime;
    char tmp[10]="";
    SNMP_PRIV *priv;

    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);

    priv = module->priv;

    uname(&utsName);
    snprintf(version_descr, sizeof(version_descr),
            "%s %s %s %s %s", utsName.sysname,
            utsName.nodename, utsName.release, utsName.version,
            utsName.machine);
    version_descr[ sizeof(version_descr)-1 ] = 0;

    uptime = (time(0)-startup_time)*100;
    snprintf(tmp, sizeof(tmp), "%lu", uptime);

	  respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 7,
                        version_descr,
                        priv->sysOID,
                        tmp, priv->sysContact, priv->sysName,
                        priv->sysLocation, "72");
    msg_send(respMsg);
    return 0;
}

static void snmp_sendError(struct cm_module *module,
                           struct msg_base  *msg,
                           int err)
{
    struct msg_base *respMsg;
    char tmp[10]="";
    
    snprintf(tmp, sizeof(tmp), "%d", err);
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, "ERROR");
    msgr_addTransValueSingle(respMsg, tmp);
    msg_send(respMsg);
}

static void snmp_sendReqNext(struct cm_module *module,
                             struct msg_base  *msg,
                             int sameTable, char *newIndex)
{    
    struct msg_base *respMsg;
	char tmp1[10]="";

    snprintf(tmp1, sizeof(tmp1), "%d", sameTable);
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, "NEXT");
    msgr_addTransValues(respMsg, 2, tmp1, newIndex);
    msg_send(respMsg);
}


long sysOR_lastchange;

static int snmp_sendSysOrChange(struct cm_module *module,
                                struct msg_base  * msg)
{
    // return in the following format
    // sysORLastChange
    long          res;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp[10]="";

    res = (sysOR_lastchange - startup_time) * 100;
        
    snprintf(tmp, sizeof(tmp), "%lu", res);

    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValueSingle(respMsg, tmp);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendSysOrTblInfo(struct cm_module *module,
                                 struct msg_base  * msg,
                                 struct sysORTable *pOrIndx,
                                 char *newIndex)
{
    // return 3 entries one in each row and
    // index:sysORID:sysORDesc:sysORUpTime
    
    struct msg_base *respMsg;
    int i, len;
    char *key = msg_getTransKey(msg);
    char tmp[10]="";
    char tmp1[40]="";
    char tmp2[10]="";
    char *dest;
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    
    // snprintf(tmp1, sizeof(tmp1), "%u", );
    snprintf(tmp2, sizeof(tmp2), "%lu",
             pOrIndx->OR_uptime - startup_time);
    
    // handle OID conversion from integer to string
    dest = tmp1;
    for (i = 0; i < pOrIndx->OR_oidlen; i++)
    {
        len = sprintf(tmp,"%u", (int)(*(pOrIndx->OR_oid+i)));
        memmove(dest, tmp, len);
        dest += len;
        *dest = '.';
        dest++;
    }
    *(--dest) = 0;
    
    msgr_addTransValues(respMsg, 4,
                        newIndex, tmp1,
                        pOrIndx->OR_descr, tmp2);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIfNum(struct cm_module *module,
                          struct msg_base  * msg)
{
    // return in the following format
    // ifNum:     value

    int       count;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp[10]="";

    count = Interface_Scan_Get_Count();
        
    snprintf(tmp, sizeof(tmp), "%d", count);

    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValueSingle(respMsg, tmp);
    
    msg_send(respMsg);
    return 0;
}

static conf_if_list *conf_list;

static int snmp_sendIfGenInfo1(struct cm_module *module,
                               struct msg_base  * msg, int intf)
{
    // return in the following format
    // index:ifDescr:ifType:ifMtu:ifSpeed:ifPhyAddr
    static char     Name[16]="";
    static struct ifnet ifnet;
    static struct in_ifaddr in_ifaddr;
    conf_if_list   *if_ptr = conf_list;

    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[10]="";
    char tmp2[10]="";
    char tmp3[10]="";
    char tmp4[10]="";
    char tmp5[20]="";
    extern struct ifnet *saveifnetaddr; 
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    Interface_Scan_By_Index(intf, Name, &ifnet, &in_ifaddr);

    if ((*saveifnetaddr->if_hwaddr == 0) &&
        (*(saveifnetaddr->if_hwaddr+1) == 0) &&
        (*(saveifnetaddr->if_hwaddr+2) == 0) &&
        (*(saveifnetaddr->if_hwaddr+3) == 0) &&
        (*(saveifnetaddr->if_hwaddr+4) == 0) &&
        (*(saveifnetaddr->if_hwaddr+5) == 0))
    {
        snprintf(tmp5, sizeof(tmp5), " ");
    }
    else
    {
        sprintf(tmp5, "%u:%u:%u:%u:%u:%u",
                (*saveifnetaddr->if_hwaddr) & 0xff,
                (*(saveifnetaddr->if_hwaddr+1)) & 0xff,
                (*(saveifnetaddr->if_hwaddr+2)) & 0xff, 
		(*(saveifnetaddr->if_hwaddr+3)) & 0xff,
                (*(saveifnetaddr->if_hwaddr+4)) & 0xff, 
		(*(saveifnetaddr->if_hwaddr+5)) & 0xff);
    }
    
    snprintf(tmp1, sizeof(tmp1), "%u", intf);

    if (if_ptr)
    {
        snprintf(tmp2, sizeof(tmp2), "%u", if_ptr->type);
        snprintf(tmp4, sizeof(tmp4), "%lu", if_ptr->speed);
    }
    else
    {
        snprintf(tmp2, sizeof(tmp2), "%u", ifnet.if_type);
        snprintf(tmp4, sizeof(tmp4), "%lu", ifnet.if_speed);
    }
    
    snprintf(tmp3, sizeof(tmp3), "%u", ifnet.if_mtu);
    
    
    msgr_addTransValues(respMsg, 7,
                        tmp1, Name, tmp2, tmp3, tmp4, tmp5, " ");
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIfGenInfo2(struct cm_module *module,
                               struct msg_base  * msg, int intf)
{
    // return in the following format
    // index:ifAdmin:ifOper:ifSpecific
    static char     Name[16]="";
    static struct ifnet ifnet;
    static struct in_ifaddr in_ifaddr;
    
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    char tmp[10]="";
    char tmp1[10]="";
    char tmp2[10]="";
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    Interface_Scan_By_Index(intf, Name, &ifnet, &in_ifaddr);

    snprintf(tmp, sizeof(tmp), "%u", intf);
    snprintf(tmp1, sizeof(tmp1), "%u",
             ifnet.if_flags & IFF_UP ? 1 : 2);
    snprintf(tmp2, sizeof(tmp2), "%u",
             ifnet.if_flags & IFF_RUNNING ? 1 : 2);

    msgr_addTransValues(respMsg, 4,
                        tmp, tmp1, tmp2, "0.0");
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIfInStats(struct cm_module *module,
                              struct msg_base  * msg, int intf)
{
    // return in the following format.
    // index:ifInOcts:ifInUcastPkt:ifInDiscards:ifInErrs
    static char     Name[16]="";
    static struct ifnet ifnet;
    static struct in_ifaddr in_ifaddr;
    
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[10]="";
    char tmp2[10]="";
    char tmp3[10]="";
    char tmp4[10]="";
    char tmp5[10]="";

    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    Interface_Scan_By_Index(intf, Name, &ifnet, &in_ifaddr);

    snprintf(tmp1, sizeof(tmp1), "%u", intf);
    snprintf(tmp2, sizeof(tmp2), "%lu", ifnet.if_ibytes);
    snprintf(tmp3, sizeof(tmp3), "%lu", ifnet.if_ipackets);
    snprintf(tmp4, sizeof(tmp4), "%lu", ifnet.if_iqdrops);
    snprintf(tmp5, sizeof(tmp5), "%lu", ifnet.if_ierrors);

    msgr_addTransValues(respMsg, 5,
                        tmp1, tmp2, tmp3, tmp4, tmp5);
    msg_send(respMsg);
   
    return 0;
}

static int snmp_sendIfOutStats(struct cm_module *module,
                               struct msg_base  * msg, int intf)
{
    // return in the following format.
    // index:ifOutOcts:ifOutUcastPkt:ifOutDiscards:
    // ifOutErrs:ifOutQlen
    
    static char     Name[16]="";
    static struct ifnet ifnet;
    static struct in_ifaddr in_ifaddr;
    
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[10]="";
    char tmp2[10]="";
    char tmp3[10]="";
    char tmp4[10]="";
    char tmp5[10]="";
    char tmp6[10]="";

    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    Interface_Scan_By_Index(intf, Name, &ifnet, &in_ifaddr);

    snprintf(tmp1, sizeof(tmp1), "%u", intf);
    snprintf(tmp2, sizeof(tmp2), "%lu", ifnet.if_obytes);
    snprintf(tmp3, sizeof(tmp3), "%lu", ifnet.if_opackets);
    snprintf(tmp4, sizeof(tmp4), "%u", ifnet.if_snd.ifq_drops);
    snprintf(tmp5, sizeof(tmp5), "%lu", ifnet.if_oerrors);
    snprintf(tmp6, sizeof(tmp6), "%u", ifnet.if_snd.ifq_len);

    msgr_addTransValues(respMsg, 6,
                        tmp1, tmp2, tmp3,
                        tmp4, tmp5, tmp6);
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIpGenInfo(struct cm_module *module,
                              struct msg_base  * msg)
{
    // return in the following format
    // (Row 1)
    // ipForwarding:ipDefaultTTL:ipRouteDiscards
    
    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct ip_mib ipstat;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";

    memset((char *) &ipstat, (0), sizeof(struct ip_mib));
    ret_value = read_ip_stat((void*)(&ipstat), IP_STATS);
    
	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    snprintf(tmp1, sizeof(tmp1), "%lu", ipstat.ipForwarding);
    snprintf(tmp2, sizeof(tmp2), "%lu", ipstat.ipDefaultTTL);
    snprintf(tmp3, sizeof(tmp3), "%lu", ipstat.ipRoutingDiscards);

    msgr_addTransValues(respMsg, 3, tmp1, tmp2, tmp3);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIpInStat(struct cm_module *module,
                             struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1)
    // ipInRecvs:ipInHdrErrs:ipInAddrErrs:ipForDatagram:
    // ipInUnknownPro:ipInDiscards:ipInDelivers
    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct ip_mib ipstat;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";
	char tmp7[10]="";

    memset((char *) &ipstat, (0), sizeof(struct ip_mib));
    ret_value = read_ip_stat((void*)(&ipstat), IP_STATS);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    snprintf(tmp1, sizeof(tmp1), "%lu", ipstat.ipInReceives);
    snprintf(tmp2, sizeof(tmp2), "%lu", ipstat.ipInHdrErrors);
    snprintf(tmp3, sizeof(tmp3), "%lu", ipstat.ipInAddrErrors);
    snprintf(tmp4, sizeof(tmp4), "%lu", ipstat.ipForwDatagrams);
    snprintf(tmp5, sizeof(tmp5), "%lu", ipstat.ipInUnknownProtos);
    snprintf(tmp6, sizeof(tmp6), "%lu", ipstat.ipInDiscards);
    snprintf(tmp7, sizeof(tmp7), "%lu", ipstat.ipInDelivers);
    msgr_addTransValues(respMsg, 7,
                        tmp1, tmp2, tmp3, tmp4,
                        tmp5, tmp6, tmp7);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIpOutStat(struct cm_module *module,
                              struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1)
    // ipOutRequests:ipOutDiscards:ipOutNoRoutes

    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct ip_mib ipstat;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";

    memset((char *) &ipstat, (0), sizeof(struct ip_mib));
    ret_value = read_ip_stat((void*)(&ipstat), IP_STATS);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    snprintf(tmp1, sizeof(tmp1), "%lu", ipstat.ipOutRequests);
    snprintf(tmp2, sizeof(tmp2), "%lu", ipstat.ipOutDiscards);
    snprintf(tmp3, sizeof(tmp3), "%lu", ipstat.ipOutNoRoutes);
    msgr_addTransValues(respMsg, 3, tmp1, tmp2, tmp3);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIpReasmStat(struct cm_module *module,
                                struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1)
    // ipReasmTimeout:ipReasmReqds:ipReasmOKs:ipReasmFails:
    // ipFragOKs:ipFragsCreates
    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct ip_mib ipstat;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";
	char tmp7[10]="";

    memset((char *) &ipstat, (0), sizeof(struct ip_mib));
    ret_value = read_ip_stat((void*)(&ipstat), IP_STATS);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    snprintf(tmp1, sizeof(tmp1), "%lu", ipstat.ipReasmTimeout);
    snprintf(tmp2, sizeof(tmp2), "%lu", ipstat.ipReasmReqds);
    snprintf(tmp3, sizeof(tmp3), "%lu", ipstat.ipReasmOKs);
    snprintf(tmp4, sizeof(tmp4), "%lu", ipstat.ipReasmFails);
    snprintf(tmp5, sizeof(tmp5), "%lu", ipstat.ipFragOKs);
    snprintf(tmp6, sizeof(tmp6), "%lu", ipstat.ipFragFails);
    snprintf(tmp7, sizeof(tmp7), "%lu", ipstat.ipFragCreates);
    msgr_addTransValues(respMsg, 7,
                        tmp1, tmp2, tmp3, tmp4,
                        tmp5, tmp6, tmp7);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIpAddrTblInfo(struct cm_module *module,
                                  struct msg_base  * msg,
                                  char *ipAddrIndx,
                                  int *lowinterface,
                                  struct ifnet *lowin_ifnet)
{
    // return in the following format.
    // ipAdEntAddr:ipAdEntIfIndx:ipAdEntNetMask:ipAdEntBcastAddr
    
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[16]="";
    char tmp2[10]="";
    char tmp3[40]="";
    char tmp4[10]="";
    
    long bcastAddr;

    long netMask;

    long ipAddr;
    long ip[4];
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    netMask = ((struct sockaddr_in *) &lowin_ifnet->
               ia_subnetmask)->sin_addr.s_addr;
    bcastAddr = ntohl(((struct sockaddr_in *)
                       &lowin_ifnet->ifu_broadaddr)->
                      sin_addr.s_addr) & 1;

    sscanf(ipAddrIndx, "%lu.%lu.%lu.%lu",
           &ip[0], &ip[1], &ip[2], &ip[3]);
    ipAddr = (ip[0]) | (ip[1] << 8) | (ip[2] << 16) | (ip[3] << 24);
        

    snprintf(tmp1, sizeof(tmp1), "%lu", ipAddr);
    snprintf(tmp2, sizeof(tmp2), "%u", *lowinterface);
    snprintf(tmp3, sizeof(tmp3), "%lu",  netMask);
    snprintf(tmp4, sizeof(tmp4), "%lu", bcastAddr);
    
    msgr_addTransValues(respMsg, 4,
                        tmp1, tmp2,
                        tmp3, tmp4);
    msg_send(respMsg);
    return 0;
}

extern struct rtentry **rthead;

static int snmp_sendIpRtTblInfo1(struct cm_module *module,
                                 struct msg_base  * msg,
                                 char *ipRtIndx, int RtIndex)
{
    // return in the following format.
    // ipRtDest:ipRtIfIndx:ipRtMetric1:ipRtNxtHop:ipRtType:

    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[16]="";
    char tmp2[10]="";
    char tmp3[10]="";
    char tmp4[16]="";
    char tmp5[10]="";
    
    char *tmp;
    long ipRtDest;
    long ip[4];
    long long_return;
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    sscanf(ipRtIndx, "%lu.%lu.%lu.%lu",
           &ip[0], &ip[1], &ip[2], &ip[3]);
    ipRtDest = (ip[0]) | (ip[1] << 8) | (ip[2] << 16) | (ip[3] << 24);
        

    snprintf(tmp1, sizeof(tmp1), "%lu", ipRtDest);
    snprintf(tmp2, sizeof(tmp2), "%lu", rthead[RtIndex]->rt_pad1);
    snprintf(tmp3, sizeof(tmp3), "%u", (rthead[RtIndex]->rt_flags &
                                         RTF_GATEWAY) ? 1 : 0);

    tmp = (char *) & ((struct sockaddr_in *) &rthead[RtIndex]->
                       rt_gateway)->sin_addr.s_addr;
    ipRtDest = (*tmp) | ((*(tmp+1))<<8) | ((*(tmp+2))<<16)  | ((*(tmp+3))<<24);
    snprintf(tmp4, sizeof(tmp4), "%lu", ipRtDest);
    

    if (rthead[RtIndex]->rt_flags & RTF_UP)
    {
        if (rthead[RtIndex]->rt_flags & RTF_GATEWAY)
            long_return = 4;        /*  indirect(4)  */
        else
            long_return = 3;        /*  direct(3)  */
    }
    else
        long_return = 2;    /*  invalid(2)  */

    snprintf(tmp5, sizeof(tmp5), "%lu", long_return);
        
    msgr_addTransValues(respMsg, 5,
                        tmp1, tmp2, tmp3, tmp4, tmp5);
    //                        tmp6, tmp7, tmp8, "0.0");
    
    msg_send(respMsg);

    return 0;
}

static int snmp_sendIpRtTblInfo2(struct cm_module *module,
                                 struct msg_base  * msg,
                                 char *ipRtIndx, int RtIndex)
{
    // return in the following format.
    // ipRtDest:ipRtProto:ipRtAge:ipRtMask:ipRtInfo

    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    char *tmp;
	char tmp1[16]="";
    char tmp6[10]="";
    char tmp7[10]="";
    char tmp8[16];
    
    long ipRtDest;
    long ip[4];
    long long_return;
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);

    sscanf(ipRtIndx, "%lu.%lu.%lu.%lu",
           &ip[0], &ip[1], &ip[2], &ip[3]);
    ipRtDest = (ip[0]) | (ip[1] << 8) | (ip[2] << 16) | (ip[3] << 24);
        

    snprintf(tmp1, sizeof(tmp1), "%lu", ipRtDest);

    long_return = (rthead[RtIndex]->rt_flags & RTF_DYNAMIC) ? 4 : 2;
    snprintf(tmp6, sizeof(tmp6), "%lu", long_return);
        
    long_return = 0;
    snprintf(tmp7, sizeof(tmp7), "%lu", long_return);
        
    if (((struct sockaddr_in *) &rthead[RtIndex]->rt_dst)->sin_addr.
        s_addr == 0)
        strcpy(tmp8, "0.0.0.0");    /* Default route */
    else
    {
        tmp = (char *) &
                (((struct sockaddr_in *) &(rthead[RtIndex]->rt_genmask))->
                 sin_addr.s_addr);
        sprintf(tmp8, "%u.%u.%u.%u", (*tmp) & 0xff, (*tmp+1) & 0xff,
                (*tmp+2) & 0xff, (*tmp+3) & 0xff);
    }
        
    msgr_addTransValues(respMsg, 5,
                        tmp1, tmp6, tmp7, tmp8, "0.0");
    
    msg_send(respMsg);

    return 0;
}

static int snmp_sendIpN2MTblInfo(struct cm_module *module,
                                 struct msg_base  * msg,
                                 oid *LowAddr, u_long lowIfIndex,
                                 u_long lowIfType, char *LowPhysAddr)
{
    // return in the following format.
    // ipN2MIfIndex:ipN2MPhysAddr:ipN2MNetAddr:ipN2MType
    
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
	char tmp1[10]="";
    char tmp2[20];
    char tmp3[16]="";
    char tmp4[10]="";
    long ipAddr;
    
    respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);


    snprintf(tmp1, sizeof(tmp1), "%lu", lowIfIndex);
    sprintf(tmp2, "%u:%u:%u:%u:%u:%u",
           LowPhysAddr[0] & 0xff, LowPhysAddr[1] & 0xff, LowPhysAddr[2] & 0xff,
           LowPhysAddr[3] & 0xff, LowPhysAddr[4] & 0xff, LowPhysAddr[5] & 0xff);

    ipAddr = (*LowAddr) | ((*(LowAddr+1))<<8) | ((*(LowAddr+2))<<16) | ((*(LowAddr+3))<<24);
    snprintf(tmp3, sizeof(tmp3), "%lu", ipAddr);
    snprintf(tmp4, sizeof(tmp4), "%lu", lowIfType);
    
    msgr_addTransValues(respMsg, 4,
                        tmp1, tmp3,
                        tmp2, tmp4);
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIcmpInStats1(struct cm_module *module,
                                 struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1, 7 columns)
    // icmpInMsgs:icmpInErrs:icmpInDestUnrchs:icmpInTimeExcds:
    // icmpInParmProbs:icmpInSrcQuenchs:icmpInRedirects

    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct icmp_mib icmp;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";
	char tmp7[10]="";

    memset((char *) &icmp, (0), sizeof(struct icmp_mib));
    ret_value = read_ip_stat((void*)(&icmp), ICMP_STATS);

    snprintf(tmp1, sizeof(tmp1), "%lu", icmp.icmpInMsgs);
    snprintf(tmp2, sizeof(tmp2), "%lu", icmp.icmpInErrors);
    snprintf(tmp3, sizeof(tmp3), "%lu", icmp.icmpInDestUnreachs);
    snprintf(tmp4, sizeof(tmp4), "%lu", icmp.icmpInTimeExcds);
    snprintf(tmp5, sizeof(tmp5), "%lu", icmp.icmpInParmProbs);
    snprintf(tmp6, sizeof(tmp6), "%lu", icmp.icmpInSrcQuenchs);
    snprintf(tmp7, sizeof(tmp7), "%lu", icmp.icmpInRedirects);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 7,
                        tmp1, tmp2, tmp3, tmp4,
                        tmp5, tmp6, tmp7);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIcmpInStats2(struct cm_module *module,
                                 struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1, 6 columns)
    // icmpInEchos:icmpInEchoReps:icmpInTimestamps
    // icmpInTimestReps:icmpInAddrMasks:icmpInAdMaskRep
    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct icmp_mib icmp;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";

    memset((char *) &icmp, (0), sizeof(struct icmp_mib));
    ret_value = read_ip_stat((void*)(&icmp), ICMP_STATS);

    snprintf(tmp1, sizeof(tmp1), "%lu", icmp.icmpInEchos);
    snprintf(tmp2, sizeof(tmp2), "%lu", icmp.icmpInEchoReps);
    snprintf(tmp3, sizeof(tmp3), "%lu", icmp.icmpInTimestamps);
    snprintf(tmp4, sizeof(tmp4), "%lu", icmp.icmpInTimestampReps);
    snprintf(tmp5, sizeof(tmp5), "%lu", icmp.icmpInAddrMasks);
    snprintf(tmp6, sizeof(tmp6), "%lu", icmp.icmpInAddrMaskReps);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 6,
                        tmp1, tmp2, tmp3,
                         tmp4,tmp5, tmp6);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIcmpOutStats1(struct cm_module *module,
                                  struct msg_base  * msg)
{
    // return in the following format.
    // (Row 1, 7 columns)
    // icmpOutMsgs:icmpOutErrs:icmpOutDestUnrchs:
    // icmpOutTimeExcds:icmpOutParmProbs:icmpOutSrcQuenchs:
    // icmpOutRedirects
    
    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct icmp_mib icmp;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";
	char tmp7[10]="";

    memset((char *) &icmp, (0), sizeof(struct icmp_mib));
    ret_value = read_ip_stat((void*)(&icmp), ICMP_STATS);

    snprintf(tmp1, sizeof(tmp1), "%lu", icmp.icmpOutMsgs);
    snprintf(tmp2, sizeof(tmp2), "%lu", icmp.icmpOutErrors);
    snprintf(tmp3, sizeof(tmp3), "%lu", icmp.icmpOutDestUnreachs);
    snprintf(tmp4, sizeof(tmp4), "%lu", icmp.icmpOutTimeExcds);
    snprintf(tmp5, sizeof(tmp5), "%lu", icmp.icmpOutParmProbs);
    snprintf(tmp6, sizeof(tmp6), "%lu", icmp.icmpOutSrcQuenchs);
    snprintf(tmp7, sizeof(tmp7), "%lu", icmp.icmpOutRedirects);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 7,
                        tmp1, tmp2, tmp3, tmp4,
                        tmp5, tmp6, tmp7);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_sendIcmpOutStats2(struct cm_module *module,
                                  struct msg_base  * msg)
{
    // (Row 1, 6 columns)
    // icmpOutEchos:icmpOutEchoReps:icmpOutTimestamps:
    // icmpOutTimestReps:icmpOutAddrMasks:icmpOutAdMaskRep

    int ret_value;
    struct msg_base *respMsg;
    char *key = msg_getTransKey(msg);
    struct icmp_mib icmp;
	char tmp1[10]="";
	char tmp2[10]="";
	char tmp3[10]="";
	char tmp4[10]="";
	char tmp5[10]="";
	char tmp6[10]="";

    memset((char *) &icmp, (0), sizeof(struct icmp_mib));
    ret_value = read_ip_stat((void*)(&icmp), ICMP_STATS);

    snprintf(tmp1, sizeof(tmp1), "%lu", icmp.icmpOutEchos);
    snprintf(tmp2, sizeof(tmp2), "%lu", icmp.icmpOutEchoReps);
    snprintf(tmp3, sizeof(tmp3), "%lu", icmp.icmpOutTimestamps);
    snprintf(tmp4, sizeof(tmp4), "%lu", icmp.icmpOutTimestampReps);
    snprintf(tmp5, sizeof(tmp5), "%lu", icmp.icmpOutAddrMasks);
    snprintf(tmp6, sizeof(tmp6), "%lu", icmp.icmpOutAddrMaskReps);

	respMsg = msgm_makeTransResponseEmpty(module,
                                          msg->from, 0,
                                          msg->seq,
                                          0, key);
    msgr_addTransValues(respMsg, 6,
                        tmp1, tmp2, tmp3,
                        tmp4, tmp5, tmp6);
    
    msg_send(respMsg);
    return 0;
}

static int snmp_query(struct cm_module *module,
                      struct msg_base  *msg,
                      char             *key)
{
    struct key_parts kp;
    int notGetNext, append, sameTable;
    char newIndex[40]="";
				
    if(key_split(key, &kp) < 0)
        return RET_MSG_BAD_KEY; // key malformed

    while ((kp.keytype != KEY_SETTINGS) &&
           (kp.keytype != KEY_STATUS))
    {
        if (key_split(key, &kp) < 0)
            return RET_MSG_BAD_KEY;
    }

    switch(kp.keytype)
    {
        case KEY_SETTINGS:
            // community name and others
            if (!strcmp(kp.key, "community/entry"))
            	snmp_sendCommunityEntryInfo(module, msg);
            else if(!strcmp(kp.key, "community"))
              snmp_sendCommunityInfo(module, msg);
            else if (!strcmp(kp.key, "system/idletime"))
              snmp_sendSystemIdletime(module, msg);
            else if (!strncmp(kp.key, "system/",strlen("system/")))
              snmp_sendSystemElements(module, msg);
            else if (!strncmp(kp.key, "system",strlen("system")))
              snmp_sendSystemInfo(module, msg);
            else
              return RET_MSG_BAD_KEY;
            return RET_MSG_OK;
        case KEY_STATUS:
            // Other OIDs
            if (!strncmp(kp.key, "sysorchange",
                         strlen("sysorchange")))
            {
                snmp_sendSysOrChange(module, msg);
            }
            else if (!strncmp(kp.key, "sysortbl",
                         strlen("sysortbl")))
            {
                struct sysORTable *pORIndx;

                pORIndx = get_ORIndex(key, &notGetNext,
                                      newIndex, &append,
                                      &sameTable);
                if (pORIndx == NULL)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else
                    snmp_sendSysOrTblInfo(module, msg,
                                          pORIndx, newIndex);
            }
            else if (!strncmp(kp.key, "ifnum",
                              strlen("ifnum")))
            {
                snmp_sendIfNum(module, msg);
            }
            else if (!strncmp(kp.key, "ifgeneral1",
                              strlen("ifgeneral1")))
            {
                
                int intf = get_Interface(key, &notGetNext,
                                         newIndex, &append,
                                         &sameTable);
                if (intf < 0)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else
                    snmp_sendIfGenInfo1(module, msg, intf);
            }
            else if (!strncmp(kp.key, "ifgeneral2",
                              strlen("ifgeneral2")))
            {
                int intf = get_Interface(key, &notGetNext,
                                         newIndex, &append,
                                         &sameTable);
                if (intf < 0)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else
                    snmp_sendIfGenInfo2(module, msg, intf);
            }            
            else if (!strncmp(kp.key, "ifinstats",
                              strlen("ifinstats")))
            {
                int intf = get_Interface(key, &notGetNext,
                                         newIndex, &append,
                                         &sameTable);
                if (intf < 0)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else
                    snmp_sendIfInStats(module, msg, intf);
            }
            else if (!strncmp(kp.key, "ifoutstats",
                              strlen("ifoutstats")))
            {
                int intf = get_Interface(key, &notGetNext,
                                         newIndex, &append,
                                         &sameTable);
                if (intf < 0)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else
                    snmp_sendIfOutStats(module, msg, intf);
            }
            else if (!strncmp(kp.key, "ipgeneral",
                              strlen("ipgeneral")))
            {
                snmp_sendIpGenInfo(module, msg);
            }
            else if (!strncmp(kp.key, "ipinstats",
                              strlen("ipinstats")))
            {
                snmp_sendIpInStat(module, msg);
            }
            else if (!strncmp(kp.key, "ipoutstats",
                              strlen("ipoutstats")))
            {
                snmp_sendIpOutStat(module, msg);
            }
            else if (!strncmp(kp.key, "ipreasmstats",
                              strlen("ipreasmstats")))
            {
                snmp_sendIpReasmStat(module, msg);
            }
            else if (!strncmp(kp.key, "ipaddrtbl",
                              strlen("ipaddrtbl")))
            {
                int    lowinterface = 0;
                struct ifnet lowin_ifnet;
                char *ipAddrIndx = get_IpAddrIndex(key, &notGetNext,
                                                   newIndex, &append,
                                                   &sameTable,
                                                   &lowinterface,
                                                   &lowin_ifnet);
                if (ipAddrIndx == NULL)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else                
                    snmp_sendIpAddrTblInfo(module, msg,
                                           ipAddrIndx, &lowinterface,
                                           &lowin_ifnet);
            }
            else if (!strncmp(kp.key, "iprttbl1",
                              strlen("iprttbl1")))
            {
                int RtIndex;
                char *ipRtIndx = get_IpRouteIndex(key, &notGetNext,
                                                  newIndex, &append,
                                                  &sameTable, &RtIndex);
                if (ipRtIndx == NULL)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else                
                    snmp_sendIpRtTblInfo1(module, msg,
                                         ipRtIndx, RtIndex);
            }
            else if (!strncmp(kp.key, "iprttbl2",
                              strlen("iprttbl2")))
            {
                int RtIndex;
                char *ipRtIndx = get_IpRouteIndex(key, &notGetNext,
                                                  newIndex, &append,
                                                  &sameTable, &RtIndex);
                if (ipRtIndx == NULL)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else                
                    snmp_sendIpRtTblInfo2(module, msg,
                                         ipRtIndx, RtIndex);
            }
            else if (!strncmp(kp.key, "ipn2mtbl",
                              strlen("ipn2mtbl")))
            {
                char   LowPhysAddr[6]="";
                oid    LowAddr[5];
                u_long lowIfIndex, lowIfType;
                int result;
                
                result = get_IpAtIndex(key, &notGetNext,
                                       newIndex, &append,
                                       &sameTable, &lowIfIndex,
                                       &lowIfType, LowPhysAddr,
                                       LowAddr);
                if (result == 0)
                {
                    if (notGetNext)
                        snmp_sendError(module, msg,
                                       SNMP_ERR_NOSUCHNAME);
                    else
                        snmp_sendReqNext(module, msg,
                                         sameTable,
                                         newIndex);
                }
                else                
                    snmp_sendIpN2MTblInfo(module, msg,
                                          LowAddr+1, lowIfIndex,
                                          lowIfType, LowPhysAddr);
            }
            else if (!strncmp(kp.key, "icmpinstats1",
                              strlen("icmpinstats1")))
            {                
                snmp_sendIcmpInStats1(module, msg);
            }
            else if (!strncmp(kp.key, "icmpinstats2",
                              strlen("icmpinstats2")))
            {                
                snmp_sendIcmpInStats2(module, msg);
            }
            else if (!strncmp(kp.key, "icmpoutstats1",
                              strlen("icmpoutstats1")))
            {
                snmp_sendIcmpOutStats1(module, msg);
            }
            else if (!strncmp(kp.key, "icmpoutstats2",
                              strlen("icmpoutstats2")))
            {
                snmp_sendIcmpOutStats2(module, msg);
            }
            else
                return RET_MSG_BAD_KEY;
            
            return RET_MSG_OK;
    }
    
    return RET_MSG_BAD_KEY;
}

static int snmp_start(struct cm_module *module)
{
#if 0
  SNMP_PRIV *priv;
  char *value;

  priv = module->priv;


  /* Register for global events */
  event_global_register(module);
  /* Check if LAN access is enabled */
  value = tree_getKeyContent(module->config,"system/lanaccess");
  if( (value != NULL) && (!strcmp(value,"1")) )
    event_broadcast(module, "flan", EVT_REPORT_FW_SNMP, (void *)1);
  else		
    event_broadcast(module, "flan", EVT_REPORT_FW_SNMP, (void *)0);
  /* Check if WAN access is enabled */
  value = tree_getKeyContent(module->config,"system/wanaccess");
  if( (value != NULL) && (!strcmp(value,"1")) )
    event_broadcast(module, "fwan", EVT_REPORT_FW_SNMP, (void *)1);
  else		
    event_broadcast(module, "fwan", EVT_REPORT_FW_SNMP, (void *)0);
#endif
  return 0;
}

static int snmp_stop(struct cm_module *module)
{
    // ??? where to send this STOP event ???
    event_send(module, DIR_UNKNOWN, EVT_REPORT_MODULE_STOP, 0);
    return 0;
}

int snmp_event(struct cm_module *module, struct cm_event *event)
{
#if 0
	SNMP_PRIV *pPriv;
  char *value;

  pPriv = module->priv;

  switch(event->type)
  {
		case EVT_REPORT_FW_UP :
			{

  			/* Check if LAN access is enabled */
  			value = tree_getKeyContent(module->config,"system/lanaccess");
  			if( (value != NULL) && (!strcmp(value,"1")) )
    			event_broadcast(module, "flan", EVT_REPORT_FW_SNMP, (void *)1);
  			else		
    			event_broadcast(module, "flan", EVT_REPORT_FW_SNMP, (void *)0);
  			/* Check if WAN access is enabled */
  			value = tree_getKeyContent(module->config,"system/wanaccess");
  			if( (value != NULL) && (!strcmp(value,"1")) )
    			event_broadcast(module, "fwan", EVT_REPORT_FW_SNMP, (void *)1);
  			else		
    			event_broadcast(module, "fwan", EVT_REPORT_FW_SNMP, (void *)0);


			}

			break;
    default:
			break;
  }
#endif

  return 0;
}

static int snmp_message(struct cm_module *module,
                        struct msg_base *msg)
{
    char             *key;
    
    switch (msg->msg_type)
    {
        case MSG_TRANSACTION:
            switch (msg_getTransType(msg))
            {
                case MSG_TRANS_QUERY:
                    key = msg_getTransKey(msg);
                    
                    return snmp_query(module,
                                      msg,
                                      key);

                case MSG_TRANS_SET:
                    return snmp_set(module,
                                    msg_getTransKey(msg),
                                    msg_getTransSingleValue(msg));
                    
                case MSG_TRANS_APPLY:
                    return snmp_apply(module);
                    
                case MSG_TRANS_REJECT:
                    return snmp_reject(module);
                    
                case MSG_TRANS_GROUP_BEGIN: 
                case MSG_TRANS_GROUP_END:
                    return RET_MSG_OK;
                    
                default:
                    logger(module, -1, LOG_ERR,
	 		"snmp_message: Unexpected transaction type\n");
                    break;
            }
            break;
            
        default:
            logger(module, -1, LOG_ERR,
                   "snmp_message: Unexpected message type\n");
            break;
    }

    return RET_MSG_ERROR;
}

static int snmp_init(struct cm_module *module)
{
    SNMP_PRIV *priv;

    priv = malloc(sizeof(SNMP_PRIV));
    if (!priv)
    {
        logger(module, -1, LOG_ERR,
               "snmp_init: Malloc failure\n");
        
        return -1;
    }
    
    module->start = snmp_start;
    module->stop = snmp_stop;
    module->do_message = snmp_message;
    module->do_event = snmp_event;
    module->detach = NULL;

    module->priv = priv;
    
    // Any private field initialization goes to here
    memset (priv->community, 0x00,
            sizeof(COMMUNITY) * MAX_COMMUNITY_NUM);

    if (module->config)
    {
        snmp_load_community(module);
    }
    else
    {
        priv->community[0].name = default_community;
        priv->community[0].acsRight = default_acsRight;
        priv->community[1].name = default_community;
        priv->community[1].acsRight = default_acsRight;
        priv->community[2].name = default_community;
        priv->community[2].acsRight = default_acsRight;
        priv->idleTimeout = SNMP_LISTEN_INET_TIMEOUT;
        priv->sysName 		= SNMP_DEF_NAME;
        priv->sysLocation = SNMP_DEF_LOCATION;
        priv->sysContact 	= SNMP_DEF_CONTACT;
        priv->sysOID 			= SNMP_DEF_OID;
    }

    init_mib();
    sysOR_lastchange = time(0);
    
    return 0;
}

int snmp_register(void)
{
    return register_module("snmp", snmp_init);
}
