#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <malloc.h>
#include <unistd.h>
#include <net/if.h>
#include <netinet/if_ether.h>
#include <sys/ioctl.h>
#include <time.h>
#include <net/route.h>


#include <sys/sysctl.h>

#include "../../snmp/include/snmp_types.h"
#include "snmp_mib.h"

extern long sysOR_lastchange;
static struct sysORTable *table = NULL;


struct sysORTable *get_ORIndex(char *key, int *notGetNext,
                               char *newIndex, int *append,
                               int *sameTable)
{
    char *opIn, *indxIn;
    int index = 0;

    unsigned long   i = 0;
    struct sysORTable *ptr = table;
    
    *append = 0;
    *sameTable = 1;
    opIn = strchr(key, '@');
    indxIn = strrchr(key, '@');

    *notGetNext = strncmp(opIn+1, "NEXT", strlen("NEXT"));
    
    // No index in the key
    if (indxIn == opIn)
    {
        // if it is not getNext, that is an error
        if (*notGetNext)
            return NULL;
    }
    else // retrieve the index from key
    {
        indxIn++; // skip @
        index = atoi(++indxIn); // skip '.'
    }

    if (index == 0)
    {
        if (*notGetNext)
            return NULL;
        // GetNext to the first entry in the table
        // append the index to the key
        index = 1;
        *append = 1;
        *sameTable = 1;
        sprintf(newIndex, "%d", index);
        return ptr;
    }

    for (i = 1; ptr != NULL; ptr = ptr->next, i++)
    {
        // GET and SET has to be exact
        if (i == index)
        {
            if (*notGetNext)
                return ptr;
        }

        if (i > index)
        {
            // need to modify index with the new value
            sprintf(newIndex, "%lu", i);
            *append = -1;
            *sameTable = 1;
            return ptr;            
        }
    }    
    *sameTable = 0;
    return NULL;    
}


#define IFNET_SYMBOL "ifnet"

static struct ifconf ifc;
static struct ifreq *ifr;
static int ifr_counter;

static time_t   scan_time = 0;


static conf_if_list *conf_list;
static int      saveIndex = 0;
static char     saveName[16];
static struct ifnet *ifnetaddr, saveifnet;
struct ifnet *saveifnetaddr;
static struct ifnet *ifnetaddr_list;
static int      Interface_Count = 0;

unsigned int getIfSpeed(int fd, struct ifreq ifr)
{
	unsigned int retspeed = 10000000;

/* the code is based on mii-diag utility by Donald Becker
* see ftp://ftp.scyld.com/pub/diag/mii-diag.c
*/
	ushort *data = (ushort *)(&ifr.ifr_data);
	unsigned phy_id;
	unsigned char new_ioctl_nums = 0;
	int mii_reg, i;
	ushort mii_val[32];
	ushort bmcr, bmsr, nway_advert, lkpar;
	const unsigned int media_speeds[] = {10000000, 10000000, 100000000, 100000000, 10000000, 0};	
/* It corresponds to "10baseT", "10baseT-FD", "100baseTx", "100baseTx-FD", "100baseT4", "Flow-control", 0, */


	data[0] = 0;

	if (ioctl(fd, 0x8947, &ifr) >= 0) {
		new_ioctl_nums = 1;
	} else if (ioctl(fd, SIOCDEVPRIVATE, &ifr) >= 0) {
		new_ioctl_nums = 0;
	} else {
		return retspeed;
	}
/* Begin getting mii register values */
	phy_id = data[0];
	for (mii_reg = 0; mii_reg < 8; mii_reg++){
		data[0] = phy_id;
		data[1] = mii_reg;
		ioctl(fd, new_ioctl_nums ? 0x8948 : SIOCDEVPRIVATE+1, &ifr);

		mii_val[mii_reg] = data[3];		
	}
/*Parsing of mii values*/
/*Invalid basic mode control register*/
	if (mii_val[0] == 0xffff  ||  mii_val[1] == 0x0000) {
		return retspeed;
	}
	/* Descriptive rename. */
	bmcr = mii_val[0]; 	  /*basic mode control register*/
	bmsr = mii_val[1]; 	  /* basic mode status register*/
	nway_advert = mii_val[4]; /* autonegotiation advertisement*/
	lkpar = mii_val[5]; 	  /*link partner ability*/
	
/*Check for link existence, returns 0 if link is absent*/
	if ((bmsr & 0x0016) != 0x0004){
		retspeed = 0;
		return retspeed;
	}
	
	if(!(bmcr & 0x1000) ){
		retspeed = bmcr & 0x2000 ? 100000000 : 10000000;
		return retspeed;
	}
/* Link partner got our advertised abilities */	
	if (lkpar & 0x4000) {
		int negotiated = nway_advert & lkpar & 0x3e0;
		int max_capability = 0;
		/* Scan for the highest negotiated capability, highest priority
		   (100baseTx-FDX) to lowest (10baseT-HDX). */
		int media_priority[] = {8, 9, 7, 6, 5}; 	/* media_names[i-5] */
		for (i = 0; media_priority[i]; i++){
			if (negotiated & (1 << media_priority[i])) {
				max_capability = media_priority[i];
				break;
			}
		}
		if (max_capability)
			retspeed = media_speeds[max_capability - 5];
	}
	return retspeed;
}

static int
if_type_from_name(const char *pcch)
{
    typedef struct _match_if {
        int             mi_type;
        const char     *mi_name;
    }              *pmatch_if, match_if;

    static match_if lmatch_if[] = {
        {24, "lo"},
        {6, "eth"},
        {9, "tr"},
        {23, "ppp"},
        {28, "sl"},
        {0, 0}                  /* end of list */
    };

    int             ii, len;
    register pmatch_if pm;

    for (ii = 0, pm = lmatch_if; pm->mi_name; pm++) {
        len = strlen(pm->mi_name);
        if (0 == strncmp(pcch, pm->mi_name, len)) {
            return (pm->mi_type);
        }
    }
    return (1);                 /* in case search fails */
}


static void Interface_Scan_Init(void)
{
    char            line[256], ifname_buf[64], *ifname, *ptr;
    struct ifreq    ifrq;
    struct ifnet  **ifnetaddr_ptr;
    FILE           *devin, *intfFd;
    unsigned long   rec_pkt, rec_oct, rec_err, rec_drop;
    unsigned long   snd_pkt, snd_oct, snd_err, snd_drop, coll;
    int             i, fd;
    conf_if_list   *if_ptr;
    const char     *scan_line_2_2 =
        "%lu %lu %lu %lu %*lu %*lu %*lu %*lu %lu %lu %lu %lu %*lu %lu";
    const char     *scan_line_2_0 =
        "%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
    const char     *scan_line_to_use;

    saveIndex = 0;

    while (ifnetaddr_list) {
        struct ifnet   *old = ifnetaddr_list;
        ifnetaddr_list = ifnetaddr_list->if_next;
        free(old->if_name);
        free(old->if_unit);
        free(old);
    }

    ifnetaddr = 0;
    ifnetaddr_ptr = &ifnetaddr_list;

    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        return; /** exit (1); **/
    
    /*
     * build up ifnetaddr list by hand: 
     */

    /*
     * at least linux v1.3.53 says EMFILE without reason... 
     */
    if (!(devin = fopen("/proc/net/dev", "r"))) {
        close(fd);
        return; /** exit (1); **/
    }

    i = 0;

    /*
     * read the second line (a header) and determine the fields we
     * should read from.  This should be done in a better way by
     * actually looking for the field names we want.  But thats too
     * much work for today.  -- Wes 
     */
    fgets(line, sizeof(line), devin);
    fgets(line, sizeof(line), devin);
    if (strstr(line, "compressed")) {
        scan_line_to_use = scan_line_2_2;
    } else {
        scan_line_to_use = scan_line_2_0;
    }


    while (fgets(line, sizeof(line), devin)) {
        struct ifnet   *nnew;
        char           *stats, *ifstart = line;

        if (line[strlen(line) - 1] == '\n')
            line[strlen(line) - 1] = '\0';

        while (*ifstart && *ifstart == ' ')
            ifstart++;

        if (!*ifstart || ((stats = strrchr(ifstart, ':')) == NULL)) {
            continue;
        }

        *stats   = 0;
        strncpy(ifname_buf, ifstart, sizeof(ifname_buf));
        ifname_buf[ sizeof(ifname_buf)-1 ] = 0;
        *stats++ = ':';
        while (*stats == ' ')
            stats++;

        if ((scan_line_to_use == scan_line_2_2 &&
             sscanf(stats, scan_line_to_use, &rec_oct, &rec_pkt, &rec_err,
                    &rec_drop, &snd_oct, &snd_pkt, &snd_err, &snd_drop,
                    &coll) != 9) || (scan_line_to_use == scan_line_2_0
                                     && sscanf(stats, scan_line_to_use,
                                               &rec_pkt, &rec_err,
                                               &snd_pkt, &snd_err,
                                               &coll) != 5)) {
            continue;
        }

        nnew = (struct ifnet *) calloc(1, sizeof(struct ifnet));
        if (nnew == NULL)
            break;              /* alloc error */

        /*
         * chain in: 
         */
        *ifnetaddr_ptr = nnew;
        ifnetaddr_ptr = &nnew->if_next;
        i++;

        /*
         * linux previous to 1.3.~13 may miss transmitted loopback pkts: 
         */
        if (!strcmp(ifname_buf, "lo") && rec_pkt > 0 && !snd_pkt)
            snd_pkt = rec_pkt;

        nnew->if_ipackets = rec_pkt;
        nnew->if_ierrors = rec_err;
        nnew->if_opackets = snd_pkt;
        nnew->if_oerrors = snd_err;
        nnew->if_collisions = coll;
        if (scan_line_to_use == scan_line_2_2) {
            nnew->if_ibytes = rec_oct;
            nnew->if_obytes = snd_oct;
            nnew->if_iqdrops = rec_drop;
            nnew->if_snd.ifq_drops = snd_drop;
        } else {
            nnew->if_ibytes = rec_pkt * 308;
            nnew->if_obytes = snd_pkt * 308;
        }

        /*
         * ifnames are given as ``   eth0'': split in ``eth'' and ``0'': 
         */
        for (ifname = ifname_buf; *ifname && *ifname == ' '; ifname++);

        /*
         * set name and interface# : 
         */
        nnew->if_name = (char *) strdup(ifname);
        for (ptr = nnew->if_name; *ptr && (*ptr < '0' || *ptr > '9');
             ptr++);
        nnew->if_unit = strdup(*ptr ? ptr : "");
        *ptr = 0;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFADDR, &ifrq) < 0)
            memset((char *) &nnew->if_addr, 0, sizeof(nnew->if_addr));
        else
            nnew->if_addr = ifrq.ifr_addr;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFBRDADDR, &ifrq) < 0)
            memset((char *) &nnew->ifu_broadaddr, 0,
                   sizeof(nnew->ifu_broadaddr));
        else
            nnew->ifu_broadaddr = ifrq.ifr_broadaddr;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFNETMASK, &ifrq) < 0)
            memset((char *) &nnew->ia_subnetmask, 0,
                   sizeof(nnew->ia_subnetmask));
        else
            nnew->ia_subnetmask = ifrq.ifr_netmask;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_flags = ioctl(fd, SIOCGIFFLAGS, &ifrq) < 0
            ? 0 : ifrq.ifr_flags;

        nnew->if_type = 0;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        if (ioctl(fd, SIOCGIFHWADDR, &ifrq) < 0)
            memset(nnew->if_hwaddr, (0), 6);
        else {
            memcpy(nnew->if_hwaddr, ifrq.ifr_hwaddr.sa_data, 6);

            switch (ifrq.ifr_hwaddr.sa_family) {
            case ARPHRD_ETHER:
                nnew->if_type = 6;
                break;
            case ARPHRD_TUNNEL:
            case ARPHRD_TUNNEL6:
            case ARPHRD_IPGRE:
            case ARPHRD_SIT:
                nnew->if_type = 131;
                break;          /* tunnel */
            case ARPHRD_SLIP:
            case ARPHRD_CSLIP:
            case ARPHRD_SLIP6:
            case ARPHRD_CSLIP6:
                nnew->if_type = 28;
                break;          /* slip */
            case ARPHRD_PPP:
                nnew->if_type = 23;
                break;          /* ppp */
            case ARPHRD_LOOPBACK:
                nnew->if_type = 24;
                break;          /* softwareLoopback */
            case ARPHRD_FDDI:
                nnew->if_type = 15;
                break;
            case ARPHRD_ARCNET:
                nnew->if_type = 35;
                break;
            case ARPHRD_LOCALTLK:
                nnew->if_type = 42;
                break;
            case ARPHRD_ATM:
                nnew->if_type = 37;
                break;
            }
        }
        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_metric = ioctl(fd, SIOCGIFMETRIC, &ifrq) < 0
            ? 0 : ifrq.ifr_metric;

        strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
        ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
        nnew->if_mtu = (ioctl(fd, SIOCGIFMTU, &ifrq) < 0)
            ? 0 : ifrq.ifr_mtu;

        for (if_ptr = conf_list; if_ptr; if_ptr = if_ptr->next)
            if (!strcmp(if_ptr->name, ifname))
                break;

        if (if_ptr) {
            nnew->if_type = if_ptr->type;
            nnew->if_speed = if_ptr->speed;
        } else {
            /*
             * do only guess if_type from name, if we could not read
             * * it before from SIOCGIFHWADDR 
             */
            if (!nnew->if_type)
                nnew->if_type = if_type_from_name(nnew->if_name);
            nnew->if_speed = nnew->if_type == 6 ? getIfSpeed(fd, ifrq) :
                nnew->if_type == 24 ? 10000000 :
                nnew->if_type == 9 ? 4000000 : 0;
            /*Zero speed means link problem*/
            /* check the link status from proc files */
            if (!strncmp(nnew->if_name, "lo", 2))
            {
                continue;
            }
            if (!strncmp(nnew->if_name, "usb", 3))
            {
                if (!(intfFd = fopen("/proc/avalanche/usb_link", "r")))
                    nnew->if_flags &= ~IFF_RUNNING;
                else
                {
                    int linkStatus;
                    
                    fgets(line, sizeof(line), intfFd);
                    sscanf(line, "%d", &linkStatus);
                    if (linkStatus == 0)
                        nnew->if_flags &= ~IFF_RUNNING;
                    fclose(intfFd);
                }
            }
            else if (!strncmp(nnew->if_name, "eth", 3))
            {
                if (!(intfFd = fopen("/proc/avalanche/cpmac_link", "r")))
                    nnew->if_flags &= ~IFF_RUNNING;
                else
                {
                    char *linkStatus;
                    /* second line contains the ethernet status */
                    fgets(line, sizeof(line), intfFd);
                    fgets(line, sizeof(line), intfFd);
                    linkStatus = strchr(line, ':');
                    linkStatus = strchr(linkStatus, ':');
                    linkStatus++;
                    if (!strncmp(linkStatus, "DOWN", 4))
                        nnew->if_flags &= ~IFF_RUNNING;
                    fclose(intfFd);
                }
            }
            else
            {
                if (!(intfFd = fopen("/proc/avalanche/avsar_modem_training", "r")))
                    nnew->if_flags &= ~IFF_RUNNING;
                else
                {
                    fgets(line, sizeof(line), intfFd);
                    if (!strncmp(line, "IDLE", 4))
                        nnew->if_flags &= ~IFF_RUNNING;
                    fclose(intfFd);
                }
            }
            
#if 0
            if(nnew->if_speed == 0 && nnew->if_flags & IFF_UP){
                nnew->if_flags &= ~IFF_RUNNING;
            }
#endif
        }

    }                           /* while (fgets ... */

    ifnetaddr = ifnetaddr_list;

    fclose(devin);
    close(fd);     
}

static int Interface_Scan_Next(short *Index, char *Name,
                        struct ifnet *Retifnet,
                        struct in_ifaddr *dummy)
{
    struct ifnet    ifnet;
    register char  *cp;

    while (ifnetaddr) {
        /*
         *      Get the "ifnet" structure and extract the device name
         */
        ifnet = *ifnetaddr;
        strncpy(saveName, ifnet.if_name, sizeof(saveName));

        if (strcmp(saveName, "ip") == 0) {
            ifnetaddr = ifnet.if_next;
            continue;
        }



        saveName[sizeof(saveName) - 1] = '\0';
        cp = (char *) strchr(saveName, '\0');
        strncat(cp, ifnet.if_unit, sizeof(saveName)-strlen(saveName)-1);
        saveName[sizeof(saveName) - 1] = '\0';
        if (1 || strcmp(saveName, "lo0") != 0) {        /* XXX */

            if (Index)
                *Index = ++saveIndex;
            if (Retifnet)
                *Retifnet = ifnet;
            if (Name)
                strcpy(Name, saveName);
            saveifnet = ifnet;
            saveifnetaddr = ifnetaddr;
            ifnetaddr = ifnet.if_next;

            return (1);         /* DONE */
        }
        ifnetaddr = ifnet.if_next;
    }
    return (0);                 /* EOF */
}


int Interface_Scan_Get_Count(void)
{
    time_t          time_now = time(NULL);

    if (!Interface_Count || (time_now > scan_time + 60)) {
        scan_time = time_now;
        Interface_Scan_Init();
        Interface_Count = 0;
        while (Interface_Scan_Next(NULL, NULL, NULL, NULL) != 0) {
            Interface_Count++;
        }
    }
    return (Interface_Count);
}

int Interface_Scan_By_Index(int Index,
                            char *Name,
                            struct ifnet *Retifnet,
                            struct in_ifaddr *Retin_ifaddr)
{
    short           i;

    Interface_Scan_Init();
    while (Interface_Scan_Next(&i, Name, Retifnet, Retin_ifaddr)) {
        if (i == Index)
            break;
    }
    if (i != Index)
        return (-1);            /* Error, doesn't exist */

    return (0);                 /* DONE */
}


int Interface_Index_By_Name(char *Name, int Len)
{
    short           ifIndex = 0;
    char            ifName[20];

    Interface_Scan_Init();
    while (Interface_Scan_Next(&ifIndex, ifName, NULL, NULL)
           && strcmp(Name, ifName));
    return ifIndex;
}


int get_Interface(char *key, int *notGetNext,
                  char *newIndex, int *append, int *sameTable)
{
    char *opIn, *indxIn;
    int count, index = 0, interface;

    *append = 0;
    *sameTable = 1;
    opIn = strchr(key, '@');
    indxIn = strrchr(key, '@');

    *notGetNext = strncmp(opIn+1, "NEXT", strlen("NEXT"));
    
    // No index in the key
    if (indxIn == opIn)
    {
        // if it is not getNext, that is an error
        if (*notGetNext)
            return -1;
    }
    else // retrieve the index from key
    {
        indxIn++; // skip @
        index = atoi(++indxIn); // skip '.'
    }
    
    count = Interface_Scan_Get_Count();

    // no interface
    if (count == 0)
        return -1;

    // no index in the OID
    if (index == 0)
    {
        if (*notGetNext)
            return -1;
        // GetNext to the first entry in the table
        // append the index to the key
        interface = 1;
        *append = 1;
        *sameTable = 1;
        sprintf(newIndex, "%d", interface);
        return interface; 
    }

    // It's an error for Get, but GetNext goes to the
    // next leaf
    if (index > count)
    {
        *sameTable = 0;
        return -1;
    }
    
    for (interface = 1; interface <= count; interface++)
    {
        // GET and SET has to be exact
        if (*notGetNext)
        {
            if (interface == index)
                return interface;
        }
        
        // find the interface after NEXT requested
        if (interface > index)
        {
            // need to modify index with the new value
            sprintf(newIndex, "%d", interface);
            *append = -1;
            *sameTable = 1;
            return interface;
        }
    }

    *sameTable = 0;
    return -1;
}


int oid_cmp(oid *in_name1, oid *in_name2, int len1, int len2)
{
    register int    len;
    register const oid *name1 = in_name1;
    register const oid *name2 = in_name2;

    /*
     * len = minimum of len1 and len2 
     */
    if (len1 < len2)
        len = len1;
    else
        len = len2;
    /*
     * find first non-matching OID 
     */
    while (len-- > 0) {
        /*
         * these must be done in seperate comparisons, since
         * subtracting them and using that result has problems with
         * subids > 2^31. 
         */
        if (*(name1) != *(name2)) {
            if (*(name1) < *(name2))
                return -1;
            return 1;
        }
        name1++;
        name2++;
    }
    /*
     * both OIDs equal up to length of shorter OID 
     */
    if (len1 < len2)
        return -1;
    if (len2 < len1)
        return 1;
    return 0;
}

static void Address_Scan_Init(void)
{
    int num_interfaces = 0;
    int fd;

    /* get info about all interfaces */

    ifc.ifc_len = 0;
    ifc.ifc_buf = NULL;
    ifr_counter = 0;

    do
    {
    num_interfaces += 16;

    ifc.ifc_len = sizeof(struct ifreq) * num_interfaces;
    ifc.ifc_buf = (char*) realloc(ifc.ifc_buf, ifc.ifc_len);
    
    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        return;
    
    if (ioctl(fd, SIOCGIFCONF, &ifc) < 0)
    {
        ifr=NULL;
        close(fd);
        return;
    }
    }
    while (ifc.ifc_len >= (sizeof(struct ifreq) * num_interfaces));
    
    ifr = ifc.ifc_req;
    close(fd);
}

static int Address_Scan_Next(short *Index, struct ifnet *Retifnet)
{
    struct ifnet   ifnet_store;
    int fd;
    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        return(0);

    while (ifr)
    {
    
        ifnet_store.if_addr = ifr->ifr_addr;

        if (Retifnet)
        {
            Retifnet->if_addr = ifr->ifr_addr;
        
            if (ioctl(fd, SIOCGIFBRDADDR, ifr) < 0)
            {
                memset((char *) &Retifnet->ifu_broadaddr, 0, sizeof(Retifnet->ifu_broadaddr));
            }
            else
                Retifnet->ifu_broadaddr = ifr->ifr_broadaddr;
            
            ifr->ifr_addr = Retifnet->if_addr;
            if (ioctl(fd, SIOCGIFNETMASK, ifr) < 0)
            {
                memset((char *) &Retifnet->ia_subnetmask, 0, sizeof(Retifnet->ia_subnetmask));
            }
            else
                Retifnet->ia_subnetmask = ifr->ifr_netmask;
            
        }
        
        if (Index)
        {
            ifr->ifr_addr = ifnet_store.if_addr;
            if (ioctl(fd, SIOCGIFINDEX, ifr) < 0)
            {
                *Index = -1;
            }
            else
                *Index = ifr->ifr_ifindex;
        }
        
        ifr++;
        ifr_counter+=sizeof(struct ifreq);
        if (ifr_counter >= ifc.ifc_len)
        {
            ifr = NULL; /* beyond the end */
        }
        
        close(fd);
        return (1);             /* DONE */
    }
    close(fd);
    return (0);                 /* EOF */
}

int cnvtStr2Oid(char *indxIn, oid *name, int len)
{
    char *tmp;
    int   nameLen = 0;

    tmp = strchr(indxIn, '.');
    
    for (nameLen = 0; tmp && nameLen < len; nameLen++)
    {
        *tmp = 0;
        *(name+nameLen) = atol(indxIn);
        indxIn = tmp+1;
        tmp = strchr(indxIn, '.');
    }
    if (strlen(indxIn) != 0)
       *(name+nameLen) = atol(indxIn);

    return ++nameLen;
}


// Do we need append at all?????
char *get_IpAddrIndex(char *key, int *notGetNext,
                      char *newIndex, int *append,
                      int *sameTable, int *lowinterface,
                      struct ifnet *lowin_ifnet)
{
    oid          current[4], *op;
    oid          lowAddr[4], name[4];
    char         lowIndex[40];
    u_char         *cp;
    
    static struct ifnet ifnet;

    char *opIn, *indxIn;
    short interface;
    int result, nameLen = 0;
    u_long foundone = 0;

    *append = 0;
    *sameTable = 1;
    opIn = strchr(key, '@');
    indxIn = strrchr(key, '@');

    *notGetNext = strncmp(opIn+1, "NEXT", strlen("NEXT"));
    memset(name, 0x00, 4 * sizeof(oid));
    memset(lowAddr, 0x00, 4 * sizeof(oid));
    
    // No index in the key
    if (indxIn == opIn)
    {
        // if it is not getNext, that is an error
        if (*notGetNext)
            return NULL;

        indxIn = NULL;
    }
    else // retrieve the index from key
    {
        indxIn += 2; // skip @ and '.'
        nameLen = cnvtStr2Oid(indxIn, name, 4);
    }
    
    Address_Scan_Init();
    
    for (;;) {
        // no IP address
        if (Address_Scan_Next(&interface, &ifnet) == 0)
            break;

        cp = (u_char *) &(((struct sockaddr_in *) &(ifnet.if_addr))->
                           sin_addr.s_addr);

        op = current;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;

        sprintf(newIndex, "%lu.%lu.%lu.%lu",
                current[0], current[1], current[2], current[3]);
        
        // no index in OID
        if (indxIn == NULL)
        {
            if (*notGetNext)
                return NULL;
            // GetNext to the first entry in table
            // append the index to the key
        }

        if ((result = oid_cmp(current, name, 4, nameLen)) == 0)
        {
            // GET and SET has to be exact,
            // GetNext need to find the next one
            if (*notGetNext)
            {
                *lowinterface = interface;
                foundone = 1;
                memmove(lowin_ifnet, &ifnet, sizeof(struct ifnet));
                return indxIn;
            }
        }
        else if ((result > 0) &&
                 ((foundone == 0)  ||
                  (oid_cmp(current, lowAddr, 4, 4) < 0)))
        {
            strcpy(lowIndex, newIndex);
            memmove(lowAddr, current, 4 * sizeof(oid));
            // find the ipAddress after the one GETNEXT requested
            *append = -1;
            *sameTable = 1;
            *lowinterface = interface;
            foundone = 1;
            memmove(lowin_ifnet, &ifnet, sizeof(struct ifnet));
        }
    }

    free(ifc.ifc_buf);

    if (foundone == 0)
    {
        *sameTable = 0;
        return NULL;
    }
    else
    {
        strcpy(newIndex, lowIndex);
        return newIndex;
    }
}


static int  rtsize = 0, rtallocate = 0;
struct rtentry **rthead = 0;

static int
qsort_compare(const void *v1, const void *v2)
{
    struct rtentry * const *r1 = (struct rtentry * const *) v1;
    struct rtentry * const *r2 = (struct rtentry * const *) v2;
    register u_long dst1 =
        ntohl(((const struct sockaddr_in *) &((*r1)->rt_dst))->sin_addr.
              s_addr);
    register u_long dst2 =
        ntohl(((const struct sockaddr_in *) &((*r2)->rt_dst))->sin_addr.
              s_addr);
    if (dst1 == dst2)
        return (0);
    if (dst1 > dst2)
        return (1);
    return (-1);
}

#define MAX_ROUTE_CACHE 50
static void
Route_Scan_Reload(void){
    FILE           *in;
    char            line[256];
    struct rtentry *rt;
    char            name[16], temp[16];
    
    if (!rthead) {
        rthead = (struct rtentry **) calloc(MAX_ROUTE_CACHE,
                                            sizeof(struct rtentry *));
        if (!rthead)
            return;
        rtallocate = MAX_ROUTE_CACHE;
    }

    rtsize = 0;

    if (!(in = fopen("/proc/net/route", "r")))
        return;

    Interface_Scan_Init();
    
    while (fgets(line, sizeof(line), in)) {
        struct rtentry  rtent;
        char            rtent_name[32];
        int             refcnt, flags, metric;
        unsigned        use;

        rt = &rtent;
        memset((char *) rt, (0), sizeof(*rt));
        rt->rt_dev = rtent_name;

        /*
         * as with 1.99.14:
         * Iface Dest GW Flags RefCnt Use Metric Mask MTU Win IRTT
         * eth0 0A0A0A0A 00000000 05 0 0 0 FFFFFFFF 1500 0 0 
         */
        if (8 != sscanf(line, "%s %x %x %x %u %d %d %x %*d %*d %*d\n",
                        rt->rt_dev,
                        &(((struct sockaddr_in *) &(rtent.rt_dst))->
                          sin_addr.s_addr),
                        &(((struct sockaddr_in *) &(rtent.rt_gateway))->
                          sin_addr.s_addr),
                        /*
                         * XXX: fix type of the args 
                         */
                        &flags, &refcnt, &use, &metric,
                        &(((struct sockaddr_in *) &(rtent.rt_genmask))->
                          sin_addr.s_addr)))
            continue;

        strncpy(name, rt->rt_dev, sizeof(name));
        name[ sizeof(name)-1 ] = 0;

        /*
         * linux says ``lo'', but the interface is stored as ``lo0'': 
         */
        // Intel platform shows otherwise
        // if (!strcmp(name, "lo"))
        //     strcat(name, "0");

        rt->rt_flags = flags, rt->rt_pad3 = refcnt;
        // rt->rt_use = use, rt->rt_metric = metric;

        while (Interface_Scan_Next
               ((short *) &rt->rt_pad1, temp, NULL, NULL) != 0)
            if (strcmp(name, temp) == 0)
                break;

        /*
         *  Allocate a block to hold it and add it to the database
         */
        if (rtsize >= rtallocate) {
            rthead = (struct rtentry **) realloc((char *) rthead,
                                                 2 * rtallocate *
                                                 sizeof(struct rtentry *));
            memset(&rthead[rtallocate], 0,
                   rtallocate * sizeof(struct rtentry *));
            rtallocate *= 2;
        }
        if (!rthead[rtsize])
            rthead[rtsize] =
                (struct rtentry *) malloc(sizeof(struct rtentry));
        /*
         *  Add this to the database
         */
        memcpy((char *) rthead[rtsize], (char *) rt,
               sizeof(struct rtentry));
        rtsize++;
    }

    fclose(in);

    /*
     *  Sort it!
     */
    qsort((char *) rthead, rtsize, sizeof(rthead[0]), qsort_compare);
}
        
char *get_IpRouteIndex(char *key, int *notGetNext,
                       char *newIndex, int*append,
                       int *sameTable, int *RtIndex)
{
    oid             current[4], *op;
    u_char         *cp;

    char *opIn, *indxIn;
    int result;
    
    *append = 0;
    *sameTable = 1;
    opIn = strchr(key, '@');
    indxIn = strrchr(key, '@');

    *notGetNext = strncmp(opIn+1, "NEXT", strlen("NEXT"));
    
    // No index in the key
    if (indxIn == opIn)
    {
        // if it is not getNext, that is an error
        if (*notGetNext)
            return NULL;

        indxIn = NULL;
    }
    else // retrieve the index from key
        indxIn += 2; // skip @ and '.'    

    Route_Scan_Reload();

    for (*RtIndex = 0; *RtIndex < rtsize; (*RtIndex)++)
    {
        cp = (u_char *) &
            (((struct sockaddr_in *) &(rthead[*RtIndex]->rt_dst))->
             sin_addr.s_addr);
        op = current;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;

        sprintf(newIndex, "%lu.%lu.%lu.%lu",
                current[0], current[1], current[2], current[3]);

        if (indxIn == NULL)
        {
            if (*notGetNext)
                return NULL;
            // GetNext to the first entry in table
            // append the index to the key
            *append = 1;
            *sameTable = 1;            
            return newIndex;
        }

        result = strcmp(newIndex, indxIn);    

        if (result == 0)
        {
            // GET and SET has to be exact,
            // GetNext need to find the next one
            if (*notGetNext)
                return indxIn;
        }
        else if (result > 0)
        {
            // find the ipAddress after the one GETNEXT requested
            *append = -1;
            *sameTable = 1;
            return newIndex;
        }        
    }
    
    *sameTable = 0;
    return (NULL);
}

#define ARP_CACHE_INCR 128
static int      arptab_size, arptab_current;
static struct arptab *at = NULL;
static int      arptab_curr_max_size = 0;

static int ARP_Scan_Next(u_long * IPAddr, char *PhysAddr,
                         u_long * ifType, u_short * ifIndex)
{
    if (arptab_current < arptab_size) {
        /*
         * copy values 
         */
        *IPAddr = at[arptab_current].at_iaddr.s_addr;
        *ifType =
            (at[arptab_current].
             at_flags & ATF_PERM) ? 4 /*static */ : 3 /*dynamic */ ;
        *ifIndex = at[arptab_current].if_index;
        memcpy(PhysAddr, &at[arptab_current].at_enaddr,
               sizeof(at[arptab_current].at_enaddr));

        /*
         * increment to point next entry 
         */
        arptab_current++;
        /*
         * return success 
         */
        return (1);
    }

    return 0;    /* we need someone with an irix box to fix this section */
}

    
static void ARP_Scan_Init(void)
{
    static time_t   tm = 0;     /* Time of last scan */
    FILE           *in;
    int             i;
    char            line[128];
    int             za, zb, zc, zd, ze, zf, zg, zh, zi, zj;
    char            ifname[21];

    arptab_current = 0; /* Anytime this is called we need to reset 'current' */

    if (time(NULL) < tm + 1) {  /*Our cool one second cache implementation :-) */
        return;
    }

    in = fopen("/proc/net/arp", "r");
    if (!in) {
        arptab_size = 0;
        return;
    }

    fgets(line, sizeof(line), in);

    i = 0;
    while (fgets(line, sizeof(line), in)) {
        u_long          tmp_a;
        int             tmp_flags;
        if (i >= arptab_curr_max_size) {
            struct arptab  *newtab = (struct arptab *)
                realloc(at, (sizeof(struct arptab) *
                             (arptab_curr_max_size + ARP_CACHE_INCR)));
            if (newtab == at) {
                break;
            } else {
                arptab_curr_max_size += ARP_CACHE_INCR;
                at = newtab;
            }
        }
        if (12 !=
            sscanf(line,
                   "%d.%d.%d.%d 0x%*x 0x%x %x:%x:%x:%x:%x:%x %*[^ ] %20s\n",
                   &za, &zb, &zc, &zd, &tmp_flags, &ze, &zf, &zg, &zh, &zi,
                   &zj, ifname)) {
            continue;
        }
        /*
         * Invalidated entries have their flag set to 0.
         * * We want to ignore them 
         */
        if (tmp_flags == 0) {
            continue;
        }
        at[i].at_flags = tmp_flags;
        at[i].at_enaddr[0] = ze;
        at[i].at_enaddr[1] = zf;
        at[i].at_enaddr[2] = zg;
        at[i].at_enaddr[3] = zh;
        at[i].at_enaddr[4] = zi;
        at[i].at_enaddr[5] = zj;
        tmp_a = ((u_long) za << 24) |
            ((u_long) zb << 16) | ((u_long) zc << 8) | ((u_long) zd);
        at[i].at_iaddr.s_addr = htonl(tmp_a);
        at[i].if_index = Interface_Index_By_Name(ifname, strlen(ifname));
        i++;
    }
    arptab_size = i;

    fclose(in);
    time(&tm);
}
int get_IpAtIndex(char *key, int *notGetNext,
                  char *newIndex, int *append,
                  int *sameTable, u_long *lowIfIndex,
                  u_long *lowIfType, char *LowPhysAddr,
                  oid *LowAddr)
{
    u_char      *cp;
    oid         *op;
    oid          current[8];
    oid          name[5];
    oid          lowAddr[5];
    char         lowIndex[40];
    static char  PhysAddr[6];
    u_long       Addr, foundone;
    u_short      ifIndex;
    int          result, nameLen = 0;
    u_long       ifType;
    char *opIn, *indxIn;
    
    *lowIfIndex = 0;
    *lowIfType = 0;
    
    
    *append = 0;
    *sameTable = 1;
    opIn = strchr(key, '@');
    indxIn = strrchr(key, '@');

    *notGetNext = strncmp(opIn+1, "NEXT", strlen("NEXT"));
    memset(name, 0x00, 5 * sizeof(oid));
    memset(lowAddr, 0x00, 5 * sizeof(oid));
    
    // No index in the key
    if (indxIn == opIn)
    {
        // if it is not getNext, that is an error
        if (*notGetNext)
            return 0;

        indxIn = NULL;
    }
    else // retrieve the index from key
    {
        indxIn += 2; // skip @ and '.'

        nameLen = cnvtStr2Oid(indxIn, name, 5);
        
    }
    
    foundone = 0;
    
    ARP_Scan_Init();
    
    for (;;) {
        if (ARP_Scan_Next(&Addr, PhysAddr,
                          &ifType, &ifIndex) == 0)
            break;
        
        current[0] = ifIndex;
        
        cp = (u_char *) & Addr;
        op = current+1;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;
        *op++ = *cp++;

        sprintf(newIndex, "%lu.%lu.%lu.%lu.%lu",
                current[0], current[1], current[2],
                current[3], current[4]);
        
        // no index in OID
        if (indxIn == NULL)
        {
            if (*notGetNext)
                return 0;
            // GetNext to the first entry in table
            // append the index to the key
        }
        
        if ((result = oid_cmp(current, name, 5, nameLen)) == 0)
        {
            // GET and SET has to be exact,
            // GetNext need to find the next one
            if (*notGetNext)
            {
                *lowIfIndex = ifIndex;
                foundone = 1;
                *lowIfType = ifType;
                memcpy(LowPhysAddr, PhysAddr, sizeof(PhysAddr));
                memcpy(LowAddr, current, 5 * sizeof(oid));
                return 1;
            }
        }
        else if ((result > 0) &&
                 ((foundone == 0) ||
                  (oid_cmp(current, lowAddr, 5, 5) < 0)))
        {
            strcpy (lowIndex, newIndex);
            memcpy (lowAddr, current, 5 * sizeof(oid));

            // find the ipAddress after the one GETNEXT requested
            *append = -1;
            *sameTable = 1;
            *lowIfIndex = ifIndex;
            foundone = 1;
            *lowIfType = ifType;
            memcpy(LowPhysAddr, PhysAddr, sizeof(PhysAddr));
        }
    }

    if (foundone == 0)
    {
        *sameTable = 0;
        return 0;
    }
    else
    {
        memcpy (LowAddr, lowAddr, 5 * sizeof(oid));
        return 1;
    }
}

static int      numEntries = 0;

int register_sysORTable(oid * oidin, size_t oidlen,
                        const char *descr)
{
    struct sysORTable **ptr = &table;

    while (*ptr != NULL)
        ptr = &((*ptr)->next);
    *ptr = (struct sysORTable *) malloc(sizeof(struct sysORTable));
    if (*ptr == NULL) {
        return SYS_ORTABLE_REGISTRATION_FAILED;
    }
    (*ptr)->OR_descr = (char *) malloc(strlen(descr) + 1);
    if ((*ptr)->OR_descr == NULL) {
        free(*ptr);
        return SYS_ORTABLE_REGISTRATION_FAILED;
    }
    strcpy((*ptr)->OR_descr, descr);
    (*ptr)->OR_oidlen = oidlen;
    (*ptr)->OR_oid = (oid *) malloc(sizeof(oid) * oidlen);
    if ((*ptr)->OR_oid == NULL) {
        free(*ptr);
        free((*ptr)->OR_descr);
        return SYS_ORTABLE_REGISTRATION_FAILED;
    }
    memcpy((*ptr)->OR_oid, oidin, sizeof(oid) * oidlen);
    (*ptr)->OR_uptime = time(0);
    sysOR_lastchange = time(0);
    (*ptr)->next = NULL;
    numEntries++;

    return SYS_ORTABLE_REGISTERED_OK;
}

oid system_module_oid[] = { SNMP_OID_SNMPMODULES, 1 };
oid interfaces_module_oid[] = { SNMP_OID_MIB2, 2 };
oid ip_module_oid[] = { SNMP_OID_MIB2, 4 };


void init_mib(void)
{
    register_sysORTable(system_module_oid,
                        sizeof(system_module_oid)/sizeof(oid),
                        "The MIB module for SNMPv2 entities");
    register_sysORTable(interfaces_module_oid,
                        sizeof(interfaces_module_oid)/sizeof(oid),
                        "The MIB module for network interface sub-layers");
    register_sysORTable(ip_module_oid,
                        sizeof(ip_module_oid)/sizeof(oid),
                        "The MIB module for IP and ICMP implementations");    
}

