#ifndef __SNMP_MIB_H__
#define __SNMP_MIB_H__

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include "../../snmp/include/snmp_types.h"

#define RTA_NETMASK 0x04
#define RTA_IFP     0x10
#define RTA_BRD     0x80

#define RTM_NEWADDR 0x0c
#define RTM_IFINFO  0x0e

#define SNMP_OID_INTERNET		1, 3, 6, 1
#define SNMP_OID_ENTERPRISES	SNMP_OID_INTERNET, 4, 1
#define SNMP_OID_MIB2			SNMP_OID_INTERNET, 2, 1
#define SNMP_OID_SNMPV2			SNMP_OID_INTERNET, 6
#define SNMP_OID_SNMPMODULES		SNMP_OID_SNMPV2, 3

#define SYS_ORTABLE_REGISTERED_OK              0
#define SYS_ORTABLE_REGISTRATION_FAILED       -1

struct in_ifaddr {
    int  dummy;
};

struct ifnet {
    char           *if_name;       /* name, e.g. ``en'' or ``lo'' */
    char           *if_unit;       /* sub-unit for lower level driver */
    short           if_mtu;        /* maximum transmission unit */
    short           if_flags;      /* up/down, broadcast, etc. */
    int             if_metric;     /* routing metric (external only) */
    char            if_hwaddr[6];  /* ethernet address */
    int             if_type;       /* interface type: 1=generic,
                                    * 28=slip, ether=6, loopback=24 */
    u_long          if_speed;      /* interface speed: in bits/sec */
    
    struct sockaddr if_addr;       /* interface's address */
    struct sockaddr ifu_broadaddr; /* broadcast address */
    struct sockaddr ia_subnetmask; /* interface's mask */
    
    struct ifqueue {
        int             ifq_len;
        int             ifq_drops;
    } if_snd;              /* output queue */
    u_long          if_ibytes;     /* octets received on interface */
    u_long          if_ipackets;   /* packets received on interface */
    u_long          if_ierrors;    /* input errors on interface */
    u_long          if_iqdrops;    /* input queue overruns */
    u_long          if_obytes;     /* octets sent on interface */
    u_long          if_opackets;   /* packets sent on interface */
    u_long          if_oerrors;    /* output errors on interface */
    u_long          if_collisions; /* collisions on csma interfaces */
    /*
     * end statistics 
     */
    struct ifnet   *if_next;
};

typedef struct _conf_if_list {
    char           *name;
    int             type;
    u_long          speed;
    struct _conf_if_list *next;
} conf_if_list;

struct small_ifaddr {
    struct in_addr  sifa_addr;
    struct in_addr  sifa_netmask;
    struct in_addr  sifa_broadcast;
};

struct arptab {
    int             at_flags;
    char            at_enaddr[6];
    struct in_addr  at_iaddr;
    int             if_index;
};

struct sysORTable {
    char           *OR_descr;
    oid            *OR_oid;
    size_t          OR_oidlen;
    long            OR_uptime;
    struct sysORTable *next;
};

void init_mib(void);
struct sysORTable *get_ORIndex(char *key, int *notGetNext,
                               char *newIndex, int *append,
                               int *sameTable);
int Interface_Scan_Get_Count(void);
int get_Interface(char *key, int *notGetNext,
                  char *newIndex, int *append, int *sameTable);
int Interface_Scan_By_Index(int Index,
                            char *Name,
                            struct ifnet *Retifnet,
                            struct in_ifaddr *Retin_ifaddr);

char *get_IpAddrIndex(char *key, int *notGetNext,
                      char *newIndex, int *append,
                      int *sameTable, int *lowinterface,
                      struct ifnet *lowin_ifnet);
char *get_IpRouteIndex(char *key, int *notGetNext,
                       char *newIndex, int*append,
                       int *sameTable, int *RtIndex);
int get_IpAtIndex(char *key, int *notGetNext,
                  char *newIndex, int *append,
                  int *sameTable, u_long *lowIfIndex,
                  u_long *lowIfType, char *LowPhysAddr,
                  oid *LowAddr);

#endif
