
#include <stdlib.h>
#include <stdio.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "utility.h"

#define dprintf(...)

int getIfinfo(char *ifname ,char *buf,int type)
{
	struct ifreq ifr;
	struct sockaddr_in *saddr;
	int fd = -1;
	int ret = 0;
	
	
        fd = socket(AF_INET, SOCK_DGRAM, 0);
	
	if (fd < 0 )
		return -1;
	
			
	memset(&ifr,0,sizeof(ifr));
	strcpy(ifr.ifr_name, ifname);
	ifr.ifr_addr.sa_family = AF_INET;
		
	switch(type){
		case IF_MTU:
			if ((ret=ioctl(fd, SIOCGIFMTU, &ifr)) == 0){
				*(int *)buf=ifr.ifr_mtu;			
			}
			break;
		case IF_IPADDR:
			if ((ret=ioctl(fd, SIOCGIFADDR, &ifr)) == 0){
				saddr = (struct sockaddr_in *)&ifr.ifr_addr;
				*(int *)buf=*(unsigned long *)&saddr->sin_addr;			
				//strcpy(buf,inet_ntoa(saddr->sin_addr));
			}
			break;
		case IF_NETMASK:
			if ((ret=ioctl(fd, SIOCGIFNETMASK, &ifr)) == 0){
				saddr = (struct sockaddr_in *)&ifr.ifr_netmask;
				*(int *)buf=*(unsigned long *)&saddr->sin_addr;			
				//strcpy(buf,inet_ntoa(saddr->sin_addr));
			}
			break;
		case IF_BRDADDR:
			if ((ret=ioctl(fd, SIOCGIFBRDADDR, &ifr)) == 0){
				saddr = (struct sockaddr_in *)&ifr.ifr_broadaddr;
				*(int *)buf=*(unsigned long *)&saddr->sin_addr;			
				//strcpy(buf,inet_ntoa(saddr->sin_addr));
			}
			break;

		case IF_MAC:
			if ((ret=ioctl(fd,SIOCGIFHWADDR, &ifr)) == 0){
				sprintf(buf,"%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX"
						,ifr.ifr_hwaddr.sa_data[0]
						,ifr.ifr_hwaddr.sa_data[1]
						,ifr.ifr_hwaddr.sa_data[2]
						,ifr.ifr_hwaddr.sa_data[3]
						,ifr.ifr_hwaddr.sa_data[4]
						,ifr.ifr_hwaddr.sa_data[5]);

			}

			break;
		case IF_QLEN:
			if ((ret=ioctl(fd, SIOCGIFTXQLEN, &ifr)) == 0){
				*(int *)buf=ifr.ifr_qlen;			
			}

			break;
		default:
			ret=-1;
	}
			
	close(fd);
	return ret; 
}

int get_dev_info(char *ifname ,dev_info_t *info)
{
	char tmp[4096]="";
	FILE *fp;
	char *pt;
	fp=fopen("/proc/net/dev","r");
	if(!fp)
		return -1;
	fread(tmp,4096,1,fp);
	fclose(fp);
	if((pt=strstr(tmp,ifname))==NULL)
		return -1;

      	sscanf(pt,"%*[^:]:%lu %lu %lu %lu %*u %*u %*u %lu %lu %lu %lu %lu %*u %*u %*u %lu"
			,&info->in_bytes
			,&info->in_packets
			,&info->in_err
			,&info->in_drop
			,&info->in_mcast
			,&info->out_bytes
			,&info->out_packets
			,&info->out_err
			,&info->out_drop
			,&info->out_mcast);

	return 0;
}

int convertStr2PhysAddr(char *dest, char *src)
{
    char *del;
    char *string = src;
    int i, tmp;
    
    del = strchr(string, ':');
    for (i = 1; del != NULL; i++)
    {
        *del = 0;
        tmp = strtol(string,NULL,16);
        *dest = tmp & 0xff;
        dest++;
        string = del+1;
        del = strchr(string, ':');
    }

    if (strlen(string) != 0)
    {
        tmp = strtol(string,NULL,16);
        *dest = tmp & 0xff;
    }
    
    return i;
}

int get_snmp_info(snmp_info_t *info)
{
	FILE *fp;
	fp=fopen("/proc/net/snmp","r");
	if(fp){
		/* ip */
		while ( getc (fp) != '\n' );
		fscanf (fp,"%*[^:]:%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu"
				,&info->ip[0]
				,&info->ip[1]
				,&info->ip[2]
				,&info->ip[3]
				,&info->ip[4]
				,&info->ip[5]
				,&info->ip[6]
				,&info->ip[7]
				,&info->ip[8]
				,&info->ip[9]
				,&info->ip[10]
				,&info->ip[11]
				,&info->ip[12]
				,&info->ip[13]
				,&info->ip[14]
				,&info->ip[15]
				,&info->ip[16]
				,&info->ip[17]
				,&info->ip[18]);
		
		/* icmp */
		while ( getc (fp) != '\n' );
		while ( getc (fp) != '\n' );
		
		fscanf (fp,"%*[^:]:%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu"
				,&info->icmp[0]
				,&info->icmp[1]
				,&info->icmp[2]
				,&info->icmp[3]
				,&info->icmp[4]
				,&info->icmp[5]
				,&info->icmp[6]
				,&info->icmp[7]
				,&info->icmp[8]
				,&info->icmp[9]
				,&info->icmp[10]
				,&info->icmp[11]
				,&info->icmp[12]
				,&info->icmp[13]
				,&info->icmp[14]
				,&info->icmp[15]
				,&info->icmp[16]
				,&info->icmp[17]
				,&info->icmp[18]
				,&info->icmp[19]
				,&info->icmp[20]
				,&info->icmp[21]
				,&info->icmp[22]
				,&info->icmp[23]
				,&info->icmp[24]
				,&info->icmp[25]);

		/* tcp */
		while ( getc (fp) != '\n' );
		while ( getc (fp) != '\n' );
		
		fscanf (fp,"%*[^:]:%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu"
				,&info->tcp[0]
				,&info->tcp[1]
				,&info->tcp[2]
				,&info->tcp[3]
				,&info->tcp[4]
				,&info->tcp[5]
				,&info->tcp[6]
				,&info->tcp[7]
				,&info->tcp[8]
				,&info->tcp[9]
				,&info->tcp[10]
				,&info->tcp[11]
				,&info->tcp[12]
				,&info->tcp[13]);

		/* udp */
		while ( getc (fp) != '\n' );
		while ( getc (fp) != '\n' );
		
		fscanf (fp,"%*[^:]:%lu %lu %lu %lu"
				,&info->udp[0]
				,&info->udp[1]
				,&info->udp[2]
				,&info->udp[3]);
		fclose(fp);
	}
	return 0;
}

int get_route_info(route_table_t *rt,int item)
{
	FILE *fp;
	int i=0;
	char tmp[1024]="";
	fp=fopen("/proc/net/route","r");
	if(!fp)
		return -1;
	
	fgets(tmp,1024,fp);
	while(!feof(fp)){
		if(fgets(tmp,1024,fp)==NULL)
			break;;
		i++;
		if(i==item)
			break;
	}
	fclose(fp);
	if(i!=item){
		return -1;
	}
	
        sscanf(tmp, "%s %x %x %*s %*s %*s %d %x %*d %*d %*d\n",
                        rt->name,
			&rt->dest.sin_addr.s_addr,
			&rt->gateway.sin_addr.s_addr,
			&rt->metric,
			&rt->mask.sin_addr.s_addr
			);
	
	return 0;
}

int get_arp_info(struct arp_info *info,int item)
{
	FILE *fp;
	char tmp[1024];
	int i=0;
	
	fp=fopen("/proc/net/arp","r");
	if(!fp)
		return -1;
	fgets(tmp,1024,fp);
	while(!feof(fp)){
		if(fgets(tmp,1024,fp)==NULL)
			break;;
		i++;
		if(i==item)
			break;
	}

	fclose(fp);
	if(i!=item){
		return -1;
	}
	
        sscanf(tmp, "%s %*s %*s %s %*s %s"
			,info->ipaddr
			,info->mac
			,info->ifname);
	

	return 0;
}
