/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *
 *   \file  init.c
 *   \brief PURPOSE: initialize wcfg library
 *
 */


#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <sys/ioctl.h>
#include <net/if_arp.h>         /* For ARPHRD_ETHER */
#include <sys/socket.h>         /* For AF_INET & struct sockaddr */
#include <netinet/in.h>         /* For struct sockaddr_in */
#include <netinet/if_ether.h>

#include <linux/wireless.h>


#include "wcfglib.h"


//wcfgd *tiap_wcfg_init(wcfginit *);

int main(int argc, char *argv[]) 
{
	wcfgd *wd = NULL;
	wcfginit init;
	char driver_load_failed;

	if( argc < 4 || argc > 6 ) {
		fprintf(stderr, "usage: %s wpa_auth module wlan0 [br0] [fw trace app]\n", argv[0]);
		return 1;
	}

	memset(&init, 0, sizeof(init));

#ifndef MWCODE
	strncpy(init.storage_file, WCFG_MTDBLOCK, sizeof(init.storage_file));
	init.storage_file[sizeof(init.storage_file)-1] = '\0';
#endif /* MWCODE */

	strncpy(init.wpaauth_file, argv[1], sizeof(init.wpaauth_file));
	init.wpaauth_file[sizeof(init.wpaauth_file)-1] = '\0';

	strncpy(init.apmodule_file, argv[2], sizeof(init.apmodule_file));
	init.apmodule_file[sizeof(init.apmodule_file)-1] = '\0';

	strncpy(init.dev_name, argv[3], sizeof(init.dev_name));
	init.dev_name[sizeof(init.dev_name)-1] = '\0';

	if( argc >= 5 ) {
		strncpy(init.br_name, argv[4], sizeof(init.br_name));
		init.br_name[sizeof(init.br_name)-1] = '\0';
	}

#ifdef TIWLAN_FW_TRACE
	if ( argc != 6) {
		puts("Need FW trace app filename !!!");
		return 1;
	} else
		strncpy(init.fw_trace_app_filename, argv[5], sizeof(init.fw_trace_app_filename));
#endif

	if( !(wd =  tiap_wcfg_init(&init, &driver_load_failed)) ) {
		return 1;
	}

	tiap_wcfg_light_deinit(wd);
	printf("\ninit: lightly deinitializing wcfglib\n\n"); fflush(NULL);

	return 0;
}

