/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */


/*!
 *
 *   \file  wcfg_ioctl.c
 *   \brief PURPOSE: wcfg_ioctl.c header file
 *
 */

#ifndef WCFG_IOCTL_H
#define WCFG_IOCTL_H

#include <linux/config.h>

#ifdef CONFIG_NET_RADIO
#include <linux/wireless.h>
#else
#error Linux wireless LAN is not enabled
#endif /* CONFIG_NET_RADIO */

#include "wcfg_storage.h"
#include "wcfglib.h"

/*!	\def MAX_FW_SIZE
  	\brief max size of buffer to read from firmware file
 */
#define MAX_FW_SIZE 200000

/*!	\def IW_MAX_PRIV_DEF
  	\brief max size of device ioctls table
 */
#define IW_MAX_PRIV_DEF	128


int wcfg_ioctl_init(wcfgd *wcfg_data, tiap_param_eeprom_t * eeprom);

int ioctl_get_tx_info(int Qid, int QprintEn, char * buf, int buf_len, char * ifname);
int ioctl_get_challenge(int index, char * buf, int buf_len, char * ifname);
int ioctl_get_find_sta(char * mac, char * buf, int buf_len, char * ifname);
int ioctl_get_insert_sta(char * mac, int wone_index, char * buf, int buf_len, char * ifname);
int ioctl_get_tx_queue(int Qid, char * buf, int buf_len, char * ifname);
int ioctl_get_eer(int addr, char * buf, int buf_len, char * ifname);
int ioctl_get_mac_reg_read(int addr, char * buf, int buf_len, char * ifname);
int ioctl_get_phy_reg_read(int addr, char * buf, int buf_len, char * ifname);
int ioctl_get_trace_print(int num, int from, char * buf, int buf_len, char * ifname);
int ioctl_get_wlan_params(int wone_index, char *buf, int buf_len, char *ifname);
int ioctl_get_max_rates(__u8, __u8 *, char *);
int ioctl_set_burst_limit(__u32 burst_limit, int ioctl_sock, char *ifname);
int ioctl_get_burst_limit(char *buf, int buf_len, int ioctl_sock, char *ifname);
#ifdef REPORT_DEBUG
int ioctl_get_logged_chars(char *buf, int buf_len, int ioctl_sock, char *ifname);
int ioctl_set_logged_chars(__u32 logged_chars, int ioctl_sock, char *ifname);
#endif

int exec_priv_ioctl_get (const __u32 we_id, void *param, int param_size, int ioctl_sock, char *ifname);
int exec_priv_ioctl_set (const __u32 we_id, void *param, int param_size, int ioctl_sock, char *ifname);
int ioctl_get_essid (char *essid, int ioctl_sock, char *ifname);
int ioctl_get_rts(__s32 *rts,int ioctl_sock, char *ifname);
int ioctl_get_frag(__s32 *frag, int ioctl_sock, char *ifname);
int ioctl_get_retry(__s32 *retry, int type, int ioctl_sock, char *ifname);
int ioctl_get_txpow(tiap_param_txpower_t *txpow, int ioctl_sock, char *ifname);

int ioctl_set_go(wcfgd *wcfg_data, int go_stage);

int ioctl_set_wds_pid(char *pDevName, pid_t pid);

#endif
