 /***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	ap_port.c	      										   */
/*   PURPOSE:	ap_port functions.	      			 				   */
/*																									*/
/***************************************************************************/
#include <stdarg.h>
#include "utils.h"
#include "wpaIfc.h"
#include <wpa.h>
#include <netinet/in.h>
#include <stdio.h>
#ifndef VXWORKS
#include "linux_ipc_user.h"
#else
#include "pform_types.h"
#endif
#include "dda_wpa.h"

/*-----------------------------------------------------------*/
/*         Local Prototypes									 */
/*-----------------------------------------------------------*/


int send_eapol(auth1x_pc_t *t, void *buf, int len, int encrypt, u16 eth_type);

/*-----------------------------------------------------------*/
/*         Implementation									 */
/*-----------------------------------------------------------*/

/****************************************************************************
 *                      wdrv_wpaApPortInitCallbacks()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_wpaApPortInitCallbacks(auth1x_pc_ops_t *portCBs)
{	memset(portCBs, 0, sizeof(auth1x_pc_ops_t));

	portCBs->send_eapol		= send_eapol;

	portCBs->mode_set		= NULL;
	portCBs->mac_auth		= NULL;
	portCBs->mac_unauth		= NULL;
	portCBs->pass_single_unauth_set		= NULL;
	portCBs->pass_single_unauth_get		= NULL;
	portCBs->mode_get					= NULL;
	portCBs->mac_get_next				= NULL;
	portCBs->use_count_get				= NULL;

	return OK;
 
}

/* send the specified EAPoL packet through the device. Packet should
 * contain ethernet header. Returns number of bytes sent, -1 on error. 
 
   TODO: remove ethernet type argument - obsolete */
int send_eapol(auth1x_pc_t *t, void *buf, int len, int encrypt, u16 eth_type)
{
	wpaMessage_T	msg;
#ifndef VXWORKS
	int rc;
#endif

	msg.type = EAPOL_FRAME;
	memcpy(msg.content.eapolFrame.pEapol,buf,len);
	msg.content.eapolFrame.encrypted = encrypt;
	msg.content.eapolFrame.eapolDataLen = len;

#ifdef VXWORKS
	dda_wpa_receive(&msg, sizeof(msg));
#else
	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("send_eapol:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return NOK;
	}
#endif

	return len;
}


