/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*											    		    				*/
/*	 MODULE:	hiddenSecurityProfileAPI.h					   			   */
/*   PURPOSE:	hidden Security Profile API functions   				   */
/*												    						*/
/***************************************************************************/
/***************************************************************************/
#ifndef HIDDEN_PROF_API_H
#define HIDDEN_PROF_API_H

#include  "802_11Defs.h"
#ifdef VXWORKS
#include "socket.h"
#else /* ifdef VXWORKS */
#include <sys/socket.h>
#include <sys/un.h>
#endif /* ifdef VXWORKS */

enum SEC_AUTH_STATUS_e
{
	SEC_AUTH_STATUS_OK,
	SEC_AUTH_STATUS_FAILURE
};

typedef struct
{
	UINT8  secPartyMac[MAC_ADDR_LEN];
}dynamicSecureCfg_T;

typedef void (* secAuthStatusCB_T) (UINT8 *pMac, enum SEC_AUTH_STATUS_e status);

typedef struct
{
	UINT8 secPortEnabled;
	UINT8 maxNumAssocs;
    UINT32 reportMsgType;
    /* Socket creation parameters */
    int domain;
    int type;
    int protocol;
    /* Binding parameters */
#ifdef VXWORKS
    struct sockaddr_in addr;
#else /* ifdef VXWORKS */
    struct sockaddr_un addr;
#endif /* ifdef VXWORKS */
    int addrLen;
	UINT8  pskPasswd[MAX_PSK_PWD_LEN];
	UINT32 pskPasswdLen;
} initHiddenProfConf_T;

/* Report security result to some one */
typedef struct {
    UINT32 msgType;     /* From initHiddenProfConf_T->reportMsgType */
    UINT8  linkPartnerMac[MAC_ADDR_LEN];  /* Link partner MAC address */
    UINT32 result;                         /* Authentication result OK or NOK */
    UINT32 failReson;                      /* Failure details */
} hiddenProfSecurityResp_s;


u32 hidden_profile_Init(initHiddenProfConf_T *initParams);
u32 hidden_profile_SecureAuthStart(dynamicSecureCfg_T *linkParams);
u32 hidden_profile_SecureSuplStart(dynamicSecureCfg_T *linkParams);
u32 hidden_profile_SecureAbort(dynamicSecureCfg_T *linkParams);
u32 hidden_profile_AuthLinkClose(dynamicSecureCfg_T *linkParams);
u32 hidden_profile_SuplLinkClose(dynamicSecureCfg_T *linkParams);


#endif /* ifndef HIDDEN_PROF_API_H */
