#ifndef VXWORKS
#include <stdio.h>
#include <stdlib.h>
#include <asm/types.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <net_utils.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include "linux_ipc_user.h"

int ipc_accept_buf_read(int fd, void *data, int data_len) {
	int res;
	struct sockaddr_nl sanl;
	socklen_t sanllen;

	sanllen = sizeof(struct sockaddr_nl);
	res = recvfrom(fd, data, data_len, MSG_DONTWAIT, (struct sockaddr*)&sanl, &sanllen);

	if (res <= 0) {
		if(res != -1)	{
			printf("res=%d\n",res);
			perror(__FUNCTION__);
		}
		return -1;
	}

	return 0;
}

int ipc_bind_listen_port(unsigned short port) {
	int fd;
	struct sockaddr_nl      local;

	fd = socket(AF_NETLINK, SOCK_DGRAM, NETLINK_USERSOCK);
	if (fd < 0) {
		perror(__FUNCTION__);
		return -1;
	}

	memset(&local, 0, sizeof(local));
	local.nl_family = AF_NETLINK;
	//	local.nl_groups = RTMGRP_LINK;

	if (bind(fd, (struct sockaddr*)&local, sizeof(local)) < 0) {
		perror(__FUNCTION__);
		return -1;
	}

	return fd;
}

int ipc_connect_buf_write(int fd, void *data, int data_len) {
	
	struct sockaddr_nl  danl;

    danl.nl_family = AF_NETLINK;
	danl.nl_pad = 0;
	danl.nl_pid = 0;		// target in kernel
	danl.nl_groups = 0;

	return sendto(fd, data, data_len, 0, (struct sockaddr *)&danl, sizeof(struct  sockaddr_nl));
}

/******************************************************************************
*                      ipc_create_sock()								  *
*******************************************************************************
DESCRIPTION:	Creates and binds a socket for interprocess communication in LINUX
                                                                         
INPUTS:			sock_path - socket name

OUTPUT:			pSock 	  - an output buffer for socket

RETURNS:		OK/NOK                                             
******************************************************************************/
int ipc_create_sock(char *sock_path, int *pSock)
{

	int  sock;
    struct sockaddr_un name;

    /* Create socket from which to read. */
    sock = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (sock < 0) {
        printf("WPA HIDDEN INST.:error opening datagram socket for WDS\n");
        return NOK;
    }
    /* Create name. */
    name.sun_family = AF_UNIX;

    strcpy((char*)name.sun_path,sock_path);

    unlink(sock_path);

    /* Bind the  domain address to the created socket */
    if (bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0 )
	{
		  printf("WPA: error binding datagram socket\n");
          return NOK;
    }

	*pSock = sock;
	return OK;

}

#endif
