/* LICENSE_CODE JUNGO */

#include <event.h>
#include <rg_memory.h>
#include <openssl/rc4.h>
#include <rg_error.h>
#include <802.1x/wep_port.h>
#include <kos/knet.h>
#include "auth1x_conn.h"
#include "wpa_key_utils.h"
#include "wpa_port.h"
#include "wpa_assoc.h"
#include "wpa_log.h"
#include "wpa_common.h"
#include "wpa_pmk_cache.h"
#include "hiddenSecurityProfileAPI.h"
#include "debug_report.h"

/*********/

#define HANDSHAKE_MAX_RETRIES 3

/* Time to wait for a station response for an EAPOL packet before retrying */
#define  PTK_HANDSHAKE_TIMEOUT 1000 /* ms */
#define  GTK_HANDSHAKE_TIMEOUT 2000 /* ms */

/* If we're not getting any data from station within this period since the
* station associates, we assume it's a WEP-legacy station
*/
#define WPA_WAIT_FOR_FIRST_PACKET_TIMEOUT 2000 /* ms */

#define TEST_VER2BIT  0x0002


#define KEY_INFO_INDEX_BITS 4 /* position of bits from right */

#define SET_BIT(info, mask) ((info) |= (mask))
#define TEST_BITS(info, mask) (((info) & (mask)) == (mask))
#undef MIN
#define MIN(a, b) ((a)<(b)?(a):(b))

#define TSC_MASK 0x00FFFFFF /* 6 least significant bytes of the TSC are used as 
* RSC
*/

/* Tests if the key descriptor indicates a MIC failure report */
#define IS_MIC_FAILURE_REPORT(key_desc) \
    TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT | \
KEY_INFO_ERROR_BIT | KEY_INFO_MIC_BIT))

/* Tests if the key descriptor indicates a request for a 4-way-handshake/group 
* key update
*/
#define IS_KEY_REQUEST(key_desc) \
    (TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT)) && \
    !TEST_BITS(key_desc->info, htons(KEY_INFO_ERROR_BIT)) && \
!TEST_BITS(key_desc->info, htons(KEY_INFO_MIC_BIT)))

/* A WPA2 GTK IE to be sent in Message3 of 4-way handshake key descriptor */
typedef struct
{
    u8 type;
    u8 length;
    u8 oui[3];
    u8 datatype;
    u8 keyid;
    u8 reserved;
    u8 gtk[WPA_TKIP_KEY_LEN];
} __attribute__ ((packed)) eapol_gtk_ie_t;

typedef struct
{
    u8 type;
    u8 length;
    u8 oui[3];
    u8 datatype;
    u8 pmk_id[PMK_CACHE_ID_SIZE];
} __attribute__ ((packed)) eapol_pmk_id_ie_t;

#define EAPOL_IE_GTK_DATA_TYPE      0x1;
#define EAPOL_IE_PMK_ID_DATA_TYPE   0x4;


typedef enum {
    WPA_START_PTK = 0,
		WPA_NEGOTIATING_PTK = 1,
		WPA_START_GTK = 2,
} eapol_key_type_t;

static u8 rsnoui[3] = {0x00, 0x0F, 0xAC};

BOOL	micFailureReportFromStation = TRUE;

u8* DefaultIV(void);


static void calc_mic_on_eapol_pkt(wpa_assoc_t *assoc, eapol_pkt_t *eapol_pkt, 
								  wpa_eapol_mic_t *mic)
{
	if(assoc->sta_type != WPA_STA_TYPE_WPA2)
	{
		wpa_key_util_hmac_md5((u8 *)mic, (u8 *)&eapol_pkt->proto_ver, 
			ntohs(eapol_pkt->pkt_body_len) + 4,
			(u8 *)&assoc->ptk.eapol_key_mic_key, sizeof(wpa_eapol_mic_t));
	}
	else
	{
		u8 buff[20]; // need spare space !!!
		//		printf("calc mic with hmac_sha1 func \n");
		hmac_sha1((u8 *)&buff, (u8 *)&eapol_pkt->proto_ver, 
			ntohs(eapol_pkt->pkt_body_len) + 4, /* TODO: 128 func ???*/
			(u8 *)&assoc->ptk.eapol_key_mic_key, sizeof(wpa_eapol_mic_t));
		
		// truncate the MIC to the actual size
		memcpy((u8 *)&mic->data, (u8 *)&buff, sizeof(wpa_eapol_mic_t));
	}
}

/* Create an EAPOL-key packet */
eapol_pkt_t *create_eapol_key_pkt(wpa_assoc_t *assoc, int key_info_bits, 
										 int key_index, u64 rsc, int key_len, wpa_nonce_t *nonce, u8 *iv, 
										 u8 *key_data, int key_data_len)
{
    eapol_pkt_t *eapol_pkt;
    eapol_key_desc_t *key_desc;
    int eapol_pkt_len;
    int key_desc_len;
    RC4_KEY rc4_key;
    u8 rc4_key_data[32];
    u8 dummy[256];
	
    key_desc_len = sizeof(eapol_key_desc_t) + key_data_len - 1;
    eapol_pkt_len = sizeof(eapol_pkt_t) + key_desc_len - 1;
    eapol_pkt = (eapol_pkt_t *)zalloc_l(eapol_pkt_len + 8);
    if (!eapol_pkt)
		return NULL;
	
    /* EAPOL packet header */
    memcpy(eapol_pkt->dest, &assoc->sta_mac, sizeof(ether_mac_t));
    memcpy(eapol_pkt->src, &assoc->port->params.mac, sizeof(ether_mac_t));
    eapol_pkt->ether_type = htons(EAPOL_TYPE);
    eapol_pkt->proto_ver = EAPOL_VER;
    eapol_pkt->pkt_type = ET_KEY;
    eapol_pkt->pkt_body_len = htons(key_desc_len); 
	
    /* Key descriptor header */
    key_desc = (eapol_key_desc_t *)eapol_pkt->pkt_body;
    /* check this field value !!!*/
	if(assoc->sta_type == WPA_STA_TYPE_WPA2 )
		key_desc->type = KEY_DESC_TYPE_WPA2;
	else
		key_desc->type = KEY_DESC_TYPE;
	
	/* todo :we dont need this ifs i think !!!*/
	if(key_info_bits & KEY_INFO_VERSION_2_BIT)
	{
		//		printf("setting eapol version to 2 in eapol packet \n");
		key_desc->info = key_info_bits | KEY_INFO_VERSION_2_BIT;
	}
    if(key_info_bits & KEY_INFO_VERSION_1_BIT)
	{
		//		printf("setting eapol version to 1 in eapol packet \n");
		key_desc->info = key_info_bits | KEY_INFO_VERSION_1_BIT;
	}
	if (key_index)
		key_desc->info |= (key_index << KEY_INFO_INDEX_BITS);
    key_desc->info = htons(key_desc->info);
	
    /* Note that key_len and key_data_len have different meanings. key_len is
	* the length of the key that should be eventually installed to station/AP
	* while key_data_len is the length of the data actually included in this
	* message.
	*/
    key_desc->len = htons(key_len); 
	
    /* Increment replay counter and put it in packet */
    assoc->ap_to_sta_replay_count++;
    key_desc->replay_count = knet_htonll(assoc->ap_to_sta_replay_count);
	
    memcpy(&key_desc->nonce, nonce, sizeof(key_desc->nonce));
	
    key_desc->rsc = rsc;
	
    if (iv)
        memcpy(&key_desc->iv, iv, sizeof(key_desc->iv));
	
    key_desc->data_len = htons(key_data_len);
    
    /* In EAPOL-key messages which are part of the GTK update, the key is stored
	* encrypted with RC4
	*/
	
    if (
		(!(key_info_bits & KEY_INFO_PAIRWISE_BIT)) &&
		(assoc->sta_type == WPA_STA_TYPE_WPA )
		)
		/* RC4 key = Key IV || EAPOL-Key encryption key */
	{
		//		printf("create eapol , pair_wise bit is off, use rc4!!!\n");
		memcpy(rc4_key_data, key_desc->iv, sizeof(key_desc->iv));
		memcpy(&(rc4_key_data[sizeof(key_desc->iv)]), 
			&assoc->ptk.eapol_key_encryption_key,
			sizeof(assoc->ptk.eapol_key_encryption_key));
		
		/* Initialize RC4 key */
		RC4_set_key(&rc4_key, sizeof(rc4_key_data), rc4_key_data);
		
		/* Skip first 256 bytes of key stream (spec says so) */
		RC4(&rc4_key, sizeof(dummy), dummy, dummy);
		
		/* Encrypt GTK */
		RC4(&rc4_key, key_data_len, (u8 *)&assoc->port->gtk, key_desc->data);
	}
	
    else
    {
		/* Copy key data to packet */
		if (key_data_len > 0)
			memcpy(key_desc->data, key_data, key_data_len);
    }
	
	/* Calculate MIC if it should be included in the packet */
    if (key_info_bits & KEY_INFO_MIC_BIT)
	{
		//		printf("create_eapol_key_pkt:calc mic on eapol packet !\n");
		calc_mic_on_eapol_pkt(assoc, eapol_pkt, &key_desc->mic);
    }
	
    return eapol_pkt;	
}

/* Check whether a replay counter contained in a key descriptor is valid */
static int replay_verify(wpa_assoc_t *assoc, eapol_key_desc_t *key_desc)
{
	int result = 1;
	
	/* The replay counter is verified differently for replys and requests */
	if (TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT)))
	{
		/* For request EAPOL-Key messages, replay counter must be monotonic */
		if (!assoc->sta_to_ap_replay_count || 
			knet_ntohll(key_desc->replay_count) > assoc->sta_to_ap_replay_count)
		{
			result = 0;
			
			/* Remember the last replay counter we received */
			assoc->sta_to_ap_replay_count = knet_ntohll(key_desc->replay_count);
		}
	}
	else
	{
	/* For reply EAPOL-Key messages, the replay counter in the received 
	* EAPOL-Key must be identical to the replay counter in the last 
	* EAPOL-Key we sent
		*/
		if (assoc->ap_to_sta_replay_count == knet_ntohll(key_desc->replay_count))
			result = 0;
	}
	
	if (result)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Invalid replay counter in message received from station %s", 
			mactoa(&assoc->sta_mac)); 
	}
	
	return result;
}

int mic_check(wpa_assoc_t *assoc, eapol_pkt_t *eapol_pkt)
{
    wpa_eapol_mic_t mic_recv, mic_calc;
    eapol_key_desc_t *key_desc = (eapol_key_desc_t *)eapol_pkt->pkt_body;
    int result;
	
    memcpy(&mic_recv, &key_desc->mic, sizeof(wpa_eapol_mic_t));
    
    /* MIC should be calculated on packet when its MIC field is zero */
    MZERO(key_desc->mic);
    
    calc_mic_on_eapol_pkt(assoc, eapol_pkt, &mic_calc);
	
    result = memcmp(&mic_recv, &mic_calc, sizeof(wpa_eapol_mic_t));
	
    if (result)
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Bad MIC detected in EAPOL-Key packet\n");
	
    return result;
}

static void ptk_neg_timeout_cb(void *context);
static void gtk_neg_timeout_cb(void *context);

/* Sends an EAPOL-key packet */
static void send_eapol_key_pkt(wpa_assoc_t *assoc, 
							   eapol_pkt_t *eapol_pkt, int is_ptk_neg_pkt)
{
    etimer_func_t timeout_cb = is_ptk_neg_pkt ? ptk_neg_timeout_cb : 
							   gtk_neg_timeout_cb;
	double timeout = is_ptk_neg_pkt ? PTK_HANDSHAKE_TIMEOUT : GTK_HANDSHAKE_TIMEOUT;


	event_timer_set(timeout, timeout_cb, assoc);

	/* Send EAPOL packet. If a pairwise key is installed, use it to encrypt 
	* EAPOL-Key packet, otherwise send it unencrypted.
	*/
	auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, 
							   assoc->pairwise_cipher ? 1 : 0);
	
	free(eapol_pkt);
}

/* create msg in 4 way handshake(cause its different between wpa and wpa2*/
static eapol_pkt_t *create_msg3_eapol_key_pkt(wpa_assoc_t *assoc)
{
	if(assoc->sta_type == WPA_STA_TYPE_WPA2)
	{
		eapol_gtk_ie_t    gtk_ie;
		u8 key_data[256];  
		u32 key_data_length = 0; 
		int gtk_key_size ;
		
		u8 key_data_after_aes[1024];
		u32 out_key_data_length = 0;
		u8 *key_iv;
		memset(&gtk_ie, 0, sizeof(gtk_ie));
		gtk_ie.type = 0xDD;
		gtk_ie.length = sizeof(gtk_ie) - 2;
		memcpy((u8 *)gtk_ie.oui, (u8 *)rsnoui, sizeof(rsnoui));
		gtk_ie.datatype = EAPOL_IE_GTK_DATA_TYPE; // GTK_IE
		gtk_ie.keyid = assoc->port->current_gtk_index;    
		gtk_ie.reserved = 0;
		
		
		if(assoc->port->mix_mode_flag)
		{
			gtk_key_size = WPA_TKIP_KEY_LEN;
		}
		else
		{	
			gtk_ie.length -= WPA_AES_KEY_LEN;
			gtk_key_size = WPA_AES_KEY_LEN;
		}
		
		memcpy((char*)gtk_ie.gtk , (u8 *)&assoc->port->gtk , gtk_key_size);  // Fill in GTK
		
		memset(key_data, 0, sizeof(key_data));
		
		if(assoc->sta_type == WPA_STA_TYPE_WPA)
		{
			memcpy((u8 *)key_data, (u8 *)assoc->port->ap_wpa_info_elem, wpa_info_elem_get_len(assoc->port->ap_wpa_info_elem));
			key_data_length += wpa_info_elem_get_len(assoc->port->ap_wpa_info_elem);
		}
		else /* wpa2 station*/
		{
			memcpy((u8 *)key_data, (u8 *)assoc->port->ap_rsn_info_elem, wpa_info_elem_get_len(assoc->port->ap_rsn_info_elem));
			key_data_length += wpa_info_elem_get_len(assoc->port->ap_rsn_info_elem);
		}
		
		memcpy((u8 *)key_data + key_data_length, (u8 *)&gtk_ie, sizeof(gtk_ie));
		key_data_length += gtk_ie.length + 2 /*sizeof(gtk_ie)*/;
		
		
		if (key_data_length % 8)
		{
			key_data[key_data_length] = 0xDD;
			key_data_length += 8 - (key_data_length % 8);
		}
		
		out_key_data_length = key_data_length   + 8;	
				
		key_iv = DefaultIV();		
		out_key_data_length = key_data_length   + 8;	
		
		if(
			wpa_aes_Encrypt((u8*)key_data_after_aes,
			(u8*)key_data,key_data_length,
			(u8 *)&assoc->ptk.eapol_key_encryption_key,
			WPA_AES_KEY_LEN,
			key_iv) == FALSE
			)
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\n@@ ERROR encrypt eapol key data with  AES wrap @@\n");
		}
		
		
		return  create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_2_BIT|
			KEY_INFO_INSTALL_BIT | KEY_INFO_ACK_BIT | KEY_INFO_MIC_BIT |
			KEY_INFO_SECURE_BIT | KEY_INFO_ENCRYPTED_KEY_DATA_BIT,
			0, 0,
			16/*gtk_key_size*/, &assoc->nonce, key_iv, (u8 *)key_data_after_aes,
			out_key_data_length);
	}
	
	else  /* no wpa2 station*/
		return create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_1_BIT |
		KEY_INFO_INSTALL_BIT | KEY_INFO_ACK_BIT | KEY_INFO_MIC_BIT, 0, 0,
		WPA_TKIP_KEY_LEN, &assoc->nonce, 0, (u8 *)assoc->port->ap_wpa_info_elem,
		wpa_info_elem_get_len(assoc->port->ap_wpa_info_elem));
	
	
}
/* PTK (4 way handshake) negotiation uses the following states:
* WPA_START_MSG1, WPA_WAIT_MSG2, WPA_SEND_MSG3, WPA_WAIT_MSG4, WPA_DONE */
static void wpa_assoc_ptk_neg(wpa_assoc_t *assoc, eapol_pkt_t *in_eapol_pkt)
{
    eapol_key_desc_t *key_desc = NULL;
    eapol_pkt_t *out_eapol_pkt;
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"ptk neg receive event STA %02X:%02X:%02X:%02X:%02X:%02X!!!!\n",
				  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
				  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
	
	/* PMK could have become invalid since 4-way handshake started*/
	if (!assoc->pmk_valid) 
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"PMK is not valid, terminating the 4-way handshake");
		return;
	}
    /* eapol_pkt points to a received EAPOL packet when the function is called
	* in response to a received packet, or NULL when the function is called to
	* initiate a new 4-way handshake or to retry sending an EAPOL packet 
	*/
    if (in_eapol_pkt)
    {
		key_desc = (eapol_key_desc_t *)in_eapol_pkt->pkt_body;
		
		/* Verify replay counter, ignore packet if invalid */
		if (replay_verify(assoc, key_desc))
			return;
		
			/* In case the last message we sent was message 1, the message we just
			* received is supposed to be message 2. We must check the MIC on the
			* received message before moving to the next state, but before we can
			* check the MIC we must calculate the PTK
		*/
		if(assoc->ptk_state == WPA_START_MSG1)
		{
			/* Derive PTK */
			wpa_key_utils_calc_ptk(&assoc->ptk, &assoc->pmk, &assoc->nonce, 
				&key_desc->nonce, assoc->port->params.mac, assoc->sta_mac);
		}
		
		/* Ignore packet if it has a bad MIC */
		if (mic_check(assoc, in_eapol_pkt))
			return;
		else
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa: mic check success\n");
		}
		
		event_timer_del(ptk_neg_timeout_cb, assoc);
		
		/* Check if the station reported of a unicast key MIC failure */
		if (IS_MIC_FAILURE_REPORT(key_desc))
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Station %s reports of unicast key MIC "
				"failure", mactoa(&assoc->sta_mac));
			
			/* Tell the port to handle the MIC failure */
			if (micFailureReportFromStation)
				wpa_port_mic_failure(assoc->port, assoc);
			return;
		}
		
	
		/* Check if the station requested a 4-way handshake */
		if (IS_KEY_REQUEST(key_desc))
		{
			//		printf("4-way handshake request received from station %s",
			
			//	mactoa(&assoc->sta_mac));
			
			/* Initiate a new 4-way handshake */
			wpa_assoc_start_ptk_neg(assoc);
			return;
		}
		
		assoc->ptk_state++;
		assoc->ptk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
	
    }/* eapol key */
	
    switch (assoc->ptk_state) 
    {
        case WPA_START_MSG1:        
		/* Generate ANonce */
		wpa_port_generate_nonce(assoc->port, &assoc->nonce);
		
		/* Create new EAPOL packet */
		if(assoc->sta_type == WPA_STA_TYPE_WPA)
		{
			out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_1_BIT |
				KEY_INFO_ACK_BIT, 0, 0, WPA_TKIP_KEY_LEN, &assoc->nonce, 0, NULL, 
				0);
		}
		else /* wpa2 ,handle use of pmk cache*/
		{
			eapol_pmk_id_ie_t  pmk_id_ie;
			u8 *key_data = NULL;
			int key_data_len = 0;

			/* case pmk cache was used ,we need to had it to key data.*/
			if(assoc->pmk_from_cache == 1)
			{
				pmk_id_ie.type = 0xdd;
				pmk_id_ie.length =  PMK_CACHE_ID_SIZE + 4;
				memcpy((u8 *)pmk_id_ie.oui, (u8 *)rsnoui, sizeof(rsnoui));
				pmk_id_ie.datatype = EAPOL_IE_PMK_ID_DATA_TYPE;
				memcpy(pmk_id_ie.pmk_id,assoc->pmk_cach_id,PMK_CACHE_ID_SIZE );
				key_data = (u8*)&pmk_id_ie;
				key_data_len = pmk_id_ie.length + 2;
			}
			out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_2_BIT |
				KEY_INFO_ACK_BIT, 0, 0, WPA_AES_KEY_LEN, &assoc->nonce, 0, key_data, 
				key_data_len);
         }
		if (!out_eapol_pkt)
			goto Error;
		/* Send EAPOL packet */
		send_eapol_key_pkt(assoc, out_eapol_pkt, 1);
		break;
    case WPA_WAIT_MSG2:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Received message 2 of 4-way handshake");
		
		/* Information element received in packet must be identical to the one
		* received in the station association request 
		*/
		if (memcmp(assoc->sta_info_elem, key_desc->data, 
			MIN(wpa_info_elem_get_len(assoc->sta_info_elem), 
			ntohs(key_desc->data_len)))) 
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Information element in message 2 of "
				"4-way handshake doesn't match one in station's association "
				"request.\nExpected (length=%d):\n%s\nReceived "
				"(length=%d):\n%s", 
				wpa_info_elem_get_len(assoc->sta_info_elem), 
				wpa_log_format_buf((u8 *)assoc->sta_info_elem, 
				wpa_info_elem_get_len(assoc->sta_info_elem)),
				ntohs(key_desc->data_len),
				wpa_log_format_buf(key_desc->data, key_desc->data_len));
			
			goto Error;
		}
		
		assoc->ptk_state++;
		
		/* Fall Through */
		/* WPA_WAIT_MSG2 and WPA_SEND_MSG3 are two separate states because we should
		* be able to resend message 3 (on retries) without doing the parsing on
		* message 2 (in the retransmission case, message 2 would be NULL)
		*/
    case WPA_SEND_MSG3:
		/* Create new EAPOL packet */
		out_eapol_pkt = create_msg3_eapol_key_pkt(assoc);
		
		if (!out_eapol_pkt)
			goto Error;
		
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Sending message 3 of 4-way handshake");
		
		/* Send EAPOL packet */
		send_eapol_key_pkt(assoc, out_eapol_pkt, 1);
		
		if(assoc->sta_type == WPA_STA_TYPE_WPA2)
			assoc->has_gtk = 1;
		break;
		
    case WPA_WAIT_MSG4:	

		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Received message 4 of 4-way handshake");
		
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Installing pairwise key for station %s",
				mactoa(&assoc->sta_mac));
		
		/* Install pairwise key to AP */
		if(assoc->sta_type == WPA_STA_TYPE_WPA)
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"TKIP encryption key=\n%s",
				wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.tkip.tkip_key, 
				sizeof(assoc->ptk.temporal_key.data.tkip.tkip_key)));
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"TX MIC key=\n%s",
				wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.tkip.tx_mic_key, 
				sizeof(assoc->ptk.temporal_key.data.tkip.tx_mic_key)));
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"RX MIC key=\n%s",
				wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.tkip.rx_mic_key, 
				sizeof(assoc->ptk.temporal_key.data.tkip.rx_mic_key)));
			
			assoc->port->params.ap_dev_ops->set_mapping_key(assoc->port->port_profile_index	, 
				assoc->sta_mac, WPA_CIPHER_TKIP, &assoc->ptk.temporal_key);
			
			assoc->pairwise_cipher = WPA_CIPHER_TKIP;
		}
		
		else if(assoc->sta_type == WPA_STA_TYPE_WPA2)
		{	
			if(WPA_TKIP_KEY_LEN == assoc->ptk.temporal_key.dataSize )
				assoc->pairwise_cipher = WPA_CIPHER_TKIP;
			else
				assoc->pairwise_cipher = WPA_CIPHER_CCMP;
			
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"AES key=\n%s",
				wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.aes.key, 
				sizeof(assoc->ptk.temporal_key.data.aes.key)));

			assoc->port->params.ap_dev_ops->set_mapping_key(assoc->port->port_profile_index, 
				assoc->sta_mac, assoc->pairwise_cipher, &assoc->ptk.temporal_key);
			
		   /* This station have just acknowledged receiving the GTK. Check if
			* gtk update started during the station association and now is in process
			* for this port. In case it was the last station, we can install the new GTK now.
			*/
			assoc->gtk_state = WPA_DONE;
			wpa_port_gtk_neg_end(assoc->port, assoc, 1);

		}
		
		/* Once a pairwise key is installed to the AP, we can stop blocking
		* traffic going to the station
		*/
		assoc->port->params.ap_dev_ops->block_unicast_traffic(assoc->port->ap_dev, 0, assoc->sta_mac);
		assoc->ptk_state = WPA_DONE;
		
		wpa_port_ptk_neg_end(assoc->port, assoc, 1);
		
		break;
		
		/* XXX Is this case really needed? */
    case WPA_DONE:
		break;
    }
	
    return;
	
Error:
    /* Notify port that the 4-way handshake failed */
    wpa_port_ptk_neg_end(assoc->port, assoc, 0);
}

/* GTK negotiation uses the following states:
* WPA_START_MSG1, WPA_WAIT_MSG2, WPA_DONE */
static void wpa_assoc_gtk_neg(wpa_assoc_t *assoc, eapol_pkt_t *in_eapol_pkt)
{
	
    eapol_key_desc_t *key_desc;
    u8 iv[IV_FIELD_LEN];
    u64 rsc = 0;
    eapol_pkt_t *out_eapol_pkt;
	
    /* eapol_pkt points to a received EAPOL packet when the function is called
	* in response to a received packet, or NULL when the function is called to
	* initiate a new group key update or to retry sending an EAPOL packet
	*/
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH," start gtk shake for STA %02X:%02X:%02X:%02X:%02X:%02X!!!!\n",
				  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
				  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
    if (in_eapol_pkt)
    {
		key_desc = (eapol_key_desc_t *)in_eapol_pkt->pkt_body;
		
		/* Ignore message if replay counter is invalid */
		if (replay_verify(assoc, key_desc))
			return;
		
		/* Ignore packet if it has a bad MIC */
		if (mic_check(assoc, in_eapol_pkt))
			return;
		
		event_timer_del(gtk_neg_timeout_cb, assoc);
		
		/* Check if the station reported of a group key MIC failure */
		if (IS_MIC_FAILURE_REPORT(key_desc))
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Station %s reports of group key MIC failure", 
				mactoa(&assoc->sta_mac));
			
			/* Tell the port to handle the MIC failure */
			if (micFailureReportFromStation)
				wpa_port_mic_failure(assoc->port, assoc);
			return;
		}
		
		/* Check if the station requested a group key update */
		if (IS_KEY_REQUEST(key_desc))
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Group key update request received from station %s", 
				mactoa(&assoc->sta_mac));
			
			wpa_port_gtk_update_request(assoc->port, assoc);
			return;
		}
		
		assoc->gtk_state++;
		assoc->gtk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
    }
	
    switch (assoc->gtk_state) 
    {
    case WPA_START_MSG1:
		{
			/* RSC field should contain TSC for TKIP or 0 for WEP */
			if (assoc->port->multicast_cipher == WPA_CIPHER_TKIP)
			{
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH," port multicast cipher WPA_CIPHER_TKIP \n");
				/* XXX This is splitted into 2 lines because of B7579 */
				rsc = (u64)((u32)assoc->port->params.ap_dev_ops->get_tsc(assoc->port->ap_dev) &
					 TSC_MASK);
				rsc = knet_cpu_to_le64(rsc);
			}
			
			/* For the key IV we use the lower 16 bytes of the key counter. Our key
			* counter is always big endian so we take its last 16 bytes
			*/
			memcpy(iv, &assoc->port->key_counter.data[16], sizeof(iv));
			
			/* Create EAPOL packet */
			if(assoc->sta_type  == WPA_STA_TYPE_WPA)
			{
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"creating gtk eapol for wpa1\n");
				out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_ACK_BIT | KEY_INFO_VERSION_1_BIT |
					KEY_INFO_MIC_BIT | KEY_INFO_SECURE_BIT, 
					assoc->port->current_gtk_index, rsc, 
					wpa_key_utils_get_key_len(assoc->port->multicast_cipher),
					&assoc->port->gnonce, iv, (u8 *)&assoc->port->gtk, 
					wpa_key_utils_get_key_len(assoc->port->multicast_cipher));
			}
			else
			{
				eapol_gtk_ie_t    gtk_ie;
				u8 key_data[256];  //should be enough for RSN IE and GTK IE
				u32 key_data_length = 0;  // actual length of key_data
				int gtk_key_size;
				
				u8 key_data_after_aes[1024];
				u32 out_key_data_length = 0;
				u8 *key_iv;

				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"start bulding gtk eapol msg \n"); 
				/* Prepare GTK IE*/
				memset(&gtk_ie, 0, sizeof(gtk_ie));
				gtk_ie.type = 0xDD;
				gtk_ie.length = sizeof(gtk_ie) - 2;
				memcpy((u8 *)gtk_ie.oui, (u8 *)rsnoui, sizeof(rsnoui));
				gtk_ie.datatype = 1; /* GTK_IE */
				gtk_ie.keyid = assoc->port->current_gtk_index;
				gtk_ie.reserved = 0;
				
				if(assoc->port->mix_mode_flag)
					gtk_key_size = WPA_TKIP_KEY_LEN;
				else
				{	
					gtk_ie.length -= WPA_AES_KEY_LEN;
					gtk_key_size = WPA_AES_KEY_LEN;
				}
				
				memcpy((char*)gtk_ie.gtk , (u8 *)&assoc->port->gtk , gtk_key_size);  // Fill in GTK
				
				memset(key_data, 0, sizeof(key_data));
				
				memcpy((u8 *)key_data, (u8 *)&gtk_ie, sizeof(gtk_ie));
				key_data_length += gtk_ie.length + 2 /*sizeof(gtk_ie)*/;
				
				if (key_data_length % 8)
				{
					key_data[key_data_length] = 0xDD;
					key_data_length += 8 - (key_data_length % 8);
				}
				
				out_key_data_length = key_data_length   + 8;	
				
				key_iv = DefaultIV();
				if(
					wpa_aes_Encrypt((u8*)key_data_after_aes,
					(u8*)key_data,key_data_length,
					(u8 *)&assoc->ptk.eapol_key_encryption_key,
					WPA_AES_KEY_LEN,
					key_iv) == FALSE
					)
				{
					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\n@@ ERROR encrypt eapol key data with  AES wrap @@\n");
				}
				
				
				out_eapol_pkt =  create_eapol_key_pkt(assoc,  KEY_INFO_VERSION_2_BIT |
					KEY_INFO_ACK_BIT | KEY_INFO_MIC_BIT |
					KEY_INFO_SECURE_BIT | KEY_INFO_ENCRYPTED_KEY_DATA_BIT,
					0, 0,
					gtk_key_size, &assoc->nonce, key_iv, (u8 *)key_data_after_aes,
					out_key_data_length);
			}
			if (!out_eapol_pkt)
				goto Error;
			
				/* Once we sent the GTK to the station, it potentially has the key. We
				* intentionally update this flag as early as now and not when the 
				* station acknowledges receiving the key, to handle the case where
				* a hostile station interested in sniffing the network may request us 
				* for a key and not acknowledge receiving in order to confuse us.
			*/
			assoc->has_gtk = 1;
			
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Sending message 1 of group key update STA = %02X:%02X:%02X:%02X:%02X:%02X ",
						  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
						  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
			
			/* Send EAPOL packet */
			send_eapol_key_pkt(assoc, out_eapol_pkt, 0);
			break;
		}
		
    case WPA_WAIT_MSG2:
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Received message 2 of group key update STA = %02X:%02X:%02X:%02X:%02X:%02X\n",
						  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
						  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
			
			assoc->gtk_state = WPA_DONE;
			
			/* Notify port that group key update with this station succeeded */
			wpa_port_gtk_neg_end(assoc->port, assoc, 1);
			break;
			
			/* XXX Is this case really needed? */
		}
    case WPA_DONE:
		break;
		
    default:
		/* Should never reach here */
		break;
    }
	
    return;
	
Error:
    /* Notify port the the group key update with this station failed */
    wpa_port_gtk_neg_end(assoc->port, assoc, 0);
    return;
}

static void send_pairwise_key_to_8021x_sta(wpa_assoc_t *assoc)
{
    eapol_pkt_t *eapol_pkt;
    u8 *pmk = NULL;
    int pmk_len;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Sending pairwise key to %s in 802.1X EAPOL-Key", 
		mactoa(&assoc->sta_mac));
	
		/* Get the PMK that has been created as result of the 802.1X authentication
	*/
    auth1x_conn_pmk_get(assoc->auth1x_conn, &pmk, &pmk_len);
	
    /* Create an EAPOL-Key packet that only specifies the length of the key, 
	* the station will use the PMK as its unicast key
	*/
    eapol_pkt = eap_rekey_pkt_create(&assoc->sta_mac, &assoc->port->params.mac,
		NULL, 
		wpa_key_utils_get_key_len(assoc->port->params.rekeying_wep_cipher), 0,
		1, pmk, 0);
	
    if (!eapol_pkt)
    {
		/* Notify port that we failed to send the key */
		wpa_port_ptk_neg_end(assoc->port, assoc, 0);
		return;
    }
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Installing %s pairwise key for station %s. Key=\n%s",
		wpa_log_cipher_to_str(assoc->port->params.rekeying_wep_cipher), 
		mactoa(&assoc->sta_mac), wpa_log_format_buf(pmk,
		wpa_key_utils_get_key_len(assoc->port->params.rekeying_wep_cipher)));
    
    assoc->port->params.ap_dev_ops->set_mapping_key(assoc->port->port_profile_index, 
		assoc->sta_mac, assoc->port->params.rekeying_wep_cipher, 
		(wpa_temporal_key_t *)pmk);
	
    assoc->pairwise_cipher = assoc->port->params.rekeying_wep_cipher;
	
    /* Unicast traffic to station can now be encrypted using the unicast key, so
	* we can stop blocking it
	*/
    assoc->port->params.ap_dev_ops->block_unicast_traffic(assoc->port->ap_dev, 
		0, assoc->sta_mac);
	
    /* Send EAPOL-Key packet */
    auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, 0);
	
    free(eapol_pkt);
	
    /* Notify port that the key was sent successfully */
    wpa_port_ptk_neg_end(assoc->port, assoc, 1);
}

/* Starts PTK (4 way handshake) negotiation */
void wpa_assoc_start_ptk_neg(wpa_assoc_t *assoc)
{
	if (!assoc->pmk_valid)
    {
		wpa_port_ptk_neg_end(assoc->port, assoc, 0);
		return;
    }
	
    /* Use 4-way handshake to send unicast key to WPA stations or a single
	* EAPOL-Key to send it to WEP-802.1X stations 
	*/
    if (assoc->sta_type == WPA_STA_TYPE_WPA || assoc->sta_type == WPA_STA_TYPE_WPA2)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_assoc_start_ptk_neg: starting wpa/ ptk nag\n");
		assoc->ptk_state = WPA_START_MSG1;
		assoc->ptk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
		
		wpa_assoc_ptk_neg(assoc, NULL);
    }
    else
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_assoc_start_ptk_neg: starting 8021x  key nag\n");
		send_pairwise_key_to_8021x_sta(assoc);
	}
}

/* Sends GTK in a legacy 802.1X EAPOL-Key message */
static void send_group_key_to_8021x_sta(wpa_assoc_t *assoc)
{
    eapol_pkt_t *eapol_pkt;
    u8 *pmk = NULL;
    int pmk_len;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Sending group key to %s in 802.1X EAPOL-Key", 
		mactoa(&assoc->sta_mac));
	
	/* Get the PMK that has been created as result of the 802.1X authentication
	*/
    auth1x_conn_pmk_get(assoc->auth1x_conn, &pmk, &pmk_len);
    if (!pmk)
		return;
	
    /* Create an EAPOL-Key packet containing the GTK */
    eapol_pkt = eap_rekey_pkt_create(&assoc->sta_mac, &assoc->port->params.mac, 
		(u8 *)&assoc->port->gtk, 
		wpa_key_utils_get_key_len(assoc->port->multicast_cipher), 
		assoc->port->current_gtk_index, 0, pmk, 0);
	
    if (!eapol_pkt)
    {
		/* Notify port that we failed to send the group key */
		wpa_port_gtk_neg_end(assoc->port, assoc, 0);
		return;
    }
	
    /* Send EAPOL-Key packet */
    auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, assoc->pairwise_cipher ? 1 : 0);
	
    free(eapol_pkt);
	
    /* We sent the key to the station, we'll never know if it got it or not
	* (because there is no ack defined in the protocol) but we have to assume 
	* that it potentially has the key
	*/
    assoc->has_gtk = 1;
	
    /* Notify port the the group key update with this station succeeded */
    wpa_port_gtk_neg_end(assoc->port, assoc, 1);
}

/* Starts GTK negotiation */
void wpa_assoc_start_gtk_neg(wpa_assoc_t *assoc)
{
/* We send the GTK to WPA stations by doing group key update and to 
* WEP-802.1X stations by using a single EAPOL-Key message
	*/
    if (assoc->sta_type == WPA_STA_TYPE_WPA || assoc->sta_type == WPA_STA_TYPE_WPA2)
    {
		assoc->gtk_state = WPA_START_MSG1;
		assoc->gtk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
		wpa_assoc_gtk_neg(assoc, NULL);
    }
    else
	/* We want the completion function called from within
	* send_group_key_to_8021x_sta to be called only after the current
	* execution thread finishes
	*/
	event_timer_set(0, (etimer_func_t)send_group_key_to_8021x_sta, assoc);
}

/* Receive and process EAPoL-key  */
static void wpa_assoc_eapol_key_rcv(void *context, eapol_pkt_t *eapol_pkt,
									int is_encrypted)
{
    wpa_assoc_t *assoc = context;
	
    /* Check if this EAPOL-Key is part of 4-way handshake or group key update 
	* and pass it to the correct state machine
	*/
    if (TEST_BITS(((eapol_key_desc_t *)eapol_pkt->pkt_body)->info, 
		ntohs(KEY_INFO_PAIRWISE_BIT)))
    {
		wpa_assoc_ptk_neg(assoc, eapol_pkt);
    }
    else
		wpa_assoc_gtk_neg(assoc, eapol_pkt);
}

/* 802.1x authentication end callback */
static void wpa_assoc_auth1x_end(void *context, auth1x_conn_t *conn, 
								 int is_authenticated, u8 *pmk, int pmk_len,double session_timeout)
{
    wpa_assoc_t *assoc = context;
	
    if (!is_authenticated)
		goto Exit;
	
    if (pmk && pmk_len >= sizeof(wpa_pmk_t))
    {
		memcpy(&assoc->pmk, pmk, sizeof(wpa_pmk_t));
		assoc->pmk_valid = 1;
    }
    else
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: 802.1X authentication produced a PMK which is too"
			"small: %d bytes, expected at least %d bytes", pmk_len, 
			sizeof(wpa_pmk_t));
		
		is_authenticated = 0;
    }
	
Exit:
    wpa_port_8021x_auth_end(assoc->port, assoc, is_authenticated,session_timeout); 
}

/* Called when the station type is detected to update the association object and
* the port counters
*/
void wpa_assoc_set_sta_type(wpa_assoc_t *assoc, wpa_sta_type_t sta_type)
{
/* In some cases we assume the station is of a certain type and then change
* the type when we find out we were wrong, so update the relevant counter
	*/
    if (assoc->sta_type)
		wpa_port_update_sta_counter(assoc->port, sta_type, -1);
    
    assoc->sta_type = sta_type;
	
    /* Count station by its new type */
    wpa_port_update_sta_counter(assoc->port, sta_type, 1);
}

/* Called when no data is received from a station since it associated for a
* defined timeout
*/
static void wait_for_first_packet_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = (wpa_assoc_t *)context;
    
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Didn't receive data from station %s. It's assumed to be "
		"WEP-legacy station", mactoa(&assoc->sta_mac));
    
		/* Nothing has been received from the station yet, we assume it's a legacy
		* station (a 802.1X station would have sent EAPOL-start packet by now)
	*/
    wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_LEGACY);
	
    /* Now that we know the station type, let the port do type-specific handling
	* for the station
	*/
    wpa_port_sta_type_detected(assoc->port, assoc);
}

/* Called when we found out whether a certain WEP station is legacy or 802.1X */
void wpa_assoc_sta_type_detected(wpa_assoc_t *assoc)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station %s detected to be %s", mactoa(&assoc->sta_mac), 
		wpa_log_sta_type_to_str(assoc->sta_type));
    
    if (assoc->sta_type == WPA_STA_TYPE_WEP_LEGACY)
    {
		if (assoc->auth1x_conn)
		{
		/* There is no point in trying to do 802.1X authentication with a
		* non-802.1X station so close 802.1X connection
			*/
			auth1x_conn_close(assoc->auth1x_conn);
			assoc->auth1x_conn = NULL;
		}
		
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Unblocking unicast traffic to station %s", 
			mactoa(&assoc->sta_mac));
		
			/* No authentication is required with legacy stations, so we can stop 
			* blocking unicast traffic to the station now
		*/
		assoc->port->params.ap_dev_ops->block_unicast_traffic(
			assoc->port->ap_dev, 0, assoc->sta_mac);
    }
	
    /* We just found out the station type so we no longer need the timeout */
    event_timer_del(wait_for_first_packet_timeout_cb, assoc);
}

static void ptk_neg_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = context;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Timeout waiting for reply from station %02X:%02X:%02X:%02X:%02X:%02X!!!!\n",
				  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
				  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
    
    /* Retry sending the last packet */
    if (assoc->ptk_neg_retries_left--)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Retrying to send EAPOL-Key message retries left =%d ", assoc->ptk_neg_retries_left);
		
		wpa_assoc_ptk_neg(assoc, NULL);
		
		return;
    }
	
    /* Number of retries exceeded, notify port the 4-way handshake failed */
    wpa_port_ptk_neg_end(assoc->port, assoc, 0);
}


/* Called when a station doesn't respond to one of our messages */
static void gtk_neg_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = context;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Timeout waiting for reply from station %02X:%02X:%02X:%02X:%02X:%02X!!!!\n",
				  assoc->sta_mac[0],assoc->sta_mac[1],assoc->sta_mac[2],
				  assoc->sta_mac[3],assoc->sta_mac[4],assoc->sta_mac[5]);
    
    /* Retry sending the last packet */
    if (assoc->gtk_neg_retries_left--)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Retrying to send EAPOL-Key message, retries left =%d ",
					  assoc->gtk_neg_retries_left);
		
		wpa_assoc_gtk_neg(assoc, NULL);
		
		return;
    }
	
    /* Number of retries exceeded, notify port that the group key update
	* failed
	*/
    wpa_port_gtk_neg_end(assoc->port, assoc, 0);
}

/* Terminate 802.1X connection  */
static void wpa_assoc_auth1x_conn_term(void *context, auth1x_conn_t *conn)
{

    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"802.1x: terminating connection to station %s\n", 
					mactoa(auth1x_conn_mac_get(conn)));
	/* 802.1X connection would be closed upon receiving notification from driver about 
	   STA deauthentication */
	wpa_assoc_deauth(context);
}


/* Callbacks that the 802.1X package will use to notify us of important events
*/
static auth1x_conn_cb_t auth1x_conn_cb = {
    wpa_assoc_eapol_key_rcv,
		wpa_assoc_auth1x_end,
	wpa_assoc_auth1x_conn_term
	
};

/* Opens a new association */
wpa_assoc_t *wpa_assoc_open(wpa_port_t *port, ether_mac_t sta_mac, 
							wpa_info_elem_t *sta_info_elem , UINT16 ether_type)
{
    wpa_assoc_t *assoc = (wpa_assoc_t *)zalloc_l(sizeof(wpa_assoc_t));

    if (!assoc)
		return NULL;
    
    /* Fill new association context */
    assoc->port = port;
    memcpy(assoc->sta_mac, sta_mac, sizeof(ether_mac_t));
	
	assoc->pre_auth_conn = 0;
	assoc->pmk_from_cache = 0;
    /* indicate that the assoc is not participating in current gtk update if any in progress */
	assoc->in_gtk_update = 0;

    if (sta_info_elem)
    {
		/* This station has an information element so it's a WPA station(1 or 2 ?) */
		if(wpa_info_elem_get_info_type(sta_info_elem) == WPA_STA_TYPE_WPA)
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"setting station type wpa1\n");
			wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WPA);
			assoc->ptk.temporal_key.dataSize = wpa_key_utils_get_key_len(WPA_CIPHER_TKIP);
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"connection key data size = %d\n",assoc->ptk.temporal_key.dataSize);
		}
		else
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"setting station type wpa2\n");
			wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WPA2);
			assoc->ptk.temporal_key.dataSize = 16;
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"connection key data size = %d\n",assoc->ptk.temporal_key.dataSize);
		}
		/* set ptk len*/
		
	}
    /* If WEP-legacy stations are allowed and WEP-802.1X are not allowed, we
	* will assume that this non-WPA station is a WEP-legacy station instead
	* of waiting for the first packet coming from it and telling the 
	* station type by it. If we didn't use this assumption we would have to
	* block traffic to this station for up to 2 seconds.
	*/
    else if ((assoc->port->params.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) 
		&& !(assoc->port->params.allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
    {
		wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_LEGACY);
    }
    /* If both WEP-legacy and WEP-802.1X stations are allowed, we still have
	* no way to tell whether it's a 802.1X station or a legacy station so 
	* we wait for the first packet it sends us. If no packet arrives within
	* 2 seconds, we assume it's a legacy station because a 802.1X is 
	* expected to send us an EAPOL packet right after association. Traffic
	* going to this station is blocked until we detect its type.
	*/ 
    else
    {
		event_timer_set(WPA_WAIT_FOR_FIRST_PACKET_TIMEOUT,
			wait_for_first_packet_timeout_cb, assoc);
    }
	
    /* Do stuff which is relevant only for WPA stations */
    if (assoc->sta_type == WPA_STA_TYPE_WPA || assoc->sta_type == WPA_STA_TYPE_WPA2)
    {
		/* Save station's information element */
		assoc->sta_info_elem =  zalloc_l(wpa_info_elem_get_len(sta_info_elem));
		if (!assoc->sta_info_elem)
			goto Error;
		
		memcpy(assoc->sta_info_elem, sta_info_elem, 
			wpa_info_elem_get_len(sta_info_elem));
		
		/* If we're working in PSK mode, set PMK to be the PSK */
		if (assoc->port->params.psk_param != WPA_PSK_PARAM_NONE)
		{
			memcpy(&assoc->pmk, &assoc->port->psk, sizeof(assoc->pmk));
			
			assoc->pmk_valid = 1;
			
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Using PSK as PMK, PMK =\n%s", 
				wpa_log_format_buf((u8 *)&assoc->pmk, sizeof(assoc->pmk)));
		}
    }
	
    /* Open a 802.1X connection for types of stations that need it. This 
	* includes also stations which we don't know their type yet, because they 
	* might be WEP-802.1X stations, so we must be prepared to handle incoming
	* EAPOL-Start messages from those stations.
	*/
    if (!assoc->sta_type || (assoc->sta_type & (WPA_STA_TYPE_WPA | WPA_STA_TYPE_WPA2|
		WPA_STA_TYPE_WEP_8021X)))
    {
		/* open regular connection (not pre-authentication) so the eth type = 0x888e */
		assoc->auth1x_conn = auth1x_conn_open(assoc->port->params.auth1x_port, 
			&assoc->sta_mac, &auth1x_conn_cb,ether_type ,assoc);
		if (!assoc->auth1x_conn)
			goto Error;
		

		/* Unless this is a WPA-PSK station, we also have to be prepared for the
		* station starting 802.1X authentication
		*/
		if (!((assoc->sta_type == WPA_STA_TYPE_WPA ||assoc->sta_type == WPA_STA_TYPE_WPA2)  && 
			(assoc->port->params.psk_param != WPA_PSK_PARAM_NONE)))
		{
			u8 *pmk_id_list;
			u8 num_of_pmk_id = 0;
			/* check if we have pmk id in the station rsn*/
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH," checking if station has pmk in our list !!!\n");
			if(wpa_info_elem_get_pmk_id(sta_info_elem,&pmk_id_list,&num_of_pmk_id) == 0)
			{
				wpa_pmk_t sta_pmk;

				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"station found PMK %s!!!\n",wpa_log_format_buf((u8 *)pmk_id_list, 16));
				/*
				if we have pmk id in the rsn,and we will find
				it in our pmk cache list ,we dont need to start authentication server
				connection.
				todo :add loop for pmk_id_list
				*/
				if(wpa_pmk_cache_check_pmk_valid(port,pmk_id_list,&sta_pmk) == OK)
				{
					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"station has valid PMK !!!\n");
					memcpy(&assoc->pmk, &sta_pmk, sizeof(assoc->pmk));
					memcpy(assoc->pmk_cach_id,pmk_id_list,PMK_CACHE_ID_SIZE);
					assoc->pmk_valid = 1;
					assoc->pmk_from_cache = 1;
					if ( (wpa_pmk_cache_update_assoc(port, pmk_id_list, assoc)) != OK)
					{
						DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH, "%s: failed to update assoc in pmk cache\n",__func__);
						goto Error;

					}
					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Using PMK CACHE as PMK, PMK =\n%s", 
						wpa_log_format_buf((u8 *)&assoc->pmk, sizeof(assoc->pmk)));
				}
			}
			
            /* case no pmk found*/
			if(assoc->pmk_valid != 1)
				auth1x_conn_auth_start(assoc->auth1x_conn);
		}
    }
    
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"finish open assoc !!!\n");
	
    return assoc;
	
Error:
    if (assoc->auth1x_conn)
		auth1x_conn_close(assoc->auth1x_conn);
    
    if (assoc->sta_info_elem)
		wpa_info_elem_close(assoc->sta_info_elem);
	
    event_timer_del(wait_for_first_packet_timeout_cb, assoc);
    
    nfree(assoc);
	
    return NULL;
}

/* Disassociates a station */
void wpa_assoc_disassoc(wpa_assoc_t *assoc)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Disassociating station %s", mactoa(&assoc->sta_mac));
    
    /* Disassociate station */
    assoc->port->params.ap_dev_ops->disassoc_sta(assoc->port->ap_dev, 
		assoc->sta_mac);
}

/* Deauthenticates a station */
void wpa_assoc_deauth(wpa_assoc_t *assoc)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Deauthenticating station %s", mactoa(&assoc->sta_mac));
    
    /* Deauthenticate station */
    assoc->port->params.ap_dev_ops->deauth_sta(assoc->port->ap_dev, 
		assoc->sta_mac);
}

/* Cleans up and frees an association object */
void wpa_assoc_close(wpa_assoc_t *assoc)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Removing pairwise key of station %s", mactoa(&assoc->sta_mac)); 
    
    /* Remove PTK from AP */
    if (assoc->pairwise_cipher)
    {
		assoc->port->params.ap_dev_ops->remove_mapping_key(assoc->port->ap_dev,
			assoc->sta_mac, assoc->pairwise_cipher);
		
		assoc->pairwise_cipher = 0;
    }
	
    event_timer_del(ptk_neg_timeout_cb, assoc);
    event_timer_del(gtk_neg_timeout_cb, assoc);
    event_timer_del(wait_for_first_packet_timeout_cb, assoc);
	
    if (assoc->auth1x_conn)
		auth1x_conn_close(assoc->auth1x_conn);
	
    if (assoc->sta_info_elem)
		wpa_info_elem_close(assoc->sta_info_elem);
	
    free(assoc);
}

int wpa_assoc_is_in_gtk_handshake(wpa_assoc_t *assoc)
{
    /* Only WPA stations may be in the process of a GTK update */
    if (assoc->sta_type != WPA_STA_TYPE_WPA && assoc->sta_type != WPA_STA_TYPE_WPA2)
		return 0;
    
    return assoc->gtk_state == WPA_START_MSG1;
}

void wdrv_wpaSetMicFailureReportFromStation(BOOL value)
{
	micFailureReportFromStation = value;
}

BOOL wdrv_wpaGetMicFailureReportFromStation(void)
{
	return micFailureReportFromStation;
}

auth1x_conn_t * openAuthxConn(wpa_assoc_t *assoc)
{
	/* open pre-authentication,  eth type = 0x88c7 (unic type for pre-authentication packets)*/
	return auth1x_conn_open(assoc->port->params.auth1x_port, 
				&assoc->sta_mac, &auth1x_conn_cb,EAPOL_ETHER_PRE_AUTH_TYPE ,assoc);

}

/* Prints PMKSA for the given assoc */
void printPMKSA(wpa_assoc_t *assoc)
{
	printf("PMK ID: %s\n", wpa_log_format_buf(assoc->pmk_cach_id, PMK_CACHE_ID_SIZE));
	printf("PMK = %s\n", wpa_log_format_buf((u8 *)&assoc->pmk, sizeof(assoc->pmk)));
	printf("AKMP: %s\n", (assoc->port->params.psk_param == WPA_PSK_PARAM_NONE) ? "802.1x auth" : "PSK");
	printf("PMK from cache: %s\n", (assoc->pmk_from_cache ? "True" : "False"));
	printf("Preauth connection: %s\n", (assoc->pre_auth_conn ? "True" : "False"));

}

/* Prints PTKSA for the given assoc */
void printPTKSA(wpa_assoc_t *assoc)
{
	printf("Pairwise cipher suite = 0x%02X\n", assoc->pairwise_cipher); 
	if (assoc->ptk_state == WPA_DONE) 
	{
		if (assoc->sta_type == WPA_STA_TYPE_WPA) 
		{
			printf("PTK = %s\n", wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.tkip.tkip_key, 
													sizeof(assoc->ptk.temporal_key.data.tkip.tkip_key)));
		}
		else if (assoc->sta_type == WPA_STA_TYPE_WPA2) 
		{
			printf("PTK = %s\n", wpa_log_format_buf((u8 *)assoc->ptk.temporal_key.data.aes.key, 
				sizeof(assoc->ptk.temporal_key.data.aes.key)));
		}
	}
	else 
	{
		printf("Current PTK state 	  = %d\n", assoc->ptk_state);
		printf("PTK retries left 	  = %d\n", assoc->ptk_neg_retries_left);
	}
	
}

/* Prints GTKSA for the given assoc */
void printGTKSA(wpa_assoc_t *assoc)
{
	printf("STA received GTK      : %s\n", (assoc->has_gtk ? "True" : "False"));	
}


/* Called whenever EAPOL-Start is received from supplicant */
void wpa_assoc_eapol_start_cb(void *context, ether_mac_t *mac, int *no_auth1x)
{
	wpa_assoc_t *assoc = (wpa_assoc_t *)context;
	wpa_pmk_cache_t *cache_elem;
	
	if (!assoc) 
		return;

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH, "%s: EAPOL-start received from STA %02X:%02X:%02X:%02X:%02X:%02X ",
				  __func__, assoc->sta_mac[0],assoc->sta_mac[1], assoc->sta_mac[2],
				  assoc->sta_mac[3], assoc->sta_mac[4], assoc->sta_mac[5] );

	/* If it is 802.1X port indicate that 802.1X authentication can be started*/
	if (assoc->port->params.psk_param == WPA_PSK_PARAM_NONE) 
		*no_auth1x = 0; /* allow 802.1X authenctication */
	else
		*no_auth1x = 1; /* disallow 802.1X authenctication*/

	/* if connection has SA record in cache - remove it */
	if (assoc->pmk_valid && assoc->pmk_from_cache) 
	{
		cache_elem = wpa_pmk_cache_get_station_by_id(assoc->pmk_cach_id,
													 assoc->port->port_profile_index);
		assoc->pmk_valid = 0;
		assoc->pmk_from_cache = 0;
		if (cache_elem)  /* remove the entry from cache */
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH, "Removing PMK record from cache, PMK ID=\n%s",wpa_log_format_buf((u8 *)&assoc->pmk_cach_id, PMK_CACHE_ID_SIZE));
			wpa_pmk_cache_remove (cache_elem);
		}
	}
			
}

