/* LICENSE_CODE JUNGO */

#ifndef _WPA_ASSOC_H_
#define _WPA_ASSOC_H_

#include <802.1x/auth1x_conn.h>
#include "wpa_port.h"

#define IV_FIELD_LEN  16
#define ID_FIELD_LEN  8

#define KEY_DESC_TYPE 254
#define KEY_DESC_TYPE_WPA2 2

/* Key info bitmask */
#define KEY_INFO_VERSION_1_BIT 0x0001
#define KEY_INFO_VERSION_2_BIT 0x0002
#define KEY_INFO_PAIRWISE_BIT  0x0008
#define KEY_INFO_INSTALL_BIT   0x0040
#define KEY_INFO_ACK_BIT       0x0080
#define KEY_INFO_MIC_BIT       0x0100
#define KEY_INFO_SECURE_BIT    0x0200
#define KEY_INFO_ERROR_BIT     0x0400
#define KEY_INFO_REQUEST_BIT   0x0800
#define KEY_INFO_ENCRYPTED_KEY_DATA_BIT   0x1000

typedef enum {
    WPA_START_MSG1 = 0,
    WPA_WAIT_MSG2  = 1,
    WPA_SEND_MSG3  = 2,
    WPA_WAIT_MSG4  = 3,
    WPA_DONE       = 4,
} wpa_assoc_state_t;

/* A WPA EAPOL-key key descriptor */
typedef struct {
    u8 type;
    u16 info;
    u16 len;
    u64 replay_count;
    wpa_nonce_t nonce;
    u8 iv[IV_FIELD_LEN];
    u64 rsc;
    u8 id[ID_FIELD_LEN];
    wpa_eapol_mic_t mic;
    u16 data_len;
    u8 data[1];
} __attribute__ ((packed)) eapol_key_desc_t;


struct wpa_assoc_t {
    wpa_assoc_t *next;
    wpa_port_t *port;
    ether_mac_t sta_mac;
    wpa_info_elem_t *sta_info_elem;
    wpa_assoc_state_t ptk_state;  /* Current ptk negotiation state */
    wpa_assoc_state_t gtk_state;  /* Current gtk negotiation state */
    wpa_nonce_t nonce;            /* ANonce or GNonce depending on current 
				   * negotiation 
				   */
    u64 ap_to_sta_replay_count;   /* Replay counter maintained by the AP */
    u64 sta_to_ap_replay_count;   /* Replay counter maintained by the STA */
    wpa_pmk_t pmk;
	u8        pmk_cach_id[16];
    int pmk_valid;
    wpa_ptk_t ptk;
    int ptk_neg_retries_left;
    int gtk_neg_retries_left;
    int has_gtk;
	int in_gtk_update; /* Indicate that STA is currently participating in GTK update */
	u8  pre_auth_conn;
	u8  pmk_from_cache;
    auth1x_conn_t *auth1x_conn;
    wpa_sta_type_t sta_type;
    wpa_cipher_t pairwise_cipher; /* Cipher of pairwise key installed for the 
				   * station or 0 if no pairwise key is 
				   * installed 
				   */
};

/* Opens a new association */
wpa_assoc_t *wpa_assoc_open(wpa_port_t *port, ether_mac_t sta_mac, 
    wpa_info_elem_t *sta_info_elem,UINT16 ether_type);

/* Cleans up and free association */
void wpa_assoc_close(wpa_assoc_t *assoc);

/* Starts 802.1x authentication */
void wpa_assoc_start_auth1x(wpa_assoc_t *assoc); 
    
/* Starts PTK (4 way handshake) negotiation */
void wpa_assoc_start_ptk_neg(wpa_assoc_t *assoc);

/* Starts GTK negotiation */
void wpa_assoc_start_gtk_neg(wpa_assoc_t *assoc);

/* Sends a GTK using a legacy 802.1X EAPOL-Key message */
void wpa_assoc_send_gtk_using_8021x(wpa_assoc_t *assoc);

/* Disassociates station */
void wpa_assoc_disassoc(wpa_assoc_t *t);

/* Deauthenticates a station */
void wpa_assoc_deauth(wpa_assoc_t *t);

/* Returns 1 if we're waiting for this station to acknowledge receipt of the
 * group key
 */ 
int wpa_assoc_is_in_gtk_handshake(wpa_assoc_t *t);

/* Called when a station type is detected */
void wpa_assoc_sta_type_detected(wpa_assoc_t *t);

/* Called to update the station type */
void wpa_assoc_set_sta_type(wpa_assoc_t *assoc, wpa_sta_type_t sta_type);

auth1x_conn_t* openAuthxConn(wpa_assoc_t *assoc);

void wdrv_wpaSetMicFailureReportFromStation(BOOL value);

/* Creates EAPOL key packet*/
eapol_pkt_t *create_eapol_key_pkt(wpa_assoc_t *assoc, int key_info_bits, 
										 int key_index, u64 rsc, int key_len, wpa_nonce_t *nonce, u8 *iv, 
										 u8 *key_data, int key_data_len);

/* Verify an EAPOL-key MIC */
int mic_check(wpa_assoc_t *assoc, eapol_pkt_t *eapol_pkt);

/* Get local AP ESSID from driver*/
int get_essid(char *essid);

/* Handle EAPOL-Start packet received */
void wpa_assoc_eapol_start_cb(void *context, ether_mac_t *mac, int *no_auth1x);


/* print SA tables*/
void printGTKSA(wpa_assoc_t *assoc);
void printPTKSA(wpa_assoc_t *assoc);
void printPMKSA(wpa_assoc_t *assoc);


#endif
