/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

#ifndef _WPA_PMK_CACHE_H_
#define _WPA_PMK_CACHE_H_

#define PMK_CACHE_NUMBER_OF_STA            128    /* max number of station cached */
#define PMK_CACHE_ID_SIZE                  16     /* pmk id size*/
#define CHECK_LIST_TIME_CYCLE              5000   /* check list entries validity cycle */
#define SESSION_DEFAULT_TIMEOUT			   1000*60*60*24 /* session default timeout is 24 hours*/	

/* pmk cache list entry states*/
typedef enum 
{
    PMK_CACHE_ENRTY_NOT_ACTIVE  = 0,
	PMK_CACHE_STATION_ASSOC     = 1,
} wpa_pmk_cache_state_t;

/* pmk cache list entry
   holds pmk received from authentication server
   for one station.
*/ 
typedef struct 
{
	wpa_pmk_cache_state_t session_state; 
	char				 pmk_cache_id[PMK_CACHE_ID_SIZE + 4 ];
	ether_mac_t			 mac; 
	wpa_pmk_t			 pmk;
    wpa_port_t           *port;
    wpa_assoc_t          *assoc;
	double		         session_timeout;
	double		         session_start_TS;
	int					 cache_idx;
} wpa_pmk_cache_t;

/* init pmk cahch list ,update all entries to not active*/
int wpa_pmk_cache_init(wpa_port_t *t,char *ap_mac, int profile_index);
/* insert station to pmk cache list
   first cahck if pmk cach is active and if we have place.	
*/
int wpa_pmk_cache_insert(wpa_assoc_t *assoc,double session_timeout);
/* get station entry index with pmk id input*/
wpa_pmk_cache_t*  wpa_pmk_cache_get_station_by_id(char *pmk_id, int profile_index);
/* get station entry index with mac input*/
wpa_pmk_cache_t*  wpa_pmk_cache_get_station_by_mac(ether_mac_t  mac, int profile_index);
/* remove station from pmk cache list and disassociate it*/
int wpa_dissasoc_pmk_cache_remove(wpa_pmk_cache_t* elem);
/* remove station from pmk cache list */
int wpa_pmk_cache_remove(wpa_pmk_cache_t* elem);
/* print list*/
void wpa_pmk_cache_dbg_print(void);
/* release pmk cach mem*/
void wpa_pmk_cache_kill(int profile_index);
/* check whether pkm is valid */
int wpa_pmk_cache_check_pmk_valid(wpa_port_t *port, char *pmk_cach_id, wpa_pmk_t *sta_pmk);
/* update assoc pointer in the cache entry */
int wpa_pmk_cache_update_assoc(wpa_port_t *port, char *pmk_cach_id, wpa_assoc_t *assoc);



#endif /* _WPA_PMK_CACHE_H_ */
