
/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	dda_wds.c	      										   */
/*   PURPOSE:	DDA implementation of the WDS manger.				       */
/*																		   */
/***************************************************************************/

#define __DDA_WDS_C__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <asm/types.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <linux/if_bridge.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/ioctl.h>

#include "pform_types_user.h"
#include "802_11Defs.h"
#include "dda_wext_ioctl.h"
#include "hiddenSecurityProfileAPI.h"
#include "wds.h"

#include "dda_wds.h"
#include "wds.h"
#include "wdsMngr.h"
#include "wdsProtocol.h"

#undef WDS_PRINT_MSG

#define TIAP_SOCKADDR_WDS_IPC "/var/tmp/wdsipc"
#define TIAP_SOCKADDR_WDS_IPC_REP "/var/tmp/wdsrep"
#define TIAP_SOCKADDR_WDS_ISC "/var/tmp/wdsisc"

#define WDS_WAIT_FOR_CHILD_TIME 3
#define WDS_READ_BUFFER_SIZE    256

void dda_wdsMainLoop(void); 

UINT32 dda_wdsPseudoStart(void);
static int dda_wdsIoctlExchange(char *buf, int buf_len, int cmd, char *ifname, 
                                int param1, int param2, char *mac); 
static int dda_wdsGetAfterGoFlag(char *pDevName, UINT32 *pFlag);
static int dda_wdsSetBssServiceState(char *pDevName, UINT32 enable); 
static int dda_wdsDevSetFlags(const char *devname, int flag, int enable);
static int dda_wdsAddRemoveBridgeIfc(const char *br_name, const char *ifname, UINT32 addIfc);


/*
 *Function Name:dda_wdsStart
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsStart(pid_t *pChildPid)
{
    pid_t pid;
    int wait, status;
    char name[128];
    struct stat buff;
    int sockIpc;
    struct sockaddr_un sock_name;
    wdsIpcMsg_s repMsg;
    int retCode;

    g_wdsPrintSeverity = (WDS_SEV_ERROR | WDS_SEV_WARNING); /* Enable all messages */
    /* Create socket for inter process communications. */
    sockIpc = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (sockIpc < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error creating socket for inter process communications: %s.\n",
                             strerror(errno));
        return (NOK);
    }
    /* Create name. */
    sock_name.sun_family = AF_UNIX;

    sprintf((char*)sock_name.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC_REP);
    unlink(sock_name.sun_path);
    /* Bind the UNIX domain address to the created socket */
    retCode = bind(sockIpc, (struct sockaddr *) &sock_name, sizeof(struct sockaddr_un));
    if (retCode < 0 ) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error binding socket for inter process communications - %s.\n", 
               strerror(errno));
        close(sockIpc);
        return (NOK);
    }

    pid = fork();
    if(pid == -1) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, 
                             "%d - Can not create execution context for WDS manager.\n", __LINE__);
        return (NOK);
    }
    if(pid != 0) {
        retCode = recvfrom(sockIpc, (void *) &repMsg, sizeof(wdsIpcMsg_s), 0, NULL, NULL);
        if (retCode < 0) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error reading IPC message: %s", strerror(errno));
        }
        if(repMsg.type != WDS_MSG_TYPE_REPORT_PID) {
            return (NOK);
        }
        *pChildPid = repMsg.body.param;

        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "WDS manager started. pid = %d.\n", *pChildPid);
        /* Check for child process is ready. */
        /* The last thing of the child's init process is */
        /* IPC socket creation. Let's see is it created.*/
        sprintf(name, "%s", TIAP_SOCKADDR_WDS_IPC); 
        for(wait = 0; wait < WDS_WAIT_FOR_CHILD_TIME; wait++) { 
            if(stat(name, &buff) == 0)
                break;
            sleep(1);
        }
        if( waitpid(pid, &status, 0) == -1 ) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Failed in waiting for the child.\n");
            return (NOK);
        }

        if(wait == WDS_WAIT_FOR_CHILD_TIME) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "WDS manager process is not responding.\n");
            return (NOK);
        }

        return (OK);
    }
    /* We are in WDS process. Let's create sockets */
    if(dda_wdsPseudoStart() != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, 
                             "%d - Can not create execution context for WDS manager.\n", __LINE__);
        return (NOK);
    }
    exit(1);
}

 /*
  *Function Name:dda_wdsPseudoStart
  *
  *Parameters:
  *
  *Description:
  *
  *Returns:
  *
  */
 
UINT32 dda_wdsPseudoStart(void)
{
    pid_t pid;
    int sock;
    struct sockaddr_un name;
    wdsIpcMsg_s repMsg;

    pid = fork();
    if(pid == -1) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Can not create execution context for WDS manager.");
        return (NOK);
    }
    if(pid != 0) {
#if 0
        pid_t retPid;
        int status;
        /* pseudo start process */
		if( (retPid = waitpid(pid, &status, 0)) == -1 ) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Failed in waiting for the child, reason: %s\n", 
                                 strerror(errno));

            return -1;
		}
        if( WIFEXITED(status) ) {
            if( WEXITSTATUS(status) == 1 ) /*since child did exit(1)*/
                fprintf(stderr, "The child execl'ed process is exited prematurely %d\n",
                        WEXITSTATUS(status));
            else fprintf(stderr, "Internal error %d\n",
                    WEXITSTATUS(status));
            return -1;
        }
        else if( WIFSIGNALED(status) ) {
            fprintf(stderr, "The child execl'ed process is %d\n",
                    WTERMSIG(status));
            return -1;
        }
#endif /* if 0 */
        return (OK);
    }
    g_wdsPrintSeverity = (WDS_SEV_ERROR | WDS_SEV_WARNING); /* Enable all messages */
    /* Report WDS pid to the caller */

	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Opening datagram socket: %s\n", strerror(errno));
		return (NOK);
	}
    /* Construct name of socket to send to. */
    name.sun_family = AF_UNIX;
    sprintf(name.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC_REP);

    repMsg.type = WDS_MSG_TYPE_REPORT_PID;
    repMsg.body.param = (UINT32) getpid();
    /* Send message. */
    if (sendto(sock, &repMsg, sizeof(wdsIpcMsg_s), 0,
                (struct sockaddr *)&name, sizeof(struct sockaddr_un)) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Sending IPC message: %s\n", strerror(errno));
        close(sock);
        return (NOK);
    }
    close(sock);

    dda_wdsMainLoop();
    unlink(name.sun_path);
    exit(1);

}
/*
 *Function Name:dda_wdsSendIpcMsg
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsSendIpcMsg(wdsIpcMsg_s *pMsg) 
{
    int sock;
    struct sockaddr_un name;

	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Opening datagram socket: %s\n", strerror(errno));
		return (NOK);
	}
    /* Construct name of socket to send to. */
    name.sun_family = AF_UNIX;
    sprintf(name.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC);
    /* Send message. */
    if (sendto(sock, pMsg, sizeof(wdsIpcMsg_s), 0,
                (struct sockaddr *)&name, sizeof(struct sockaddr_un)) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Sending IPC message: %s\n", strerror(errno));
        close(sock);
        return (NOK);
    }
    close(sock);
    return (OK);
}

/*
 *Function Name:dda_wdsPassConfig
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsPassConfig(wdsMngrCfg_s *pCfg)
{
    wdsIpcMsg_s cfgMsg;

    /* Validate configuration */
    if(wds_validateCfg(pCfg) != OK) {
        return (NOK);
    }

    cfgMsg.type = WDS_MSG_TYPE_CFG;
    cfgMsg.body.cfg = *pCfg;
    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Send config.\n");
    return(dda_wdsSendIpcMsg(&cfgMsg));
}

/*
 *Function Name:dda_wdsRelease
 *
 *Parameters:  param - not used, added to be comatible with signal handler prototype
 *
 *Description:
 *
 *Returns:
 *
 */
void dda_wdsRelease(int param)
{
    wdsIpcMsg_s relMsg;

    relMsg.type = WDS_MSG_TYPE_EXIT;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Send release msg to WDS manager.\n");
    dda_wdsSendIpcMsg(&relMsg);
}

/*
 *Function Name:dda_wdsGetReport
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsGetReport(wdsMngrReportRequest_s *pRequest)
{
    wdsIpcMsg_s relMsg;

    relMsg.type = WDS_MSG_TYPE_REPORT;
    relMsg.body.reportReq = *pRequest;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Send report request to WDS manager.\n");
    return(dda_wdsSendIpcMsg(&relMsg));
}

/*
 *Function Name:dda_wdsGetStat
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsGetStat(wdsMngrReportRequest_s *pRequest)
{
    wdsIpcMsg_s relMsg;

    relMsg.type = WDS_MSG_TYPE_STAT;
    relMsg.body.reportReq = *pRequest;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Send stat request to WDS manager.\n");
    return(dda_wdsSendIpcMsg(&relMsg));
}

/*
 *Function Name:dda_wdsGetStat
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsGetInfoDb(wdsMngrReportRequest_s *pRequest)
{
    wdsIpcMsg_s relMsg;

    relMsg.type = WDS_MSG_TYPE_INFO_DB;
    relMsg.body.reportReq = *pRequest;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Send info db request to WDS manager.\n");
    return(dda_wdsSendIpcMsg(&relMsg));
}


/*
 *Function Name:dda_wdsRestart
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 dda_wdsRestart(UINT32 clearConfig)
{
    wdsIpcMsg_s relMsg;

    relMsg.type = WDS_MSG_TYPE_RESTART;
    relMsg.body.param = clearConfig;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Restart WDS manager.\n");
    return(dda_wdsSendIpcMsg(&relMsg));
}

/*
 *Function Name:dda_wdsAlarmHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void dda_wdsAlarmHandler(int signum)
{
    wdsIpcMsg_s relMsg;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Timeout.\n");
    relMsg.type = WDS_MSG_TYPE_ALARM;
    dda_wdsSendIpcMsg(&relMsg);
}

/*
 *Function Name:dda_wdsMainLoop
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void dda_wdsMainLoop(void)
{
    wdsMngr_s * pWdsMngr;
    int sockIpc, sockIsc;
    struct sockaddr_un name;
	struct sockaddr_nl local;
    fd_set activeFdSet, readFdSet;
    UINT32 wdsTimeout;
    struct timeval reqTimeout;
    wdsIpcMsg_s ipcMsg;
    UINT8 buffer[WDS_READ_BUFFER_SIZE];
    int retCode;
    struct itimerval alarmTimeout;


    /* Create socket for inter process communications. */
    sockIpc = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (sockIpc < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error creating socket for inter process communications : %s.\n", 
                             strerror(errno));
        return ;
    }
    /* Create name. */
    name.sun_family = AF_UNIX;

    sprintf((char*)name.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC);

    unlink((char*)name.sun_path);
    /* Bind the UNIX domain address to the created socket */
    retCode = bind(sockIpc, (struct sockaddr *) &name, sizeof(struct sockaddr_un));
    if (retCode < 0 ) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error binding socket for inter process communications - %s.\n", 
               strerror(errno));
        close(sockIpc);
        return ;
    }

    sockIsc = socket(AF_NETLINK, SOCK_DGRAM, NETLINK_USERSOCK);
    if (sockIsc < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error creating socket for inter space communications: %s.\n", strerror(errno));
        close(sockIpc);
        return ;
    }

	memset(&local, 0, sizeof(local));
	local.nl_family = AF_NETLINK;

    /* Bind the NET LINK to the created socket */
    retCode = bind(sockIsc, (struct sockaddr *) &local, sizeof(local));
    if (retCode < 0 ) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error binding socket for inter space communications - %s.\n", 
               strerror(errno));
        close(sockIsc);                               
        close(sockIpc);
        return ;
    }

    if(wds_ddcInit(&pWdsMngr, &wdsTimeout) != OK) {
        close(sockIsc);
        close(sockIpc);
        return;
    }
    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start WDS manager main loop.\n");

    /* Setup TERM handler */
    if(signal (SIGTERM, dda_wdsRelease) == SIG_ERR) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error setting term handler: %s.", strerror(errno));
        goto exit;
    }
    /* Initialize the set of active sockets. */
    FD_ZERO(&activeFdSet);
    FD_SET(sockIpc, &activeFdSet);
    FD_SET(sockIsc, &activeFdSet);
    /* WDS DDCinit returns requered time out value in milliseconds. */
    reqTimeout.tv_sec = wdsTimeout / 1000;
    reqTimeout.tv_usec = ((wdsTimeout % 1000) * 1000);

    /* Setup alarm handler */
    if(signal (SIGALRM, dda_wdsAlarmHandler) == SIG_ERR) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error setting alarm handler: %s.", strerror(errno));
        goto exit;
    }

    alarmTimeout.it_interval = reqTimeout;
    alarmTimeout.it_value = reqTimeout;
    if(setitimer(ITIMER_REAL, &alarmTimeout, NULL) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error setting timer: %s.", strerror(errno));
        goto exit;
    }
    while(1) {
        readFdSet = activeFdSet;
        retCode = select(FD_SETSIZE, &readFdSet, NULL, NULL, NULL);
        if(retCode < 0) {
            if(errno == EINTR) {
                /* Ignore interrupted by signal */
                continue;
            }
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error returned by select: %s.", strerror(errno));
            break;
        }
        if(FD_ISSET(sockIpc, &readFdSet)) {
            /* Inter process communications. */
            retCode = recvfrom(sockIpc, (void *)&ipcMsg, sizeof(wdsIpcMsg_s), 0, NULL, NULL);
            if (retCode < 0) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error reading IPC message.");
            }
            else {
                if(ipcMsg.type == WDS_MSG_TYPE_EXIT) {
                    dda_wdsDelIfFromBridge(pWdsMngr->cfg.brgDevName);
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Release WDS manager.\n");
                    break;
                }
                if(ipcMsg.type == WDS_MSG_TYPE_ALARM) {
                    wds_messageHandler(pWdsMngr, WDS_MSG_SRC_TIMER, NULL, 0);
                }
                else {
                    wds_messageHandler(pWdsMngr, WDS_MSG_SRC_IPC, (UINT8 *)&ipcMsg, sizeof(wdsIpcMsg_s));
                }
            }
        }
        if(FD_ISSET(sockIsc, &readFdSet)) {
            /* Inter space communications. */
            retCode = recvfrom(sockIsc, buffer, WDS_READ_BUFFER_SIZE, 0, NULL, NULL);
            if (retCode < 0) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error reading ISC message.");
            }
            else {
                wds_messageHandler(pWdsMngr, WDS_MSG_SRC_LINK_PARTNER, buffer, retCode);
            }
        }
    }
exit:
    printf("\n\nExit from WDS manager.\n\n");

    wds_ddcRelease(pWdsMngr);
    close(sockIsc);
    close(sockIpc);
    sprintf((char*)name.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC);
    unlink((char*)name.sun_path);
    return ;

}

/*
 *Function Name:wds_ddaSendMsgToDriver
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_ddaSendMsgToDriver(char *pMsg, UINT32 msgLength, UINT8 *pDevName)
{
    return (dda_wdsIoctlExchange(pMsg, msgLength, TIAP_GENERIC_IOCTL_SEND_WDS_PKT, 
                                 pDevName, 0, 0, NULL));
}

/*
 *Function Name:wds_ddaStopBss
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_ddaStopBss(wdsMngr_s *pMngrStruct)
{
    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, " called.\n");
    dda_wdsSetBssServiceState(pMngrStruct->cfg.wlanDevName, FALSE); 

}

/*
 *Function Name:wds_ddaStartBss
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_ddaStartBss(wdsMngr_s *pMngrStruct)
{
    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, " called.\n");
    dda_wdsSetBssServiceState(pMngrStruct->cfg.wlanDevName, TRUE); 
}

/*
 *Function Name:wds_ddaPrintSeveritySet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_ddaPrintSeveritySet(UINT32 error, UINT32 warning, UINT32 info)
{
    wdsIpcMsg_s relMsg;
    UINT32 wdsPrintSeverity = 0;

    if(error)
        wdsPrintSeverity |= WDS_SEV_ERROR;
    if(warning)
        wdsPrintSeverity |= WDS_SEV_WARNING;
    if(info)
        wdsPrintSeverity |= WDS_SEV_INFO;

    relMsg.type = WDS_MSG_TYPE_SEVERITY_SET;
    relMsg.body.param = wdsPrintSeverity;

    dda_wdsSendIpcMsg(&relMsg);

}

/*
 *Function Name:wds_ddaNetIfcEnable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaNetIfcEnable(char *pDevName, char *pMac)
{
    if(dda_wdsDevSetFlags(pDevName, IFF_UP, TRUE) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "dda_wdsDevSetFlags returns error.\n");
        return (NOK);
    }
    return (OK);

}

/*
 *Function Name:wds_ddaAddToStaTable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaAddToStaTable(char *pDevName, char *pMac, UINT32 fourXSupport)
{
    char dummy_buf;

    if(dda_wdsIoctlExchange(&dummy_buf, sizeof(dummy_buf), TIAP_GENERIC_IOCTL_WDS_LINK_ADD, 
                            pDevName, fourXSupport, 0, pMac)!= 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_set_wds_link_up returns error.\n");
        return (NOK);
    }
    return (OK);

}

/*
 *Function Name:wds_ddaNetIfcDisable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaNetIfcDisable(char *pDevName, char *pMac)
{
    if(dda_wdsDevSetFlags(pDevName, ~IFF_UP, FALSE) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "dda_wdsDevSetFlags returns error.\n");
        return (NOK);
    }
    return (OK);

}

/*
 *Function Name:wds_ddaRemoveFromStaTable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaRemoveFromStaTable(char *pDevName, char *pMac)
{
    char dummy_buf;

    if(dda_wdsIoctlExchange(&dummy_buf, sizeof(dummy_buf), TIAP_GENERIC_IOCTL_WDS_LINK_REMOVE, 
                            pDevName, 0, 0, pMac) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_set_wds_link_down returns error.\n");
        return (NOK);
    }

    return (OK);
}

/*
 *Function Name:dda_wdsAddIfToBridge
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void dda_wdsAddIfToBridge(char *pBrName)
{
    char devName[128];
    int i;

    sprintf(devName, "%s%d", WDS_DEVNAME_UL, 0);
    dda_wdsAddRemoveBridgeIfc(pBrName, devName, TRUE);

    for(i = 0; i < WDS_MAX_DL_NUMBER; i++) {
        sprintf(devName, "%s%d", WDS_DEVNAME_DL, i);
        dda_wdsAddRemoveBridgeIfc(pBrName, devName, TRUE);
    }
}

/*
 *Function Name:dda_wdsDelIfFromBridge
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void dda_wdsDelIfFromBridge(char *pBrName)
{
    char devName[128];
    int i;

    sprintf(devName, "%s%d", WDS_DEVNAME_UL, 0);
    if(dda_wdsDevSetFlags(devName, ~IFF_UP, FALSE) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "dda_wdsDevSetFlags returns error.\n");
    }
    dda_wdsAddRemoveBridgeIfc(pBrName, devName, FALSE);

    for(i = 0; i < WDS_MAX_DL_NUMBER; i++) {
        sprintf(devName, "%s%d", WDS_DEVNAME_DL, i);
        if(dda_wdsDevSetFlags(devName, ~IFF_UP, FALSE) != 0) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "dda_wdsDevSetFlags returns error.\n");
        }
        dda_wdsAddRemoveBridgeIfc(pBrName, devName, FALSE);
    }
}

 /*
  *Function Name:wds_ddaNetIfcStatGet
  *
  *Parameters:
  *
  *Description:
  *
  *Returns:
  *
  */
 
UINT32 wds_ddaNetIfcStatGet(char *pDevName, char *pBuff, int len)
{
    if(dda_wdsIoctlExchange(pBuff, len, TIAP_GENERIC_IOCTL_WDS_IFC_STAT_GET, 
                            pDevName, 0, 0, NULL) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_get_wds_ifc_stat returns error.\n");
        return (NOK);
    }
    return (OK);
}

/*
 *Function Name:wds_ddaNetIfcStatGet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaDrvStatGet(char *pDevName, char *pBuff, int len)
{
    if(dda_wdsIoctlExchange(pBuff, len, TIAP_GENERIC_IOCTL_WDS_DRV_STAT_GET, 
                            pDevName, 0, 0, NULL) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_get_wds_drv_stat returns error.\n");
        return (NOK);
    }
    return (OK);
}

/*
 *Function Name:wds_ddaBrgStatGet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaBrgStatGet(char *pDevName, char *pBuff, int len)
{
    if(dda_wdsIoctlExchange(pBuff, len, TIAP_GENERIC_IOCTL_WDS_BRG_STAT_GET, 
                            pDevName, 0, 0, NULL) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_get_wds_brg_stat returns error.\n");
        return (NOK);
    }
    return (OK);
}

/*
 *Function Name:wds_ddaStatGet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddaStatGet(char *pDevName, char *pBuff, int len)
{
    if(dda_wdsIoctlExchange(pBuff, len, TIAP_GENERIC_IOCTL_WDS_STAT_GET, pDevName, 0, 0, NULL) != 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl_get_wds_stat returns error.\n");
        return (NOK);
    }
    return (OK);
}

/*
 *Function Name:dda_wdsSendReport
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void dda_wdsSendReport(char *pReport, UINT32 reportLen, wdsMngrReportRequest_s *pRequest)
{
    int sockRep;

    /* Create socket for inter process communications. */
    sockRep = socket(pRequest->domain, pRequest->type, pRequest->protocol);
    if (sockRep < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error creating report socket: %s.\n", strerror(errno));
        return;
    }
    if (sendto(sockRep, pReport, reportLen, 0,
                (struct sockaddr *)&pRequest->addr, pRequest->addrLen) < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error binding report socket: %s.\n", strerror(errno));
        close(sockRep);
        return;
    }

    close(sockRep);
}

/*
 *Function Name:dda_wdsIpAddrGet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 dda_wdsIpAddrGet(UINT8 *pBrgDevName)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;
    UINT32 ipAddr;

	if (!pBrgDevName) 
        return 0;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 0;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, pBrgDevName, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, SIOCGIFADDR, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", pBrgDevName, strerror(errno));
        close(fd);
		return 0;
	}

	ipAddr = sin->sin_addr.s_addr;
    close(fd);
	return (ipAddr);
}

/*
 *Function Name:dda_wdsInitSecurityProfile
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 dda_wdsInitSecurityProfile(UINT8 privacy, UINT8 *pSecret, UINT32 secretLen, UINT8 wdsLinksNumber)
{
    initHiddenProfConf_T secureInit;

    secureInit.secPortEnabled = privacy;
    secureInit.pskPasswdLen = secretLen;
    strncpy(secureInit.pskPasswd, pSecret, secretLen);
    secureInit.maxNumAssocs = WDS_MAX_DL_NUMBER + 1;
    secureInit.reportMsgType = WDS_MSG_TYPE_WPA_RESP;
    secureInit.domain = AF_UNIX;
    secureInit.type = SOCK_DGRAM;
    secureInit.protocol = 0;
    secureInit.addr.sun_family = AF_UNIX;
    sprintf(secureInit.addr.sun_path,"%s", TIAP_SOCKADDR_WDS_IPC);
    secureInit.addrLen = sizeof(struct sockaddr_un);
    if(hidden_profile_Init(&secureInit)) {
        return (NOK);
    }

    return (OK);
}

/*
 *Function Name:dda_wdsIsDrvReady
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 dda_wdsIsDrvReady(char *pDevName)
{
    UINT32 afterGoFlag;

    if(dda_wdsGetAfterGoFlag(pDevName, &afterGoFlag) != 0) {
        return FALSE;
    }
    return (afterGoFlag);
}

/*
 *Function Name:dda_wdsSleepOneSec
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void dda_wdsSleepOneSec(void)
{
    sleep(1);
}

/*
 *Function Name:dda_wdsSetBssServiceState
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int dda_wdsSetBssServiceState(char *pDevName, UINT32 enable) 
{
	char dummy_buf;

    return (dda_wdsIoctlExchange(&dummy_buf, sizeof(dummy_buf), 
                                 TIAP_GENERIC_IOCTL_BSS_SERVICE_STATE, pDevName, enable, 0, NULL));

}

/*
 *Function Name:dda_wdsGetAfterGoFlag
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int dda_wdsGetAfterGoFlag(char *pDevName, UINT32 *pFlag) 
{
    if( (dda_wdsIoctlExchange((char *)pFlag, sizeof(UINT32),
                                  TIAP_GENERIC_IOCTL_AFTER_GO_FLAG_GET, pDevName, 0, 0, NULL)) != 0) {
        *pFlag = 0;
        return (NOK);
    }
    return (OK);
}

/*
 *Function Name:dda_wdsIoctlExchange
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int dda_wdsIoctlExchange(char *buf, int buf_len, int cmd, char *ifname, 
                                int param1, int param2, char *mac) 
{
	int ioctl_sock, ret = 0;
	generic_ioctl_buf_t data;
	struct ifreq ifr;

	if (!buf && !buf_len) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Both buf and buf_len must be valid (cmd=%d)\n", cmd);
		return (NOK);
	}

    if ((ioctl_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "socket create failed - %s\n", strerror(errno));
		return (NOK);
	}


	data.buf = buf;
	data.buf_len = buf_len;
	data.cmd = cmd;
	data.param1 = param1;
	data.param2 = param2;
	if (mac != NULL)
		memcpy(data.mac, mac, 6);

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	ifr.ifr_data = (char *)&data;

	ret = ioctl(ioctl_sock, SIOCDEVPRIVATE, &ifr);	
	if (ret < 0)
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "ioctl() failed - %s. Dev name %s\n", strerror(errno), ifname);

	close(ioctl_sock);

	return ret;
}

/*
 *Function Name:dda_wdsDevSetFlags
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int dda_wdsDevSetFlags(const char *devname, int flag, int enable)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname) 
        return (NOK);

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, " %s\n", strerror(errno));
		return (NOK);
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, SIOCGIFFLAGS, &ifr) == -1) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "%s: %s\n", devname, strerror(errno));
        close(fd);
		return (NOK);
	}

	if( !enable )
		ifr.ifr_flags &= flag;
	else ifr.ifr_flags |= flag;

	if (ioctl(fd, SIOCSIFFLAGS, &ifr) == -1) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "%s: %s\n", devname, strerror(errno));
        close(fd);
		return (NOK);
	}

    close(fd);
	return (OK);
}

/*
 *Function Name:dda_wdsAddRemoveBridgeIfc
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int dda_wdsAddRemoveBridgeIfc(const char *br_name, const char *ifname, UINT32 addIfc)
{
	unsigned long args[4];
	unsigned long ifindex;
	struct ifreq ifr;
	int fd, err;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "%s\n", strerror(errno));
		return (NOK);
	}

	strncpy (ifr.ifr_name, ifname, sizeof (ifr.ifr_name));
	if (ioctl (fd, SIOCGIFINDEX, &ifr) < 0) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, " %s\n", strerror(errno));
		close(fd);
		return (NOK);
	}

	ifindex = ifr.ifr_ifindex;

	if (!ifindex) {
		WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, " interface %s does not exist!\n", ifname);
		return (NOK);
	}

	args[0] = (addIfc) ? BRCTL_ADD_IF : BRCTL_DEL_IF;
	args[1] = ifindex;
	args[2] = 0;
	args[3] = 0;

	memcpy(ifr.ifr_name, br_name, IFNAMSIZ);
	((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;

	if( (err = ioctl(fd, SIOCDEVPRIVATE, &ifr)) )
		switch ((err = ioctl(fd, SIOCDEVPRIVATE, &ifr))) {
			case EBUSY:
				WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "device %s is already a member of a bridge; "
						"can't enslave it to bridge %s.\n", ifname, br_name);
				break;

			case ELOOP:
				WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, " device %s is a bridge device itself; "
						"can't enslave a bridge device to a bridge device.\n", ifname);
				break;

			default:
				WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "%s\n", strerror(errno));
				break;
		}

	close(fd);
	return err;
}


