/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/***************************************************************************
**                                                                        **
**   MODULE:    dda_wds.h                                                 **
**   PURPOSE:   WDS manager APIs'                                         **
**                                                                        **
***************************************************************************/
#ifndef __DDA_WDS__H__
#define  __DDA_WDS__H__

#ifdef __linux__
#include <sys/un.h>
#else /* ifdef __linux__ */
#endif /* ifdef __linux__ */
#ifdef __DDA_WDS_C__
#define PUBLIC
#else /* ifdef __WDS_MNGR_C__ */
#define PUBLIC extern
#endif /* ifdef __WDS_MNGR_C__ */


#define WDS_SHARED_SECRET_STR_LEN   32 /* Shared secret max string length */
#define WDS_NAME_MAX_LEN             8 /* WDS name max string length */
#define WDS_DEFAULT_COMM_CH_NUM      0 /* Default communication channel */
#define WDS_DEV_NAME_LEN            16 /* Network device name length */
#define WDS_MNGR_INFO_TABLE_SIZE    21 /* Elements number in information database*/



/* Return 0 if all bytes in MAC address are zero */
#define WDS_IS_MAC_NOT_ZERO_m(mac)  (*((UINT8 *)(mac)) | *((UINT8 *)(mac) + 1) | \
                                    *((UINT8 *)(mac) + 2) | *((UINT8 *)(mac) + 3) | \
                                    *((UINT8 *)(mac) + 4) | *((UINT8 *)(mac) + 5))


#define WDS_MAC_CPY(dst, src)       memcpy(dst, src, MAC_ADDR_LEN)

#define WDS_MAC_CMP(addr1, addr2)   memcmp(addr1, addr2, MAC_ADDR_LEN)

typedef enum  {
    WDS_MNGR_STATE_INIT = 0,
    WDS_MNGR_STATE_CH_SEL,
    WDS_MNGR_STATE_UPLINK_CREATE,
    WDS_MNGR_STATE_IN_SERVICE,
    WDS_MNGR_STATE_CRUDE,
    WDS_MNGR_STATE_MAX
}  wdsMngrStates_e;

typedef enum  {
    WDS_HANDLER_STATE_INIT = 0,
    WDS_HANDLER_STATE_JOIN,
    WDS_HANDLER_STATE_SECURE,
    WDS_HANDLER_STATE_MAINTAIN,
    WDS_HANDLER_STATE_MAX
}  wdsLinkHandlerStates_e;


typedef struct {
    UINT8  chSelectMode;               /* channel selection mode: 0 - provided by user, 1 - automatic */
    UINT8  hop;                        /* WDS hop number. 0 - root WDS bridge */
    UINT8  privacy;                    /* WDS privacy mode: 0 - disabled, 1 - enabled */
    UINT8  sharedSecret[WDS_SHARED_SECRET_STR_LEN]; /* Shared secret string */
    UINT8  autoCfg;                    /* WDS automatic configuration mode: 0 - disabled, 1 - enabled */
    UINT8  wirelessIfMode;             /* Wireless interface mode: 0 - disabled, 1 - enabled */
    UINT8  crudeMode;                  /* Set WDS crud mode - no link maintanence is requered*/ 
    UINT8  name[WDS_NAME_MAX_LEN];     /* WDS name */
    UINT8  uplinkMac[MAC_ADDR_LEN];    /* uplink MAC address for manual configuration */
    UINT8  downlinkMac0[MAC_ADDR_LEN]; /* downlink0 MAC address for manual configuration */
    UINT8  downlinkMac1[MAC_ADDR_LEN]; /* downlink1 MAC address for manual configuration */
    UINT8  downlinkMac2[MAC_ADDR_LEN]; /* downlink2 MAC address for manual configuration */
    UINT8  downlinkMac3[MAC_ADDR_LEN]; /* downlink3 MAC address for manual configuration */
} wdsStorageCfg_s;


typedef struct  {
    UINT8  bss_id[MAC_ADDR_LEN];       /* MAC address of the AP */
    UINT8  brgDevName[WDS_DEV_NAME_LEN];   /* Local bridge name. Used to get device IP address */
    UINT8  wlanDevName[WDS_DEV_NAME_LEN];  /* Wlan network interface name */

    UINT8  chSelectMode;               /* channel selection mode: 0 - provided by user, 1 - automatic */
    UINT8  hop;                        /* WDS hop number. 0 - root WDS bridge */
    UINT8  privacy;                    /* WDS privacy mode: 0 - disabled, 1 - enabled */
    UINT8  sharedSecret[WDS_SHARED_SECRET_STR_LEN]; /* Shared secret string */
    UINT8  autoCfg;                    /* WDS automatic configuration mode: 0 - disabled, 1 - enabled */
    UINT8  wirelessIfMode;             /* Wireless interface mode: 0 - disabled, 1 - enabled */
    UINT8  crudeMode;                  /* Set WDS crud mode - no link maintanence is requered*/ 
    UINT8  name[WDS_NAME_MAX_LEN];     /* WDS name */
    UINT8  uplinkMac[MAC_ADDR_LEN];    /* uplink MAC address for manual configuration */
    UINT8  downlinkMac0[MAC_ADDR_LEN]; /* downlink0 MAC address for manual configuration */
    UINT8  downlinkMac1[MAC_ADDR_LEN]; /* downlink1 MAC address for manual configuration */
    UINT8  downlinkMac2[MAC_ADDR_LEN]; /* downlink2 MAC address for manual configuration */
    UINT8  downlinkMac3[MAC_ADDR_LEN]; /* downlink3 MAC address for manual configuration */
} wdsMngrCfg_s;



/****************************************************
* Report and statistics structures and definitions  *
****************************************************/
typedef struct {
    /* Socket creation parameters */
    int domain;
    int type;
    int protocol;
    /* Binding parameters */
#ifdef __linux__
    struct sockaddr_un addr;
#else /* ifdef __linux__ */
    struct sockaddr_in addr;
#endif /* ifdef __linux__ */
    int addrLen;
} wdsMngrReportRequest_s;

typedef struct {
    wdsLinkHandlerStates_e state;
    UINT8  linkPartner[MAC_ADDR_LEN];
    short    rssi;
} wdsLinkReport_s;


typedef struct {
    /* WDS common section */
    UINT32 wdsLibVerMajor;
    UINT32 wdsLibVerMinor;
    UINT32 wdsLibVerPatch;
    wdsMngrStates_e wdsMngrState;
    UINT32 actualHopNum;
    UINT32 dlNumber;
    /* Configuration */
    UINT8  wdsName[WDS_NAME_MAX_LEN];
    UINT8  wdsHop; 
    UINT8  autoCfg;
    UINT8  chSelect;
    UINT8  privacy;
    UINT8  sharedSecret[WDS_SHARED_SECRET_STR_LEN];
    UINT8  bssId[MAC_ADDR_LEN];
    /* Links */
    wdsLinkReport_s uplink;
    wdsLinkReport_s downlink[WDS_MAX_DL_NUMBER];
} wdsMngrReport_s;

typedef struct {
    UINT32 linkEstablished;
    UINT32 secureFailed;
    UINT32 linkFailed;
    UINT32 lastKeepALiveId;
    wdsNetIfcStat_s ifStat;
    UINT8  devName[WDS_DEV_NAME_LEN];
} wdsLinkStat_s;

typedef struct {
    /* WDS manager statistics */
    wdsDrvMngrStat_s        mngrStat;
    wdsDrvStat_s            drvStat;
    wdsDrvSwtchingEntry_s   switchTalbe[WDS_LINKS_NUMBER];
    wdsLinkStat_s           linkStat[WDS_LINKS_NUMBER];
} wdsMngrStat_s;

struct wdsLinkInfo_s {
    UINT8   mac[MAC_ADDR_LEN];
    short     rssi;
};

typedef struct {
    UINT8   sender[MAC_ADDR_LEN];
    UINT32  ip;
    struct wdsLinkInfo_s ul;
    struct wdsLinkInfo_s dl[WDS_MAX_DL_NUMBER];
    UINT32  timestamp;
} wdsMngrInfoRecord_s;

/**********************************************************
*  The commNum parameter of API functions is used         *
*  for stand alone debug of the WDS manager debug only.   *
*  WDS_DEFAULT_COMM_CH_NUM must be used in regular case.  *
**********************************************************/

PUBLIC UINT32 dda_wdsStart(pid_t *pChildPid);
PUBLIC void dda_wdsRelease(int param);
PUBLIC UINT32 dda_wdsRestart(UINT32 clearConfig);
PUBLIC UINT32 dda_wdsPassConfig(wdsMngrCfg_s *pCfg);
PUBLIC UINT32 dda_wdsGetReport(wdsMngrReportRequest_s *pRequest);
PUBLIC UINT32 dda_wdsGetStat(wdsMngrReportRequest_s *pRequest);
PUBLIC void wds_ddaPrintSeveritySet(UINT32 error, UINT32 warning, UINT32 info);

PUBLIC UINT32 wds_ddaNetIfcEnable(char *pDevName, char *pMac);
PUBLIC UINT32 wds_ddaNetIfcDisable(char *pDevName, char *pMac);
PUBLIC UINT32 wds_ddaAddToStaTable(char *pDevName, char *pMac, UINT32 fourXSupport);
PUBLIC UINT32 wds_ddaRemoveFromStaTable(char *pDevName, char *pMac);
PUBLIC void dda_wdsAddIfToBridge(char *pBrName);
PUBLIC void dda_wdsDelIfFromBridge(char *pBrName);
PUBLIC UINT32 wds_ddaNetIfcStatGet(char *pDevName, char *pBuff, int len);
PUBLIC UINT32 wds_ddaDrvStatGet(char *pDevName, char *pBuff, int len);
PUBLIC UINT32 wds_ddaBrgStatGet(char *pDevName, char *pBuff, int len);
PUBLIC UINT32 wds_ddaStatGet(char *pDevName, char *pBuff, int len);
PUBLIC void dda_wdsSendReport(char *pReport, UINT32 reportLen, wdsMngrReportRequest_s *pRequest);
PUBLIC UINT32 dda_wdsIpAddrGet(UINT8 *pBrgDevName);
PUBLIC UINT32 dda_wdsGetInfoDb(wdsMngrReportRequest_s *pRequest);
PUBLIC UINT32 dda_wdsInitSecurityProfile(UINT8 privacy, UINT8 *pSecret, UINT32 secretLen, UINT8 wdsLinksNumber);
PUBLIC UINT32 dda_wdsIsDrvReady(char *pDevName);
PUBLIC void dda_wdsSleepOneSec(void);




#ifdef __DDA_WDS_C__
UINT8 *wds_stateToString[WDS_MNGR_STATE_MAX] = 
        {"INIT", "CH SELECT", "UL CREATE", "IN SERVICE", "CRUDE"};
UINT8 *wds_hdlrStateToStr[WDS_HANDLER_STATE_MAX] = {"INIT", "JOIN", "SECURE", "MAINTAIN"};
#else /* ifdef __DDA_WDS_C__ */
PUBLIC UINT8 *wds_stateToString[WDS_MNGR_STATE_MAX];
UINT8 *wds_hdlrStateToStr[WDS_HANDLER_STATE_MAX];
#endif /* ifdef __DDA_WDS_C__ */
#undef PUBLIC
#endif /* ifndef __DDA_WDS__H__ */
