
/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	wdsMngr.c	      										   */
/*   PURPOSE:	WDS manger DDC  .	      			 				       */
/*																		   */
/***************************************************************************/

#define __WDS_MNGR_C__

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <sys/socket.h>
#include <in.h>
#include "pform_types_user.h"

#include "802_11Defs.h"
#include "hiddenSecurityProfileAPI.h"
#include "wds.h"
#include "dda_wds.h"
#include "wds.h"
#include "wdsMngr.h"
#include "wdsProtocol.h"
#undef WDS_PRINT_MAINT_INFO

#define WDS_REPORT_BUFFER_SIZE  256

#define WDS_PROTOCOL_MSG_GET_SRC_m(msg, src) { \
                        dot11_header_T * pHdr = (dot11_header_T *)(msg); \
                        (src) = pHdr->address2.addr; }

#define WDS_PROTOCOL_MSG_GET_DEST_m(msg, dest) { \
                        dot11_header_T * pHdr = (dot11_header_T *)(msg); \
                        (dest) = pHdr->address1.addr; }

#define WDS_PROTOCOL_MSG_GET_BODY_m(msg, protocolMsg) { \
                        dot11_header_T * pHdr = (dot11_header_T *)(msg); \
                        memcpy(&protocolMsg, pHdr->address3.addr, MAC_ADDR_LEN); \
                        memcpy(((UINT8 *)&protocolMsg) + MAC_ADDR_LEN, pHdr->address4.addr, MAC_ADDR_LEN); }
                        

static UINT8 wdsBroadcastMac[MAC_ADDR_LEN] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
static wdsMngrInfoRecord_s wds_maintainDb[WDS_MNGR_INFO_TABLE_SIZE];


/* Forward functions declarations */

UINT32 wds_ddaSendMsgToDriver(char *pMsg, UINT32 msgLength, UINT8 *pDevName);
void wds_ddaStopBss(wdsMngr_s *pMngrStruct);
void wds_ddaStartBss(wdsMngr_s *pMngrStruct);
void wds_mngrAbortSecure(UINT8 *pMac);
void wds_mngrWpaSuplEnable(UINT8 *pMac);
void wds_mngrWpaAuthlEnable(UINT8 *pMac);

/* Common */
UINT32 wds_sendProtocolMsg(UINT8 *pMacTo, UINT8 *pMacFrom, wdsProtocolMsg_s * pMsg, 
                           UINT8 privacy, UINT8 *pWlanDev);
void wds_mngrStopUplink(wdsMngr_s *pMngrStruct);
UINT32 wds_mngrGetDlIndex(wdsMngr_s *pMngrStruct, UINT8 * pMsg);
void wds_mngrStartDownLinks(wdsMngr_s *pMngrStruct);
void wds_mngrStopDownLinks(wdsMngr_s *pMngrStruct);
WDS_PRO_MSG_TYPES_e wds_mngrProtocolMsgGetType(wdsMngr_s *pMngrStruct, UINT8 * pMsg);
void wds_mngrSendJoinResp(wdsMngr_s *pMngrStruct, UINT8 *pOrigMsg, WDS_PRO_JOIN_REASON_e reason);
void wds_mngrCreateReport(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc);
void wds_mngrCreateStat(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc);
void wds_mngrProvideInfoDb(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc);
UINT32 wds_mngrGetActiveDlNumber(wdsMngr_s *pMngrStruct);
void wds_mngrMaintamceFrameSend(wdsMngr_s *pMngrStruct);
void wds_mngrHandleMaintenInfo(wdsMngr_s *pMngrStruct, UINT8 *pMsg, UINT32 msgLen);
void wds_mngrUpdateLinksDb(wdsMngr_s *pMngrStruct,  dot11_header_T  *pPktHdr, void *pBody, UINT32 ieLen);
void wds_mngrAgeInfoDb(wdsMngr_s *pMngrStruct);
int wds_mngrGetRecordIndex(UINT8 *pMac);
void wds_mngrCleanupLink(wdsLinkHandler_s *pLinkStruct, UINT32 isUplink);


/* Uplink handler */
void wds_ulHandlerInit(wdsLinkHandler_s *pUlStruct);
UINT32 wds_ulHandlerJoin(wdsLinkHandler_s *pUlStruct, UINT8 * pMsg, UINT8 autoCfg, UINT8 *pCurrentHop);
UINT32 wds_ulHandlerJoinResp(wdsLinkHandler_s *pUlStruct, UINT8 * pMsg);
UINT32 wds_ulHandlerTimerHook(wdsLinkHandler_s *pUlStruct);
void wds_ulHandlerStartMaintain(wdsLinkHandler_s *pUlStruct);
UINT32 wds_ulHandlerAuthDone(wdsLinkHandler_s *pUlStruct, UINT32 result);
UINT32 wds_ulHandlerAbort(wdsLinkHandler_s *pUlStruct, UINT8 *pMsg);
void wds_ulHandlerSendKeepAlive(wdsLinkHandler_s *pUlStruct);
void wds_ulHandlerKeepAliveResp(wdsLinkHandler_s *pUlStruct, UINT8 *pMsg);

/* Downlink handler */
UINT32 wds_dlHandlerJoinReq(wdsLinkHandler_s *pDlStruct, UINT8 * pMsg, UINT32 dlNumber);
UINT32 wds_dlHandlerTimerHook(wdsLinkHandler_s *pDlStruct, UINT32 dlNumber);
UINT32 wds_dlHandlerAuthDone(wdsLinkHandler_s *pDlStruct, UINT32 result, UINT32 dlNumber);
UINT32 wds_dlHandlerAbort(wdsLinkHandler_s *pDlStruct, UINT8 *pMsg, UINT32 dlNumber);
UINT32 wds_dlHandlerKeepLink(wdsLinkHandler_s *pDlStruct, UINT8 * pMsg, UINT32 dlNumber);

/*
 *Function Name:wds_ddcInit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ddcInit(wdsMngr_s **pMngrStruct, UINT32 *pTimeout)
{
    *pMngrStruct = (wdsMngr_s *) malloc(sizeof(wdsMngr_s));
    if(*pMngrStruct == NULL) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR,"Error allocation memory!.\n");
        return (NOK);
    }
    /* Reset all fields in g_wdsMngrMainStruct */
    memset((void *) *pMngrStruct, 0, sizeof(wdsMngr_s));
    (*pMngrStruct)->state = WDS_MNGR_STATE_INIT;
    *pTimeout = WDS_TICK;
    (*pMngrStruct)->isConfigured = FALSE;
    return (OK);
}


void wds_ddcRelease(wdsMngr_s *pMngrStruct)
{
    wds_mngrStopDownLinks(pMngrStruct);
    wds_mngrStopUplink(pMngrStruct);
    wds_ddaStartBss(pMngrStruct);

    free(pMngrStruct);
}

/*
 *Function Name:wds_validateCfg
 *
 *Parameters:
 *
 *Description:
 *
 *Returns: TRUE if O.K. otherwise NOK
 *
 */
UINT32 wds_validateCfg(wdsMngrCfg_s *pCfg)
{
    if(pCfg->autoCfg) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Auto config does not supported in current version.\n");
        return (NOK);
    }
    /* ARTIOM : Disabled for DEBUG*/
    if(pCfg->chSelectMode) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Automatic channel selection does not supported in current version.\n");
        return (NOK);
    }
    if((pCfg->hop != 0) && (pCfg->hop != 1)) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Hop number must be 0 for root device or 1 for any other.\n");
        return (NOK);
    }

    return (OK);
}

/*
 *Function Name:wds_mngrStartChScan
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e  wds_mngrStartChScan(wdsMngr_s *pMngrStruct)
{
    if(pMngrStruct->cfg.chSelectMode == WDS_CH_SELECT_MODE_MANUAL)
        return (WDS_MNGR_STATE_UPLINK_CREATE);
    /* Start channel scan here */
    return(pMngrStruct->state);
}

/*
 *Function Name:wds_mngrLookForChannel
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e  wds_mngrLookForChannel(wdsMngr_s *pMngrStruct, UINT8 * pMsg)
{
    /* Look for WDS name in the advertisement message */
    /* If channel selected configure driver and update state */
    return(pMngrStruct->state);
}

/*
 *Function Name:wds_mngrRestart
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e  wds_mngrRestart(wdsMngr_s *pMngrStruct)
{
    wds_ddaStopBss(pMngrStruct);
    wds_mngrStopDownLinks(pMngrStruct);
    wds_mngrStopUplink(pMngrStruct);

    return (WDS_MNGR_STATE_INIT);
}

/*
 *Function Name:wds_mngrSendAdvertisementMsg
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_mngrSendAdvertisementMsg(wdsMngr_s *pMngrStruct)
{
    wdsProtocolMsg_s msg;

    if((wds_mngrGetActiveDlNumber(pMngrStruct) < WDS_MAX_DL_NUMBER) && 
       (pMngrStruct->currentHop < WDS_MAX_HOPS_NUMBER))  {
        memset(&msg, 0, sizeof(wdsProtocolMsg_s));
        WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_ADVERTISEMENT);
        WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION);
        strncpy(msg.body.advertisement.WdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
        WDS_PROT_SET_HOP_NUM_m(msg, pMngrStruct->currentHop);
        WDS_PROT_SET_DL_NUM_m(msg, wds_mngrGetActiveDlNumber(pMngrStruct));

        /* Build and send advertisement message */
        if(wds_sendProtocolMsg(wdsBroadcastMac, pMngrStruct->cfg.bss_id, &msg, 
                               pMngrStruct->cfg.privacy, pMngrStruct->cfg.wlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send advertisement message.\n");
        }
        else {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Sent advertisement message.\n");
        }
    }
}

/*
 *Function Name:wds_mngrInitMsgHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e wds_mngrInitMsgHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, 
                                       UINT8 * pMsg, UINT32 msgLen)
{
    wdsMngrStates_e newState = pMngrStruct->state;
    int devNum;

    switch(msgSource) {
        case WDS_MSG_SRC_START_STATE:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start init state received.\n");
            /* When WDS is already configured decide about next state here */
            if(pMngrStruct->isConfigured) {
	             /* Wait for WLAN driver before doing any think */
	             while(dda_wdsIsDrvReady(pMngrStruct->cfg.wlanDevName) == 0) {
	                 dda_wdsSleepOneSec();
	             }
                if(pMngrStruct->cfg.crudeMode) {
                    /* Skip all management go directly to crud state*/
                    newState = WDS_MNGR_STATE_CRUDE;
                }
                else {
                    /* For root bridge skip channel select and uplink create states */
                    newState = (pMngrStruct->cfg.hop == WDS_ROOT_HOP_NUMBER) ? 
                                WDS_MNGR_STATE_IN_SERVICE : WDS_MNGR_STATE_CH_SEL;
                }
            }
            break;
        case WDS_MSG_SRC_TIMER:
            break;
        case WDS_MSG_SRC_LINK_PARTNER:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Packet received.\n");
            break;
        case WDS_MSG_SRC_IPC:
            {
            wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "IPC message %d received.\n",  pMsgIpc->type);
            switch(pMsgIpc->type) {
                case WDS_MSG_TYPE_RESTART:
                    pMngrStruct->isConfigured = (pMsgIpc->body.param ? FALSE : TRUE);
                    newState = wds_mngrRestart(pMngrStruct);
                    break;
                case WDS_MSG_TYPE_CFG:
                    pMngrStruct->cfg = pMsgIpc->body.cfg;
                    /* Set uplink and downlink partners addresses */
                    if(pMsgIpc->body.cfg.hop == 0) {
                        /* Zero uplink MAC for root device */
                        memset(pMngrStruct->cfg.uplinkMac, 0, MAC_ADDR_LEN);
                    }
                    memcpy((void *)pMngrStruct->uplink.linkPartnerMac,  
                           (const void *)pMngrStruct->cfg.uplinkMac, MAC_ADDR_LEN);
                    memcpy((void *)pMngrStruct->uplink.selfMac,  
                           (const void *)pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
                    pMngrStruct->uplink.privacy = pMngrStruct->cfg.privacy;
                    memcpy((void *)pMngrStruct->downlink[0].linkPartnerMac,  (const void *)pMngrStruct->cfg.downlinkMac0, MAC_ADDR_LEN);
                    memcpy((void *)pMngrStruct->downlink[0].selfMac,  (const void *)pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
                    pMngrStruct->downlink[0].privacy = pMngrStruct->cfg.privacy;
                    memcpy((void *)pMngrStruct->downlink[1].linkPartnerMac,  (const void *)pMngrStruct->cfg.downlinkMac1, MAC_ADDR_LEN);
                    memcpy((void *)pMngrStruct->downlink[1].selfMac,  (const void *)pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
                    pMngrStruct->downlink[1].privacy = pMngrStruct->cfg.privacy;
                    memcpy((void *)pMngrStruct->downlink[2].linkPartnerMac,  (const void *)pMngrStruct->cfg.downlinkMac2, MAC_ADDR_LEN);
                    memcpy((void *)pMngrStruct->downlink[2].selfMac,  (const void *)pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
                    pMngrStruct->downlink[2].privacy = pMngrStruct->cfg.privacy;
                    memcpy((void *)pMngrStruct->downlink[3].linkPartnerMac,  (const void *)pMngrStruct->cfg.downlinkMac3, MAC_ADDR_LEN);
                    memcpy((void *)pMngrStruct->downlink[3].selfMac,  (const void *)pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
                    pMngrStruct->downlink[3].privacy = pMngrStruct->cfg.privacy;
                    strncpy(pMngrStruct->uplink.wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
                    strncpy(pMngrStruct->downlink[0].wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
                    strncpy(pMngrStruct->downlink[1].wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
                    strncpy(pMngrStruct->downlink[2].wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
                    strncpy(pMngrStruct->downlink[3].wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
                    pMngrStruct->uplink.fourXEnabled = TRUE;
                    pMngrStruct->uplink.pWlanDevName = pMngrStruct->cfg.wlanDevName;
                    /* Fill device name field */
                    devNum = 0;
                    sprintf(pMngrStruct->uplink.devName, "%s%d", WDS_DEVNAME_UL, devNum);
                    for (devNum = 0; devNum < WDS_MAX_DL_NUMBER; devNum++) {
                   		sprintf(pMngrStruct->downlink[devNum].devName, "%s%d", WDS_DEVNAME_DL, devNum);
                        pMngrStruct->downlink[devNum].fourXEnabled = TRUE;
                        pMngrStruct->downlink[devNum].pWlanDevName = pMngrStruct->cfg.wlanDevName;
                    }
                    pMngrStruct->isConfigured = TRUE;
                    /* Wait for WLAN driver before doing any think else */
                    while(dda_wdsIsDrvReady(pMngrStruct->cfg.wlanDevName) == 0) {
                        dda_wdsSleepOneSec();
                    }
                    /* Configure security profile */
                    if(dda_wdsInitSecurityProfile(pMngrStruct->cfg.privacy, 
                                                  pMngrStruct->cfg.sharedSecret, 
                                                  WDS_SHARED_SECRET_STR_LEN,
                                                  WDS_MAX_DL_NUMBER + 1) != OK) {
                        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error configuring security profile.\n");
                        break;
                    }
                    if(pMngrStruct->cfg.crudeMode) {
                        /* Skip all management go directly to crud state*/
                        newState = WDS_MNGR_STATE_CRUDE;
                    }
                    else {
                        /* For root bridge skip channel select and uplink create states */
                        newState = (pMngrStruct->cfg.hop == WDS_ROOT_HOP_NUMBER) ? 
                                    WDS_MNGR_STATE_IN_SERVICE : WDS_MNGR_STATE_CH_SEL;
                    }
                    break;
                default:
                    break;
            }
            }
        break;
        default:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined message type %d.\n", msgSource);
            break;

    }
    return (newState);
}

/*
 *Function Name:wds_mngrChSelMsgHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

wdsMngrStates_e wds_mngrChSelMsgHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, 
                                        UINT8 * pMsg, UINT32 msgLen)
{
    wdsMngrStates_e newState = pMngrStruct->state;

    switch(msgSource) {
        case WDS_MSG_SRC_START_STATE:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start ch select state received.\n");
            newState = wds_mngrStartChScan(pMngrStruct);
            break;
        case WDS_MSG_SRC_TIMER:
            break;
        case WDS_MSG_SRC_LINK_PARTNER:
            newState = wds_mngrLookForChannel(pMngrStruct, pMsg);
            break;
        case WDS_MSG_SRC_IPC:
            {
                wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "IPC message %d received.\n",  pMsgIpc->type);
                switch(pMsgIpc->type) {
                    case WDS_MSG_TYPE_RESTART:
                        pMngrStruct->isConfigured = (pMsgIpc->body.param ? FALSE : TRUE);
                        newState = wds_mngrRestart(pMngrStruct);
                        break;
                    default:
                        break;
                }
            }
            break;
        default:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined message type %d.\n", msgSource);
            break;
    }
    return (newState);
}

/*
 *Function Name:wds_mngrUlCreateMsgHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e wds_mngrUlCreateMsgHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, 
                                           UINT8 * pMsg, UINT32 msgLen)
{
    wdsMngrStates_e newState = pMngrStruct->state;
    WDS_PRO_MSG_TYPES_e msgType;
    switch(msgSource) {
        case WDS_MSG_SRC_START_STATE:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start UL create state received.\n");
            /* Clear info BD*/
            memset(wds_maintainDb, 0, sizeof(wds_maintainDb));
            /* start UL creation */
            wds_ddaStopBss(pMngrStruct);
            wds_mngrStopDownLinks(pMngrStruct);
            wds_ulHandlerInit(&pMngrStruct->uplink);
            break;
        case WDS_MSG_SRC_TIMER:
            if(wds_ulHandlerTimerHook(&pMngrStruct->uplink) != WDS_LINK_OK) {
                newState = WDS_MNGR_STATE_INIT;
            }
            break;
        case WDS_MSG_SRC_LINK_PARTNER:
            msgType = wds_mngrProtocolMsgGetType(pMngrStruct, pMsg);
            switch(msgType) {
                case WDS_MSG_JOIN_RESPONSE:
                    /* Pass WDS message to UL handler only */
                    if(wds_ulHandlerJoinResp(&pMngrStruct->uplink,  pMsg) == WDS_LINK_OK) {
                        newState = WDS_MNGR_STATE_IN_SERVICE;
                        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Uplink is established.\n");
                    }
                    break;
                case WDS_MSG_ADVERTISEMENT:
                    if(wds_ulHandlerJoin(&pMngrStruct->uplink, pMsg, pMngrStruct->cfg.autoCfg, 
                                         &pMngrStruct->currentHop) != OK) {
                        newState = wds_mngrRestart(pMngrStruct);
                    }
                    break;
                case WDS_MSG_LINK_ABORT:
                    /* Unhandled message in this state. Nothing to be done */
                    break;
                case WDS_MSG_JOIN_REQUEST:
                    /* Unhandled message in this state. Nothing to be done */
                    break;
                case WDS_MSG_KEEP_ALIVE_RESP:
                    wds_ulHandlerKeepAliveResp(&pMngrStruct->uplink,  pMsg);
                    break;
                default:
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Undefined WDS message type %d.\n", msgType);
                    break;
            }
            break;
        case WDS_MSG_SRC_IPC:
            {
                wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "IPC message %d received.\n",  pMsgIpc->type);
                switch(pMsgIpc->type) {
                    case WDS_MSG_TYPE_RESTART:
                        pMngrStruct->isConfigured = (pMsgIpc->body.param ? FALSE : TRUE);
                        newState = wds_mngrRestart(pMngrStruct);
                        break;
                    case WDS_MSG_TYPE_WPA_RESP:
                        if(memcmp(pMsgIpc->body.wpaResp.linkPartnerMac, pMngrStruct->uplink.linkPartnerMac, MAC_ADDR_LEN) == 0) {
                            if(wds_ulHandlerAuthDone(&pMngrStruct->uplink, 
                                                     pMsgIpc->body.wpaResp.result) == WDS_LINK_OK) {
                                newState = WDS_MNGR_STATE_IN_SERVICE;
                                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Authentification complete  peer address <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n", 
                                        pMsgIpc->body.wpaResp.linkPartnerMac[0], pMsgIpc->body.wpaResp.linkPartnerMac[1],
                                        pMsgIpc->body.wpaResp.linkPartnerMac[2], pMsgIpc->body.wpaResp.linkPartnerMac[3],
                                        pMsgIpc->body.wpaResp.linkPartnerMac[4], pMsgIpc->body.wpaResp.linkPartnerMac[5]);
                            }
                            else {
                                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Authentification failed %d.\n",  pMsgIpc->body.wpaResp.failReson);
                                newState = wds_mngrRestart(pMngrStruct);
                            }
                        }
                        else {
                            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, 
                               "WPA message received about unknown link partner <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n",  
                               pMsgIpc->body.wpaResp.linkPartnerMac[0], pMsgIpc->body.wpaResp.linkPartnerMac[1],
                               pMsgIpc->body.wpaResp.linkPartnerMac[2], pMsgIpc->body.wpaResp.linkPartnerMac[3],
                               pMsgIpc->body.wpaResp.linkPartnerMac[4], pMsgIpc->body.wpaResp.linkPartnerMac[5]);
                        }
                        break;
                    default:
                        break;
                }
            }
            break;
        default:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined message source %d.\n", msgSource);
            break;
    }
    return (newState);
}

/*
 *Function Name:wds_mngrInSrvMsgHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
wdsMngrStates_e wds_mngrInSrvMsgHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, 
                                        UINT8 * pMsg, UINT32 msgLen)
{
    wdsMngrStates_e newState = pMngrStruct->state;
    UINT32 dlIndex = -1;

    switch(msgSource) {
        case WDS_MSG_SRC_START_STATE:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start service state received.\n");
            wds_mngrStartDownLinks(pMngrStruct);
            wds_mngrSendAdvertisementMsg(pMngrStruct);
            wds_ulHandlerStartMaintain(&pMngrStruct->uplink);
            pMngrStruct->maintenInfoTimeOut = WDS_MAINEN_INFO_TIMEOUT;
            wds_mngrMaintamceFrameSend(pMngrStruct);
            if(pMngrStruct->cfg.wirelessIfMode) {
                wds_ddaStartBss(pMngrStruct);
            }
            break;
        case WDS_MSG_SRC_TIMER:
            if(pMngrStruct->cfg.hop != WDS_ROOT_HOP_NUMBER) {
                /* There are no uplink in the root bridge */
                if(wds_ulHandlerTimerHook(&pMngrStruct->uplink) != WDS_LINK_OK) {
                    newState = WDS_MNGR_STATE_UPLINK_CREATE;
                    /* exit from this case - state changed.*/
                    break;
                }
            }
            wds_mngrSendAdvertisementMsg(pMngrStruct);
            /* Call downlink timer hook here */
            for(dlIndex = 0; dlIndex < WDS_MAX_DL_NUMBER; dlIndex++) {
                if(wds_dlHandlerTimerHook(&pMngrStruct->downlink[dlIndex], dlIndex) == WDS_LINK_FAIL) {
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Downlink %d is lost.\n", dlIndex);
                    wds_mngrMaintamceFrameSend(pMngrStruct);
                }
            }
            pMngrStruct->maintenInfoTimeOut--;
            pMngrStruct->timestamp++;
            if(pMngrStruct->maintenInfoTimeOut == 0) {
                wds_mngrAgeInfoDb(pMngrStruct);
                wds_mngrMaintamceFrameSend(pMngrStruct);
                pMngrStruct->maintenInfoTimeOut = WDS_MAINEN_INFO_TIMEOUT;
            }
            break;
        case WDS_MSG_SRC_LINK_PARTNER:
            if(msgLen == sizeof(dot11_header_T)) {
                switch(wds_mngrProtocolMsgGetType(pMngrStruct,pMsg)) {
                    case WDS_MSG_JOIN_REQUEST:
                        dlIndex = wds_mngrGetDlIndex(pMngrStruct, pMsg);
                        if(dlIndex == -1) {
                            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Wrong join request sender.\n"); 
                            wds_mngrSendJoinResp(pMngrStruct, pMsg, WDS_JOIN_FAIL_PEER_NOT_ALLOWED);
                        }
                        else {
                            if(wds_dlHandlerJoinReq(&pMngrStruct->downlink[dlIndex], pMsg, dlIndex) == WDS_LINK_OK) {
                                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Downlink %d is established.\n", dlIndex);
                                wds_mngrMaintamceFrameSend(pMngrStruct);
                            }
                        }
    
                        break;
                    case WDS_MSG_KEEP_ALIVE_REQ:
                        dlIndex = wds_mngrGetDlIndex(pMngrStruct, pMsg);
                        if(dlIndex != -1) {
                            wds_dlHandlerKeepLink(&pMngrStruct->downlink[dlIndex],  pMsg, dlIndex);
                        }
                        break;
                    case WDS_MSG_KEEP_ALIVE_RESP:
                        wds_ulHandlerKeepAliveResp(&pMngrStruct->uplink,  pMsg);
                        break;
                    case WDS_MSG_LINK_ABORT:
                        if(wds_ulHandlerAbort(&pMngrStruct->uplink,  pMsg) != WDS_LINK_OK) {
                            newState = WDS_MNGR_STATE_UPLINK_CREATE;
                        }
                        else {
                            dlIndex = wds_mngrGetDlIndex(pMngrStruct, pMsg);
                            if(dlIndex != -1) {
                                wds_dlHandlerAbort(&pMngrStruct->downlink[dlIndex], pMsg, dlIndex);
                                wds_mngrMaintamceFrameSend(pMngrStruct);
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
            else {
                wds_mngrHandleMaintenInfo(pMngrStruct, pMsg, msgLen);
            }
            break;
        case WDS_MSG_SRC_IPC:
            {
                wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "IPC message %d received.\n",  pMsgIpc->type);
                switch(pMsgIpc->type) {
                    case WDS_MSG_TYPE_RESTART:
                        pMngrStruct->isConfigured = (pMsgIpc->body.param ? FALSE : TRUE);
                        newState = wds_mngrRestart(pMngrStruct);
                        break;
                    case WDS_MSG_TYPE_WPA_RESP:
                        for(dlIndex = 0; dlIndex < WDS_MAX_DL_NUMBER; dlIndex++) {
                            if(memcmp(pMngrStruct->downlink[dlIndex].linkPartnerMac, 
                                      pMsgIpc->body.wpaResp.linkPartnerMac, MAC_ADDR_LEN) == 0) {
                                break;
                            }
                        }
                        if(dlIndex < WDS_MAX_DL_NUMBER) {
                            if(wds_dlHandlerAuthDone(&pMngrStruct->downlink[dlIndex], 
                                                     pMsgIpc->body.wpaResp.result, dlIndex) == WDS_LINK_FAIL) {
                                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Authentification failed %d.\n",  pMsgIpc->body.wpaResp.failReson);
                            }
                            else {
                                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Authentification complete  peer address <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n", 
                                    pMsgIpc->body.wpaResp.linkPartnerMac[0], pMsgIpc->body.wpaResp.linkPartnerMac[1],
                                    pMsgIpc->body.wpaResp.linkPartnerMac[2], pMsgIpc->body.wpaResp.linkPartnerMac[3],
                                    pMsgIpc->body.wpaResp.linkPartnerMac[4], pMsgIpc->body.wpaResp.linkPartnerMac[5]);
                            }
                        }
                        else {
                            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Unexpected peer address <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n", 
                                pMsgIpc->body.wpaResp.linkPartnerMac[0], pMsgIpc->body.wpaResp.linkPartnerMac[1],
                                pMsgIpc->body.wpaResp.linkPartnerMac[2], pMsgIpc->body.wpaResp.linkPartnerMac[3],
                                pMsgIpc->body.wpaResp.linkPartnerMac[4], pMsgIpc->body.wpaResp.linkPartnerMac[5]);
                        }
                    default:
                        break;
                }
            }
            break;
        default:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined message type %d.\n", msgSource);
            break;
    }
    return (newState);
}

/*
 *Function Name:wds_mngrCrudeMsgHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

wdsMngrStates_e wds_mngrCrudeMsgHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, 
                                        UINT8 * pMsg, UINT32 msgLen)
{
    wdsMngrStates_e newState = pMngrStruct->state;
    UINT32 dlIndex = -1;

    switch(msgSource) {
        case WDS_MSG_SRC_START_STATE:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Start crude state received.\n");
            if(WDS_IS_MAC_NOT_ZERO_m(pMngrStruct->uplink.linkPartnerMac)) {
                if(wds_ddaAddToStaTable(pMngrStruct->uplink.devName, 
                                       pMngrStruct->uplink.linkPartnerMac, FALSE) != OK) {
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "UL Error enabling net ifc %s!.\n",
                                         pMngrStruct->uplink.devName);
                }
    
                if(wds_ddaNetIfcEnable(pMngrStruct->uplink.devName, 
                                       pMngrStruct->uplink.linkPartnerMac) != OK) {
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "UL Error enabling net ifc %s!.\n",
                                         pMngrStruct->uplink.devName);
                }
                pMngrStruct->uplink.state = WDS_HANDLER_STATE_MAINTAIN;
                pMngrStruct->uplink.linkEstablished++;
            }

            for(dlIndex = 0; dlIndex < WDS_MAX_DL_NUMBER; dlIndex++) {
                if(WDS_IS_MAC_NOT_ZERO_m(pMngrStruct->downlink[dlIndex].linkPartnerMac)) {
                    if(wds_ddaAddToStaTable(pMngrStruct->downlink[dlIndex].devName, 
                                           pMngrStruct->downlink[dlIndex].linkPartnerMac, FALSE) != OK) {
                        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error enabling net ifc %s!.\n",
                                             dlIndex, pMngrStruct->downlink[dlIndex].devName);
                    }
                    if(wds_ddaNetIfcEnable(pMngrStruct->downlink[dlIndex].devName, 
                                           pMngrStruct->downlink[dlIndex].linkPartnerMac) != OK) {
                        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error enabling net ifc %s!.\n",
                                             dlIndex, pMngrStruct->downlink[dlIndex].devName);
                    }
                    pMngrStruct->downlink[dlIndex].linkEstablished++;
                    pMngrStruct->downlink[dlIndex].state = WDS_HANDLER_STATE_MAINTAIN;
                }
            }
            if(pMngrStruct->cfg.wirelessIfMode) {
                wds_ddaStartBss(pMngrStruct);
            }

            break;
        case WDS_MSG_SRC_TIMER:
            break;
        case WDS_MSG_SRC_LINK_PARTNER:
            break;
        case WDS_MSG_SRC_IPC:
            {
                wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "IPC message %d received.\n",  pMsgIpc->type);
                switch(pMsgIpc->type) {
                    case WDS_MSG_TYPE_RESTART:
                        pMngrStruct->isConfigured = (pMsgIpc->body.param ? FALSE : TRUE);
                        newState = wds_mngrRestart(pMngrStruct);
                        break;
                    default:
                        break;
                }
            }
            break;
        default:
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined message type %d.\n", msgSource);
            break;
    }
    return (newState);
}

/*
 *Function Name:wds_messageHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_messageHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, UINT8 * pMsg, UINT32 msgLen)
{
    UINT32 nextPass = 0;
    wdsMngrStates_e newState;
    wdsIpcMsg_s *pMsgIpc = (wdsIpcMsg_s *)pMsg;
    UINT32 continueFlag = TRUE;
        
    /* Catch maintenance messages first */
    if(msgSource == WDS_MSG_SRC_IPC) {
        switch(pMsgIpc->type) {
            case WDS_MSG_TYPE_REPORT:
                wds_mngrCreateReport(pMngrStruct, pMsgIpc);
                continueFlag = FALSE;
                break;
            case WDS_MSG_TYPE_STAT:
                wds_mngrCreateStat(pMngrStruct, pMsgIpc);
                continueFlag = FALSE;
                break;
            case WDS_MSG_TYPE_SEVERITY_SET:
                g_wdsPrintSeverity = pMsgIpc->body.param;
                continueFlag = FALSE;
                break;
            case WDS_MSG_TYPE_INFO_DB:
                wds_mngrProvideInfoDb(pMngrStruct, pMsgIpc);
                continueFlag = FALSE;
                break;
            default:
                continueFlag = TRUE;
                break;
        }

    }
    if(continueFlag) {
        do {
            switch(pMngrStruct->state) {
                case WDS_MNGR_STATE_INIT:
                    newState = wds_mngrInitMsgHandler(pMngrStruct, msgSource, pMsg, msgLen);
                    break;
                case WDS_MNGR_STATE_CH_SEL:
                    newState = wds_mngrChSelMsgHandler(pMngrStruct, msgSource, pMsg, msgLen);
                    break;
                case WDS_MNGR_STATE_UPLINK_CREATE:
                    newState = wds_mngrUlCreateMsgHandler(pMngrStruct, msgSource, pMsg, msgLen);
                    break;
                case WDS_MNGR_STATE_IN_SERVICE:
                    newState = wds_mngrInSrvMsgHandler(pMngrStruct, msgSource, pMsg, msgLen);
                    break;
                case WDS_MNGR_STATE_CRUDE:
                    newState = wds_mngrCrudeMsgHandler(pMngrStruct, msgSource, pMsg, msgLen);
                    break;
                default:
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Undefined state of the WDS manager %d.\n",  
                                         pMngrStruct->state);
                    newState =  WDS_MNGR_STATE_INIT;
                    break;
            }
            if(newState != pMngrStruct->state) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "WDS mngr state changed from %s to %s.\n",  
                                     wds_stateToString[pMngrStruct->state], wds_stateToString[newState]);
                 pMngrStruct->state = newState;
                 nextPass = 1;
                 msgSource = WDS_MSG_SRC_START_STATE;
                 pMsg = NULL;
            }
            else {
                nextPass = 0;
            }
        } while ( nextPass != 0 );
    }
}

/*
 *Function Name:wds_sendProtocolMsg
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_sendProtocolMsg(UINT8 *pMacTo, UINT8 *pMacFrom, wdsProtocolMsg_s * pMsg, 
                           UINT8 privacy, UINT8 *pWlanDev)
{
    dot11_header_T dot11Msg;
    UINT8 * pSecondHalf;

    /* Clear all*/
    memset(&dot11Msg, 0, sizeof(dot11_header_T));
    memcpy((void *)&dot11Msg.address1.addr, pMacTo, MAC_ADDR_LEN);
    memcpy((void *)&dot11Msg.address2.addr, pMacFrom, MAC_ADDR_LEN);
    /* Set frame control bits */
    dot11Msg.fc = (DOT11_FC_TYPE_DATA | DOT11_FC_SUB_NULL_FUNCTION | DOT11_FC_TO_DS | 
                   DOT11_FC_FROM_DS /*| DOT11_FC_PWR_MGMT | DOT11_FC_ORDER*/);

    if(privacy) {
        dot11Msg.fc |= DOT11_FC_WEP;
    }
    pSecondHalf = ((UINT8 *)pMsg) + MAC_ADDR_LEN;
    memcpy((void *)&dot11Msg.address3.addr, pMsg, MAC_ADDR_LEN);
    memcpy((void *)&dot11Msg.address4.addr, pSecondHalf, MAC_ADDR_LEN);

    /* Send message to driver */
    return (wds_ddaSendMsgToDriver((char *)&dot11Msg, sizeof(dot11_header_T), pWlanDev));
}

/*
 *Function Name:wds_mngrGetDlIndex
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_mngrGetDlIndex(wdsMngr_s *pMngrStruct, UINT8 * pMsg)
{
    UINT8 *pSrc, *pDest;
    UINT32 index;

    WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);
    WDS_PROTOCOL_MSG_GET_DEST_m(pMsg, pDest);

    if(memcmp(pMngrStruct->cfg.bss_id, pDest, MAC_ADDR_LEN) != 0) {
        return (-1);
    }
    for(index = 0; index < WDS_MAX_DL_NUMBER; index++) {
        if(memcmp(pMngrStruct->downlink[index].linkPartnerMac, pSrc, MAC_ADDR_LEN) == 0) {
            return (index);
        }
    }
    return (-1);
}

/*
 *Function Name:wds_mngrStartDownLinks
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_mngrStartDownLinks(wdsMngr_s *pMngrStruct)
{
    UINT32 index;
    for(index = 0; index < WDS_MAX_DL_NUMBER; index++) {
        pMngrStruct->downlink[index].secureElapsedTime = 0;
        pMngrStruct->downlink[index].unsuccessfulKeepAliveNumber = 0;
        pMngrStruct->downlink[index].state = WDS_HANDLER_STATE_JOIN;
    }
}

/*
 *Function Name:wds_mngrStopDownLinks
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_mngrStopDownLinks(wdsMngr_s *pMngrStruct)
{
    UINT32 index;
    for(index = 0; index < WDS_MAX_DL_NUMBER; index++) {
        if(pMngrStruct->downlink[index].state == WDS_HANDLER_STATE_MAINTAIN) {
            wdsProtocolMsg_s msg;

            memset(&msg, 0, sizeof(wdsProtocolMsg_s));
            WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_LINK_ABORT);
            WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION);
            msg.body.linkAbort.reason = CONNECTION_LOST;

            /* Build and send abort message */
            if(wds_sendProtocolMsg(pMngrStruct->downlink[index].linkPartnerMac, 
                                   pMngrStruct->downlink[index].selfMac, &msg, 
                                   pMngrStruct->downlink[index].privacy,
                                   pMngrStruct->cfg.wlanDevName) != OK) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send abort message.\n");
            }
            wds_mngrCleanupLink(&(pMngrStruct->downlink[index]), FALSE);

            if(wds_ddaNetIfcDisable(pMngrStruct->downlink[index].devName, 
                                    pMngrStruct->downlink[index].linkPartnerMac) != OK) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error disabling net ifc %s!.\n", pMngrStruct->downlink[index].devName);
            }

            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Downlink %d is released.\n", index);
        }
        else if(pMngrStruct->downlink[index].state == WDS_HANDLER_STATE_SECURE) {
            wds_mngrAbortSecure(pMngrStruct->downlink[index].linkPartnerMac);
        }

        pMngrStruct->downlink[index].state = WDS_HANDLER_STATE_INIT;
    }
}

/*
 *Function Name:wds_mngrStopUplink
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrStopUplink(wdsMngr_s *pMngrStruct)
{
    if(pMngrStruct->uplink.state == WDS_HANDLER_STATE_MAINTAIN) {

        /* If Root --> Don't need to send Abort Msg to Uplink */
        if(pMngrStruct->currentHop != 0) {
            
            wdsProtocolMsg_s msg;
            
            memset(&msg, 0, sizeof(wdsProtocolMsg_s));
            
            WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_LINK_ABORT);
            WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION);
            msg.body.linkAbort.reason = CONNECTION_LOST;
            
            /* Build and send advertisement message */
            if(wds_sendProtocolMsg(pMngrStruct->uplink.linkPartnerMac, 
                pMngrStruct->uplink.selfMac, &msg, 
                pMngrStruct->uplink.privacy,
                pMngrStruct->cfg.wlanDevName) != OK) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send abort message.\n");
            }
        }
    }
    else if(pMngrStruct->uplink.state == WDS_HANDLER_STATE_SECURE) {
        wds_mngrAbortSecure(pMngrStruct->uplink.linkPartnerMac);
    }
    wds_mngrCleanupLink(&(pMngrStruct->uplink), TRUE);
    if(wds_ddaNetIfcDisable(pMngrStruct->uplink.devName, 
                            pMngrStruct->uplink.linkPartnerMac) != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error disabling net ifc %s!.\n", pMngrStruct->uplink.devName);
    }

    pMngrStruct->uplink.state = WDS_HANDLER_STATE_INIT;
}
/*
 *Function Name:wds_mngrProtocolMsgGetType
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

WDS_PRO_MSG_TYPES_e wds_mngrProtocolMsgGetType(wdsMngr_s *pMngrStruct, UINT8 * pMsg)
{
    WDS_PRO_MSG_TYPES_e msg_type;
    UINT8 proto_version;
    dot11_header_T *pHdr;
    wdsProtocolMsg_s *pProtoMsg;
    pHdr = (dot11_header_T *) pMsg;

    pProtoMsg = (wdsProtocolMsg_s *)pHdr->address3.addr;

    WDS_PROT_GET_MSG_TYPE_m((*pProtoMsg), msg_type);
    WDS_PROT_GET_VERSION_m((*pProtoMsg), proto_version);

    if(proto_version != WDS_PROTOCOL_VERSION) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Unsupported protocol version (%d) message.\n", proto_version);
        msg_type = WDS_MSG_RESERVED;
    }
    else {
        if(((pHdr->fc & DOT11_FC_WEP) != 0) && (pMngrStruct->cfg.privacy == 0)) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, "Msg received with privacy enabled "
                                 "from %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
                                 pHdr->address2.addr[0], pHdr->address2.addr[1], 
                                 pHdr->address2.addr[2], pHdr->address2.addr[3], 
                                 pHdr->address2.addr[4], pHdr->address2.addr[5]);
            msg_type = WDS_MSG_RESERVED;
        }
        else if(((pHdr->fc & DOT11_FC_WEP) == 0) && (pMngrStruct->cfg.privacy != 0)) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, "Msg received with privacy disabled "
                                 "from %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n",
                                 pHdr->address2.addr[0], pHdr->address2.addr[1], 
                                 pHdr->address2.addr[2], pHdr->address2.addr[3], 
                                 pHdr->address2.addr[4], pHdr->address2.addr[5]);
            msg_type = WDS_MSG_RESERVED;
        }
    }
    return (msg_type);
}

/*
 *Function Name:wds_mngrSendJoinResp
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrSendJoinResp(wdsMngr_s *pMngrStruct, UINT8 *pOrigMsg, WDS_PRO_JOIN_REASON_e reason)
{
    wdsProtocolMsg_s msg;
    UINT8 *pSrc;
    
    /* Send join response message here*/
    memset(&msg, 0, sizeof(wdsProtocolMsg_s));
    WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_JOIN_RESPONSE);
    WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION); 

    msg.body.joinResp.reason = reason;

    WDS_PROTOCOL_MSG_GET_SRC_m(pOrigMsg, pSrc);

    if(wds_sendProtocolMsg(pSrc, pMngrStruct->cfg.bss_id, &msg, 
                           pMngrStruct->cfg.privacy, pMngrStruct->cfg.wlanDevName) != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send join response message.\n");
        return;
    }
}

 /*
  *Function Name:wds_mngrCalcRssi
  *
  *Parameters:
  *
  *Description:
  *
  *Returns:
  *
  */
 
short wds_mngrCalcRssi(short *pRssiBuff)
{
    short sum = 0;
    UINT8 index, calculated = 0;

    for (index = 0; index < WDS_RSSI_WINDOW_SIZE; index++) {
        if(*(pRssiBuff + index) == 0)
            continue;
        sum += *(pRssiBuff + index);
        calculated++;
    }
    if(calculated == 0)
        return(0);
    return (sum/calculated);
}

 /*
  *Function Name:wds_mngrCreateReport
  *
  *Parameters:
  *
  *Description:
  *
  *Returns:
  *
  */

void wds_mngrCreateReport(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc)
{
    int i;
    wdsMngrReport_s report;

    report.wdsLibVerMajor = WDS_VER_MAJOR;
    report.wdsLibVerMinor = WDS_VER_MINOR;
    report.wdsLibVerPatch = WDS_VER_PATCH;
    report.wdsMngrState = pMngrStruct->state;
    report.actualHopNum = pMngrStruct->currentHop;
    report.dlNumber = wds_mngrGetActiveDlNumber(pMngrStruct);
    strncpy(report.wdsName, pMngrStruct->cfg.name, WDS_NAME_MAX_LEN);
    report.wdsHop = pMngrStruct->cfg.hop;
    report.autoCfg = pMngrStruct->cfg.autoCfg;
    report.chSelect = pMngrStruct->cfg.chSelectMode;
    report.privacy = pMngrStruct->cfg.privacy;
    strncpy(report.sharedSecret, pMngrStruct->cfg.sharedSecret, WDS_SHARED_SECRET_STR_LEN);
    memcpy(report.bssId, pMngrStruct->cfg.bss_id, MAC_ADDR_LEN);
    /* Uplink */
    report.uplink.state = pMngrStruct->uplink.state;
    memcpy(report.uplink.linkPartner, pMngrStruct->uplink.linkPartnerMac, MAC_ADDR_LEN);
    report.uplink.rssi = wds_mngrCalcRssi(pMngrStruct->uplink.rssiBuffer);
    for(i = 0; i < WDS_MAX_DL_NUMBER;i++) {
        report.downlink[i].state = pMngrStruct->downlink[i].state;
        memcpy(report.downlink[i].linkPartner, pMngrStruct->downlink[i].linkPartnerMac, MAC_ADDR_LEN);
        if(pMngrStruct->downlink[i].state != WDS_HANDLER_STATE_MAINTAIN) {
            report.downlink[i].rssi = 0;
        }
        else {
            report.downlink[i].rssi = wds_mngrCalcRssi(pMngrStruct->downlink[i].rssiBuffer);
        }
    }
    dda_wdsSendReport((char *)&report, sizeof(wdsMngrReport_s), &pMsgIpc->body.reportReq);
}

/*
 *Function Name:wds_mngrCreateStat
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_mngrCreateStat(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc)
{
    int devNum, i;
    char buff[WDS_REPORT_BUFFER_SIZE];
    wdsDrvSwtchingEntry_s *pSwitchingEntry;
    wdsMngrStat_s mngrStat;

    mngrStat.linkStat[0].linkEstablished = pMngrStruct->uplink.linkEstablished;
    mngrStat.linkStat[0].linkFailed = pMngrStruct->uplink.linkFailed;
    mngrStat.linkStat[0].secureFailed = pMngrStruct->uplink.secureFailed;
    mngrStat.linkStat[0].lastKeepALiveId = pMngrStruct->uplink.lastKeepAliveId;

    for (i = 0; i < WDS_MAX_DL_NUMBER; i++) {
        mngrStat.linkStat[i + 1].linkEstablished = pMngrStruct->downlink[i].linkEstablished;
        mngrStat.linkStat[i + 1].linkFailed = pMngrStruct->downlink[i].linkFailed;
        mngrStat.linkStat[i + 1].secureFailed = pMngrStruct->downlink[i].secureFailed;
        mngrStat.linkStat[i + 1].lastKeepALiveId = pMngrStruct->downlink[i].lastKeepAliveId;
    }

    wds_ddaStatGet(pMngrStruct->uplink.devName, (char *) &mngrStat.mngrStat, 
                   sizeof(mngrStat.mngrStat));

    wds_ddaBrgStatGet(pMngrStruct->uplink.devName, buff, WDS_REPORT_BUFFER_SIZE);
    pSwitchingEntry = (wdsDrvSwtchingEntry_s *)buff;
    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        mngrStat.switchTalbe[i] = *(pSwitchingEntry + i);
    }

    wds_ddaDrvStatGet(pMngrStruct->uplink.devName, (char *)&mngrStat.drvStat, 
                      sizeof(mngrStat.drvStat));

    wds_ddaNetIfcStatGet(pMngrStruct->uplink.devName, (char *)&mngrStat.linkStat[0].ifStat, 
                         sizeof(mngrStat.linkStat[0].ifStat));
    strncpy(mngrStat.linkStat[0].devName, pMngrStruct->uplink.devName, WDS_DEV_NAME_LEN);

    for (devNum = 0; devNum < WDS_MAX_DL_NUMBER; devNum++) {
        wds_ddaNetIfcStatGet(pMngrStruct->downlink[devNum].devName, 
                             (char *)&mngrStat.linkStat[devNum + 1].ifStat, 
                             sizeof(mngrStat.linkStat[devNum + 1].ifStat));
        strncpy(mngrStat.linkStat[devNum + 1].devName, pMngrStruct->downlink[devNum].devName, 
                WDS_DEV_NAME_LEN);
    }
    dda_wdsSendReport((char *)&mngrStat, sizeof(wdsMngrStat_s), &pMsgIpc->body.reportReq);
}

/*
 *Function Name:wds_ulHandlerInit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_ulHandlerInit(wdsLinkHandler_s *pUlStruct)
{
    pUlStruct->secureElapsedTime = 0;
    pUlStruct->unsuccessfulKeepAliveNumber = 0;
    pUlStruct->state = WDS_HANDLER_STATE_INIT;
    memset(pUlStruct->rssiBuffer, 0, sizeof(pUlStruct->rssiBuffer));
    pUlStruct->rssiBuffIndex = 0;
}
/*
 *Function Name:wds_ulHandlerJoin
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_ulHandlerJoin(wdsLinkHandler_s *pUlStruct, UINT8 * pMsg, UINT8 autoCfg, UINT8 *pCurrentHop)
{
    wdsProtocolMsg_s msg;
    UINT8 *pSrc = NULL;
    wdsProtocolMsg_s protoMsg;
    UINT8 hop;

    if((pUlStruct->state == WDS_HANDLER_STATE_INIT) || 
       (pUlStruct->state == WDS_HANDLER_STATE_JOIN)) {
        WDS_PROTOCOL_MSG_GET_BODY_m(pMsg, protoMsg);
        /* Check WDS name first and ignore messages from other WDS */
        if(strncmp(protoMsg.body.advertisement.WdsName, pUlStruct->wdsName, WDS_NAME_MAX_LEN) != 0) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Message from foreign WDS %s is ignored.\n", 
                                 protoMsg.body.advertisement.WdsName);
            return (OK);
        }
        if(autoCfg) {
            /* In dynamic configuration mode examinate message */
            /* NOT SUPPORTED - return */
            return(OK);
        }
        else {
            /* In static configuration mode check advertisement message sender */
            WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);

            if(memcmp(pUlStruct->linkPartnerMac, pSrc, MAC_ADDR_LEN) != 0) {
                return (OK);
            }
            WDS_PROT_GET_HOP_NUM_m(protoMsg, hop);

            if(hop > (WDS_MAX_HOPS_NUMBER - 1)) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Missconfiguration detected Uplink hop number too big. Can not connect.\n");
                return (OK);
            }
            *pCurrentHop = hop + 1;
        }

        /* Send join message here*/
        memset(&msg, 0, sizeof(wdsProtocolMsg_s));
        WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_JOIN_REQUEST);
        WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION);
    
        if(wds_sendProtocolMsg(pUlStruct->linkPartnerMac, pUlStruct->selfMac, &msg, 
                               pUlStruct->privacy, pUlStruct->pWlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send join message.\n");
            return (NOK);
    
        }
        pUlStruct->state = WDS_HANDLER_STATE_JOIN;
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Sent join req.\n");
    }
    pUlStruct->unsuccessfulKeepAliveNumber = 0;
    return (OK);

}

/*
 *Function Name:wds_ulHandlerJoinResp
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_ulHandlerJoinResp(wdsLinkHandler_s *pUlStruct, UINT8 * pMsg)
{
    char * pStrReason = NULL;
    wdsProtocolMsg_s protoMsg;
    UINT8 *pSrc = NULL, *pDest = NULL;


    WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);
    WDS_PROTOCOL_MSG_GET_DEST_m(pMsg, pDest);

    if(memcmp(pUlStruct->linkPartnerMac, pSrc, MAC_ADDR_LEN) != 0) {
        return (WDS_LINK_FAIL);
    }
    if(memcmp(pUlStruct->selfMac, pDest, MAC_ADDR_LEN) != 0) {
        return (WDS_LINK_FAIL);
    }

    WDS_PROTOCOL_MSG_GET_BODY_m(pMsg, protoMsg);
    if(protoMsg.body.joinResp.reason == WDS_JOIN_SUCCESS) {
        if(wds_ddaAddToStaTable(pUlStruct->devName, pUlStruct->linkPartnerMac, 
                               (pUlStruct->fourXEnabled ? TRUE : FALSE)) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error enabling net ifc %s!.\n", pUlStruct->devName);
        }
        if(pUlStruct->privacy) {
            if(pUlStruct->state != WDS_HANDLER_STATE_SECURE) {
            wds_mngrWpaSuplEnable(pUlStruct->linkPartnerMac);
            pUlStruct->secureElapsedTime = 0;
            pUlStruct->state = WDS_HANDLER_STATE_SECURE;
            }
            pUlStruct->unsuccessfulKeepAliveNumber = 0;
            return (WDS_LINK_FAIL);
        }
        if(wds_ddaNetIfcEnable(pUlStruct->devName, pUlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error enabling net ifc %s!.\n", pUlStruct->devName);
        }
        pUlStruct->linkEstablished++;
        return (WDS_LINK_OK);
    }
    else {
        /* Inform user about fail reason */
        switch(protoMsg.body.joinResp.reason) {
            case WDS_JOIN_FAIL_GENERIC :
                pStrReason = "generic fail"; 
                break;
            case WDS_JOIN_FAIL_NO_RESOURCES :    
                pStrReason = "no resources"; 
                break;
            case WDS_JOIN_FAIL_PEER_NOT_ALLOWED :
                pStrReason = "peer not allowed"; 
                break;
            case WDS_JOIN_FAIL_PARM_MISMATCH :   
                pStrReason = "parameters mismatch"; 
                break;
            case WDS_JOIN_FAIL_RSSI_LOW :         
                pStrReason = "low RSSI"; 
                break;
            default:
                pStrReason = "unknown"; 
                break;
        }

        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Join fail. Reason : %s\n", pStrReason);
        pUlStruct->state = WDS_HANDLER_STATE_INIT;
        return (WDS_LINK_FAIL);
    }
}

/*
 *Function Name:wds_dlHandlerKeepLink
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_dlHandlerKeepLink(wdsLinkHandler_s *pDlStruct, UINT8 * pMsg, UINT32 dlNumber)
{
    wdsProtocolMsg_s replMsg;
    wdsProtocolMsg_s protoMsg;
    UINT8 *pSrc, *pDest;
    dot11_header_T *pHdr;

    pHdr = (dot11_header_T *)pMsg;
    WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);
    WDS_PROTOCOL_MSG_GET_DEST_m(pMsg, pDest);

    if(memcmp(pDlStruct->linkPartnerMac, pSrc, MAC_ADDR_LEN) != 0) {
        return (OK);
    }
    if(memcmp(pDlStruct->selfMac, pDest, MAC_ADDR_LEN) != 0) {
        return (OK);
    }

    WDS_PROTOCOL_MSG_GET_BODY_m(pMsg, protoMsg);

    if((pDlStruct->state != WDS_HANDLER_STATE_SECURE) && 
       (pDlStruct->state != WDS_HANDLER_STATE_MAINTAIN)) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, "DL #%d Keep alive received in wrong state.\n", dlNumber);
    }
    else {
        pDlStruct->unsuccessfulKeepAliveNumber = 0;
        memset(&replMsg, 0, sizeof(wdsProtocolMsg_s));
        WDS_PROT_SET_MSG_TYPE_m(replMsg, WDS_MSG_KEEP_ALIVE_RESP);
        WDS_PROT_SET_VERSION_m(replMsg, WDS_PROTOCOL_VERSION); 
        replMsg.body.keepAlive.id = protoMsg.body.keepAlive.id;
        if(wds_sendProtocolMsg(pDlStruct->linkPartnerMac, pDlStruct->selfMac, 
                               &replMsg, pDlStruct->privacy, pDlStruct->pWlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error send keep alive response message.\n", dlNumber);
            return (NOK);
        }
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "DL #%d Keep alive request id %d.\n", dlNumber, protoMsg.body.keepAlive.id);
        /* Update RSSI information */
        pDlStruct->rssiBuffer[pDlStruct->rssiBuffIndex] = (short)(pHdr->duration);
        pDlStruct->rssiBuffIndex = (pDlStruct->rssiBuffIndex + 1) % WDS_RSSI_WINDOW_SIZE;
        /* Last keep alive ID is stored for debug purpose only */
        pDlStruct->lastKeepAliveId = ntohl(protoMsg.body.keepAlive.id);
    }
    return (OK);
}

/*
 *Function Name:wds_ulHandlerTimerHook
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ulHandlerTimerHook(wdsLinkHandler_s *pUlStruct)
{
    UINT32 retCode = WDS_LINK_OK;

    switch(pUlStruct->state) {
        case WDS_HANDLER_STATE_INIT:
            break;
        case WDS_HANDLER_STATE_JOIN:
            break;
        case WDS_HANDLER_STATE_SECURE:
            pUlStruct->secureElapsedTime++;
            if(pUlStruct->secureElapsedTime > WDS_SECURE_LINK_TIME_OUT) {
                retCode = WDS_LINK_FAIL;
                pUlStruct->state = WDS_HANDLER_STATE_INIT;
                pUlStruct->secureFailed++;
                wds_mngrCleanupLink(pUlStruct, TRUE) ;
            }
            pUlStruct->unsuccessfulKeepAliveNumber++;
            wds_ulHandlerSendKeepAlive(pUlStruct);
            break;
        case WDS_HANDLER_STATE_MAINTAIN:
            pUlStruct->unsuccessfulKeepAliveNumber++;
            wds_ulHandlerSendKeepAlive(pUlStruct);
            break;
        default:
            break;
    }
    if(pUlStruct->unsuccessfulKeepAliveNumber > WDS_NO_KEEP_ALIVE_MAX_NUMBER) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Keep alive time out!.\n");
        retCode = WDS_LINK_FAIL;
        pUlStruct->state = WDS_HANDLER_STATE_INIT;
        wds_mngrCleanupLink(pUlStruct, TRUE) ;
        if(wds_ddaNetIfcDisable(pUlStruct->devName, pUlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error disabling net ifc %s!.\n", pUlStruct->devName);
        }
        pUlStruct->linkFailed++;
    }
    return (retCode);
}

/*
 *Function Name:wds_ulHandlerStartMaintain
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_ulHandlerStartMaintain(wdsLinkHandler_s *pUlStruct)
{
    pUlStruct->state = WDS_HANDLER_STATE_MAINTAIN;
}

/*
 *Function Name:wds_ulHandlerAuthDone
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_ulHandlerAuthDone(wdsLinkHandler_s *pUlStruct, UINT32 result)
{
    pUlStruct->secureElapsedTime = 0;
    if(result == WDS_AUTHENTICATION_SUCCESS) {
        pUlStruct->state = WDS_HANDLER_STATE_MAINTAIN;
        if(wds_ddaNetIfcEnable(pUlStruct->devName, pUlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error enabling net ifc %s!.\n", pUlStruct->devName);
        }
        pUlStruct->linkEstablished++;
        return (WDS_LINK_OK);
    }
    else {
        pUlStruct->secureFailed++;
        pUlStruct->state = WDS_HANDLER_STATE_INIT;
        return (WDS_LINK_FAIL);
    }

}

UINT32 wds_ulHandlerAbort(wdsLinkHandler_s *pUlStruct, UINT8 *pMsg)
{
    UINT8 *pSrc;

    WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);

    if(memcmp(pSrc, pUlStruct->linkPartnerMac, MAC_ADDR_LEN) == 0) {

        WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, 
           "Abort message received from uplink partner <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n",  
           pSrc[0], pSrc[1], pSrc[2], pSrc[3], pSrc[4], pSrc[5]);
        pUlStruct->state = WDS_HANDLER_STATE_INIT;
        wds_mngrCleanupLink(pUlStruct, TRUE) ;
        if(wds_ddaNetIfcDisable(pUlStruct->devName, pUlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error disabling net ifc %s!.\n", pUlStruct->devName);
        }
        pUlStruct->linkFailed++;

        return (WDS_LINK_FAIL);
    }
    return (WDS_LINK_OK);

}



void wds_ulHandlerSendKeepAlive(wdsLinkHandler_s *pUlStruct)
{
    wdsProtocolMsg_s msg;

    if((pUlStruct->state == WDS_HANDLER_STATE_SECURE) || 
       (pUlStruct->state == WDS_HANDLER_STATE_MAINTAIN)) {
        /* Send keep alive message here*/
        memset(&msg, 0, sizeof(wdsProtocolMsg_s));
    
        WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_KEEP_ALIVE_REQ);
        WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION); 
    
        pUlStruct->lastKeepAliveId++;
        msg.body.keepAlive.id = htonl(pUlStruct->lastKeepAliveId);
    
        if(wds_sendProtocolMsg(pUlStruct->linkPartnerMac, pUlStruct->selfMac, &msg, 
                               pUlStruct->privacy, pUlStruct->pWlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send keep alive request message.\n");
            return ;
    
        }
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Sent keepalive id %d.\n", pUlStruct->lastKeepAliveId);
    }
}

/*
 *Function Name:wds_ulHandlerKeepAliveResp
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
void wds_ulHandlerKeepAliveResp(wdsLinkHandler_s *pUlStruct, UINT8 *pMsg)
{
    wdsProtocolMsg_s protoMsg;
    dot11_header_T *pHdr;
    UINT32 keepAliveId;

    pHdr = (dot11_header_T *)pMsg;
    WDS_PROTOCOL_MSG_GET_BODY_m(pMsg, protoMsg);
    keepAliveId = ntohl(protoMsg.body.keepAlive.id);

    if(keepAliveId == pUlStruct->lastKeepAliveId) {
        pUlStruct->unsuccessfulKeepAliveNumber = 0;
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "Keep alive id %d responded.\n", keepAliveId);
    }
    else {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, "Keep alive responded for id %d, expected %d.\n", 
                             keepAliveId, pUlStruct->lastKeepAliveId);
    }
    /* Update RSSI information */
    pUlStruct->rssiBuffer[pUlStruct->rssiBuffIndex] = (short)pHdr->duration;
    pUlStruct->rssiBuffIndex = (pUlStruct->rssiBuffIndex + 1) % WDS_RSSI_WINDOW_SIZE;

}

/*
 *Function Name:wds_dlHandlerJoinReq
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_dlHandlerJoinReq(wdsLinkHandler_s *pDlStruct, UINT8 * pMsg, UINT32 dlNumber)
{
    wdsProtocolMsg_s msg;

    if(pDlStruct->state != WDS_HANDLER_STATE_INIT) {
        /* Send join response message here*/
        memset(&msg, 0, sizeof(wdsProtocolMsg_s));
        WDS_PROT_SET_MSG_TYPE_m(msg, WDS_MSG_JOIN_RESPONSE);
        WDS_PROT_SET_VERSION_m(msg, WDS_PROTOCOL_VERSION); 

        msg.body.joinResp.reason = WDS_JOIN_SUCCESS;

        if(wds_sendProtocolMsg(pDlStruct->linkPartnerMac, pDlStruct->selfMac, &msg, 
                               pDlStruct->privacy, pDlStruct->pWlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error send join response message.\n", dlNumber);
            return (WDS_LINK_FAIL);
    
        }
        if(wds_ddaAddToStaTable(pDlStruct->devName, pDlStruct->linkPartnerMac,
                               (pDlStruct->fourXEnabled ? TRUE : FALSE)) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error enabling net ifc %s!.\n"
                                 , dlNumber, pDlStruct->devName);
        }
        if(pDlStruct->privacy) {
            if(pDlStruct->state != WDS_HANDLER_STATE_SECURE) {
                pDlStruct->state = WDS_HANDLER_STATE_SECURE;
                pDlStruct->secureElapsedTime = 0;
                wds_mngrWpaAuthlEnable(pDlStruct->linkPartnerMac);
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "DL #%d Join request received. Start security.\n", dlNumber);
            }
            pDlStruct->unsuccessfulKeepAliveNumber = 0;
            return (WDS_LINK_FAIL);

        }
        else {
            pDlStruct->state = WDS_HANDLER_STATE_MAINTAIN;
            if(wds_ddaNetIfcEnable(pDlStruct->devName, pDlStruct->linkPartnerMac) != OK) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error enabling net ifc %s!.\n"
                                     , dlNumber, pDlStruct->devName);
            }
            pDlStruct->linkEstablished++;
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "DL #%d Join request received.\n", dlNumber);
        }
    }
    pDlStruct->unsuccessfulKeepAliveNumber = 0;

    return (WDS_LINK_OK);
}


/*
 *Function Name:wds_dlHandlerAbort
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */
UINT32 wds_dlHandlerAbort(wdsLinkHandler_s *pDlStruct, UINT8 *pMsg, UINT32 dlNumber)
{
    UINT8 *pSrc;
    UINT32 retCode = WDS_LINK_OK;

    WDS_PROTOCOL_MSG_GET_SRC_m(pMsg, pSrc);
    if(pDlStruct->state == WDS_HANDLER_STATE_MAINTAIN) {
        /* Notify WDS manager link is realy failed */
        retCode = WDS_LINK_FAIL;
        wds_mngrCleanupLink(pDlStruct, FALSE); 
        if(wds_ddaNetIfcDisable(pDlStruct->devName, pDlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error disabling net ifc %s!.\n", dlNumber, pDlStruct->devName);
        }

        pDlStruct->linkFailed++;
    }

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, 
       "DL #%d Abort message received from link partner <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>.\n",  
       dlNumber, pSrc[0], pSrc[1], pSrc[2], pSrc[3], pSrc[4], pSrc[5]);
    pDlStruct->state = WDS_HANDLER_STATE_JOIN;
    return (retCode);
}
/*
 *Function Name:wds_dlHandlerTimerHook
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_dlHandlerTimerHook(wdsLinkHandler_s *pDlStruct, UINT32 dlNumber)
{
    UINT32 retCode = WDS_LINK_OK;

    switch(pDlStruct->state) {
        case WDS_HANDLER_STATE_INIT:
            break;
        case WDS_HANDLER_STATE_JOIN:
            break;
        case WDS_HANDLER_STATE_SECURE:
            pDlStruct->secureElapsedTime++;
            if(pDlStruct->secureElapsedTime > WDS_SECURE_LINK_TIME_OUT) {
                retCode = WDS_LINK_FAIL;
                pDlStruct->secureFailed++;
                pDlStruct->state = WDS_HANDLER_STATE_JOIN;
                wds_mngrCleanupLink(pDlStruct, FALSE); 
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Link secure time out!.\n", dlNumber);
            }
            pDlStruct->unsuccessfulKeepAliveNumber++;
            break;
        case WDS_HANDLER_STATE_MAINTAIN:
            pDlStruct->unsuccessfulKeepAliveNumber++;
            if(pDlStruct->unsuccessfulKeepAliveNumber > WDS_NO_KEEP_ALIVE_MAX_NUMBER) {
                WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Keep alive time out!.\n", dlNumber);
                retCode = WDS_LINK_FAIL;
                wds_mngrCleanupLink(pDlStruct, FALSE); 
                if(wds_ddaNetIfcDisable(pDlStruct->devName, pDlStruct->linkPartnerMac) != OK) {
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error disabling net ifc %s!.\n"
                                         , dlNumber, pDlStruct->devName);
                }

                pDlStruct->linkFailed++;
                pDlStruct->state = WDS_HANDLER_STATE_JOIN;
            }
            break;
        default:
            break;
    }
    return (retCode);
}

/*
 *Function Name:wds_dlHandlerAuthDone
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_dlHandlerAuthDone(wdsLinkHandler_s *pDlStruct, UINT32 result, UINT32 dlNumber)
{
    pDlStruct->secureElapsedTime = 0;
    if(result == WDS_AUTHENTICATION_SUCCESS) {
        pDlStruct->state = WDS_HANDLER_STATE_MAINTAIN;
        if(wds_ddaNetIfcEnable(pDlStruct->devName, pDlStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "DL #%d Error enabling net ifc %s!.\n"
                                 , dlNumber, pDlStruct->devName);
        }
        pDlStruct->linkEstablished++;
        return (WDS_LINK_OK);
    }
    else {
        pDlStruct->secureFailed++;
        pDlStruct->state = WDS_HANDLER_STATE_JOIN;
        return (WDS_LINK_FAIL);
    }

}

/*
 *Function Name:wds_mngrGetActiveDlNumber
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wds_mngrGetActiveDlNumber(wdsMngr_s *pMngrStruct)
{
    int i, dlNum = 0;
    for(i = 0; i < WDS_MAX_DL_NUMBER; i++) {
        if((pMngrStruct->downlink[i].state == WDS_HANDLER_STATE_SECURE) ||
           (pMngrStruct->downlink[i].state == WDS_HANDLER_STATE_MAINTAIN))
            dlNum++;
    }
    return (dlNum);
}

/*
 *Function Name:wds_mngrAbortSecure
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrAbortSecure(UINT8 *pMac)
{
    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "hidden_profile_SecureAbort not implementedd yet.\n");
}

/*
 *Function Name:wds_mngrWpaSuplEnable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrWpaSuplEnable(UINT8 *pMac)
{
    dynamicSecureCfg_T secureCfg;

    WDS_MAC_CPY(secureCfg.secPartyMac, pMac);
    if(hidden_profile_SecureSuplStart(&secureCfg) != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error start suplicant.\n");
    }
}

/*
 *Function Name:wds_mngrWpaAuthlEnable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrWpaAuthlEnable(UINT8 *pMac)
{
    dynamicSecureCfg_T secureCfg;

    WDS_MAC_CPY(secureCfg.secPartyMac, pMac);
    if(hidden_profile_SecureAuthStart(&secureCfg) != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error start authentication.\n");
    }
}

/*
 *Function Name:wds_mngrCleanupLink
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrCleanupLink(wdsLinkHandler_s *pLinkStruct, UINT32 isUplink) 
{
    dynamicSecureCfg_T linkParams;
    UINT8 mac[MAC_ADDR_LEN] = {0};


    if(pLinkStruct->privacy) {
        /* Only MAC address field is used */
        memcpy(linkParams.secPartyMac, pLinkStruct->linkPartnerMac, MAC_ADDR_LEN);

        if(isUplink) {
            hidden_profile_SuplLinkClose(&linkParams);
        }
        else {
            hidden_profile_AuthLinkClose(&linkParams);
        }
    }

    /* If Root - don't send event to STA table for Uplink */
    if( (!isUplink) || (memcmp(mac, pLinkStruct->linkPartnerMac, MAC_ADDR_LEN)) )
    {
        if(wds_ddaRemoveFromStaTable(pLinkStruct->devName, pLinkStruct->linkPartnerMac) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error removing station from stations table.\n");
        }
    }

    /* Clean Rssi Buffer */
    memset(pLinkStruct->rssiBuffer, 0, sizeof(pLinkStruct->rssiBuffer));
    pLinkStruct->rssiBuffIndex = 0;

}

/*
 *Function Name:wds_mngrMaintamceFrameSend
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrMaintamceFrameSend(wdsMngr_s *pMngrStruct)
{
    char wdsTiSnapHdr[] = WDS_TI_SNAP_HEADER;
    dot11_header_T  *pPktHdr;
    char buffer[256];
    UINT32 *pType;
    wdsInformationElement_s *pIe;
    int i;
    UINT32 msgLength, msgType;
    UINT32 ipAddr;
    UINT16 currElemLength;

    if(pMngrStruct->cfg.hop == 0) {
        /* Skip for WDS root bridge */
        return;
    }
    memset(buffer, 0, 256);
    /* Fill 802.11 header */
    pPktHdr = (dot11_header_T  *)buffer;
    pPktHdr->fc = (DOT11_FC_TYPE_DATA | DOT11_FC_SUB_DATA | DOT11_FC_TO_DS | DOT11_FC_FROM_DS);
    if(pMngrStruct->cfg.privacy) {
        pPktHdr->fc |= DOT11_FC_WEP;
    }
    WDS_MAC_CPY(pPktHdr->address1.addr, pMngrStruct->cfg.uplinkMac);
    WDS_MAC_CPY(pPktHdr->address2.addr, pMngrStruct->cfg.bss_id);
    WDS_MAC_CPY(pPktHdr->address3.addr, pMngrStruct->cfg.uplinkMac);
    WDS_MAC_CPY(pPktHdr->address4.addr, pMngrStruct->cfg.bss_id);

    /* Copy TI SNAP header */
    memcpy(buffer + sizeof(dot11_header_T), wdsTiSnapHdr, sizeof(wdsTiSnapHdr));

    /* Set type field */
    pType = (UINT32 *)(buffer + sizeof(dot11_header_T) + sizeof(wdsTiSnapHdr));
    msgType = htonl(WDS_BRIDGE_STATUS_REPORT_TYPE);
	 /* memcpy used to fix aligement problem */
    memcpy((char *)pType, (char *)&msgType, sizeof(msgType));

    /* Create Information elements */
    pIe = (wdsInformationElement_s *)(pType + 1);
    ipAddr = dda_wdsIpAddrGet(pMngrStruct->cfg.brgDevName);
    if(ipAddr != 0) {
        pIe->id = WDS_IE_ID_OWN_IP;
        currElemLength = sizeof(UINT32);
        pIe->length = htons(currElemLength);
        pIe->body.ipAddr = ipAddr; /* IP address should be in network byte order */
        (UINT32)pIe += currElemLength + WDS_IE_PREFIX_LENGTH;
    }
    pIe->id = WDS_IE_ID_UL_STATUS;
    currElemLength = sizeof(struct wdsIeWbniPeerStatus_s);
    pIe->length = htons(currElemLength);
    WDS_MAC_CPY(pIe->body.peerStatus.peerMac, pMngrStruct->uplink.linkPartnerMac);
    pIe->body.peerStatus.rssi = htons(wds_mngrCalcRssi(pMngrStruct->uplink.rssiBuffer));
    (UINT32)pIe += currElemLength + WDS_IE_PREFIX_LENGTH;
    for (i = 0; i < WDS_MAX_DL_NUMBER; i++) {
        if(WDS_IS_MAC_NOT_ZERO_m(pMngrStruct->downlink[i].linkPartnerMac)) {
            pIe->id = WDS_IE_ID_DL_STATUS;
            currElemLength = sizeof(struct wdsIeWbniPeerStatus_s);
            pIe->length = htons(currElemLength);
            WDS_MAC_CPY(pIe->body.peerStatus.peerMac, pMngrStruct->downlink[i].linkPartnerMac);
            if(pMngrStruct->downlink[i].state != WDS_HANDLER_STATE_MAINTAIN) {
                pIe->body.peerStatus.rssi = 0;
            }
            else {
                pIe->body.peerStatus.rssi = htons(wds_mngrCalcRssi(pMngrStruct->downlink[i].rssiBuffer));
            }
            (UINT32)pIe += currElemLength + WDS_IE_PREFIX_LENGTH;
        }
    }

    /* Send mesage to driver */
    msgLength = (UINT32)pIe - (UINT32)buffer;
    if(wds_ddaSendMsgToDriver(buffer, msgLength, pMngrStruct->cfg.wlanDevName) != OK) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send WDS maintenence informatin.\n");
    }
    else {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "WDS maintenence informatin sent. Length %d\n", msgLength);

    }
}

 /*
  *Function Name:wds_mngrHandleMaintenInfo
  *
  *Parameters:
  *
  *Description:
  *
  *Returns:
  *
  */
 
void wds_mngrHandleMaintenInfo(wdsMngr_s *pMngrStruct, UINT8 *pMsg, UINT32 msgLen)
{
    dot11_header_T  *pPktHdr;
    char wdsTiSnapHdr[] = WDS_TI_SNAP_HEADER;
    char *pType;
    UINT32 ieLength, Type;

    WDS_OUTPUT_MESSAGE_m(WDS_SEV_INFO, "WDS maintenence informatin received.\n");
    pPktHdr = (dot11_header_T  *)pMsg;
    pType = (char *)(pMsg + sizeof(dot11_header_T) + sizeof(wdsTiSnapHdr));
	 /* memcpy used to fix aligement problem */
    memcpy((char *)&Type, pType, sizeof(Type));
    Type = ntohl(Type);
    if(Type != WDS_BRIDGE_STATUS_REPORT_TYPE) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_WARNING, "Wrong type of WDS maintenence informatin: %d.\n", Type);
        return;
    }
    /* Relay this message if this is not root bridge */
    if(pMngrStruct->cfg.hop != 0) {
        /* Update addr1 and addr2 fields */
        WDS_MAC_CPY(pPktHdr->address1.addr, pMngrStruct->uplink.linkPartnerMac);
        WDS_MAC_CPY(pPktHdr->address2.addr, pMngrStruct->cfg.bss_id);
        if(wds_ddaSendMsgToDriver(pMsg, msgLen, pMngrStruct->cfg.wlanDevName) != OK) {
            WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error send WDS maintenence informatin.\n");
        }
    }
    ieLength = (msgLen - sizeof(dot11_header_T) - sizeof(wdsTiSnapHdr) - sizeof(UINT32));
    wds_mngrUpdateLinksDb(pMngrStruct, pPktHdr, (void *)(pType + sizeof(Type)), ieLength);
}




/*
 *Function Name:wds_mngrUpdateLinksDb
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrUpdateLinksDb(wdsMngr_s *pMngrStruct,  dot11_header_T  *pPktHdr, void *pBody, UINT32 ieLen)
{
    wdsInformationElement_s *pIe;
    UINT32 parsedLen = 0;
    int index;
    int dlIndex;

    pIe = (wdsInformationElement_s *)pBody;
    index = wds_mngrGetRecordIndex(pPktHdr->address4.addr);
    if(index < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error storing WDS maintenence informatin.\n");
        return;
    }
    memset(&wds_maintainDb[index], 0, sizeof(wdsMngrInfoRecord_s));
    WDS_MAC_CPY(wds_maintainDb[index].sender, pPktHdr->address4.addr);
    wds_maintainDb[index].timestamp = pMngrStruct->timestamp;
#ifdef WDS_PRINT_MAINT_INFO
    printf("Sender %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX, uplink %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX.\n",
           pPktHdr->address4.addr[0], pPktHdr->address4.addr[1], pPktHdr->address4.addr[2], 
           pPktHdr->address4.addr[3], pPktHdr->address4.addr[4], pPktHdr->address4.addr[5], 
           pPktHdr->address3.addr[0], pPktHdr->address3.addr[1], pPktHdr->address3.addr[2], 
           pPktHdr->address3.addr[3], pPktHdr->address3.addr[4], pPktHdr->address3.addr[5]);

#endif /* ifdef WDS_PRINT_MAINT_INFO */
    while (ieLen > parsedLen) {
        switch(pIe->id) {
            case WDS_IE_ID_OWN_IP:
                wds_maintainDb[index].ip = pIe->body.ipAddr;
                break;
            case WDS_IE_ID_UL_STATUS:
                WDS_MAC_CPY(wds_maintainDb[index].ul.mac, pIe->body.peerStatus.peerMac);
                wds_maintainDb[index].ul.rssi = ntohs(pIe->body.peerStatus.rssi);
#ifdef WDS_PRINT_MAINT_INFO
                printf("Uplink status length %d, peer %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX rssi %d.\n", 
                       ntohs(pIe->length), wds_maintainDb[index].ul.mac[0], wds_maintainDb[index].ul.mac[1],
                       wds_maintainDb[index].ul.mac[2], wds_maintainDb[index].ul.mac[3], 
                       wds_maintainDb[index].ul.mac, wds_maintainDb[index].ul.mac[5],
                       wds_maintainDb[index].ul.rssi);
#endif /* ifdef WDS_PRINT_MAINT_INFO */
                break;
            case WDS_IE_ID_DL_STATUS:
                dlIndex = 0;
                for(dlIndex = 0; dlIndex < WDS_MAX_DL_NUMBER; dlIndex++) {
                    if(!WDS_IS_MAC_NOT_ZERO_m(wds_maintainDb[index].dl[dlIndex].mac))
                       break;
                }
                if(dlIndex == WDS_MAX_DL_NUMBER) {
                    WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "WDS maintenence informatin wrong.\n");
                    return;
                }
                WDS_MAC_CPY(wds_maintainDb[index].dl[dlIndex].mac, pIe->body.peerStatus.peerMac);
                wds_maintainDb[index].dl[dlIndex].rssi = ntohs(pIe->body.peerStatus.rssi);
#ifdef WDS_PRINT_MAINT_INFO
                printf("Downlink status length %d, peer %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX rssi %d.\n", 
                       ntohs(pIe->length), 
                       wds_maintainDb[index].dl[dlIndex].mac[0], wds_maintainDb[index].dl[dlIndex].mac[1],
                       wds_maintainDb[index].dl[dlIndex].mac[2], wds_maintainDb[index].dl[dlIndex].mac[3], 
                       wds_maintainDb[index].dl[dlIndex].mac[4], wds_maintainDb[index].dl[dlIndex].mac[5],
                       wds_maintainDb[index].dl[dlIndex].rssi);
#endif /* ifdef WDS_PRINT_MAINT_INFO */
                break;
            default:
                return;
                break;

        }
        parsedLen += WDS_IE_PREFIX_LENGTH + ntohs(pIe->length);
        (UINT32)pIe = (UINT32)pBody + parsedLen;
    }
}

/*
 *Function Name:wds_mngrAgeInfoDb
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrAgeInfoDb(wdsMngr_s *pMngrStruct)
{
    int index;

    for (index = 0; index < WDS_MNGR_INFO_TABLE_SIZE; index++) {
        if(wds_maintainDb[index].timestamp == 0)
            continue;
        if((pMngrStruct->timestamp - wds_maintainDb[index].timestamp) > 20) {
            memset(&wds_maintainDb[index], 0, sizeof(wdsMngrInfoRecord_s));
        }
    }
}

/*
 *Function Name:wds_mngrGetRecordIndex
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

int wds_mngrGetRecordIndex(UINT8 *pMac)
{
    int index;
    int emptyRecord = -1;

    for (index = 0; index < WDS_MNGR_INFO_TABLE_SIZE; index++) {
        if(WDS_MAC_CMP(wds_maintainDb[index].sender, pMac) == 0) {
            return (index);
        }
        if(emptyRecord == -1) {
            if(WDS_IS_MAC_NOT_ZERO_m(wds_maintainDb[index].sender))
                continue;
            else 
                emptyRecord = index;
            
        }
    }
    return (emptyRecord);
}

/*
 *Function Name:wds_mngrProvideInfoDb
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wds_mngrProvideInfoDb(wdsMngr_s *pMngrStruct, wdsIpcMsg_s *pMsgIpc)
{
    int index, i, dlIndex;

    /* Add self info to db first */
    index = wds_mngrGetRecordIndex(pMngrStruct->cfg.bss_id);
    if(index < 0) {
        WDS_OUTPUT_MESSAGE_m(WDS_SEV_ERROR, "Error storing WDS maintenence informatin.\n");
        return;
    }
    memset(&wds_maintainDb[index], 0, sizeof(wdsMngrInfoRecord_s));
    WDS_MAC_CPY(wds_maintainDb[index].sender, pMngrStruct->cfg.bss_id);
    wds_maintainDb[index].timestamp = pMngrStruct->timestamp;
    wds_maintainDb[index].ip = dda_wdsIpAddrGet(pMngrStruct->cfg.brgDevName);
    WDS_MAC_CPY(wds_maintainDb[index].ul.mac, pMngrStruct->uplink.linkPartnerMac);
    wds_maintainDb[index].ul.rssi = wds_mngrCalcRssi(pMngrStruct->uplink.rssiBuffer);
    for(i = 0, dlIndex = 0; i < WDS_MAX_DL_NUMBER; i++) {
        if(WDS_IS_MAC_NOT_ZERO_m(pMngrStruct->downlink[i].linkPartnerMac)) {
            WDS_MAC_CPY(wds_maintainDb[index].dl[dlIndex].mac, pMngrStruct->downlink[i].linkPartnerMac);
            if(pMngrStruct->downlink[i].state != WDS_HANDLER_STATE_MAINTAIN) {
                wds_maintainDb[index].dl[dlIndex].rssi = 0;
            }
            else {
                wds_maintainDb[index].dl[dlIndex].rssi = wds_mngrCalcRssi(pMngrStruct->downlink[i].rssiBuffer);
            }
            dlIndex++;
        }
    }
    dda_wdsSendReport((char *)wds_maintainDb, sizeof(wds_maintainDb), &pMsgIpc->body.reportReq);
}
