/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/***************************************************************************
**                                                                        **
**   MODULE:    wdsMngr.h                                                 **
**   PURPOSE:   WDS manager Common Definitions                            **
**                                                                        **
***************************************************************************/

#ifndef __WDS_MNGR_H__
#define  __WDS_MNGR_H__

#ifdef __WDS_MNGR_C__
#define PUBLIC
#else /* ifdef __WDS_MNGR_C__ */
#define PUBLIC extern
#endif /* ifdef __WDS_MNGR_C__ */

#define WDS_VER_MAJOR 0
#define WDS_VER_MINOR 0
#define WDS_VER_PATCH 4

#ifdef VXWORKS
#undef printf
#define WDS_OUTPUT_MESSAGE_m(sev, fmt, args...) if(g_wdsPrintSeverity & (sev)) { \
                                                   char buff[32];   \
                                                   time_t currTime; \
												   int len;			\
                                                   if(time(&currTime) == ((time_t) -1)) printf("Error reading time"); \
                                                   ctime_r (&currTime, buff, &len); \
                                                   printf("%s %s : ", buff, __FUNCTION__); \
                                                   printf(fmt, ##args);}
#else /* ifdef VXWORKS */
#define WDS_OUTPUT_MESSAGE_m(sev, fmt, args...) if(g_wdsPrintSeverity & (sev)) { \
                                                   char buff[32];   \
                                                   time_t currTime; \
                                                   if(time(&currTime) == ((time_t) -1)) printf("Error reading time"); \
                                                   ctime_r (&currTime, buff); \
                                                   printf("%s %s : ", buff, __FUNCTION__); \
                                                   printf(fmt, ##args);}
#endif /* ifdef VXWORKS */
/* Hard coded WDS definitions */
#define WDS_TICK                  1000 /* WDS tick in msecs */
#define WDS_KEEP_ALIVE_PERIOD        1 /* Keep alive period in "WDS ticks" */
#define WDS_NO_KEEP_ALIVE_MAX_NUMBER 6 /* Max number of unsuccessful keep alive messages */  
#define WDS_SECURE_LINK_TIME_OUT     6 /* Secure link timeout in "WDS ticks" */
#define WDS_ADVERTISE_PERIOD         1 /* WDS advertise message send period */
#define WDS_ROOT_HOP_NUMBER          0 /* WDS root bridge hop number */
#define WDS_MAX_HOPS_NUMBER          2 /* Max number of bridging hops */
#define WDS_RSSI_WINDOW_SIZE        30 /* RSSI calculation window size */
#define WDS_BRIDGE_STATUS_REPORT_TYPE   3 /* Type field filler in WDS maintenance frame */
#define WDS_MAINEN_INFO_TIMEOUT         10 /* Maintenance information period */

/* Severity difinitions */
#define WDS_SEV_ERROR   1
#define WDS_SEV_WARNING 2
#define WDS_SEV_INFO    4
#define WDS_SEV_ALL     (WDS_SEV_ERROR | WDS_SEV_WARNING | WDS_SEV_INFO)

#define WDS_CH_SELECT_MODE_MANUAL   0
#define WDS_CH_SELECT_MODE_AUTO     1

/* WPA authentication result */
#define WDS_AUTHENTICATION_SUCCESS  0
#define WDS_AUTHENTICATION_FAIL     1

/* Link state */
#define WDS_LINK_OK     0
#define WDS_LINK_FAIL   1


/* Inter process communications messages types */
#define WDS_MSG_TYPE_CFG            1
#define WDS_MSG_TYPE_RESTART        2
#define WDS_MSG_TYPE_WPA_RESP       3
#define WDS_MSG_TYPE_EXIT           4
#define WDS_MSG_TYPE_REPORT         5
#define WDS_MSG_TYPE_REPORT_PID     6
#define WDS_MSG_TYPE_SEVERITY_SET   7
#define WDS_MSG_TYPE_ALARM          8
#define WDS_MSG_TYPE_STAT           9
#define WDS_MSG_TYPE_INFO_DB        10


typedef enum  {
    WDS_MSG_SRC_START_STATE,    /* Start this state notification */
    WDS_MSG_SRC_TIMER,          /* Time out notification */
    WDS_MSG_SRC_LINK_PARTNER,   /* WDS protocol packet arrived */
    WDS_MSG_SRC_IPC             /* Inter process communication packet arrived */
}  wdsMsgSource_e;



typedef struct  {
    wdsLinkHandlerStates_e state;       /* Link handler state */
    UINT8  selfMac[MAC_ADDR_LEN];       /* Self MAC address */
    UINT8  linkPartnerMac[MAC_ADDR_LEN];/* Link partner MAC address */
    UINT32 unsuccessfulKeepAliveNumber; /* Number of the unsuccessful keep alives*/
    UINT32 lastKeepAliveId;             /* Last sent keep alive msg ID */
    UINT32 secureElapsedTime;           /* Elapsed time */
    UINT8  privacy;                    /* WDS privacy mode: 0 - disabled, 1 - enabled */
    UINT8  wdsName[WDS_NAME_MAX_LEN];     /* WDS name */
    UINT8  devName[WDS_DEV_NAME_LEN];
    UINT8  fourXEnabled;                /* 4x enabled */
    short  rssiBuffer[WDS_RSSI_WINDOW_SIZE];
    UINT32 rssiBuffIndex;
    UINT8  *pWlanDevName;                   /* Device name used to pass messages to the driver */
    /* Statistics */
    UINT32 linkEstablished;
    UINT32 secureFailed;
    UINT32 linkFailed;
} wdsLinkHandler_s;

typedef struct  {
    UINT32 isConfigured;            /* Configured flag */
    wdsMngrCfg_s cfg;               /* WDS configuration */
    wdsMngrStates_e state;          /* WDS manager state */
    wdsLinkHandler_s uplink;      /* WDS uplink handler */
    wdsLinkHandler_s downlink[WDS_MAX_DL_NUMBER]; /* WDS downlink hadlers */
    UINT8 currentHop;               /* Current hop number */
    UINT32 maintenInfoTimeOut;      /* Maintenance information sending timeout */
    UINT32 timestamp;               /* WDS timestamp - value incremented every WDS tick */
} wdsMngr_s;


typedef struct  {
    UINT8  linkPartnerMac[MAC_ADDR_LEN];  /* Link partner MAC address */
    UINT32 result;                         /* Authentication result */
    UINT32 failReson;                      /* Failure details */
} wdsMngrWpaResponce_s;

typedef struct  {
    UINT32 type;
    union {
        wdsMngrCfg_s            cfg;
        wdsMngrWpaResponce_s    wpaResp;
        UINT32                  param;
        wdsMngrReportRequest_s  reportReq;
    }  body;
} wdsIpcMsg_s;


PUBLIC UINT32 wds_ddcInit(wdsMngr_s **pMngrStruct, UINT32 *pTimeout);
void wds_ddcRelease(wdsMngr_s *pMngrStruct);
PUBLIC void wds_messageHandler(wdsMngr_s *pMngrStruct, wdsMsgSource_e msgSource, UINT8 * pMsg, UINT32 msgLen);
PUBLIC UINT32 wds_validateCfg(wdsMngrCfg_s *pCfg);
PUBLIC UINT32 wds_mngrGetInfoDb(void *pBuffer);


PUBLIC UINT32 g_wdsPrintSeverity;

#undef PUBLIC
#endif /* ifndef __WDS_MNGR_H__ */

