/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	bssbridge.c	      														*/
/*   PURPOSE:	Bssbridge functions.	      			 								*/
/*																									*/
/***************************************************************************/
#include "pform_types.h"
#include <stdio.h>
#include "bssbridge.h"
#include "mainsm.h"
#include "802_11Defs.h"
#include "tx.h"
#include "wlan_pal.h"
#include "dda_wpa_interface.h"
#include "dda_contexts.h"
#include "QoSClassifier.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtIf.h"
#include "dda.h"
#include "mgmtSsidHandler.h"
#include "mgmtSsidHandler.h"



/*
This is an Ethernet Version 2 frame:

       +--------------+
       |              | The destination address is a six byte Media Access
       | Destination  | Control (MAC) address, usually burned into the
       |   6 bytes    | ROM of the Ethernet card.
       +--------------+
       |              | The source address is a six byte MAC address, and
       |   Source     | can signify a physical station or a broadcast.
       |   6 bytes    |
       +--------------+
       |     Type     | The Type field it must be grater then 1500 dec.
       |    2 bytes   |
       +--------------+
       |              |  Any higher layer information is placed in the
       |    Data      |  data field, which could contain protocol
       |              |  information or user data.
       ~              ~
       ~              ~
       |  46 to 1500  |
       |    bytes     |
       |              |
       +--------------+
       |     FCS      |  
       |   4 bytes    |  
       +--------------+  
                         


*/
/*

        802.2 SNAP DATA Frame 
       +----------------+
       |                |
       |  Destination   |
       |    6 bytes     |
       +----------------+
       |                |
       |     Source     |
       |    6 bytes     |
       +----------------+
       |  Frame Length  | Must be <= 1500 Dec.
       |    2 bytes     |
       +----------------+
       |  DSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |  SSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |Control - 1 byte| = 0x03 
       +----------------+ 
       | OUI - 3 bytes  | = 0x0
       |                | 
       +----------------+
       | Type - 2 bytes |  = Ethernet type (IP=0x0800)
       +----------------+ 
       |                | 
       |      Data      |
       |                |
       ~                ~
       ~                ~
       |   46 to 1500   |
       |     bytes      |
       |                |
       +----------------+
       |      FCS       |
       |    4 bytes     |
       +----------------+




        802.11 DATA Frame
       +----------------+
       |                |
       |  Frame Control |
       |    2 bytes     | 
       +----------------+
       |                |
       |  Duration ID   |
       |    2 bytes     |        
       +----------------+
       |                |
       |   Address 1    |
       |    6 bytes     |
       +----------------+
       |                |
       |   Address 2    |
       |    6 bytes     |
       +----------------+
       |                |
       |   Address 3    |
       |    6 bytes     |
       +----------------+
       |                |
       | Sequence Cntrl |
       |    2 bytes     |
       +----------------+
       |                |
       |   Address 4    |
       |    6 bytes     |
       +----------------+
       |  DSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |  SSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |Control - 1 byte| = 0x03 
       +----------------+ 
       | OUI - 3 bytes  | = 0x0
       |                | 
       +----------------+
       | Type - 2 bytes |  = Ethernet type (IP=0x0800)
       +----------------+ 
       |                | 
       |      Data      |
       |                |
       ~                ~
       ~                ~
       |   46 to 1500   |
       |     bytes      |
       |                |
       +----------------+
       |      FCS       |
       |    4 bytes     |
       +----------------+

*/

#pragma pack(2)
typedef struct
{
  macAddress_T	DstAddr;
  macAddress_T	SrcAddr;
  UINT16    Length;
  UINT8     DSAP;
  UINT8     SSAP;
  UINT8     Control;
  UINT8     OUI[3];
  UINT16		Type;
} LlcSnapHeader_T;
#pragma pack()

static UINT8 SNAP_OUI_802_1H[] = { SNAP_OUI_802_1H_BYTE0, SNAP_OUI_802_1H_BYTE1, SNAP_OUI_802_1H_BYTE2 };


static UINT8 SNAP_OUI_RFC1042[] = { SNAP_OUI_RFC1042_BYTE0, SNAP_OUI_RFC1042_BYTE1, SNAP_OUI_RFC1042_BYTE2 };
#define BSS_PERF_DBG

extern void wdrv_dbGetWlanServiceState ( BOOL *state );

/* Gather all statistics in one structure */
static struct bssBrgStat {
   UINT32 pktsUnicWlanToWlan;
   UINT32 pktsUnicWlanToEthr;
   UINT32 pktsUnicEthrToWlan;
   UINT32 pktsBrcstFromWlan;
   UINT32 pktsBrcstFromEthr;
   UINT32 pktsFromEthrToWlanErrStaNotFound;
   UINT32 pktsDroppedDueToIsolation;
} bssBrgStat = {0};

static UINT32 bssBridgeExcludedCount;
static UINT32 bssBridgeTkipMicFailureCount;
static UINT32 bssBridgeRxVLANDropCount;
static UINT32 bssBridgeTxWrongVLANDRopCount;
static UINT32 bssBridgeTxmSSIDnoVLANDropCount;

/* Demiurg: WME */
static wdrv_QoSACParams_t wmeACParams;

/****************************************************************************
 *                      wdrv_bssbridgeInit()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_bssbridgeInit(void) {
	STATUS ret = OK;

	bssBridgeExcludedCount = 0;
	bssBridgeTkipMicFailureCount = 0;
	bssBridgeRxVLANDropCount = 0;
	bssBridgeTxWrongVLANDRopCount = 0;
	bssBridgeTxmSSIDnoVLANDropCount = 0;

	wdrv_mainsmSendModuleUp(WDRV_BSS_BRIDGE_MODULE_ID);

	if (wdrv_dbGetWMEEnabled())
		wmeACParams = wdrv_dbGetWMEParams();

	return ret;
}

STATUS wdrv_bssBridgeShutdown(void) 
{
	return OK;
}


/* TODO : eliminate this function, we already have wlan_memMngrCopyMsdu */

/****************************************************************************
 *                      copyMsdu()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:
 *
 *
 *
 * OUTPUT:	None
 *
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS copyMsdu(mem_MSDU_T *pDstMsdu, mem_MSDU_T *pSrcMsdu )
{
    mem_BD_T* srcBdPtr;
    mem_BD_T* dstBdPtr;
    mem_BD_T* tmpDstBdPtr;

   srcBdPtr = pSrcMsdu->firstBDPtr;
   dstBdPtr = pDstMsdu->firstBDPtr;

   dstBdPtr->length =  srcBdPtr->length;
   dstBdPtr->dataOffset = srcBdPtr->dataOffset;
   PAL_memCopy( dstBdPtr->data +  dstBdPtr->dataOffset,
           srcBdPtr->data + srcBdPtr->dataOffset,
           srcBdPtr->length );
  
   pDstMsdu->dataLen =  srcBdPtr->length;

   while( pDstMsdu->dataLen <  pSrcMsdu->dataLen ){

       srcBdPtr = srcBdPtr->nextBDPtr;
       
       if( wlan_memMngrAllocDataBuf(&tmpDstBdPtr, DATA_BUF_LEN) == NOK){
			wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION,
							"WDRV_DB: BSS bridge, copyMsdu - fail to allocate payloadBdPtr\n");
            return NOK;
       }
       dstBdPtr->nextBDPtr = tmpDstBdPtr;
       dstBdPtr = tmpDstBdPtr;
    
       dstBdPtr->length =  srcBdPtr->length;
       dstBdPtr->dataOffset = srcBdPtr->dataOffset;
	    PAL_memCopy( dstBdPtr->data +  dstBdPtr->dataOffset,
          srcBdPtr->data + srcBdPtr->dataOffset,
          srcBdPtr->length );
       
       pDstMsdu->dataLen +=  srcBdPtr->length;
   }


   pDstMsdu->dataLen = pSrcMsdu->dataLen;

   pDstMsdu->timestamp = pSrcMsdu->timestamp;

	 pDstMsdu->wmeAC = pSrcMsdu->wmeAC;

   return OK;
}


/****************************************************************************
 *                      wdrv_bssBridgeAddBd4header()
 ****************************************************************************
 * DESCRIPTION:	Adds additional BD for WLAN header
 *
 * INPUTS: pMsdu - Msdu to be sent to ETH/WLAN
 *
 *
 * 
 * OUTPUT:	The same Msdu
 * 
 * RETURNS:
 ****************************************************************************/
STATUS wdrv_bssBridgeAddBd4header(mem_MSDU_T *pMsdu, BOOL addQoSHeader)
{
   mem_BD_T        *pBD, *tempBD;
   int             hasQoSHeader, hdrLen;
   dot11_header_T  *pDot11Hdr;

   pDot11Hdr = (dot11_header_T*) memMgr_BufData(pMsdu->firstBDPtr) ;
	 (ULONG)pDot11Hdr += memMgr_BufOffset(pMsdu->firstBDPtr) ;

   /* before sending to TX, allocate additional BD for WLAN header */
   hasQoSHeader = ((pDot11Hdr->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_QOS_DATA) || 
		 ((pDot11Hdr->fc & DOT11_FC_TYPESUBTYPE_MASK ) == DOT11_FC_QOS_NULL);
   hdrLen = WLAN_HDR_LEN + (hasQoSHeader || addQoSHeader)*WLAN_QOS_HDR_LEN;
   wlan_memMngrAllocDataBuf(&pBD, hdrLen);
   if (pBD == NULL)
   {
	   printf("%s: failed to allocate BD\n",__FUNCTION__);
	   return NOK;
   }

   /* link the BD to the Msdu */
   tempBD = pMsdu->firstBDPtr;
   pMsdu->firstBDPtr = pBD; 
   pMsdu->firstBDPtr->nextBDPtr = pMsdu->lastBDPtr = tempBD;
   /* copy WLAN header to the new BD */
   PAL_memCopy(pBD->data,
               pMsdu->firstBDPtr->nextBDPtr->data+pMsdu->firstBDPtr->nextBDPtr->dataOffset,
               hdrLen);
   pBD->length = hdrLen;
	 pMsdu->firstBDPtr->nextBDPtr->dataOffset += hdrLen - addQoSHeader*WLAN_QOS_HDR_LEN;   
   pMsdu->firstBDPtr->nextBDPtr->length -= hdrLen - addQoSHeader*WLAN_QOS_HDR_LEN;
	 if (addQoSHeader) {
		 dot11_QoSControlHeader_t * qosHeader;

		 pDot11Hdr = (dot11_header_T*) memMgr_BufData(pMsdu->firstBDPtr) ;

		 qosHeader = (dot11_QoSControlHeader_t *)((ULONG)pDot11Hdr + WLAN_HDR_LEN);

		 PAL_memSet(qosHeader, 0, WLAN_QOS_HDR_LEN);
		 DOT11_QOS_ACK_SET(qosHeader, wmeACParams.ap_edcfParams[WDRV_QOS_CLASSIFIER_AC_BE].ackPolicy);
		 DOT11_QOS_UP_SET(qosHeader, 0); 
		 pDot11Hdr->fc |= DOT11_FC_QOS_DATA;
	 }

	 return OK;
}

extern int wlan_have_packages;
/****************************************************************************
 *                      wdrv_bssbridgeInit()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function
 *
 * INPUTS:
 *
 *
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_bssBridgeFromWlanFrame(mem_MSDU_T *pMsdu,UINT32 pktEmulate, BOOL micOK, UINT16 srcStaIndex)
{
	typedef enum 
	{
	  WLAN = 0x1,
	  END  = 0x2,
	} dest_T;

	mem_MSDU_T *pBrctMsdu;
	UINT8 stationCipher = CIPHER_NONE;
	BOOL	srcAuthorized = FALSE, wepOn = FALSE, frameIsEapolToAp = FALSE;
	UINT8 *dataBuf, DA[6], apMac[6], *SA;
	register dot11_header_T  *pDot11Hdr;
	UINT16 staIndex;
	dest_T DEST = 0;
	ULONG txFlags = 0; /*WDRV_TX_LOW_PRIORITY;*/
    UINT8 wone_index, totalWones;
    mem_MSDU_T *pCopyMsdu = NULL;
	BOOL		bssBridgeIsolationOn;
	BOOL    packetHasQoSHeader = FALSE;
	BOOL addQoSHeader = FALSE;
	UINT8 *pDest;
	UINT16 etherType;
    drvDbHashStaType_e staType;

  
	bm_trace(2,0,0);
	wlan_have_packages++;

	dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
	pDot11Hdr = (dot11_header_T*) dataBuf;

    SA = pDot11Hdr->address2.addr;
#ifdef BSS_PERF_DBG
    MACDUP2( DA, (UINT8*)&pDot11Hdr->address3);
#else
	PAL_memCopy( DA, (UINT8*)&pDot11Hdr->address3, 6 );
#endif


	/* Demiurg: WME */
    /* set the txFlags for the relevant AC */
	if (! wmeACParams.enabled) 
		txFlags = WDRV_TX_LOW_PRIORITY;
	else if (wdrv_dbGetStaWME(srcStaIndex))  {
		wdrv_QoSClassifierAC_t wmeAC;
		if (((pDot11Hdr->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_QOS_DATA) || ((pDot11Hdr->fc & DOT11_FC_TYPESUBTYPE_MASK ) == DOT11_FC_QOS_NULL))  {
			dot11_QoSControlHeader_t * qosHeader;
			UINT8 up;
			
			qosHeader = (dot11_QoSControlHeader_t *)((ULONG)pDot11Hdr + WLAN_HDR_LEN);
			up = DOT11_QOS_UP(qosHeader);
			if (up > WDRV_QOS_MAX_UP)  {
				if (REPORT_LEVEL(RL_BRIDGE)) 
					wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "%s : Wrong UP in QoS header !\n", __FUNCTION__);
				wmeAC = WDRV_QOS_CLASSIFIER_AC_BE;
			} else  {
				wmeAC = wdrv_QoSAuxMapUPtoAC[up];
				packetHasQoSHeader = TRUE;
			}
		} else {
			wmeAC = WDRV_QOS_CLASSIFIER_AC_BE;
		}
		
		txFlags |= wdrv_QoSAuxMapACtoTxFlags[wmeAC];
        pMsdu->wmeAC = (UINT8)wmeAC;
	}

	wdrv_dbGetBssid(apMac);

    if (pDot11Hdr->fc & DOT11_FC_FROM_DS)
    {
        pDest = pDot11Hdr->address1.addr;
    } else
    {
        pDest = pDot11Hdr->address3.addr;
    }

#ifdef BSS_PERF_DBG
    if ((isEapol(pMsdu,packetHasQoSHeader, FALSE, &etherType)) && MACCMP(apMac, pDest))
#else
    if ((isEapol(pMsdu,packetHasQoSHeader, FALSE, &etherType)) && (PAL_memCmp(apMac, pDest, MAC_ADDR_LEN) == 0))
#endif
    {
        frameIsEapolToAp = TRUE;
    }

		if (etherType == ETHERTYPE_802_1D) {
			++bssBridgeRxVLANDropCount;
			wlan_memMngrFreeMSDU(pMsdu->handle);
			if(REPORT_LEVEL(RL_BRIDGE)) 
				wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "Dropped packet with VLAN on RX!\n");
			return;
		}

    if (wdrv_dbIsDataFirstFromStation(srcStaIndex))
        dda_wpa_first_dataFromStation(srcStaIndex, frameIsEapolToAp);


    /* At this point the WPA task may delete the station from the table. */  	
    if( wdrv_dbFindStaIndex(SA, &staIndex) != OK )
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_INFORMATION, "WDRV_DB: BSS bridge, station removed from database becasue of first data\n");
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return;
	}

	if(REPORT_LEVEL(RL_BRIDGE)) 
		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, SRC station %d (%02hx-%02hx-%02hx-%02hx-%02hx-%02hx) was found\n",staIndex,
						 SA[0],SA[1],SA[2],SA[3],SA[4],SA[5]);

	srcAuthorized = wdrv_mgmtIsStationAuthorized(staIndex);
	if  (((pDot11Hdr->fc) & DOT11_FC_WEP) == DOT11_FC_WEP)
		wepOn = TRUE; 
	wdrv_dbGetStationSuiteType(staIndex, &stationCipher);
	
	if (srcAuthorized == FALSE)
	{
		/* Drop all frames with WEP ON received from non-authorized station. */
		if (wepOn == TRUE)
		{
			wlan_memMngrFreeMSDU(pMsdu->handle);
			return;
		}
		
		else	/* WEP is off, than except only EAPOL frames. */
		{
			if (frameIsEapolToAp == TRUE)
			{
				wdrv_bssBridgeSendPktToEthernet(pMsdu ,TRUE);
				return;
			}			
			else		/* Not EAPOL */
			{
				wlan_memMngrFreeMSDU(pMsdu->handle);
				return;
			}

		}
	} 
	if(REPORT_LEVEL(RL_BRIDGE)) 
		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, SRC station is authorized\n");


	/* If we are here, it means that the station is authorized. */

	
	/* Exclude if needed any frames with WEP OFF, but forward EAPOLS */
	if (wepOn == FALSE)
	{
		if (frameIsEapolToAp == TRUE)		/* EAPOL */
		{
			
			wdrv_bssBridgeSendPktToEthernet(pMsdu,TRUE);
			return;
		}
		else		/* Not EAPOL */
		{
			if(wdrv_mgmt_ssidIsStationPrivOptOn(srcStaIndex))
			{
				wlan_memMngrFreeMSDU(pMsdu->handle);
				bssBridgeExcludedCount++;
				return;
			}
		}
	}
	
	else	/* WEP is ON */
	{
		if (stationCipher == CIPHER_TKIP)		/* if TKIP, check MIC */
		{
			if (micOK == FALSE)			/* MIC Failure */
			{

				dda_wpa_SetStationSecAttack(staIndex, TRUE);		/* Set the security attack flag for this station */
				dda_wpa_rxMicFailure(staIndex);
				bssBridgeTkipMicFailureCount++;
				wlan_memMngrFreeMSDU(pMsdu->handle);
				return;
			}

		}
		
		/* Now if it is a EAPOL we have to forward it to WPA */
		if (frameIsEapolToAp)
		{
			wdrv_bssBridgeSendPktToEthernet(pMsdu,TRUE);
			return;
		}
	}
	
	
	/*--------------------------------------------------*/
	/*             Destination determination            */
	/*--------------------------------------------------*/
   
	wdrv_dbGetBssIsolationState(&bssBridgeIsolationOn);
	if( DA[0] & 1 ) 
	{ 
        int profile_index = 0;
        wdrv_mgmt_ssidGetStationSsidIndex(staIndex,&profile_index);
        staIndex = profile_index;

      /* broadcast or multicast, bit40 of MAC address is set */
		/* we allow forwarding it to the WLAN only if the broadcst MAC is authorized. */    
		DEST = END;

		if (wdrv_mgmtIsBroadcastAuthorized() && !bssBridgeIsolationOn)
			DEST |= WLAN;

		bssBrgStat.pktsBrcstFromWlan++;
   }
    
   else 
   {  
       /* Unicast */
    if( wdrv_dbFindStaIndex(DA, &staIndex) == OK ) {

            wdrv_dbGetStationType(staIndex, &staType);
            /* Check STA type - regular STA or WDS STA */
            if(staType == DRV_DB_HASH_STA_TYPE_STATION)
            {
                if(REPORT_LEVEL(RL_BRIDGE)) 
                    wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, wdrv_dbFindStaIndex(%02hx-%02hx-%02hx-%02hx-%02hx-%02hx, %d) == OK\n",
                    DA[0],DA[1],DA[2],DA[3],DA[4],DA[5],staIndex);
                
                if (wdrv_dbGetStaWME(staIndex) )
                {
                    if( !packetHasQoSHeader)  
                    {
                        addQoSHeader = TRUE;
                        txFlags &= (~WDRV_TX_AC_MASK);
                        txFlags &= ~(WDRV_TX_LOW_PRIORITY);
                        txFlags &= ~(WDRV_TX_HIGH_PRIORITY);
                        txFlags |= wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_BE];
                    }
                }
                else
                {
                    addQoSHeader = FALSE;
                    txFlags &= (~WDRV_TX_AC_MASK);
                    txFlags &= ~(WDRV_TX_LOW_PRIORITY);
                    txFlags &= ~(WDRV_TX_HIGH_PRIORITY);
                    txFlags |= wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_BE];
                }
                
                /* TODO(shlomi)  :add check if it is on the same vlan case multi ssid*/ 
                if (wdrv_mgmtIsStationAuthorized(staIndex))	{
                    if(REPORT_LEVEL(RL_BRIDGE)) 
                        wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = WLAN\n");
                    DEST = WLAN; 
                    bssBrgStat.pktsUnicWlanToWlan++;  
                }	 else	 {
                /* station is not authorized, but still appears in the table (e.g. authenticated).
                One case is fast roaming. Station is disassociation from this AP, but now
                associated on another AP, probably on the same LAN, maybe this AP connected
                directly with this AP.
                For that, forward the packet on the bridge..
                    */
                    if(REPORT_LEVEL(RL_BRIDGE)) 
                        wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = END\n");
                    DEST = END;
                    bssBrgStat.pktsUnicWlanToEthr++;
                }
                
            }
            else /* If STA is WDS STA */
            {
                if(REPORT_LEVEL(RL_BRIDGE)) 
                    wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = END\n");
                DEST = END;
                bssBrgStat.pktsUnicWlanToEthr++;
            }
    }
    else 
	  { 
		  if(REPORT_LEVEL(RL_BRIDGE)) 
			  wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = END\n");
         DEST = END;
         bssBrgStat.pktsUnicWlanToEthr++;
      }
   }
   
   if(DEST == WLAN && bssBridgeIsolationOn)
   {
		wlan_memMngrFreeMSDU(pMsdu->handle);
		bssBrgStat.pktsDroppedDueToIsolation++;
		return;
   }

#if 0
	if(pktEmulate == FALSE)
      rc = wdrv_ackEmulationRxPacket(pMsdu);
#endif

   if( DEST == END ){
      /* Packet destined to the END, send it and return. */
	   if(REPORT_LEVEL(RL_BRIDGE)) 
		   wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, Packet destined to the END, send it and return.\n");
   	  
	   wdrv_bssBridgeSendPktToEthernet(pMsdu,FALSE);
      return;
   }

   
   if( DEST == WLAN ) { 
         txFlags |= WDRV_TX_UNICAST_MSDU;
   }

   else /*DEST = WLAN | END */ 
   {
	   if(REPORT_LEVEL(RL_BRIDGE)) 
		  wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = WLAN | END.\n");

      if( wlan_memMngrAllocMSDU ( &pBrctMsdu, pMsdu->dataLen ) == OK )
      {

         if(copyMsdu( pBrctMsdu, pMsdu ) == OK)
			 wdrv_bssBridgeSendPktToEthernet(pMsdu,FALSE);
      }
      else
      {
          if (REPORT_LEVEL(RL_MEM))
              wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"failed in wlan_memMngrAllocMSDU");
		  wlan_memMngrFreeMSDU(pMsdu->handle);
		  return;
      }

      txFlags |= WDRV_TX_BRCST_MTCST_MSDU;   
   }

   if(!(txFlags & WDRV_TX_BRCST_MTCST_MSDU))
   {
	   pBrctMsdu = pMsdu;
   }
   else
   {
	   dataBuf = memMgr_BufData(pBrctMsdu->firstBDPtr);
       pDot11Hdr = (dot11_header_T*) dataBuf;
   }
   /* If the destination is no WME station or broadcast remove the QOS header */
   if (packetHasQoSHeader && (!wdrv_dbGetStaWME(staIndex) || (txFlags & WDRV_TX_BRCST_MTCST_MSDU))) 
   {
       /* shift the 802.11 header 2 byte */
       PAL_memMove((UINT8 *)((ULONG)dataBuf + WLAN_QOS_HDR_LEN), dataBuf, WLAN_HDR_LEN);
       (ULONG)memMgr_BufOffset(pBrctMsdu->firstBDPtr) += WLAN_QOS_HDR_LEN;
       (ULONG)dataBuf += WLAN_QOS_HDR_LEN;
       pDot11Hdr = (dot11_header_T*) dataBuf;
			 pDot11Hdr->fc &= ~(DOT11_FC_QOS_DATA | DOT11_FC_QOS_NULL);
			 pDot11Hdr->fc |= DOT11_FC_DATA;
   }

   /*--------------------------------------------------*/
   /* Sending the original buffer to the WLAN          */
   /*--------------------------------------------------*/
   pDot11Hdr->fc &= ~( DOT11_FC_TO_DS | DOT11_FC_MORE_FRAG | DOT11_FC_RETRY );
   pDot11Hdr->fc |= DOT11_FC_FROM_DS;

#ifdef BSS_PERF_DBG
   MACDUP2( (void*)&pDot11Hdr->address3, (void*)&pDot11Hdr->address2);
   MACDUP2( (void*)&pDot11Hdr->address2, (void*)&pDot11Hdr->address1);
   MACDUP2( (void*)&pDot11Hdr->address1, DA);
#else
   PAL_memCopy( (void*)&pDot11Hdr->address3, (void*)&pDot11Hdr->address2, 6 );
   PAL_memCopy( (void*)&pDot11Hdr->address2, (void*)&pDot11Hdr->address1, 6 );
   PAL_memCopy( (void*)&pDot11Hdr->address1, DA, 6);
#endif

   pBrctMsdu->lastBDPtr = pBrctMsdu->firstBDPtr;


   if (txFlags & WDRV_TX_BRCST_MTCST_MSDU)
   {
      wdrv_dbGetNumSupWone(&totalWones);
      if(totalWones == WONE_DUAL_BAND)
      {
         /* duplicate broadcast packet for second wone */

            if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(pBrctMsdu)) == OK )
            {
                wlan_memMngrCopyMsdu( pCopyMsdu, pBrctMsdu);
                if(wdrv_bssBridgeAddBd4header(pCopyMsdu, addQoSHeader) == OK)
					wdrv_txSendMsdu( pCopyMsdu, 0, txFlags,SECOND_WONE_NET);
				else
					wlan_memMngrFreeMSDU(pCopyMsdu->handle);
            } 
            else
            {
                if (REPORT_LEVEL(RL_MEM))
                    wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"1 fail to wlan_memMngrAllocMSDU");
            }
      }
      wone_index = FIRST_WONE_NET;
   } 
   else
   {
      wdrv_dbGetStationWoneIndex(staIndex, &wone_index);
   }

   if(wdrv_bssBridgeAddBd4header(pBrctMsdu, addQoSHeader) == OK)
	   wdrv_txSendMsdu( pBrctMsdu, staIndex, txFlags,wone_index);
   else
	   wlan_memMngrFreeMSDU(pBrctMsdu->handle);
}



/****************************************************************************
 *                      wdrv_bssBridgeSendPktToEthernet()                   *
 ****************************************************************************
 * DESCRIPTION:	Send the WLAN packet to the END using Ethernet header.	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_bssBridgeSendPktToEthernet(mem_MSDU_T *pMsdu,BOOL isEapol)
{
   
   EthernetHeader_T EthHeader;
 
   Wlan_LlcHeader_T  *pWlanSnapHeader;   
   UINT8 			*dataBuf;
   register dot11_header_T  	*pdot11Header;
   UINT32   lengthDelta;
   UINT8    createEtherIIHeader;
	 UINT8    hasQoSHeader;

   /* 802.1q changes*/
   Ethernet802_1qHeader_T	ExtEthHeader;
   UINT32			etherHdrAddition;
   UINT32			wlanHdrAddition;
   UINT8			tag[4];
   UINT16           multi_stat;
   UINT8            BSSID[6];  	
	 UINT8 up;

   dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
   pdot11Header = (dot11_header_T*) dataBuf;

   wdrv_dbGetBssid(BSSID);

   wdrv_mgmt_ssidGetMultipleSsidStatus(&multi_stat);
   pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + (ULONG)WLAN_HDR_LEN);
   wlanHdrAddition = 0;

   if(multi_stat && !isEapol)
   {
	   etherHdrAddition = 4; /* Todo : Add define */
   }
   else
   {
	   etherHdrAddition = 0;
   }

	 hasQoSHeader = ((pdot11Header->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_QOS_DATA) || 
		 ((pdot11Header->fc & DOT11_FC_TYPESUBTYPE_MASK ) == DOT11_FC_QOS_NULL);
	 (char *)pWlanSnapHeader += hasQoSHeader*WLAN_QOS_HDR_LEN;

	 if (hasQoSHeader) {
			dot11_QoSControlHeader_t * qosHeader;
			
			qosHeader = (dot11_QoSControlHeader_t *)((ULONG)pdot11Header + WLAN_HDR_LEN);
			up = DOT11_QOS_UP(qosHeader);
	} else
		up = 0;

   /* See if the LLC header in the frame shows the SAP SNAP... */
   if ( ( SNAP_CHANNEL_ID == pWlanSnapHeader->DSAP ) &&
        ( SNAP_CHANNEL_ID == pWlanSnapHeader->SSAP ) &&
        ( LLC_CONTROL_UNNUMBERED_INFORMATION ==  pWlanSnapHeader->Control ) ) 
   {

      /* Check for the Bridge Tunnel OUI in the SNAP Header... */
      if((SNAP_OUI_802_1H_BYTE0 == pWlanSnapHeader->OUI[ 0 ]) &&
         (SNAP_OUI_802_1H_BYTE1 == pWlanSnapHeader->OUI[ 1 ]) &&
         (SNAP_OUI_802_1H_BYTE2 == pWlanSnapHeader->OUI[ 2 ])) {
         /* Strip the SNAP header by skipping over it.                  */
         /* Start moving data from the Ethertype field in the SNAP      */
         /* header.  Move to the TypeLength field in the 802.3 header.  */
         createEtherIIHeader = TRUE;        
         }

      /* Check for the RFC 1042 OUI in the SNAP Header   */
      else {
           /* See if the Ethertype is in our selective translation table  */
           /* (Appletalk AARP and DIX II IPX are the two protocols in     */
           /* our 'table')                                                */
            if (( ETHERTYPE_APPLE_AARP == htons(pWlanSnapHeader->Type) ) ||
                ( ETHERTYPE_DIX_II_IPX == htons(pWlanSnapHeader->Type) ))
			{
               /* Strip the SNAP header by skipping over it. */
                   createEtherIIHeader = FALSE;
            }
            /* All the rest SNAP types are transformed to EthernetII format */
            else 
			{
               createEtherIIHeader = TRUE;
            }
         }
      }

   else 
   { 
         /* Non-SNAP packets strip out the WLAN header and create the dst,src,len header. */
      createEtherIIHeader = FALSE;
   }
      
   

   if(etherHdrAddition == 0)
   {
       UINT8 *pDest, *pSrc;
       /* Find DA in the packet */
       if(pdot11Header->fc & DOT11_FC_FROM_DS) {
           pDest = pdot11Header->address1.addr;
           pSrc = pdot11Header->address3.addr;
       }
       else {
           pDest = pdot11Header->address3.addr;
           pSrc = pdot11Header->address2.addr;
       }

#ifdef BSS_PERF_DBG
   MACDUP2( &EthHeader.DstAddr, pDest);
   MACDUP2( &EthHeader.SrcAddr, pSrc);
#else
   CopyMacAddress(&EthHeader.DstAddr, pDest);
   CopyMacAddress(&EthHeader.SrcAddr, pSrc);
#endif
   }
   
   if( etherHdrAddition == 4)  /* Todo : Add define */
   {
#ifdef BSS_PERF_DBG
           MACDUP2( &ExtEthHeader.DstAddr, &pdot11Header->address3);
           MACDUP2( &ExtEthHeader.SrcAddr, &pdot11Header->address2);
#else
	   CopyMacAddress(&ExtEthHeader.DstAddr, &pdot11Header->address3);
	   CopyMacAddress(&ExtEthHeader.SrcAddr, &pdot11Header->address2);
#endif
   }
  

   if( createEtherIIHeader == TRUE ){
      /* The LEN/TYPE bytes are set to TYPE, the entire WLAN+SNAP is removed.*/
      lengthDelta = WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN + hasQoSHeader*WLAN_QOS_HDR_LEN - ETHERNET_HDR_LEN - etherHdrAddition;
      /* To do if  etherHdrAddition*/
      EthHeader.TypeLength = pWlanSnapHeader->Type;
      ExtEthHeader.TypeLength = pWlanSnapHeader->Type;
   }
   else {
      /* The LEN/TYPE bytes are set to frame LEN, only the WLAN header is removed, */
      /* the entire 802.3 or 802.2 header is not removed.*/
      lengthDelta = WLAN_HDR_LEN + hasQoSHeader*WLAN_QOS_HDR_LEN - ETHERNET_HDR_LEN - etherHdrAddition;
      /* To do if  etherHdrAddition*/
      EthHeader.TypeLength = htons(pMsdu->dataLen - WLAN_HDR_LEN - hasQoSHeader*WLAN_QOS_HDR_LEN);
      ExtEthHeader.TypeLength = htons(pMsdu->dataLen - WLAN_HDR_LEN - hasQoSHeader*WLAN_QOS_HDR_LEN);

}


   /* Put 802.1q tag */
   if(etherHdrAddition == 4) /* Todo : Add define */
   {
	   UINT16 vlan_id;
	   if(wdrv_mgmt_ssidGetStaVlanTag(ExtEthHeader.SrcAddr.addr,&vlan_id) != OK)
	   {
		   printf("bss setting vlan id to 0 \n");
		   vlan_id = 0;
	   }

	   /* insert priority into the 802.1D Tag field of Ethernet header */
	   tag[0] = 0x81;
	   tag[1] = 0x00;
		 /*	tag[2] = (vlan_id << 8) & 0xf;*/ /*(priority <<5) & 0xE0;*/
		 tag[2] = (vlan_id >> 8) & 0x0F;
		 tag[2] |= (up & 7) << 5;

	   tag[3] = vlan_id & 0xff;

	   PAL_memCopy(&ExtEthHeader.Tag, tag, sizeof(ExtEthHeader.Tag));
   }

   /* Replace the 802.11 header and the LLC with Ethernet packet. */
   dataBuf += lengthDelta;
   if(etherHdrAddition == 0)
   PAL_memCopy( dataBuf, (void*)&EthHeader, ETHERNET_HDR_LEN );
   else 
   {
	   PAL_memCopy( dataBuf, (void*)&ExtEthHeader, ETHERNET_HDR_LEN  + 4);/* Todo : Add define */
   }


   pMsdu->firstBDPtr->dataOffset = lengthDelta;
   pMsdu->dataLen -= lengthDelta;
   pMsdu->firstBDPtr->length -= lengthDelta;


   if (REPORT_LEVEL(RL_BRIDGE)) 
   {
   		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "w->e :: msdu->len=%d :: msdu->firstBD->len=%d\n",
   			pMsdu->dataLen, pMsdu->firstBDPtr->length );
      }
   	 
   tiwlan_ddaReceive(pMsdu);
}


/****************************************************************************
 *                      wdrv_bssBridgePktFromEthrToWlan()                   *
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_bssBridgePktFromEthrToWlan(mem_MSDU_T *pMsdu) 
{
   EthernetHeader_T   *pEthHeader;
   dot11_header_T     *pdot11Header;
   Wlan_LlcHeader_T  *pWlanSnapHeader;  
   UINT8 BSSID[6];
   UINT16 staIndex;
   ULONG txFlags = WDRV_TX_LOW_PRIORITY | WDRV_TX_FROM_ETHER;
   UINT16 lenType = 0;
   UINT8 createSNAPHeader;
   UINT8 bytesToskeep;
   mem_BD_T	   *BDptr;
   BOOL			WlanState, hasQoSHeader = 0;
   UINT8 wone_index, totalWones;
   mem_MSDU_T *pCopyMsdu = NULL;
   UINT8 *pEthHeaderTmp;
   int profile_index = 0;
   int lenTypeOffset = 0;
   UINT16 multiSssidEnabled;
   UINT32       isZeroLengthPkt = FALSE;
   drvDbHashStaType_e staType;

   
   wdrv_dbGetWlanServiceState( &WlanState );


   /* The data packet is placed in the second buffer in order to keep place for 
      for the WLAN + SNAP header.
   */
   pEthHeader = (EthernetHeader_T*)memMgr_BufData(pMsdu->firstBDPtr->nextBDPtr);
   pEthHeaderTmp = (UINT8 *) memMgr_BufData(pMsdu->firstBDPtr->nextBDPtr);
  

 
  /* check if this is 802.1d tagged frame and use different Ethernet header for it*/
   lenType = ((pEthHeaderTmp[2*MAC_ADDR_LEN])<<8) + pEthHeaderTmp[2*MAC_ADDR_LEN+1];

	 wdrv_mgmt_ssidGetMultipleSsidStatus(&multiSssidEnabled);
   if (multiSssidEnabled)  {
		 if (lenType == ETHERTYPE_802_1D) {
			 /* read priority from the tag*/
			 lenTypeOffset = 4;
			 if(wdrv_mgmt_ssidHandleRecvTagPkt((char*)pEthHeader ,&profile_index) != OK) {
				 ++bssBridgeTxWrongVLANDRopCount;
				 if (REPORT_LEVEL(RL_BRIDGE)) 
					 wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "Dropped packet with unknown VLAN tag on RX when Multiple SSID is enabled! \n");
				 wlan_memMngrFreeMSDU(pMsdu->handle);
				 return OK;
			 }
			 /* read real typeLength that comes after 4 bytes 802.1d tag */
			 lenType = ((pEthHeaderTmp[2*MAC_ADDR_LEN + lenTypeOffset]) << 8) + (pEthHeaderTmp[2*MAC_ADDR_LEN + lenTypeOffset + 1]);
		 } else {
			 ++bssBridgeTxmSSIDnoVLANDropCount;
			 if (REPORT_LEVEL(RL_BRIDGE)) 
				 wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "Dropped packet without VLAN tag on TX when Multiple SSID is enabled! \n");
     
			 wlan_memMngrFreeMSDU(pMsdu->handle);
		   return OK;
		 }
   }  else  {
	   lenTypeOffset = 0;
   }
  
   /* The data packet is placed in the second buffer in order to keep place for 
      for the WLAN + SNAP header.
   */
   BDptr=pMsdu->firstBDPtr;
   while(BDptr->nextBDPtr)
		 BDptr = BDptr->nextBDPtr;
   pMsdu->lastBDPtr = BDptr;

	 if ( !WlanState )	
         {
		 wlan_memMngrFreeMSDU(pMsdu->handle);
		 return OK;
	 }


   if( (*(char*)pEthHeader) & 0x01 )
   {
   		if ( (wdrv_mgmtIsBroadcastAuthorized() == FALSE) || (wdrv_mgmtIsAnyStationAuthorized() == FALSE) )
		{
			 wlan_memMngrFreeMSDU(pMsdu->handle);
			 return OK;
		}


		txFlags |= WDRV_TX_BRCST_MTCST_MSDU;
		bssBrgStat.pktsBrcstFromEthr++;
   }
   
   else
   {
      txFlags |= WDRV_TX_UNICAST_MSDU;
      if( wdrv_dbFindStaIndex((char*)&pEthHeader->DstAddr, &staIndex) != OK )
	  {
		  wlan_memMngrFreeMSDU(pMsdu->handle);
                  bssBrgStat.pktsFromEthrToWlanErrStaNotFound++ ;
                  return OK;
	  }

      wdrv_dbGetStationType(staIndex, &staType);
      /* Check STA type - regular STA or WDS STA */
      if(staType != DRV_DB_HASH_STA_TYPE_STATION)
      {
          if (REPORT_LEVEL(RL_BRIDGE)) 
              wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "Dropped packet that the Dest addr is a WDS STA\n");
          
          wlan_memMngrFreeMSDU(pMsdu->handle);
         return OK;
      }
      
	  if (wdrv_mgmtIsStationAuthorized(staIndex) == FALSE)
	  {
         wlan_memMngrFreeMSDU(pMsdu->handle);
         return OK;
      }
      bssBrgStat.pktsUnicEthrToWlan++;
   }


   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 1: Create the WLAN Header in the head of the first BD. */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* The WLAN header will be created in the first BS */
   pdot11Header = (dot11_header_T*)memMgr_BufData(pMsdu->firstBDPtr);
   
   /* The first BD contains the WLAN and SNAP headers only. */
   pMsdu->firstBDPtr->length = WLAN_HDR_LEN;
   pMsdu->firstBDPtr->dataOffset = 0;
   pMsdu->dataLen += WLAN_HDR_LEN;

   pMsdu->headerLen = WLAN_HDR_LEN;
   pdot11Header->fc = DOT11_FC_FROM_DS;
   wdrv_dbGetBssid(BSSID);

#ifdef BSS_PERF_DBG
   MACDUP2((void*)&pdot11Header->address1, (void*)&pEthHeader->DstAddr);
   MACDUP2((void*)&pdot11Header->address2, (void*)BSSID);
   MACDUP2((void*)&pdot11Header->address3, (void*)&pEthHeader->SrcAddr);
#else
   CopyMacAddress((void*)&pdot11Header->address1,(void*)&pEthHeader->DstAddr);
   CopyMacAddress((void*)&pdot11Header->address2, (void*)BSSID);
   CopyMacAddress((void*)&pdot11Header->address3,(void*)&pEthHeader->SrcAddr);
#endif

   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 2: Detect the packet type and decide if to create a    */
   /*          new SNAP or leave the original LLC.                 */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   if ( lenType	 > 1500 ){
      /*----------------------------------------------------------------*/
      /*                    Ethernet Packet                             */
      /*----------------------------------------------------------------*/
      /* Need to cut the Ethernet header off and create SNAP in the     */
      /* first BD. (just remember the type field.)                      */
      /*----------------------------------------------------------------*/
      bytesToskeep = ETHERNET_HDR_LEN;
      /* Move the Ethertype into the SNAP header */
      createSNAPHeader = TRUE;

   }
    else if (htons(pEthHeader->TypeLength) == 0) { /* Check is ethernet length 0 prior to any others checks */
        bytesToskeep = ETHERNET_HDR_LEN;
        createSNAPHeader = FALSE;
        isZeroLengthPkt = TRUE;
    }
   else if(
		   ( *(unsigned char*)((ULONG)pEthHeader+14 + lenTypeOffset) == 0xFF )
			&&
		   ( *(unsigned char*)((ULONG)pEthHeader+15+lenTypeOffset) == 0xFF )
		  ) 
   {

      /*----------------------------------------------------------------*/
      /*                    RAW 802.3  Packet                           */
      /*----------------------------------------------------------------*/
      /* Just jump over this header.                                    */
      /*----------------------------------------------------------------*/
      bytesToskeep = IEEE802_3_HDR_LEN;
      createSNAPHeader = FALSE; 
   }
   else{
      /*----------------------------------------------------------------*/
      /*                   802.2 Packet (RAW or SNAP)                    */
      /*----------------------------------------------------------------*/
      /* Cut off the first 14 bytes, the LLC will be glued to the WLAN  */
      /* header.                                                        */
      /*----------------------------------------------------------------*/
      bytesToskeep = ETHERNET_HDR_LEN;
      createSNAPHeader = FALSE;
   }
 
	 /* Demiurg: WME */
    if (txFlags & WDRV_TX_UNICAST_MSDU)
    {
        if (wmeACParams.enabled && wdrv_dbGetStaWME(staIndex))
        {
			 dot11_QoSControlHeader_t * qosHeader;
			 wdrv_QoSClassifierAC_t wmeAC;
			 UINT8 up;
				 
			 txFlags &= ~WDRV_TX_LOW_PRIORITY;
			 wmeAC = wdrv_QoSClassifierClassify( (UINT8 *)pEthHeaderTmp, (UINT8 *)(pEthHeader) + bytesToskeep + lenTypeOffset, &up);
			 txFlags |= wdrv_QoSAuxMapACtoTxFlags[wmeAC];

			 qosHeader = (dot11_QoSControlHeader_t *)((ULONG)pdot11Header + WLAN_HDR_LEN);
			 PAL_memSet(qosHeader, 0, WLAN_QOS_HDR_LEN);
			 DOT11_QOS_ACK_SET(qosHeader, wmeACParams.ap_edcfParams[wmeAC].ackPolicy);
			 DOT11_QOS_UP_SET(qosHeader, up); 
			 
			 pMsdu->firstBDPtr->length += WLAN_QOS_HDR_LEN;
			 pMsdu->dataLen += WLAN_QOS_HDR_LEN;

			 pdot11Header->fc |= DOT11_FC_QOS_DATA;

			 hasQoSHeader = 1;
		 } else
			 pdot11Header->fc |= DOT11_FC_DATA;
	 } else 
		 pdot11Header->fc |= DOT11_FC_DATA;


   if(isZeroLengthPkt) {
       /* Free second BD */
       pMsdu->dataLen -= bytesToskeep;
       wlan_memMngrFreeBD(pMsdu->firstBDPtr->nextBDPtr->handle);
       pMsdu->lastBDPtr = pMsdu->firstBDPtr;
       pMsdu->firstBDPtr->nextBDPtr = NULL;
   }
   else {
       /* In the second BD we skeep header bytes accourding to the frame type */
       pMsdu->dataLen -= bytesToskeep +lenTypeOffset;
       pMsdu->firstBDPtr->nextBDPtr->length -= bytesToskeep +lenTypeOffset;
       pMsdu->firstBDPtr->nextBDPtr->dataOffset = bytesToskeep +lenTypeOffset;
   }


   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 3: If we need to create SNAP header, it is created in  */
   /*          the first BD, after the WLAN header.                */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/   
   if ( createSNAPHeader == TRUE ){
      /*--------------------------------------------------------*/
      /* Create the SNAP Header                                 */
      /*--------------------------------------------------------*/

      pMsdu->firstBDPtr->length += WLAN_SNAP_HDR_LEN;
      pMsdu->dataLen += WLAN_SNAP_HDR_LEN;
      pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)pdot11Header + WLAN_HDR_LEN + hasQoSHeader*WLAN_QOS_HDR_LEN);
      pWlanSnapHeader->DSAP = SNAP_CHANNEL_ID;
      pWlanSnapHeader->SSAP = SNAP_CHANNEL_ID;
      pWlanSnapHeader->Control = LLC_CONTROL_UNNUMBERED_INFORMATION;
      

      /* Check to see if the Ethertype matches anything in the translation     */
      /* table (Appletalk AARP or DixII/IPX).  If so, add the 802.1h           */
      /* SNAP.                                                                 */
      if ( ( ETHERTYPE_APPLE_AARP == lenType ) ||
           ( ETHERTYPE_DIX_II_IPX == lenType )    ) {
        
        /* Fill out the SNAP Header with 802.1H extention   */
#ifdef BSS_PERF_DBG
         UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_802_1H, 0);
         UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_802_1H, 1);
         UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_802_1H, 2);
#else
        PAL_memCopy( &pWlanSnapHeader->OUI, SNAP_OUI_802_1H, 
                        sizeof( pWlanSnapHeader->OUI ) );
#endif
      }
      else {
#ifdef BSS_PERF_DBG
         UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_RFC1042, 0);
         UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_RFC1042, 1);
         UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_RFC1042, 2);
#else
         /* otherwise, add the RFC1042 SNAP   */
         PAL_memCopy( &pWlanSnapHeader->OUI, SNAP_OUI_RFC1042, 
                    sizeof( pWlanSnapHeader->OUI) );
#endif
      }

      pWlanSnapHeader->Type = htons(lenType);
   }

   
   if (txFlags & WDRV_TX_BRCST_MTCST_MSDU)
   {
      wdrv_dbGetNumSupWone(&totalWones);
      if(totalWones == WONE_DUAL_BAND)
      {
         /* duplicate broadcast packet for second wone */

         if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(pMsdu)) == OK )
         {
            wlan_memMngrCopyMsdu( pCopyMsdu, pMsdu);
            wdrv_txSendMsdu( pCopyMsdu, profile_index, txFlags,SECOND_WONE_NET);
         }
         else
         {
             if (REPORT_LEVEL(RL_MEM))
                 wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"fail to wlan_memMngrAllocMSDU");
         }
      }
      wone_index = FIRST_WONE_NET;
   } 
   else
   {
      wdrv_dbGetStationWoneIndex(staIndex, &wone_index);

   }

   if (txFlags & WDRV_TX_BRCST_MTCST_MSDU)
   {
	   wdrv_txSendMsdu( pMsdu, profile_index, txFlags,FIRST_WONE_NET);
   }
   else
   wdrv_txSendMsdu( pMsdu, staIndex, txFlags ,wone_index);
   return OK;
}


/****************************************************************************
 *                      wdrv_bssBridgeShowStatistics()                      *
 ****************************************************************************
 * DESCRIPTION:
 * 
 * INPUTS:
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int wdrv_bssBridgeShowStatistics( char *buf ){
   int len = 0;
	 UINT16 multiSssidEnabled;

	 wdrv_mgmt_ssidGetMultipleSsidStatus(&multiSssidEnabled);

   len += sprintf(buf+len,"------------------------- Bss Bridge Statistics ---------------------\n");
   len += sprintf(buf+len,"Unicast:  Wlan->Wlan     : %8d\n", bssBrgStat.pktsUnicWlanToWlan );
   len += sprintf(buf+len,"          Wlan->Ethernet : %8d\n", bssBrgStat.pktsUnicWlanToEthr );         
   len += sprintf(buf+len,"          Ethrnet->Wlan  : %8d\n", bssBrgStat.pktsUnicEthrToWlan );
   len += sprintf(buf+len,"\n");
   len += sprintf(buf+len,"Broacast: From Wlan      : %8d\n", bssBrgStat.pktsBrcstFromWlan  );
   len += sprintf(buf+len,"          From Ethernet  : %8d\n", bssBrgStat.pktsBrcstFromEthr  );
   len += sprintf(buf+len,"\n");
   len += sprintf(buf+len,"Error:    Unouwn Station : %8d\n", bssBrgStat.pktsFromEthrToWlanErrStaNotFound );
   len += sprintf(buf+len,"BSS Isolation dropped    : %8d\n", bssBrgStat.pktsDroppedDueToIsolation);    
	 if (multiSssidEnabled) {
		 len += sprintf(buf+len,"-----------------------Multiple SSID drop counts---------------------\n");
		 len += sprintf(buf+len,"TX: wrong VLAN id        : %8d\n", bssBridgeTxWrongVLANDRopCount);
		 len += sprintf(buf+len,"TX: no VLAN tag          : %8d\n", bssBridgeTxmSSIDnoVLANDropCount);
		 len += sprintf(buf+len,"RX: VLAN tag present     : %8d\n", bssBridgeRxVLANDropCount);
	 }
   len += sprintf(buf+len,"---------------------------------------------------------------------\n");                                              
	
   return len;
}

void wdrv_bssBridgeGetLanSendUnicast(UINT32 *LanSendUnicast)
{
	*LanSendUnicast = bssBrgStat.pktsUnicWlanToEthr;
}

void wdrv_bssBridgeGetLanSendMulticast(UINT32 *LanSendMulticast)
{
	*LanSendMulticast = bssBrgStat.pktsBrcstFromWlan;
}

void wdrv_bssBridgeGetLanRecvUnicast(UINT32 *LanRecvUnicast)
{
	*LanRecvUnicast = bssBrgStat.pktsUnicEthrToWlan;
}

void wdrv_bssBridgeGetLanRecvMulticast(UINT32 *LanRecvMulticast)
{
	*LanRecvMulticast = bssBrgStat.pktsBrcstFromEthr;
}

/* WPA support section */
BOOL isEapol(mem_MSDU_T *pMsdu, BOOL packetHasQoSHeader, UINT32 apToApPkt, UINT16 * etherType)
{
	Wlan_LlcHeader_T	*pWlanSnapHeader;   
	UINT8 				*dataBuf;
    UINT32              headerLen;

    headerLen = (apToApPkt) ? sizeof(dot11_header_T) : WLAN_HDR_LEN;

	dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
	pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + headerLen + (ULONG)packetHasQoSHeader*WLAN_QOS_HDR_LEN);

	if (etherType)
		*etherType = htons(pWlanSnapHeader->Type);

	/* See if the frame is EAPOL... */
	if (( SNAP_CHANNEL_ID == pWlanSnapHeader->DSAP ) &&
		( SNAP_CHANNEL_ID == pWlanSnapHeader->SSAP ) &&
		( LLC_CONTROL_UNNUMBERED_INFORMATION ==  pWlanSnapHeader->Control ) &&
		(SNAP_OUI_RFC1042_BYTE0 == pWlanSnapHeader->OUI[ 0 ]) &&
		(SNAP_OUI_RFC1042_BYTE1 == pWlanSnapHeader->OUI[ 1 ]) &&
		(SNAP_OUI_RFC1042_BYTE2 == pWlanSnapHeader->OUI[ 2 ]) &&             
		(EAPOL_ETHER_TYPE == htons(pWlanSnapHeader->Type)))
		return TRUE;

	return FALSE;
}


UINT32 wdrv_bssbridgeGetExcludedCount(void)
{
	return bssBridgeExcludedCount;		
}

UINT32 wdrv_bssbridgeGetTKIPMicFailure(void)
{
	return bssBridgeTkipMicFailureCount;		
}


/* send msdu to bridge*/
STATUS wdrv_bssBridgeMsduToBridge(mem_MSDU_T *pMsdu)
{
	return tiwlan_ddaReceive( pMsdu );
}

