/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		mgmtParser.c											*/
/*		PURPOSE:	Management parser object								*/
/*																			*/
/****************************************************************************/
#include "802_11Defs.h"
#include "utils.h"
#include "mgmtSwap.h"
#include "wlan_pal.h"


/* The following structures are used in the swap */
typedef struct {
	UINT16			algoNumber;
	UINT16			transSeqNumber;
	UINT16			status;
} authSwap_T;

typedef struct {
	UINT16			reason;
} deAuthSwap_T;

typedef struct {
	UINT16				capabilities;
	UINT16				listenInterval;
} assocReqSwap_T;

typedef struct {
	UINT16				capabilities;
	UINT16				status;
	UINT16				AID;
} assocRspSwap_T, reAssocRspSwap_T;

typedef struct {
	UINT16				capabilities;
	UINT16				listenInterval;
} reAssocReqSwap_T;

typedef struct {
	UINT16			reason;
} disAssocSwap_T;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
void authSwap(authSwap_T *body);
void deAuthSwap(deAuthSwap_T *body);
void assocReqSwap(assocReqSwap_T *body);
void assocRspSwap(assocRspSwap_T *body);
void reAssocReqSwap(reAssocReqSwap_T *body);
void reAssocRspSwap(reAssocRspSwap_T *body);
void DisAssocSwap(disAssocSwap_T *body);

/*************************************************************************
 *                        wdrv_mgmtSwap                                  *
 *************************************************************************
DESCRIPTION: This function swaps a mgmt frame.
			 Called by the mgmt parser upon receiving a frame.
			 Called by the mgmt builder before sending a frame.
                                                                                                   
INPUT:      frame pointer

OUTPUT:     None. 

RETURN:     None. 

************************************************************************/
void wdrv_mgmtSwap(dot11_mgmtFrame_T *mmpduPtr)
{
	/* call the corresopnding swap function according to the frame subtype */
	switch (mmpduPtr->hdr.fc  & DOT11_FC_SUB_MASK)
	{
		case DOT11_FC_AUTH:
			authSwap((authSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_DEAUTH:
			deAuthSwap((deAuthSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_ASSOC_REQ:
			assocReqSwap((assocReqSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_ASSOC_RESP:
			assocRspSwap((assocRspSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_REASSOC_REQ:
			reAssocReqSwap((reAssocReqSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_REASSOC_RESP:
			reAssocRspSwap((reAssocRspSwap_T *)&mmpduPtr->body);
			break;

		case DOT11_FC_DISASSOC:
			DisAssocSwap((disAssocSwap_T *)&mmpduPtr->body);
			break;

		default:
			break;
	}

	return;	
}

/******************************************************************************
*                               authSwap          	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in authentication message.                    
                                                                         
INPUTS:			body - pointer to authentication message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void authSwap(authSwap_T *body)
{
	body->algoNumber = ENDIAN_HANDLE_WORD(body->algoNumber);
	body->transSeqNumber = ENDIAN_HANDLE_WORD(body->transSeqNumber);
	body->status = ENDIAN_HANDLE_WORD(body->status);
}

/******************************************************************************
*                               deAuthSwap        	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in Deauthentication message.                    
                                                                         
INPUTS:			body - pointer to Deauthentication message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void deAuthSwap(deAuthSwap_T *body)
{
	body->reason = ENDIAN_HANDLE_WORD(body->reason);
}

/******************************************************************************
*                               assocReqSwap         	                      *
*******************************************************************************
DESCRIPTION:	Swap fields in association request message.                    
                                                                         
INPUTS:			body - pointer to association request message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void assocReqSwap(assocReqSwap_T *body)
{
	body->capabilities = ENDIAN_HANDLE_WORD(body->capabilities);
	body->listenInterval = ENDIAN_HANDLE_WORD(body->listenInterval);
}

/******************************************************************************
*                            assocRspSwap         	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in association response message.                    
                                                                         
INPUTS:			body - pointer to association response message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void assocRspSwap(assocRspSwap_T *body)
{
	body->capabilities = ENDIAN_HANDLE_WORD(body->capabilities);
	body->status = ENDIAN_HANDLE_WORD(body->status);
	body->AID = ENDIAN_HANDLE_WORD(body->AID);
}

/******************************************************************************
*                            reAssocReqSwap      	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in Reassociation request message.                    
                                                                         
INPUTS:			body - pointer to Reassociation request message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void reAssocReqSwap(reAssocReqSwap_T *body)
{
	body->capabilities = ENDIAN_HANDLE_WORD(body->capabilities);
	body->listenInterval = ENDIAN_HANDLE_WORD(body->listenInterval);
}

/******************************************************************************
*                         reAssocRspSwap        	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in Reassociation response message.                    
                                                                         
INPUTS:			body - pointer to Reassociation response message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void reAssocRspSwap(reAssocRspSwap_T *body)
{
	body->capabilities = ENDIAN_HANDLE_WORD(body->capabilities);
	body->status = ENDIAN_HANDLE_WORD(body->status);
	body->AID = ENDIAN_HANDLE_WORD(body->AID);
}

/******************************************************************************
*                             DisAssocSwap       	                    	  *
*******************************************************************************
DESCRIPTION:	Swap fields in Disassociation request message.                    
                                                                         
INPUTS:			body - pointer to Disassociation request message.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void DisAssocSwap(disAssocSwap_T *body)
{
	body->reason = ENDIAN_HANDLE_WORD(body->reason);
}
