/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	tx.h													   */
/*    PURPOSE:	Tx module					 							   */
/*																		   */
/***************************************************************************/
#ifndef _TX_INIT_H_
#define _TX_INIT_H_

#include "pform_types.h"
#include "envList.h"
#include "wlan_pal.h"


#define TOTAL_NUM_OF_DATA_ENVELOPS      		192
#define TOTAL_NUM_OF_MNGT_ENVELOPS      		192
#define TOTAL_NUM_OF_WONE_WHIT_DATA_ENVELOPS    64 /*120*/

/* Tx Flags */
#define   WDRV_TX_HIGH_PRIORITY      0x01
#define   WDRV_TX_LOW_PRIORITY       0x02
// WME TODO : remove high and low flags
#define   WDRV_TX_UNICAST_MSDU       0x04
#define   WDRV_TX_BRCST_MTCST_MSDU   0x08
#define   WDRV_TX_FROM_ETHER		 0x20
#define   WDRV_TX_FORCE_UN_ENCRYPTED 0x40
#define   WDRV_TX_PS_NOT_ALLOWED	 0x80
#define   WDRV_TX_EAPOL_FRAME		 0x100

#define   WDRV_TX_AC_MASK 0xE00 /* 11 1000000000 */

#define   WDRV_TX_AC_BE 0x200 /* 00 1000000000 */
#define   WDRV_TX_AC_BK 0x600 /* 01 1000000000 */
#define   WDRV_TX_AC_VI 0xA00 /* 10 1000000000 */
#define   WDRV_TX_AC_VO 0xE00 /* 11 1000000000 */

/* AP to AP TX flag used to mark WDS frames */
#define   WDRV_TX_AP_TO_AP  0x1000
/* We need to differentiate between management and AC VO frames */
#define   WDRV_TX_MGMT  0x2000
/* We need to differentiate between probe Response and other management */
#define   WDRV_TX_PROBE_RESPONSE  0x4000


typedef enum {
   WDRV_TX_MSDU_TIMEOUT,
   WDRV_TX_MSDU_SEND,
   WDRV_TX_COMPLETE,
   WDRV_TX_MSDU_PS_POLL,
   WDRV_TX_PS_AGING,
   WDRV_TX_MSDU_PS_STATUS_CHANGE,
   WDRV_TX_MSDU_PS_CLEAN_STATION_LIST,
} TxMsg_T;


typedef struct {
   TxMsg_T type;  /* Type of message (from the list above) */
   union {
      long MessageInfo[3]; /* queue-message size */

      /* Ps message types. 802.11(Ps-poll, Ps-wake) */
      struct {
         long  staIndex;
         long  status /* reserve1 */;
         long  woneIndex;
      } psMsg;
   } msg;
} TxMessage_T;


/*******************************************************************/
/*                    Testing routines                             */ 
/*******************************************************************/
void wdrv_txSendMsduTest( void );


/*******************************************************************/


/*************************************************************************
 *                        wdrv_txInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Tx module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_txInit(void);

/*************************************************************************
 *                        wdrv_txUnload                                  *
 *************************************************************************
DESCRIPTION: This function deinitializes the Tx module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:     

************************************************************************/
void wdrv_txUnload(void);

/*************************************************************************
 *                        wdrv_txSendMsdu                                *
 *************************************************************************
DESCRIPTION: This prepares the msdu to be sent. It takes an empty envelop              
             from the free list, put the msdu in to it, and inserts the 
             envelop to the waiting list.
INPUT:       msduPtr : Pointer to the MSDU to send
			 stationIndex : Index of the STA the msdu is sent to.
             txFlags : TX flags indicating priority, unicast/broadcast, AC.
             woneIndex : wone index of the STA the msdu is sent to

OUTPUT:      

RETURN:      OK : MSDU has been inserted to the waiting list.
             NOK: Couldn't insert the MSDU to the waiting list.                                                                                                                                             
************************************************************************/
STATUS wdrv_txSendMsdu( mem_MSDU_T *msduPtr, UINT16 stationIndex, 
                       ULONG txFlags, UINT8 woneIndex);

/*************************************************************************
 *                        wdrv_txSendPsPollMsg                           *
 *************************************************************************
DESCRIPTION: This function sends ps-poll message to the DDA context engine.

INPUT :      staIndex : The index of the station the Ps-Poll message
						received from.
OUTPUT:      

RETURN:                                                                                  
************************************************************************/
void wdrv_txSendPsPollMsg( UINT32 staIndex );


/*************************************************************************
 *                        wdrv_txCheckWaitPckts                          *
 *************************************************************************
DESCRIPTION: This function calls HAL's function to clean up HAL's queues,
			 verifies if there waiting packets in the waiting list and
             sends a TIMEOUT message to the DDA context engine.

OUTPUT:      

RETURN:
************************************************************************/
void wdrv_txCheckWaitPckts (void );

/*************************************************************************
 *                        wdrv_txCheckWaitPcktsFromIntr                  *
 *************************************************************************
DESCRIPTION: This function sends TX complete message to the DDA context
			 engine
OUTPUT:      

RETURN:
************************************************************************/
void wdrv_txCheckWaitPcktsFromIntr (void);

/*************************************************************************
 *                        wdrv_txFreeEnv                                 *
 *************************************************************************
DESCRIPTION: This function puts the envelope into corresponding free list.

INPUT :      envelopPtr : The envelop to be released.
OUTPUT:      

RETURN:                                                                                                                                            
************************************************************************/
void wdrv_txFreeEnv( TxEnvelop_T **envelopPtr );

/*************************************************************************
 *                        wdrv_txEnable                                  *
 *************************************************************************
DESCRIPTION: This function calls HAL to enable TX complete interrupts and
			 passes Core callback function.
             
INPUT :      
OUTPUT:      

RETURN:      OK:
			 NOK: 
************************************************************************/ 
STATUS wdrv_txEnable(void);

/*************************************************************************
 *                        wdrv_txSendPsChangeStateMsg                    *
 *************************************************************************
DESCRIPTION: This function sends ps-Status Change message to the DDA
			 context engine.

INPUT :      staIndex : The index of the station that is the subject of
						changing the power save state.
			 woneIndex: STA's wone index.
OUTPUT:      

RETURN:                                                                                                                   
************************************************************************/
void wdrv_txSendPsChangeStateMsg( UINT32 staIndex ,UINT16 status, UINT8 woneIndex);

/****************************************************************************
 *                      wdrv_txSendPsCleanStaListMsg()                       *
 ****************************************************************************
  DESCRIPTION: This function sends ps-clean stations list message to the 
	 		   DDA context engine.
 
  INPUT :      staIndex : The index of the station that is the subject of
						  changing the power save state.
			   woneIdx:   STA's wone index.

  OUTPUT:	None
  
  RETURNS:	None
****************************************************************************/
void  wdrv_txSendPsCleanStaListMsg(UINT16 stationIndex, UINT8 woneIdx) ;

/*************************************************************************
 *                        wdrv_txRateAdapSetRate                         *
 *************************************************************************
DESCRIPTION:  Sets a station rate parameters in the rate adaptation table.
      
INPUT :   stationIndex : The index of the STA, the rate is configured for.
		  rate		   : The configured rate in units of ACX rate bitmask.
OUTPUT:     

RETURN:  OK:  The setting succeeded.
		 NOK: The configured rate is invalid. 
************************************************************************/
STATUS wdrv_txRateAdapSetRate( UINT16 stationIndex, UINT16 rate);


/*************************************************************************
 *                        wdrv_txSetConcatStatus                         *
 *************************************************************************
DESCRIPTION:  Setting the concatenation mode/status.
      
INPUT :   status: ON/OFF.
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetConcatStatus( UINT8 status );

/*************************************************************************
 *                        wdrv_txSetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Setting the max concatenation size.
      
INPUT :   size : the value of max concatenation size.
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetMaxConcatSize( UINT32 size );


/*************************************************************************
 *                        wdrv_tx_getEvent		                         *
 *************************************************************************
DESCRIPTION:  This function is TX event handler .It is called by DDA
			  upon TX event reception.
      
INPUT :   pTxMessage : structure that describes the received event.
OUTPUT:     

RETURN:   OK  : the event is handled successfully
		  NOK : failed to handle the event      
************************************************************************/
STATUS wdrv_tx_getEvent(TxMessage_T *pTxMessage);


/*************************************************************************
 *                        wdrv_txGetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Getting the max concatenation size.
      
INPUT :   
OUTPUT:     

RETURN:  Max concatenation size.
************************************************************************/
int wdrv_txGetMaxConcatSize( void );

/*************************************************************************
 *                        wdrv_txGetTxMulticastCount                     *
 *************************************************************************
DESCRIPTION:  This function returns number of multicats frames that passed
			  through the TX module.
	  
INPUT :   
OUTPUT:   counter : number of multicast frames.  

RETURN:        
*************************************************************************/
void wdrv_txGetTxMulticastCount(UINT32 *counter);

/*************************************************************************
 *                        wdrv_txPrintStatistics                         *
 *************************************************************************
DESCRIPTION:  This function returns TX module statistics.
	  
INPUT :   buf : buffer to fill.
OUTPUT:   buf : filled buffer.  

RETURN:   Length of the data inside the buffer.     
*************************************************************************/
int  wdrv_txPrintStatistics(char *buf);

/*************************************************************************
 *                        wdrv_txPrintRateAdaptation                     *
 *************************************************************************
DESCRIPTION:  This function returns rate adaptation statistics.
	  
INPUT :   buf : buffer to fill.
OUTPUT:   buf : filled buffer.  

RETURN:   Length of the data inside the buffer.     
************************************************************************/
int  wdrv_txPrintRateAdaptation( char *buf);

/*************************************************************************
 *                        tx_getStaActiveRate	                         *
 *************************************************************************
DESCRIPTION:  This function returns STA's current active rate according to
			  the rate adaptation algorithm.
	  
INPUT :   staIndex : the index of the station, the activate rate is
					 required for.
OUTPUT:     

RETURN:   STA's current active rate.
************************************************************************/
UINT16 tx_getStaActiveRate(UINT16 staIndex);

#endif
