/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	wdsDrv.c                							       */
/*   PURPOSE:	WDS driver functions.	      			 			       */
/*																		   */
/***************************************************************************/
#define __WDS_DRV_C__


#include "pform_types.h"
#include <stdio.h>
#include "mainsm.h"
#include "802_11Defs.h"
#include "tx.h"
#include "wlan_pal.h"
#include "dda_wpa_interface.h"
#include "dda_contexts.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtIf.h"
#include "AcxCommands.h"
#include "whalAcxCmd.h"
#include "whalParams.h"
#include "Shutdown.h"
#include "bssbridge.h"
#include "dda.h"
#include "wds.h"
#include "wdsDrv.h"


static UINT8 SNAP_OUI_802_1H[] = { SNAP_OUI_802_1H_BYTE0, SNAP_OUI_802_1H_BYTE1, SNAP_OUI_802_1H_BYTE2};
static UINT8 SNAP_OUI_RFC1042[] = { SNAP_OUI_RFC1042_BYTE0, SNAP_OUI_RFC1042_BYTE1, SNAP_OUI_RFC1042_BYTE2};

static UINT32 g_wdsIsBssEnabled=TRUE; /* Enable BSS by default */
wdsDrvStat_s wdsDrvStat;

/*
 *Function Name:wdrv_wdsDrvPktFromEthrToWlan
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

STATUS wdrv_wdsDrvPktFromEthrToWlan(mem_MSDU_T *pMsdu, UINT8 *pLinkPartner) 
{
    EthernetHeader_T   *pEthHeader;
    dot11_header_T     *pdot11Header;
    Wlan_LlcHeader_T  *pWlanSnapHeader;  
    UINT8 BSSID[6];
    UINT16 staIndex;
    ULONG txFlags = WDRV_TX_LOW_PRIORITY | WDRV_TX_FROM_ETHER | WDRV_TX_AP_TO_AP;
    UINT16 lenType = 0;
    UINT8 createSNAPHeader;
    UINT8 bytesToskeep;
    mem_BD_T    *BDptr;
    BOOL         WlanState;
    UINT8 wone_index;
    UINT32       isZeroLengthPkt = FALSE;
    wdrv_dbGetWlanServiceState( &WlanState );
    wdrv_dbGetBssid(BSSID);



    /* The data packet is placed in the second buffer in order to keep place for 
       for the WLAN + SNAP header.
    */
    pEthHeader = (EthernetHeader_T*)memMgr_BufData(pMsdu->firstBDPtr->nextBDPtr);


    BDptr=pMsdu->firstBDPtr;
    while (BDptr->nextBDPtr)
        BDptr = BDptr->nextBDPtr;
    pMsdu->lastBDPtr = BDptr;

    if ( !WlanState ) {
        wlan_memMngrFreeMSDU(pMsdu->handle);
        return OK;
    }

    txFlags |= WDRV_TX_UNICAST_MSDU;
    if ( wdrv_dbFindStaIndex(pLinkPartner, &staIndex) != OK ) {
        wlan_memMngrFreeMSDU(pMsdu->handle); 
        wdsDrvStat.tx_staIndexNotFound++;
        return OK;
    }
    if (wdrv_mgmtIsStationAuthorized(staIndex) == FALSE) {
        wlan_memMngrFreeMSDU(pMsdu->handle);
        wdsDrvStat.tx_staIndexNotAuthorized++;
        return OK;
    }
    wdsDrvStat.tx_pkts++;

    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/
    /* Phase 1: Create the WLAN Header in the head of the first BD. */
    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/
    /* The WLAN header will be created in the first BS */
    pdot11Header = (dot11_header_T*)(memMgr_BufData(pMsdu->firstBDPtr));

    /* The first BD contains the WLAN and SNAP headers only. */
    pMsdu->firstBDPtr->length = sizeof(dot11_header_T);
    pMsdu->firstBDPtr->dataOffset = 0;
    pMsdu->dataLen += sizeof(dot11_header_T);

    pMsdu->headerLen = sizeof(dot11_header_T);
    pdot11Header->fc = (DOT11_FC_DATA | DOT11_FC_TO_DS | DOT11_FC_FROM_DS);

    MACDUP2((void*)&pdot11Header->address1, (void*)pLinkPartner);
    MACDUP2((void*)&pdot11Header->address2, (void*)BSSID);
    MACDUP2((void*)&pdot11Header->address3, (void*)&pEthHeader->DstAddr);
    MACDUP2((void*)&pdot11Header->address4, (void*)&pEthHeader->SrcAddr);

    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/
    /* Phase 2: Detect the packet type and decide if to create a    */
    /*          new SNAP or leave the original LLC.                 */
    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/
    if ( htons(pEthHeader->TypeLength) > 1500 ) {
        /*----------------------------------------------------------------*/
        /*                    Ethernet Packet                             */
        /*----------------------------------------------------------------*/
        /* Need to cut the Ethernet header off and create SNAP in the     */
        /* first BD. (just remember the type field.)                      */
        /*----------------------------------------------------------------*/
        bytesToskeep = ETHERNET_HDR_LEN;
        /* Move the Ethertype into the SNAP header */
        lenType = pEthHeader->TypeLength;
        createSNAPHeader = TRUE;
    }

    else if (htons(pEthHeader->TypeLength) == 0) { /* Check is ethernet length 0 prior to any others checks */
        bytesToskeep = ETHERNET_HDR_LEN;
        createSNAPHeader = FALSE;
        isZeroLengthPkt = TRUE;
    }
    else if (( *(unsigned char*)((ULONG)pEthHeader+14) == 0xFF )&&( *(unsigned char*)((ULONG)pEthHeader+15) == 0xFF)) {
        /*----------------------------------------------------------------*/
        /*                    RAW 802.3  Packet                           */
        /*----------------------------------------------------------------*/
        /* Just jump over this header.                                    */
        /*----------------------------------------------------------------*/
        bytesToskeep = IEEE802_3_HDR_LEN;
        createSNAPHeader = FALSE; 
    } else {
        /*----------------------------------------------------------------*/
        /*                   802.2 Packet (RAW or SNAP)                    */
        /*----------------------------------------------------------------*/
        /* Cut off the first 14 bytes, the LLC will be glued to the WLAN  */
        /* header.                                                        */
        /*----------------------------------------------------------------*/
        bytesToskeep = ETHERNET_HDR_LEN;
        createSNAPHeader = FALSE;
    }
    if(isZeroLengthPkt) {
        /* Free second BD */
        pMsdu->dataLen -= bytesToskeep;
        wlan_memMngrFreeBD(pMsdu->firstBDPtr->nextBDPtr->handle);
        pMsdu->lastBDPtr = pMsdu->firstBDPtr;
        pMsdu->firstBDPtr->nextBDPtr = NULL;
    }
    else {
        /* In the second BD we skeep header bytes accourding to the frame type */
        pMsdu->dataLen -= bytesToskeep;
        pMsdu->firstBDPtr->nextBDPtr->length -= bytesToskeep;
        pMsdu->firstBDPtr->nextBDPtr->dataOffset = bytesToskeep;
    }


    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/
    /* Phase 3: If we need to create SNAP header, it is created in  */
    /*          the first BD, after the WLAN header.                */
    /*--------------------------------------------------------------*/
    /*--------------------------------------------------------------*/   
    if ( createSNAPHeader == TRUE ) {
        /*--------------------------------------------------------*/
        /* Create the SNAP Header                                 */
        /*--------------------------------------------------------*/
        pWlanSnapHeader = (Wlan_LlcHeader_T*)((UINT32)(memMgr_BufData(pMsdu->firstBDPtr)) 
                                              + sizeof(dot11_header_T));
        pWlanSnapHeader->DSAP = SNAP_CHANNEL_ID;
        pWlanSnapHeader->SSAP = SNAP_CHANNEL_ID;
        pWlanSnapHeader->Control = LLC_CONTROL_UNNUMBERED_INFORMATION;

        /* Check to see if the Ethertype matches anything in the translation     */
        /* table (Appletalk AARP or DixII/IPX).  If so, add the 802.1h           */
        /* SNAP.                                                                 */
        if ( ( ETHERTYPE_APPLE_AARP == htons(lenType) ) ||
             ( ETHERTYPE_DIX_II_IPX == htons(lenType) )    ) {

            /* Fill out the SNAP Header with 802.1H extention   */
            UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_802_1H, 0);
            UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_802_1H, 1);
            UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_802_1H, 2);
        } else {
            UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_RFC1042, 0);
            UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_RFC1042, 1);
            UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_RFC1042, 2);
        }

        pWlanSnapHeader->Type = lenType;
        pMsdu->firstBDPtr->length += sizeof(Wlan_LlcHeader_T);
        pMsdu->dataLen += sizeof(Wlan_LlcHeader_T);

    }


    wdrv_dbGetStationWoneIndex(staIndex, &wone_index);

    if(wdrv_txSendMsdu( pMsdu, staIndex, txFlags ,wone_index) != OK) {
        wdsDrvStat.tx_fails++;
        return (NOK);
    }
    return (OK);
}

wdsDrvSwtchingEntry_s g_wdsSwitchIfcs[WDS_LINKS_NUMBER];

/*
 *Function Name:wdrv_wdsInterfaceCookieGet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void * wdrv_wdsInterfaceCookieGet(char *pMac)
{
    int i;
    unsigned int flags;
    void *pResult;

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);      /* START OF CRITICAL SECTION */
    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        if ((g_wdsSwitchIfcs[i].inUse != 0) && (MACCMP(pMac, g_wdsSwitchIfcs[i].mac) != 0)) {
            g_wdsSwitchIfcs[i].used++;
            break;
        }
    }
    pResult = (i == WDS_LINKS_NUMBER) ? NULL : g_wdsSwitchIfcs[i].pCookie;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);    /* END OF CRITICAL SECTION */
    return(pResult);
}

/*
 *Function Name:wdrv_wdsInterfaceCookieGetByIndex
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void * wdrv_wdsInterfaceCookieGetByIndex(UINT32 wIndex)
{
    return ((g_wdsSwitchIfcs[wIndex].inUse == 0) ? NULL : g_wdsSwitchIfcs[wIndex].pCookie);
}

/*
 *Function Name:wdrv_wdsInterfaceCookieRemove
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsInterfaceCookieRemove(char *pMac)
{
    int i;
    unsigned int flags;

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);      /* START OF CRITICAL SECTION */
    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        if ((g_wdsSwitchIfcs[i].inUse != FALSE) && (MACCMP(pMac, g_wdsSwitchIfcs[i].mac) != 0)) {
            break;
        }
    }
    if (i != WDS_LINKS_NUMBER) {
        g_wdsSwitchIfcs[i].inUse = FALSE;
    }
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);    /* END OF CRITICAL SECTION */
    return;
}

/*
 *Function Name:wdrv_wdsInterfaceCookieAdd
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsInterfaceCookieAdd(char *pMac, void *pCookie)
{                              
    int i;
    unsigned int flags;

    /* Remove if present */
    if (wdrv_wdsInterfaceCookieGet(pMac) != NULL) {
        wdrv_wdsInterfaceCookieRemove(pMac);
    }
    /* Critical section */
    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);      /* START OF CRITICAL SECTION */
    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        if (g_wdsSwitchIfcs[i].inUse == 0)
            break;
    }
    if (i != WDS_LINKS_NUMBER) {
        MACDUP2(g_wdsSwitchIfcs[i].mac, pMac);
        g_wdsSwitchIfcs[i].pCookie = pCookie;
        g_wdsSwitchIfcs[i].inUse = TRUE;
    }
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);    /* END OF CRITICAL SECTION */
    if (i == WDS_LINKS_NUMBER) {
        printk("%s : %d No free room to add interface to WDS bridge.\n", __FUNCTION__, __LINE__);
    }
    return;
}

void wdrv_wdsBridgeInfoGet(wdsDrvSwtchingEntry_s *pBuff)
{
    int i;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        *(pBuff + i) = g_wdsSwitchIfcs[i];

    }
}
extern STATUS tiwlan_ddaWdsReceive(void *pCookie, mem_MSDU_T * pMSDU);

/*
 *Function Name:wdrv_wdsBridgeSendPktToEthernet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsBridgeSendPktToEthernet(void *pCookie, mem_MSDU_T *pMsdu)
{

    EthernetHeader_T EthHeader;
    Wlan_LlcHeader_T  *pWlanSnapHeader;   
    UINT8            *dataBuf;
    register dot11_header_T      *pdot11Header;
    UINT32           lengthDelta;
    UINT8          createEtherIIHeader;
    UINT32          isEapolFrame;


    isEapolFrame = isEapol(pMsdu, FALSE, TRUE, NULL);
    dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
    pdot11Header = (dot11_header_T*) dataBuf;
    pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + sizeof(dot11_header_T));


    /* See if the LLC header in the frame shows the SAP SNAP... */
    if ( ( SNAP_CHANNEL_ID == pWlanSnapHeader->DSAP ) &&
         ( SNAP_CHANNEL_ID == pWlanSnapHeader->SSAP ) &&
         ( LLC_CONTROL_UNNUMBERED_INFORMATION ==  pWlanSnapHeader->Control ) ) {

        /* Check for the Bridge Tunnel OUI in the SNAP Header... */
        if ((SNAP_OUI_802_1H_BYTE0 == pWlanSnapHeader->OUI[ 0 ]) &&
            (SNAP_OUI_802_1H_BYTE1 == pWlanSnapHeader->OUI[ 1 ]) &&
            (SNAP_OUI_802_1H_BYTE2 == pWlanSnapHeader->OUI[ 2 ])) {
            /* Strip the SNAP header by skipping over it.                  */
            /* Start moving data from the Ethertype field in the SNAP      */
            /* header.  Move to the TypeLength field in the 802.3 header.  */
            createEtherIIHeader = TRUE;        
        }

        /* Check for the RFC 1042 OUI in the SNAP Header   */
        else {
            /* See if the Ethertype is in our selective translation table  */
            /* (Appletalk AARP and DIX II IPX are the two protocols in     */
            /* our 'table')                                                */
            if (( ETHERTYPE_APPLE_AARP == htons(pWlanSnapHeader->Type) ) ||
                ( ETHERTYPE_DIX_II_IPX == htons(pWlanSnapHeader->Type) )) {
                /* Strip the SNAP header by skipping over it. */
                createEtherIIHeader = FALSE;
            }
            /* All the rest SNAP types are transformed to EthernetII format */
            else {
                createEtherIIHeader = TRUE;
            }
        }
    }

    else {
        /* Non-SNAP packets strip out the WLAN header and create the dst,src,len header. */
        createEtherIIHeader = FALSE;
    }


    /* Prepare the Ethernet header. */
    MACDUP2( &EthHeader.DstAddr, &pdot11Header->address3);
    MACDUP2( &EthHeader.SrcAddr, &pdot11Header->address4);

    if ( createEtherIIHeader == TRUE ) {
        /* The LEN/TYPE bytes are set to TYPE, the entire WLAN+SNAP is removed.*/
        lengthDelta = sizeof(dot11_header_T) + WLAN_SNAP_HDR_LEN - ETHERNET_HDR_LEN;
        EthHeader.TypeLength = pWlanSnapHeader->Type;
    } else {
        /* The LEN/TYPE bytes are set to frame LEN, only the WLAN header is removed, */
        /* the entire 802.3 or 802.2 header is not removed.*/
        EthHeader.TypeLength = htons(pMsdu->dataLen - sizeof(dot11_header_T));
        lengthDelta = sizeof(dot11_header_T) - ETHERNET_HDR_LEN;
    }

    /* Replace the 802.11 header and the LLC with Ethernet packet. */
    dataBuf += lengthDelta;
    PAL_memCopy( dataBuf, (void*)&EthHeader, ETHERNET_HDR_LEN );

    pMsdu->firstBDPtr->dataOffset = lengthDelta;
    pMsdu->dataLen -= lengthDelta;
    pMsdu->firstBDPtr->length -= lengthDelta;

    if (REPORT_LEVEL(RL_BRIDGE)) {
        wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "w->e :: msdu->len=%d :: msdu->firstBD->len=%d\n",
                         pMsdu->dataLen, pMsdu->firstBDPtr->length );
    }
    wdsDrvStat.rx_pkts++;
    tiwlan_ddaWdsReceive(pCookie, pMsdu);
}

/*
 *Function Name:wdrv_wdsBridgeFromWlanFrame
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsBridgeFromWlanFrame(mem_MSDU_T *pMsdu,UINT32 pktEmulate, BOOL micOK, UINT16 srcStaIndex)
{
    BOOL    srcAuthorized = FALSE;
    UINT8 *dataBuf, *RA, *TA, apMac[6];
    register dot11_header_T  *pDot11Hdr;
    void *pCookie;


    dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
    pDot11Hdr = (dot11_header_T*) dataBuf;


    RA = pDot11Hdr->address1.addr;
    TA = pDot11Hdr->address2.addr;


    wdrv_dbGetBssid(apMac);

    if (!MACCMP(apMac, RA)) {
        /* Not our packet - drop */
        wlan_memMngrFreeMSDU(pMsdu->handle);
        wdsDrvStat.rx_wrongRA++;
#if 0
        printk("Wrong RA %02hx-%02hx-%02hx-%02hx-%02hx-%02hx.\n", RA[0],RA[1],RA[2],RA[3],RA[4],RA[5]);
#endif/* if 0 */
        return;
    }

    if (REPORT_LEVEL(RL_BRIDGE))
        wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "WDS bridge, SRC bridge %d (%02hx-%02hx-%02hx-%02hx-%02hx-%02hx) was found\n",
                         srcStaIndex, TA[0],TA[1],TA[2],TA[3],TA[4],TA[5]);

    srcAuthorized = wdrv_mgmtIsStationAuthorized(srcStaIndex);

    if (srcAuthorized == FALSE) {
        wlan_memMngrFreeMSDU(pMsdu->handle);
        wdsDrvStat.rx_srcNotAuthorized++;
        if (REPORT_LEVEL(RL_BRIDGE))
            wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "WDS bridge, SRC bridge is not authorized\n");
        return;
    }
    if (REPORT_LEVEL(RL_BRIDGE))
        wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "WDS bridge, SRC bridge is authorized\n");




    /*--------------------------------------------------*/
    /*             Destination determination            */
    /*--------------------------------------------------*/

    pCookie = wdrv_wdsInterfaceCookieGet(TA);
    if (pCookie == NULL) {
        wdsDrvStat.rx_TAnotFound++;
        wlan_memMngrFreeMSDU(pMsdu->handle);
        if (REPORT_LEVEL(RL_BRIDGE))
            wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "WDS bridge, SRC (%02hx-%02hx-%02hx-%02hx-%02hx-%02hx) is not found\n",
                             TA[0],TA[1],TA[2],TA[3],TA[4],TA[5]);
        return;
    }

    wdrv_wdsBridgeSendPktToEthernet(pCookie,pMsdu);
}

/*
 *Function Name:wdrv_wdsGetStat
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsGetStat(wdsDrvStat_s *pBuff)
{
    *pBuff = wdsDrvStat;
}

/*
 *Function Name:wdrv_wdsBeaconsSendEnable
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsBeaconsSendEnable(UINT32 enable)
{
    wdrv_wdsBssServiceStateSet(enable);

    if (enable) {
        whal_acxCmdBeaconActive(TRUE);
    } else {
        /* Configure firmware to infrastructure mode to stop sending beacons */
        whal_acxCmdBeaconActive(FALSE);
        wdrv_DeauthStations();

    }
}

/*
 *Function Name:wdrv_wdsCheckForRestart
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

UINT32 wdrv_wdsIsBssInService(void) 
{
    return (g_wdsIsBssEnabled);
}

/*
 *Function Name:wdrv_wdsBssServiceStateSet
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void wdrv_wdsBssServiceStateSet(UINT32 inService)
{
    g_wdsIsBssEnabled = inService;
}
