/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004    Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*                                                                        */
/*  MODULE:     whalAcxFWTrace.c                                          */
/*  PURPOSE:    Firmware trace information sending agent implementation.  */
/*              It is not going to be available in official release.      */
/*                                                                        */
/**************************************************************************/

#ifndef TIWLAN_FW_TRACE
#error "FW trace NOT supported !!!"
#else

#include "pform_types.h"
#include "wlan_pal.h"
#include "whalCommon.h"

#include "whalAcxFWTrace.h"
#include "whalInterrupt.h"
#include "whalAcxConfig.h"
#include "whalParams.h"
#include "whalAcxAccess.h"

/**************************************************************************/
/*                                                                        */
/*                         Local variables                                */
/*                                                                        */
/**************************************************************************/

static volatile UINT8 *         mappedFWBuf[2];
static UINT8 *                  driverBuf = NULL;
static int                      driverBufSize,
                                driverBufCount,
                                mappedFWBufSize;
static UINT32                   sequence_counter,
                                timer_counter,
                                no_interrupt_count;
static int                      disabled;
static int                      nextBufNum = 1;

static PALSRV_TcbObjTYPE        FWDebugTraceTimer;

#define WHAL_ACX_FW_TRACE_TIMER 1000


/****************************************************************************
 *                         whal_AcxFWTraceCallback()
 ****************************************************************************
 * DESCRIPTION: Firmware trace agent callback function.
 *              Copies the appropriate firmware trace buffer to host buffer
 *              and sends it to application for future processing.
 * 
 * INPUTS:  
 *          int bufNum          Number of firmware buffer ( 1 or 2 )
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

void whal_AcxFWTraceCallback( int     bufNum )
{
    /* NO Critical Section !!! Assuming this runs in interrupt only !!! */

    if (disabled)
    {
        return;
    }

/*  PAL_cacheInvalidate(0, mappedFWBuf[bufNum - 1], mappedFWBufSize); */
    
    whal_acxMemCopy(    driverBuf + (mappedFWBufSize + sizeof(int)) * driverBufCount + sizeof(int) , 
                        mappedFWBuf[bufNum - 1], 
                        mappedFWBufSize );
    
    
    *((UINT32 *)( driverBuf + (mappedFWBufSize + sizeof(int)) * driverBufCount) ) = htonl(sequence_counter++);

    if (++driverBufCount >= driverBufSize)
    {
        whal_AcxFWTraceSend( driverBuf, (mappedFWBufSize + sizeof(int)) * driverBufSize );
        driverBufCount  = 0;
        timer_counter   = 0;
    }

    ++timer_counter;
    no_interrupt_count = 0;
    
    /* I assume that FW never uses the same buffer twice in a row.  */
    /* I also assume that FW uses buffer 0 first                    */
    
    nextBufNum = (bufNum == 1) ? 2 : 1;
}


/****************************************************************************
 *                         whal_AcxFWTraceTimer()
 ****************************************************************************
 * DESCRIPTION: Firmware trace agent timer handler.
 *              Copies the appropriate firmware trace buffer to host buffer
 *              and sends it to application for future processing.
 *              In case there were no reports from firmware for 10 times,
 *              it sends both buffers.
 * 
 * INPUTS:  
 *          int bufNum          Number of firmware buffer ( 1 or 2 )
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

void whal_AcxFWTraceTimer(void)
{
    unsigned int       flags;

    /***********************************************************
    
     I assume this timer is never called from interrupt context,
    hence no protection in whal_AcxFWTraceCallback.
    
    ************************************************************/
    
    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  

    if (driverBufCount == 0)
    {
        ++no_interrupt_count;
    }

    if (no_interrupt_count == 10)
    { 
        /****************************************************************
        
        Special case hack in case we did not receive a single interrupt.
        data buffer may contain partial messages.
        
        *****************************************************************/
        
/*      PAL_cacheInvalidate(0, mappedFWBuf[bufNum - 1], mappedFWBufSize);  */
        
        whal_acxMemCopy( driverBuf + sizeof(int),
                         mappedFWBuf[nextBufNum - 1],
                         mappedFWBufSize );
        
        *(UINT32 *)driverBuf = htonl(sequence_counter++);
        
        nextBufNum = (nextBufNum == 1) ? 2 : 1;
        
        whal_acxMemCopy( driverBuf + sizeof(int) * 2 + mappedFWBufSize,
                         mappedFWBuf[nextBufNum - 1], 
                         mappedFWBufSize );
        
        *(UINT32 *)(driverBuf + sizeof(int) + mappedFWBufSize) = htonl(sequence_counter++);
        
        whal_AcxFWTraceSend(driverBuf, (mappedFWBufSize + sizeof(int))*2);
        
        no_interrupt_count = 0;
        goto exit;
    }

    if ((timer_counter == 0) && (driverBufCount != 0))
    {
        whal_AcxFWTraceSend( driverBuf, (mappedFWBufSize + sizeof(int)) * driverBufCount );
        driverBufCount = 0;
    }

    timer_counter = 0;
    
exit:
    PALSRV_tcbQueueAct(&FWDebugTraceTimer,
                       WHAL_ACX_FW_TRACE_TIMER,
                       whal_AcxFWTraceTimer,
                       0, 0);

    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  
}


/****************************************************************************
 *                         whal_AcxFWTraceInit()
 ****************************************************************************
 * DESCRIPTION: Firmware trace agent initialization API.
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/

int whal_AcxFWTraceInit(void)
{
    AcxElm_MemoryMap_T  memMap;
    int                 ret = OK;

    driverBufSize   = TIWLAN_FW_TRACE_DRIVER_BUF_SIZE;
    mappedFWBufSize = TIWLAN_FW_TRACE_FW_BUF_SIZE;

    sequence_counter    = 0;
    timer_counter       = 0;
    no_interrupt_count  = 0;

    if ((driverBuf = PAL_memAlloc(0, driverBufSize*(mappedFWBufSize + sizeof(int)), 0, 0)) == NULL)
    {
        printf("%s : Can't allocate %d bytes of memory !!!\n", __FUNCTION__, driverBufSize*mappedFWBufSize);
        ret = NOK;
        goto exit;
    }

    driverBufCount = 0;

    whal_acxConfigRead_MemoryMap(&memMap); 

    if ( (memMap.DebugBuffer1Start == NULL) || 
         (memMap.DebugBuffer2Start == NULL) )
    {
        printf("%s : Current firmware doesn't support trace [ %p, %p ] !!!\n", __FUNCTION__,
                     memMap.DebugBuffer1Start, memMap.DebugBuffer2Start);
        ret = NOK;
        goto exit;
    }

    printf("%s: Setting FW trace buffer addresses to \n %p and %p\n %p and %p\n", 
                 __FUNCTION__,
                 memMap.DebugBuffer1Start,
                 memMap.DebugBuffer2Start,
                 AcxMemAddr + memMap.DebugBuffer1Start,
                 AcxMemAddr + memMap.DebugBuffer2Start );
    
    mappedFWBuf[0] = AcxMemAddr + memMap.DebugBuffer1Start;
    mappedFWBuf[1] = AcxMemAddr + memMap.DebugBuffer2Start;

    PALSRV_tcbNew(&FWDebugTraceTimer);

    disabled = 0;

    PALSRV_tcbQueueAct( &FWDebugTraceTimer,
                        WHAL_ACX_FW_TRACE_TIMER,
                        whal_AcxFWTraceTimer,
                        0, 0 );
 exit:
    if (ret == NOK && driverBuf)
    {
        PAL_memFree(0, driverBuf, 0);
    }
    return ret;
}

/****************************************************************************
 *                         whal_acxFWTraceShutdown()
 ****************************************************************************
 * DESCRIPTION: Firmware trace agent shutdown API.
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/

int whal_acxFWTraceShutdown(void)
{
    disabled=1;
    
    printf("%s\n", __FUNCTION__);
    
    PALSRV_tcbDelete(&FWDebugTraceTimer);
    
    if (driverBuf)
    {
        PAL_memFree(0, driverBuf, 0);
    }

    return OK;
}

#endif

