/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxProcess.c
 *   PURPOSE: Implements action on the Acx card (Reset, Run, SendCmd, Sw Download)
 *
 ****************************************************************************/
#include "wlan_pal.h"
#include "whalCommon.h"
#include "whal.h"
#include "whalAcxRegs.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalAcxProcess.h"
#include "whalAcxMbox.h"
#include "whalAcxConfig.h"
#include "whalRxQueue.h"
#include "whalTxQueue.h"
#include "whalRxBufs.h"
#include "whalRxMaster.h"
#include "whalTxBufs.h"
#include "whalParams.h"
#include "whalAcxCmd.h"
#include "whalAcxEeprom.h"
#include "QoS/QoSClassifier.h"
#include "QoS/QoSACParams.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "dda.h"
#include "acx_hw.h"
#include "mgmtSsidHandler.h"
#include "whalEEPromLess.h"



#define FW_DL_MAX_SIZE 0x20000

/* 
   Global optional User Call-Back. The user can define this function as extern in his
   BSP and return the Host SDRAM Base address he wants to exposure to the VLYNQ remote
   module.  See implementation in whal_acxConfigDma() .
   
e.g:  from the BSP (outside the WLAN Driver module scope): 

#define HOST_SDRAM_BASE_ADDRESS    0x14000000

extern  unsigned int(*getHostSdram_CB)(void);  
unsigned int acxGetHostSdram(void);

    getHostSdram_CB=acxGetHostSdram;
    UINT32 acxGetHostSdram(void) { HOST_SDRAM_BASE_ADDRESS }
   
*/   

UINT32(*getHostSdram_CB)(void) = NULL;  

/****************************************************************************
 *               External labels, taken from acx100_bin.S
 ****************************************************************************
 * 
 ****************************************************************************/

UINT8 * Acx100_bin_0_Array = NULL;
UINT8 * Acx100_bin_0_Array_Extra = NULL;
UINT32 Acx100_bin_0_Array_Len = 0;
UINT32 Acx100_bin_0_Array_Extra_Len = 0;

/* Firmware generated assembly file externals */
extern UINT8   _ACX111_REG_binArrayStart[];    /* Debug image           */
extern UINT8   _ACX111_REG_binArrayEnd[];      /* end of debug image    */
extern UINT8   _ACX111_REG_Version[];          /* debug image version   */

/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 * 
 ****************************************************************************/
int  whal_acxProcWaitInitComplete(void);
void whal_acxProcRun(void);
void whal_acxProcReset(void);
int  whal_acxProcLoadFwImage(UINT32 Acx_BaseMemAddr, UINT32 AddrOffset, UINT32 *RetDataLen);
int  whal_acxProcDownloadFwImageFTP(unsigned char **pBuf, int *Len, int Type);
int  whal_acxProcDownloadFwImageTFTP(unsigned char **pBuf, int *Len, int Type);
int  whal_acxProcGetFwImage(unsigned char **pBuf, int *Len, unsigned char **, int *);
int  whal_acxProcGetRadioImage(unsigned char **pBuf, int *Len);
int  whal_acxProcInitiateDebug(void);

int  whal_acxConfigDma(void);

/****************************************************************************
 *                      whal_acxProcMemoryConfigure()
 ****************************************************************************
 * DESCRIPTION: Configure the TNETW memory
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int wDbgWait1=0;
int whal_acxProcMemoryConfigure(void)
{
    dot11_tim_T *pWlanElm_Tim = whal_ParamsGetElm_Tim();
    WlanElm_QueueHead_T WlanElm_QueueHead;
    UINT32 TxQueuesSize;
    UINT32 HostRxDescriptorsAddr;
    int Stt;
    int Qid;
    int    originalSupportTIcapTemplate;
    int saveLengthTim;
    UINT8 WoneIndex, TotalWones;
    edcf_info_t  edcfInfo;
    int   edcfInfo_RC;

    UINT32  queueThresh, queueHighThresh;


    wdrv_dbGetNumSupWone(&TotalWones);


    /*
         * Configure Beacon and ProbeResponse templates
         * --------------------------------------------
         */

    for (WoneIndex = 0 ; WoneIndex < TotalWones ; WoneIndex++)
        {

        printf("whal_acxProcConfigure: templates (beacon, probe response, tim) for [%d] Network\n", WoneIndex);
        /* configure the ACX100 templates */
        #ifndef NO_PBCC_CHECK
        if (whal_ParamsGetNvParams()->ProbePatch == 1)
            printf("whal_acxProcConfigure: NO probe response template\n");
        else
            #endif

            {
            /* First ProbeRsp template should be of max. len, so include Rsne IE if required with max length */
            dot11_TIcap_T TIcapStruct;

            /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
                {
                    saveLengthRsne = pWlanElm_Rsne->hdr.eleLen;
                    pWlanElm_Rsne->hdr.eleLen = DOT11_RSNE_MAX;
                }*/
            originalSupportTIcapTemplate = whal_ParamsGetGenParams(0)->SupportTIcapTemplate;
            /* First ProbeRsp template should be of max. len, so include 4x IE */
            whal_ParamsGetGenParams(0)->SupportTIcapTemplate = TRUE; 
            /* Set the TI Capability (4x) to its maximum size so the Probe Response template's IE has the maximum size */
            PAL_memSet(&TIcapStruct,0,sizeof(dot11_TIcap_T));
            TIcapStruct.hdr.eleLen = DOT11_TI_CAP_MAX_LEN;
            whal_ParamsSetElm_TIcapPrivateSupport(&TIcapStruct);
            /* whal_acxCmdProbeRespTemplate(WoneIndex); */
            whal_ParamsSetElm_TIcapSupport();   /* Restore the IE to the real one */
            whal_ParamsGetGenParams(0)->SupportTIcapTemplate = originalSupportTIcapTemplate;
            /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
                {
                    pWlanElm_Rsne->hdr.eleLen = saveLengthRsne;
                }*/
            /* whal_acxCmdProbeRespTemplate(WoneIndex); */
        }

        whal_acxCmdBeaconTemplate(WoneIndex); 

        /* Tim template, first reserve space (len=MAX), second init to empty */
        saveLengthTim = pWlanElm_Tim->hdr.eleLen;
        pWlanElm_Tim->hdr.eleLen = 3+DOT11_PARTIAL_VIRTUAL_BITMAP_MAX;
        /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
            {
                pWlanElm_Rsne->hdr.eleLen = DOT11_RSNE_MAX;
            }*/

        whal_acxCmdTimTemplate(WoneIndex);

        pWlanElm_Tim->hdr.eleLen = saveLengthTim;
        /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
            {
                pWlanElm_Rsne->hdr.eleLen = saveLengthRsne;
            }*/
        whal_acxCmdTimTemplate(WoneIndex);      
    }

    if (TotalWones > 1)
        {
        whal_acxCmdWoneToggleTemplate();
    }    

    /*
         * Configure memory queues and pools
         * ---------------------------------
         */
    HostRxDescriptorsAddr = (UINT32)whal_rxMaster_getHostQueueAddr();

    printf("whal_acxProcConfigure: Host Queue is in 0x%X\n", (int)HostRxDescriptorsAddr);
    Stt = whal_acxConfigMemory(HostRxDescriptorsAddr);
    if (Stt != OK)
        {
        printf("whal_acxProcMemoryConfigure: ERROR: whal_acxConfigMemory failed\n");
        return(NOK);
    }

    /*
         * Configure Feature Options
         * -------------------------
         */

    whal_acxConfigFeatureOptions(0);    

    /*
         * Read Queue heads
         * ----------------
         */
    printf("whal_acxProcConfigure: read queue heads\n");    
    Stt = whal_acxConfigRead_QueueHead(&WlanElm_QueueHead);
    if (Stt != OK)
        {
        printf("whal_acxProcMemoryConfigure: ERROR: whal_acxConfigRead_QueueHead failed\n");
        return(NOK);
    }

    while(wDbgWait1)
        {
        PAL_sleepLong(500);
    }
    /* 
         * configure RxQueue/TxQueue objects 
         */
    printf("whal_acxProcConfigure: configure RxQueue/TxQueue objects\n");

    TxQueuesSize = 0;

    for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
        {   
        /*
             * Tx queue config 
             */
        int ac;
        for (ac=0; ac < WDRV_MAX_TX_QUEUE ; ac++)
            {
            if (wdrv_dbGetAcxGlobalQindex(WoneIndex,ac,&Qid)==OK)
                {
                TxQueuesSize  += whal_txQueue_Config(Qid, (UINT32)WlanElm_QueueHead.TxQueueHead[Qid].Addr);

                /* If required, configure the ACX Tx queue EDCF QoS parameters & packet bursting parameters. */

                edcfInfo_RC = wdrv_dbGetEdcfParams(WoneIndex, ac, &edcfInfo);
                if(edcfInfo_RC)
                    {
                    whal_acxConfigAccessCategory(Qid, &edcfInfo);
                    printf(" whal_acxProcConfigure: AC=%d \n", edcfInfo.acID);
                }
                /* If required, configure the ACX Tx queue buffers usage threshold. */
                if ( wdrv_dbGetQueueThresh(WoneIndex, ac, &queueThresh) ) { 
									wdrv_dbGetQueueHighThresh(WoneIndex, ac, &queueHighThresh);
									printf(" whal_acxProcConfigure: Qud=%d Guaranteed threshold = %d, High threshold = %d blocks \n", Qid, queueThresh, queueHighThresh);
									whal_acxConfigAccessCategoryCfg( WoneIndex, edcfInfo.acID, Qid, (UINT16)queueThresh, (UINT16)queueHighThresh);
                }
            }
        }
    }

    whal_ParamsPrintMemMap();
    return OK;
}


/****************************************************************************
 *                      whal_acxProcInitiate()
 ****************************************************************************
 * DESCRIPTION: Download firmware code to the ACX100 and run it
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxJTAGDebugFWReady = 0;

int whal_acxProcInitiate(void) 
{
    int Stt;
    UINT32 DeviceVendorId;
    UINT32 ImageDataLen;
    EepromlessCfg_T EepromlessCfg;


#if( USE_ENDIAN_SWAP == 0 )
    /*for  little endian system.*/
    whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0x00000000);
#else
    /* use endian swap */
    whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0xffffffff);
#endif


#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150 )
    /*
     * 1150 Problem: PCI registers are not set - no vendor/device on PCI_BE_VENDOR_ID_REG register
     * 1150 Workaround: Identification test by Chip ID 
     */
    DeviceVendorId = whal_acxRegRead(CHIP_ID);
    printf("%s: Got CHIP_ID = %#x\n", __func__, DeviceVendorId);
    
    if( (DeviceVendorId != ((0x0401<<16) | 0x0101)) /* PG1.0 */  &&
        (DeviceVendorId != ((0x0402<<16) | 0x0101)) /* PG1.1 */  &&
        (DeviceVendorId != ((0x0403<<16) | 0x0101)) /* PG2.0 */  &&
        (DeviceVendorId != ((0x0601<<16) | 0x0101)) /* 1350a */ 
		)
    {
        printf( "whal_acxProcInitiate: ERROR, Fail to identify TNETW card Addr 0x%x = 0x%x and not 0x%x\n",
                CHIP_ID, DeviceVendorId, ((0x0401<<16) | 0x0101));
        return NOK;
    }
#else
    /*
     * Identification test by Device and Vendor ID
     */
    DeviceVendorId = whal_acxRegRead(REG_DEVICE_VENDOR_ID);
    if (DeviceVendorId != ((ACX_DEVICE_ID<<16) | ACX_VENDOR_ID))    
    {
        printf( "whal_acxProcInitiate: ERROR, Fail to identify TNETW card Addr 0x%x = 0x%x and not 0x%x\n",
                REG_DEVICE_VENDOR_ID, DeviceVendorId, ((ACX_DEVICE_ID<<16) | ACX_VENDOR_ID));
        return NOK;
    }

    printf( "whal_acxProcInitiate: found DEVICE_VENDOR ID(0x%x) = 0x%x, 0x%x\n",
                REG_DEVICE_VENDOR_ID, DeviceVendorId, ((ACX_DEVICE_ID<<16) | ACX_VENDOR_ID));
#endif

    whal_acxProcReset();

	/* 
	 * Prepare the boot in case of eeprom present and eepromless
	 */
	if ( whal_ParamsGetEeepromLess() )
	{
		printf( "*****  EepromlessSupport !!!!!!!!!\n");

		/* Set EEPROM NOT present mode operation on SCR_PAD2 */
		whal_acxRegWrite(ACX_EEPROMLESS_IND_REG, HW_EEPROM_NOT_PRESENTED);

		/* Prepare the download */
		whal_FwCtrl_Eepromless_PrepareBoot( &EepromlessCfg);
	}
	else
	{
		/* Set EEPROM present mode operation on SCR_PAD2 */
		whal_acxRegWrite( ACX_EEPROMLESS_IND_REG, HW_EEPROM_PRESENTED);
	}


	/*
     * Read Halt bit
     */
    if (whal_acxRegIsBitSet( ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT ))         
    {
        printf("whal_acxProcInitiate: Cpu halt -> download code\n");
        
        if (! Acx100_bin_0_Array )
        {
            Acx100_bin_0_Array      =   _Acx100_bin_0_ArrayStart;
            Acx100_bin_0_Array_Len  =   _Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart;        
        }
        
        Stt = whal_acxProcLoadFwImage(ACX100_MEM_ADDRESS, 0, &ImageDataLen);
        
        if (Stt != OK)       
        {
            printf("whal_acxProcInitiate: download failed\n");
            return NOK;
        }
    }

    whal_acxConfigDma();

    printf("whal_acxProcInitiate: Run\n");
    
    whal_acxProcRun(); 

    if ( whal_ParamsGetEeepromLess() ) 
		whal_FwCtrl_Eepromless_AfterBoot(&EepromlessCfg,whal_apiGetAcxMemAddr());


    if ( whal_acxProcWaitInitComplete() != OK )   
    {
        printf("whal_acxProcInitiate: Timeout waiting for the ACX100 to complete Initialization\n");
        return NOK;
    }

    printf("whal_acxProcInitiate: Firmware is running...\n");    
    return OK;
}

/****************************************************************************
 *                      whal_acxProcRun()
 ****************************************************************************
 * DESCRIPTION: Run the ACX firmware
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
void whal_acxProcRun(void)
{
    /*
     * Run Acx Cpu
     */
    whal_acxRegResetBitVal(ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT);
    /*PAL_sleepLong(500);*/
}



/****************************************************************************
 *                      whal_acxProcReset()
 ****************************************************************************
 * DESCRIPTION: Reset the ACX100
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
void whal_acxProcReset(void)
{
    /*
     * Halt the Acx Cpu
     */
    whal_acxRegSetBitVal(ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT);
}



/****************************************************************************
 *                      whal_acxProcWaitInitComplete()
 ****************************************************************************
 * DESCRIPTION: Wait (polling) for init complete interrupt
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxProcWaitInitComplete(void)
{
    int i;

    printf("whal_acxProcWaitInitComplete:\n");
    
    for(i=0; i<40; i++)
    {
        PAL_sleepLong(50);
        
        if (whal_acxIntrCheckAndAck(INTR_INIT_COMPLETE))
        {
            return OK;
        }
    }

    return NOK;
}



/****************************************************************************
 *                      whal_acxProcLoadFwImage()
 ****************************************************************************
 * DESCRIPTION: Load Image from the host and download into the Acx100
 *                AddrOffset == 0 : FW is loaded
 *                AddrOffset != 0 : Module is loaded
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxProcLoadFwImage(UINT32 Acx_BaseMemAddr, UINT32 AddrOffset, UINT32 *RetDataLen)
{
    unsigned char *pNextChar;
    unsigned int  checksumCalc = 0;
    unsigned int  idx, temp;
    int i;
    unsigned int  FileCsum;
    unsigned int  DataLen;
    unsigned int  AcxOffset;
    unsigned int  NumWrittenChars;
    int fRC, Status;

    unsigned char *FileBuf = 0, *ExtraFileBuf = 0;
    int FileLen = 0, ExtraFileLen = 0;
    BOOL inExtraFile = FALSE;

    printf( "whal_acxProcLoadFwImage: 0x%x, 0x%x\n", Acx_BaseMemAddr, AddrOffset);
    /*
     * get image 
     */
    if (AddrOffset == 0)
    {
        /* firmware image */
      puts("whal_acxProcLoadFwImage() -- Loading FW image");
      Status = whal_acxProcGetFwImage(&FileBuf, &FileLen, &ExtraFileBuf, &ExtraFileLen);
    }
    else
    {
        /* radio image */
      puts("whal_acxProcLoadFwImage() -- Loading radio image");
        Status = whal_acxProcGetRadioImage(&FileBuf, &FileLen);
    }

    printf( "whal_acxProcLoadFwImage: 1, pBuf=0x%x, len=0x%x. Extra pBuf=%#x, len=%#x\n",
      (int)FileBuf, FileLen, (int)ExtraFileBuf, ExtraFileLen);
    if (Status < 0)
    {
        printf ("whal_acxProcLoadImage: can't load Image\n");
        return NOK;
    }

    printf( "whal_acxProcLoadFwImage: 2, pBuf=0x%x, len=0x%x. Extra pBuf=%#x, len=%#x\n",
      (int)FileBuf, FileLen, (int)ExtraFileBuf, ExtraFileLen);
    /*
     * Extract and Calculate CheckSum and Length
     */
    FileCsum = ENDIAN_HANDLE_LONG(*(unsigned long *)&FileBuf[0]);
    DataLen  = ENDIAN_HANDLE_LONG(*(unsigned long *)&FileBuf[4]);

    *RetDataLen = DataLen;
    pNextChar = &FileBuf[4];

    checksumCalc = 0;

    /* Add the NumBytes to the checksum calculation. */
    for ( idx = 0; idx < 4; idx++ )  {
        checksumCalc += (unsigned int)*pNextChar++;
    }


    AcxOffset = AddrOffset;
    NumWrittenChars = 0;
    i = 3;
    temp = 0;

    printf( "whal_acxProcLoadFwImage: 3, pBuf=0x%x, len=0x%x, DataLen=0x%x\n", (int)FileBuf, FileLen, DataLen);
    /*
     * download the image into the MemMapped area in the Acx100 board
     */
    while ( NumWrittenChars < DataLen )     {
        temp |= ((unsigned int)(*pNextChar)) << ( 8 * i );
        checksumCalc += ((unsigned int)(*pNextChar));
        ++pNextChar;
        ++NumWrittenChars;
        if ( --i < 0 )      {
            temp = ENDIAN_HANDLE_LONG(temp);

         /*
         if( AddrOffset)
         {
            printf( "whal_acxMemCopy(%p, %p, %d)\n", (char *)(Acx_BaseMemAddr+AcxOffset), (char *)&temp, sizeof(temp));
            PAL_sleepLong(100);
         }
         */

            whal_acxMemCopy((char *)(Acx_BaseMemAddr+AcxOffset), (char *)&temp, sizeof(temp));

            AcxOffset += 4;
            temp = 0;
            i = 3;
        }
        if ((pNextChar >= FileBuf + FileLen) && !inExtraFile) {
            pNextChar = ExtraFileBuf;
            inExtraFile = TRUE;
        }
    }

    printf( "whal_acxProcLoadFwImage: 4, pBuf=0x%x, len=0x%x\n", (int)FileBuf, FileLen);
    /*
     * CheckSum
     */
    printf ("whal_acxProcLoadFwImage: Checksum, calc=0x%x, file=0x%x\n", checksumCalc, FileCsum);
    fRC = (checksumCalc == FileCsum) ? OK : NOK ;
    PAL_sleepLong(300);

    return(fRC);
}



/****************************************************************************
 *                      whal_acxProcDownloadFwImageFTP()
 ****************************************************************************
 * DESCRIPTION: Download image from the host (via FTP)
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxProcDownloadFwImageFTP(unsigned char **pBuf, int *Len, int Type)
{
    return OK;
}



#ifndef TIWLAN_NO_FIRMWARE_TFTP

/****************************************************************************
 *                      whal_acxProcDownloadFwImage()
 ****************************************************************************
 * DESCRIPTION: Download image from the host (via TFTP)
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxProcDownloadFwImageTFTP(unsigned char **pBuf, int *Len, int Type)
{
    return OK;
}

#endif /* TIWLAN_NO_FIRMWARE_TFTP */



/****************************************************************************
 *                      whal_acxProcGetFwImage()
 ****************************************************************************
 * DESCRIPTION: get firmware image pointer and length
 *      The FW file ACX100.bin converted to ASM file using BinToAsm util
 *      and have two global symbols Acx100_binArrayStart and Acx100_binArrayEnd
 *      for each image
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
STATUS whal_acxProcGetFwImage(
                                unsigned char **    pBuf,
                                int *               len,
                                unsigned char **    extraBuf,
                                int *               extraLen )
{
    printf("Compiled for %s radio\n", XXstr(TIWLAN_RADIO_NAME));

#if defined(TIWLAN_FW_RADIA)
    wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
#elif defined(TIWLAN_FW_RADIA_DB)
    wdrv_dbSetRadioFrequency(RADIO_BAND_2_4_AND_5GHZ);
#elif defined(TIWLAN_FW_RADIA_CRT)
    wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
#elif defined(TIWLAN_FW_RADIA_WBR)
    wdrv_dbSetRadioFrequency(RADIO_BAND_2_4_AND_5GHZ);
#elif defined(TIWLAN_FW_RADIA_DCR)
    wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
#elif
     #error "Unknwon radio type !!!\n"
#endif

    *pBuf   = Acx100_bin_0_Array;
    *len    = Acx100_bin_0_Array_Len;
    
    if (Acx100_bin_0_Array_Extra)
    {
        *extraBuf = Acx100_bin_0_Array_Extra;
        *extraLen = Acx100_bin_0_Array_Extra_Len;
    }
    else
    {
        *extraBuf = NULL;
        *extraLen = 0;
    }
    
    return OK;
}


/****************************************************************************
 *                      whal_acxProcGetRadioImage()
 ****************************************************************************
 * DESCRIPTION: Gets the Radio Module image
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/
int whal_acxProcGetRadioImage( unsigned char **pBuf, int *Len)
{
   /* Stub */
   *pBuf = NULL;
   *Len = 0;
   return( OK);
}/* END whal_acxProcGetRadioImage */




/****************************************************************************
 *                      whal_acxConfigDma()
 ****************************************************************************
 * DESCRIPTION: Configure ACX DMA for Indirect Memory Access Master Mode.
 * 
 * INPUTS:  None    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/
int  whal_acxConfigDma(void)
{
    UINT32 HostSdramBaseAddress = EVM3_SDRAM_MEM_ADDR;  /* Default if not applied User Callback */
    
    /*
       The Call Back is initialized with NULL value. If you intends to assign a function to the Empty Call-Back,
       make sure that the assignment is done before this point...
    */
   if( getHostSdram_CB != NULL )
     HostSdramBaseAddress = getHostSdram_CB();
        
   /* configure ACX DMA controls to enable HW translation of pointers */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
   whal_acxRegWrite(DMA_HDESC_OFFSET, ACX111_VL1_REMOTE_BASE - PHYS_ADDR(HostSdramBaseAddress)); 
   whal_acxRegWrite(DMA_HDATA_OFFSET, ACX111_VL1_REMOTE_BASE - PHYS_ADDR(HostSdramBaseAddress));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
   whal_acxRegWrite(DMA_HDESC_OFFSET, - PHYS_ADDR(HostSdramBaseAddress));
   whal_acxRegWrite(DMA_HDATA_OFFSET, - PHYS_ADDR(HostSdramBaseAddress));
#else
   #error WLAN_BUS_TYPE must be defined correctly.
#endif /* WLAN_BUS_TYPE */

#if( USE_ENDIAN_SWAP == 1 )
   /* Big endian CPU host */
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
   /* Big endian CPU host */
    /* 
     * 1150 Problem: Could not work with endian as in 1130, DMA fail
     * 1150 Workaround: Use Host Descriptor as big endianess 
     * don't convert host desc (whalHostDesc.h)
     * The VLYNQ configuration convert from big to little so the host data
     * should not be converted
     */
   whal_acxRegWrite(DMA_GLB_CFG, 2); /* 1150 endian problem - Host descriptor is big endian */
#else
   /* 1130 Big endian CPU host */
   whal_acxRegWrite(DMA_GLB_CFG, 1);
#endif
#else /* USE_ENDIAN_SWAP != 1 */
   /* Little endian CPU host */
   whal_acxRegWrite(DMA_GLB_CFG, 0);
#endif /* USE_ENDIAN_SWAP */

   whal_acxRegWrite(ACX_DMA_CFG0_REG   , ACX_DMA_HOST_INDIRECT);
   whal_acxRegWrite(ACX_DMA_CFG1_REG   , ACX_DMA_HOST_INDIRECT); 
   
   
   return 0;
}/* END whal_acxConfigDma() */




