/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE:  whalRx.c
 *   PURPOSE: Main receive module
 *   
 ****************************************************************************/

/****************************
 * HAL Include files
 ****************************/
#include "whalCommon.h"
#include "whal.h"
#include "whalRx.h"
#include "acx_hw.h"
#include "whalRxMaster.h"

/***************************
 * Function Implementation
 ***************************/

/****************************************************************************
 *                      whal_rxRecv()
 ****************************************************************************
 * DESCRIPTION:   Receives a buffer from the WLAN chip.
 * 
 * INPUT :  None  
 * OUTPUT:  INT32 *MpduBufLen = Returned MPDU length.
 *          RxAttr_T *pRxAttr = Storage of received attributes
 * 
 * RETURN:  MEM_HANDLE: Handle of the buffer descriptor (BD). NULL if fail.
 ****************************************************************************/
MEM_HANDLE whal_rxRecv(int *MpduBufLen, RxAttr_T *pRxAttr)
{
	return whal_rxMasterRecv( MpduBufLen, pRxAttr );	
}/* END whal_rxRecv() */

/****************************************************************************
 *                      whal_rxInit()
 ****************************************************************************
 * DESCRIPTION:	Initializes the Rx queue object and the Rx bufs object.
 *
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURNS:	int : OK or NOK
 ****************************************************************************/
int whal_rxInit(void)
{
	return whal_rxMasterInit();	
}/* END whal_rxInit() */

/****************************************************************************
 *                      whal_rxClean()
 ****************************************************************************
 * DESCRIPTION: Cleans the already processed Queue descriptors and buffers.
 *
 * NOTE  :  Empty function
 *
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURN:  None
 ****************************************************************************/
void whal_rxClean(void)
{
	return;	
}/* END whal_rxClean() */

/****************************************************************************
 *                      whal_rxInfoPrint()
 ****************************************************************************
 * DESCRIPTION:	Prints Rx queue/bufs info.
 *
 * INPUT :  None
 * OUTPUT:  char *buf
 * 
 * RETURN:  Output length. 
 ****************************************************************************/
int  whal_rxInfoPrint(char *buf)
{
	int len = 0;
	return (len + whal_rxMasterInfoPrint(buf));
}/* END whal_rxInfoPrint() */

