/**********************************************************************************************************************
  FILENAME:       tx_memory_calculation_driver.h

  DESCRIPTION:    Internal header file for the tx memory calculation mechanism to be used in the driver

  Copyright (c) 2004 Texas Instruments Inc. 
  All Rights Reserved This program is the confidential and proprietary 
  product of Texas Instruments Inc.  Any Unauthorized use, reproduction or 
  transfer of this program is strictly prohibited.
***********************************************************************************************************************/
#ifndef TX_MEMORY_CALCULATION_H
#define TX_MEMORY_CALCULATION_H

/* Basic Types needed are:
typedef unsigned char UINT8;
typedef unsigned short UINT16;
typedef unsigned int UINT32;
*/
#include "wlan_pal.h"

typedef UINT32 MBlkAddr_t; //holds 19-bit memory block address for 32-byte boundaries

typedef struct
{
    UINT16          fragThreshold;
    UINT16		    wpa_MPDU_addition;
    UINT16          wpa_MSDU_addition;
    UINT16          blk_size_addition;
    UINT16          driverPaddingSize;
    UINT16          numMPDUs;
} FragmentationData_t;

/*typedef struct
{
    UINT16          finalFragThreshold;
    UINT8           totalNumBlocksToAllocate;
    UINT8           fragmentedFrame;
} MemBlocksTxAllocation_t;
*/
typedef enum
{
    TX_MEM_CALC_RATE_INDEX_1MBPS   =  0,
    TX_MEM_CALC_RATE_INDEX_2MBPS   =  1,
    TX_MEM_CALC_RATE_INDEX_5_5MBPS =  2,
    TX_MEM_CALC_RATE_INDEX_6MBPS   =  3,
    TX_MEM_CALC_RATE_INDEX_9MBPS   =  4,
    TX_MEM_CALC_RATE_INDEX_11MBPS  =  5,
    TX_MEM_CALC_RATE_INDEX_12MBPS  =  6,
    TX_MEM_CALC_RATE_INDEX_18MBPS  =  7,
    TX_MEM_CALC_RATE_INDEX_22MBPS  =  8,
    TX_MEM_CALC_RATE_INDEX_24MBPS  =  9,
    TX_MEM_CALC_RATE_INDEX_36MBPS  =  10,
    TX_MEM_CALC_RATE_INDEX_48MBPS  =  11,
    TX_MEM_CALC_RATE_INDEX_54MBPS  =  12,
    TX_MEM_CALC_RATE_INDEX_MAX     =  TX_MEM_CALC_RATE_INDEX_54MBPS,
    TX_MEM_CALC_MAX_RATE_INDEX,
    TX_MEM_CALC_INVALID_RATE_INDEX = TX_MEM_CALC_MAX_RATE_INDEX
} TxMemCalcRateIndex_e;


// Structure for parameters to functions
typedef struct
{
    UINT16 txRate;
    UINT16 txCtlRate;
    UINT16 memBlkSize;
    UINT16 dataLength;
    UINT16 txOpLimit;
    UINT16 originalFragThreshold;
    UINT8 encryptionType; 
    UINT8 mpduHeaderLength;
    UINT8 unicastFrame;
    UINT8 driverFragmentationOn;
    UINT8 txCts;
    UINT8 txRts;
    UINT8 txNoAck; 
    UINT8 tx4X;
} FragmentationDecisionData_t;




/* though the SIFS duration can either be 10usec(B) or 16usec(A) we will take the worst case on the effect 
   on the txOp fragmentation threshold*/
#define TX_MEM_CALC_SIFS_DURATION_FOR_FRAG 16
#define TX_MEM_CALC_4X_FRAG_THRESHOLD      4096

#define TX_MEM_CALC_FCS_LENGTH  4

#define TX_MEM_CALC_MIN_FRAG_THRESH        256

#define TX_MEM_CALC_WEP_TX_IV_SIZE         4 
#define TX_MEM_CALC_WEP_TX_ICV_SIZE        4
#define TX_MEM_CALC_TKIP_TX_EIV_SIZE       4
 

#define TX_MEM_CALC_RSN_HEADER_LEN         8	  // Length of RSN Header in packet
#define TX_MEM_CALC_WPA_MIC_LEN            8	  // Length of MIC field in packet
#define TX_MEM_CALC_AES_PAD_SIZE           (TX_MEM_CALC_RSN_HEADER_LEN + TX_MEM_CALC_WPA_MIC_LEN)

#define TX_MEM_CALC_CTS_FRAG_DATA_TIME      248

// AVISHAI_----- Arik fill this in!!!!!!
#define TX_MEM_CALC_RTS_FRAG_DATA_TIME      248


// Function Prototypes
/******************************/


/**********************************************************************************************************************
   CalculateHeaderSize

   DESCRIPTION:  Calculates the total header size needed for each fragment (after encryption if needed)

   PARAMETERS:   CipherSuite_e encryptionType - specifies the type of encryption for the frame
                                                valid values:
                                                      CIPHER_NONE = 0,
                                                      CIPHER_WEP  = 1,
                                                      CIPHER_TKIP = 2,
                                                      CIPHER_AES  = 3
                 UINT8 mpduHdrLength          - the header length of the original frame

   RETURNS:      Nothing (sets the header size and the aes overhead in the global structure)
***********************************************************************************************************************/
void CalculateHeaderSize( UINT8 encryptionType, UINT8 mpduHdrLength);


/**********************************************************************************************************************
   CalculateFragmentationThreshold

   DESCRIPTION:  Calculates the fag threshold per frame according to the frag threshold defined by
                 the user and the TxOp fragmentation (if WME is used).

   PARAMETERS:   UINT16 originalFragmentThreshold - the user specified fragmentation threshold
                 UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
                 UINT32 txRate                    - the rate field as it is put in the TxDesc
                 UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise
                 UINT8  txCts                     - "1" if the frame is sent with CTS protection "0" otherwise  
                 UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame
                 UINT8  tx4X                      - "1" if the frame is sent with 4X on "0" otherwise  
   
   RETURNS:      Nothing (Sets the fragThresh in the global structure)
***********************************************************************************************************************/
void CalculateFragmentationThreshold(FragmentationDecisionData_t* fragDecisionData);


/**********************************************************************************************************************
   GetTxOpFragThreshold

   DESCRIPTION:  Calculates the fragmentation threshold caused by the TxOpLimit.

   PARAMETERS:   UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
                 UINT32 txRate                    - the rate field as it is put in the TxDesc
                 UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise  
                 UINT8  txCts                     - "1" if we are working with CTS protection "0" otherwise  
                 UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame

   RETURNS:      The fragmentation threshold calculated
***********************************************************************************************************************/
UINT16 GetTxOpFragThreshold(FragmentationDecisionData_t* fragDecisionData);


/**********************************************************************************************************************
   CalculateNumberOfFragments

   DESCRIPTION:  Calculates the number of fragments according to the decided upon frag threshold

   PARAMETERS:   UINT16 dataLength        - the length of the data in teh frame
                 UINT8 mpduHeaderLength   - the header length of the original frame
                 UINT8 unicastFrame       - "0" if the frame is Bcast / Multicast "1" if unicast
                 UINT8 driverFragmentationOn - bit_0 of the dataFlowOptions IE specifies whether driver encryption is used
 
   RETURNS:      Nothing (updates the number of fragments in the global structure)
***********************************************************************************************************************/
void CalculateNumberOfFragments(UINT16 dataLength, UINT8 mpduHeaderLength, UINT8 unicastFrame, UINT8 driverFragmentationOn);


/**********************************************************************************************************************
   SelectNumberOfMemBlocks

   DESCRIPTION:   Selects the number of mem blocks to allocate according to the following function:
                  #extraBlocksToAllocate = min(number of blocks per frag , number of frags)

   PARAMETERS:   UINT16 dataLength - the length of the data in the frame
                 UINT16 memBlkSize - the size of a memory blocks in bytes
                 UINT8  mpduHeaderLength - the header length of the original frame
 
   RETURNS:      The total number of memory blocks to allocate (including the frag overhead)
***********************************************************************************************************************/
UINT8 SelectNumberOfMemBlocks(UINT16 dataLength,UINT16 memBlkSize, UINT8 mpduHeaderLength, UINT8 encryptionType);


/**********************************************************************************************************************
   NumMemBlocksTxAllocation

   DESCRIPTION: Utility function: finds the correct fragmentation threshold and the number of memory blocks to allocate   

   PARAMETERS: UINT16 originalFragmentThreshold - the user specified fragmentation threshold
               UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
               UINT32 txRate                    - the rate field as it is put in the TxDesc
               UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise
               UINT8  unicastFrame              - "0" if the frame is Bcast / Multicast "1" if unicast
               UINT8  driverFragmentationOn        - bit_0 of the dataFlowOptions IE specifies whether driver encryption is used
               UINT8  txCts                     - bit_0 of the ctrl2 field "1" if there is CTS protection on the frame
               UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame
               UINT16 dataLength                - the length of the data in the frame
               UINT16 memBlkSize                - the size of a memory blocks in bytes
               UINT8  mpduHeaderLength          - the header length of the original frame
               UINT8  tx4X                      - "1" if the frame is sent with 4X on "0" otherwise  
               CipherSuite_e encryptionType     - specifies the type of encryption for the frame
                                                  valid values:
                                                      CIPHER_NONE = 0,
                                                      CIPHER_WEP  = 1,
                                                      CIPHER_TKIP = 2,
                                                      CIPHER_AES  = 3

                UINT8* fragmentedFrame           - return value whether or not the frame will be fragmented
                UINT8* totalNumBlocksToAllocate  - return value number of blocks to allocate
                UINT16* finalFragThreshold       - return value the frag threshold used for the calculatio

 
   RETURNS:      NOTHING

***********************************************************************************************************************/
void NumMemBlocksTxAllocation(UINT8* fragmentedFrame, UINT8* totalNumBlocksToAllocate, UINT16* finalFragThreshold, FragmentationDecisionData_t* fragDecisionData);


uint16 TxMemCalcGetCtsDuration  ( uint16 rateField, uint16 controlRateIndex );
uint16 TxMemCalcComputeDuration ( uint16 length,    uint16 rateField,       uint32 noAck );


#endif //TX_MEMORY_CALCULATION_H
