#ifndef __TI_TYPES_H__
#define __TI_TYPES_H__

#ifdef VXWORKS
  #include "vxWorks.h"
  #define  PHYSADDR(x) ((x) & 0x1FFFFFFF)
#else
  #ifndef __KERNEL__
    #include <stdint.h>
  #else
    #include <linux/types.h>
  #endif
#endif /* VXWORKS */ 

#ifndef K1BASE
#define K1BASE KSEG1
#endif

#ifndef PHYS_ADDR
#define PHYS_ADDR(x) PHYSADDR(x)
#endif

#ifndef PHYS_TO_K1
#define PHYS_TO_K1(X) (PHYSADDR(X) | K1BASE)
#endif

#ifndef PHYS_TO_K0
#define PHYS_TO_K0(X) (PHYSADDR(X) | K0BASE)
#endif


#define IMPORT extern
#define FAST   register

#define TRUE 1
#define FALSE 0

#define OFF    0
#define ON     1



#define NOK    1
#define OK	   0
#ifndef ERROR
  #define ERROR -1
#endif



#ifdef VXWORKS
typedef UINT8  __u8;
typedef UINT16 __u16;
typedef UINT32 __u32;
typedef signed char __s8;
typedef INT16  __s16;
typedef INT32  __s32;
typedef UINT8  u8;
typedef UINT16 u16;
typedef UINT32 u32;
typedef char   Char; 
typedef UINT8  Uint8;
typedef UINT16 Uint16;
typedef UINT32 Uint32;
typedef long long u64;
#endif

#ifndef uint32
#define uint32 UINT32
#endif

#ifndef uint16
#define uint16 UINT16
#endif

#ifndef uint8
#define uint8 UINT8
#endif

#define MAC_ADDR_LEN    6   /* MAC Address length in bytes  */
typedef unsigned char ether_mac_t[MAC_ADDR_LEN];

#define UST1(a,x)      (*( (unsigned char *) ( ((unsigned long)(a)) + ((int)(x)) ) ))
#define UST2(a,x)      (*( (unsigned short*) ( ((unsigned long)(a)) + ((int)(x)) ) ))
#define UST4(a,x)      (*( (unsigned long *) ( ((unsigned long)(a)) + ((int)(x)) ) ))
#define XORUST1(a,b,x) ( UST1((a),(x)) ^ UST1((b),(x)))
#define XORUST2(a,b,x) ( UST2((a),(x)) ^ UST2((b),(x)))
#define XORUST4(a,b,x) ( UST4((a),(x)) ^ UST4((b),(x)))
#define MACCMP(a,b)    ( MACNCMP((a),(b)) == 0)


#ifndef VXWORKS
/* Type safe MAC comparison macros, assuming MAC are at least 2 bytes aligned */
#define MACCMPNZ(a)    ( UST2((a),0) | UST2((a),2) | UST2((a),4))
#define MACCMPZ(a)     ( MACCMPNZ(a) == 0)
#define MACCMPF(a)     (( UST2((a),0) & UST2((a),2) & UST2((a),4)) == 0xffff)
#define MACNCMP(a,b)   ( XORUST2((a),(b),0) | XORUST2((a),(b),2) | XORUST2((a),(b),4))
#define MACDUP2(dst,src) { \
   UST2((dst),0) = UST2((src),0); \
   UST2((dst),2) = UST2((src),2); \
   UST2((dst),4) = UST2((src),4); \
}
#else /* VXWORKS */	/* prevent non-align access */
/* Type safe MAC comparison macros, assuming MAC is not aligned */
#define MACCMPNZ(a)    ( UST1((a),0) | UST1((a),1) | UST1((a),2) | \
						 UST1((a),3) | UST1((a),4) | UST1((a),5))
#define MACCMPZ(a)     ( MACCMPNZ(a) == 0)
#define MACCMPF(a)     (( UST1((a),0) & UST1((a),1) & UST1((a),2) & \
						  UST1((a),3) & UST1((a),4) & UST1((a),5)) == 0xff)

#define MACNCMP(a,b)   (XORUST1((a),(b),0) | XORUST1((a),(b),1) | XORUST1((a),(b),2) | \
						XORUST1((a),(b),3) | XORUST1((a),(b),4) | XORUST1((a),(b),5))
#define MACDUP2(dst,src) { \
   UST1((dst),0) = UST1((src),0); \
   UST1((dst),1) = UST1((src),1); \
   UST1((dst),2) = UST1((src),2); \
   UST1((dst),3) = UST1((src),3); \
   UST1((dst),4) = UST1((src),4); \
   UST1((dst),5) = UST1((src),5); \
}
#endif /* VXWORKS */

#ifdef  __cplusplus
typedef void            (*VOIDFUNCPTR) (...); 
typedef double          (*DBLFUNCPTR) (...);  
typedef float           (*FLTFUNCPTR) (...);  
#else
typedef void            (*VOIDFUNCPTR) (void);
typedef double          (*DBLFUNCPTR) (void); 
typedef float           (*FLTFUNCPTR) (void); 
#endif

#endif // __TI_TYPES_H__
