/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda.h                                                
 *   \brief PURPOSE: headers and definitions for DDA
 *                                                                         
 */
 
#ifndef __TIWLAN_DDA_H
#define __TIWLAN_DDA_H

#include <linux/module.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/mm.h>
#include <linux/slab.h>

#if     defined(CONFIG_MIPS_WIFI_1130_LED)
#define WLAN_WIFI_1130_LED
#undef  WLAN_WIFI_PSP_LED
#undef  WLAN_WIFI_PSP_COLORED_LED
#elif   defined(CONFIG_MIPS_AVALANCHE_LED)
#undef  WLAN_WIFI_1130_LED
#define WLAN_WIFI_PSP_LED
#undef  WLAN_WIFI_PSP_COLORED_LED
#elif   defined(CONFIG_MIPS_AVALANCHE_COLORED_LED)
#undef  WLAN_WIFI_1130_LED
#undef  WLAN_WIFI_PSP_LED
#define WLAN_WIFI_PSP_COLORED_LED
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#include <linux/tqueue.h>
#else
#include <linux/workqueue.h>
#endif
#include <linux/if.h>
#include <asm-mips/addrspace.h>
#ifdef WLAN_WIFI_PSP_COLORED_LED
#include <asm/avalanche/generic/led_drv_kernel.h>
#include <asm/avalanche/generic/led_drv_user.h>
#endif

#include "whal.h"
#include "drvDbDefs.h"
#include "memMngr.h"
#include "whalAcxRegs.h"


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif

#ifndef vbase1
# define vbase1 TNETD53XX_PUMA_VLBASE_1
#else
# error "name clashing, vbase1 is defined"
#endif

/* we want dynamic name */
/*TODO: pass the dynamic name to the user*/
#define DDA_DEVNAME	"wlan%d"

#define DDA_DRV_RECEIVED_GO (1)
#define DDA_DRV_STARTED	(1<<2)

struct dda_priv {
	struct net_device_stats stats;
	__u32 wpa_authenticator_pid, driver_started;
	struct sock * user_nl;
#ifdef  TIWLAN_FW_TRACE
	__u32 fw_trace_app_pid;
#endif
    __u32 wds_manager_pid;
    #ifdef NSP_WAN_BRIDGE
		void *pWanBridgeHandle;
    #endif /* ifdef NSP_WAN_BRIDGE */
};

struct wds_priv {
	struct net_device_stats stats;
    char linkPartner[ETH_ALEN];
};


/* wireless extensions staff */
#define DDA_IOCTL_PARAM (SIOCIWFIRSTPRIV + 0)
#define DDA_IOCTL_GET_PARAM (SIOCIWFIRSTPRIV + 1)








typedef enum {
    TIWLAN_ALLOC_INIT_PHASE,
    TIWLAN_ALLOC_RUNTIME_PHASE
} tiwlan_allocPhase_t;



void tiwlan_ddaDeleteMacAddrFunc(char *mac);
UINT8 tiwlan_ddaRand(void);
void tiwlan_ddaSrand(UINT16 seed);
STATUS 	tiwlan_ddaIsMemMngrInitNeeded(void);
STATUS tiwlan_ddaReceive(mem_MSDU_T *);

#define TIWLAN_DDA_BUFFER_FROM_TOKEN(token) (((struct sk_buff *)token)->data)
STATUS tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, UINT8 ** buf, tiwlan_allocPhase_t phase);
STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size );

typedef enum {
	TIWLAN_NETLINLK_DESTINATION_WPA,
	TIWLAN_NETLINK_DESTINATION_FW_TRACE,
    TIWLAN_NETLINK_DESTINATION_WDS_MNGR
} tiwlan_ddaNetlinkDestination_t;

STATUS tiwlan_ddaNetLinkSend(void * data, int data_len, tiwlan_ddaNetlinkDestination_t dst);

STATUS tiwlan_ddaCreateNetlinkSocket(void);

STATUS tiwlan_ddaGo(initConfigStruct *init_cfg);

STATUS tiwlan_dda_wpaGo(void);

STATUS tiwlan_dda_waitForWpa(void);

STATUS tiwlan_ddaInitInterrupts(void);

STATUS tiwlan_ddaSuspend(void);
STATUS tiwlan_ddaResume(void);

STATUS tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr);
STATUS tiwlan_ddaUnloadPSPVlynq(void);
STATUS tiwlan_ddaReInit(void);
STATUS tiwlan_ddaRecvWdsPkt(UINT8 *pBuff, UINT32 length);
STATUS tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr); 

int tiwlan_setupWdsNetDevs(int wdsDlDevsNumber);

#define tiwlan_ddaRxFreeBuffer(x) \
do{ \
	if (!(((struct sk_buff *)x)->list)) dev_kfree_skb((struct sk_buff *)x);\
	else printk("kfree_skb warning: the skb is on the list !!!\n");\
} while (0);

#define boardInitAcx111Addr(RegAddr, MemAddr) tiwlan_ddaInitPSPVlynq(RegAddr, MemAddr)
#define boardUnloadAcx111() tiwlan_ddaUnloadPSPVlynq()

STATUS wdrv_DDA_queue_stop(void);
STATUS wdrv_DDA_queue_start(void);

#ifdef TIWLAN_DEBUG
void inline bm_trace(int loc, unsigned long p1, unsigned long p2);
#else
#define bm_init()
#define bm_trace(a, b, c) 
#define bm_shutdown()
#define bm_disable()
#endif




#ifdef DDA_WLAN_LED

#ifdef WLAN_WIFI_PSP_COLORED_LED
#define TIAP_LED_OFF		LED_MODE_OFF
#define TIAP_LED_ON		    LED_MODE_ON
#define TIAP_LED_BLINK_OFF	LED_MODE_ONE_SHOT_OFF
#define TIAP_LED_BLINK_ON	LED_MODE_ONE_SHOT_ON
#define TIAP_LED_FLASH		LED_MODE_BLINK 
#else
#define TIAP_LED_OFF		0
#define TIAP_LED_ON		1
#define TIAP_LED_BLINK_OFF	2
#define TIAP_LED_BLINK_ON	3
#define TIAP_LED_FLASH		4
#endif

extern UINT8 wdrv_dbGetApBridgeMode(void);
#if defined(WLAN_WIFI_1130_LED)

extern atomic_t tiap_led_oper_state, tiap_led_blinked;
extern UINT16 tiap_activity_led_mask;

extern __u8 tiap_gpio_owner;
extern void whal_apiMacWrite(UINT32 MacRegAddr, UINT32 DataVal);

__inline__ static void tiwlan_ddaLedAction(int action)
{
	/* It mustn't be a function call in an inline function like this one.
	 * Performance is crucial.
	 */
	if (wdrv_dbGetApBridgeMode() || !tiap_gpio_owner)
		return;

	switch(action)
	{
		case TIAP_LED_OFF:
			whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
			atomic_set(&tiap_led_oper_state, action);
			break;
		case TIAP_LED_ON:
			whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
			atomic_set(&tiap_led_oper_state, action);
			break;
		case TIAP_LED_BLINK_OFF:
		case TIAP_LED_BLINK_ON:
			atomic_set(&tiap_led_blinked, 1);
			break;
	}
}

#define PAL_ConfigVWiLeds whal_acxConfigVWiLeds

#elif defined(WLAN_WIFI_PSP_LED)
extern void *wlan_led;
#define PAL_ConfigVWiLeds()
#define tiwlan_ddaLedAction(action)	\
	if (!wdrv_dbGetApBridgeMode())	\
		avalanche_led_action(wlan_led, action)

#elif defined(WLAN_WIFI_PSP_COLORED_LED)
extern struct led_action led_action;
#define PAL_ConfigVWiLeds()
#define tiwlan_ddaLedAction(action)	\
    led_action.mode = action; \
	if (!wdrv_dbGetApBridgeMode())	\
		led_hal_action(&led_action)

#endif
#endif /* DDA_WLAN_LED */

#define XXstr(x) (x)

#endif //__TIWLAN_DDA_H


