/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda_contexts.c                                                
 *   \brief PURPOSE: implementation of DDA event scheduling
 *                                                                         
 */
/***********************************************************
	 includes.
*******************************************************/
#include "wlan_pal.h"
#include <linux/config.h>
#include <linux/version.h>
#include <linux/list.h>
#include <linux/tqueue.h>
#include <linux/interrupt.h>

#include <memMngr.h>
#include "whal.h"
#include "802_11Defs.h"
#include "QoS/QoSClassifier.h"
#include "QoS/QoSACParams.h"
#include "dda.h"
#include "dda_contexts.h"
#include "pform_types.h"


/***********************************************************
	TODO:
	 1. case preamble tasklet we will need to change loop
		logic in process_event tasklet func,couse
		we cant free elements inside the loop !!!(maybe anther loop)
		maybe we will make define handle for preemble tasklet ? 
     2. why list_entry MACRO wont compile ?
	 3. add include to STATUS ,OK,NOK defs.
	 4. call wdr functoin and erase tmp section !
*******************************************************/



/***********************************************************
	defs
*******************************************************/

#define FREE_LIST_HEAD_IMMIDATE

/***********************************************************
	static variables
*******************************************************/



/* tasklett structs definitions*/
struct tasklet_struct  main_system_tasklet; 
struct tasklet_struct  mbox_tasklet;

/*! \brief System tasklet lists struct */
typedef struct
{
	struct list_head mgmt_list_head;
	struct list_head rx_list_head;
	struct list_head tx_list_head;
	struct list_head main_list_head;
} tiap_dda_tasklets_lists_T;

static tiap_dda_tasklets_lists_T tasklets_lists_head;
static struct list_head 	mb_list_head;

/*! \brief events data struct */
typedef	struct {
	unsigned long   type;
	union{
		  mgmtMessage_T    mgmt;
		  mainsmMessage_T  mainsm;
		  RxMessage_T      rx;
		  TxMessage_T	   tx;
	}data;
}event_data;

/*! \brief pool element struct*/
typedef struct {
	struct   list_head  list;
	event_data event;
}list_ele;

static list_ele	  **pool_list;
static unsigned long list_index;
static unsigned long list_size;

static struct tq_struct  restart_tq_task;

/***********************************************************
	static functions
*******************************************************/
static inline void mgmt_process_event(struct list_head*  head, int *event_count);
static inline void rx_process_event(struct list_head*  head, int *event_count);
static inline void tx_process_event(struct list_head*  head, int *event_count);
static inline void main_sm_process_event(struct list_head*  head, int *event_count);
static inline void mailbox_process_event(struct list_head*  head, int *event_count);
static inline void main_sm_process_event(struct list_head*  head, int *event_count);
static void main_system_process_event(unsigned long  data);
static void mbox_tasklet_func(unsigned long data);
static inline list_ele* tasklet_get_list_ele(void);
static inline void tasklet_ret_list_ele(list_ele *ele);

/*******************************************************************************************/
/*!
	\def    set_event(event_type, ele_head, message, len).
	\brief	add event to event list (ele_head).
			copy data to list element event data and update.
			event type field.
*/
/*******************************************************************************************/
#define set_event(event_type, ele_head, message, len) \
do { \
  unsigned int flags; \
  list_ele *ele = NULL; \
  PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  \
  if( (ele = tasklet_get_list_ele()) ) \
	{ \
	  ele->event.type = event_type;    \
		memcpy((void *)&ele->event.data, (void *)message, len);\
		list_add_tail(&ele->list, ele_head); \
	} else \
	  printk("\n\n\n no element in the free entries pool !!!!!!! \n"); \
  PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  \
} while(0)

/*******************************************************************************************/
/*!
	\def 	set_event_nocopy
	\brief  add event to events list(ele_head)
			update event type field.
*/
/*******************************************************************************************/
#define set_event_nocopy(event_type, ele_head) \
do { \
        unsigned int flags; \
        list_ele *ele = NULL; \
        PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  \
        if ( !(ele = tasklet_get_list_ele()) ) { \
          PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  \
          return NOK; \
        } \
        ele->event.type = event_type; \
        list_add_tail(&(ele->list), ele_head); \
        PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  \
} while(0)

/*******************************************************************************************/
/*!
	\brief Initialize free events entries pool
                                                                         
	\param pool_size   size of the pool

	\return  OK/NOK   		                                                    
*/
/*******************************************************************************************/
STATUS tasklet_init_pool(unsigned long pool_size)
{
	int i;

	/* malloc list*/
	if((pool_list = kmalloc(sizeof(list_ele*) * pool_size, GFP_KERNEL)) == NULL)
	{
		printk("\n\rerror malloc tasklet buffer pool list");
		return NOK;
	}

	/* it can be done staticly if we want*/
	for(i = 0;i < pool_size ; i++ )
	{
		if((pool_list[i] = kmalloc(sizeof(list_ele),GFP_KERNEL)) == NULL)
		{
			printk("\n\rerror malloc tasklet buffer pool list element %d",i);
			return NOK;
		}
	}
	list_index = 0;
	list_size = pool_size;

	return OK;
}

#define MAIN_SYS_TASKLET_MAX_COUNT 6

/*******************************************************************************************/
/*!
	\brief Process all main system tasklet events
	
	\param data  A pointer to tasklet lists structure
*/
/*******************************************************************************************/
static inline void main_system_process_event(unsigned long data)
{
    tiap_dda_tasklets_lists_T *head;
	int 				 	   event_count;
	int 					   maxLoopCount = MAIN_SYS_TASKLET_MAX_COUNT;

	head = (tiap_dda_tasklets_lists_T *)data;

	while(--maxLoopCount)
	{
		event_count = 0;
		rx_process_event(&head->rx_list_head, &event_count);
		tx_process_event(&head->tx_list_head, &event_count);
		mgmt_process_event(&head->mgmt_list_head, &event_count);
		main_sm_process_event(&head->main_list_head, &event_count);

		if(event_count == 0)
			break;
	}
}

#define MBOX_TASKLET_MAX_COUNT 10
/*******************************************************************************************/
/*!
	\brief Process mailbox tasklet events
	
	\param data  A pointer to events list
*/
/*******************************************************************************************/
static void mbox_tasklet_func(unsigned long data)
{
	int event_count;
	int maxLoopCount = MAIN_SYS_TASKLET_MAX_COUNT;

	struct list_head *mb_list_head = (struct list_head *) data;

	while(--maxLoopCount)
	{	
		event_count = 0;
		mailbox_process_event(mb_list_head, &event_count);
		if(event_count == 0)
			break;
	}

}

/*******************************************************************************************/
/*!
	\brief Get a free event from the head of events list
	
	\return a pointer to the event
*/
/*******************************************************************************************/
static inline list_ele* tasklet_get_list_ele(void)
{

	list_ele *tmp;

	if(list_index != (list_size - 1))
	{
		tmp =  pool_list[list_index++];
	}
	else
	{
		printk("\n tasklet buffer pool empty");
		tmp =  NULL; 		
	}

	return tmp; 
}

/*******************************************************************************************/
/*!
	\brief Return handled event to the list of free events
*/
/*******************************************************************************************/
static inline void tasklet_ret_list_ele(list_ele *ele)
{	
	pool_list[--list_index] = ele;    
}

/*******************************************************************************************/
/*!
	\brief Initalize tasklet queues and free events pool
	
	\return OK/NOK
*/
/*******************************************************************************************/
STATUS dda_tasklet_init(unsigned long pool_size)
{
	INIT_LIST_HEAD(&tasklets_lists_head.mgmt_list_head);
	INIT_LIST_HEAD(&tasklets_lists_head.rx_list_head);
	INIT_LIST_HEAD(&tasklets_lists_head.tx_list_head);
	INIT_LIST_HEAD(&tasklets_lists_head.main_list_head);
	
	INIT_LIST_HEAD(&mb_list_head);

    tasklet_init(&main_system_tasklet,main_system_process_event,(unsigned long) &tasklets_lists_head);
	tasklet_init(&mbox_tasklet, mbox_tasklet_func, (unsigned long) &mb_list_head);

	/* init taskQ*/
	restart_queue_init();
	
#ifdef DDA_CONTENT_TEST_ONLY	
	test_init();
#endif
	return tasklet_init_pool(pool_size);
}

/*******************************************************************************************/
/*!
	\brief free events pool and kill the tasklet structure
*/
/*******************************************************************************************/
void dda_tasklet_kill(void)
{
	int i;
	tasklet_kill(&main_system_tasklet);
	tasklet_kill(&mbox_tasklet);

	/*free list elem*/
	for(i = 0;i < list_size ; i++ )
		kfree(pool_list[i]);

	/*free list*/
	kfree(pool_list);

	list_size = 0;
}

/*******************************************************************************************/
/*!
	\brief Process all  management events
	
	\param 
	
	\return OK/NOK
	
*/
/*******************************************************************************************/
STATUS wdrv_DDA_mgmt_processAllEvents(void) {
	struct list_head * head = &tasklets_lists_head.mgmt_list_head;
	int event_count = 1;
	while (event_count) {
		event_count = 0;
		mgmt_process_event(head, &event_count);
		if (event_count == 0)
			break;
	}

	return OK;
}

/*******************************************************************************************/
/*!
	\brief Process or schedule a given management event
	
	\param message 	Management event structure
	
	\return OK/NOK
	
*/
/*******************************************************************************************/
STATUS wdrv_DDA_mgmt_setEvent( mgmtMessage_T *message)
{
	switch(message->type)
	{
	case MGMT_FRAME:
	case CLASS3:
	case BEACON_FRAME:
	
		wdrv_mgmt_getEvent(message);
		break;

	case AUTH_RESPONSE_TIMEOUT:
	case AGING:
	case DEAUTH_MNG:
	case DISASSOC_MNG:
	case BEACON_ERP_TIMEOUT:
	case BEACON_PBCC_TIMEOUT:
	case DYNAMIC_ALG_TIMEOUT:
	case SET_TEMPLATE_REQ:
	case RADIO_CALIBRATE_TIMEOUT:
	case RADIO_CALIBRATE_TIMEOUT_CRT:
	case MAC_AUTHORIZE:
	case MAC_DE_AUTHORIZE:
	case WPA_ASSOC_REQ_REP:
    case WDS_LINK_MNG:
    case DEAUTH_TX_COMPLETE:
		set_event(message->type, &tasklets_lists_head.mgmt_list_head, message, sizeof(mgmtMessage_T));
		tasklet_schedule(&main_system_tasklet);
		break;
	default: 
		printk("Unknown mgmt event %ul\n", message->type);
		return NOK;
	}
	
	return OK; 	
}

/*******************************************************************************************/
/*!
	\brief Handle all events in management tasklets queue
		   Return handled events to free events pool
	
	\param head 		Management tasklets queue head
	\param event_count	a pointer to the global event count
	
*/
/*******************************************************************************************/
static inline void mgmt_process_event(struct list_head*  head, int *event_count)
{
	struct list_head *p,new_head;
	unsigned int flags;


#ifdef FREE_LIST_HEAD_IMMIDATE
	
	/* return in case there are no events in the list */
	if(head->next == head)
	{		
		return;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 

	new_head.next = head->next;
	new_head.prev = head->prev;
	

	/*assign last element on the list next ptr to point new head */
	(new_head.prev)->next = &new_head;

	(new_head.next)->prev = &new_head;

	/* free list head*/
	INIT_LIST_HEAD(head);

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	/* run on all list entries and call DDC func to handle event */
	list_for_each(p, &new_head)
	/*for(p = new_head.next; p != &new_head; p = p->next)*/
	{
#else
	list_for_each(p, head)
	{
#endif
		list_ele *ele = (list_ele *)p;
		if(wdrv_mgmt_getEvent(&(ele->event.data.mgmt)) != OK)
			printk("\n\rerror handle event %lu, elem_ptr=%p",ele->event.type, ele);
		(*event_count)++;
	}
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	list_for_each(p, &new_head)
	{
		tasklet_ret_list_ele((list_ele*)p);
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/*******************************************************************************************/
/*!
	\brief Schedule a given main SM event
	
	\param message 	Main SM event structure
	
	\return OK
*/
/*******************************************************************************************/
STATUS wdrv_DDA_mainSm_setEvent(mainsmMessage_T *message)
{
	set_event(message->type, &tasklets_lists_head.main_list_head, message,sizeof(mainsmMessage_T));
	tasklet_schedule(&main_system_tasklet);
	return OK; 
}

/*******************************************************************************************/
/*!
	\brief Handle all events in Main SM tasklets queue
		   Return handled events to free events pool
	
	\param head 		Main SM tasklets queue head
	\param event_count	a pointer to the global event count
	
*/
/*******************************************************************************************/
static inline void main_sm_process_event(struct list_head*  head, int *event_count)
{
	struct list_head *p,new_head;
	unsigned int flags;

		
#ifdef FREE_LIST_HEAD_IMMIDATE
    /* return in case there are no events in the list */
	if(head->next == head)
	{
		return;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 

	new_head.next = head->next;
	new_head.prev = head->prev;
	

	/*assign last element on the list next ptr to point new head */
	(new_head.prev)->next = &new_head;

	(new_head.next)->prev = &new_head;

	/* free list head*/
	INIT_LIST_HEAD(head);

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	/* run on all list entries and call DDC func to handle event */
	/*list_for_each(p, &new_head)*/
	for(p = new_head.next; p != &new_head; p = p->next)
	{
#else

	list_for_each(p, head)
	{
#endif
		list_ele * ele = (list_ele*)p;	

		if(wdrv_mainSm_getEvent(&(ele->event.data.mainsm)) != OK)
			printk("\n\rerror handle event %lu, elem_ptr=%p",ele->event.type, ele);
		(*event_count)++;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	list_for_each(p, &new_head)
	{
		tasklet_ret_list_ele((list_ele*)p);
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/*! \brief Count of scheduled RX events waiting in the queue 
*/
static int rx_sched_event_count=0;

/*******************************************************************************************/
/*! \brief Sets the count of scheduled RX events waiting in the queue to zero
*/
/*******************************************************************************************/
void inline wdrv_DDA_init_rx_sched_count(void)  {rx_sched_event_count=0;}

/*******************************************************************************************/
/*!
	\brief Schedule a given RX event
	
	\param message 	RX event structure
	
	\return OK
	
*/
/*******************************************************************************************/
STATUS wdrv_DDA_rx_setEvent(RxMessage_T *message)
{
	/* if there is already RX event scheduled in the queue there is no need to set more
		 since all RX events will be handled together */
	if(rx_sched_event_count > 0)
		return OK;
	rx_sched_event_count++;

	set_event(message->type,&tasklets_lists_head.rx_list_head, message,sizeof(RxMessage_T));
	tasklet_schedule(&main_system_tasklet);
	return OK; 
}

/*******************************************************************************************/
/*!
	\brief Handle all events in RX tasklets queue
		   Return handled events to free events pool
	
	\param head 		RX tasklets queue head
	\param event_count	a pointer to the global event count
	
*/
/*******************************************************************************************/
static inline void rx_process_event(struct list_head*  head, int *event_count)
{

	struct list_head *p,new_head;
	unsigned int flags;
		

#ifdef FREE_LIST_HEAD_IMMIDATE
	/* return in case there are no events in the list */
	if(head->next == head)
	{
		return;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 

	new_head.next = head->next;
	new_head.prev = head->prev;
	

	/*assign last element on the list next ptr to point new head */
	(new_head.prev)->next = &new_head;

	(new_head.next)->prev = &new_head;

	/* free list head*/
	INIT_LIST_HEAD(head);

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	/* run on all list entries and call DDC func to handle event */
	list_for_each(p, &new_head)
	/*for(p = new_head.next; p != &new_head; p = p->next)*/
	{
#else

	list_for_each(p, head)
	{
#endif
		list_ele * ele = (list_ele*)p;

		if(wdrv_rx_getEvent(&(ele->event.data.rx)) != OK)
			printk("\n\rerror handle event %lu, elem_ptr=%p, head=%p, new_head=%p",(unsigned long)ele->event.type, ele, head, (void *)&new_head);
		(*event_count)++;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	list_for_each(p, &new_head)
	{
		tasklet_ret_list_ele((list_ele*)p);
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/*! \brief Count of scheduled TX events waiting in the queue
*/
static int tx_sched_event_count=0;

/*******************************************************************************************/
/*! \brief Sets the count of scheduled TX events waiting in the queue to zero
*/
/*******************************************************************************************/
void inline wdrv_DDA_init_tx_sched_count(void)  {tx_sched_event_count=0;}

/*******************************************************************************************/
/*!
	\brief Process or schedule a given TX event
	
	\param message 	TX event structure
	
	\return OK
*/
/*******************************************************************************************/
STATUS wdrv_DDA_tx_setEvent(TxMessage_T *message)
{
	switch(message->type)
	{
	case WDRV_TX_MSDU_SEND:
	case WDRV_TX_COMPLETE:
	case WDRV_TX_MSDU_PS_POLL:
	case WDRV_TX_PS_AGING:
	case WDRV_TX_MSDU_PS_STATUS_CHANGE:
	case WDRV_TX_MSDU_PS_CLEAN_STATION_LIST:
		wdrv_tx_getEvent(message);
		break;
	
	case WDRV_TX_MSDU_TIMEOUT:
		 /* if there is already TX event scheduled in the queue there is no need to set more
		 since all TX events will be handled together */
		if(tx_sched_event_count > 0) 
			break;
		tx_sched_event_count++;
		set_event(message->type,&tasklets_lists_head.tx_list_head, message,sizeof(TxMessage_T));
		tasklet_schedule(&main_system_tasklet);
		break;
	
	default: 
		printk("Unknown tx event %ul\n", message->type);
		return NOK;

	}

	return OK;
}

/*******************************************************************************************/
/*!
	\brief Handle all events in TX tasklets queue
		   Return handled events to free events pool
	
	\param head 		TX tasklets queue head
	\param event_count	a pointer to the global event count
	
*/
/*******************************************************************************************/
static inline void tx_process_event(struct list_head*  head, int *event_count)
{
	struct list_head *p,new_head;
	unsigned int flags;


#ifdef FREE_LIST_HEAD_IMMIDATE
	/* return in case there are no events in the list */
	if(head->next == head)
	{
		return;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 

	new_head.next = head->next;
	new_head.prev = head->prev;
	

	/*assign last element on the list next ptr to point new head */
	(new_head.prev)->next = &new_head;

	(new_head.next)->prev = &new_head;

	/* free list head*/
	INIT_LIST_HEAD(head);

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	/* run on all list entries and call DDC func to handle event */
	list_for_each(p, &new_head)
	{
#else

	list_for_each(p, head)
	{
#endif
		list_ele * ele = (list_ele*)p;	

  		if(wdrv_tx_getEvent(&(ele->event.data.tx)) != OK)
			printk("\n\rerror handle event %lu, elem_ptr=%p\n",(unsigned long)ele->event.type,ele);
		(*event_count)++;
	}
	
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	list_for_each(p, &new_head)
	{
		tasklet_ret_list_ele((list_ele*)p);
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/*******************************************************************************************/
/*!
	\brief Schedule a given Mailbox event
	
	\param event_type tasklet events queue type	
	
	\return OK
*/
/*******************************************************************************************/

STATUS wdrv_DDA_mailbox_setEvent(unsigned long  event_type)
{
	set_event_nocopy(event_type,&mb_list_head);
	tasklet_schedule(&mbox_tasklet);
	return OK; 
}

/*******************************************************************************************/
/*!
	\brief Handle all events in Mailbox tasklets queue
		   Return handled events to free events pool
	
	\param head 		Mailbox tasklets queue head
	\param event_count	a pointer to the global event count
*/
/*******************************************************************************************/
static inline void mailbox_process_event(struct list_head*  head, int *event_count)
{
	struct list_head *p,new_head;
	unsigned int flags;


#ifdef FREE_LIST_HEAD_IMMIDATE
	/* return in case there are no events in the list */
	if(head->next == head)
	{
		return;
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 

	new_head.next = head->next;
	new_head.prev = head->prev;
	

	/*assign last element on the list next ptr to point new head */
	(new_head.prev)->next = &new_head;

	(new_head.next)->prev = &new_head;

	/* free list head*/
	INIT_LIST_HEAD(head);

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	/* run on all list entries and call DDC func to handle event */
	list_for_each(p, &new_head)
	{
#else

	list_for_each(p, head)
	{
#endif

		list_ele * ele = (list_ele*)p;

		if(wdrv_mailbox_getEvent(ele->event.type) != OK)
			printk("\n\rerror handle event %lu",(unsigned long)ele->event.type);
		(*event_count)++;
	}
	
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	list_for_each(p, &new_head)
	{
		tasklet_ret_list_ele((list_ele*)p);
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}


/******************************  task Q *********************************/

/*******************************************************************************************/
/*!
	\brief Schedule restart event 
	
	\return OK
	
*/
/*******************************************************************************************/
STATUS wdrv_DDA_restart_setEvent(void)
{
	printk("wdrv_DDA_restart_setEvent ..........\n");
	schedule_task(&restart_tq_task);
	return OK;
}

/*******************************************************************************************/
/*! \brief Activate restart
*/
/*******************************************************************************************/
void restart_proccess_event(void)
{
	wdrv_ActivateRestart();	
}

/*******************************************************************************************/
/*!
	\brief A wrapper for linux INIT_TQUEUE macro
*/
/*******************************************************************************************/
void restart_queue_init(void)
{
	INIT_TQUEUE(&restart_tq_task,(void (*)(void*))restart_proccess_event,0);
}

/*******************************************************************************************/
/*!
	\brief A wrapper for linux wake_up_interruptible function
	
	\param wait_q 	waiting queue head
*/
/*******************************************************************************************/
void   wdrv_DDA_wake(void *wait_q)
{
	wake_up_interruptible((wait_queue_head_t *)wait_q);
}

/*******************************************************************************************/
/*!
	\brief A wrapper for linux interruptible_sleep_on_timeout function
	
	\param wait_q 	waiting queue head
	\param timeout	timeout
	
	\return timeout
	
*/
/*******************************************************************************************/
long   wdrv_DDA_sleep_timeout(void *wait_q,long timeout)
{
/*
	DEBUG:
	long timeout = interruptible_sleep_on_timeout((wait_queue_t*)wait_h, msecs_to_jiffies(timeout));

	//printk("\n%s: timeout = %d, %#x\n", __func__, timeout, (unsigned long)wait_h);

	return timeout;
*/
	
	return interruptible_sleep_on_timeout((wait_queue_head_t *)wait_q, msecs_to_jiffies(timeout));
}





























/******************************  test func *********************************/



#ifdef DDA_CONTENT_TEST_ONLY


#include <linux/timer.h>

static struct timer_list test_timer1,test_timer2 ;

static int tmpcount1 = 0;
void test_rx_tasklet(void)
{
	wdrv_DDA_rx_setEvent(10);
	if(	tmpcount1++ > 10)
		return;

	test_timer1.expires = jiffies + HZ; 
	test_timer1.data = 0; 
	test_timer1.function = test_rx_tasklet; 
	add_timer(&test_timer1);	
}

static int tmpcount2 = 0;
void test_task_q(void)
{
	dda_restart_setEvent();
	if(	tmpcount2++ > 10)
		return;

	test_timer1.expires = jiffies + HZ; 
	test_timer1.data = 0; 
	test_timer1.function = test_task_q; 
	add_timer(&test_timer1);	

}

void test_init(void)
{
	printk("\nim in test init");
	init_timer(&test_timer1);

	test_timer1.expires = jiffies + HZ; 
	test_timer1.data = 0; 
	test_timer1.function = test_rx_tasklet; 
/*	add_timer(&test_timer1); */

	init_timer(&test_timer2);
	test_timer2.expires = jiffies + HZ; 
	test_timer2.data = 0; 
	test_timer2.function = test_task_q; 
	add_timer(&test_timer2); 
}


#endif /* DDA_CONTENT_TEST_ONLY*/
