/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda_main.c                                                
 *   \brief PURPOSE: implementation of DDA interface
 *                                                                         
 */
#include "wlan_pal.h"
#include <linux/config.h>
#include <linux/version.h>
#include <linux/skbuff.h>
#include <linux/mm.h>
#include <linux/wireless.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <asm/uaccess.h>
#include <net/sock.h>


#ifdef PSP7
  #include <asm/avalanche/generic/pal.h>
  #include <asm/avalanche/generic/pal_vlynqDev.h>
  #define DDA_RESET_CTRL(pin, mode)   PAL_sysResetCtrl((pin),(mode))
#else /* NOT PSP7 */
#include <asm/avalanche/generic/avalanche_intc.h>
#include <asm/avalanche/generic/avalanche_misc.h>
#include <asm/avalanche/generic/vlynq_hal.h>
#endif   /* PSP7 */

#ifdef DDA_WLAN_LED
#if defined(WLAN_WIFI_PSP_LED)
#include <asm/avalanche/generic/led_hal.h>
#endif
#endif

#ifdef NSP_WAN_BRIDGE
#include <net/wb.h>
#endif /* ifdef NSP_WAN_BRIDGE */


#include <linux/random.h>
#include <whalAcxProcess.h>


#include "whal.h"
#include "802_11Defs.h"
#include "QoS/QoSClassifier.h"
#include "QoS/QoSACParams.h"
#include "dda_wpa_interface.h"
#include "dda.h"
#include "dda_proc.h"
#include "dda_contexts.h"
#include "mainsm.h"    
#include "memMngr.h"
#include "whalInterrupt.h"
#include "dda_wpaauth_ioctl.h"
#include "bssbridge.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "whalApi.h"
#include "tx.h"
#include "whalAcxRegs.h"
#include "acx_hw.h"
#include "dda_wdsDrv.h"
#include "Shutdown.h"
#include "wds.h"
#include "wdsDrv.h"
#include "dda_wpa.h"
#include "dda_wext_ioctl.h"
#include "mgmtSsidHandler.h"

/****************************************
 * Interrupt vector registration
 ****************************************/

#if( WLAN_ADAPTOR_VERSION == 1150)
   #ifdef PSP7
      #define WLAN_MAP_VEC    0
   #else  /* PSP6_1 */
      #define WLAN_MAP_VEC    7
   #endif /* PSP7 */
   #define WLAN_VEC_NAME   "TNETW1150"
#else    /* WLAN_ADAPTOR_VERSION  Not 1150 */
   #define WLAN_MAP_VEC    0
   #define WLAN_VEC_NAME   "TNETW1130"
#endif   /* WLAN_ADAPTOR_VERSION */

/*
 * For VLYNQ0, PSP maps interrupt vector to system interrupt 80 + map_vector.
 * For VLYNQ1, PSP maps interrupt vector to system interrupt 112 + map_vector.
 */
#define WLAN_IVEC    (80 + (WLAN_MAP_VEC))

/******************************
 * Module version information
 ******************************/
MODULE_AUTHOR("Texas Instruments");
MODULE_DESCRIPTION("TI WLAN AP driver, version 5.6"); 

#define WAIT_FOR_WPA_MAX_RETRY    10
#define DDA_RX_SKIP_BLINK 16

 
extern const struct iw_handler_def dda_iw_handler_def;
/*!
	\brief Single Global variable. We need DDA context of some kind to get rid of it.
*/
struct net_device *wlan_dev = NULL;
signed int pal_replev;

#if REPORT_DEBUG
	/*! \brief current number of characters logged in kernel message buffer */
   int logged_chars = 0; 
#endif
UINT16 configTxopLimit;

/* Local prototypes, defined here for convinience */
static int tiwlan_ddaInit(struct net_device *dev);
static int tiwlan_ddaSend(struct sk_buff * skb, struct net_device * dev);
static void tiwlan_ddaTxTimeoutHandler(struct net_device * dev);
static int tiwlan_ddaOpen(struct net_device *dev);
static int tiwlan_ddaStop(struct net_device *dev);
static int tiwlan_ddaSetMacAddress(struct net_device *, void *);
struct net_device_stats * tiwlan_ddaGetStats(struct net_device * dev);
static void tiwlan_ddaNetlinkRecv(struct sock *sk, int len) ;
static STATUS initTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr, int reset); 

/*! \brief list of timers which used for stopping/resuming timers upon recovery */
LIST_HEAD(pal_tcb_list_head);

/*! \brief list of netlink socket messages */
LIST_HEAD(netlink_list_head);
static STATUS tilwan_ddaAddMsgToNetLinkList(void *data, int data_len, int pid);
static STATUS tiwlan_ddaNetLinkTransmit(void * data, int data_len, int pid);
static void   tiwlan_ddaUnloadNetlinkList(void);

struct netlink_list {
	struct list_head list;
	void   *data;
	int    data_len;
	__u32  pid;
};

#define NETLINK_LIST_MAX_SIZE 128
static int netlinkListCurrSize;

__u16 tiap_activity_led_mask;
__u8 tiap_gpio_owner = 1; /*1 for driver, 0 for fw*/
#if defined(WLAN_WIFI_PSP_COLORED_LED)
struct led_action led_action;
struct led_object led;
#elif defined(WLAN_WIFI_PSP_LED)
void *wlan_led;
#elif defined(WLAN_WIFI_1130_LED)
#define TIAP_LED_TIMER_PERIOD 30
PALSRV_TcbObjTYPE tiap_led_timer;


atomic_t tiap_led_cur_state = ATOMIC_INIT(TIAP_LED_OFF),
	tiap_led_oper_state = ATOMIC_INIT(TIAP_LED_OFF),
	tiap_led_blinked = ATOMIC_INIT(0);

/*************************************************************************/
/*! \brief Implements a led timer
	\param data 	not used 
*/
/*************************************************************************/
void tiap_led_timer_func(__u32 data)
{
	if ((atomic_read(&tiap_led_oper_state) == TIAP_LED_OFF) || !tiap_gpio_owner)
		goto out;
		
	if (atomic_read(&tiap_led_cur_state) == TIAP_LED_OFF) {
		whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
		atomic_set(&tiap_led_cur_state, TIAP_LED_ON);
		goto out_blinked;
	}

	if (atomic_read(&tiap_led_blinked)) {
		whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
		atomic_set(&tiap_led_cur_state, TIAP_LED_OFF);
	}

out_blinked:
	atomic_set(&tiap_led_blinked, 0);
out:
	if (tiap_gpio_owner)
		PALSRV_tcbQueueAct(&tiap_led_timer, TIAP_LED_TIMER_PERIOD, tiap_led_timer_func, 0, 0);
}
#endif	

/*************************************************************************/
/*! \brief Copies from an input buffer ACX image
	\param buf contains the FW image 
	\param len buffer length
	
	\return OK/NOK
*/
/*************************************************************************/
static int tiwlan_ddaPutFwImage(unsigned char * buf, int len) {

	if (! Acx100_bin_0_Array ) {
		Acx100_bin_0_Array = _Acx100_bin_0_ArrayStart;
		Acx100_bin_0_Array_Len =  _Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart;		
	}
	copy_from_user(Acx100_bin_0_Array, buf, _Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart);
	Acx100_bin_0_Array_Extra_Len = len - (_Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart);
	if (Acx100_bin_0_Array_Extra_Len) {
		if (Acx100_bin_0_Array_Extra)
			kfree(Acx100_bin_0_Array_Extra);
		Acx100_bin_0_Array_Extra = kmalloc(Acx100_bin_0_Array_Extra_Len, GFP_KERNEL);
		copy_from_user(Acx100_bin_0_Array_Extra, buf + (_Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart), Acx100_bin_0_Array_Extra_Len);
	} else {
		Acx100_bin_0_Array_Extra_Len = 0;
		Acx100_bin_0_Array_Extra = NULL;
	}
	

	return OK;
}

/*************************************************************************/
/*! \brief WLAN device ioctl handler
	\param dev ignored 
	\param rq  contains ioctl data
	\param cmd ioctl command ID (should be SIOCDEVPRIVATE)
	
	\return 0 in case of success, negative value otherwise
*/
/*************************************************************************/
static int tiwlan_ddaDoIoctl(struct net_device * dev, struct ifreq * rq, int cmd) {
	int ret = 0;
	generic_ioctl_buf_t ioctl_buf;
	char *buf = NULL;
	UINT16 idx;

	if (cmd != SIOCDEVPRIVATE) {
#ifdef NSP_WAN_BRIDGE
        switch(cmd) {
            case TIWAN_ADD_WAN_BRIDGE_IOCTL:
                ((struct dda_priv *)wlan_dev->priv)->pWanBridgeHandle = (void *)rq->ifr_ifru.ifru_data;
                goto out;
                break;
            case TIWAN_DEL_WAN_BRIDGE_IOCTL:
                ((struct dda_priv *)wlan_dev->priv)->pWanBridgeHandle = NULL;
                goto out;
                break;
            default:
                break;

        }
#endif /* ifdef NSP_WAN_BRIDGE */
		printk("%s : Unknown ioctl %d\n", __FUNCTION__, cmd);
		ret = -1;
		goto out;
	}

	copy_from_user(&ioctl_buf, rq->ifr_data, sizeof(generic_ioctl_buf_t));

	if (ioctl_buf.buf && ioctl_buf.buf_len) {
		if ( ioctl_buf.cmd != TIAP_GENERIC_IOCTL_DOWNLOAD_FW &&
			 ioctl_buf.cmd != TIAP_GENERIC_IOCTL_SET_BURST_LIMIT ){  /* hack - we don't want to allocate 100K memory */
			buf = kmalloc(ioctl_buf.buf_len, GFP_KERNEL);
			if (!buf) {
				printk("%s : failed to allocate %d bytes of memory \n", __FUNCTION__, ioctl_buf.buf_len);
				return -ENOMEM;
			}
			copy_from_user(buf, ioctl_buf.buf, ioctl_buf.buf_len);
		}
	}  else {
		printk(KERN_ERR "dda: Both buf and buf_len must be valid.\n");
		return -EINVAL;
	}

	switch (ioctl_buf.cmd) {
	case TIAP_GENERIC_IOCTL_GET_TX_INFO:
		whal_apiPrintTxInfo(ioctl_buf.param1, /* Qid */ ioctl_buf.param2, /* QprintEn */ buf);
		break;
		
	case TIAP_GENERIC_IOCTL_TST_FINDSTA: 
		if (wdrv_dbFindStaIndex(ioctl_buf.mac, &idx) == OK) 
			sprintf(buf, "station found for mac %s,idex %d", ioctl_buf.mac, idx);	
		else {
			ret = -1;
			goto out;
		}
		break;
		
	case TIAP_GENERIC_IOCTL_TST_FIND_INS:
		if (wdrv_dbFindAndInsertStaIndex(ioctl_buf.mac, ioctl_buf.param1, 
                                         &idx, DRV_DB_HASH_STA_TYPE_STATION) == OK)
			sprintf(buf, "station found/created for mac %s,idex %d", ioctl_buf.mac, idx);	
		else {
			ret = -1;
			goto out;
		}
		break;
		
	case TIAP_GENERIC_IOCTL_TST_CHALLENGE:			
		wdrv_mgmtPrintChallenge(ioctl_buf.param1,/* index */buf);
		break;

	case TIAP_GENERIC_IOCTL_GET_TX_QUEUE:
		whal_apiPrintTxQueue(ioctl_buf.param1/* Qid */,buf);
		break;
		
	case TIAP_GENERIC_IOCTL_GET_EER:
		whal_apiReadEEPROM(ioctl_buf.param1/* address*/, buf);
		break;

	case TIAP_GENERIC_IOCTL_GET_MAC_REGREAD:
		whal_apiMacRead(ioctl_buf.param1/* read mac addr reg*/, buf);
		break;
	case TIAP_GENERIC_IOCTL_GET_PHY_REGREAD:
		whal_apiPhyRead(ioctl_buf.param1, /* read phy addr reg*/buf);
		break;
	case TIAP_GENERIC_IOCTL_TRACE_PRINT:
		whal_apiTracePrint(ioctl_buf.param1/*NUM */, ioctl_buf.param2/*FROM */, buf);
		break;
	case TIAP_GENERIC_IOCTL_WLAN_PARAMS:
		proc_srv_read_wlan_params(ioctl_buf.param1/* woneIndex */,buf);
		break;
	case TIAP_GENERIC_IOCTL_DOWNLOAD_FW:
		if (atomic_read(&after_go)) {
			printk("%s : Not after AP is started !\n", __FUNCTION__);
			ret = -1;
			goto out;
		}

		ret = tiwlan_ddaPutFwImage(ioctl_buf.buf, ioctl_buf.buf_len) == OK ? 0 : -1;
		if (ret == 0) {
			configured_params.fw = 1;
		} else
			printk("%s : whal_acxProcPutFwImage() failed !\n", __FUNCTION__);
		goto out;

		break;
	case TIAP_GENERIC_IOCTL_SET_BURST_LIMIT:
		configTxopLimit = ioctl_buf.param1;
		goto out;
		break;
	case TIAP_GENERIC_IOCTL_GET_BURST_LIMIT:
		wdrv_dbGetBurstLimit((UINT16 *)buf);
		break;
    case TIAP_GENERIC_IOCTL_SET_WDS_PID:
        if(wlan_dev) {
            ((struct dda_priv *)wlan_dev->priv)->wds_manager_pid = ioctl_buf.param1;
        }
        else {
            printk("%s : wlan_dev is not initialazed.\n", __FUNCTION__);
        }
        goto out;
        break;
    case TIAP_GENERIC_IOCTL_BSS_SERVICE_STATE:
        if(ioctl_buf.param1) {
            wdrv_wdsBeaconsSendEnable(TRUE);
            tiwlan_ddaLedAction(TIAP_LED_ON);
        }
        else {
            wdrv_wdsBeaconsSendEnable(FALSE);
            tiwlan_ddaLedAction(TIAP_LED_OFF);
        }
        goto out;
        break;
    case TIAP_GENERIC_IOCTL_SEND_WDS_PKT:
        tiwlan_ddaSendWdsPkt(buf, ioctl_buf.buf_len);
        goto out;
        break;
    case TIAP_GENERIC_IOCTL_AFTER_GO_FLAG_GET:
        *((UINT32 *)buf) = atomic_read(&after_go);
        break;
            
	case TIAP_GENERIC_IOCTL_SET_GPIO_OWNER:
		tiap_gpio_owner = (__u8)ioctl_buf.param1;
		goto out;

#if REPORT_DEBUG
	case TIAP_GENERIC_IOCTL_GET_LOGGED_CHARS:
		
		
		{
			unsigned int flags;
			PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
			printk("\n\nlogged_chars = %d\n\n",logged_chars);
			logged_chars = 0;
			PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
		}
		
		break;
	
	case TIAP_GENERIC_IOCTL_SET_LOGGED_CHARS:
		
		{
			unsigned int flags;
			PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
			logged_chars = ioctl_buf.param1; 
			PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
		}
		
		break;
#endif
	default:
		printk("%s : Unknown ioctl command %d\n", __FUNCTION__, ioctl_buf.cmd);
		ret = -1;
		goto out;
		break;
	}


	copy_to_user(rq->ifr_data, &ioctl_buf, sizeof(generic_ioctl_buf_t));
	if( buf )
		copy_to_user(((generic_ioctl_buf_t *)rq->ifr_data)->buf, buf, ioctl_buf.buf_len);

out:
	if (buf)
	kfree(buf);

	return ret;
}

/*! \brief A flag which is TRUE if NetLink socket is up */
BOOL netlinkSockUp = FALSE;

/*************************************************************************/
/*! \brief Creates a netlink socket for WPA
	
	\return OK/NOK
*/
/*************************************************************************/
STATUS  tiwlan_ddaCreateNetlinkSocket(void) {
	struct sock * ret;
	
	ret = netlink_kernel_create(NETLINK_USERSOCK, tiwlan_ddaNetlinkRecv);
	if (ret == NULL){
		printk(KERN_INFO "dda: netlink_kernel_create() failed !\n");
		return NOK;
	}
	
	((struct dda_priv *)wlan_dev->priv)->user_nl = ret;
	netlinkSockUp = TRUE;

	netlinkListCurrSize = 0;
	
	return OK;
}

/**************************************************************************************/
/*! \brief A callback which is called upon reception of data on the netlink socket
	
	\param sk  the netlink socket ID
	\param len data len
	
*/
/**************************************************************************************/
static void tiwlan_ddaNetlinkRecv(struct sock *sk, int len)  {
	struct sk_buff * skb;
#ifdef TIWLAN_FW_TRACE
	int * type;
#endif

	if (!sk) return;

	while (recoveryIsInProcess) {
		PAL_sleepLong(10);
	}

	while ((skb = skb_dequeue(&sk->receive_queue)) != NULL) 
        {
#ifdef TIWLAN_FW_TRACE
		type = (int *)skb->data;
		
		if (*type == FW_TRACE_HELLO) 
        {
			((struct dda_priv *)wlan_dev->priv)->fw_trace_app_pid = NETLINK_CB(skb).pid;
			printk("%s : received FW trace app pid = %d\n", __FUNCTION__, NETLINK_CB(skb).pid);
		}	
        else
#endif 
        {
            ((struct dda_priv *)wlan_dev->priv)->wpa_authenticator_pid = NETLINK_CB(skb).pid;
			if(dda_wpa_receive(skb->data, skb->len) != OK) {
                printk("%s wrong msg sender %d\n", __FUNCTION__, NETLINK_CB(skb).pid);
            }
		}

		kfree_skb(skb);
	}
}

/**************************************************************************************/
/*! \brief  Sends messages (if any) from the netlink queue through the netlink socket 
			Sends or inserts to the queue the input data 
	
	\param data  		data buffer
	\param data_len 	data length
	
	\return OK/NOK
	
*/
/**************************************************************************************/
STATUS tiwlan_ddaNetLinkSend(void * data, int data_len, tiwlan_ddaNetlinkDestination_t dst) {
	struct netlink_list * netlink_list_elem = NULL;
	struct list_head * current = NULL; 
	BOOL   sendFailed = FALSE;
	struct dda_priv * priv;
	__u32 pid;
	char *pDest = "None";

	priv = (struct dda_priv *)wlan_dev->priv;
	
	switch (dst) {
	case TIWLAN_NETLINLK_DESTINATION_WPA:
	    pDest = "WPA";
		pid = priv->wpa_authenticator_pid;
		break;
#ifdef TIWLAN_FW_TRACE
	case TIWLAN_NETLINK_DESTINATION_FW_TRACE:
	    pDest = "FW_TRACE";
		pid = priv->fw_trace_app_pid;
		break;
#endif
    case TIWLAN_NETLINK_DESTINATION_WDS_MNGR:
	    pDest = "WDS_MNGR";
        pid = priv->wds_manager_pid; 
        break;

	default:
	    pDest = "Undef";
		printk("%s : unknown destination\n", __FUNCTION__);
		return NOK;
	}

	if (pid == 0) {
		printk("%s : Netlink usermode destination is not connected !dest = %s\n", 
			   __FUNCTION__, pDest);
		return NOK;
	}

	/* try to send messages from the list, if any */
	list_for_each(current, &netlink_list_head) {
		netlink_list_elem = list_entry(current, struct netlink_list, list);
		if(tiwlan_ddaNetLinkTransmit(netlink_list_elem->data, netlink_list_elem->data_len, netlink_list_elem->pid) == OK) {			
			list_del(&netlink_list_elem->list); 
			kfree(netlink_list_elem->data);
			kfree(netlink_list_elem); 
			netlinkListCurrSize--;
		}
		else {
			sendFailed = TRUE;
			break;
		}
	}

	if(!sendFailed) {
		/* send the new message */
		if(tiwlan_ddaNetLinkTransmit(data, data_len, pid) == OK)
			return OK;
	}

	/* insert the new message to the list */
	return tilwan_ddaAddMsgToNetLinkList(data, data_len, pid);
}

/*******************************************************************************************/
/*! \brief Inserts new data to the queue of messages to be sent through the netlink socket 
	
	\param data  		data buffer
	\param data_len 	data length
	
	\return OK/NOK
	
*/
/*******************************************************************************************/
STATUS tilwan_ddaAddMsgToNetLinkList(void *data, int data_len, int pid) {

	struct netlink_list * netlink_list_elem = kmalloc(sizeof(struct netlink_list), GFP_ATOMIC); 	
	if(!netlink_list_elem)
		return NOK;

	if(netlinkListCurrSize >= NETLINK_LIST_MAX_SIZE)
	{
		printf("tilwan_ddaAddMsgToNetLinkList: the list is full\n");
		kfree(netlink_list_elem);
		return NOK;
	}		
	
	netlink_list_elem->data = kmalloc(data_len, GFP_ATOMIC);
	if(netlink_list_elem->data)
	{
		memcpy(netlink_list_elem->data, data, data_len);
	}
	else
	{
		kfree(netlink_list_elem);
		return NOK;
	}
	netlink_list_elem->data_len = data_len;
	netlink_list_elem->pid = pid;
	list_add_tail(&netlink_list_elem->list, &netlink_list_head);
	netlinkListCurrSize++;

	return OK;
}

/*******************************************************************************************/
/*! \brief  Sends a message through the netlink socket 
	
	\param data  		data buffer
	\param data_len 	data length
	
	\return OK/NOK
	
*/
/*******************************************************************************************/
STATUS tiwlan_ddaNetLinkTransmit(void * data, int data_len, int pid) {
	struct sk_buff *skb;
	int res;
	struct dda_priv * priv;

	priv = (struct dda_priv *)wlan_dev->priv;

	skb = alloc_skb(data_len, GFP_ATOMIC);
	if (!skb) {
		printk("%s : alloc_skb failed !\n", __FUNCTION__);
		return NOK;
	}

	memcpy(skb_put(skb, data_len), data, data_len);

	NETLINK_CB(skb).dst_groups = RTMGRP_LINK;
	NETLINK_CB(skb).dst_pid = pid;

	res = netlink_unicast(priv->user_nl, skb, pid, MSG_DONTWAIT);
	if (res != data_len) {
		/* Removed the following print since it's not critical failure: */

        /*printk("%s : send to pid %d fail.\n", __FUNCTION__, pid);*/
		return NOK;
	}

	return OK;
}

/*******************************************************************************************/
/*! \brief  Removes all the messages from the netlink queue	
*/
/*******************************************************************************************/
void tiwlan_ddaUnloadNetlinkList(void)
{
	struct netlink_list * netlink_list_elem = NULL;
	struct list_head * current = NULL;

	list_for_each(current, &netlink_list_head) {
		netlink_list_elem = list_entry(current, struct netlink_list, list);
		list_del(&netlink_list_elem->list); 
		kfree(netlink_list_elem->data);
		kfree(netlink_list_elem); 
		netlinkListCurrSize--;
	}
}

/*******************************************************************************************/
/*! \brief  Initializes the Vlynq

	\param  RegAddr 	an output buffer for Vlynq registers base address
	\param  MemAddr	 	an output buffer for Vlynq memory base address
	
	\return OK/NOK
*/
/*******************************************************************************************/

STATUS tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr) {
	
    if ( initTnetwAddr( RegAddr, MemAddr, 1 /* reset */ ) != OK)
    {
        printk("%s: Error reading TNETW mem map.\n" __FUNCTION__);
        return NOK;
    }

	return tiwlan_ddaInitInterrupts();
}


/******************************************************************************/

/*! \brief 	Sets and returns TNETW memory map
	\param - pointer to VLYNQ registers base addresses to return the value 
	\param - pointer to VLYNQ memory base ddresses to return the value
	
	\return OK if operation is successful or NOK in the case of error
*/
/******************************************************************************/

STATUS tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr) 
{

    if ( initTnetwAddr( RegAddr, MemAddr, 0 /* no reset */ ) != OK ) 
    {
        printk("%s: Error reading TNETW mem map.\n" __FUNCTION__);
        return NOK;
}

	return OK;
}

/******************************************************************************/

/*! \brief 	Sets and returns TNETW memory map
	\param - pointer to VLYNQ registers base addresses to return the value 
	\param - pointer to VLYNQ memory base ddresses to return the value
	
	\return OK if operation is successful or NOK in the case of error
*/
/******************************************************************************/

static STATUS initTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr, int reset) 
{
	PAL_VLYNQ_DEV_HND *vlynq_dev;
	PAL_VLYNQ_HND	  *vlynq;

	if( ! (vlynq_dev = PAL_vlynqDevFind("wlan", 0)))
	{
		printk("%s: Could not find the WLAN device configuration for instance 0\n", __FUNCTION__);
		return NOK;
	}

	if( ! (vlynq = PAL_vlynqDevGetVlynq(vlynq_dev)))
	{
		printk("Problems in the WLAN VLYNQ configuration for instance 0\n", __FUNCTION__);
		return NOK;
	}

	if( ! (PAL_vlynqGetLinkStatus(vlynq))) 
	{
		printk("We did not detect Link for WLAN device instance 0\n", __FUNCTION__);
		return NOK;
	}

    if (reset) 
    {
        DDA_RESET_CTRL(64, IN_RESET);
        PAL_sleepLong(40);
        DDA_RESET_CTRL(64, OUT_OF_RESET);
    }

    if ( PAL_vlynqGetDevBase(vlynq, VLYNQ_ACX111_MEM_OFFSET, MemAddr, vlynq_dev) )
    {
        printk ("Error getting the WLAN mapped memory area from the VLYNQ driver\n", __FUNCTION__);
        return NOK;
    }

	/* Convert to KSEG1 */
	*MemAddr = PHYS_TO_K1( *MemAddr);
	
	if ( PAL_vlynqGetDevBase(vlynq, VLYNQ_ACX111_REG_OFFSET, RegAddr, vlynq_dev) )
    {
        printk ("Error getting the WLAN mapped register area from the VLYNQ driver\n", __FUNCTION__);
        return NOK;
    }
	
	/* Convert to KSEG1 */
	*RegAddr = PHYS_TO_K1( *RegAddr);

	printk("%s: MemAddr=0x%x, RegAddr=0x%x\n", __FUNCTION__, *MemAddr, *RegAddr);


	return OK;
}


/*******************************************************************************************/
/*! \brief  Unloads the Vlynq
	
	\return OK/NOK
*/
/*******************************************************************************************/
STATUS tiwlan_ddaUnloadPSPVlynq(void) {
    if (((struct dda_priv *)wlan_dev->priv)->driver_started & DDA_DRV_STARTED)
    {
        /* Free any related ISR registration, before unloading bus driver */
        free_irq( WLAN_IVEC, wlan_dev);
        /* unload & reinit the VLYNQ driver */
        DDA_RESET_CTRL(64, IN_RESET);
        WLAN_PAL_sleepShort(10);        
        DDA_RESET_CTRL(64, OUT_OF_RESET);
    }
    return OK;
}

/*******************************************************************************************/
/*! \brief  initializes the Vlynq (stub function required for VxWorks)
	
	\return OK/NOK
*/
/*******************************************************************************************/
STATUS tiwlan_ddaReInit(void)
{
    return OK;
}

#ifdef TIWLAN_DEBUG
atomic_t pal_alloc_count;
#endif


#ifdef TIWLAN_DEBUG
#define BM_NUM_ENTRIES 64
#define BM_NUM_MASK 0x3F

typedef struct {
      unsigned long loc;/* trace entry identification */
      unsigned long ts; /* Timestamp */
      unsigned long p1; /* Parameter 1 */
      unsigned long p2; /* Parameter 2 */
} bm_entry_t;

typedef struct {
      int pos;
      UINT32 count;
      int print_on;
      unsigned long self_delay;
      bm_entry_t entry; /* Array of entries */
} bm_control_t;

bm_control_t *bm_control = NULL;
bm_entry_t   *bm_entry[BM_NUM_ENTRIES];

char *bm_trace_print_lut[] =
							   {"EMPTY     ", /* 0 */
								"TX START  ", /* 1 */
								"BSS_WLAN_R", /* 2 */
								"CHKSIZE_F ", /* 3 */
								"RX INTR   ", /* 4 */
								"RX EVENT  ", /* 5 */
								"TX_C_INTR ", /* 6 */
								"SND_Q_OK  ", /* 7 */
								"SND_Q_BUSY", /* 8 */
								"TX_C_EVENT", /* 9 */
								"TX_E_START", /*10 */
								"TX_MASTR_E", /*11 */
								"TX_TIMEOUT", /*12 */
								"SKB_FAIL  ", /*13 */
								"RX_EN_TO  ", /*14 */
                                "MBOX_TRC  "  /*15 */ 
                                   };

/*******************************************************************************************/
/*!
	\brief Serves for internal use (benchmarking )
	\return 0 in case of success
*/
/*******************************************************************************************/
int bm_init(void)
{
   int i;

   bm_control = (bm_control_t *)PAL_memAlloc(0,sizeof(bm_control_t),0,0);
   PAL_memSet(bm_control, 0, sizeof(bm_control));
   for(i=0; i<BM_NUM_ENTRIES; i++)
   {
	   bm_entry[i] = PAL_memAlloc(0,sizeof(bm_entry_t),0,0);
	   PAL_memSet(bm_entry[i], 0, sizeof(bm_entry_t));
   }
   bm_control->print_on = FALSE;
   return 0;
}

/*******************************************************************************************/
/*!
	\brief Serves for internal use (benchmarking )
*/
/*******************************************************************************************/
void bm_shutdown(void) {
   int i;
	 
	 for(i=0; i<BM_NUM_ENTRIES; i++) 
	   PAL_memFree(0, bm_entry[i], 0);

	 PAL_memFree(0, bm_control, 0);

}

/*******************************************************************************************/
/*!
	\brief Serves for internal use (benchmarking )
*/
/*******************************************************************************************/
void inline bm_trace(int loc, unsigned long p1, unsigned long p2)
{   
   int pos;

   if(bm_control->print_on)
	   return;

   pos = bm_control->pos;
   bm_control->pos = (pos+1) & BM_NUM_MASK;
   ++bm_control->count;
   bm_entry[pos]->ts = PAL_GET_TIMESTAMP();
   bm_entry[pos]->loc= loc;
   bm_entry[pos]->p1 = p1;
   bm_entry[pos]->p2 = p2;
}

/*******************************************************************************************/
/*!
	\brief Serves for internal use (benchmarking )
*/
/*******************************************************************************************/
void bm_disable(void) {
	bm_control->print_on = TRUE;
}

/*******************************************************************************************/
/*!
	\brief Serves for internal use (benchmarking )
*/
/*******************************************************************************************/
void bm_print(char *buf, int *length)
{
	int len=0, i;


	bm_control->print_on = TRUE;

	len += sprintf(buf+len, "\n------------------ TRACE BUFFER, pos = %d, count = %ul --------------------\n\n", bm_control->pos, bm_control->count); 
	for(i=0; i<BM_NUM_ENTRIES; i++)
	{
		len += sprintf(buf+len, "entry %d  event %s  time %lu  p1 %lu  p2 %lu\n",
					   i, bm_trace_print_lut[bm_entry[i]->loc], bm_entry[i]->ts, bm_entry[i]->p1, bm_entry[i]->p2);
	}
	len += sprintf(buf+len, "\n------------------------ END OF TRACE BUFFER -------------------------\n\n"); 

	bm_control->print_on = FALSE;
	*length = len;
}

void bm_print_to_console(void)
{
	int i;


	bm_control->print_on = TRUE;

	printk("\n------------------ TRACE BUFFER, pos = %d, count = %ul, ts = %lu --------------------\n\n", 
           bm_control->pos, bm_control->count, PAL_GET_TIMESTAMP()); 
	for(i=0; i<BM_NUM_ENTRIES; i++)
	{
        if(bm_entry[i]->p1 < 100000) {
            printk("entry %d  event %s  time %lu  p1 %lu  p2 %lu\n",
                           i, bm_trace_print_lut[bm_entry[i]->loc], 
                           bm_entry[i]->ts, bm_entry[i]->p1, bm_entry[i]->p2);
        }
        else {
            printk("entry %d  event %s  time %lu  p1 %s  p2 %lu - %s\n",
                           i, bm_trace_print_lut[bm_entry[i]->loc], 
                           bm_entry[i]->ts, (char *)bm_entry[i]->p1, (bm_entry[i]->p2) & 0xFFFF,
                   ((bm_entry[i]->p2) & 0x10000) ? "out" : "in");

        }
	}
	printk("\n------------------------ END OF TRACE BUFFER -------------------------\n\n"); 

	bm_control->print_on = FALSE;
}

#endif /* TIWLAN_DEBUG */

/*******************************************************************************************/
/*! \brief empty function */
/*******************************************************************************************/
void tiwlan_ddaDeleteMacAddrFunc(char *mac)
{	
}

/******************************************************************************/

/*! \brief 	Determines whether to initialize memory management module
	
	\return OK if memory management initialization is required and NOK if not
*/
/******************************************************************************/
STATUS 	tiwlan_ddaIsMemMngrInitNeeded(void)
{
	return OK;
}

/*******************************************************************************************/
/*! \brief A wrapper for linux get_random_bytes function 
	\return random bytes
*/
/*******************************************************************************************/
UINT8 tiwlan_ddaRand(void)
{
	UINT8 temp;
	get_random_bytes(&temp, 1);
	return temp;
}

/*******************************************************************************************/
/*! \brief empty function in Linux */
/*******************************************************************************************/
void tiwlan_ddaSrand(UINT16 seed)
{
	/* empty function in Linux */
}

/*******************************************************************************************/
/*! \brief Handle a Vlynq interrupt 
	
	\param	irq 	ignored
	\param 	dev_id	ignored 
	\param  regs	ignored
	
	To be as much compatible, as it is possible with 2.6, follow these guidelines:
   return IRQ_HANDLED - when we did have a valid interrupt and handled it
          IRQ_NONE - the opposite of IRQ_HANDLED
 */
/*******************************************************************************************/
static irqreturn_t tiwlan_ddaVlynqIsr(int irq, void *dev_id, struct pt_regs *regs) 
{
	whal_acxIntrHandler() ;
	return IRQ_HANDLED;
}

/*******************************************************************************************/
/*! \brief Connects to WPA module. Waits for hello message from WPA module 
	
	\return OK/NOK
 */
/*******************************************************************************************/
STATUS tiwlan_dda_waitForWpa(void)
{
	int retryCount = 0;
	
	/* connect to wpa module */
	/* wait for hello msg */
	while(retryCount++ < WAIT_FOR_WPA_MAX_RETRY)
	{
		if(dda_wpa_wait_for_wpaHello() != OK)
			printk("\nDRV WAITING FOR WPA MODULE TO COME UP !!!\n");
		else
			return OK;
	}
	return NOK;
}

/*******************************************************************************************/
/*! \brief Sends hello message to WPA module 
	
	\return OK/NOK
 */
/*******************************************************************************************/
STATUS tiwlan_dda_wpaGo(void)
{
	dda_wpa_drv_send_hello();
	return OK;
}

/*******************************************************************************************/
/*! \brief Initializes WLAN device interrupts
	
	\return OK/NOK
 */
/*******************************************************************************************/
STATUS tiwlan_ddaInitInterrupts(void) {
	/* 0 is pulsed/edge,  1 is level based interrupt */
	avalanche_intr_type_set( WLAN_IVEC, 1);
	/* 0 is active high, 1 is active low */
	avalanche_intr_polarity_set( WLAN_IVEC, 0);
	if( request_irq( WLAN_IVEC, tiwlan_ddaVlynqIsr, 0, WLAN_VEC_NAME, wlan_dev)) {
		printk( "Interrupt handler registration (request_irq) failed !\n");
		return NOK;
	}

	((struct dda_priv *)wlan_dev->priv)->driver_started |= DDA_DRV_STARTED;

	return OK;
}

/*******************************************************************************************/
/*! \brief Initializes the tasklet module
		   Restarts kernel queue 
		   Initializes proc fs support
		  
	\return OK/NOK
 */
/*******************************************************************************************/
STATUS tiwlan_ddaGo(initConfigStruct *init_cfg) {
	/*	int ret; */

	/* init tasklet module*/
	if(dda_tasklet_init(DDA_TASKLET_BUFFER_POOL_SIZE) != OK)
	{
		printk(KERN_INFO "dda: error init tasklet module\n");
		return NOK;		
	}

	/* restart kernel queue init*/
	restart_queue_init();

	/*init proc fs support*/
	if(dda_init_proc_fs() != OK)
	{
		printk(KERN_INFO "dda: error init proc module\n");
		dda_tasklet_kill();
		return NOK;		
	}

	((struct dda_priv *)wlan_dev->priv)->driver_started |= DDA_DRV_RECEIVED_GO;
	return OK;

}

/*******************************************************************************************/
/*! \brief WLAN device callback. Allocates MSDU for a given skb and sends it to the WLAN
	\param skb 	skb buffer which contains the data 
	\param dev  WLAN device structure
	
	\return 0 or a negative value in case of error
*/
/*******************************************************************************************/
static int tiwlan_ddaSend(struct sk_buff * skb, struct net_device * dev) {
#define HEADER_EXTRA_LENGTH 32
	mem_MSDU_T * pMSDU;
    mem_BD_T   * pBD;
	struct net_device_stats * stats;

	if(recoveryIsInProcess == FALSE) {

		netif_stop_queue(wlan_dev);

		bm_trace(1,0,0);

		/*
			Currently no skb fragmentation support
		*/
	#ifdef TIWLAN_DEBUG
		SKB_FRAG_ASSERT(skb);
	#endif /* TIWLAN_DEBUG */

		wlan_memMngrAllocMSDU (&pMSDU, HEADER_EXTRA_LENGTH);
		if (pMSDU == NULL)
		{
			printk("wlan_memMngrAllocMSDU failed !\n");
			kfree_skb(skb);
			goto exit;
		}
		
		pMSDU->dataLen = skb->len;
		pMSDU->freeFunc = (ap_FreeMemFunc)kfree_skb;
		pMSDU->freeArgs[0] = (UINT32)skb;
		pMSDU->firstBDPtr->bufToken = 1;

		wlan_memMngrAllocBDs(1, &pBD);
		if (pBD == NULL)
		{
			printk("wlan_memMngrAllocBDs failed !\n");
			wlan_memMngrFreeMSDU(pMSDU->handle);
			goto exit;
		}

		pBD->data = skb->data;
		pBD->length = skb->len;
		pMSDU->firstBDPtr->nextBDPtr = pMSDU->lastBDPtr = pBD;

		wdrv_bssBridgePktFromEthrToWlan(pMSDU);

		dev->trans_start = jiffies;
		stats = &((struct dda_priv *)dev->priv)->stats;

		stats->tx_bytes += skb->len;
		stats->tx_packets++;
		
		netif_wake_queue(wlan_dev);
	 exit:
		return 0;
	}
	else {
        kfree_skb(skb);
        return 0;   
    }
}

/*******************************************************************************************/
/*! \brief Receives MSDU. Copies it to skb.

	\param pMSDU 	a pointer to MSDU buffer
	
	\return OK/NOK
*/
/*******************************************************************************************/
STATUS tiwlan_ddaReceive(mem_MSDU_T * pMSDU) {
	register struct sk_buff * skb;
	int    res = OK;
	struct net_device_stats * stats;
	register mem_BD_T   * pBD = pMSDU->firstBDPtr;
#ifdef DDA_WLAN_LED
	static int skip_blink;
#endif

#ifdef NSP_WAN_BRIDGE
    /* For adding 802.1q tag to EAPOLs */
    EthernetHeader_T        *ethHdr;
    Ethernet802_1qHeader_T	ExtEthHeader;
    UINT8			        tag[4];
    UINT16                  vlan_id;
    UINT16                  ethType;
#endif /* ifdef NSP_WAN_BRIDGE */

    
	/* For BSS bridging purpose only, when interface is down */
	if (!netif_running(wlan_dev)) {
		if( (res = wlan_memMngrFreeMSDU(pMSDU->handle)) != OK)
			printk("%s: wlan_memMngrFreeMSDU failed !\n", __func__);
		return res;
	}
	
	/*
		Assumes that MSDU has only one BD
	*/
#ifdef TIWLAN_DEBUG
	if (pBD->nextBDPtr) {
		printk("%s : Only single BD MSDUs are supported !\n", __func__);
		return NOK;
	}
#endif // TIWLAN_DEBUG


#ifdef NSP_WAN_BRIDGE
    if(((struct dda_priv *)wlan_dev->priv)->pWanBridgeHandle != NULL)
    {
        ethHdr = (EthernetHeader_T*)(pBD->data+pBD->dataOffset);
        ethType = htons(ethHdr->TypeLength);
        /* If EAPOL --> Add 802.1q tag to EAPOLs */
        if( (ethType == EAPOL_ETHER_TYPE) || (ethType == 0x88c7 /* Pre-Auth EthType*/) )
        {
            if(pBD->dataOffset < 4)
            {
                printk("%s : Not enough space to add vlan tag !\n", __func__);
                res = wlan_memMngrFreeMSDU(pMSDU->handle);
                if (res != OK) 
                {
                    printk("%s: wlan_memMngrFreeMSDU failed !\n", __func__);
                }
                return NOK;
            }
            
            pBD->dataOffset -=4;
            memcpy(ExtEthHeader.DstAddr.addr, ethHdr->DstAddr.addr, MAC_ADDR_LEN*2);
            ExtEthHeader.TypeLength = ethHdr->TypeLength;
            
            if(wdrv_mgmt_ssidGetStaVlanTag(ExtEthHeader.SrcAddr.addr,&vlan_id) != OK)
            {
                printk("%s : bss setting vlan id to 0 \n", __func__);
                vlan_id = 0;
            }
            
            /* insert priority into the 802.1D Tag field of Ethernet header */
            tag[0] = 0x81;
            tag[1] = 0x00;
            tag[2] = (vlan_id >> 8) & 0x0F;
            
            tag[3] = vlan_id & 0xff;
            
            memcpy(&ExtEthHeader.Tag,tag,sizeof(ExtEthHeader.Tag));
            
            memcpy(pBD->data+pBD->dataOffset, &ExtEthHeader, sizeof(ExtEthHeader));            
        }
    }
#endif /* ifdef NSP_WAN_BRIDGE */

	skb = (struct sk_buff *)pBD->bufToken;
	if(!skb)
	{
		printk("tiwlan_ddaReceive: NULL skb received\n");
		return NOK;
	}

	skb->dev = wlan_dev ;

	stats = &((struct dda_priv *)wlan_dev->priv)->stats;

	skb_reserve(skb,(int)memMgr_BufData(pBD) + (int)memMgr_BufOffset(pBD) - (int)skb->data);
    skb_put(skb, memMgr_BufLength(pMSDU->firstBDPtr));

	wlan_dev->last_rx = jiffies;

#ifdef NSP_WAN_BRIDGE
    if(((struct dda_priv *)wlan_dev->priv)->pWanBridgeHandle != NULL) {
        skb->ip_summed = CHECKSUM_NONE;
        wb_receive_frame(skb, ((struct dda_priv *)wlan_dev->priv)->pWanBridgeHandle);
    }
    else {
#endif /* ifdef NSP_WAN_BRIDGE */

    /* #ifndef TI_SLOW_PATH */
#if 1 //def CONFIG_MIPS_AVALANCHE_FAST_ROUTE

	if (!wlan_dev->br_port)
#endif
        skb->protocol = eth_type_trans(skb, wlan_dev);
	skb->ip_summed = CHECKSUM_NONE;
	netif_rx(skb);
#ifdef NSP_WAN_BRIDGE
    }
#endif /* ifdef NSP_WAN_BRIDGE */

#ifdef DDA_WLAN_LED
	if (!wdrv_dbGetApBridgeMode() && !(++skip_blink % DDA_RX_SKIP_BLINK))
		tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF);
#endif

	stats->rx_packets++;
	stats->rx_bytes += skb->len;
	
	/* signal to memMngr not to free the skb */
	pBD->bufToken = 0;
	res = wlan_memMngrFreeMSDU(pMSDU->handle);
	if (res != OK) {
		printk("%s: wlan_memMngrFreeMSDU failed !\n", __func__);
	}

	return res;
}

/*******************************************************************************************/
/*! \brief Registered WLAN device TX timeout handler
	\param dev  WLAN device structure
	
*/
/*******************************************************************************************/
static void tiwlan_ddaTxTimeoutHandler(struct net_device * dev) {
	bm_trace(12,0,0);
	if (!recoveryIsInProcess) {
		printk("tiwlan_ddaTxTimeoutHandler activated => start queue !!!!!!!!!!!\n");
		netif_wake_queue(wlan_dev);
	}
}

/*******************************************************************************************/
/*! \brief Allocates skb of a given length

	\param len  		the length
	\param dataBufToken an output buffer: a pointer to allocated skb
	\param buf			an output buffer: a pointer to the skb data  
	
	\return OK
*/
/*******************************************************************************************/
STATUS  tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, UINT8 ** buf, tiwlan_allocPhase_t phase) {

    if(phase == TIWLAN_ALLOC_RUNTIME_PHASE)
        *dataBufToken = (UINT32)alloc_skb(len, GFP_ATOMIC);
    else
        *dataBufToken = (UINT32)alloc_skb(len, GFP_KERNEL);

	if((*dataBufToken) != 0 )
	{
		*buf = (UINT8 *)((struct sk_buff *)*dataBufToken)->data;
	}
	else
	{
		*buf = NULL;
	}
		
	return OK;
}

/*******************************************************************************************/
/*! \brief detect whether allocation of pool is needed
                                                                         
*/
/*******************************************************************************************/
STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size )
{
	
	return OK;
}


/*******************************************************************************************/
/*! \brief Registered WLAN device open handler
	\param dev  WLAN device structure
	
	\return 0
	
*/
/*******************************************************************************************/
int tiwlan_ddaOpen(struct net_device *dev) {
	netif_start_queue(dev);
	printk(KERN_INFO "%s device is activated\n", dev->name);
	return 0;
}

/*******************************************************************************************/
/*! \brief Registered WLAN device init handler
	\param dev  WLAN device structure
	
	\return 0
	
*/
/*******************************************************************************************/
static int tiwlan_ddaInit(struct net_device *dev) {

	SET_MODULE_OWNER(dev);

	dev->wireless_handlers = (struct iw_handler_def *)&dda_iw_handler_def;
	dda_wext_init();
	
	dev->hard_start_xmit = tiwlan_ddaSend;
	dev->open = tiwlan_ddaOpen;
	dev->stop = tiwlan_ddaStop;
	dev->get_stats = tiwlan_ddaGetStats;
	dev->do_ioctl = tiwlan_ddaDoIoctl;
	dev->set_mac_address = tiwlan_ddaSetMacAddress;

	dev->tx_timeout = tiwlan_ddaTxTimeoutHandler;
	dev->watchdog_timeo = HZ; /* 1 jiffies = 10 ms */

	dev->if_port = 3; /* QoS support for NSP */

	init_etherdev(dev, sizeof(struct dda_priv));

	memset(dev->priv, 0, sizeof(struct dda_priv));
#ifdef NSP_WAN_BRIDGE
    ((struct dda_priv *)dev->priv)->pWanBridgeHandle = NULL;        
#endif /* ifdef NSP_WAN_BRIDGE */
		
	wlan_dev = dev;

	return 0;
}

/*******************************************************************************************/
/*! \brief Registered WLAN device statistics handler
	\param dev  WLAN device structure
	
	\return a pointer to statistics structure
	
*/
/*******************************************************************************************/
struct net_device_stats * tiwlan_ddaGetStats(struct net_device * dev) {
	return &((struct dda_priv *)dev->priv)->stats;
}

/*******************************************************************************************/
/*! \brief Registered WLAN device stop handler
	\param dev  WLAN device structure
	
	\return 0
*/
/*******************************************************************************************/
static int tiwlan_ddaStop(struct net_device *dev) {
	netif_stop_queue(dev);
	printk(KERN_INFO "%s device is de-activated\n", dev->name);
	return 0;
}

/* Matthew Add for Board startup wireless module detection */

/* DG834G led function */
/* GPIO definations */
#define ACX111_GPIO_0_BASE 0xa8610900
#define ACX111_GPIO_OUT   (*(volatile unsigned int *)(ACX111_GPIO_0_BASE+0x4))
#define ACX111_GPIO_DIR   (*(volatile unsigned int *)(ACX111_GPIO_0_BASE+0x8))
#define ACX111_GPIO_EN    (*(volatile unsigned int *)(ACX111_GPIO_0_BASE+0xc))
#define DG834G_WLLED_GPIO (1<<12)
#define LED_THRESHOLD_LEVEL     7
#define LED_BLINKING_LEVEL      3

#define AVALANCHE_VLYNQ0_INT         21

static int wlan_have_packages = 0;
static struct timer_list wlan_led_timer;

static void blink_wireless_led(unsigned long data) {
        if(ACX111_GPIO_OUT & DG834G_WLLED_GPIO) { /* current off */
                ACX111_GPIO_OUT &= ~DG834G_WLLED_GPIO; /* turn on */
        } else if(wlan_have_packages){
                ACX111_GPIO_OUT |= DG834G_WLLED_GPIO;
                wlan_have_packages = 0;
        }
        wlan_led_timer.expires = jiffies + 5;
        add_timer(&wlan_led_timer);
}
/* DG834G led function */

/*******************************************************************************************/
/*! \brief Initializes WLAN device
	
	\return zero or error code in case of falure
*/
/*******************************************************************************************/
static int __init tiwlan_ddaInitModule(void) 
{
	/* Matthew Add for Board startup wireless module detection */
        printk("**************************************\n");
        printk("**  APDK 6.2.0.20  (Patch 0.3.3)    **\n");
        printk("**************************************\n");
	/* Set GPIO Direction to Output 2005.11.10 (turn wlan led on)*/
        ACX111_GPIO_DIR &= ~DG834G_WLLED_GPIO;

        wlan_led_timer.function = blink_wireless_led;
        wlan_led_timer.expires = jiffies + 5;
        add_timer(&wlan_led_timer);
#ifdef DDA_WLAN_LED
#if defined(WLAN_WIFI_PSP_LED)

	if( !(wlan_led = avalanche_led_register("wlan", 0)) ) {
		printk("\ndda: Failed to register wlan led\n\n");
		return -ENODEV;
	}
#elif defined(WLAN_WIFI_PSP_COLORED_LED)	
    led.led_funcs = NULL;
    led.num_of_gpios = 1;
#if defined(CONFIG_MIPS_TITAN)
    led.led_gpios[0] = 9;
#elif defined(CONFIG_MIPS_AR7VWI)
	led.led_gpios[0] = 6;
#endif
    led_action.led_handle = led_hal_register(&led);
    led_action.color_index = 0;
    led_action.param1 = 50; /* 50 ms delay */
    if(led_action.led_handle < 0) {
		printk("\ndda: Failed to register wlan led, err=%d\n\n",led_action.led_handle);
		return -ENODEV;
    }
#elif defined(WLAN_WIFI_1130_LED)
	PALSRV_tcbNew(&tiap_led_timer);
	PALSRV_tcbQueueAct(&tiap_led_timer, TIAP_LED_TIMER_PERIOD, tiap_led_timer_func, 0, 0);
#endif
#endif

	if( !(wlan_dev = alloc_etherdev(sizeof(struct dda_priv))) )
		return -ENODEV;

	strncpy(wlan_dev->name, DDA_DEVNAME, IFNAMSIZ);
	wlan_dev->init = tiwlan_ddaInit;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	dev->destructor = free_netdev;
#else
	/* Linux 2.4.x uses dev entry (reads dev->features) after
	 * dev->destructor has returned, so must not free struct net_device
	 * here. Set NETIF_F_DYNALLOC flag so that netdev_finish_unregister()
	 * frees the entry. */
	wlan_dev->features |= NETIF_F_DYNALLOC;
#endif
	
	if( register_netdev(wlan_dev) ) {
		printk(KERN_INFO "dda: error registering %s\n", wlan_dev->name);
		kfree(wlan_dev);
		return -ENODEV;
	}

#ifdef TIWLAN_DEBUG	
	atomic_set(&pal_alloc_count, 0);
#endif

	bm_init();

    if(tiwlan_setupWdsNetDevs(WDS_MAX_DL_NUMBER) != 0) {
		printk(KERN_INFO "\ndda: error setting up WDS network interfaces\n");
        unregister_netdev(wlan_dev);
        tiwlan_ddaRemoveWdsNetDevs();
        wlan_dev = NULL;
		return -ENODEV;
    }
	return 0;
}

char *get_mac_addr(void);

/*! \brief Registered WLAN device handler. Changes hardware address of WLAN device
	\param dev  WLAN device structure
	\param addr MAC address structure
	
	\return 0 or error code in case of falure
*/
int tiwlan_ddaSetMacAddress(struct net_device *dev, void *addr)
{
	struct  sockaddr *hwaddr = addr;
	u8 *p = get_mac_addr();
	
	if( !p || !dev ) return -EINVAL;

	memcpy(dev->dev_addr, hwaddr->sa_data, ETH_ALEN);
	dev->addr_len = ETH_ALEN;
	memcpy(p, dev->dev_addr, dev->addr_len);
	p = dev->dev_addr;
	printk("\n\n%s: set hw addr=%02X:%02X:%02X:%02X:%02X:%02X\n\n\n", __func__,
			p[0], p[1], p[2], p[3], p[4], p[5]);

        return 0;
}

extern struct tasklet_struct  main_system_tasklet; 

/*******************************************************************************************/
/*! \brief Suspends WLAN device. 
	
	\return OK
*/
/*******************************************************************************************/
STATUS tiwlan_ddaSuspend(void) { 
	unsigned int flags = 0;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	PALSRV_tcbSuspend();	
	netif_stop_queue(wlan_dev);
    if(wdrv_dbGetWdsEnable()) {
        tiwlan_ddaWdsSuspedTransmit();
    }
	tasklet_disable(&main_system_tasklet);
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
	return OK;
}

/*******************************************************************************************/
/*! \brief Resumes WLAN device. 
	
	\return OK
*/
/*******************************************************************************************/
STATUS tiwlan_ddaResume(void) {	
	unsigned int flags = 0;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	tasklet_enable(&main_system_tasklet);
	PALSRV_tcbResume();
	netif_wake_queue(wlan_dev);
    if(wdrv_dbGetWdsEnable()) {
        tiwlan_ddaWdsResumeTransmit();
    }
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	return OK;
}

/*******************************************************************************************/
/*! \brief Performs a clean up for DDA Module. 

*/
/*******************************************************************************************/
static void __exit tiwlan_ddaCleanupModule(void) {
	struct sock * sock /*ret*/;
	BOOL driver_started = FALSE;

	printk("dda: starting cleanup: \n");

	del_timer_sync(&wlan_led_timer);

	netif_stop_queue(wlan_dev);


	if (((struct dda_priv *)wlan_dev->priv)->driver_started & DDA_DRV_STARTED) {

        wdrv_DeauthStations();
        wdrv_DDA_mgmt_processAllEvents();

        free_irq(WLAN_IVEC, wlan_dev);
		printk("dda: cleanup finish succesfully ,good-bye.\n");
		driver_started = TRUE;
	}

	tiwlan_ddaWdsSuspedTransmit();

	if(netlinkSockUp)
	{
		if( !(sock =  (struct sock *)((struct dda_priv *)wlan_dev->priv)->user_nl) )
			printk("\n%s: BUG line=%d !!!\n", __func__, __LINE__);
		else sock_release(sock->socket);

		tiwlan_ddaUnloadNetlinkList();
	}	

	if (((struct dda_priv *)wlan_dev->priv)->driver_started & DDA_DRV_RECEIVED_GO) {
		dda_tasklet_kill();
		dda_clean_proc();
		wdrv_unload(driver_started);
	}

	if (Acx100_bin_0_Array_Extra)
		kfree(	Acx100_bin_0_Array_Extra);

#ifdef TIWLAN_DEBUG
	printk("Memory debug : pal_alloc_count = %d\n", atomic_read(&pal_alloc_count));
#endif

	bm_disable();
	bm_shutdown();

#ifdef DDA_WLAN_LED
#if defined(WLAN_WIFI_PSP_LED)
	if( wlan_dev ) {
		tiwlan_ddaLedAction(TIAP_LED_OFF);
		avalanche_led_unregister(wlan_led);
	}
#elif defined(WLAN_WIFI_PSP_COLORED_LED)
    if(led_action.led_handle >= 0) {
		BOOL sts;
        tiwlan_ddaLedAction(TIAP_LED_OFF);
        if((sts = led_hal_deregister(led_action.led_handle)) < 0) {
            printk("Failed to unregister led, err=%d\n",sts);
        }
    }
#elif defined(WLAN_WIFI_1130_LED)
	PALSRV_tcbDelete(&tiap_led_timer);
#endif
#endif

	unregister_netdev(wlan_dev);
    tiwlan_ddaRemoveWdsNetDevs();
	wlan_dev = NULL;

	/* 2005.1.14 change to turn off LED */
        ACX111_GPIO_OUT |= (1<<12);

	printk("dda: cleanup finish succesfully, good-bye.\n");
}

module_init(tiwlan_ddaInitModule);
module_exit(tiwlan_ddaCleanupModule);

