/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	dda_wdsDrv.c	  										   */
/*   PURPOSE:	DDA implementation of the WDS driver.				       */
/*																		   */
/***************************************************************************/

#define __DDA_WDS_DRV_C__

#include "wlan_pal.h"
#include <linux/skbuff.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>

#include <asm/uaccess.h>

#include <net/sock.h>
#include "mainsm.h"
#include "802_11Defs.h"
#include "public_security_keys.h"
#include "tx.h"
#include "dda_wpa_interface.h"
#include "dda_contexts.h"
#include "dda_wext_ioctl.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtIf.h"
#include "mgmtSsidHandler.h"
#include "dda.h"
#include "wds.h"
#include "wdsDrv.h"


#define WDS_HEADER_EXTRA_LENGTH (sizeof(dot11_header_T) + sizeof(Wlan_LlcHeader_T))
extern UINT32 tiwlan_wdsRecvProtMsgs;
void tiwlan_ddaGetWdsCounters(wdsDrvMngrStat_s *pBuff);


/*
 *Function Name:tiwlan_ddaWdsSend
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int tiwlan_ddaWdsSend(struct sk_buff * skb, struct net_device * dev) {

	mem_MSDU_T * pMSDU;
    mem_BD_T   * pBD;
	struct net_device_stats * stats;
    struct wds_priv *pWdsDev = ((struct wds_priv *) dev->priv);

	netif_stop_queue(dev);

    wlan_memMngrAllocMSDU (&pMSDU, WDS_HEADER_EXTRA_LENGTH);
    if (pMSDU == NULL)
	{
		printk("wlan_memMngrAllocMSDU failed !\n");
		kfree_skb(skb);
		goto exit;
	}
	pMSDU->dataLen = skb->len;
	pMSDU->freeFunc = (ap_FreeMemFunc)kfree_skb;
	pMSDU->freeArgs[0] = (UINT32)skb;
	pMSDU->firstBDPtr->bufToken = 1;

    wlan_memMngrAllocBDs(1, &pBD);
    if (pBD == NULL)
    {
		printk("wlan_memMngrAllocBDs failed !\n");
		wlan_memMngrFreeMSDU(pMSDU->handle);
		goto exit;
	}

    pBD->data = skb->data;
    pBD->length = skb->len;
    pMSDU->firstBDPtr->nextBDPtr = pMSDU->lastBDPtr = pBD;

    wdrv_wdsDrvPktFromEthrToWlan(pMSDU, pWdsDev->linkPartner);

    dev->trans_start = jiffies;
	stats = &((struct wds_priv *)dev->priv)->stats;

	stats->tx_bytes += skb->len;
	stats->tx_packets++;
	
	netif_wake_queue(dev);

exit:
	return 0;
}

/*
 *Function Name:tiwlan_ddaWdsReceive
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

STATUS tiwlan_ddaWdsReceive(void *pCookie, mem_MSDU_T * pMSDU) 
{
	int    res = OK;
	register mem_BD_T   * pBD = pMSDU->firstBDPtr;
    register struct sk_buff * skb;
    struct net_device *dev = (struct net_device *)pCookie;
    struct net_device_stats * stats;

    EthernetHeader_T * pEthHdr;

    /* When interface is down drop the packet except EAPOLs */
	if (!netif_running(dev)) {
        /* When WDS network interface still down pass EAPOLs over NetLink socket. */
        pEthHdr = (EthernetHeader_T *) (memMgr_BufData(pBD) + memMgr_BufOffset(pBD));
        if(htons(pEthHdr->TypeLength) == EAPOL_ETHER_TYPE) {
            wpaMessage_T	msg;

            msg.type = EAPOL_FRAME_INT_PROF;
            if(wdrv_mgmt_ssidGetInternalSsidIndex(&msg.profile_index) == OK) { 
                msg.content.eapolFrame.eapolDataLen = memMgr_BufLength(pBD);
                memcpy((void *)msg.content.eapolFrame.pEapol, (const void *)pEthHdr, memMgr_BufLength(pBD));
                msg.content.eapolFrame.encrypted = TRUE;

                if(tiwlan_ddaNetLinkSend((void *) &msg, sizeof(msg), 
                                         TIWLAN_NETLINLK_DESTINATION_WPA) != OK) {
                    printk("%s: Send to WPA over NetLink socket failed !\n", __FUNCTION__);
                }
            }
        }
		if( (res = wlan_memMngrFreeMSDU(pMSDU->handle)) != OK)
			printk("%s: wlan_memMngrFreeMSDU failed !\n", __FUNCTION__);
		return (res);
	}
	
	/*
		Assumes that MSDU has only one BD
	*/
#ifdef TIWLAN_DEBUG
	if (pBD->nextBDPtr) {
		printk("%s : Only single BD MSDUs are supported !\n", __FUNCTION__);
		return (NOK);
	}
#endif // TIWLAN_DEBUG


	skb = (struct sk_buff *)pBD->bufToken;
	if(!skb)
	{
		printk("%s : tiwlan_ddaReceive: NULL skb received\n", __FUNCTION__);
		return (NOK);
	}

	skb->dev = dev ;

	skb_reserve(skb,(int)memMgr_BufData(pBD) + (int)memMgr_BufOffset(pBD) - (int)skb->data);
    skb_put(skb, memMgr_BufLength(pMSDU->firstBDPtr));
/* #ifndef TI_SLOW_PATH */
#if 1   //def CONFIG_MIPS_AVALANCHE_FAST_ROUTE
	if (!dev->br_port)
#endif
       skb->protocol = eth_type_trans(skb, dev);
    skb->ip_summed = CHECKSUM_NONE;
    dev->last_rx = jiffies;

    netif_rx(skb);

    stats = &((struct wds_priv *)dev->priv)->stats;
    stats->rx_packets++;
	stats->rx_bytes += skb->len;

    /* signal to memMngr not to free the skb */
	pBD->bufToken = 0;
	res = wlan_memMngrFreeMSDU(pMSDU->handle);
	if (res != OK) {
		printk("%s: wlan_memMngrFreeMSDU failed !\n", __FUNCTION__);
	}

	return res;
}

/*
 *Function Name:tiwlan_ddaWdsOpen
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

int tiwlan_ddaWdsOpen(struct net_device *dev) {
	netif_start_queue(dev);
	printk(KERN_INFO "%s device is activated\n", dev->name);
	return 0;
}

/*
 *Function Name:tiwlan_ddaWdsStop
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int tiwlan_ddaWdsStop(struct net_device *dev) {
	netif_stop_queue(dev);
	printk(KERN_INFO "%s device is de-activated\n", dev->name);
	return 0;
}

/*
 *Function Name:tiwlan_ddaWdsGetStats
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

struct net_device_stats * tiwlan_ddaWdsGetStats(struct net_device * dev) {
	return &((struct wds_priv *)dev->priv)->stats;
}

/*
 *Function Name:tiwlan_ddaWdsDoIoctl
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int tiwlan_ddaWdsDoIoctl(struct net_device * dev, struct ifreq * rq, int cmd)
{
    int ret = 0;
    generic_ioctl_buf_t ioctl_buf;
    wdsNetIfcStat_s *pIfcStat;
    struct wds_priv *pDev = ((struct wds_priv *) (dev->priv));
	char *buff = NULL;

	if (cmd != SIOCDEVPRIVATE) {
		printk("%s : Unknown ioctl %d\n", __FUNCTION__, cmd);
		ret = -1;
		return (ret);
	}

	copy_from_user(&ioctl_buf, rq->ifr_data, sizeof(generic_ioctl_buf_t));

	if (ioctl_buf.buf && ioctl_buf.buf_len) {
        buff = kmalloc(ioctl_buf.buf_len, GFP_KERNEL);
        if (!buff) {
            printk("%s : failed to allocate %d bytes of memory \n", __FUNCTION__, ioctl_buf.buf_len);
            return -ENOMEM;
        }
        copy_from_user(buff, ioctl_buf.buf, ioctl_buf.buf_len);
	}  else {
		printk(KERN_ERR "%s: Both buf and buf_len must be valid.\n", __FUNCTION__);
		return -EINVAL;
	}

	switch (ioctl_buf.cmd) {
    	case TIAP_GENERIC_IOCTL_WDS_LINK_ADD:
            PAL_memCopy(pDev->linkPartner, ioctl_buf.mac, ETH_ALEN);
            wdrv_mgmtWdsLinkAdd(pDev->linkPartner, ioctl_buf.param1);
            wdrv_wdsInterfaceCookieAdd(pDev->linkPartner, (void *)dev);
            break;
        case TIAP_GENERIC_IOCTL_WDS_LINK_REMOVE:
            wdrv_wdsInterfaceCookieRemove(pDev->linkPartner);
            wdrv_mgmtWdsLinkRemove(pDev->linkPartner);
            break;
        case TIAP_GENERIC_IOCTL_WDS_STAT_GET:
            tiwlan_ddaGetWdsCounters((wdsDrvMngrStat_s *)buff);
            break;
        case TIAP_GENERIC_IOCTL_WDS_IFC_STAT_GET:
            pIfcStat = (wdsNetIfcStat_s *)buff;
            pIfcStat->rx_packets = pDev->stats.rx_packets;
            pIfcStat->rx_errors = pDev->stats.rx_errors;
            pIfcStat->rx_dropped = pDev->stats.rx_dropped;
            pIfcStat->tx_packets = pDev->stats.tx_packets;
            pIfcStat->tx_errors = pDev->stats.tx_errors;
            pIfcStat->tx_dropped = pDev->stats.tx_dropped;
            break;
        case TIAP_GENERIC_IOCTL_WDS_DRV_STAT_GET:
            wdrv_wdsGetStat((wdsDrvStat_s *)buff);
            break;
        case TIAP_GENERIC_IOCTL_WDS_BRG_STAT_GET:
            wdrv_wdsBridgeInfoGet((wdsDrvSwtchingEntry_s *)buff);
            printk("%s\n", buff);
            break;

        default:
            printk("%s : Unknown ioctl %d\n", __FUNCTION__, cmd);
            ret = -1;
            break;
    }
	copy_to_user(rq->ifr_data, &ioctl_buf, sizeof(generic_ioctl_buf_t));
	if(buff != NULL) {
        copy_to_user(((generic_ioctl_buf_t *)rq->ifr_data)->buf, buff, ioctl_buf.buf_len);
        kfree(buff);
    }
    return (ret);
}

/*
 *Function Name:tiwlan_ddaWdsSetMacAddress
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

int tiwlan_ddaWdsSetMacAddress(struct net_device *dev, void *addr)
{
	printk("\n\n%s: set hw addr called.\n\n", dev->name);

        return 0;
}

/*
 *Function Name:tiwlan_ddaWdsTxTimeoutHandler
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static void tiwlan_ddaWdsTxTimeoutHandler(struct net_device * dev) 
{
    netif_wake_queue(dev);
}

/*
 *Function Name:tiwlan_ddaWdsDevsInit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

static int tiwlan_ddaWdsDevsInit(struct net_device *dev) {

	SET_MODULE_OWNER(dev);

	dev->hard_start_xmit = tiwlan_ddaWdsSend;
	dev->open = tiwlan_ddaWdsOpen;
	dev->stop = tiwlan_ddaWdsStop;
	dev->get_stats = tiwlan_ddaWdsGetStats;
	dev->do_ioctl = tiwlan_ddaWdsDoIoctl;
	dev->set_mac_address = tiwlan_ddaWdsSetMacAddress;

	dev->tx_timeout = tiwlan_ddaWdsTxTimeoutHandler;
	dev->watchdog_timeo = 1000; /* 1 jiffies = 10 ms */

	dev->if_port = 3; /* QoS support for NSP */

	init_etherdev(dev, sizeof(struct wds_priv));

	memset(dev->priv, 0, sizeof(struct wds_priv));
		
	return 0;
}

static struct net_device * wdsNetDevs[WDS_LINKS_NUMBER];
/*
 *Function Name:tiwlan_setupWdsNetDevs
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

int tiwlan_setupWdsNetDevs(int wdsDlDevsNumber)
{
    struct net_device *wds_dev = NULL;
    int i;

    /* create uplink device */
    if ( !(wds_dev = alloc_etherdev(sizeof(struct wds_priv))) )
        return -ENODEV;

    strncpy(wds_dev->name, WDS_DEVNAME_UL, (IFNAMSIZ - 2));
    wds_dev->name[IFNAMSIZ - 2] = 0;
    strcat(wds_dev->name, "%d");
    wds_dev->init = tiwlan_ddaWdsDevsInit;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
    dev->destructor = free_netdev;
#else
    /* Linux 2.4.x uses dev entry (reads dev->features) after
     * dev->destructor has returned, so must not free struct net_device
     * here. Set NETIF_F_DYNALLOC flag so that netdev_finish_unregister()
     * frees the entry. */
    wds_dev->features |= NETIF_F_DYNALLOC;
#endif

    if ( register_netdev(wds_dev) ) {
        printk(KERN_INFO "dda: error registering %s\n", wds_dev->name);
        kfree(wds_dev);
        return -ENODEV;
    }
    wdsNetDevs[0] = wds_dev;
    /* create downlink devices */
    for (i = 0; i < wdsDlDevsNumber; i++) {
        if ( !(wds_dev = alloc_etherdev(sizeof(struct wds_priv))) )
            return -ENODEV;

        strncpy(wds_dev->name, WDS_DEVNAME_DL, (IFNAMSIZ - 2));
        wds_dev->name[IFNAMSIZ - 2] = 0;
        strcat(wds_dev->name, "%d");
        wds_dev->init = tiwlan_ddaWdsDevsInit;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
        dev->destructor = free_netdev;
#else
        /* Linux 2.4.x uses dev entry (reads dev->features) after
         * dev->destructor has returned, so must not free struct net_device
         * here. Set NETIF_F_DYNALLOC flag so that netdev_finish_unregister()
         * frees the entry. */
        wds_dev->features |= NETIF_F_DYNALLOC;
#endif

        if ( register_netdev(wds_dev) ) {
            printk(KERN_INFO "dda: error registering %s\n", wds_dev->name);
            kfree(wds_dev);
            return -ENODEV;
        }
        wdsNetDevs[i + 1] = wds_dev;

    }
    return 0;
}

/*
 *Function Name:tiwlan_ddaRemoveWdsNetDevs
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaRemoveWdsNetDevs(void)
{
    UINT32 i;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        if (wdsNetDevs[i] != NULL) {
            unregister_netdev(wdsNetDevs[i]);
        }
    }
}

static UINT32 tiwlan_wdsSentProtMsgs = 0;
static UINT32 tiwlan_wdsSentProtMsgsFail = 0;
/*
 *Function Name:tiwlan_ddaSendWdsPkt
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaSendWdsPkt(UINT8 *pBuff, UINT32 length)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	mem_BD_T *BD;
    dot11_header_T *pHdr;
    int profileIndex;

    /* Check for recovery before sending packet */
    if(recoveryIsInProcess)
        return;
	/* compute the total len of the frame */
	totalLen = length;
		
	/* Allocate data buffers and take spare for security */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen + MIC_LEN) != OK) {
		printk("%s : wlan_memMngrAllocMSDU failed !\n", __FUNCTION__);
        return;
    }

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = length;

	BD = MSDUPtr->firstBDPtr;
	BD->dataOffset = 0;
	BD->length = 0;

    PAL_memCopy((BD->data + BD->dataOffset), pBuff, length);

	BD->length += length;	


    pHdr = (dot11_header_T *) BD->data;
    if(pHdr->address1.addr[0] == 0xFF) {
        /* Set broadcast flag */
        txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_BRCST_MTCST_MSDU;
        if(wdrv_mgmt_ssidGetInternalSsidIndex(&profileIndex) != OK)  {
            wlan_memMngrFreeMSDU(MSDUPtr->handle);
            printf("%s wdrv_mgmt_ssidGetInternalSsidIndex returns error.\n", __FUNCTION__);
            return;
        }
        if(wdrv_txSendMsdu(MSDUPtr, profileIndex, txFlags, 0) == OK) {
            tiwlan_wdsSentProtMsgs++;
        }
        else {
            tiwlan_wdsSentProtMsgsFail++;
        }

    }
    else {
        UINT16 staIndex;

        txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU;
        if( wdrv_dbFindStaIndex(pHdr->address1.addr, &staIndex) != OK ) {
            staIndex = INVALID_INDEX;
        }
        else {
            if (wdrv_mgmtIsStationAuthorized(staIndex) == FALSE) {
                staIndex = INVALID_INDEX;
            }
        }

        if(wdrv_txSendMsdu(MSDUPtr, staIndex, txFlags, 0) == OK) {
            tiwlan_wdsSentProtMsgs++;
        }
        else {
            tiwlan_wdsSentProtMsgsFail++;
        }
    }

}

/*
 *Function Name:tiwlan_ddaGetWdsCounters
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaGetWdsCounters(wdsDrvMngrStat_s *pBuff)
{
    pBuff->sentProtocolMsgs = tiwlan_wdsSentProtMsgs;
    pBuff->sentProtocolMsgsFails = tiwlan_wdsSentProtMsgsFail;
    pBuff->recvProtocolMsgs = tiwlan_wdsRecvProtMsgs;
}

/*
 *Function Name:tiwlan_ddaRecvWdsPkt
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

STATUS tiwlan_ddaRecvWdsPkt(UINT8 *pBuff, UINT32 length)
{
    STATUS retCode;
    retCode = tiwlan_ddaNetLinkSend((void *) pBuff, length, TIWLAN_NETLINK_DESTINATION_WDS_MNGR);

    return (retCode);
    
}

/*
 *Function Name:tiwlan_ddaWdsSuspedTransmit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaWdsSuspedTransmit(void)
{
    UINT32 i;
    struct net_device * pDev;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        pDev = (struct net_device *)wdrv_wdsInterfaceCookieGetByIndex(i);
        if(pDev != NULL) {
            netif_stop_queue(pDev);
        }
    }
}

/*
 *Function Name:tiwlan_ddaWdsResumeTransmit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaWdsResumeTransmit(void)
{
    UINT32 i;
    struct net_device * pDev;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        pDev = (struct net_device *)wdrv_wdsInterfaceCookieGetByIndex(i);
        if(pDev != NULL) {
            netif_wake_queue(pDev);
        }
    }
}


