/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000-2004 Texas Instruments Incorporated |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/*!
 *    \file  	wlan_pal.h                                                        
 *    \brief	WLAN PAL definitions                                            
 *
 *    authors:	Felix Rubinstein   <felixr@ti.com>                         
 *              Shlomi Atias       <shlomi.atias@ti.com>                   
 *                                                                         
*/



#ifndef __WLAN_PAL_H
#define __WLAN_PAL_H

/****************************
    includes
*****************************/


#include <linux/config.h>
#include <linux/version.h>
#ifdef PSP7
#include <linux/modversions.h>
#include <asm/avalanche/generic/pal.h> 
#endif

#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <asm/cacheops.h>
/*#include <asm-mips/softirq.h>*/
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <asm/atomic.h>
#include <asm/byteorder.h>
#include <asm/hardirq.h>
#include "pform_types.h"
/*
#ifdef PSP7
   #include <asm/avalanche/generic/haltypes.h>
#endif  */

typedef int WLAN_PAL_Result;
typedef void * WLAN_Ptr;
typedef struct timer_list PALSRV_TcbObjTYPE;
typedef int PAL_MemAddrSpace;
typedef atomic_t PAL_atomic_t;

#define STATUS	int

#ifdef TIWLAN_DEBUG
#define INTLOCK_DEBUG
#define MAX_INTLOCK_TRACE 100
#define INTLOCK_TRACE_PRINT_THRESHOLD 15
#endif
/*
	For debug
*/

#define FUNC printk("%s : %d\n", __FUNCTION__, __LINE__)

#define WLAN_PAL_SOK (0)


/* macros definitions */

/* key definition */
#define RL_ROOT		1
#define RL_TEST		2
#define RL_TCB			3
#define RL_IDLE		4
#define RL_DB			5
#define RL_MAINSM	   6
#define RL_MGMT	   7
#define RL_HAL	      8
#define RL_TX        9
#define RL_SM	   10
#define RL_RX	   11
#define RL_RX_SNIFF	   12
#define RL_TX_SNIFF	   13
#define RL_BRIDGE	   14
#define RL_WPA			15
#define RL_HTTP	16
#define RL_4X		17
#define RL_MEM		18
#define RL_DYNAMIC_ALG	19

/*! \brief level definition */
typedef enum {
   PRI_ALWAYS_PRINT = 1,
   PRI_EMERGENCY,
   PRI_ALERT,
   PRI_CRITICAL,
   PRI_ERROR,
   PRI_WARNING,
   PRI_NOTICE,
   PRI_INFORMATION,
   PRI_DEBUG
} Priority_T;

extern signed int pal_replev;

struct pal_tcb_list {
	struct list_head list;
	PALSRV_TcbObjTYPE * timer;
	int pending, expires;
};

extern struct list_head pal_tcb_list_head;

extern BOOL recoveryIsInProcess;

#define TIWLAN_DEBUG_PRINT printk("%s : %d\n", __FUNCTION__, __LINE__)

#define PAL_GET_TIMESTAMP() read_32bit_cp0_register(CP0_COUNT)

/*
static inline int PAL_GET_TIMESTAMP(void) {
	volatile int tmp;
	__asm __volatile ( "mfc0    %0,$9"  : "=d" (tmp));
	return tmp;
}
*/
/*! \brief for ReportMode debugging */
#define REPORT_DEBUG 0 

/*! \def PALSRV_report (key, level, len, format, args...)
   \brief Print routine implemented with debug levels
   
   \param key: the modules bitmask
   \param level: the debug level
   \param len: is ignored
   \param format: the format string
   \param args:	format arguments	

   Note: uses gcc extension namely - Macros with a Variable Number of Arguments
   TODO: The level argument should be dynamically mapped to one of linux kernel defines, i.e. KERN_DEBUG
 */

#define PALSRV_report(key, level, len, format, args...) \
do { \
			printk("%lu: ", jiffies/(1000/HZ)); \
			printk(format, ##args);\
} while(0)

/*!
   \def PALSRV_report_set(key, level)
   \brief set key to the modules bitmask
    
	\param key 		the key value to set
	\param level 	ignored
 */
#define PALSRV_report_set(key, level) \
	do { pal_replev |= 1 << (key); } while(0)
	
/*! \def PALSRV_report_unset(key, level) 
   \brief unset key from the modules bitmask
   
   \param key: 		the key value to unset
   \param level: 	ignored
 */
#define PALSRV_report_unset(key, level) \
	do { pal_replev &= ~(1 << (key)); } while(0)

/*! \def PALSRV_report_disable(level) 
   \brief set all the bits in the bitmask to zero
  
   \param level: 	ignored
 */
#define PALSRV_report_disable(level) \
	do { pal_replev = 0; } while(0)

/*! \def PALSRV_report_get() 
   \brief returns the pal_replev bitmask

 */
#define PALSRV_report_get() pal_replev
#define printf(s,args...)	PALSRV_report(0,PRI_ALWAYS_PRINT,0,s,##args)
#define logMsg(s,args...)	PALSRV_report(0,PRI_ALWAYS_PRINT,0,s,##args)
#define wdrv_utilsReport(key, level, s, args...) PALSRV_report(key,level,0,s,##args)

#define puts(s) printf("%s\n", s)
	
#ifdef TIWLAN_DEBUG
	#define REPORT_LEVEL(key)	(pal_replev & (1<<(key)))
#else
	#define REPORT_LEVEL(key) 0
#endif

/* endianess converion */

#ifdef __LITTLE_ENDIAN
#define PAL_LITTLE_ENDIAN
#endif

#define PAL_LE2UINT16 le16_to_cpu
#define PAL_BE2UINT16 be16_to_cpu
#define	PAL_LE2UINT32 le32_to_cpu
#define	PAL_BE2UINT32 be32_to_cpu

#define ENDIAN_HANDLE_WORD  PAL_LE2UINT16
#define ENDIAN_HANDLE_LONG	PAL_LE2UINT32

#ifdef INTLOCK_DEBUG
extern unsigned int wlan_inIntLock;
extern char *wlan_intLockFunc[];
extern unsigned int wlan_intLockLine[];
#endif /* ifdef INTLOCK_DEBUG */


/*! \def PAL_protectEntry(level, cookie)
	\brief Level - lock level required, cuurently supports only interrupt locking
	\brief cookie - flags for interrupt locking
	\brief Saves current interrupt state in flags and disables the interrupts
	\param flags: current interrupt state is saved in flags
 */
#ifdef INTLOCK_DEBUG
#define PAL_protectEntry(level, cookie) {PAL_osProtectEntry(level,cookie); \
                        wlan_intLockFunc[wlan_inIntLock] = __FUNCTION__; \
                        wlan_intLockLine[wlan_inIntLock] = __LINE__; wlan_inIntLock ++;\
                        if(wlan_inIntLock == INTLOCK_TRACE_PRINT_THRESHOLD){ \
                          int i; \
                          printk("\n\n**** %s:%d Max nesting level reached.*****\n\n", __FUNCTION__, __LINE__); \
                          for(i=0;i<INTLOCK_TRACE_PRINT_THRESHOLD;i++) \
                             printk("\nLock %d set at %s:%d\n\n", i, wlan_intLockFunc[i], wlan_intLockLine[i]);}}
#else  /* ifdef INTLOCK_DEBUG */
#define PAL_protectEntry(level, cookie) PAL_osProtectEntry(level,cookie)
#endif /* ifdef INTLOCK_DEBUG */

/*! \def PAL_protectExit(level, cookie) 
	\brief Level - lock level required, cuurently supports only interrupt locking
	\brief cookie - flags for interrupt locking
	\brief Enables the interrupts and restores interrupt state saved in flags
	\param flags: interrupt state was saved in flags
 */
#ifdef INTLOCK_DEBUG
#define PAL_protectExit(level, cookie) { if(wlan_inIntLock == 0) \
                {printk("\n\n**** %s:%d Unbalanced int locks number.*****\n\n", __FUNCTION__, __LINE__);*(int *)0 = 0;}\
                 wlan_inIntLock --; PAL_osProtectExit(level,cookie);}
#else  /* ifdef INTLOCK_DEBUG */
#define PAL_protectExit(level, cookie) PAL_osProtectExit(level,cookie)
#endif /* ifdef INTLOCK_DEBUG */

/* Atomic handling */
#define PAL_atomic_set(var, val) \
	do { atomic_set(var, val); } while (0);
	
#define PAL_atomic_read(var, val) \
	do { val = atomic_read(var); } while (0);

	
/*! \def PAL_sleepShort(usecs)
	\brief Delay execution for usecs microseconds
		   Can be called from interrupt
			
	\param usecs number of microseconds to sleep		
   Note: if you know for sure what you're doing 
   #define notdef 
   before including linux/delay.h (I've warned you!)
 */
#define WLAN_PAL_sleepShort(usecs) mdelay(usecs)


/*! \brief don't remove n, since constant argumentis are perfectly possible */
#define msecs_to_jiffies(msecs)						\
	({ 								\
		unsigned long n = msecs; 				\
		if ((unsigned long) n < MAX_SCHEDULE_TIMEOUT / HZ) 	\
			n = (unsigned long)(n*HZ+999)/1000+1; 		\
		else n = MAX_SCHEDULE_TIMEOUT; 				\
		n; 							\
	})

	
/*! \def PAL_sleepLong(msecs)
	\brief Delay execution for msecs milliseconds
	
	\param msecs number of milliseconds to sleep
 */	
#ifdef INTLOCK_DEBUG
#define PAL_sleepLong(msecs) 							\
	({	signed long timeout = (msecs); 					\
        if(wlan_inIntLock) {printk("\n%s:%d Warning: Called sleep when interrupt was locked. Lock set at %s:%d\n\n",\
         __FUNCTION__, __LINE__, wlan_intLockFunc[wlan_inIntLock -1], wlan_intLockLine[wlan_inIntLock - 1]);}\
        if (unlikely(in_interrupt())) {printk("*****\n\n%s:%d sleep called from interrupt context.*****\n\n", \
                                                __FUNCTION__, __LINE__);*(int *)0=0;}\
        while( timeout ) { 						\
			set_current_state(TASK_INTERRUPTIBLE); 			\
			timeout = schedule_timeout(msecs_to_jiffies(timeout)); 	\
		} 								\
		WLAN_PAL_SOK; 							\
	})
#else /* ifdef INTLOCK_DEBUG */
#define PAL_sleepLong(msecs) 							\
	({	signed long timeout = (msecs); 					\
        while( timeout ) { 						\
			set_current_state(TASK_INTERRUPTIBLE); 			\
			timeout = schedule_timeout(msecs_to_jiffies(timeout)); 	\
		} 								\
		WLAN_PAL_SOK; 							\
	})
#endif /* ifdef INTLOCK_DEBUG */


STATUS PALSRV_tcbNew(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbDelete(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbCancel(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbSuspend(void);
STATUS PALSRV_tcbResume(void);

/*! \def  PALSRV_tcbQueueAct(timer, timeout, func, dat, periodic)
   \brief start a timer
   
   \param timer: the timer to be added
   \param timeout: when the timer expires in milliseonds
   \param func: the function to be executed when the timer expires
   \param dat: data to be passed to the function
   \param periodic: ignored

   Note: don't change dat to data !!!
   RETURNS WLAN_PAL_SOK/~WLAN_PAL_SOK
 */
#define PALSRV_tcbQueueAct(timer, timeout, func, dat, periodic) 	\
	({ 								\
	 	if( timeout ) { 					\
			(timer)->expires = jiffies + msecs_to_jiffies(timeout); 	\
			(timer)->data = (unsigned long)dat; 		\
			(timer)->function = (void (*)(unsigned long))func; 			\
			add_timer(timer); 				\
			WLAN_PAL_SOK; 					\
		} 							\
		else ~WLAN_PAL_SOK; 						\
	})


/*! \def PALSRV_getTick()
	\brief return number of clock ticks
 */
#define PALSRV_getTick() jiffies

/*! \def PALSRV_msec2ticks(msecs)
	\brief	convert number of milliseconds to number of clock ticks 
   
	\param  msecs: number of milliseconds
 */
#define PALSRV_msec2ticks(msecs) msecs_to_jiffies(msecs)

/*! \def PALSRV_tick2usecs
	\brief	convert ticks to micro seconds
   
	\param  ticks
 */
#if HZ == 100
#define PALSRV_ticks2usec(ticks) (ticks*10000)
#else
#define PALSRV_ticks2usec(ticks) (ticks/HZ*1000000)
#endif

/*! \def PAL_vmPhy2VirtAddr(phys, virt)
	\brief  map physical address to virtual
	\param  phys: address to remap
	\param  virt: the result of the remap
 */
#define PAL_vmPhy2VirtAddr(phys, virt) \
		({ virt = phys_to_virt((volatile void *)(phys)); WLAN_PAL_SOK; })

/*! \def  PAL_vmVirt2PhyAddr(virt, phys)
	\brief  map virtual address to physical
	\param  virt: address to remap
	\param  phys: the result of the remap
 */
#define PAL_vmVirt2PhyAddr(virt, phys) \
		({ phys = virt_to_phys((volatile void *)(virt)); WLAN_PAL_SOK; })

#ifdef TIWLAN_DEBUG
extern atomic_t pal_alloc_count;
#endif

/*! \def PAL_memAlloc(segId, size, alignment, memAddr)
	\brief  allocate memory
	\param 	size: how many bytes of memory are required
	\param 	segId: ignored
	\param 	alignment: ignored
	\param 	memAddr: ignored


   GFP_KERNEL - Allocate normal kernel ram.  May sleep.
 */
#ifdef TIWLAN_DEBUG
#define PAL_memAlloc(segId, size, alignment, memAddr) (atomic_add(1, &pal_alloc_count), kmalloc(size, GFP_KERNEL))
#else
#define PAL_memAlloc(segId, size, alignment, memAddr) kmalloc(size, GFP_KERNEL)
#endif
		
/*! \def PAL_memFree(segId, addr, numBytes)
	\brief free previously allocated memory
	
	\param addr: pointer returned by PAL_memAlloc
	\param segId  	ignored
	\param numBytes	ignored
 */
#ifdef TIWLAN_DEBUG
#define PAL_memFree(segId, addr, numBytes) (atomic_sub(1, &pal_alloc_count), kfree(addr) )
#else
#define PAL_memFree(segId, addr, numBytes) kfree(addr)
#endif


#define PAL_memMove(dest, src, count) memmove(dest, src, count)

/*! \def PAL_memCopy 
	\brief Copy one area of memory to another
	
	\param dest: Where to copy to
	\param src: Where to copy from
	\param count: The size of the area
 */
#define PAL_memCopy(dest, src, count) memcpy(dest, src, count)

/*! \def PAL_memSet
	\brief Fill a region of memory with the given value
	\param s: Pointer to the start of the area
	\param c: The byte to fill the area with
	\param count: The size of the area
 */
#define PAL_memSet(s, c, count) memset(s, c, count)


/*! \def PAL_memCmp
	\brief Compare two areas of memory
	\param cs: One area of memory
	\param ct: Another area of memory
	\param count: The size of the area
 */
#define PAL_memCmp(cs, ct, count) memcmp(cs, ct, count)


/* excerted from include/asm-mips/r4kcache.h */
#define cache_op(op,addr)                                               \
	__asm__ __volatile__(                                           \
	"       .set    noreorder               \n"                     \
	"       .set    mips3\n\t               \n"                     \
	"       cache   %0, %1                  \n"                     \
	"       .set    mips0                   \n"                     \
	"       .set    reorder"                                        \
	:                                                               \
	: "i" (op), "m" (*(unsigned char *)(addr)))


#define cache_align_addr(a, end, addr, cl_size) 	\
do { 							\
	a = (addr) & ~((cl_size) - 1); 			\
	end = ((addr) + (size) - 1) & ~((cl_size) - 1); \
} while(0)

/*! \def PAL_cacheInvalidate 
	\brief Invalidate cache address
	\param addrSpace: ignored
	\param addr: address to invalidate from
	\param size: size byte
 */

__inline__ static WLAN_PAL_Result PAL_cacheInvalidate(PAL_MemAddrSpace addrSpace, WLAN_Ptr addr, UINT32 size)
{
	unsigned long a, end, cl_size = 16;

		/* no need to do it twice, since anyway size will be verified beforehand*/
		cache_align_addr(a, end, (unsigned long)addr, cl_size);

		while (1) {
			cache_op(Hit_Invalidate_D, a);
			if (a == end) break;
			a += cl_size;
		}
	
	return WLAN_PAL_SOK;
}

#if defined(CONFIG_MIPS_PUMAS)
#define PAL_cacheFlush(a, b, c) 

#else
/*! \def PAL_cacheFlush
	\brief Flush cache address
	\param  addrSpace: ignored
	\param  addr: address to flush from
	\param  size: size byte
 */
__inline__ static WLAN_PAL_Result PAL_cacheFlush(PAL_MemAddrSpace addrSpace, WLAN_Ptr addr, UINT32 size)
{
	unsigned long a, end, cl_size = 16;

	 /* no need to do it twice, since anyway size will be verified beforehand*/
		cache_align_addr(a, end, (unsigned long)addr, cl_size);

		while (1) {
			cache_op(Hit_Writeback_D, a);
			if (a == end) break;
			a += cl_size;
		}
	
	return WLAN_PAL_SOK;
}


#endif /* CONFIG_MIPS_PUMA_S */


#define _min(a,b) ( (a) < (b) ? (a) : (b))
#define _max(a,b) ( (a) > (b) ? (a) : (b))


#endif /* __WLAN_PAL_H */

