/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda.h                                               
    \brief Device Driver Adaptation Layer for VxWorks Operating System
*/

/******************************************************************************/

#ifndef __TIWLAN_DDA_H
#define __TIWLAN_DDA_H


#include "wlan_pal.h"
#include "memMngr.h"
#include "drvDbDefs.h"
#include "dda_trace.h"
#include "mgmtSm.h"
#include "mainsm.h"
#include "rx.h"
#include "tx.h"
#include "whalAcxMbox.h"
#include "whalAcxRegs.h"
#include "dda_contexts.h"
#include "release.h"
#include "pal.h"
#include "whalAcxRegs.h"

#ifdef _DDA_MAIN_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

#define DDA_DEV_NAME		"wlan"
#define DDA_DEVNAME			"wlan%d"
#define DDA_DEV_UNIT_NUM	0
#define DDA_DEV_IP_ADR		"127.1.1.222"		   /* WLAN END dummy IP address */
#define EIOCTI   			_IOWR('i', 324, int)   /* define code for TI EIOC   */

#define DDA_TASK_PRIORITY   45


#define TIAP_LED_OFF		0
#define TIAP_LED_ON		    1
#define TIAP_LED_BLINK_OFF	2
#define TIAP_LED_BLINK_ON	3
#define TIAP_LED_FLASH		4

#ifdef DDA_WLAN_LED
#ifdef CONFIG_MIPS_AR7VWI
#define PAL_ConfigVWiLeds()

extern int tiap_led_oper_state, tiap_led_blinked;
extern UINT16 tiap_activity_led_mask;
extern __u8 tiap_gpio_owner;
UINT8 wdrv_dbGetApBridgeMode(void);
__inline__ static void tiwlan_ddaLedAction(int action)
{
	/* It mustn't be a function call in an inline function like this one.
	 * Performance is crucial.
	 */
	if (wdrv_dbGetApBridgeMode() || !tiap_gpio_owner)
		return;

	switch(action)
	{
    case TIAP_LED_OFF:            
#if 0
            whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
#else
            PAL_sysGpioOutBit( 6, 1);
#endif
			tiap_led_oper_state = action;
			break;
    case TIAP_LED_ON:            
#if 0
			whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
#else
            PAL_sysGpioOutBit( 6, 0);
#endif
			tiap_led_oper_state = action;
			break;
		case TIAP_LED_BLINK_OFF:
		case TIAP_LED_BLINK_ON:
			tiap_led_blinked = 1;
			break;
	}
}
#else
#define tiwlan_ddaLedAction(action)
#define PAL_ConfigVWiLeds()
#endif
#else
#define tiwlan_ddaLedAction(action)
#define PAL_ConfigVWiLeds()
#endif

#define GET_MAJOR_SW_VER SW_VERSION_MAJOR
#define GET_MINOR_SW_VER SW_VERSION_MINOR
  
#define ETH_ALEN 	6

typedef enum {
    TIWLAN_ALLOC_INIT_PHASE,
    TIWLAN_ALLOC_RUNTIME_PHASE
} tiwlan_allocPhase_t;

#define tiwlan_ddaRxFreeBuffer(x) 			/* nothing to be done in VxWorks */
								  
#define TIWLAN_DDA_BUFFER_FROM_TOKEN(token) token

#define boardInitAcx111Addr(RegAddr, MemAddr) \
				tiwlan_ddaInitPSPVlynq(RegAddr, MemAddr)

#define boardUnloadAcx111() tiwlan_ddaUnloadPSPVlynq()

#define XStr(x)  #x
#define XXstr(x) XStr(x)

PUBLIC UINT8 	tiap_gpio_owner; 			/* 1 for driver, 0 for fw */
PUBLIC STATUS 	tiwlan_ddaInit(void);
PUBLIC STATUS   tiwlan_ddaReInit(void);
PUBLIC STATUS 	tiwlan_ddaSuspend(void);
PUBLIC STATUS 	tiwlan_ddaResume(void);
PUBLIC STATUS 	tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr);
PUBLIC STATUS	tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr); 
PUBLIC STATUS 	tiwlan_ddaUnloadPSPVlynq(void);
PUBLIC UINT8 	tiwlan_ddaRand(void);
PUBLIC void 	tiwlan_ddaSrand(UINT16 seed);
PUBLIC STATUS 	tiwlan_ddaSend(mem_MSDU_T * pMSDU);
PUBLIC STATUS 	tiwlan_ddaReceive(mem_MSDU_T * pMSDU);
PUBLIC void 	tiwlan_ddaDeleteMacAddrFunc(char *mac);
PUBLIC STATUS 	tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, 
												UINT8 ** buf, tiwlan_allocPhase_t phase);
PUBLIC STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size );
PUBLIC STATUS 	tiwlan_dda_waitForWpa(void);
PUBLIC STATUS 	tiwlan_ddaStart(void);
PUBLIC STATUS 	tiwlan_ddaCleanup(void);
PUBLIC STATUS   tiwlan_dda_wpaGo(void);
PUBLIC STATUS 	tiwlan_ddaIsMemMngrInitNeeded(void);

#undef  PUBLIC

#endif /* __TIWLAN_DDA_H */

