/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_EventQueue.h                                               
    \brief This module implements DDA Event Queue                                                                          
*/
/******************************************************************************/

#ifndef _DDA_EVENT_QUEUE_H
#define _DDA_EVENT_QUEUE_H

#undef PUBLIC

#ifdef _DDA_EVENT_QUEUE_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

typedef UINT32 EVQ_EVENT_TYPE_T;

/* Event structure, variable size */
typedef	struct EvqEventS {
	struct EvqEventS * next;
	EVQ_EVENT_TYPE_T   type;
    UINT32             dataLen;
	void             * data;     /* variable length user defined data */

} EVQ_EVENT_S;

typedef EVQ_EVENT_S * EVQ_EVENT_P;


/* Event Pool structure */
typedef	struct {
    UINT8       * poolAddr;
    EVQ_EVENT_P * eventArray;
    UINT32        freeIndex;
    UINT32        dataSize;
    UINT32        poolSize;

} EVQ_POOL_S;

typedef EVQ_POOL_S * EVQ_POOL_P;

/* Event action funciton type */
typedef STATUS (*EVQ_ACTION_FUN_P) ( void * );

/* Event List structure */
typedef struct 
{
   EVQ_EVENT_P      head;
   EVQ_EVENT_P      tail;
   UINT32           count;
   UINT32           maxCount;
   EVQ_ACTION_FUN_P pActionFun;

} EVQ_LIST_S;

typedef EVQ_LIST_S * EVQ_LIST_P;

/* Event Queue structure */
typedef struct 
{
   SEM_ID        waitSem;
   UINT32        numOfEvents;
   UINT32        maxProcessNum;
   UINT32        eventDataSize;
   UINT32        numOfLists;
   EVQ_POOL_P	 pEventPool;
   EVQ_LIST_P *  eventLists;

} EVQ_QUEUE_S;

typedef EVQ_QUEUE_S * EVQ_QUEUE_P;



PUBLIC EVQ_QUEUE_P EVQ_Create( const UINT32 numOfLists, 
                               const UINT32 numOfEvents, 
                               const UINT32 maxProcessNum,
                               const UINT32 eventDataSize,
                               const EVQ_ACTION_FUN_P actionFun[] );

PUBLIC STATUS EVQ_PutEvent( const EVQ_QUEUE_P      pEvQueue,
                            const EVQ_EVENT_TYPE_T eventType,
                            const UINT32           listNum, 
                            const UINT8 *          message,
                            const UINT32           messageLen );


PUBLIC STATUS EVQ_WaitAndProcess( const EVQ_QUEUE_P  pEvQueue );


PUBLIC STATUS EVQ_Delete( const EVQ_QUEUE_P pEvQueue );


#undef PUBLIC
#endif  /* _DDA_EVENT_QUEUE_H */

