/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_contextsTest.c                                               
    \brief Contains routines for dda_EventQueue unit testing                                                                          
*/
/******************************************************************************/

#include "wdLib.h"
#include "semLib.h"

int testTask1id;
int testTask2id;

UINT32 SendCount1_MB     = 0;  
UINT32 SendCount1_Rx     = 0;  
UINT32 SendCount1_MainSm = 0;  


UINT32 SendCount2_MB     = 0;  
UINT32 SendCount2_Rx     = 0;  
UINT32 SendCount2_MainSm = 0;  


UINT32 SendCount3_MB     = 0;  
UINT32 SendCount3_Rx     = 0;  
UINT32 SendCount3_MainSm = 0;  

UINT32 SendFail_MB     = 0;  
UINT32 SendFail_Rx     = 0;  
UINT32 SendFail_MainSm = 0;  

WDOG_ID TestWd;
int TestWdDisable = 0;

/******************************************************************************/

/*! \brief 	Test function to put events into queue
	\return void
*/
/******************************************************************************/

void TestTask1Fun(void)
{
 STATUS status;
 char * message1 = "12345678";
 char * message2 = "abcdefgh";
 unsigned long  event_type = 0;

    for(;;)
    {
        status = EVQ_PutEvent( ddaTaskQueue, EVENT_MB_A, MB_LIST_A,
                               (UINT8 *) &event_type, sizeof(unsigned long) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "TestTask1Fun: Failed to put event\n" );
            SendFail_MB++;
        }
        else
        {
            SendCount1_MB++;
            event_type++;
        }

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_MAINSM_A, MAINSM_LIST_A,
                               (UINT8 *) message1, sizeof(mainsmMessage_T) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "TestTask1Fun: Failed to put event\n" );
            SendFail_MainSm++;
        }
        else
        {
            SendCount1_MainSm++;
        }

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_RX_A, RX_LIST_A,
                               (UINT8 *) message2, sizeof(RxMessage_T) );
        if (status != OK) 
        {
           DDA_LOG( DDA_LOG_ERR, "TestTask1Fun: Failed to put event\n" );
           SendFail_Rx++;
        }
        else
        {
           SendCount1_Rx++;
        }
    }
}

/******************************************************************************/

/*! \brief 	Test function to put events into queue
	\return void
*/
/******************************************************************************/

void TestTask2Fun(void)
{
 STATUS status;
 char * message1 = "12345678";
 char * message2 = "abcdefgh";
 unsigned long  event_type = 0;
 int i;

    for(;;)
    {
        
      for(i=0; i<20; i++)
      {
        taskDelay(1);

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_MB_A, MB_LIST_A,
                               (UINT8 *) &event_type, sizeof(unsigned long) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "TestTask2Fun: Failed to put event\n" );
            SendFail_MB++;
        }
        else
        {
            SendCount2_MB++;
            event_type++;
        }

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_MAINSM_A, MAINSM_LIST_A,
                               (UINT8 *) message1, sizeof(mainsmMessage_T) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "TestTask2Fun: Failed to put event\n" );
            SendFail_MainSm++;
        }
        else
        {
            SendCount2_MainSm++;
        }

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_RX_A, RX_LIST_A,
                               (UINT8 *) message2, sizeof(RxMessage_T) );
        if (status != OK) 
        {
           DDA_LOG( DDA_LOG_ERR, "TestTask2Fun: Failed to put event\n" );
           SendFail_Rx++;
        }
        else
        {
           SendCount2_Rx++;
        }
      }
    }
}

/******************************************************************************/

/*! \brief 	Test function to put events into queue from interrupt context
	\return void
*/
/******************************************************************************/

void TestWdFun(void)
{
 STATUS status;
 char * message1 = "12345678";
 char * message2 = "abcdefgh";
 unsigned long  event_type = 0;
 int i;

 if (TestWdDisable) return;

 for(i=0; i<10; i++)
 {
    status = EVQ_PutEvent( ddaTaskQueue, EVENT_MB_A, MB_LIST_A,
                           (UINT8 *) &event_type, sizeof(unsigned long) );
    if (status != OK) 
    {
        DDA_LOG( DDA_LOG_ERR, "TestWdFun: Failed to put event\n" );
        SendFail_MB++;
    }
    else
    {
        SendCount3_MB++;
        event_type++;
    }
 }
    
 for(i=0; i<15; i++)
 {
    status = EVQ_PutEvent( ddaTaskQueue, EVENT_MAINSM_A, MAINSM_LIST_A,
                           (UINT8 *) message1, sizeof(mainsmMessage_T) );
    if (status != OK) 
    {
        DDA_LOG( DDA_LOG_ERR, "TestWdFun: Failed to put event\n" );
        SendFail_MainSm++;
    }
    else
    {
        SendCount3_MainSm++;
    }

    status = EVQ_PutEvent( ddaTaskQueue, EVENT_RX_A, RX_LIST_A,
                           (UINT8 *) message2, sizeof(RxMessage_T) );
    if (status != OK) 
    {
       DDA_LOG( DDA_LOG_ERR, "TestWdFun: Failed to put event\n" );
       SendFail_Rx++;
    }
    else
    {
        SendCount3_Rx++;
    }
 }

 wdStart( TestWd,              /* watchdog ID                           */
          10,                   /* delay count, in ticks                 */
          (FUNCPTR) TestWdFun, /* routine to call on time-out           */
          0                    /* parameter with which to call routine  */
        );

}

/******************************************************************************/

/*! \brief 	Test initialization function
	\return void
*/
/******************************************************************************/

void DCX_TestInit( void )
{
    
    testTask1id= taskSpawn( "tTest1",               /* name of task             */
                           210,                     /* priority of task         */
                           0,                       /* task option word         */
                           10000,                   /* task stack size (bytes)  */
                           (FUNCPTR) TestTask1Fun,  /* entry point of DDA task  */
                           0,0,0,0,0,0,0,0,0,0      /* arguments to pass        */
                         );                       


    testTask2id= taskSpawn( "tTest2",               /* name of task             */
                           190,                     /* priority of task         */
                           0,                       /* task option word         */
                           10000,                   /* task stack size (bytes)  */
                           (FUNCPTR) TestTask2Fun,  /* entry point of DDA task  */
                           0,0,0,0,0,0,0,0,0,0      /* arguments to pass        */
                         );                       

 TestWd = wdCreate ();
 wdStart( TestWd,              /* watchdog ID                           */
          10,                  /* delay count, in ticks                 */
          (FUNCPTR) TestWdFun, /* routine to call on time-out           */
          0                    /* parameter with which to call routine  */
          );
 TestWdDisable = 0;

}

/******************************************************************************/

/*! \brief 	Test statistics show function
	\return void
*/
/******************************************************************************/


void DCX_TestShow(void)
{
    printf("Send     MB=%u Rx=%u MainSm=%u\n",
            SendCount1_MB, SendCount1_Rx, SendCount1_MainSm);
    printf("Send     MB=%u Rx=%u MainSm=%u\n",
            SendCount2_MB, SendCount2_Rx, SendCount2_MainSm);
    printf("Send     MB=%u Rx=%u MainSm=%u\n",
            SendCount3_MB, SendCount3_Rx, SendCount3_MainSm);
    printf("SendTot  MB=%u Rx=%u MainSm=%u\n",
            SendCount1_MB+SendCount2_MB+SendCount3_MB, 
            SendCount1_Rx+SendCount2_Rx+SendCount3_Rx, 
            SendCount1_MainSm+SendCount2_MainSm+SendCount3_MainSm);
    printf("SendFail MB=%u Rx=%u MainSm=%u\n",
            SendFail_MB, SendFail_Rx, SendFail_MainSm);

    printf("Recv     MB=%u Rx=%u MainSm=%u\n",
            RecvCount_MB, RecvCount_Rx, RecvCount_MainSm);

}

/******************************************************************************/

/*! \brief 	Stops test execution
	\return void
*/
/******************************************************************************/

void DCX_TestStop(void)
{
    taskSuspend( testTask1id );
    taskSuspend( testTask2id );
    TestWdDisable = 1;
}

/******************************************************************************/

/*! \brief 	Starts test execution
	\return void
*/
/******************************************************************************/

void DCX_TestStart(void)
{
    taskResume( testTask1id );
    taskResume( testTask2id );
    TestWdDisable = 0;
    TestWdFun();
}

DDA_DECLARE_WAIT_MECHANISM(DCX_sem);

/******************************************************************************/

/*! \brief 	Tests sleep with timeout function
	\return void
*/
/******************************************************************************/

void DCX_sleep(void)
{
    wdrv_DDA_sleep_timeout(&DCX_sem, 10000);
}

/******************************************************************************/

/*! \brief 	Tests awake function
	\return void
*/
/******************************************************************************/

void DCX_awake(void)
{
    wdrv_DDA_wake( &DCX_sem );
}


