/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_ioctl.h                                               
    \brief This module implements WLAN driver IOCTL functions for configuration 
		   and monitoring                                                                          
*/
/******************************************************************************/

#ifndef DDA_IOCTL_H
#define DDA_IOCTL_H

#ifdef _DDA_IOCTL_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

		/* define code for TI EIOC */
#define DDA_TI_WLAN_IOCTL 	_IOWR('i', 324, int)

/* utils */

#define ETH_ALEN 	6

#define LO_BYTE(x) ((x) & 0xff)
#define HI_BYTE(x) (((x) >> 8) & 0xff)

/* Maximum size of the ESSID string */
#define IW_ESSID_MAX_SIZE	32


enum {
	TIAP_PARAM_ERANGE = 1,	
	TIAP_PARAM_ERUNTIME,	
	TIAP_PARAM_ELOGICAL,	
	TIAP_PARAM_ERESOURCE, 
	TIAP_PARAM_EHWINIT,
	TIAP_PARAM_EPARAMNOTSET
};

/* Demiurg: WME */
#define WCFG_NUM_OF_QOS_CONFIGS 2

#define TIAP_PARAM_COUNTRY_STRING_LEN                    (3)
#define TIAP_PARAM_MIN_REPLEV                           (-31)
#define TIAP_PARAM_MAX_REPLEV                           (31)
#define TIAP_PARAM_MAX_RADIO_CAL                        (0xF)
#define TIAP_PARAM_MIN_POWER_CONS                       (-50)
#define TIAP_PARAM_MAX_POWER_CONS                       (50)
#define TIAP_PARAM_MIN_TX_POW_LEVEL                     (1)
#define TIAP_PARAM_MAX_TX_POW_LEVEL                     (5)
#define TIAP_PARAM_RTS_TRESHOLD_DEFAULT                 (2347)
#define TIAP_PARAM_RTS_TRESHOLD_MAX                     (4096)
#define TIAP_PARAM_MIN_FRAGMENTATION_TRESHOLD           (256)
#define TIAP_PARAM_MAX_FRAGMENTATION_TRESHOLD           (4096)
#define TIAP_PARAM_FRAG_TRESH_DEFAULT	                (2346)
#define TIAP_PARAM_MAX_DTIM_PERIOD                      (255)
#define TIAP_PARAM_ACCESS_LIST_SIZE	                	(64)
#define TIAP_PARAM_NUM_MULTIPLE_SSID                    (5) 
#define TIAP_PARAM_MAX_PRIMARY_SSIDS					(1)
#define TIAP_PARAM_MAX_SECONDARY_SSIDS					(3)
#define TIAP_PARAM_MAX_INTERNAL_SSIDS					(1)
#define TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL              \
	TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES * 		\
	TIAP_PARAM_MAX_RADIO_PARAM_POW_LEVELS
#define TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES    		(4)
#define TIAP_PARAM_MAX_RADIO_PARAM_POW_LEVELS    		(5)
#define TIAP_PARAM_MAX_VHCF_CLASSIFIERS                 (2)
#define TIAP_PARAM_RATES_MAX                            (8)
#define TIAP_PARAM_MAX_STATIONS                         (64)
#define TIAP_PARAM_MAX_NUM_OF_RANGES_A                  (8)
#define TIAP_PARAM_MAX_NUM_OF_RANGES_G                  (4)
#define TIAP_PARAM_NUM_TXPOW_LEVELS                     (5)
#define TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED               (2)
#define TIAP_PARAM_MIB_OPEN_SYSTEM                      (1)
#define TIAP_PARAM_MIB_SHARED_KEY                       (2)
#define TIAP_PARAM_MIN_DTIM_PERIOD						(1)
#define TIAP_PARAM_WONE_SINGLE_BAND						(1)
#define TIAP_PARAM_WONE_DUAL_BAND						(2)
#define TIAP_PARAM_RADIO_BAND_2_4GHZ 					(0)
#define TIAP_PARAM_RADIO_BAND_5GHZ 						(1)
#define TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ 				(2)
#define TIAP_PARAM_CCK_PBCC_RATES_MAX                   (5)
#define TIAP_PARAM_OFDM_RATES_MAX						(8)

#define TIAP_PARAM_PBCC_ALGO_DISABLED                   (0)
#define TIAP_PARAM_PBCC_ALGO_ENABLED                    (1)
#define TIAP_PARAM_PBCC_ALGO_LOCAL_STATIONS           	(2)
#define TIAP_PARAM_PBCC_ALGO_DYNAMIC                    (3)
#define TIAP_PARAM_PBCC_ALGO_ENHANCED_DYNAMIC         	(4)

#define TIAP_PARAM_PREAMBLE_MODE_LONG                   (0)
#define TIAP_PARAM_PREAMBLE_MODE_SHORT                  (1)
#define TIAP_PARAM_PREAMBLE_MODE_LOCAL_STATIONS 		(2)

#define TIAP_PARAM_SLOTTIME_MODE_DISABLED             	(0)
#define TIAP_PARAM_SLOTTIME_MODE_ENABLED               	(1)
#define TIAP_PARAM_SLOTTIME_MODE_LOCAL_STATIONS    		(2)
#define TIAP_PARAM_SLOTTIME_MODE_ENHANCED_DYNAMIC  		(3)

#define TIAP_PARAM_PROTECTION_MODE_DISABLED            	(0)
#define TIAP_PARAM_PROTECTION_MODE_ENABLED             	(1)
#define TIAP_PARAM_PROTECTION_MODE_LOCAL_STATIONS   	(2)
#define TIAP_PARAM_PROTECTION_MODE_DYNAMIC             	(3)
#define TIAP_PARAM_PROTECTION_MODE_ENHANCED_DYNAMIC 	(4)

#define TIAP_PARAM_PROTECTION_TYPE_RTS_CTS             	(0)
#define TIAP_PARAM_PROTECTION_TYPE_CTS_TOSELF           (1)
#define TIAP_PARAM_PROTECTION_TYPE_TI                   (2)

#define TIAP_PARAM_PROTECTION_TI_DISABLED              	(0)
#define TIAP_PARAM_PROTECTION_TI_ENABLED              	(1)

#define TIAP_PARAM_DOT11_CWMIN_DISABLED        			(0)
#define TIAP_PARAM_DOT11_CWMIN_ENABLED                 	(1)
#define TIAP_PARAM_DOT11_CWMIN_DOT11_MODE      			(2)

#define TIAP_PARAM_MIN_KEY_ID              				(0)
#define TIAP_PARAM_MAX_KEY_ID              				(3)

#define TIAP_PARAM_MAX_WD_INTVL							(30)

/* default values */
#define TIAP_PARAM_DEF_A_CHANNEL						(44)
#define TIAP_PARAM_DEF_CHANNEL							(11)
#define TIAP_PARAM_DEF_WONE_BEACON_PERIOD				(200)

enum {
	TIAP_PARAM_RATE_NOT_SUPPORTED,
	TIAP_PARAM_RATE_SUPPORTED,
	TIAP_PARAM_RATE_BASIC_SUPPORTED
};

enum {
	TIAP_PARAM_RETRY_LONG,
	TIAP_PARAM_RETRY_SHORT,
	TIAP_PARAM_LIFETIME_TX,
	TIAP_PARAM_LIFETIME_RX
};


typedef enum {
    TIAP_PARAM_BSS_BR_BLOCK,
    TIAP_PARAM_BSS_BR_ENABLE,
    TIAP_PARAM_BSS_BR_DROP
} tiap_param_bss_br_state_t;	

typedef enum {
    TIAP_PARAM_SECURITY_WPA	 	= 1,
    TIAP_PARAM_SECURITY_WEP	 	= 2,
    TIAP_PARAM_SECURITY_802_1X	= 4,
   	TIAP_PARAM_SECURITY_WPA2	= 8
} tiap_param_security_type_t;

typedef enum {
  TIAP_PARAM_RD_FCC     = 0x10,
  TIAP_PARAM_RD_IC      = 0x20,
  TIAP_PARAM_RD_ETSI    = 0x30,
  TIAP_PARAM_RD_SPAIN   = 0x31,
  TIAP_PARAM_RD_FRANCE  = 0x32,
  TIAP_PARAM_RD_MKK     = 0x40,
  TIAP_PARAM_RD_MKK1    = 0x41,
  TIAP_PARAM_RD_ISRAEL  = 0x43,
  TIAP_PARAM_RD_US      = 0x50,
  TIAP_PARAM_RD_WORLD   = 0x51,
  TIAP_PARAM_RD_CUSTOM  = 0x60,
  TIAP_PARAM_RD_DEFAULT = TIAP_PARAM_RD_WORLD
} tiap_param_cur_reg_domain_type_t;




/* state machine states */
typedef enum {
	TIAP_PARAM_SM_STATE_IDLE			= 0,
	TIAP_PARAM_SM_STATE_AUTH_KEY  		= 1,
	TIAP_PARAM_SM_STATE_AUTHENTICATED 	= 2,
	TIAP_PARAM_SM_STATE_WAIT_FOR_ASS_RES= 3,
	TIAP_PARAM_SM_STATE_ASSOCIATED		= 4,
	TIAP_PARAM_SM_STATE_AUTHORIZED		= 5,
	TIAP_PARAM_SM_NUM_STATES			= 6
} tiap_param_sm_states;


enum {
	/*basic*/
	TIAP_PARAM_RTS = 1,
	TIAP_PARAM_FRAG,
	TIAP_PARAM_REPLEV,		
	TIAP_PARAM_PRIV_OPTION,
	TIAP_PARAM_BEACON_PERIOD,
	TIAP_PARAM_DTIM_PERIOD,
	TIAP_PARAM_BSS_BRIDGE_STATE,
	TIAP_PARAM_WONE_TIME_MODE,
	TIAP_PARAM_RATE_FALLBACK,
	TIAP_PARAM_WONE_NUM,          /* 10 */
	TIAP_PARAM_RADIO_CAL,
	TIAP_PARAM_RADIO_CAL_INT,       
	TIAP_PARAM_ENERGY_DETECT,	
	TIAP_PARAM_HIDDEN_SSID,
	TIAP_PARAM_MULTI_DOMAIN_CAPAB,
	TIAP_PARAM_SPECTRUM_MGMT,
	TIAP_PARAM_LEGACY_STA,
	TIAP_PARAM_LOCAL_POWER_CONS,
	TIAP_PARAM_MAX_TX_POWER_LEVEL,
	TIAP_PARAM_OPER_STATE,         /* 20 */
	TIAP_PARAM_RADAR_DETECT_MODE,	
	TIAP_PARAM_AP_BRIDGE_MODE,
	TIAP_PARAM_WDS_ENABLE,
	TIAP_PARAM_EEPROM_LESS_ENABLE,

	/*complex*/
	TIAP_PARAM_EEPROM,
	TIAP_PARAM_AUTH_TYPE,
	TIAP_PARAM_WD,
	TIAP_PARAM_ACCESS_LIST,
	TIAP_PARAM_4X,			
	TIAP_PARAM_MULTI_SSID,
	TIAP_PARAM_CFG_POWER_TABLE,
	TIAP_PARAM_CHANNEL_RANGE,      /* 29	 */
	TIAP_PARAM_VHCF,
	TIAP_PARAM_WONE_ELEM,		
	TIAP_PARAM_STA_TBL,
	TIAP_PARAM_PRIV_REG_DOMAIN,
	TIAP_PARAM_CURR_REG_DOMAIN,
    TIAP_PARAM_INTERF_WD_CONFIG,
	TIAP_PARAM_NET_STATS,
	TIAP_PARAM_ESSID,
	TIAP_PARAM_TXPOWER,
	TIAP_PARAM_RETRY,
	TIAP_PARAM_COUNTRY_STR,
	TIAP_PARAM_NAME,
	TIAP_PARAM_QOS,
	TIAP_PARAM_BIT,
	TIAP_PARAM_HW_INTERF_CFG,
	TIAP_PARAM_DIE_ID,
	TIAP_PARAM_LAST
};

#define TIAP_PARAM_NUM_PARAMS TIAP_PARAM_LAST
#define TIAP_WDS_SSID  "\x1BTI_WDS" /*include non-printable character ESC*/

typedef struct {
	UINT16 algo;
	UINT8 enable;
} tiap_param_auth_algo_t;

typedef enum { 
	TIAP_PARAM_WD_ENABLE,
    TIAP_PARAM_WD_INTERF,  /* The whole ACI/NB interference WA in driver */
	TIAP_PARAM_WD_RXFREE, 
	TIAP_PARAM_WD_BEACONS,
    TIAP_PARAM_WD_TXSTUCK,
    TIAP_PARAM_WD_FCSERR,
	TIAP_PARAM_WD_ACI,
	TIAP_PARAM_WD_NB
} tiap_param_wd_cmd_t;


typedef struct {
	tiap_param_wd_cmd_t cmd;
       	union {
		UINT16 val16;
		UINT8  val8;
	} value;
} tiap_param_wd_t;

/* Interference Watch Dog state machine configuration enumerator*/
typedef enum {

	TIAP_PARAM_PD_LOW,
	TIAP_PARAM_PD_HIGH,
	TIAP_PARAM_ACI_TIMEOUT,
	TIAP_PARAM_ACI_EVENT_THR,
	TIAP_PARAM_NB_TIMEOUT,
	TIAP_PARAM_ACI_RATE_THR,
	TIAP_PARAM_MEDIUM_BUSY_THR
}tiap_param_interf_cmd_t;

/* Interference Watch Dog state machine configuration parameter type*/
typedef struct {
	tiap_param_interf_cmd_t cmd;
	union {
		UINT16 	val16;
		UINT32 	val32;
	} value;
}tiap_param_interference_wd_t;

/* HW interference handling configuration parameters */
typedef enum {

	TIAP_PARAM_INPUT_CCA,
	TIAP_PARAM_TX_CCA,
	TIAP_PARAM_QUALIF_CCA,
	TIAP_PARAM_STOMP_RX,
	TIAP_PARAM_STOMP_TX,
}tiap_param_hw_interf_cmd_t;

/* HW interference handling parameter type*/
typedef struct {
	tiap_param_hw_interf_cmd_t cmd;
	__u8	value;
}tiap_param_hw_interf_t;

typedef enum {
	TIAP_PARAM_ACCESS_SET_TYPE,
	TIAP_PARAM_ACCESS_LST,
	TIAP_PARAM_ACCESS_ADD,
	TIAP_PARAM_ACCESS_DEL_BY_INDEX,
	TIAP_PARAM_ACCESS_DEL_BY_ADDR,
	TIAP_PARAM_ACCESS_RESET
} tiap_param_access_list_cmd_t;

typedef struct {
	tiap_param_access_list_cmd_t cmd;
	__s8 enabled;
	__s8 type;
	__s8 index;
	__s8 list[TIAP_PARAM_ACCESS_LIST_SIZE][ETH_ALEN];
} tiap_param_access_list_t;

#define TIAP_PARAM_DIE_ID_SIZE  (2 * sizeof(__u32))

typedef struct {
	__u32 die_id_lo;
	__u32 die_id_hi;
}tiap_param_die_id_t;


#define TIAP_PARAM_EEPROM_MAX_SIZE 1000

typedef struct {
	char image[TIAP_PARAM_EEPROM_MAX_SIZE];
	__u32 len;
	tiap_param_die_id_t die_id;

}tiap_param_eeprom_t;


typedef enum {
	TIAP_PARAM_4X_FEATSET,
	TIAP_PARAM_4X_4X,
	TIAP_PARAM_4X_CAT,
	TIAP_PARAM_4X_PKTBURST,
	TIAP_PARAM_4X_MIXMOD
} tiap_param_4x_cmd_t;


typedef struct {
	tiap_param_4x_cmd_t cmd;
	UINT8 value;
} tiap_param_4x_t;


typedef enum
{
	TIAP_PARAM_SSID_NON_ACTIVE,
	TIAP_PARAM_SSID_PRIMARY,
	TIAP_PARAM_SSID_SECONDARY,
	TIAP_PARAM_SSID_INTERNAL
}tiap_param_ssid_type_t;

typedef struct
{
	__s8 ssid[IW_ESSID_MAX_SIZE+1];
	__u8 sec_profile_index; /* index of corresponding security profile */
	__u16 vlan;
	tiap_param_ssid_type_t ssid_type;
}tiap_param_multi_ssid_elem_t;

typedef struct {
	__u16 secondary_enabled;
	tiap_param_multi_ssid_elem_t tbl[TIAP_PARAM_NUM_MULTIPLE_SSID];
} tiap_param_multi_ssid_t;

enum {
	TIAP_PARAM_PWRTBL = 1
};



typedef struct {
	UINT32 type;
	__s8 param[TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL];
} tiap_param_radio_pwrtbl_t;

typedef struct {
	UINT32 mask;
	tiap_param_radio_pwrtbl_t tbl;
} tiap_param_radio_t;

typedef struct {
	UINT8 start;
	UINT8 stop;
	UINT8 step;
	__s8 power_limit;
} tiap_param_channel_rang_elem_t;



typedef struct {
	UINT8 band;
	tiap_param_cur_reg_domain_type_t type;
} tiap_param_cur_reg_domain_t;


/* Demiurg: WME */
// TODO

typedef struct {
	__u8 cwMin, aifsn, shortRetryLimit, longRetryLimit, ackPolicy;
	__u16 cwMax, txopLimit11b, txopLimit11g, txOpContinuation;
	__u32 txMaxLifetime;
} tiap_param_qos_ac_t;

/* Duplicates definitions in QoS/QoSACParams.h */
#define TIAP_PARAM_QOS_MAX_CP 64
/*
typedef struct {
	__u8 codePoint, vlanPriority;
} tiap_param_qos_diffserv_map_t;
*/
typedef struct {
	__u8 numOfCP;
	__u8 map[TIAP_PARAM_QOS_MAX_CP];
} tiap_param_qos_diffserv_t;


typedef struct {
	__u8 enabled;
	__u8 classifierMode;
	//	wdrv_QoSClassifierMode_t mode;
	tiap_param_qos_ac_t edcf_info[2][4];
   	tiap_param_qos_diffserv_t diffserv;
} tiap_param_qos_t;

typedef enum {
	TIAP_PARAM_QOS_CLASSIFIER_MODE_VLAN = 0,
	TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV,
	TIAP_PARAM_QOS_CLASSIFIER_MODE_UDP_PORT,
	TIAP_PARAM_QOS_CLASSIFIER_MODE_UNDEFINED
} tiap_param_qos_classifier_mode_t;

enum {
	TIAP_PARAM_QOS_AC_AP_PARAMS = 0,
	TIAP_PARAM_QOS_AC_STA_PARAMS = 1
}; 

enum {
	TIAP_PARAM_QOS_CLASSIFIER_AC_BE = 0,
	TIAP_PARAM_QOS_CLASSIFIER_AC_BK = 1,
	TIAP_PARAM_QOS_CLASSIFIER_AC_VI = 2,
	TIAP_PARAM_QOS_CLASSIFIER_AC_VO = 3,
	TIAP_PARAM_QOS_CLASSIFIER_BROADCAST = 4,
	TIAP_PARAM_QOS_CLASSIFIER_AC_UNDEFINED = -1
};

enum {
	TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE = 0,
	TIAP_PARAM_QOS_ACK_POLICY_NO_ACK = 1
};

typedef struct {
	UINT8 type;
	union {
		tiap_param_channel_rang_elem_t a[TIAP_PARAM_MAX_NUM_OF_RANGES_A];
		tiap_param_channel_rang_elem_t g[TIAP_PARAM_MAX_NUM_OF_RANGES_G];
	} rng;
	UINT8 ranges_nr;
} tiap_param_channel_rang;

enum {
    TIAP_PARAM_RANG_G,
	TIAP_PARAM_RANG_A
};

typedef enum {
	vhcf_prot_none,
	vhcf_prot_tcp,
	vhcf_prot_udp
} tiap_param_vhcf_proto_t;

typedef struct {
	tiap_param_vhcf_proto_t proto;
	UINT16  dest_port;
	UINT32  dest_ip;
} tiap_param_vhcf_class_t;



typedef struct {
	UINT8    enable;
	tiap_param_vhcf_class_t classifier[TIAP_PARAM_MAX_VHCF_CLASSIFIERS];
} tiap_param_vhcf_t;


typedef struct {
	UINT8 rates[TIAP_PARAM_RATES_MAX];
	UINT8 len;
} tiap_param_rates_t;


enum {
	TIAP_PARAM_D11MODE_MIXED,
	TIAP_PARAM_D11MODE_B,
	TIAP_PARAM_D11MODE_B_PLUS,
	TIAP_PARAM_D11MODE_OFDM,
	TIAP_PARAM_D11MODE_A
};

typedef union
{
	tiap_param_rates_t b_legacy;
	tiap_param_rates_t ofdm_legacy;
	tiap_param_rates_t ofdm_extend;
	UINT8 ch;
	UINT8 band;
	UINT8 pbcc_mode;
	UINT8 rate_adapt;
	UINT8 pream_mode;
	UINT8 slot_mode;
	UINT8 erp_mode;
	UINT8 erp_type;
	UINT8 erp_ti;
	UINT8 dot11_mode;
	UINT8 cwmin_mode;
	UINT8 usage_time;
} tiap_param_wone_elem_t;

typedef enum {
	wone_oper_rate,
	wone_oper_add,
	wone_oper_ofdm,
	wone_ch,
	wone_band,
	wone_pbcc,
	wone_rate_adapt,
	wone_pream_mode,
	wone_slot_mode,
	wone_erp_mode,
	wone_erp_type,
	wone_erp_ti,
	wone_dot11_mode,
	wone_cwmin_mode,
	wone_usage_time 
} tiap_param_wone_elem_type_t;

typedef struct {
	UINT8 index;
	tiap_param_wone_elem_type_t type;
	tiap_param_wone_elem_t elem;
} tiap_param_wone_t;



typedef struct {
	__s8 hwaddr[ETH_ALEN];
	tiap_param_sm_states state;
	__s8 ssid[IW_ESSID_MAX_SIZE];
	UINT8 active_rate;
	__u32 txCounter;
	__u32 rxCounter;
	__u32 wme;
} tiap_param_statbl_t;



typedef enum {
        txpower_g,
        txpower_a,
} tiap_param_txpower_hr;


typedef struct {
	UINT8 hr;
	__s8  txpower[TIAP_PARAM_NUM_TXPOW_LEVELS];
} tiap_param_txpower_t;

enum {
	BAND_B,
	BAND_A,
	BAND_DEFAULT
};

typedef struct {
	UINT8 band;
	UINT8 domain;
} tiap_param_priv_reg_domain_t;

typedef struct {
	int 	buf_len, cmd;
	UINT32 	param1, param2, param3, param4, param5, param6, param7, param8;
	UINT8 	mac[ETH_ALEN];
	char  * buf;

} DDA_IOCTL_GENERIC_PAR_S;

typedef enum {
	TIAP_GENERIC_IOCTL_GET_TX_INFO,
	TIAP_GENERIC_IOCTL_GET_TX_QUEUE,
	TIAP_GENERIC_IOCTL_TST_CHALLENGE,
	TIAP_GENERIC_IOCTL_TST_FINDSTA,
	TIAP_GENERIC_IOCTL_TST_FIND_INS,
	TIAP_GENERIC_IOCTL_GET_EER,		/*5*/
	TIAP_GENERIC_IOCTL_GET_MAC_REGREAD,
	TIAP_GENERIC_IOCTL_GET_PHY_REGREAD,
	TIAP_GENERIC_IOCTL_TRACE_PRINT,
	TIAP_GENERIC_IOCTL_WLAN_PARAMS,
	TIAP_GENERIC_IOCTL_DOWNLOAD_FW,
	TIAP_GENERIC_IOCTL_SET_BURST_LIMIT,
	TIAP_GENERIC_IOCTL_GET_BURST_LIMIT,
    TIAP_GENERIC_IOCTL_SEND_WDS_PKT,
    TIAP_GENERIC_IOCTL_WDS_LINK_ADD,
    TIAP_GENERIC_IOCTL_WDS_LINK_REMOVE,
    TIAP_GENERIC_IOCTL_WDS_STAT_GET,
    TIAP_GENERIC_IOCTL_WDS_IFC_STAT_GET,
    TIAP_GENERIC_IOCTL_WDS_DRV_STAT_GET,
    TIAP_GENERIC_IOCTL_WDS_BRG_STAT_GET,
    TIAP_GENERIC_IOCTL_BSS_SERVICE_STATE,
    TIAP_GENERIC_IOCTL_AFTER_GO_FLAG_GET,
	TIAP_GENERIC_IOCTL_SET_GPIO_OWNER,
	#ifdef REPORT_DEBUG
	TIAP_GENERIC_IOCTL_GET_LOGGED_CHARS,
	TIAP_GENERIC_IOCTL_SET_LOGGED_CHARS
	#endif
} generic_ioctl_cmd_t;


typedef struct {
	UINT32 rx_frames;
	UINT32 tx_frames;
	UINT32 rx_multicast_frames;
	UINT32 tx_multicast_frames;
} tiap_param_net_bss_stats;

typedef struct {
	UINT32 tx_mpdus;
	UINT32 tx_msdus;
	UINT32 tx_multicast_msdus;
	UINT32 tx_failed_msdus;
	UINT32 tx_retry_msdus;
	
	UINT32 rx_mpdus;
	UINT32 rx_msdus;
	UINT32 rx_multicast_msdus;
	UINT32 rx_fcs_error_mpdus;
	UINT32 rx_mic_failure_msdus;
	UINT32 rx_decrypt_error_mpdus;
} tiap_param_net_wireless_stats;


typedef struct {
	tiap_param_net_bss_stats bss;
	tiap_param_net_wireless_stats wireless;
} tiap_param_net_stats;

typedef struct 
{
	UINT32  len;
	char    essid[ IW_ESSID_MAX_SIZE + 1 ];
} tiap_param_essid_t;

typedef struct 
{
	char    name[IFNAMSIZ];

} tiap_param_drvname_t;

typedef struct 
{
	char    name[TIAP_PARAM_COUNTRY_STRING_LEN];

} tiap_param_country_t;

typedef struct 
{
	UINT32  flags;
	UINT32  value;

}tiap_param_retry_t;

/* Structs to pass BIT commands arguments to Driver through IOCTLs */
typedef enum {
    BIT_FCC,
    BIT_TELEC
} bitCmdType_e;

typedef struct {
    bitCmdType_e    cmdType;
	__u32           band;
	__u32           channel;
	__u32           rate;
	__s32           mode;
} tiap_param_bit_args_t;
/* End of BIT commands structs */

#define GENERIC_IOCTL_BUF_SIZE 20000

/* It's the most easiest way to compute the size of the largest struct.
   Since wireless extensions use the same size for both the real ioctl
   and the sub-ioctls, we must compute the largests size of all structs.
   In the future, if you need to add support for another struct, simply
   add it to the struct below.
 */
typedef union {
	tiap_param_auth_algo_t		auth_tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_param_wd_t			    wd;
	tiap_param_interference_wd_t interf_wd_conf;
	tiap_param_access_list_t	access_list;
	tiap_param_4x_t			    _4x;
	tiap_param_multi_ssid_t		multi_ssid;
	tiap_param_radio_t		    radio;
	tiap_param_channel_rang		ch_rang;
	tiap_param_vhcf_t		    vhcf;
	tiap_param_wone_t		    wone;
	tiap_param_statbl_t		    sta_tbl[TIAP_PARAM_MAX_STATIONS];
	tiap_param_txpower_t		txpow;
	tiap_param_priv_reg_domain_t priv_regdomain;
	tiap_param_cur_reg_domain_t	cur_regdomain;
	tiap_param_net_stats		net_stats;
	tiap_param_essid_t			essid;
	tiap_param_drvname_t		name;
	tiap_param_country_t		country;
	tiap_param_retry_t			retry;
	tiap_param_qos_t 			qos;
    tiap_param_bit_args_t       bit;
	tiap_param_hw_interf_t		hw_interf_param;
	tiap_param_eeprom_t 		eeprom;
	tiap_param_die_id_t			die_id;
	
} TI_IOCTL_PARAM_T;

typedef struct
{
	INT32 				subcmd;
	TI_IOCTL_PARAM_T    params;

} DDA_IOCTL_COMPLEX_PAR_S;


/* Wireless extensions version 16 does not allow more then IW_PRIV_SIZE_MASK args.
   Each arg has it own size, since union the_largest_struct larger than IW_PRIV_SIZE_MASK,
   we do the following trick, we say that we have sizeof(union the_largest_struct)/sizeof(int)
   number of args, each of which has sizeof(int) (IW_PRIV_TYPE_INT|IW_PRIV_SIZE_FIXED).
 */
#define COMPLEX_PARAM_SIZE IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | (sizeof(union the_largest_struct)/sizeof(int))

typedef enum
{
	DDA_IOCTL_TYPE_GENERIC_A = 1,
	DDA_IOCTL_TYPE_BASIC_A,
	DDA_IOCTL_TYPE_COMPLEX_A,
	DDA_IOCTL_TYPE_GO_A

} DDA_IOCTL_DATA_TYPE_E;

typedef enum
{
	DDA_IOCTL_CMD_SET_A = 1,
	DDA_IOCTL_CMD_GET_A,

} DDA_IOCTL_CMD_TYPE_E;


typedef struct
{
	INT32	parType;
	INT32  	parValue;

}  DDA_IOCTL_BASIC_PAR_S;

typedef struct
{
	UINT8 	dev_addr[ETH_ALEN];

}  DDA_IOCTL_GO_PAR_S;

typedef union
{
	DDA_IOCTL_GENERIC_PAR_S	genericPar;
	DDA_IOCTL_BASIC_PAR_S	basicPar;
	DDA_IOCTL_COMPLEX_PAR_S complexPar;
	DDA_IOCTL_GO_PAR_S 		goPar;

} DDA_IOCTL_COM_DATA_S;

typedef struct 
{
	DDA_IOCTL_CMD_TYPE_E  	cmd;
    DDA_IOCTL_DATA_TYPE_E	dataType;
	DDA_IOCTL_COM_DATA_S	dataBuf;

}  DDA_IOCTL_DATA_S;

PUBLIC UINT16 configTxopLimit;

PUBLIC STATUS DDA_InitIoctl(void);
PUBLIC STATUS DDA_DeleteIoctl(void);
PUBLIC STATUS DDA_doIoctl(const DDA_IOCTL_DATA_S * pData );
PUBLIC STATUS DDA_GetInitStatus( void );

#undef PUBLIC
#endif  /* DDA_IOCTL_H */

