/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_proc.c                                               
    \brief This module implementats functions for linux style proc support of
		   managemnet application                                                                          
*/
/******************************************************************************/

#define _DDA_PROC_C


/***********************************************************
	includes
*******************************************************/

#include "wlan_pal.h"
#include "dda.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "whalApi.h"
#include "mgmtSm.h"
#include "mainsm.h"
#include "mgmtSsidHandler.h"
#include "whalDefs.h"
#include "bssbridge.h"
#include "dda_contexts.h"
#include "mgmtParser.h"
#include "powerSave.h"
#include "Test/mgmtTst.h"
#include "release.h"
#include "dda_ioctl.h"
#include "dda_proc_fs_defs.h"
#include "dda_proc.h"


/***********************************************************
	defs
*******************************************************/

typedef int	(*PROC_FUNC) ( char * buf,  int buf_len);

extern UINT8 whal_ParamsGetRadioChannel(UINT8 WoneIndex);

#define DDA_PROC_ENRTY_NUM    128
#define CMD_NOT_SUPPORT_STR   "cmd not supported."

#undef printf
#undef puts

/***********************************************************
	static variables
*******************************************************/


/***********************************************************
	froc function decleration
*******************************************************/

/******* MAIN funtions *********/

int proc_main_read_ver( char * buf, int buf_len);

/******* BIT functions *********/
int proc_bit_write_fcc( char * buf, int buf_len);
int proc_bit_write_telec( char * buf, int buf_len);
int proc_bit_write_txCfg( char * buf, int buf_len);
int proc_bit_write_rxCfg( char * buf, int buf_len);
int proc_bit_write_clrStat( char * buf, int buf_len);
int proc_bit_write_txStop( char * buf, int buf_len);
int proc_bit_write_bitStop( char * buf, int buf_len);
int proc_bit_read_rxStat( char * buf, int buf_len);
int proc_bit_read_txStat( char * buf, int buf_len);
int proc_bit_read_test( char * buf, int buf_len);

/************* mgmt func *************/
int proc_mgmt_read_staTable( char * buf, int buf_len);
int proc_mgmt_read_staStatistics( char * buf, int buf_len);
int proc_mgmt_read_staHash( char * buf, int buf_len);
int proc_mgmt_read_staRates( char * buf, int buf_len);
int proc_mgmt_read_4xtable( char * buf, int buf_len);
int proc_mgmt_write_deAuth( char * buf, int buf_len);
int proc_mgmt_write_disAss( char * buf, int buf_len);
int proc_mgmt_read_counters( char * buf, int buf_len);
int proc_mgmt_read_show( char * buf, int buf_len);
int proc_mgmt_write_aging( char * buf, int buf_len);
int proc_mgmt_write_setLog( char * buf, int buf_len);
/* mgmt test func */
int proc_mgmt_write_null( char * buf, int buf_len);
int proc_mgmt_write_TstAuth1( char * buf, int buf_len);
int proc_mgmt_write_TstAuth3( char * buf, int buf_len);
int proc_mgmt_write_TstAss( char * buf, int buf_len);
int proc_mgmt_write_TstReass( char * buf, int buf_len);
int proc_mgmt_write_TstDeAuth( char * buf, int buf_len);
int proc_mgmt_write_TstDisAss( char * buf, int buf_len);
int proc_mgmt_write_TstClass3( char * buf, int buf_len);
int proc_mgmt_write_TstRemStat( char * buf, int buf_len);
int proc_mgmt_write_TstReset( char * buf, int buf_len);


/************** wpa func **********/
int proc_wpa_write_TstReset( char * buf, int buf_len);

/************** privacy func **********/
int proc_priv_read_notify( char * buf, int buf_len);
int proc_priv_read_cfg( char * buf, int buf_len);
int proc_priv_read_wepkeys( char * buf, int buf_len);
int proc_priv_read_counters( char * buf, int buf_len);
int proc_priv_write_invoke( char * buf, int buf_len);
int proc_priv_write_exclude( char * buf, int buf_len);

/***********  serv func*/
int proc_srv_read_rates( char * buf, int buf_len);
int proc_srv_read_counters( char * buf, int buf_len);
int proc_srv_read_radioFreq( char * buf, int buf_len);  
int proc_srv_read_radioParamas( char * buf, int buf_len);  
int proc_srv_read_power_level( char * buf, int buf_len);  

/************** show func **********/
int proc_show_read_bss( char * buf, int buf_len);
int proc_show_read_ps( char * buf, int buf_len);
int proc_show_read_lanMib( char * buf, int buf_len);
int proc_show_read_lanStat( char * buf, int buf_len);
int proc_show_read_lanPool( char * buf, int buf_len);
int proc_show_read_mgmtStat( char * buf, int buf_len);
int proc_show_read_RA( char * buf, int buf_len);
int proc_show_read_txStat( char * buf, int buf_len);

/*************  hal func ********/
int proc_whal_read_fwVer( char * buf, int buf_len);
int proc_whal_read_acxParams( char * buf, int buf_len);
int proc_whal_read_rxInfo( char * buf, int buf_len);
int proc_whal_read_rxQueue( char * buf, int buf_len);
int proc_whal_read_Intr( char * buf, int buf_len);
int proc_whal_read_Mbox( char * buf, int buf_len);
int proc_whal_read_acxStat( char * buf, int buf_len);
int proc_whal_read_restratStat( char * buf, int buf_len);
int proc_whal_read_led( char * buf, int buf_len);
int proc_whal_read_fwVer_only( char * buf, int buf_len);
int proc_whal_read_radioVer_only( char * buf, int buf_len);
int proc_whal_read_eepromVer_only( char * buf, int buf_len);
int proc_whal_write_led( char * buf, int buf_len);
int proc_whal_show_InterfWAparams( char* buf,  int count );
int proc_whal_read_MboxErrTrace( char * buf, int buf_len);
int proc_whal_read_mediumUsage( char * buf, int buf_len);

/************* hal diag ************/
int proc_whal_read_eed( char * buf, int buf_len);
int proc_whal_write_eew( char * buf, int buf_len);
int proc_whal_write_trsh( char * buf, int buf_len);
int proc_whal_write_BssPs( char * buf, int buf_len);
int proc_whal_read_Regs( char * buf, int buf_len);
int proc_whal_write_reg( char * buf, int buf_len);
int proc_whal_read_reg( char * buf, int buf_len);
int proc_whal_write_macReg( char * buf, int buf_len);
int proc_whal_write_rx_promis( char * buf, int buf_len);
int proc_whal_write_sniffmod( char * buf, int buf_len);
int proc_whal_write_FwSelc( char * buf, int buf_len);
int proc_whal_write_QueueSizeType( char * buf, int buf_len);
int proc_whal_write_SelcRadionum( char * buf, int buf_len);
int proc_whal_write_NvByte( char * buf, int buf_len);
int proc_whal_write_traceEn( char * buf, int buf_len);
int proc_whal_read_traceRxStat( char * buf, int buf_len);
int proc_whal_read_RxBufs( char * buf, int buf_len);

/*4x*/
int proc_4x_write_max_concatsize( char * buf, int buf_len);
int proc_4x_show_max_concatsize( char * buf, int buf_len);

int exec_proc_entry(  DPR_PROC_DIR_NAME_E dir, int entry, char *buf, int buf_len);

typedef int (*PROC_FUN_P) ( char *buf, int buf_len);


#define MAX_PROC_DIRS 		WLAN_PROC_DIR_NAME_LAST		/* 10 */
#define MAX_PROC_ENTRIES    35								

static PROC_FUN_P procFunctionTable[ MAX_PROC_DIRS ] [ MAX_PROC_ENTRIES ] = 
{
	/* base */
	{ 	
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* BIT */
	{ 	
		proc_bit_write_fcc, 	proc_bit_write_telec, 	proc_bit_write_txCfg,
		proc_bit_write_rxCfg,   proc_bit_write_clrStat, proc_bit_write_txStop,
		proc_bit_write_bitStop,	proc_bit_read_rxStat, 	proc_bit_read_txStat,
		NULL, NULL,	NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* MGMT */
	{ 	
		proc_mgmt_write_deAuth,		    proc_mgmt_write_disAss,     proc_mgmt_read_staTable,
        proc_mgmt_read_staStatistics,   proc_mgmt_read_4xtable,		proc_mgmt_read_staHash,	
        proc_mgmt_read_staRates,		proc_mgmt_read_counters,	proc_mgmt_write_setLog,	
        proc_mgmt_write_aging,  		proc_mgmt_read_show,		proc_mgmt_write_null,	
        proc_mgmt_write_TstAuth1,		proc_mgmt_write_TstAuth3, 	proc_mgmt_write_TstAss, 
        proc_mgmt_write_TstReass,		proc_mgmt_write_TstDeAuth,	proc_mgmt_write_TstDisAss, 
        proc_mgmt_write_TstClass3,		NULL,						NULL,					
        NULL,                   		proc_mgmt_write_TstRemStat,	proc_mgmt_write_TstReset, 
        NULL, NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
	},

	/* WPA */
	{ 	
		proc_wpa_write_TstReset,	NULL,					NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* Privacy */
	{
		proc_priv_read_cfg,			proc_priv_read_notify,	proc_priv_read_wepkeys,
		proc_priv_read_counters, 	proc_priv_write_invoke, proc_priv_write_exclude,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* Serv */
	{
		NULL/*proc_srv_read_wlan_params*/,proc_srv_read_rates,	proc_srv_read_counters,
		proc_srv_read_radioFreq,	proc_srv_read_radioParamas,	proc_srv_read_power_level, 
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* Show */
	{
		proc_show_read_bss,			proc_show_read_ps,		proc_show_read_txStat,
		proc_show_read_lanMib,		proc_show_read_lanStat,	proc_show_read_lanPool,
		proc_show_read_mgmtStat,	proc_show_read_RA,		NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* HAL */
	{
		proc_whal_read_fwVer,		    proc_whal_read_acxParams,	    proc_whal_read_rxInfo,
		proc_whal_read_rxQueue,		    proc_whal_read_Intr,		    proc_whal_read_Mbox,		
        proc_whal_read_acxStat,         proc_whal_read_restratStat,	    proc_whal_read_led,
		proc_whal_read_eed,			    proc_whal_write_eew,		    proc_whal_write_trsh,
		proc_whal_read_fwVer_only,	    proc_whal_read_radioVer_only,	proc_whal_read_eepromVer_only,
		proc_whal_write_BssPs,		    proc_whal_read_Regs,		    proc_whal_write_reg,
		proc_whal_write_macReg,		    proc_whal_write_rx_promis,	    proc_whal_write_sniffmod,
		proc_whal_write_FwSelc,		    proc_whal_write_QueueSizeType,  proc_whal_write_NvByte,		
        proc_whal_write_SelcRadionum,   proc_whal_write_traceEn,	    proc_whal_read_traceRxStat,	
        proc_whal_read_RxBufs,          proc_whal_show_InterfWAparams,  proc_whal_read_MboxErrTrace,
        proc_whal_read_mediumUsage,     NULL, NULL, NULL,
	},

	/* 4X */
	{
		proc_4x_write_max_concatsize, 	proc_4x_show_max_concatsize,	NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	},

	/* Prod */
	{
		proc_main_read_ver, 	NULL,	NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL,
		NULL, NULL, NULL, NULL,
	}

};

/*******************************************************************************
		FUNCTION IMPL.
*******************************************************************************/

/******************************************************************************/

/*! \brief 	Executes function from proc table specified by dir and entry number
	\param	dir 	- directory of proc table
	\param	entry 	- entry of proc table
	\param 	buf		- pointer to output buffer containing read data	
	\param 	buf_len - size of output data
	
	\return number of bytes copied to buf or ERROR
*/
/******************************************************************************/

int dda_read_proc_entry( const int dir, const int entry, 
						 const char *buf, const int buf_len)
{
	if( buf_len <= 0 || buf==NULL)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}
 
	return exec_proc_entry( dir, entry, (char *) buf, buf_len);
}

/******************************************************************************/

/*! \brief 	Executes write function from proc table specified by dir and 
			entry number
	\param	dir 	- directory of proc table
	\param	entry 	- entry of proc table
	\param	buf		- pointer to data buffer for write
	\param 	buf_len - size of buf
	
	\return number of bytes copied or ERROR
*/
/******************************************************************************/

int dda_write_proc_entry( const int dir, const int entry, 
						  const char *buf, const int buf_len)
{
 return exec_proc_entry( dir, entry, (char *) buf, buf_len);
}

/******************************************************************************/

/*! \brief 	Executes print proc function from proc table specified by dir and 
			entry number
	\param	dir 	- directory of proc table
	\param	entry 	- entry of proc table
	
	\return number of bytes printed or ERROR
*/
/******************************************************************************/

int dda_print_proc_entry( const int dir, const int entry )
{
 STATUS res;
 UINT16 buf[10000];	 /* WHAL MailBox reply buffer is to be 2 byte aligned */
	
 res = exec_proc_entry( dir, entry, (char*) buf, sizeof(buf) );
 if (res != ERROR)
 {
  	printf("%s\n", (char *) buf);
 }
 return res;

}

/******************************************************************************/

/*! \brief 	Executes proc function from proc table specified by dir and 
			entry number
	\param	dir 	- directory of proc table
	\param	entry 	- entry of proc table
	\param	buf		- pointer to data buffer
	\param 	buf_len - size of buf
	
	\return number of output bytes
*/
/******************************************************************************/

int exec_proc_entry( DPR_PROC_DIR_NAME_E dir, int entry, char *buf, int buf_len)
{
	int len = ERROR;

	if ( (dir >= 0)   && (dir < MAX_PROC_DIRS) &&
		 (entry >= 0) && (entry	< MAX_PROC_ENTRIES) )
	{
		if ( procFunctionTable[ dir ][ entry ] )
		{
			 len = procFunctionTable[ dir ][ entry ]( buf, buf_len );
		}
		else
		{
			DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d dir=%d entry=%d not defined\n",
						 __FUNCTION__, __LINE__, dir, entry, 0,0 );

		}
	}
	else
	{
		DDA_LOG_ARG( DDA_LOG_ERR,
					 "%s: line=%d dir=%d entry=%d not valid, max_dir=%d max_ent=%d\n",
					 __FUNCTION__, __LINE__, dir, entry, MAX_PROC_DIRS, MAX_PROC_ENTRIES );

	}
	return len;
}


/******************************************************************************/

/*! \brief 	Detects whether channel is inside band range
	\param	band 	- radio band RADIO_BAND_2_4GHZ or RADIO_BAND_5GHZ 
	\param	chID 	- channel number
	
	\return TRUE or FALSE
*/
/******************************************************************************/

BOOL IsChannelInsideBandRange(int band, int chID)
{
	BOOL found = FALSE;
	int ch;

	if (band == RADIO_BAND_2_4GHZ)
	{
		if ((chID < 1) || (chID > 14))
			return FALSE;
		return TRUE;
	}
	else
	{
		if ((chID < 36) || (chID > 161))
			return FALSE;
		/* channel is inside the range, let's verify its value */
		for (ch=36; ch<=64;ch++)
			if (ch == chID)
			{
				found=TRUE;
				break;
			}
		if (!found)
			for (ch=100; ch<=140;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
		if (!found)
			for (ch=149; ch<=161;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
	}
	return found;
}

/******************************************************************************/

/*! \brief 	Reads WLAN driver version
	\param	page    - pointer to buffer to output data
	\param	buf_len - buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_main_read_ver( char* page,  int buf_len)
{
	int len = 0;

	len = sprintf(page,"%s, %d/%d/%d", SW_VERSION_STR, SW_RELEASE_MONTH, SW_RELEASE_DAY,SW_RELEASE_YEAR);
	
	return len;
}

/******* BIT functions *********/
/******************************************************************************/

/*! \brief 	Performs TNETW BIT
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes  or ERROR
*/
/******************************************************************************/
int proc_bit_write_fcc( char * buffer,  int count)
{ 
	proc_params in_params;
	int band,chID,rate,mode;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/
	rate = in_params.s.Param3; /* rate*/
	mode = in_params.s.Param4; /* mode*/

	if (IsChannelInsideBandRange(band,chID) == FALSE)
		return -1;

	/* OK.. let's do it */
	wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
	wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

	if(whal_apiAcxBIT_Fcc(band, chID, rate, mode) != OK)
		return -1;
	
	return count; 
}

/******************************************************************************/

/*! \brief 	Writes telec
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes  or ERROR
*/
/******************************************************************************/

int proc_bit_write_telec( char *buffer,  int count)
{
	proc_params in_params;
	int band,chID;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/


   if (band < 0)
	   return -1;
   if(chID <= 0)
	   return -1;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   return -1;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
   wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

   if(whal_apiAcxBIT_Telec(band, chID) != OK)
	   return -1;
   
   return count;
}

/******************************************************************************/

/*! \brief 	Writes transmitting TNETW configuration
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes or ERROR
*/
/******************************************************************************/

int proc_bit_write_txCfg( char *buffer,  int count)
{
	proc_params in_params;
	int band,chID,rate,mode,preamble,numFrames,gap,bodySize;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/
	rate = in_params.s.Param3; /* rate*/;
	mode = in_params.s.Param4; /* mode*/;
	preamble = in_params.s.Param5; /* preamble*/
    numFrames = in_params.s.Param6; /* frame num*/
	gap = in_params.s.Param7; /* gap*/;
	bodySize = in_params.s.Param8; /* frame budy size*/;

	wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */

    if(whal_apiAcxBIT_perTxCfg(band, chID, rate, mode, preamble,
							   numFrames, gap, bodySize, (char*)in_params.s.mac
							  ) != OK)
		return -1;

	return count;
}

/******************************************************************************/

/*! \brief 	Writes receiving TNETW configuration
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes or ERROR
*/
/******************************************************************************/

int proc_bit_write_rxCfg( char *buffer,  int count)
{
	proc_params in_params;
	int band,chID;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* channel id*/


   if (band < 0)
	   return -1;
   if(chID <= 0)
	   return -1;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   return -1;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */

   if(whal_apiAcxBIT_perRxCfg(band, chID) != OK)
	   return -1;
   return count;
}

/******************************************************************************/

/*! \brief 	Writes receiving TNETW clear status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes or ERROR
*/
/******************************************************************************/

int proc_bit_write_clrStat( char *buffer,  int count)
{
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if(whal_apiAcxBIT_perClrStats() != OK)
		return -1;
	return count;
}

/******************************************************************************/

/*! \brief 	Stops TNETW BIT transmitting
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes or ERROR
*/
/******************************************************************************/

int proc_bit_write_txStop( char *buffer,  int count)
{
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if(whal_apiAcxBIT_perTxStop() != OK)
		return -1;
	return count; 
}


/******************************************************************************/

/*! \brief 	Exits TNETW BIT test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_bit_write_bitStop( char *buffer,  int count)
{
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
   
	whal_apiAcxBIT_ExitTest();
	whal_apiAcxReset();

//	cm_hwReset(); /* nothing else matters... */
	
	return count; 
}

/******************************************************************************/

/*! \brief 	Reads TNETW BIT receiving status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_bit_read_rxStat(  char* page,  int count )
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiAcxBIT_perRxStats(page);

	return len;
}

/******************************************************************************/

/*! \brief 	Reads TNETW BIT transmitting status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_bit_read_txStat( char* page,  int count)
{
	int len = 0;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	len = whal_apiAcxBIT_perTxStats(page);

	return len;
}

/******************************************************************************/

/*! \brief 	Reads TNETW BIT status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_bit_read_test(  char* page,  int count ) 
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	/* tiwlan_ddaNetLinkSend(buff, 32); */
	len = sprintf(page, "%s Done !!!\n", __FUNCTION__);

	return len;
}

/******************************************************************************/

/*! \brief 	Reads station table
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_staTable( char * page,  int count)
{
	int len;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_dbPrintStaTable(page);

    return len;

}

/******************************************************************************/

/*! \brief 	Reads station statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_staStatistics( char * page,  int count)
{
	int len;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_dbPrintStaStatistics(page);

    return len;

}

/******************************************************************************/

/*! \brief 	Reads station hash table
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_staHash( char * page,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_dbPrintHashTable(page);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Reads station rates table
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_staRates( char * page,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_dbPrintRatesTable(page);

    return len;
}

/******************************************************************************/

/*! \brief 	Reads station 4x table
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_4xtable( char * buf,  int count)
{
	int	len;
    
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	len = wdrv_dbPrintFourXTable(buf);
    
	return len;
}

/******************************************************************************/

/*! \brief 	Sends deauthentication event
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_deAuth( char *buffer,  int count)
{
	mgmtMessage_T	msg;
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	msg.type = DEAUTH_MNG;
	PAL_memCopy(msg.content.deAuthMng.macAddress, in_params.s.mac, MAC_ADDR_LEN);
	msg.content.deAuthMng.reason = in_params.s.Param1 /* reason */;
    wdrv_DDA_mgmt_setEvent(&msg);
	
	return count;
}

/******************************************************************************/

/*! \brief 	Sends deassosiation event
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_disAss( char *buffer,  int count)
{
	mgmtMessage_T	msg;
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	msg.type = DISASSOC_MNG;
	PAL_memCopy(msg.content.deAuthMng.macAddress, in_params.s.mac, MAC_ADDR_LEN);
	msg.content.deAuthMng.reason = in_params.s.Param1; /* reason */
    wdrv_DDA_mgmt_setEvent(&msg);
	
	return count;
}

/******************************************************************************/

/*! \brief 	Reads management statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_counters( char* page,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_mgmtPrintCounters(page);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Shows management status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_read_show( char* page,  int count )
{
	BOOL	enable4x;
	dot11_TIcap_T	TIcap;
	int len = 0;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	wdrv_dbGet4xEnable(&enable4x);
	wdrv_dbGetTIInfoElement(TIcap.cap, &(TIcap.hdr.eleLen));		
		
	len=wdrv_dbPrintNumOfStations(page);
	
	if (wdrv_mgmtLogEnable())
		len += sprintf(page +len,"Management Log:		Enable\n");
	else
		len += sprintf(page +len,"Management Log:		Disable\n");
	
	if (wdrv_mgmtAgingEnable())
		len += sprintf(page +len,"Management Aging:	Enable\n");
	else
		len += sprintf(page +len,"Management Aging:	Disable\n");
	
	if(enable4x == TRUE)
	{
		int i;
		len += sprintf(page +len,"4x:	Enable\n");
		len += sprintf(page +len,"0x%x ", TIcap.hdr.eleId);
		len += sprintf(page +len,"0x%x ", TIcap.hdr.eleLen);
		for(i=0;i < TIcap.hdr.eleLen;i++)
			len += sprintf(page +len,"0x%x ", TIcap.cap[i]);
		len += sprintf(page +len,"\n");
	}
	else
		len += sprintf(page +len,"4x:	Disable\n");
			
	return len;
}

/******************************************************************************/

/*! \brief 	Sets aging
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_aging( char* buffer,  int count)
{
	int flag;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&flag, buffer, count);
	
	if ((flag!= 0) && (flag != 1))
	{
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d invalid param\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}
	wdrv_mgmtSetAging((BOOL)flag);

	return count;
}

/******************************************************************************/

/*! \brief 	Sets logging option
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_setLog( char* buffer,  int count)
{
	int flag;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&flag, buffer, count);
	
	if ((flag!= 0) && (flag != 1))
	{
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d invalid param\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	wdrv_mgmtSetLog((BOOL)flag);
	return count;
}



/******************************************************************************/

/*! \brief 	Performs management null test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_null( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstNull( params.s.Param1, /* fc 	*/	
					  params.s.Param2, /* SA 	*/
					  params.s.Param3, /* da 	*/
					  params.s.Param4  /* bssid */
					 );

	return count;
}


/******************************************************************************/

/*! \brief 	Performs management authentication test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstAuth1( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstAuth1( params.s.Param1, /* SA		*/
					   params.s.Param2  /* algoNum  */
					 );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management authentication test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/


int proc_mgmt_write_TstAuth3( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	
	wdrv_mgmtTstAuth3( params.s.Param1, 		/* SA 		 */
					   (UINT16)params.s.Param2, /* algoNum	 */
					   params.s.Param3 			/* challenge */
					 );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management association test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstAss( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstAssoc( params.s.Param1,		 	/* SA 		*/
					   (BOOL)params.s.Param2, 	/* pbcc		*/
					   (BOOL)params.s.Param3, 	/* privacy 	*/
					   (BOOL)params.s.Param4 	/* fourx 	*/
					 );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management reaassociation test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstReass( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstReassoc( params.s.Param1,		  	/* SA 		*/
					     (BOOL)params.s.Param2, 	/* pbcc		*/
					     (BOOL)params.s.Param3, 	/* privacy 	*/
					     (BOOL)params.s.Param4 		/* fourx 	*/
					   );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management deauthentication test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstDeAuth( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstDeAuth (
					    params.s.Param1,		/* SA 		*/
					    (UINT16)params.s.Param2 /* reason	*/
					   );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management disassociation test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstDisAss( char* buffer,  int count)
{
	
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstDisAssoc (
					      params.s.Param1,		  /* SA */
					      (UINT16)params.s.Param2 /* reason*/
					     );
	return count;
}

/******************************************************************************/

/*! \brief 	Performs management class 3 test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstClass3( char* buffer,  int count)
{
	proc_params params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&params,buffer, count);
	wdrv_mgmtTstClass3(
					   params.s.Param1, 		/* SA 	 */
					   (BOOL)params.s.Param2 	/* index */
					  );
	return count;
}


/******************************************************************************/

/*! \brief 	Performs management disassociation test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstRemStat( char* buffer,  int count)
{
	/* call proc_mgmt_write_deAuth (this whats happend in command.c)*/
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	return count;
}


/******************************************************************************/

/*! \brief 	Performs management reset test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_mgmt_write_TstReset( char* buffer,  int count)
{
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	wdrv_mgmtReset();
	return count;
}


/******************************************************************************/

/*! \brief 	Performs management reset test
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_wpa_write_TstReset( char* buffer,  int count)
{
	return count;
}

/******************************************************************************/

/*! \brief 	Reads disassociation/deauthentication privacy statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_read_notify( char* page,  int count)
{
	UINT16	value;
	UINT8 	macaddr[ETH_ALEN];
	int 	len=0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	wdrv_dbGetStationConfigDisassociateReason(&value);
	wdrv_dbGetStationConfigDisassociateStation(macaddr);
	len +=sprintf(page+len,"Disassociate reason: %d\n", value);
	len +=sprintf(page+len,"Disassociates station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
		  macaddr[4],macaddr[5]);

	wdrv_dbGetStationConfigDeauthenticateReason(&value);
	wdrv_dbGetStationConfigDeauthenticateStation(macaddr);
	len +=sprintf(page+len,"Deauthenticate reason: %d\n", value);
	len +=sprintf(page+len,"Deauthenticate station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
		  macaddr[4],macaddr[5]);

	wdrv_dbGetStationConfigDeauthenticateFailStatus(&value);
	wdrv_dbGetStationConfigeauthenticateFailStation((char*)macaddr);
	len +=sprintf(page+len,"Deauthenticate Fail status: %d\n", value);
	len +=sprintf(page+len,"Deauthenticate Fail station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
		  macaddr[4],macaddr[5]);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Reads privacy configuration
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_read_cfg( char* page,  int count)
{
	int 	len = 0, value = TRUE, i;
	UINT32 	value32;
	UINT16 	value16;
	UINT8  	value8;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	/* wdrv_dbGetStationConfigPrivacyOptionImplemented(&value); */
	if (value == TRUE)
		len +=sprintf(page+len,"Privacy option:		On\n");
	else
		len +=sprintf(page+len,"Privacy option:		Off\n");

/*	wdrv_dbGetPrivacyInvoked(&value); */
	if (value == TRUE)
		len +=sprintf(page+len,"Privacy Invoked:	On\n");
	else
		len +=sprintf(page+len,"Privacy Invoked:	Off\n");

/*	wdrv_dbGetPrivacyExcludeUnencrypted(&value); */
	if (value == TRUE)
		len +=sprintf(page+len,"Exclude Unencrypted:	On\n");
	else
		len +=sprintf(page+len,"Exclude Unencrypted:	Off\n");

	wdrv_dbGetPrivacyWEPDefaultKeyId(&value8);
	len +=sprintf(page+len,"WEP Default Key Id:	%u\n", value8);


	wdrv_dbGetStationConfigAuthRspTimeout(&value32);
	len +=sprintf(page+len,"Auth response timeout:	%d msecs\n", value32);
	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		
		wdrv_dbGetAuthAlgo(i, (UINT16 *)&value16);
		if (value16 == MIB_OPEN_SYSTEM)
			len +=sprintf(page+len,"Authentication algorithm: Open system	");
		else if (value16 == MIB_SHARED_KEY)
			len +=sprintf(page+len,"Authentication algorithm: Shared key	");
		else 
			len +=sprintf(page+len,"Authentication algorithm: Unknown	");

		wdrv_dbGetAuthAlgoEnable(i, &value);
		if (value == TRUE)
			len +=sprintf(page+len,"Enable\n");
		else
			len +=sprintf(page+len,"Disable\n");
	}
	return len;
}

/******************************************************************************/

/*! \brief 	Reads privacy WEP keys configuration
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_read_wepkeys( char* page,  int count)
{

	char 	key[WEP_KEY_LEN_MAX];
	int 	i, j,len = 0;
	UINT16 	Keylen;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	for (i = 0; i < DOT11_DEFS_KEY_NAL_LEN; i++)
	{
		wdrv_dbGetDefaultWepKey(i, &Keylen, key);
		len+=sprintf(page+len,"Default key index: %d\n", i);
		len+=sprintf(page+len,"len: %d\n", Keylen);
		len+=sprintf(page+len,"key value: ");
		for (j = 0; j < Keylen; j++)
			len+=sprintf(page+len,"0x%hhX ", key[j]);
		len+=sprintf(page+len,"\n\n");
	}
	return len;
}

/******************************************************************************/

/*! \brief 	Reads privacy WEP statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_read_counters( char* page,  int count)
{
	int len = 0;
	UINT32 count14, count15, count16, count17;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	wdrv_dbGetCountersWEPUndecryptableCount(&count14);
	wdrv_dbGetPrivacyWEPICVErrorCount(&count15);
	wdrv_dbGetPrivacyWEPExcludedCount(&count16);
	wdrv_dbGetPrivacyTKIPMicFailure(&count17);

	len = sprintf(page,"***Privacy Counters***\n"); 
	len +=sprintf(page+len,"WEP Undecryptable Count	= %d\n", count14); 
	len +=sprintf(page+len,"WEP ICV Error Count		= %d\n", count15); 
	len +=sprintf(page+len,"WEP Excluded Count		= %d\n", count16); 
	len +=sprintf(page+len,"TKIP MIC Failure Count	= %d\n", count17); 

	return len;
}

/******************************************************************************/

/*! \brief 	Seads privacy invoke parameter
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_write_invoke( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	!= 0) && (in_params.s.Param1 != 1))
	{
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d Invalid param=%u.\n", __FUNCTION__, __LINE__,
					 in_params.s.Param1, 0,0,0 );
		return -1;
	}
	/* wdrv_dbSetPrivacyInvoked((BOOL)in_params.s.Param1); */

	return count;
}


/******************************************************************************/

/*! \brief 	Reads privacy configuration
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_priv_write_exclude( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	!= 0) && (in_params.s.Param1 != 1))
	{
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d Invalid param=%u.\n", __FUNCTION__, __LINE__,
					 in_params.s.Param1, 0,0,0 );
		return -1;
	}
/* 	wdrv_dbSetPrivacyExcludeUnencrypted((BOOL)in_params.s.Param1); */

	return count;
}


/******************************************************************************/

/*! \brief 	Sets 4x concation size
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_4x_write_max_concatsize( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	< WLAN_MIN_CONCAT_SIZE) || (in_params.s.Param1 > WLAN_MAX_CONCAT_SIZE))
	{
		printf("4x Max Concat size: %d - the value is out of range\n", in_params.s.Param1); 
		return -1;
	}
	wdrv_txSetMaxConcatSize((UINT32)in_params.s.Param1);

	return count;
}


/******************************************************************************/

/*! \brief 	Gets 4x concation size
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_4x_show_max_concatsize( char* buffer,  int count)
{
	int len = 0;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
    len = sprintf(buffer, "Max Concatenation Size    =%8d\n", wdrv_txGetMaxConcatSize());
	
	return len;
}

/******************************************************************************/

/*! \brief 	Gets AP rates
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_srv_read_rates( char* page,  int count)
{
	int i, len= 0;

	char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 ratesLen;
	char extRates[OFDM_RATES_MAX];
	UINT8 extRatesLen;
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;

	UINT8 wone_index;
	UINT8  num_sup_wone;
	
	wdrv_dbGetNumSupWone(&num_sup_wone);	
	for (wone_index = 0 ; wone_index < num_sup_wone; wone_index ++)
	{
		wdrv_dbGetStationConfigOperationalRateSet(wone_index, rates, &ratesLen);
		wdrv_dbGetStationConfigOperationalOFDMRateSet(wone_index, extRates, &extRatesLen);
		wdrv_dbGetStationConfigOperationalAllRateSet(wone_index, allRates, &allRatesLen);

		len +=sprintf(page +len , "\n"); 
		len +=sprintf(page +len ,"Wone Index %d\n",wone_index); 

		len +=sprintf(page +len ,"Rates in Legacy IE =    "); 
		for (i = 0; i < ratesLen; i++)
		{
			len +=sprintf(page +len ,"  "); 
			len += wdrv_printRateInMbpsUnit(rates[i], page + len);
		}
		len +=sprintf(page +len ,"\n\n"); 
		len +=sprintf(page +len ,"Rates in Extended IE =    "); 
		for (i = 0; i < extRatesLen; i++)
		{
			len +=sprintf(page +len ,"  "); 
		len += wdrv_printRateInMbpsUnit(extRates[i], page + len);
		}
		len +=sprintf(page +len ,"\n\n"); 
		len +=sprintf(page +len ,"All Rates =    "); 
		for (i = 0; i < allRatesLen; i++)
		{
			len += sprintf(page+len,"  "); 
			len += wdrv_printRateInMbpsUnit(allRates[i], page + len);
		}
		len += sprintf(page+len,"\n"); 
	}
	return len;
}

/******************************************************************************/

/*! \brief 	Gets AP statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_srv_read_counters( char* page,  int count)
{
	UINT32 count1, count2, count3, count4, count5, count6, count7;
	UINT32 count8, count9, count10, count11, count12, count13, count14;
	UINT32 count15, count16;

	int len = 0;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	wdrv_dbGetCountersTransmittedFragmentCount(&count1);
	wdrv_dbGetCountersMulticastTransmittedFrameCount(&count2);
	wdrv_dbGetCountersFailedCount(&count3);
	wdrv_dbGetCountersRetryCount(&count4);
	wdrv_dbGetCountersMultipleRetryCount(&count5);
	wdrv_dbGetCountersFrameDuplicateCount(&count6);
	wdrv_dbGetCountersRTSSuccessCount(&count7);
	wdrv_dbGetCountersRTSFailureCount(&count8);
	wdrv_dbGetCountersACKFailureCount(&count9);
	wdrv_dbGetCountersReceivedFragmentCount(&count10);
	wdrv_dbGetCountersMulticastReceivedFrameCount(&count11);
	wdrv_dbGetCountersFCSErrorCount(&count12);
	wdrv_dbGetCountersTransmittedFrameCount(&count13);
	wdrv_dbGetCountersWEPUndecryptableCount(&count14);
	wdrv_dbGetPrivacyWEPICVErrorCount(&count15);
	wdrv_dbGetPrivacyWEPExcludedCount(&count16);

	len += sprintf(page + len,"***Counters***\n"); 
	len += sprintf(page + len,"Transmitted Fragment Count		= %d\n", count1); 
	len += sprintf(page + len,"Multicast Transmitted Frame Count= %d\n", count2); 
	len += sprintf(page + len,"Failed Count						= %d\n", count3); 
	len += sprintf(page + len,"Retry Count						= %d\n", count4); 
	len += sprintf(page + len,"Multiple Retry Count				= %d\n", count5); 
	len += sprintf(page + len,"Frame Duplicate Count			= %d\n", count6); 
	len += sprintf(page + len,"RTS Success Count				= %d\n", count7); 
	len += sprintf(page + len,"RTS Failure Count				= %d\n", count8); 
	len += sprintf(page + len,"ACK Failure Count				= %d\n", count9); 
	len += sprintf(page + len,"Received Fragment Count			= %d\n", count10); 
	len += sprintf(page + len,"Multicast Received Frame Count	= %d\n", count11); 
	len += sprintf(page + len,"FCS Error Count					= %d\n", count12); 
	len += sprintf(page + len,"Transmitted Frame Count			= %d\n", count13); 
	len += sprintf(page + len,"WEP Undecryptable Count			= %d\n", count14); 
	len += sprintf(page + len,"WEP ICV Error Count				= %d\n", count15); 
	len += sprintf(page + len,"WEP Excluded Count				= %d\n", count16); 
	
	return len;
}


/******************************************************************************/

/*! \brief 	Gets radio frequency
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_srv_read_radioFreq( char* buffer,  int count)
{
	UINT8 value;
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	wdrv_dbGetRadioFrequency(&value);
	len = sprintf(buffer,"%u\n",value);
	
	return len;
}



/******************************************************************************/

/*! \brief 	Reads power levels
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_srv_read_power_level( char* buffer,  int count)
{
	UINT8 value;
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	wdrv_dbGetPowerLevel(whal_ParamsGetRadioChannel(0), &value);
	len = sprintf(buffer, "%d", value);
	
	return len;
}



/******************************************************************************/

/*! \brief 	Reads radio parameters
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_srv_read_radioParamas( char* buffer,  int count)
{
		
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
			
	wdrv_dbGetRadioParams((RadioParam_t *)buffer);
	return sizeof(RadioParam_t);
		
}

/******************************************************************************/

/*! \brief 	Reads BSS briadge statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_bss( char* buffer,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_bssBridgeShowStatistics(buffer);

	return  len;
}

/******************************************************************************/

/*! \brief 	Reads power save status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_ps( char* buffer,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = wdrv_powerSavePrintDb(buffer);
	return 	len;
}

/******************************************************************************/

/*! \brief 	Reads LAN mib
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_lanMib( char* buffer,  int count)
{
	/* TODO:do we have support in MIB2 for linux */
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	return -1;
}

/******************************************************************************/

/*! \brief 	Reads LAN statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_lanStat( char* buffer,  int count)
{
/* TODO:do we have support in MIB2 for linux */
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	return -1;
}

/******************************************************************************/

/*! \brief 	Reads LAN pool status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_lanPool( char* buffer,  int count)
{
/* TODO:do we have support in MIB2 for linux */
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	return -1;
}

/******************************************************************************/

/*! \brief 	Reads memory managment statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_mgmtStat( char* page,  int count)
{
	int len = 0;
	memMngr_Statistics_T s;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = sprintf(page,"------------ MemMngr Stat ------------");
	if( OK == wlan_memMngrPrintStatistics(&s))
	{
		len +=sprintf(page+len,"msduPool BDs location   @ %p\n", (void*) s.msduPool);
		len +=sprintf(page+len,"bdPool data BDs         @ %p\n", (void*) s.bdPool);
		len +=sprintf(page+len,"List of data buffers    @ %p\n", (void*) s.dataBufPool);
		len +=sprintf(page+len,"firstFreeMSDU           @ %p\n", (void*) s.firstFreeMSDU);
		len +=sprintf(page+len,"firstFreeBD             @ %p\n", (void*) s.firstFreeBD);
		len +=sprintf(page+len,"firstFreeDataBuf64      @ %p\n", (void*) s.firstFreeDataBuf[0]);
		len +=sprintf(page+len,"firstFreeDataBuf2048    @ %p\n", (void*) s.firstFreeDataBuf[1]);
		len +=sprintf(page+len,"numFreeMSDU             = %d\n", s.numFreeMSDU     );
		len +=sprintf(page+len,"numFreeBD               = %d\n", s.numFreeBD       );
		len +=sprintf(page+len,"numFreeDataBuf64        = %d\n", s.numFreeDataBuf[0]);
		len +=sprintf(page+len,"numFreeDataBuf2048      = %d\n", s.numFreeDataBuf[1]);
		len +=sprintf(page+len,"allocFailuresMSDU       = %u\n", s.MsduAllocFailures);
		len +=sprintf(page+len,"allocFailuresBD         = %u\n", s.BdAllocFailures);
		len +=sprintf(page+len,"allocFailuresBuf64      = %u\n", s.BufAllocFailures[0]);
		len +=sprintf(page+len,"allocFailuresBuf2048    = %u\n", s.BufAllocFailures[1]);
	}
	return len;
}

/******************************************************************************/

/*! \brief 	Reads rate addaptation status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_show_read_RA( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = wdrv_txPrintRateAdaptation(buf);
	return len;
}

/******************************************************************************/

/*! \brief 	Shows transmitting statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/


int proc_show_read_txStat( char* buf,  int count)
{
	int len = 0;
	
	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	len = wdrv_txPrintStatistics(buf);
	return len;
}


/*************  hal func ********/


/******************************************************************************/

/*! \brief 	read callback which reads FW HDK and EEPROM version from kernel into input buffer
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_fwVer( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiPrintFwVersion(buf,FALSE);	/* FALSE - long format */
	if (((UINT32)buf+len) & 0x01)	/* Whal MailBox buffer is to be aligned */
	{
		buf[len++]='\n';
	}
	if (DDA_GetInitStatus())
	{
		len += whal_apiGetHDKVersion(buf + len,FALSE);	/* FALSE, long format */
	}
	else
	{					  
		/* Driver has not been initialized yet, access to HAL is forbidden */
		/* return zero version */
		len += sprintf(buf + len, "Radio Module Version = %d.%d.%d.%d\n", 0,0,0,0 ); 
	}

    len += whal_apiGetEEPROMVersion(buf + len,FALSE);
	
	return len;
}


/******************************************************************************/

/*! \brief 	Shows TNETW parameters
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_acxParams( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiPrintAcxParams(buf);
	
	return len;
}



/******************************************************************************/

/*! \brief 	Shows TNETW parameters
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_rxInfo( char* buf,  int count) 
{
	static int len;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiPrintRxInfo(buf);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL receiving queue status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_rxQueue( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiPrintRxQueue(buf);

	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL interrupt status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_Intr( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintIntr(buf);
	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL MailBox status queu status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_Mbox( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintMbox(buf);
	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL MailBox errors trace buffer
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_MboxErrTrace( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintMboxErrTrace(buf);
	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_acxStat( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintAcxStatistics(buf);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL restart statistics
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_restratStat( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintRestartStatistics(buf);
	
	return len;
}

/******************************************************************************/

/*! \brief 	Read meduim usage
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_mediumUsage( char* buf,  int count) 
{
	static int len;
    UINT32 usage, intvl;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );


    if(whal_apiGetMediumUsage(&usage, &intvl) != OK) {
        len = sprintf(buf, "\nRead medium usage error.\n");
    }
    else {
        len = sprintf(buf, "\nRead medium usage %u, interval %u\n", usage, intvl);
    }

    return len;
}

/******************************************************************************/

/*! \brief 	Shows HAL led status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/


int proc_whal_read_led( char *page,  int count)
{
	int len = 0;
	extern UINT8 tiap_gpio_owner;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = sprintf(page, "%s owns gpio\n", tiap_gpio_owner ? "driver" : "fw");

	return len;
}


/******************************************************************************/

/*! \brief 	Sets HAL leds 
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/


int proc_whal_write_led( char* buf,  int count)
{
	char 	byte;
	extern 	UINT8 tiap_gpio_owner;
	UINT16 	tmp;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	byte = * buf;
	
	sscanf(&byte, "%hu", &tmp);
	tiap_gpio_owner = (UINT8)tmp;

	return count;
}


/******************************************************************************/

/*! \brief 	Reads TNETW EEPROM
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_eed( char* buf,  int count) 
{
	int len=0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	len = whal_apiDumpEEPROM(buf, count);

	return len;
}


/******************************************************************************/

/*! \brief 	Writes to TNETW EEPROM
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_eew( char* buf,  int count)
{
	proc_params in_params;
	int addr,val;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );

	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buf, count);
	addr = in_params.s.Param1; 	/* band			*/
    val  = in_params.s.Param2; 	/* channel id	*/

	if(whal_apiWriteEEPROM(addr,val) != OK)
		return -1;
	
	return count;

}


/******************************************************************************/

/*! \brief 	Writes transmitting complete threshold
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_trsh( char* buf,  int count)
{
	proc_params in_params;
	int trsh ;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}
	
	PAL_memCopy((char*)&in_params, buf, count);
	trsh = in_params.s.Param1; /* treshhold */

	if(whal_apiSetTxCompleteThreshold(trsh)== OK)
		return count;
	return -1;
}


/******************************************************************************/

/*! \brief 	read the FW version as a readable string
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/
int proc_whal_read_fwVer_only( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiPrintFwVersion(buf,TRUE);	/* TRUE - short format */
	if (((UINT32)buf+len) & 0x01)	/* Whal MailBox buffer is to be aligned */
	{
		buf[len++]='\n';
	}

	return len;
}


/******************************************************************************/

/*! \brief 	read the Radio version as a readable string
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/
int	proc_whal_read_radioVer_only( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
		
	if (DDA_GetInitStatus())
	{
		len = whal_apiGetHDKVersion(buf,TRUE); /* TRUE, short format */
	}
	else
	{					  
		/* Driver has not been initialized yet, access to HAL is forbidden */
		/* return zero version */
		len = sprintf(buf, "%d.%d.%d.%d", 0,0,0,0 ); 
	}

	if (((UINT32)buf+len) & 0x01)	/* Whal MailBox buffer is to be aligned */
	{
		buf[len++]='\n';
	}

	return len;
}


/******************************************************************************/

/*! \brief 	read the EEPROM version as a readable string
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/
int	proc_whal_read_eepromVer_only( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiGetEEPROMVersion(buf,TRUE); /* TRUE, short format */
	if (((UINT32)buf+len) & 0x01)	/* Whal MailBox buffer is to be aligned */
	{
		buf[len++]='\n';
	}

	return len;
}


/******************************************************************************/

/*! \brief 	Sets BSS bridge power save mode
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_BssPs( char* buf,  int count)
{
	proc_params in_params;
	int BssPsMode;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buf, count);
	BssPsMode = in_params.s.Param1; /* BssPsMode */

	if(whal_apiSetBssPs(BssPsMode) != OK)
		return -1;
	return count; 
}

/******************************************************************************/

/*! \brief 	Gets BSS bridge power save mode
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_Regs( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}
	
	len = whal_apiPrintRegs(buf);
	
	return len;
}


/******************************************************************************/

/*! \brief 	Write to TNETW register
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_reg( char* buf,  int count)
{
	proc_params in_params;
	int addr,val;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buf, count);
	addr = in_params.s.Param1; /* RegAdd */
	val = in_params.s.Param2; /* val */

	whal_apiPhyWrite(addr,val);
	
	return count;
}



/******************************************************************************/

/*! \brief 	Sets TNETW MAC register
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_macReg( char* buffer,  int count)
{
	proc_params in_params;
	int mac_addr,val;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);
	mac_addr = in_params.s.Param1; 	/* RegAdd 	*/
	val = in_params.s.Param2; 		/* val 		*/

	whal_apiMacWrite(mac_addr,val);
	
	return count;
}


/******************************************************************************/

/*! \brief 	Sets TNETW promiscuous mode
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_rx_promis( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiRxPromiscuous( in_params.s.Param1 /* promiscuous flag*/ );
	
	return count;
}

/******************************************************************************/

/*! \brief 	Sets sniffer mode
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_sniffmod( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiModeSniffer( in_params.s.Param1 /* sniff flag*/ );
	
	return count;
}


/******************************************************************************/

/*! \brief 	Sets selected firmware version number
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_FwSelc( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiSelectFwNumber( in_params.s.Param1 /*  selection val*/ );

	return count;
	
}


/******************************************************************************/

/*! \brief 	Sets HAL queue size type
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_QueueSizeType( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiSetQueueSizeType( in_params.s.Param1/*  queue size*/ );

	return count;
}


/******************************************************************************/

/*! \brief 	Select radio number
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_SelcRadionum( char* buffer,  int count)
{
	
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiSelectRadioNumber( in_params.s.Param1/*  selection val*/ );
	return count;
}



/******************************************************************************/

/*! \brief 	Sets NOVRAM bytes
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_NvByte( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiSetNvByte( in_params.s.Param1 ,/* radio num*/	
					   in_params.s.Param2 /*  selection val*/
					 );
	return count;
}


/******************************************************************************/

/*! \brief 	Sets HAL trace enable
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_write_traceEn( char* buffer,  int count)
{
	proc_params in_params;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	if( count <= 0)
	{	
		DDA_LOG_ARG( DDA_LOG_ERR,"%s: line=%d empty buffer\n", __FUNCTION__, __LINE__,
					 0,0,0,0 );
		return -1;
	}

	PAL_memCopy((char*)&in_params, buffer, count);

	whal_apiTraceEnable( in_params.s.Param1/* val*/ );

	return count;
}


/******************************************************************************/

/*! \brief 	Gets HAL trace status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_traceRxStat( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	
	len = whal_apiTracePrintRxStat(buf);
	
	return len;
}



/******************************************************************************/

/*! \brief 	Shows HAL receiving buffer status
	\param	pointer to buffer to output data
	\param	buffer size
	
	\return number of output bytes
*/
/******************************************************************************/

int proc_whal_read_RxBufs( char* buf,  int count)
{
	int len = 0;

	DDA_LOG_ARG( DDA_LOG_PROC,"%s is called.\n", __FUNCTION__, 0,0,0,0,0 );
	len = whal_apiPrintRxBufs(buf);
	
	return len;
}

/*******************************************************************************/
/*!                        
	\brief 	read callback which reads interference WD parameters 
	   
	\param 	buf	a buffer to write into
	\param 	count	at most count bytes should be written	  
	\return number of bytes written to buffer 	                                                    
*/
/*******************************************************************************/
int proc_whal_show_InterfWAparams( char* buf,  int count )
{
	int len = 0;
	UINT32 aciRateThr, aciEventThr, aciTimeout, nbTimeout,  busyMedium;
	UINT16 pdHighThresh, pdLowThresh;
	
	wdrv_dbGetAciRateThreshold(&aciRateThr);
	wdrv_dbGetAciEventsThreshold(&aciEventThr);
	wdrv_dbGetAciStateTimeout(&aciTimeout);
	wdrv_dbGetNbStateTimeout(&nbTimeout);
	wdrv_dbGetPacketDetectLowThresh(&pdLowThresh);
	wdrv_dbGetPacketDetectHighThresh(&pdHighThresh);
	wdrv_dbGetMediumBusyThresh(&busyMedium);

	len =  sprintf(buf+len, "Interference workaround configuration:\n");
	len += sprintf(buf+len, "ACI rate threshold     = %u\n", aciRateThr);
	len += sprintf(buf+len, "ACI events threshold   = %u\n", aciEventThr);
	len += sprintf(buf+len, "ACI state timeout      = %u sec\n", aciTimeout);
	len += sprintf(buf+len, "NB state timeout       = %u sec\n", nbTimeout);
	len += sprintf(buf+len, "Low PD threshold       = 0x%hx\n", pdLowThresh);
	len += sprintf(buf+len, "High PD threshold      = 0x%hx\n", pdHighThresh);
	len += sprintf(buf+len, "Busy medium threshold  = %u percents\n", busyMedium);

	/*FW configuration*/
	{
		UINT32 pdThresh, edThresh, rxDot11Mode;

		whal_apiGetPDthreshold(&pdThresh);
		whal_apiGetEnergyDetect(&edThresh);
		whal_apiGetRxDot11mode(&rxDot11Mode);
		len += sprintf(buf+len, "\nFW configuration:\n");
		len += sprintf(buf+len, "PD threshold  = 0x%x\n", pdThresh);
		len += sprintf(buf+len, "ED threshold  = 0x%x\n", edThresh);
		len += sprintf(buf+len, "RX dot11 mode = %d\n", rxDot11Mode);
		
	}

	return len;
}

