/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanBridgeIf.h                                               
    \brief This module implements WLAN driver interface to the Local Bridge
*/
/******************************************************************************/


#ifndef _WLAN_BRIDGE_IF_H_
#define _WLAN_BRIDGE_IF_H_


#ifdef _WLAN_BRIDGE_IF_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

typedef STATUS (*WBIF_SEND_FUN_T) (mem_MSDU_T * pMsdu, char * mac);
typedef STATUS (*WBIF_IOCTL_FUN_T)(const void * pBrgIf, DDA_IOCTL_DATA_S * pData );

typedef struct
{
	UINT32 rx_packets;
	UINT32 rx_drops;
	UINT32 tx_packets;
	UINT32 tx_drops;

} WBIF_STATISTICS_T;

typedef struct
{
	UINT32            numOfBlocks;	 /* number of M and Cluster Control blocks */
	UINT32            numOfClusters; /* number of Data Cluster buffers 		   */

} WBIF_POOL_DESCR_T;

typedef struct
{
	END_OBJ           end;			 /* Pointert to END driver, must be the first */
	void *			  pCookie;		 /* End device cookie, must be the second 	  */ 
	void *       	  bridgeIfHandle;/* Bridge interface handler 			*/
	UINT32            unit;			 /* Unit number 						*/
	char 			  name[IFNAMSIZ];/* Interface name						*/
	CL_POOL_ID        poolId;		 /* Cluster pool ID 					*/
	WBIF_SEND_FUN_T	  pSendFun;		 /* Pointer to send function to WLAN 	*/
	WBIF_IOCTL_FUN_T  pIoctlFun;	 /* Pointer to Ioctl function   		*/
	WBIF_POOL_DESCR_T poolDesc;		 /* Pool descriptor						*/
	WBIF_STATISTICS_T statistics;
	char 			  linkPartner[ETH_ALEN]; /* WDS peer AP MAC address     */

} WBIF_INTERF_T;


PUBLIC WBIF_INTERF_T *	WBIF_CreateInerface( const char * 				ifName,
											 const UINT32 		   	 	unitNum,
											 const WBIF_SEND_FUN_T 	 	pSendFun,
											 const WBIF_IOCTL_FUN_T 	pIoctlFun,
											 const WBIF_POOL_DESCR_T * 	pPoolDesc );

PUBLIC STATUS  WBIF_RecvPacket( WBIF_INTERF_T * pBrgIf, 
							    const mem_MSDU_T * pMsdu );

PUBLIC STATUS  WBIF_InterfUp( WBIF_INTERF_T * pBrgIf );

PUBLIC STATUS  WBIF_InterfDown( WBIF_INTERF_T * pBrgIf );

PUBLIC STATUS  WBIF_GetFlags( WBIF_INTERF_T * pBrgIf, UINT32 * pFlags );

PUBLIC void    WBIF_GetStatistics( const WBIF_INTERF_T 	* pBrgIf,
								   WBIF_STATISTICS_T 	* pStatistics);

PUBLIC void    WBIF_ResetStatistics( WBIF_INTERF_T * pBrgIf );

PUBLIC void    WBIF_ShowStatistics( const WBIF_INTERF_T * pBrgIf );

PUBLIC STATUS  WBIF_Unload( WBIF_INTERF_T * pBrgIf );

#undef PUBLIC

#endif   /* _WLAN_BRIDGE_IF_H_ */

