/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanMIB2.h                                               
    \brief This module implements SNMP MIB data base for WLAN END driver
*/
/******************************************************************************/

/***********************************************************
	 includes.
*******************************************************/

#ifndef	WLAN_MIB2_H
#define	WLAN_MIB2_H

/**************************************************************************/
/*                 MIB 2 Admin/Oper Status Values                         */
/**************************************************************************/
/* Device is in operational status unless status is down. */
#define MIB2_STATUS_UP     	1
#define MIB2_STATUS_DOWN   	2
#define MIB2_STATUS_TEST   	3
#define MIB2_STATUS_UNKNOWN	4
#define MIB2_STATUS_DORMANT	5

#define	MIB2_UNKNOWN_DUPLEX		1
#define	MIB2_HALF_DUPLEX		2
#define	MIB2_FULL_DUPLEX		3

/**************************************************************************/
/*                 MIB 2 byte/packet Counter Types                        */
/**************************************************************************/
typedef enum
{
    InBytes = 0,
    InBytesHigh,
    InUnicastPkts,
    InUnicastPktsHigh,
    InMulticastPkts,
    InMulticastPktsHigh,
    InBroadcastPkts,
    InBroadcastPktsHigh,
    InDiscardPkts,
    InErrorPkts,
    InUnknownProtPkts,
    OutBytes,
    OutBytesHigh,
    OutUnicastPkts,
    OutUnicastPktsHigh,
    OutMulticastPkts,
    OutMulticastPktsHigh,
    OutBroadcastPkts,
    OutBroadcastPktsHigh,
    OutDiscardPkts,
    OutErrorPkts
} MIB2_CounterType;


typedef struct UINT_64_s
        {
        UINT32  high;        /* the high (msb) 4 bytes of the uint */
        UINT32  low;         /* the low (lsb) 4 bytes of the uint  */
} UINT_64_t;

#define wlan_UInt64_Add(self, add)	\
{										\
	unsigned long	oldLow = (self)->low;	\
										\
	(self)->low += (add);					\
	if ((self)->low < oldLow)++(self)->high;	\
} 

#define UI64_Zero(N) (N)->high = (N)->low = 0L
#define UI64_Increment(N) if (++((N)->low) == 0) (N)->high++

/* First arguments contains the sum */
#define wlan_UInt64_Add64(self, add)	\
{										\
	unsigned long	oldLow = (self)->low;	\
										\
	(self)->low += (add)->low;	  		    \
	(self)->high += (add)->high;	  		    \
	if ((self)->low < oldLow)++(self)->high;	\
}


/**************************************************************************/
/*  Usbwlan_ExtraStats Statistics in addition to mib_stat */
/**************************************************************************/
typedef struct WLAN_MIB_DATA_S
{
	/* These are from the PC's point of view */

	/*  Rx:  Frames without error */
	unsigned long		goodFramesOut;
	
	/*  Pc Rx:  Unicast bytes (64 bit number) */
	UINT_64_t		unicastBytesOut;

	/*  Pc Rx:  Multicast bytes (64 bit number) */
	UINT_64_t		multicastBytesOut;

	/*  Pc Rx:  Broadcast bytes (64 bit number) */
	UINT_64_t		broadcastBytesOut;

	/*  Pc Rx:  Unicast Frames */
	unsigned long		unicastFramesOut; 

	/*  Pc Rx:  Multicast Frames */
	unsigned long		multicastFramesOut;

	/*  Pc Rx:  Broadcast Frames */
	unsigned long		broadcastFramesOut;

	/*  Pc Tx:  Unicast bytes (64 bit number) */
	UINT_64_t		unicastBytesIn;

	/*  Pc Tx:  Multicast bytes (64 bit number) */
	UINT_64_t		multicastBytesIn;

	/*  Pc Tx:  Broadcast bytes (64 bit number) */
	UINT_64_t		broadcastBytesIn;

	/*  Pc Tx:  Unicast Frames */
	unsigned long		unicastFramesIn; 

	/*  Pc Tx:  Multicast Frames */
	unsigned long		multicastFramesIn;

	/*  Pc Tx:  Broadcast Frames */
	unsigned long		broadcastFramesIn;

	unsigned long		inerrors;
	unsigned long		outerrors;
	unsigned long		indiscards;
	unsigned long		outdiscards;
	unsigned long		InUnknownProtos;

} WLAN_MIB_DATA;

void wlanResetMibData(void);
void wlanMib2SetOperStatus(int status);

#endif

