/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2005 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/*!
 *
 *   \file  debug_print.h
 *   \brief PURPOSE: infrastructure for user mode runtime debug printouts
 *
 */

#ifndef  _DEBUG_PRINT_
#define  _DEBUG_PRINT_

/******************************* Defines **************************************************/

#ifdef MAIN_MODULE
	#define PUBLIC
#else /* ifdef MAIN_MODULE */
	#define PUBLIC extern
#endif /* ifdef MAIN_MODULE */

#ifdef VXWORKS
#include "socket.h"
#else /* ifdef VXWORKS */
#include <sys/socket.h>
#include <sys/un.h>
#endif /* ifdef VXWORKS */

#ifdef VXWORKS
#undef printf
#define DEBUG_PRINT_m(sev, mod, fmt, args...) if( (dbgSeverity & (sev)) && (dbgModule & (mod)) ) { \
                                                   char buff[32];   \
                                                   time_t currTime; \
												   int len;			\
                                                   if(time(&currTime) == ((time_t) -1)) printf("Error reading time"); \
                                                   ctime_r (&currTime, buff, &len); \
                                                   printf("%s %s : ", buff, __FUNCTION__); \
                                                   printf(fmt, ##args);}
#define IPC_LOCAL_INET_ADDR "127.0.0.1"
#else /* ifdef VXWORKS */
#define DEBUG_PRINT_m(sev, mod, fmt, args...) if( (dbgSeverity & (sev)) && (dbgModule & (mod)) ) { \
                                                   char buff[32];   \
                                                   time_t currTime; \
                                                   if(time(&currTime) == ((time_t) -1)) printf("Error reading time"); \
                                                   ctime_r (&currTime, buff); \
                                                   printf("%s %s : ", buff, __FUNCTION__); \
                                                   printf(fmt, ##args);}
#endif /* ifdef VXWORKS */

/* Severity definitions */
#define SEV_ERROR   2    
#define SEV_WARNING 4
#define SEV_INFO    8
#define SEV_ALL     (SEV_ERROR | SEV_WARNING | SEV_INFO)

#ifdef VXWORKS
#define TIAP_SOCKADDR_CFG_IPC_IP        "127.0.0.1"
#endif

/* Port on which WPA would listen for debug requests*/
#define DBG_WPA_SOCK_PORT 5130
/* Path to socket on which WPA would listen for debug requests*/
#define DBG_WPA_SOCK_PATH "/var/tmp/wpa_debug_sock"

/* Debug message types*/
typedef enum {
	SET_SEVERITY	 = 0,
	SET_REP_MODULE	 = 1,
	PRINT_PROF_STAT  = 2,
	PRINT_PROF_TBLS  = 3,
	PRINT_PROF_CFG   = 4,
    PRINT_PMK_CACHE  = 5,
    GET_PSK          = 6,
    RECV_PSK         = 7,
}dbgMsgType_e;

#define PSK_LEN     32

/************************** Typedefs *********************************/

typedef struct {
	UINT32 type;
	union {
		UINT32 severity;
		UINT32 module;
		UINT32 profile_ind;
	}content;
    UINT32 reportMsgType;
    /* Socket creation parameters */
    int domain;
    int SocketType;
    int protocol;
    /* Binding parameters */
#ifdef VXWORKS
    struct sockaddr_in addr;
#else /* ifdef VXWORKS */
    struct sockaddr_un addr;
#endif /* ifdef VXWORKS */
    int addrLen;
}dbgMsg_T;

typedef struct {
    UINT32 msgType;     /* From dbgMsg_T->reportMsgType */
    UINT8  psk[PSK_LEN];
} msgReport_t;

/************************ Global variables ***************************/

/* Printouts severity level bit map*/
PUBLIC UINT32 dbgSeverity;

/* Printouts subject bit map*/
PUBLIC UINT32 dbgModule;

#undef PUBLIC

#endif /* ifdef _DEBUG_PRINT_*/
