/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.2.1
 *
 * Author:      Felix Rubinstein, <felixr@ti.com>
 *
 */


#ifndef __WCFG_IPC_H
#define __WCFG_IPC_H

#include <limits.h>

#include "wcfg_storage.h"

#define WCFG_MANDATORY_LOCK 1
#define WCFG_MANDATORY_UNLOCK 0

struct ipc_shared {
	struct storage storage;
	pid_t wpaauth_pid, wdsmngr_pid;
};

struct ipc_private {
	int fd, orig_mode;
#ifdef WCFG_SYS_V_IPC
	int shmid;
#endif	
	char storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char apmodule_name[_POSIX_NAME_MAX+1];
	char dev_name[IFNAMSIZ];
	char br_name[IFNAMSIZ];
};

struct ipc {
	struct ipc_private priv;
	struct ipc_shared *shared;
};


int ipc_create_shm(struct ipc *);
int ipc_remove_shm(struct ipc *);
int ipc_update_shm(struct ipc_shared *);

int ipc_readw_lock(struct ipc_private *);
int ipc_writew_lock(struct ipc_private *);
int ipc_unlock(struct ipc_private *);

void ipc_save_priv_during_restart(struct ipc_private *, struct ipc_private *);

#endif
