/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */


/*!
 *
 *   \file  wcfg_storage.c
 *   \brief PURPOSE: wcfg library persistent storage manipulations
 *
 */

#ifndef __WCFG_NVRAM_H
#define __WCFG_NVRAM_H


#ifndef VXWORKS
#include <linux/config.h>
#include "iwlib.h"
#include "dda_wext_ioctl.h"
#include "dda_wpaauth_ioctl.h"
#include "pform_types_user.h"
#endif
#include "wds.h"
#include "Wds/dda_wds.h"
/* Each time when struct wcfg_storage_t is being changed
   one MUST change (increment) the value of WCFG_MAGIC*/ 
#define WCFG_MAGIC 0x123457b7

#if defined(CONFIG_MIPS_AR7WI) || defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7VW) || defined(CONFIG_MIPS_AR7WRD)
	#define WCFG_BOOTLOADER "/proc/ticfg/env"
#elif CONFIG_MIPS_PUMAS
	#define WCFG_BOOTLOADER "/proc/avalanche/env"
#elif CONFIG_MIPS_APEX
	#define WCFG_BOOTLOADER "/proc/avalanche/env"
#else
	#error Unknown board type.
#endif

#ifdef MWCODE
#define WCFG_WLAN_HWADDR "maca"
#else
#define WCFG_WLAN_HWADDR "WLAN_HWADDR"
#endif /* MWCODE */

/*!	\def WCFG_WLAN_HWADDR
	\brief WCFG_WLAN_HWADDR

	access point hardware address boot-loader environment variable.
*/

#define WCFG_WLAN_SVR_IPDDR 		"IPA_SVR"

#define MAX_EEPROM_ENV_CHUNKS 		50
#define MAX_EEPROM_ENV_CHUNK_SIZE 	150
#define NVS_ENV_VAR_NAME 			"WLAN_EEPROM"
#define NVS_ENV_BLOCK_SIZE 			32 				/* Eeprom bytes in one env var */
#define NVS_TFTP_LOAD_ENV_VAR_NAME	"NVS_TFTP_LOAD"
#define NVS_DEFAULT_ENV_VAR_NAME	"NVS_DEFAULT"

/*!	\struct struct wcfg_storage_t
	\brief struct wcfg_storage_t

	hold all driver configuration parameters.
 */
struct wcfg_storage_t 
{

	/*linux specific staff*/
	__u8				hwaddr[ETH_ALEN];
	__u32				ipaddr, netmask, default_gw;
	
	/*basic data types*/

	/*standard*/
	__s8               		essid[IW_ESSID_MAX_SIZE+1];
	__u16             		rts;
	__u16             		frag;
	__u8             		retry_long, retry_short;
	__u32				lifetime_tx, lifetime_rx;
	tiap_param_txpower_t		txpow_a, txpow_g;
	
	/*private*/
	__u32              		replev; /* bit map */
	__u8               		priv_opt;
	__u16             		beacon_period;
	__u8               		dtim_period;
	tiap_param_bss_br_state_t    	bss_bridge_state;
	
	__u8               		rate_fallback;
	__u32             		radio_cal;
	__u16             		cal_interval;
	__u8               		energy_detect;
	__u8               		eeprom_less;
	__u16					pd_low_thresh;
	__u16					pd_high_thresh;
	__u32					aci_state_timeout;
	__u32					nb_state_timeout;
	__u32					aci_rate_thresh;
	__u32					aci_event_thresh;
	__u32					medium_busy_thresh;
	__u16             		hidden_ssid;
	__u8               		multi_domain_cap;
	__u8               		spectrum_mgmt;
	__u8               		legacy_sta;
	__s8               		local_pow_cons;
	__u8               		max_txpow_lev;
	__s8 				country_str[TIAP_PARAM_COUNTRY_STRING_LEN];
	__u8              		oper_state;
	__u8               		wone_num;
	__u8               		wone_timemod;
	__u8				radar_detect;
	__u16				txop_limit;
	__u8				bridge_mode;
	__u8				gpio_owner;
	__u16				primary_vlan; 

	/*complex data types*/
	tiap_param_auth_algo_t		auth_tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_param_wd_t			wd_en, wd_interf, wd_rxfree, wd_beacons, wd_txstuck, wd_fcserrs, wd_aci, wd_nb;
	tiap_param_access_list_t	access_list;
	tiap_param_4x_t			_4x_fs, _4x_4x, _4x_cat, _4x_pb, _4x_mm;
	tiap_param_multi_ssid_t		multi_ssid;
	tiap_param_radio_t		radio;
	tiap_param_channel_rang		ch_rang_a, ch_rang_g;
	tiap_param_vhcf_t		vhcf;
	tiap_param_wone_t		oper_rate[TIAP_PARAM_WONE_DUAL_BAND], oper_add[TIAP_PARAM_WONE_DUAL_BAND],
					oper_ofdm[TIAP_PARAM_WONE_DUAL_BAND], brcst_default_rate[TIAP_PARAM_WONE_DUAL_BAND],
					mcst_default_rate[TIAP_PARAM_WONE_DUAL_BAND],tx_default_rate[TIAP_PARAM_WONE_DUAL_BAND],
					ch[TIAP_PARAM_WONE_DUAL_BAND],band[TIAP_PARAM_WONE_DUAL_BAND], pbcc[TIAP_PARAM_WONE_DUAL_BAND],
					rate_adapt[TIAP_PARAM_WONE_DUAL_BAND], pream_mode[TIAP_PARAM_WONE_DUAL_BAND],
					slot_mode[TIAP_PARAM_WONE_DUAL_BAND], erp_mode[TIAP_PARAM_WONE_DUAL_BAND],
					erp_type[TIAP_PARAM_WONE_DUAL_BAND], erp_ti[TIAP_PARAM_WONE_DUAL_BAND],
					dot11_mode[TIAP_PARAM_WONE_DUAL_BAND], cwmin_mode[TIAP_PARAM_WONE_DUAL_BAND],
					usage_time[TIAP_PARAM_WONE_DUAL_BAND];
	
	tiap_param_statbl_t		sta_tbl[TIAP_PARAM_MAX_STATIONS];
	tiap_param_priv_reg_domain_t	priv_regdomain;
	tiap_param_cur_reg_domain_t	cur_regdomain[TIAP_PARAM_WONE_DUAL_BAND];
	/* Demiurg : WME */
	tiap_param_qos_t qos;
	/* HW interference handling params - used only with 1150 family */
	tiap_param_hw_interf_t  input_cca, tx_cca, qualif_cca, stomp_tx, stomp_rx ;
    __u8                wdsEnable;

};

/*!	\struct struct tiap_debug_cfg_t
	\brief struct tiap_debug_cfg_t

	debug configuration structure.
 */
struct tiap_debug_cfg_t {
	__u32 debug_server_ip_addr;
	__u8 automatic_fw_load_enabled, jtag_enabled;
	__u32	   				wpa_report_sev; /* wpa debug reports severity bit map */
	__u32					wpa_report_mod; /* wpa debug reports subjects bit map */
	__u32					wds_report_sev; /* wds debug reports severity bit map */
	__u32					wds_report_mod; /* wds debug reports subjects bit map */

};

/**
 *	struct storage - stored in the persistent storage 
 *
 *	@magic - unique number to differ previously stored configuration.
 *	@defaults - defaults are set when set to 1, otherwise is set to 0.
 *	@ldb - local db, stores ioctls configuration.
 *	@wpa - stores wpa authenticator configuration.
 */
struct storage {
	unsigned int magic, defaults;
	struct wcfg_storage_t ldb;
	tiap_wpa_auth_t wpa; 
    wdsStorageCfg_s wds;
	struct tiap_debug_cfg_t debug;
};

int  wcfg_storage_read(struct storage *, const char *);
int  wcfg_storage_write(const struct storage *, const char *);
void wcfg_storage_setdefaults(struct storage *);
void wcfg_storage_setdefaults_ldb(struct wcfg_storage_t *);
void wcfg_storage_setdefaults_ldb_qos(struct wcfg_storage_t *ldb);
void wcfg_storage_setdefaults_wpa(tiap_wpa_auth_t *);
void wcfg_storage_setdefaults_debug(struct tiap_debug_cfg_t * debug );
void wcfg_storage_setdefaults_wds(wdsStorageCfg_s * wdsCfg);
void wcfg_storage_setdefaults_sec_profile(tiap_sec_profile_t *profile);
int  wcfg_storage_getServerIpAdr(char * pIpDotAdr);
int  wcfg_storage_readNvsData(int * pNvsTftpLoadEnable, int * pNvsDefault, 
                              tiap_param_eeprom_t * pEeprom);
int  wcfg_storage_writeNvsData(int TftpLoadEnable, tiap_param_eeprom_t * pEeprom);
void wcfg_storage_set_default_nvs_data( tiap_param_eeprom_t * pEeprom );
int  wcfg_storage_setDefaultNvs(int enableDefaultNvs);



#endif
