/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */

#ifndef __WCFGLIB_H
#define __WCFGLIB_H

#include <sys/types.h>
#include <netinet/if_ether.h>
#include <linux/if.h>
#include <limits.h>

#include "wcfg_storage.h"

/*number of seconds to wait to decide process is successfully started*/
#define WCFG_MAXWAIT 1
#define WCFG_MAXWPAWAIT 5
#define WCFG_MTDBLOCK "/dev/mtdblock/3"
#define WCFG_PATHPREFIX	"/var/tmp/"
#define WCFG_CFGFILE "/var/tmp/.wcfglib"
#define WCFG_INITCFGFILE "/var/tmp/.wcfglib_init"
#define WCFG_WPAPIDFILE "/var/tmp/.wcfgwpapid"
#ifdef TIWLAN_FW_TRACE
#define WCFG_FW_TRACE_PID_FILE "/var/tmp/.wcfgfwtracepid"
#endif
#define TIAP_SOCKADDR_WPAAUTH "/var/tmp/wpaauth"

#define WCFG_RADIO_PARAM_OVERRIDE_BMASK 0
#define WCFG_RADIO_PARAM_POWER_TABLE 1

#define ENABLE  1
#define DISABLE 0

#define MAX_REPLEV 27
#define MIN_REPLEV -27

#define WCFG_TMP_DIR "/var/tmp"

typedef struct {
	char storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	struct storage storage;
	pid_t wpaauth_pid, fw_trace_pid, wdsmngr_pid;
	char apmodule_name[_POSIX_NAME_MAX+1];
	char dev_name[IFNAMSIZ];
	char br_name[IFNAMSIZ];
	char fw_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char fw_trace_app_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
} wcfgd;

typedef struct {
	char storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char wpaauth_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char apmodule_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char dev_name[IFNAMSIZ];
	char br_name[IFNAMSIZ];
	char fw_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char fw_trace_app_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
} wcfginit;

#define UPDATE_STORAGE  1
#define FROM_STORAGE	1

#define INET_ADDR_LEN   INET_ADDRSTRLEN

wcfgd * tiap_wcfg_init(wcfginit *, char * driver_load_failed);
int tiap_wcfg_light_deinit(const wcfgd *);
int tiap_wcfg_deinit(const wcfgd *);
wcfgd *tiap_wcfg_restart(wcfgd *, char *);
void tiap_wcfg_system_reboot(const wcfgd *);


pid_t pseudo_start(const char *path, const char * pid_path, const char * arg);

/* set/get hardware address */
int tiap_wcfg_set_hwaddr(wcfgd *, const __u8 *);
int tiap_wcfg_get_hwaddr(const wcfgd *, __u8 *);

/* WE standard */
int tiap_wcfg_siwrts(wcfgd *, const __s32, int); /*Felix R.: Should it be __s32 ?*/
int tiap_wcfg_giwrts(const wcfgd *wd, __s32 *rts, int from_storage);
int tiap_wcfg_siwfrag(wcfgd *, const __s32, int);
int tiap_wcfg_giwfrag(const wcfgd *, __s32 *, int);
int tiap_wcfg_siwretry(wcfgd *, const __u32, const int, int);
int tiap_wcfg_giwretry(const wcfgd *, __u32 *, int, int from_storage);
int tiap_wcfg_siwtxpow(wcfgd *, const tiap_param_txpower_t *, int);
	
/* Private */
int tiap_wcfg_set_defaults(wcfgd *);

/*basic*/
int tiap_wcfg_set_basic(wcfgd *, const __u32, void *,int);
int tiap_wcfg_get_basic(wcfgd *, const __u32, void *, int from_storage);
int tiap_wcfg_giwtxpow(const wcfgd *, tiap_param_txpower_t *, int from_storage);

/*complex*/
int tiap_wcfg_set_vhcf(const wcfgd *, const tiap_param_vhcf_t *, const int);
int tiap_wcfg_get_vhcf(const wcfgd *, tiap_param_vhcf_t *, int from_storage);

int tiap_wcfg_set_wone(wcfgd *, const tiap_param_wone_t *, const int);
int tiap_wcfg_get_wone(const wcfgd *, tiap_param_wone_t *, int from_storage);

int tiap_wcfg_set_4x(wcfgd *wd, const tiap_param_4x_t *_4x, int idx, const int);
int tiap_wcfg_get_4x(const wcfgd *wd, tiap_param_4x_t *_4x, int from_storage);
int tiap_wcfg_get_sta_table(wcfgd *, tiap_param_statbl_t *);
int tiap_wcfg_get_access_list(wcfgd *wd, tiap_param_access_list_t *tbl, int from_storage);
int tiap_wcfg_set_access_list(wcfgd *wd, const tiap_param_access_list_t *tbl, int update_storage);
int tiap_wcfg_set_auth_type(const wcfgd *wd, const tiap_param_auth_algo_t *tbl, int update_storage);
int tiap_wcfg_get_auth_type(const wcfgd *wd, tiap_param_auth_algo_t *tbl, int from_storage);
int tiap_wcfg_set_multi_ssid(wcfgd *, const tiap_param_multi_ssid_t *, int);
int tiap_wcfg_get_multi_ssid(const wcfgd *wd, tiap_param_multi_ssid_t *multi_ssid, int from_storage);
int tiap_wcfg_set_cfg_pow_tbl(wcfgd *, const tiap_param_radio_t *, int update_storage);
int tiap_wcfg_set_wep_default_key_id(wcfgd *wd, __u8 key_id,int profile_index, int update_storage);
int tiap_wcfg_get_wep_default_key_id(wcfgd *, __u8 *,int profile_index);
int tiap_wcfg_set_def_wep_key(wcfgd *, int, tiap_wpa_wep_key_t *,int profile_index, const int);
int tiap_wcfg_get_def_wep_key(wcfgd *, int, tiap_wpa_wep_key_t *,int profile_index);
int tiap_wcfg_set_psk_passphrase_mode(wcfgd *wd, int value,int profile_index, int update_storage);
int tiap_wcfg_set_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 passwd_len,int profile_index, int update_storage);
int tiap_wcfg_get_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 *passwd_len,int profile_index);
int tiap_wcfg_set_default_cipher(wcfgd *wd, __u32 cipher,int profile_index, int update_storage);
int tiap_wcfg_set_gtk_interval(wcfgd *wd, __u32 intvl,int profile_index, int update_storage);
int tiap_wcfg_get_default_cipher(wcfgd *wd, __u32 *cipher,int profile_index);
int tiap_wcfg_get_radius_port(wcfgd *wd, __u16 *port,int profile_index);
int tiap_wcfg_get_radius_ipaddr(wcfgd *wd, __u32 *ipaddr,int profile_index);
int tiap_wcfg_get_gtk_interval(wcfgd *wd, __u32 *intvl,int profile_index);
int tiap_wcfg_get_psk_passphrase_mode(wcfgd *wd, int *value,int profile_index);
int tiap_wcfg_get_priv_regdomain(wcfgd *, tiap_param_priv_reg_domain_t *, int from_storage);
int tiap_wcfg_set_dot11mode(wcfgd *, int, int, int);
int tiap_wcfg_set_wd(const wcfgd *wd, const tiap_param_wd_t *w, int);
int tiap_wcfg_get_interf_wd_config(const wcfgd *wd, tiap_param_interference_wd_t *param, int from_storage);
int tiap_wcfg_set_interf_wd_config(wcfgd *wd, tiap_param_interference_wd_t *param, int update_storage);
int tiap_wcfg_get_priv_net_stats(wcfgd *, tiap_param_net_stats *);

int tiap_wcfg_set_eeprom_image(wcfgd * wd, const tiap_param_eeprom_t * eeprom);

void  print_mac(const char *mac);
int tiap_wcfg_get_cur_regdomain(wcfgd *, tiap_param_cur_reg_domain_t *, int from_storage);
int tiap_wcfg_set_priv_regdomain(wcfgd *, tiap_param_priv_reg_domain_t *, const int);
int tiap_wcfg_set_ch_range(const wcfgd *, const tiap_param_channel_rang *, int update_storage);
int tiap_wcfg_get_ch_range(const wcfgd *, tiap_param_channel_rang *, int from_storage);
int tiap_wcfg_set_cur_regdomain(wcfgd *, tiap_param_cur_reg_domain_t *, const int);

int tiap_wcfg_set_pre_auth_flag(wcfgd *, int,int profile_index, int update_storage);
int tiap_wcfg_get_pre_auth_flag(wcfgd *, __u8*,int profile_index);
int tiap_wcfg_set_sa_max_duration(wcfgd *wd, __u32 interval,int profile_index, int update_storage);
int tiap_wcfg_get_sa_max_duration(wcfgd *wd, __u32* interval ,int profile_index);

int tiap_wcfg_set_radius_port(wcfgd *, __u16,int profile_index, int update_storage);
int tiap_wcfg_set_radius_ipaddr(wcfgd *, __u32,int profile_index, int update_storage);
int tiap_wcfg_set_radius_secret(wcfgd *wd, __s8 *, __u8,int profile_index, int update_storage);
int tiap_wcfg_get_radius_secret(wcfgd *wd, __s8 *, __u8 *,int profile_index);

int tiap_wcfg_set_dual_ap(wcfgd *wd, __u8 wone_nr, int update_storage);
int tiap_wcfg_set_gpio_owner(wcfgd *, __u8, int update_storage);

int tiap_wcfg_get_num_of_multi_ssid(wcfgd *wd);

int tiap_wcfg_debug_set_server_ip(wcfgd *wd, __u32 ipaddr);

int tiap_wcfg_debug_automatic_fw_load(wcfgd *wd, char enable);
int tiap_wcfg_debug_jtag(wcfgd *wd, char enable);
int tftp_get_file(const char * filename, const char * addr, const char * dir);
int tiap_inet_aton(char *str, struct in_addr * inetAddress);
int tiap_wcfg_get_security_type(wcfgd *wd, tiap_param_security_type_t  *sec_type ,int profile_index);
int tiap_wcfg_set_security_type(wcfgd *wd, tiap_param_security_type_t sec_type, int profile_index, int update_storage);
int tiap_wcfg_set_primary_ssid(wcfgd *wd, __u8 *ssid, const int update_storage);
int tiap_wcfg_get_primary_ssid(wcfgd *wd, __u8 *ssid,  int from_storage);
int tiap_wcfg_get_primary_profileIndex(wcfgd *wd, int *profile_index,  int from_storage);
int tiap_wcfg_get_hw_interf_param(const wcfgd *wd, tiap_param_hw_interf_t *param, int from_storage);
int tiap_wcfg_set_hw_interf_param(wcfgd *wd, tiap_param_hw_interf_t *param, int update_storage);


/* QoS */

int tiap_wcfg_set_qos_params(wcfgd *, const tiap_param_qos_t *, int update_storage);
int tiap_wcfg_set_qos_ac_params(wcfgd * wd, const tiap_param_qos_t * params, int, int update_storage);
int tiap_wcfg_get_qos_params(wcfgd *, tiap_param_qos_t *, int from_storage);

int  tiap_wcfg_enableWds(wcfgd *wd, __u8 enable, int update_storage);
int  tiap_wcfg_configWds(const wcfgd *wd, int restart, int update_storage);
int  tiap_wcfg_wdsHopSet(wcfgd *wd, __u8 hop, int update_storage);
void tiap_wcfg_wdsHopGet(const wcfgd *wd, __u8 *pHop);
int  tiap_wcfg_wdsChSelModeSet(wcfgd *wd, __u8 chSelectMode, int update_storage);
void tiap_wcfg_wdsChSelModeGet(const wcfgd *wd, __u8 *pChSelectMode);
int  tiap_wcfg_wdsAutoCfgSet(wcfgd *wd, __u8 autoCfg, int update_storage);
void tiap_wcfg_wdsAutoCfgGet(const wcfgd *wd, __u8 *pAutoCfg);
int  tiap_wcfg_wdsPrivacySet(wcfgd *wd, __u8 privacy, int update_storage);
void tiap_wcfg_wdsPrivacyGet(const wcfgd *wd, __u8 *pPrivacy);
int  tiap_wcfg_wdsNameSet(wcfgd *wd, const __u8 *name, int update_storage);
void tiap_wcfg_wdsNameGet(const wcfgd *wd, __u8 *name);
int  tiap_wcfg_wdsWirelessIfModeSet(wcfgd *wd, __u8 wirelessIfMode, int update_storage);
void tiap_wcfg_wdsWirelessIfModeGet(const wcfgd *wd, __u8 *pWirelessIfMode);
int  tiap_wcfg_wdsCrudeModeSet(wcfgd *wd, __u8 crudeMode, int update_storage);
void tiap_wcfg_wdsCrudeModeGet(const wcfgd *wd, __u8 *pCrudeMode);
int  tiap_wcfg_wdsSharedSecretSet(wcfgd *wd, const __u8 *sharedSecret, int update_storage);
void tiap_wcfg_wdsSharedSecretGet(const wcfgd *wd, __u8 *sharedSecret);
int  tiap_wcfg_wdsUlMacSet(wcfgd *wd, __u8 *mac, int update_storage);
void tiap_wcfg_wdsUlMacGet(const wcfgd *wd, __u8 *mac);
int  tiap_wcfg_wdsDl0MacSet(wcfgd *wd, __u8 *mac, int update_storage);
void tiap_wcfg_wdsDl0MacGet(const wcfgd *wd, __u8 *mac);
int  tiap_wcfg_wdsDl1MacSet(wcfgd *wd, __u8 *mac, int update_storage);
void tiap_wcfg_wdsDl1MacGet(const wcfgd *wd, __u8 *mac);
int  tiap_wcfg_wdsDl2MacSet(wcfgd *wd, __u8 *mac, int update_storage);
void tiap_wcfg_wdsDl2MacGet(const wcfgd *wd, __u8 *mac);
int  tiap_wcfg_wdsDl3MacSet(wcfgd *wd, __u8 *mac, int update_storage);
void tiap_wcfg_wdsDl3MacGet(const wcfgd *wd, __u8 *mac);
void tiap_wcfg_wdsReport(const wcfgd *wd);
void tiap_wcfg_wdsStat(const wcfgd *wd);
void tiap_wcfg_wdsInfoDb(const wcfgd *wd);
const char * tiap_wcfg_wdsLinkStateGet(const wcfgd *wd, int link/* 0 - uplink, 1-4 - downlinks*/);
const char * tiap_wcfg_wdsMngrStateGet(const wcfgd *wd, int link/* ignored */);
void tiap_wcfg_wdsSeveritySet(const wcfgd *wd, UINT32 error, UINT32 warning, UINT32 info);
void tiap_wcfg_wdsRestart(const wcfgd *wd);
int tiap_wcfg_setWdsDefaults(wcfgd *wd);
int tiap_wcfg_wdsPrepareReportSocket(wdsMngrReportRequest_s *pRequest);
int tiap_wcfg_wdsReadResponse(void * pResponse, int size, int sock);

int tiap_wcfg_lookUpMultiSsidBySsid(tiap_param_multi_ssid_t *multi_ssid, __u8 *ssid,tiap_param_multi_ssid_elem_t *elem, int *elem_ind);
int tiap_wcfg_lookUpMultiSsidByVlan(tiap_param_multi_ssid_t *multi_ssid, __u16 vlan,tiap_param_multi_ssid_elem_t *elem, int *elem_ind);
int tiap_wcfg_lookUpMultiSsidByType(tiap_param_multi_ssid_t *multi_ssid, tiap_param_ssid_type_t type,
					                tiap_param_multi_ssid_elem_t *elem, int *elem_ind);
int tiap_wcfg_isCipherExistsInMultiSsid(wcfgd *wd, __u32 cipher, BOOL *isCipherExists, 
                                        int *profile_index);
int tiap_wcfg_isPreAuthOnInMultiSsid(wcfgd *wd, BOOL *isPreAuthOn, int *profile_index);
int tiap_wcfg_delMultiSsid(wcfgd *wd,  int ssid_id, int update_storage);
int tiap_wcfg_addMultiSsid(wcfgd *wd,  tiap_param_multi_ssid_elem_t *elem,  int update_storage);
int tiap_wcfg_set_primary_vlan(wcfgd *wd, __u16 vlan_id,const int update_storage);
int tiap_wcfg_get_primary_vlan(wcfgd *wd, __u16 *vlan_id,  int from_storage);
int tiap_wcfg_get_freeSecProfileIndex(wcfgd *wd, int *profile_index);
int tiap_wcfg_get_actual_ssid_ind(tiap_param_multi_ssid_t *multi_ssid, int ssid_seq_num, int *actual_index);
int tiap_wcfg_sendIpcMsg(char *sock_path, void *msg, int msg_len);
int tiap_wcfg_wpaSetRepModule(wcfgd *wd, UINT32 mod);
int tiap_wcfg_wpaSetSeverity(wcfgd *wd, UINT32 severity);
int tiap_wcfg_wpaGetRepModule(wcfgd *wd, UINT32 *mod);
int tiap_wcfg_wpaGetSeverity(wcfgd *wd, UINT32 *severity);
int tiap_wcfg_wpaPrintStaTbl(int profile_index);
int tiap_wcfg_wpaPrintProfStatistics(int profile_index);
int tiap_wcfg_wpaPrintProfCfg(int profile_index);
int tiap_wcfg_wpaPrintPmkCacheList(void);
int tiap_wcfg_wpaGetPSKinHexFormat(int profile_index, char *psk);
/* BIT */
int tiap_wcfg_set_bitArgs(wcfgd *wd, tiap_param_bit_args_t *bit_args, int update_storage);

int tiap_wcfg_get_die_id(const wcfgd *wd, tiap_param_die_id_t *param, int from_storage);
int tiap_tftp_read_file(const char * filename, const char * addr, char * pBuf, int bufSize);

#define TIAP_PARAM_BURST_LIMIT 100

#endif
