/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*											    		    				*/
/*	 MODULE:	securityProfile.h					   			   			*/
/*   PURPOSE:	security Profile headers  						   			*/
/*												    						*/
/***************************************************************************/
/***************************************************************************/
#ifndef SEC_PROF_H
#define SEC_PROF_H

#ifdef VXWORKS			
#define TO_WPA_HIDDEN_PROFILE_SOCK_PORT	5128
#define TO_WPA_HIDDEN_PROFILE_SOCK_RESP_PORT	5129
#else /* ifdef VXWORKS */
#define TO_WPA_HIDDEN_PROFILE_SOCK  "/var/tmp/to_wpa_hidden_sock"
#define TO_WPA_HIDDEN_PROFILE_SOCK_RESP	"/var/tmp/hidden_to_wpa_sock"
#endif /* ifdef VXWORKS */
typedef enum
{
	SEC_PROFILE_NON_ACTIVE,
	SEC_PROFILE_REGULAR,
	SEC_PROFILE_HIDDEN
}profile_type_t;

typedef struct 
{ 
	profile_type_t		profile_type;
	auth1x_port_params_t	auth_8021x_params;		/* Forwarded in the 'auth1x_port_open()' function */
	void					*pXPortContext;	/* The WPA port context. Used in every call to WPA port functions. */
	char					pskPwd[MAX_PSK_PWD_LEN + 1];
	char   				Ssid[DOT11_SSID_MAX+1];
	wpa_port_params_t	portParams;			/* Forwarded in the 'wpa_port_open()' function */
	wpa_port_t			*pWpaPortContext;	/* The WPA port context. Used in every call to WPA port functions. */
	int					secAttackTable[MAX_STATIONS];
	int					micFailureReportInRx;
	radius_params_t		   radiusParams;
	server_list_t		   serverList;
	/* Radius client Section */
	void *radius_client;			/* returned from the ;radius_client_open()' function */
} security_port_data_t;

typedef enum
{
	HIDDEN_PROF_INIT,
	HIDDEN_PROF_AUTH_START,  /* Start secure authentication */
    HIDDEN_PROF_SUPL_START,  /* Start secure suplicant */
	HIDDEN_PROF_WPAS_ABORT,  /* Abort supplicant message*/
	HIDDEN_PROF_AUTH_CLOSE,  /* Close authenticator connection */
	HIDDEN_PROF_SUPL_CLOSE,	 /* Close supplicant connection */
    HIDDEN_PROF_INIT_COMPLETE, /* Hidden profile init complete*/
    HIDDEN_PROF_LINK_CLOSE_COMPLETE, /* Link close confirmation message */

} hiddenWpaInstMsgType;

/* Message to hidden profile WPA instance */
typedef struct
{
	hiddenWpaInstMsgType   type;  /* Message type */
	union
	{  	 
		initHiddenProfConf_T    initParams;
		dynamicSecureCfg_T		confParams;     /* Current authentication dynamic params */

	}u;
	
}hiddenWpaInstanceMsg_T;


security_port_data_t *wdrv_wpaGetHiddenSecPortData(void);
ap_dev_cbs_t 		 *wdrv_wpaGetApDevCbs(void);
STATUS  hidden_profile_PrepareRequestsSock(UINT32 *pSock);
UINT32 	isAssocWithHiddenInstance(ether_mac_t staMac);
void    hidden_profile_ProcessInputMsg(int sock, void *data);
void 	hidden_profile_Disable(void);
UINT32	hidden_profile_IsEnabled(void);
UINT32  hidden_profile_IsSecured(void);
void    hidden_profile_ReportAuthStatus(ether_mac_t staMac, UINT32 status);
UINT32  hidden_profile_IsEapolToSup(eapol_pkt_t *eapol);
void    hidden_profile_CloseAuth(UINT8 *pMac);
#endif
