
#include <rg_error.h>
#ifndef VXWORKS
#include "malloc.h"
#include <sys/socket.h>
#else
#include "wlan_pal.h"
#endif
#include "dda_wpaauth_ioctl.h"
#include "wpa_port.h"
#include "wpa_assoc.h"
#include "wpa_pmk_cache.h"
#include "stdio.h"
#include <util/memory.h>
#include <wpa_log.h>
#include <wpa_key_utils.h>
#include "debug_report.h"

/* pmk cache list*/
static wpa_pmk_cache_t  *pmk_cache_list[PMK_CACHE_NUMBER_OF_STA][TIAP_MAX_NUM_OF_SUPPORTED_PROFILES];
static unsigned long    pmk_list_index[TIAP_MAX_NUM_OF_SUPPORTED_PROFILES];
static BOOL				is_pmk_cache_active[TIAP_MAX_NUM_OF_SUPPORTED_PROFILES] = {FALSE,FALSE,FALSE,FALSE};
static char             APMAC[sizeof(ether_mac_t)];


/* get free entry*/
static wpa_pmk_cache_t* wpa_pmk_cache_get_free_elem(int profile_index);
/* return ele to list*/
static void wpa_pmk_cache_ret_list_ele(int profile_index, wpa_pmk_cache_t *elem);
/* calc station pmk cache ID.*/
static int wpa_pmk_cache_calc_id(ether_mac_t  mac,wpa_pmk_t pmk,u8 *pmk_id);
/* check list valid timer call back.*/
static void wpa_pmk_cache_timer_cb(void *data);


/* init pmk cahch list ,update all entries to not active*/
int wpa_pmk_cache_init(wpa_port_t *t,char *ap_mac, int profile_index)
{
	int i;
	
	/* malloc list elements, it can be done staticly if we want */
	for(i = 0; i < PMK_CACHE_NUMBER_OF_STA; i++)
	{
		if((pmk_cache_list[i][profile_index] = (wpa_pmk_cache_t*)zalloc_l(sizeof(wpa_pmk_cache_t))) == NULL)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"\n\rerror malloc pmk cache list entry %d",i);
			return -1;
		}
		
		pmk_cache_list[i][profile_index]->session_state = PMK_CACHE_ENRTY_NOT_ACTIVE;
        pmk_cache_list[i][profile_index]->port = t;
		pmk_cache_list[i][profile_index]->cache_idx = i;
	}
	
	pmk_list_index[profile_index] = 0;
	is_pmk_cache_active[profile_index] = TRUE;
	
	memcpy(APMAC, ap_mac, sizeof(ether_mac_t));
	
	/* start list check timer.*/
	event_timer_set(CHECK_LIST_TIME_CYCLE,(void*)wpa_pmk_cache_timer_cb, (void *)profile_index);
	
	return OK;
}

/* release pmk cache mem*/
void wpa_pmk_cache_kill(int profile_index)
{
	int i;

	is_pmk_cache_active[profile_index] = FALSE;

	event_timer_del(wpa_pmk_cache_timer_cb, (void *)profile_index);

	/*free list elem*/
	for(i = 0;i < PMK_CACHE_NUMBER_OF_STA ; i++ )
		free(pmk_cache_list[i][profile_index]);
}

/* get free entry*/
static wpa_pmk_cache_t* wpa_pmk_cache_get_free_elem(int profile_index)
{
	wpa_pmk_cache_t* tmp = NULL;
	
	if(pmk_list_index[profile_index] != PMK_CACHE_NUMBER_OF_STA)
	{
		int i;
		for (i=0; i < PMK_CACHE_NUMBER_OF_STA; i++) 
		{
			if (pmk_cache_list[i][profile_index]->session_state == PMK_CACHE_ENRTY_NOT_ACTIVE) 
			{
				tmp = pmk_cache_list[i][profile_index];
				tmp->session_state = PMK_CACHE_STATION_ASSOC;
				pmk_list_index[profile_index]++;
				break;
			}
		}
	}
	else
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"\n\rpmk list full");
		return NULL;
	}
	
	return tmp; 
}

/* return ele to list*/
static void wpa_pmk_cache_ret_list_ele(int profile_index, wpa_pmk_cache_t *elem)
{
	pmk_list_index[profile_index]--;
	pmk_cache_list[elem->cache_idx][profile_index]->session_state = PMK_CACHE_ENRTY_NOT_ACTIVE;    
}

/* calc station pmk cache ID.*/
static int wpa_pmk_cache_calc_id(ether_mac_t  mac,wpa_pmk_t pmk,u8 *pmk_id)
{
	/* section 8.5.1.2 */
	/* pmk Id = sha1(pmk, "pmk name" || AP addr || STA add) */
	
	const char *pmk_name = "PMK Name";
	char pt[64];
	int txt_size = sizeof(ether_mac_t)*2 +strlen(pmk_name);
	

	memcpy(pt,pmk_name,strlen(pmk_name));
	memcpy(&pt[strlen(pmk_name)],APMAC ,sizeof(ether_mac_t));
	memcpy(&pt[strlen(pmk_name) + sizeof(ether_mac_t)],mac ,sizeof(ether_mac_t));

	
	hmac_sha1(pmk_id, pt,txt_size ,pmk.data, sizeof(wpa_pmk_t));
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"PMK-ID created  - %s  \n",wpa_log_format_buf(pmk_id ,16));
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nPMK-ID created for station %s\n" ,mactoa((ether_mac_t *)&mac));

	return OK;
}

/* insert station to pmk cache list
   first check if pmk cache is active and if we have place.	
*/
int wpa_pmk_cache_insert(wpa_assoc_t *assoc,double session_timeout)
{
	wpa_pmk_cache_t* list_elem;
    int              profile_index;
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"insert station %s to pmk cache,timeout = %ld !!!\n",mactoa(&assoc->sta_mac),(long)session_timeout);
    profile_index = assoc->port->port_profile_index;

	/* check if pmk cache active*/
	if(!is_pmk_cache_active[profile_index])
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_pmk_cache_insert: pmk cache flag is down.");
		return -1;
	}    
	
	/* case this station allready got pmk id entry*/

	if((list_elem  = wpa_pmk_cache_get_station_by_mac(assoc->sta_mac, profile_index)) != NULL)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_pmk_cache_insert: found station %s in cache,replacing pmk id ", mactoa(&assoc->sta_mac));
	}
	else /* new station ,get new list elem.*/
	{
		/* get free entry*/
		list_elem = wpa_pmk_cache_get_free_elem(profile_index);
		if(list_elem == NULL)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_pmk_cache_insert: no free entries found");
			return -1;
		}
	}
	
	/* calc pmk id for new station*/
	if(wpa_pmk_cache_calc_id(assoc->sta_mac,
							assoc->pmk,
							list_elem->pmk_cache_id) == -1)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_pmk_cache_insert: error calc pmk id for station %s", mactoa(&assoc->sta_mac));
		wpa_pmk_cache_ret_list_ele(profile_index, list_elem);
		return -1;
	}
	
	/* fill fields*/
	memcpy((char*)list_elem->mac,(char*)assoc->sta_mac,sizeof(ether_mac_t));
	memcpy((char*)list_elem->pmk.data,(char*)assoc->pmk.data,sizeof(wpa_pmk_t));
	/*memcpy((char*)assoc->pmk_cach_id, (char *)list_elem->pmk_cache_id, PMK_CACHE_ID_SIZE);*/
	list_elem->session_timeout	 = session_timeout;
	list_elem->session_start_TS = get_time_from_boot();
    list_elem->assoc = assoc;
   
	
	return OK;
	
}

/* get station entry index with pmk id input*/
wpa_pmk_cache_t*  wpa_pmk_cache_get_station_by_id(char *pmk_id, int profile_index)
{
	int i;

	for(i = 0; i < PMK_CACHE_NUMBER_OF_STA; i++)
	{
		if (pmk_cache_list[i][profile_index]->session_state != PMK_CACHE_ENRTY_NOT_ACTIVE) 
		{
			if(memcmp(pmk_cache_list[i][profile_index]->pmk_cache_id, pmk_id ,PMK_CACHE_ID_SIZE ) == 0)
			{
				return pmk_cache_list[i][profile_index];
			}
		}	
	}
	
	return NULL;
}

/* get station entry index with mac input*/
wpa_pmk_cache_t*  wpa_pmk_cache_get_station_by_mac(ether_mac_t  mac, int profile_index)
{
	int i;

	for(i = 0; i < PMK_CACHE_NUMBER_OF_STA; i++)
	{
		if (pmk_cache_list[i][profile_index]->session_state != PMK_CACHE_ENRTY_NOT_ACTIVE) 
		{
			if(memcmp(pmk_cache_list[i][profile_index]->mac, mac ,sizeof(ether_mac_t) ) == 0)
			{
				return pmk_cache_list[i][profile_index];
			}
		}
	}
	
	return NULL;
}

/* disassociate station and remove station from pmk cache list*/
int wpa_pmk_cache_disassoc_remove(wpa_pmk_cache_t* elem)
{	
    wpa_assoc_disassoc(elem->assoc);

	wpa_pmk_cache_remove(elem);

	return OK;
}

/* remove station from pmk cache list */
int wpa_pmk_cache_remove(wpa_pmk_cache_t* elem)
{
	
	memset(elem->pmk_cache_id, 0, PMK_CACHE_ID_SIZE + 4);
	memset (elem->mac, 0, sizeof(ether_mac_t)); 
	memset (&elem->pmk, 0, sizeof(wpa_pmk_t));
    elem->assoc = NULL;
	elem->session_timeout  = 0;
	elem->session_start_TS = 0;

	wpa_pmk_cache_ret_list_ele(elem->port->port_profile_index, elem);

	return OK;

}

/* check if station in PMK cache list*/
int wpa_pmk_cache_check_pmk_valid(wpa_port_t *port, char *pmk_cach_id, wpa_pmk_t *sta_pmk)
{
	wpa_pmk_cache_t* elem;
    int              profile_index = port->port_profile_index;

	if(!is_pmk_cache_active[profile_index])
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_pmk_cach_check_pmk_valid: pmk cache flag is down.");
		return -1;
	}
	
	if((elem = wpa_pmk_cache_get_station_by_id(pmk_cach_id, profile_index)) == NULL)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_pmk_cach_check_pmk_valid: station pmk not found,starting regular connection .");
		return -1;
	}
	
	memcpy(sta_pmk->data,elem->pmk.data,sizeof(wpa_pmk_t));
	
	return OK;
}

/* Update a pointer to assoc structure in the pmk cache entry*/
int wpa_pmk_cache_update_assoc_by_mac(wpa_port_t *port, ether_mac_t sta_mac, wpa_assoc_t *assoc)
{
	wpa_pmk_cache_t* elem;
	int              profile_index = port->port_profile_index;

	if(!is_pmk_cache_active[profile_index])
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_pmk_cache_update_assoc: pmk cache flag is down.");
		return NOK;
	}
	
	if((elem = wpa_pmk_cache_get_station_by_mac(sta_mac, profile_index)) == NULL)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_pmk_cache_update_assoc: station pmk not found,starting regular connection .");
		return NOK;
	}

	elem->assoc = assoc;

	return OK;
}
/* check list valid timer call back.*/
/* TODO : check if its not killing us when list full*/
static void wpa_pmk_cache_timer_cb(void *data)
{
	int i, profile_index = (int)data;
	double cur_time = get_time_from_boot();

    for(i = 0; i < PMK_CACHE_NUMBER_OF_STA; i++)
    {
		if (pmk_cache_list[i][profile_index]->session_state != PMK_CACHE_ENRTY_NOT_ACTIVE) 
		{
			if(cur_time > pmk_cache_list[i][profile_index]->session_start_TS + pmk_cache_list[i][profile_index]->session_timeout)
			{
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_pmk_cache_timer_cb: pmk expired for station %s", mactoa(&pmk_cache_list[i][profile_index]->mac));
				wpa_pmk_cache_disassoc_remove(pmk_cache_list[i][profile_index]);
			}
		}
        
    }
	
    event_timer_set(CHECK_LIST_TIME_CYCLE,(void*)wpa_pmk_cache_timer_cb, data);
}


/* print list*/
void wpa_pmk_cache_dbg_print(void)
{
	int i,k;
	
    printf("\n PMK CACHE LIST:\n");
    printf("-----------------\n");
    for(k = 0; k < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; k++)
    {
        if(!is_pmk_cache_active[k])
            continue;

        if(pmk_list_index[k] > 0)
        {
            printf("\n------------------ Profile index %d -----------------\n",k);
            for(i = 0; i < PMK_CACHE_NUMBER_OF_STA; i++)
            {
                if (pmk_cache_list[i][k]->session_state != PMK_CACHE_ENRTY_NOT_ACTIVE)
                {
                    printf("station %s pmk list data:\n" ,mactoa(&pmk_cache_list[i][k]->mac ));
                    printf("state    - %d\n",pmk_cache_list[i][k]->session_state);
                    printf("PMK      - %s\n",wpa_log_format_buf((u8 *)&pmk_cache_list[i][k]->pmk ,32));
                    printf("PMK-ID   - %s\n",wpa_log_format_buf((u8 *)pmk_cache_list[i][k]->pmk_cache_id ,16));
                    printf("session life time   - %ld seconds\n",(long)(pmk_cache_list[i][k]->session_timeout/1000));
                    printf("session start time  - %ld\n\n",(long)pmk_cache_list[i][k]->session_start_TS);
                }
            }
        }
    }
}



