/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	wpaTask.c	      										   */
/*   PURPOSE:	WPA main task functions.	   			 				   */
/*																									*/
/***************************************************************************/

#define MAIN_MODULE

#include "rg_error.h"
#include <signal.h>
#include <802.1x/wep_port.h>
#include <wpa/wpa_port.h>
#include <ipc.h>
#include <event.h>
#include <802.1x/eap_radius.h>
#include <wpa/wpa_info_elem.h>
#include <wpa/wpa_log.h>
#include <util/random.h>
#include <wpa/wpa_key_utils.h>

#include "hiddenSecurityProfileAPI.h"
#include "wpa_supplicant.h"
#include "dda_wpa.h"
#include "wpaIfc.h"
#include "wpa_common.h"
#include "wpa_assoc.h"
#include "dda_wpaauth_ioctl.h"
#include "securityProfile.h"
#include "debug_report.h"
#include "wpa_pmk_cache.h"


#ifdef VXWORKS
#include "../../cfg/cfgvxw/wcfglib.h"
#include "pform_types.h"
#include "mgmtSsidHandler.h"
#include "wpaIfVxWorks.h"
#include "drvDbMain.h"
#else
#include "wcfglib.h"
#include "linux_ipc_user.h"
#include "pform_types_user.h"
#include <sys/un.h>
#include <stdio.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdio.h>
#include <netinet/if_ether.h>
#include <sys/ioctl.h>
#endif

/*-----------------------------------------------------------*/
/*         defines										 */
/*-----------------------------------------------------------*/
#define NUMBER_OF_SENDING_HELLO_MSG     10

#define WPA_TASK_PRIORITY 190
#define WPA_TASK_STACK_SIZE 16384

#define CFG_PORT_WPA 8801

/* Alarm callback - called by driver with relevant info */
typedef  void (*wdrv_reportFunc)(char *buffer);

/*-----------------------------------------------------------*/
/*         Local Vartiables									 */
/*-----------------------------------------------------------*/

/* socket fd's*/

static int wpaSocket; /* for communication with kernel */
static int dbgReqSock; /* for debug info requests in user space */

#ifndef VXWORKS
static int eapol_fd;
static int pre_auth_eapol_fd;

#else
static int    cfgSocket;
static int    wpaTaskId;
static tiap_wpa_config_t wpaCfg;
#endif

/* Hidden security profile section - used for internal purposes and not for stations */
#ifdef WPA_HIDDEN_PROFILE_SUPPORT
	static int hiddenProfileSock;    	  		/* a socket for hidden profile authentication triggers */
#endif/*if WPA_HIDDEN_PROFILE_SUPPORT*/

eap_method_radius_params_t clientParams;

static int radius_client_index;


/* all port relevant data per ssid plus hidden profile */
static security_port_data_t sec_port_data[TIAP_MAX_NUM_OF_SUPPORTED_PROFILES];
static int wpa_port_started = FALSE;
static ap_dev_ops_t			deviceCBs;		/* To contain the callbacks that the AP supplies to the WPA module */
static unsigned int				apPortHandle = 2;		/* We initializes our AP device in dummy handle. */
static auth1x_pc_t				*pApPort = (auth1x_pc_t *)&apPortHandle;
/* WPA Port Section */
static ap_dev_cbs_t			ap_dev_cbs;		/* This structure used to hold the Callbacks functions 
											that the WPA package supplies to the AP driver. */
static UINT32		apDeviceHandle = 1;		/* We initializes our AP device in dummy handle. */
static ap_dev_t		pApDev = &apDeviceHandle;
static UINT32		local_dev_id = 1;		/* The device ID, we don't use it at all. */
static auth1x_port_cb_t			auth_8021x_callbacks;	/* Forwarded in the '-()' function  */
static auth1x_pc_ops_t 			portCBs;			/* To contain the callbacks that the AP supplies to the 802.1X module */
static auth1x_pkt_handler_t		eapolPacketHandler = NULL;
static BOOL 		wpaStopped = FALSE;

/*-----------------------------------------------------------*/
/*         Local Prototypes									 */
/*-----------------------------------------------------------*/
static UINT16 get_eth_type(u8 *buf);
/* The following callbacks are forwarded to the 802.1X port as part of the port params in the 'auth1x_port_open()' function */
/* WEP Port Section  - temporal only, in order to enable to the 802.1X port to work without WPA. */
static void		    init8021xPortParams(auth1x_port_params_t *params ,int max_sa_duration, int profile_index);
static auth1x_pc_t *open_by_x_port(void *dev_id,
								   auth1x_pkt_handler_t auth1x_pkt_handler, 
								   void *context);

static void	 close_by_x_port(auth1x_pc_t *t);

static eap_method_id_t client_get(void *context,u8 *id, int id_len, int profile_index, void **eap_method_params);
/* The following callbacks are forwarded to the WPA package as part of the port params in the 'wpa_port_open()' function */
static ap_dev_t	*open_by_wpa_port(void *dev_id,
								  ap_dev_cbs_t *cb,
								  void *context);   
static void			initWpaPortParams(wpa_port_params_t *pPortParams);
static int			init_wpa_eapol_handler(void);
static int			init_pre_auth_eapol_handler(void);
static void			close_by_wpa_port(ap_dev_t *t);
static void         set_ap_info_elem(wpa_port_params_t *params , wpa_info_elem_t **ap_info_elem,int *p_info_elem_len,wpa_info_elem_t **rsn_elem,int *p_rsn_elem_len);
void				wdrv_wpaSetMicFailureReportInRx(int value);
int				wdrv_wpaGetMicFailureReportInRx(void);
static int			send_ass_res(UINT16 index,int flag);
int				wdrv_wpaGetStationSecAttack(UINT16 index);
STATUS				canStationAssc(u8 *info_elem, int info_elem_len,int profile_index);	
#ifdef VXWORKS
static void 		wpaTask(void);
static void         wpa_cfg_read(int sock, void *data);
static STATUS       tiwlan_wpaStart(void);
static int          tiwlan_wpaIsInitiated(void);
#else
static int			wpa_wait_for_cfg(tiap_wpa_config_t *wpa_cfg );
static void			wpaTask(tiap_wpa_config_t *wpa_cfg );
static void			eapol_packet_recive(int sock, void *data);
static void			pre_auth_eapol_packet_recive(int sock, void *data);
#endif /*ifdef VXWORKS*/
static void         eapol_packet_process(char * buffer, int bread);
static void			wpa_input(int sock, void *data);
static void 		start_wpa_port(void);
static void 		stop_wpa_port(void);
void 				wdrv_wpaSetStationSecAttack(UINT16 index, int value,int profile_index);
int					wdrv_wpaGetHiddenProfileIndex(void);
static int 			isPortOpen(security_port_data_t *pPortData);
static void			processDbgRequest(int sock, void *data);
static void         printProfileTbls(int profile_index);
static void         printProfileCfg(int profile_index);
static void         getPSK(dbgMsg_T *msg);
static void         wpaGetPSK(int profile_index, char *psk);

static STATUS       isCipherAndPreAuthOK(__u32 default_cipher, __u8 pre_auth_flag, 
                                         tiap_param_security_type_t sec_type, 
                                         BOOL *isWEPexists, BOOL *isPreAuthExists);

#ifndef VXWORKS
/****************************************************************************
 *                      wpa_wait_for_cfg()
 ****************************************************************************
 * DESCRIPTION:	this function creates a UNIX domain datagram socket, binds a name to it,
 * then reads from the socket.	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
static int wpa_wait_for_cfg(tiap_wpa_config_t *wpa_conf) {
	int sock, ret;
	struct sockaddr_un name;

	/* Create socket from which to read. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		printf("wpa_wait_for_cfg: error opening datagram socket\n");
		return -1;
	}

	memset(&name, 0, sizeof(struct sockaddr_un));
	/* Create name. */
	name.sun_family = AF_UNIX;
	
	strcpy((char*)name.sun_path,TIAP_SOCKADDR_WPAAUTH);

	unlink(TIAP_SOCKADDR_WPAAUTH);

	/* Bind the UNIX domain address to the created socket */
	if (bind(sock, (struct sockaddr *) &name, sizeof(struct sockaddr_un)) < 0 )	{
		printf("wpa_wait_for_cfg: error binding datagram socket\n");
		ret = -1;
		goto exit;
	}
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC,"socket %s successfully created, wpa module waits for configuration\n", TIAP_SOCKADDR_WPAAUTH);

	/* Read from the socket */
	if ((ret = read(sock, (char*)wpa_conf, sizeof(*wpa_conf))) < 0)	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"wpa_wait_for_cfg: receiving error datagram packet\n");
		ret = -1;
		goto exit;
	}

	if(ret != sizeof(*wpa_conf)) {
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"wpa_wait_for_cfg: receive error cfg size ,size receive = %d\n",ret);
		ret = -1;
		goto exit;
	}

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "wpa_wait_for_cfg:received configuration\n");
	ret = 0;
 exit:
	close(sock);
	unlink(TIAP_SOCKADDR_WPAAUTH);

	return ret;
}
/****************************************************************************
 *                      main()
 ****************************************************************************
 * DESCRIPTION:	WPA module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int main(void) 
{
	tiap_wpa_config_t wpa_conf ;
	int i;

	wpa_wait_for_cfg(&wpa_conf);

	wdrv_wpaSetMicFailureReportFromStation(TRUE);

	for(i = 0;i< TIAP_MAX_NUM_OF_SUPPORTED_PROFILES;i++)
	{
		sec_port_data[i].micFailureReportInRx = TRUE;
		sec_port_data[i].radius_client = NULL;
		
		memset(sec_port_data[i].secAttackTable, 0, sizeof(sec_port_data[i].secAttackTable));
	}

	wpaTask(&wpa_conf);
	return OK;
}
#endif

/****************************************************************************
 *                      init8021xPortParams()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	profile index.
 *			auth1x_port_params_t params.	
 *		
 * OUTPUT:  auth1x_port_params_t params.
 * 
 * RETURNS:	non
 ****************************************************************************/
static void	init8021xPortParams(auth1x_port_params_t *params,int max_sa_duration, int profile_index)
{
#ifdef VXWORKS
	
	memset(params, 0, sizeof(auth1x_port_params_t));

    params->dev_id = (void *)&local_dev_id;
	wdrv_dbGetOperationMacAddress(params->mac);

#else

	int fd;
	struct ifreq mac;	
	memset(params, 0, sizeof(auth1x_port_params_t));

    params->dev_id = (void *)&local_dev_id;
	fd = socket(AF_INET,SOCK_DGRAM,0);
	
	if(fd < 0)
	{
		printf("init8021xPortParams:error create mac ioctl fd\n");
		return; 
	}


	memset(params->mac, 0, ETH_ALEN);
	mac.ifr_hwaddr.sa_family = ARPHRD_ETHER;
	memcpy(mac.ifr_name, "wlan0", IFNAMSIZ);
	ioctl(fd, SIOCGIFHWADDR,&mac);
	close(fd);
	memcpy(params->mac,
		   mac.ifr_hwaddr.sa_data,
		   ETH_ALEN);
#endif

	memcpy(sec_port_data[profile_index].portParams.mac,params->mac,ETH_ALEN);

	params->is_wireless = 1;
	params->mtu = 1420/* device MTU */;
    
    params->glob_ext_vars.initialize = 1;
    params->glob_ext_vars.port_control = ACPC_AUTO;
    params->glob_ext_vars.port_enabled = 1;
    params->glob_ext_vars.admin_controlled_directions = ACD_BOTH;
    params->glob_ext_vars.bridge_detected = 0;
    /*  params are taken from default params as appear in 802.1X-2001 section
     *  8.5. */
    params->params.quiet_period = 60;
    params->params.re_auth_max = 2;
    params->params.tx_period = 30;
    params->params.key_tx_enabled = 1;
    params->params.re_auth_period = 3600;
    params->params.re_auth_enabled = 0;
    params->params.supp_timeout = 30;
    params->params.server_timeout = 30;
    params->params.max_req = 2;
	params->params.max_sa_duration = max_sa_duration;
    params->profile_index = profile_index;



	/* Now init the AP device CB functions */
	wdrv_wpaApPortInitCallbacks(&portCBs);

	/* Now fill the rest of the AP port CB functions */
	portCBs.open = open_by_x_port;
	portCBs.close = close_by_x_port;
}

/****************************************************************************
 *                      initWpaPortParams()
 ****************************************************************************
 * DESCRIPTION: init all ports params	
 * 
 * INPUTS:	wpa_port_params_t *pPortParams
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	non
 ****************************************************************************/
static void	initWpaPortParams(wpa_port_params_t *pPortParams)
{
	pPortParams->dev_id = &local_dev_id;

	/* Now init the AP device CB functions */

	wdrv_wpaApDevInitCallbacks(&deviceCBs);
	pPortParams->ap_dev_ops = &deviceCBs;
	
	/* Now fill the rest of the AP device CB functions */
	pPortParams->ap_dev_ops->open = open_by_wpa_port;
	pPortParams->ap_dev_ops->close = close_by_wpa_port;
}
/******************************************************************************
*																			  *
*******************************************************************************
DESCRIPTION:init all wpa db,this db will be input for the authenticator package.  	   
                                                                         
INPUTS:		

OUTPUT:			None

RETURNS:		OK.                                                    
******************************************************************************/
static STATUS initWpaDb(tiap_wpa_config_t *wpa_conf,
						wpa_info_elem_t **ap_info_elem,
						int *p_info_elem_len,
						wpa_info_elem_t **rsn_info_elem,
						int *p_rsn_len,
						int profile_index,
						server_list_t **first)
{
	int i, sec_profile_index;

	   sec_profile_index = wpa_conf->mssid.tbl[profile_index].sec_profile_index;
		/* WPA parameters */
	   sec_port_data[profile_index].portParams.allowed_sta_types = wpa_conf->sec_profiles[sec_profile_index].sec.type;	
	   sec_port_data[profile_index].portParams.pre_auth_on = wpa_conf->sec_profiles[sec_profile_index].pmk.pre_auth_flag;
	   sec_port_data[profile_index].portParams.sa_max_duration = wpa_conf->sec_profiles[sec_profile_index].pmk.sa_max_duration;
	   sec_port_data[profile_index].portParams.psk_param   = wpa_conf->sec_profiles[sec_profile_index].psk.pass_phrase_mode;

       memcpy(sec_port_data[profile_index].pskPwd,wpa_conf->sec_profiles[sec_profile_index].psk.psk_passwd,
              wpa_conf->sec_profiles[sec_profile_index].psk.psk_passwd_len);
       
	   if (sec_port_data[profile_index].portParams.psk_param == WPA_PSK_PARAM_HEX)
       {
		   memcpy(sec_port_data[profile_index].portParams.psk.hex.data, 
                  wpa_conf->sec_profiles[sec_profile_index].psk.psk_passwd, WPA_PSK_KEY_LEN);
       }
	   else if (sec_port_data[profile_index].portParams.psk_param == WPA_PSK_PARAM_ASCII)
	   {
		   sec_port_data[profile_index].portParams.psk.ascii = sec_port_data[profile_index].pskPwd;
	   }
	   
	   sec_port_data[profile_index].portParams.rekeying_wep_cipher = wpa_conf->sec_profiles[sec_profile_index].sec.default_cipher;
	   
	   sec_port_data[profile_index].portParams.gtk_update_interval = wpa_conf->sec_profiles[sec_profile_index].sec.interval;// * 1000000; /*3600 * 1000;*/

	/* WEP Keys parameters */
    if(sec_port_data[profile_index].portParams.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) 
    {
    	for (i = 0; i < WPA_NUM_DEFAULT_KEYS; i++)
    	{
    		   switch (wpa_conf->sec_profiles[sec_profile_index].sec.keys[i].len)
    		{
    		case 5:
    			   sec_port_data[profile_index].portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_40;
    			   memcpy(sec_port_data[profile_index].portParams.wep_keys[i].data, wpa_conf->sec_profiles[sec_profile_index].sec.keys[i].key, WPA_WEP_40_KEY_LEN);
    
    			break;
    
    		case 13:
    			   sec_port_data[profile_index].portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_104;
    			   memcpy(sec_port_data[profile_index].portParams.wep_keys[i].data, wpa_conf->sec_profiles[sec_profile_index].sec.keys[i].key, WPA_WEP_104_KEY_LEN);
    
    			break;
    
    		case 29:
    			   sec_port_data[profile_index].portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_232;
    			   memcpy(sec_port_data[profile_index].portParams.wep_keys[i].data, wpa_conf->sec_profiles[sec_profile_index].sec.keys[i].key, WPA_WEP_232_KEY_LEN);
    			break;
    
            default:
                
    			break;
    		}
    	}

        sec_port_data[profile_index].portParams.wep_key_index = wpa_conf->sec_profiles[sec_profile_index].sec.wep_def_key_id;
    }
	   

	/* RADIUS parameters */
    if((sec_port_data[profile_index].portParams.allowed_sta_types & WPA_STA_TYPE_WPA) || 
       (sec_port_data[profile_index].portParams.allowed_sta_types & WPA_STA_TYPE_WPA2) ||
       (sec_port_data[profile_index].portParams.allowed_sta_types & WPA_STA_TYPE_WEP_8021X))  
    {
	   if(wpa_conf->sec_profiles[sec_profile_index].psk.pass_phrase_mode == 0)
	   {
		   sec_port_data[profile_index].radiusParams.servers = &sec_port_data[profile_index].serverList;
		   sec_port_data[profile_index].serverList.next = NULL;
		   memset(sec_port_data[profile_index].serverList.secret, 0, MAX_RADIUS_SECRET_LEN);
		   memcpy(sec_port_data[profile_index].serverList.secret,wpa_conf->sec_profiles[sec_profile_index].radius.secret,
				   wpa_conf->sec_profiles[sec_profile_index].radius.secret_len); 
		   sec_port_data[profile_index].serverList.secret[wpa_conf->sec_profiles[sec_profile_index].radius.secret_len] = 0;
		   sec_port_data[profile_index].serverList.secret_len = wpa_conf->sec_profiles[sec_profile_index].radius.secret_len;
		   sec_port_data[profile_index].serverList.port = wpa_conf->sec_profiles[sec_profile_index].radius.port;
		   sec_port_data[profile_index].serverList.ip.s_addr = wpa_conf->sec_profiles[sec_profile_index].radius.serv_ip_addr;
           sec_port_data[profile_index].serverList.profile_index = profile_index;
		   
		   if(*first == NULL)
		   {
			   *first = sec_port_data[profile_index].radiusParams.servers;
               radius_client_index = profile_index;
			   /*printf(" ** setting radius client index to %d\n",radius_client_index);*/
		   }
		   else
		   {
			   while((*first)->next)
				   (*first) = (*first)->next;
			   (*first)->next = sec_port_data[profile_index].radiusParams.servers;

		   }
	   }

       /* We have to configure the information element to the HAL before we init the HAL. */	   
       set_ap_info_elem(&sec_port_data[profile_index].portParams,ap_info_elem,p_info_elem_len,rsn_info_elem,p_rsn_len);
    }
	   
	return OK;
}

/******************************************************************************
*                      printProfileCfg()											  *
*******************************************************************************
DESCRIPTION:	print profile configuration 
                                                                         
INPUTS:			profile_index - profile index.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void printProfileCfg(int profile_index)
{
	/* primary ssid */

	int mode,value;
	struct in_addr addr;
	
	if (sec_port_data[profile_index].profile_type == SEC_PROFILE_NON_ACTIVE) 
	{
		printf("The given profile is inactive\n");
		return;
	}
	printf("\n");
	mode = sec_port_data[profile_index].portParams.allowed_sta_types;

	if (mode & TIAP_PARAM_SECURITY_WEP)
		printf("	Legacy Stations		- On\n");
	else
		printf("	Legacy Stations		- Off\n");
	
	if (mode & TIAP_PARAM_SECURITY_802_1X)
		printf("	EAP-TLS Stations	- On\n");
	else
		printf("	EAP-TLS Stations	- Off\n");
	
	if (mode & TIAP_PARAM_SECURITY_WPA)
		printf("	WPA Stations		- On\n");
	else
		printf("	WPA Stations		- Off\n");
	
	if (mode & TIAP_PARAM_SECURITY_WPA2)
		printf("	WPA2 Stations		- On\n");
	else
		printf("	WPA2 Stations		- Off\n");
	
	
	value = sec_port_data[profile_index].portParams.psk_param;

	
	if (value == TIAP_WPA_PSK_PARAM_NONE)
		printf("	Pass Phrase Mode	- None\n");
	else if (value == TIAP_WPA_PSK_PARAM_HEX)
		printf("	Pass Phrase Mode	- HEX\n");
	else if (value == TIAP_WPA_PSK_PARAM_ASCII)
		printf("	Pass Phrase Mode	- ASCII\n");
	else 
		printf("	Pass Phrase Mode	- Invalid\n");
	
	value = sec_port_data[profile_index].portParams.rekeying_wep_cipher;
	
	if (value == TIAP_WPA_CIPHER_NON)
		printf("	Default Cipher		- NONE\n");
	else if (value == TIAP_WPA_CIPHER_WEP_40)
		printf("	Default Cipher		- WEP - 40\n");
	else if (value == TIAP_WPA_CIPHER_WEP_104)
		printf("	Default Cipher		- WEP - 104\n");
	else if (value == TIAP_WPA_CIPHER_WEP_232)
		printf("	Default Cipher		- WEP - 232\n");
	else if (value == TIAP_WPA_CIPHER_TKIP)
		printf("	Default Cipher		- TKIP\n");
	else if (value == TIAP_WPA_CIPHER_CCMP)
		printf("	Default Cipher		- CCMP\n");
	else
		printf("	Default Cipher		Invalid\n");
	
	
	printf("	PSK Password:			%s\n", sec_port_data[profile_index].pskPwd);
	
	value = sec_port_data[profile_index].portParams.gtk_update_interval;

	printf("	Group Key update Interval:	%d	seconds\n", value / 1000);
	
	if(sec_port_data[profile_index].portParams.pre_auth_on)
		printf("	pre authentication support:	on\n" );
	else
		printf("	pre authentication support:	off\n" );
	
	printf("	Radius Secret:			%s\n", sec_port_data[profile_index].serverList.secret);
	
	printf("	Radius Port:			%hu\n", sec_port_data[profile_index].serverList.port);
	
	addr.s_addr = sec_port_data[profile_index].serverList.ip.s_addr;

	printf("	Radius IP Address:		%s\n", inet_ntoa(addr));
		
#ifndef VXWORKS
	fflush(NULL);
#endif
}

#ifdef VXWORKS
/******************************************************************************
*                      wpa_dda_send_msg()									  *
*******************************************************************************
DESCRIPTION:	sends message to the WPA task via socket	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int wpa_dda_send_msg(UINT16 port, void *msg, int len)
{
	UINT32 rc;

	rc = ipc_buf_write(port, msg, len);
	if (rc)
	{
		printf("WPA: (%s) Error in 'ipc_buf_write'. Error code %d\n",__func__, rc);
	}

	return rc;
}

/******************************************************************************
*                      recvEapolPacket()  VXWORKS   								  *
*******************************************************************************
DESCRIPTION:	handles reception of packets from MUX, filters out non EAPOL packets	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static BOOL recvEapolPacket(void *pCookie, long type, 
					 M_BLK_ID pMblk, LL_HDR_INFO *pLinkHdrInfo,
					 void * pMode)
{
	UINT32          rc;
	wpaMessage_T	msg;
	int  			len = 0, i;
    M_BLK_ID 		pTmpMblk = pMblk;
    char 			*pData = msg.content.eapolFrame.pEapol;
	UINT16			ether_type;

	/* check if WPA port is open */
	if(wpa_port_started == FALSE)
		return FALSE;    

	ether_type = get_eth_type(pTmpMblk->mBlkHdr.mData);
	if(ether_type == EAPOL_ETHER_TYPE)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Eapol frame received from OS\n");
        msg.type = EAPOL_FRAME;
	}
    else if (ether_type == EAPOL_ETHER_PRE_AUTH_TYPE)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Preath Eapol frame received from OS\n");
        msg.type = EAPOL_FRAME_PRE_AUTH;
	}
    else
		/* indicate MUX to pass packet to the stack */
		return FALSE;

	/* copy EAPOL frame from the mblk to the msg */
    while( pTmpMblk)
    {
        memcpy( pData, pTmpMblk->mBlkHdr.mData, pTmpMblk->mBlkHdr.mLen);
		len += pTmpMblk->mBlkHdr.mLen;
        pData += pTmpMblk->mBlkHdr.mLen;
        pTmpMblk = pTmpMblk->mBlkHdr.mNext;
    }

	/* find station port*/
    if(ether_type == EAPOL_ETHER_TYPE)    
    {

#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		/* If WPA supplicant is enabled */
		/* Check if the packet is addressed to it and not to authenticator */        
		if(hidden_profile_IsEapolToSup((eapol_pkt_t *)msg.content.eapolFrame.pEapol))
		{
			int hiddenProfileInd = wdrv_wpaGetHiddenProfileIndex();
			
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Eapol to supplicant \n");
			
			if (hiddenProfileInd == -1) {
				netMblkClChainFree (pMblk);
				return TRUE;
			}
			wpas_HandleEapol((eapol_pkt_t *)msg.content.eapolFrame.pEapol,len, 
							 sec_port_data[hiddenProfileInd].pWpaPortContext);
			netMblkClChainFree (pMblk);
			return TRUE;
		}
		else
#endif
		{
			/* find station port*/
			for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
			{
                if(!isPortOpen(&(sec_port_data[i])))
                    continue;

				if(wpa_port_is_station_exist(sec_port_data[i].pWpaPortContext,&msg.content.eapolFrame.pEapol[6]))
                    break;
			}
		}
	}
    else /* preauthentication */
    {
    	for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
    	{
            if(!isPortOpen(&(sec_port_data[i])))
                continue;

            /*if(wpa_port_is_station_exist(sec_port_data[i].pXPortContext,&msg.content.eapolFrame.pEapol[6]))*/
            if( (sec_port_data[i].profile_type == SEC_PROFILE_REGULAR) && 
                (sec_port_data[i].portParams.pre_auth_on == 1) )
            {
                break;
            }
        }
    }

    if(i == TIAP_MAX_NUM_OF_SUPPORTED_PROFILES)
    {
		/* free the mblk */
		netMblkClChainFree (pMblk);
		return TRUE;
    }
	
    msg.profile_index = i;
	msg.content.eapolFrame.encrypted = 0;
	msg.content.eapolFrame.eapolDataLen = len;
	
	/* send the EAPOL packet to the WPA task */
	rc = ipc_buf_write(htons(RG_IPC_PORT_WPA), (void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"WPA: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
	}

	/* free the mblk */
	netMblkClChainFree (pMblk);

	/* indicate MUX not to pass the packet to the stack */
	return TRUE;
}

/******************************************************************************
*                      tiwlan_wpaInit()     	        					  *
*******************************************************************************
DESCRIPTION:    Crates WPA task and creates mutex used for synchronization with cfg  
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:	    None
******************************************************************************/
void tiwlan_wpaInit(void)
{
	STATUS ret;

	/* If WPA task was already created do nothing */
    if(tiwlan_wpaIsInitiated())
		return;

	wpaTaskId = taskSpawn( "tWPA",           	       	  /* name of WPA task         */
						   WPA_TASK_PRIORITY,          	  /* priority of WPA task     */
						   0,                      	   	  /* task option word         */
						   WPA_TASK_STACK_SIZE,    	  	  /* task stack size (bytes)  */
						   (FUNCPTR) wpaTask,   	   	  /* entry point of DDA task  */
						   0,0,0,0,0,0,0,0,0,0            /* arguments to pass        */
						 ); 

	if (wpaTaskId == ERROR) 
	{
		printf("WDRV_WPA: Failed to create WPA task\n" );
		exit(1);
	}

    cfgSocket = ipc_bind_listen_port(htons(CFG_PORT_WPA));

	if (cfgSocket < 0)
	{
		printf("WDRV_WPA: WPA cfg socket init failure \n");
		exit(1);
	}

	/* socket for debug info requests*/
	ret = ipc_create_sock(DBG_WPA_SOCK_PORT, &dbgReqSock);
	if (ret != OK) 
	{
		printf("WDRV_WPA: Failed to create a socket debug requests \n");
		exit(1);
	}

}

/******************************************************************************
*                      tiwlan_wpaCleanup()     	        					  *
*******************************************************************************
DESCRIPTION:    Cleans up WPA module by sending STOP message to WPA task  
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:	    OK/NOK
******************************************************************************/
STATUS tiwlan_wpaCleanup(void)
{
	int		        rc;
	wpaMessage_T	msg;

	/* send stop message to WPA task */
	msg.type = WPA_STOP;	
	rc = ipc_buf_write(htons(RG_IPC_PORT_WPA), (void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printf("WPA: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
	}
	
    return OK;
}

/******************************************************************************
*                      wpa_cfg_read()      		        					  *
*******************************************************************************
DESCRIPTION:	Gets WPA configuration via predefined socket, calls WPA init function      
                                                                         
INPUTS:			sock - socket opened for configuration

OUTPUT:			data - not relevant

RETURNS:	    None
******************************************************************************/
static void wpa_cfg_read(int sock, void *data)
{
	int rc;

	rc = ipc_accept_buf_read(sock, &wpaCfg, sizeof(wpaCfg));

	if (rc)
	{
	    DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"Got %d bytes, expect %d", rc, sizeof(wpaCfg));
	    return;
	}

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC,"WPA: Configuration received \n");

    tiwlan_wpaStart();
}

/******************************************************************************
*                      tiwlan_sendCfg()     	        					  *
*******************************************************************************
DESCRIPTION:	This service function sends configuration to the WPA task      
                                                                         
INPUTS:			pCfg - pointer to the configuration structure

OUTPUT:			None

RETURNS:	    OK/NOK
******************************************************************************/
STATUS tiwlan_sendCfg(tiap_wpa_config_t *pCfg)
{
	UINT32 rc;

	rc = ipc_connect_buf_write(htons(CFG_PORT_WPA), (void *)pCfg, sizeof(*pCfg));
	if (rc)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"WPA: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}

/******************************************************************************
*                      tiwlan_wpaRestart()      							  *
*******************************************************************************
DESCRIPTION:	Restarts WPA authenticator, sends HELLO message to the driver      
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:	    OK/NOK
******************************************************************************/
static STATUS tiwlan_wpaStart(void)
{
    tiap_wpa_config_t *wpa_cfg = &wpaCfg;
	wpaMessage_T msg;
	int 		 rc, i;
	wpa_info_elem_t *ap_info_elem = NULL;
	wpa_info_elem_t *rsn_info_elem = NULL;
	int info_elem_len = 0,rsn_len = 0;
	int profile_index;
    int num_hidden_profiles = 0;
	wpaMessage_T hello_message;
    server_list_t   *radius_list_first = NULL;
    BOOL isWEPexists = FALSE, isPreAuthExists = FALSE;

    memset(sec_port_data, 0, sizeof(sec_port_data));

    radius_client_index = 0;

	dbgSeverity = wpa_cfg->dbg_report_sev;
	dbgModule = wpa_cfg->dbg_report_mod;

    for(i = 0;i< MAX_NUM_OF_SUPPORTED_PROFILES;i++)
	{
		sec_port_data[i].micFailureReportInRx = TRUE;
	}

    wdrv_wpaSetMicFailureReportFromStation(TRUE);
	/* Multiple ssid init */
	/* One primary profile always exists even if multiple ssid is disabled and AP is configured with no privacy*/
	
	for (i=0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++) 
	{
        if(wpa_cfg->mssid.tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE)
        {  
            /* If the current profile is configured with static WEP or the          *
             * Pre-Auth flag is ON, and a preevious profile is also configured      *
             * with static WEP or Pre-Auth, respectivaly --> return NOK(Abort Init) *
             *                                                                      *
             * NOTE: In each iteration, the function uses the values of             *
             *       'isWEPexists' & 'isPreAuthExists' that were returned from      *
             *       the same function in the previous iteration.                   */
            if(isCipherAndPreAuthOK(wpa_cfg->sec_profiles[i].sec.default_cipher, 
                wpa_cfg->sec_profiles[i].pmk.pre_auth_flag, 
                wpa_cfg->sec_profiles[i].sec.type, 
                &isWEPexists, &isPreAuthExists) != OK)
            {
                return NOK;
            }
        }

		switch (wpa_cfg->mssid.tbl[i].ssid_type) 
		{
		case TIAP_PARAM_SSID_NON_ACTIVE:
				sec_port_data[i].profile_type = SEC_PROFILE_NON_ACTIVE;
				break; 
		case TIAP_PARAM_SSID_INTERNAL:
				sec_port_data[i].profile_type = SEC_PROFILE_HIDDEN;
				num_hidden_profiles ++;
				break;

		case TIAP_PARAM_SSID_PRIMARY:
		case TIAP_PARAM_SSID_SECONDARY:
				sec_port_data[i].profile_type = SEC_PROFILE_REGULAR;
				break;
		default:
			printf("wpaTask: wrong multiple ssid type received\n");
			return NOK;
		}

		if (sec_port_data[i].profile_type == SEC_PROFILE_NON_ACTIVE) 
			continue;

		strcpy(sec_port_data[i].Ssid,wpa_cfg->mssid.tbl[i].ssid);
		sec_port_data[i].portParams.ssid = sec_port_data[i].Ssid;
		sec_port_data[i].portParams.ssid_len = strlen(sec_port_data[i].portParams.ssid);		
	}

	/* set hidden profile index */
#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		if (num_hidden_profiles == 0) 
		{
			printf("WPA: Error: hidden profile configuration wasn't supplied\n");
			hidden_profile_Disable();
		}
#endif /* ifdef WPA_HIDDEN_PROFILE_SUPPORT */

	hello_message.type = WPA_HELLO;
	for(profile_index = 0;profile_index < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; profile_index++)
	{
		info_elem_len = 0;rsn_len = 0;

		if (sec_port_data[profile_index].profile_type != SEC_PROFILE_NON_ACTIVE) 
		{
		
			/* create wpa db */
			initWpaDb(wpa_cfg,&ap_info_elem,&info_elem_len,&rsn_info_elem,&rsn_len,profile_index,&radius_list_first);

			/* create hello msg */
			if(info_elem_len > 0)
				{
					memcpy(hello_message.content.hello.info[profile_index],(UINT8 *)ap_info_elem,info_elem_len);
					wpa_info_elem_close(ap_info_elem);
				}
					
	
			if(rsn_len > 0)
			{
				memcpy(hello_message.content.hello.rsn[profile_index],(UINT8 *)rsn_info_elem,rsn_len);
				wpa_info_elem_close(rsn_info_elem);
			}
			hello_message.content.hello.cipher[profile_index] = wpa_cfg->sec_profiles[profile_index].sec.default_cipher;
            hello_message.content.hello.secType[profile_index] = wpa_cfg->sec_profiles[profile_index].sec.type;
			/*printf("*** sending hello msg with cipher  = %d ***\n",hello_message.content.hello.cipher[profile_index]);*/

		}

		hello_message.content.hello.info_len[profile_index] = info_elem_len;
		hello_message.content.hello.rsn_len[profile_index] = rsn_len;

	}

	dda_wpa_receive(&hello_message, sizeof(hello_message));

    if(tiwlan_wpaIsInitiated()) {
    	/* send resume message to WPA task */
    	msg.type = WPA_RESUME;	
    	rc = ipc_buf_write(htons(RG_IPC_PORT_WPA), (void *)&msg, sizeof(wpaMessage_T));
    	if (rc)
    	{
    		printf("WPA: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
    	}
    }

	return OK;
}

/******************************************************************************
*                      tiwlan_wpaIsInitiated()  							  *
*******************************************************************************
DESCRIPTION:	returns an indication whether WPA task is up already	   
                                                                         
INPUTS:			

OUTPUT:			None

RETURNS:	    WPA task ID
******************************************************************************/
static int tiwlan_wpaIsInitiated(void)
{
	return wpaTaskId;
}

/******************************************************************************
*                      tiwlan_wpaSuspend()  							  *
*******************************************************************************
DESCRIPTION:	suspends WPA task	   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:	    None
******************************************************************************/
void tiwlan_wpaSuspend(void)
{
	if (wpaTaskId)
	{
		if( taskSuspend( wpaTaskId ) != OK )
		{
			printf("tiwlan_wpaSuspend: task suspend failed\n");
		}
	}
}

/******************************************************************************
*                      tiwlan_wpaResume()  							  *
*******************************************************************************
DESCRIPTION:	resumes WPA task	   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:	    None
******************************************************************************/
void tiwlan_wpaResume(void)
{
	if (wpaTaskId)
	{
		if( taskResume( wpaTaskId ) != OK )
		{
			printf("tiwlan_wpaResume: task resume failed\n");
		}
	}
}

/******************************************************************************
*                      wpaTask()	VXWORKS										  *
*******************************************************************************
DESCRIPTION:	Initialize WPA task   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wpaTask(void)
{
	STATUS ret;

    if ( ipc_create_sock(htons(RG_IPC_PORT_WPA), &wpaSocket) )
	{
		printf("%s: ipc_create_sock failed\n", __func__);
		return;
	}
	
	if (wpaSocket < 0)
	{
		printf("WDRV_WPA: WPA socket init failure \n");
		return;
	}

    random_init();

	event_loop_init();

	/* init eapol handle */
	if(init_wpa_eapol_handler() == -1)
	{
		printf("WDRV_WPA: WPA module error intialized\n");
		return;
	}

	/* init pre auth handler */
	if(init_pre_auth_eapol_handler() == -1)
	{
		printf("WDRV_WPA: WPA module error intialized\n");
		return;
	}

	#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		/* init a socket for listening for messages to hidden authenticator/supplicant instance*/
		ret = hidden_profile_PrepareRequestsSock(&hiddenProfileSock);
		if (ret != OK) 
		{
			printf("WDRV_WPA: Failed to create a socket for hidden WPA instance \n");
			hidden_profile_Disable();
		}
		else
		{
			/* Add the socket to the read descriptors list */
			event_fd_set(hiddenProfileSock, EFD_READ, hidden_profile_ProcessInputMsg, NULL);
		}		
	#endif /* ifdef WPA_HIDDEN_PROFILE_SUPPORT */

	event_fd_set(dbgReqSock, EFD_READ, processDbgRequest, NULL);

	event_fd_set(wpaSocket, EFD_READ, wpa_input, NULL);

    event_fd_set(cfgSocket, EFD_READ, wpa_cfg_read, NULL);

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"WDRV_WPA: WPA module initialized successfully ,waiting for driver hello..\n");

	event_loop();
}
#else
/******************************************************************************
*                      wpaTask()	LINUX									  *
*******************************************************************************
DESCRIPTION:	Initialize WPA task   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void wpaTask(tiap_wpa_config_t *wpa_cfg) 
{ 
	wpaMessage_T hello_message;
	wpa_info_elem_t *ap_info_elem = NULL;
	wpa_info_elem_t *rsn_info_elem = NULL;
    int info_elem_len,rsn_len,profile_index,rc = 0,retryCount = 0;
	server_list_t   *radius_list_first = NULL;
	int num_hidden_profiles = 0;
	int i,ret;
    BOOL isWEPexists = FALSE, isPreAuthExists = FALSE;
	
	random_init();
	
    radius_client_index = 0;

	dbgSeverity = wpa_cfg->dbg_report_sev;
	dbgModule = wpa_cfg->dbg_report_mod;

	/* Multiple ssid init */
	/* One primary profile always exists even if multiple ssid is disabled and AP is configured with no privacy*/
	
	for (i=0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++) 
	{
        if(wpa_cfg->mssid.tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE)
        {
            /* If the current profile is configured with static WEP or the          *
             * Pre-Auth flag is ON, and a preevious profile is also configured      *
             * with static WEP or Pre-Auth, respectivaly --> return NOK(Abort Init) *
             *                                                                      *
             * NOTE: In each iteration, the function uses the values of             *
             *       'isWEPexists' & 'isPreAuthExists' that were returned from      *
             *       the same function in the previous iteration.                   */
            if(isCipherAndPreAuthOK(wpa_cfg->sec_profiles[i].sec.default_cipher, 
                wpa_cfg->sec_profiles[i].pmk.pre_auth_flag, 
                wpa_cfg->sec_profiles[i].sec.type, 
                &isWEPexists, &isPreAuthExists) != OK)
            {
                return;
            }
        }
        
		switch (wpa_cfg->mssid.tbl[i].ssid_type) 
		{
		case TIAP_PARAM_SSID_NON_ACTIVE:
				sec_port_data[i].profile_type = SEC_PROFILE_NON_ACTIVE;
				break; 
		case TIAP_PARAM_SSID_INTERNAL:
				sec_port_data[i].profile_type = SEC_PROFILE_HIDDEN;
				num_hidden_profiles ++;
				break;

		case TIAP_PARAM_SSID_PRIMARY:
		case TIAP_PARAM_SSID_SECONDARY:
				sec_port_data[i].profile_type = SEC_PROFILE_REGULAR;
				break;
		default:
			printf("wpaTask: wrong multiple ssid type received\n");
			return;
		}

		if (sec_port_data[i].profile_type == SEC_PROFILE_NON_ACTIVE) 
			continue;

		strcpy(sec_port_data[i].Ssid,wpa_cfg->mssid.tbl[i].ssid);
		sec_port_data[i].portParams.ssid = sec_port_data[i].Ssid;
		sec_port_data[i].portParams.ssid_len = strlen(sec_port_data[i].portParams.ssid);
		

	}

	/* set hidden profile index */
#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		if (num_hidden_profiles == 0) 
		{
			printf("WPA: Error: hidden profile configuration wasn't supplied\n");
			hidden_profile_Disable();
		}
#endif /* ifdef WPA_HIDDEN_PROFILE_SUPPORT */

	hello_message.type = WPA_HELLO;
	for(profile_index = 0;profile_index < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; profile_index++)
	{
		info_elem_len = 0;rsn_len = 0;

		if (sec_port_data[profile_index].profile_type != SEC_PROFILE_NON_ACTIVE) 
		{
		
			/* create wpa db */
			initWpaDb(wpa_cfg,&ap_info_elem,&info_elem_len,&rsn_info_elem,&rsn_len,profile_index,&radius_list_first);

			/* create hello msg */
			if(info_elem_len > 0)
				{
					memcpy(hello_message.content.hello.info[profile_index],(UINT8 *)ap_info_elem,info_elem_len);
					wpa_info_elem_close(ap_info_elem);
				}
					
	
			if(rsn_len > 0)
			{
				memcpy(hello_message.content.hello.rsn[profile_index],(UINT8 *)rsn_info_elem,rsn_len);
				wpa_info_elem_close(rsn_info_elem);
			}
			hello_message.content.hello.cipher[profile_index] = wpa_cfg->sec_profiles[profile_index].sec.default_cipher;
            hello_message.content.hello.secType[profile_index] = wpa_cfg->sec_profiles[profile_index].sec.type;
			printf("*** sending hello msg with cipher  = %d SecurityType = %d***\n", 
                   hello_message.content.hello.cipher[profile_index], 
                   hello_message.content.hello.secType[profile_index]);

		}

		hello_message.content.hello.info_len[profile_index] = info_elem_len;
		hello_message.content.hello.rsn_len[profile_index] = rsn_len;

	}


	wpaSocket = ipc_bind_listen_port(0);

	/* send hello to drv*/
	while( retryCount++ < NUMBER_OF_SENDING_HELLO_MSG)
	{
		rc = ipc_connect_buf_write(wpaSocket, &hello_message, sizeof(wpaMessage_T));
	
		if (rc == -1)
		{
			printf("wpa auth:waiting for driver to come up...\n");
		}
		else
			break;
		if (wpaSocket < 0)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT,"WDRV_WPA: WPA socket init failure, trying again... \n");
		}
		
		usleep(2000000);
	}
	

	if(retryCount == NUMBER_OF_SENDING_HELLO_MSG)
	{
		printf("WDRV_WPA: error %d in 'ipc_connect_buf_write \nCANT FIND DIVER,ABORT \n",rc);
		return;
	}
	
	event_loop_init();

	/* init eapol handle*/
	if(init_wpa_eapol_handler() == -1)
	{
		printf("WDRV_WPA: WPA eapol_handler module error intialized\n");
		return;
	}

	/* init pre auth handler*/
	if(init_pre_auth_eapol_handler() == -1)
	{
		printf("WDRV_WPA: WPA pre_auth_eapol module error intialized\n");
		return;
	}

	#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		/* init a socket for listening for messages to hidden authenticator/supplicant instance*/
		ret = hidden_profile_PrepareRequestsSock(&hiddenProfileSock);
		if (ret != OK) 
		{
			printf("WDRV_WPA: Failed to create a socket for hidden WPA instance \n");
			hidden_profile_Disable();
		}
		else
		{
			/* Add the socket to the read descriptors list */
			event_fd_set(hiddenProfileSock, EFD_READ, hidden_profile_ProcessInputMsg, NULL);
		}		
	#endif /* ifdef WPA_HIDDEN_PROFILE_SUPPORT */
	
	ipc_create_sock(DBG_WPA_SOCK_PATH, &dbgReqSock);

	event_fd_set(dbgReqSock,EFD_READ,processDbgRequest, NULL);

	event_fd_set(wpaSocket, EFD_READ, wpa_input, NULL);

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"WDRV_WPA: WPA module initialized successfully ,waiting for driver hello..\n");

	event_loop();

}
#endif


/******************************************************************************
*                      start_wpa_port()											  *
*******************************************************************************
DESCRIPTION:  start wpa port
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void start_wpa_port(void)
{

	int i = 0;

	for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
	{
	    if(!isPortOpen(&(sec_port_data[i])))
            continue;
	/* Init the 802.1X port params */
		init8021xPortParams(&sec_port_data[i].auth_8021x_params,
							sec_port_data[i].portParams.sa_max_duration,i);

		sec_port_data[i].portParams.ssid = sec_port_data[i].Ssid;
		sec_port_data[i].portParams.ssid_len = strlen(sec_port_data[i].portParams.ssid);
		

	/* Init 802.1X port callback */
        auth_8021x_callbacks.client_get = client_get;
        auth_8021x_callbacks.new_assoc = NULL;
		auth_8021x_callbacks.eapol_start_received = wpa_assoc_eapol_start_cb;

	/* Now open the radius client */	

		if(radius_client_index == i)
		{
            sec_port_data[i].radius_client = radius_client_open(&sec_port_data[i].radiusParams);
            
            if (sec_port_data[i].radius_client == NULL)
            {
                printf("WDRV_WPA: Failed to open radius client\n");
                return;
            }
            /*printf("** radius  for index %d = 0x%x\n",i,(unsigned int)sec_port_data[i].radius_client);*/
        }
		

	/* Open 802.1X port */
		sec_port_data[i].pXPortContext = auth1x_port_open(&sec_port_data[i].auth_8021x_params, 
	    &auth_8021x_callbacks, &portCBs, NULL);
		if (sec_port_data[i].pXPortContext == NULL)
	{
		printf("WDRV_WPA: Failed to open X port\n");
		return;
	}
	/* Init the WPA port params */
		initWpaPortParams(&sec_port_data[i].portParams);
		sec_port_data[i].portParams.auth1x_port = sec_port_data[i].pXPortContext;
		

	/* Now open the WPA port */
        sec_port_data[i].pWpaPortContext = wpa_port_open(&sec_port_data[i].portParams,i);
        if (sec_port_data[i].pWpaPortContext == NULL)
        {
            printf("WDRV_WPA: Failed to open WPA port ,in index %d\n",i);
            return;
        }
	}

	wpa_port_started = TRUE;
}

/******************************************************************************
*                      stop_wpa_port()											  *
*******************************************************************************
DESCRIPTION:
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void stop_wpa_port(void)
{
    int i;

    wpa_port_started = FALSE;
	for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
	{
		if (sec_port_data[i].profile_type == SEC_PROFILE_NON_ACTIVE) 
			continue;

        if((sec_port_data[i].portParams.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) &&
           (sec_port_data[i].portParams.rekeying_wep_cipher == WPA_CIPHER_NON))
            continue;
        
		if(radius_client_index == i)
            radius_client_close(sec_port_data[i].radius_client);
        auth1x_port_close(sec_port_data[i].pXPortContext);
        wpa_port_close(sec_port_data[i].pWpaPortContext,i);        
    }
}


/******************************************************************************
*                      wpa_input()											  *
*******************************************************************************
DESCRIPTION:  reciev an event from  pending socket.
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void wpa_input(int sock, void *data)
{	
	wpaMessage_T	message;
	int rc;

	rc = read(sock, (char *)&message, sizeof(wpaMessage_T) );

	if ( rc != sizeof(wpaMessage_T) )
	{
	    DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"Got %d bytes. expect %d", rc,
		sizeof(wpaMessage_T));
	    return;
	}

	/* check if this is local WPA message */
	if(message.type == WPA_STOP)
	{
		wpaStopped = TRUE;
        stop_wpa_port();
		return;
	}
	else if(message.type == WPA_RESUME)
	{
		printf("WPA resume message received...\n");
		wpaStopped = FALSE;
		return;
	}

	/* don't proceed in case WPA is stopped */
	if(wpaStopped == TRUE)
		return;

    if (! wpa_port_started && message.type != WPA_DRV_HELLO) {
        printf("%s : WPA is not ready yet! msg_type=%d\n", __FUNCTION__,message.type);
        return;
    }

	switch (message.type)
	{
		case EAPOL_FRAME:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: EAPOL_FRAME message received\n");
			if (eapolPacketHandler != NULL)
			{
				
				eapolPacketHandler(sec_port_data[message.profile_index].pXPortContext,
								   message.content.eapolFrame.pEapol,
								   message.content.eapolFrame.eapolDataLen,
								   message.content.eapolFrame.encrypted);
			}
			break;

        case EAPOL_FRAME_PRE_AUTH:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: EAPOL_FRAME_PRE_AUTH message received from driver\n");
			if (eapolPacketHandler != NULL)
			{
				receivePreAuthRequest(sec_port_data[message.profile_index].pWpaPortContext,
                                      message.content.eapolFrame.pEapol);
				eapolPacketHandler(sec_port_data[message.profile_index].pXPortContext,
								   message.content.eapolFrame.pEapol,
								   message.content.eapolFrame.eapolDataLen,
								   message.content.eapolFrame.encrypted);
			}
			break;
        case EAPOL_FRAME_INT_PROF:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: EAPOL_FRAME_INT_PROF message received\n");
            eapol_packet_process(message.content.eapolFrame.pEapol, message.content.eapolFrame.eapolDataLen);
            break;

		case NOTIFY_STATION_CONNECT:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: NOTIFY_STATION_CONNECT message received from driver\n");
			if (ap_dev_cbs.sta_assoc_done != NULL)
			{
				if(message.profile_index < 0 || message.profile_index > 4)
				{
					DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"NOTIFY_STATION_CONNECT: received error profile index %d\n", message.profile_index);
					return;
				}
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "NOTIFY_STATION_CONNECT: profile index = %d, STA mac = %02X:%02X:%02X:%02X:%02X:%02X\n", message.profile_index,
                              (UINT8)message.content.stationConnect.mac[0], (UINT8)message.content.stationConnect.mac[1],(UINT8) message.content.stationConnect.mac[2],
						  (UINT8)message.content.stationConnect.mac[3],(UINT8)message.content.stationConnect.mac[4],(UINT8)message.content.stationConnect.mac[5]);
				/* If the eleLen is zero, than the RSN info element is NULL */
				if (message.content.stationConnect.rsne.hdr.eleLen > 0)
				{
					ap_dev_cbs.sta_assoc_done(sec_port_data[message.profile_index].pWpaPortContext,
											  message.content.stationConnect.mac,
											  (UINT8 *)&(message.content.stationConnect.rsne),
											  message.content.stationConnect.rsne.hdr.eleLen + 2);
				}
				else
				{

					ap_dev_cbs.sta_assoc_done(sec_port_data[message.profile_index].pWpaPortContext,
											  message.content.stationConnect.mac,
											  NULL,
											  message.content.stationConnect.rsne.hdr.eleLen + 2);
				}
			}
			break;

		case NOTIFY_STATION_DIS_CONNECT:				
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: NOTIFY_STATION_DIS_CONNECT message received from driver\n");
			if(message.profile_index < 0 || message.profile_index > 4)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"NOTIFY_STATION_DIS_CONNECT: received error profile index %d\n", message.profile_index);
				return;
			}
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "NOTIFY_STATION_DIS_CONNECT: profile index = %d, STA mac = %02X:%02X:%02X:%02X:%02X:%02X\n",
						  message.profile_index, (UINT8)message.content.stationDisConnect.mac[0], (UINT8)message.content.stationDisConnect.mac[1],(UINT8)message.content.stationDisConnect.mac[2],
						  (UINT8)message.content.stationDisConnect.mac[3],(UINT8)message.content.stationDisConnect.mac[4],(UINT8)message.content.stationDisConnect.mac[5]);
			if (ap_dev_cbs.sta_disassoc != NULL) {
                if(message.profile_index == wdrv_wpaGetHiddenProfileIndex()) {
                    hidden_profile_CloseAuth(message.content.stationDisConnect.mac);
                }
                else {
                    ap_dev_cbs.sta_disassoc(sec_port_data[message.profile_index].pWpaPortContext, 
                                            message.content.stationDisConnect.mac);
                }
            }
			break;

		case FIRST_DATA_FROM_STATION:				
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: FIRST_DATA_FROM_STATION message received from driver\n");
			if(message.profile_index < 0 || message.profile_index > 4)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"FIRST_DATA_FROM_STATION: received error profile index %d\n", message.profile_index);
				return;
			}
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "FIRST_DATA_FROM_STATION: profile index = %d\n", message.profile_index);
			if (ap_dev_cbs.first_data_packet_received != NULL)
				ap_dev_cbs.first_data_packet_received(sec_port_data[message.profile_index].pWpaPortContext, message.content.firstDataFromStation.isEapol, message.content.firstDataFromStation.mac);
			break;

		case MIC_FAILURE:				
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: MIC_FAILURE message received from driver\n");
			if(message.profile_index < 0 || message.profile_index > 4)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"MIC_FAILURE: received error profile index %d\n", message.profile_index);
				return;
			}
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "MIC_FAILURE: profile index = %d\n", message.profile_index);
			if (ap_dev_cbs.unicast_mic_failure != NULL)
			{
				if (sec_port_data[message.profile_index].micFailureReportInRx)
					ap_dev_cbs.unicast_mic_failure(sec_port_data[message.profile_index].pWpaPortContext, message.content.micFailure.mac);
			}
			break;
		case CAN_STA_ASS:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: CAN_STA_ASS message received from driver\n");
			if(message.profile_index < 0 || message.profile_index > 4)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"CAN_STA_ASS: received error profile index %d\n", message.profile_index);
				return;
			}
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "CAN_STA_ASS: profile index = %d\n", message.profile_index);
			if(canStationAssc(message.content.canStaAss.info_elem,
							  message.content.canStaAss.info_elem_len,
							  message.profile_index) == OK)
			{
				send_ass_res(message.content.canStaAss.index,1);
			}
			else
			{
				send_ass_res(message.content.canStaAss.index,0);
				/*printf("station can not !!!!! assoc\n" );fflush(NULL);*/
			}
			break;
		case WPA_DRV_HELLO:
            DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: WPA_DRV_HELLO message received from driver\n");
			start_wpa_port();            
			break;

		case SET_STA_ATTACK:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: SET_STA_ATTACK message received from driver\n");
			if(message.profile_index < 0 || message.profile_index > 4)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"SET_STA_ATTACK: received error profile index %d\n", message.profile_index);
				return;
			}
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "SET_STA_ATTACK: profile index = %d, STA index = %d, attack flag = %d\n", message.profile_index,
						  message.content.StaAttack.index, message.content.StaAttack.flag );
			wdrv_wpaSetStationSecAttack(message.content.StaAttack.index,
										message.content.StaAttack.flag,
										message.profile_index);
			break;
		default:
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"WDRV_WPA: receive unknown msg type %d\n",message.type);
			break;
	} /* switch */

}

/******************************************************************************
*                      init_wpa_eapol_handler()											  *
*******************************************************************************
DESCRIPTION:	start eapols socket	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		0 ,-1.                                                    
******************************************************************************/
static int init_wpa_eapol_handler(void)
{
#ifndef VXWORKS
	struct ifreq  ifinfo;

	/* create eapols fd*/
	eapol_fd = socket(AF_INET, SOCK_PACKET, htons(0x888e));
	if (eapol_fd < 0) {
		perror(__FUNCTION__);
		return -1;
	}

	/* update promisc mode*/
	ioctl(eapol_fd,SIOCGIFFLAGS,&ifinfo);
	ifinfo.ifr_ifru.ifru_flags |= IFF_PROMISC;
	ioctl(eapol_fd,SIOCSIFFLAGS,&ifinfo); 

	event_fd_set(eapol_fd, EFD_READ, eapol_packet_recive, NULL);
#else
	END_OBJ* pCookie;

	char * name="brgEnd";
                                                                         
	pCookie = muxBind(name, 32, recvEapolPacket, NULL, NULL,NULL,MUX_PROTO_SNARF,
					  "EAPOL", NULL);

	if (pCookie == NULL)
{
		printf("Could not bind to driver %s\n",(int)name);
		return -1;
	}
#endif

	return 0;
}

/******************************************************************************
*                      init_pre_auth_eapol_handler()						  *
*******************************************************************************
DESCRIPTION:	init pre authentication socket.	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static int init_pre_auth_eapol_handler(void)
{
#ifndef VXWORKS
	struct ifreq  ifinfo;

	pre_auth_eapol_fd = socket(AF_INET, SOCK_PACKET, htons(0x88c7));
	if (pre_auth_eapol_fd < 0) {
		perror(__FUNCTION__);
		return -1;
	}

	ioctl(pre_auth_eapol_fd,SIOCGIFFLAGS
		  ,&ifinfo);
	ifinfo.ifr_ifru.ifru_flags |= IFF_PROMISC;
	ioctl(pre_auth_eapol_fd,SIOCSIFFLAGS,&ifinfo); 

	event_fd_set(pre_auth_eapol_fd, EFD_READ,pre_auth_eapol_packet_recive, NULL);
#endif
	return 0;
}


/******************************************************************************
*                      get_eth_type()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static UINT16 get_eth_type(u8 *buf)
{
	return (((UINT16)buf[12]) << 8 | buf[13]);
}

/******************************************************************************
*                      eapol_packet_recive()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
#ifndef VXWORKS
static void eapol_packet_recive(int sock, void *data)
{
	int bread = 0;
	char buffer[2000];

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Eapol frame received from OS network stack \n");
	bread = read( eapol_fd, buffer, 2000 );
	if( bread <0)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC, "WPA_DRV: Error receiving eapol frame from OS network stack \n");
		return;
	}
    eapol_packet_process(buffer, bread);
}

/******************************************************************************
*                      pre_auth_eapol_packet_recive()										  *
*******************************************************************************/
static void pre_auth_eapol_packet_recive(int sock, void *data)
{
	int bread = 0;
	char buffer[2000];
	UINT16 ether_type;
	int i;

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Preauth Eapol frame received from OS network stack \n");
	bread = read( pre_auth_eapol_fd, buffer, 2000 );
	if( bread <0)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"error reading eapol buffer\n");
		return;
	}

	ether_type = get_eth_type(buffer);
	if( ether_type != EAPOL_ETHER_PRE_AUTH_TYPE)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"%s error reading pre auth ethet type from buffer ,type = 0x%x\n", __FUNCTION__, ether_type);
		return;
	}
	/* find station port*/
	for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
	{
        if(!isPortOpen(&(sec_port_data[i])))
            continue;
        
        if( (sec_port_data[i].profile_type == SEC_PROFILE_REGULAR) && 
            (sec_port_data[i].portParams.pre_auth_on == 1) )
		{
			receivePreAuthRequest((wpa_port_t *)sec_port_data[i].pWpaPortContext,buffer);
			eapolPacketHandler(sec_port_data[i].pXPortContext,buffer,bread,0);
			return; 
		}
	}
	DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"%s cant find station wp port \n", __FUNCTION__);
}
#endif
/******************************************************************************
*                      eapol_packet_process()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			buffer - received EAPOL packet.
                bread - number of bytes in EAPOL packet.
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void eapol_packet_process(char * buffer, int bread)
{
	UINT16 ether_type;

    if (eapolPacketHandler != NULL)
	{
		int i;
		
		ether_type = get_eth_type(buffer);
		if( ether_type != EAPOL_ETHER_TYPE)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"error reading ethet type from buffer ,type = 0x%x\n",ether_type);
			return;
		}
#ifdef WPA_HIDDEN_PROFILE_SUPPORT
		/* If WPA supplicant is enabled */
		/* Check if the packet is addressed to it and not to authenticator */
		if(hidden_profile_IsEapolToSup((eapol_pkt_t *)buffer))
		{
			int hiddenProfileInd = wdrv_wpaGetHiddenProfileIndex();

			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_IFC, "WPA_DRV: Eapol to supplicant \n");
			if (hiddenProfileInd == -1) 
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"%s:Hidden profile index is -1\n",__func__);
				return;
			}
			wpas_HandleEapol((eapol_pkt_t *)buffer,bread, 
							 sec_port_data[hiddenProfileInd].pWpaPortContext);
			return;
		}
		else
#endif
		{
			/* find station port*/
			for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++)
			{
                if(!isPortOpen(&(sec_port_data[i])))
                    continue;
				if(wpa_port_is_station_exist(sec_port_data[i].pWpaPortContext,&buffer[6]))
				{
                    /* TODO: check if hidden profile enabled if eapol is to hidden instance*/
					eapolPacketHandler(sec_port_data[i].pXPortContext,buffer,bread,0);
					return; 
				}
			}
		}
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"%s cant find station wpa port \n", __FUNCTION__);
		
	}
    else {
        DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC,"%s:%d eapolPacketHandler == NULL\n", __FUNCTION__, __LINE__);
	}
}


/******************************************************************************
*                      send_ass_res()											  *
*******************************************************************************
DESCRIPTION:	send ass response	
		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static int send_ass_res(UINT16  index,int flag)
{
	wpaMessage_T	msg;
#ifndef VXWORKS
	int rc;
#endif

	msg.type = CAN_STA_ASS_RES;
	msg.content.canStaAssRes.index = index;
	msg.content.canStaAssRes.flag  = flag;

#ifdef VXWORKS
	dda_wpa_receive(&msg, sizeof(msg));
#else
	rc = ipc_connect_buf_write(wpaSocket,(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("send_eapol:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return NOK;
	}
#endif

	return OK;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int    wdrv_wpaGetListenSock(void)
{
	return wpaSocket;
}

/******************************************************************************
*                      open_by_wpa_port()									  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static ap_dev_t *open_by_wpa_port(void *dev_id,     /* object context */
					  ap_dev_cbs_t *cb, /* callbacks to use */
					  void *context)	/* context to be passed when calling callbacks */
{
	
	/* Now init the WPA package callbacks tp be called */
	memset(&ap_dev_cbs, 0, sizeof(ap_dev_cbs_t));
	memcpy(&ap_dev_cbs, cb, sizeof(ap_dev_cbs_t));

	if (ap_dev_cbs.sta_assoc_request == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT,"WDRV_WPA: sta_assoc_request callback is NULL\n");
	if (ap_dev_cbs.sta_assoc_done == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT, "WDRV_WPA: sta_assoc_done callback is NULL\n");
	if (ap_dev_cbs.sta_disassoc == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT, "WDRV_WPA: sta_disassoc callback is NULL\n");
	if (ap_dev_cbs.unicast_mic_failure == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT, "WDRV_WPA: unicast_mic_failure callback is NULL\n");
	if (ap_dev_cbs.first_data_packet_received == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT, "WDRV_WPA: first_data_packet_received callback is NULL\n");

	
	return &pApDev;
}

/******************************************************************************
*                      close_by_wpa_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void close_by_wpa_port(ap_dev_t *t)
{
}

/******************************************************************************
*                      canStationAssc()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS canStationAssc(u8 *info_elem, int info_elem_len,int profile_index)
{
	if (ap_dev_cbs.sta_assoc_request != NULL)
	{
		if(profile_index < 0 || profile_index > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1)
		{
			printf("can station assoc: receive error profile index\n");
			return NOK;
		}

		if (sec_port_data[profile_index].profile_type != SEC_PROFILE_REGULAR) 
		{
			printf("can station assoc: invalid profile received \n");
			return NOK;
		}

		if (info_elem_len) 
			return ap_dev_cbs.sta_assoc_request(sec_port_data[profile_index].pWpaPortContext, info_elem, info_elem_len);
		else
			return ap_dev_cbs.sta_assoc_request(sec_port_data[profile_index].pWpaPortContext, NULL, info_elem_len);
		
	}
		
	else
		return OK;
}

/******************************************************************************
*                      open_by_x_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static auth1x_pc_t *open_by_x_port(void *dev_id,
				 auth1x_pkt_handler_t auth1x_pkt_handler, 
				 void *context)
{
	/* Now init the EAPOL recv callback */
	eapolPacketHandler = auth1x_pkt_handler;
	if (eapolPacketHandler == NULL)
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_INIT, "WDRV_WPA: eapolPacketHandler callback is NULL\n");
	
	return pApPort;
}

/******************************************************************************
*                      close_by_x_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void close_by_x_port(auth1x_pc_t *t)
{
}
/******************************************************************************
*                      client_get()											  *
*******************************************************************************
DESCRIPTION:
 * Look up the client specified by id and id_len.
 * If not found - return EMI_NONE, otherwise return EAP method id and set
 * *eap_method_params to its client parameters.
 *	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/

static eap_method_id_t client_get(void *context,u8 *id, int id_len, int profile_index, void **eap_method_params)
{


    clientParams.identity = id;
    clientParams.identity_len = id_len;
    clientParams.radius_client = sec_port_data[radius_client_index].radius_client; 
    clientParams.profile_index = profile_index;

    *eap_method_params = &clientParams;
	
    return EMI_RADIUS;
}


/******************************************************************************
*                      wdrv_wpaReConf()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS wdrv_wpaReConf(wpa_port_params_t *pWpaDbParams)
{
	memcpy(&sec_port_data[0].portParams, pWpaDbParams, sizeof(wpa_port_params_t));

	/* We overrided this value, than set them again. */
	sec_port_data[0].portParams.dev_id = &local_dev_id;
	sec_port_data[0].portParams.auth1x_port = sec_port_data[1].pXPortContext;
	sec_port_data[0].portParams.ap_dev_ops = &deviceCBs;

	wpa_port_reconf(sec_port_data[0].pWpaPortContext, &sec_port_data[1].portParams);
	
	return OK;
}

/******************************************************************************
*                      set_ap_info_elem()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void set_ap_info_elem(wpa_port_params_t *params,
					  wpa_info_elem_t **ap_info_elem,
					  int *p_info_elem_len, 
					  wpa_info_elem_t **rsn_info_elem,
					  int *p_rsn_len )
{
    /* If WPA stations are supported, we're going to need an information element
     */
    if (params->allowed_sta_types & WPA_STA_TYPE_WPA)
    {
		/*printf("init SSN info element , unicast cipher  = %d\n",params->rekeying_wep_cipher);*/

		/* Build information element */
		*ap_info_elem = wpa_info_elem_open(select_multicast_cipher(params), 
			WPA_CIPHER_TKIP, params->psk_param == WPA_PSK_PARAM_NONE ?
			WPA_AUTH_8021X : WPA_AUTH_PSK);

		if (!*ap_info_elem)
			return;

		*p_info_elem_len = wpa_info_elem_get_len(*ap_info_elem);
	}

	if (params->allowed_sta_types & WPA_STA_TYPE_WPA2)
    {
		/*printf("init RSN info element , unicast cipher  = %d\n",params->rekeying_wep_cipher);*/

		/* Build information element */
		*rsn_info_elem = wpa_rsn_elem_open(select_multicast_cipher(params), 
			WPA_CIPHER_CCMP/*params->rekeying_wep_cipher*/, params->psk_param == WPA_PSK_PARAM_NONE ?
			WPA_AUTH_8021X : WPA_AUTH_PSK,params->pre_auth_on);

		if (!*rsn_info_elem)
			return;

		*p_rsn_len = wpa_info_elem_get_len(*rsn_info_elem);
	}
}

/******************************************************************************
*                      wdrv_wpaSetMicFailureReportInRx()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_wpaSetMicFailureReportInRx(int value)
{
	sec_port_data[0].micFailureReportInRx = value;
}

/******************************************************************************
*                      wdrv_wpaGetMicFailureReportInRx()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int wdrv_wpaGetMicFailureReportInRx(void)
{
	return sec_port_data[0].micFailureReportInRx;
}

/******************************************************************************
*                      wdrv_wpaSetStationSecAttack()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_wpaSetStationSecAttack(UINT16 index, int value,int profile_index)	
{
	sec_port_data[profile_index].secAttackTable[index] = value;
}

/******************************************************************************
*                      wdrv_wpaGetStationSecAttack()								  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int wdrv_wpaGetStationSecAttack(UINT16 index)	
{
	return sec_port_data[0].secAttackTable[index];
}


/******************************************************************************
*                      wdrv_wpaGetHiddenSecPortData()								  *
*******************************************************************************
DESCRIPTION:	Gets hidden security profile structure	   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		a pointer to security profile structure                                                   
******************************************************************************/
security_port_data_t *wdrv_wpaGetHiddenSecPortData(void)
{
	int hiddenProfileInd = wdrv_wpaGetHiddenProfileIndex();
	
	if (hiddenProfileInd == -1) 
	{
		printf("%s: hidden security port data not found. Returning NULL!!!\n",__FUNCTION__);
		return NULL;
	}
	else
	{
		return &sec_port_data[hiddenProfileInd];
	}
}

/******************************************************************************
*                      wdrv_wpaGetHiddenSecPortData()								  *
*******************************************************************************
DESCRIPTION:	Gets ap device callbacks structure	   
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		a pointer to the structure                                                   
******************************************************************************/
ap_dev_cbs_t *wdrv_wpaGetApDevCbs(void)
{
	return &ap_dev_cbs;
}


/******************************************************************************
*                      authStatusReport()								  *
*******************************************************************************
DESCRIPTION:	Reports authentication status (used only for hidden profile BY NOW)	   
                                                                         
INPUTS:			mac, status

OUTPUT:			None

RETURNS:	    None                                                  
******************************************************************************/
void authStatusReport(ether_mac_t mac, u32 status)
{
	if (hidden_profile_IsSecured()) 
	{
		if (isAssocWithHiddenInstance(mac))
		{
			hidden_profile_ReportAuthStatus(mac, status);
		}
	}
}


/******************************************************************************
*                      wdrv_wpaGetHiddenProfileIndex()						  *
*******************************************************************************
DESCRIPTION:	Determines  the hidden profile index  
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		index or -1 in case if failed                                                   
******************************************************************************/
int wdrv_wpaGetHiddenProfileIndex(void)
{
	int i, hidden_index = -1;

	for (i=0; i<TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++) 
	{
		if (sec_port_data[i].profile_type == SEC_PROFILE_HIDDEN) 
		{
			hidden_index = i;
			break;
		}
	}

	return hidden_index;
}

/******************************************************************************
*                      isPortOpen()											  *
*******************************************************************************
DESCRIPTION:	Determines  if secure port is open for a given profile 
                                                                         
INPUTS:			profile data

OUTPUT:			None

RETURNS:		TRUE/FALSE                                                 
******************************************************************************/
static int isPortOpen(security_port_data_t *pPortData)
{
    if (pPortData->profile_type == SEC_PROFILE_NON_ACTIVE) 
        return (FALSE);

    if((pPortData->portParams.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) &&
       (pPortData->portParams.rekeying_wep_cipher == WPA_CIPHER_NON))
        return (FALSE);
    return (TRUE);
}

/******************************************************************************
*                      processDbgRequest()							          *
*******************************************************************************
DESCRIPTION:	process Debuge Request 
                                                                         
INPUTS:			int sock    - Input sock
                void *data  - Input message

OUTPUT:			None

RETURNS:		TRUE/FALSE                                                 
******************************************************************************/
static void processDbgRequest(int sock, void *data)
{
	int ret;
	dbgMsg_T msg;

	ret = read(sock, (char *)&msg, sizeof(msg) );

	if (ret < 0) {
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_IFC, "WPA dbg: error reading a message from socket\n");
		return;
	}

	switch (msg.type) 
	{
	case SET_SEVERITY:
		dbgSeverity = msg.content.severity;
		break;
	case SET_REP_MODULE:
		dbgModule = msg.content.module;
		break;
	case PRINT_PROF_STAT:
		if (msg.content.profile_ind > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_IFC, "WPA DBG: PRINT_PROF_STAT request - Invalid profile index\n");
			return;
		}
		wpa_port_print_statistics(sec_port_data[msg.content.profile_ind].pWpaPortContext);
		break;
	case PRINT_PROF_TBLS:
		if (msg.content.profile_ind > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_IFC, "WPA DBG: PRINT_PROF_TBLS request - Invalid profile index\n");
			return;
		}
		printProfileTbls(msg.content.profile_ind);
		break;
	case PRINT_PROF_CFG:
		if (msg.content.profile_ind > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_IFC, "WPA DBG: PRINT_PROF_CFG request - Invalid profile index\n");
			return;
		}
		printProfileCfg(msg.content.profile_ind);
		break;
    case PRINT_PMK_CACHE:
		wpa_pmk_cache_dbg_print();
		break;
    case GET_PSK: 
        getPSK(&msg);
	default:
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_IFC, "WPA: Invalid debug request type\n");	
	}
   
}

/******************************************************************************
*                      printProfileTbls()							          *
*******************************************************************************
DESCRIPTION:	print the SA data for STAs associated with the given profile
                                                                         
INPUTS:			int profile_index  - profile index

OUTPUT:			None

RETURNS:		TRUE/FALSE                                                 
******************************************************************************/
static void printProfileTbls(int profile_index)
{
	security_port_data_t *profile_data = &sec_port_data[profile_index];
	wpa_assoc_t *assoc;
	int i;
	
	printf("\n");
	printf("STA table for profile index %d\n", profile_index);
	printf("Allowed station types:");

	/* If profile is non-active or privacy is disabled - nothing to print*/
	if (!isPortOpen(profile_data)) 
	{
		printf("Legacy\n");
		printf("Privacy disabled for a given profile - no internal info in WPA\n");
		return;
	}
	else
	{
		if (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) 
			printf(" WEP legacy ");
		if (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WEP_8021X) 
			printf(" WEP 802.1X ");
		if (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WPA) 
			printf(" WPA ");
		if (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WPA2) 
			printf(" WPA2 ");
		
		printf("\n");
		
		if ( (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) ||
			 (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
		{
			printf("WEP default keys:\n");
			for (i=0; i < 4; i++) 
			{
				printf("Index %d. Key=%s\n", i, 
					   wpa_log_format_buf((u8 *)profile_data->portParams.wep_keys[i].data,
					   wpa_key_utils_get_key_len(profile_data->portParams.wep_keys[i].cipher)));
			}
			printf("Default key ID = %d\n", profile_data->portParams.wep_key_index);
	
		}
		printf("Group cipher suite    = 0x%02X\n", profile_data->pWpaPortContext->multicast_cipher);
		printf("Current gtk index     = %d\n", profile_data->pWpaPortContext->current_gtk_index);
		printf("Profile GTK           = %s\n", wpa_log_format_buf((u8 *)&profile_data->pWpaPortContext->gtk, wpa_key_utils_get_key_len(profile_data->pWpaPortContext->multicast_cipher)));
		printf("-------------------------------------------------------------\n\n");
		i = 0;
		for (assoc = profile_data->pWpaPortContext->assoc_list; assoc; assoc = assoc->next)
		{
			printf("STATION %d:\n", i);
			printf("STA MAC %02X-%02X-%02X-%02X-%02X-%02X\n", assoc->sta_mac[0], assoc->sta_mac[1],assoc->sta_mac[2],
														assoc->sta_mac[3], assoc->sta_mac[4], assoc->sta_mac[5]);
			if ( (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WPA) ||
			 (profile_data->portParams.allowed_sta_types & WPA_STA_TYPE_WPA2)) 
			{
				printPMKSA(assoc);
				printPTKSA(assoc);
			}
			printGTKSA(assoc);
			i++;
			printf("\n\n");
		}
	}

}

/******************************************************************************
*                               getPSK()							          *
*******************************************************************************
DESCRIPTION:	Get the PSK of the requested profile in Hex format.
                                                                         
INPUTS:			msg  - The massage that contains the request parameters.

OUTPUT:			None

RETURNS:		None                                                 
******************************************************************************/
static void getPSK(dbgMsg_T *msg)
{
    int sock;
    msgReport_t report;

    wpaGetPSK(msg->content.profile_ind, report.psk);
    report.msgType = msg->reportMsgType;
    /* Create socket on which to send. */ 
	sock = socket(msg->domain, msg->SocketType, msg->protocol);
	if (sock < 0) {
        printf("%s : Opening datagram socket: %s\n", __FUNCTION__, strerror(errno));
	}
    /* Send message. */
    if (sendto(sock, (void *)&report, sizeof(msgReport_t), 0,
                (struct sockaddr *)&msg->addr, msg->addrLen) < 0) {
		printf("%s : Sending IPC message: %s\n", __FUNCTION__, strerror(errno));
        close(sock);
    }
    close(sock);
}

/******************************************************************************
*                               wpaGetPSK  									  *
*******************************************************************************
DESCRIPTION:	Get the PSK key in Hex Format  
                                                                         
INPUTS:			profile_index - The index of the profile

OUTPUT:			psk - the PSK key

RETURNS:		None.                                                    
******************************************************************************/
static void   wpaGetPSK(int profile_index, char *psk)
{
    memcpy(psk, sec_port_data[profile_index].pWpaPortContext->psk.data, 32);
}

/******************************************************************************
*                      isCipherAndPreAuthOK()						          *
*******************************************************************************
DESCRIPTION:        This function is called for each profile. It checks the 
                    cipher and pre-auth flag of this profile. If the cihper is
                    WEP and there is already a previous profile with static WEP, 
                    or if the Pre-Auth flag is On and there is already a previous
                    profile with Pre-Auth flag On --> Aborts the Init process.
                    If not, sets the 'isWEPexists' & 'isPreAuthExists' flags 
                    if necessary and returns OK.


INPUTS:			    default_cipher  - Current Porfile's Cipher
                    pre_auth_flag   - Current Porfile's Pre-Authentication flag
                    sec_type        - Current Porfile's Security Type


INPUTS & OUTPUT     isWEPexists     - Indicates whether or not a profile with 
                                      WEP was found. If the current profile is
                                      configured to WEP --> set to TRUE.

                    isPreAuthExists - Indicates whether or not a profile with 
                                      Pre-Auth was found.If the current profile
                                      is configured to Pre-Auth --> set to TRUE.


RETURNS:		    OK on success, NOK otherwise.                                                 
******************************************************************************/
static STATUS isCipherAndPreAuthOK(__u32 default_cipher, __u8 pre_auth_flag, 
                                   tiap_param_security_type_t sec_type, 
                                   BOOL *isWEPexists, BOOL *isPreAuthExists)
{
    /*  Only one profile can be configured to work with type STATIC WEP and chipher WEP.
        If current profile's cipher is WEP --> 
        check if there is already another profile configured to WEP. 
        If there is --> abort configuration!!                        */
    if( (sec_type != TIAP_PARAM_SECURITY_802_1X) && 
        ((default_cipher &= TIAP_WPA_CIPHER_WEP_40) ||
         (default_cipher &= TIAP_WPA_CIPHER_WEP_232) ||
         (default_cipher &= TIAP_WPA_CIPHER_WEP_104)) )
    {
        if(*isWEPexists)
        {
            printf("ERROR while configuring WPA Authenticator! Abort Configuration!\nCan't configure 2 profiles with WEP.\n***** Please return to defualt values and configure the profiles again *****\n");
            return NOK;
        }
        else
        {
            *isWEPexists = TRUE;
        }
    }
    
    /* Only one profile can be configured to work with Pre-Auth.
    If in current profile Pre-Auth is ON --> 
    check if there is already another profile with Pre-Auth ON.
    If there is --> abort configuration!!                       */
    if(pre_auth_flag == 1)
    {
        if( !(sec_type & TIAP_PARAM_SECURITY_WPA2) )
        {
            printf("ERROR while configuring WPA Authenticator! Abort Configuration!\nCan't configure pre-Authentication to a non-WPA2 profile.\n***** Please return to defualt values and configure the profiles again *****\n");
            return NOK;
        }
        else
        {
            if(*isPreAuthExists)
            {
                printf("ERROR while configuring WPA Authenticator! Abort Configuration!\nCan't configure 2 profiles with Pre-authentication ON.\n***** Please return to defualt values and configure the profiles again *****\n");
                return NOK;
            }
            else
            {
                *isPreAuthExists = TRUE;
            }
        }
    }

    return OK;
}
