/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/***************************************************************************
**                                                                        **
**   MODULE:    wdsProtocol.h                                             **
**   PURPOSE:   WDS protocol elemets definitions                          **
**                                                                        **
***************************************************************************/

#ifndef __WDS_PROTOCOL_H__
#define __WDS_PROTOCOL_H__

#define WDS_PROTOCOL_VERSION            0
#define WDS_PROTOCOL_MSG_BODY_LENGTH    11 
#define WDS_IE_PREFIX_LENGTH            3 /* two fields id - 1 byte, length - 2 bytes */

#define WDS_PROT_GET_VERSION_m(msg, ver) (ver) = ((msg.ver_type) & 0xE0) >> 5
#define WDS_PROT_SET_VERSION_m(msg, ver) (msg.ver_type) |= ((ver) << 5)
#define WDS_PROT_GET_MSG_TYPE_m(msg, type) (type) = ((msg.ver_type) & 0x1F)
#define WDS_PROT_SET_MSG_TYPE_m(msg, type) (msg.ver_type) |= ((type) & 0x1F)

#define WDS_PROT_GET_HOP_NUM_m(msg, hop) (hop) = ((msg.body.advertisement.info) & 0xF0) >> 4
#define WDS_PROT_SET_HOP_NUM_m(msg, hop) (msg.body.advertisement.info) |= ((hop) << 4)
#define WDS_PROT_GET_DL_NUM_m(msg, dlNum) (dlNum) = ((msg.body.advertisement.info) & 0x0E) >> 1
#define WDS_PROT_SET_DL_NUM_m(msg, dlNum) (msg.body.advertisement.info) |= ((dlNum) & 0x07) << 1

                 
typedef enum{
    WDS_MSG_RESERVED = 0,
    WDS_MSG_ADVERTISEMENT,
    WDS_MSG_JOIN_REQUEST,
    WDS_MSG_JOIN_RESPONSE,
    WDS_MSG_KEEP_ALIVE_REQ,
    WDS_MSG_KEEP_ALIVE_RESP,
    WDS_MSG_LINK_ABORT
} WDS_PRO_MSG_TYPES_e;

typedef enum {
    CONNECTION_LOST = 0,
} WDS_PRO_LINK_ABORT_REASON_e;

typedef enum {
    WDS_JOIN_SUCCESS    = 0,
    WDS_JOIN_FAIL_GENERIC,
    WDS_JOIN_FAIL_NO_RESOURCES,
    WDS_JOIN_FAIL_PEER_NOT_ALLOWED,
    WDS_JOIN_FAIL_PARM_MISMATCH,
    WDS_JOIN_FAIL_RSSI_LOW
} WDS_PRO_JOIN_REASON_e;

#define WDS_IE_ID_OWN_IP    (UINT8) 0
#define WDS_IE_ID_UL_STATUS (UINT8) 1
#define WDS_IE_ID_DL_STATUS (UINT8) 2

#pragma pack(1)

typedef struct {
    UINT8   WdsName[WDS_NAME_MAX_LEN];
    UINT8   info; /* hop number : 4, dlNum : 3*/
    UINT8   reserved;
} wdsAdvertiseMsg_s;

typedef struct {
    UINT8   reason;
} wdsJoinRespMsg_s;

typedef struct {
    UINT32  id;
} wdsKeepAliveMsg_s;

typedef struct {
    UINT8 reason;
} wdsAbortMsg_s;

typedef struct {
    UINT8  ver_type; /* 3 bits version, 5 bits message type*/
    union {
        UINT8  msg_body[WDS_PROTOCOL_MSG_BODY_LENGTH];
        wdsAdvertiseMsg_s advertisement;
        wdsJoinRespMsg_s  joinResp;
        wdsKeepAliveMsg_s keepAlive;
        wdsAbortMsg_s     linkAbort;
    } body; 
} wdsProtocolMsg_s;

struct wdsIeWbniPeerStatus_s{
    UINT8  peerMac[MAC_ADDR_LEN];
    short  rssi;
};

typedef struct {
    UINT8   id;     /* IE(Information element) identifier */
    UINT16  length; /* IE body length */
    union {
        UINT32 ipAddr;
        struct wdsIeWbniPeerStatus_s    peerStatus;
    } body;
} wdsInformationElement_s;

#pragma pack()


#endif /* ifndef __WDS_PROTOCOL_H__ */

