/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:		drvDbMain.c											   */
/*		PURPOSE:	AP Main database. This file holds the main database	   */
/*					of the AP, such as system global variable and MIB tables.      */
/*																	 	   */
/***************************************************************************/

#include "wlan_pal.h"
#include "whal.h"
#include "802_11Defs.h"
#include <QoS/QoSClassifier.h>
#include <QoS/QoSACParams.h>
#include "drvDbHash.h"
#include "whalDefs.h"
#include "whalParams.h"
#include "4xMngr.h"
#include "utils.h"
#include "drvDbMain.h"
#include "drvDbRegDomain.h"
#include "bssbridge.h"
#include "tx.h"
#include "rx.h"
#include "dda_wpa.h"
#include "mgmtAccess.h"
#include "mgmtSsidHandler.h" 
#include "wdsDrv.h"


typedef struct
{
	UINT32	suite;
	BOOL	enable;
} suite_t;


/*-----------------------------------------------------------*/
/*         EDCF Definitions                                  */
/*-----------------------------------------------------------*/

#define AIFS_PIFS  1  /* slots number to start tx at PIFS interval. */
#define AIFS_DIFS  2  /* slots number to start tx at DIFS interval (normal DCF access). */
#define MAX_AIFS   0xFF
#define MAX_EDCF_SLOTS  10   /* FW allows AIFSN up to this number of additional slots beyond DIFS. */
#define NO_RX_TIMEOUT   0    /* rxTimeout default value */
#define SINGLE_MPDU_TXOP_LIMIT  0    /* Only send a single MPDU when a TXOP is obtained. */
#define MSDU_LIFETIME_DEFAULT   512  /* In TUs (1024 usec Time Units). */

/* Demiurg: WME */
static UINT32 wme_ac_ucast_threshold[WDRV_QOS_MAX_AC] = {17, 7, 21, 21};

static UINT32 no_wme_ac_ucast_threshold[WDRV_QOS_MAX_AC] = {17, 0, 0, 0};
#define WDRV_DB_MAIN_WME_BCAST_THRESHOLD 7
#define WDRV_DB_MAIN_BCAST_THRESHOLD 7

static UINT32 wme_ac_ucast_high_threshold[WDRV_QOS_MAX_AC] = {34, 7, 28, 28};
static UINT32 no_wme_ac_ucast_high_threshold[WDRV_QOS_MAX_AC] = {65, 65, 65, 65};
#define WDRV_DB_MAIN_WME_BCAST_HIGH_THRESHOLD 7
#define WDRV_DB_MAIN_BCAST_HIGH_THRESHOLD 21


/*-----------------------------------------------------------*/
/*         wep  params                                       */
/*-----------------------------------------------------------*/


static defaultWepKey_T defaultWepKeys[WPA_NUM_DEFAULT_KEYS];
static UINT8 wepDefaultKeyId;


/*-----------------------------------------------------------*/
/*         Wone  Info                                        */
/*-----------------------------------------------------------*/


typedef struct 
{
    UINT8 active;
    UINT8 type;
    UINT8 Priority;
    UINT8 globalQid;
    UINT32 threshold; /* Limit buffer pool usage by a specific tx queue in the ACX in 5 percent
					    units from ACX buffer pool. 0 = no limit, so no need to send IE. */
	UINT32 high_threshold;
	BOOL edcfInfoRequired;  /* TRUE = need to configure EDCF parameters to FW for this tx queue. */
	edcf_info_t  edcfInfo;  /* defined in whal.h */
} queue_info_t;

typedef struct 
{
	queue_info_t ac_queue[WDRV_MAX_TX_QUEUE];
    UINT32                  mgmtAccessCategory;
   UINT8                   numOfQueue;
} 
queues_table_t;

typedef struct  
{    
    dot11_supRates_T       apRates;
    dot11_supRates_T       apRatesTemplate;
    dot11_extSupRates_T    apOFDMRates;
    UINT16                 apCapabilities;
    UINT16                 defaultActiveRateAcx;
    UINT16                 defaultActiveRateAcxWithFallback;
	UINT16				   defaultActiveRateAcxBrcst;
	UINT16				   defaultActiveRateAcxBrcstFallback;
	UINT16				   defaultActiveRateAcxMcst;
	UINT16				   defaultActiveRateAcxMcstFallback;
    UINT16                 allApSupRatesLen;
    queues_table_t         queue_table;
    UINT8                  channel;
    UINT8                  band;
    UINT8                  primaryAcxQindex;
    UINT8                  psBroadcastQindex;
    UINT8                  UsageTime;      
    UINT8                  PreambleMode;
    UINT8                  PbccAlgorithmMode;
    UINT8                  RateAdaptation;
    UINT8                  CwMinMode;
    UINT8                  SlotTimeMode;
    UINT8                  ERPProtectionMode;
    UINT8                  ERPProtectionType;
    UINT8                  ERPProtectionTI;
    UINT8                  defaultActiveRate;
	UINT8				   genTxRate;
	UINT8                  maxActiveRate;
    UINT8                  maxCCKactiveRate;
    UINT8                  maxCCKbasicRate;
    UINT8                  maxOFDMbasicRate;
    UINT8                  minCCKbasicRate;
    UINT8                  minOFDMbasicRate;
    UINT8                  allApSupRates[DOT11_SUPPORTED_RATES_MAX +
                                         DOT11_EXT_SUPPORTED_RATES_MAX];
    UINT8                  Dot11Mode;
    UINT8                  PowerLevel;
} wone_info_T;




static UINT8 numSupWone;    /* Number of supported wone object. Default is 1  */
static UINT8 usageTimeMode;      /* 0 = short usage time, 1 = long usage time */




static wone_info_T wone_info[MAX_WONE_NET];  /* Wone Info table */



/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/

/* Inplements the privacy part in the station config table */
static struct 
{
	UINT16	disassociateReason;
	char	disassociateStation[MAC_ADDR_LEN];
	UINT16	deauthenticateReason;
	char	deauthenticateStation[MAC_ADDR_LEN];
	UINT16	deauthenticateFailStatus;
	char	deauthenticateFailStation[MAC_ADDR_LEN];
} dot11StationConfigTable; /* TODO */


/* Dynamic Algorithms Timestamps*/
static struct
{
	UINT32	pbccTimeStamp;
	UINT32  erpTimeStamp;
	UINT32  RxTxBrateTimestamp;
	UINT32  RxTxBrateCount;
	BOOL	pbccValid;
	BOOL	erpValid;
} DynamicAlgInfo; 

/* AP's MAC address*/
static char APMAC[MAC_ADDR_LEN] = {0xaa,0xaa,0xbb,0xcc,0xdd,0xee};
/* AP's BSSID */
static char BSSID[MAC_ADDR_LEN];

/* AP's beacon period. */
static UINT16 apBeaconPeriod;	
/* AP's DTIM period. */
static UINT8 apDtimPeriod;	
/* AP's RTS threshold. */
static UINT16 apRtsThreshold;	
/* AP's short retry limit. */
static UINT8 apShortRetryLimit;	
/* AP's long retry limit. */
static UINT8 apLongRetryLimit;	
/* AP's Frag threshold. */
static UINT16 apFragThreshold;	
/* AP's max transmit lifetime. */
static UINT32 apMaxTxLifetime;	
/* AP's max receive lifetime. */
static UINT32 apMaxRxLifetime;	
/* AP's BSS isolation state */
static BOOL bssBridgeIsolationOn;
/* AP's 4x mode*/
static dot11_TIcap_T	TIcap;
static BOOL	enable4x;
static BOOL	enableConcat;
static BOOL	enableAckEmul;
static BOOL enable4xMixMode;

static UINT8 FeatureSet;
/* AP's Contention window minimum */
static UINT16	apCWMinWindow;
/* AP uses this to decide how to set the supported rates IE in (re)associate response */
static UINT8 apRespRateMode;



static char TI_OUI[DOT11_OUI_LEN] = {0x08, 0x00, 0x28};

static BOOL  authOpenSystemEnable;
static UINT8 authOpenSystemIndex;

static BOOL  authSharedKeyEnable;
static UINT8 authSharedKeyIndex;

static BOOL  Agile; 


static UINT8 RateFallback;


static UINT16 SlotTimeThreshold;



static UINT32 DynamicAlgorithmsTimeout;

static UINT32 RadioCalibrateOption;
static UINT16 RadioCalibrateInterval;


/*		WPA Section		*/

/* broadcast suite*/
UINT8	broadcastCipherSuiteType;		

static BOOL WlanServiceState ;
static UINT8 SavedEnRestart;
static BOOL RestoreSavedEnRestart;
/* Driver Workaround variables */
static UINT8 EnRestart;
static UINT8 EnInterfWA;            /* when true the ACI/NB workaround in the driver is enabled */
static UINT8 RxFreeMemOptionWA;
static UINT8 BeaconNotSentOptionWA;
static UINT8 TxStuckOptionWA;
static UINT16 ConsFcsErrOptionWA;
static UINT8  aciOptionWA;
static UINT8  nbOptionWA;

/* Interference WA State Machine configuration */
static UINT32 aciStateTimeout;
static UINT32 nbStateTimeout;
static UINT32 aciRateThreshold;
static UINT32 aciEventThreshold;
static UINT32 mediumBusyThresh;

/* FW Workaround variables */
static UINT8 EnEnergyDetect;
static UINT16 pdUnderAciLowThrsh;
static UINT16 pdUnderAciHighThrsh;

/* EEPROM Less support variable */
static UINT8 EEPromLessSupport;

/* HW ACI/NB interfernce handling configuration */
static HwInterfParams_t hwInterfParams;

/* radio support variable */
static UINT8 RadioFrequency = RADIO_BAND_2_4_AND_5GHZ;

/* radio parameters */
static RadioParam_t dbRadioParams;

/* packet bursting Information */
static UINT16 txopLimit;    

extern UINT16 configTxopLimit;

static UINT8 RadarDetectMode;
static UINT8 ApBridgeMode;
static UINT8 wdsEnabled;

/* Demiurg: WME */
static wdrv_QoSACParams_t wmeACParams;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static void	   initMibTables(void);	
static BOOL    isBasic(UINT8 rate);
static int     initDefaultActiveRate(UINT8 woneIndex, initConfigStruct *pInitConfigStruct);
INIT_RET_CODE  checkInitStructValid(initConfigStruct *pInitConfigStruct);	

/*************************************************************************
 *                        edcfInfoInit                                   *
 *************************************************************************
DESCRIPTION: Initialize Tx queue's EDCF parameters.
			 Currently assumes that it is called only for low and high priority
			 tx queues and only if vHCF or concatination is enabled!!!
INPUT:   queueType, wmeACParams,  wone_index   

OUTPUT:   edcfInfoPtr   

RETURN:  
**************************************************************************/
void edcfInfoInit(int queueType ,edcf_info_t *edcfInfoPtr,  wdrv_QoSACParams_t *  wmeACParams, int wone_index) 
    {   
    if (wmeACParams->enabled && edcfInfoPtr->acID != AC_PS_BROADCAST)
    {
		 edcfInfoPtr->trafficType = QOS_DATA_EDCF;
		 edcfInfoPtr->txopLimit = (wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY || wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY_PLUS) ? 
			 edcfInfoPtr->txopLimit11b : edcfInfoPtr->txopLimit11g;
		 edcfInfoPtr->rxTimeout = NO_RX_TIMEOUT;
		 edcfInfoPtr->retryPremption = FALSE;
		 edcfInfoPtr->rxTimeout = NO_RX_TIMEOUT;
    } else
    {
        switch (edcfInfoPtr->acID)
        {
    case AC_BEST_EFFORT  :
    case AC_PS_BROADCAST :
        {
            int enablePb = ((enableConcat) && (numSupWone == WONE_SINGLE_BAND));

            edcfInfoPtr->trafficType        = (enablePb) ? QOS_DATA_EDCF : DATA_DCF;
            edcfInfoPtr->aifsn              = AIFS_DIFS ;
            edcfInfoPtr->cwMin              = 0x0F;
            edcfInfoPtr->cwMax              = 0x03FF; 
            edcfInfoPtr->txopContinuation   = (enablePb) ? TRUE : FALSE;
            edcfInfoPtr->txopLimit          = (enablePb) ? txopLimit : 0;
            
            edcfInfoPtr->dot11EDCATableMSDULifetime = MSDU_LIFETIME_DEFAULT;
            edcfInfoPtr->rxTimeout                  = NO_RX_TIMEOUT;
            edcfInfoPtr->ackPolicy                  = ACK_IMMEDIATE_POLICY;
			wdrv_dbGetOperationShortRetryLimit(&edcfInfoPtr->shortRetryLimit);
			wdrv_dbGetOperationLongRetryLimit(&edcfInfoPtr->longRetryLimit);
            edcfInfoPtr->retryPremption     = FALSE;
            break;
        }
    case AC_BKGRND       :
    case AC_VIDEO        :
    case AC_VOICE        :
    default              :
            memset(edcfInfoPtr,0,sizeof(edcf_info_t));
    }
	 }
}

/*************************************************************************
 *                         LookforBasicRate                              *
 *************************************************************************
DESCRIPTION: Checks basic rate 
                                                                                                   
INPUT:  woneIndex,  pInitConfigStruct    

OUTPUT:      

RETURN:  resukt (TRUE/FALSE)    
                                                                                                                                                          
************************************************************************/
static BOOL LookforBasicRate(int woneIndex,initConfigStruct *pInitConfigStruct)
{
	int i = 0,break_flag;
    while (1)
	{
		break_flag = 1;	
        if (pInitConfigStruct->wone_info[woneIndex].apRates.hdr.eleLen > i)
		{
			break_flag = 0;
            if (pInitConfigStruct->wone_info[woneIndex].apRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
        if (pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen > i)
		{
			break_flag = 0;
            if (pInitConfigStruct->wone_info[woneIndex].apOFDMRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
        if (pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen > i)
		{
			break_flag = 0;
            if (pInitConfigStruct->wone_info[woneIndex].apAddRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
		i++;
		
        if (break_flag)
			break;
	}
	return FALSE; 
}




/*************************************************************************
 *          checkInitStructValid                                         *
 *************************************************************************
DESCRIPTION: Checks whether init structure is valid
                                                                                                   
INPUT:  pInitConfigStruct     

OUTPUT:      

RETURN: result     
                                                                                                                                                          
************************************************************************/
INIT_RET_CODE  checkInitStructValid(initConfigStruct *pInitConfigStruct)
{
	int woneIndex;
    for (woneIndex = 0;woneIndex < pInitConfigStruct->numSupWone;woneIndex++)
	{
        switch (pInitConfigStruct->wone_info[woneIndex].Dot11Mode)
		{
		case DOT11_MODE_MIXED:

			/* check band*/
                if (pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
                if (
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

						/* check at list one basic rate*/


			break;
		
		case DOT11_MODE_B_ONLY:

			/* additional rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* chack band*/
                if (pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
                if (
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;
		
		case DOT11_MODE_B_ONLY_PLUS:			

			/* additional rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check band*/
                if (pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
                if (
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;
		
		case DOT11_MODE_OFDM_ONLY:

			/* check band*/
                if (pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
                if (
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;

		case DOT11_MODE_A_ONLY:
			
			/* base rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* additional rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen == 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
                if (pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* chack band*/
                if (pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_5GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
                if (
				(pInitConfigStruct->wone_info[woneIndex].channel < 36 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 161 )
			  )
			{
				printf("\n\nwrong channel: %d\n\n\n", pInitConfigStruct->wone_info[woneIndex].channel);
				return DDC_INIT_CFG_LOGICAL_ERROR;
			}
			
			break;
		
		default:
			return DDC_INIT_CFG_RANGE_ERROR;
		}

		/* check for basic rates */
        if (LookforBasicRate(woneIndex,pInitConfigStruct) == FALSE)
		{
			printf("LookforBasicRate:no basic rates found for wone index %d.\n",woneIndex);
			return DDC_INIT_CFG_LOGICAL_ERROR;
		}
	}

	
    if (pInitConfigStruct->wmeACParams.enabled && pInitConfigStruct->numSupWone > 1)
    {
		printf("%s : Currently WME do not coexist with wOne\n", __FUNCTION__);
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	if (pInitConfigStruct->ApBridgeMode > 1)
	{
		printf("%s: ap bridge mode %d.\n", __func__, pInitConfigStruct->ApBridgeMode);
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

    if (pInitConfigStruct->wdsEnable > 1)
    {
        printf("%s: wdsEnable must be 0 or 1. Parameter %d.\n", __func__, pInitConfigStruct->wdsEnable);
        return DDC_INIT_CFG_LOGICAL_ERROR;
    }

	/* check WME with vlan classifier vs. multiple ssid configuration */
	if(!pInitConfigStruct->apMultipleSsidFlag && pInitConfigStruct->wmeACParams.enabled && pInitConfigStruct->wmeACParams.classifierMode == WDRV_QOS_CLASSIFIER_MODE_VLAN)
	{
		printf("%s: WME with VLAN classifier requires multiple ssid\n",__FUNCTION__);
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	return DDC_INIT_SUCCESS;
}

/*************************************************************************
 *                        wdrv_dbMainInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the AP's main database:
			1. Reset the MIB tables.                 
			2. Load the database variables with the values from the nvram.
			3. Fixes the AP's defaulte active rate.
			4. Builds information elements based on the database. Thius elements
				are sent in management messges.
                                                                                                   
INPUT:    pInitConfigStruct   

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
INIT_RET_CODE wdrv_dbMainInit(initConfigStruct *pInitConfigStruct)
{
	INIT_RET_CODE ret;
	int i, j;
	UINT16 MaxConcatSize;
    int wone_index;
    UINT16   value16;
    BOOL     value;
    UINT8 global_Qid =0;
	UINT8 apDomain;

	/* check params validity*/
    if (((ret = checkInitStructValid(pInitConfigStruct)) != DDC_INIT_SUCCESS ))
	{
		printf("checkInitStructValid failed\n");
		return ret;
	}

		if (whal_FwCtrl_Eepromless_SetImage(&pInitConfigStruct->eeprom) != OK) {
			printf("No eeprom image received\n");
			return DDC_INIT_CFG_LOGICAL_ERROR;
		}

    PAL_memSet(wone_info,0,sizeof(wone_info_T));

	/* init mib parameters*/
	initMibTables();
   
	/* init mac, bssid (same parameter in init stage)*/
    PAL_memCopy(APMAC,pInitConfigStruct->APMAC,MAC_ADDR_LEN);
    PAL_memCopy(BSSID,pInitConfigStruct->APMAC,MAC_ADDR_LEN);
   


	enable4x      = pInitConfigStruct->enable4x;
	enableConcat  = pInitConfigStruct->enableConcat;
	enableAckEmul = FALSE;
	#ifdef TIWLAN_DEBUG
		txopLimit     = configTxopLimit; /* MicroSec */
	#else
		txopLimit     = 3000; /* MicroSec */
	#endif
	
    /* init 802.11 d & h DB */
    if ((ret = wdrv_dbRegDomainInit(&pInitConfigStruct->RegInit,
								   pInitConfigStruct->wone_info[0].band	
                                   )) != DDC_INIT_SUCCESS)
	{
		printf("wdrv_dbRegDomainInit failed\n");
		return ret;
	}

	/* init mgmt access list params*/
    if ((ret=mgmtAccessInit(&(pInitConfigStruct->mgmtAccessList))) != OK)
	{
		printf("\n\r error init access list");
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	/* init ssid support*/
    if (wdrv_mgmt_ssidInit(pInitConfigStruct) != OK)
	{
		printf("\n\r error init ssid support");
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	bssBridgeIsolationOn = !(pInitConfigStruct->bssBridgeState == BSS_TRAFFIC_EN);

	usageTimeMode = pInitConfigStruct->usageTimeMode;
	numSupWone    = pInitConfigStruct->numSupWone;

	Agile = FALSE;

	/* Demiurg: WME */

	wmeACParams = pInitConfigStruct->wmeACParams;


    if (wmeACParams.enabled)
    {
		printf("%s : WME enabled\n", __FUNCTION__);
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_BE] = TOTAL_NUM_OF_DATA_ENVELOPS / 3;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_BK] = TOTAL_NUM_OF_DATA_ENVELOPS / 3;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_VI] = TOTAL_NUM_OF_DATA_ENVELOPS / 3;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_VO] = TOTAL_NUM_OF_MNGT_ENVELOPS;
    } else
    {
		printf("%s : WME disabled\n", __FUNCTION__);
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_BE] = TOTAL_NUM_OF_DATA_ENVELOPS - 5;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_BK] = 0;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_VI] = 0;
		wmeACParams.queueSizes[WDRV_QOS_CLASSIFIER_AC_VO] = TOTAL_NUM_OF_MNGT_ENVELOPS;
        apShortRetryLimit = pInitConfigStruct->apShortRetryLimit;
        apLongRetryLimit  = pInitConfigStruct->apLongRetryLimit;

	}

	for (wone_index = 0, global_Qid=0;wone_index < numSupWone; wone_index++)  
	{
        PAL_memSet(&wone_info[wone_index].queue_table,0,sizeof(queues_table_t));
		/* If vHCF is enabled set 3 Tx queues per Wone (low priority, high priority
		   and ps-bcast), otherwise (default) set 2 Tx queues (regular and ps-bcast). */
		
		wone_info[wone_index].Dot11Mode     = pInitConfigStruct->wone_info[wone_index].Dot11Mode;

        for (i=0; i<WDRV_QOS_MAX_AC; ++i, ++global_Qid)
        {
				++wone_info[wone_index].queue_table.numOfQueue;

				wone_info[wone_index].queue_table.ac_queue[i].active = TRUE;
				wone_info[wone_index].queue_table.ac_queue[i].globalQid = global_Qid;
				wone_info[wone_index].queue_table.ac_queue[i].Priority = global_Qid;
				wone_info[wone_index].queue_table.ac_queue[i].type = 0; /*regular */

				wone_info[wone_index].queue_table.ac_queue[i].edcfInfoRequired = wmeACParams.enabled;

            if (wmeACParams.enabled)
            {
                wone_info[wone_index].queue_table.ac_queue[i].edcfInfoRequired = TRUE;
                wone_info[wone_index].queue_table.ac_queue[i].threshold = wme_ac_ucast_threshold[i];
								wone_info[wone_index].queue_table.ac_queue[i].high_threshold = wme_ac_ucast_high_threshold[i];
					wone_info[wone_index].queue_table.ac_queue[i].edcfInfo = 	wmeACParams.ap_edcfParams[i];
					wone_info[wone_index].queue_table.ac_queue[i].edcfInfo.acID = wdrv_QoSACtoACID[i];
					edcfInfoInit(i, &(wone_info[wone_index].queue_table.ac_queue[i].edcfInfo), &wmeACParams, wone_index);
					wone_info[wone_index].queue_table.mgmtAccessCategory = WDRV_QOS_CLASSIFIER_AC_VO;
            } else
            {
					wone_info[wone_index].queue_table.mgmtAccessCategory = WDRV_QOS_CLASSIFIER_AC_BE;
					wone_info[wone_index].queue_table.ac_queue[i].edcfInfoRequired = TRUE;
					wone_info[wone_index].queue_table.ac_queue[i].threshold = no_wme_ac_ucast_threshold[i];
					wone_info[wone_index].queue_table.ac_queue[i].high_threshold = no_wme_ac_ucast_high_threshold[i];
					wone_info[wone_index].queue_table.ac_queue[i].edcfInfo.acID = AC_BEST_EFFORT;
					edcfInfoInit(i, &(wone_info[wone_index].queue_table.ac_queue[i].edcfInfo), &wmeACParams, wone_index);
					global_Qid++;
					break;
				}				
			}

        /* ********************* */
        /* Set PS brodcast Queue */

			wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].active = TRUE;
			wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].globalQid = global_Qid;
			wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].Priority  = global_Qid;
			wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].type = 1; /* PS broadcast */
            if (wmeACParams.enabled)
            {
                wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].threshold = WDRV_DB_MAIN_WME_BCAST_THRESHOLD;
                wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].high_threshold = WDRV_DB_MAIN_WME_BCAST_HIGH_THRESHOLD;
            }
            else
            {
                wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].threshold = WDRV_DB_MAIN_BCAST_THRESHOLD;
                wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].high_threshold = WDRV_DB_MAIN_BCAST_HIGH_THRESHOLD;
            }
            wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].edcfInfo.acID = AC_PS_BROADCAST;
			wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].edcfInfoRequired = TRUE;
            edcfInfoInit( WDRV_QOS_CLASSIFIER_BROADCAST, &(wone_info[wone_index].queue_table.ac_queue[WDRV_QOS_CLASSIFIER_BROADCAST].edcfInfo), &wmeACParams, wone_index);
            ++wone_info[wone_index].queue_table.numOfQueue;
            
        /* ********************* */

			global_Qid ++;


			/* Channel & Band*/
		wone_info[wone_index].band    = pInitConfigStruct->wone_info[wone_index].band;
		wone_info[wone_index].channel = pInitConfigStruct->wone_info[wone_index].channel;

        /* Regulatory Domain */
        if ((ret = wdrv_dbGetdot11CurrentRegDomain(wone_info[wone_index].band, &apDomain)) != OK)
		{
			printf("wdrv_dbGetdot11CurrentRegDomain failed\n");
			return ret;
		}
		
		/* check if channel allowed */
		if (IsChannelAllowed(wone_index,wone_info[wone_index].channel) != TRUE)
		{
            if ((ret = wdrv_dbRegDomainGetDefaultChannel(apDomain, &(wone_info[wone_index].channel))) != OK)
			{
				printf("wdrv_dbRegDomainGetDefaultChannel failed\n");
				return ret;
			}
			printf("setting channel to default according to regulatory domain......\n");
		}


        /* build the apRates from the OperationalRateSet & OperationalAdditionalRateSet */
        /* depending on the Dot11 mode */
        wone_info[wone_index].apRates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;  
        
        if (wone_info[wone_index].band == RADIO_BAND_2_4GHZ)
        {
			wone_info[wone_index].apRates.hdr.eleLen = pInitConfigStruct->wone_info[wone_index].apRates.hdr.eleLen;
			PAL_memCopy(wone_info[wone_index].apRates.rates,
				   pInitConfigStruct->wone_info[wone_index].apRates.rates,
				   wone_info[wone_index].apRates.hdr.eleLen);
        } else
        {
            wone_info[wone_index].apRates.hdr.eleLen = 0;
        }


        if ((wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY) && 
            (wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY_PLUS) &&
			(wone_info[wone_index].channel != 14))
        {
            
			PAL_memCopy((char*)&wone_info[wone_index].apRates.rates[pInitConfigStruct->wone_info[wone_index].apRates.hdr.eleLen],
				   pInitConfigStruct->wone_info[wone_index].apAddRates.rates,
				   pInitConfigStruct->wone_info[wone_index].apAddRates.hdr.eleLen
				  );

            wone_info[wone_index].apRates.hdr.eleLen += pInitConfigStruct->wone_info[wone_index].apAddRates.hdr.eleLen;				   
        }

        /* Get CWmin */
		wone_info[wone_index].CwMinMode = pInitConfigStruct->wone_info[wone_index].CwMinMode;

		if (wone_info[wone_index].CwMinMode == DOT11_CWMIN_ENABLED)
            apCWMinWindow = 15;
        else if (wone_info[wone_index].CwMinMode == DOT11_CWMIN_DISABLED)
            apCWMinWindow = 31;
        else /* behave as dot11mode dependent */
        {
            if (wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY || wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY_PLUS)
                apCWMinWindow = 31;
            else
                apCWMinWindow = 15;
        }

        /* build the apOFDMRates from the OperationalOFDMRateSet */
        if (wone_info[wone_index].band == RADIO_BAND_2_4GHZ)
        {
            wone_info[wone_index].apOFDMRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
            if ((wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY) && 
                (wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY_PLUS) &&
				(wone_info[wone_index].channel != 14))
            {

				wone_info[wone_index].apOFDMRates.hdr.eleLen = pInitConfigStruct->wone_info[wone_index].apOFDMRates.hdr.eleLen;
				PAL_memCopy(wone_info[wone_index].apOFDMRates.rates,
					   pInitConfigStruct->wone_info[wone_index].apOFDMRates.rates,
					   pInitConfigStruct->wone_info[wone_index].apOFDMRates.hdr.eleLen);

			
			}
        } else
        {
            wone_info[wone_index].apOFDMRates.hdr.eleLen = 0;
        }


        wone_info[wone_index].allApSupRatesLen = 0;

	   	wone_info[wone_index].genTxRate	 = pInitConfigStruct->wone_info[wone_index].genTxRate;
		
				/* Wi-Fi mode */
				ApBridgeMode = pInitConfigStruct->ApBridgeMode;
				if (ApBridgeMode && wmeACParams.enabled) {
                    /* In Wi-Fi mode, set the QoS Lifetime to 9000000 */
                    wmeACParams.ap_edcfParams[WDRV_QOS_CLASSIFIER_AC_BE].dot11EDCATableMSDULifetime = 9000000;
                    wmeACParams.ap_edcfParams[WDRV_QOS_CLASSIFIER_AC_BK].dot11EDCATableMSDULifetime = 9000000;
                    wmeACParams.ap_edcfParams[WDRV_QOS_CLASSIFIER_AC_VI].dot11EDCATableMSDULifetime = 9000000;
                    wmeACParams.ap_edcfParams[WDRV_QOS_CLASSIFIER_AC_VO].dot11EDCATableMSDULifetime = 9000000;
				}

				/* Wi-Fi mode - remove 22 rate in .11g mode */
				if (ApBridgeMode) {
					if ((wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY) && 
							(wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY_PLUS)) {
						int new_len;
						
						for (i=0, new_len = wone_info[wone_index].apRates.hdr.eleLen; i<wone_info[wone_index].apRates.hdr.eleLen; i++)	{
							if (wone_info[wone_index].apRates.rates[i] == DOT11_RATE_22) {
								--new_len;								
								break;
							}
						}
						wone_info[wone_index].apRates.hdr.eleLen = new_len;
					}
				}

        for (i=0; i<wone_info[wone_index].apRates.hdr.eleLen; i++)
        {
            wone_info[wone_index].allApSupRates[wone_info[wone_index].allApSupRatesLen++] = 
            wone_info[wone_index].apRates.rates[i];
        }

        for (i=0; i<wone_info[wone_index].apOFDMRates.hdr.eleLen; i++)
        {
            wone_info[wone_index].allApSupRates[wone_info[wone_index].allApSupRatesLen++] =
            wone_info[wone_index].apOFDMRates.rates[i];
        }
        /* Sort allApSupRates in ascending order */
        for (i=0; i<wone_info[wone_index].allApSupRatesLen; i++)
        {
            for (j=i+1; j<wone_info[wone_index].allApSupRatesLen; j++)
            {
                if ((wone_info[wone_index].allApSupRates[j] & 0x7F) < (wone_info[wone_index].allApSupRates[i] & 0x7F))
                {
                    char temp = wone_info[wone_index].allApSupRates[i];
                    wone_info[wone_index].allApSupRates[i] = wone_info[wone_index].allApSupRates[j];
                    wone_info[wone_index].allApSupRates[j] = temp;
                }
            }
        }
        /* Add to handle special test case where a rate is set more than once */
        for (i=1; i<wone_info[wone_index].allApSupRatesLen; i++)
        {
            if (wone_info[wone_index].allApSupRates[i] == wone_info[wone_index].allApSupRates[i-1])
            {
                for (j=i; j<wone_info[wone_index].allApSupRatesLen-1; j++)
                {
                    wone_info[wone_index].allApSupRates[j] = wone_info[wone_index].allApSupRates[j+1];
                }
                wone_info[wone_index].allApSupRatesLen--;
            }
        }

		wone_info[wone_index].ERPProtectionTI = pInitConfigStruct->wone_info[wone_index].ERPProtectionTI;
	
		FeatureSet = pInitConfigStruct->FeatureSet;
       
		TIcap.hdr.eleId = DOT11_TI_CAP_ELE_ID;
        TIcap.hdr.eleLen = DOT11_4X_FEATURE_SET_ZERO_LEN;
        PAL_memCopy(TIcap.cap, TI_OUI, DOT11_OUI_LEN);
        TIcap.cap[DOT11_OUI_LEN] = FeatureSet;
        if (FeatureSet == 0)
            TIcap.hdr.eleLen = DOT11_4X_FEATURE_SET_ZERO_LEN;
        else
        {
            int curr = DOT11_4X_FEATURE_SET_ZERO_LEN;
            if (enable4x)
            {
                /* declare concatenation */
                if (enableConcat)
                {
                    MaxConcatSize = ENDIAN_HANDLE_WORD(WLAN_MAX_CONCAT_SIZE);
                    TIcap.cap[curr] = TI_CAP_4X_CONCATENATION;
                    curr += sizeof(UINT16);
                    TIcap.cap[curr] = 2;
                    curr += sizeof(UINT16);
                    PAL_memCopy(&TIcap.cap[curr], (char *)&MaxConcatSize, sizeof(MaxConcatSize));
                    curr += 2;
                }
                /* declare Ack emulation */
                if (enableAckEmul)
                {
                    TIcap.cap[curr] = TI_CAP_4X_TCP_ACK_EMUL;
                    curr += sizeof(UINT16);
                    TIcap.cap[curr] = 0;
                    curr += sizeof(UINT16);
                }
            }
            /* declare trick packet */
            if (wone_info[wone_index].ERPProtectionTI)
            {
                TIcap.cap[curr] = TI_CAP_TRICK_PACKET_ERP;
                curr += sizeof(UINT16);
                TIcap.cap[curr] = 0;
                curr += 2;
            }

            if (curr <= DOT11_TI_CAP_MAX_LEN)
                TIcap.hdr.eleLen = curr;
            else
            {
                printf("there is not enough space allocated for 4x IE (need:%d, have:%d)\n",
                       curr, DOT11_TI_CAP_MAX_LEN);
                return DDC_INIT_RESOURCE_ERROR;
            }
        }

        if ((ret = initDefaultActiveRate(wone_index, pInitConfigStruct)) != OK)
		{
			printf("initDefaultActiveRate failed\n");
			return DDC_INIT_CFG_LOGICAL_ERROR;
		}

		wone_info[wone_index].PbccAlgorithmMode = pInitConfigStruct->wone_info[wone_index].PbccAlgorithmMode;
        if (wone_info[wone_index].PbccAlgorithmMode != PBCC_ALGO_DISABLED)
            wdrv_dbSetCurrentPBCCSupportMode(TRUE);
		wone_info[wone_index].RateAdaptation = pInitConfigStruct->wone_info[wone_index].RateAdaptation;
		/* Set PreambleMode to LOMG when in WiFi WMM mode only to pass WiFi WMM Test2 (b mode)          */
		/* Note: this change ovewrites any manual configuration                                         */
		if (ApBridgeMode && wmeACParams.enabled)
		{
			wone_info[wone_index].PreambleMode = PREAMBLE_MODE_LONG;
		}
		else
		{
			wone_info[wone_index].PreambleMode = pInitConfigStruct->wone_info[wone_index].PreambleMode;
		}
		wone_info[wone_index].SlotTimeMode = pInitConfigStruct->wone_info[wone_index].SlotTimeMode;
		wone_info[wone_index].ERPProtectionMode = pInitConfigStruct->wone_info[wone_index].ERPProtectionMode;
		wone_info[wone_index].ERPProtectionType = pInitConfigStruct->wone_info[wone_index].ERPProtectionType;
        wone_info[wone_index].apCapabilities = 0;
        wone_info[wone_index].apCapabilities |= (TRUE << CAP_ESS_SHIFT);
        if (wone_info[wone_index].PbccAlgorithmMode != PBCC_ALGO_DISABLED)
            wone_info[wone_index].apCapabilities |= (1 << CAP_PBCC_SHIFT);
        wone_info[wone_index].apCapabilities |= (Agile << CAP_AGILE_SHIFT);
        if (wone_info[wone_index].PreambleMode != PREAMBLE_MODE_LONG)
        {
            wone_info[wone_index].apCapabilities |= (1 << CAP_PREAMBLE_SHIFT);
        }
        if (wone_info[wone_index].SlotTimeMode != SLOTTIME_MODE_DISABLED)
        {
            wone_info[wone_index].apCapabilities |= (1 << CAP_SLOTTIME_SHIFT);
        }


        PAL_memSet(&DynamicAlgInfo, 0, sizeof(DynamicAlgInfo));
        		
		wone_info[wone_index].UsageTime = pInitConfigStruct->wone_info[wone_index].UsageTime;
        /* if usage time mode is short -> divide UsageTime n=by 2 */
        if (usageTimeMode == 0)
          wone_info[wone_index].UsageTime = ((wone_info[wone_index].UsageTime)/2);
	} /* for wone */

	
	enable4xMixMode		     = pInitConfigStruct->enable4xMixMode;		
	apRespRateMode		     = ASSOCRESP_RATE_AP_AND_STA;
	RadioCalibrateOption     = pInitConfigStruct->RadioCalibrateOption;
	RadioCalibrateInterval   = pInitConfigStruct->RadioCalibrateInterval;
	RateFallback		     = pInitConfigStruct->RateFallback;

	apBeaconPeriod    = pInitConfigStruct->apBeaconPeriod;
	apDtimPeriod      = pInitConfigStruct->apDtimPeriod;
	apRtsThreshold    = pInitConfigStruct->apRtsThreshold;
	if (wmeACParams.enabled)
		apFragThreshold   = MAX_FRAG_THRESHOLD;
	else
		apFragThreshold   = pInitConfigStruct->apFragThreshold;
	apMaxTxLifetime   = pInitConfigStruct->apMaxTxLifetime;
	apMaxRxLifetime   = pInitConfigStruct->apMaxRxLifetime; 
	EnEnergyDetect	  = pInitConfigStruct->enableEnergyDetect;
	EEPromLessSupport = pInitConfigStruct->enableEEPromLessSupport;
	
	DynamicAlgorithmsTimeout = DEF_DYN_ALG_TIMEOUT;

	
	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
   {

		   value16 = pInitConfigStruct->dot11AuthenticationAlgorithmTable[i].authAlgo;
		   value   = pInitConfigStruct->dot11AuthenticationAlgorithmTable[i].enable;

		   if (value16 == MIB_OPEN_SYSTEM)
		   {
			   authOpenSystemIndex = i;

			   if (value == TRUE)
				   authOpenSystemEnable = TRUE;
			   else
				   authOpenSystemEnable = FALSE;
		   }

		   else if (value16 == MIB_SHARED_KEY)
		   {
			   authSharedKeyIndex = i;

			   if (value == TRUE)
				   authSharedKeyEnable = TRUE;
			   else
				   authSharedKeyEnable = FALSE;
		   }

		   else
		   {
			   printf("WDRV_DB: FATAL ERROR, receive error auth algorithm from init struct.\n");
			   return DDC_INIT_CFG_RANGE_ERROR;
		   }
   }

	WlanServiceState = pInitConfigStruct->WlanServiceState;
 	

	EnRestart = pInitConfigStruct->EnRestart;
    SavedEnRestart =        EnRestart;
    RestoreSavedEnRestart =        FALSE;
    if (!WlanServiceState)
		EnRestart = 0;
    

	EnInterfWA			  = pInitConfigStruct->EnInterfWA;
	RxFreeMemOptionWA     = pInitConfigStruct->RxFreeMemOptionWA; 
    BeaconNotSentOptionWA = pInitConfigStruct->BeaconNotSentOptionWA;
    TxStuckOptionWA       = pInitConfigStruct->TxStuckOptionWA;
    ConsFcsErrOptionWA    = pInitConfigStruct->ConsFcsErrOptionWA; 
	aciOptionWA			  = pInitConfigStruct->aciOptionWA;
	nbOptionWA			  = pInitConfigStruct->nbOptionWA;
	aciStateTimeout		  = pInitConfigStruct->interfWAparams.aciStateTimeout;
	nbStateTimeout		  = pInitConfigStruct->interfWAparams.nbStateTimeout;
	aciRateThreshold	  = pInitConfigStruct->interfWAparams.aciRateThreshold;
	aciEventThreshold	  = pInitConfigStruct->interfWAparams.aciEventThreshold;
	pdUnderAciLowThrsh	  = pInitConfigStruct->interfWAparams.pdUnderAciLowThrsh;
	pdUnderAciHighThrsh   = pInitConfigStruct->interfWAparams.pdUnderAciHighThrsh;
	mediumBusyThresh	  = pInitConfigStruct->interfWAparams.mediumBusyThresh;


	hwInterfParams.inputCCA     = pInitConfigStruct->hwInterfParams.inputCCA;
	hwInterfParams.qualifiedCCA = pInitConfigStruct->hwInterfParams.qualifiedCCA;
	hwInterfParams.stompForRx   = pInitConfigStruct->hwInterfParams.stompForRx;
	hwInterfParams.stompForTx   = pInitConfigStruct->hwInterfParams.stompForTx;
	hwInterfParams.txCCA        = pInitConfigStruct->hwInterfParams.txCCA;

    if (pInitConfigStruct->wone_info[0].Dot11Mode == DOT11_MODE_OFDM_ONLY)
		SlotTimeThreshold = 3;
	else
		SlotTimeThreshold = 0;

   /******************************************
    * Radio Params - Read them all from NVRAM
    ******************************************/
   PAL_memSet( (char*)&dbRadioParams, 0, sizeof( dbRadioParams));

   dbRadioParams.overrideBitmask      = pInitConfigStruct->dbRadioParams.overrideBitmask;
   dbRadioParams.powerTable_paramType = pInitConfigStruct->dbRadioParams.powerTable_paramType;
   PAL_memCopy(dbRadioParams.powerTable_param,
		  pInitConfigStruct->dbRadioParams.powerTable_param,
		  MAX_RADIO_PARAM_POWER_TABLE);
   
   RadarDetectMode = pInitConfigStruct->RadarDetectMode;

   /* Wi-Fi mode */
    if (ApBridgeMode)
    {
	   /* Reflects Wi-Fi Alliance changes */
	   apBeaconPeriod = 100;
	   apDtimPeriod = 1;
   }

   printf("%s: wdsEnabled=%d\n",__func__, wdsEnabled);			
	wdsEnabled = pInitConfigStruct->wdsEnable;

   printf("Configuration succeeded !!!\n");

   return DDC_INIT_SUCCESS;
}/* END wdrv_dbMainInit() */

/*************************************************************************
 *                        initDefaultActiveRate                          *
 *************************************************************************
DESCRIPTION: This function initializes the AP's default active rate.
				THe default active rate is the highest basic rate supportted 
				by the AP.
			There are 2 variables:
			1. defaultActiveRate is the default active rate in standard units.
			2. defaultActiveRateAcx is the default active rate in ACX units.
			3. defaultActiveRateAcxWithFallback is the default active rate in
               ACX units with other rates used in FW rate fallback.
                                                                                         
INPUT:   woneIndex    

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
static int initDefaultActiveRate(UINT8 woneIndex, initConfigStruct *pInitConfigStruct)
{
	int i;
	UINT16 activeRateAcx = 0;

	
	wone_info[woneIndex].maxActiveRate = 0;
	wone_info[woneIndex].maxCCKactiveRate = DOT11_RATE_1;
	wone_info[woneIndex].maxCCKbasicRate = DOT11_RATE_1;
    wone_info[woneIndex].maxOFDMbasicRate = DOT11_RATE_6;
	wone_info[woneIndex].minCCKbasicRate = DOT11_RATE_11 | 0x80;
    wone_info[woneIndex].minOFDMbasicRate = DOT11_RATE_54 | 0x80;
	wone_info[woneIndex].defaultActiveRate = 0;
	wone_info[woneIndex].defaultActiveRateAcx = 0;
	wone_info[woneIndex].defaultActiveRateAcxWithFallback  = 0;
	wone_info[woneIndex].defaultActiveRateAcxBrcst = 0;
	wone_info[woneIndex].defaultActiveRateAcxMcst = 0;
	wone_info[woneIndex].defaultActiveRateAcxBrcstFallback = 0;
	wone_info[woneIndex].defaultActiveRateAcxMcstFallback = 0;
	
	for (i = 0; i < wone_info[woneIndex].allApSupRatesLen; i++)
	{
		/* Find max of all rates (basic and not basic) */
		wone_info[woneIndex].maxActiveRate = 
                    _max(wone_info[woneIndex].allApSupRates[i] & 0x7F, wone_info[woneIndex].maxActiveRate);
        
		if (((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_1) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_2) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_5_5) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_11) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_22))			
		{
            if ((wone_info[woneIndex].allApSupRates[i] & 0x7F) > wone_info[woneIndex].maxCCKactiveRate)
				wone_info[woneIndex].maxCCKactiveRate = wone_info[woneIndex].allApSupRates[i] & 0x7F;
		}
		   

		if (isBasic(wone_info[woneIndex].allApSupRates[i]) == TRUE)
		{
			/* Find min/max of basic rates only */
			wone_info[woneIndex].defaultActiveRate = 
                 max(wone_info[woneIndex].allApSupRates[i], wone_info[woneIndex].defaultActiveRate);

			switch (wone_info[woneIndex].allApSupRates[i] & 0x7F)
			{
				case DOT11_RATE_1:
					activeRateAcx = ACX_RATE_1;
                    if (wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_2:
					activeRateAcx = ACX_RATE_2;
                    if (wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_5_5:
					activeRateAcx = ACX_RATE_5_5;
                    if (wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_6:
					activeRateAcx = ACX_RATE_6;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_9:
					activeRateAcx = ACX_RATE_9;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_11:
					activeRateAcx = ACX_RATE_11;
                    if (wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_12:
					activeRateAcx = ACX_RATE_12;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_18:
					activeRateAcx = ACX_RATE_18;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					
					break;

				case DOT11_RATE_22:
					activeRateAcx = ACX_RATE_22;
					break;

				case DOT11_RATE_24:
					activeRateAcx = ACX_RATE_24;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_36:
					activeRateAcx = ACX_RATE_36;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_48:
					activeRateAcx = ACX_RATE_48;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_54:
					activeRateAcx = ACX_RATE_54;
                    if (wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if (wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				default:
					wdrv_utilsReport(RL_DB, PRI_INFORMATION,
                                     "WDRV_DB: Warning, Invalid default active rate.\n");
			}

			/* Find max of basic rates only in ACX units */
			wone_info[woneIndex].defaultActiveRateAcx = 
                max(activeRateAcx, wone_info[woneIndex].defaultActiveRateAcx);

			/* Set all basic rates in ACX units, used when fallback is enabled */
			wone_info[woneIndex].defaultActiveRateAcxWithFallback |= activeRateAcx;

		}
	
	}
       /* Init broadcast/multicast rates */

	   if (wdrv_dbGetApBridgeMode()) /* WI-FI mode */
	   {
		   if (wone_info[woneIndex].band == RADIO_BAND_2_4GHZ) 
		   {
			   wone_info[woneIndex].defaultActiveRateAcxBrcst = ACX_RATE_2;
			   wone_info[woneIndex].defaultActiveRateAcxMcst  = ACX_RATE_2;
		   }
		   else /* 5 GHZ*/
		   {
			   wone_info[woneIndex].defaultActiveRateAcxBrcst = ACX_RATE_6;
			   wone_info[woneIndex].defaultActiveRateAcxMcst  = ACX_RATE_6;
		   }	   
		   
	   }
	   else
	   {
		   /* If broadcast default rate was configured convert it to ACX rate from dot11 rate*/
			if (pInitConfigStruct->wone_info[woneIndex].brsctDefActiveRate) 
				wone_info[woneIndex].defaultActiveRateAcxBrcst = 
					convertDOT11RatesToAcxBitWiseFormat(&pInitConfigStruct->wone_info[woneIndex].brsctDefActiveRate,1);
			else
			{
				if (wone_info[woneIndex].band == RADIO_BAND_2_4GHZ) 
					wone_info[woneIndex].defaultActiveRateAcxBrcst = BRCST_MTCST_ACX_RATE;
				else
					wone_info[woneIndex].defaultActiveRateAcxBrcst = BRCST_MTCST_OFDM_ACX_RATE;
			}

			/* If multicast default rate was configured convert it to ACX rate from dot11 rate*/
			if (pInitConfigStruct->wone_info[woneIndex].mcstDefActiveRate) 
				wone_info[woneIndex].defaultActiveRateAcxMcst = 
					convertDOT11RatesToAcxBitWiseFormat(&pInitConfigStruct->wone_info[woneIndex].mcstDefActiveRate,1);
			else
			{
				if (wone_info[woneIndex].band == RADIO_BAND_2_4GHZ) 
					wone_info[woneIndex].defaultActiveRateAcxMcst = BRCST_MTCST_ACX_RATE;
				else
					wone_info[woneIndex].defaultActiveRateAcxMcst = BRCST_MTCST_OFDM_ACX_RATE;
			}
			
	   }
	   /* Fallback is not performed for broadcast/multicast but need to set it for interface compatibility */
	   wone_info[woneIndex].defaultActiveRateAcxBrcstFallback |= wone_info[woneIndex].defaultActiveRateAcxBrcst;
	   wone_info[woneIndex].defaultActiveRateAcxMcstFallback |= wone_info[woneIndex].defaultActiveRateAcxMcst;

		
	return OK;
}

/*************************************************************************
 *                        initMibTables                                  *
 *************************************************************************
DESCRIPTION: This function initializes mib table
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void	initMibTables(void)
{
	PAL_memSet(&dot11StationConfigTable, 0, sizeof(dot11StationConfigTable));
}

/*************************************************************************
 *                        wdrv_dbGetBssid                                  *
 *************************************************************************
DESCRIPTION: get bss id
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBssid(char *value)
{
	PAL_memCopy(value, BSSID, MAC_ADDR_LEN);	
}

/*************************************************************************
 *	wdrv_dbGetApCapabilities 					                                  *
 *************************************************************************
DESCRIPTION: Returns AP capabilities
                                                                                                   
INPUT:     woneIndex  

OUTPUT:    cap  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetApCapabilities(UINT8 woneIndex, UINT16 *cap)
{
	PAL_memCopy(cap, &(wone_info[woneIndex].apCapabilities), FIX_FIELD_LEN);
}

/*************************************************************************
 *		wdrv_dbSetApCapabilities 				                                  *
 *************************************************************************
DESCRIPTION: Configures AP capabilities
                                                                                                   
INPUT:     woneIndex,   cap

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetApCapabilities(UINT8 woneIndex, UINT16 cap)
{
	PAL_memCopy(&(wone_info[woneIndex].apCapabilities), &cap, FIX_FIELD_LEN);
}

/*************************************************************************
 *	wdrv_dbGetDefaultActiveRate				                                  *
 *************************************************************************
DESCRIPTION: Returns default active rate
                                                                                                   
INPUT:    woneIndex   

OUTPUT:   defaultRate   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRate(UINT8 woneIndex ,UINT8 *defaultRate)
{
	*defaultRate = wone_info[woneIndex].defaultActiveRate;
}


/*************************************************************************
 *		wdrv_dbGetMaxCCKbasicRate 				                                  *
 *************************************************************************
DESCRIPTION: Returns max CCK basic rate
                                                                                                   
INPUT:   woneIndex    

OUTPUT:   maxBasicCCKRate   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxCCKbasicRate(UINT8 woneIndex ,UINT8 *maxBasicCCKRate)
{
	*maxBasicCCKRate = wone_info[woneIndex].maxCCKbasicRate;
}

/*************************************************************************
 *	 wdrv_dbGetMinCCKbasicRate				                                  *
 *************************************************************************
DESCRIPTION: Returns min CCK basioc rate 
                                                                                                   
INPUT:     woneIndex  

OUTPUT:   minBasicCCKRate   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMinCCKbasicRate(UINT8 woneIndex ,UINT8 *minBasicCCKRate)
{
	*minBasicCCKRate = wone_info[woneIndex].minCCKbasicRate;
}

/*************************************************************************
 *	wdrv_dbGetMaxOFDMbasicRate				                                  *
 *************************************************************************
DESCRIPTION: Returns MAX OFDM basic rate
                                                                                                   
INPUT:   woneIndex    

OUTPUT:  maxBasicOFDMRate    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxOFDMbasicRate(UINT8 woneIndex ,UINT8 *maxBasicOFDMRate)
{
	*maxBasicOFDMRate = wone_info[woneIndex].maxOFDMbasicRate;
}

/*************************************************************************
 *	wdrv_dbGetMinOFDMbasicRate					                                  *
 *************************************************************************
DESCRIPTION: Returns min OFDM basic rate
                                                                                                   
INPUT: woneIndex      

OUTPUT:  minBasicOFDMRate    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMinOFDMbasicRate(UINT8 woneIndex ,UINT8 *minBasicOFDMRate)
{
	*minBasicOFDMRate = wone_info[woneIndex].minOFDMbasicRate;
}

/*************************************************************************
 *	wdrv_dbGetDefaultActiveRateBrcst					                                  *
 *************************************************************************
DESCRIPTION: Returns default broadcast dot11 rate
                                                                                                   
INPUT: woneIndex      

OUTPUT:  rate 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRateBrcst(UINT8 woneIndex ,UINT8 *rate)
{
	*rate = convertAcxRateToDOT11Rate(wone_info[woneIndex].defaultActiveRateAcxBrcst);
}

/*************************************************************************
 *	wdrv_dbGetDefaultActiveRateMcst					                                  *
 *************************************************************************
DESCRIPTION: Returns default multicast dot11 rate
                                                                                                   
INPUT: woneIndex      

OUTPUT:  rate 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRateMcst(UINT8 woneIndex ,UINT8 *rate)
{
	*rate = convertAcxRateToDOT11Rate(wone_info[woneIndex].defaultActiveRateAcxMcst);
}

/*************************************************************************
 *	wdrv_dbGetGenTxActiveRate					                                  *
 *************************************************************************
DESCRIPTION: Returns general TX dot11 rate with which beacons, probe responses,
			 RTS frames and PS-Polls are transmitted
                                                                                                   
INPUT: woneIndex      

OUTPUT:  rate 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetGenTxActiveRate(UINT8 woneIndex ,UINT8 *rate)
{
	*rate = wone_info[woneIndex].genTxRate;
}


/*************************************************************************
 *		wdrv_dbGetMaxCCKactiveRate			                                  *
 *************************************************************************
DESCRIPTION: Returns MAX CK active rate
                                                                                                   
INPUT:    woneIndex   

OUTPUT:     maxActiveCCKRate 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxCCKactiveRate(UINT8 woneIndex ,UINT8 *maxActiveCCKRate)
{
	*maxActiveCCKRate = wone_info[woneIndex].maxCCKactiveRate;	
}

/*************************************************************************
 *	wdrv_dbGetDefaultActiveRateAcx		                                  *
 *************************************************************************
DESCRIPTION: Returns default active rate (ACX)
                                                                                                   
INPUT:     woneIndex  , pktType

OUTPUT:      defaultRate, defaultRateWithFallback

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRateAcx(UINT8 woneIndex ,UINT16 *defaultRate, 
									UINT16 *defaultRateWithFallback, UINT8 pktType)
{
	switch(pktType)
	{
		case TXATTR_BRCST:
			*defaultRate = wone_info[woneIndex].defaultActiveRateAcxBrcst;
			*defaultRateWithFallback = wone_info[woneIndex].defaultActiveRateAcxBrcstFallback;
			break;
		case TXATTR_MTCST:
			*defaultRate = wone_info[woneIndex].defaultActiveRateAcxMcst;
			*defaultRateWithFallback = wone_info[woneIndex].defaultActiveRateAcxMcstFallback;
			break;
		case TXATTR_UNICAST:
			*defaultRate = wone_info[woneIndex].defaultActiveRateAcx;
			*defaultRateWithFallback = wone_info[woneIndex].defaultActiveRateAcxWithFallback;			
			break;
	}
}


/*************************************************************************
 *	wdrv_dbGetMaxActiveRate						                                  *
 *************************************************************************
DESCRIPTION: Returns Max active rate
                                                                                                   
INPUT:       

OUTPUT:   activeRate   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxActiveRate(UINT8 *activeRate)
{
	*activeRate = wone_info[0].maxActiveRate;
}

/*************************************************************************
 *	wdrv_dbGetDynamicAlgorithmsTimeout                                  *
 *************************************************************************
DESCRIPTION: Returns dynamic algorithm timeout
                                                                                                   
INPUT:       

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDynamicAlgorithmsTimeout(UINT32 *value)
{
	*value = DynamicAlgorithmsTimeout;
}

/*************************************************************************
 *	wdrv_dbGetRadioCalibrateOptio			                                  *
 *************************************************************************
DESCRIPTION: Retuns radio calibrate option
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioCalibrateOption(UINT32 *value)
{
	*value = RadioCalibrateOption;
}

/*************************************************************************
 *	wdrv_dbGetRadioCalibrateInterva		                                  *
 *************************************************************************
DESCRIPTION: Retuns radio calibrate interval
                                                                                                   
INPUT:       

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioCalibrateInterval(UINT16 *value)
{
	*value = RadioCalibrateInterval;
}

/*************************************************************************
 *	wdrv_dbGetPreambleMode						                                  *
 *************************************************************************
DESCRIPTION: Returns preamble mode
                                                                                                   
INPUT:  woneIndex     

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPreambleMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].PreambleMode;
}

/*************************************************************************
 *	wdrv_dbGetSlotTimeMode						                                  *
 *************************************************************************
DESCRIPTION: Returns slot time mode
                                                                                                   
INPUT:    woneIndex   

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSlotTimeMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].SlotTimeMode;
}

/*************************************************************************
 *	 wdrv_dbGetSlotTimeThreshold			                                  *
 *************************************************************************
DESCRIPTION: Returns slot time threshold
                                                                                                   
INPUT:       

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSlotTimeThreshold(UINT16 *value)
{
	*value = SlotTimeThreshold;
}

/*************************************************************************
 *		wdrv_dbGetERPProtectionMode			                                  *
 *************************************************************************
DESCRIPTION: Retyurns ERP protection mode
                                                                                                   
INPUT:  woneIndex     

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionMode;	
}

/*************************************************************************
 *		wdrv_dbSetERPprotectionMode			                                  *
 *************************************************************************
DESCRIPTION: Sets the ERP Protection mode in runtime without updating the NVRAM
                                                                                                   
INPUT:   woneIndex,    value 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

void wdrv_dbSetERPprotectionMode(UINT8 woneIndex ,UINT8 value)
{	
	 unsigned int flags;

	 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	 wone_info[woneIndex].ERPProtectionMode = value;
	 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbGetERPProtectionType 		                                  *
 *************************************************************************
DESCRIPTION: Returns ERP protection type
                                                                                                   
INPUT:     woneIndex  

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionType(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionType;	
}

/*************************************************************************
 *	wdrv_dbSetDefaultWepKey						                                  *
 *************************************************************************
DESCRIPTION: Sets default WEP key
                                                                                                   
INPUT:  index,     keySize,  keyValue

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetDefaultWepKey(UINT8 index,UINT16 keySize,UINT8 *keyValue)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION - incase of restart workaround*/

    if (keyValue == NULL)
    {
        PAL_memSet(defaultWepKeys[index].WepKey,0,keySize);
        defaultWepKeys[index].keySize = 0;

    } else
    {
        PAL_memCopy(defaultWepKeys[index].WepKey,keyValue,keySize);
        defaultWepKeys[index].keySize = keySize;

    }
	
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

}

/*************************************************************************
 *wdrv_dbGetDefaultWepKey								                                  *
 *************************************************************************
DESCRIPTION: Returns default WEP key
                                                                                                   
INPUT:    index,    

OUTPUT:   keySize,  keyValue  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultWepKey(UINT8 index,UINT16 *keySize,UINT8 *keyValue)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION - incase of restart workaround*/

    PAL_memCopy(keyValue,defaultWepKeys[index].WepKey,defaultWepKeys[index].keySize);
    *keySize = defaultWepKeys[index].keySize;

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *		wdrv_dbGetWepDefaultKeyId					                                  *
 *************************************************************************
DESCRIPTION: Returns default key id
                                                                                                   
INPUT:       

OUTPUT:   index   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWepDefaultKeyId(UINT8 *index)
{
    *index = wepDefaultKeyId;
}

/*************************************************************************
 *	 wdrv_dbGetPrivacyWEPDefaultKeyId	                                  *
 *************************************************************************
DESCRIPTION: Returns default key id
                                                                                                   
INPUT:       

OUTPUT:   index   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyWEPDefaultKeyId(UINT8 *value)
{
    *value = wepDefaultKeyId;
}
/*************************************************************************
 *	wdrv_dbSetWepDefaultKeyId									                            *
 *************************************************************************
DESCRIPTION: sets default WEP key id
                                                                                                   
INPUT:  index     

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWepDefaultKeyId(UINT8 index)
{
    wepDefaultKeyId = index;
}
/*************************************************************************
 *		wdrv_dbGetWEP_DefaultKeyId								                                  *
 *************************************************************************
DESCRIPTION: Returns default key id
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWEP_DefaultKeyId(UINT8 *index)
{
	*index = wepDefaultKeyId;
}

/*************************************************************************
 *	wdrv_dbGetERPProtectionTI									                                  *
 *************************************************************************
DESCRIPTION: Returns ERP protection TI
                                                                                                   
INPUT:   woneIndex    

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionTI(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionTI;	
}

/*************************************************************************
 *	wdrv_dbGetFeatureSet							                                  *
 *************************************************************************
DESCRIPTION: Returns feature set
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetFeatureSet(UINT8 *value)
{
	*value = FeatureSet;	
}
 
/*************************************************************************
 *		wdrv_dbGetDot11Mode								                                  *
 *************************************************************************
DESCRIPTION: Returns dot11 mode
                                                                                                   
INPUT:  woneIndex     

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDot11Mode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].Dot11Mode;	
}

/*************************************************************************
 *	wdrv_dbGetPBCCAlgorithmMode									                                  *
 *************************************************************************
DESCRIPTION: Returns PBCC algorithm mode
                                                                                                   
INPUT:  woneIndex     

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPBCCAlgorithmMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].PbccAlgorithmMode;
}

/*************************************************************************
 *					wdrv_dbGetPowerLevel					                                  *
 *************************************************************************
DESCRIPTION:
 This is the 1st phase of implementation of Get PowerLevel. 
 In this phase there is no band involved and required, due  
 to a FW limitation (only one value can be sent down.)      
 Second phase (TBD..) will involve also the woneIndex, so   
 the interface will be probably changed (and so this note..) 
                                                                                                  
INPUT:  channel     
  
OUTPUT:  power level      

RETURN:  void    
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPowerLevel(UINT8 channel, UINT8 *value)
{	
    INT8  powerAdbm, poweBdbm, userTxPower, maxTxPower=0, txPowerPerLevel, band;
    UINT8 status, index, powerConst, step, ch;
    UINT8 currentRegDomain, numOfRanges, isPrivateDomain, userTxPowerLevel, TotalWones;
    tChannelRangeElementStruct rangeElementStruct;
    UINT32 level;

    wdrv_dbGetMaxTxPowerLevel(&userTxPowerLevel);
    wdrv_dbGetdot11MultiDomainCapabilityStatus(&status);

    if (status == 0)     /* 802.11d is unused */
    {
        *value = userTxPowerLevel;
        return;
    } else                 /* 802.11d is used */
    {
        /* Check power level for band A for given channel */
        powerAdbm = 0;
        wdrv_dbGetTxPowerLevels(RADIO_BAND_5GHZ, userTxPowerLevel, &userTxPower);
        wdrv_dbGetRegDomainInformation(WLAN_BAND_A, 
                                       &currentRegDomain, 
                                       &numOfRanges, 
                                       &isPrivateDomain);

        for (index = 1; index <= numOfRanges; index++) 
        {
            if (isPrivateDomain) 
                wdrv_dbGetPrivateRangeElement(WLAN_BAND_A,index,&rangeElementStruct);
            else
                wdrv_dbGetRangeElement(WLAN_BAND_A,
                                       (IntegerRegulatoryDomains_e)currentRegDomain,
                                       index,
                                       &rangeElementStruct);
            step = rangeElementStruct.stepOpt & 0xF;
            for (ch =  rangeElementStruct.startCh;
                 ch <= rangeElementStruct.stopCh;
                 ch += step) 
            {
                if (ch == channel)
                {
                    powerAdbm = rangeElementStruct.powerLimit;
                    break;
                }
            }
            if (powerAdbm > 0)
            {
                wdrv_dbGetdot11LocalPowerConstraint(&powerConst);
                powerAdbm -= powerConst;
                if (powerAdbm > userTxPower)
                    powerAdbm = userTxPower;
                break;
            }
        }

        /* Check power level for band B for given channel */
        poweBdbm = 0;
        wdrv_dbGetTxPowerLevels(RADIO_BAND_2_4GHZ, userTxPowerLevel, &userTxPower);
        wdrv_dbGetRegDomainInformation(WLAN_BAND_B,
                                       &currentRegDomain, 
                                       &numOfRanges, 
                                       &isPrivateDomain);

        for (index = 1; index <= numOfRanges; index++) 
        {
            if (isPrivateDomain) 
                wdrv_dbGetPrivateRangeElement(WLAN_BAND_B,index,&rangeElementStruct);
            else
                wdrv_dbGetRangeElement(WLAN_BAND_B,
                                       (IntegerRegulatoryDomains_e)currentRegDomain,
                                       index,
                                       &rangeElementStruct);
            step = rangeElementStruct.stepOpt & 0xF;
            for (ch =  rangeElementStruct.startCh;
                 ch <= rangeElementStruct.stopCh;
                 ch += step) 
            {
                if (ch == channel)
                {
                    poweBdbm = rangeElementStruct.powerLimit;
                    break;
                }
            }
            if (poweBdbm > 0)
            {
                wdrv_dbGetdot11LocalPowerConstraint(&powerConst);
                poweBdbm -= powerConst;
                if (poweBdbm > userTxPower)
                    poweBdbm = userTxPower;
                break;
            }
        }
        /* Get the lower between the TxPowers of A and B */
        if ((powerAdbm == 0) && (poweBdbm == 0))
        {
            *value = 0;     /* channel not exists */
            return;
        } else if (powerAdbm == 0)   /* channel belongs to B band */
        {
            maxTxPower = poweBdbm;
            band = RADIO_BAND_2_4GHZ;
        } else if (poweBdbm == 0)   /* channel belongs to A band */
        {
            maxTxPower = powerAdbm;
            band = RADIO_BAND_5GHZ;
        } else                    /* channel contains in both A and B bands */
        {
            wdrv_dbGetNumSupWone(&TotalWones);
            if (TotalWones == WONE_SINGLE_BAND)
            {
                wdrv_dbGetBand(0,&band);
            } else
            {
                maxTxPower = min(powerAdbm,poweBdbm);
                band = (powerAdbm < poweBdbm) ? RADIO_BAND_5GHZ : RADIO_BAND_2_4GHZ;
            }
        }
    }
    /* Now we have the maxTxPower - should find the correct entry in power level tables. */
    for (level=1; level <= 5; level++) 
    {
        wdrv_dbGetTxPowerLevels(band, level, &txPowerPerLevel);
        if (txPowerPerLevel <= maxTxPower) 
        {
            *value = (UINT8)level;
            return;
        }
    }
    /* txPower is below the minimum level */
	*value = 5;
}


/*************************************************************************
 *	wdrv_dbGetRateFallback									                                  *
 *************************************************************************
DESCRIPTION: Returns rate fallback status
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRateFallback(UINT8 *value)
{
	*value = RateFallback;
}

/*************************************************************************
 *		wdrv_dbGetRateAdaptation								                                  *
 *************************************************************************
DESCRIPTION: Returns Rate adaptation
                                                                                                   
INPUT:    woneIndex   

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRateAdaptation(UINT8 woneIndex, UINT8 *value)
{
	*value = wone_info[woneIndex].RateAdaptation;
}


/*************************************************************************
 *			wdrv_dbGet4xEnable							                                  *
 *************************************************************************
DESCRIPTION: Returns 4x status 
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGet4xEnable(BOOL *value)
{
	*value = enable4x;	
}

/*************************************************************************
 *	wdrv_dbSet4xEnable									                                  *
 *************************************************************************
DESCRIPTION: Sets 3x status
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSet4xEnable(BOOL value)
{
	enable4x = value;

	wdrv_dbSetCwMinWinSize(15);
}

/*************************************************************************
 *		wdrv_dbGetConcatEnable								                                  *
 *************************************************************************
DESCRIPTION: Returns concatenation status
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetConcatEnable(BOOL *value)
{
	*value = enableConcat;	
}

/*************************************************************************
 *wdrv_dbSetConcatEnable										                                  *
 *************************************************************************
DESCRIPTION: Sets concatenatiuon status
                                                                                                   
INPUT:  value     

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetConcatEnable(BOOL value)
{
	enableConcat = value;
	
	if (value == TRUE)
		wdrv_4xMngr_EnableConcatenation();
	else
		wdrv_4xMngr_DisableConcatenation(); 
}
/*************************************************************************
 *		wdrv_dbGetAckEmulEnable								                                  *
 *************************************************************************
DESCRIPTION:                  Returns ACK emulation status                                                                                  
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAckEmulEnable(BOOL *value)
{
	*value = enableAckEmul;	
}

/*************************************************************************
 *		wdrv_dbSetAckEmulEnable								                                  *
 *************************************************************************
DESCRIPTION: Set ack amulation status
                                                                                                   
INPUT:   value    

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetAckEmulEnable(BOOL value)
{
	enableAckEmul = value;
	
	if (value == TRUE)
		wdrv_4xMngr_EnableAckEmulation();
	else
		wdrv_4xMngr_DisableAckEmulation(); 
}

/*************************************************************************
 *				wdrv_dbGetCwMinMode						                                  *
 *************************************************************************
DESCRIPTION: Returns cwmin mode
                                                                                                   
INPUT:   woneIndex    

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCwMinMode(UINT8 woneIndex, UINT8 *value)
{
	*value = wone_info[woneIndex].CwMinMode;	
}

/*************************************************************************
 *	wdrv_dbGetCwMinWinSize									                                  *
 *************************************************************************
DESCRIPTION: Returns cwmin win size
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCwMinWinSize(UINT16 *value)
{
	*value = apCWMinWindow;
}

/*************************************************************************
 *			wdrv_dbSetCwMinWinSize							                                  *
 *************************************************************************
DESCRIPTION: Sets cwmin win size
                                                                                                   
INPUT:   value    

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCwMinWinSize(UINT16 value)
{
	apCWMinWindow = value;
	
	/* pass the updated CWmin to the HAL */
	whal_apiSet4xCwMin(apCWMinWindow);
}

/*************************************************************************
 *		wdrv_dbSetBssIsolationState								                                  *
 *************************************************************************
DESCRIPTION: Sets bss islation status
                                                                                                   
INPUT:  value     

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetBssIsolationState(UINT8 value)
{
	bssBridgeIsolationOn = !(value == BSS_TRAFFIC_EN);
}

/*************************************************************************
 *		wdrv_dbGetBssIsolationState								                                  *
 *************************************************************************
DESCRIPTION: Returns bss isolation status
                                                                                                   
INPUT:       

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBssIsolationState(BOOL *value)
{
	*value = bssBridgeIsolationOn;
}


/*************************************************************************
 *		 wdrv_dbGetTIInfoElement								                                  *
 *************************************************************************
DESCRIPTION: Returns TI IE 
                                                                                                   
INPUT:       

OUTPUT:   value,   len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetTIInfoElement(char *value, UINT8 *len)
{
	*len = TIcap.hdr.eleLen;
	PAL_memCopy(value, TIcap.cap, TIcap.hdr.eleLen);	
}
/*************************************************************************
 *	wdrv_dbGet4xMixMode									                                  *
 *************************************************************************
DESCRIPTION: Returns 4x mode
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void	wdrv_dbGet4xMixMode(BOOL *value)
{
   *value = enable4xMixMode; 
}

/*************************************************************************
 *				wdrv_dbSet4xMixMode						                                  *
 *************************************************************************
DESCRIPTION: Sets 4x mode
                                                                                                   
INPUT:     value  

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSet4xMixMode(BOOL value)
{
   enable4xMixMode = value;
}

/*************************************************************************
 *					wdrv_dbGetWlanServiceState					                                  *
 *************************************************************************
DESCRIPTION: Returns Wlan service state
                                                                                                   
INPUT:       

OUTPUT:      state

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWlanServiceState ( BOOL *state )
{
	*state = WlanServiceState;
}

/*************************************************************************
 *			wdrv_dbSetWlanServiceState							                                  *
 *************************************************************************
DESCRIPTION: Sets wlan service state
                                                                                                   
INPUT:      state 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWlanServiceState ( BOOL state )
{
   /*-------------------------------------------------------------*/
   /* Only in case we are going from OFF to ON,                   */
   /* let the wdrv_dbGetEnRestart retreive the saved transitions. */
    if (!WlanServiceState && state)
	{
		RestoreSavedEnRestart = TRUE;
    } else
	{
		RestoreSavedEnRestart = FALSE;
	}
   /*-------------------------------------------------------------*/
	WlanServiceState = state;
}

/*************************************************************************
 *			wdrv_dbGetEnRestart							                                  *
 *************************************************************************
DESCRIPTION: Returns En Restart
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetEnRestart(UINT8 *value)
{
   	if (RestoreSavedEnRestart)
	{
	   /*------------------------------------*/
	   /* Return saved transitions only once */
		*value = SavedEnRestart;
		RestoreSavedEnRestart = FALSE;
	   /*------------------------------------*/
    } else
	{
        *value = EnRestart; 
    }
}

/*************************************************************************
 *	wdrv_dbGetEnInterfWA									                                  *
 *************************************************************************
DESCRIPTION: Returns EnInterfWA
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void	wdrv_dbGetEnInterfWA(UINT8 *value)
{
   *value = EnInterfWA; 
}

/*************************************************************************
 *			wdrv_dbSetEnRestart							                                  *
 *************************************************************************
DESCRIPTION: Sets En Restart
                                                                                                   
INPUT:   value,   updateNvram  

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetEnRestart(UINT8 value, BOOL updateNvram)
{
   if (WlanServiceState)
   {
      EnRestart = value;
    } else
   {
	    SavedEnRestart = value;
   }
}
/*************************************************************************
 *						wdrv_dbGetWAoptions(				                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      rxFreeMem bcnNotSent txStuck consFcsErr aci nb

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWAoptions(UINT8 *rxFreeMem,UINT8 *bcnNotSent,UINT8 *txStuck,
						  UINT16 *consFcsErr, UINT8 *aci, UINT8 *nb)
{
   *rxFreeMem  = RxFreeMemOptionWA; 
   *bcnNotSent = BeaconNotSentOptionWA; 
   *txStuck    = TxStuckOptionWA; 
   *consFcsErr = ConsFcsErrOptionWA; 
   *aci		   = aciOptionWA;
   *nb		   = nbOptionWA;
}
/*************************************************************************
 *							wdrv_dbSetWAoptions(			                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:     rxFreeMem   bcnNotSent txStuck consFcsErr aci nb

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWAoptions(UINT8 rxFreeMem,UINT8 bcnNotSent,UINT8 txStuck,
						  UINT16 consFcsErr, UINT8 aci, UINT8 nb)
{
}
/*************************************************************************
 *					wdrv_dbGetEnergyDetect					                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetEnergyDetect(UINT8 *value)
{
   *value = EnEnergyDetect; 
}
/*************************************************************************
 *				wdrv_dbSetEnergyDetect						                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetEnergyDetect(UINT8 value)
{
   EnEnergyDetect = value;
}

/*************************************************************************
 *					wdrv_dbGetEepromLess					                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetEepromLess(UINT8 *value)
{
   *value = EEPromLessSupport; 
}
/*************************************************************************
 *				wdrv_dbSetEepromLess						                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetEepromLess(UINT8 value)
{
   EEPromLessSupport = value;
}

/*************************************************************************
 *						wdrv_dbGetRespRateMode				                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRespRateMode(UINT8 *value)
{
	*value = apRespRateMode;
}

/*************************************************************************
 *								wdrv_dbGetRadioFrequency		                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioFrequency(UINT8 *value)
{
    *value = RadioFrequency;
}
/*************************************************************************
 *										                                  *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioFrequency(UINT8 value)
{
    RadioFrequency = value;
}


/*************************************************************************
 *                        wdrv_dbRatesSupport					         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station rates or not                 
                                                                                                   
INPUT:       The station rates resides in one or two information elements

OUTPUT:      

RETURN:      OK : Rates supportted.
             NOK: Rates not supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbRatesSupport(UINT8 woneIndex, dot11_supRates_T *ratesP, dot11_extSupRates_T *extRatesP)
{
	UINT8 i, j;
	BOOL	rateFound;
	
	/* now, for every basic rate of the AP, find it in the station rates */
	for (i = 0; i < wone_info[woneIndex].allApSupRatesLen; i++)
	{
		if (isBasic(wone_info[woneIndex].allApSupRates[i]) == TRUE)
		{
		   rateFound = FALSE;
		
		   for (j = 0; j < ratesP->hdr.eleLen; j++)
		   {
			   if (wone_info[woneIndex].allApSupRates[i] == (ratesP->rates[j] | 0x80))
			   {
				   rateFound = TRUE;
				   break;
			   }
		   }

		   /* If rate was not found and the extended supported rates exist -> search it */
		   if ((extRatesP != NULL) && (rateFound == FALSE))
		   {
			   for (j = 0; j < extRatesP->hdr.eleLen; j++)
			   {
				   if (wone_info[woneIndex].allApSupRates[i] == (extRatesP->rates[j] | 0x80))
				   {
					   rateFound = TRUE;
					   break;
				   }
			   }
		   }

		   if (rateFound == FALSE)
			   return FALSE;
		}
	}
	return TRUE;
}
/*************************************************************************
 *					isBasic					                                  *
*************************************************************************
DESCRIPTION:return true if rate is basic rate, i.e., the msb is set 
                                                                                                   
INPUT:     rate  

OUTPUT:      

RETURN:     result 
                                                                                                                                                          
************************************************************************/
static BOOL isBasic(UINT8 rate)
{
	if (rate & 0x80)
		return TRUE;
	else
		return FALSE;
}


/*************************************************************************
 *                        wdrv_dbAuthAlgoSupported					     *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the authentication algorithm                 
                                                                                                   
INPUT:       The authentication algorithm as received in the authentication frame.

OUTPUT:      

RETURN:      OK : Algorithm supportted.
             NOK: Algorithm not supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbAuthAlgoSupported(UINT16 algoNumber)
{
	int i;
	UINT16	algo;
	BOOL	enable;

	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		wdrv_dbGetAuthAlgo(i, &algo);
		wdrv_dbGetAuthAlgoEnable(i, &enable);
		
		if ((algo == algoNumber + 1) && (enable == TRUE))
			return TRUE;
	}
	return FALSE;
}

/*************************************************************************
 *							 wdrv_dbApSupport			                                  *
*************************************************************************
DESCRIPTION: Returns TRUE if input MAC address is 
			 equal to the AP's MAC address, otherwise, returns FALSE. 
                                                                                                   
INPUT: mac      

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbApSupport(char *mac)
{
	BOOL	compare;

	compare = (!PAL_memCmp(mac, APMAC, MAC_ADDR_LEN));

	return compare;
}


/***********************************************************************************
 The following functions implement the MIB. Each group is correspond to a mib table 
************************************************************************************/	
/*************************************************************************
 * wdrv_dbGetStationConfigAuthRspTimeou                   *
*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigAuthRspTimeout(UINT32 *value)
{
	*value = 2000;
}

/*************************************************************************
*		                wdrv_dbRemoveRate                                *
*************************************************************************
DESCRIPTION:   Removes a received rate from the rates list.
                                                                                                   
INPUT:       woneIndex
             rate - the rate to remove

OUTPUT:      

RETURN:   BOOL - TRUE if rate was removed, FALSE otherwise   
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbRemoveRate(UINT8 woneIndex, UINT8 rate)
{
    int temp = wone_info[woneIndex].apRates.hdr.eleLen;
    int i;
    BOOL rateFound = FALSE;

    for(i=0; i < temp; i++)
    {
        if(wone_info[woneIndex].apRates.rates[i] == rate)
        {
            wone_info[woneIndex].apRates.rates[i] = 0;
            wone_info[woneIndex].apRates.hdr.eleLen--;
            rateFound = TRUE;
        }
    }

    temp = wone_info[woneIndex].apOFDMRates.hdr.eleLen;
    for(i=0; i < temp; i++)
    {
        if(wone_info[woneIndex].apOFDMRates.rates[i] == rate)
        {
            wone_info[woneIndex].apOFDMRates.rates[i] = 0;
            wone_info[woneIndex].apOFDMRates.hdr.eleLen--;
            rateFound = TRUE;
        }
    }
    
    temp = wone_info[woneIndex].allApSupRatesLen;
    for(i=0; i < temp; i++)
    {
        if(wone_info[woneIndex].allApSupRates[i] == rate)
        {
            wone_info[woneIndex].allApSupRates[i] = 0;
            wone_info[woneIndex].allApSupRatesLen--;
            rateFound = TRUE;
        }
    }

    return rateFound;
}

/*************************************************************************
 *									wdrv_dbGetStationConfigOperationalRateSet	                                  *
*************************************************************************
DESCRIPTION:  802.11b rates 
                                                                                                   
INPUT:       woneIndex

OUTPUT:      value len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].apRates.hdr.eleLen;       
   	PAL_memCopy(value, wone_info[woneIndex].apRates.rates, wone_info[woneIndex].apRates.hdr.eleLen);	
}
/*************************************************************************
 *										    wdrv_dbGetStationConfigOperationalOFDMRateSet                              *
*************************************************************************
DESCRIPTION:  802.11g OFDM rates 
                                                                                                   
INPUT:       woneIndex

OUTPUT:    value   len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalOFDMRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].apOFDMRates.hdr.eleLen;
	PAL_memCopy(value, wone_info[woneIndex].apOFDMRates.rates, wone_info[woneIndex].apOFDMRates.hdr.eleLen);	
}
/*************************************************************************
 *									wdrv_dbGetStationConfigOperationalAllRateSet	                                  *
*************************************************************************
DESCRIPTION:  802.11g all rates 
                                                                                                   
INPUT:      woneIndex 

OUTPUT:      value len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalAllRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].allApSupRatesLen;
	PAL_memCopy(value, wone_info[woneIndex].allApSupRates, wone_info[woneIndex].allApSupRatesLen);	
}
/*************************************************************************
 *										     wdrv_dbGetStationConfigBeaconPeriod                             *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigBeaconPeriod(UINT16 *value)
{
	*value = apBeaconPeriod;
}
/*************************************************************************
 *						wdrv_dbGetStationConfigDtimPeriod				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDtimPeriod(UINT8 *value)
{
	*value = apDtimPeriod;
}
/*************************************************************************
 *							wdrv_dbGetStationConfigDisassociateReason			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDisassociateReason(UINT16 *value)
{
	*value = dot11StationConfigTable.disassociateReason;
}
/*************************************************************************
 *										 wdrv_dbGetStationConfigDisassociateStation                                 *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDisassociateStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.disassociateStation, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *							wdrv_dbGetStationConfigDeauthenticateReason			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateReason(UINT16 *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*value = dot11StationConfigTable.deauthenticateReason;
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *							wdrv_dbGetStationConfigDeauthenticateStation			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.deauthenticateStation, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *										               wdrv_dbGetStationConfigDeauthenticateFailStatus                   *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateFailStatus(UINT16 *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*value = dot11StationConfigTable.deauthenticateFailStatus;
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *								wdrv_dbGetStationConfigeauthenticateFailStation		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigeauthenticateFailStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.deauthenticateFailStation, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *							wdrv_dbSetStationConfigDisassociateReason			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDisassociateReason(UINT16 value)
{
	dot11StationConfigTable.disassociateReason = value;
}

/*************************************************************************
 *										  wdrv_dbSetStationConfigDisassociateStation                                *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDisassociateStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */ 
	PAL_memCopy(dot11StationConfigTable.disassociateStation, value, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *										wdrv_dbSetStationConfigDeauthenticateReason                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateReason(UINT16 value)
{
	dot11StationConfigTable.deauthenticateReason = value;
}

/*************************************************************************
 *									wdrv_dbSetStationConfigDeauthenticateStation	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(dot11StationConfigTable.deauthenticateStation, value, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *						wdrv_dbSetStationConfigDeauthenticateFailStatus				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateFailStatus(UINT16 value)
{
	dot11StationConfigTable.deauthenticateFailStatus = value;
}

/*************************************************************************
 *					wdrv_dbSetStationConfigDeauthenticateFailStation					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateFailStation(char *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(dot11StationConfigTable.deauthenticateFailStation, value, MAC_ADDR_LEN);
  	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/**********************************************************************
	Authentication Algorithms table
************************************************************************/

/*************************************************************************
DESCRIPTION:  wdrv_dbGetAuthAlgo
                                                                                                   
INPUT:    index   

OUTPUT: value     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAuthAlgo(UINT32 index, UINT16 *value)
{
	if (index == authOpenSystemIndex)
		*value = MIB_OPEN_SYSTEM;
	else if (index == authSharedKeyIndex)
		*value = MIB_SHARED_KEY;
}

/*************************************************************************
 *										wdrv_dbGetAuthAlgoEnable                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:     index  

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAuthAlgoEnable(UINT32 index, BOOL *value)
{
	if (index == authOpenSystemIndex)
		*value = authOpenSystemEnable;
	else if (index == authSharedKeyIndex)
		*value = authSharedKeyEnable;
}


/*************************************************************************
 *										    wdrv_dbGetPrivacyWEPExcludedCount                              *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyWEPExcludedCount(UINT32 *value)
{
	* value = wdrv_bssbridgeGetExcludedCount();
}

/*************************************
	MultiDomainCapability table
**************************************/

/*************************************************************************
 *					wdrv_dbGetChannel					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetChannel(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].channel;
}

/*************************************************************************
 *					wdrv_dbSetChannel					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetChannel(UINT8 woneIndex ,UINT8 value)
{
	wone_info[woneIndex].channel = value;
}

/*************************************************************************
 *					wdrv_dbGetBand					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:      woneIndex 

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBand(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].band;
}

/*************************************************************************
 *								wdrv_dbGetPrimaryAcxQindex		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:    woneIndex   

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrimaryAcxQindex(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].primaryAcxQindex;
}

/*************************************************************************
 *							wdrv_dbSetPrimaryAcxQindex(			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPrimaryAcxQindex(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].primaryAcxQindex = value;
}



/*************************************************************************
 *										wdrv_dbGetPsBroadcastQindex                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPsBroadcastQindex(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].psBroadcastQindex;
}

/*************************************************************************
 *					wdrv_dbSetPsBroadcastQindex					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPsBroadcastQindex(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].psBroadcastQindex = value;
}


/*************************************************************************
 *							wdrv_dbGetUsageTime			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:    woneIndex   

OUTPUT:      a

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetUsageTime(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].UsageTime;
}

/*************************************************************************
 *			wdrv_dbUpdateUsageTime							                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       woneIndex value

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbUpdateUsageTime(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].UsageTime = value;
}




/*************************************
	Operation table
**************************************/

/*************************************************************************
 *						wdrv_dbGetOperationMacAddress				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:   value   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMacAddress(char *value)
{
	PAL_memCopy(value, APMAC, MAC_ADDR_LEN);	
}

/*************************************************************************
 *								wdrv_dbGetOperationRTSThreshold		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationRTSThreshold(UINT16 *value)
{
	*value = apRtsThreshold;	
}

/*************************************************************************
 *					wdrv_dbGetOperationShortRetryLimit					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationShortRetryLimit(UINT8 *value)
{
	*value = apShortRetryLimit;	
}

/*************************************************************************
 *										                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationLongRetryLimit(UINT8 *value)
{
	*value = apLongRetryLimit;	
}

/*************************************************************************
 *						wdrv_dbGetOperationFragmentationThreshold				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationFragmentationThreshold(UINT16 *value)
{
	*value = apFragThreshold;	
}

/*************************************************************************
 *									wdrv_dbGetOperationMaxTransmitMSDULifetime	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMaxTransmitMSDULifetime(UINT32 *value)
{	
	*value = apMaxTxLifetime;	
}

/*************************************************************************
 *							wdrv_dbGetOperationMaxReceiveLifetime			                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMaxReceiveLifetime(UINT32 *value)
{
	*value = apMaxRxLifetime;	
}


/*************************************
	Counters table
**************************************/

/*************************************************************************
 *									wdrv_dbGetCountersTransmittedFragmentCount	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersTransmittedFragmentCount(UINT32 *value)
{
	whal_apiGetTxFragmentCount(0,value);
}

/*************************************************************************
 *								wdrv_dbGetCountersMulticastTransmittedFrameCount		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMulticastTransmittedFrameCount(UINT32 *value)
{
	wdrv_txGetTxMulticastCount(value);
}

/*************************************************************************
 *				wdrv_dbGetCountersFailedCount						                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFailedCount(UINT32 *value)
{
	whal_apiGetFailedCount(0,value);
}

/*************************************************************************
 *									wdrv_dbGetCountersRetryCount	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:    value  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRetryCount(UINT32 *value)
{
	whal_apiGetRetryCount(0,value);	
}

/*************************************************************************
 *						wdrv_dbGetCountersMultipleRetryCount				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMultipleRetryCount(UINT32 *value)
{
	whal_apiGetMultipleRetryCount(0,value);	
}

/*************************************************************************
 *				wdrv_dbGetCountersFrameDuplicateCoun						                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:  value    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFrameDuplicateCount(UINT32 *value)
{
	wdrv_rxGetCountersFrameDuplicateCount(value);
}

/*************************************************************************
 *									wdrv_dbGetCountersRTSSuccessCount	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRTSSuccessCount(UINT32 *value)
{
	whal_apiGetRtsSuccCount(0,value);
}

/*************************************************************************
 *								wdrv_dbGetCountersRTSFailureCount		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRTSFailureCount(UINT32 *value)
{
	whal_apiGetRtsFailCount(0,value);
}

/*************************************************************************
 *								 wdrv_dbGetCountersACKFailureCount		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersACKFailureCount(UINT32 *value)
{
	whal_apiGetAckFailCount(0,value);	
}

/*************************************************************************
 *									wdrv_dbGetCountersReceivedFragmentCount	                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersReceivedFragmentCount(UINT32 *value)
{
	wdrv_rxGetReceivedFragmentCount(value);
}

/*************************************************************************
 *						wdrv_dbGetCountersMulticastReceivedFrameCount				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMulticastReceivedFrameCount(UINT32 *value)
{
	wdrv_rxGetMulticastReceivedFrameCount(value);
}

/*************************************************************************
 *						wdrv_dbGetCountersFCSErrorCount				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFCSErrorCount(UINT32 *value)
{
	whal_apiGetFcsError(value);	
}

/*************************************************************************
 *					wdrv_dbGetCountersTransmittedFrameCount					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     value 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersTransmittedFrameCount(UINT32 *value)
{
	whal_apiGetTxFrameCount(0,value);	
}

/*************************************************************************
 *				wdrv_dbSetPBCCdynamicStatus						                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:    status   

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPBCCdynamicStatus(BOOL status)
{
	DynamicAlgInfo.pbccValid = status;
}

/*************************************************************************
 *				wdrv_dbGetPBCCdynamicStatus						                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     status 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPBCCdynamicStatus(BOOL *status)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*status = DynamicAlgInfo.pbccValid;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *					wdrv_dbSetERPdynamicStatus					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:      status 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetERPdynamicStatus(BOOL status)
{
	DynamicAlgInfo.erpValid = status;
}

/*************************************************************************
 *					wdrv_dbGetERPdynamicStatus					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:     status 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPdynamicStatus(BOOL *status)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*status = DynamicAlgInfo.erpValid;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *	wdrv_dbSetNotPBCCTimestamp									                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:      timestamp 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetNotPBCCTimestamp(UINT32 timestamp)
{
	DynamicAlgInfo.pbccTimeStamp = timestamp;
}

/*************************************************************************
 *			 wdrv_dbGetNotPBCCTimestamp							                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:  timestamp    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNotPBCCTimestamp(UINT32 *timestamp)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.pbccTimeStamp;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbSetRxTxBrate								                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       timestamp count

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRxTxBrate(UINT32 timestamp, UINT32 count)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	DynamicAlgInfo.RxTxBrateTimestamp = timestamp;
	if (count != 0)
	{
		DynamicAlgInfo.RxTxBrateCount += count;
    } else
	{
		DynamicAlgInfo.RxTxBrateCount = 0;
	}
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *						wdrv_dbGetRxTxBrate				                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      timestamp count

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRxTxBrate(UINT32 *timestamp, UINT32 *count)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.RxTxBrateTimestamp;
	*count = DynamicAlgInfo.RxTxBrateCount;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *			wdrv_dbSetNotErpTimestamp							                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:   timestamp    

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetNotErpTimestamp(UINT32 timestamp)
{
	DynamicAlgInfo.erpTimeStamp = timestamp;
}

/*************************************************************************
 *			wdrv_dbGetNotErpTimestamp							                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:   timestamp   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNotErpTimestamp(UINT32 *timestamp)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.erpTimeStamp;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbGetNumSupWone						                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:   TotalWones   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumSupWone(UINT8 * TotalWones)
{
	*TotalWones = numSupWone;
}

/*************************************************************************
 *								wdrv_dbGetNumSupWoneQueues		                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumSupWoneQueues	(UINT8 woneIndex, UINT8 * WoneTxQueues )
{
	*WoneTxQueues = wone_info[woneIndex].queue_table.numOfQueue;
}

/*************************************************************************
 *					wdrv_dbGetAcxQindex					                                  *
*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAcxQindex	(	UINT8 		woneIndex,
								UINT8 		ac,
								UINT8 *		pType,
                                UINT8 *		pPriority,
                                UINT8 *     pActive	,
                                UINT32 *     pGlobalQid) 
{      
	*pType = (UINT8)(wone_info[woneIndex].queue_table.ac_queue[ac].type);
	
	*pPriority = (UINT8)(wone_info[woneIndex].queue_table.ac_queue[ac].Priority);
	
    *pActive = (UINT8)(wone_info[woneIndex].queue_table.ac_queue[ac].active);
    
    *pGlobalQid = (UINT8)(wone_info[woneIndex].queue_table.ac_queue[ac].globalQid);
}

/*************************************************************************
 *						 wdrv_dbGetAcxGlobalQindex				                                  *
*************************************************************************
DESCRIPTION:  return the global queue index
                                                                                                   
INPUT:       wone index
             Access category     

OUTPUT:      global queue index

RETURN:      OK if queu is active, otherwise NOK 
                                                                                                                                                          
************************************************************************/
int wdrv_dbGetAcxGlobalQindex(UINT8 woneIndex, wdrv_QoSClassifierAC_t ac, int * globalQid) 
{
    int rc;
    if (wone_info[woneIndex].queue_table.ac_queue[ac].active==TRUE)
    {
	*globalQid = wone_info[woneIndex].queue_table.ac_queue[ac].globalQid;
         rc=OK;
    } else
    {
        *globalQid = 0xff;
         rc=NOK;
    }
    return rc;
}



/*************************************************************************
 *							wdrv_dbGetmgmtAccessCategory			                                  *
 *************************************************************************
DESCRIPTION: return the AC for management packet.
                                                                                                   
INPUT:   woneIndex  

OUTPUT:  Voice ac if QOS enable otherwise best effort   
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetmgmtAccessCategory(UINT8 woneIndex ,UINT32 *value)
{
	*value = wone_info[woneIndex].queue_table.mgmtAccessCategory;	
}


/*************************************************************************
 *							wdrv_dbGetMgmtAccessCategoryTx			                                  *
 *************************************************************************
DESCRIPTION: return return the AC for management packet.
                                                                                                   
INPUT:   woneIndex  

OUTPUT:  Voice ac if QOS enable otherwise best effort   
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMgmtAccessCategoryTx(UINT8 woneIndex ,UINT32 *value)
{
	if (wmeACParams.enabled)
	{
		*value = wone_info[woneIndex].queue_table.mgmtAccessCategory;
	}
	else
	{
		*value = WDRV_QOS_CLASSIFIER_AC_VO;
	}
}



/************************************************************************************
 *                        wdrv_dbGetEdcfParams                                      *
 ************************************************************************************
  DESCRIPTION:  Provide the requested tx queue EDCF parameters. 
				Return FALSE if not required (vHCF disabled or PS-Broadcast queue).
 ************************************************************************************/
BOOL wdrv_dbGetEdcfParams(UINT8 woneIndex, UINT8 localQueueId, edcf_info_t * edcfInfoPtr) {
	if (wone_info[woneIndex].queue_table.ac_queue[localQueueId].edcfInfoRequired)	
        {
	    PAL_memCopy( (void *)edcfInfoPtr,
								 (void *)&(wone_info[woneIndex].queue_table.ac_queue[localQueueId].edcfInfo),
				sizeof(edcf_info_t) );
		return TRUE;
    } else
		return FALSE;
}

/************************************************************************************
 *                        wdrv_dbGetQueueHighThresh                                 *
 ************************************************************************************
  DESCRIPTION:  Provide the requested tx queue buffers usage limit (in 5% units). 
				Return FALSE if not required (vHCF disabled or not the low priority queue).
 ************************************************************************************
INPUT:  woneIndex      localQueueId

OUTPUT:    thresholdPtr  

RETURN:      STATUS
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbGetQueueHighThresh(UINT8 woneIndex, UINT8 localQueueId, UINT32 * thresholdPtr) {
	if (wone_info[woneIndex].queue_table.ac_queue[localQueueId].high_threshold) {
		*thresholdPtr = wone_info[woneIndex].queue_table.ac_queue[localQueueId].high_threshold;
		return TRUE;
	} else	{
		return FALSE;
	}
}


/************************************************************************************
 *                        wdrv_dbGetQueueThresh                                     *
 ************************************************************************************
  DESCRIPTION:  Provide the requested tx queue buffers usage limit (in 5% units). 
				Return FALSE if not required (vHCF disabled or not the low priority queue).
 ************************************************************************************
INPUT:  woneIndex      localQueueId

OUTPUT:    thresholdPtr  

RETURN:      STATUS
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbGetQueueThresh(UINT8 woneIndex, UINT8 localQueueId, UINT32 * thresholdPtr) {
	if (wone_info[woneIndex].queue_table.ac_queue[localQueueId].threshold) {
		*thresholdPtr = wone_info[woneIndex].queue_table.ac_queue[localQueueId].threshold;
		return TRUE;
	} else	{
		return FALSE;
	}
}

/*************************************************************************
 *				wdrv_dbGetUsageTimeMode						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetUsageTimeMode(UINT8 *value)
{	
	*value = usageTimeMode;
}

/*************************************************************************
 *				wdrv_dbGetRadioParams						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     ptr 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioParams( RadioParam_t *ptr)
{
   PAL_memCopy( ptr, &dbRadioParams, sizeof(dbRadioParams));
}/* END wdrv_dbGetRadioParams() */

/*************************************************************************
 *		 wdrv_dbSetRadioParams								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       ptr parameterType updateNvram

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioParams( RadioParam_t *ptr, UINT32 parameterType, BOOL updateNvram)
{
    switch (parameterType)
   {
   case RADIO_PARAM_OVERRIDE_BMASK:
      dbRadioParams.overrideBitmask = ptr->overrideBitmask;
      break;
   case RADIO_PARAM_POWER_TABLE:
      dbRadioParams.powerTable_paramType = ptr->powerTable_paramType;
      PAL_memCopy( dbRadioParams.powerTable_param,
         ptr->powerTable_param,
         sizeof( dbRadioParams.powerTable_param));
      break;
   default:
      break;
   }

}/* END wdrv_dbSetRadioParams() */

/*************************************************************************
 *						wdrv_dbGetCountersWEPUndecryptableCount				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      val

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersWEPUndecryptableCount(UINT32 *val)
{
	*val = whal_ParamsGetWepParams()->dot11WEPUndecryptableCount;
}

/*************************************************************************
 *					wdrv_dbGetPrivacyWEPICVErrorCount					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      val

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyWEPICVErrorCount(UINT32 *val)
{
	*val = whal_ParamsGetWepParams()->dot11WEPICVErrorCount;
}

/*************************************************************************
 *						wdrv_dbGetPrivacyTKIPMicFailure				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:    val  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyTKIPMicFailure(UINT32 *val)
{
	*val = wdrv_bssbridgeGetTKIPMicFailure();
}


/*************************************************************************
 *						wdrv_dbGetRadarDetectMode				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     val 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadarDetectMode(UINT8 *val)
{
	*val = RadarDetectMode;
}
/*************************************************************************
 *					wdrv_dbGetBurstLimit					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     val 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBurstLimit(UINT16 *val)
{
	*val = txopLimit;
}

/*************************************************************************
 *				wdrv_dbGetApBridgeMode						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:   ApBridgeMode   
                                                                                                                                                          
************************************************************************/
UINT8 wdrv_dbGetApBridgeMode(void)
{
	return ApBridgeMode;
}

/*************************************************************************
 *			wdrv_dbGetAciStateTimeout							                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:   timeout   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAciStateTimeout( UINT32 *timeout)
{
	*timeout = aciStateTimeout;
}

/*************************************************************************
 *				wdrv_dbGetNbStateTimeout						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:   timeout   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNbStateTimeout( UINT32 *timeout)
{
	*timeout = nbStateTimeout;
}

/*************************************************************************
 *					wdrv_dbGetAciRateThreshold					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      thresh

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAciRateThreshold( UINT32 *thresh)
{
	*thresh = aciRateThreshold;
}

/*************************************************************************
 *						wdrv_dbGetAciEventsThreshold				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     thresh 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAciEventsThreshold( UINT32 *thresh)
{
	*thresh = aciEventThreshold;
}


/*************************************************************************
 *					wdrv_dbGetPacketDetectHighThresh					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     pdHighThresh 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPacketDetectHighThresh(UINT16 *pdHighThresh)
{
	*pdHighThresh = pdUnderAciHighThrsh;
}

/*************************************************************************
 *					wdrv_dbGetPacketDetectLowThresh					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      pdLowThresh

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPacketDetectLowThresh(UINT16 *pdLowThresh)
{
	*pdLowThresh = pdUnderAciLowThrsh;
}

/*************************************************************************
 *		wdrv_dbGetMediumBusyThresh								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      thresh

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMediumBusyThresh(UINT32 *thresh)
{
	*thresh = mediumBusyThresh;
}

/*************************************************************************
 *		wdrv_dbGetWdsEnable								                 *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:  

RETURN: TRUE if WDS enabled, otherwise - FALSE     
                                                                                                                                                          
************************************************************************/
UINT8 wdrv_dbGetWdsEnable(void)
{
	return wdsEnabled;
}

/*************************************************************************
 *		wdrv_dbSetWdsEnable								                 *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT: WDS state       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWdsEnable(UINT8 enable)
{
    wdsEnabled = enable;
    /* Disable BSS. WDS manager will enable BSS according to his state machine */
    wdrv_wdsBssServiceStateSet(FALSE);
}

/*************************************************************************
 *		 wdrv_dbGetWMEEnabled								            *
*************************************************************************
DESCRIPTION: Returns WME support status
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbGetWMEEnabled(void) {
	return wmeACParams.enabled;
}

/*************************************************************************
 *				wdrv_dbSetWMEEnabled						                                  *
*************************************************************************
DESCRIPTION: Sets WME support status
                                                                                       
INPUT:   enabled,    mode 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWMEEnabled(BOOL enabled, wdrv_QoSClassifierAC_t mode) {
	wmeACParams.enabled = enabled;
	wmeACParams.classifierMode = mode;
}

/*************************************************************************
 *		wdrv_dbGetTxOPLimit								                                  *
*************************************************************************
DESCRIPTION:  Returns TX Op limit for queue
                                                                                       
INPUT:   wOneIndex,  TxQid   

OUTPUT:      

RETURN:   tx op limit   
                                                                                                                                                          
************************************************************************/
UINT16 wdrv_dbGetTxOPLimit(UINT8 wOneIndex, TxQueueId_T 	TxQid) {
	int i;

	for (i=0; i<wone_info[wOneIndex].queue_table.numOfQueue; ++i)
		if (wone_info[wOneIndex].queue_table.ac_queue[i].globalQid == TxQid) 
			return wone_info[wOneIndex].queue_table.ac_queue[i].edcfInfo.txopLimit;

	printf("%s: Could not find tx op limit in DB for queue = %d\n", __FUNCTION__, TxQid);
	return 0;
}

/*************************************************************************
 *		wdrv_dbGetWMEParams								                                  *
*************************************************************************
DESCRIPTION:  Returns WME parameters
                                                                                       
INPUT:       

OUTPUT:      

RETURN: WME parameters     
                                                                                                                                                          
************************************************************************/
wdrv_QoSACParams_t wdrv_dbGetWMEParams(void) {
	return wmeACParams;
}

/*************************************************************************
 *                        wdrv_dbGetWMElifetime ()                       *
 *************************************************************************
DESCRIPTION: Retreive packet's lifetime according to the Access Category.                                    
                                                                                                   
INPUT:       Access Category identifier.
OUTPUT:      lifetime                                           
RETURN:      OK if succeeded.
            NOK if the parameters are incorrect.
                                                                                                   
************************************************************************/
int wdrv_dbGetWMEpktLifetime( wdrv_QoSClassifierAC_t ac, UINT32 * lifetime )
{

    if ((ac >= WDRV_QOS_CLASSIFIER_AC_BE)      &&
        (ac <  WDRV_QOS_CLASSIFIER_BROADCAST)  &&
        ( NULL != lifetime ))
    {
        if (wmeACParams.enabled)
        {
            *lifetime = wmeACParams.ap_edcfParams[ac].dot11EDCATableMSDULifetime;
        } else
        {
            *lifetime = WDRV_QOS_DEFAULT_MSDU_LIFETIME;                          
        }
        return OK;
    }
    return NOK;
}

/*************************************************************************
 *		wdrv_dbGetHwInterfParams								                                  *
*************************************************************************
DESCRIPTION:  Returns HW ACI/NB interference handling configuration
                                                                                       
INPUT:       

OUTPUT: HW interference parameters          

RETURN: 
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetHwInterfParams(HwInterfParams_T *params) 
{
	PAL_memCopy(params, &hwInterfParams, sizeof(HwInterfParams_T));
}
