/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*                                                                          */
/*      MODULE:	mgmtAccess.c                                                */
/*      PURPOSE: MAC access list functionality                              */
/*                                                                          */
/****************************************************************************/

#include "wlan_pal.h"
#include "QoS/QoSClassifier.h"
#include "mgmtAccess.h"
#include "app_driver_api.h"



static UINT8 mgmt_access_enabled = 0;
static char mgmt_access_list[MGMT_ACCESS_LIST_SIZE][MAC_ADDR_LEN];
static mgmt_access_list_type_t mgmt_access_list_type = MGMT_ACCESS_BLACK_LIST;
static int NumOfStationsInList = 0;

/* MW CODE BEGIN */
/* 2006.05.11 Banned List*/
#define MAX_BANNED      8
typedef struct banned_list_s {
	macAddress_T mac;
	UINT32 timeStamp;
} banned_list_t;

static banned_list_t banned_list[MAX_BANNED];
static int banlist_index=0;
/* 2006.05.11 Print Banned List */
#define BANLIST_TIMEOUT	60000
int wdrv_mgmtPrintBanList(char *buf) {
	int i, time;

	if(!mgmt_access_enabled)
		return -1;

	buf[0]='\0';
	time=jiffies;
	for (i = 0; i < MAX_BANNED; i++)
	{
		/* 2006.05.13 fix timeStamp zero bug */
		if((banned_list[i].timeStamp>0)&&((time-banned_list[i].timeStamp)<BANLIST_TIMEOUT))
			sprintf(buf,\
				"%s%02X:%02X:%02X:%02X:%02X:%02X",\
				buf, banned_list[i].mac.addr[0], \
				banned_list[i].mac.addr[1], \
				banned_list[i].mac.addr[2],\
		       		banned_list[i].mac.addr[3],\
			       	banned_list[i].mac.addr[4],\
		       		banned_list[i].mac.addr[5]);
	}
	return 0; 
}
/* MW CODE END */

/******************************************************************************
*                           mgmtAccessInit()                				  *
*******************************************************************************
DESCRIPTION:	Initialize access list.
                                                                         
INPUTS:			pInitAccessList - access list init structure.

OUTPUT:			None.

RETURNS:		OK                                                    
******************************************************************************/
STATUS mgmtAccessInit(mgmt_accesslist_init_struct *pInitAccessList)
{
	
    UINT8 type;
    int i=0;
    
	/* 2006.05.11 initial banned list */
        memset(banned_list, 0, MAX_BANNED*sizeof(banned_list_t));
	mgmt_access_enabled = pInitAccessList->mgmt_access_enabled;
	
	type = pInitAccessList->mgmt_access_list_type;
  	mgmt_access_list_type = (type == 0) ? MGMT_ACCESS_WHITE_LIST : MGMT_ACCESS_BLACK_LIST;
    
	memcpy((char*)mgmt_access_list,
		   (char*)(pInitAccessList->mgmt_access_list),
			MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN);

	wdrv_accessSetAuthCallback(mgmtAccessCallback);
	wdrv_accessSetAssocCallback(mgmtAccessCallback);

    for (i=0; i<MGMT_ACCESS_LIST_SIZE; i++) 
    {
        if( MACCMPZ( mgmt_access_list[i]))
            break;
        NumOfStationsInList++;
    }

	return OK;
}


/******************************************************************************
*                           mgmtAccessGetList()	              				  *
*******************************************************************************
DESCRIPTION:	Copy management access list to buffer.
                                                                         
INPUTS:			list - pointer to buffer, 
                pLength - pointer to buffer length.

OUTPUT:			pLength - copied access list length.

RETURNS:		OK if success, NOK -if buffer is small then access list.                                                    
******************************************************************************/
STATUS mgmtAccessGetList(char *list, int *pLength) 
{
    if (*pLength < MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN) 
        return NOK;
	PAL_memCopy(list,mgmt_access_list,MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN);
    *pLength = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    return OK;
}

/******************************************************************************
*                           mgmtAccessGetEnabled()          				  *
*******************************************************************************
DESCRIPTION:	Get access enable flag.
                                                                         
INPUTS:			None.

OUTPUT:			*enabled - Copy of the access enable flag.

RETURNS:		None.                                                    
******************************************************************************/
void mgmtAccessGetEnabled(UINT8 *enabled)
{
	*enabled = mgmt_access_enabled;
}

/******************************************************************************
*                           mgmtAccessGetType()	               				  *
*******************************************************************************
DESCRIPTION:	Get access list type.
                                                                         
INPUTS:			None.

OUTPUT:			*type - Copy of the access list type.

RETURNS:		None.                                                    
******************************************************************************/
void mgmtAccessGetType(mgmt_access_list_type_t *type)
{
	*type = mgmt_access_list_type;
}

/******************************************************************************
*                           mgmtAccessSetMAC()	       				          *
*******************************************************************************
DESCRIPTION:	Add a given MAC Address to the B/W table.
                                                                         
INPUTS:			mac_addr - pointer to MAC address.

OUTPUT:			None.

RETURNS:		OK if success, ERROR if empty or broadcast MAC provided.                                                    
******************************************************************************/
STATUS mgmtAccessSetMAC(char *mac_addr) 
{
    int i;
    UINT16 idx;

    if (NumOfStationsInList == MGMT_ACCESS_LIST_SIZE) 
    {
        return ERROR;
    }

    /* an empty MAC (00-00-00-00-00-00), a broadcast MAC (ff-ff-ff-ff-ff-ff) */
    if( MACCMPZ(mac_addr) || MACCMPF(mac_addr))
        return ERROR;

    for (i=0; i<MGMT_ACCESS_LIST_SIZE; i++) 
    {
        if( MACCMP( mgmt_access_list[i], mac_addr)) 
            return OK;  /* already exists */
    }

    MACDUP2( mgmt_access_list[NumOfStationsInList], mac_addr);
    NumOfStationsInList++;
    if (mgmt_access_enabled && (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST)) 
    {
        if (wdrv_dbFindStaIndex(mac_addr,&idx) == OK) 
        {   /* station is already exists - send DIS_ASSOC msg and remove it. */
            wdrv_mgmtDeAuthMng(mac_addr,12/*STATUS_ASSOC_DENIED_OUT_OF_SCOPE*/);
        }
    }
	return OK;
}

/******************************************************************************
*                               mgmtAccessRemMAC()	           				  *
*******************************************************************************
DESCRIPTION:	Remove a given index from the B/W table.
                                                                         
INPUTS:			index - entry index to be removed.

OUTPUT:			None.

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS mgmtAccessRemMAC(UINT16 index)
{
    int i;
    UINT16 idx;
    UINT8 mac_addr[MAC_ADDR_LEN];
    register void *pTmp;
   
    if ((index == 0) || (index > MGMT_ACCESS_LIST_SIZE))
	    return NOK;
 
    if (index > NumOfStationsInList)
        return OK;


    MACDUP2( mac_addr, mgmt_access_list[index-1]);
    
    /* shrink the table by one */
    for( i=index; i<NumOfStationsInList; ++i) {
       register void *pTmp1 = (void*)(mgmt_access_list[i]);
       pTmp  = (void*)(mgmt_access_list[i-1]);
       MACDUP2( pTmp, pTmp1);
    }
   
    --NumOfStationsInList;
    /* Set last MAC address to 0 */
    pTmp = (void*)(mgmt_access_list[NumOfStationsInList]);
    UST2( pTmp, 0) = 0;
    UST2( pTmp, 2) = 0;
    UST2( pTmp, 4) = 0;

  
    if (mgmt_access_enabled && (mgmt_access_list_type == MGMT_ACCESS_WHITE_LIST))
    {
	   
        if (wdrv_dbFindStaIndex(mac_addr,&idx) == OK) 
        {   /* station is already exists - send DIS_ASSOC msg and remove it. */
	     wdrv_mgmtDeAuthMng(mac_addr,12/*STATUS_ASSOC_DENIED_OUT_OF_SCOPE*/);
        }
    }

    return OK;
}

/******************************************************************************
*                       wdrv_RestartWatchDog_Start()	       				  *
*******************************************************************************
DESCRIPTION:	Remove a given index from the B/W table.
                                                                         
INPUTS:			mac_addr - pointer to MAC address.

OUTPUT:			None.

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS mgmtAccessRemMACbyAddr(char *mac_addr)
{
    int i;

    for (i=0; i < NumOfStationsInList; i++) 
    {
       if( MACCMP( mgmt_access_list[i], mac_addr)) 
            break;
    }

    if (i == NumOfStationsInList)
        return OK;  /* not found */

    return mgmtAccessRemMAC(i+1);
}

/******************************************************************************
*                        mgmtAccessCallback()	               				  *
*******************************************************************************
DESCRIPTION:	Callback function called to check is mac address in black list.
                                                                         
INPUTS:			mac_addr - pointer to MAC address.
                mac_addr_len - MAC address length.
OUTPUT:			None.

RETURNS:		OK if MAC address not found in black list, otherwise NOK                                                    
******************************************************************************/
STATUS mgmtAccessCallback(char * mac_addr, int mac_addr_len) 
{
	int i;

	if (! mgmt_access_enabled)
		return OK;

	if (mac_addr_len != MAC_ADDR_LEN)
		return NOK;

	for (i=0; i<NumOfStationsInList; ++i) 
    {
        if ( MACCMP( mac_addr, mgmt_access_list[i]))
            return (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST) ? NOK : OK;
	}


	/* 2006.05.10 Banned List */
        for(i=0;i<MAX_BANNED;i++)
                if(memcmp(&banned_list[i].mac, mac_addr, MAC_ADDR_LEN) == 0){
                        banned_list[i].timeStamp=jiffies;
                        return NOK;
                }

        if(++banlist_index > MAX_BANNED)
                banlist_index=0;

        memcpy(&banned_list[banlist_index].mac, mac_addr, MAC_ADDR_LEN);
        banned_list[banlist_index].timeStamp=jiffies;
        //printk("banned_list[%d].mac=%08x\n", banlist_index, banned_list[banlist_index].mac);
	return (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST) ? OK : NOK;
}


/******************************************************************************
*                           mgmtAccessListStatus()	           				  *
*******************************************************************************
DESCRIPTION:	Given a buffer, prints to it the B/W table contents. 
                The buffer is ready tobe printed out.
                                                                         
INPUTS:			list - pointer to buffer, max_list - buffer length.

OUTPUT:			*pNumOfElements - number of copied elements.

RETURNS:		None.                                                    
******************************************************************************/
void mgmtAccessListStatus(char *list, int max_list, int *pNumOfEntries) 
{
    int i, res, count;
    *list = '\0';
    
    
    for (i=0, count=0, *pNumOfEntries=0; i<MGMT_ACCESS_LIST_SIZE; ++i) 
    {
       /* if empty mac, return */

        if( MACCMPZ( mgmt_access_list[i]))
        {
            return;     /* end of table */
        }
        res = sprintf(list+count,   
                      "%2d : %02hx-%02hx-%02hx-%02hx-%02hx-%02hx\n", 
                      i+1, 
                      mgmt_access_list[i][0], mgmt_access_list[i][1],
                      mgmt_access_list[i][2], mgmt_access_list[i][3],
                      mgmt_access_list[i][4], mgmt_access_list[i][5]);
        if (count+5 > max_list)    /* almost end of given buffer */
            return;
        if (res<0) 
            return;
        else
            count+=res;
        (*pNumOfEntries)++;
    }
}


