/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:		mgmtSsidHandler.h									   */
/*		PURPOSE:	Management ssid handeling							   */
/*																	 	   */
/***************************************************************************/

#include "mgmtSsidHandler.h"
#include "drvDbHash.h" 



/**********************************************
  TODO:
	1. check performance in all  data path functions.
	2. check wep function in all profiles.
	3. cheeck mix-mode.
	4. check several radiuses.
***********************************************/

/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/

/* profile struct ,include all profile 
   privecy parameters 
*/
typedef struct
{
	apMultiSsidElem_t           multi_ssid;
	dot11_rsne_T	            rsne_ie;    /* wpa2 info element */ 
	dot11_rsne_T	            ssn_ie;     /* wpa info element */
	BOOL                        is_authenticator_support;   /* do we need the aothenticator support
													        ,case wep legacy the privecy bit is on but
													        no aothentication is needed*/
	BOOL                        priv_opt_implemented;       /* privecy opt bit is on*/
	UINT8                       cipher;     /* profile cipher*/
    wpa_sta_type_t			    secType;
	wpa_temporal_key_t          default_key;  /* profile braodcast key ,for recovery*/
	int                         default_key_index; /* profile braodcast key index ,for recovery*/
} profile_elem;


/* AP's hidden SSID status */
static UINT16 apHiddenSSID;
/* Multiple SSID flag */
static UINT16 apMultipleSsidFlag; 
/* is module started flag*/
/*static BOOL   is_started =0;*/


/************************************************ 
  security profiles per ssid table:
**************************************************/
static profile_elem privacy_table[NUM_MULTIPLE_SSID];


/******************************************************************************
*                       wdrv_mgmt_ssidGetNumOfSsid         	              	  *
*******************************************************************************
DESCRIPTION:	Get number of supported SSID's
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		number of supported SSID's
******************************************************************************/
int     wdrv_mgmt_ssidGetNumOfSsid(void)
{
	int i, apNumOfMultipleSsid = 0;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		if (privacy_table[i].multi_ssid.ssidType != SSID_NON_ACTIVE) 
		   apNumOfMultipleSsid++; 
	}
	return apNumOfMultipleSsid;
}

/******************************************************************************
*                       wdrv_mgmt_ssidInit                 	              	  *
*******************************************************************************
DESCRIPTION:	Initialize SSID management.                    
                                                                         
INPUTS:			pInitConfigStruct - pointer Configuration structure.

OUTPUT:			None

RETURNS:		OK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidInit(initConfigStruct *pInitConfigStruct)
{
	int i;
	
	/* hidden PRIMARY ssid flag*/
	apHiddenSSID = pInitConfigStruct->apHiddenSSID;

	/* indicate if secondary ssids additional to primary are enabled */
	apMultipleSsidFlag = pInitConfigStruct->apMultipleSsidFlag;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		memcpy(&privacy_table[i].multi_ssid, &pInitConfigStruct-> apMultiSsid[i], sizeof(apMultiSsidElem_t));
		if (privacy_table[i].multi_ssid.ssidType != SSID_NON_ACTIVE) 
		{
			privacy_table[i].multi_ssid.ssid.hdr.eleId = DOT11_SSID_ELE_ID;
            
            if( (privacy_table[i].secType & WPA_STA_TYPE_WPA) || 
                (privacy_table[i].secType & WPA_STA_TYPE_WPA2) || 
                (privacy_table[i].secType & WPA_STA_TYPE_WEP_8021X) )
            {
                privacy_table[i].is_authenticator_support = TRUE;
            }
            else
            {
                privacy_table[i].is_authenticator_support = FALSE;
            }
            
			if (privacy_table[i].multi_ssid.ssidType != SSID_INTERNAL) 
				printf("For SSID %d vlan tag received is %d\n",i,privacy_table[i].multi_ssid.vlanId);
			
		}
	}
	
	return OK;
}

/*************************************************************************
 *                        wdrv_mgmt_ssidIsSSIDSupported					 *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station SSID or not                 
                                                                                                   
INPUT:       The station SSID as received in the assoc/reassoc request.

OUTPUT:      

RETURN:      TRUE if SSID supported, otherwise FALSE.
			 Note: returns FALSE for Internal Ssids (e.g. WDS ssid)

************************************************************************/
BOOL wdrv_mgmt_ssidIsSSIDSupported(dot11_SSID_T	*ssid)
{
	BOOL	compare = FALSE;
    int     index;

	if(ssid == NULL) {
        return FALSE;
    }

    /* Check Multiple SSID */
    for (index = 0; index < NUM_MULTIPLE_SSID; index++) 
    {
		if (privacy_table[index].multi_ssid.ssidType == SSID_NON_ACTIVE ||
			privacy_table[index].multi_ssid.ssidType == SSID_INTERNAL)
			continue;
		if ((ssid->hdr.eleLen) != privacy_table[index].multi_ssid.ssid.hdr.eleLen)
            continue;
		compare = (PAL_memCmp(ssid->serviceSetId, privacy_table[index].multi_ssid.ssid.serviceSetId, ssid->hdr.eleLen)==0);
        if (compare)
		{
            return TRUE;
		}
    }
	return FALSE;
}

/******************************************************************************
*                      wdrv_mgmt_ssidGetHiddenSsidStatus           	          *
*******************************************************************************
DESCRIPTION:	Get status of the hidden SSID support.                    
                                                                         
INPUTS:			None.

OUTPUT:			

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetHiddenSsidStatus(UINT16 *value)
{
    *value = apHiddenSSID;
}

/*************************************************************************
 *                        wdrv_mgmt_ssidGetPrimarySsid                   *
 *************************************************************************
DESCRIPTION:Returns the SSID 
			value is accepted to point to a buffer in len MAX_SSID_LENGTH + 1 

                                                                                                   
INPUT:       

OUTPUT:         *value - copy of primary SSID

RETURN:         OK/NOK
                                                                                                                                                          
************************************************************************/
STATUS wdrv_mgmt_ssidGetPrimarySsid(char *value)
{
	int i;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		if (privacy_table[i].multi_ssid.ssidType == SSID_PRIMARY) 
		{
			PAL_memCopy(value, privacy_table[i].multi_ssid.ssid.serviceSetId, 
						privacy_table[i].multi_ssid.ssid.hdr.eleLen);	
			value[privacy_table[i].multi_ssid.ssid.hdr.eleLen] = '\0';
			return OK;
		}
	}
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetMultipleSsidStatus()              	  *
*******************************************************************************
DESCRIPTION:	Get status of the muliple SSID support..                    
                                                                         
INPUTS:			None.

OUTPUT:			*value - copy of multiple SSID support flag

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetMultipleSsidStatus(UINT16 *value)
{
    *value = apMultipleSsidFlag;
}

/******************************************************************************
*            wdrv_mgmt_ssidGetElementFromMultipleSsidActualTable           	  *
*******************************************************************************
DESCRIPTION:	Get element from the multiple SSID table.                    
                                                                         
INPUTS:			index - entry index.

OUTPUT:			*mSsidElem - a pointer multiple SSID table entry

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetElementFromMultipleSsidActualTable(apMultiSsidElem_t *mSsidElem, UINT16 index)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memCopy(mSsidElem, &privacy_table[index].multi_ssid, sizeof(apMultiSsidElem_t));
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}	

/******************************************************************************
*                     wdrv_mgmt_ssidGetSsidPrivacyIE     	              	  *
*******************************************************************************
DESCRIPTION:	Get SSID privacy information element.                    
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidGetSsidPrivacyIE(dot11_SSID_T *ssid,dot11_rsne_T **p_rsn,dot11_rsne_T **p_ssn)
{
	int index,compare;

    for (index = 0; index < NUM_MULTIPLE_SSID; index++) 
    {
		if (privacy_table[index].multi_ssid.ssidType == SSID_NON_ACTIVE)
			continue;

        if ((ssid->hdr.eleLen) != strlen(privacy_table[index].multi_ssid.ssid.serviceSetId))
            continue;

        compare = (PAL_memCmp(ssid->serviceSetId,
							  privacy_table[index].multi_ssid.ssid.serviceSetId,
							  ssid->hdr.eleLen)==0);
		if (compare)
		{
			if(privacy_table[index].rsne_ie.hdr.eleLen != 0)
			{
				*p_rsn = &privacy_table[index].rsne_ie;
			}

			if(privacy_table[index].ssn_ie.hdr.eleLen != 0)
			{
				*p_ssn = &privacy_table[index].ssn_ie;
			}

            return OK;
		}
    }
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetVlanPofileIndex    	              *
*******************************************************************************
DESCRIPTION:	Get VLAN profile index.                    
                                                                         
INPUTS:			vlan - requered vlan tag.

OUTPUT:			*p_index - copy of vlan profile index 

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidGetVlanPofileIndex(int vlan,int *p_index)
{

	int i;

	for(i =0; i<NUM_MULTIPLE_SSID; i++)
	{
		if (privacy_table[i].multi_ssid.ssidType == SSID_NON_ACTIVE ||
			privacy_table[i].multi_ssid.ssidType == SSID_INTERNAL)
			continue;

		if(privacy_table[i].multi_ssid.vlanId == vlan)
		{
			*p_index = i;
			return OK;
		}
	}

	return NOK;
}
/******************************************************************************
*                       wdrv_mgmt_ssidGetSsidPofileIndex      	              *
*******************************************************************************
DESCRIPTION:	Get SSID profile index.                    
                                                                         
INPUTS:			ssid - pointer to requered ssid.

OUTPUT:			*p_index - copy of ssid profile index 

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidGetSsidPofileIndex(dot11_SSID_T *ssid , int *p_index)
{
	int index,compare;
	
	if(ssid == NULL)
	{
		printk("wdrv_mgmt_ssidGetSsidPofileIndex:receive null ssid\n");
		return NOK;
	}

    for (index = 0; index < NUM_MULTIPLE_SSID; index++) 
    {
        if (privacy_table[index].multi_ssid.ssidType == SSID_NON_ACTIVE)
			continue;
		
		if (ssid->hdr.eleLen != privacy_table[index].multi_ssid.ssid.hdr.eleLen)
            continue;
        
		compare = (PAL_memCmp(ssid->serviceSetId, privacy_table[index].multi_ssid.ssid.serviceSetId,
							   privacy_table[index].multi_ssid.ssid.hdr.eleLen)==0);

		if (compare)
		{
			*p_index =  index;
			return OK;
		}
	}
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetSsn          	              	      *
*******************************************************************************
DESCRIPTION:	Get ssid ssn.                    
                                                                         
INPUTS:			profile_index - SSID profile index.

OUTPUT:			ssn - copy of ssn

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetSsn(int profile_index,dot11_rsne_T *ssn)	
{

	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	PAL_memSet(ssn, 0, sizeof(dot11_rsne_T));	

	PAL_memCopy((void*)ssn,(void*)&privacy_table[profile_index].ssn_ie, privacy_table[profile_index].ssn_ie.hdr.eleLen + 2);	

 	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}

/******************************************************************************
*                       wdrv_mgmt_ssidSetSsn              	              	  *
*******************************************************************************
DESCRIPTION:	Set profile ssn .                    
                                                                         
INPUTS:			len - ssn length.
                pInfo - pointer to ssn buffer
                profile_index - profile index.
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidSetSsn(UINT8 len, UINT8 *pInfo,int profile_index)
{

	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	PAL_memSet(&privacy_table[profile_index].ssn_ie, 0, sizeof(dot11_rsne_T));	


	if (pInfo != NULL)
	{
		PAL_memCopy(&privacy_table[profile_index].ssn_ie, pInfo, len);
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);/* END OF CRITICAL SECTION */
}


/******************************************************************************
*                       wdrv_mgmt_ssidGetRsne          	              	      *
*******************************************************************************
DESCRIPTION:	Get profile rsne.                    
                                                                         
INPUTS:			profile_index - profile index.

OUTPUT:			rsne - copy of rsne

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetRsne(int profile_index,dot11_rsne_T *rsne)	
{

	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	PAL_memSet(rsne, 0, sizeof(dot11_rsne_T));	

	PAL_memCopy((void*)rsne,(void*)&privacy_table[profile_index].rsne_ie, privacy_table[profile_index].rsne_ie.hdr.eleLen + 2);	

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); /* START OF CRITICAL SECTION */
}

/******************************************************************************
*                       wdrv_mgmt_ssidSetRsne              	              	  *
*******************************************************************************
DESCRIPTION:	Set profile rsne .                    
                                                                         
INPUTS:			len - rsne length.
                pInfo - pointer to rsne buffer
                profile_index - profile index.
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidSetRsne(UINT8 len, UINT8 *pInfo,int profile_index)
{

	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	PAL_memSet(&privacy_table[profile_index].rsne_ie, 0, sizeof(dot11_rsne_T));	

	if (pInfo != NULL)
	{
		PAL_memCopy(&privacy_table[profile_index].rsne_ie, pInfo, len);
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); /* START OF CRITICAL SECTION */
}

/******************************************************************************
*                       wdrv_mgmt_ssidSetPrivOpt          	              	  *
*******************************************************************************
DESCRIPTION:	Set profile privacy option.                    
                                                                         
INPUTS:			profile_index - profile index.
                val - privacy option
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidSetPrivOpt(int profile_index ,BOOL val)
{

	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	privacy_table[profile_index].priv_opt_implemented = val;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); /* START OF CRITICAL SECTION */
}

/******************************************************************************
*                      wdrv_mgmt_ssidGetStationSsidIndex   	              	  *
*******************************************************************************
DESCRIPTION:	Get station ssid index                    
                                                                         
INPUTS:			idx - station index.

OUTPUT:			*p_profile_idx - copy of profile index.

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidGetStationSsidIndex(UINT16 idx , int  *p_profile_idx)
{
	int ssid_len = 0;
	char ssid_str[WHAL_SSID_STRING_LEN]; 
	dot11_SSID_T ssid;

	*p_profile_idx = 0;

	if(idx >= MAX_STATIONS)
	{
		return NOK;
	}
	
	wdrv_dbGetStaSsidByIndex(idx,ssid_str,&ssid_len);
	memcpy(ssid.serviceSetId, ssid_str, ssid_len);
	ssid.hdr.eleLen = ssid_len;

	if(wdrv_mgmt_ssidGetSsidPofileIndex(&ssid,p_profile_idx ) != OK)
	{
		return NOK;
	}

	return OK; 
}

/******************************************************************************
*                      wdrv_mgmt_ssidSetBroadcastSuiteType   	              *
*******************************************************************************
DESCRIPTION:	Set broadcast cipher type.                    
                                                                         
INPUTS:			cipher - broadcast cipher.
                prof_index - profile index
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidSetBroadcastSuiteType(UINT8 cipher ,int prof_index)
{
	privacy_table[prof_index].cipher = cipher;
	return OK;
}

/******************************************************************************
*                 wdrv_mgmt_ssidGetBroadcastSuiteType     	              	  *
*******************************************************************************
DESCRIPTION:	Set broadcast cipher type.                    
                                                                         
INPUTS:			prof_index - profile index

OUTPUT:			*cipher - copy of broadcast cipher

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidGetBroadcastSuiteType(UINT8 *cipher,int prof_index)
{
	*cipher = privacy_table[prof_index].cipher;
	return OK;
}

/******************************************************************************
*                  wdrv_mgmt_ssidSetBroadcastKey      	              	      *
*******************************************************************************
DESCRIPTION:	Set SSID broadcast key                    
                                                                         
INPUTS:			prof_index - profile index
                *p_key - broadcast key
                key_idx - key index
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidSetBroadcastKey(int prof_index,wpa_temporal_key_t *p_key,int key_idx)
{
	memcpy(&privacy_table[prof_index].default_key,p_key,sizeof(wpa_temporal_key_t));
	privacy_table[prof_index].default_key_index = key_idx;
	return OK;
}

/******************************************************************************
*                    wdrv_mgmt_ssidGetBroadcastKey    	              	      *
*******************************************************************************
DESCRIPTION:	Get SSID broadcast key.                    
                                                                         
INPUTS:			prof_index - profile index

OUTPUT:			*p_key - copy of broadcast key
                *p_key_idx - copy of key index

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidGetBroadcastKey(int prof_index,wpa_temporal_key_t *p_key,int *p_key_idx)
{
	memcpy(p_key,&privacy_table[prof_index].default_key,sizeof(wpa_temporal_key_t));
	*p_key_idx = privacy_table[prof_index].default_key_index;
	return OK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidSetSecurityType      	              	  *
*******************************************************************************
DESCRIPTION:	Set profile security type.                    
                                                                         
INPUTS:			profile_index - profile index.
                type - security type
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidSetSecurityType(int profile_index ,wpa_sta_type_t type)
{

	/*PAL_spinLock(&ssidLock); *//* START OF CRITICAL SECTION */
	privacy_table[profile_index].secType = type;

	/*PAL_spinUnLock(&ssidLock);*/ /* START OF CRITICAL SECTION */
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetSecurityType      	              	  *
*******************************************************************************
DESCRIPTION:	Get profile security type.                    
                                                                         
INPUTS:			profile_index - profile index.
                type - security type
OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_mgmt_ssidGetSecurityType(int profile_index ,wpa_sta_type_t *type)
{
	*type = privacy_table[profile_index].secType;
}

/******************************************************************************
*                 wdrv_mgmt_ssidUpdateCapPrivBySsid      	              	  *
*******************************************************************************
DESCRIPTION:	Update privacy capabilities by SSID                    
                                                                         
INPUTS:			*cap - privacy capabilities.
                *ssid - 802.11 SSID.
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidUpdateCapPrivBySsid(UINT16 *cap,dot11_SSID_T	*ssid)
{
	int profile_index;

	if(ssid->hdr.eleLen > 32)
	{
		printf("wdrv_mgmt_ssidUpdateCapabilitiesPriv:receive error ssid len\n");
		return NOK;
	}

	if(wdrv_mgmt_ssidGetSsidPofileIndex(ssid,&profile_index) != OK)
	{
		printf("wdrv_mgmt_ssidUpdateCapabilitiesPriv:cant find ssid index ,setting primary ssid privecy cap\n");
		wdrv_mgmt_ssidGetPrimarySsidIndex(&profile_index);
	}

	if(privacy_table[profile_index].priv_opt_implemented)
		(*cap) |= (1 << CAP_PRIVACY_SHIFT);
	else
		(*cap) &= ~CAP_PRIVACY_SHIFT;

	return OK;
}

/******************************************************************************
*                 wdrv_mgmt_ssidUpdateCapPrivProfileIndex	              	  *
*******************************************************************************
DESCRIPTION:	Update privacy capabilities by profile index.
                                                                         
INPUTS:			*cap - privacy capabilities.
                profile_index - profile index
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidUpdateCapPrivProfileIndex(UINT16 *cap,int profile_index)
{
	if(privacy_table[profile_index].priv_opt_implemented)
		(*cap) |= (1 << CAP_PRIVACY_SHIFT);
	else
		(*cap) &= (~CAP_PRIVACY_SHIFT);

	return OK;
}

/******************************************************************************
*                 wdrv_mgmt_ssidUpdateCapPrivByStaIdx	              	      *
*******************************************************************************
DESCRIPTION:	Update privacy capabilities by station index.
                                                                         
INPUTS:			*cap - privacy capabilities.
                idx - station index
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidUpdateCapPrivByStaIdx(UINT16 *cap,UINT16 idx)
{
	if(wdrv_mgmt_ssidIsStationPrivOptOn(idx))
		(*cap) |= 1 << CAP_PRIVACY_SHIFT;
	else
		(*cap) &=   ~CAP_PRIVACY_SHIFT;

	return OK;
}

/******************************************************************************
*                    wdrv_mgmt_ssidIsStationPrivOptOn     	              	  *
*******************************************************************************
DESCRIPTION:	Query station privacy options.                    
                                                                         
INPUTS:			index - station index.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
BOOL wdrv_mgmt_ssidIsStationPrivOptOn(UINT16 index)
{
	int profile_index;

	if(wdrv_mgmt_ssidGetStationSsidIndex(index,&profile_index) != OK)
	{
		return FALSE;
	}
	
	if(privacy_table[profile_index].priv_opt_implemented)
	{
		return TRUE;
	}
	return FALSE;
}

/******************************************************************************
*                    wdrv_mgmt_ssidIsStationNeedAuthSupprt     	              *
*******************************************************************************
DESCRIPTION:	Query station authentication support.                    
                                                                         
INPUTS:			index - station index.

OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS wdrv_mgmt_ssidIsStationNeedAuthSupprt(UINT16 index)
{
	int profile_index;

	if(wdrv_mgmt_ssidGetStationSsidIndex(index,&profile_index) != OK)
	{
		return 0xff;
	}
	
	if(privacy_table[profile_index].is_authenticator_support)
	{
		return OK;
	}

	return NOK;
}

/*************************************************************************
 *                        wdrv_mgmt_ssidCapabilitiesPrivSupport			         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station
			 capabilities as received in the association request.
			 Right now it checks the provacy bit only.	
                                                                                                   
INPUT:       The station capabilities.

OUTPUT:      

RETURN:      OK : Capabilities supportted.
             NOK: Capabilities supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_mgmt_ssidCapabilitiesPrivSupport(UINT16	*caps,dot11_SSID_T *ssid)
{
	int profile_index;

	if(wdrv_mgmt_ssidGetSsidPofileIndex(ssid ,&profile_index) != OK)
	{
		profile_index = 0;
	}

	if((privacy_table[profile_index].priv_opt_implemented) == (((*caps) >> CAP_PRIVACY_SHIFT) & 1))
		return TRUE;
	else if(privacy_table[profile_index].priv_opt_implemented)
		return TRUE;
	else	
		return FALSE;
}

/******************************************************************************
*                       wdrv_mgmt_ssidIs1PrivecyCfg()           	          *
*******************************************************************************
DESCRIPTION:	Check for privacy capabilities at least one of the 
                supported SSID's.                    
INPUTS:			None.

OUTPUT:			None

RETURNS:		TRUE if supported, otherwise FALSE.                                                    
******************************************************************************/
BOOL    wdrv_mgmt_ssidIs1PrivecyCfg(void)
{
	int i;
	for(i=0; i < NUM_MULTIPLE_SSID; i++)
	{
	   if (privacy_table[i].multi_ssid.ssidType == SSID_NON_ACTIVE)
			continue;

		if(privacy_table[i].priv_opt_implemented)
			return TRUE;
	}
	return FALSE;
}

/******************************************************************************
*                    wdrv_mgmt_ssidUpdateHashTable      	              	  *
*******************************************************************************
DESCRIPTION:	Update hash table for station.                    
                                                                         
INPUTS:			idx - station index.
                *ssid - 802.11 SSID
                SsidLen - length 0f *ssid
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS	wdrv_mgmt_ssidUpdateHashTable(UINT16 idx,char *ssid_str, int SsidLen)
{
	int profile_index;
	dot11_SSID_T ssid;

	if(wdrv_dbSetStaSsidByIndex(idx,ssid_str,SsidLen) != OK)
	{
		printf("wdrv_mgmt_ssidUpdateHashTable:error setting ssid in db hash\n");
		return NOK;
	}
	
	ssid.hdr.eleLen = SsidLen;
	memcpy(ssid.serviceSetId, ssid_str, WHAL_SSID_STRING_LEN);
	
	if(wdrv_mgmt_ssidGetSsidPofileIndex(&ssid, &profile_index) != OK)
	{
		printf("wdrv_mgmt_ssidUpdateHashTable:error getting ssid profile index\n");
		return NOK;
	}
	
	wdrv_dbSetStaProfileIndex(idx,profile_index);
	
	return OK;

}
/******************************************************************************
*                      wdrv_mgmt_ssidHandleRecvTagPkt      	              	  *
*******************************************************************************
DESCRIPTION:	Checks VLAN tag from packet according to SSID settings.                    
                                                                         
INPUTS:			*pPkt - packet to be examinated.
                *profile - profile 
OUTPUT:			None

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidHandleRecvTagPkt(char *pPkt , int *profile)
{
	UINT16 vlan_tag;
 											
	/*  get vlan tag from packet*/
	vlan_tag = (((UINT16)pPkt[14] << 8) | (UINT8)pPkt[15]) & 0xfff;

	if(wdrv_mgmt_ssidGetVlanPofileIndex(vlan_tag,profile) != OK)
		return NOK; 
	return OK;
}

/******************************************************************************
*                      wdrv_mgmt_ssidGetStaVlanTag      	              	  *
*******************************************************************************
DESCRIPTION:	Get VLAN tag for station.                    
                                                                         
INPUTS:			*mac - station MAC address.

OUTPUT:			*vlan - station VLAN tag if found

RETURNS:		OK if success, otherwise - NOK.                                                    
******************************************************************************/
STATUS   wdrv_mgmt_ssidGetStaVlanTag(char *mac ,UINT16 *vlan)
{
	UINT16 idx;
	int profile;	
	if(wdrv_dbFindStaIndex(mac,&idx) != OK)
	{
		printf("wdrv_mgmt_ssidGetStaVlanTag : cant find station\n");
		return NOK;
	}

	if(wdrv_dbGetStaProfileByIndex(idx,&profile) != OK)
	{
		printf("wdrv_mgmt_ssidGetStaVlanTag : cant find station profile index\n");
		return NOK;
	}

	*vlan = privacy_table[profile].multi_ssid.vlanId;
	return OK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetPrimaryVlan       	              	  *
*******************************************************************************
DESCRIPTION:	Get VLAN tag for primary SSID.                    
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		ok/nok                                                    
******************************************************************************/
STATUS     wdrv_mgmt_ssidGetPrimaryVlan(UINT16 *vlan)
{
	int i;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		if (privacy_table[i].multi_ssid.ssidType == SSID_PRIMARY) 
		{
			*vlan = privacy_table[i].multi_ssid.vlanId;
			return OK;
		}
	}
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidIsProfilePrivOptOn                 	  *
*******************************************************************************
DESCRIPTION:	Get privacy options of profile.                    
                                                                         
INPUTS:			profile_index - profile index.

OUTPUT:			None

RETURNS:		TRUE if implemented, otherwise FALSE.                                                    
******************************************************************************/
BOOL   wdrv_mgmt_ssidIsProfilePrivOptOn(int profile_index)
{
	if(privacy_table[profile_index].priv_opt_implemented)
	{
		return TRUE;
	}
	return FALSE;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetPrimarySsidIndex                 	  *
*******************************************************************************
DESCRIPTION:	Get primary profile index.                    
                                                                         
INPUTS:			index - profile index.

OUTPUT:			None

RETURNS:		OK/NOK.                                                    
******************************************************************************/

STATUS wdrv_mgmt_ssidGetPrimarySsidIndex(int *index)
{
	int i;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		if (privacy_table[i].multi_ssid.ssidType == SSID_PRIMARY) 
		{
			*index = i;
			return OK;
		}
	}
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetInternalSsidIndex                 	  *
*******************************************************************************
DESCRIPTION:	Get internal profile index.                    
                                                                         
INPUTS:			index - profile index.

OUTPUT:			None

RETURNS:		OK/NOK.                                                    
******************************************************************************/

STATUS wdrv_mgmt_ssidGetInternalSsidIndex(int *index)
{
	int i;

	for (i=0; i<NUM_MULTIPLE_SSID; i++) 
	{
		if (privacy_table[i].multi_ssid.ssidType == SSID_INTERNAL) 
		{
			*index = i;
			return OK;
		}
	}
	return NOK;
}

/******************************************************************************
*                       wdrv_mgmt_ssidGetPrimarySsidIndex                 	  *
*******************************************************************************
DESCRIPTION:	Determine if SSID is active                    
                                                                         
INPUTS:			index -  index.

OUTPUT:			None

RETURNS:		OK/NOK.                                                    
******************************************************************************/

BOOL wdrv_mgmt_ssidIsActive(int index)
{
	if (privacy_table[index].multi_ssid.ssidType == SSID_NON_ACTIVE)
		return FALSE;
	else
		return TRUE;
}
