/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004    Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*	MODULE:	QoSClassifier.c		 		                                        */
/*    	PURPOSE:	Implementation of WME QoS                               */
/*  Author : Demiurg (Alexander Sirotkin)                                 */
/**************************************************************************/


#include "wlan_pal.h"
#include "802_11Defs.h"
#include "QoSClassifier.h"
#include "QoSACParams.h"
#include <Tx/tx.h>
#include <AcxInfoElements.h>
#include <BssBridge/bssbridge.h>
#include <whalAcxAccess.h>
#include <whalAcxRegs.h>

#define WDRV_QOS_MAX_VLAN 8
#define WDRV_QOS_MAX_UDP_PORT 8
#define WDRV_QOS_MAX_DIFFSERVCP 8

/*
     Auxillary Maps (used for iptimization, to get read of all that switches) : 
     AC to UP
     UP to AC
     AC to txFlags
     txFlags to AC
*/

UINT8 wdrv_QoSAuxMapACtoUP[WDRV_QOS_MAX_AC];

/* This table map Tx Flags Bit map to AC  */
wdrv_QoSClassifierAC_t wdrv_QoSAuxMapUPtoAC[WDRV_QOS_MAX_UP];

/* This table maps unicast traffic TX priorities to AC  */
wdrv_QoSClassifierAC_t wdrv_QoSAuxMapPriorToAC[WDRV_QOS_MAX_UP];

/* This table map AC to Tx Flags Bit map */
UINT32 wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_MAX_AC];

wdrv_QoSClassifierAC_t wdrv_QoSAuxMapsTxFlagstoAC[WDRV_QOS_MAX_AC];

UINT8 wdrv_QoSACtoACID[WDRV_QOS_MAX_AC] = {AC_BEST_EFFORT, AC_BKGRND, AC_VIDEO, AC_VOICE};

#define TIAP_PARAM_QOS_MAX_CP 64

static void wdrv_QoSAuxMapsInit(void);

/* auxillary arrays for AC to UP to vlan priority to diffserv codepoint conversion */

static wdrv_QoSClassifierAC_t configuredMode =  WDRV_QOS_CLASSIFIER_MODE_UNDEFINED;
static wdrv_QoSClassifierAC_t * vlanToACMap = NULL;
static wdrv_QoSClassifierUP_t * diffServCPToUPMap = NULL;

static UINT8 *refPortToUP = NULL;

static UINT8 numOfCP;

static wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperVLAN(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up);
static wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperDiffServ(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up);
static wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperUdpPort(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up);
wdrv_QoSClassifierAC_t (*wdrv_QoSClassifierClassify)(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up);

#if 0
static void  wdrv_QoSTimestampTimerFunc(void);
static int wdrv_QoSTimestampDelta = 0;
static PALSRV_TcbObjTYPE wdrv_QoSTimestampTimer;

static void  wdrv_QoSTimestampTimerFunc(void) {
	wdrv_QoSTimestampDelta = whal_acxRegRead( ACX_MAC_TIMESTAMP_REG ) - PALSRV_ticks2usec(PALSRV_getTick());
}
#endif

/*************************************************************************
 *		wdrv_QoSAuxMapsInit								                                  *
*************************************************************************
DESCRIPTION: Initialize auxillary conversion maps
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

static void wdrv_QoSAuxMapsInit(void) {
    wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_BE] = WDRV_TX_AC_BE | WDRV_TX_LOW_PRIORITY;
    wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_BK] = WDRV_TX_AC_BK;
    wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_VI] = WDRV_TX_AC_VI;
    wdrv_QoSAuxMapACtoTxFlags[WDRV_QOS_CLASSIFIER_AC_VO] = WDRV_TX_AC_VO | WDRV_TX_HIGH_PRIORITY;

    wdrv_QoSAuxMapUPtoAC[1] = WDRV_QOS_CLASSIFIER_AC_BK;
    wdrv_QoSAuxMapUPtoAC[2] = WDRV_QOS_CLASSIFIER_AC_BK;
    wdrv_QoSAuxMapUPtoAC[0] = WDRV_QOS_CLASSIFIER_AC_BE;
    wdrv_QoSAuxMapUPtoAC[3] = WDRV_QOS_CLASSIFIER_AC_BE;
    wdrv_QoSAuxMapUPtoAC[4] = WDRV_QOS_CLASSIFIER_AC_VI;
    wdrv_QoSAuxMapUPtoAC[5] = WDRV_QOS_CLASSIFIER_AC_VI;
    wdrv_QoSAuxMapUPtoAC[6] = WDRV_QOS_CLASSIFIER_AC_VO;
    wdrv_QoSAuxMapUPtoAC[7] = WDRV_QOS_CLASSIFIER_AC_VO;

	wdrv_QoSAuxMapPriorToAC[0] = WDRV_QOS_CLASSIFIER_AC_BK;
	wdrv_QoSAuxMapPriorToAC[1] = WDRV_QOS_CLASSIFIER_AC_BE;
	wdrv_QoSAuxMapPriorToAC[2] = WDRV_QOS_CLASSIFIER_AC_VI;
	wdrv_QoSAuxMapPriorToAC[3] = WDRV_QOS_CLASSIFIER_AC_VO;
}

#if 0
/*************************************************************************
 *             wdrv_QoSGetTimestamp                                      *
 *************************************************************************
 DESCRIPTION: Returns pseudo MAC timestamp with 10 ms resolution

 INPUT:
 OUTPUT:
 RETURN: timestamp
**************************************************************************/
UINT32 wdrv_QoSGetTimestamp(void) {
    return PALSRV_ticks2usec(PALSRV_getTick()) + wdrv_QoSTimestampDelta;
}
#endif

/*************************************************************************
 *			wdrv_QoSClassifierInit							                                  *
*************************************************************************
DESCRIPTION: Initialize QoS classifier
                                                                                       
INPUT:    mode,  in_numOfCP,  inDiffServMap 

OUTPUT:      

RETURN: result (OK/NOK)
                                                                                                                                                          
************************************************************************/
STATUS wdrv_QoSClassifierInit(BOOL enabled, wdrv_QoSClassifierMode_t mode, UINT8 in_numOfCP, wdrv_QoSClassifierUP_t * inDiffServMap) {    

    if(enabled) {

        configuredMode = mode;

   
        vlanToACMap = PAL_memAlloc(0, sizeof(wdrv_QoSClassifierAC_t)*WDRV_QOS_MAX_VLAN, 0, 0);
        if (vlanToACMap == NULL)
        {
            wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "%s: Memory allocation failed \n", __FUNCTION__);
            return NOK;
        }
        vlanToACMap[0] =    WDRV_QOS_CLASSIFIER_AC_BE;
        vlanToACMap[1] =    WDRV_QOS_CLASSIFIER_AC_BK;
        vlanToACMap[2] =    WDRV_QOS_CLASSIFIER_AC_BK;
        vlanToACMap[3] =    WDRV_QOS_CLASSIFIER_AC_BE;
        vlanToACMap[4] =    WDRV_QOS_CLASSIFIER_AC_VI;
        vlanToACMap[5] =    WDRV_QOS_CLASSIFIER_AC_VI;
        vlanToACMap[6] =    WDRV_QOS_CLASSIFIER_AC_VO;
        vlanToACMap[7] =    WDRV_QOS_CLASSIFIER_AC_VO;


        refPortToUP = PAL_memAlloc(0, sizeof(UINT16)*WDRV_QOS_MAX_UDP_PORT, 0, 0);
        if (vlanToACMap == NULL)
        {
            wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "%s: Memory allocation failed \n", __FUNCTION__);
            return NOK;
        }
		refPortToUP[0] = 0;
		refPortToUP[1] = 1;
		refPortToUP[2] = 4;
		refPortToUP[3] = 6;
		refPortToUP[4] = 0;
		refPortToUP[5] = 0;
		refPortToUP[6] = 0;
		refPortToUP[7] = 0;

        switch (configuredMode)
        {
            case    WDRV_QOS_CLASSIFIER_MODE_VLAN:
				wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "%s : CLassifier mode = VLAN !\n", __FUNCTION__);
                wdrv_QoSClassifierClassify = wdrv_QoSClassifierClassifyHelperVLAN;
                break;
            case    WDRV_QOS_CLASSIFIER_MODE_DIFFSERV:
				wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "%s : CLassifier mode DiffServ !\n", __FUNCTION__);
                numOfCP = in_numOfCP;
                diffServCPToUPMap = PAL_memAlloc(0, sizeof(wdrv_QoSClassifierUP_t)*WDRV_QOS_MAX_CP/*inDiffServMapSize*/, 0, 0);
                if (diffServCPToUPMap == NULL)
                {
                    wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "%s: Memory allocation failed \n", __FUNCTION__);
                    return NOK;
                }
    
                PAL_memCopy(diffServCPToUPMap, inDiffServMap, sizeof(wdrv_QoSClassifierUP_t)*WDRV_QOS_MAX_CP);
    
                wdrv_QoSClassifierClassify = wdrv_QoSClassifierClassifyHelperDiffServ;
                break;

		case 	WDRV_QOS_CLASSIFIER_MODE_UDP_PORT:
				wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "%s : CLassifier mode = udp port !\n", __FUNCTION__);

				wdrv_QoSClassifierClassify = wdrv_QoSClassifierClassifyHelperUdpPort;
				break;
            default:
                wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "%s: Wrong mode \n", __FUNCTION__);
                return NOK;
        }
    
        wdrv_QoSAuxMapsInit();
    } else {
			configuredMode =  WDRV_QOS_CLASSIFIER_MODE_UNDEFINED;
		}

#if 0
	wdrv_QoSTimestampTimerFunc();
	PALSRV_tcbNew(&wdrv_QoSTimestampTimer);
	PALSRV_tcbQueueAct(&wdrv_QoSTimestampTimer, 100, wdrv_QoSTimestampTimerFunc, 0, 0);
#endif
    return OK;
}

/*************************************************************************
 *		wdrv_QoSClassifierShutdown								                                  *
*************************************************************************
DESCRIPTION: Shutdown QoS module
                                                                                       
INPUT:       

OUTPUT:      

RETURN: result (OK/NOK)     
                                                                                                                                                          
************************************************************************/

STATUS wdrv_QoSClassifierShutdown(void) {
    switch (configuredMode)
    {
        case    WDRV_QOS_CLASSIFIER_MODE_DIFFSERV:
			if(diffServCPToUPMap)
				PAL_memFree(0, diffServCPToUPMap, 0);
        case WDRV_QOS_CLASSIFIER_MODE_VLAN:
			if(vlanToACMap)
				PAL_memFree(0, vlanToACMap, 0);
            break;
        case    WDRV_QOS_CLASSIFIER_MODE_UNDEFINED:
            //		PALSRV_report(RL_BRIDGE, PRI_ERROR, 0, "%s: QoS was not initialized !!! \n", __FUNCTION__);
            return OK;
        default:
            wdrv_utilsReport(RL_BRIDGE, PRI_ERROR, "%s: Sanity check failed, need professional help \n", __FUNCTION__);
            return NOK;
    }

    return OK;
}

/*************************************************************************
 *			wdrv_QoSClassifierClassifyHelperVLAN							                                  *
*************************************************************************
DESCRIPTION: Classifier for VLAN priority
                                                                                       
INPUT:    ethernetHeader,  ipHeader  

OUTPUT:    up  

RETURN:   Access Category   
                                                                                                                                                          
************************************************************************/
wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperVLAN(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up) {
    UINT8 vlanPriority;

	if( ntohs(((wdrv_QoSEthernetHeader_t *)ethernetHeader)->vlan_proto) ==  ETHERTYPE_802_1D)
	{
		vlanPriority = (ntohs(((wdrv_QoSEthernetHeader_t *)ethernetHeader)->vlan_TCI) >> 13) & 0x7;
		*up = vlanPriority;

		return vlanToACMap[vlanPriority];
	}
	else
	{
		*up = 0;
		return WDRV_QOS_CLASSIFIER_AC_BE;
	}
}

/*************************************************************************
 *		wdrv_QoSClassifierClassifyHelperDiffServ								                                  *
*************************************************************************
DESCRIPTION: Classifier for DiffServ CodePoint
                                                                                       
INPUT:      ethernetHeader,  ipHeader

OUTPUT:   up   

RETURN:    Access Category  
                                                                                                                                                          
************************************************************************/
wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperDiffServ(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up) {
    UINT8 diffServCP;

	/* Handle case of ethernet header with extra 4 VLAN bytes */
	if( ntohs(((wdrv_QoSEthernetHeader_t *)ethernetHeader)->vlan_proto) ==  ETHERTYPE_802_1D)
	{
		ethernetHeader += 4; /* only for extracting the real TypeLength */
	}

    if (htons(((EthernetHeader_T *)ethernetHeader)->TypeLength) == 0x800)
    {
        diffServCP = 0x3F & (((wdrv_QoSIPHeader_t *)ipHeader)->tos >> 2);
        *up = diffServCPToUPMap[diffServCP];
        return vlanToACMap[*up];
    } else
    {
        *up = 0;
        return WDRV_QOS_CLASSIFIER_AC_BE;
    }
}

/*************************************************************************
 *		wdrv_QoSClassifierClassifyHelperDiffServ								                                  *
*************************************************************************
DESCRIPTION: Classifier for DiffServ CodePoint
                                                                                       
INPUT:      ethernetHeader,  ipHeader

OUTPUT:   up   

RETURN:    Access Category  
                                                                                                                                                          
************************************************************************/
static UINT8 demoPort = 0;
static wdrv_QoSClassifierAC_t wdrv_QoSClassifierClassifyHelperUdpPort(UINT8 * ethernetHeader, UINT8 * ipHeader, UINT8 * up)
{
    UINT8              ipHeaderLen;
    UINT16             dstPortNum;
    UINT8              *pUdpHeader;



	ipHeaderLen = ((*(unsigned char*)ipHeader  & 0x0f) * 4);

	pUdpHeader = ipHeader +  ipHeaderLen;

	dstPortNum = htons(*((UINT16 *)(pUdpHeader + 2)));


    if(dstPortNum == 5008 )
	{
		*up = refPortToUP[demoPort];
		demoPort++;
		demoPort = demoPort%4;
		return vlanToACMap[*up];
	}
	else if(dstPortNum < 5000 || dstPortNum > 5003) 
	{
	   *up = 0;
	   return WDRV_QOS_CLASSIFIER_AC_BE;
	}
	else 
	{
		*up = refPortToUP[dstPortNum-5000];
		return vlanToACMap[*up];
	}
}
