/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*													   					  */
/*	  MODULE:	concatenator.c	                                          */ 
/*   PURPOSE:	Performs concatenation.				 					  */ 
/*																		  */
/**************************************************************************/

#include "pform_types.h"
#include "802_11Defs.h"
#include "memMngr.h"

#define SNAPDUP2(dst,src) { \
   UST2((dst),0) = UST2((src),0); \
   UST2((dst),2) = UST2((src),2); \
   UST2((dst),4) = UST2((src),4); \
   UST2((dst),6) = UST2((src),6); \
}

static void wdrv_txReplaceWlanHeader( mem_MSDU_T* msduPtr);

UINT8   tiSnapDataArray[8] = {0xAA,0xAA,0x03,0x08,0x00,0x28,0x60,0xD0};


/*************************************************************************
 *                        wdrv_txConcatMsduList                          *
 *************************************************************************
DESCRIPTION: This function get a list of MSDUs (The first MSDU pointes 
             on the others) and concatenate them into one MPDU by linking the
             buffers BDs. In return only the first MSDU struct is in use, 
             and it contains the entire concatenated MPDU.                
                                                                                                   
INPUT:       buildMsduPtr - Msdu containing the headers on base of which the
							concatenation is done.
			 firstMsduPtr - Pointer to the first MSDU.
             concatFlags - Concatenation flags.
             
             
OUTPUT:      firstMsduPtr - The msdu containig concatenated BDs.

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_txConcatMsduList( mem_MSDU_T *buildMsduPtr, mem_MSDU_T** firstMsduPtr, UINT16 concatFlags )
{
	mem_MSDU_T     *currMsduPtr;
	UINT8          *srcDataPtr;
	UINT8          *buildDataPtr;
    Wdrv4xHeader_T *w4xHeaderPtr;
    dot11_header_T *pDot11Hdr;
    UINT16         dot11HdrLen; 
	
	/* Create the first MSDU.(contains 802.11 header, TI wlan SNAP & Concat headr) */
	srcDataPtr = memMgr_MsduHdrAddr( *firstMsduPtr );
	buildDataPtr = memMgr_MsduHdrAddr( buildMsduPtr);

    /* Calculate 802.11 header length */
    dot11HdrLen = (concatFlags & WLAN_4X_CONCAT_AP_TO_AP) ? sizeof(dot11_header_T) : WLAN_HDR_LEN;
    pDot11Hdr = (dot11_header_T *) buildDataPtr;
    /* Copy the 802.11 header. */
#if 1
	PAL_memCopy( buildDataPtr, srcDataPtr, dot11HdrLen );
#else
   bcopyWords( srcDataPtr, buildDataPtr, dot11HdrLen/2);
#endif

   if(concatFlags & WLAN_4X_CONCAT_AP_TO_AP) {
       /* Duplicate RA(address1) to DA(address3) and TA(addres2) to SA(address4)*/
       MACDUP2(pDot11Hdr->address3.addr, pDot11Hdr->address1.addr);
       MACDUP2(pDot11Hdr->address4.addr, pDot11Hdr->address2.addr);
   }
   else {
       /* We send the frame from the AP so the BSSID is copied to the SA .*/
       MACDUP2( buildDataPtr+WLAN_SA_FIELD_OFFSET, buildDataPtr+WLAN_BSSID_FIELD_OFFSET);
   }
   buildDataPtr += dot11HdrLen;

	/* create a TI WLAN SNAP */
   SNAPDUP2( buildDataPtr ,tiSnapDataArray);
	buildDataPtr += WLAN_SNAP_HDR_LEN;
	
	/* create 4X header.   */
    w4xHeaderPtr =  (Wdrv4xHeader_T*)buildDataPtr;
    w4xHeaderPtr->type = WLAN_HEADER_TYPE_CONCATENATION;
    w4xHeaderPtr->headerLen = WLAN_CONCAT_HEADER_LEN;
	/* lior - endianess fix */
    w4xHeaderPtr->txFlags = PAL_BE2UINT16(concatFlags);
	
	buildMsduPtr->firstBDPtr->length = dot11HdrLen + WLAN_SNAP_HDR_LEN + 
										WLAN_4X_CONCAT_HDR_LEN;

    buildMsduPtr->dataLen =  dot11HdrLen + WLAN_SNAP_HDR_LEN + 
										WLAN_4X_CONCAT_HDR_LEN;

    buildMsduPtr->headerLen = dot11HdrLen;
    buildMsduPtr->nextConcatMSDU =  (*firstMsduPtr);

	/* Link the new MSDU to the first MSDU to imitate the MSDU list format.     */
	buildMsduPtr->firstBDPtr->nextBDPtr =  (*firstMsduPtr)->firstBDPtr;
	
	/* Start Concatenating the MSDUs, payload is copied from the SNAP to       */
	/* payload  end.                                                           */        
	currMsduPtr = *firstMsduPtr;
	while( 	currMsduPtr != NULL )
	{
        wdrv_txReplaceWlanHeader( currMsduPtr );

        /* Update the size of the concatenated MSDU. */		
        buildMsduPtr->dataLen += currMsduPtr->dataLen;

        if(  currMsduPtr->nextConcatMSDU != NULL )
        {
            /* Link last BD of the current MSDU to the first BD of the next MSDU.*/
             currMsduPtr->lastBDPtr->nextBDPtr = currMsduPtr->nextConcatMSDU->firstBDPtr;
		
            /* Jump for the next MSDU */
            currMsduPtr = currMsduPtr->nextConcatMSDU;
        }
        else {
			/* Last MSDU */
            currMsduPtr->lastBDPtr->nextBDPtr = NULL;
            currMsduPtr = NULL;
		}
	}

	*firstMsduPtr = buildMsduPtr;		
	
	return;
}/* END wdrv_txConcatMsduList() */


/*************************************************************************
 *                        wdrv_txReplaceWlanHeader                       *
 *************************************************************************
DESCRIPTION: This function replaces the 802.11 header with length + SA as
             prefix to the concatenated MSDU.                
                                                                                                   
INPUT:       msduPtr     - Pointer to the first MSDU.
             
             

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
void wdrv_txReplaceWlanHeader( mem_MSDU_T* msduPtr)
{
    UINT8   *firstDataBuf;
   register UINT     numOfPadBytes;
   
	/* Replace the 802.11 header with 2 length bytes and 6 SA . */
	firstDataBuf = msduPtr->firstBDPtr->data + 
						  msduPtr->firstBDPtr->dataOffset;
    {
        UINT16 *pDest, *pSrc;
        pDest = (UINT16 *)(firstDataBuf + WLAN_HDR_LEN - WLAN_SA_FIELD_LEN);
        pSrc  = (UINT16 *)(firstDataBuf + WLAN_SA_FIELD_OFFSET);
        *(pDest + 2) = *(pSrc + 2);
        *(pDest + 1) = *(pSrc + 1);
        *pDest       = *pSrc;
    
    }
    msduPtr->firstBDPtr->dataOffset += WLAN_CONCAT_HDR_OFFSET;
    msduPtr->firstBDPtr->length -= WLAN_CONCAT_HDR_OFFSET;
    msduPtr->dataLen -= WLAN_CONCAT_HDR_OFFSET;
    /* Fill the length bytes.   */
    /* lior - endianess fix */
    (*(UINT16*)(firstDataBuf+ WLAN_CONCAT_HDR_OFFSET)) = 
      PAL_BE2UINT16((UINT16)msduPtr->dataLen - WLAN_4X_LEN_FIELD_LEN);

   /* Padding the last buffer with zeros. 4 byte alignment */
   numOfPadBytes = ((UINT)(msduPtr->dataLen)) & 3;
      if( numOfPadBytes > 0 )
      { 
      /* padding required */
      UINT  i = 0;
      register UINT8 *tmpPtr;

      /* to padd */
      numOfPadBytes = (4 - numOfPadBytes);
          tmpPtr = (char*) ((UINT32)msduPtr->lastBDPtr->data + msduPtr->lastBDPtr->length +
              msduPtr->lastBDPtr->dataOffset);
     
      while( i < numOfPadBytes ) {
         tmpPtr[i++] = 0x00;
      }

      /* update length */
          msduPtr->lastBDPtr->length += numOfPadBytes;
          msduPtr->dataLen += numOfPadBytes;
      }
}/* END wdrv_txReplaceWlanHeader() */
