/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																	                  	   */
/*		MODULE:	envList.c											            	   */
/*    PURPOSE:	Envelope list implementation        				       	   */
/*																		                     */
/***************************************************************************/

#ifndef ENV_LIST_H
#define ENV_LIST_H

#include "pform_types.h"
#include "whal.h"
#include "memMngr.h"
#include "wlan_pal.h"


/* Defining an element in the envelop link list. */

/*************************************************/
/* Envelops definition.                          */
/*************************************************/
typedef struct TxEnvelop_T TxEnvelop_T;

struct TxEnvelop_T {
   UINT16         staIndex;     /* The station index.  */ 
   mem_MSDU_T    *msduPtr;      /* Pointer to the MSDU to be send. */
   ULONG          txFlags;      /* Transmition flags.  */
   UINT32         expirTick;    /* In which tick this pkt can be expired (Used for PWR save MNG)*/  
   TxEnvelop_T   *nextEnvPtr;   /* Pointer to the next envelop. (Uesd when in queue) */ 
   TxEnvelop_T   *prevEnvPtr;   /* Pointer to the previous envelop. (Uesd when in queue) */ 
   TxEnvelop_T   *nextSameStaEnv; /* Next envelop from the same station. (for concatenation) */		
   UINT8         woneIndex;  
}; 

/*************************************************/
/* Envelops link list definition.                */
/*************************************************/
typedef struct EnvList_T EnvList_T;
struct EnvList_T {
   TxEnvelop_T *first;      /* Points to the first envelop is the list. */
   TxEnvelop_T *last;       /* Points to the last envelop is the list. */
   UINT16   maxNumOfEnv;   /* The maximum number of envelops allowded to be in the queue. */
   UINT16   CurrNumOfEnv;  /* The current number of envelops.         */
   UINT16   maxStoredEnv;  /* Holds the maximum number of stored envelops */
};

/*************************************************************************
 *                        wdrv_txNewEnvList                              *
 *************************************************************************
DESCRIPTION: This function creates new envelop list.                 
                                                                                                   
INPUT:       maxNumOfElements : The maximum number of elements allwd inition)    
OUTPUT:                                                 
RETURN:      (a list of output parameters (returned by pointer) 
                       and their definitions)                                                 
                                                                                                   
************************************************************************/
EnvList_T* wdrv_txNewEnvList(UINT16 maxNumOfElements );

/*************************************************************************
 *                        wdrv_txFreeEnvList                              *
 *************************************************************************
DESCRIPTION: This function frees envelop list.                 
                                                                                                   
INPUT:   this - pointer to the envelope list    
OUTPUT:       
RETURN:      None                                                 
                                                                                                   
************************************************************************/
void wdrv_txFreeEnvList(EnvList_T *this);

/*************************************************************************
 *                        wdrv_txEnvListIns                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. If the 
                           list is full the first envelop will be taken 
                           out and will be returned in this pointer.

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS  wdrv_txEnvListIns( EnvList_T* this , TxEnvelop_T  **envelopPtr );

/*************************************************************************
 *                        wdrv_txEnvListInsFirst                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this in the first location.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. If the 
                           list is full the first envelop will be taken 
                           out and will be returned in this pointer.

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 
             wasQueueFull: Indicates if the list was full.

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS wdrv_txEnvListInsFirst (EnvList_T* this , TxEnvelop_T  **envelopPtr, BOOL *wasQueueFull );

/*************************************************************************
 *                        wdrv_txEnvListGetFirst                         *
 *************************************************************************
DESCRIPTION: This function takes the first envelop from the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListGetFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr);

/*************************************************************************
 *                        wdrv_txEnvListWatchFirst                       *
 *************************************************************************
DESCRIPTION: This function watch at the first envelop in the list. The 
             envelop is not removed yet from the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListWatchFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr);

/*************************************************************************
 *                        wdrv_txEnvListGetFirst                         *
 *************************************************************************
DESCRIPTION: This function takes the envelop from the middle of the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to take from. 

OUTPUT:      *envelopPtr : The envelop . 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListGetMiddle( EnvList_T *this, TxEnvelop_T  *envelopPtr);

/*************************************************************************
 *                        wdrv_txEnvListGetMaxStored                     *
 *************************************************************************
DESCRIPTION: This function returns maximal number of elements that were
			 stored in the specified list ever.
                                                                                                                              
INPUT:      this - pointer to the list. 

OUTPUT:      
RETURN:     maximal number of elements that were
			 stored in the specified list ever              
             
************************************************************************/
UINT16 wdrv_txEnvListGetMaxStored( EnvList_T *this );

/*************************************************************************
 *                   wdrv_txEnvListGetCurrentStored                      *
 *************************************************************************
DESCRIPTION: This function returns current number of elements stored in the list.
                              
                                                                                                   
INPUT:       this - pointer to the list.

OUTPUT:      
RETURN:      current number of elements stored in the list.
                            
************************************************************************/
UINT16 wdrv_txEnvListGetCurrentStored( EnvList_T *this );

/*************************************************************************
 *                        wdrv_txStaEnvListGetFirst                      *
 *************************************************************************
DESCRIPTION: This function takes the first envelop from a station list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.
************************************************************************/     
STATUS wdrv_txStaEnvListGetFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr);

/*************************************************************************
 *                        wdrv_txEnvListIns                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. If the 
                           list is full the first envelop will be taken 
                           out and will be returned in this pointer.

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS  wdrv_txStaEnvListIns( EnvList_T* this , TxEnvelop_T  **envelopPtr );

#endif /* ENV_LIST_H */
