/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*	  MODULE:	powerSave.h	                                                */ 
/*   PURPOSE:	Power Save module implementation.	 							   */ 
/*																									*/
/***************************************************************************/
#ifndef TX_POWER_SAVE_H
#define TX_POWER_SAVE_H


/*************************************************************************
 *                        wdrv_psInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of supported station.

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_psInit(UINT16 numOfStations );

/*************************************************************************
 *                        wdrv_psUnload                                    *
 *************************************************************************
DESCRIPTION: This function deinitializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of station in the system 

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_psUnload(UINT16 numOfStations);

/*************************************************************************
 *                        wdrv_psSetPsStatus                             *
 *************************************************************************
DESCRIPTION: This function called to report to the power save module of
			 changing ps status of specific STA.                 
                                                                                                   
INPUT:      staIndex - The station index.
			woneIndex - The station wone index.
			status - the new STA's power save status.

OUTPUT:      

RETURN:  TRUE  - PS in ON .  
         FALSE - PS is OFF.                                                                                                                                              
************************************************************************/
inline void wdrv_psSetPsStatus( UINT16 staIndex, UINT8 woneIndex, UINT16 status );

/*************************************************************************
 *                        wdrv_psGetStaStatus                            *
 *************************************************************************
DESCRIPTION: This function returns the power save status of the station.                
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - PS in ON .  
        FALSE - PS is OFF.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psGetStaStatus( UINT16 stationIndex );

/*************************************************************************
 *                        wdrv_psPsOnAndPksInQueue                       *
 *************************************************************************
DESCRIPTION: This function returns TRUE if the station is in PS mode and
             there are more packets in the PS queue.                 
                                                                                                   
INPUT:       staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - Station in PS and more packets to send .  
        FALSE - Otherwise.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psPsOnAndPksInQueue( UINT16 staIndex );

/*************************************************************************
 *                        wdrv_updateRecoveryFlag                        *
 *************************************************************************
DESCRIPTION: This function updates the 'firstTimeAfterRecoveryFlag' flag. 
             This flag is used to determine if to update the virtual bitmap
             when receiving a packet to put in the queue.                
                                                                                                   
INPUT:                  

OUTPUT:      

RETURN:                                                                                                                                                     
************************************************************************/
void wdrv_updateRecoveryFlag(void);


/*************************************************************************
 *                        wdrv_psStoreEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the corresponding PS envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                
************************************************************************/
STATUS wdrv_psStoreEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac );


/*************************************************************************
 *                        wdrv_psWatchEnv                                *
 *************************************************************************
DESCRIPTION: This function returns an indication whether there is at least
			 one envelope in the list. In case it is, the first envelope
			 pointer is returned in the envelopPtr.
                                                                                                   
INPUT:       staIndex - The station index.
                        

OUTPUT:      envelopPtr - Pointer to the first envelop in the list.

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psWatchEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac);

/*************************************************************************
 *                        wdrv_psGetEnv                                  *
 *************************************************************************
DESCRIPTION: This function retrieves envelope from the list.               
                                                                                                   
INPUT:       staIndex - The station index.
                        

OUTPUT:      envelopPtr - Pointer to the retrievde envelope. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psGetEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac);


/*************************************************************************
 *                        wdrv_psGetTotalStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode.

INPUT :
OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalStaInPSMode( void );

/*************************************************************************
 *                        wdrv_psGetTotalWoneStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode for wone.

INPUT : wone_Index : index of wone on which the number of STAs in PS requested.
OUTPUT:      

RETURN: The number of packets in PS mode for wone.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalWoneStaInPSMode( UINT8 wone_Index );

/*************************************************************************
 *                        wdrv_powerSavePrintDb                     *
 *************************************************************************
DESCRIPTION: Returns PS module statistics

INPUT : buf - buffer to be filled with the statistics
OUTPUT:      

RETURN: The length of data in the buffer.                                                                                                                                              
************************************************************************/
int wdrv_powerSavePrintDb(char *buf);

/*************************************************************************
 *                        wdrv_txPowerSaveAgingFunc                      *
 *************************************************************************
DESCRIPTION: Power Save aging management function.

OUTPUT:      

RETURN:                                                                                                                                               
************************************************************************/
void wdrv_txPowerSaveAgingFunc(void);

/****************************************************************************
 *                      wdrv_powerSaveCleanStaList()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.

 * INPUTS: stationIndex - The station index
 *		   woneIdx - The station wone index
 * 
 * OUTPUT:
 * 
 * RETURNS:
 ****************************************************************************/
void wdrv_powerSaveCleanStaList( UINT16 stationIndex, UINT8 woneIdx );

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFail()							*
 ****************************************************************************
 * DESCRIPTION: Return the status of previous Ps unicast packet send 
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * 
 * OUTPUT:	prevTxFail - Indecate if this station failed to send previous 
 *          unicast packet
 * 
 * RETURNS:
 ****************************************************************************/
void  wdrv_psGetPsPrevTxFail( UINT8 stationIndex ,UINT8 *prevTxFail);

/****************************************************************************
 *                      wdrv_psSetPsPrevTxFail()							*
 ****************************************************************************
 * DESCRIPTION: Set the PS Prev send fail flag.
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * RETURNS:
 ****************************************************************************/
void  wdrv_psSetPsPrevTxFail( UINT8 stationIndex);

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFailCounter()                       *
 ****************************************************************************
 * DESCRIPTION: return the number of station with PS Prev send fail flag on.
 * 
 * INPUTS:
 *
 * OUTPUT: 	PsTxFailurs - Number of station with PS Prev send fail flag on	      
 *		
 * RETURNS:
 ****************************************************************************/
inline void  wdrv_psGetPsPrevTxFailCounter(UINT8 *PsTxFailurs);

/*************************************************************************
 *                        wdrv_psPowerSaveStatusChangeFunc                *
 *************************************************************************
DESCRIPTION: This function called to update power sae status of specific STA.
                                                                                                   
INPUT:      staIndex - The station index.
		    status - The new status.
			woneIndex - The station wone index.

OUTPUT:      

RETURN:                                                                                                                                                
************************************************************************/
void wdrv_psPowerSaveStatusChangeFunc(UINT32 staIndex, UINT16 status, UINT8 woneIndex) ;

/****************************************************************************
 *                      wdrv_powerSaveCleanStaListFunc()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.

 * INPUTS: stationIndex - The station Index
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaListFunc( UINT16 stationIndex ) ;

/****************************************************************************
 *                      wdrv_psGetEnvList			                        *
 ****************************************************************************
 * DESCRIPTION: return a specific PsEnvList for a specific ac and station
 *
 * INPUTS:
 *
 * OUTPUT: 	       
 *		
 * RETURNS:
 ****************************************************************************/

STATUS wdrv_psGetEnvList( UINT16 staIndex, EnvList_T **envelopListPtr, wdrv_QoSClassifierAC_t ac);

/****************************************************************************
 *                      wdrv_psResetStaCounters		                        *
 ****************************************************************************
 * DESCRIPTION: reset counters for a specific station
 *
 * INPUTS:
 *
 * OUTPUT: 	       
 *		
 * RETURNS:
 ****************************************************************************/
void wdrv_psResetStaCounters( UINT16 staIndex);

/****************************************************************************
 *                      updatePartVirtBitmap		                        *
 ****************************************************************************
 * DESCRIPTION: update the partial virtual bitmap in beacon
 *
 * INPUTS:
 *
 * OUTPUT: 	       
 *		
 * RETURNS:
 ****************************************************************************/
STATUS updatePartVirtBitmap(UINT16 staIndex, UINT16 status );



#endif /* TX_POWER_SAVE_H */

