/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*********************************************************************************
*                                                                                     
*   MODULE:  whalEEPromLess.h                                                                            
*   PURPOSE: EEProm Less operation
*                                                                                                          
**********************************************************************************/
#ifndef _WHAL_EEPROM_LESS_H
#define _WHAL_EEPROM_LESS_H


/********************************************/
/*			EEPROM LESS						*/
/********************************************/


/****************************
 EEPROMless support constants
*****************************/
#define EEPROM_BR_ENTRY_LEN					1			// in units of 4 bytes
#define EEPROM_BR_ENTRY_ADDR_LEN			2			// in bytes
#define EEPROM_BR_ENTRY_DATA_START_OFFSET	((EEPROM_BR_ENTRY_LEN) + (EEPROM_BR_ENTRY_ADDR_LEN))
#define EEPROM_TABLE_OFFSET_LEN             2			// in bytes
#define EEPROM_BR_ENTRY_DATA_UNIT_LEN		4			// in bytes
#define NVS_DATA_BUNDARY_ALIGNMENT			4


#define HW_EEPROM_PRESENTED					 1
#define HW_EEPROM_NOT_PRESENTED				 0

/*
 * EEPROMless support registers
*/
#define ACX_EEPROMLESS_IND_REG				(SCR_PAD2)
#define ACX_EEPROMLESS_HAND_SHAKE_REG       (SCR_PAD1)
#define RESUME_FW_RUN_INDICATION             0xFF

#define HW_EEPROM_CARD_TYPE_ADDR	0x004

#define EE_START_MAX_TIME		3000000
#define EE_START_STALL_TIME		1000
#define SOFT_RESET_MAX_TIME		1000000
#define SOFT_RESET_STALL_TIME	1000

/* Inon Debug */
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
#define EEPROM_LEN		353
#else /*130*/ 
#define EEPROM_LEN		493
#endif

#define MAX_EEPROM_LEN  1000

/*#define DEBUG_EPROMLESS*/


/*
 * Eepromless API
 */


typedef struct 
{
	UINT16 tblsOffset;
	UINT16 NvsDataLen;
	UINT32 SavedEEPROMlessInd;
} EepromlessCfg_T;

typedef struct {
	UINT8 image[MAX_EEPROM_LEN];
	UINT32 len;
} eepromImage_t;

UINT32 whal_FwCtrl_Eepromless_PrepareBoot ( EepromlessCfg_T *pEepromlessCfg);
int whal_FwCtrl_Eepromless_AfterBoot( EepromlessCfg_T *pEepromlessCfg, UINT32 AcxMemAddr);
UINT32 whal_FwCtrl_Eepromless_GetRadioType(void);

void whal_hwCtrl_ProcessEepromBREntry( UINT8 *pCurrEntry);
void whal_hwCtrl_ProcessEepromBR( UINT16 *Offset);
UINT16 whal_hwCtrl_GetNvsDataSize( UINT16 TblPtrOffset);
void whal_hwCtrl_CopyNvsData(UINT16 tblsOffset, UINT16 NvsDataLen, UINT32 AcxMemAddr);
BOOL whal_hwCtrl_WaitFwWhenEepromless( UINT32 WaitRegVal);
void HexDumpEep ( UINT8 *data, int datalen);
UINT16 whal_hwCtrl_ReadEepromU16value(UINT8  *pCurrLocation);
UINT32 whal_hwCtrl_ReadEepromU32value(UINT8  *pCurrLocation);
UINT32 whal_hwCtrl_Eepromless_ReadByte(UINT16 wAddr,UINT8 *pbVal);

STATUS whal_FwCtrl_Eepromless_SetImage(eepromImage_t *image);

//BOOL os_getEeepromImage(UINT8 **pBuffer,UINT32 *length);

#endif
