/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE:  whalHostDesc.h
 *   PURPOSE: Rx Host Descriptor Manager - Header file
 *
 ****************************************************************************/

#ifndef WHAL_RX_HOST_DESC_H
#define WHAL_RX_HOST_DESC_H 

#include "acx_hw.h"		/* for USE_ENDIAN_SWAP and WLAN_ADAPTOR_VERSION_1150 definitions */

/****************************
 * Constants
 ****************************/
/*
 *  Host Description Definitions.        
 */
#define ACX_HOST_DESC_STAT_ERR_MASK         0x7 /* dma error mask, parse in case of error */
#define ACX_HOST_DESC_STAT_TAR_ABORT        0x1 /* dma error codes */
#define ACX_HOST_DESC_STAT_MASTER_ABORT     0x2
#define ACX_HOST_DESC_STAT_REQ_BEF_COMP     0x3
#define ACX_HOST_DESC_STAT_UNEXP_HOST       0x4
#define ACX_HOST_DESC_STAT_UNEXP_ACX        0x5
#define ACX_HOST_DESC_STAT_PCI_DISABLED     0x6

#define ACX_HOST_DESC_STAT_DMA_ERR          (1ul<<7)   /* DMA error during transaction */

#define ACX_HOST_DESC_STAT_OWN             (1ul<<31) /* Set when host has the descriptor,
                                        should be cleared by host when the host is done */

#define ACX_HOST_DESC_STAT_ALIGN_BIT_LOCATION 8
#define ACX_HOST_DESC_STAT_ALIGNMENT       (1ul<<ACX_HOST_DESC_STAT_ALIGN_BIT_LOCATION) /* bit indicating whether status begins with 2 bytes offset from block ptr */


/****************************
 * Structures Definitions
 ****************************/
#pragma pack(2)

typedef struct
{
  UINT32	blk_ptr;  /* Pointer to data buffer (in ACX address space). */
 /* 
  * In the TNETW1150, use the structure as in the DPG and dont swap words
  * The fix is in the VLYNQ conversion and in GLB_CFG register
  */
#if (USE_ENDIAN_SWAP == 1 && ( WLAN_ADAPTOR_VERSION != WLAN_ADAPTOR_VERSION_1150)) /*BIG ENDIAN MODE */
  UINT16 reserved;
  UINT16	blk_offset;
  UINT16	blk_len;
  UINT16 ctrl;          /* should be 0 when host sends data, and cleared when
                           host gets data */

#else
  UINT16	blk_offset;
  UINT16	reserved;
  UINT16	ctrl;          /* should be 0 when host sends data, and cleared when host gets data */
  UINT16	blk_len;
#endif

  UINT32	next_desc_ptr; /*Pointer to the next desc in the list.
  								(in ACX address space)*/
  /* valid in rx */
  UINT32	next_desc_ptr_to_use;   /* put 0 */
  UINT32 status;        /* bit31 is own, sould be 0 by the host */
 
  /**  DRIVER SPACE (HostCookie) ****/
  UINT32    msduPtr;      /* Pointer to the MSDU handler, used for free func
                             after the sending. */
  UINT32   bufToken;      /* aligning stucture to 32 bytes for cache invalidate (2 lines) */
  UINT8    DriverWorkspace[32];
} HostDesc_T;
#pragma pack()

/* managed by host, no need of pack() */
typedef struct
{
  UINT32	blk_ptr;  /* Pointer to data buffer (in ACX address space). */
  UINT16	blk_offset;
  UINT16 ctrl;
  UINT16	blk_len;
  UINT32	next; /*Pointer to the next desc in the list.
  								(in ACX address space)*/
  UINT32 status;
  UINT32 msduPtr;
  UINT32 bufToken;
} HostDescAttr_T;

/*********************************
 * External Functions Prototypes
 *********************************/
void whal_HostDesc_setNextDesc( HostDesc_T* hostDesc, HostDesc_T *nextHostDesc );
HostDesc_T* whal_HostDesc_getNext(  HostDesc_T* this );
void whal_HostDesc_setBuffAddr(  HostDesc_T* this, UINT8* dataBuff , UINT32 bufToken, UINT16 len);
void whal_HostDesc_getDataBuff(  HostDesc_T* this, UINT8** dataBuff, UINT16* len );
void whal_HostDesc_getAttr(  HostDesc_T* this, HostDescAttr_T* attr );
void whal_HostDesc_clearToken( HostDesc_T* this);
void whal_HostDesc_setCtrl(  HostDesc_T* this, UINT16 ctrl);
void whal_HostDesc_setStatus(  HostDesc_T* this, UINT32 status);
void whal_HostDesc_setAttr(  HostDesc_T* this, HostDescAttr_T* attr );


#endif /* WHAL_RX_HOST_DESC_H */

