/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE:  whalTxQueue.h
 *   PURPOSE: WLAN chip Tx Queues manager
 * 
 ****************************************************************************/

#ifndef _WHAL_TX_QUEUE_H
#define _WHAL_TX_QUEUE_H

/****************************
 * HAL Include files
 ****************************/
#include "AcxDataIf.h"
#include "whalTxDesc.h"
#include "whalHostQueue.h"

/****************************
 * Constants
 ****************************/
#define ALL_TX_QUEUES   NUM_TX_QUEUES
#define QIND_MAX_COUNT     10
#define QIND_DELAY_VALUE   10

/****************************
 * Data Structures
 ****************************/
typedef struct 
{
   UINT32      NumDesc;        /* Number of descriptors in the queue */
   TxDesc_T    *HwBaseAddr;    /* ACX100 address of the queue        */
   TxDesc_T    *pFree;         /* first free queue descriptor        */
   TxDesc_T    *pAlloc;        /* first allocate queue descriptor    */
   int         NumFree;        /* Number of free descriptors         */
   
   UINT32      RtsFailCounter; /* total RTS failures    */
   UINT32      RtsSuccCounter; /* total RTS scuccesses  */
   UINT32      AckFailCounter; /* total ACK failures    */
   UINT32      RetryCounter;   /* total retries counter */
   UINT32      MultipleRetryCounter; /* multiple retries counter */
   UINT32      RetryFailCounter;     /* retry failures counter   */
   UINT32      OtherFailCounter;     /* other failures counter   */
   UINT32      SuccCounter;          /* success counter          */
   
   UINT32      DmaErrorCounter;            /* DMA errors */
   UINT32      TimeoutErrorCounter;        /* Timeout errors */
   
   /* Details about DMA errors */
   UINT32      DmaErrorInProgress;     /* DMA is still on progress */
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1130)
   UINT32      DmaErrorNoErr_spur;     /* false detection of DMA error */
   UINT32      DmaErrorTargetAbort;    /* Target Abort */
   UINT32      DmaErrorMasterAbort;    /* Master Abort */
#endif   /*  WLAN_ADAPTOR_VERSION_1130 */
   UINT32      DmaErrorClearedAbort;   /* Request cleared before operation completion */
   UINT32      DmaErrorHostEndAbort;   /* Unexpected end of host descriptor list */
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
   UINT32      DmaErrorPrevError;      /* error in previous transaction */
   UINT32      DmaLocalReadError;      /* local read error */
   UINT32      DmaLocalWriteError;     /* local write error */
   UINT32      DmaErrorHostOwnError;   /* own error in descriptor */
#else /* not WLAN_ADAPTOR_VERSION_1150 */
   UINT32      DmaErrorAcxEndAbort;    /* unexpected end of ACX memory list */
   UINT32      DmaPciDisabled;         /* pci not enabled */
#endif   /* WLAN_ADAPTOR_VERSION_1150 */
   UINT32      DmaErrorUnspecified;    /* undocumented/unspecified error */
   UINT16 txOpLimit;
	UINT8        acID; /* AC ID (in FW notation - BE = 1) */
} TxQueue_T;

/****************************
 * Function Definitions
 ****************************/
extern int       whal_txQueue_Init(TxQueueId_T TxQid, int NumTxAcxDesc, UINT16 txOpLimit, UINT8 acID);
extern UINT32    whal_txQueue_Config(TxQueueId_T TxQid, UINT32 TxDesc_OffsetAddr);
extern TxDesc_T *whal_txQueue_Alloc(TxQueueId_T Qid);
extern int       whal_txQueue_Free(TxQueueId_T Qid, HostQueue_Obj* phostQueue);
extern UINT32    whal_txQueue_GetSize(TxQueueId_T Qid);
extern UINT32    whal_txQueue_GetBaseAddr(TxQueueId_T Qid);
extern UINT32    whal_txQueue_GetNumFree(TxQueueId_T Qid);
extern int       whal_txQueue_Print(TxQueueId_T Qid,char *buf);
extern int       whal_txQueue_InfoPrint(TxQueueId_T Qid,char *buf);
extern void      whal_txIndicatorSetAddr(UINT32 QueueIndicatorAddress);
extern int       whal_txIndicatorSet(TxQueueId_T Qid);

extern UINT16 whal_txQueueGetTxOpLimit(TxQueueId_T Qid);

/*
 * Counters api
 */
extern int      whal_txQueue_GetRetryFailCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetRetryCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetMultipleRetryCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetAckFailCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetRtsFailCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetRtsSuccCount(TxQueueId_T Qid);
extern int      whal_txQueue_GetSuccCount(TxQueueId_T Qid);

#endif   /* _WHAL_TX_QUEUE_H */
