/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda.h                                                
 *   \brief PURPOSE: headers and definitions for DDA
 *                                                                         
 */
 
#ifndef __TIWLAN_DDA_H
#define __TIWLAN_DDA_H

#include <linux/module.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/psp_trace.h>

#if     defined(CONFIG_MIPS_WIFI_1130_LED)
#define WLAN_WIFI_1130_LED
#undef  WLAN_WIFI_PSP_LED
#undef  WLAN_WIFI_PSP_COLORED_LED
#elif   defined(CONFIG_MIPS_AVALANCHE_LED)
#undef  WLAN_WIFI_1130_LED
#define WLAN_WIFI_PSP_LED
#undef  WLAN_WIFI_PSP_COLORED_LED
#elif   defined(CONFIG_MIPS_AVALANCHE_COLORED_LED)
#undef  WLAN_WIFI_1130_LED
#undef  WLAN_WIFI_PSP_LED
#define WLAN_WIFI_PSP_COLORED_LED
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#include <linux/tqueue.h>
#else
#include <linux/workqueue.h>
#endif
#include <linux/if.h>
#include <asm-mips/addrspace.h>
#ifdef WLAN_WIFI_PSP_COLORED_LED
#include <asm/avalanche/generic/led_manager.h>
#endif

#include "whal.h"
#include "drvDbDefs.h"
#include "memMngr.h"
#include "whalAcxRegs.h"


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif

#ifndef vbase1
# define vbase1 TNETD53XX_PUMA_VLBASE_1
#else
# error "name clashing, vbase1 is defined"
#endif

/* we want dynamic name */
/*TODO: pass the dynamic name to the user*/
#define DDA_DEVNAME	"wlan%d"

#define DDA_DRV_RECEIVED_GO (1)
#define DDA_DRV_STARTED	(1<<2)

#if defined (CONFIG_MIPS_AVALANCHE_PROTOCOL_INSPECT)
#define PSP_PIL_QOS
typedef enum
{
    TIWLAN_PIL_UN_TRUSTED = 0,
    TIWLAN_PIL_TRUSTED
} TIWLAN_PIL_MODE;
#endif

struct dda_priv {
	struct net_device_stats stats;
	__u32 wpa_authenticator_pid, driver_started;
	struct sock * user_nl;
#ifdef  TIWLAN_FW_TRACE
	__u32 fw_trace_app_pid;
#endif
    __u32 wds_manager_pid;
    #ifdef NSP_WAN_BRIDGE
		void *pWanBridgeHandle;
    #endif /* ifdef NSP_WAN_BRIDGE */
#ifdef PSP_PIL_QOS
        TIWLAN_PIL_MODE pil_mode;
#endif
};

struct wds_priv {
	struct net_device_stats stats;
    char linkPartner[ETH_ALEN];
#ifdef PSP_PIL_QOS
        TIWLAN_PIL_MODE pil_mode;
#endif
};


/* wireless extensions staff */
#define DDA_IOCTL_PARAM (SIOCIWFIRSTPRIV + 0)
#define DDA_IOCTL_GET_PARAM (SIOCIWFIRSTPRIV + 1)








typedef enum {
    TIWLAN_ALLOC_INIT_PHASE,
    TIWLAN_ALLOC_RUNTIME_PHASE
} tiwlan_allocPhase_t;



void tiwlan_ddaDeleteMacAddrFunc(char *mac);
UINT8 tiwlan_ddaRand(void);
void tiwlan_ddaSrand(UINT16 seed);
STATUS 	tiwlan_ddaIsMemMngrInitNeeded(void);
STATUS tiwlan_ddaReceive(mem_MSDU_T *);

#define TIWLAN_DDA_BUFFER_FROM_TOKEN(token) (((struct sk_buff *)token)->data)
STATUS tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, UINT8 ** buf, tiwlan_allocPhase_t phase);
STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size );

typedef enum {
	TIWLAN_NETLINLK_DESTINATION_WPA,
	TIWLAN_NETLINK_DESTINATION_FW_TRACE,
    TIWLAN_NETLINK_DESTINATION_WDS_MNGR
} tiwlan_ddaNetlinkDestination_t;

typedef enum {
    TIWLAN_SOFT_RESET,
    TIWLAN_HARD_RESET,
}TIWLAN_RESET_TYPE_E;


STATUS tiwlan_ddaNetLinkSend(void * data, int data_len, tiwlan_ddaNetlinkDestination_t dst);

STATUS tiwlan_ddaCreateNetlinkSocket(void);

STATUS tiwlan_ddaGo(initConfigStruct *init_cfg);

STATUS tiwlan_dda_wpaGo(void);

STATUS tiwlan_dda_waitForWpa(void);

STATUS tiwlan_ddaInitInterrupts(void);

STATUS tiwlan_ddaSuspend(void);
STATUS tiwlan_ddaResume(void);

STATUS tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr);
STATUS tiwlan_ddaUnloadPSPVlynq(void);
STATUS tiwlan_ddaReInit(void);
STATUS tiwlan_ddaRecvWdsPkt(UINT8 *pBuff, UINT32 length);
STATUS tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr);
void   tiwlan_setAcxResetType(TIWLAN_RESET_TYPE_E type);
TIWLAN_RESET_TYPE_E tiwlan_getAcxResetType(void);

int tiwlan_setupWdsNetDevs(int wdsDlDevsNumber);

#define tiwlan_ddaRxFreeBuffer(x) \
do{ \
	if (!(((struct sk_buff *)x)->list)) dev_kfree_skb((struct sk_buff *)x);\
	else printk("kfree_skb warning: the skb is on the list !!!\n");\
} while (0);

#define boardInitAcx111Addr(RegAddr, MemAddr) tiwlan_ddaInitPSPVlynq(RegAddr, MemAddr)
#define boardUnloadAcx111() tiwlan_ddaUnloadPSPVlynq()

STATUS wdrv_DDA_queue_stop(void);
STATUS wdrv_DDA_queue_start(void);

#ifdef TIWLAN_DEBUG
void inline bm_trace(int loc, unsigned long p1, unsigned long p2);
void bm_print_to_console(void);
void bm_disable(void);
#else
#define bm_init()
#define bm_trace(a, b, c) 
#define bm_shutdown()
#define bm_disable()

#define bm_print_to_console()
#endif




#ifdef DDA_WLAN_LED

#ifdef WLAN_WIFI_PSP_COLORED_LED
#define WLAN_LED_STATE_LINK_DOWN   0x0
#define WLAN_LED_STATE_LINK_UP     0x1
#define WLAN_LED_STATE_PKT_RCV     0x2
#define WLAN_LED_STATE_PKT_XMIT    0x3
#else
#define TIAP_LED_OFF		0
#define TIAP_LED_ON	        1
#define TIAP_LED_BLINK_OFF	2
#define TIAP_LED_BLINK_ON	3
#define TIAP_LED_FLASH		4
#endif

extern UINT8 wdrv_dbGetApBridgeMode(void);

#if defined(WLAN_WIFI_1130_LED)

	extern atomic_t tiap_led_oper_state, tiap_led_blinked;
	extern UINT16 tiap_activity_led_mask;
	
	extern __u8 tiap_gpio_owner;
	extern void whal_apiMacWrite(UINT32 MacRegAddr, UINT32 DataVal);
	
	__inline__ static void tiwlan_ddaLedAction(int action)
	{
		/* It mustn't be a function call in an inline function like this one.
		 * Performance is crucial.
		 */
		if (wdrv_dbGetApBridgeMode() || !tiap_gpio_owner)
			return;
	
		switch(action)
		{
			case TIAP_LED_OFF:
				whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
				atomic_set(&tiap_led_oper_state, action);
				break;
			case TIAP_LED_ON:
				whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
				atomic_set(&tiap_led_oper_state, action);
				break;
			case TIAP_LED_BLINK_OFF:
			case TIAP_LED_BLINK_ON:
				atomic_set(&tiap_led_blinked, 1);
				break;
		}
	}

	#define PAL_ConfigVWiLeds whal_acxConfigVWiLeds
	#define TIWLAN_LED_LINK_UP		tiwlan_ddaLedAction(TIAP_LED_ON)
	#define TIWLAN_LED_LINK_DOWN	tiwlan_ddaLedAction(TIAP_LED_OFF)
	#define TIWLAN_LED_PKT_RCV		tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF)
	#define TIWLAN_LED_PKT_XMIT		tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF)
#elif defined(WLAN_WIFI_PSP_LED)
	extern void *wlan_led;
	#define PAL_ConfigVWiLeds()
	#define tiwlan_ddaLedAction(action)	\
		if (!wdrv_dbGetApBridgeMode())	\
			avalanche_led_action(wlan_led, action)
	#define TIWLAN_LED_LINK_UP		tiwlan_ddaLedAction(TIAP_LED_ON)
	#define TIWLAN_LED_LINK_DOWN	tiwlan_ddaLedAction(TIAP_LED_OFF)
	#define TIWLAN_LED_PKT_RCV		tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF)
	#define TIWLAN_LED_PKT_XMIT		tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF)
#elif defined(WLAN_WIFI_PSP_COLORED_LED)
	extern void *wlan_led_handle;
	#define PAL_ConfigVWiLeds()
	#define tiwlan_ddaLedAction(state_id)	\
		if (!wdrv_dbGetApBridgeMode())	\
			led_manager_led_action(wlan_led_handle, state_id)
	
	#define TIWLAN_LED_LINK_UP   tiwlan_ddaLedAction(WLAN_LED_STATE_LINK_UP)
	#define TIWLAN_LED_LINK_DOWN tiwlan_ddaLedAction(WLAN_LED_STATE_LINK_DOWN)
	#define TIWLAN_LED_PKT_RCV   tiwlan_ddaLedAction(WLAN_LED_STATE_PKT_RCV)
	#define TIWLAN_LED_PKT_XMIT  tiwlan_ddaLedAction(WLAN_LED_STATE_PKT_XMIT)
#endif /* WLAN_WIFI_1130_LED */
#else
	#define TIWLAN_LED_LINK_UP
	#define TIWLAN_LED_LINK_DOWN
	#define TIWLAN_LED_PKT_RCV
	#define TIWLAN_LED_PKT_XMIT
#endif /* DDA_WLAN_LED */

#define XXstr(x) (x)

/****************************************************************************/
/********************PSP Instrumentation support ****************************/
/****************************************************************************/

/* AP execution time */
/* Event Id AP_DRV_ISR_ENTER is traced without parameters */
#define TIAP_TRACE_DRV_ISR_ENTER()          psp_trace(AP_DRV_ISR_ENTER)

/* Event Id AP_DRV_ISR_EXIT is traced with interrupt reason parameters */
#define TIAP_TRACE_DRV_ISR_EXIT(reason)     psp_trace_par(AP_DRV_ISR_EXIT,reason)

/* Event Id AP_DRV_TASKLET_ENTER is traced with tasklet id parameter */

#define TIAP_TRACE_DRV_TASKLET_ENTER(tsklId)    psp_trace_par(AP_DRV_TASKLET_ENTER,tsklId)

/* Event Id AP_DRV_TASKLET_EXIT is traced with tasklet id parameter */

#define TIAP_TRACE_DRV_TASKLET_EXIT(tsklId)    psp_trace_par(AP_DRV_TASKLET_EXIT,tsklId)

/* Tasklet Id definition */

typedef enum
{
    TIAP_TRACE_TASKLET_MAIN,
    TIAP_TRACE_TASKLET_MBOX
} TIAP_TRACE_TASKLET_ID_E;

/* Timers */

/* Event Id AP_DRV_TMR_ENTER is traced with timer id parameter */

#define TIAP_TRACE_DRV_TMR_ENTER(tmr)           psp_trace_par(AP_DRV_TMR_ENTER,tmr)

/* Event Id AP_DRV_TMR_EXIT is traced with timer id parameter */

#define TIAP_TRACE_DRV_TMR_EXIT(tmr)            psp_trace_par(AP_DRV_TMR_EXIT,tmr)

/* Timer ID definition */

typedef enum
{
    TIAP_TRACE_TMR_LED,
    TIAP_TRACE_TMR_RESTART,
    TIAP_TRACE_TMR_AGING,
    TIAP_TRACE_TMR_DYNAMIC_ALGORITHM,
    TIAP_TRACE_TMR_STA_AUTH,
    TIAP_TRACE_TMR_STA_DEAUTH,
    TIAP_TRACE_TMR_ERP_BEACON_AGING,
    TIAP_TRACE_TMR_PBCC_BEACON_AGING,
    TIAP_TRACE_TMR_RADIO_CALIB,
    TIAP_TRACE_TMR_CRT_RADIO_CALIB,
    TIAP_TRACE_TMR_RX,
    TIAP_TRACE_TMR_PS_AGING,
    TIAP_TRACE_TMR_TX_WAIT_LIST,
    TIAP_TRACE_BSS_BRIDGE_ISOLATION

} TIAP_TRACE_TMR_ID_E;

/* AP TX data path */

/*  Event id AP_DRV_PKT_TX_ENTER is traced with two parameters, 
    when first parameter is skb pointer and second identifies an entry point 
    or internal packet ( such as management frames ) */
// #define TIAP_TRACE_PKT_TX_ENTER(ptr,entrypoint)  psp_trace_2_par(AP_DRV_PKT_TX_ENTER,ptr,entrypoint)
/* We change this macro for nsp because they want to measure only the xmit path*/
#define TIAP_TRACE_PKT_TX_ENTER(ptr,entrypoint)  psp_trace_2_par(AP_DRV_PKT_TX_ENTER,0,0)

/* Entry points definition */
typedef enum {
    TIAP_TRACE_XMIT = 0,
    TIAP_TRACE_XMIT_WDS,
    TIAP_TRACE_XMIT_WDS_PKT,
    TIAP_TRACE_WLAN_RX_BRDCST,
    TIAP_TRACE_WLAN_RX_UNICAST,
    TIAP_TRACE_EAPOL,
    TIAP_TRACE_PROBE_RSP,
    TIAP_TRACE_AUTH,
    TIAP_TRACE_ASS_RES

} TIAP_TRACE_ENTRY_POINT_E;

/*  Event id AP_DRV_PKT_TX_EXIT is traced with two parameters, 
    when first parameter is skb pointer and second is Tx result */

/* We change this macro for nsp because they want to measure only the xmit path*/
#define TIAP_TRACE_PKT_TX_EXIT(ptr,result)    

#define TIAP_TRACE_PKT_TX_EXIT_FOR_NSP(ptr,result)      psp_trace_2_par(AP_DRV_PKT_TX_EXIT,0,0)

/* Tx result definition */

typedef enum {
    TIAP_TRACE_TX_RESULT_OK,
    TIAP_TRACE_TX_RESULT_NOK,
    TIAP_TRACE_TX_RESULT_NO_BD_FOR_HEADER,
    TIAP_TRACE_TX_RESULT_ALLOC_MSDU_FAIL,
    TIAP_TRACE_TX_RESULT_ALLOC_BD_FAIL

} TIAP_TRACE_TX_RESULT_E;

/* Event Id AP_DRV_PKT_TX_COMPLETE_ENTER is traced with one parameter (skb pointer)*/
#define TIAP_TRACE_PKT_TX_COMPLETE_ENTER(par)    psp_trace_par(AP_DRV_PKT_TX_COMPLETE_ENTER,par)

/* Event Id AP_DRV_PKT_TX_COMPLETE_EXIT is traced with one parameter (skb pointer)*/
#define TIAP_TRACE_PKT_TX_COMPLETE_EXIT(par)    psp_trace_par(AP_DRV_PKT_TX_COMPLETE_EXIT,par)

/* Event Id AP_DRV_TX_ENTER is traced without parameters */
#define TIAP_TRACE_DRV_TX_ENTER()               psp_trace(AP_DRV_TX_COMPLETE_ENTER)

/* Event Id AP_DRV_TX_EXIT is traced with one parameter */
#define TIAP_TRACE_DRV_TX_EXIT(par)             psp_trace_par(AP_DRV_TX_COMPLETE_EXIT,par)

/* Driver Tx Exit status definition */

typedef enum
{
    TIAP_TRACE_DRV_TX_EXIT_OK,
    TIAP_TRACE_DRV_TX_EXIT_CONTINUE

} TIAP_TRACE_DRV_TX_EXIT_STATUS;

/* AP Rx data path */

/* Event Id AP_DRV_RX_ENTER is traced without parameters */

#define TIAP_TRACE_DRV_RX_ENTER()           psp_trace(AP_DRV_RX_ENTER)

/* Event Id AP_DRV_TX_EXIT is traced with one parameter */
#define TIAP_TRACE_DRV_RX_EXIT(par)         psp_trace_par(AP_DRV_RX_EXIT,par)

/* Driver Tx Exit status definition */

typedef enum
{
    TIAP_TRACE_DRV_RX_EXIT_OK,
    TIAP_TRACE_DRV_RX_EXIT_CONTINUE

} TIAP_TRACE_DRV_RX_EXIT_STATUS;

/* Event Id AP_DRV_PKT_RX_ENTER is traced with one parameter ( skb pointer ) */

#define TIAP_TRACE_DRV_PKT_RX_ENTER(ptr)            psp_trace_par(AP_DRV_PKT_RX_ENTER,ptr)

/*  Event Id AP_DRV_PKT_RX_EXIT is traced with two parameters ( skb pointer and
    Rx result ) */

#define TIAP_TRACE_DRV_PKT_RX_EXIT(ptr,result)   psp_trace_2_par(AP_DRV_PKT_RX_EXIT,ptr,result)

/* Rx result definition */

typedef enum {
    TIAP_TRACE_RX_RESULT_OK,
    TIAP_TRACE_RX_RESULT_NOK,
    TIAP_TRACE_RX_RESULT_DMA_ERROR,
    TIAP_TRACE_RX_RESULT_MAX_DECONACT_LEN,
    TIAP_TRACE_RX_RESULT_ALLOC_BD_FAIL,
    TIAP_TRACE_RX_RESULT_ALLOC_MSDU_FAIL,
    TIAP_TRACE_RX_RESULT_FRAME_FILTERED,
    TIAP_TRACE_RX_RESULT_NULL_DATA_FRAME

} TIAP_TRACE_RX_RESULT_E;

/* QOS Support */
/* Function prototypes */

/* Get pointer to network device for WDS */
#define tiwlan_dda_get_wds_device(TA)  wdrv_wdsInterfaceCookieGet(TA)

#ifdef PSP_PIL_QOS

int tiwlan_ddaL2_pil_classify(mem_MSDU_T * pMSDU, void *netdevice);
int tiwlan_ddaL3_pil_classify(mem_MSDU_T * pMSDU, void *netdevice, void *ptr_iphdr);
int tiwlan_dda_pil_get_mode(void *net_device);
int tiwlan_dda_eapol_pil_classify(mem_MSDU_T * pMSDU, void *netdevice);
/* This function returns pointer to wlan_dev */
void *tiwlan_dda_get_wlan_device(void);

#else
#define tiwlan_ddaL2_pil_classify(msdu,dev)             (1)
#define tiwlan_ddaL3_pil_classify(msdu,dev,iphdr)       (1)
#define tiwlan_dda_eapol_pil_classify(msdu,dev)         (1)
#define tiwlan_dda_pil_get_mode(dev)                    (TIWLAN_PIL_UN_TRUSTED)
#define tiwlan_dda_get_wlan_device()                    (0)
#endif
void tiwlan_dda_set_pil_mode(int val);
int tiwlan_dda_print_pil_mode(char *buf);
#endif //__TIWLAN_DDA_H


