/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda_wext_ioctl.c                                                
 *   \brief PURPOSE: implementation of public ioctl interface 													       
 *                                                                         
 */
#include "wlan_pal.h"
#include <asm/uaccess.h>
#include "mgmtSsidHandler.h" 
#include "whal.h"
#include "QoS/QoSClassifier.h"
#include "QoS/QoSACParams.h"
#include "dda.h"
#include "dda_wext_ioctl.h"
#include "drvDbDefs.h"
#include "drvDbRegDomain.h"
#include "drvDbHash.h"
#include "mgmtAccess.h"
#include "mainsm.h"
#include "mgmtSm.h"
#include "Shutdown.h"  
#include  "dda_wpa.h"
#include "bssbridge.h"
#include "rx.h"
#include "drvDbHash.h"
#include "mgmtSsidHandler.h" 
#include "dda_wpaauth_ioctl.h"


/*! \brief This struct contains driver initial configuration.
		   It is filled and passed to the driver at init time (before GO command)
 */
static initConfigStruct init_cfg;


/*! \brief This struct contaions a field for each initial driver configuration parameter
		   The field is non-zero if respective parameter has been configured 
*/
struct configured_params configured_params;



/*! \brief a boolean variable that is true when it's a runtime cofiguration */
atomic_t after_go;

/*! \brief a semaphore for  synchronizing access to the config struct */
static struct semaphore sem;  


/* static functions */
static int dda_ioctl_priv_basic_param(struct net_device *, struct iw_request_info *, void *, char *);
static int dda_ioctl_priv_get_basic_param(struct net_device *, struct iw_request_info *, void *, char *);
static int dda_ioctl_priv_go(struct net_device *, struct iw_request_info *, void *, char *);
static int dda_ioctl_giwname(struct net_device *, struct iw_request_info *, char *, char *);
static int dda_ioctl_siwrts(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_giwrts(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_siwfrag(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_giwfrag(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_siwretry(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_giwretry(struct net_device *dev, struct iw_request_info *info,struct iw_param *rts, char *dummy);
static int dda_ioctl_siwtxpow(struct net_device *, struct iw_request_info *, struct iw_point *, char *);
static int dda_ioctl_giwtxpow(struct net_device *, struct iw_request_info *, struct iw_point *, char *);
static int dda_ioctl_priv_complex_param(struct net_device *, struct iw_request_info *,
					union iwreq_data *, char *);
static int dda_ioctl_priv_get_complex_param(struct net_device *, struct iw_request_info *,
					union iwreq_data *, char *);
/* functions called by private ioctls DDA_IOCTL_BASIC_PARAM and DDA_IOCTL_GET_BASIC_PARAM */

static int tiap_param_set_replev(int);
static int tiap_param_get_replev(int *);
static int tiap_param_set_beaconperiod(initConfigStruct *, __u16);
static int tiap_param_get_beaconperiod(int *);
static int tiap_param_set_dtim_period(initConfigStruct *, __u8);
static int tiap_param_get_dtim_period(int *);
static int tiap_param_set_bss_bridge_state(initConfigStruct *, tiap_param_bss_br_state_t);
static int tiap_param_get_bss_bridge_state(int *);
static int tiap_param_set_rate_fallback(initConfigStruct *, __u8);
static int tiap_param_get_rate_fallback(int *);
static int tiap_param_get_radio_cal(int *);
static int tiap_param_set_radio_cal(initConfigStruct *, __u32);
static int tiap_param_get_radio_cal_int(int *);
static int tiap_param_set_radio_cal_int(initConfigStruct *, __u16);
static int tiap_param_get_energy_detect(int *);
static int tiap_param_set_energy_detect(initConfigStruct *, __u8);
static int tiap_param_get_eeprom_less(int *);
static int tiap_param_set_eeprom_less(initConfigStruct *, __u8);
static int tiap_param_get_hidden_ssid(int *);
static int tiap_param_set_hidden_ssid(initConfigStruct *, __u16);
static int tiap_param_get_multidomain_cap(int *);
static int tiap_param_set_multidomain_cap(initConfigStruct *, __u8);
static int tiap_param_get_spectrum_mgmt(int *);
static int tiap_param_set_spectrum_mgmt(initConfigStruct *, __u8);
static int tiap_param_get_legacy_sta(int *);
static int tiap_param_set_legacy_sta(initConfigStruct *, __u8);
static int tiap_param_get_local_power_cons(int *);
static int tiap_param_set_local_power_cons(initConfigStruct *, __s8);
static int tiap_param_get_max_tx_pow_level(int *);
static int tiap_param_set_max_tx_pow_level(initConfigStruct *, __u8);
static int tiap_param_get_country_str(int *);
static int tiap_param_set_country_str(initConfigStruct *, char *);
static int tiap_param_get_oper_state(int *);
static int tiap_param_set_oper_state(initConfigStruct *, __u8);
static int tiap_param_get_wone_num(int *);
static int tiap_param_set_wone_num(initConfigStruct *, __u8);
static int tiap_param_get_wone_timemod(int *);
static int tiap_param_set_wone_timemod(initConfigStruct *, __u8);
static int tiap_param_set_radar_detect_mode(initConfigStruct *, __u8);
static int tiap_param_get_radar_detect_mode(int *);
static int tiap_param_set_bridge_mode(initConfigStruct *, __u8);
static int tiap_param_get_bridge_mode(int *);
static int tiap_param_set_wds_enable(initConfigStruct *init_cfg, __u8 val);
static int tiap_param_get_wds_enable(int *val);

int check_if_all_configured(void);


/*! \brief Array of Linux Wireless Extension standard ioctl handlers
*/

static const iw_handler dda_handler[] =
{
	(iw_handler) NULL,                              /* SIOCSIWCOMMIT */
	(iw_handler) dda_ioctl_giwname,                 /* SIOCGIWNAME */
	(iw_handler) NULL,                              /* SIOCSIWNWID */
	(iw_handler) NULL,                              /* SIOCGIWNWID */
	(iw_handler) NULL,              		/* SIOCSIWFREQ */
	(iw_handler) NULL,              		/* SIOCGIWFREQ */
	(iw_handler) NULL,		              	/* SIOCSIWMODE */
	(iw_handler) NULL,	        		/* SIOCGIWMODE */
	(iw_handler) NULL,              		/* SIOCSIWSENS */
	(iw_handler) NULL,              		/* SIOCGIWSENS */
	(iw_handler) NULL /* not used */,               /* SIOCSIWRANGE */
	(iw_handler) NULL,             			/* SIOCGIWRANGE */
	(iw_handler) NULL /* not used */,               /* SIOCSIWPRIV */
	(iw_handler) NULL /* kernel code */,            /* SIOCGIWPRIV */
	(iw_handler) NULL /* not used */,               /* SIOCSIWSTATS */
	(iw_handler) NULL /* kernel code */,            /* SIOCGIWSTATS */
	(iw_handler) NULL,                      	/* SIOCSIWSPY */
	(iw_handler) NULL,                              /* SIOCGIWSPY */
	(iw_handler) NULL,              	        /* SIOCSIWTHRSPY */
	(iw_handler) NULL,                          	/* SIOCGIWTHRSPY */
	(iw_handler) NULL,		                /* SIOCSIWAP */
	(iw_handler) NULL,  			        /* SIOCGIWAP */
	(iw_handler) NULL,                              /* -- hole -- */
	(iw_handler) NULL,   			        /* SIOCGIWAPLIST */
	(iw_handler) NULL,                              /* SIOCSIWSCAN */
	(iw_handler) NULL,                              /* SIOCGIWSCAN */
	(iw_handler) NULL,  			        /* SIOCSIWESSID */
	(iw_handler) NULL,             /* SIOCGIWESSID */
	(iw_handler) NULL,             /* SIOCSIWNICKN */
	(iw_handler) NULL,             /* SIOCGIWNICKN */
	(iw_handler) NULL,                              /* -- hole -- */
	(iw_handler) NULL,                              /* -- hole -- */
	(iw_handler) NULL,              /* SIOCSIWRATE */
	(iw_handler) NULL,              /* SIOCGIWRATE */
	(iw_handler) dda_ioctl_siwrts,   /* SIOCSIWRTS */
	(iw_handler) dda_ioctl_giwrts,  /* SIOCGIWRTS */
	(iw_handler) dda_ioctl_siwfrag, /* SIOCSIWFRAG */
	(iw_handler) dda_ioctl_giwfrag, /* SIOCGIWFRAG */
	(iw_handler) dda_ioctl_siwtxpow,             /* SIOCSIWTXPOW */
	(iw_handler) dda_ioctl_giwtxpow,             /* SIOCGIWTXPOW */
	(iw_handler) dda_ioctl_siwretry,  /* SIOCSIWRETRY */
	(iw_handler) dda_ioctl_giwretry, /* SIOCGIWRETRY */
	(iw_handler) NULL,            /* SIOCSIWENCODE */
	(iw_handler) NULL,            /* SIOCGIWENCODE */
	(iw_handler) NULL,             /* SIOCSIWPOWER */
	(iw_handler) NULL             /* SIOCGIWPOWER */
};



/*! \brief Initialization private (sub-)ioctl interface information
		   For each private (sub-)ioctl ID there may be two elements: for set and for get
*/
static const struct iw_priv_args dda_priv[] = 
{
	/* --- raw access to basic sub-ioctls --- */
	{ DDA_IOCTL_BASIC_PARAM,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 2, 0, "dda_baspar" },
	{ DDA_IOCTL_GET_BASIC_PARAM,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "dda_g_baspar" },
	/* --- basic sub-ioctls handlers --- */
	{ DDA_IOCTL_BASIC_PARAM,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "" },
	{ DDA_IOCTL_GET_BASIC_PARAM,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "" },
	/* --- basic sub-ioctls definitions --- */
	{ TIAP_PARAM_REPLEV,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "replev" },
	{ TIAP_PARAM_REPLEV,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_replev" },
	{ TIAP_PARAM_BEACON_PERIOD,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "beaconperiod" },
	{ TIAP_PARAM_BEACON_PERIOD,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_beaconperiod" },
	{ TIAP_PARAM_DTIM_PERIOD,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "dtim_period" },
	{ TIAP_PARAM_DTIM_PERIOD,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_dtim_period" },
	{ TIAP_PARAM_BSS_BRIDGE_STATE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "bss_bridge" },
	{ TIAP_PARAM_BSS_BRIDGE_STATE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_bss_bridge" },
	{ TIAP_PARAM_RATE_FALLBACK,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "rfallback" },
	{ TIAP_PARAM_RATE_FALLBACK,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_rfallback" },
	{ TIAP_PARAM_RADIO_CAL,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "radio_cal" },
	{ TIAP_PARAM_RADIO_CAL,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_radio_cal" },
	{ TIAP_PARAM_RADIO_CAL_INT,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "radio_calint" },
	{ TIAP_PARAM_RADIO_CAL_INT,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_radio_calint" },
	{ TIAP_PARAM_ENERGY_DETECT,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "energy_det" },
	{ TIAP_PARAM_ENERGY_DETECT,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_energy_det" },
	{ TIAP_PARAM_HIDDEN_SSID,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "hidden_ssid" },
	{ TIAP_PARAM_HIDDEN_SSID,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_hidden_ssid" },
	{ TIAP_PARAM_MULTI_DOMAIN_CAPAB,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "multidom_cap" },
	{ TIAP_PARAM_MULTI_DOMAIN_CAPAB,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_multidom_cap" },
	{ TIAP_PARAM_SPECTRUM_MGMT,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "spectr_mgmt" },
	{ TIAP_PARAM_SPECTRUM_MGMT,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_spectr_mgmt" },
	{ TIAP_PARAM_LEGACY_STA,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "legacy_sta" },
	{ TIAP_PARAM_LEGACY_STA,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_legacy_sta" },
	{ TIAP_PARAM_LOCAL_POWER_CONS,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "power_cons" },
	{ TIAP_PARAM_LOCAL_POWER_CONS,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_power_cons" },
	{ TIAP_PARAM_MAX_TX_POWER_LEVEL,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "max_tx_pow" },
	{ TIAP_PARAM_MAX_TX_POWER_LEVEL,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_max_tx_pow" },
	{ TIAP_PARAM_COUNTRY_STR,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "country" },
	{ TIAP_PARAM_COUNTRY_STR,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_country" },
	{ TIAP_PARAM_OPER_STATE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "operstate" },
	{ TIAP_PARAM_OPER_STATE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_operstate" },
	{ TIAP_PARAM_WONE_NUM,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "wone_num" },
	{ TIAP_PARAM_WONE_NUM,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_wone_num" },
	{ TIAP_PARAM_WONE_TIME_MODE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "wone_timemod" },
	{ TIAP_PARAM_WONE_TIME_MODE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_wone_timemod" },
	{ TIAP_PARAM_RADAR_DETECT_MODE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "radar_detect" },
	{ TIAP_PARAM_RADAR_DETECT_MODE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_radar_detect" },
	{ TIAP_PARAM_AP_BRIDGE_MODE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "bridge_mode" },
	{ TIAP_PARAM_AP_BRIDGE_MODE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_bridge_mode" },
	{ TIAP_PARAM_WDS_ENABLE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "wds_enable" },
	{ TIAP_PARAM_WDS_ENABLE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_wds_enable" },
	{ TIAP_PARAM_EEPROM_LESS_ENABLE,
	  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "ee_less_enable" },
	{ TIAP_PARAM_EEPROM_LESS_ENABLE,
	  0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "g_ee_less_enable" },

	/* --- raw access to complex sub-ioctls --- */
	{ DDA_IOCTL_COMPLEX_PARAM,
	  COMPLEX_PARAM_SIZE, 0, "cplex_param" },
	{ DDA_IOCTL_GET_COMPLEX_PARAM,
	  COMPLEX_PARAM_SIZE,
	  COMPLEX_PARAM_SIZE, "g_cplex_param" },
	/* --- complex sub-ioctls handlers --- */
	{ DDA_IOCTL_COMPLEX_PARAM,
	  COMPLEX_PARAM_SIZE, 0, "" },
	{ DDA_IOCTL_GET_COMPLEX_PARAM,
	  0, COMPLEX_PARAM_SIZE, "" },
	/* --- complex sub-ioctls definitions --- */
	{ TIAP_PARAM_AUTH_TYPE,
	  COMPLEX_PARAM_SIZE, 0, "auth_type" },
	{ TIAP_PARAM_AUTH_TYPE,
	  0, COMPLEX_PARAM_SIZE, "g_auth_type" },
	{ TIAP_PARAM_WD,
	  COMPLEX_PARAM_SIZE, 0, "wd" },
	{ TIAP_PARAM_WD,
	  0, COMPLEX_PARAM_SIZE, "g_wd" },
	{ TIAP_PARAM_INTERF_WD_CONFIG,
	  COMPLEX_PARAM_SIZE, 0, "interf_par" },
	{ TIAP_PARAM_INTERF_WD_CONFIG,
	  0, COMPLEX_PARAM_SIZE, "g_interf_par" },
	{ TIAP_PARAM_ACCESS_LIST,
	  COMPLEX_PARAM_SIZE, 0, "acess_list" },
	{ TIAP_PARAM_ACCESS_LIST,
	  0, COMPLEX_PARAM_SIZE, "g_access_list" },
	{ TIAP_PARAM_4X,
	  COMPLEX_PARAM_SIZE, 0, "4x" },
	{ TIAP_PARAM_4X,
	  0, COMPLEX_PARAM_SIZE, "g_4x" },
	{ TIAP_PARAM_MULTI_SSID,
	  COMPLEX_PARAM_SIZE, 0, "multi_ssid" },
	{ TIAP_PARAM_MULTI_SSID,
	  0, COMPLEX_PARAM_SIZE, "g_multi_ssid" },
	{ TIAP_PARAM_CFG_POWER_TABLE,
	  COMPLEX_PARAM_SIZE, 0, "cfg_pwr_tbl" },
	{ TIAP_PARAM_CFG_POWER_TABLE,
	  0, COMPLEX_PARAM_SIZE, "g_cfg_pwr_tbl" },
	{ TIAP_PARAM_CHANNEL_RANGE,
	  COMPLEX_PARAM_SIZE, 0, "chan_rang" },
	{ TIAP_PARAM_CHANNEL_RANGE,
	  0, COMPLEX_PARAM_SIZE, "g_chan_rang" },
	{ TIAP_PARAM_WONE_ELEM,
	  COMPLEX_PARAM_SIZE, 0, "wone_elem" },
	{ TIAP_PARAM_WONE_ELEM,
	  0, COMPLEX_PARAM_SIZE, "g_wone_elem" },
	{ TIAP_PARAM_STA_TBL,
	  0, COMPLEX_PARAM_SIZE, "g_sta_tbl" },
	{ TIAP_PARAM_PRIV_REG_DOMAIN,
	  COMPLEX_PARAM_SIZE, 0,  "priv_regdom" },
	{ TIAP_PARAM_PRIV_REG_DOMAIN,
	  0, COMPLEX_PARAM_SIZE,  "g_priv_regdom" },
	{ TIAP_PARAM_CURR_REG_DOMAIN,
	  COMPLEX_PARAM_SIZE, 0,  "curr_regdom" },
	{ TIAP_PARAM_CURR_REG_DOMAIN,
	  0, COMPLEX_PARAM_SIZE,  "g_curr_regdom" },
	{ TIAP_PARAM_NET_STATS,
	  0, COMPLEX_PARAM_SIZE, "g_net_stats" },
	{ TIAP_PARAM_QOS,
		COMPLEX_PARAM_SIZE, 0, "qos" },
	{ TIAP_PARAM_QOS,
		0, COMPLEX_PARAM_SIZE, "g_qos"},
    { TIAP_PARAM_BIT,
		COMPLEX_PARAM_SIZE, 0, "bit" },
	{ TIAP_PARAM_BIT,
		0, COMPLEX_PARAM_SIZE, "g_bit"},
	{ TIAP_PARAM_HW_INTERF_CFG,
		COMPLEX_PARAM_SIZE, 0, "hw_aci" },
	{ TIAP_PARAM_HW_INTERF_CFG,
		0, COMPLEX_PARAM_SIZE, "g_hw_aci"},
	{ TIAP_PARAM_EEPROM,
	  COMPLEX_PARAM_SIZE, 0, "eeprom" },
	{ TIAP_PARAM_DIE_ID,
	  0, COMPLEX_PARAM_SIZE, "g_die_id" },

	  /* raw access for GO */
	{ DDA_IOCTL_GO,
	  IW_PRIV_TYPE_NONE | IW_PRIV_SIZE_FIXED | 1, 0, "go" },
};


/*! \brief Array of Linux Wireless Extension private ioctl handlers

	For each private subioctl one of the following handlers is called 
	Which in its turn calls that subioctl's handler
*/
static const iw_handler dda_private_handler[] =
{                                                       /* SIOCIWFIRSTPRIV + */
	(iw_handler) dda_ioctl_priv_basic_param,    	/* 0 */
	(iw_handler) dda_ioctl_priv_get_basic_param, 	/* 1 */
	(iw_handler) dda_ioctl_priv_complex_param,    	/* 2 */
	(iw_handler) dda_ioctl_priv_get_complex_param, 	/* 3 */
	(iw_handler) dda_ioctl_priv_go,			/* 4 */
};


/*! \brief Wireless Extension ioctl handlers */
const struct iw_handler_def dda_iw_handler_def =
{
        .num_standard   = sizeof(dda_handler) / sizeof(iw_handler),
        .num_private    = sizeof(dda_private_handler) / sizeof(iw_handler),
        .num_private_args = sizeof(dda_priv) / sizeof(struct iw_priv_args),
        .standard       = (iw_handler *)dda_handler,
        .private        = (iw_handler *)dda_private_handler,
        .private_args   = (struct iw_priv_args *)dda_priv,
};


/* functions implementation */

/*******************************************************************************************/
/* dda_ioctl_giwname */
/*! 
	\brief Standard WE ioctl handler. Get wireless interface name
	
	\param dev		ignored
	\param info 	ignored
	\param name 	a buffer for output
	\param extra	ignored 
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_giwname(struct net_device *dev, struct iw_request_info *info, char *name, char *extra)
{
	if( name )
		strcpy(name, "wlan0");
	else printk(KERN_INFO "dda: name is NULL\n");

	return 0;
}


/* implementation of complex data types */

/*******************************************************************************************/
/* param_auth_tbl */
/*! 
	\brief Private WE subioctl handler. Sets authentication algorithms table
	
	\param init_cfg		initial driver configuration structure
	\param auth_tbl		input table
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_auth_tbl(initConfigStruct *init_cfg, tiap_param_auth_algo_t *auth_tbl)
{
int ret = 0;

	if( !atomic_read(&after_go) )
	{
		if( ((auth_tbl[0].algo == TIAP_PARAM_MIB_OPEN_SYSTEM && auth_tbl[1].algo == TIAP_PARAM_MIB_SHARED_KEY) ||
		     (auth_tbl[1].algo == TIAP_PARAM_MIB_OPEN_SYSTEM && auth_tbl[0].algo == TIAP_PARAM_MIB_SHARED_KEY)) ) 
		    
		{
			if ( ((auth_tbl[0].enable == FALSE) || (auth_tbl[0].enable == TRUE)) &&
			     ((auth_tbl[1].enable == FALSE) || (auth_tbl[1].enable == TRUE)) )
			{
				init_cfg->dot11AuthenticationAlgorithmTable[0].authAlgo = auth_tbl[0].algo;
				init_cfg->dot11AuthenticationAlgorithmTable[0].enable = auth_tbl[0].enable;
				init_cfg->dot11AuthenticationAlgorithmTable[1].authAlgo = auth_tbl[1].algo;
				init_cfg->dot11AuthenticationAlgorithmTable[1].enable = auth_tbl[1].enable;
				configured_params.auth_tbl = 1;
			}
			else{
				printk(KERN_INFO "%s: algorithm enable value: invalid parameter %d %d\n", __func__,
						auth_tbl[0].enable, auth_tbl[1].enable);
				return TIAP_PARAM_ERANGE ;
			}

		}
		else {
			printk(KERN_INFO "%s: auth algorithm: invalid parameter %d %d\n", __func__,
				auth_tbl[0].algo, auth_tbl[1].algo);
			return TIAP_PARAM_ERANGE ;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets WatchDog parameters
	
	\param init_cfg		initial driver configuration structure
	\param wd			Watch Dog structure: command ID and input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_wd(initConfigStruct *init_cfg, tiap_param_wd_t *wd)
{
int ret = 0;

	if( !atomic_read(&after_go) )
	{
		switch( wd->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->EnRestart = wd->value.val8;
					configured_params.wd_en = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  enable :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}
				break;
			case TIAP_PARAM_WD_INTERF:
				if ( (wd->value.val8 == 0) || (wd->value.val8 == 1) || (wd->value.val8 == 2) )
				{
					init_cfg->EnInterfWA = wd->value.val8;
					configured_params.wd_interf = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  enable :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}
				break;	
			case TIAP_PARAM_WD_RXFREE:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->RxFreeMemOptionWA = wd->value.val8;
					configured_params.wd_rxfree = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  rx free :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}

				break;

			case TIAP_PARAM_WD_BEACONS:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->BeaconNotSentOptionWA = wd->value.val8;
					configured_params.wd_beacons = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  beacons :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}

				break;

			case TIAP_PARAM_WD_TXSTUCK:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->TxStuckOptionWA = wd->value.val8;
					configured_params.wd_txstuck = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  tx stuck :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}
				break;

			case TIAP_PARAM_WD_FCSERR:
				if ( wd->value.val16 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->ConsFcsErrOptionWA = wd->value.val16;
					configured_params.wd_fcserrs = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  fcs err :invalid parameter %d \n", __func__,
						wd->value.val16);
					return TIAP_PARAM_ERANGE ;
				}
				break;

			case TIAP_PARAM_WD_ACI:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->aciOptionWA = wd->value.val8;
					configured_params.wd_aci = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  enable :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}
				break;
			
			case TIAP_PARAM_WD_NB:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->nbOptionWA = wd->value.val8;
					configured_params.wd_nb = 1;
				}
				else
				{
					printk(KERN_INFO "%s:  enable :invalid parameter %d \n", __func__,
						wd->value.val8);
					return TIAP_PARAM_ERANGE ;
				}
				break;


			default:
				printk(KERN_INFO "%s: invalid type %d\n", __func__, wd->cmd);
				return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret_err(ret, wd->cmd);
	
	return ret;
}

/* Set ACI/NB interference Watch Dog configuration parameters*/
static int param_interference_wd_config(initConfigStruct *init_cfg, tiap_param_interference_wd_t *interf_param)
{
	int ret = 0;

	if( !atomic_read(&after_go) )
	{
		switch (interf_param->cmd) {
	
			case  TIAP_PARAM_ACI_TIMEOUT:
				  init_cfg->interfWAparams.aciStateTimeout = interf_param->value.val32;
				  configured_params.aci_state_timeout = 1;
				  break;
			case  TIAP_PARAM_NB_TIMEOUT:
				  init_cfg->interfWAparams.nbStateTimeout = interf_param->value.val32;
				  configured_params.nb_state_timeout = 1;
				  break;
			case  TIAP_PARAM_ACI_RATE_THR:
				  init_cfg->interfWAparams.aciRateThreshold = interf_param->value.val32;
				  configured_params.aci_rate_thresh = 1;
				  break;
			case  TIAP_PARAM_ACI_EVENT_THR:
				  init_cfg->interfWAparams.aciEventThreshold = interf_param->value.val32;
				  configured_params.aci_event_thresh = 1;
				  break;
			case  TIAP_PARAM_PD_LOW:
				  init_cfg->interfWAparams.pdUnderAciLowThrsh = interf_param->value.val16;
				  configured_params.pd_low_thresh = 1;
				  break;
			case  TIAP_PARAM_PD_HIGH:
				  init_cfg->interfWAparams.pdUnderAciHighThrsh = interf_param->value.val16;
				  configured_params.pd_high_thresh = 1;
				  break; 
			case  TIAP_PARAM_MEDIUM_BUSY_THR:
				  if (interf_param->value.val32 <= 100) {
					  init_cfg->interfWAparams.mediumBusyThresh = interf_param->value.val32;
					  configured_params.medium_busy_thresh = 1;
				  }
				  else {
						printk(KERN_INFO "%s:  invalid  medium busy threshold %d \n", __func__,
							interf_param->value.val32);
						ret = TIAP_PARAM_ERANGE ;
				  }
	
				  break;
			
			default:
					printk(KERN_INFO "%s: invalid command type %d\n", __func__, interf_param->cmd);
					ret = TIAP_PARAM_ERANGE;

		}
	}
	else /* after go */
		dda_wext_rt_ret_err(ret, interf_param->cmd);
	
	return ret;
}

/* Set HW interference handling parameters */
static int param_hw_interf_cfg(initConfigStruct *init_cfg, tiap_param_hw_interf_t *interf_param)
{
	int ret = 0;

	if( !atomic_read(&after_go) )
	{
 
		switch (interf_param->cmd) {
	
			case  TIAP_PARAM_INPUT_CCA:
				  init_cfg->hwInterfParams.inputCCA = interf_param->value;
				  configured_params.hw_input_cca = 1;
				  break;
			case  TIAP_PARAM_TX_CCA:
				  init_cfg->hwInterfParams.txCCA = interf_param->value;
				  configured_params.hw_tx_cca = 1;
				  break;
			case  TIAP_PARAM_QUALIF_CCA:
				  init_cfg->hwInterfParams.qualifiedCCA = interf_param->value;
				  configured_params.hw_qualif_cca = 1;
				  break;
			case  TIAP_PARAM_STOMP_RX:
				  init_cfg->hwInterfParams.stompForRx = interf_param->value;
				  configured_params.hw_stomp_rx = 1;
				  break;
			case  TIAP_PARAM_STOMP_TX:
				  init_cfg->hwInterfParams.stompForTx = interf_param->value;
				  configured_params.hw_stomp_tx = 1;
				  break;
					
			default:
					printk(KERN_INFO "%s: invalid command type %d\n", __func__, interf_param->cmd);
					ret = TIAP_PARAM_ERANGE;

		}
	}
	else /* after go */
		dda_wext_rt_ret_err(ret, interf_param->cmd);
	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets EEPROM image
	
	\param init_cfg		initial driver configuration structure
	\param eeprom eeprom image
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_eeprom(initConfigStruct *init_cfg, tiap_param_eeprom_t * eeprom) {
	if (eeprom->len > MAX_EEPROM_LEN )
		return TIAP_PARAM_ELOGICAL;
	memcpy((void *)init_cfg->eeprom.image, eeprom->image, eeprom->len);
	init_cfg->eeprom.len = eeprom->len;
	configured_params.eeprom_image = 1;

	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets access list
	
	\param init_cfg		initial driver configuration structure
	\param access_list  access list structure: command ID and input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_access_list(initConfigStruct *init_cfg, tiap_param_access_list_t *access_list)
{
int ret = 0;

	switch( access_list->cmd )
	{
		case TIAP_PARAM_ACCESS_SET_TYPE:
			printk(KERN_INFO "setting access list type is not allowed\n");
			ret = 1;
			break;

		case TIAP_PARAM_ACCESS_LST:
			if( !atomic_read(&after_go) )
		       	{
				if ( (access_list->enabled == TRUE) || (access_list->enabled == FALSE) )
			       	{
					if ( (access_list->type == 1) || (access_list->type == 0) ) {
						init_cfg->mgmtAccessList.mgmt_access_list_type = access_list->type;
						memcpy((void *)init_cfg->mgmtAccessList.mgmt_access_list,
							(void *)access_list->list, TIAP_PARAM_ACCESS_LIST_SIZE * ETH_ALEN);
						init_cfg->mgmtAccessList.mgmt_access_enabled = access_list->enabled;
						configured_params.access_list = 1;
					}
				}
			}
			else dda_wext_rt_ret_err(ret, access_list->cmd);
			break;

		case TIAP_PARAM_ACCESS_ADD:
			if( !atomic_read(&after_go) )
				dda_wext_rt_ret_err(ret, access_list->cmd);
			else {
				if (mgmtAccessSetMAC(access_list->list[0]) != OK)
					ret = TIAP_PARAM_ELOGICAL;

			}
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_INDEX:
			if ( (access_list->index >= 0) && (access_list->index <=MGMT_ACCESS_LIST_SIZE) ){
				if( !atomic_read(&after_go) )
					dda_wext_rt_ret_err(ret, access_list->cmd);
				else {
					if (mgmtAccessRemMAC(access_list->index) != OK)
						ret = TIAP_PARAM_ELOGICAL;
				}
			}
			else {
				printk(KERN_INFO "%s:  index :invalid parameter %d \n", __func__,
							access_list->index);
				ret = TIAP_PARAM_ERANGE ;
			}

			break;

		case TIAP_PARAM_ACCESS_DEL_BY_ADDR:
			if( !atomic_read(&after_go) )
				dda_wext_rt_ret_err(ret, access_list->cmd);
			else {
				if (mgmtAccessRemMACbyAddr(access_list->list[0]) != OK)
					ret = TIAP_PARAM_ELOGICAL;
			}
			break;

		case TIAP_PARAM_ACCESS_RESET:
			if ( access_list->enabled == FALSE)
			{
				init_cfg->mgmtAccessList.mgmt_access_enabled = access_list->enabled;
				memset(init_cfg->mgmtAccessList.mgmt_access_list,
							0, TIAP_PARAM_ACCESS_LIST_SIZE * ETH_ALEN);
	
				configured_params.access_list = 1;
			}
			else {
				printk(KERN_INFO "%s:  enabled :invalid parameter %d \n", __func__,
							access_list->enabled);
				ret = TIAP_PARAM_ERANGE ;
			}
			
			break;
			

		default:
			printk(KERN_INFO "%s: invalid command type %d\n", __func__, access_list->cmd);
			ret = TIAP_PARAM_ERANGE ;
	}
			
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets 4x parameters
	
	\param init_cfg		initial driver configuration structure
	\param _4x			4x structure: command ID and input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_4x(initConfigStruct *init_cfg, tiap_param_4x_t *_4x)
{
	int ret = 0;



	if( !atomic_read(&after_go) )
	{
		if( (_4x->value == 0) || (_4x->value == 1) )
		{	
			switch( (tiap_param_4x_cmd_t)_4x->cmd )
			{
				case TIAP_PARAM_4X_FEATSET:
					init_cfg->FeatureSet = _4x->value;
					configured_params._4x_fs = 1;
					break;
				case TIAP_PARAM_4X_4X:
					init_cfg->enable4x = _4x->value;
					configured_params._4x_4x = 1;
					break;
				case TIAP_PARAM_4X_CAT:
					init_cfg->enableConcat = _4x->value;
					configured_params._4x_cat = 1;
					break;
				case TIAP_PARAM_4X_PKTBURST:
					configured_params._4x_pb = 1;
					break;
				case TIAP_PARAM_4X_MIXMOD:
					init_cfg->enable4xMixMode = _4x->value;
					configured_params._4x_mm = 1;
					break;
				default:
					printk(KERN_INFO "%s:  index :invalid parameter %d \n", __func__,
							_4x->cmd);
					return TIAP_PARAM_ERANGE ;
			}
			
		}
		else {
			printk(KERN_INFO "%s: invalid parameter %d \n", __func__,_4x->value);
			return TIAP_PARAM_ERANGE ;
		}
	}
	else dda_wext_rt_ret(ret);
	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets Multiple Ssid parameters
	
	\param init_cfg		initial driver configuration structure
	\param multi_ssid	multiple ssid structure: command ID and input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_multi_ssid(initConfigStruct *init_cfg, tiap_param_multi_ssid_t *multi_ssid)
{
int ret = 0, i;

	if( !init_cfg || !multi_ssid )
		return -EINVAL;

	if( !atomic_read(&after_go) ) {
			for (i=0; i< TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
		{
				
				memcpy(init_cfg->apMultiSsid[i].ssid.serviceSetId, multi_ssid->tbl[i].ssid,IW_ESSID_MAX_SIZE);
				init_cfg->apMultiSsid[i].ssid.hdr.eleLen = strlen(multi_ssid->tbl[i].ssid);
				init_cfg->apMultiSsid[i].ssidType = multi_ssid->tbl[i].ssid_type;
				init_cfg->apMultiSsid[i].vlanId = multi_ssid->tbl[i].vlan;
				/* Verify that primary SSID is configured */
				if (init_cfg->apMultiSsid[i].ssidType == TIAP_PARAM_SSID_PRIMARY)
					configured_params.essid = 1;
			}
			
			init_cfg->apMultipleSsidFlag  = multi_ssid->secondary_enabled;
    			
			configured_params.multi_ssid = 1;
	}
	else dda_wext_rt_ret(ret);
	
	return ret;
}


/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets radio parameters
	
	\param init_cfg		initial driver configuration structure
	\param radio		input radio structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_radio(initConfigStruct *init_cfg, tiap_param_radio_t *radio)
{
int ret = 0;

	init_cfg->dbRadioParams.overrideBitmask = radio->mask;
	if( radio->tbl.type != TIAP_PARAM_PWRTBL ) {
		printk(KERN_INFO "%s: type: invalid parameter %d \n", __func__,radio->tbl.type);
		return TIAP_PARAM_ERANGE ;
	}

	init_cfg->dbRadioParams.powerTable_paramType = radio->tbl.type;
	/*TODO check range*/
	/*
	for(i=0; i<sizeof(radio->tbl.param); i++)
		if( radio->tbl.param[i] < -5 || radio->tbl.param[i] > 5 ) {
			printk(KERN_INFO "%s: tbl.param: invalid parameter %d \n", __func__,radio->tbl.param[i]);
			return TIAP_PARAM_ERANGE ;
		}
	*/

	memcpy((void *)&init_cfg->dbRadioParams.powerTable_param, (void *)&radio->tbl.param, 
			sizeof(radio->tbl.param));

	/* set radio params according to the new configuration */
	if( atomic_read(&after_go) ) {
		wdrv_dbSetRadioParams((RadioParam_t *)&init_cfg->dbRadioParams, RADIO_PARAM_POWER_TABLE, 0);
		wdrv_dbSetRadioParams((RadioParam_t *)&init_cfg->dbRadioParams, RADIO_PARAM_OVERRIDE_BMASK, 0);
		whal_apiSetRadioParams();
	}

	configured_params.radio = 1;
		
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets channel range parameters
	
	\param init_cfg		initial driver configuration structure
	\param ch_rang		input channel range structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_ch_rang(initConfigStruct *init_cfg, tiap_param_channel_rang *ch_rang)
{
	int i, max_index, ret=0;


	if( !atomic_read(&after_go) )
	{
		if( ch_rang->type == TIAP_PARAM_RANG_A ) 
			max_index = TIAP_PARAM_MAX_NUM_OF_RANGES_A;
		else 
		{
			if ( ch_rang->type == TIAP_PARAM_RANG_G ) 
				max_index = TIAP_PARAM_MAX_NUM_OF_RANGES_G;
			else
			{
				printk(KERN_INFO "%s: type: invalid parameter %d \n", __func__,ch_rang->type);
				return  TIAP_PARAM_ERANGE ;
			}
		}

		if (ch_rang->ranges_nr < max_index ) {
			for (i=0 ; i < ch_rang->ranges_nr; i++)
            {
                if( ch_rang->type == TIAP_PARAM_RANG_A )
                {
                    if ( (ch_rang->rng.a[i].start > ch_rang->rng.a[i].stop) ||
                        (ch_rang->rng.a[i].power_limit < 0) ||
                        (ch_rang->rng.a[i].step < 1) ||
                        (ch_rang->rng.a[i].step > 15) )
                    {
                        printk(KERN_INFO "%s: invalid parameter  \n", __func__);
                        return TIAP_PARAM_ERANGE ;
                    }
                }
                else
                {
                    if ( (ch_rang->rng.g[i].start > ch_rang->rng.g[i].stop) ||
                        (ch_rang->rng.g[i].power_limit < 0) ||
                        (ch_rang->rng.g[i].step < 1) ||
                        (ch_rang->rng.g[i].step > 15) )
                    {
                        printk(KERN_INFO "%s: invalid parameter  \n", __func__);
                        return TIAP_PARAM_ERANGE ;
                    }
                }
            }
		 }
		 else {
			printk(KERN_INFO "%s: num ranges: invalid parameter %d  \n", __func__, ch_rang->ranges_nr);
			return TIAP_PARAM_ERANGE ;
		 }	



	
		if( ch_rang->type == TIAP_PARAM_RANG_A ) {
			init_cfg->RegInit.numOfRanges_802_11_a = ch_rang->ranges_nr;
			memcpy((void *)&init_cfg->RegInit.privateRangeElements_a, 
					(void *)&ch_rang->rng.a, sizeof(ch_rang->rng.a));
			configured_params.ch_rang_a = 1;

		}
		else {
			init_cfg->RegInit.numOfRanges_802_11_b = ch_rang->ranges_nr;
			memcpy((void *)&init_cfg->RegInit.privateRangeElements_b, 
				       (void *)&ch_rang->rng.g, sizeof(ch_rang->rng.g));
			configured_params.ch_rang_g = 1;
		}

				
	}		
	else dda_wext_rt_ret(ret);

	return ret;
}

static int param_wone(initConfigStruct *init_cfg, tiap_param_wone_t *wone)
{
	u8 index = wone->index;
	int  i,k, ret = 0;
	u8 cckRateValue[TIAP_PARAM_CCK_PBCC_RATES_MAX] = 
		{DOT11_RATE_1,DOT11_RATE_2,DOT11_RATE_5_5,DOT11_RATE_11,DOT11_RATE_22};
	u8 ofdmRateValue[TIAP_PARAM_OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };

	if( !atomic_read(&after_go) )
	{
		switch(wone->type)
		{
			case wone_oper_rate:
				if(wone->elem.b_legacy.len > 5) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				dda_wext_convert_rates(b_legacy, apRates, cckRateValue);
				init_cfg->wone_info[index].apRates.hdr.eleLen = k;
				configured_params.oper_rate[index] = 1;
				break;

			case wone_oper_add:
				if(wone->elem.ofdm_legacy.len > 8) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				dda_wext_convert_rates(ofdm_legacy, apAddRates, ofdmRateValue);
				init_cfg->wone_info[index].apAddRates.hdr.eleLen = k;
				configured_params.oper_add[index] = 1;
				break;

			case wone_oper_ofdm:
				if(wone->elem.ofdm_extend.len > 8) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				dda_wext_convert_rates(ofdm_extend, apOFDMRates, ofdmRateValue);
				init_cfg->wone_info[index].apOFDMRates.hdr.eleLen = k;
				configured_params.oper_ofdm[index] = 1;
				break;

			 case wone_brcst_rate:
				
				if (wone->elem.brcst_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.brcst_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						printk (KERN_INFO "dda: set wone: broadcast rate : value is out of range\n");
						ret = TIAP_PARAM_ERANGE;
						break;
					}
				}
				
				init_cfg->wone_info[index].brsctDefActiveRate = wone->elem.brcst_default_rate;
				configured_params.brcst_rate[index] = 1;
				break;
			case wone_mcst_rate:
				
				if (wone->elem.mcst_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.mcst_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						printk (KERN_INFO "dda: set wone: multicast rate : value is out of range\n");
						ret = TIAP_PARAM_ERANGE;
						break;
					}
				}
				
				init_cfg->wone_info[index].mcstDefActiveRate = wone->elem.mcst_default_rate;
				configured_params.mcst_rate[index] = 1;
				break;
			case wone_tx_rate:
				
				if (wone->elem.tx_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.tx_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						printk (KERN_INFO "dda: set wone: tx default rate : value is out of range\n");
						ret = TIAP_PARAM_ERANGE;
						break;
					}
				}
				
				init_cfg->wone_info[index].genTxRate = wone->elem.tx_default_rate;
				configured_params.tx_rate[index] = 1;
				break;
			case wone_ch:
				init_cfg->wone_info[index].channel = wone->elem.ch;
				
				if (init_cfg->wone_info[index].Dot11Mode == TIAP_PARAM_D11MODE_A)
				{
					if( (init_cfg->wone_info[index].channel < 36) ||
						(init_cfg->wone_info[index].channel > 64) ||
						(init_cfg->wone_info[index].channel % 4) ){
							printk (KERN_INFO "dda: set wone: channel: value is out of range\n");
							ret = TIAP_PARAM_ERANGE;
							break;
					}
				}
				else {
					if  (init_cfg->wone_info[index].channel > 14) {
							printk (KERN_INFO "dda: set wone: channel : value is out of range\n");
							ret = TIAP_PARAM_ERANGE;
							break;
					}
				}
				configured_params.ch[index] = 1;
				break;

			case wone_band:
				init_cfg->wone_info[index].band = wone->elem.band;
				if((init_cfg->wone_info[index].band != RADIO_BAND_5GHZ) &&
					(init_cfg->wone_info[index].band != RADIO_BAND_2_4GHZ)) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				configured_params.band[index] = 1;
				break;

			case wone_pbcc:
				init_cfg->wone_info[index].PbccAlgorithmMode = wone->elem.pbcc_mode;
				if(init_cfg->wone_info[index].PbccAlgorithmMode >
						TIAP_PARAM_PBCC_ALGO_ENHANCED_DYNAMIC) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				configured_params.pbcc[index] = 1;
				break;

			case wone_rate_adapt:
				init_cfg->wone_info[index].RateAdaptation = wone->elem.rate_adapt;
				if(init_cfg->wone_info[index].RateAdaptation > 1) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.rate_adapt[index] = 1;
				break;

			case wone_pream_mode:
				init_cfg->wone_info[index].PreambleMode = wone->elem.pream_mode;
				if(init_cfg->wone_info[index].PreambleMode >
						TIAP_PARAM_PREAMBLE_MODE_LOCAL_STATIONS) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.pream_mode[index] = 1;
				break;

			case wone_slot_mode:
				init_cfg->wone_info[index].SlotTimeMode = wone->elem.slot_mode;
				if(init_cfg->wone_info[index].SlotTimeMode >
						TIAP_PARAM_SLOTTIME_MODE_ENHANCED_DYNAMIC) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.slot_mode[index] = 1;
				break;

			case wone_erp_mode:	
				init_cfg->wone_info[index].ERPProtectionMode = wone->elem.erp_mode;
				if(init_cfg->wone_info[index].ERPProtectionMode > 
						TIAP_PARAM_PROTECTION_MODE_ENHANCED_DYNAMIC) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.erp_mode[index] = 1;
				break;

			case wone_erp_type:
				init_cfg->wone_info[index].ERPProtectionType = wone->elem.erp_type;
				if(init_cfg->wone_info[index].ERPProtectionType >
						TIAP_PARAM_PROTECTION_TYPE_CTS_TOSELF) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.erp_type[index] = 1;
				break;

			case wone_erp_ti:
				init_cfg->wone_info[index].ERPProtectionTI = wone->elem.erp_ti;
				if(init_cfg->wone_info[index].ERPProtectionTI > 1) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.erp_ti[index] = 1;
				break;

			case wone_dot11_mode:
				init_cfg->wone_info[index].Dot11Mode = wone->elem.dot11_mode;
				if(init_cfg->wone_info[index].Dot11Mode > DOT11_MODE_A_ONLY) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.dot11_mode[index] = 1;
				break;

			case wone_cwmin_mode:
				init_cfg->wone_info[index].CwMinMode = wone->elem.cwmin_mode;
				if(init_cfg->wone_info[index].CwMinMode > DOT11_CWMIN_DOT11_MODE) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.cwmin_mode[index] = 1;
				break;

			case wone_usage_time:
				init_cfg->wone_info[index].UsageTime = wone->elem.usage_time;
				if((init_cfg->wone_info[index].UsageTime > 32) ||
					(init_cfg->wone_info[index].UsageTime < 8)) {
					printk (KERN_INFO "dda: set wone: value is out of range\n");
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.usage_time[index] = 1;
				break;

			default:
				printk (KERN_INFO "dda: set wone: value is out of range\n");
				ret = TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret_err(ret, wone->type);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets private regulatory domain parameters
	
	\param init_cfg			initial driver configuration structure
	\param priv_regdomain	input private regulatory domain structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_priv_regdomain(initConfigStruct *init_cfg, tiap_param_priv_reg_domain_t *priv_regdomain) 
{
	int ret = 0;
	/*range_min = 1, range_max;*/

	if( !atomic_read(&after_go) )
	{
		/*switch (priv_regdomain ->band)
		{
			case BAND_B : range_max = 4;
				      break;
			case BAND_A  : range_max = 8;
				       break;
			case BAND_DEFAULT:
				       range_min = range_max = 0;
				       break;
			default: 
				       printk (KERN_INFO "dda: %s: %d - band value is out of range \n",
						       __func__, priv_regdomain->band);
				       return TIAP_PARAM_ERANGE;
		}
		if ( priv_regdomain->domain >= range_min && priv_regdomain->domain <= range_max )
		{*/
			/*Felix R.: the band is unused for now*/

			init_cfg->RegInit.dot11PrivateRegulatoryDomain = priv_regdomain->domain;
            init_cfg->RegInit.dot11PrivateRegulatoryBand = priv_regdomain->band;
			configured_params.priv_regdomain = 1;
			   
		/*}
		else
		{
			printk (KERN_INFO "dda: %s: %d - domain value is out of range \n",
						 __func__, priv_regdomain->domain);
			return TIAP_PARAM_ERANGE;
		}*/
	}
	else dda_wext_rt_ret(ret);

	return ret;


}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets current regulatory domain parameters
	
	\param init_cfg			initial driver configuration structure
	\param val				input current regulatory domain structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_curr_reg_domain(initConfigStruct *init_cfg, tiap_param_cur_reg_domain_t *val)
{
	int ret = 0;
	__u8 *dest; 

	if (!atomic_read(&after_go)){
		if (val){
			
			switch (val->type )
			{
			case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_IC:
			case TIAP_PARAM_RD_ETSI:
			case TIAP_PARAM_RD_SPAIN:
			case TIAP_PARAM_RD_FRANCE:
			case TIAP_PARAM_RD_MKK:
			case TIAP_PARAM_RD_MKK1:
            case TIAP_PARAM_RD_ISRAEL:
			case TIAP_PARAM_RD_US:
			case TIAP_PARAM_RD_WORLD:
            case TIAP_PARAM_RD_CUSTOM:
				 dest = &(init_cfg->RegInit.dot11CurrentRegDomain); 
				 memset(dest,0,sizeof(*dest));
				 memcpy(dest, &val->type, sizeof(*dest));
				 configured_params.cur_regdomain = 1;
				break;
			default:
				printk (KERN_INFO "dda: set curr_regdomain: %d - value is out of range \n",val->type);
				ret = TIAP_PARAM_ERANGE;
			}
		}
		else printk (KERN_INFO "dda: %s: input pointer is NULL\n", __func__);
	}
	else 
		dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/* IsChannelInsideBandRange() */
/*! 	
	\brief serves for internal use: verfies if a given channel is valid for a given band	   
                                                                         
	\param band 		band 
	\param chID			channel number 
   
	\return TRUE/FALSE.			                                                    
*/
/*******************************************************************************************/
static BOOL IsChannelInsideBandRange(int band, int chID)
{
	BOOL found = FALSE;
	int ch;

	if (band == RADIO_BAND_2_4GHZ)
	{
		if ((chID < 1) || (chID > 14))
			return FALSE;
		return TRUE;
	}
	else
	{
		if ((chID < 36) || (chID > 161))
			return FALSE;
		/* channel is inside the range, let's verify its value */
		for (ch=36; ch<=64;ch++)
			if (ch == chID)
			{
				found=TRUE;
				break;
			}
		if (!found)
			for (ch=100; ch<=140;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
		if (!found)
			for (ch=149; ch<=161;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
	}
	return found;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets BIT commands parameters
	
	\param init_cfg			initial driver configuration structure
	\param val				input BIT arguments structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_bit_args(initConfigStruct *init_cfg, tiap_param_bit_args_t *val)
{
	int ret = 0;

   if (val->band < 0)
	   return TIAP_PARAM_ERANGE;

   if(val->channel <= 0)
	   return TIAP_PARAM_ERANGE;
   
   if (IsChannelInsideBandRange(val->band,val->channel) == FALSE)
   {
	   return TIAP_PARAM_ERANGE;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
   wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

   if(val->cmdType == BIT_FCC)
   {
       if(whal_apiAcxBIT_Fcc(val->band, val->channel, val->rate, val->mode) != OK)
           return TIAP_PARAM_ERANGE;
   }
   else
   {
       if(whal_apiAcxBIT_Telec(val->band, val->channel) != OK)
           return TIAP_PARAM_ERANGE;
   }
     
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief  Common private ioctl handler for all  set subioctls which get complex data type input. 
			Calls appropriate set subioctl handler depending on input subioctl ID
	
	\param dev 		ignored
	\param info		ignored
	\param wrqu 	contains subioctl ID (wrqu->data.flags)
	\param extra	input for subioctl	
	
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_priv_complex_param(struct net_device *dev, struct iw_request_info *info, 
					union iwreq_data *wrqu, char *extra) {
	union the_largest_struct *tls = (union the_largest_struct *)extra;
	int subcmd = wrqu->data.flags;
	int ret = 0, i;


	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	switch (subcmd)	{
	case TIAP_PARAM_QOS:
		memset(&init_cfg.wmeACParams, 0, sizeof(wdrv_QoSACParams_t));
		init_cfg.wmeACParams.enabled = tls->qos.enabled;
		if(tls->qos.enabled) {
		for (i=0; i<4; ++i) {
            init_cfg.wmeACParams.ap_edcfParams[i].aifsn =   tls->qos.edcf_info[0][i].aifsn;
            init_cfg.wmeACParams.sta_edcfParams[i].aifsn =  tls->qos.edcf_info[1][i].aifsn;

            init_cfg.wmeACParams.ap_edcfParams[i].cwMin =   tls->qos.edcf_info[0][i].cwMin;
            init_cfg.wmeACParams.sta_edcfParams[i].cwMin =  tls->qos.edcf_info[1][i].cwMin;

            init_cfg.wmeACParams.ap_edcfParams[i].cwMax =   tls->qos.edcf_info[0][i].cwMax;
            init_cfg.wmeACParams.sta_edcfParams[i].cwMax =  tls->qos.edcf_info[1][i].cwMax;
            
            init_cfg.wmeACParams.ap_edcfParams[i].txopLimit11b =   tls->qos.edcf_info[0][i].txopLimit11b;
            init_cfg.wmeACParams.sta_edcfParams[i].txopLimit11b =  tls->qos.edcf_info[1][i].txopLimit11b;
            
            init_cfg.wmeACParams.ap_edcfParams[i].txopLimit11g =   tls->qos.edcf_info[0][i].txopLimit11g;
            init_cfg.wmeACParams.sta_edcfParams[i].txopLimit11g =  tls->qos.edcf_info[1][i].txopLimit11g;

            init_cfg.wmeACParams.ap_edcfParams[i].ackPolicy =   tls->qos.edcf_info[0][i].ackPolicy;
            init_cfg.wmeACParams.sta_edcfParams[i].ackPolicy =  tls->qos.edcf_info[1][i].ackPolicy;
            
            init_cfg.wmeACParams.ap_edcfParams[i].shortRetryLimit =   tls->qos.edcf_info[0][i].shortRetryLimit;
            init_cfg.wmeACParams.sta_edcfParams[i].shortRetryLimit =  tls->qos.edcf_info[1][i].shortRetryLimit;
			
            init_cfg.wmeACParams.ap_edcfParams[i].longRetryLimit =   tls->qos.edcf_info[0][i].longRetryLimit;
            init_cfg.wmeACParams.sta_edcfParams[i].longRetryLimit =  tls->qos.edcf_info[1][i].longRetryLimit;
            
            init_cfg.wmeACParams.ap_edcfParams[i].dot11EDCATableMSDULifetime =   tls->qos.edcf_info[0][i].txMaxLifetime;
            init_cfg.wmeACParams.sta_edcfParams[i].dot11EDCATableMSDULifetime =  tls->qos.edcf_info[1][i].txMaxLifetime;
            
            /* TODO : Finish configuration */
            
            init_cfg.wmeACParams.ap_edcfParams[i].txopContinuation =   tls->qos.edcf_info[0][i].txOpContinuation;
            init_cfg.wmeACParams.sta_edcfParams[i].txopContinuation =  tls->qos.edcf_info[1][i].txOpContinuation;
		}

		init_cfg.wmeACParams.classifierMode = tls->qos.classifierMode;

		if (tls->qos.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) {
			init_cfg.wmeACParams.numOfCP = tls->qos.diffserv.numOfCP;
			memcpy(init_cfg.wmeACParams.diffServCPToUPMap, tls->qos.diffserv.map, WDRV_QOS_MAX_CP*sizeof(__u8));
		} 
		}
		break;
		case TIAP_PARAM_AUTH_TYPE:
			if( param_auth_tbl(&init_cfg, tls->auth_tbl) )
				ret = TIAP_PARAM_ERANGE; 
			break;
		case TIAP_PARAM_WD:
			if( param_wd(&init_cfg, &tls->wd) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_INTERF_WD_CONFIG:
			if ( param_interference_wd_config( &init_cfg, &tls->interf_wd_conf)) {
				ret = TIAP_PARAM_ERANGE;
			}
			break;
		case TIAP_PARAM_ACCESS_LIST:
			if( param_access_list(&init_cfg, &tls->access_list) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_4X:
			if( param_4x(&init_cfg, &tls->_4x) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_MULTI_SSID:
			if( param_multi_ssid(&init_cfg, &tls->multi_ssid) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CFG_POWER_TABLE:
			if( param_radio(&init_cfg, &tls->radio) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CHANNEL_RANGE:
			if( param_ch_rang(&init_cfg, &tls->ch_rang) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_WONE_ELEM:
			if( param_wone(&init_cfg, &tls->wone) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_PRIV_REG_DOMAIN:
			if( param_priv_regdomain(&init_cfg, &tls->priv_regdomain) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CURR_REG_DOMAIN:
			if( param_curr_reg_domain(&init_cfg, &tls->cur_regdomain) )
				ret = TIAP_PARAM_ERANGE;
			break;
        case TIAP_PARAM_BIT:
			if( param_bit_args(&init_cfg, &tls->bit) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_HW_INTERF_CFG:
			if (param_hw_interf_cfg(&init_cfg, &tls->hw_interf) ) 
				ret = TIAP_PARAM_ERANGE;
			break;
	case TIAP_PARAM_EEPROM:
		if (param_eeprom(&init_cfg, &tls->eeprom))
			ret = TIAP_PARAM_ERANGE;
		break;

    default:
			printk("%s failed - Invalid argument\n", __FUNCTION__);
			ret = -EINVAL;
	}

	up(&sem);
	return ret;
}


/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets authentication algorithm table
	
	\param auth_tbl			a buffer for output by referrence
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_auth_tbl(tiap_param_auth_algo_t *auth_tbl)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		BOOL temp_status; /*BOOL defined as u32, but 'enable' field as u8*/

		wdrv_dbGetAuthAlgo(0, &auth_tbl[0].algo);
		wdrv_dbGetAuthAlgoEnable(0, &temp_status);
		auth_tbl[0].enable = (u8)temp_status;
		wdrv_dbGetAuthAlgo(1, &auth_tbl[1].algo);
		wdrv_dbGetAuthAlgoEnable(1, &temp_status);
		auth_tbl[1].enable = (u8)temp_status;
	}
	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets Watch Dog parameters
	
	\param wd			wd structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_wd(tiap_param_wd_t *wd)
{
	int ret = 0;
	__u8  a, b, c, e, f;
	__u16 d;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret_err(ret, wd->cmd);
	else
	{
		a = b = c = d = 0;
		switch( wd->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				wdrv_dbGetEnRestart(&wd->value.val8);
				break;

			case TIAP_PARAM_WD_INTERF:
				wdrv_dbGetEnInterfWA(&wd->value.val8);
				break;
				
			case TIAP_PARAM_WD_RXFREE:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val8 = a;
				break;

			case TIAP_PARAM_WD_BEACONS:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val8 = b;
				break;

			case TIAP_PARAM_WD_TXSTUCK:
				wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				wd->value.val8 = c;
				break;

			case TIAP_PARAM_WD_FCSERR:
				wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				wd->value.val16 = d;
				break;

			case TIAP_PARAM_WD_ACI:
				 wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				 wd->value.val8 = e;

			 case TIAP_PARAM_WD_NB:
				 wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				 wd->value.val8 = f;	 

			default:
				ret = TIAP_PARAM_ERANGE;
				printk(KERN_INFO "%s: invalid type %d\n", __func__, wd->cmd);
		}
	}
	
	return ret;
}

static int get_param_interference_wd(tiap_param_interference_wd_t *interf_param)
{
	int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		switch ( interf_param->cmd) {
		
			case TIAP_PARAM_ACI_TIMEOUT:
				wdrv_dbGetAciStateTimeout(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_NB_TIMEOUT:
				wdrv_dbGetNbStateTimeout(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_ACI_RATE_THR:
				wdrv_dbGetAciRateThreshold(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_ACI_EVENT_THR:
				wdrv_dbGetAciEventsThreshold(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_PD_HIGH:
				wdrv_dbGetPacketDetectHighThresh(&(interf_param->value.val16));
				break;
			case TIAP_PARAM_PD_LOW:
				wdrv_dbGetPacketDetectLowThresh(&(interf_param->value.val16));
				break;
			case TIAP_PARAM_MEDIUM_BUSY_THR:
				wdrv_dbGetMediumBusyThresh(&(interf_param->value.val32));
				break;
			
			default:
				 printk(KERN_INFO "%s: Invalid command type: %d\n", __func__, interf_param->cmd);
				 ret = TIAP_PARAM_ERANGE;
		}
	}

	return ret;
}

static int get_param_hw_interf(tiap_param_hw_interf_t *interf_param)
{
	int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		HwInterfParams_T hw_interf_params;
		wdrv_dbGetHwInterfParams(&hw_interf_params);

		switch ( interf_param->cmd) {
		
			case TIAP_PARAM_INPUT_CCA:
				interf_param->value = hw_interf_params.inputCCA;
				break;
			case TIAP_PARAM_TX_CCA:
				interf_param->value = hw_interf_params.txCCA;
				break;
			case TIAP_PARAM_QUALIF_CCA:
				interf_param->value = hw_interf_params.qualifiedCCA;
				break;
			case TIAP_PARAM_STOMP_RX:
				interf_param->value = hw_interf_params.stompForRx;
				break;
			case TIAP_PARAM_STOMP_TX:
				interf_param->value = hw_interf_params.stompForTx;
				break;
			
			default:
				 printk(KERN_INFO "%s: Invalid command type: %d\n", __func__, interf_param->cmd);
				 ret = TIAP_PARAM_ERANGE;
		}
	}

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets access list
	
	\param access_list	access list structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_access_list(tiap_param_access_list_t *access_list)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret_err(ret, access_list->cmd);
	else {
		if (access_list->cmd == TIAP_PARAM_ACCESS_LST )
		{
			int n = sizeof(access_list->list);
			
			mgmtAccessGetEnabled(&access_list->enabled);
			mgmtAccessGetType((mgmt_access_list_type_t *)&access_list->type);
			mgmtAccessGetList((char *)access_list->list, &n);
			
		}
		else
		{
			printk(KERN_INFO "%s: Invalid command type: %d\n", __func__, access_list->cmd);
			ret = TIAP_PARAM_ERANGE;
		}
	}

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets 4x parameters
	
	\param _4x			4x structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_4x(tiap_param_4x_t *_4x)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret_err(ret, _4x->cmd);
	else
	{
		BOOL temp_status;

		switch( _4x->cmd )
		{
			case TIAP_PARAM_4X_FEATSET:
				wdrv_dbGetFeatureSet(&_4x->value);
				break;
			case TIAP_PARAM_4X_4X:
				wdrv_dbGet4xEnable(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			case TIAP_PARAM_4X_CAT:
				wdrv_dbGetConcatEnable(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			case TIAP_PARAM_4X_PKTBURST:
				/*not supported*/
				break;
			case TIAP_PARAM_4X_MIXMOD:
				wdrv_dbGet4xMixMode(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			default:
				dda_wext_init_ret_err(ret, _4x->cmd);
		}
	}

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets multiple ssid parameters
	
	\param multi_ssid			mutliple ssid structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_multi_ssid(tiap_param_multi_ssid_t *multi_ssid)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		int i;

		apMultiSsidElem_t apMultiSsid;
		for(i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
		{
			wdrv_mgmt_ssidGetElementFromMultipleSsidActualTable(&apMultiSsid, i);
			memcpy(multi_ssid->tbl[i].ssid, apMultiSsid.ssid.serviceSetId, IW_ESSID_MAX_SIZE);
			multi_ssid->tbl[i].ssid[apMultiSsid.ssid.hdr.eleLen] = '\0';
			multi_ssid->tbl[i].ssid_type = apMultiSsid.ssidType;
			multi_ssid->tbl[i].vlan = apMultiSsid.vlanId;
		}
		wdrv_mgmt_ssidGetMultipleSsidStatus(&multi_ssid->secondary_enabled);
		
	}

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets radio parameters
	
	\param radio			radio parameters structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_radio(tiap_param_radio_t *radio)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		RadioParam_t r;
		wdrv_dbGetRadioParams(&r);
		radio->mask = r.overrideBitmask;
		radio->tbl.type = r.powerTable_paramType;
		memcpy((void *)&radio->tbl.param, (void *)&r.powerTable_param, sizeof(radio->tbl.param));
	}

	return ret;
}


/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets channel range parameters
	
	\param ch_rang		channel range parameters structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_ch_rang(tiap_param_channel_rang *ch_rang)
{
int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else
	{
		int i;
		
		if( ch_rang->type != TIAP_PARAM_RANG_A && ch_rang->type != TIAP_PARAM_RANG_G ) {
			printk(KERN_INFO "%s: invalid argument %d\n", __func__, ch_rang->type);
			return 1;
		}

		wdrv_dbGetNumOfPrivateRanges(ch_rang->type, &ch_rang->ranges_nr);
		
		for(i=1; i<=ch_rang->ranges_nr; i++)
			if( ch_rang->type == TIAP_PARAM_RANG_A )
				/* Warning: if tChannelRangeElementStruct changes this won't work right ! */
				wdrv_dbGetPrivateRangeElement(ch_rang->type, i, (tChannelRangeElementStruct *)(&ch_rang->rng.a[i-1]));

			else 
				/* Warning: if tChannelRangeElementStruct changes this won't work right ! */
				wdrv_dbGetPrivateRangeElement(ch_rang->type, i, (tChannelRangeElementStruct *)(&ch_rang->rng.g[i-1]));
	}

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler.
		   Gets parameters which are configured separately for each wone index
	
	\param wone	   a structure which contains parameter type and value for that parameter
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_wone(tiap_param_wone_t *wone)
{
	u8 index = wone->index;
	int   i,k, ret = 0;
	u8 cckRateValue[CCK_PBCC_RATES_MAX] = 
		{DOT11_RATE_1,DOT11_RATE_2,DOT11_RATE_5_5,DOT11_RATE_11,DOT11_RATE_22};
	u8 ofdmRateValue[OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };
    char extRates[OFDM_RATES_MAX];
    UINT8 extRatesLen;
    char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
    UINT8 ratesLen;
    UINT8 ratesCounter = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret_err(ret, wone->type);
	else
	{
		switch(wone->type)
		{
			/*TODO: Felix R.: implement functions for rates. */
			case wone_oper_rate:
                wdrv_dbGetStationConfigOperationalRateSet(index, rates, &ratesLen);

				for(i=0; i<CCK_PBCC_RATES_MAX; i++)
				{
					wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<ratesLen; k++)
					{
						if(rates[k] == cckRateValue[i])
						{
							wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
                            ratesCounter++;
							break;
						}
						else if((rates[k] & 0x7F)== cckRateValue[i])
						{
							wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
                            ratesCounter++;
							break;
						}
					}
				}
				wone->elem.b_legacy.len = ratesCounter;
				break;

			case wone_oper_add:
                wdrv_dbGetStationConfigOperationalRateSet(index, rates, &ratesLen);
                
				for(i=0; i<OFDM_RATES_MAX; i++)
				{
					wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<ratesLen; k++)
					{
						if(rates[k] == ofdmRateValue[i])
						{
							wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
                            ratesCounter++;
							break;
						}
						else if((rates[k] & 0x7F)== ofdmRateValue[i])
						{
							wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
                            ratesCounter++;
							break;
						}
					}
				}
				wone->elem.ofdm_legacy.len = ratesCounter;
				break;

			case wone_oper_ofdm:		
                wdrv_dbGetStationConfigOperationalOFDMRateSet(index, extRates, &extRatesLen);

				for(i=0; i<OFDM_RATES_MAX; i++)
				{
					wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<extRatesLen; k++)
					{
						if(extRates[k] == ofdmRateValue[i])
						{
							wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
							break;
						}
						else if((extRates[k] & 0x7F) == ofdmRateValue[i])
						{
							wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
							break;
						}
					}
				}
				wone->elem.ofdm_extend.len = extRatesLen;
				break;

			case wone_brcst_rate:
				wdrv_dbGetDefaultActiveRateBrcst(index, &wone->elem.brcst_default_rate);
				break;
			case wone_mcst_rate:
				wdrv_dbGetDefaultActiveRateMcst(index, &wone->elem.mcst_default_rate);
				break;
			case wone_tx_rate:
				wdrv_dbGetGenTxActiveRate(index, &wone->elem.tx_default_rate);
				break;
			case wone_ch:
				wdrv_dbGetChannel(index, &wone->elem.ch);
				break;

			case wone_band:
				wdrv_dbGetBand(index, &wone->elem.band);
				break;

			case wone_pbcc:
				wdrv_dbGetPBCCAlgorithmMode(index, &wone->elem.pbcc_mode);
				
				break;

			case wone_rate_adapt:
				wdrv_dbGetRateAdaptation(index, &wone->elem.rate_adapt);
				
				break;

			case wone_pream_mode:
				wdrv_dbGetPreambleMode(index, &wone->elem.pream_mode);
				
				break;

			case wone_slot_mode:
				wdrv_dbGetSlotTimeMode(index, &wone->elem.slot_mode);
				
				break;

			case wone_erp_mode:	
				wdrv_dbGetERPProtectionMode(index, &wone->elem.erp_mode);
				
				break;

			case wone_erp_type:
				wdrv_dbGetERPProtectionType(index, &wone->elem.erp_type);
				
				break;

			case wone_erp_ti:
				wdrv_dbGetERPProtectionTI(index, &wone->elem.erp_ti);
				
				break;

			case wone_dot11_mode:
				wdrv_dbGetDot11Mode(index, &wone->elem.dot11_mode);
				
				break;

			case wone_cwmin_mode:
				wdrv_dbGetCwMinMode(index, &wone->elem.cwmin_mode);
				
				break;

			case wone_usage_time:
				wdrv_dbGetUsageTime(index, &wone->elem.usage_time);
				
				break;

			default:
				return DDC_INIT_CFG_LOGICAL_ERROR;

		}
	}

	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets station table
	
	\param sta_tbl		a buffer for output by referrence 
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_sta_tbl(tiap_param_statbl_t *sta_tbl)
{
	int     ssid_len = 0, ret = 0, profile_index = 0;
	__u8	brcst_cipher;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else
	{
		__u16 i;
		for(i=0; i<TIAP_PARAM_MAX_STATIONS; i++)
		{
			wdrv_dbGetStaMacAddress(i, sta_tbl[i].hwaddr);
			wdrv_dbGetStaSsidByIndex(i, sta_tbl[i].ssid, &ssid_len);
			sta_tbl[i].ssid[ssid_len] = '\0';
			wdrv_mgmtGetState(i, &sta_tbl[i].state);
			/*removed since it returns max active rate,
			 i.e. not updated after rate adaptation*/
			/*wdrv_dbGetActiveRate(i, &sta_tbl[i].active_rate);*/
			sta_tbl[i].active_rate = 
				convertAcxRateToDOT11Rate(tx_getStaActiveRate(i));
			wdrv_dbGetStationTxPacketCounter(i, &(sta_tbl[i] .txCounter));
			wdrv_dbGetStationRxPacketCounter(i, &sta_tbl[i].rxCounter);
			sta_tbl[i].wme = wdrv_dbGetStaWME(i);

			wdrv_mgmt_ssidGetStationSsidIndex(i, &profile_index);
			wdrv_mgmt_ssidGetBroadcastSuiteType(&brcst_cipher, profile_index);
			if (brcst_cipher == CIPHER_WEP) 
			{
				__u16	key_len;
				__u8    dummy[TIAP_WPA_WEP_KEY_LEN_MAX];
				
				wdrv_dbGetDefaultWepKey(0, &key_len, dummy);
				switch(key_len) 
				{
				case TIAP_WPA_WEP_40_KEY_LEN:
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_40;
					break;
				case TIAP_WPA_WEP_104_KEY_LEN :
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_104;
					break;
				case TIAP_WPA_WEP_232_KEY_LEN :
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_232;
					break;
				default:
				}
			}
			else
			{
				wdrv_dbGetStationSuiteType(i, &sta_tbl[i].cipher);
			}
		}
	}
	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets private regulatory domain parameters
	
	\param priv_regdom  private regulatory domain parameters structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_priv_regdomain(tiap_param_priv_reg_domain_t *priv_regdom)
{

	if (priv_regdom)
	{
		wdrv_dbGetPrivateRegulatoryDomain(&priv_regdom->domain, priv_regdom->band);
	}
	else printk (KERN_INFO "dda: %s: input pointer is NULL\n", __func__);
	
	return 0;	
}				


/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets current regulatory domain parameters
	
	\param val  current regulatory domain parameters structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_curr_reg_domain(tiap_param_cur_reg_domain_t *val)
{

	if (val)
	{
		val->type = 0; /*don't remove it*/
		wdrv_dbGetdot11CurrentRegDomain(val->band,(__u8 *)&val->type);
	}
	else printk (KERN_INFO "dda: %s: input pointer is NULL\n",__func__);
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets bss statistics parameters
	
	\param stats  a buffer for output by referrence
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int get_param_stats(tiap_param_net_stats *stats)
{
	int ret = 0;

	if( !atomic_read(&after_go) )
		dda_wext_init_ret(ret);
	else {
		/* 
		 * bss statistics
		 */
		/*tx*/
		wdrv_bssBridgeGetLanSendUnicast(&stats->bss.tx_frames);
		wdrv_bssBridgeGetLanSendMulticast(&stats->bss.tx_multicast_frames);
		/*rx*/
		wdrv_bssBridgeGetLanRecvUnicast(&stats->bss.rx_frames);
		wdrv_bssBridgeGetLanRecvMulticast(&stats->bss.rx_multicast_frames);

		/* 
		 * wireless statistics
		 */
		/*tx*/
		wdrv_dbGetCountersTransmittedFragmentCount(&stats->wireless.tx_mpdus);
		wdrv_dbGetCountersTransmittedFrameCount(&stats->wireless.tx_msdus);
		wdrv_dbGetCountersMulticastTransmittedFrameCount(&stats->wireless.tx_multicast_msdus);
		wdrv_dbGetCountersFailedCount(&stats->wireless.tx_failed_msdus);
		wdrv_dbGetCountersRetryCount(&stats->wireless.tx_retry_msdus);

		/*rx*/
		wdrv_dbGetCountersReceivedFragmentCount(&stats->wireless.rx_mpdus);
		wdrv_rxGetRxMsduReadyCount(&stats->wireless.rx_msdus);
		wdrv_dbGetCountersMulticastReceivedFrameCount(&stats->wireless.rx_multicast_msdus);
		wdrv_dbGetCountersFCSErrorCount(&stats->wireless.rx_fcs_error_mpdus);
		wdrv_dbGetPrivacyTKIPMicFailure(&stats->wireless.rx_mic_failure_msdus);
		wdrv_dbGetPrivacyWEPICVErrorCount(&stats->wireless.rx_decrypt_error_mpdus);
	}
	
	return ret;
}


/******************************************************************************/

/*! \brief 	Private subioctl handler to get TNETW die id

	\param 	pDieId  - pointer to die id structure to copy result

	\return  0 in case of success, error code otherwise
*/
/******************************************************************************/

static int get_param_die_id( tiap_param_die_id_t * pDieId )
{
 tnetw_die_id_t pTnetwDieId; 

 if (!pDieId) 
 {
     printk(KERN_INFO "%s: NULL pointer to die id structure.\n", __func__);
     return 1 ;
 }
 
 if (whal_apiGetTnetwDieId( &pTnetwDieId ) == NOK)
     return 1;

 pDieId->die_id_lo = pTnetwDieId.die_id_lo;
 pDieId->die_id_hi = pTnetwDieId.die_id_hi;
 
 return 0;
}


/*******************************************************************************************/
/*! 
	\brief  Common private ioctl handler for all get subioctls which have complex data type argument
			Calls appropriate get subioctl handler depending on input subioctl ID
	
	\param dev 		ignored
	\param info		ignored
	\param wrqu 	contains subioctl ID (wrqu->data.flags) and  a pointer to an input in user space
	\param extra	used as pointer to an input for subioctl in kernel space.
					Input is copied to this pointer from user.	
	
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_priv_get_complex_param(struct net_device *dev, struct iw_request_info *info, 
					union iwreq_data *wrqu, char *extra) {
union the_largest_struct *tls = (union the_largest_struct *)extra;
int subcmd = wrqu->data.flags;
int ret = 0;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	/* Standard Wireless Extensions don't provide a way to pass information
	from user in GET ioctls besides data.flags and data.length .
	Thus we're enforced to do it by ourselves */
	copy_from_user(extra, wrqu->data.pointer, sizeof(union the_largest_struct));

	switch (subcmd)	{
	case TIAP_PARAM_QOS: 
		{
			wdrv_QoSACParams_t params;
			int i;
			printk("%s: Get QoS ioctl\n", __FUNCTION__);
			params = wdrv_dbGetWMEParams();

			memset(&tls->qos, 0, sizeof(tiap_param_qos_t));

			tls->qos.enabled = params.enabled;

			for (i=0; i<4; ++i) {
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].aifsn = params.ap_edcfParams[i].aifsn;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].aifsn = params.sta_edcfParams[i].aifsn;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].cwMin = params.ap_edcfParams[i].cwMin;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].cwMin = params.sta_edcfParams[i].cwMin;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].cwMax = params.ap_edcfParams[i].cwMax;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].cwMax = params.sta_edcfParams[i].cwMax;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txopLimit11b = params.ap_edcfParams[i].txopLimit11b;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txopLimit11b = params.sta_edcfParams[i].txopLimit11b;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txopLimit11g = params.ap_edcfParams[i].txopLimit11g;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txopLimit11g = params.sta_edcfParams[i].txopLimit11g;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].ackPolicy = params.ap_edcfParams[i].ackPolicy;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].ackPolicy = params.sta_edcfParams[i].ackPolicy;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].shortRetryLimit = params.ap_edcfParams[i].shortRetryLimit;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].shortRetryLimit = params.sta_edcfParams[i].shortRetryLimit;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].longRetryLimit = params.ap_edcfParams[i].longRetryLimit;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].longRetryLimit = params.sta_edcfParams[i].longRetryLimit;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txMaxLifetime = params.ap_edcfParams[i].dot11EDCATableMSDULifetime;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txMaxLifetime = params.sta_edcfParams[i].dot11EDCATableMSDULifetime;
				
				tls->qos.edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txOpContinuation = params.ap_edcfParams[i].txopContinuation;
				tls->qos.edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txOpContinuation = params.sta_edcfParams[i].txopContinuation;
			}

			tls->qos.classifierMode = params.classifierMode;

			if (tls->qos.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) {
				tls->qos.diffserv.numOfCP = params.numOfCP;
				memcpy(tls->qos.diffserv.map, params.diffServCPToUPMap, TIAP_PARAM_QOS_MAX_CP*sizeof(__u8));
			} 
		}
		break;
		case TIAP_PARAM_AUTH_TYPE:
			if( get_param_auth_tbl(tls->auth_tbl) )
				ret = TIAP_PARAM_ERANGE; 
			break;
		case TIAP_PARAM_WD:
			if( get_param_wd(&tls->wd) )
				ret = TIAP_PARAM_ERANGE;
			break;

    	case TIAP_PARAM_INTERF_WD_CONFIG:
			if( get_param_interference_wd(&tls->interf_wd_conf) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_ACCESS_LIST:
			if( get_param_access_list(&tls->access_list) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_4X:
			if( get_param_4x(&tls->_4x) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_MULTI_SSID:
			if( get_param_multi_ssid(&tls->multi_ssid) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CFG_POWER_TABLE:
			if( get_param_radio(&tls->radio) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CHANNEL_RANGE:
			if( get_param_ch_rang(&tls->ch_rang) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_WONE_ELEM:
			if( get_param_wone(&tls->wone) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_STA_TBL:
			if( get_param_sta_tbl(tls->sta_tbl) )
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_PRIV_REG_DOMAIN:
			if (get_param_priv_regdomain(&tls->priv_regdomain))
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_CURR_REG_DOMAIN:
			if (get_param_curr_reg_domain(&tls->cur_regdomain))
				ret = TIAP_PARAM_ERANGE;
			break;
		case TIAP_PARAM_NET_STATS:
			get_param_stats(&tls->net_stats);
			break;
		case TIAP_PARAM_HW_INTERF_CFG:
			get_param_hw_interf(&tls->hw_interf);
			break;
        case TIAP_PARAM_DIE_ID:
            if (get_param_die_id(&tls->die_id))
               ret = TIAP_PARAM_ERANGE;
            break;
    
        default:
			ret = -EINVAL;
	}


	up(&sem);
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Set TX power parameters
	
	\param dev		ignored
	\param info 	ignored
	\param rrq		input tx power structure
	\param extra	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_siwtxpow(struct net_device *dev, struct iw_request_info *info,
			 struct iw_point *rrq, char *extra)
{
	int ret = 0;
	tiap_param_txpower_t *txpower = NULL;

	if( !(txpower = (tiap_param_txpower_t *)rrq) )
		return -EINVAL;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	switch( txpower->hr )
	{
		case txpower_a:
			memcpy((void *)init_cfg.RegInit.txPowerLevels_a, (void *)txpower->txpower,
			 	sizeof(txpower->txpower));
			configured_params.txpow_a = 1;
			break;
		case txpower_g:
			memcpy((void *)init_cfg.RegInit.txPowerLevels_g, (void *)txpower->txpower,
				sizeof(txpower->txpower));
			configured_params.txpow_g = 1;
			break;
		default:
			printk(KERN_INFO "%s: invalid type %d\n", __func__, txpower->hr);
			dda_wext_init_ret_err(ret, txpower->hr);
	}

	up(&sem);
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Get TX power parameters
	
	\param dev		ignored
	\param info 	ignored
	\param rrq		tx power structure
	\param extra	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_giwtxpow(struct net_device *dev, struct iw_request_info *info,
			 struct iw_point *rrq, char *extra)
{
	tiap_param_txpower_t *txpower = NULL;
	int lev;

	if( !(txpower = (tiap_param_txpower_t *)rrq) )
		return -EINVAL;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	switch( txpower->hr )
	{

		case txpower_a:
			for(lev=1; lev<=TIAP_PARAM_NUM_TXPOW_LEVELS; lev++)
				wdrv_dbGetTxPowerLevels(RADIO_BAND_5GHZ, lev, &(txpower->txpower[lev-1]));
			break;
		case txpower_g:
			for(lev=1; lev<=TIAP_PARAM_NUM_TXPOW_LEVELS; lev++)
				wdrv_dbGetTxPowerLevels(RADIO_BAND_2_4GHZ, lev, &(txpower->txpower[lev-1]));
			break;
		default:
			printk(KERN_INFO "%s: invalid type %d\n", __func__, txpower->hr);
	}

	
	up(&sem);
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Set RTS threshold
	
	\param dev		ignored
	\param info 	ignored
	\param rts		input rts value
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_siwrts(struct net_device *dev, struct iw_request_info *info,
		struct iw_param *rts, char *dummy)
{
	__u16 value, 
	      *dest;
	int   ret = 0;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if( !atomic_read(&after_go) )
	{
		if (rts){
			value = (__u16)rts->value;
			if ( value <= TIAP_PARAM_RTS_TRESHOLD_MAX ){
				dest = &init_cfg.apRtsThreshold; 
				memset ( dest, 0 , sizeof(*dest)); 
				memcpy ( dest, &value, sizeof (*dest));
				configured_params.rts = 1;
			}	
			else {
				printk (KERN_INFO "dda: %s: %d - value is out of range \n", __func__, value);
				ret = -EINVAL;
			}
		}
		else 
			printk (KERN_INFO "dda: siwrts:input pointer is NULL\n");
	}
	else
	      dda_wext_rt_ret(ret);

	up(&sem);	
	return ret;

}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Get RTS threshold
	
	\param dev		ignored
	\param info 	ignored
	\param rts		output buffer
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_giwrts(struct net_device *dev, struct iw_request_info *info,
							struct iw_param *rts, char *dummy)
{
	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if (rts){
		wdrv_dbGetOperationRTSThreshold((__u16 *)&rts->value);
		rts -> fixed = 1;
	}
	else printk (KERN_INFO "dda: giwrts: input pointer is NULL\n");
	
	up(&sem);
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Set Fragmentation threshold
	
	\param dev		ignored
	\param info 	ignored
	\param frag		input fragmentation threshold value
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_siwfrag (struct net_device *dev, struct iw_request_info *info,
		struct iw_param *frag, char *dummy)
{
	int ret = 0;
	__u16 value;
	__u16 *dest;


	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if( !atomic_read(&after_go) )
	{
		if (frag) {
			value = (__u16)frag->value;
			if (value >=TIAP_PARAM_MIN_FRAGMENTATION_TRESHOLD &&
			    value <= TIAP_PARAM_MAX_FRAGMENTATION_TRESHOLD 
						   && value % 2==0 ) /* even numbers only */
			{
				dest = &init_cfg.apFragThreshold; 
				memset ( dest, 0 , sizeof(*dest)); 
				memcpy ( dest, &value, sizeof (*dest));
				configured_params.frag = 1;
			
			}
			else {
				printk (KERN_INFO "dda: %s: %d - value is out of range \n", __func__, value);
				ret = -EINVAL;
			}
		}
		
		else printk (KERN_INFO "dda: frag: input pointer  is NULL\n");
	}
	else
	    dda_wext_rt_ret(ret);

	up(&sem);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Get Fragmentation threshold
	
	\param dev		ignored
	\param info 	ignored
	\param frag		output buffer
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_giwfrag (struct net_device *dev, struct iw_request_info *info,
		struct iw_param *frag, char *dummy)
{
	
	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if (frag) {
		wdrv_dbGetOperationFragmentationThreshold ((__u16 *)&frag -> value );
		frag->fixed = 1;
			
	}
	else printk (KERN_INFO "dda: frag: input pointer is NULL\n");

	up(&sem);
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Set short/long retry limit
	
	\param dev		ignored
	\param info 	ignored
	\param retry	input retry limit structure
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_siwretry (struct net_device *dev, struct iw_request_info *info,
		struct iw_param *retry, char *dummy)
{
	int ret = 0;
	
	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if (!atomic_read(&after_go))
	{
		if (retry)
		{
			if (retry->value)
			{ 
				/*the each retry limit should be non-zero value */
				switch(retry->flags) /*tells which retry limit to set */
				{
				case TIAP_PARAM_RETRY_LONG: /* long retry limit*/ 
					init_cfg.apLongRetryLimit = (__u8)retry->value;
					configured_params.retry_long = 1;
					break;
				case TIAP_PARAM_RETRY_SHORT: /* short retry limit*/ 
					init_cfg.apShortRetryLimit = (__u8)retry->value;
					configured_params.retry_short = 1;
					break;
				case TIAP_PARAM_LIFETIME_TX:
					init_cfg.apMaxTxLifetime = retry->value;
					configured_params.lifetime_tx = 1;
					break;
				case TIAP_PARAM_LIFETIME_RX:
					init_cfg.apMaxRxLifetime = retry->value;
					configured_params.lifetime_rx = 1;
					break;
				default:printk(KERN_INFO "%s: flag is out of range %d - \n",
						__func__, retry->flags);
					ret = -EINVAL;
				}
					
			}
			else {
				printk(KERN_INFO "%s: %d - value is out of range, flag=%d\n", 
						__func__, retry->value, retry->flags);
				ret = -EINVAL;
			}	
		}
		else 
			printk (KERN_INFO "dda: retry: input pointer is NULL\n");
	}
	else
		dda_wext_rt_ret(ret);
	
	up(&sem);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Standard WE ioctl handler. Get short/long retry limit
	
	\param dev		ignored
	\param info 	ignored
	\param retry	retry limit structure
	\param dummy	ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_giwretry (struct net_device *dev, struct iw_request_info *info, 
							   struct iw_param *retry, char *dummy)
{
	__u8 *dest;
	int ret = 0;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if (retry)
	{
		dest = (__u8 *)&retry->value;
		switch (retry -> flags)
		{
		case 0:
			 
			wdrv_dbGetOperationLongRetryLimit(dest);
			break;
		case 1:
			
			wdrv_dbGetOperationShortRetryLimit(dest+1);
			break;
		case 2:
			
			wdrv_dbGetOperationLongRetryLimit(dest);
			wdrv_dbGetOperationShortRetryLimit(dest+1);
			break;
		default:printk (KERN_INFO "dda: %s: value is out of range %d -  \n",__func__, retry->flags);
			ret = -EINVAL;
		}

		retry->fixed = 1;
	}
	else 
		printk (KERN_INFO "dda: %s: input pointer is NULL\n", __func__);

	up(&sem);
	return ret;
}


/* PRIVATE */

/* basic data types */

/*******************************************************************************************/
/*! 
	\brief  Common private ioctl handler for all set subioctls which get simple data type input. 
			Calls appropriate set subioctl handler depending on input subioctl ID
	
	\param dev 		ignored
	\param info		ignored
	\param dummy 	ignored
	\param data		first byte contains subioctl ID. The rest is input for subioctl	
	
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_priv_basic_param(struct net_device *dev, struct iw_request_info *info, 
								void *dummy, char *data)
{
int *d = (int *)data;
int param = *d;
int *val = d + 1;
int ret = 0;
	
	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	if( !val ) {
		printk(KERN_WARNING "%s: input pointer is NULL\n", __func__);
		return -EINVAL;
	}

	switch (param)
	{
		case TIAP_PARAM_REPLEV:
			ret = tiap_param_set_replev(*(__s8 *)val);
			break;
		case TIAP_PARAM_BEACON_PERIOD:
			ret = tiap_param_set_beaconperiod(&init_cfg, *(__u16 *)val);
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			ret = tiap_param_set_dtim_period(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			ret = tiap_param_set_bss_bridge_state(&init_cfg, *(tiap_param_bss_br_state_t *)val);
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			ret = tiap_param_set_rate_fallback(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_RADIO_CAL:
			ret = tiap_param_set_radio_cal(&init_cfg, *(__u32 *)val);
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			ret = tiap_param_set_radio_cal_int(&init_cfg, *(__u16 *)val);
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			ret = tiap_param_set_energy_detect(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			ret = tiap_param_set_hidden_ssid(&init_cfg, *(__u16 *)val);
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			ret = tiap_param_set_multidomain_cap(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			ret = tiap_param_set_spectrum_mgmt(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_LEGACY_STA:
			ret = tiap_param_set_legacy_sta(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			ret = tiap_param_set_local_power_cons(&init_cfg, *(__s8 *)val);
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			ret = tiap_param_set_max_tx_pow_level(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_COUNTRY_STR:
			ret = tiap_param_set_country_str(&init_cfg, (char *)val);
			break;
		case TIAP_PARAM_OPER_STATE:
			ret = tiap_param_set_oper_state(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_WONE_NUM:
			ret = tiap_param_set_wone_num(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			ret = tiap_param_set_wone_timemod(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			ret = tiap_param_set_radar_detect_mode(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			tiap_param_set_bridge_mode(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_WDS_ENABLE:
			tiap_param_set_wds_enable(&init_cfg, *(__u8 *)val);
			break;
		case TIAP_PARAM_EEPROM_LESS_ENABLE:
			ret = tiap_param_set_eeprom_less(&init_cfg, *(__u8 *)val);
			break;

		default:
			ret = -EINVAL;
	}

	up(&sem);	
	return ret;
}


/*******************************************************************************************/
/*! 
	\brief  Common private ioctl handler for all set subioctls which get simple data type input. 
			Calls appropriate set subioctl handler depending on input subioctl ID
	
	\param dev 		ignored
	\param info		ignored
	\param dummy 	ignored
	\param data		first byte contains input subioctl ID. And it is an output buffer at the same time
	
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_priv_get_basic_param(struct net_device *dev, struct iw_request_info *info,
		void *dummy, char *data)
{
int *d = (int *)data;
int *param = d;
int ret = 0;

	
	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	switch (*param)
	{
		case TIAP_PARAM_REPLEV:
			ret = tiap_param_get_replev(param);
			break;
		case TIAP_PARAM_BEACON_PERIOD:
			ret = tiap_param_get_beaconperiod(param);
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			ret = tiap_param_get_dtim_period(param);
			break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			ret = tiap_param_get_bss_bridge_state(param);
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			ret = tiap_param_get_rate_fallback(param);
			break;
		case TIAP_PARAM_RADIO_CAL:
			ret = tiap_param_get_radio_cal(param);
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			ret = tiap_param_get_radio_cal_int(param);
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			ret = tiap_param_get_energy_detect(param);
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			ret = tiap_param_get_hidden_ssid(param);
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			ret = tiap_param_get_multidomain_cap(param);
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			ret = tiap_param_get_spectrum_mgmt(param);
			break;
		case TIAP_PARAM_LEGACY_STA:
			ret = tiap_param_get_legacy_sta(param);
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			ret = tiap_param_get_local_power_cons(param);
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			ret = tiap_param_get_max_tx_pow_level(param);
			break;
		case TIAP_PARAM_COUNTRY_STR:
			ret = tiap_param_get_country_str(param);
			break;
		case TIAP_PARAM_OPER_STATE:
			ret = tiap_param_get_oper_state(param);
			break;
		case TIAP_PARAM_WONE_NUM:
			ret = tiap_param_get_wone_num(param);
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			ret = tiap_param_get_wone_timemod(param);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			ret = tiap_param_get_radar_detect_mode(param);
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			ret = tiap_param_get_bridge_mode(param);
			break;
		case TIAP_PARAM_WDS_ENABLE:
			ret = tiap_param_get_wds_enable(param);
			break;
		case TIAP_PARAM_EEPROM_LESS_ENABLE:
			ret = tiap_param_get_eeprom_less(param);
			break;

	
		default:
			ret = -EINVAL;
	}	

	up(&sem);	
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets or unsets given report level 
	
	\param value	report level number
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_replev(int value)
{
	if (value>=TIAP_PARAM_MIN_REPLEV && value<=TIAP_PARAM_MAX_REPLEV)
	{
		if (value > 0)
			PALSRV_report_set(value,0);
		else if (value < 0)
			PALSRV_report_unset(value * (-1),0);
		else
			PALSRV_report_disable(0);

		configured_params.replev = 1;
	}
	else {
		printk (KERN_INFO "dda: %s: %d - value is out of range \n", __func__, value);
		return TIAP_PARAM_ERANGE;
	}
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets report level bitmask
	
	\param val	output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_replev(int *val)
{
	if (val)
	{
		*val = PALSRV_report_get();

	}
	else printk (KERN_INFO "dda: replev: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets beacon period  
	
	\param init_cfg		init configuration structure
	\param value		beacon period value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_beaconperiod(initConfigStruct *init_cfg, __u16 value)
{
	int ret = 0;

	if (!atomic_read(&after_go))
	{
		if (value > 0){

			memset(&init_cfg->apBeaconPeriod,0,sizeof(init_cfg->apBeaconPeriod));
			memcpy(&init_cfg->apBeaconPeriod, (void *)&value, sizeof(init_cfg->apBeaconPeriod));
			configured_params.beacon_period = 1;
		}
		else {
			printk (KERN_INFO "dda: set beacon_period: %d - value is out of range \n", value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);


	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets beacon period  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_beaconperiod(int *val)
{
	if (val)
	{
		memset(val, 0, sizeof(*val));
		wdrv_dbGetStationConfigBeaconPeriod((__u16 *)val);
	}
	else printk (KERN_INFO "dda: beacon: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets dtim period
	
	\param init_cfg		init configuration structure
	\param value		dtim period value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_dtim_period(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go))
	{
		if (value >= TIAP_PARAM_MIN_DTIM_PERIOD && value <= TIAP_PARAM_MAX_DTIM_PERIOD){
			init_cfg->apDtimPeriod = value;
			configured_params.dtim_period = 1;
		}
		else {
			printk (KERN_INFO "dda: set dtim: %d - value is out of range \n", value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets DTIM period  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_dtim_period(int *val)
{
	if (val)
	{
		memset(val, 0, sizeof(*val));
		wdrv_dbGetStationConfigDtimPeriod ((__u8 *)val);
	}
	else printk (KERN_INFO "dda: %s: input pointer is NULL\n", __func__);
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets bss bridge state
	
	\param init_cfg		init configuration structure
	\param value		bss bridge state
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_bss_bridge_state(initConfigStruct *init_cfg, tiap_param_bss_br_state_t value)
{
	__u32 bss_bridge_state;

	switch (value)
	{
		case TIAP_PARAM_BSS_BR_ENABLE:
		case TIAP_PARAM_BSS_BR_DROP  :
						bss_bridge_state = BSS_TRAFFIC_EN;
						break;
		case TIAP_PARAM_BSS_BR_BLOCK :
						bss_bridge_state = BSS_TRAFFIC_BLOCK;
						break;
		default:
			printk (KERN_INFO "dda: set bssbridge: %d - value is out of range \n", value);
			return TIAP_PARAM_ERANGE;
	}

	if (!atomic_read(&after_go)) {
		init_cfg->bssBridgeState = bss_bridge_state; 
				configured_params.bss_bridge_state = 1;
			}
			else
			{
		wdrv_dbSetBssIsolationState(bss_bridge_state);
			}

	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets bss bridge state  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_bss_bridge_state(int *val)
{
	__u32 bss_bridge_state;

	if (val)
	{
		memset(val, 0, sizeof(*val));
		/* TODO uncomment when the func is ready */
		/* wdrv_dbGetBssBridgeState(&bss_isolation);*/

		/* TODO to remove the following line when the above function is ready*/
		bss_bridge_state = init_cfg.bssBridgeState;
		
		if (bss_bridge_state == BSS_TRAFFIC_BLOCK)
			*val = TIAP_PARAM_BSS_BR_BLOCK;
		else
			*val = TIAP_PARAM_BSS_BR_ENABLE;
			

	}
	else printk (KERN_INFO "dda: bss bridge: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets rate fallback value
	
	\param init_cfg		init configuration structure
	\param value		input rate fallback value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_rate_fallback(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go))
	{
		init_cfg->RateFallback = value;
		configured_params.rate_fallback = 1;
	}
	else dda_wext_rt_ret(ret);


	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets rate fallback value 
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_rate_fallback(int *val)
{
	if (val)
	{
		memset(val, 0, sizeof(*val));
		wdrv_dbGetRateFallback ((__u8 *)val);
	}
	else printk (KERN_INFO "dda: rfallback: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets radio calibaration value
	
	\param init_cfg		init configuration structure
	\param value		input radio calibration value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_radio_cal(initConfigStruct *init_cfg, __u32 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value <= TIAP_PARAM_MAX_RADIO_CAL){
			init_cfg->RadioCalibrateOption = value;
			configured_params.radio_cal = 1;
		}
		else{
			printk (KERN_INFO "dda: set radiocal: %d - value is out of range \n", value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets radio calibration value 
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_radio_cal(int *val)
{
	if (val) {
		wdrv_dbGetRadioCalibrateOption (val);
	}
	else 
		printk (KERN_INFO "dda: radiocal: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets radio calibaration interval value
	
	\param init_cfg		init configuration structure
	\param value		input radio calibration interval value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_radio_cal_int(initConfigStruct *init_cfg, __u16 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		init_cfg->RadioCalibrateInterval = value;
		configured_params.cal_interval = 1;
	}
	else dda_wext_rt_ret(ret);

	return ret ;
	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets radio calibration interval
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_radio_cal_int(int *val)
{
	if (val){
		wdrv_dbGetRadioCalibrateInterval((__u16 *)val);
	}
	else 
		printk (KERN_INFO "dda: radiocalint: input pointer is NULL\n");
	
	return 0;

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets radar detection mode  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_radar_detect_mode(int *val)
{
	if (val){
		wdrv_dbGetRadarDetectMode((__u8 *)val);
	}
	else 
		printk (KERN_INFO "%s: input pointer is NULL\n", __func__);
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets bridge mode  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_bridge_mode(int *val)
{
	if (val)
		*val = wdrv_dbGetApBridgeMode();
	else printk (KERN_INFO "%s: input pointer is NULL\n", __func__);
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets wds mode  
	
	\param val		output buffer
	
	RETURNS: 0 in case of success, error code otherwise
*/
static int tiap_param_get_wds_enable(int *val)
{
	if (val)
		*val = wdrv_dbGetWdsEnable();
	else printk (KERN_INFO "%s: input pointer is NULL\n", __func__);
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets energy detection mode
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_energy_detect(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value == 0 || value == 1){
			init_cfg->enableEnergyDetect = value;
			configured_params.energy_detect = 1;
		}
		else {
			printk (KERN_INFO "dda: set energy detect: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets energy detection mode
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_energy_detect(int *val)
{
	if (val) {
		wdrv_dbGetEnergyDetect ((__u8 *)val);
	}
	else 
		printk (KERN_INFO "dda: energydet: input pointer is NULL\n");
	
	return 0;
	
}
/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets eeprom less mode
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_eeprom_less(int *val)
{
	if (val) {
		wdrv_dbGetEepromLess ((__u8 *)val);
	}
	else 
		printk (KERN_INFO "dda: eepromLess: input pointer is NULL\n");
	
	return 0;
	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets hidden ssid mode
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_hidden_ssid(initConfigStruct *init_cfg, __u16 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if ( value == 0 || value == 1){
			init_cfg->apHiddenSSID = value;
			configured_params.hidden_ssid = 1;
		}
		else {
			printk (KERN_INFO "dda: set hidden ssid: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets hidden ssid modes  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_hidden_ssid(int *val)
{

	if (val)
	{
		wdrv_mgmt_ssidGetHiddenSsidStatus((__u16 *)val);
	}
	else 
		printk (KERN_INFO "dda: hidden ssid: input pointer is NULL\n");
	
	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets multidomain mode
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_multidomain_cap(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if ( value == 0 || value == 1){
			init_cfg->RegInit.dot11MultiDomainCapabilityEnable = value;
			configured_params.multi_domain_cap = 1;
		}
		else {	
			printk (KERN_INFO "dda: set multidomain: %d - value is out of range \n", value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets multidomain mode  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_multidomain_cap(int *val)
{

	if (val)
	{
		wdrv_dbGetdot11MultiDomainCapabilityStatus((__u8 *)val);
	}
	else printk (KERN_INFO "dda: multidomain: input pointer is NULL\n");
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets spectrum management mode
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_spectrum_mgmt(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value == 0 || value == 1){
			init_cfg->RegInit.dot11SpectrumManagementRequired = value;
			configured_params.spectrum_mgmt = 1;
		}
		else	
			return TIAP_PARAM_ERANGE;
	}
	else dda_wext_rt_ret(ret);
	

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets spectrum management mode  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_spectrum_mgmt(int *val)
{

	if (val)
	{
		wdrv_dbGetdot11SpectrumManagementRequiredStatus((__u8 *)val);
	}
	else printk (KERN_INFO "dda: spectrum_mgmt: input pointer is NULL\n");
	
	return 0;	

}


/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets legacy stations mode
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_legacy_sta(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value == 0 || value == 1){
			init_cfg->RegInit.whalEnableLegacyStations = value;
			configured_params.legacy_sta = 1;
		}
		else {
			printk (KERN_INFO "dda: set legacysta: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets legacy stations mode
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_legacy_sta(int *val)
{

	if (val)
	{
		wdrv_dbGetLegacyStatusEnableStatus((__u8 *)val);
		
	}
	else printk (KERN_INFO "dda: legacy_sta: input pointer is NULL\n");
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets local power constraint
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_local_power_cons(initConfigStruct *init_cfg, __s8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value >= TIAP_PARAM_MIN_POWER_CONS &&
		    value <= TIAP_PARAM_MAX_POWER_CONS){
			init_cfg->RegInit.dot11LocalPowerConstraint = value;
			configured_params.local_pow_cons = 1;
		}
		else	
			return TIAP_PARAM_ERANGE;
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets local power constraint modes  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_local_power_cons(int *val)
{

	if (val)
	{
		wdrv_dbGetdot11LocalPowerConstraint((__s8 *)val);
	}
	else 
		printk (KERN_INFO "dda: power_constr: input pointer is NULL\n");
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets max TX power level
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_max_tx_pow_level(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value >= TIAP_PARAM_MIN_TX_POW_LEVEL &&
				value <= TIAP_PARAM_MAX_TX_POW_LEVEL ){
			init_cfg->RegInit.whalMaxTxPowerLevel = value;
			configured_params.max_txpow_lev = 1;
		}
		else {
			printk (KERN_INFO "dda: set tx_powlevel: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Max TX power level
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_max_tx_pow_level(int *val)
{

	if (val)
	{
		wdrv_dbGetMaxTxPowerLevel((__s8 *)val);
	}
	else printk (KERN_INFO "dda: txpow_level: input pointer is NULL\n");
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets country string
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_country_str(initConfigStruct *init_cfg, char *value)
{
	int ret = 0;

	/*Warning: value is NOT null-terminated*/

	if (!atomic_read(&after_go)){
		switch (value[2])
		{
			case 'i':
			case 'o':
			case 'I':
			case 'O':
			case ' ':
				memset(init_cfg->RegInit.dot11CountryString,0,TIAP_PARAM_COUNTRY_STRING_LEN);
				memcpy(init_cfg->RegInit.dot11CountryString,value,TIAP_PARAM_COUNTRY_STRING_LEN);
				configured_params.country_str = 1;
				break;
			default:value[TIAP_PARAM_COUNTRY_STRING_LEN + 1] = '\0';
				printk (KERN_INFO "dda: %s: %s - value is out of range \n", __func__, value);
				return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

    return ret;	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets country string
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_country_str( int *val)
{
    if (val){	
	memset(val, 0, COUNTRY_STRING_LENGTH);
	wdrv_dbGetdot11CountryString((__s8 *)val);	
    }
    else 
	    printk (KERN_INFO "dda: country_str: input pointer is NULL\n");
	
    return 0;	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Enables/Disables WLAN service
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_oper_state(initConfigStruct *init_cfg, __u8 value)
{
	if (value == 0 || value == 1)
		if (!atomic_read(&after_go)){
			init_cfg->WlanServiceState = value;
			configured_params.oper_state = 1;
		}
		else{
			if(value == 0)
				wdrv_WlanServiceDisable();
			else
				wdrv_WlanServiceEnable();
		}		
	else
	{   
		printk (KERN_INFO "dda: set oper_state: %d - value is out of range \n",value);
		return TIAP_PARAM_ERANGE;
	}

	return 0;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets operational state  
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_oper_state(int *val)
{

	if (val)
	{
		wdrv_dbGetWlanServiceState(val);
	}
	else printk (KERN_INFO "dda: oper_state: input pointer is NULL\n");
	
	return 0;	

}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets wone number
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_wone_num(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value == 1 || value == 2){ /* 1- single band, 2-dual band*/
			init_cfg->numSupWone = value;
			configured_params.wone_num = 1;
		}
		else
		{   
			printk (KERN_INFO "dda: set wone_num: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets current wone number 
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_wone_num(int *val)
{

	if (val) wdrv_dbGetNumSupWone((__u8 *)val);
	else printk (KERN_INFO "dda: wone_num: input pointer is NULL\n");
	
	return 0;	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets wone time mode 
	
	\param init_cfg		init configuration structure
	\param value		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_wone_timemod(initConfigStruct *init_cfg, __u8 value)
{
	int ret = 0;

	if (!atomic_read(&after_go)){
		if (value == 0 || value == 1){ /* 0- short, 1-long*/
			init_cfg->usageTimeMode = value;
			configured_params.wone_timemod = 1;
		}
		else
		{   
			printk (KERN_INFO "dda: set wone_timemod: %d - value is out of range \n",value);
			return TIAP_PARAM_ERANGE;
		}
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets wone time mode
	
	\param val		output buffer
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_get_wone_timemod(int *val)
{

	if (val)
	{
		wdrv_dbGetUsageTimeMode((__u8 *)val);
	}
	else 
		printk (KERN_INFO "dda: wone timemode: input pointer is NULL\n");
	
	return 0;	
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets radar detection mode
	
	\param init_cfg		init configuration structure
	\param val		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_radar_detect_mode(initConfigStruct *init_cfg, __u8 val)
{
	int ret = 0;

	if (!atomic_read(&after_go)) {
		if (!val || val == 1) {
			init_cfg->RadarDetectMode = val;
			configured_params.radar_detect = 1;
		}
		else dda_wext_init_ret(ret);
	}
	else dda_wext_rt_ret(ret);

	return ret;


}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Enables/disables bridge mode
	
	\param init_cfg		init configuration structure
	\param val		input value
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int tiap_param_set_bridge_mode(initConfigStruct *init_cfg, __u8 val)
{
	int ret = 0;

	if (!atomic_read(&after_go)) {
		if (!val || val == 1) {
			init_cfg->ApBridgeMode = val;
			configured_params.bridge_mode = 1;
		}
		else dda_wext_init_ret(ret);
	}
	else dda_wext_rt_ret(ret);

	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Enables/disables WDS
	
	\param init_cfg		init configuration structure
	\param val		input value
	
	RETURNS: 0 in case of success, error code otherwise
*/
static int tiap_param_set_wds_enable(initConfigStruct *init_cfg, __u8 val)
{
	int ret = 0;

    if (!val || val == 1) {
        init_cfg->wdsEnable = val;
        configured_params.wdsEnable = 1;
        wdrv_dbSetWdsEnable(val);
    }
    else {
        dda_wext_init_ret(ret);
    }

    return (ret);
}
/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. set Eeprom Less Mode
	
	\param init_cfg		init configuration structure
	\param val		input value
	
	RETURNS: 0 in case of success, error code otherwise
*/
static int tiap_param_set_eeprom_less(initConfigStruct *init_cfg, __u8 val)
{
	int ret = 0;

    if (!val || val == 1) {
        init_cfg->enableEEPromLessSupport = val;
        configured_params.eeprom_less = 1;
        wdrv_dbSetEepromLess(val);
    }
    else {
        dda_wext_init_ret(ret);
    }

    return (ret);
}

/*******************************************************************************************/
/*! 
	\brief Intialization function for ioctl interface  
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
int dda_wext_init(void)
{
	/*otherwise wireless extensions won't work*/
	if ((sizeof(union the_largest_struct)/sizeof(int)) > IW_PRIV_SIZE_MASK) {
		printk(KERN_WARNING
			"The size of union the_largest_struct is too big, think of other trick.\n");
		return -ENOMEM;
	}

	sema_init(&sem,1);
	
	memset(&configured_params,0,sizeof(configured_params));

	atomic_set(&after_go, 0);

	return 0;
}

#include "whalAcxRegs.h"
#include "whalAcxAccess.h"
#include "whalAcxEeprom.h"

/*******************************************************************************************/
/*! 
	\brief Verfies if all initial parameters  have been configured. If so:
		   Calls driver intialization functions with that confgiguration
		   Starts the driver
	
	\param dev	 contains MAC address (dev->dev_addr)
	\param info  ignored
	\param dummy ignored
	\param extra ignored
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int dda_ioctl_priv_go(struct net_device *dev, struct iw_request_info *info,
														 void *dummy, char *extra) {
	int ret = 0, state;

	if (down_interruptible(&sem))
		return -ERESTARTSYS;

	memcpy(init_cfg.APMAC, dev->dev_addr, ETH_ALEN);
	configured_params.hwaddr = 1;

	if (! check_if_all_configured()) {
			printk(KERN_INFO "One or more configuration parameters were not configured\n");
			ret = TIAP_PARAM_EPARAMNOTSET;
			goto out;
	}
		
	tiwlan_ddaGo(&init_cfg);
    	/* create net link socket,connection to user space */
	if( tiwlan_ddaCreateNetlinkSocket() == NOK )
		return NOK;
		
    ret = tiwlan_dda_waitForWpa();
	if(ret == NOK) {
		ret = TIAP_PARAM_ELOGICAL;
		goto out;
	}

	ret = wdrv_init(&init_cfg);

	switch (ret)	{
		case DDC_INIT_SUCCESS:		
			ret = 0;
			break;
		case DDC_INIT_CFG_RANGE_ERROR:
			ret = TIAP_PARAM_ERANGE;
			goto out;
			break;
		case DDC_INIT_CFG_LOGICAL_ERROR:
			ret = TIAP_PARAM_ELOGICAL;
			goto out;
			break;
		case DDC_INIT_RESOURCE_ERROR:
			ret = TIAP_PARAM_ERESOURCE;
			goto out;
			break;
		case DDC_INIT_HW_INIT_ERROR:
			ret = TIAP_PARAM_EHWINIT;
			goto out;
			break;
		default: ret = -1;
			 goto out;

	}

	tiwlan_dda_wpaGo();
	
	wdrv_start();

	wdrv_dbGetWlanServiceState(&state);
	if (state)
	{
		TIWLAN_LED_LINK_UP;
	}
	atomic_set(&after_go,1);
out:
	up(&sem);
	printk(KERN_INFO "dda: %s in initializing %s wireless extensions: ret = %d\n",
		dev->name, (!ret ? "Succeeded" : "Failed"), ret);
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Gets AP MAC address string that was configured at init time 
		
	\return  MAC address field value from init config structure
*/
/*******************************************************************************************/
char *get_mac_addr(void)
{
	return (char *)init_cfg.APMAC;
}
		   
/*******************************************************************************************/
/*! 
	\brief Verifies if all required configuration parameters have already been configured 
		
	\return  0 in case of success, -1 if at least one parameter remained unconfigured
*/
/*******************************************************************************************/
int check_if_all_configured(void)
{
	int i;

	if(configured_params.hwaddr != 1)
		return 0;
	
	if(configured_params.essid != 1)
		return 0;

	if(configured_params.rts != 1)
		return 0;

	if(configured_params.frag != 1)
		return 0;
	
	if(configured_params.retry_long != 1)
		return 0;
	
	if(configured_params.retry_short != 1)
		return 0;

	if(configured_params.lifetime_tx != 1)
		return 0;
	
	if(configured_params.lifetime_rx != 1)
		return 0;

	if(configured_params.txpow_a != 1)
		return 0;

	if(configured_params.txpow_g != 1)
		return 0;
	if(configured_params.replev != 1)
		return 0;

	if(configured_params.beacon_period != 1)
		return 0;

	if(configured_params.dtim_period != 1)
		return 0;
	if(configured_params.bss_bridge_state != 1)
		return 0;
	if(configured_params.rate_fallback != 1)
		return 0;
	if(configured_params.radio_cal != 1)
		return 0;
	if(configured_params.cal_interval != 1)
		return 0;
	if(configured_params.energy_detect != 1)
		return 0;
	if(configured_params.hidden_ssid != 1)
		return 0;
	if(configured_params.multi_domain_cap != 1)
		return 0;
	if(configured_params.spectrum_mgmt != 1)
		return 0;
	if(configured_params.legacy_sta != 1)
		return 0;
	if(configured_params.local_pow_cons != 1)
		return 0;
	if(configured_params.max_txpow_lev != 1)
		return 0;
	if(configured_params.country_str != 1)
		return 0;
	if(configured_params.oper_state != 1)
		return 0;
	if(configured_params.wone_num != 1)
		return 0;
	if(configured_params.wone_timemod != 1)
		return 0;
	if(configured_params.auth_tbl != 1)
		return 0;
	if(configured_params.wd_en != 1)
		return 0;
	if(configured_params.wd_interf != 1)
		return 0;
	if(configured_params.wd_rxfree != 1)
		return 0;
	if(configured_params.wd_beacons != 1)
		return 0;
	if(configured_params.wd_fcserrs != 1)
		return 0;
	if(configured_params.wd_txstuck != 1)
		return 0;
	if(configured_params.wd_aci != 1)
		return 0;
	if(configured_params.wd_nb != 1)
		return 0;
	if (configured_params.aci_state_timeout != 1)
		return 0;
	if (configured_params.nb_state_timeout != 1)
		return 0;
	if (configured_params.aci_rate_thresh != 1) 
		return 0;
	if (configured_params.pd_low_thresh != 1) 
		return 0;
	if (configured_params.pd_high_thresh != 1) 
		return 0;
	if (configured_params.medium_busy_thresh != 1) 
		return 0;
	if(configured_params.access_list != 1)
		return 0;
	if(configured_params._4x_fs != 1)
		return 0;
	if(configured_params._4x_4x != 1)
		return 0;
	if(configured_params._4x_cat != 1)
		return 0;
	if(configured_params._4x_pb != 1)
		return 0;
	if(configured_params._4x_mm != 1)
		return 0;
	if(configured_params.multi_ssid != 1)
		return 0;
	if(configured_params.radio != 1)
		return 0;
	if(configured_params.ch_rang_a != 1)
		return 0;
	if(configured_params.ch_rang_g != 1)
		return 0;
	if(configured_params.radar_detect != 1)
		return 0;
	if(configured_params.bridge_mode != 1)
		return 0;

	for (i=0; i<TIAP_PARAM_WONE_DUAL_BAND; i++)
	{
		if(configured_params.oper_rate[i] != 1)
			return 0;
		if(configured_params.oper_add[i] != 1)
			return 0;
		if(configured_params.oper_ofdm[i] != 1)
			return 0;
		if(configured_params.brcst_rate[i]  != 1)
			return 0;
		if(configured_params.mcst_rate[i]  != 1)
			return 0;
		if(configured_params.tx_rate[i]  != 1)
			return 0;
		if(configured_params.ch[i] != 1)
			return 0;
		if(configured_params.band[i] != 1)
			return 0;
		if(configured_params.pbcc[i] != 1)
			return 0;
		if(configured_params.rate_adapt[i] != 1)
			return 0;
		if(configured_params.pream_mode[i] != 1)
			return 0;
		if(configured_params.slot_mode[i] != 1)
			return 0;
		if(configured_params.erp_mode[i] != 1)
			return 0;
		if(configured_params.erp_type[i] != 1)
			return 0;
		
		if(configured_params.erp_ti[i] != 1)
			return 0;
		if(configured_params.dot11_mode[i] != 1)
			return 0;
		if(configured_params.cwmin_mode[i] != 1)
			return 0;
		if(configured_params.usage_time[i] != 1)
			return 0;
	}
	if(configured_params.priv_regdomain != 1)
		return 0;
	if(configured_params.cur_regdomain != 1)
		return 0;
	if (configured_params.wdsEnable != 1) 
		return 0;
    if(configured_params.wdsEnable != 1)
        return 0;

	if(configured_params.hw_input_cca != 1)
        return 0;
	if(configured_params.hw_qualif_cca != 1)
        return 0;
	if(configured_params.hw_tx_cca != 1)
        return 0;
	if(configured_params.hw_stomp_rx != 1)
        return 0;
	if(configured_params.hw_stomp_tx != 1)
        return 0;

	if (configured_params.eeprom_less != 1)
		return 0;

	return 1;
	
}
