/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *
 *   \file  dda_wpa.h                                                
 *   \brief PURPOSE: headers and definitions for WPA interface													       
 *
 */

#ifndef _DDA_WPA_H_
#define _DDA_WPA_H_



/*************************************************************************
	includes
**************************************************************************/
#include "802_11Defs.h"
//#include "dda_wext_ioctl.h"	

/*************************************************************************
	defs
**************************************************************************/



#define MAX_STATIONS				64
#define REPORT_MSG_LENGTH 512
#define RG_IPC_PORT_WPA	  8800

#ifndef WPA_TKIP_KEY_LEN 
#define WPA_TKIP_ENCRYPTION_KEY_LEN 16 
#define WPA_TKIP_KEY_LEN            WPA_TKIP_ENCRYPTION_KEY_LEN + 2*  \
									WPA_MIC_KEY_LEN
#endif // WPA_TKIP_KEY_LEN 

#define WPA_MIC_KEY_LEN             8

#define WPA_NUM_DEFAULT_KEYS        4

#define WPA_WEP_40_KEY_LEN          5
#define WPA_WEP_104_KEY_LEN         13
#define WPA_WEP_232_KEY_LEN         29
#define WPA_MAX_WEP_KEY_LEN         WPA_WEP_232_KEY_LEN
#define WPA_PSK_KEY_LEN             32 
#define WPA_TSC_LEN                 6
#define WPA_AES_KEY_LEN             16

#ifdef __KERNEL__

/*************************************************************************
	includes (krnel only)
**************************************************************************/
#include "public_security_keys.h"
#include "memMngr.h"

/*************************************************************************
	defs
**************************************************************************/

/*!\brief Station configuration types */ 
typedef enum {
/*WPA_STA_TYPE_SEC_NONE   = 0x0,  no security */ 
    WPA_STA_TYPE_WPA        = 0x1, /* WPA station */ 
    WPA_STA_TYPE_WEP_LEGACY = 0x2, /* WEP station using fixed keys */ 
    WPA_STA_TYPE_WEP_8021X  = 0x4, /* WEP station supporting 802.1X  authentication and key exchange */ 
	WPA_STA_TYPE_WPA2       = 0x8,                                   
} wpa_sta_type_t;

/*! \brief  Used to specify how the PSK is passed as a parameter to wpa_port */
typedef enum {
    WPA_PSK_PARAM_NONE  = 0,
    WPA_PSK_PARAM_HEX   = 1,
    WPA_PSK_PARAM_ASCII = 2,
} wpa_psk_param_t;


/*! \typedef wpa_temporal_key_t
	\brief  Temporal key is the part of the PTK and GTK which is fed to the data 
		    encryption engine. It is used differently depending on the cipher suite in
	        use  */
 

typedef struct {

    
	union{
		struct {
			u8 tkip_key[WPA_TKIP_ENCRYPTION_KEY_LEN];
			u8 tx_mic_key[WPA_MIC_KEY_LEN];
			u8 rx_mic_key[WPA_MIC_KEY_LEN];
		} tkip;
		
		struct {
			u8 wep_key[WPA_MAX_WEP_KEY_LEN];
		} wep;
		
		struct {
			u8 key[WPA_AES_KEY_LEN];
		}aes;
	}data;
	int dataSize;

} wpa_temporal_key_t;





/*! \brief  Pairwise and group cipher suites */ 
typedef enum {
    WPA_CIPHER_NON     = 0x0,
    WPA_CIPHER_TKIP    = 0x1,
	WPA_CIPHER_WEP_40  = 0x2,
    WPA_CIPHER_CCMP    = 0x4,
    WPA_CIPHER_WEP_232 = 0x8,
    WPA_CIPHER_WRAP    = 0x16,
	WPA_CIPHER_WEP_104 = 0x32,
} wpa_cipher_t;


#endif /* __KERNEL__ */

#include <WPA/include/net_types.h>

/*! \brief  WPA Incoming Messages type definition */
typedef enum {
	EAPOL_FRAME					=	1,	/* EAPOL frame received in the RX */
	NOTIFY_STATION_CONNECT		=	2,	/* Station associated */
	NOTIFY_STATION_DIS_CONNECT	=	3,	/* Station Dis associated */
	FIRST_DATA_FROM_STATION		=	4,	/* First data frame received from station */
	MIC_FAILURE					=	5,	/* A MIC failure detected in the RX */
	CAN_STA_ASS					=   6,	
	RESTART_MODULE				=	7,	/* Restrat module needed  */
	SET_DEFAULT_KEY				=	8,	/* Restrat module needed  */
	SET_DEFAULT_KEY_ID          =   9,
	REMOVE_DEFAULT_KEY		    =   10,
	SET_MAP_KEY                 =   11,
	REMOVE_MAP_KEY              =   12, 
	DEAUTH_STA					=   13,
	DISASS_STA                  =   14,
	BLOCK_UNICAST				=   15,
	BLOCK_MULTYCAST             =   16,
	CAN_STA_ASS_RES             =   17,
	WPA_HELLO                   =   18,
	WPA_DRV_HELLO               =   19,
	SET_STA_ATTACK				=   20,
	EAPOL_FRAME_PRE_AUTH        =   21,
	WPA_STOP					=   22,
	WPA_RESUME					=   23,
    WPA_SET_PROFILE_PRIV_OPT    =   24,
    EAPOL_FRAME_INT_PROF        =   25,

#ifdef TIWLAN_FW_TRACE
	/* TODO : write normal generic kernel/user communication mechanism 
		 for now will piggy back on WPA communication mechanism for other 
		 modules too */
	FW_TRACE_HELLO = 26,
#endif
} wpaMsgType;

#define EAPOL_TYPE  0x888e 

#define MAX_NUM_OF_SUPPORTED_PROFILES   5

/*#pragma pack(1)*/
/*! \brief  Management messages structure */
typedef struct 
{
	wpaMsgType type;
	int profile_index;
	union 
	{

/* driver wpa module msg */
		struct 
		{
			char   pEapol[MAX_DATA_BODY_LENGTH + sizeof(dot11_header_T)];	
			int    eapolDataLen;
			int   encrypted;
		} eapolFrame;

		struct 
		{
			char		mac[MAC_ADDR_LEN];	
		} stationDisConnect;

		struct 
		{
			char		mac[MAC_ADDR_LEN];	
			int		isEapol;
		} firstDataFromStation;

		struct 
		{
			char		mac[MAC_ADDR_LEN];	
		} micFailure;
		
		struct
		{
			char		 mac[MAC_ADDR_LEN];
			UINT16       index;
			dot11_rsne_T rsne;
		}stationConnect;

		struct
		{
			UINT16 index;
			unsigned char info_elem[DOT11_RSNE_MAX + sizeof(dot11_eleHdr_T)];
			UINT16		  info_elem_len;
		}canStaAss;

		struct
		{
			UINT16 index;
			int   flag;
		}StaAttack;

 
/* wpa module driver msg */

		struct
		{ 
			UINT16                      info_len[MAX_NUM_OF_SUPPORTED_PROFILES];
			UINT8		                info[MAX_NUM_OF_SUPPORTED_PROFILES][128];
			UINT16                      rsn_len[MAX_NUM_OF_SUPPORTED_PROFILES];
			UINT8		                rsn[MAX_NUM_OF_SUPPORTED_PROFILES][128];
			wpa_cipher_t                cipher[MAX_NUM_OF_SUPPORTED_PROFILES];
            wpa_sta_type_t				secType[MAX_NUM_OF_SUPPORTED_PROFILES];
		}hello;

		struct
		{
			ether_mac_t  mac;
			wpa_cipher_t cipher;
			wpa_temporal_key_t temporal_key;
		}setMapKey;

		struct
		{
            wpa_sta_type_t     staType;
			wpa_cipher_t	   cipher;
			UINT16			   index;
			int                profile_index;
			wpa_temporal_key_t temporal_key;
		} setDefaultKey;
		
		struct 
		{
			UINT16 index;
		}setDefaultKeyId;

		struct 
		{
			UINT16 index;
			wpa_cipher_t cipher;
			int profile_index;
		}removeDefaultKey;


		struct
		{
			ether_mac_t  mac; 
			wpa_cipher_t cipher;
		}removeMapKey;
		
		struct
		{
			ether_mac_t  mac; 
		}deAssAuthSta;

		struct
		{	
			int block;
		}blockMcast;

		struct
		{
			int block;
			ether_mac_t  mac; 
		}blockUcast;

		struct
		{
			UINT16 index;
			int flag;
		}canStaAssRes;

        struct 
        {
           UINT32 profile_index;
           UINT32 priv_opt;
        }profilePrivOpt;


	} content;
} __attribute__ ((packed)) wpaMessage_T;

typedef struct{
    UINT8  WepKey[WPA_MAX_WEP_KEY_LEN];
    UINT16 keySize;
} defaultWepKey_T;


#endif /* _DDA_WPA*/
