/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  wlan_pal.c  

 \brief PURPOSE: implementation of platform abstraction layer API											       
 *                                                                                                                   
 *                                                                         
 */

#include "wlan_pal.h"

/*******************************************************************************************/
/*!
	\brief Suspends all the timers in tcb list
	
	\return  OK
*/
/*******************************************************************************************/
STATUS PALSRV_tcbSuspend(void) {
	struct pal_tcb_list * pal_tcb_list_elem = NULL; 
	struct list_head * current = NULL; 

	list_for_each(current, &pal_tcb_list_head) {
		pal_tcb_list_elem = list_entry(current, struct pal_tcb_list, list); 
		if (timer_pending(pal_tcb_list_elem->timer)) {
			pal_tcb_list_elem->pending = 1;
			pal_tcb_list_elem->expires = pal_tcb_list_elem->timer->expires - jiffies;
			printk("%s: deleting timer %#x\n", __func__, (int)pal_tcb_list_elem->timer->function);
			del_timer(pal_tcb_list_elem->timer);
		} else
			pal_tcb_list_elem->pending = 0;	
	}

	return OK;
}

/*******************************************************************************************/
/*!
	\brief Restores all the timers in tcb list
	
	\return  OK
*/
/*******************************************************************************************/
STATUS PALSRV_tcbResume(void) {
	struct pal_tcb_list * pal_tcb_list_elem = NULL; 
	struct list_head * current = NULL; 
	
	list_for_each(current, &pal_tcb_list_head) {
		pal_tcb_list_elem = list_entry(current, struct pal_tcb_list, list); 
		if (pal_tcb_list_elem->pending) {
			pal_tcb_list_elem->timer->expires = jiffies + pal_tcb_list_elem->expires + 1;
			add_timer(pal_tcb_list_elem->timer);
		}
	}

	return OK;
}

/*******************************************************************************************/
/*!
	\brief Adds a new timer in to the tail of tcb list
	
	\param timer input timer struct
	
	\return  WLAN_PAL_SOK
*/
/*******************************************************************************************/
STATUS PALSRV_tcbNew(PALSRV_TcbObjTYPE * timer) {
	struct pal_tcb_list * pal_tcb_list_elem = kmalloc(sizeof(struct pal_tcb_list), GFP_KERNEL); 	

	init_timer(timer); 
	pal_tcb_list_elem->timer = timer; 
	list_add_tail(&pal_tcb_list_elem->list, &pal_tcb_list_head); 

	return WLAN_PAL_SOK; 
}

/*******************************************************************************************/
/*!
	\brief Deletes a given timer from tcb list
	
	\param timer input timer struct
	
	\return  WLAN_PAL_SOK
*/
/*******************************************************************************************/
STATUS PALSRV_tcbDelete(PALSRV_TcbObjTYPE * timer) {
	struct pal_tcb_list * pal_tcb_list_elem = NULL; 
	struct list_head * current = NULL; 

	list_for_each(current, &pal_tcb_list_head) {
		pal_tcb_list_elem = list_entry(current, struct pal_tcb_list, list); 
		if (pal_tcb_list_elem->timer == timer) {
			list_del(&pal_tcb_list_elem->list); 
			kfree(pal_tcb_list_elem); 
		} 
	} 

	del_timer(timer); 

	return WLAN_PAL_SOK;
}

/*******************************************************************************************/
/*!
	\brief Cancels a given timer 
	
	\param timer input timer struct
	
	\return  WLAN_PAL_SOK
*/
/*******************************************************************************************/
STATUS PALSRV_tcbCancel(PALSRV_TcbObjTYPE * timer) {
	struct pal_tcb_list * pal_tcb_list_elem = NULL; 
	struct list_head * current = NULL; 

	list_for_each(current, &pal_tcb_list_head) {
		pal_tcb_list_elem = list_entry(current, struct pal_tcb_list, list); 
		if (pal_tcb_list_elem->timer == timer) {
			list_del(&pal_tcb_list_elem->list); 
			kfree(pal_tcb_list_elem); 
		} 
	} 

	del_timer(timer); 

	return WLAN_PAL_SOK;
}

#ifdef INTLOCK_DEBUG
unsigned int wlan_inIntLock = 0;
char *wlan_intLockFunc[MAX_INTLOCK_TRACE];
unsigned int wlan_intLockLine[MAX_INTLOCK_TRACE];
#endif /* ifdef INTLOCK_DEBUG */

