/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda.h                                               
    \brief Device Driver Adaptation Layer for VxWorks Operating System
*/

/******************************************************************************/

#ifndef __TIWLAN_DDA_H
#define __TIWLAN_DDA_H


#include "wlan_pal.h"
#include "memMngr.h"
#include "drvDbDefs.h"
#include "dda_trace.h"
#include "mgmtSm.h"
#include "mainsm.h"
#include "rx.h"
#include "tx.h"
#include "whalAcxMbox.h"
#include "whalAcxRegs.h"
#include "dda_contexts.h"
#include "release.h"
#include "pal.h"
#include "whalAcxRegs.h"

#ifdef _DDA_MAIN_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

#define DDA_DEV_NAME		"wlan"
#define DDA_DEVNAME			"wlan%d"
#define DDA_DEV_UNIT_NUM	0
#define DDA_DEV_IP_ADR		"127.1.1.222"		   /* WLAN END dummy IP address */
#define EIOCTI   			_IOWR('i', 324, int)   /* define code for TI EIOC   */

#define DDA_TASK_PRIORITY   45


#define TIAP_LED_OFF		0
#define TIAP_LED_ON		    1
#define TIAP_LED_BLINK_OFF	2
#define TIAP_LED_BLINK_ON	3
#define TIAP_LED_FLASH		4

#ifdef DDA_WLAN_LED
	#if defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7VW)
		#define PAL_ConfigVWiLeds()

		extern int tiap_led_oper_state, tiap_led_blinked;
		extern UINT16 tiap_activity_led_mask;
		extern __u8 tiap_gpio_owner;
		UINT8 wdrv_dbGetApBridgeMode(void);
		__inline__ static void tiwlan_ddaLedAction(int action)
		{
			/* It mustn't be a function call in an inline function like this one.
			 * Performance is crucial.
			 */
			if (wdrv_dbGetApBridgeMode() || !tiap_gpio_owner)
				return;
		
			switch(action)
			{
			case TIAP_LED_OFF:            
			#if 0
						whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
			#else
						PAL_sysGpioOutBit( 6, 1);
			#endif
						tiap_led_oper_state = action;
					break;
			case TIAP_LED_ON:            
			#if 0
					whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
			#else
					PAL_sysGpioOutBit( 6, 0);
			#endif
					tiap_led_oper_state = action;
					break;
			case TIAP_LED_BLINK_OFF:
			case TIAP_LED_BLINK_ON:
					tiap_led_blinked = 1;
					break;
			}
		}
	#else 
		#define tiwlan_ddaLedAction(action)
		#define PAL_ConfigVWiLeds()
	#endif/*defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7VW)*/ 
	#define TIWLAN_LED_LINK_UP 	  tiwlan_ddaLedAction(TIAP_LED_ON)
	#define TIWLAN_LED_LINK_DOWN  tiwlan_ddaLedAction(TIAP_LED_OFF)
	#define TIWLAN_LED_PKT_XMIT   tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF)
	#define TIWLAN_LED_PKT_RCV    tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF) 
#else
	#define TIWLAN_LED_LINK_UP
	#define TIWLAN_LED_LINK_DOWN
	#define TIWLAN_LED_PKT_RCV
	#define TIWLAN_LED_PKT_XMIT
	#define PAL_ConfigVWiLeds()
#endif/*DDA_WLAN_LED*/


#define GET_MAJOR_SW_VER SW_VERSION_MAJOR
#define GET_MINOR_SW_VER SW_VERSION_MINOR
  
#define ETH_ALEN 	6

typedef enum {
    TIWLAN_ALLOC_INIT_PHASE,
    TIWLAN_ALLOC_RUNTIME_PHASE
} tiwlan_allocPhase_t;


typedef enum {
    TIWLAN_SOFT_RESET,
    TIWLAN_HARD_RESET,
}TIWLAN_RESET_TYPE_E;


#define tiwlan_ddaRxFreeBuffer(x) 			/* nothing to be done in VxWorks */
								  
#define TIWLAN_DDA_BUFFER_FROM_TOKEN(token) token

#define boardInitAcx111Addr(RegAddr, MemAddr) \
				tiwlan_ddaInitPSPVlynq(RegAddr, MemAddr)

#define boardUnloadAcx111() tiwlan_ddaUnloadPSPVlynq()

#define XStr(x)  #x
#define XXstr(x) XStr(x)

PUBLIC UINT8 	tiap_gpio_owner; 			/* 1 for driver, 0 for fw */
PUBLIC STATUS 	tiwlan_ddaInit(void);
PUBLIC STATUS   tiwlan_ddaReInit(void);
PUBLIC STATUS 	tiwlan_ddaSuspend(void);
PUBLIC STATUS 	tiwlan_ddaResume(void);
PUBLIC STATUS 	tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr);
PUBLIC STATUS	tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr); 
PUBLIC STATUS 	tiwlan_ddaUnloadPSPVlynq(void);
PUBLIC UINT8 	tiwlan_ddaRand(void);
PUBLIC void 	tiwlan_ddaSrand(UINT16 seed);
PUBLIC STATUS 	tiwlan_ddaSend(mem_MSDU_T * pMSDU);
PUBLIC STATUS 	tiwlan_ddaReceive(mem_MSDU_T * pMSDU);
PUBLIC void 	tiwlan_ddaDeleteMacAddrFunc(char *mac);
PUBLIC STATUS 	tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, 
												UINT8 ** buf, tiwlan_allocPhase_t phase);
PUBLIC STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size );
PUBLIC STATUS 	tiwlan_dda_waitForWpa(void);
PUBLIC STATUS 	tiwlan_ddaStart(void);
PUBLIC STATUS 	tiwlan_ddaCleanup(void);
PUBLIC STATUS   tiwlan_dda_wpaGo(void);
PUBLIC STATUS 	tiwlan_ddaIsMemMngrInitNeeded(void);
PUBLIC void     tiwlan_setAcxResetType(TIWLAN_RESET_TYPE_E type);

/****************************************************************************/
/********************PSP Instrumentation support ****************************/
/****************************************************************************/
/* Instrumentation is not implemented for vxWorks - all macros are empty */
/* AP execution time */
/* Event Id AP_DRV_ISR_ENTER is traced without parameters */
#define TIAP_TRACE_DRV_ISR_ENTER()

/* Event Id AP_DRV_ISR_EXIT is traced with interrupt reason parameters */
#define TIAP_TRACE_DRV_ISR_EXIT(reason)

/* Event Id AP_DRV_TASKLET_ENTER is traced with tasklet id parameter */

#define TIAP_TRACE_DRV_TASKLET_ENTER(tsklId)

/* Event Id AP_DRV_TASKLET_EXIT is traced with tasklet id parameter */

#define TIAP_TRACE_DRV_TASKLET_EXIT(tsklId)

/* Tasklet Id definition */

typedef enum
{
    TIAP_TRACE_TASKLET_MAIN,
    TIAP_TRACE_TASKLET_MBOX
} TIAP_TRACE_TASKLET_ID_E;

/* Timers */

/* Event Id AP_DRV_TMR_ENTER is traced with timer id parameter */

#define TIAP_TRACE_DRV_TMR_ENTER(tmr)

/* Event Id AP_DRV_TMR_EXIT is traced with timer id parameter */

#define TIAP_TRACE_DRV_TMR_EXIT(tmr)

/* Timer ID definition */

typedef enum
{
    TIAP_TRACE_TMR_LED,
    TIAP_TRACE_TMR_RESTART,
    TIAP_TRACE_TMR_AGING,
    TIAP_TRACE_TMR_DYNAMIC_ALGORITHM,
    TIAP_TRACE_TMR_STA_AUTH,
    TIAP_TRACE_TMR_STA_DEAUTH,
    TIAP_TRACE_TMR_ERP_BEACON_AGING,
    TIAP_TRACE_TMR_PBCC_BEACON_AGING,
    TIAP_TRACE_TMR_RADIO_CALIB,
    TIAP_TRACE_TMR_CRT_RADIO_CALIB,
    TIAP_TRACE_TMR_RX,
    TIAP_TRACE_TMR_PS_AGING,
    TIAP_TRACE_TMR_TX_WAIT_LIST,
    TIAP_TRACE_BSS_BRIDGE_ISOLATION

} TIAP_TRACE_TMR_ID_E;

/* AP TX data path */

/*  Event id AP_DRV_PKT_TX_ENTER is traced with two parameters, 
    when first parameter is skb pointer and second identifies an entry point 
    or internal packet ( such as management frames ) */
#define TIAP_TRACE_PKT_TX_ENTER(ptr,entrypoint)

/* Entry points definition */
typedef enum {
    TIAP_TRACE_XMIT = 0,
    TIAP_TRACE_XMIT_WDS,
    TIAP_TRACE_XMIT_WDS_PKT,
    TIAP_TRACE_WLAN_RX_BRDCST,
    TIAP_TRACE_WLAN_RX_UNICAST,
    TIAP_TRACE_EAPOL,
    TIAP_TRACE_PROBE_RSP,
    TIAP_TRACE_AUTH,
    TIAP_TRACE_ASS_RES

} TIAP_TRACE_ENTRY_POINT_E;

/*  Event id AP_DRV_PKT_TX_EXIT is traced with two parameters, 
    when first parameter is skb pointer and second is Tx result */

#define TIAP_TRACE_PKT_TX_EXIT(ptr,result)

/* Tx result definition */

typedef enum {
    TIAP_TRACE_TX_RESULT_OK,
    TIAP_TRACE_TX_RESULT_NOK,
    TIAP_TRACE_TX_RESULT_NO_BD_FOR_HEADER,
    TIAP_TRACE_TX_RESULT_ALLOC_MSDU_FAIL,
    TIAP_TRACE_TX_RESULT_ALLOC_BD_FAIL

} TIAP_TRACE_TX_RESULT_E;

/* Event Id AP_DRV_PKT_TX_COMPLETE_ENTER is traced with one parameter (skb pointer)*/
#define TIAP_TRACE_PKT_TX_COMPLETE_ENTER(par)

/* Event Id AP_DRV_PKT_TX_COMPLETE_EXIT is traced with one parameter (skb pointer)*/
#define TIAP_TRACE_PKT_TX_COMPLETE_EXIT(par)

/* Event Id AP_DRV_TX_ENTER is traced without parameters */
#define TIAP_TRACE_DRV_TX_ENTER()

/* Event Id AP_DRV_TX_EXIT is traced with one parameter */
#define TIAP_TRACE_DRV_TX_EXIT(par)

/* Driver Tx Exit status definition */

typedef enum
{
    TIAP_TRACE_DRV_TX_EXIT_OK,
    TIAP_TRACE_DRV_TX_EXIT_CONTINUE

} TIAP_TRACE_DRV_TX_EXIT_STATUS;

/* AP Rx data path */

/* Event Id AP_DRV_RX_ENTER is traced without parameters */

#define TIAP_TRACE_DRV_RX_ENTER()

/* Event Id AP_DRV_TX_EXIT is traced with one parameter */
#define TIAP_TRACE_DRV_RX_EXIT(par)

/* Driver Tx Exit status definition */

typedef enum
{
    TIAP_TRACE_DRV_RX_EXIT_OK,
    TIAP_TRACE_DRV_RX_EXIT_CONTINUE

} TIAP_TRACE_DRV_RX_EXIT_STATUS;

/* Event Id AP_DRV_PKT_RX_ENTER is traced with one parameter ( skb pointer ) */

#define TIAP_TRACE_DRV_PKT_RX_ENTER(ptr)

/*  Event Id AP_DRV_PKT_RX_EXIT is traced with two parameters ( skb pointer and
    Rx result ) */

#define TIAP_TRACE_DRV_PKT_RX_EXIT(ptr,result)

/* Rx result definition */

typedef enum {
    TIAP_TRACE_RX_RESULT_OK,
    TIAP_TRACE_RX_RESULT_NOK,
    TIAP_TRACE_RX_RESULT_DMA_ERROR,
    TIAP_TRACE_RX_RESULT_MAX_DECONACT_LEN,
    TIAP_TRACE_RX_RESULT_ALLOC_BD_FAIL,
    TIAP_TRACE_RX_RESULT_ALLOC_MSDU_FAIL,
    TIAP_TRACE_RX_RESULT_FRAME_FILTERED,
    TIAP_TRACE_RX_RESULT_NULL_DATA_FRAME

} TIAP_TRACE_RX_RESULT_E;

/* QOS Support */
/* Function prototypes */

/* Get pointer to network device for WDS */
#define tiwlan_dda_get_wds_device(TA)  wdrv_wdsInterfaceCookieGet(TA)

#if defined (CONFIG_MIPS_AVALANCHE_PROTOCOL_INSPECT)
#define PSP_PIL_QOS
typedef enum
{
    TIWLAN_PIL_UN_TRUSTED=0,
    TIWLAN_PIL_TRUSTED
} TIWLAN_PIL_MODE;

int tiwlan_ddaL2_pil_classify(mem_MSDU_T * pMSDU, void *netdevice);
int tiwlan_ddaL3_pil_classify(mem_MSDU_T * pMSDU, void *netdevice, void *ptr_iphdr);
int tiwlan_dda_pil_get_mode(void *net_device);
/* This function returns pointer to wlan_dev */
void *tiwlan_dda_get_wlan_device(void);

#else
#define tiwlan_ddaL2_pil_classify(msdu,dev)             (1)
#define tiwlan_ddaL3_pil_classify(msdu,dev,iphdr)       (1)
#define tiwlan_dda_eapol_pil_classify(msdu,dev)         (1)
#define tiwlan_dda_pil_get_mode(dev)                    (TIWLAN_PIL_UN_TRUSTED)
#define tiwlan_dda_get_wlan_device()                    (0)
#endif


#undef  PUBLIC

#endif /* __TIWLAN_DDA_H */

