/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_ioctl.c                                               
    \brief This module implements WLAN driver IOCTL functions for configuration
	 and monitoring                                                                          
*/
/******************************************************************************/


#define _DDA_IOCTL_C

/***********************************************************
	includes
*******************************************************/

#include "dda.h"
#include "dda_trace.h"
#include "dda_ioctl.h"
#include "drvDbDefs.h"
#include "drvDbRegDomain.h"
#include "drvDbHash.h"
#include "mgmtAccess.h"
#include "mgmtSsidHandler.h"
#include "mainsm.h"
#include "mgmtSm.h"
#include "Shutdown.h"  
#include "whal.h"
#include "whalApi.h"
#include "whalAcxProcess.h"
#include "bssbridge.h"
#include "rx.h"
#include "drvDbHash.h"
#include "WlanEnd.h"
#include "dda_wdsDrv.h"
#include "wdsDrv.h"
#include "dda_wpaauth_ioctl.h"

/***********************************************************
	defs
***********************************************************/

typedef struct _configured_params
{

	UINT8				hwaddr;
	
	/*basic data types*/

	/*standard*/
	UINT8               essid;
	UINT8             	rts;
	UINT8             	frag;
	UINT8             	retry_long, retry_short;
	UINT8				lifetime_tx, lifetime_rx;
	UINT8				txpow_a, txpow_g;
	
	/*private*/
	UINT8               replev;
	UINT8             	beacon_period;
	UINT8               dtim_period;
	UINT8			    bss_bridge_state;
	UINT8               rate_fallback;
	UINT8             	radio_cal;
	UINT8             	cal_interval;
	UINT8               energy_detect;
 	UINT8				pd_low_thresh;
	UINT8				pd_high_thresh;
	UINT8				aci_state_timeout;
	UINT8				nb_state_timeout;
	UINT8				aci_rate_thresh;
	UINT8				aci_event_thresh;
	UINT8				medium_busy_thresh;
	UINT8             	hidden_ssid;
	UINT8               multi_domain_cap;
	UINT8               spectrum_mgmt;
	UINT8               legacy_sta;
	UINT8               local_pow_cons;
	UINT8               max_txpow_lev;
	UINT8 				country_str;
	UINT8              	oper_state;
	UINT8               wone_num;
	UINT8               wone_timemod;
	UINT32				txop_limit;
	UINT8		        radar_detect;
	UINT8				bridge_mode;
	UINT8				eeprom_less;
	UINT8       eeprom_image;

	/*complex data types*/
	UINT8				auth_tbl;
	UINT8				wd_en, wd_interf, wd_rxfree, wd_beacons, wd_txstuck, wd_fcserrs, wd_aci, wd_nb;
	UINT8 				hw_input_cca, hw_tx_cca, hw_qualif_cca, hw_stomp_rx, hw_stomp_tx;
	UINT8				access_list;
	UINT8				_4x_fs, _4x_4x, _4x_cat, _4x_pb, _4x_mm;
	UINT8				multi_ssid;
	UINT8				radio;
	UINT8				ch_rang_a, ch_rang_g;
	UINT8				vhcf;
	UINT8				oper_rate[TIAP_PARAM_WONE_DUAL_BAND], oper_add[TIAP_PARAM_WONE_DUAL_BAND],
    					oper_ofdm[TIAP_PARAM_WONE_DUAL_BAND], brcst_rate[TIAP_PARAM_WONE_DUAL_BAND], ch[TIAP_PARAM_WONE_DUAL_BAND],
    					mcst_rate[TIAP_PARAM_WONE_DUAL_BAND],tx_rate[TIAP_PARAM_WONE_DUAL_BAND],
						band[TIAP_PARAM_WONE_DUAL_BAND], pbcc[TIAP_PARAM_WONE_DUAL_BAND],
    					rate_adapt[TIAP_PARAM_WONE_DUAL_BAND], pream_mode[TIAP_PARAM_WONE_DUAL_BAND],
    					slot_mode[TIAP_PARAM_WONE_DUAL_BAND], erp_mode[TIAP_PARAM_WONE_DUAL_BAND],
    					erp_type[TIAP_PARAM_WONE_DUAL_BAND], erp_ti[TIAP_PARAM_WONE_DUAL_BAND],
    					dot11_mode[TIAP_PARAM_WONE_DUAL_BAND], cwmin_mode[TIAP_PARAM_WONE_DUAL_BAND],
    					usage_time[TIAP_PARAM_WONE_DUAL_BAND];
	
	UINT8				priv_regdomain;
	UINT8				cur_regdomain;
	UINT8               fw;
   UINT8				wdsEnable;

} CONFIGURED_PARAMS_S;

/***********************************************************
	macros
***********************************************************/

#define DDA_IOCTL_RUNTIME_ERR(ret)      \
do { 						            \
	ret = TIAP_PARAM_ERUNTIME;	        \
    DDA_LOG_ARG(DDA_LOG_ERR, "Func %s: run-time parameter is not allowed\n", \
                __FUNCTION__, 0,0,0,0,0);                                          \
} while(0)

#define DDA_IOCTL_RUNTIME_PAR_ERR(ret,val)    \
do { 						    \
	ret = TIAP_PARAM_ERUNTIME;	\
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: run-time parameter %d is not allowed\n", \
                __FUNCTION__, __LINE__, val, 0,0,0);                                          \
} while(0)


#define DDA_IOCTL_RANGE_ERR(ret,val)        \
do { 						                \
	ret = TIAP_PARAM_ERANGE;	            \
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Invalid value range %d\n", \
                __FUNCTION__, __LINE__, val, 0,0,0);                       \
} while(0)

#define DDA_IOCTL_NULL_PTR_ERR(ret)             \
do { 						                    \
	ret = EINVAL;	                            \
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Pointer is NULL\n", \
                __FUNCTION__, __LINE__,0,0,0,0);              \
} while(0)
        
#define DDA_IOCTL_INV_PAR_ERR(ret,val)          \
do { 						                    \
	ret = TIAP_PARAM_ERANGE;	                              \
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Ivalid param %d\n", \
                __FUNCTION__, __LINE__, val, 0,0,0);          \
} while(0)

#define DDA_IOCTL_INTI_ERR(ret,val)        \
do { 						               \
	ret = TIAP_PARAM_ERANGE;	           \
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Parametr is not allowed %d\n",  \
                __FUNCTION__, __LINE__, val, 0,0,0);                      \
} while(0)

#define dda_wext_convert_rates(type, tr, tv)	\
do { 						                    \
for(i=0,k=0; i<wone->elem.type.len; i++)	    \
	if(wone->elem.type.rates[i] == TIAP_PARAM_RATE_SUPPORTED)		        \
		init_cfg->wone_info[index].tr.rates[k++] = tv[i];		            \
	else if(wone->elem.type.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)	\
		init_cfg->wone_info[index].tr.rates[k++] = tv[i] | 0x80; 	        \
} while(0)


/***********************************************************
	static variables
***********************************************************/

static initConfigStruct         init_cfg;
static CONFIGURED_PARAMS_S      configured_params;

/* a boolean variable that is true when it's a runtime cofiguration*/
UINT32  after_go;

/* a semaphore for  synchronizing access to the config struct */
static SEM_ID   sem;  

UINT16 tiap_activity_led_mask;
UINT8  tiap_gpio_owner;     /*1 for driver, 0 for fw*/

#if REPORT_DEBUG
   int logged_chars; /* current number of characters logged in message buffer */
#endif

/***********************************************************
	Local Function Prototypes
***********************************************************/
static int param_eeprom(initConfigStruct *init_cfg, tiap_param_eeprom_t * eeprom);
static int doGenericIoctl(const DDA_IOCTL_GENERIC_PAR_S * pIoctl, int cmd );
static int doBasicIoctl( DDA_IOCTL_BASIC_PAR_S  * pIoctl, int cmd );
static int doComplexIoctl( DDA_IOCTL_COMPLEX_PAR_S  * pIoctl, int cmd );
static int doGoIoctl( DDA_IOCTL_GO_PAR_S  * pIoctl, int cmd );

static int get_param_name( tiap_param_drvname_t * pName );
static int param_tx_power(initConfigStruct *init_cfg, 
                          tiap_param_txpower_t * txpower);
static int param_retry(initConfigStruct *init_cfg, tiap_param_retry_t * retry);
static int get_param_retry( tiap_param_retry_t * retry );

static int get_param_tx_power( tiap_param_txpower_t * txpower );
static int param_auth_tbl(initConfigStruct *init_cfg, 
						  tiap_param_auth_algo_t *auth_tbl);
static int param_wd(initConfigStruct *init_cfg, tiap_param_wd_t *wd);
static int param_interference_wd_config(initConfigStruct *init_cfg, 
										tiap_param_interference_wd_t *interf_param);
static int param_hw_interf_cfg(initConfigStruct *init_cfg, tiap_param_hw_interf_t *interf_param);
static int param_access_list(initConfigStruct *init_cfg, 
							 tiap_param_access_list_t *access_list);
static int param_4x(initConfigStruct *init_cfg, tiap_param_4x_t *_4x);
static int param_multi_ssid(initConfigStruct *init_cfg, 
							tiap_param_multi_ssid_t *multi_ssid);
static int param_radio(initConfigStruct *init_cfg, tiap_param_radio_t *radio);
static int param_ch_rang(initConfigStruct *init_cfg, 
						 tiap_param_channel_rang *ch_rang);
static int param_wone(initConfigStruct *init_cfg, tiap_param_wone_t *wone);
static int param_priv_regdomain(initConfigStruct *init_cfg, 
								tiap_param_priv_reg_domain_t *priv_regdomain);
static int param_curr_reg_domain(initConfigStruct *init_cfg, 
								 tiap_param_cur_reg_domain_t *val);
static int param_set_country_str(initConfigStruct *init_cfg, tiap_param_country_t *country);
static int param_set_qos( initConfigStruct *init_cfg, tiap_param_qos_t * pQoS );
static int param_set_bit_args(initConfigStruct *init_cfg, tiap_param_bit_args_t *val);

static int get_param_country_str(tiap_param_country_t *country);
static int get_param_qos(tiap_param_qos_t * pQoS);
static int get_param_auth_tbl(tiap_param_auth_algo_t *auth_tbl);
static int get_param_wd(tiap_param_wd_t *wd);
static int get_param_interference_wd(tiap_param_interference_wd_t *interf_param);
static int get_param_hw_interf(tiap_param_hw_interf_t *interf_param);
static int get_param_access_list(tiap_param_access_list_t *access_list);
static int get_param_4x(tiap_param_4x_t *_4x);
static int get_param_multi_ssid(tiap_param_multi_ssid_t *multi_ssid);
static int get_param_radio(tiap_param_radio_t *radio);
static int get_param_ch_rang(tiap_param_channel_rang *ch_rang);
static int get_param_wone(tiap_param_wone_t *wone);
static int get_param_sta_tbl(tiap_param_statbl_t *sta_tbl);
static int get_param_priv_regdomain(tiap_param_priv_reg_domain_t *priv_regdom);
static int get_param_curr_reg_domain(tiap_param_cur_reg_domain_t *val);
static int get_param_stats(tiap_param_net_stats *stats);
static int get_param_die_id( tiap_param_die_id_t * pDieId );

/* functions called by private ioctls DDA_IOCTL_BASIC_PARAM and DDA_IOCTL_GET_BASIC_PARAM */

static int tiap_param_set_replev(int);
static int tiap_param_get_replev(int *);
static int tiap_param_set_beaconperiod(initConfigStruct *, UINT16);
static int tiap_param_get_beaconperiod(int *);
static int tiap_param_set_dtim_period(initConfigStruct *, UINT8);
static int tiap_param_get_dtim_period(int *);
static int tiap_param_set_bss_bridge_state(initConfigStruct *, tiap_param_bss_br_state_t);
static int tiap_param_get_bss_bridge_state(int *);
static int tiap_param_set_rate_fallback(initConfigStruct *, UINT8);
static int tiap_param_get_rate_fallback(int *);
static int tiap_param_get_radio_cal(int *);
static int tiap_param_set_radio_cal(initConfigStruct *, UINT32);
static int tiap_param_get_radio_cal_int(int *);
static int tiap_param_set_radio_cal_int(initConfigStruct *, UINT16);
static int tiap_param_get_energy_detect(int *);
static int tiap_param_set_energy_detect(initConfigStruct *, UINT8);
static int tiap_param_get_eeprom_less(int *);
static int tiap_param_set_eeprom_less(initConfigStruct *, UINT8);
static int tiap_param_get_hidden_ssid(int *);
static int tiap_param_set_hidden_ssid(initConfigStruct *, UINT16);
static int tiap_param_get_multidomain_cap(int *);
static int tiap_param_set_multidomain_cap(initConfigStruct *, UINT8);
static int tiap_param_get_spectrum_mgmt(int *);
static int tiap_param_set_spectrum_mgmt(initConfigStruct *, UINT8);
static int tiap_param_get_legacy_sta(int *);
static int tiap_param_set_legacy_sta(initConfigStruct *, UINT8);
static int tiap_param_get_local_power_cons(int *);
static int tiap_param_set_local_power_cons(initConfigStruct *, __s8);
static int tiap_param_get_max_tx_pow_level(int *);
static int tiap_param_set_max_tx_pow_level(initConfigStruct *, UINT8);
static int tiap_param_get_oper_state(int *);
static int tiap_param_set_oper_state(initConfigStruct *, UINT8);
static int tiap_param_get_wone_num(int *);
static int tiap_param_set_wone_num(initConfigStruct *, UINT8);
static int tiap_param_get_wone_timemod(int *);
static int tiap_param_set_wone_timemod(initConfigStruct *, UINT8);
static int tiap_param_set_radar_detect_mode(initConfigStruct *, UINT8);
static int tiap_param_get_radar_detect_mode(int *);
static int tiap_param_set_bridge_mode(initConfigStruct *, UINT8);
static int tiap_param_get_bridge_mode(int *);
static int tiap_param_set_rts(initConfigStruct * init_cfg, UINT16 val );
static int tiap_param_get_rts( UINT32 * pParam );
static int tiap_param_set_frag(initConfigStruct * init_cfg, UINT16 val );
static int tiap_param_get_frag (UINT32 * pParam);
static int check_if_all_configured(void);
static int  read_wlan_params(UINT8 currentConfigWoneInex, char *page);
static int  putFwImage(unsigned char * buf, int len) ;
static int tiap_param_set_wds_enable(initConfigStruct *init_cfg, __u8 val);
static int tiap_param_get_wds_enable(int *val);


/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Processes IOCTL request
	\param 	pData - pointer to IOCTL data
	\return status of operation OK or code of error
*/
/******************************************************************************/


STATUS DDA_doIoctl(const DDA_IOCTL_DATA_S * pData )
{
 STATUS status;

 if ( semTake(sem, WAIT_FOREVER) != OK) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, 
                 "Func %s: Failed to take semaphore %x\n",__FUNCTION__, 
                 sem,0,0,0,0 );
     return ERROR;
 }

 switch (pData->dataType)
 {
    case DDA_IOCTL_TYPE_BASIC_A:
        status = doBasicIoctl((DDA_IOCTL_BASIC_PAR_S*)&pData->dataBuf, pData->cmd);
        break;
    
    case DDA_IOCTL_TYPE_GENERIC_A:
        status = doGenericIoctl((DDA_IOCTL_GENERIC_PAR_S *)&pData->dataBuf, pData->cmd );
        break;
    
    case DDA_IOCTL_TYPE_COMPLEX_A:
        status = doComplexIoctl((DDA_IOCTL_COMPLEX_PAR_S*) &pData->dataBuf, pData->cmd );
        break;

    case DDA_IOCTL_TYPE_GO_A:
        status = doGoIoctl((DDA_IOCTL_GO_PAR_S*) &pData->dataBuf, pData->cmd );
        break;

    default:
        DDA_LOG_ARG(DDA_LOG_ERR, 
                    "Func %s: Invalid IOCTL data type %x\n",__FUNCTION__, 
                    pData->dataType, 0,0,0,0 );
        status = ERROR;
 }
 
 if (semGive( sem ) != OK) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, 
                 "Func %s: Failed to give semaphore %x\n",__FUNCTION__, 
                 sem,0,0,0,0 );
     return ERROR;
 }
 
 return (status);

}

/******************************************************************************/

/*! \brief 	Initilaize local variables
	\return status of operation OK	or ERROR in case of semaphore creation failure
*/
/******************************************************************************/

STATUS DDA_InitIoctl( void )
{
 after_go = 0;
 tiap_gpio_owner = 1;

#if REPORT_DEBUG
 logged_chars = 0;
#endif

 memset(&configured_params,0,sizeof(configured_params));
 memset(&init_cfg,0,sizeof(init_cfg));

 sem = semBCreate( SEM_Q_FIFO, SEM_FULL );
 if (sem == NULL)
 {
     DDA_LOG_ARG(DDA_LOG_ERR, 
                 "Func %s: Failed to create garding semaphore\n",__FUNCTION__, 
                 0,0,0,0,0 );
     return ERROR;
 }
 return OK;
}

/******************************************************************************/

/*! \brief 	Deletes local variables
	\return status of operation OK
*/
/******************************************************************************/

STATUS DDA_DeleteIoctl( void )
{
 if (sem)
 {
	 semDelete(sem);
 }
 return OK;
}


/******************************************************************************/

/*! \brief 	Get status of driver initialization
	\return status : TRUE - initialized OK, FALSE - not initialized
*/
/******************************************************************************/

STATUS DDA_GetInitStatus( void )
{
 return after_go;
}


/*******************************************************************************
		LOCAL FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Performs generic IOCTL request
	\param 	pIoctl - pointer to IOCTL data
	\param 	cmd    - IOCTL command value
	\return status of operation OK, ERROR or TIAP_PARAM_ERxxx error code
*/
/******************************************************************************/

static int doGenericIoctl(const DDA_IOCTL_GENERIC_PAR_S * pIoctl, int cmd )
{
 int status = OK;
 UINT16 idx;
 
 if (!pIoctl->buf || !pIoctl->buf_len) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, "Func %s: Invalid IOCTL buf and buf_len\n",
                 __FUNCTION__,0,0,0,0,0);
     return EINVAL;
 }

 switch (pIoctl->cmd) 
 {
     case TIAP_GENERIC_IOCTL_GET_TX_INFO:
         whal_apiPrintTxInfo( pIoctl->param1 /* Qid */, pIoctl->param2 /* QprintEn */, 
                              pIoctl->buf);
         break;
    
     case TIAP_GENERIC_IOCTL_TST_FINDSTA: 
         if (wdrv_dbFindStaIndex( (char*) pIoctl->mac, &idx) == OK)
         {
             sprintf( pIoctl->buf, "station found for mac %s, index %d", 
                      (char*) &pIoctl->mac[0], idx);	
         }
         else 
         {
             status = ERROR;
         }
         break;
    
     case TIAP_GENERIC_IOCTL_TST_FIND_INS:
         if (wdrv_dbFindAndInsertStaIndex((char*) pIoctl->mac, 
                                          pIoctl->param1 /* wone index*/, 
                                          &idx, DRV_DB_HASH_STA_TYPE_STATION) == OK)
         {
             sprintf(pIoctl->buf, "station found/created for mac %x:%x:%x:%x:%x:%x, idex=%d", 
                     pIoctl->mac[0],pIoctl->mac[1],pIoctl->mac[2],
                     pIoctl->mac[3],pIoctl->mac[4],pIoctl->mac[5], idx);
         }
         else 
         {
             status = ERROR;
         }
         break;
    
     case TIAP_GENERIC_IOCTL_TST_CHALLENGE:			
         wdrv_mgmtPrintChallenge( pIoctl->param1/* index */, pIoctl->buf);
         break;
    
     case TIAP_GENERIC_IOCTL_GET_TX_QUEUE:
         whal_apiPrintTxQueue( pIoctl->param1/* Qid */, pIoctl->buf );
         break;
    
     case TIAP_GENERIC_IOCTL_GET_EER:
         whal_apiReadEEPROM( pIoctl->param1/* address*/, pIoctl->buf);
         break;
    
     case TIAP_GENERIC_IOCTL_GET_MAC_REGREAD:
         whal_apiMacRead( pIoctl->param1/* read mac addr reg*/, pIoctl->buf);
         break;

     case TIAP_GENERIC_IOCTL_GET_PHY_REGREAD:
         whal_apiPhyRead( pIoctl->param1 /* read phy addr reg*/, pIoctl->buf);
         break;

     case TIAP_GENERIC_IOCTL_TRACE_PRINT:
         whal_apiTracePrint( pIoctl->param1/* num */, pIoctl->param2 /* from */, 
                             pIoctl->buf);
         break;

     case TIAP_GENERIC_IOCTL_WLAN_PARAMS:
         read_wlan_params( pIoctl->param1/* woneIndex */, pIoctl->buf);
         break;

     case TIAP_GENERIC_IOCTL_DOWNLOAD_FW:
         if (after_go) 
         {
             DDA_LOG(DDA_LOG_ERR, "doGenericIoctl: AP is started !\n");
             status = ERROR;
         }
         else
         {
             status = putFwImage(pIoctl->buf, pIoctl->buf_len);
             if (status == OK) 
             {
                 configured_params.fw = 1;
             } 
             else
             {
                 DDA_LOG(DDA_LOG_ERR, 
                         "doGenericIoctl: whal_acxProcPutFwImage() failed !\n");
                 status = ERROR;
             }
         }
         break;
     
    case TIAP_GENERIC_IOCTL_SET_BURST_LIMIT:
         configTxopLimit = pIoctl->param1;
         break;
     
    case TIAP_GENERIC_IOCTL_GET_BURST_LIMIT:
         wdrv_dbGetBurstLimit( (UINT16 *) pIoctl->buf );
         break;

   case TIAP_GENERIC_IOCTL_BSS_SERVICE_STATE:
        if(pIoctl->param1) 
        {
            wdrv_wdsBeaconsSendEnable(TRUE);
			TIWLAN_LED_LINK_UP;
        }
        else 
        {
            wdrv_wdsBeaconsSendEnable(FALSE);
            TIWLAN_LED_LINK_DOWN;
        }
        break;
        
    case TIAP_GENERIC_IOCTL_SEND_WDS_PKT:
        tiwlan_ddaSendWdsPkt(pIoctl->buf, pIoctl->buf_len);
        break;
        
  	 case TIAP_GENERIC_IOCTL_AFTER_GO_FLAG_GET:
    	  *((UINT32 *)pIoctl->buf) = after_go;
        break;
          
    case TIAP_GENERIC_IOCTL_SET_GPIO_OWNER:
         tiap_gpio_owner = (UINT8) pIoctl->param1;
         break;
    case TIAP_GENERIC_IOCTL_SET_PIL_MODE:
        DDA_LOG_ARG( DDA_LOG_ERR, "Func %s: Protocol Inspectation Layer is not supported for vxWorks!(%d)\n",
                          __FUNCTION__, pIoctl->param1, 0,0,0,0 );
         break;
#if REPORT_DEBUG
    case TIAP_GENERIC_IOCTL_GET_LOGGED_CHARS:
         {
             unsigned int flags;
			 int          tmpLoggedChars;
             PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
             tmpLoggedChars = logged_chars
             logged_chars = 0;
             PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
             DDA_LOG_ARG( DDA_LOG_ERR, "Func %s: logged chars=%d\n",
                          __FUNCTION__, tmpLoggedChars, 0,0,0,0 );
         }
         break;
    
     case TIAP_GENERIC_IOCTL_SET_LOGGED_CHARS:
         {
             unsigned int flags;
             PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
             logged_chars = pIoctl->param1; 
             PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
         }
         break;
#endif
     
    default:
        DDA_LOG_ARG( DDA_LOG_ERR,"Func %s: Unknown ioctl command %d\n",
                     __FUNCTION__, pIoctl->cmd, 0,0,0,0 );
        status = ERROR;
        break;
 }

 return status;

}


/******************************************************************************/

/*! \brief 	Executes basic IOCTL request
	\param 	pIoctl - pointer to IOCTL data
	\param 	cmd    - IOCTL command value
	\return status of operation OK, ERROR or TIAP_PARAM_ERxxx error code
*/
/******************************************************************************/

static int doBasicIoctl( DDA_IOCTL_BASIC_PAR_S  * pIoctl, int cmd )
{
 int  ret  = OK;
 INT32   type;
 INT32   val;
 INT32 * pParam;


 if( !pIoctl ) 
 {
    DDA_IOCTL_NULL_PTR_ERR(ret);
 	return ret;
 }
 type = pIoctl->parType;	
 val  = pIoctl->parValue;	
 pParam = &pIoctl->parValue;

 DDA_LOG_ARG(DDA_LOG_IO,"Func %s: cmd=%d type=%d value=%d\n",__FUNCTION__, 
             cmd, type, val, 0,0);

 switch (cmd)
 {
    case DDA_IOCTL_CMD_SET_A:
        switch (type)
        {
           case TIAP_PARAM_RTS:
               ret = tiap_param_set_rts( &init_cfg, (UINT16) val );
               break;
           case TIAP_PARAM_FRAG:
               ret = tiap_param_set_frag( &init_cfg, (UINT16) val );
               break;
           case TIAP_PARAM_REPLEV:
               ret = tiap_param_set_replev( val );
               break;
           case TIAP_PARAM_BEACON_PERIOD:
               ret = tiap_param_set_beaconperiod( &init_cfg, (UINT16)val);
               break;
           case TIAP_PARAM_DTIM_PERIOD:
               ret = tiap_param_set_dtim_period( &init_cfg, (UINT8)val);
               break;
           case TIAP_PARAM_BSS_BRIDGE_STATE:
               ret = tiap_param_set_bss_bridge_state( &init_cfg, 
                                                     (tiap_param_bss_br_state_t) val);
               break;
           case TIAP_PARAM_RATE_FALLBACK:
               ret = tiap_param_set_rate_fallback( &init_cfg, (UINT8) val );
               break;
           case TIAP_PARAM_RADIO_CAL:
               ret = tiap_param_set_radio_cal( &init_cfg, (UINT32) val );
               break;
           case TIAP_PARAM_RADIO_CAL_INT:
               ret = tiap_param_set_radio_cal_int(&init_cfg, (UINT16) val );
               break;
           case TIAP_PARAM_ENERGY_DETECT:
               ret = tiap_param_set_energy_detect( &init_cfg, (UINT8) val );
               break;
           case TIAP_PARAM_HIDDEN_SSID:
               ret = tiap_param_set_hidden_ssid( &init_cfg, (UINT16)val );
               break;
           case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
               ret = tiap_param_set_multidomain_cap( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_SPECTRUM_MGMT:
               ret = tiap_param_set_spectrum_mgmt( &init_cfg, (UINT8)val);
               break;
           case TIAP_PARAM_LEGACY_STA:
               ret = tiap_param_set_legacy_sta( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_LOCAL_POWER_CONS:
               ret = tiap_param_set_local_power_cons( &init_cfg, (__s8)val);
               break;
           case TIAP_PARAM_MAX_TX_POWER_LEVEL:
               ret = tiap_param_set_max_tx_pow_level( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_OPER_STATE:
               ret = tiap_param_set_oper_state( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_WONE_NUM:
               ret = tiap_param_set_wone_num( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_WONE_TIME_MODE:
               ret = tiap_param_set_wone_timemod( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_RADAR_DETECT_MODE:
               ret = tiap_param_set_radar_detect_mode( &init_cfg, (UINT8)val );
               break;
           case TIAP_PARAM_AP_BRIDGE_MODE:
               tiap_param_set_bridge_mode( &init_cfg, (UINT8)val );
               break;
		   case TIAP_PARAM_WDS_ENABLE:
				tiap_param_set_wds_enable(&init_cfg, (UINT8)val);
				break;
		   case TIAP_PARAM_EEPROM_LESS_ENABLE:
				ret = tiap_param_set_eeprom_less(&init_cfg, (UINT8)val);
				break;

           
           default:
               DDA_LOG_ARG( DDA_LOG_IO, "Func %s: Invalid type=%d cmd=%d val=%d\n",
                            __FUNCTION__, type, cmd, val, 0,0 );
               ret = EINVAL;
       }
    break;

    
    case DDA_IOCTL_CMD_GET_A:
        switch (type)
        {
            case TIAP_PARAM_RTS:
                ret = tiap_param_get_rts(pParam);
                break;
            case TIAP_PARAM_FRAG:
                ret = tiap_param_get_frag(pParam);
                break;
			case TIAP_PARAM_REPLEV:
                ret = tiap_param_get_replev(pParam);
                break;
            case TIAP_PARAM_BEACON_PERIOD:
                ret = tiap_param_get_beaconperiod(pParam);
                break;
            case TIAP_PARAM_DTIM_PERIOD:
                ret = tiap_param_get_dtim_period(pParam);
                break;
            case TIAP_PARAM_BSS_BRIDGE_STATE:
                ret = tiap_param_get_bss_bridge_state(pParam);
                break;
            case TIAP_PARAM_RATE_FALLBACK:
                ret = tiap_param_get_rate_fallback(pParam);
                break;
            case TIAP_PARAM_RADIO_CAL:
                ret = tiap_param_get_radio_cal(pParam);
                break;
            case TIAP_PARAM_RADIO_CAL_INT:
                ret = tiap_param_get_radio_cal_int(pParam);
                break;
            case TIAP_PARAM_ENERGY_DETECT:
                ret = tiap_param_get_energy_detect(pParam);
                break;
            case TIAP_PARAM_HIDDEN_SSID:
                ret = tiap_param_get_hidden_ssid(pParam);
                break;
            case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
                ret = tiap_param_get_multidomain_cap(pParam);
                break;
            case TIAP_PARAM_SPECTRUM_MGMT:
                ret = tiap_param_get_spectrum_mgmt(pParam);
                break;
            case TIAP_PARAM_LEGACY_STA:
                ret = tiap_param_get_legacy_sta(pParam);
                break;
            case TIAP_PARAM_LOCAL_POWER_CONS:
                ret = tiap_param_get_local_power_cons(pParam);
                break;
            case TIAP_PARAM_MAX_TX_POWER_LEVEL:
                ret = tiap_param_get_max_tx_pow_level(pParam);
                break;
            case TIAP_PARAM_OPER_STATE:
                ret = tiap_param_get_oper_state(pParam);
                break;
            case TIAP_PARAM_WONE_NUM:
                ret = tiap_param_get_wone_num(pParam);
                break;
            case TIAP_PARAM_WONE_TIME_MODE:
                ret = tiap_param_get_wone_timemod(pParam);
                break;
            case TIAP_PARAM_RADAR_DETECT_MODE:
                ret = tiap_param_get_radar_detect_mode(pParam);
                break;
            case TIAP_PARAM_AP_BRIDGE_MODE:
                ret = tiap_param_get_bridge_mode(pParam);
                break;
			case TIAP_PARAM_WDS_ENABLE:
				ret = tiap_param_get_wds_enable(pParam);
				break;
			case TIAP_PARAM_EEPROM_LESS_ENABLE:
				ret = tiap_param_get_eeprom_less(pParam);
				break;


            default:
                DDA_LOG_ARG( DDA_LOG_ERR, "Func %x: Invalid type=%d cmd=%d val=%d\n",
                             __FUNCTION__, type, cmd, val, 0,0 );
                ret = EINVAL;
        }	
    
    break;
    
    default:
        DDA_LOG_ARG( DDA_LOG_IO,"Func %s:Invalid command cmd=%d type=%d value=%d\n",
                     __FUNCTION__, cmd, type, val, 0,0 );
        ret = EINVAL;
 }
 
 return ret;

}

/******************************************************************************/

/*! \brief 	Performs complex IOCTL request
	\param 	pIoctl - pointer to IOCTL data
	\param 	cmd    - IOCTL command value
	\return status of operation OK, ERROR or TIAP_PARAM_ERxxx error code
*/
/******************************************************************************/

static int doComplexIoctl( DDA_IOCTL_COMPLEX_PAR_S  * pIoctl, int cmd )
{
 int    ret = OK;
 INT32  subcmd;

 if( !pIoctl ) 
 {
     DDA_IOCTL_NULL_PTR_ERR(ret); 	
     return ret;
 }

 subcmd = pIoctl->subcmd;	

 DDA_LOG_ARG( DDA_LOG_IO, "Func %s: cmd=%d subcmd=%d\n", __FUNCTION__,
              cmd, subcmd, 0,0,0 );

 switch (cmd)
 {
    case DDA_IOCTL_CMD_SET_A:
        
        switch (subcmd)
        {
            case TIAP_PARAM_AUTH_TYPE:
                if( param_auth_tbl(&init_cfg, pIoctl->params.auth_tbl) )
                    ret = TIAP_PARAM_ERANGE; 
                break;
				case TIAP_PARAM_EEPROM:
					if (param_eeprom(&init_cfg, &pIoctl->params.eeprom))
						ret = TIAP_PARAM_ERANGE;
					break;
            case TIAP_PARAM_WD:
                if( param_wd(&init_cfg, &pIoctl->params.wd) )
                    ret = TIAP_PARAM_ERANGE;
                break;
			case TIAP_PARAM_INTERF_WD_CONFIG:
				if ( param_interference_wd_config( &init_cfg, &pIoctl->params.interf_wd_conf))
					ret = TIAP_PARAM_ERANGE;				
				break;
			case TIAP_PARAM_HW_INTERF_CFG:
				if ( param_hw_interf_cfg( &init_cfg, &pIoctl->params.hw_interf_param))
					ret = TIAP_PARAM_ERANGE;				
				break;
            case TIAP_PARAM_ACCESS_LIST:
                if( param_access_list(&init_cfg, &pIoctl->params.access_list) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_4X:
                if( param_4x(&init_cfg, &pIoctl->params._4x) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_MULTI_SSID:
                if( param_multi_ssid(&init_cfg, &pIoctl->params.multi_ssid) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CFG_POWER_TABLE:
                if( param_radio(&init_cfg, &pIoctl->params.radio) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CHANNEL_RANGE:
                if( param_ch_rang(&init_cfg, &pIoctl->params.ch_rang) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_WONE_ELEM:
                if( param_wone(&init_cfg, &pIoctl->params.wone) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_PRIV_REG_DOMAIN:
                if( param_priv_regdomain(&init_cfg, &pIoctl->params.priv_regdomain) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CURR_REG_DOMAIN:
                if( param_curr_reg_domain(&init_cfg, &pIoctl->params.cur_regdomain) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_RETRY:
                if( param_retry(&init_cfg, &pIoctl->params.retry) )
                ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_TXPOWER:
                if( param_tx_power(&init_cfg, &pIoctl->params.txpow) )
                ret = TIAP_PARAM_ERANGE;
                break;
			case TIAP_PARAM_COUNTRY_STR:
				if( param_set_country_str( &init_cfg, &pIoctl->params.country ))
					ret = TIAP_PARAM_ERANGE;
				break;
			case TIAP_PARAM_QOS:
				if( param_set_qos( &init_cfg, &pIoctl->params.qos ))
					ret = TIAP_PARAM_ERANGE;
				break;
            case TIAP_PARAM_BIT:
                if ( param_set_bit_args( &init_cfg, &pIoctl->params.bit ))
                    ret = TIAP_PARAM_ERANGE;
                break;
            default:
                DDA_LOG_ARG( DDA_LOG_ERR, 
                             "doComplexIoctl: Invalid sub command cmd=%d subcmd=%d\n",
                             cmd, subcmd, 0,0,0,0 );
                ret = EINVAL;
        }
        break;

    case DDA_IOCTL_CMD_GET_A:
        switch (subcmd)
        {
            case TIAP_PARAM_AUTH_TYPE:
                if( get_param_auth_tbl(pIoctl->params.auth_tbl) )
                    ret = TIAP_PARAM_ERANGE; 
                break;
            case TIAP_PARAM_WD:
                if( get_param_wd(&pIoctl->params.wd) )
                    ret = TIAP_PARAM_ERANGE;
                break;
			case TIAP_PARAM_INTERF_WD_CONFIG:
				if( get_param_interference_wd(&pIoctl->params.interf_wd_conf) )
					ret = TIAP_PARAM_ERANGE;
				break;
			case TIAP_PARAM_HW_INTERF_CFG:
				if( get_param_hw_interf(&pIoctl->params.hw_interf_param) )
					ret = TIAP_PARAM_ERANGE;
				break;
            case TIAP_PARAM_ACCESS_LIST:
                if( get_param_access_list(&pIoctl->params.access_list) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_4X:
                if( get_param_4x(&pIoctl->params._4x) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_MULTI_SSID:
                if( get_param_multi_ssid(&pIoctl->params.multi_ssid) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CFG_POWER_TABLE:
                if( get_param_radio(&pIoctl->params.radio) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CHANNEL_RANGE:
                if( get_param_ch_rang(&pIoctl->params.ch_rang) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_WONE_ELEM:
                if( get_param_wone(&pIoctl->params.wone) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_STA_TBL:
                if( get_param_sta_tbl(pIoctl->params.sta_tbl) )
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_PRIV_REG_DOMAIN:
                if (get_param_priv_regdomain(&pIoctl->params.priv_regdomain))
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_CURR_REG_DOMAIN:
                if (get_param_curr_reg_domain(&pIoctl->params.cur_regdomain))
                    ret = TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_NET_STATS:
                get_param_stats(&pIoctl->params.net_stats);
                break;
            case TIAP_PARAM_RETRY:
                if (get_param_retry(&pIoctl->params.retry))
                    ret = TIAP_PARAM_ERANGE;
            case TIAP_PARAM_TXPOWER:
                if (get_param_tx_power(&pIoctl->params.txpow))
                    ret = TIAP_PARAM_ERANGE;
                break;
			case TIAP_PARAM_NAME:
				if (get_param_name(&pIoctl->params.name))
					ret = TIAP_PARAM_ERANGE;
				break;
            case TIAP_PARAM_COUNTRY_STR:
                if (get_param_country_str(&pIoctl->params.country))
					ret	= TIAP_PARAM_ERANGE;
                break;
            case TIAP_PARAM_QOS:
                if (get_param_qos(&pIoctl->params.qos))
					ret	= TIAP_PARAM_ERANGE;
                break;            
            case TIAP_PARAM_DIE_ID:
                if (get_param_die_id(&pIoctl->params.die_id))
					ret	= TIAP_PARAM_ERANGE;
                break;                     
			default:
                DDA_LOG_ARG( DDA_LOG_ERR,"Func %s: Invalid sub command cmd=%d subcmd=%d\n",
                             __FUNCTION__, cmd, subcmd, 0,0,0 );
                ret = EINVAL;
        }
        break;

    default:
        DDA_LOG_ARG( DDA_LOG_ERR, "Func %s:Invalid command cmd=%d subcmd=%d\n",
                     __FUNCTION__, cmd, subcmd, 0,0,0 );
     ret = EINVAL;
 }

 return ret;
}

/******************************************************************************/

/*! \brief 	Sets QoS (WME) parameters
	\param 	init_cfg - pointer to initialization data structure
	\param 	pQoS     - pointer to QoS structure
	\return status of operation OK
*/
/******************************************************************************/

static int param_set_qos( initConfigStruct *init_cfg, tiap_param_qos_t * pQoS )
{
 int ret = OK;
 int i; 

 memset(&init_cfg->wmeACParams, 0, sizeof(wdrv_QoSACParams_t));

 init_cfg->wmeACParams.enabled = pQoS->enabled;
 if (pQoS->enabled) {
 for (i=0; i<4; ++i) 
 {
	 init_cfg->wmeACParams.ap_edcfParams[i].aifsn = pQoS->edcf_info[0][i].aifsn;
	 init_cfg->wmeACParams.sta_edcfParams[i].aifsn= pQoS->edcf_info[1][i].aifsn;

	 init_cfg->wmeACParams.ap_edcfParams[i].cwMin = pQoS->edcf_info[0][i].cwMin;
	 init_cfg->wmeACParams.sta_edcfParams[i].cwMin= pQoS->edcf_info[1][i].cwMin;

	 init_cfg->wmeACParams.ap_edcfParams[i].cwMax = pQoS->edcf_info[0][i].cwMax;
	 init_cfg->wmeACParams.sta_edcfParams[i].cwMax= pQoS->edcf_info[1][i].cwMax;

	 init_cfg->wmeACParams.ap_edcfParams[i].txopLimit11b = 
								pQoS->edcf_info[0][i].txopLimit11b;
	 init_cfg->wmeACParams.sta_edcfParams[i].txopLimit11b= 
								pQoS->edcf_info[1][i].txopLimit11b;

	 init_cfg->wmeACParams.ap_edcfParams[i].txopLimit11g = pQoS->edcf_info[0][i].txopLimit11g;
	 init_cfg->wmeACParams.sta_edcfParams[i].txopLimit11g= pQoS->edcf_info[1][i].txopLimit11g;

	 init_cfg->wmeACParams.ap_edcfParams[i].ackPolicy = pQoS->edcf_info[0][i].ackPolicy;
	 init_cfg->wmeACParams.sta_edcfParams[i].ackPolicy= pQoS->edcf_info[1][i].ackPolicy;

	 init_cfg->wmeACParams.ap_edcfParams[i].shortRetryLimit = 
								pQoS->edcf_info[0][i].shortRetryLimit;
	 init_cfg->wmeACParams.sta_edcfParams[i].shortRetryLimit= 
								pQoS->edcf_info[1][i].shortRetryLimit;

	 init_cfg->wmeACParams.ap_edcfParams[i].longRetryLimit = 
								pQoS->edcf_info[0][i].longRetryLimit;
	 init_cfg->wmeACParams.sta_edcfParams[i].longRetryLimit= 
		 pQoS->edcf_info[1][i].longRetryLimit;

	 init_cfg->wmeACParams.ap_edcfParams[i].dot11EDCATableMSDULifetime =
								pQoS->edcf_info[0][i].txMaxLifetime;
	 init_cfg->wmeACParams.sta_edcfParams[i].dot11EDCATableMSDULifetime=  
								pQoS->edcf_info[1][i].txMaxLifetime;

	 init_cfg->wmeACParams.ap_edcfParams[i].txopContinuation = 
								pQoS->edcf_info[0][i].txOpContinuation;
	 init_cfg->wmeACParams.sta_edcfParams[i].txopContinuation= 
								pQoS->edcf_info[1][i].txOpContinuation;
 }
 init_cfg->wmeACParams.classifierMode = pQoS->classifierMode;

 if (pQoS->classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) 
 {
 	init_cfg->wmeACParams.numOfCP = pQoS->diffserv.numOfCP;
 	memcpy(init_cfg->wmeACParams.diffServCPToUPMap, pQoS->diffserv.map, 
		   WDRV_QOS_MAX_CP * sizeof(UINT8));
 } 
 }

 return ret;
}

/******************************************************************************/

/*! \brief 	Gets QoS (WME) parameters
	\param 	pQoS     - pointer to QoS structure
	\return status of operation OK
*/
/******************************************************************************/

static int  get_param_qos(tiap_param_qos_t * pQoS)
{
	int ret = OK;
    int i;
	wdrv_QoSACParams_t params;
	
	params = wdrv_dbGetWMEParams();
	
    memset(pQoS, 0, sizeof(tiap_param_qos_t));
	pQoS->enabled = (__u8)params.enabled;

	for (i=0; i<4; ++i) 
	{
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].aifsn = 
			params.ap_edcfParams[i].aifsn;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].aifsn = 
			params.sta_edcfParams[i].aifsn;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].cwMin = 
			params.ap_edcfParams[i].cwMin;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].cwMin = 
			params.sta_edcfParams[i].cwMin;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].cwMax = 
			params.ap_edcfParams[i].cwMax;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].cwMax = 
			params.sta_edcfParams[i].cwMax;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txopLimit11b = 
			params.ap_edcfParams[i].txopLimit11b;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txopLimit11b = 
			params.sta_edcfParams[i].txopLimit11b;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txopLimit11g = 
			params.ap_edcfParams[i].txopLimit11g;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txopLimit11g = 
			params.sta_edcfParams[i].txopLimit11g;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].ackPolicy = 
			params.ap_edcfParams[i].ackPolicy;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].ackPolicy = 
			params.sta_edcfParams[i].ackPolicy;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].shortRetryLimit = 
			params.ap_edcfParams[i].shortRetryLimit;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].shortRetryLimit = 
			params.sta_edcfParams[i].shortRetryLimit;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].longRetryLimit = 
			params.ap_edcfParams[i].longRetryLimit;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].longRetryLimit = 
			params.sta_edcfParams[i].longRetryLimit;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txMaxLifetime = 
			params.ap_edcfParams[i].dot11EDCATableMSDULifetime;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txMaxLifetime = 
			params.sta_edcfParams[i].dot11EDCATableMSDULifetime;
		
		pQoS->edcf_info[WDRV_QOS_AC_AP_PARAMS][i].txOpContinuation = 
			params.ap_edcfParams[i].txopContinuation;
		pQoS->edcf_info[WDRV_QOS_AC_STA_PARAMS][i].txOpContinuation = 
			params.sta_edcfParams[i].txopContinuation;
	}

	pQoS->classifierMode = params.classifierMode;

	if (pQoS->classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) 
	{
		pQoS->diffserv.numOfCP = params.numOfCP;
		memcpy( pQoS->diffserv.map, params.diffServCPToUPMap, 
			    TIAP_PARAM_QOS_MAX_CP * sizeof(__u8));
	} 

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets WLAN driver name
	\param 	pName  - pointer to buffer to copy driver name string
	\return status of operation OK
*/
/******************************************************************************/

static int get_param_name( tiap_param_drvname_t * pName )
{
 int ret = OK;

 if (!pName) 
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return (ret);
 }
 strcpy( pName->name, "wlan0" );
 
 return ret;
}

/******************************************************************************/

/*! \brief 	Gets TNETW die id
	\param 	pDieId  - pointer to die id structure to copy result
	\return status of operation OK
*/
/******************************************************************************/

static int get_param_die_id( tiap_param_die_id_t * pDieId )
{
 int ret = OK;
 tnetw_die_id_t pTnetwDieId; 

 if (!pDieId) 
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return (ret);
 }
 
 if (whal_apiGetTnetwDieId( &pTnetwDieId ) == NOK)
     return TIAP_PARAM_ERANGE;
 
 pDieId->die_id_lo = pTnetwDieId.die_id_lo;
 pDieId->die_id_hi = pTnetwDieId.die_id_hi;
 
 return ret;
}

/******************************************************************************/

/*! \brief 	Sets RTS threshold value
	\param 	init_cfg - pointer to initialization data structure
	\param 	val   	 - RTS threshold value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_rts( initConfigStruct * init_cfg, UINT16 val )
{
 int  ret = OK;

	
 if( after_go )
 {
     DDA_IOCTL_RUNTIME_ERR(ret);
     return ret;
 }

 if ( val > TIAP_PARAM_RTS_TRESHOLD_MAX )
 {
     DDA_IOCTL_RANGE_ERR(ret, val);
     return ret;
 }	

 init_cfg->apRtsThreshold = val; 
 configured_params.rts = 1;

 return ret;

}

/******************************************************************************/

/*! \brief 	Gets RTS threshold value
	\param 	pParam - pointer to return RTS threshold value
	\return status of operation OK or EINVAL if parameters is null pointer
*/
/******************************************************************************/

static int tiap_param_get_rts( UINT32 * pParam )
{
 int ret = OK;
 UINT16 rts;
    
 if( !pParam ) 
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return ERROR;
 }
	
 wdrv_dbGetOperationRTSThreshold( &rts );	
 *pParam = rts;

 return ret;
}

/******************************************************************************/

/*! \brief 	Sets fragmentation value
	\param 	init_cfg - pointer to initialization data structure
	\param 	val   	 - fragmentation value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_frag(initConfigStruct * init_cfg, UINT16 val )
{
 int  ret = OK;

 if( after_go )
 {
     DDA_IOCTL_RUNTIME_ERR(ret);
     return ret;
 }

 if ( val < TIAP_PARAM_MIN_FRAGMENTATION_TRESHOLD ||
      val > TIAP_PARAM_MAX_FRAGMENTATION_TRESHOLD ||
      val % 2 /* odd number */ )
 {
     DDA_IOCTL_RANGE_ERR(ret, val);
     return ret;
 }	

 init_cfg->apFragThreshold = (UINT16) val; 
 configured_params.frag = 1;

 return ret;

}

/******************************************************************************/

/*! \brief 	Gets fragmentation value
	\param 	pParam - pointer to return fragmentation value
	\return status of operation OK or EINVAL if parameters is null pointer
*/
/******************************************************************************/

static int tiap_param_get_frag (UINT32 * pParam)
{
 int  	ret = OK;
 UINT16 frag;

 if( !pParam ) 
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return ERROR;
 }

 wdrv_dbGetOperationFragmentationThreshold ( &frag );
 *pParam = frag;
  
 return OK;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets EEPROM image
	
	\param init_cfg		initial driver configuration structure
	\param eeprom eeprom image
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_eeprom(initConfigStruct *init_cfg, tiap_param_eeprom_t * eeprom) {
	int ret = 0;
	
	if (eeprom->len > MAX_EEPROM_LEN ) {
	  DDA_IOCTL_RANGE_ERR(ret, eeprom->len);
		return ret;
	}

	memcpy((void *)init_cfg->eeprom.image, eeprom->image, eeprom->len);
	init_cfg->eeprom.len = eeprom->len;
	configured_params.eeprom_image = 1;

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets retry parameter value
	\param 	init_cfg - pointer to initialization data structure
	\param 	retry    - pointer to input retry parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_retry(initConfigStruct *init_cfg, tiap_param_retry_t * retry)
{
 int ret = 0;
	
 if (!after_go)
 {
	 if (retry)
	 {
		if (retry->value)
		{ 
			/*the each retry limit should be non-zero value */
			switch(retry->flags) /*tells which retry limit to set */
			{
				case TIAP_PARAM_RETRY_LONG: /* long retry limit*/ 
					init_cfg->apLongRetryLimit = (UINT8)retry->value;
					configured_params.retry_long = 1;
					break;
				
				case TIAP_PARAM_RETRY_SHORT: /* short retry limit*/ 
					init_cfg->apShortRetryLimit = (UINT8)retry->value;
					configured_params.retry_short = 1;
					break;
				
				case TIAP_PARAM_LIFETIME_TX:
					init_cfg->apMaxTxLifetime = retry->value;
					configured_params.lifetime_tx = 1;
					break;
				
				case TIAP_PARAM_LIFETIME_RX:
					init_cfg->apMaxRxLifetime = retry->value;
					configured_params.lifetime_rx = 1;
					break;
				
				default:
					DDA_IOCTL_RANGE_ERR(ret, retry->flags);
			}
		}
		else 
		{
			DDA_IOCTL_RANGE_ERR(ret, retry->value);
		}	
	}
	else
	{
		DDA_IOCTL_NULL_PTR_ERR(ret);
	}
 }
 else
 {
	 DDA_IOCTL_RUNTIME_ERR(ret);
 }
	
 return ret;
}

/******************************************************************************/

/*! \brief 	Gets retry parameter value
	\param 	pParam - pointer to retry structure to copy data
	\return status of operation OK or EINVAL if parameters is null pointer
*/
/******************************************************************************/

static int get_param_retry( tiap_param_retry_t * retry )
{
 int ret = 0;

 if (retry)
 {
	switch (retry -> flags)
	{
		case TIAP_PARAM_RETRY_LONG:
			wdrv_dbGetOperationLongRetryLimit((UINT8*) &retry->value);
			break;
		
		case TIAP_PARAM_RETRY_SHORT:
			wdrv_dbGetOperationShortRetryLimit((UINT8*) &retry->value);
			break;
		
		case TIAP_PARAM_LIFETIME_TX:
			wdrv_dbGetOperationMaxTransmitMSDULifetime(&retry->value);
			break;
		
		case TIAP_PARAM_LIFETIME_RX:
			wdrv_dbGetOperationMaxReceiveLifetime(&retry->value);
			break;
		
		default:
			DDA_IOCTL_RANGE_ERR(ret, retry->flags);
	}
 }
 else 
 {
	 DDA_IOCTL_NULL_PTR_ERR(ret);
 }

 return ret;
}

/******************************************************************************/

/*! \brief 	Sets transmitting power value
	\param 	init_cfg - pointer to initialization data structure
	\param 	txpower  - pointer to input transmitting power structure
	\return status of operation OK or EINVAL if parameter is invalid 
*/
/******************************************************************************/


static int param_tx_power(initConfigStruct *init_cfg, tiap_param_txpower_t * txpower)
{
 int ret = OK;

 if (!txpower)
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return ret;
 }

 switch( txpower->hr )
 {
    case txpower_a:
        memcpy((void *)init_cfg->RegInit.txPowerLevels_a, (void *)txpower->txpower,
               sizeof(txpower->txpower));
        configured_params.txpow_a = 1;
        break;
    case txpower_g:
        memcpy((void *)init_cfg->RegInit.txPowerLevels_g, (void *)txpower->txpower,
               sizeof(txpower->txpower));
        configured_params.txpow_g = 1;
        break;
    default:
        DDA_LOG_ARG( DDA_LOG_ERR, "Func %s:invalid power type\n", __FUNCTION__,
                     txpower->hr, 0,0,0,0);
        ret = EINVAL;
 }

 return ret;

}

/******************************************************************************/

/*! \brief 	Gets transmitting power parameter values
	\param 	txpower - pointer to return fragmentation value
	\return status of operation OK or EINVAL if input parameter is invalid
*/
/******************************************************************************/

static int get_param_tx_power( tiap_param_txpower_t * txpower )
{
 int ret = OK;
 int lev;

 if (!txpower)
 {
     DDA_IOCTL_NULL_PTR_ERR(ret);
     return ret;
 }

 switch( txpower->hr )
 {

    case txpower_a:
        for(lev=1; lev<=TIAP_PARAM_NUM_TXPOW_LEVELS; lev++)
            wdrv_dbGetTxPowerLevels(RADIO_BAND_5GHZ, lev, &(txpower->txpower[lev-1]));
        break;
    case txpower_g:
        for(lev=1; lev<=TIAP_PARAM_NUM_TXPOW_LEVELS; lev++)
            wdrv_dbGetTxPowerLevels(RADIO_BAND_2_4GHZ, lev, &(txpower->txpower[lev-1]));
        break;
    default:
        DDA_LOG_ARG( DDA_LOG_ERR, "Func %s:invalid power type\n", __FUNCTION__,
                     txpower->hr, 0,0,0,0);
        ret = EINVAL;
 }
 return ret;
}

/* implementation of complex data types */

/* set */

/******************************************************************************/

/*! \brief 	Sets authentication algorithm parameter value
	\param 	init_cfg - pointer to initialization data structure
	\param 	auth_tbl - pointer to authentication algorithm parameter
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_auth_tbl(initConfigStruct *init_cfg, tiap_param_auth_algo_t *auth_tbl)
{
int ret = 0;

	if( !after_go )
	{
		if( ((auth_tbl[0].algo == TIAP_PARAM_MIB_OPEN_SYSTEM && auth_tbl[1].algo == TIAP_PARAM_MIB_SHARED_KEY) ||
		     (auth_tbl[1].algo == TIAP_PARAM_MIB_OPEN_SYSTEM && auth_tbl[0].algo == TIAP_PARAM_MIB_SHARED_KEY)) ) 
		    
		{
			if ( ((auth_tbl[0].enable == FALSE) || (auth_tbl[0].enable == TRUE)) &&
			     ((auth_tbl[1].enable == FALSE) || (auth_tbl[1].enable == TRUE)) )
			{
				init_cfg->dot11AuthenticationAlgorithmTable[0].authAlgo = auth_tbl[0].algo;
				init_cfg->dot11AuthenticationAlgorithmTable[0].enable = (BOOL) auth_tbl[0].enable;
				init_cfg->dot11AuthenticationAlgorithmTable[1].authAlgo = auth_tbl[1].algo;
				init_cfg->dot11AuthenticationAlgorithmTable[1].enable = (BOOL) auth_tbl[1].enable;
				configured_params.auth_tbl = 1;
			}
			else{
                DDA_LOG_ARG(DDA_LOG_ERR, 
                            "Func %s: algorithm enable value: invalid parameter %d %d\n",
                            __FUNCTION__, auth_tbl[0].enable, auth_tbl[1].enable,0,0,0 );
				return TIAP_PARAM_ERANGE ;
			}

		}
		else {
            DDA_LOG_ARG(DDA_LOG_ERR, 
                        "%s: auth algorithm: invalid parameter %d %d\n", __FUNCTION__,
                        auth_tbl[0].algo, auth_tbl[1].algo, 0,0,0);
			return TIAP_PARAM_ERANGE ;
		}
	}
	else
    {
        DDA_IOCTL_RUNTIME_ERR(ret);
    }

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets watch dog for software recovery
	\param 	init_cfg - pointer to initialization data structure
	\param 	wd 		-  pointer to input watch dog structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_wd(initConfigStruct *init_cfg, tiap_param_wd_t *wd)
{
int ret = 0;

	if( !after_go )
	{
		switch( wd->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->EnRestart = wd->value.val8;
					configured_params.wd_en = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  enable :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;
				
			case TIAP_PARAM_WD_INTERF:
				if ( (wd->value.val8 == 0) || (wd->value.val8 == 1) || (wd->value.val8 == 2))
				{
					init_cfg->EnInterfWA = wd->value.val8;
					configured_params.wd_interf = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  enable :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;
			case TIAP_PARAM_WD_RXFREE:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->RxFreeMemOptionWA = wd->value.val8;
					configured_params.wd_rxfree = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR,
                                "%s:  rx free :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}

				break;

			case TIAP_PARAM_WD_BEACONS:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->BeaconNotSentOptionWA = wd->value.val8;
					configured_params.wd_beacons = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  beacons :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}

				break;

			case TIAP_PARAM_WD_TXSTUCK:
				if ( wd->value.val8 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->TxStuckOptionWA = wd->value.val8;
					configured_params.wd_txstuck = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  tx stuck :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;

			case TIAP_PARAM_WD_FCSERR:
				if ( wd->value.val16 <= TIAP_PARAM_MAX_WD_INTVL) 
				{
					init_cfg->ConsFcsErrOptionWA = wd->value.val16;
					configured_params.wd_fcserrs = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  fcs err :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val16, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;

			case TIAP_PARAM_WD_ACI:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->aciOptionWA = wd->value.val8;
					configured_params.wd_aci = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  enable :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;
			
			case TIAP_PARAM_WD_NB:
				if ( (wd->value.val8 == TRUE) || (wd->value.val8 == FALSE) )
				{
					init_cfg->nbOptionWA = wd->value.val8;
					configured_params.wd_nb = 1;
				}
				else
				{
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  enable :invalid parameter %d \n", __FUNCTION__,
                                wd->value.val8, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
				}
				break;
			
			
			
			default:
				DDA_LOG_ARG(DDA_LOG_ERR, 
                            "%s: invalid type %d\n", __FUNCTION__, 
                            wd->cmd, 0,0,0,0);
				return TIAP_PARAM_ERANGE;
		}
	}
	else
    {
        DDA_IOCTL_RUNTIME_ERR(ret);
    }
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets ACI/NB interference Watch Dog configuration parameters
	\param 	init_cfg - pointer to initialization data structure
	\param 	interf_param -  pointer to input interface parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameters are invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_interference_wd_config(initConfigStruct *init_cfg, tiap_param_interference_wd_t *interf_param)
{
	int ret = 0;

	if( !after_go )
	{
		switch (interf_param->cmd) {
	
			case  TIAP_PARAM_ACI_TIMEOUT:
				  init_cfg->interfWAparams.aciStateTimeout = interf_param->value.val32;
				  configured_params.aci_state_timeout = 1;
				  break;
			case  TIAP_PARAM_NB_TIMEOUT:
				  init_cfg->interfWAparams.nbStateTimeout = interf_param->value.val32;
				  configured_params.nb_state_timeout = 1;
				  break;
			case  TIAP_PARAM_ACI_RATE_THR:
				  init_cfg->interfWAparams.aciRateThreshold = interf_param->value.val32;
				  configured_params.aci_rate_thresh = 1;
				  break;
			case  TIAP_PARAM_ACI_EVENT_THR:
				  init_cfg->interfWAparams.aciEventThreshold = interf_param->value.val32;
				  configured_params.aci_event_thresh = 1;
				  break;
			case  TIAP_PARAM_PD_LOW:
				  init_cfg->interfWAparams.pdUnderAciLowThrsh = interf_param->value.val16;
				  configured_params.pd_low_thresh = 1;
				  break;
			case  TIAP_PARAM_PD_HIGH:
				  init_cfg->interfWAparams.pdUnderAciHighThrsh = interf_param->value.val16;
				  configured_params.pd_high_thresh = 1;
				  break; 
			case  TIAP_PARAM_MEDIUM_BUSY_THR:
				  if (interf_param->value.val32 <= 100) {
					  init_cfg->interfWAparams.mediumBusyThresh = interf_param->value.val32;
					  configured_params.medium_busy_thresh = 1;
				  }
				  else {
						DDA_LOG_ARG(DDA_LOG_ERR, 
									"%s:  invalid  medium busy threshold %d \n", 
									__FUNCTION__, interf_param->value.val32, 0,0,0,0 );
						ret = TIAP_PARAM_ERANGE ;
				  }
	
				  break;
			
			default:
					DDA_LOG_ARG(DDA_LOG_ERR, "%s: invalid command type %d\n", 
								__FUNCTION__, interf_param->cmd, 0,0,0,0);
					ret = TIAP_PARAM_ERANGE;

		}
	}
	else /* after go */
		DDA_IOCTL_RUNTIME_PAR_ERR(ret, interf_param->cmd);
	return ret;
}

/******************************************************************************/

/*! \brief 	Set HW interference handling parameters
	\param 	init_cfg - pointer to initialization data structure
	\param 	interf_param -  pointer to input structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
*/
/******************************************************************************/

static int param_hw_interf_cfg(initConfigStruct *init_cfg, tiap_param_hw_interf_t *interf_param)
{
	int ret = 0;

	if( !after_go )
	{
 
		switch (interf_param->cmd) {
	
			case  TIAP_PARAM_INPUT_CCA:
				  init_cfg->hwInterfParams.inputCCA = interf_param->value;
				  configured_params.hw_input_cca = 1;
				  break;
			case  TIAP_PARAM_TX_CCA:
				  init_cfg->hwInterfParams.txCCA = interf_param->value;
				  configured_params.hw_tx_cca = 1;
				  break;
			case  TIAP_PARAM_QUALIF_CCA:
				  init_cfg->hwInterfParams.qualifiedCCA = interf_param->value;
				  configured_params.hw_qualif_cca = 1;
				  break;
			case  TIAP_PARAM_STOMP_RX:
				  init_cfg->hwInterfParams.stompForRx = interf_param->value;
				  configured_params.hw_stomp_rx = 1;
				  break;
			case  TIAP_PARAM_STOMP_TX:
				  init_cfg->hwInterfParams.stompForTx = interf_param->value;
				  configured_params.hw_stomp_tx = 1;
				  break;
					
			default:
					DDA_LOG_ARG(DDA_LOG_ERR, "%s: invalid command type %d\n", 
								__FUNCTION__, interf_param->cmd, 0,0,0,0);
					ret = TIAP_PARAM_ERANGE;

		}
	}
	else /* after go */
		DDA_IOCTL_RUNTIME_PAR_ERR(ret, interf_param->cmd);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets station access list parameters
	\param 	init_cfg - pointer to initialization data structure
	\param 	access_list -  pointer to input station access list structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
*/
/******************************************************************************/

static int param_access_list(initConfigStruct *init_cfg, tiap_param_access_list_t *access_list)
{
int ret = 0;

	switch( access_list->cmd )
	{
		case TIAP_PARAM_ACCESS_SET_TYPE:
			DDA_LOG_ARG(DDA_LOG_ERR, 
                        "%s:setting access list type is not allowed\n",
                        __FUNCTION__, 0,0,0,0,0);
			ret = 1;
			break;

		case TIAP_PARAM_ACCESS_LST:
			if( !after_go )
		       	{
				if ( (access_list->enabled == TRUE) || (access_list->enabled == FALSE) )
			       	{
					if ( (access_list->type == 1) || (access_list->type == 0) ) {
						init_cfg->mgmtAccessList.mgmt_access_list_type = access_list->type;
						memcpy((void *)init_cfg->mgmtAccessList.mgmt_access_list,
							(void *)access_list->list, TIAP_PARAM_ACCESS_LIST_SIZE * ETH_ALEN);
						init_cfg->mgmtAccessList.mgmt_access_enabled = access_list->enabled;
						configured_params.access_list = 1;
					}
				}
			}
			else DDA_IOCTL_RUNTIME_PAR_ERR(ret, access_list->cmd);
			break;

		case TIAP_PARAM_ACCESS_ADD:
			if( !after_go )
            {
				DDA_IOCTL_RUNTIME_PAR_ERR(ret, access_list->cmd);
            }
			else 
            {
				if (mgmtAccessSetMAC(access_list->list[0]) != OK)
					ret = TIAP_PARAM_ELOGICAL;
			}
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_INDEX:
			if ( (access_list->index >= 0) && (access_list->index <=MGMT_ACCESS_LIST_SIZE) ){
				if( !after_go )
					DDA_IOCTL_RUNTIME_PAR_ERR(ret, access_list->cmd);
				else {
					if (mgmtAccessRemMAC(access_list->index) != OK)
						ret = TIAP_PARAM_ELOGICAL;
				}
			}
			else {
				DDA_LOG_ARG(DDA_LOG_ERR,  
                            "%s:  index :invalid parameter %d \n", __FUNCTION__,
							access_list->index, 0,0,0,0);
				ret = TIAP_PARAM_ERANGE ;
			}

			break;

		case TIAP_PARAM_ACCESS_DEL_BY_ADDR:
			if( !after_go)
				DDA_IOCTL_RUNTIME_PAR_ERR(ret, access_list->cmd);
			else {
				if (mgmtAccessRemMACbyAddr(access_list->list[0]) != OK)
					ret = TIAP_PARAM_ELOGICAL;
			}
			break;

		case TIAP_PARAM_ACCESS_RESET:
			if ( access_list->enabled == FALSE)
			{
				init_cfg->mgmtAccessList.mgmt_access_enabled = access_list->enabled;
				memset(init_cfg->mgmtAccessList.mgmt_access_list,
							0, TIAP_PARAM_ACCESS_LIST_SIZE * ETH_ALEN);
	
				configured_params.access_list = 1;
			}
			else 
            {
				DDA_LOG_ARG(DDA_LOG_ERR,
                            "%s:  enabled :invalid parameter %d \n", __FUNCTION__,
							access_list->enabled, 0,0,0,0);
				ret = TIAP_PARAM_ERANGE ;
			}
			
			break;
			

		default:
			DDA_LOG_ARG(DDA_LOG_ERR,
                        "%s: invalid command type %d\n", __FUNCTION__, 
                        access_list->cmd, 0,0,0,0 );
			ret = TIAP_PARAM_ERANGE ;
	}
			
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets 4x parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	_4x 	 - pointer to input 4x parameter structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_4x(initConfigStruct *init_cfg, tiap_param_4x_t *_4x)
{
	int ret = 0;

	if( !after_go )
	{
		if( (_4x->value == 0) || (_4x->value == 1) )
		{	
			switch( (tiap_param_4x_cmd_t)_4x->cmd )
			{
				case TIAP_PARAM_4X_FEATSET:
					init_cfg->FeatureSet = _4x->value;
					configured_params._4x_fs = 1;
					break;
				case TIAP_PARAM_4X_4X:
					init_cfg->enable4x = _4x->value;
					configured_params._4x_4x = 1;
					break;
				case TIAP_PARAM_4X_CAT:
					init_cfg->enableConcat = _4x->value;
					configured_params._4x_cat = 1;
					break;
				case TIAP_PARAM_4X_PKTBURST:
					configured_params._4x_pb = 1;
					break;
				case TIAP_PARAM_4X_MIXMOD:
					init_cfg->enable4xMixMode = _4x->value;
					configured_params._4x_mm = 1;
					break;
				default:
					DDA_LOG_ARG(DDA_LOG_ERR, 
                                "%s:  index :invalid parameter %d \n", __FUNCTION__,
                                _4x->cmd, 0,0,0,0);
					return TIAP_PARAM_ERANGE ;
			}
			
		}
		else {
			DDA_LOG_ARG(DDA_LOG_ERR, 
                        "%s: invalid parameter %d \n", __FUNCTION__,
                        _4x->value, 0,0,0,0);
			return TIAP_PARAM_ERANGE ;
		}
	}
	else
    {
       DDA_IOCTL_RUNTIME_ERR(ret);
    }
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets multiple SSID parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	multi_ssid - pointer to input multiple SSID parameter structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_multi_ssid(initConfigStruct *init_cfg, tiap_param_multi_ssid_t *multi_ssid)
{
	int ret = 0, i;

	if( !init_cfg || !multi_ssid )
		return EINVAL;

	if( !after_go ) {
		for (i=0; i< TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
			{
				memcpy(init_cfg->apMultiSsid[i].ssid.serviceSetId, multi_ssid->tbl[i].ssid,IW_ESSID_MAX_SIZE);
				init_cfg->apMultiSsid[i].ssid.hdr.eleLen = strlen(multi_ssid->tbl[i].ssid);
				init_cfg->apMultiSsid[i].ssidType = multi_ssid->tbl[i].ssid_type;
				init_cfg->apMultiSsid[i].vlanId = multi_ssid->tbl[i].vlan;
				/* Verify that primary SSID is configured */
				if (init_cfg->apMultiSsid[i].ssidType == TIAP_PARAM_SSID_PRIMARY) {
					configured_params.essid = 1;
				}
			}
			
			init_cfg->apMultipleSsidFlag  = multi_ssid->secondary_enabled;
			configured_params.multi_ssid = 1;
	}
	else
	{
		DDA_IOCTL_RUNTIME_ERR(ret);
	}
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets radio parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	radio    - pointer to input radio parameter structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
*/
/******************************************************************************/

static int param_radio(initConfigStruct *init_cfg, tiap_param_radio_t *radio)
{
int ret = 0;

	init_cfg->dbRadioParams.overrideBitmask = radio->mask;
	if( radio->tbl.type != TIAP_PARAM_PWRTBL ) {
		DDA_LOG_ARG(DDA_LOG_ERR, 
                    "%s: type: invalid parameter %d \n", __FUNCTION__,
                    radio->tbl.type, 0,0,0,0);
		return TIAP_PARAM_ERANGE ;
	}

	init_cfg->dbRadioParams.powerTable_paramType = radio->tbl.type;
	//TODO check range
	/*
	for(i=0; i<sizeof(radio->tbl.param); i++)
		if( radio->tbl.param[i] < -5 || radio->tbl.param[i] > 5 ) {
			printk(KERN_INFO "%s: tbl.param: invalid parameter %d \n", __FUNCTION__,radio->tbl.param[i]);
			return TIAP_PARAM_ERANGE ;
		}
	*/

	memcpy((void *)&init_cfg->dbRadioParams.powerTable_param, (void *)&radio->tbl.param, 
			sizeof(radio->tbl.param));

	/* set radio params according to the new configuration */
	if( after_go ) {
		wdrv_dbSetRadioParams((RadioParam_t *)&init_cfg->dbRadioParams, RADIO_PARAM_POWER_TABLE, 0);
		wdrv_dbSetRadioParams((RadioParam_t *)&init_cfg->dbRadioParams, RADIO_PARAM_OVERRIDE_BMASK, 0);
		whal_apiSetRadioParams();
	}

	configured_params.radio = 1;
		
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets channel range parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	ch_rang  - pointer to input channel range structure
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_ch_rang(initConfigStruct *init_cfg, tiap_param_channel_rang *ch_rang)
{
	int i, max_index, ret=0;


	if( !after_go )
	{
		if( ch_rang->type == TIAP_PARAM_RANG_A ) 
			max_index = TIAP_PARAM_MAX_NUM_OF_RANGES_A;
		else 
		{
			if ( ch_rang->type == TIAP_PARAM_RANG_G ) 
				max_index = TIAP_PARAM_MAX_NUM_OF_RANGES_G;
			else
			{
				DDA_IOCTL_INV_PAR_ERR(ret,ch_rang->type); 
				return ret;
			}
		}

		if (ch_rang->ranges_nr < max_index ) {
			for (i=0 ; i < ch_rang->ranges_nr; i++)
            {
                if( ch_rang->type == TIAP_PARAM_RANG_A )
                {
				if ( (ch_rang->rng.a[i].start > ch_rang->rng.a[i].stop) ||
				     (ch_rang->rng.a[i].power_limit < 0) ||
				     (ch_rang->rng.a[i].step < 1) ||
				     (ch_rang->rng.a[i].step > 15) )
				{
                        DDA_LOG_ARG(DDA_LOG_ERR, "%s: invalid parameter  \n", __FUNCTION__, 0, 0, 0, 0, 0);
                        return TIAP_PARAM_ERANGE ;
                    }
                }
                else
                {
                    if ( (ch_rang->rng.g[i].start > ch_rang->rng.g[i].stop) ||
                        (ch_rang->rng.g[i].power_limit < 0) ||
                        (ch_rang->rng.g[i].step < 1) ||
                        (ch_rang->rng.g[i].step > 15) )
                    {
                        DDA_LOG_ARG(DDA_LOG_ERR, "%s: invalid parameter  \n", __FUNCTION__, 0, 0, 0, 0, 0);
                        return TIAP_PARAM_ERANGE ;
                    }
                }
				}	
			
		 }
		 else {
			DDA_IOCTL_INV_PAR_ERR(ret,ch_rang->ranges_nr);
			return ret;
		 }	
		
         if( ch_rang->type == TIAP_PARAM_RANG_A ) {
			init_cfg->RegInit.numOfRanges_802_11_a = ch_rang->ranges_nr;
			memcpy((void *)&init_cfg->RegInit.privateRangeElements_a, 
					(void *)&ch_rang->rng.a, sizeof(ch_rang->rng.a));
			configured_params.ch_rang_a = 1;

		}
		else {
			init_cfg->RegInit.numOfRanges_802_11_b = ch_rang->ranges_nr;
			memcpy((void *)&init_cfg->RegInit.privateRangeElements_b, 
				       (void *)&ch_rang->rng.g, sizeof(ch_rang->rng.g));
			configured_params.ch_rang_g = 1;
		}

				
	}		
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets WONE parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	wone     - pointer to input WONE structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_wone(initConfigStruct *init_cfg, tiap_param_wone_t *wone)
{
	UINT8 index = wone->index;
	int  i,k, ret = 0;
	UINT8 cckRateValue[TIAP_PARAM_CCK_PBCC_RATES_MAX] = 
		{DOT11_RATE_1,DOT11_RATE_2,DOT11_RATE_5_5,DOT11_RATE_11,DOT11_RATE_22};
	UINT8 ofdmRateValue[TIAP_PARAM_OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };

	if( !after_go )
	{
		switch(wone->type)
		{
			case wone_oper_rate:
				if(wone->elem.b_legacy.len > 5) {
                    DDA_IOCTL_RANGE_ERR(ret,wone->elem.b_legacy.len);
					break;
				}
				dda_wext_convert_rates(b_legacy, apRates, cckRateValue);
				init_cfg->wone_info[index].apRates.hdr.eleLen = k;
				configured_params.oper_rate[index] = 1;
				break;

			case wone_oper_add:
				if(wone->elem.ofdm_legacy.len > 8) {
                    DDA_IOCTL_RANGE_ERR(ret, wone->elem.ofdm_legacy.len );
					ret = TIAP_PARAM_ERANGE;
					break;
				}
				dda_wext_convert_rates(ofdm_legacy, apAddRates, ofdmRateValue);
				init_cfg->wone_info[index].apAddRates.hdr.eleLen = k;
				configured_params.oper_add[index] = 1;
				break;

			case wone_oper_ofdm:
				if(wone->elem.ofdm_extend.len > 8) {
                    DDA_IOCTL_RANGE_ERR(ret, wone->elem.ofdm_extend.len);
					break;
				}
				dda_wext_convert_rates(ofdm_extend, apOFDMRates, ofdmRateValue);
				init_cfg->wone_info[index].apOFDMRates.hdr.eleLen = k;
				configured_params.oper_ofdm[index] = 1;
				break;

			case wone_brcst_rate:
				
				if (wone->elem.brcst_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.brcst_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						DDA_IOCTL_RANGE_ERR (ret,wone->elem.brcst_default_rate );
						break;
					}
				}
				
				init_cfg->wone_info[index].brsctDefActiveRate = wone->elem.brcst_default_rate;
				configured_params.brcst_rate[index] = 1;
				break;
			case wone_mcst_rate:
				
				if (wone->elem.mcst_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.mcst_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						DDA_IOCTL_RANGE_ERR (ret,wone->elem.mcst_default_rate );
						break;
					}
				}
				
				init_cfg->wone_info[index].mcstDefActiveRate = wone->elem.mcst_default_rate;
				configured_params.mcst_rate[index] = 1;
				break;
			case wone_tx_rate:
				
				if (wone->elem.tx_default_rate) 
				{
					/* check the range */
					for (i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++ ) 
					{
						if (cckRateValue[i] == wone->elem.tx_default_rate) 
						   break;
					}
					if (i == TIAP_PARAM_CCK_PBCC_RATES_MAX) 
					{
						DDA_IOCTL_RANGE_ERR (ret,wone->elem.tx_default_rate );
                        break;
					}
				}
				
				init_cfg->wone_info[index].genTxRate = wone->elem.tx_default_rate;
				configured_params.tx_rate[index] = 1;
				break;

			case wone_ch:
				init_cfg->wone_info[index].channel = wone->elem.ch;
				
				if (init_cfg->wone_info[index].Dot11Mode == TIAP_PARAM_D11MODE_A)
				{
					if( (init_cfg->wone_info[index].channel < 36) ||
						(init_cfg->wone_info[index].channel > 64) ||
						(init_cfg->wone_info[index].channel % 4) )
                    {
                       DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].channel);
                       break;
					}
				}
				else {
					if  (init_cfg->wone_info[index].channel > 14) {
							DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].channel); 
							break;
					}
				}
				configured_params.ch[index] = 1;
				break;

			case wone_band:
				init_cfg->wone_info[index].band = wone->elem.band;
				if((init_cfg->wone_info[index].band != RADIO_BAND_5GHZ) &&
					(init_cfg->wone_info[index].band != RADIO_BAND_2_4GHZ))
                {
                    DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].band );
					break;
				}
				configured_params.band[index] = 1;
				break;

			case wone_pbcc:
				init_cfg->wone_info[index].PbccAlgorithmMode = wone->elem.pbcc_mode;
				if(init_cfg->wone_info[index].PbccAlgorithmMode >
						TIAP_PARAM_PBCC_ALGO_ENHANCED_DYNAMIC) 
                {
                    DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].PbccAlgorithmMode );
					break;
				}
				configured_params.pbcc[index] = 1;
				break;

			case wone_rate_adapt:
				init_cfg->wone_info[index].RateAdaptation = wone->elem.rate_adapt;
				if(init_cfg->wone_info[index].RateAdaptation > 1) 
                {
					DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].RateAdaptation);
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.rate_adapt[index] = 1;
				break;

			case wone_pream_mode:
				init_cfg->wone_info[index].PreambleMode = wone->elem.pream_mode;
				if(init_cfg->wone_info[index].PreambleMode >
						TIAP_PARAM_PREAMBLE_MODE_LOCAL_STATIONS) 
                {
					DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].PreambleMode);
					ret = TIAP_PARAM_ERANGE;;
					break;
				}
				configured_params.pream_mode[index] = 1;
				break;

			case wone_slot_mode:
				init_cfg->wone_info[index].SlotTimeMode = wone->elem.slot_mode;
				if( init_cfg->wone_info[index].SlotTimeMode >
					TIAP_PARAM_SLOTTIME_MODE_ENHANCED_DYNAMIC) 
                {
					DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].SlotTimeMode);
					break;
				}
				configured_params.slot_mode[index] = 1;
				break;

			case wone_erp_mode:	
				init_cfg->wone_info[index].ERPProtectionMode = wone->elem.erp_mode;
				if(init_cfg->wone_info[index].ERPProtectionMode > 
						TIAP_PARAM_PROTECTION_MODE_ENHANCED_DYNAMIC) 
                {
					DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].ERPProtectionMode);
					break;
				}
				configured_params.erp_mode[index] = 1;
				break;

			case wone_erp_type:
				init_cfg->wone_info[index].ERPProtectionType = wone->elem.erp_type;
				if(init_cfg->wone_info[index].ERPProtectionType >
						TIAP_PARAM_PROTECTION_TYPE_CTS_TOSELF) 
                {
					DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].ERPProtectionType);
					break;
				}
				configured_params.erp_type[index] = 1;
				break;

			case wone_erp_ti:
				init_cfg->wone_info[index].ERPProtectionTI = wone->elem.erp_ti;
				if(init_cfg->wone_info[index].ERPProtectionTI > 1) 
                {
                    DDA_IOCTL_RANGE_ERR(ret,init_cfg->wone_info[index].ERPProtectionTI);
					break;
				}
				configured_params.erp_ti[index] = 1;
				break;

			case wone_dot11_mode:
				init_cfg->wone_info[index].Dot11Mode = wone->elem.dot11_mode;
				if(init_cfg->wone_info[index].Dot11Mode > DOT11_MODE_A_ONLY) 
                {
                    DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].Dot11Mode);   				
                    break;
				}
				configured_params.dot11_mode[index] = 1;
				break;

			case wone_cwmin_mode:
				init_cfg->wone_info[index].CwMinMode = wone->elem.cwmin_mode;
				if(init_cfg->wone_info[index].CwMinMode > DOT11_CWMIN_DOT11_MODE) 
                {
					DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].CwMinMode);
					break;
				}
				configured_params.cwmin_mode[index] = 1;
				break;

			case wone_usage_time:
				init_cfg->wone_info[index].UsageTime = wone->elem.usage_time;
				if((init_cfg->wone_info[index].UsageTime > 32) ||
					(init_cfg->wone_info[index].UsageTime < 8)) 
                {
                    DDA_IOCTL_RANGE_ERR(ret, init_cfg->wone_info[index].UsageTime);
                    break;
				}
				configured_params.usage_time[index] = 1;
				break;

			default:
				DDA_IOCTL_RANGE_ERR(ret, wone->type);
		}
	}
	else
    { 
        DDA_IOCTL_RUNTIME_PAR_ERR(ret, wone->type);
    }

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets private regulatory domain parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	priv_regdomain - pointer to input private regulatory domain structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if called after driver 
			has been started
*/
/******************************************************************************/

static int param_priv_regdomain(initConfigStruct *init_cfg, tiap_param_priv_reg_domain_t *priv_regdomain)
{
	int ret = 0;

	if( !after_go )
	{
		init_cfg->RegInit.dot11PrivateRegulatoryDomain = priv_regdomain->domain;
        init_cfg->RegInit.dot11PrivateRegulatoryBand = priv_regdomain->band;
		configured_params.priv_regdomain = 1;			   
	}
	else 
    {
        DDA_IOCTL_RUNTIME_ERR(ret);
    }

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets current private regulatory domain parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	priv_regdomain - pointer to input current private regulatory domain 
			structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_curr_reg_domain(initConfigStruct *init_cfg, tiap_param_cur_reg_domain_t *val)
{
	int ret = 0;

	if (!after_go)
    {
		if (val)
        {
			
			switch (val->type )
			{
			case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_IC:
			case TIAP_PARAM_RD_ETSI:
			case TIAP_PARAM_RD_SPAIN:
			case TIAP_PARAM_RD_FRANCE:
			case TIAP_PARAM_RD_MKK:
			case TIAP_PARAM_RD_MKK1:
            case TIAP_PARAM_RD_ISRAEL:
			case TIAP_PARAM_RD_US:
			case TIAP_PARAM_RD_WORLD:
            case TIAP_PARAM_RD_CUSTOM:
				 init_cfg->RegInit.dot11CurrentRegDomain = val->type;
				 configured_params.cur_regdomain = 1;
				break;
			default:
				DDA_IOCTL_RANGE_ERR(ret, val->type);
			}
		}
		else DDA_IOCTL_NULL_PTR_ERR(ret);
	}
	else 
		DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets station authentication list
	\param 	auth_tbl - pointer to otput station authentication list structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if driver 
			hasn't been started
*/
/******************************************************************************/

static int get_param_auth_tbl(tiap_param_auth_algo_t *auth_tbl)
{
int ret = 0;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret,after_go);
	else {
		BOOL temp_status; /*BOOL defined as u32, but 'enable' field as u8*/

		wdrv_dbGetAuthAlgo(0, &auth_tbl[0].algo);
		wdrv_dbGetAuthAlgoEnable(0, &temp_status);
		auth_tbl[0].enable = (u8)temp_status;
		wdrv_dbGetAuthAlgo(1, &auth_tbl[1].algo);
		wdrv_dbGetAuthAlgoEnable(1, &temp_status);
		auth_tbl[1].enable = (u8)temp_status;

	}
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets watch dog parameter
	\param 	wd - pointer to otput watch dog parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/

static int get_param_wd(tiap_param_wd_t *wd)
{
	int ret = 0;
	UINT8  a, b, c, e, f;
	UINT16 d;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret, wd->cmd);
	else
	{
		a = b = c = d = 0;
		switch( wd->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				wdrv_dbGetEnRestart(&wd->value.val8);
				break;
				
            case TIAP_PARAM_WD_INTERF:
				wdrv_dbGetEnInterfWA(&wd->value.val8);
				break;

			case TIAP_PARAM_WD_RXFREE:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val8 = a;
				break;

			case TIAP_PARAM_WD_BEACONS:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val8 = b;
				break;

			case TIAP_PARAM_WD_TXSTUCK:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val8 = c;
				break;

			case TIAP_PARAM_WD_FCSERR:
				wdrv_dbGetWAoptions(&a, &b, &c, &d, &e, &f);
				wd->value.val16 = d;
				break;

			case TIAP_PARAM_WD_ACI:
				 wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				 wd->value.val8 = e;

			case TIAP_PARAM_WD_NB:
				 wdrv_dbGetWAoptions(&a, &b, &c, &d,  &e, &f);
				 wd->value.val8 = f;	 

			default:
				DDA_IOCTL_INV_PAR_ERR(ret, wd->cmd);
		}
	}
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets watch dog interface parameter
	\param 	wd - pointer to otput watch dog parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/

static int get_param_interference_wd(tiap_param_interference_wd_t *interf_param)
{
	int ret = 0;

	if( !after_go)
	{
		DDA_IOCTL_INTI_ERR(ret,interf_param->cmd);
	}
	else {
		switch ( interf_param->cmd) {
		
			case TIAP_PARAM_ACI_TIMEOUT:
				wdrv_dbGetAciStateTimeout(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_NB_TIMEOUT:
				wdrv_dbGetNbStateTimeout(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_ACI_RATE_THR:
				wdrv_dbGetAciRateThreshold(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_ACI_EVENT_THR:
				wdrv_dbGetAciEventsThreshold(&(interf_param->value.val32));
				break;
			case TIAP_PARAM_PD_HIGH:
				wdrv_dbGetPacketDetectHighThresh(&(interf_param->value.val16));
				break;
			case TIAP_PARAM_PD_LOW:
				wdrv_dbGetPacketDetectLowThresh(&(interf_param->value.val16));
				break;
			case TIAP_PARAM_MEDIUM_BUSY_THR:
				wdrv_dbGetMediumBusyThresh(&(interf_param->value.val32));
				break;
			
			default:
				 DDA_IOCTL_INV_PAR_ERR(ret, interf_param->cmd);
		}
	}

	return ret;
}

static int get_param_hw_interf(tiap_param_hw_interf_t *interf_param)
{
	int ret = 0;

	if( !after_go )
	   DDA_IOCTL_INTI_ERR(ret,interf_param->cmd);
	else {
		HwInterfParams_T hw_interf_params;
		wdrv_dbGetHwInterfParams(&hw_interf_params);

		switch ( interf_param->cmd) {
		
			case TIAP_PARAM_INPUT_CCA:
				interf_param->value = hw_interf_params.inputCCA;
				break;
			case TIAP_PARAM_TX_CCA:
				interf_param->value = hw_interf_params.txCCA;
				break;
			case TIAP_PARAM_QUALIF_CCA:
				interf_param->value = hw_interf_params.qualifiedCCA;
				break;
			case TIAP_PARAM_STOMP_RX:
				interf_param->value = hw_interf_params.stompForRx;
				break;
			case TIAP_PARAM_STOMP_TX:
				interf_param->value = hw_interf_params.stompForTx;
				break;
			default:
				DDA_IOCTL_INV_PAR_ERR(ret, interf_param->cmd);
		}
	}

	return ret;
}


/******************************************************************************/

/*! \brief 	Gets station access list
	\param 	access_list - pointer to otput access list structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/


static int get_param_access_list(tiap_param_access_list_t *access_list)
{
	int ret = 0;
	mgmt_access_list_type_t mgmtType;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret, access_list->cmd);
	else {
		if (access_list->cmd == TIAP_PARAM_ACCESS_LST )
		{
			int n = sizeof(access_list->list);
			
			mgmtAccessGetEnabled(&access_list->enabled);
			mgmtAccessGetType( &mgmtType );
			access_list->type = (INT8) mgmtType;
			mgmtAccessGetList((char *)access_list->list, &n);
			
		}
		else
		{
            DDA_IOCTL_INV_PAR_ERR(ret, access_list->cmd );
		}
	}

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets 4x parameter structure
	\param 	_4x - pointer to otput 4x parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/

static int get_param_4x(tiap_param_4x_t *_4x)
{
int ret = 0;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret, _4x->cmd);
	else
	{
		BOOL temp_status;

        switch( _4x->cmd )
		{
			case TIAP_PARAM_4X_FEATSET:
				wdrv_dbGetFeatureSet(&_4x->value);
				break;
			case TIAP_PARAM_4X_4X:
				wdrv_dbGet4xEnable(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			case TIAP_PARAM_4X_CAT:
				wdrv_dbGetConcatEnable(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			case TIAP_PARAM_4X_PKTBURST:
				/*not supported*/
				break;
			case TIAP_PARAM_4X_MIXMOD:
				wdrv_dbGet4xMixMode(&temp_status);
				_4x->value = (u8)temp_status;
				break;
			default:
				DDA_IOCTL_INV_PAR_ERR(ret, _4x->cmd );
		}
	}

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets 4x multi SSID parameter structure
	\param 	multi_ssid - pointer to otput multi SSID parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/

static int get_param_multi_ssid(tiap_param_multi_ssid_t *multi_ssid)
{
	int ret = 0;

	if( !after_go )
    {
		DDA_IOCTL_INTI_ERR(ret, 0);
    }
	else 
    {
		int i;

		apMultiSsidElem_t apMultiSsid;
		for(i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
		{			
			wdrv_mgmt_ssidGetElementFromMultipleSsidActualTable(&apMultiSsid, i);
			memcpy(multi_ssid->tbl[i].ssid, apMultiSsid.ssid.serviceSetId, IW_ESSID_MAX_SIZE);
			multi_ssid->tbl[i].ssid[apMultiSsid.ssid.hdr.eleLen] = '\0';
			multi_ssid->tbl[i].ssid_type = apMultiSsid.ssidType;
			multi_ssid->tbl[i].vlan = apMultiSsid.vlanId;
		}
		wdrv_mgmt_ssidGetMultipleSsidStatus(&multi_ssid->secondary_enabled);
	}

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets radio parameter structure
	\param 	radio - pointer to otput radio parameter structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if driver hasn't 
			been started
*/
/******************************************************************************/

static int get_param_radio(tiap_param_radio_t *radio)
{
int ret = 0;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret, 0);
	else {
		RadioParam_t r;
		wdrv_dbGetRadioParams(&r);
		radio->mask = r.overrideBitmask;
		radio->tbl.type = r.powerTable_paramType;
		memcpy((void *)&radio->tbl.param, (void *)&r.powerTable_param, sizeof(radio->tbl.param));
	}

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets channel range parameter structure
	\param 	ch_rang - pointer to otput channel range parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid or 
			TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/

static int get_param_ch_rang(tiap_param_channel_rang *ch_rang)
{
int ret = 0;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret, ch_rang->type);
	else
	{
		int i;
		
		if( ch_rang->type != TIAP_PARAM_RANG_A && ch_rang->type != TIAP_PARAM_RANG_G ) {
			DDA_IOCTL_INV_PAR_ERR(ret,ch_rang->type);
			return ret;
		}

		wdrv_dbGetNumOfPrivateRanges(ch_rang->type, &ch_rang->ranges_nr);
		
		for(i=1; i<=ch_rang->ranges_nr; i++)
			if( ch_rang->type == TIAP_PARAM_RANG_A )
				/* Warning: if tChannelRangeElementStruct changes this won't work right ! */
				wdrv_dbGetPrivateRangeElement(ch_rang->type, i, (tChannelRangeElementStruct *)(&ch_rang->rng.a[i-1]));

			else 
				/* Warning: if tChannelRangeElementStruct changes this won't work right ! */
				wdrv_dbGetPrivateRangeElement(ch_rang->type, i, (tChannelRangeElementStruct *)(&ch_rang->rng.g[i-1]));
	}

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets WONE parameter structure
	\param 	wone - pointer to otput WONE parameter structure
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid or 
			TIAP_PARAM_ERUNTIME if driver hasn't been started
*/
/******************************************************************************/
 
static int get_param_wone(tiap_param_wone_t *wone)
{
	UINT8 index = wone->index;
	int   i,k, ret = 0;
	UINT8 cckRateValue[CCK_PBCC_RATES_MAX] = 
		{DOT11_RATE_1,DOT11_RATE_2,DOT11_RATE_5_5,DOT11_RATE_11,DOT11_RATE_22};
	UINT8 ofdmRateValue[OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };
    char extRates[OFDM_RATES_MAX];
    UINT8 extRatesLen;
    char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
    UINT8 ratesLen;
    UINT8 ratesCounter = 0;

	if( !after_go )
		DDA_IOCTL_INV_PAR_ERR(ret, wone->type);
	else
	{
		switch(wone->type)
		{
			/*TODO: Felix R.: implement functions for rates. */
			case wone_oper_rate:		
				wdrv_dbGetStationConfigOperationalRateSet(index, rates, &ratesLen);

				for(i=0; i<CCK_PBCC_RATES_MAX; i++)
				{
					wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<ratesLen; k++)
					{
						if(rates[k] == cckRateValue[i])
						{
							wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
                            ratesCounter++;
							break;
						}
						else if((rates[k] & 0x7F)== cckRateValue[i])
						{
							wone->elem.b_legacy.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
                            ratesCounter++;
							break;
						}
					}
				}
				wone->elem.b_legacy.len = ratesCounter;
				break;

			case wone_oper_add:
				wdrv_dbGetStationConfigOperationalRateSet(index, rates, &ratesLen);
                
				for(i=0; i<OFDM_RATES_MAX; i++)
				{
					wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<ratesLen; k++)
					{
						if(rates[k] == ofdmRateValue[i])
						{
							wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
                            ratesCounter++;
							break;
						}
						else if((rates[k] & 0x7F)== ofdmRateValue[i])
						{
							wone->elem.ofdm_legacy.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
                            ratesCounter++;
							break;
						}
					}
				}
				wone->elem.ofdm_legacy.len = ratesCounter;
				break;

			case wone_oper_ofdm:
				wdrv_dbGetStationConfigOperationalOFDMRateSet(index, extRates, &extRatesLen);

				for(i=0; i<OFDM_RATES_MAX; i++)
				{
					wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_NOT_SUPPORTED;
					for(k=0; k<extRatesLen; k++)
					{
						if(extRates[k] == ofdmRateValue[i])
						{
							wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_SUPPORTED;
							break;
						}
						else if((extRates[k] & 0x7F) == ofdmRateValue[i])
						{
							wone->elem.ofdm_extend.rates[i] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
							break;
						}
					}
				}
				wone->elem.ofdm_extend.len = extRatesLen;
				break;


			case wone_brcst_rate:
				wdrv_dbGetDefaultActiveRateBrcst(index, &wone->elem.brcst_default_rate);
				break;
			case wone_mcst_rate:
				wdrv_dbGetDefaultActiveRateMcst(index, &wone->elem.mcst_default_rate);
				break;
			case wone_tx_rate:
				wdrv_dbGetGenTxActiveRate(index, &wone->elem.tx_default_rate);
				break;
			case wone_ch:
				wdrv_dbGetChannel(index, &wone->elem.ch);
				break;

			case wone_band:
				wdrv_dbGetBand(index, &wone->elem.band);
				break;

			case wone_pbcc:
				wdrv_dbGetPBCCAlgorithmMode(index, &wone->elem.pbcc_mode);
				
				break;

			case wone_rate_adapt:
				wdrv_dbGetRateAdaptation(index, &wone->elem.rate_adapt);
				
				break;

			case wone_pream_mode:
				wdrv_dbGetPreambleMode(index, &wone->elem.pream_mode);
				
				break;

			case wone_slot_mode:
				wdrv_dbGetSlotTimeMode(index, &wone->elem.slot_mode);
				
				break;

			case wone_erp_mode:	
				wdrv_dbGetERPProtectionMode(index, &wone->elem.erp_mode);
				
				break;

			case wone_erp_type:
				wdrv_dbGetERPProtectionType(index, &wone->elem.erp_type);
				
				break;

			case wone_erp_ti:
				wdrv_dbGetERPProtectionTI(index, &wone->elem.erp_ti);
				
				break;

			case wone_dot11_mode:
				wdrv_dbGetDot11Mode(index, &wone->elem.dot11_mode);
				
				break;

			case wone_cwmin_mode:
				wdrv_dbGetCwMinMode(index, &wone->elem.cwmin_mode);
				
				break;

			case wone_usage_time:
				wdrv_dbGetUsageTime(index, &wone->elem.usage_time);
				
				break;

            default:
                DDA_IOCTL_RANGE_ERR(ret,wone->type); 
				return ret;
		}
	}
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets station table
	\param 	sta_tbl - pointer to otput station table structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if driver 
			has not been started
*/
/******************************************************************************/

static int get_param_sta_tbl(tiap_param_statbl_t *sta_tbl)
{
	int     ssid_len = 0, ret = 0, profile_index = 0;
	__u8	brcst_cipher;


	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret,0);
	else
	{
		UINT16 i;
		for(i=0; i<TIAP_PARAM_MAX_STATIONS; i++)
		{
			wdrv_dbGetStaMacAddress(i, sta_tbl[i].hwaddr);
			wdrv_dbGetStaSsidByIndex(i, sta_tbl[i].ssid, &ssid_len);
			sta_tbl[i].ssid[ssid_len] = '\0';
			wdrv_mgmtGetState(i, &sta_tbl[i].state);
			/*removed since it returns max active rate,
			 i.e. not updated after rate adaptation*/
			/*wdrv_dbGetActiveRate(i, &sta_tbl[i].active_rate);*/
			sta_tbl[i].active_rate = 
				convertAcxRateToDOT11Rate(tx_getStaActiveRate(i));
			wdrv_dbGetStationTxPacketCounter(i, &(sta_tbl[i].txCounter));
			wdrv_dbGetStationRxPacketCounter(i, &sta_tbl[i].rxCounter);
			sta_tbl[i].wme = wdrv_dbGetStaWME(i);

			wdrv_mgmt_ssidGetStationSsidIndex(i, &profile_index);
			wdrv_mgmt_ssidGetBroadcastSuiteType(&brcst_cipher, profile_index);
			if (brcst_cipher == CIPHER_WEP) 
			{
				__u16	key_len;
				__u8    dummy[TIAP_WPA_WEP_KEY_LEN_MAX];
				
				wdrv_dbGetDefaultWepKey(0, &key_len, dummy);
				switch(key_len) 
				{
				case TIAP_WPA_WEP_40_KEY_LEN:
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_40;
					break;
				case TIAP_WPA_WEP_104_KEY_LEN :
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_104;
					break;
				case TIAP_WPA_WEP_232_KEY_LEN :
					sta_tbl[i].cipher = TIAP_WPA_CIPHER_WEP_232;
					break;
				default:
				}
			}
			else
			{
				wdrv_dbGetStationSuiteType(i, &sta_tbl[i].cipher);
			}
		}

	}
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets private regulatory domain parameter
	\param 	priv_regdom - pointer to otput private regulatory domain structure
	\return status of operation OK or EINVAL parameter is null pointer 
*/
/******************************************************************************/

static int get_param_priv_regdomain(tiap_param_priv_reg_domain_t *priv_regdom)
{
    int ret = 0;

	if (priv_regdom)
	{
		wdrv_dbGetPrivateRegulatoryDomain(&priv_regdom->domain, priv_regdom->band);
	}
	else
    {
        DDA_IOCTL_NULL_PTR_ERR(ret);
    }
	
	return ret;	
}				

/******************************************************************************/

/*! \brief 	Gets current regulatory domain parameter structure
	\param 	val - pointer to otput current regulatory domain structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if driver 
			has not been started
*/
/******************************************************************************/

static int get_param_curr_reg_domain(tiap_param_cur_reg_domain_t *val)
{
    int ret = 0;
	UINT8 regDomain;

	if (val)
	{
		wdrv_dbGetdot11CurrentRegDomain(val->band, &regDomain);
		val->type = regDomain;
	}
	else
    {
        DDA_IOCTL_NULL_PTR_ERR(ret);
    }
	
	return 0;	

}

/******************************************************************************/

/*! \brief 	Gets driver statistics structure
	\param 	val - pointer to otput driver statistics structure
	\return status of operation OK or TIAP_PARAM_ERUNTIME if driver 
			has not been started
*/
/******************************************************************************/

static int get_param_stats(tiap_param_net_stats *stats)
{
	int ret = 0;

	if( !after_go )
		DDA_IOCTL_INTI_ERR(ret,0);
	else {
		/* 
		 * bss statistics
		 */
		/*tx*/
		wdrv_bssBridgeGetLanSendUnicast(&stats->bss.tx_frames);
		wdrv_bssBridgeGetLanSendMulticast(&stats->bss.tx_multicast_frames);
		/*rx*/
		wdrv_bssBridgeGetLanRecvUnicast(&stats->bss.rx_frames);
		wdrv_bssBridgeGetLanRecvMulticast(&stats->bss.rx_multicast_frames);

		/* 
		 * wireless statistics
		 */
		/*tx*/
		wdrv_dbGetCountersTransmittedFragmentCount(&stats->wireless.tx_mpdus);
		wdrv_dbGetCountersTransmittedFrameCount(&stats->wireless.tx_msdus);
		wdrv_dbGetCountersMulticastTransmittedFrameCount(&stats->wireless.tx_multicast_msdus);
		wdrv_dbGetCountersFailedCount(&stats->wireless.tx_failed_msdus);
		wdrv_dbGetCountersRetryCount(&stats->wireless.tx_retry_msdus);

		/*rx*/
		wdrv_dbGetCountersReceivedFragmentCount(&stats->wireless.rx_mpdus);
		wdrv_rxGetRxMsduReadyCount(&stats->wireless.rx_msdus);
		wdrv_dbGetCountersMulticastReceivedFrameCount(&stats->wireless.rx_multicast_msdus);
		wdrv_dbGetCountersFCSErrorCount(&stats->wireless.rx_fcs_error_mpdus);
		wdrv_dbGetPrivacyTKIPMicFailure(&stats->wireless.rx_mic_failure_msdus);
		wdrv_dbGetPrivacyWEPICVErrorCount(&stats->wireless.rx_decrypt_error_mpdus);
	}
	
	return ret;
}


/******************************************************************************/

/*! \brief 	Sets report level parameter
	\param 	value - input report value to set
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid
*/
/******************************************************************************/

static int tiap_param_set_replev(int value)
{
 int ret = 0;

	if (value>=TIAP_PARAM_MIN_REPLEV && value<=TIAP_PARAM_MAX_REPLEV)
	{
		if (value > 0)
			PALSRV_report_set(value,0);
		else if (value < 0)
			PALSRV_report_unset(value * (-1),0);
		else
			PALSRV_report_disable(0);

		configured_params.replev = 1;
	}
	else 
    {
		DDA_IOCTL_RANGE_ERR(ret,value);
	}
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets report level parameter
	\param 	value - input report value to set
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid
*/
/******************************************************************************/

static int tiap_param_get_replev(int *val)
{
    int ret = 0;
	
    if (val)
    {
		*val = PALSRV_report_get();
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets beacon period parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value - input beacon period value to set
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_beaconperiod(initConfigStruct *init_cfg, UINT16 value)
{
	int ret = 0;

	if (!after_go)
	{
		if (value > 0)
        {
			init_cfg->apBeaconPeriod = value;
			configured_params.beacon_period = 1;
		}
		else 
        {
            DDA_IOCTL_RANGE_ERR( ret, value );
        }
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);


	return ret;
}

/******************************************************************************/

/*! \brief 	Gets beacon period parameter
	\param 	value - pointer to output beacon period
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_beaconperiod(int *val)
{
	int ret = 0;
	UINT16 beacPeriod;
	
    if (val)
	{
		wdrv_dbGetStationConfigBeaconPeriod( &beacPeriod);
		* val = (int) beacPeriod;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets delivary traffic indication period parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value - input delivary traffic indication period value to set
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_dtim_period(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
	{
		if (value >= TIAP_PARAM_MIN_DTIM_PERIOD && value <= TIAP_PARAM_MAX_DTIM_PERIOD)
        {
			init_cfg->apDtimPeriod = value;
			configured_params.dtim_period = 1;
		}
		else 
        {
			DDA_IOCTL_RANGE_ERR( ret, value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets delivary traffic indication period parameter
	\param 	value - pointer to output delivary traffic indication period
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_dtim_period(int *val)
{
	int ret = 0;
	UINT8 dtimPeriod;

    if (val)
	{
		memset(val, 0, sizeof(*val));
		wdrv_dbGetStationConfigDtimPeriod ( &dtimPeriod);
		* val = (int) dtimPeriod;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets BSS bridge state
	\param 	init_cfg - pointer to initialization data structure
	\param 	value - input BSS bridge state value to set
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
*/
/******************************************************************************/

static int tiap_param_set_bss_bridge_state(initConfigStruct *init_cfg, tiap_param_bss_br_state_t value)
{
	UINT32 bss_bridge_state;
	int ret = 0;

	switch (value)
	{
		case TIAP_PARAM_BSS_BR_ENABLE:
		case TIAP_PARAM_BSS_BR_DROP  :
			bss_bridge_state = BSS_TRAFFIC_EN;
			break;
		case TIAP_PARAM_BSS_BR_BLOCK :
			bss_bridge_state = BSS_TRAFFIC_BLOCK;
			break;
		default:
			DDA_IOCTL_RANGE_ERR(ret,value);
            return ret;
	}

	if (!after_go) 
    {
		init_cfg->bssBridgeState = bss_bridge_state; 
		configured_params.bss_bridge_state = 1;
	}
	else
	{
		wdrv_dbSetBssIsolationState(bss_bridge_state);
	}
    return ret;
}

/******************************************************************************/

/*! \brief 	Gets BSS bridge state parameter
	\param 	val - pointer to output BSS bridge state parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_bss_bridge_state(int *val)
{
	int ret = 0;
	UINT32 bss_bridge_state;

	if (val)
	{
		bss_bridge_state = init_cfg.bssBridgeState;		
		if (bss_bridge_state == BSS_TRAFFIC_BLOCK)
			*val = TIAP_PARAM_BSS_BR_BLOCK;
		else
			*val = TIAP_PARAM_BSS_BR_ENABLE;
			

	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets rate fallback parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value - input rate fallback value to set
	\return status of operation OK or TIAP_PARAM_ERUNTIME if called after driver 
			has been started
*/
/******************************************************************************/

static int tiap_param_set_rate_fallback(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
	{
		init_cfg->RateFallback = value;
		configured_params.rate_fallback = 1;
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets rate fallback parameter
	\param 	val - pointer to output rate fallback parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_rate_fallback(int *val)
{
	int ret = 0;
	UINT8 rateFallback;

	if (val)
	{
		wdrv_dbGetRateFallback( &rateFallback );
		* val = (int) rateFallback;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets radio calibraton parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value - input radio calibraton value to set
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_radio_cal(initConfigStruct *init_cfg, UINT32 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value <= TIAP_PARAM_MAX_RADIO_CAL){
			init_cfg->RadioCalibrateOption = value;
			configured_params.radio_cal = 1;
        }
		else{
			DDA_IOCTL_RANGE_ERR(ret, value);
			return ret;
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets radio callibration parameter
	\param 	val - pointer to radio calibration parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_radio_cal(int *val)
{
	int ret = 0;
	
    if (val) {
		wdrv_dbGetRadioCalibrateOption (val);
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets radio calibraton interval parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input radio calibraton interval value to set
	\return status of operation OK or TIAP_PARAM_ERUNTIME if called after 
			driver has been started
*/
/******************************************************************************/

static int tiap_param_set_radio_cal_int(initConfigStruct *init_cfg, UINT16 value)
{
	int ret = 0;

	if (!after_go){
		init_cfg->RadioCalibrateInterval = value;
		configured_params.cal_interval = 1;
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret ;
	
}

/******************************************************************************/

/*! \brief 	Gets radio callibration interval parameter
	\param 	val - pointer to radio calibration interval parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_radio_cal_int(int *val)
{
	int ret = 0;
	
    if (val){
		wdrv_dbGetRadioCalibrateInterval((UINT16 *)val);
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;

}

/******************************************************************************/

/*! \brief 	Gets radar detection mode
	\param 	val - pointer to radar detection mode parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_radar_detect_mode(int *val)
{
	int ret = 0;
	UINT8 mode;
	
    if (val){
		wdrv_dbGetRadarDetectMode( &mode );
		* val = (int) mode;
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets bridge operation mode
	\param 	val - pointer to bridge operation mode parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_bridge_mode(int *val)
{
	int ret = 0;
	
    if (val)
		*val = wdrv_dbGetApBridgeMode();
	else 
        DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}
/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Enables/disables WDS
	
	\param init_cfg		init configuration structure
	\param val		input value
	
	RETURNS: 0 in case of success, error code otherwise
*/
static int tiap_param_set_wds_enable(initConfigStruct *init_cfg, __u8 val)
{
	int ret = 0;

    if (!val || val == 1) {
        init_cfg->wdsEnable = val;
        configured_params.wdsEnable = 1;
        wdrv_dbSetWdsEnable(val);
    }
    else {
        DDA_IOCTL_RANGE_ERR(ret, val);
    }
	return ret;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Gets wds mode  
	
	\param val		output buffer
	
	RETURNS: 0 in case of success, error code otherwise
*/
static int tiap_param_get_wds_enable(int *val)
{
	if (val)
		*val = wdrv_dbGetWdsEnable();
	
	return 0;
}

/******************************************************************************/

/*! \brief 	Sets energy detection Enable/Disable parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input energy detection Enable/Disable value to set
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/


static int tiap_param_set_energy_detect(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value == 0 || value == 1)
        {
			init_cfg->enableEnergyDetect = value;
			configured_params.energy_detect = 1;
		}
		else 
        {
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets energy detection status
	\param 	val - pointer to energy detection status parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_energy_detect(int *val)
{
	int ret = 0;
	UINT8 energDet;
	
    if (val) 
    {
		wdrv_dbGetEnergyDetect ( &energDet );
		* val = (int) energDet;
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
	
}
/******************************************************************************/

/*! \brief 	Gets eeprom less suport
	\param 	val - pointer to eeprom less suport parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_eeprom_less(int *val)
{
	int ret = 0;
	UINT8 eepromLess;
	
    if (val) 
    {
		wdrv_dbGetEepromLess ( &eepromLess );
		* val = (int) eepromLess;
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
	
}
/******************************************************************************/

/*! \brief 	Gets eeprom less suport
	\param 	val - pointer to eeprom less suport parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_set_eeprom_less(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value == 0 || value == 1)
        {
			init_cfg->enableEEPromLessSupport = value;
			configured_params.eeprom_less = 1;
		}
		else 
        {
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Sets hidden ssid status
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input hidden ssid Enable(1)/Disable(0) value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_hidden_ssid(initConfigStruct *init_cfg, UINT16 value)
{
	int ret = 0;

	if (!after_go)
    {
		if ( value == 0 || value == 1){
			init_cfg->apHiddenSSID = value;
			configured_params.hidden_ssid = 1;
		}
		else 
        {
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
	
}

/******************************************************************************/

/*! \brief 	Gets hidden ssid status
	\param 	val - pointer to hidden ssid status parameter, Enable(1)/Disable(0)
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_hidden_ssid(int *val)
{
	int ret = 0;
	UINT16 hiddedSsid;

	if (val)
	{
		wdrv_mgmt_ssidGetHiddenSsidStatus( &hiddedSsid );
		* val = (int)hiddedSsid; 
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;
}

/******************************************************************************/

/*! \brief 	Sets multi domain capability status
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input multi domain capability Enable (1) /Disable (0) value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_multidomain_cap(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if ( value == 0 || value == 1)
        {
			init_cfg->RegInit.dot11MultiDomainCapabilityEnable = value;
			configured_params.multi_domain_cap = 1;
		}
		else 
        {	
			DDA_IOCTL_RANGE_ERR(ret, value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets multi domain capability status
	\param 	val - pointer to multi domain capability status parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_multidomain_cap(int *val)
{
	int ret = 0;
	UINT8 capab;

	if (val)
	{
		wdrv_dbGetdot11MultiDomainCapabilityStatus( &capab );
		* val = (int) capab;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets spectrum management status
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input spectrum management status, Enable(1)/Disable(0)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_spectrum_mgmt(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value == 0 || value == 1)
        {
			init_cfg->RegInit.dot11SpectrumManagementRequired = value;
			configured_params.spectrum_mgmt = 1;
		}
		else	
			DDA_IOCTL_RANGE_ERR(ret, value);
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets spectrum management status
	\param 	val - pointer to spectrum management status
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_spectrum_mgmt(int *val)
{
	int ret = 0;
	UINT8 spectr;

	if (val)
	{
		wdrv_dbGetdot11SpectrumManagementRequiredStatus( &spectr );
		* val = (int) spectr;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets legacy station status
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input legacy station status, Enable(1)/Disable(0)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_legacy_sta(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go){
		if (value == 0 || value == 1){
			init_cfg->RegInit.whalEnableLegacyStations = value;
			configured_params.legacy_sta = 1;
		}
		else {
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets legacy station status
	\param 	val - pointer to legacy station status
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_legacy_sta(int *val)
{
	int ret = 0;
	UINT8 legacy;

    if (val)
	{
		wdrv_dbGetLegacyStatusEnableStatus( &legacy );
		* val = (int) legacy;

	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets local power constraint parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input ocal power constraint value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_local_power_cons(initConfigStruct *init_cfg, __s8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value >= TIAP_PARAM_MIN_POWER_CONS &&
		    value <= TIAP_PARAM_MAX_POWER_CONS){
			init_cfg->RegInit.dot11LocalPowerConstraint = value;
			configured_params.local_pow_cons = 1;
		}
		else	
			DDA_IOCTL_RANGE_ERR(ret, value);
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets local power constraint parameter
	\param 	val - pointer to local power constraint parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_local_power_cons(int *val)
{
	int ret = 0;
	__s8 power;

	if (val)
	{
		wdrv_dbGetdot11LocalPowerConstraint( &power);
		* val = (int) power;
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets maximum transmitting power level parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	value    - input maximum transmitting power level value
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_max_tx_pow_level(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value >= TIAP_PARAM_MIN_TX_POW_LEVEL &&
				value <= TIAP_PARAM_MAX_TX_POW_LEVEL )
        {
			init_cfg->RegInit.whalMaxTxPowerLevel = value;
			configured_params.max_txpow_lev = 1;
		}
		else 
        {
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets lmaximum transmitting power level parameter
	\param 	val - pointer to maximum transmitting power level parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_max_tx_pow_level(int *val)
{
	int ret = 0;
	__s8 power;

	if (val)
	{
		wdrv_dbGetMaxTxPowerLevel( &power );
		* val = (int) power;
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets country sting parameter
	\param 	init_cfg - pointer to initialization data structure
	\param 	country  - pointer to input country string
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int param_set_country_str(initConfigStruct *init_cfg, tiap_param_country_t *country)
{
	int ret = 0;

	/*Warning: value is NOT null-terminated*/

	if (!after_go){
		switch (country->name[2])
		{
			case 'i':
			case 'o':
			case 'I':
			case 'O':
			case ' ':
				memset(init_cfg->RegInit.dot11CountryString, 0, 
					   TIAP_PARAM_COUNTRY_STRING_LEN);
				memcpy(init_cfg->RegInit.dot11CountryString, country->name, 
					   TIAP_PARAM_COUNTRY_STRING_LEN);
				configured_params.country_str = 1;
				break;
			default:
				DDA_IOCTL_RANGE_ERR(ret, country->name);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

    return ret;	
}

/******************************************************************************/

/*! \brief 	Gets country sting parameter
	\param 	country - pointer to country sting parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int get_param_country_str( tiap_param_country_t *country)
{
	int ret = 0;
    
    if (country->name)
    {	
    	memset(country->name, 0, TIAP_PARAM_COUNTRY_STRING_LEN);
    	wdrv_dbGetdot11CountryString( (INT8 *) country->name );	
    }
    else 
	    DDA_IOCTL_NULL_PTR_ERR(ret);
	
    return ret;	
}

/******************************************************************************/

/*! \brief 	Sets WLAN driver operation state
	\param 	init_cfg - pointer to initialization data structure
	\param 	value  - input WLAN driver operation state Enable(1)/Disable(0)
	\return status of operation OK or TIAP_PARAM_ERANGE if parameter is invalid 
*/
/******************************************************************************/

static int tiap_param_set_oper_state(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;
	
    if (value == 0 || value == 1)
		if (!after_go)
        {
			init_cfg->WlanServiceState = value;
			configured_params.oper_state = 1;
		}
		else
        {
			if(value == 0)
				wdrv_WlanServiceDisable();
			else
				wdrv_WlanServiceEnable();
		}		
	else
	{   
		DDA_IOCTL_RANGE_ERR(ret,value);
	}
	return ret;
}

/******************************************************************************/

/*! \brief 	Gets WLAN driver operation state
	\param 	val - pointer to WLAN driver operation state parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_oper_state(int *val)
{
	int ret = 0;

	if (val)
	{
		wdrv_dbGetWlanServiceState(val);
	}
	else DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	

}

/******************************************************************************/

/*! \brief 	Sets number of WONE interfaces
	\param 	init_cfg - pointer to initialization data structure
	\param 	value  - input number of WONE interfaces (1 or 2)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_wone_num(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value == 1 || value == 2)
        {                                   /* 1- single band, 2-dual band*/
			init_cfg->numSupWone = value;
			configured_params.wone_num = 1;
		}
		else
		{   
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}


/******************************************************************************/

/*! \brief 	Gets number of WONE interfaces
	\param 	val - pointer to number of WONE interfaces parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_wone_num(int *val)
{
	int ret = 0;
	UINT8 woneN;

	if (val)
	{
		wdrv_dbGetNumSupWone( &woneN );
		* val =   (int) woneN;
	}
	else 
       DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	
}

/******************************************************************************/

/*! \brief 	Sets time mode status
	\param 	init_cfg - pointer to initialization data structure
	\param 	value  - input time mode Enable(1)/Disable(0)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_wone_timemod(initConfigStruct *init_cfg, UINT8 value)
{
	int ret = 0;

	if (!after_go)
    {
		if (value == 0 || value == 1)
        {                                           /* 0- short, 1-long*/
			init_cfg->usageTimeMode = value;
			configured_params.wone_timemod = 1;
		}
		else
		{   
			DDA_IOCTL_RANGE_ERR(ret,value);
		}
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Gets time mode status
	\param 	val - pointer to time mode status parameter
	\return status of operation OK or EINVAL if parameter is null pointer
*/
/******************************************************************************/

static int tiap_param_get_wone_timemod(int *val)
{
	int ret = 0;
	UINT8 mode;

	if (val)
	{
		wdrv_dbGetUsageTimeMode( &mode );
		* val = (int) mode;
	}
	else 
		DDA_IOCTL_NULL_PTR_ERR(ret);
	
	return ret;	
}

/******************************************************************************/

/*! \brief 	Sets radar detection mode
	\param 	init_cfg - pointer to initialization data structure
	\param 	value  - input radar detection mode Enable(1)/Disable(0)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_radar_detect_mode(initConfigStruct *init_cfg, UINT8 val)
{
	int ret = 0;

	if (!after_go) 
    {
		if (!val || val == 1) {
			init_cfg->RadarDetectMode = val;
			configured_params.radar_detect = 1;
		}
		else DDA_IOCTL_RANGE_ERR(ret,val);
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;


}

/******************************************************************************/

/*! \brief 	Sets bridge mode
	\param 	init_cfg - pointer to initialization data structure
	\param 	value  - input bridge mode Enable(1)/Disable(0)
	\return status of operation OK, TIAP_PARAM_ERANGE if parameter is invalid 
			or TIAP_PARAM_ERUNTIME if called after driver has been started
*/
/******************************************************************************/

static int tiap_param_set_bridge_mode(initConfigStruct *init_cfg, UINT8 val)
{
	int ret = 0;

	if (!after_go) 
    {
		if (!val || val == 1) 
        {
			init_cfg->ApBridgeMode = val;
			configured_params.bridge_mode = 1;
		}
		else DDA_IOCTL_RANGE_ERR(ret,val);
	}
	else DDA_IOCTL_RUNTIME_ERR(ret);

	return ret;
}

/******************************************************************************/

/*! \brief 	Performs Go IOCTL command
	\param 	pIoctl - pointer to IOCTL data
	\param 	cmd    - IOCTL command value
	\return status of operation OK, ERROR or TIAP_PARAM_ERxxx error code
*/
/******************************************************************************/


static int doGoIoctl( DDA_IOCTL_GO_PAR_S  * pIoctl, int cmd )
{
 int ret = 0, state;

 memcpy(init_cfg.APMAC, pIoctl->dev_addr, ETH_ALEN);
 configured_params.hwaddr = 1;

 if (!check_if_all_configured()) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR,"Configuration is not completed\n",__FUNCTION__, 0,0,0,0,0 );
     return TIAP_PARAM_EPARAMNOTSET;
 }
		
 ret = tiwlan_dda_waitForWpa();
 if(ret != OK)
 {
    ret = TIAP_PARAM_ELOGICAL;
    goto out;
 }

 ret = wdrv_init(&init_cfg);

 switch (ret)
 {
    case DDC_INIT_SUCCESS:		
        ret = 0;
        break;
    case DDC_INIT_CFG_RANGE_ERROR:
        ret = TIAP_PARAM_ERANGE;
        goto out;
        break;
    case DDC_INIT_CFG_LOGICAL_ERROR:
        ret = TIAP_PARAM_ELOGICAL;
        goto out;
        break;
    case DDC_INIT_RESOURCE_ERROR:
        ret = TIAP_PARAM_ERESOURCE;
        goto out;
        break;
    case DDC_INIT_HW_INIT_ERROR:
        ret = TIAP_PARAM_EHWINIT;
        goto out;
        break;
    default: 
        ret = -1;
        goto out;

 }

 tiwlan_dda_wpaGo();

 /* we have to wait until INIT task finishes all its prints, since otherwise
    Dda task is blocked on logMsg and it doesn't handle its events */
 PAL_sleepLong(4000);
 tiwlan_ddaStart();
 wdrv_start();
 WEND_Start();   /* enable packet forwarding to/from local bridge via WLAN END */ 

 wdrv_dbGetWlanServiceState(&state);

 if (state)
 {
	   TIWLAN_LED_LINK_UP;
 }
   

 after_go=1;

out:
 printf("%s: initializing %s ret = %d\n", __FUNCTION__, 
		(!ret ? "succeeded" : "failed"), ret );
 return ret;
}

/******************************************************************************/

/*! \brief 	Gets WLAN driver MAC address
	\return pointer to WLAN driver MAC address
*/
/******************************************************************************/

char *get_mac_addr(void)
{
    return (char *)init_cfg.APMAC;
}
		   
/******************************************************************************/

/*! \brief 	Checks whether all configuration parameters are set before driver start
	\return pointer OK or 1 if configuration is not completed
*/
/******************************************************************************/

	int check_if_all_configured(void)
{
	int i;

	if(configured_params.hwaddr != 1)
		return 0;
	
	if(configured_params.essid != 1)
		return 0;

	if(configured_params.rts != 1)
		return 0;

	if(configured_params.frag != 1)
		return 0;
	
	if(configured_params.retry_long != 1)
		return 0;
	
	if(configured_params.retry_short != 1)
		return 0;

	if(configured_params.lifetime_tx != 1)
		return 0;
	
	if(configured_params.lifetime_rx != 1)
		return 0;

	if(configured_params.txpow_a != 1)
		return 0;

	if(configured_params.txpow_g != 1)
		return 0;
	if(configured_params.replev != 1)
		return 0;

	if(configured_params.beacon_period != 1)
		return 0;

	if(configured_params.dtim_period != 1)
		return 0;
	if(configured_params.bss_bridge_state != 1)
		return 0;
	if(configured_params.rate_fallback != 1)
		return 0;
	if(configured_params.radio_cal != 1)
		return 0;
	if(configured_params.cal_interval != 1)
		return 0;
	if(configured_params.energy_detect != 1)
		return 0;
	if(configured_params.hidden_ssid != 1)
		return 0;
	if(configured_params.multi_domain_cap != 1)
		return 0;
	if(configured_params.spectrum_mgmt != 1)
		return 0;
	if(configured_params.legacy_sta != 1)
		return 0;
	if(configured_params.local_pow_cons != 1)
		return 0;
	if(configured_params.max_txpow_lev != 1)
		return 0;
	if(configured_params.country_str != 1)
		return 0;
	if(configured_params.oper_state != 1)
		return 0;
	if(configured_params.wone_num != 1)
		return 0;
	if(configured_params.wone_timemod != 1)
		return 0;
	if(configured_params.auth_tbl != 1)
		return 0;
	if(configured_params.wd_en != 1)
		return 0;
    if(configured_params.wd_interf != 1)
		return 0;
	if(configured_params.wd_rxfree != 1)
		return 0;
	if(configured_params.wd_beacons != 1)
		return 0;
	if(configured_params.wd_fcserrs != 1)
		return 0;
	if(configured_params.wd_txstuck != 1)
		return 0;
	if(configured_params.wd_aci != 1)
		return 0;
	if(configured_params.wd_nb != 1)
		return 0;
	if (configured_params.aci_state_timeout != 1)
		return 0;
	if (configured_params.nb_state_timeout != 1)
		return 0;
	if (configured_params.aci_rate_thresh != 1) 
		return 0;
	if (configured_params.pd_low_thresh != 1) 
		return 0;
	if (configured_params.pd_high_thresh != 1) 
		return 0;
	if (configured_params.medium_busy_thresh != 1) 
		return 0;
	if(configured_params.access_list != 1)
		return 0;
	if(configured_params._4x_fs != 1)
		return 0;
	if(configured_params._4x_4x != 1)
		return 0;
	if(configured_params._4x_cat != 1)
		return 0;
	if(configured_params._4x_pb != 1)
		return 0;
	if(configured_params._4x_mm != 1)
		return 0;
	if(configured_params.multi_ssid != 1)
		return 0;
	if(configured_params.radio != 1)
		return 0;
	if(configured_params.ch_rang_a != 1)
		return 0;
	if(configured_params.ch_rang_g != 1)
		return 0;
	if(configured_params.radar_detect != 1)
		return 0;
	if(configured_params.bridge_mode != 1)
		return 0;

	for (i=0; i<TIAP_PARAM_WONE_DUAL_BAND; i++)
	{
		if(configured_params.oper_rate[i] != 1)
			return 0;
		if(configured_params.oper_add[i] != 1)
			return 0;
		if(configured_params.oper_ofdm[i] != 1)
			return 0;
        if(configured_params.brcst_rate[i]  != 1)
			return 0;
		if(configured_params.mcst_rate[i]  != 1)
			return 0;
		if(configured_params.tx_rate[i]  != 1)
			return 0;
		if(configured_params.ch[i] != 1)
			return 0;
		if(configured_params.band[i] != 1)
			return 0;
		if(configured_params.pbcc[i] != 1)
			return 0;
		if(configured_params.rate_adapt[i] != 1)
			return 0;
		if(configured_params.pream_mode[i] != 1)
			return 0;
		if(configured_params.slot_mode[i] != 1)
			return 0;
		if(configured_params.erp_mode[i] != 1)
			return 0;
		if(configured_params.erp_type[i] != 1)
			return 0;
		
		if(configured_params.erp_ti[i] != 1)
			return 0;
		if(configured_params.dot11_mode[i] != 1)
			return 0;
		if(configured_params.cwmin_mode[i] != 1)
			return 0;
		if(configured_params.usage_time[i] != 1)
			return 0;
	}
	if(configured_params.priv_regdomain != 1)
		return 0;
	if(configured_params.cur_regdomain != 1)
		return 0;
	if(configured_params.wdsEnable != 1)
		return 0;

    if(configured_params.hw_input_cca != 1)
        return 0;
	if(configured_params.hw_qualif_cca != 1)
        return 0;
	if(configured_params.hw_tx_cca != 1)
        return 0;
	if(configured_params.hw_stomp_rx != 1)
        return 0;
	if(configured_params.hw_stomp_tx != 1)
		return 0;

	return 1;
	
}

/******************************************************************************/

/*! \brief 	Reads and formats radio parameters or specified WONE interface
	\param	page - WONE interface number
	\param	page - pointer to buffer for formated output
	\return number of output bytes
*/
/******************************************************************************/


static int read_wlan_params(UINT8 currentConfigWoneInex, char *page)
{
	int i , len = 0;
	unsigned char  macaddr[ETH_ALEN], ssid[50], bssid[ETH_ALEN];
	UINT8 shortRetryLimit, longRetryLimit, dtim, channel, defaultActiveRate, 
		  defaultActiveRateBrcst, defaultActiveRateMcst, genTxRate;
	UINT8 RateFallback, RateAdaptation;
	UINT16 capabilities, beacon, rtsThreshold, fragThreshold, stationsNotPBCCCapabale;
	UINT32 transmitLifetime, receiveLifetime;
	BOOL currentPBCCSupport;
	UINT8 PBCCAlgorithmMode; 
	UINT32 swMinor, swMajor;
	UINT8 SlotTimeMode, PreambleMode;
	UINT16 SlotTimeThreshold;
	UINT16 stationsNotSlotTimeCapabale, stationsNotShortPreambleCapabale;
	BOOL currentShortSlotTimeSupport, currentShortPreambleSupport;
	UINT8 ERPProtectionMode, ERPProtectionType;
	UINT8 ERPProtectionTI;
	BOOL currentERPProtectionMode;
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	//char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;
	UINT8 respRateMode;
	UINT8 dot11Mode,CWminMode;
	UINT16 CWminWindowSize;
	UINT32	DynAlgoTimeout;
	UINT32 RadioCalibrateOption;
	UINT16 RadioCalibrateInterval;
	UINT8 band;
	UINT8 num_sup_wone;
	UINT16 HiddenSSID;

	UINT8 usageTimeMode;
	UINT8 usageTime;

	UINT16 MultipleSsidFlag;
	UINT8  ssidStr[WHAL_SSID_STRING_LEN+1];
	apMultiSsidElem_t MultipleSsidTable[NUM_MULTIPLE_SSID];

	wdrv_dbGetNumSupWone(&num_sup_wone);
	if (num_sup_wone == WONE_DUAL_BAND )
	{
		len += sprintf(page +len,"\n---------  Dual AP ---------\n");

		wdrv_dbGetUsageTimeMode(&usageTimeMode);
		if(usageTimeMode)
			len += sprintf(page + len,"\n--------- Long Usage Time ---------\n");
		else  
			len +=sprintf(page + len,"\n---------  Short Usage Time ---------\n");
	}
	len += sprintf(page + len,"\n--------- Active Set Wone Index is %d ---------\n",currentConfigWoneInex);  

	wdrv_dbGetBand(currentConfigWoneInex,&band);


	swMajor = GET_MAJOR_SW_VER;
	swMinor = GET_MINOR_SW_VER;

	wdrv_dbGetOperationMacAddress(macaddr);
	wdrv_dbGetBssid(bssid);	
	wdrv_mgmt_ssidGetPrimarySsid(ssid);
	wdrv_mgmt_ssidGetHiddenSsidStatus(&HiddenSSID);
	wdrv_mgmt_ssidGetMultipleSsidStatus(&MultipleSsidFlag);
	if (MultipleSsidFlag)
	{
		UINT32 i;

		memset(MultipleSsidTable, 0, sizeof(MultipleSsidTable));
		for (i=0; i<NUM_MULTIPLE_SSID; i++)
			wdrv_mgmt_ssidGetElementFromMultipleSsidActualTable(&MultipleSsidTable[i], i);
	}

	wdrv_dbGetStationConfigOperationalAllRateSet(currentConfigWoneInex,allRates, &allRatesLen);
	wdrv_dbGetDefaultActiveRate(currentConfigWoneInex,&defaultActiveRate);
	wdrv_dbGetApCapabilities(currentConfigWoneInex,&capabilities);
	wdrv_dbGetDefaultActiveRateBrcst(currentConfigWoneInex ,&defaultActiveRateBrcst);
	wdrv_dbGetDefaultActiveRateMcst(currentConfigWoneInex ,&defaultActiveRateMcst);
	wdrv_dbGetGenTxActiveRate(currentConfigWoneInex ,&genTxRate);
	wdrv_dbGetChannel(currentConfigWoneInex,&channel);
	wdrv_dbGetStationConfigBeaconPeriod(&beacon);
	wdrv_dbGetStationConfigDtimPeriod(&dtim);

	wdrv_dbGetOperationRTSThreshold(&rtsThreshold);
	wdrv_dbGetOperationShortRetryLimit(&shortRetryLimit);
	wdrv_dbGetOperationLongRetryLimit(&longRetryLimit);
	wdrv_dbGetRateFallback(&RateFallback);
	wdrv_dbGetRateAdaptation(currentConfigWoneInex,&RateAdaptation);
	wdrv_dbGetOperationFragmentationThreshold(&fragThreshold);
	wdrv_dbGetOperationMaxTransmitMSDULifetime(&transmitLifetime);
	wdrv_dbGetOperationMaxReceiveLifetime(&receiveLifetime);
	wdrv_dbGetPBCCAlgorithmMode(currentConfigWoneInex,&PBCCAlgorithmMode);
	wdrv_dbGetNumOfStationNotPBCCCapable(&stationsNotPBCCCapabale);
	wdrv_dbGetCurrentPBCCSupportMode(&currentPBCCSupport);
	wdrv_dbGetDot11Mode(currentConfigWoneInex,&dot11Mode);
	wdrv_dbGetRadioCalibrateOption(&RadioCalibrateOption);
	wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);

	wdrv_dbGetPreambleMode(currentConfigWoneInex,&PreambleMode);
	wdrv_dbGetNumOfStationNotShortPreambleCapable(&stationsNotShortPreambleCapabale);
	wdrv_dbGetCurrentShortPreambleSupport(&currentShortPreambleSupport);

	wdrv_dbGetSlotTimeMode(currentConfigWoneInex,&SlotTimeMode);
	wdrv_dbGetSlotTimeThreshold(&SlotTimeThreshold);
	wdrv_dbGetNumOfStationNotSlotTimeCapable(&stationsNotSlotTimeCapabale);
	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);

	wdrv_dbGetERPProtectionMode(currentConfigWoneInex,&ERPProtectionMode);
	wdrv_dbGetCurrentERPProtectionMode(&currentERPProtectionMode);
	wdrv_dbGetERPProtectionType(currentConfigWoneInex,&ERPProtectionType);
	wdrv_dbGetERPProtectionTI(currentConfigWoneInex,&ERPProtectionTI);

	wdrv_dbGetDynamicAlgorithmsTimeout(&DynAlgoTimeout);

	wdrv_dbGetRespRateMode(&respRateMode);

	wdrv_dbGetCwMinMode(currentConfigWoneInex,&CWminMode);
	wdrv_dbGetCwMinWinSize(&CWminWindowSize);
	wdrv_dbGetUsageTime(currentConfigWoneInex,&usageTime);

	len += sprintf(page + len,"Software Version:	%d.%d\n",swMajor, swMinor);


	len += sprintf(page + len,"WLAN MAC = <%02X-%02X-%02X-%02X-%02X-%02X>\n",		
			macaddr[0],macaddr[1],macaddr[2],macaddr[3],
			macaddr[4],macaddr[5] );

	len += sprintf(page + len,"WLAN BSSID = <%02X-%02X-%02X-%02X-%02X-%02X>\n",		
			bssid[0],bssid[1],bssid[2],bssid[3],
			bssid[4],bssid[5] );

	len += sprintf(page + len,"SSID = %s\n", ssid); 

	if (band == RADIO_BAND_5GHZ)    
		len += sprintf(page + len,"Band = 5 GHZ\n");  
	else
		len += sprintf(page + len,"Band = 2.4 GHZ\n"); 

	len += sprintf(page + len,"Hidden SSID mode is %s\n", (HiddenSSID==1) ? "Enabled" : "Disabled");
	len += sprintf(page + len,"Multiple SSID mode is %s\n", (MultipleSsidFlag==1) ? "Enabled" : "Disabled");
	if (MultipleSsidFlag==1) 
	{
		len += sprintf(page + len,"List of Multiple SSID:\n");
		for (i=0; i<NUM_MULTIPLE_SSID; i++) 
		{
            if( (MultipleSsidTable[i].ssidType != SSID_SECONDARY) && (MultipleSsidTable[i].ssidType != SSID_PRIMARY) )
				continue;
			memcpy(ssidStr, MultipleSsidTable[i].ssid.serviceSetId, WHAL_SSID_STRING_LEN);
			ssidStr[MultipleSsidTable[i].ssid.hdr.eleLen] = '\0';
			len += sprintf(page + len,"\t%d. %s\n",i+1, ssidStr);

		}
		if (i==0)
			len += sprintf(page + len,"\tNone.\n");
	}

	len += sprintf(page + len,"Capabilities = AGILE  PBCC  ShortPreamble  Privacy  CFPollReq  CFPoll  IBSS  ESS\n"); 

	len += sprintf(page + len,"                 %d    ", ((capabilities >> CAP_AGILE_SHIFT) & CAP_AGILE_MASK));
	len += sprintf(page + len,"  %d   ", ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK));
	len += sprintf(page + len,"      %d       ", ((capabilities >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK));
	len += sprintf(page + len,"    %d     ", ((capabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK));
	len += sprintf(page + len,"    %d      ", ((capabilities >> CAP_CF_REQ_SHIFT) & CAP_CF_REQ_MASK));
	len += sprintf(page + len,"   %d    ", ((capabilities >> CAP_CF_POLL_SHIFT) & CAP_CF_POLL_MASK));
	len += sprintf(page + len,"  %d   ", ((capabilities >> CAP_IBSS_SHIFT) & CAP_IBSS_MASK));
	len += sprintf(page + len,"  %d\n", ((capabilities >> CAP_ESS_SHIFT) & CAP_ESS_MASK));

	len += sprintf(page + len,"Capabilities = RSN  ShortSlotTime\n"); 
	len += sprintf(page + len,"                %d         ", ((capabilities >> CAP_RSN_SHIFT) & CAP_RSN_MASK));
	len += sprintf(page + len,"%d\n", ((capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK));

	len += sprintf(page + len,"Rates =	"); 
	for (i = 0; i < allRatesLen; i++)
	{
		len += sprintf(page + len,"  "); 
		len += wdrv_printRateInMbpsUnit(allRates[i], page + len);
	}
	len += sprintf(page + len,"\n"); 

	len += sprintf(page + len,"Default active rate = "); 
	len += sprintf(page + len,"  "); 
	len += wdrv_printRateInMbpsUnit(defaultActiveRate, page + len);
	len += sprintf(page + len,"\n"); 

    len += sprintf(page + len,"\n"); 
	
	len += sprintf(page + len,"Default broadcast active rate = "); 
	len += sprintf(page + len,"  "); 
	len += wdrv_printRateInMbpsUnit(defaultActiveRateBrcst, page + len);
	len += sprintf(page + len,"\n"); 

	len += sprintf(page + len,"Default multicast active rate = "); 
	len += sprintf(page + len,"  "); 
	len += wdrv_printRateInMbpsUnit(defaultActiveRateMcst, page + len);
	len += sprintf(page + len,"\n"); 

	len += sprintf(page + len,"General TX rate = "); 
	len += sprintf(page + len,"  "); 
	if (!genTxRate) 
		len += sprintf(page + len," System default "); 
	else
		len += wdrv_printRateInMbpsUnit(genTxRate, page + len);
	len += sprintf(page + len,"\n"); 


	len += sprintf(page + len,"Channel = %d\n", channel); 

	if (channel == 14)
		len += sprintf(page + len,"Domain = MKK\n"); 
	else
		len += sprintf(page + len,"Domain = ETSI\n"); 

	len += sprintf(page + len,"Beacon period = %d TUs(msecs)\n", beacon); 

	len += sprintf(page + len,"DTIM period = %d beacons\n", dtim); 

	len += sprintf(page + len,"rtsThreshold = %d bytes\n", rtsThreshold); 
	len += sprintf(page + len,"shortRetryLimit = %d\n", shortRetryLimit); 
	len += sprintf(page + len,"longRetryLimit = %d\n", longRetryLimit); 
	len += sprintf(page + len,"rate fallback = %d\n", RateFallback); 
	if (RateAdaptation)
	{
		len += sprintf(page + len,"rate adaptation = Enabled\n");
	}
	else
	{
		len += sprintf(page + len,"rate adaptation = Disabled\n");
	}
	len += sprintf(page + len,"fragThreshold = %d bytes\n", fragThreshold); 
	len += sprintf(page + len,"transmitLifetime = %d msecs\n", transmitLifetime); 
	len += sprintf(page + len,"receiveLifetime = %d msecs\n", receiveLifetime); 
	len += sprintf(page + len,"Radio Calibrate option=%d interval=%d\n", RadioCalibrateOption, RadioCalibrateInterval); 

	if (dot11Mode == DOT11_MODE_MIXED)
	{
		len += sprintf(page + len,"Dot11 mode = Mixed mode\n");
	}
	else if (dot11Mode == DOT11_MODE_B_ONLY)
	{
		len += sprintf(page + len,"Dot11 mode = B only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_OFDM_ONLY)
	{
		len += sprintf(page + len,"Dot11 mode = G only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_B_ONLY_PLUS)
	{
		len += sprintf(page + len,"Dot11 mode = B+ only mode\n");
	}
	else
		len += sprintf(page + len,"Dot11 mode = A only mode\n");


	if (respRateMode == ASSOCRESP_RATE_ALL_AP)
	{
		len += sprintf(page + len,"AssocResp rate mode = %d (All AP rates)\n", ASSOCRESP_RATE_ALL_AP);
	}
	else if (ASSOCRESP_RATE_AP_AND_STA)
	{
		len += sprintf(page + len,"AssocResp rate mode = %d (Only AP rates supported by STA)\n", ASSOCRESP_RATE_AP_AND_STA);
	}

	len += sprintf(page + len,"Enhanced Dynamic Algorithms Timeout = %d ms\n", DynAlgoTimeout);

	len += sprintf(page + len,"CWmin mode = %d, CWminWindowSize = %d\n", CWminMode,CWminWindowSize);


	/* PBCC support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (PBCCAlgorithmMode == PBCC_ALGO_DISABLED)
		len += sprintf(page + len,"PBCC Algorithm			Always off\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS)
		len += sprintf(page + len,"PBCC Algorithm			Local Stations\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC)
		len += sprintf(page + len,"PBCC Algorithm			Dynamic\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC)
		len += sprintf(page + len,"PBCC Algorithm			Enhanced Dynamic\n");	
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENABLED)
		len += sprintf(page + len,"PBCC Algorithm			Always on\n");
	else
		len += sprintf(page + len,"PBCC Algorithm			Unrecognized\n");

	if ((PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS) || (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC) || (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		len += sprintf(page + len,"Number of station that are not PBCC capable:	%d\n", stationsNotPBCCCapabale);
		if ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		{	
			if (currentPBCCSupport)
				len += sprintf(page + len,"Current PBCC support		On\n");
			else
				len += sprintf(page + len,"Current PBCC support		Off\n");
		}
	}

	/* Preamble support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (PreambleMode == PREAMBLE_MODE_LONG)
		len += sprintf(page + len,"Preamble Mode                   Always Long\n");
	else if (PreambleMode == PREAMBLE_MODE_SHORT)
		len += sprintf(page + len,"Preamble Mode                   Always Short\n");
	else if (PreambleMode == PREAMBLE_MODE_LOCAL_STATIONS)
	{
		len += sprintf(page + len,"Preamble Mode                   Local Stations\n");
		if (currentShortPreambleSupport)
			len += sprintf(page + len,"Current Short Preamble support  On\n");
		else
			len += sprintf(page + len,"Current Short Preamble support  Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotShortPreambleCapabale);
	}

	/* Slot Time support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (SlotTimeMode == SLOTTIME_MODE_DISABLED)
		len += sprintf(page + len,"Slot Time Mode                  Always off\n");
	else if (SlotTimeMode == SLOTTIME_MODE_ENABLED)
		len += sprintf(page + len,"Slot Time Mode                  Always on\n");
	else if (SlotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS)
	{
		len += sprintf(page + len,"Slot Time Mode                  Local Stations\n");
		len += sprintf(page + len,"Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			len += sprintf(page + len,"Current Short Slot Time support On\n");
		else
			len += sprintf(page + len,"Current Short Slot Time support Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}
	else if (SlotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC)
	{
		len += sprintf(page + len,"Slot Time Mode                  Enhanced Dynamic\n");
		len += sprintf(page + len,"Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			len += sprintf(page + len,"Current Short Slot Time support On\n");
		else
			len += sprintf(page + len,"Current Short Slot Time support Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}



	/* ERP Protection support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (ERPProtectionMode == PROTECTION_MODE_DISABLED)
	{
		len += sprintf(page + len,"ERP Protection Mode             Always off\n");
	}
	else
	{
		if(ERPProtectionMode == PROTECTION_MODE_LOCAL_STATIONS)		
			len += sprintf(page + len,"ERP Protection Mode             Local Stations\n");
		else if(ERPProtectionMode == PROTECTION_MODE_DYNAMIC)
			len += sprintf(page + len,"ERP Protection Mode             Dynamic\n");
		else if(ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
			len += sprintf(page + len,"ERP Protection Mode             Enhanced Dynamic\n");
		else if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
			len += sprintf(page + len,"ERP Protection Mode             Always on\n");
		else
			len += sprintf(page + len," Undefined ERP Protection Mode\n");


		if (currentERPProtectionMode)
		{
			len += sprintf(page + len,"Current ERP Protection Mode     On\n");
		}
		else
		{
			len += sprintf(page + len,"Current ERP Protection Mode     Off\n");
		}

		if(ERPProtectionTI)
			len += sprintf(page + len,"ERP Protection Type             TI-proprietary\n");
		else
		{
			if (ERPProtectionType == PROTECTION_TYPE_RTS_CTS)
				len += sprintf(page + len,"ERP Protection Type             RTS/CTS\n");
			else
				len += sprintf(page + len,"ERP Protection Type             CTS TO SELF\n");
		}
	}
	if(num_sup_wone == 2)
		len += sprintf(page + len,"Wone Usage Time is %d\n",usageTime);

	len += sprintf(page + len, "Ap bridge mode %s\n", wdrv_dbGetApBridgeMode() ? "enabled" : "disabled");

	return len;
}

/******************************************************************************/

/*! \brief 	Put firmware code into TNETW buffer
	\param 	buf - pointer to source buffer
	\param 	len - source buffer size
	\return status of operation OK
*/
/******************************************************************************/

 static int putFwImage(unsigned char * buf, int len) 
{
    int acxArrayLen = _Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart;

    if (! Acx100_bin_0_Array ) 
    {
		Acx100_bin_0_Array = _Acx100_bin_0_ArrayStart;
		Acx100_bin_0_Array_Len = acxArrayLen;		
	}
	memcpy(Acx100_bin_0_Array, buf, acxArrayLen);

    if (len > acxArrayLen) 
    {
        Acx100_bin_0_Array_Extra_Len = len - acxArrayLen;
    }
    else
    {
       Acx100_bin_0_Array_Extra_Len = 0;
    }
	if (Acx100_bin_0_Array_Extra_Len) 
    {
		if (Acx100_bin_0_Array_Extra)
        {
			free(Acx100_bin_0_Array_Extra);
        }
		Acx100_bin_0_Array_Extra = malloc(Acx100_bin_0_Array_Extra_Len);
		memcpy(Acx100_bin_0_Array_Extra, 
               buf + acxArrayLen, 
               Acx100_bin_0_Array_Extra_Len);
	} 
    else 
    {
		if (Acx100_bin_0_Array_Extra)
        {
			free(Acx100_bin_0_Array_Extra);
        }
		Acx100_bin_0_Array_Extra = NULL;
	}
 return OK;
}

/******************************************************************************/

/*! \brief 	Shows partial configuration structure
*/
/******************************************************************************/

void ShowInitCfg(void)
{
	logMsg("int alg0=%x en0=%x alg1=%x en1=%x\n",
	   init_cfg.dot11AuthenticationAlgorithmTable[0].authAlgo,
	   init_cfg.dot11AuthenticationAlgorithmTable[0].enable,
	   init_cfg.dot11AuthenticationAlgorithmTable[1].authAlgo,
	   init_cfg.dot11AuthenticationAlgorithmTable[1].enable,
	   0,0); 

}

/*******************************************************************************************/
/* IsChannelInsideBandRange() */
/*! 	
	\brief serves for internal use: verfies if a given channel is valid for a given band	   
                                                                         
	\param band 		band 
	\param chID			channel number 
   
	\return TRUE/FALSE.			                                                    
*/
/*******************************************************************************************/
static BOOL IsChannelInsideBandRange(int band, int chID)
{
	BOOL found = FALSE;
	int ch;

	if (band == RADIO_BAND_2_4GHZ)
	{
		if ((chID < 1) || (chID > 14))
			return FALSE;
		return TRUE;
	}
	else
	{
		if ((chID < 36) || (chID > 161))
			return FALSE;
		/* channel is inside the range, let's verify its value */
		for (ch=36; ch<=64;ch++)
			if (ch == chID)
			{
				found=TRUE;
				break;
			}
		if (!found)
			for (ch=100; ch<=140;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
		if (!found)
			for (ch=149; ch<=161;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
	}
	return found;
}

/*******************************************************************************************/
/*! 
	\brief Private WE subioctl handler. Sets BIT commands parameters
	
	\param init_cfg			initial driver configuration structure
	\param val				input BIT arguments structure
	
	\return  0 in case of success, error code otherwise
*/
/*******************************************************************************************/
static int param_set_bit_args(initConfigStruct *init_cfg, tiap_param_bit_args_t *val)
{
	int ret = 0;

   if (val->band < 0)
	   return TIAP_PARAM_ERANGE;

   if(val->channel <= 0)
	   return TIAP_PARAM_ERANGE;
   
   if (IsChannelInsideBandRange(val->band,val->channel) == FALSE)
   {
	   return TIAP_PARAM_ERANGE;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
   wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

   if(val->cmdType == BIT_FCC)
   {
       if(whal_apiAcxBIT_Fcc(val->band, val->channel, val->rate, val->mode) != OK)
           return TIAP_PARAM_ERANGE;
   }
   else
   {
       if(whal_apiAcxBIT_Telec(val->band, val->channel) != OK)
           return TIAP_PARAM_ERANGE;
   }
     
	return ret;
}
