/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_main.c                                               
    \brief This module implements driver initialization, configuration 
		   and unloading functions                                                                          
*/
/******************************************************************************/

#define _DDA_MAIN_C

/***********************************************************
	 includes.
*******************************************************/
#include "wlan_pal.h"
#include "dda.h"
#include "dda_EventQueue.h"
#include "dda_trace.h"
#include "dda_contexts.h"
#include "dda_ioctl.h"
#include "dda_wdsDrv.h"
#include "wds.h"
#include "bssbridge.h"
#include "wlanEnd.h"
#include "whalInterrupt.h"
#include "sysApi.h"
#include "sangam.h"
#include "pal_vlynq.h"
#include "whalApi.h"
#include "acx_hw.h"
#include "dda_wpa_interface.h"
#include "drvDbMain.h"

/***********************************************************
	defs
************************************************************/

#define WAIT_FOR_WPA_MAX_RETRY  10

#define VLYNQ_INTR_NUM			AVALANCHE_LOW_VLYNQ_INT
#ifdef VDSP_ON_CHAIN
#define VLYNQ_IRQ_NUM			8
#else
#define VLYNQ_IRQ_NUM           0
#endif
#define TIAP_LED_TIMER_PERIOD 30
#define DDA_RX_SKIP_BLINK 16

/***********************************************************
	macros
***********************************************************/

#define DDA_INIT_ERR(val) \
do { \
    DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Initialization failed %d\n", \
                __FUNCTION__, __LINE__, (val), 0,0,0); \
} while(0)

extern void  vlynq_dev_init(void);

/***********************************************************
	Local Function Prototypes
***********************************************************/

STATUS ddaInitInterrupts(void);
static STATUS initTnetwAddr( UINT32 * RegAddr, UINT32 * MemAddr );


/***********************************************************
	static variables
************************************************************/

static BOOL wlanStarted;
static BOOL firstInit = TRUE;
static PALSRV_TcbObjTYPE tiap_led_timer;
#if (WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
static TIWLAN_RESET_TYPE_E G_tiwlan_acx_reset_type = TIWLAN_SOFT_RESET;
#else
static TIWLAN_RESET_TYPE_E G_tiwlan_acx_reset_type = TIWLAN_HARD_RESET;
#endif
#ifdef DDA_WLAN_LED
int tiap_led_cur_state = TIAP_LED_OFF;
int	tiap_led_oper_state = TIAP_LED_OFF;
int	tiap_led_blinked = 0;
UINT16 tiap_activity_led_mask;

/*************************************************************************/
/*! \brief Implements a led timer
	\param data 	not used 
*/
/*************************************************************************/
void tiap_led_timer_func(__u32 data)
{
    unsigned int flags;

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	if (tiap_led_oper_state == TIAP_LED_OFF || !tiap_gpio_owner) {
        PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
		goto out;
    }
		
	if (tiap_led_cur_state == TIAP_LED_OFF) {
#if 0
		whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
#else
        PAL_sysGpioOutBit( 6, 0);
#endif
		tiap_led_cur_state = TIAP_LED_ON;
        PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
		goto out_blinked;
	}

	if (tiap_led_blinked) {
#if 0
		whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
#else
        PAL_sysGpioOutBit( 6, 1);
#endif
		tiap_led_cur_state = TIAP_LED_OFF;
	}

    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);

out_blinked:
	tiap_led_blinked = 0;
out:
	if (tiap_gpio_owner)
		PALSRV_tcbQueueAct(&tiap_led_timer, TIAP_LED_TIMER_PERIOD, tiap_led_timer_func, 0, 0);
}
#endif

/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/
/*! \brief 	WLAN driver DDA initialization function
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS tiwlan_ddaReInit(void)
{
    if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
        vlynq_dev_init();
	return OK;
}

/******************************************************************************/

/*! \brief 	WLAN driver DDA initialization function
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS tiwlan_ddaInit(void)
{
 STATUS status;
 
 taskLock();	/* Enter critical section */
 if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
     vlynq_dev_init();
 DTR_SetReportLevel(DDA_LOG_ERR | DDA_LOG_POOL);

 PAL_Init();

 tiap_gpio_owner = 1;    /* 1 for driver, 0 for fw*/
 wlanStarted = FALSE;

 status = DDA_InitIoctl();
 if (status != OK) 
 {
	 goto ExitOnInitError;
 }
 
 status = DCX_TaskInit( DDA_TASK_PRIORITY );
 if (status != OK) 
 {
	 goto ExitOnInitError;
 }

#ifdef DDA_TRACE
 status = DTR_TRACE_INIT( 128, NULL );
 if (status != OK) 
 {
	 goto ExitOnInitError;
 }
#endif
 
 if (!WEND_IsInitialized())
 {
	 status = WEND_DrvLoad();
	 if (status != OK) 
	 {
		 goto ExitOnInitError;
	 }
 }
 
 if (!tiwlan_IsWdsNetDevInitialized())
 {
	 tiwlan_setupWdsNetDevs( WDS_MAX_DL_NUMBER );
 }

#if defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7VW)
    /* configure GPIO #6 - LED to output mode */
    PAL_sysGpioCtrl( 6, GPIO_PIN, GPIO_OUTPUT_PIN);

	PALSRV_tcbNew(&tiap_led_timer);
	PALSRV_tcbQueueAct(&tiap_led_timer, TIAP_LED_TIMER_PERIOD, tiap_led_timer_func, 0, 0);
#endif

 taskUnlock();	 /* Exit from critical section */
 return OK;

ExitOnInitError:
 
 taskUnlock();
 DDA_INIT_ERR(status);
 return NOK;

}

/******************************************************************************/

/*! \brief 	Delete Mac address from Local Bridge
	\param 	mac - pointer to MAC address entry to delete
*/
/******************************************************************************/

extern void sysBridgeDelMac (char *mac_address); /* from sysBridge.c */

void tiwlan_ddaDeleteMacAddrFunc(char *mac)
{
    char stringMac[MAC_ADDR_LEN*3];

    /* Turn the mac address into string */
    sprintf(stringMac, "%02x:%02x:%02x:%02x:%02x:%02x", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);

    sysBridgeDelMac( stringMac );
}

/******************************************************************************/

/*! \brief 	Detect whether allocation of new buffer is needed
	\param  len 		 - required data buffer size
	\param  dataBufToken - token to assign
	\param  buf			 - pointer to pointer to allocated buffer
	\param  buf			 - contains output pointer to allocated data buffer
	\param  dataBufToken - =buf
	
	\return OK in case the allocation is needed and NOK otherwise
*/
/******************************************************************************/

STATUS 	tiwlan_ddaRxAllocBufferIfNeeded( UINT32 len, UINT32 * dataBufToken, 
										 UINT8 ** buf, tiwlan_allocPhase_t phase )
{
	return NOK;
}

/******************************************************************************/

/*! \brief 	Detect whether allocation of pool is needed
	\param  pPool - pointer to pool pointer
	\param  size  - size of required pool
	
	\return OK or ERROR in case of allocation failure
*/
/******************************************************************************/


STATUS 	tiwlan_ddaAllocPoolIfNeeded( UINT8 ** pPool, UINT32 size )
{
	UINT8 * pPoolTmp;

	pPoolTmp = (UINT8*) PAL_memAlloc(0, size,0,0);
	if( pPoolTmp == NULL)
	{
		DDA_LOG_ARG(DDA_LOG_ERR, "%s line %d: Pool allocation failed\n",
					__FUNCTION__, __LINE__, 0,0,0,0);
		return NOK;
	}
	DDA_LOG_ARG(DDA_LOG_POOL, 
				"%s: Queue pool allocated at %x Size=%u\n",
				__FUNCTION__, pPoolTmp, size, 0,0,0);
	
	*pPool = pPoolTmp;
	
	return OK;
}

/******************************************************************************/

/*! \brief 	generate a pseudo-random integer
	
	\return pseudo-random byte
*/
/******************************************************************************/

UINT8 tiwlan_ddaRand(void)
{
	UINT8 temp;

	temp = (UINT8) rand();
	return temp;
}

/******************************************************************************/

/*! \brief 	Resets the value of the seed used to generate random numbers
	\param  seed - random number seed
*/
/******************************************************************************/

void tiwlan_ddaSrand(UINT16 seed)
{
	srand(seed);
}

/******************************************************************************/

/*! \brief 	Reset ACX and sets memory map. VLYNQ gets reset if reset type is defined 
            as TIWLAN_HARD_RESET.
	\param - pointer to VLYNQ registers base addresses 
	\param - pointer to VLYNQ memory base addresses
	\param - output VLYNQ registers base addresses 
	\param - output VLYNQ memory base addresses
	
	\return status of operation OK
*/
/******************************************************************************/


STATUS tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr) 
{

    if ( initTnetwAddr( RegAddr, MemAddr) != OK ) 
    {
        printf("%s: Error reading TNETW mem map.\n", __FUNCTION__);
        return NOK;
    }

	ddaInitInterrupts();

	return OK;
}


/******************************************************************************/

/*! \brief 	Sets and returns TNETW memory map
	\param - pointer to VLYNQ registers base addresses to return the value 
	\param - pointer to VLYNQ memory base ddresses to return the value
	
	\return OK if operation is successful or NOK in the case of error
*/
/******************************************************************************/

STATUS tiwlan_ddaInitTnetwAddr(UINT32 * RegAddr, UINT32 * MemAddr) 
{

    if ( initTnetwAddr( RegAddr, MemAddr) != OK ) 
    {
        printf("%s: Error reading TNETW mem map.\n", __FUNCTION__);
        return NOK;
    }

	return OK;
}


/******************************************************************************/

/*! \brief 	Resets VLYNQ
	
	\return status of operation OK
*/
/******************************************************************************/
PAL_VLYNQ_DEV_ISR_PARAM_GRP_T vlynqPar = {0};

STATUS tiwlan_ddaUnloadPSPVlynq(void) 
{
    STATUS status;
    PAL_VLYNQ_DEV_HND *vlynq_dev;
    PAL_VLYNQ_HND * pVlynq;

    if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
        return OK;
    if( ! (vlynq_dev = PAL_vlynqDevFind("wlan", 0)))
    {
        printk("Could not find the WLAN device configuration for instance 0. Abort loading.\n");
        return NOK;
    }

    if( ! (pVlynq = PAL_vlynqDevGetVlynq(vlynq_dev)))
    {
        printk("Problems in the WLAN VLYNQ configuration for instance 0. Abort loading.\n");
        return NOK;
    }

    printf("Vlynq handler=%x\n", pVlynq );

    status = PAL_vlynqDisableIrq( pVlynq, VLYNQ_IRQ_NUM );
    if (status != PAL_VLYNQ_OK)
    {
        logMsg("%s: Enabling Vlynq ISR failed, status=%x\n",
                (int)__FUNCTION__, status, 0,0,0,0);
    }

    status = PAL_vlynqRemoveIsr(pVlynq, VLYNQ_IRQ_NUM, 
                                &vlynqPar );

    if (status != PAL_VLYNQ_OK)
    {
        DDA_LOG_ARG(DDA_LOG_ERR,"%s: Add Vlynq ISR failed, status=%x\n",
                    __FUNCTION__, status, 0,0,0,0);
    }


	return OK;
}

/******************************************************************************/

/*! \brief 	Inits VLYNQ interrupt
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/


STATUS ddaInitInterrupts(void) 
{
	STATUS status;
	PAL_VLYNQ_DEV_HND *vlynq_dev;
	PAL_VLYNQ_HND * pVlynq;

	if( ! (vlynq_dev = PAL_vlynqDevFind("wlan", 0)))
	{
		printk("Could not find the WLAN device configuration for instance 0. Abort loading.\n");
		return NOK;
	}

	if( ! (pVlynq = PAL_vlynqDevGetVlynq(vlynq_dev)))
	{
		printk("Problems in the WLAN VLYNQ configuration for instance 0. Abort loading.\n");
		return NOK;
	}

	if( ! (PAL_vlynqGetLinkStatus(pVlynq))) 
	{
		printk("We did not detect Link for WLAN device instance 0.\n");
		return NOK;
	}

	printf("Vlynq handler=%x\n", pVlynq );
	
	status = PAL_vlynqAddIsr(pVlynq, VLYNQ_IRQ_NUM,
							 (PAL_VLYNQ_DEV_ISR_FN) whal_acxIntrHandler, 
							 &vlynqPar );
	
	if (status != PAL_VLYNQ_OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR,"%s: Add Vlynq ISR failed, status=%x\n",
					__FUNCTION__, status, 0,0,0,0);
	}
	
    status = PAL_vlynqEnableIrq( pVlynq, VLYNQ_IRQ_NUM );
	if (status != PAL_VLYNQ_OK)
	{
		logMsg("%s: Enabling Vlynq ISR failed, status=%x\n",
				(int)__FUNCTION__, status, 0,0,0,0);
	}
    if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
        /* Enable vlynq interrupt */
        sysHwIntEnable(VLYNQ_INTR_NUM);
	
	return status;
}

/******************************************************************************/

/*! \brief 	Send packet from END driver to WLAN
	\param 	pMSDU - pointer to MSDU to send over WLAN
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_ddaSend(mem_MSDU_T * pMSDU) 
{

	bm_trace(1,0,0);

	wdrv_bssBridgePktFromEthrToWlan(pMSDU);

	return OK;
}

/******************************************************************************/

/*! \brief 	Receive packet from WLAN and send to END driver
	\param 	pMSDU - pointer to MSDU to send to Ethernet
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/


STATUS tiwlan_ddaReceive(mem_MSDU_T * pMSDU) 
{
	STATUS status;

#ifdef DDA_WLAN_LED
    static int skip_blink;
#endif
	
 	status = WEND_EndRecv(pMSDU);

#ifdef DDA_WLAN_LED
	if (!wdrv_dbGetApBridgeMode() && !(++skip_blink % DDA_RX_SKIP_BLINK)){
		TIWLAN_LED_PKT_RCV;
	}
#endif

	
	return status;
}

/******************************************************************************/

/*! \brief 	suspend all active timers
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_ddaSuspend(void) 
{	
    taskLock(); /* START OF CRITICAL SECTION */
    tiwlan_wpaSuspend();
    if(wdrv_dbGetWdsEnable()) {
        tiwlan_ddaWdsSuspedTransmit();
    }
	PALSRV_tcbSuspend();
	DCX_suspend();
    taskUnlock(); /* END OF CRITICAL SECTION */
	
	return OK;
}

/******************************************************************************/

/*! \brief 	Resumes all active timers
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_ddaResume(void) 
{	
    taskLock(); /* START OF CRITICAL SECTION */
	DCX_resume();
    PALSRV_tcbResume();
    if(wdrv_dbGetWdsEnable()) {
        tiwlan_ddaWdsResumeTransmit();
    }	
    tiwlan_wpaResume();
    taskUnlock(); /* END OF CRITICAL SECTION */

	return OK;
}

/******************************************************************************/

/*! \brief 	Waits for WPA task to send hello
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_dda_wpaGo(void)
{
	dda_wpa_drv_send_hello();
	return OK;
}


/******************************************************************************/

/*! \brief 	wait for WPA task to wake up
	
	\return status of operation OK	or ERROR
*/
/******************************************************************************/

STATUS tiwlan_dda_waitForWpa(void)
{	
	int retryCount = 0;
	
	/* wait for hello msg */
	while(retryCount++ < WAIT_FOR_WPA_MAX_RETRY)
	{
		if(dda_wpa_wait_for_wpaHello() != OK)
			printk("\nDRV WAITING FOR WPA MODULE TO COME UP !!!\n");
		else
			return OK;
	}
	return NOK;
}

/******************************************************************************/

/*! \brief 	Starts WLAN driver
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_ddaStart(void)
{
	
	printf("%s: DDA is started.\n", __FUNCTION__);
		
	wlanStarted = TRUE;
	firstInit   = FALSE;
	
	return OK;
}


/******************************************************************************/

/*! \brief 	Determines whether to initialize memory management module
	
	\return OK if memory management initialization is required and NOK if not
*/
/******************************************************************************/

STATUS 	tiwlan_ddaIsMemMngrInitNeeded( void )
{
	return (firstInit ? OK : NOK);
}


/******************************************************************************/

/*! \brief 	Unloads DDC, DDA and PAL
	
	\return status of operation OK
*/
/******************************************************************************/

STATUS tiwlan_ddaCleanup(void)
{
#ifdef DDA_WLAN_LED
#if defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7VW)
    PALSRV_tcbDelete(&tiap_led_timer);
#endif
	TIWLAN_LED_LINK_DOWN;
#endif

	WEND_Stop();								
 
     if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
         sysHwIntDisable(VLYNQ_INTR_NUM);

	wdrv_DeauthStations();
	
	PAL_sleepLong(1000);  /* Give a time to complete Tx of Ethernet and WLAN */

	taskLock();		  	  /* Enter critical section */
	
	if (wlanStarted)
	{
		wdrv_PartialUnload();
		wlanStarted = FALSE;
	}
	
	DTR_SetReportLevel(0);
	
	PAL_reportLevel=0;

#ifdef DDA_TRACE
	DTR_TraceDelete();
#endif
	
	DCX_TaskDelete();
	
	DDA_DeleteIoctl();

	PAL_Delete();
	
	taskUnlock();		  	/* Exit critical section */

	return OK;
}

/******************************************************************************/

/*! \brief 	Set ACX reset type. if reset type is setted as HARD, the next driver
            loading resets ACX including VLYNQ device, otherwise only ACX gets reset
            without VLYNQ.
	
	\return N/A
*/
/******************************************************************************/
void tiwlan_setAcxResetType(TIWLAN_RESET_TYPE_E type)
{
    G_tiwlan_acx_reset_type = type;
}

/******************************************************************************/

/*! \brief 	 Sets and returns TNETW memory and register map
	\param - pointer to VLYNQ registers base addresses to return the value 
	\param - pointer to VLYNQ memory base ddresses to return the value
	
	\return OK if operation is successful or NOK in the case of error
*/
/******************************************************************************/

static STATUS initTnetwAddr( UINT32 * RegAddr, UINT32 * MemAddr )
{
   	PAL_VLYNQ_DEV_HND * vlynq_dev;
	PAL_VLYNQ_HND	  * vlynq;

	if( ! (vlynq_dev = PAL_vlynqDevFind("wlan", 0)))
	{
		printf("%s: Could not find the WLAN device configuration for instance 0.\n", __FUNCTION__);
		return NOK;
	}

	if( ! (vlynq = PAL_vlynqDevGetVlynq(vlynq_dev)))
	{
		printf("%s: Problems in the WLAN VLYNQ configuration for instance 0.\n", __FUNCTION__);
		return NOK;
	}

	if( ! (PAL_vlynqGetLinkStatus(vlynq))) 
	{
		printf("%s: We did not detect Link for WLAN device instance 0.\n", __FUNCTION__);
		return NOK;
	}
#if (WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1130) 
    if(G_tiwlan_acx_reset_type == TIWLAN_HARD_RESET)
    {
       logMsg("HW ACX and VLYNQ reset!!!\n",0,0,0,0,0,0);
       vlynq_dev_init();
    }
#endif
	if ( PAL_vlynqGetDevBase(vlynq, VLYNQ_ACX111_MEM_OFFSET, MemAddr, vlynq_dev) )
    {
        printf ("%s: Error getting the WLAN mapped memory area from the VLYNQ driver.\n", __FUNCTION__);
        return NOK;
    }
		
	/* Convert to KSEG1 */
	*MemAddr = PHYS_TO_K1( *MemAddr);
	
	if ( PAL_vlynqGetDevBase(vlynq, VLYNQ_ACX111_REG_OFFSET, RegAddr, vlynq_dev) )
    {
        printf ("%s: Error getting the WLAN mapped register area from the VLYNQ driver\n", __FUNCTION__);
        return NOK;
    }
	
	/* Convert to KSEG1 */
	*RegAddr = PHYS_TO_K1( *RegAddr);
   if(G_tiwlan_acx_reset_type == TIWLAN_SOFT_RESET)
    {
        logMsg("SOFT ACX reset!!!\n",0,0,0,0,0,0);
        whal_apiSoftResetAcx(); /* Reset acx */
    }


	printf("%s: MemAddr=0x%x, RegAddr=0x%x\n", __FUNCTION__, *MemAddr, *RegAddr);
    return OK;
}
