/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_trace.h                                               
    \brief This module implements DDA tracing utilities                                                                          
*/
/******************************************************************************/

#ifndef _DDA_TRACE_H
#define _DDA_TRACE_H

#undef PUBLIC

#ifdef _DDA_TRACE_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif


#define DDA_DEBUG

#define DDA_LOG_OFF         0x00000000
#define DDA_LOG_ERR         0x00000001
#define DDA_LOG_EVENT_SET   0x00000002
#define DDA_LOG_EVENT_GET   0x00000004
#define DDA_LOG_POOL        0x00000008
#define DDA_LOG_TASK        0x00000010
#define DDA_LOG_WEND        0x00000020
#define DDA_LOG_IO          0x00000040
#define DDA_LOG_TIMER       0x00000080
#define DDA_LOG_WPAIF       0x00000100
#define DDA_LOG_TRACE       0x00000200
#define DDA_LOG_PROC       	0x00000400
#define DDA_LOG_ALL         0xFFFFFFFF

#ifdef  DDA_DEBUG
	
    #define DDA_LOG(flag, s)            \
        if (DDA_LogLevel & (flag)) {       \
           logMsg((s), 0,0,0,0,0,0); }
	
    #define DDA_LOG_ARG(flag, s, a1, a2, a3, a4, a5, a6)    \
        if (DDA_LogLevel & (flag)) {                           \
           logMsg((s), (int)(a1),(int)(a2),(int)(a3),(int)(a4),(int)(a5),(int)(a6)); }

#else

    #define DDA_LOG(flg, s)
	#define DDA_LOG_ARG(flg, s, a1, a2, a3, a4, a5, a6)

#endif /* DDA_DEBUG */

#ifdef DDA_TRACE
  #define DTR_TRACE_INIT(size,adr)      DTR_TraceInit(size,adr)
  #define DTR_TRACE(type,id,par1,par2)  DTR_Trace( type, id, par1, par2)
  #define DTR_TRACE_APLIC(id,par1,par2) DTR_Trace( DTR_TYPE_APLIC_A, id, par1, par2)
#else
  #define DTR_TRACE_INIT(size,adr)
  #define DTR_TRACE(type,id,par1,par2)
  #define DTR_TRACE_APLIC(id,par1,par2)
#endif

typedef enum 
{	DTR_APPL_EMPTY_A,			/* 0  */
    DTR_APPL_TX_START_A,		/* 1  */
    DTR_APPL_BSS_WLAN_R_A,		/* 2  */
    DTR_APPL_CHKSIZE_F_A,		/* 3  */
    DTR_APPL_RX_INTR_A,         /* 4  */
    DTR_APPL_RX_EVENT_A,        /* 5  */
	DTR_APPL_TX_C_INTR_A,       /* 6  */
	DTR_APPL_SND_Q_OK_A,        /* 7  */
	DTR_APPL_SND_Q_BUSY_A,      /* 8  */
	DTR_APPL_TX_C_EVENT_A,      /* 9  */
	DTR_APPL_TX_E_START_A,      /* 10 */
	DTR_APPL_TX_MASTR_E_A,      /* 11 */
	DTR_APPL_TX_TIMEOUT_A,      /* 12 */
	DTR_APPL_SKB_FAIL_A,        /* 13 */
	DTR_APPL_RX_EN_TO_A,        /* 14 */
	DTR_APPL_WHAL_RX_A,         /* 15 */
	DTR_APPL_MNG_CHECK_A        /* 16 */

}  DTR_APPL_TYPE_E;

typedef enum
{
    DTR_TYPE_FIRST_A,
    DTR_TYPE_GET_A,
    DTR_TYPE_SET_A,
    DTR_TYPE_APLIC_A,
    DTR_TYPE_LAST_A

}   DTR_TYPE_E;


#ifdef TIWLAN_DEBUG
#define bm_init()
#define bm_trace(a, b, c) DTR_TRACE(DTR_TYPE_APLIC_A,(a),(b),(c)) 
#define bm_shutdown()
#define bm_disable()
#else
#define bm_init()
#define bm_trace(a, b, c) 
#define bm_shutdown()
#define bm_disable()
#endif


PUBLIC UINT32 	DDA_LogLevel;

PUBLIC inline 	void 	DTR_SetReportLevel( const UINT32 level );
PUBLIC inline 	UINT32 	DTR_GetReportLevel( void );
PUBLIC STATUS 	DTR_TraceInit( UINT32 maxNumOfRec, UINT32 * traceBufAdr );
PUBLIC void 	DTR_Trace( UINT16 type, UINT16 id, UINT32 par1, UINT32 par2 );
PUBLIC void 	DTR_TraceSet( BOOL status );
PUBLIC void 	DTR_TraceShow( void );
PUBLIC STATUS 	DTR_TraceDelete( void );

#undef PUBLIC
#endif  /* _DDA_TRACE_H */

