/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	dda_wdsDrv.c	  										   */
/*   PURPOSE:	VxWorks DDA implementation of the WDS driver.		       */
/*																		   */
/***************************************************************************/

#define __DDA_WDS_DRV_C__

/***********************************************************
	 Includes.
*******************************************************/

#include "end.h"        /* Common END structures. */
#include "endLib.h"
#include "dda.h"
#include "mainsm.h"
#include "802_11Defs.h"
#include "public_security_keys.h"
#include "tx.h"
#include "dda_wpa_interface.h"
#include "dda_contexts.h"
#include "dda_trace.h"
#include "dda_ioctl.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtIf.h"
#include "mgmtSsidHandler.h"
#include "dda_wpaauth_ioctl.h"
#include "wpaIfVxWorks.h"

#include "wds.h"
#include "wdsDrv.h"
#include "WlanBridgeIf.h"

/***********************************************************
	Definitions
************************************************************/


#define WDS_HEADER_EXTRA_LENGTH (sizeof(dot11_header_T) + sizeof(Wlan_LlcHeader_T))
extern 	UINT32 tiwlan_wdsRecvProtMsgs;
void 	tiwlan_ddaGetWdsCounters(wdsDrvMngrStat_s *pBuff);


/***********************************************************
	Static Variables
************************************************************/

static WBIF_INTERF_T * WdsInterfs[5];
static int  WdsSockFd;
static BOOL WdsInitialized = FALSE;

/***********************************************************
	Local Function Prototypes
***********************************************************/

static int doWdsGenericIoctl(WBIF_INTERF_T    		 * pDev,
							 DDA_IOCTL_GENERIC_PAR_S * pIoctl, 
							 int   cmd );


/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/


/******************************************************************************/

/*! \brief 	Driver receive function
	\param 	pointer to WDS bridge interface
	\param 	pointer to MSDU
	\return result of operation: OK or ERROR.
*/
/******************************************************************************/

STATUS tiwlan_ddaWdsReceive(void *pCookie, mem_MSDU_T * pMSDU) 
{
	int    res = OK;
    EthernetHeader_T * pEthHdr;
    WBIF_INTERF_T * pBrgIf = (WBIF_INTERF_T *) pCookie;
	register mem_BD_T   * pBD = pMSDU->firstBDPtr;

    /* When WDS network interface still down pass EAPOLs over NetLink socket. */
    if (!(END_FLAGS_GET(&pBrgIf->end) & IFF_UP)) {
        pEthHdr = (EthernetHeader_T *) (memMgr_BufData(pBD) + memMgr_BufOffset(pBD));
        if(htons(pEthHdr->TypeLength) == EAPOL_ETHER_TYPE) {
            wpaMessage_T	msg;
    
            msg.type = EAPOL_FRAME_INT_PROF;
            if(wdrv_mgmt_ssidGetInternalSsidIndex(&msg.profile_index) == OK) { 
                msg.content.eapolFrame.eapolDataLen = memMgr_BufLength(pBD);
                memcpy((void *)msg.content.eapolFrame.pEapol, (const void *)pEthHdr, memMgr_BufLength(pBD));
                msg.content.eapolFrame.encrypted = TRUE;
        
                if(wpa_dda_send_msg(RG_IPC_PORT_WPA, (void *) &msg, sizeof(msg)) != OK) {
                    printf("%s: Send to WPA socket failed !\n", __FUNCTION__);
                }
            }
            if( (res = wlan_memMngrFreeMSDU(pMSDU->handle)) != OK)
                printf("%s: wlan_memMngrFreeMSDU failed !\n", __FUNCTION__);
            return (res);
        }
    }

	res = WBIF_RecvPacket( pBrgIf, pMSDU );
    
	return res;
}



/******************************************************************************/

/*! \brief 	Gets address of driver statistics structure
	\param 	pointer to WDS bridge interface
	\return address of driver statistics structure
*/
/******************************************************************************/

WBIF_STATISTICS_T * tiwlan_ddaWdsGetStats(WBIF_INTERF_T * pDev) 
{
	if (pDev)
	{
		return &pDev->statistics;
	}
	else
	{
		return NULL;
	}
}


/******************************************************************************/

/*! \brief 	Processes IOCTL request
	\param 	pBrgIf - pointer to interface
	\param 	pData  - pointer to IOCTL data
	\return status of operation OK or code of error
*/
/******************************************************************************/


STATUS tiwlan_ddaWdsDoIoctl( const WBIF_INTERF_T    * pBrgIf,
							 DDA_IOCTL_DATA_S * pData )
{
 STATUS status;


 switch (pData->dataType)
 {
    
    case DDA_IOCTL_TYPE_GENERIC_A:
        status = doWdsGenericIoctl( (WBIF_INTERF_T *)  pBrgIf, 
									(DDA_IOCTL_GENERIC_PAR_S *) &pData->dataBuf, 
									pData->cmd );
        break;
    
    default:
        DDA_LOG_ARG(DDA_LOG_ERR, 
                    "Func %s: Invalid IOCTL data type %x\n",__FUNCTION__, 
                    pData->dataType, 0,0,0,0 );
        status = ERROR;
 }
  
 return (status);

}


/******************************************************************************/

/*! \brief 	Performs generic IOCTL request
	\param 	pIoctl - pointer to IOCTL data
	\param 	cmd    - IOCTL command value
	\return status of operation OK, ERROR or TIAP_PARAM_ERxxx error code
*/
/******************************************************************************/

static int doWdsGenericIoctl(WBIF_INTERF_T    		 * pDev,
							 DDA_IOCTL_GENERIC_PAR_S * pIoctl, 
							 int   cmd )
{
 int 	status = OK;
 
 if (!pIoctl->buf || !pIoctl->buf_len) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, "Func %s: Invalid IOCTL buf and buf_len\n",
                 __FUNCTION__,0,0,0,0,0);
     return EINVAL;
 }

 switch (pIoctl->cmd) 
 {
	 case TIAP_GENERIC_IOCTL_WDS_LINK_ADD:
		 PAL_memCopy(pDev->linkPartner, pIoctl->mac, ETH_ALEN);
		 wdrv_mgmtWdsLinkAdd(pDev->linkPartner, pIoctl->param1);
		 wdrv_wdsInterfaceCookieAdd(pDev->linkPartner, (void *)pDev);
		 break;
	 
	 case TIAP_GENERIC_IOCTL_WDS_LINK_REMOVE:
		 wdrv_wdsInterfaceCookieRemove(pDev->linkPartner);
		 wdrv_mgmtWdsLinkRemove((char *) pDev->linkPartner);
		 break;
	 
	 case TIAP_GENERIC_IOCTL_WDS_STAT_GET:
		 tiwlan_ddaGetWdsCounters((wdsDrvMngrStat_s *)pIoctl->buf);
		 break;
	 
	 case TIAP_GENERIC_IOCTL_WDS_IFC_STAT_GET:
		 {
			 wdsNetIfcStat_s * pIfcStat = (wdsNetIfcStat_s *)pIoctl->buf;

			 pIfcStat->rx_packets = pDev->statistics.rx_packets;
			 pIfcStat->rx_errors  = 0;			/* unsupported */
			 pIfcStat->rx_dropped = pDev->statistics.rx_drops;
			 pIfcStat->tx_packets = pDev->statistics.tx_packets;
			 pIfcStat->tx_errors  = 0;			/* unsupported */
			 pIfcStat->tx_dropped = pDev->statistics.tx_drops;
		 }
		 break;
	 
	 case TIAP_GENERIC_IOCTL_WDS_DRV_STAT_GET:
		 wdrv_wdsGetStat((wdsDrvStat_s *)pIoctl->buf);
		 break;
	 
	 case TIAP_GENERIC_IOCTL_WDS_BRG_STAT_GET:
		 wdrv_wdsBridgeInfoGet((wdsDrvSwtchingEntry_s *)pIoctl->buf);
		 break;
          
    default:
        DDA_LOG_ARG( DDA_LOG_ERR,"Func %s: Unknown ioctl command %d\n",
                     __FUNCTION__, pIoctl->cmd, 0,0,0,0 );
        status = ERROR;
        break;
 }

 return status;

}



/******************************************************************************/

/*! \brief 	Creates WDS Bridge interface devices
	\param 	number of WDS downlink interfaces
	\return result of operation: OK (0) or ERROR.
*/
/******************************************************************************/

int tiwlan_setupWdsNetDevs(int wdsDlDevsNumber)
{
	WBIF_POOL_DESCR_T 	poolDesc = { 256, 2 };
    WBIF_INTERF_T * 	pBrgIf;
    int i;
    
	WdsSockFd = socket (AF_INET, SOCK_DGRAM, 0);

    if (WdsSockFd == ERROR)
	{         
		printf("%s: error creating WDS socket\n", __FUNCTION__);         
		return (ERROR);
	} 

	/* create uplink device */

	pBrgIf = WBIF_CreateInerface( WDS_DEVNAME_UL, 
								  0 /* unit */, 
								  (WBIF_SEND_FUN_T)  wdrv_wdsDrvPktFromEthrToWlan, 
								  (WBIF_IOCTL_FUN_T) tiwlan_ddaWdsDoIoctl,
								  &poolDesc );
	WdsInterfs[0] = pBrgIf;
	if (pBrgIf == NULL)
	{
		printf("%s: error creating WDS interface %s%u\n", __FUNCTION__,
			   WDS_DEVNAME_UL,0);
		return ERROR;
	}
	
	/* create downlink devices */

    for(i = 0; i < wdsDlDevsNumber; i++) 
	{

		pBrgIf = WBIF_CreateInerface( WDS_DEVNAME_DL, 
									  i /* unit */, 
									  (WBIF_SEND_FUN_T)  wdrv_wdsDrvPktFromEthrToWlan,
									  (WBIF_IOCTL_FUN_T) tiwlan_ddaWdsDoIoctl,
									  &poolDesc );
		WdsInterfs[i+1] = pBrgIf;
		if (pBrgIf == NULL)
		{
			printf("%s: error creating WDS interface %s%u\n", __FUNCTION__,
				   WDS_DEVNAME_DL, i);
			return ERROR;
		}
    }
    WdsInitialized = TRUE;
	return 0;
}


/******************************************************************************/

/*! \brief 	Creates WDS Bridge interface devices
	\param 	number of WDS downlink interfaces
	\return result of operation: OK (0) or ERROR.
*/
/******************************************************************************/

BOOL tiwlan_IsWdsNetDevInitialized( void )
{
	return WdsInitialized;
}

static UINT32 tiwlan_wdsSentProtMsgs = 0;
static UINT32 tiwlan_wdsSentProtMsgsFail = 0;

/******************************************************************************/

/*! \brief 	Sends WDS packet to WLAN Tx module
	\param 	pointer to data buffer
	\param size of data
*/
/******************************************************************************/

void tiwlan_ddaSendWdsPkt(UINT8 *pBuff, UINT32 length)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	mem_BD_T *BD;
    dot11_header_T *pHdr;
    int profileIndex;

    /* Check for recovery before sending packet */
    if(recoveryIsInProcess)
        return;
	/* compute the total len of the frame */
	totalLen = length;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen + MIC_LEN) != OK) 
	{
		printf("%s : wlan_memMngrAllocMSDU failed !\n", __FUNCTION__);
        return;
    }

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = length;

	BD = MSDUPtr->firstBDPtr;
	BD->dataOffset = 0;
	BD->length = 0;

    PAL_memCopy((BD->data + BD->dataOffset), pBuff, length);

	BD->length += length;	


    pHdr = (dot11_header_T *) BD->data;
    if(pHdr->address1.addr[0] == 0xFF) {
        /* Set broadcast flag */
        txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_BRCST_MSDU;
        if(wdrv_mgmt_ssidGetInternalSsidIndex(&profileIndex) != OK)  {
            wlan_memMngrFreeMSDU(MSDUPtr->handle);
            printf("%s wdrv_mgmt_ssidGetInternalSsidIndex returns error.\n", __FUNCTION__);
            return;
        }
        if(wdrv_txSendMsdu(MSDUPtr, profileIndex, txFlags, 0) == OK) {
            tiwlan_wdsSentProtMsgs++;
        }
        else {
            tiwlan_wdsSentProtMsgsFail++;
        }

    }
    else {
        UINT16 staIndex;

        txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU;
        if( wdrv_dbFindStaIndex(pHdr->address1.addr, &staIndex) != OK ) {
            staIndex = INVALID_INDEX;
        }
        else {
            if (wdrv_mgmtIsStationAuthorized(staIndex) == FALSE) {
                staIndex = INVALID_INDEX;
            }
        }

        if(wdrv_txSendMsdu(MSDUPtr, staIndex, txFlags, 0) == OK) {
            tiwlan_wdsSentProtMsgs++;
        }
        else {
            tiwlan_wdsSentProtMsgsFail++;
        }
    }

}

/******************************************************************************/

/*! \brief 	Gets WDS statistics counters
	\param 	pointer to statistics sturcture to fill
*/
/******************************************************************************/


void tiwlan_ddaGetWdsCounters(wdsDrvMngrStat_s *pBuff)
{
    pBuff->sentProtocolMsgs = tiwlan_wdsSentProtMsgs;
    pBuff->sentProtocolMsgsFails = tiwlan_wdsSentProtMsgsFail;
    pBuff->recvProtocolMsgs = tiwlan_wdsRecvProtMsgs;
}

/******************************************************************************/

/*! \brief 	Sends WDS packet to WDS manager over created socket
	\param 	pointer to data buffer
	\param 	size of data
*/
/******************************************************************************/

STATUS tiwlan_ddaRecvWdsPkt(UINT8 *pBuff, UINT32 length)
{
	struct sockaddr_in  wdsAdr;
	int                 sockAdrSize;
	
	/* build WDS server socket address */      
	sockAdrSize = sizeof (struct sockaddr_in);     
    bzero ((char *) &wdsAdr, sockAdrSize);     
	wdsAdr.sin_len = (u_char) sockAdrSize;     
	wdsAdr.sin_family = AF_INET;     
	wdsAdr.sin_port = htons (TIAP_SOCKADDR_WDS_ISC_PORT);      
	
	if ((wdsAdr.sin_addr.s_addr = inet_addr (TIAP_SOCKADDR_WDS_ISC_IP)) == ERROR)
	{         
		printf("%s: inet_addr failed, IP adr %s\n", __FUNCTION__, TIAP_SOCKADDR_WDS_ISC_IP);         
		return (ERROR);         
	}      
	
	/* send message to WDS */      
	if (sendto (WdsSockFd, (caddr_t) pBuff, length, 0,         
				(struct sockaddr *) &wdsAdr, sockAdrSize) == ERROR)         
	{         
		printf("%s: sendto failed, IP adr %s\n", __FUNCTION__, TIAP_SOCKADDR_WDS_ISC_IP);         
		return (ERROR);         
	} 
	
    return OK;
}

/*
 *Function Name:tiwlan_ddaWdsSuspedTransmit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaWdsSuspedTransmit(void)
{
    UINT32 i;
    WBIF_INTERF_T * pDev;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        pDev = (WBIF_INTERF_T *)wdrv_wdsInterfaceCookieGetByIndex(i);
        if(pDev != NULL) {
            if(muxIoctl( pDev->pCookie, EIOCSFLAGS, (char *)(~IFF_UP)) != OK) {
                printf("%s: muxIoctl failed, dev=%s cookie=0x%x cmd=%x data=0x%x\n",
                       __FUNCTION__, pDev->name, pDev->pCookie, EIOCSFLAGS, (~IFF_UP));
            }
        }
    }
}

/*
 *Function Name:tiwlan_ddaWdsResumeTransmit
 *
 *Parameters:
 *
 *Description:
 *
 *Returns:
 *
 */

void tiwlan_ddaWdsResumeTransmit(void)
{
    UINT32 i;
    WBIF_INTERF_T * pDev;

    for (i = 0; i < WDS_LINKS_NUMBER; i++) {
        pDev = (WBIF_INTERF_T *)wdrv_wdsInterfaceCookieGetByIndex(i);
        if(pDev != NULL) {
            if(muxIoctl( pDev->pCookie, EIOCSFLAGS, (char *)IFF_UP) != OK) {
                printf("%s: muxIoctl failed, dev=%s cookie=0x%x cmd=%x data=0x%x\n",
                       __FUNCTION__, pDev->name, pDev->pCookie, EIOCSFLAGS, (~IFF_UP));
            }
        }
    }
}

