/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  utilsTcb.h                                               
    \brief This module implements timer call back services for VxWorks
*/
/******************************************************************************/

#ifndef __UTILS_TCB_H
#define __UTILS_TCB_H

#ifdef _UTILS_TCB_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif


#define MAX_TIMED_CALLBACKS 100

#define MAX_CLOCK 			0x7fffffff

typedef void (*lbt_tcbActTYPE) (UINT32 data);

typedef struct {
  /* call back info                       */
  unsigned long  _msgQ;
  unsigned long  _msgInfo;
  unsigned long  _msgInfoExtended[4];
  int            _isExtended;
  lbt_tcbActTYPE _act;
  UINT32		 _data;
  int            _periodic;
  unsigned int   _periodicDelta;
  unsigned long  _wakeUpTime;    	/* Tick Count To Wake Up   */  
  unsigned       _index;            /* Position In Heap Array  */

} lbt_tcbObjTYPE;

/* function prototypes                    */
PUBLIC long  lbt_tcbInit(unsigned long priority, unsigned long tcbTicks);

/* Constructor of an object               */
PUBLIC lbt_tcbObjTYPE *lbt_tcbNew(unsigned long  msgQ);

/* Destructor Of An Object                */
PUBLIC void lbt_tcbDelete(lbt_tcbObjTYPE *);


/* Adds The TCB Object To The Static Heap */
/* Message To Be Sent After Delta Ticks   */
PUBLIC void lbt_tcbQueue(lbt_tcbObjTYPE *this,unsigned long Delta,
   unsigned long msginfo,int periodic);

/* Adds The TCB Object To The Static Heap */
/* Message To Be Sent After Delta Ticks   */
PUBLIC void lbt_tcbQueueExtended(lbt_tcbObjTYPE *this,unsigned long Delta,
   unsigned long msginfo[4],int periodic);

/* Adds The TCB Object To The Static Heap */
/* act will be called After Delta Ticks   */
PUBLIC void lbt_tcbQueueAct(lbt_tcbObjTYPE *this,unsigned long Delta,
   lbt_tcbActTYPE act, UINT32 data, int periodic);

/* Delete An Element From The Static Heap */
PUBLIC void lbt_tcbCancel(lbt_tcbObjTYPE *this);


PUBLIC void lbt_tcbSuspend(void);
PUBLIC void lbt_tcbResume(void);
PUBLIC void lbt_tcbEnd(void);


#undef PUBLIC
#endif /*  __UTILS_TCB_H */



