/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      	 |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  utilsTcbTest.c                                               
    \brief This module implements unit test functions for timer call backs
*/
/******************************************************************************/

#define TIMER_TEST
#ifdef  TIMER_TEST

/***********************************************************
	defs
************************************************************/

typedef struct 
{
	int eventId;
	int timerN;

} TEST_EVENT_S;

/***********************************************************
	variables
************************************************************/

lbt_tcbObjTYPE * timer1;
lbt_tcbObjTYPE * timer2;
MSG_Q_ID		testMsgQ;
int			    testTaskId;

/***********************************************************
	Local Function Prototypes
***********************************************************/

void testTimerTask( void );
void actionFun(UINT32 data);

/******************************************************************************/

/*! \brief 	Timer test function
*/
/******************************************************************************/


void timerTest( void )
{
  lbt_tcbInit( 100, 1 );

  timer1 = lbt_tcbNew( 0 );
  timer2 = lbt_tcbNew( 0 );

  testMsgQ = msgQCreate(
				10,             /* max messages that can be queued */
				8,        		/* max bytes in a message */
				MSG_Q_PRIORITY  /* message queue options */
				);



  testTaskId = taskSpawn( "tTest1",
						210,           	/* priority of Timer task     */
						0,                     /* task option word           */
						16384,   				/* task stack size (bytes)    */
						(FUNCPTR) testTimerTask,  /* entry point of Timer task  */
						0,0,0,0,0,0,0,0,0,0       /* arguments to pass          */
					   );                       

}

/******************************************************************************/

/*! \brief 	Timer test action function
*/
/******************************************************************************/

void actionFun(UINT32 data)
{
 TEST_EVENT_S event;

 event.eventId = data;
 event.timerN  = data+10;

 msgQSend( testMsgQ, (char *) &event, sizeof(event), 
    	   WAIT_FOREVER, MSG_PRI_NORMAL );
 
}

/******************************************************************************/

/*! \brief 	Timer test task
*/
/******************************************************************************/


void testTimerTask( void )
{
 TEST_EVENT_S event;

 for(;;)
 {
	 msgQReceive( testMsgQ, (char *) &event, sizeof(event), WAIT_FOREVER );
	 logMsg("Receive: eventId %u timer# %u\n", event.eventId, event.timerN,
			0,0,0,0 );

 }

}

/******************************************************************************/

/*! \brief 	Timer start
*/
/******************************************************************************/

void timerStart( void )
{
	lbt_tcbQueueAct( timer1, 100 /* ticks */, actionFun, 
					 1 /* data */, 0 /* periodic */ ); 
}


#endif

