/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanBridgeIf.c                                               
    \brief This module implements WLAN driver interface to the Local Bridge
*/
/******************************************************************************/

#define _WLAN_BRIDGE_IF_C

/***********************************************************
	TODO:
************************************************************/

/***********************************************************
	 Includes.
*******************************************************/

#include "dda.h"
#include "pform_types.h"
#include "ipProto.h"
#include "end.h"        /* Common END structures. */
#include "endLib.h"
#include "etherLib.h"
#include "iosLib.h"
#include "cacheLib.h"
#undef   ETHER_MAP_IP_MULTICAST
#include "etherMultiLib.h"    /* multicast stuff. */
#include "memMngr.h"
#include "dda_trace.h"
#include "dda_ioctl.h"
#include "WlanBridgeIf.h"
#include "cfgMan.h"
#include "sysCfgMan.h"

extern void sysBrgIfaceModeChange(NSP_BRIDGE_IFACE_HND h_iface, 
								  NSP_BRIDGE_IFACE_MODE mode, BOOL turn_on);

/***********************************************************
	Definitions
************************************************************/

#undef WBIF_TEST

#define WDS_HEADER_EXTRA_LENGTH (sizeof(dot11_header_T) + sizeof(Wlan_LlcHeader_T))

/* Configuration items */

#define END_BUFSIZ      (ETHERMTU + ENET_HDR_REAL_SIZ + 6)
#define EH_SIZE         ENET_HDR_REAL_SIZ
#define END_SPEED_10M   10000000 	/* 10Mbs */
#define END_SPEED_100M  100000000   /* 100Mbs */
#define END_SPEED       END_SPEED_10M

#define WBIF_IP_ADR         "127.2.3.4"	/* Dummy parameters */
#define WBIF_SUBNET_MASK    "FFFFFF00"

/***********************************************************
	Macros
***********************************************************/

#define WBIF_ERR(msg)        								\
do { 						          						\
    DDA_LOG_ARG(DDA_LOG_ERR, "%s: line=%d Error: %s\n",  	\
                __FUNCTION__, __LINE__, (msg), 0,0,0);      \
} while(0)

#define WBIF_ERR_ARG(msg,arg)        						\
do { 						          						\
    DDA_LOG_ARG(DDA_LOG_ERR, "%s: line=%d Error: %s %d\n", 	\
                __FUNCTION__, __LINE__, (msg), (arg), 0,0); \
} while(0)

#define WBIF_LOG(msg)					  		\
do { 						          			\
    DDA_LOG_ARG(DDA_LOG_WEND, "%s: %s\n",		\
                __FUNCTION__,(msg),0,0,0,0);	\
} while(0)
					
/***********************************************************
	Static Variables
************************************************************/

static char MacAddress[6] = { 0x00, 0x00, 0x00, 0x00, 0x02, 0x01 }; /* Bridge interface dummy MAC Address */
static char WdsIfName[IFNAMSIZ]; /* to pass to BridgeIfEndLoad on the first call */

/***********************************************************
	Local Function Prototypes
***********************************************************/

static STATUS  AttachToBridgePort( char * devName, UINT32 unitNum );
static int 	   GetMblksLen(M_BLK_ID  pMblk);

static STATUS  BridgeIfEndStart( WBIF_INTERF_T * pBrgIf );
static STATUS  BridgeIfEndStop ( WBIF_INTERF_T * pBrgIf );
static STATUS  BridgeIfEndUnload( WBIF_INTERF_T * pBrgIf );
static int     BridgeIfEndIoctl( WBIF_INTERF_T * pBrgIf, int cmd, caddr_t data );
static STATUS  BridgeIfEndMemInit( WBIF_INTERF_T * pBrgIf );
static STATUS  BridgeIfEndMCastAdd( WBIF_INTERF_T* pBrgIf, char* pAddress );
static STATUS  BridgeIfEndMCastDel( WBIF_INTERF_T* pBrgIf, char* pAddress );
static STATUS  BridgeIfEndMCastGet( WBIF_INTERF_T* pBrgIf, MULTI_TABLE* pTable );
static STATUS  BridgeIfEndPollSend( WBIF_INTERF_T* pBrgIf, M_BLK_ID pBuf );
static STATUS  BridgeIfEndPollRcv ( WBIF_INTERF_T* pBrgIf, M_BLK_ID pBuf );
static STATUS  BridgeIfEndSend( WBIF_INTERF_T* pBrgIf, M_BLK_ID pBuf );

/* END Specific interfaces. */

static END_OBJ * BridgeIfEndLoad(char * initString, void * par );

static NET_FUNCS BridgeIfEndFuncTable =
{
	(FUNCPTR) BridgeIfEndStart,	  	/* Function to start the device 		*/
	(FUNCPTR) BridgeIfEndStop,	  	/* Function to stop the device 			*/
	(FUNCPTR) BridgeIfEndUnload,  	/* Unloading function for the driver 	*/
	(FUNCPTR) BridgeIfEndIoctl,	  	/* Ioctl function for the driver 		*/
	(FUNCPTR) BridgeIfEndSend,	  	/* Send function for the driver 		*/
	(FUNCPTR) BridgeIfEndMCastAdd,	/* Multicast add function 				*/
	(FUNCPTR) BridgeIfEndMCastDel,	/* Multicast delete function  			*/
	(FUNCPTR) BridgeIfEndMCastGet,	/* Multicast retrieve function  		*/
	(FUNCPTR) BridgeIfEndPollSend,	/* Polling send function 				*/
	(FUNCPTR) BridgeIfEndPollRcv,   /* Polling receive function 			*/
	endEtherAddressForm,	  		/* Put address info into a NET_BUFFER   */
	endEtherPacketDataGet,	  		/* Get pointer to data in NET_BUFFER    */
	endEtherPacketAddrGet		 	/* Get packet addresses 				*/
};


/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Creates Bridge Interface END driver
	\return Pointer to created interface device structure
*/
/******************************************************************************/

WBIF_INTERF_T *	WBIF_CreateInerface( const char * 				ifName,
									 const UINT32 		 	 	unitNum,
									 const WBIF_SEND_FUN_T 	 	pSendFun,
									 const WBIF_IOCTL_FUN_T 	pIoctlFun,
									 const WBIF_POOL_DESCR_T * 	pPoolDesc )
{
 END_OBJ *        	pCookie = NULL;
 WBIF_INTERF_T *	pBrgIf;

 pBrgIf = (WBIF_INTERF_T *) calloc( sizeof(WBIF_INTERF_T), 1 );
 
 if (pBrgIf == NULL)
 {
    WBIF_ERR("Can't allocate memory");
    return NULL;
 }
 
 pBrgIf->unit = unitNum;
 strncpy(pBrgIf->name, ifName, IFNAMSIZ);
 strncpy(WdsIfName, ifName, IFNAMSIZ);
 pBrgIf->poolDesc.numOfBlocks 	= pPoolDesc->numOfBlocks;
 pBrgIf->poolDesc.numOfClusters = pPoolDesc->numOfClusters;
 pBrgIf->pSendFun = pSendFun;
 pBrgIf->pIoctlFun= pIoctlFun;
 
 pCookie = muxDevLoad(	pBrgIf->unit,
						BridgeIfEndLoad,
						"1",				/* Load string 		 */
						TRUE, 				/* End loan    		 */
						pBrgIf 				/* parameter to pass */ 
					 );

 if (pCookie == NULL)
 {
		WBIF_ERR("muxDevLoad failed");
		goto CreateInerfaceErrorExit;
 }
 pBrgIf->pCookie = pCookie;

 if (muxDevStart(pCookie) == ERROR)
 {
	 WBIF_ERR("muxDevStart failed");
	 goto CreateInerfaceErrorExit;
 }

 if (AttachToBridgePort( pBrgIf->name, unitNum ) != OK)
 {
	 WBIF_ERR("AttachToBridgePort failed");
	 goto CreateInerfaceErrorExit;
 }

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: BridgeIf END loaded OK, pCookie=%x\n",
			 __FUNCTION__, pCookie, 0,0,0,0);

 return pBrgIf;

CreateInerfaceErrorExit:
 
 WBIF_Unload( pBrgIf );
 return NULL;

}

/******************************************************************************/

/*! \brief 	Checks whether Bridge Interface END driver has been initialized
	\return status TRUE if driver initiated or FALSE if not
*/
/******************************************************************************/

BOOL WBIF_IsInitialized( const WBIF_INTERF_T * pBrgIf )
{
	return (pBrgIf && ( END_FLAGS_GET((END_OBJ *) (&pBrgIf->end)) & IFF_RUNNING ));
}

/******************************************************************************/

/*! \brief 	Unloads Bridge Interface END driver
	\return status of operation OK or ERROR
*/
/******************************************************************************/


STATUS WBIF_Unload( WBIF_INTERF_T * pBrgIf )
{
 STATUS status;

 status = muxDevUnload( pBrgIf->name, pBrgIf->unit );
 if (status != OK)
 {
   	WBIF_ERR("muxDevUnload failed");
 }

 return( status );
}


/******************************************************************************/

/*! \brief 	Initialize the driver and device
	\param 	pointer to initializing string
	\param 	optional parameter
	
	\return An END object pointer or NULL on error

*/
/******************************************************************************/



static END_OBJ * BridgeIfEndLoad(char* initString, void * par )
{
 WBIF_INTERF_T * pBrgIf = (WBIF_INTERF_T *) par;

 if (!initString[0])
 {
		WBIF_LOG("Return if name");
		bcopy( WdsIfName, initString, strlen(WdsIfName) + 1);
		return(END_OBJ*)0;
 }

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: Loading BridgeIf END, par=%x\n",
			 __FUNCTION__, par, 0,0,0,0);

 if( END_OBJ_INIT( &pBrgIf->end, (DEV_OBJ *)pBrgIf, pBrgIf->name,
				    pBrgIf->unit, &BridgeIfEndFuncTable,
				    "BridgeIf END Driver.") == ERROR  || 
     END_MIB_INIT( &pBrgIf->end, M2_ifType_ethernet_csmacd,
				   (UCHAR*)MacAddress, 6, ETHERMTU, END_SPEED) == ERROR)
 {
		WBIF_ERR( "END_OBJ_INIT || END_MIB_INIT failed\n");
		return NULL;
 }

	/* Perform memory allocation/distribution */
 if (BridgeIfEndMemInit (pBrgIf) == ERROR)
 {
	WBIF_ERR("Memory initialization failed");
	return NULL;
 }

	/* set the flags to indicate readiness */
 END_OBJ_READY( &pBrgIf->end, IFF_RUNNING | IFF_NOTRAILERS | /* IFF_UP | */
				IFF_BROADCAST | IFF_MULTICAST );

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: Initialized OK pBrgIf=%x End=%x\n",
			 __FUNCTION__, pBrgIf, &pBrgIf->end, 0,0,0);
 
 return( &pBrgIf->end );

}

/******************************************************************************/

/*! \brief 	Initialize memory for the chip
	\param 	pointer to END device control structure
	
	\return An END object pointer or NULL on error

*/
/******************************************************************************/


static STATUS BridgeIfEndMemInit( WBIF_INTERF_T * pBrgIf )
{
 M_CL_CONFIG memConfig;
 CL_DESC 	 ClDescTbl[] = {{ETHERMTU + EH_SIZE + 6, /* cluster size 		*/
							 0, 					 /* number of clusters 	*/
							 NULL, 				  	 /* memory area			*/
							 0 					  	 /* memory size        	*/
							}};
 int  ClDescTblNumEnt = (NELEMENTS(ClDescTbl));
	 

 if ((pBrgIf->end.pNetPool = malloc (sizeof(NET_POOL))) == NULL)
 {
	 WBIF_ERR("Memory allocation for pool failed");	 
	 return(ERROR);
 }

 memConfig.mBlkNum  = pBrgIf->poolDesc.numOfBlocks;
 ClDescTbl[0].clNum = pBrgIf->poolDesc.numOfClusters;
 memConfig.clBlkNum = pBrgIf->poolDesc.numOfBlocks;

	/* Calculate the total memory for all the M-Blks and CL-Blks. */

 memConfig.memSize = (memConfig.mBlkNum *
					 (MSIZE + sizeof (long))) +
					 (memConfig.clBlkNum *
					 (CL_BLK_SZ + sizeof(long)));

 if ((memConfig.memArea = (char *) memalign(sizeof(long),
											memConfig.memSize)) == NULL)
 {
    WBIF_ERR("Memory area allocation failed");
   	return(ERROR);
 }

	/* Calculate the memory size of all the clusters. */
 ClDescTbl[0].memSize = (ClDescTbl[0].clNum * (END_BUFSIZ + 8)) + sizeof(int);

	/* Allocate the memory for the clusters from cache safe memory. */

 ClDescTbl[0].memArea = (char *) cacheDmaMalloc (ClDescTbl[0].memSize);

 if (ClDescTbl[0].memArea == NULL)
 {
		WBIF_ERR("Cluster pool memory allocation failed");
		return(ERROR);
 }

	/* Initialize the memory pool. */

 if (netPoolInit(pBrgIf->end.pNetPool, &memConfig,
				 &ClDescTbl[0], ClDescTblNumEnt, NULL) == ERROR)
 {
	WBIF_ERR( "netPoolInit failed");
	return(ERROR);
 }

 if ((pBrgIf->poolId = netClPoolIdGet( pBrgIf->end.pNetPool,
   										 100 /* size */, FALSE)) == NULL)
 {
	WBIF_ERR( "netClPoolIdGet failed");
   	return(ERROR);
 }

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: Memory setup complete PoolID=%x\n",
			 __FUNCTION__, pBrgIf->poolId, 0,0,0,0);

 return OK;
}

/******************************************************************************/

/*! \brief 	start the device
	\param 	pointer to END device control structure
	
	\return OK

*/
/******************************************************************************/

static STATUS BridgeIfEndStart( WBIF_INTERF_T *pBrgIf )
{
//  END_FLAGS_SET (&pBrgIf->end, IFF_UP | IFF_RUNNING);
 WBIF_LOG("Bridge interface get started");
 
 return(OK);
}

/******************************************************************************/

/*! \brief 	Bridge Interface END receive function
	\param 	pointer to sent MSDU
	
	\return status of operation OK or ERROR

*/
/******************************************************************************/


STATUS WBIF_RecvPacket( WBIF_INTERF_T 	  * pBrgIf, 
						const  mem_MSDU_T * pMsdu )
{
 M_BLK_ID       result;
 M_BLK_ID       pMblk = NULL;
 M_BLK_ID       pMblkEnd = NULL;
 CL_BLK_ID      pClBlk;
 UINT16         pktLength, length;
 unsigned long  total_packet_length;
 int            clusterIndex;
 mem_BD_T*      pBD;

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: pBrgIf=%x pMsdu=%x\n",
			 __FUNCTION__, pBrgIf, pMsdu, 0,0,0);
	
 if (!(END_FLAGS_GET(&pBrgIf->end) & IFF_UP))
 {
	if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
	{
		WBIF_ERR("wlan_memMngrFreeMSDU failed");
	}
	pBrgIf->statistics.rx_drops++;
	return NOK;
 }

	/* Receive the data, and initialize total_packet_length. */
 pBD = pMsdu->firstBDPtr;
 total_packet_length = memMgr_MsduDataSize(pMsdu);
	
	/* W/A if we get somehow Msdu with length=0, we get exception */
 if ( total_packet_length == 0)
 {
	WBIF_ERR("Packet length is 0");
   	if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
   	{
		WBIF_ERR("wlan_memMngrFreeMSDU failed");
   	}
   	return NOK;
 }

 clusterIndex = 0;
 pktLength = 0;
 result = NULL;
 
 while ( pktLength < total_packet_length)
 {
   	char*  ethernetPtr;

   	clusterIndex++;
   	length = memMgr_BufLength(pBD);	/* length of data in current BD */
   	ethernetPtr = (char*)(memMgr_BufData(pBD) + memMgr_BufOffset(pBD));

	/* The data packet is already in the cluster now. Get a cluster block to make an MBUF */
	if ((pClBlk = netClBlkGet (pBrgIf->end.pNetPool, M_DONTWAIT)) == NULL)
	{
		/* Free the result in case we are half-way through the packet construction */
		if ( result != NULL )
		{
			netMblkClFree(result);  /* Free the packet */
		}
		if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
		{
			WBIF_ERR("wlan_memMngrFreeMSDU failed");
		}
		pBrgIf->statistics.rx_drops++;
		return(ERROR);
	}

		/* Get an M_BLK_ID and link it to the cluster */
	if ((pMblk = netMblkGet (pBrgIf->end.pNetPool, M_DONTWAIT, MT_DATA)) == NULL)
	{
		/* Free the cluster control block and the data buffer */
		WBIF_ERR("netMblkGet failed");
		netClBlkFree (pBrgIf->end.pNetPool, pClBlk);
		if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
		{
			WBIF_ERR("wlan_memMngrFreeMSDU failed");
		}
		pBrgIf->statistics.rx_drops++;
		return ERROR;
	}

	pMblk->mBlkHdr.mFlags = 0;
		
		/* Remember the packet handle */
	if ( pktLength == 0 )
	{
		/* result is the first MBUF block */
		result = pMblk;
		pMblkEnd = pMblk;
		result->mBlkHdr.mFlags |= M_PKTHDR;	/* start of record */
	}

		/* Join the cluster to the MBlock */
	netClBlkJoin( pClBlk, ethernetPtr, length, wlan_memMngrFreeMSDU,
				  (clusterIndex == 1)? pMsdu->handle : WLAN_DRV_NULL_MEM_HANDLE, 0, 0);
	netMblkClJoin (pMblk, pClBlk);
	pMblk->mBlkHdr.mLen     = length;
	pMblk->mBlkHdr.mNextPkt = NULL;

		/* Link this MBUF block to the previous one */
	if (pktLength != 0)
	{
		pMblkEnd->mBlkHdr.mNext = pMblk;
		pMblkEnd = pMblk;
	}

	pktLength += length; /* Add length of current BD */
	pBD = memMgr_BufNext(pBD);
 
 } /* while */

	/* Total packet length */
 result->mBlkPktHdr.len   = pktLength;

	/* Set the end of record flag */
 pMblk->mBlkHdr.mFlags |= M_EOR;	/* end of record */

	/* Mark the end of the received packet */
 pMblk->mBlkHdr.mNext = NULL;

 pBrgIf->statistics.rx_packets++;
 
	/* Call the upper layer's receive routine. */
 END_RCV_RTN_CALL(&pBrgIf->end, result);

 return OK;


}

/******************************************************************************/

/*! \brief 	Bridge Interface END send routine
	\param 	pointer to END device structure
	\param 	pointer to MBLK to send
	
	\return status of operation OK or ERROR

*/
/******************************************************************************/

static STATUS BridgeIfEndSend( WBIF_INTERF_T * 	pBrgIf,	/* device ptr   */
							   M_BLK_ID     	pMblk	/* data to send */)
{
 mem_MSDU_T*    pMSDU = NULL;
 mem_BD_T*      pBD = NULL;
 mem_BD_T*      pCurrentMsduBd;
 int            dataLen, totalLen;

 DDA_LOG_ARG(DDA_LOG_WEND, "%s: pBrgIf=%x pMblk=%x\n",
			 __FUNCTION__, pBrgIf, pMblk, 0,0,0);
	
 if (!(END_FLAGS_GET(&pBrgIf->end) & IFF_UP))
 {
	pBrgIf->statistics.tx_drops++;
	netMblkClChainFree (pMblk);
	return NOK;
 }

 dataLen = GetMblksLen( pMblk );

 totalLen = dataLen + WDS_HEADER_EXTRA_LENGTH; /* extra length for the 802.11 Header and LLC */
 
 if (totalLen > DATA_BUF_LEN)
 {
	WBIF_ERR_ARG( "Buffer is too large", totalLen);
	pBrgIf->statistics.tx_drops++;
	netMblkClChainFree( pMblk );
	return NOK;
 }

 wlan_memMngrAllocMSDU (&pMSDU, WDS_HEADER_EXTRA_LENGTH);
 if ((pMSDU == NULL) ||
     (pMSDU->firstBDPtr == NULL) ||
	 (pMSDU->firstBDPtr->dataBuf == NULL))
 {
	netMblkClChainFree (pMblk);
	pBrgIf->statistics.tx_drops++;
	return NOK;
 }
	
 pMSDU->dataLen  = dataLen;
 pMSDU->freeFunc = (ap_FreeMemFunc)netMblkClChainFree;
 pMSDU->freeArgs[0] = (UINT32)pMblk;
 pMSDU->firstBDPtr->bufToken = 0;

 wlan_memMngrAllocBDs(1, &pBD);
 if (pBD == NULL)
 {
   	if (wlan_memMngrFreeMSDU(pMSDU->handle) != OK)
   	{
   		WBIF_ERR("wlan_memMngrFreeMSDU failed");
   	}
	netMblkClChainFree (pMblk);
	pBrgIf->statistics.tx_drops++;
	return NOK;
 }

 pBD->data = pMblk->mBlkHdr.mData;
 pBD->length = pMblk->mBlkHdr.mLen;

 pMSDU->firstBDPtr->nextBDPtr = pBD;

	/*
	 * Add other m-blocks if we didn't unite them to one buffer 
	 */

 pCurrentMsduBd = pBD;
 while ((pMblk = pMblk->mBlkHdr.mNext) != NULL)
 {
	if (pMblk->mBlkHdr.mLen == 0)
		break;

	/* Allocate new DB for the new block */
	wlan_memMngrAllocBDs(1, &pBD);
	if (pBD == NULL)
	{
		if (wlan_memMngrFreeMSDU(pMSDU->handle) != OK)
		{
			WBIF_ERR("wlan_memMngrFreeMSDU failed");
		}
		netMblkClChainFree (pMblk);
		pBrgIf->statistics.tx_drops++;
		return NOK;
	}
		/* Set pointer to the data and the data length */
	pBD->data = pMblk->mBlkHdr.mData;
	pBD->length = pMblk->mBlkHdr.mLen;

		/* Concatinate the new BD */
	pCurrentMsduBd->nextBDPtr = pBD;
	pCurrentMsduBd = pBD;

		/* Modify MSDU last BD pointer */
	pMSDU->lastBDPtr = pBD;   
 }

 DDA_LOG_ARG( DDA_LOG_WEND, "%s: Send pMsdu=%x\n",
			  __FUNCTION__, pMSDU, 0,0,0,0);
 
 pBrgIf->statistics.tx_packets++;
 
 if (pBrgIf->pSendFun)  
	 (pBrgIf->pSendFun)(pMSDU, pBrgIf->linkPartner);
 
 return OK;

}/* BridgeIfEndSend */

/******************************************************************************/

/*! \brief 	Bridge Interface END IOCTL dummy routine
	\param 	pointer to END device structure
	\param 	IOCTL command
	\param 	IOCTL data
	
	\return status of operation OK
*/
/******************************************************************************/


static int BridgeIfEndIoctl(
					 WBIF_INTERF_T * pBrgIf,  /* device receiving command 	*/
					 int 			 cmd,	  /* ioctl command code 		*/
					 caddr_t 		 data	  /* command argument 			*/
					 )
{
 STATUS error = 0;
 int    value;

 DDA_LOG_ARG( DDA_LOG_WEND, "%s: cmd=%x data=%x\n",
			  __FUNCTION__, cmd, data, 0,0,0);
	

 switch (cmd)
 {
	 case EIOCSFLAGS:
		 value = (long)data;
		 DDA_LOG_ARG(DDA_LOG_IO, "WBIF IOCTL EIOCSFLAGS cmd=%x ifrAdr=0x%x par=0x%x\n", 
					 cmd, data, 0,0,0,0);

		 if (value < 0)
		 {
			 value = -(value);
			 value--;
			 END_FLAGS_CLR (&pBrgIf->end, value);
			 if (value | IFF_UP)
			 {
				 sysBrgIfaceModeChange ( (NSP_BRIDGE_IFACE_HND) pBrgIf->bridgeIfHandle,
										 NSP_BRIDGE_IFACE_MODE_UP, FALSE );
			 }
		 }
		 else
		 {
			 END_FLAGS_SET (&pBrgIf->end, value);
			 if (value | IFF_UP)
			 {
				 sysBrgIfaceModeChange ( (NSP_BRIDGE_IFACE_HND) pBrgIf->bridgeIfHandle,
										 NSP_BRIDGE_IFACE_MODE_UP, TRUE );
			 }
		 }
		 break;

	 case EIOCGFLAGS:
		 DDA_LOG_ARG(DDA_LOG_IO, "WBIF IOCTL EIOCGFLAGS cmd=%x ifrAdr=0x%x par=0x%x\n", 
					 cmd, data, 0,0,0,0);
		 *(int *)data = END_FLAGS_GET(&pBrgIf->end);
		 break;

	 case EIOCTI:		 /* TI WLAN Configuration */

		DDA_LOG_ARG(DDA_LOG_IO, "WBIF IOCTL EIOCTI cmd=%x ifrAdr=0x%x par=0x%x\n", 
					cmd, data, 0,0,0,0);
		
		if (pBrgIf->pIoctlFun)
		{
			error = (pBrgIf->pIoctlFun)( (void *) pBrgIf, (DDA_IOCTL_DATA_S *) data);
		}		
		break;
	 
	 default:
		 error = OK;
  }

  return(error);

}


static STATUS BridgeIfEndPollRcv(
						   WBIF_INTERF_T * 	pBrgIf,  /* device to be polled */
						   M_BLK_ID      	pMblk	 /* ptr to buffer 		 */
						   )
{
	return(OK);
}

static STATUS BridgeIfEndPollSend(
						   WBIF_INTERF_T * pBrgIf, 	/* device to be polled */
						   M_BLK_ID    	   pMblk 	/* packet to send */
						   )
{
	return(OK);
}

static STATUS BridgeIfEndMCastAdd(
						   WBIF_INTERF_T * 	pBrgIf,  /* device pointer 	*/
						   char * 			pAddress /* new address to add */
						   )
{
	return(OK);
}

static STATUS BridgeIfEndMCastDel (
							WBIF_INTERF_T * pBrgIf,  /* device pointer 	   */
							char * 			pAddress /* address to be deleted */
							)
{
	return(OK);
}

static STATUS BridgeIfEndMCastGet (
							WBIF_INTERF_T * pBrgIf,  /* device pointer */
							MULTI_TABLE * 	pTable	 /* address table to be filled in */
							)
{
	return(OK);
}

static STATUS BridgeIfEndStop (
						WBIF_INTERF_T *pBrgIf	  /* device to be stopped */
						)
{
	STATUS result = OK;
	return(result);
}

static STATUS BridgeIfEndUnload (
						  WBIF_INTERF_T* pBrgIf	/* device to be unloaded */
						  )
{
	return(OK);
}


/******************************************************************************/

/*! \brief 	Sets bridge interface UP flag
	\param 	pointer to bridge interface device control structure
*/
/******************************************************************************/

STATUS WBIF_InterfUp( WBIF_INTERF_T * pBrgIf )
{
	if (pBrgIf)
	{
		if ( muxIoctl( pBrgIf->pCookie, EIOCSFLAGS, (caddr_t) IFF_UP) != OK )
		{
			WBIF_ERR("muxIoctl failed");
			return ERROR;
		}
		return OK;
	}
	else
	{
		return ERROR;
	}
}

/******************************************************************************/

/*! \brief 	Clears bridge interface UP flag
	\param 	pointer to bridge interface device control structure
*/
/******************************************************************************/

STATUS WBIF_InterfDown( WBIF_INTERF_T * pBrgIf )
{
	if (pBrgIf)
	{
		if ( muxIoctl( pBrgIf->pCookie, EIOCSFLAGS, (caddr_t) ~IFF_UP) != OK )
		{
			WBIF_ERR("muxIoctl failed");
			return ERROR;
		}
		return OK;
	}
	else
	{
		return ERROR;
	}
}


/******************************************************************************/

/*! \brief 	Clears bridge interface UP flag
	\param 	pointer to bridge interface device control structure
*/
/******************************************************************************/

STATUS WBIF_GetFlags( WBIF_INTERF_T * pBrgIf, UINT32 * pFlags )
{
	UINT32 pFlagsBuf;

	if (pBrgIf)
	{
		if ( muxIoctl( pBrgIf->pCookie, EIOCGFLAGS, (caddr_t) &pFlagsBuf) != OK )
		{
			WBIF_ERR("muxIoctl failed");
			return ERROR;
		}
		* pFlags = pFlagsBuf;
		return OK;
	}
	else
	{
		return ERROR;
	}
}


/******************************************************************************/

/*! \brief 	Gets bridge interface statistics
	\param 	pointer to bridge interface device control structure
	\param 	pointer to bridge interfaces statistics structure to copy results
*/
/******************************************************************************/

void  WBIF_GetStatistics( const WBIF_INTERF_T 	* pBrgIf,
						  WBIF_STATISTICS_T 	* pStatistics)
{
	taskLock();
	memcpy( pStatistics, &pBrgIf->statistics, sizeof(*pStatistics) ); 
	taskUnlock();
	
}

/******************************************************************************/

/*! \brief 	Clears bridge interface statistics
	\param 	pointer to bridge interface device control structure
*/
/******************************************************************************/

void  WBIF_ResetStatistics( WBIF_INTERF_T * pBrgIf )
{
	taskLock();
	memset( &pBrgIf->statistics, 0, sizeof(pBrgIf->statistics) ); 
	taskUnlock();
	
}

/******************************************************************************/

/*! \brief 	Clears bridge interface statistics
	\param 	pointer to bridge interface device control structure
*/
/******************************************************************************/

void  WBIF_ShowStatistics( const WBIF_INTERF_T * pBrgIf )
{
 printf("Rx: packets=%u drops=%u\n", 
		pBrgIf->statistics.rx_packets,
		pBrgIf->statistics.rx_drops );

 printf("Tx: packets=%u drops=%u\n",
		pBrgIf->statistics.tx_packets,
		pBrgIf->statistics.tx_drops );
	
}

/******************************************************************************/

/*! \brief 	Gets MBLK length
	\param 	pointer to MBLK
	\return MBLK data length
*/
/******************************************************************************/


static int GetMblksLen(M_BLK_ID  pMblk)
{
	int totalLen=0;

	while (pMblk)
	{
		totalLen += pMblk->mBlkHdr.mLen;
		pMblk = pMblk->mBlkHdr.mNext;
	}

	return totalLen;
}


extern STATUS sysBrgRcvRtn(END_OBJ* pEndObj, M_BLK_ID pMblk);
extern STATUS sysBrgEndConnect(END_OBJ* pEndObj, void* rcvRtn, void* hnd);
extern NSP_BRIDGE_IFACE_HND sysBrgCreateIface( 
								PORT_CONFIG_T * ptr_port_config,
								END_OBJ       * pEndObj,
								NSP_BRIDGE_IFACE_SEND_RTN  end_drv_send );

/******************************************************************************/

/*! \brief 	Attaches Wlan END driver to Local Bridge
	\return status of operation OK or ERROR
*/
/******************************************************************************/


static STATUS AttachToBridgePort( char * devName, UINT32 unitNum )
{
 PORT_CONFIG_T 	 portConfig[2];
 PORT_CONFIG_T * pConf;
 END_OBJ 	   * pEndObj;
 NSP_BRIDGE_IFACE_HND  h_iface;

 pEndObj = endFindByName( devName, unitNum );

 if (!pEndObj)
 {
	 DDA_LOG_ARG( DDA_LOG_ERR, "%s: END %s unit %u is not found\n", __FUNCTION__, 
				  devName, unitNum, 0,0,0 );
	 return ERROR;
 }
				/* Initialize the PORT configuration block. */
 memset ((char *)portConfig, 0, sizeof (portConfig));

 pConf = &portConfig[0];
 strcpy (pConf->port_name, devName);
 pConf->unit_number = unitNum;
 strcpy ((char *)pConf->ip_address, WBIF_IP_ADR);
 strcpy ((char *)pConf->subnet_mask,WBIF_SUBNET_MASK);       
 memcpy ((char *)pConf->macAddress, MacAddress, 6);
 pConf->portMedia = WIRELESS; /* VIRTUAL; */
 pConf->protocol = CSMACD;
 pConf->macSpecific.pAtm = NULL;
 pConf->macSpecific.pUsb = NULL;
 pConf->flags = NSP_MANAGED_PORT | NSP_BRIDGED_PORT;
 pConf->sys_information = sysInitializePortInfo (pConf);
 ((SYS_END_SPECIFIC_INFO_T*) pConf->sys_information)->
							 end_information.endLoadFunc = BridgeIfEndLoad;
 pConf->pNextPort = NULL;

	 
 h_iface = sysBrgCreateIface(pConf, pEndObj, (NSP_BRIDGE_IFACE_SEND_RTN)pEndObj->pFuncTable->send);
 if (!h_iface)
 {
	 DDA_LOG_ARG( DDA_LOG_ERR, "%s: failed to create Bridge interface\n", __FUNCTION__, 
				  0,0,0,0,0 );
	 return ERROR;
 }
       /* Change the receive routine in the END to point to the 
	      BRIDGE receive instead of the IP stack. */
 sysBrgEndConnect (pEndObj, (void*)sysBrgRcvRtn, h_iface);
 
 ((WBIF_INTERF_T *) pEndObj)->bridgeIfHandle = h_iface;

 return OK;
}

#ifdef WBIF_TEST
#	include "wlanBridgeIfTest.c"
#endif

