/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanEndTest.h                                               
    \brief This module implements WLAN END driver unit testing
*/
/******************************************************************************/


/******************************************************************************/

/*! \brief 	Devides MBLK cluster into 3 clusters and MBLKS
	\param 	pointer to END device control structure
	\param 	pointer to MBLK
	
	\return pointer to MBLKs

The function is ONLY for DEBUG purspose
*/
/******************************************************************************/

/* try 22 for needToCopyMblksment debug */
int dbglen = 0;

M_BLK_ID  DivideMblk(AP_END_DEVICE * pDrvCtrl, M_BLK_ID pMblk)
{
	int i;
	/* int numOfMblks = 3; */
	int  totallen = pMblk->mBlkHdr.mLen;
	CL_BLK_ID      pClBlk;
	M_BLK_ID     pMblkEnd,pMblkTmp,pMblkFirst=NULL;
	char  *pData = pMblk->mBlkHdr.mData;
	char  *cluster;
	int len = dbglen;

	/*    printf("totalLen: %d\n",totallen);*/

	for (i=0; i<3 /*numOfMblks*/ ;i++)
	{
		if ( (cluster = netClusterGet( pDrvCtrl->end.pNetPool,
									   pDrvCtrl->pClPoolId)) != NULL)
		{
			if ((pClBlk = netClBlkGet (pDrvCtrl->end.pNetPool, M_DONTWAIT)) != NULL)
			{
				if ((pMblkTmp = netMblkGet (pDrvCtrl->end.pNetPool, M_DONTWAIT, MT_DATA)) != NULL)
				{
					if (!dbglen)
					{
						if (i==0) len = 26;
						if (i==1) len = 20;
						if (i==2) len = totallen;
					}


					/* copy data */
					memcpy(cluster,pData,len);
					/* connect cluster to cluster block */
					netClBlkJoin (pClBlk, cluster, len, pMblk->pClBlk->pClFreeRtn, pMblk->pClBlk->clFreeArg1, pMblk->pClBlk->clFreeArg2, pMblk->pClBlk->clFreeArg3);

					/* modified Mblk header */

					pMblkTmp->mBlkHdr.mData = pData;
					pMblkTmp->mBlkHdr.mLen = len;
					pMblkTmp->mBlkHdr.mFlags = pMblk->mBlkHdr.mFlags;
					pMblkTmp->mBlkHdr.mNextPkt = pMblk->mBlkHdr.mNextPkt;  

					if (i)
					{
						pMblkEnd->mBlkHdr.mNext = pMblkTmp;
						pMblkEnd = pMblkTmp;
					}
					else
					{
						pMblkFirst = pMblkEnd = pMblkTmp;
						pMblkFirst->mBlkPktHdr.len = pMblk->mBlkPktHdr.len;

					}

					netMblkClJoin (pMblkTmp, pClBlk);
				}
			}
		}
		else
		{
			puts("can't get cluster | mblk | Clblk");
			break;
		}

		pData += len;
		totallen -= len;
		if (i==1) len = totallen;
	}
	if (pMblkEnd)
		pMblkEnd->mBlkHdr.mNext = NULL;

	return pMblkFirst;
}




int     WEND_SockFd = 0;

/******************************************************************************/

/*! \brief 	Sends IOCTL request via socket
	
	\return OK or ERROR

*/
/******************************************************************************/



STATUS WEND_Socket( void )
{
	int           err;
	int           sockFd;
	struct ifreq  ifr;
	u_long        ifAddr;

	/*
	 * Convert string to int
	 */

	ifAddr = inet_addr(WEND_IP_ADR);

	bzero ( (char *)&ifr, sizeof( struct ifreq ) );
	strcpy(ifr.ifr_name, "wlan0");

	/* the interface address to delete is ifAddr */
	ifr.ifr_addr.sa_len = sizeof( struct sockaddr_in );
	((struct sockaddr_in *)&ifr.ifr_addr)->sin_family = AF_INET;
	((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr = ifAddr;

	if ( (sockFd = socket( AF_INET, SOCK_RAW, 0 )) == NULL )
	{
		DDA_LOG_ARG( DDA_LOG_WEND, "WEND_Ioctl can't create socket err=%x\n",
					 sockFd, 0,0,0,0,0 );
		return( ERROR );
	}
	WEND_SockFd = sockFd; 
	return( sockFd );
}

/******************************************************************************/

/*! \brief 	Sends IOCTL request via socket
	\param 	cmd  - IOCTL command value
	\param 	data - IOCTL data
	
	\return OK or ERROR

*/
/******************************************************************************/

STATUS WEND_Ioctl(int cmd, int data)
{
	int           err;
	int           sockFd;
	struct ifreq  ifr;
	u_long        ifAddr;


	DDA_LOG_ARG( DDA_LOG_WEND, "WEND_Ioctl ifrAdr=%x data=%x\n",
				 &ifr, data, 0,0,0,0 );

	ifAddr = inet_addr(WEND_IP_ADR);

	bzero ( (char *)&ifr, sizeof( struct ifreq ) );
	strcpy(ifr.ifr_name, "wlan0");

	/* Fill interface request structure */
	ifr.ifr_addr.sa_len = sizeof( struct sockaddr_in );
	((struct sockaddr_in *)&ifr.ifr_addr)->sin_family = AF_INET;
/*    ((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr = ifAddr; */
	ifr.ifr_data = (caddr_t) data;

	err = ioctl( WEND_SockFd, cmd, (int)&ifr );

	if ( err )
	{
		DDA_LOG_ARG( DDA_LOG_WEND, "WEND_Ioctl IOCTL error=%x \n",
					 err, 0,0,0,0,0 );
		return( ERROR );
	}

	return( OK );
}

/******************************************************************************/

/*! \brief 	Sends TI IOCTL request via socket
	\param 	data - IOCTL data
	
	\return OK or ERROR

*/
/******************************************************************************/

void WEND_tiIoctl(int data)
{

	WEND_Ioctl( EIOCTI, data );

}

/******************************************************************************/

/*! \brief 	Shows IOCTL codes

*/
/******************************************************************************/

void WEND_IoctlCodes(void)
{

	logMsg("SIOCDIFADDR=%x SIOCADDMULTI=%x SIOCDELMULTI=%x SIOCGETMULTI=%x\n",
		   SIOCDIFADDR, SIOCADDMULTI, SIOCDELMULTI, SIOCGETMULTI, 0,0 ); 

	logMsg("SIOCGIFNETMASK=%x SIOCGIFBRDADDR=%x\n",
		   SIOCGIFNETMASK, SIOCGIFBRDADDR, 0,0,0,0 ); 

	logMsg("EIOCGADDR=%x EIOCGFLAGS =%x EIOCGMIB2=%x EIOCGFBUF=%x EIOCSFLAGS=%x EIOCSFLAGS=%x\n",
		   EIOCGADDR, EIOCGFLAGS, EIOCGMIB2, EIOCGFBUF, EIOCSFLAGS, EIOCSFLAGS );

}

/******************************************************************************/

/*! \brief 	Initializes IOCTL test

*/
/******************************************************************************/


void WEND_InitConfigTest(void)
{
	WEND_Socket ();
}

DDA_IOCTL_DATA_S ioctlData;


/******************************************************************************/

/*! \brief 	Gets WLAN driver name test function

*/
/******************************************************************************/


STATUS WEND_GetName(void)
{
	STATUS status;

	ioctlData.cmd = DDA_IOCTL_CMD_GET_A;
	ioctlData.dataType = DDA_IOCTL_TYPE_COMPLEX_A;
	ioctlData.dataBuf.complexPar.subcmd = TIAP_PARAM_NAME;

	status = WEND_Ioctl( EIOCTI,(int) &ioctlData );
	return(status);
}

/******************************************************************************/

/*! \brief 	Sets SSID parameter test function

*/
/******************************************************************************/


STATUS WEND_setEssid(void)
{
	STATUS status;

	ioctlData.cmd = DDA_IOCTL_CMD_SET_A;
	ioctlData.dataType = DDA_IOCTL_TYPE_COMPLEX_A;
	ioctlData.dataBuf.complexPar.subcmd = TIAP_PARAM_ESSID;
	strcpy(ioctlData.dataBuf.complexPar.params.essid.essid, "ap-igal");
	ioctlData.dataBuf.complexPar.params.essid.len = strlen("ap-igal") + 1;

	status = WEND_Ioctl( EIOCTI,(int) &ioctlData );
	return(status);
}

