/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanEnd.c                                               
    \brief This module implements WLAN END driver
*/
/******************************************************************************/

#define _WLAN_END_C

/***********************************************************
	TODO:
************************************************************/

/***********************************************************
	 includes.
*******************************************************/

#include "wlan_pal.h"
#include "pform_types.h"
#include "ipProto.h"
#include "end.h"        /* Common END structures. */
#include "endLib.h"
#include "etherLib.h"
#include "iosLib.h"
#include "cacheLib.h"
#undef   ETHER_MAP_IP_MULTICAST
#include "etherMultiLib.h"    /* multicast stuff. */
#include "memMngr.h"
#include "dda.h"
#include "dda_trace.h"
#include "dda_ioctl.h"
#include "wlanEnd.h"
#include "wlanMIB2.h"
#include "cfgMan.h"
#include "sysCfgMan.h"

extern WLAN_MIB_DATA wlanMibData;

/***********************************************************
	Defs
************************************************************/

#undef WEND_TEST

#define WEND_NAME           DDA_DEV_NAME
#define WEND_NAME_LEN       strlen(WEND_NAME)
#define WEND_UNIT_NUM       DDA_DEV_UNIT_NUM
#define WEND_LOAD_STR       WEND_NAME
#define WEND_IP_ADR         DDA_DEV_IP_ADR
#define WEND_SUBNET_MASK    "FFFFFF00"

/* Configuration items */

#define END_BUFSIZ      (ETHERMTU + ENET_HDR_REAL_SIZ + 6)
#define EH_SIZE             (14)
#define END_SPEED_10M   10000000 /* 10Mbs */
#define END_SPEED_100M  100000000   /* 100Mbs */
#define END_SPEED       END_SPEED_10M

/*
 * Default macro definitions for BSP interface.
 * These macros can be redefined in a wrapper file, to generate
 * a new module with an optimized interface.
 */

/* A shortcut for getting the hardware address from the MIB II stuff. */

#define END_HADDR(pEnd) \
      ((pEnd)->mib2Tbl.ifPhysAddress.phyAddress)

#define END_HADDR_LEN(pEnd) \
      ((pEnd)->mib2Tbl.ifPhysAddress.addrLength)

/* typedefs */
typedef struct
{
	int len;
	char * pData;
} PKT;	/* A dummy DMA data packet */

typedef struct rfd
{
	PKT *  pPkt;
	struct rfd * next;
} RFD;	/* dummy rx frame descriptor */

typedef struct free_args
{
	void* arg1;
	void* arg2;
} FREE_ARGS;

typedef struct end_device
{
	END_OBJ          end;			/* End structure, must be the first */
	void *			 pCookie;		/* End cookie, must be the second   */
	void *       	 bridgeIfHandle;/* Bridge interface handler 		*/
	int              unit;			/* unit number */
	long             flags;			/* Our local flags. */
	UCHAR            enetAddr[6];	/* ethernet address */
	CL_POOL_ID       pClPoolId;		/* cluster pool */
} AP_END_DEVICE;

/*
 * This will only work if there is only a single unit, for multiple
 * unit device drivers these should be integrated into the AP_END_DEVICE
 * structure.
 */


/***********************************************************
	Static variables
************************************************************/

static BOOL loadInitialized = FALSE;
static int  apEndEnable 	= FALSE;
static int  wlanAdminStatus = MIB2_STATUS_UP;
static char NicApMacAddress[6] = { 0x00, 0x64, 0x65, 0x66, 0x67,  0x68 }; /* The AP MAC Address */

LOCAL END_OBJ * ApEndLoad(char * initString, void * par );

LOCAL END_TBL_ENTRY wendDevTable [] =
{
	{ WEND_UNIT_NUM, ApEndLoad, WEND_LOAD_STR, 1, NULL, FALSE},
	{ 0, END_TBL_END, NULL, 0, NULL, FALSE}			  /* end of table */
};

M_CL_CONFIG ApMclBlkConfig =  /* network mbuf configuration table */
{
	/*
	 no. mBlks     no. clBlks  memArea     memSize
	 -----------      ----------  -------     -------
	 */
	0,         0,       NULL,       0
};

#define TEMPLATE_MIN_FBUF  (1536)   /* min first buffer size */

CL_DESC ApClDescTbl [] =   /* network cluster pool configuration table */
{
	/*
	clusterSize         num   memArea     memSize
	-----------         ----  -------     -------
	*/
	{ETHERMTU + EH_SIZE + 6,  0, NULL,    0}
	/*    {TEMPLATE_MIN_FBUF    , 0, NULL,    0}  */
};

static int ApClDescTblNumEnt = (NELEMENTS(ApClDescTbl));


/***********************************************************
	Local Function Prototypes
***********************************************************/

static STATUS 	AttachToBridgePort( void );
static int 		GetMblksLen(M_BLK_ID  pMblk);

/* forward static functions */

static void    ApEndReset(AP_END_DEVICE *pDrvCtrl);
static STATUS  ApEndStart(AP_END_DEVICE* pDrvCtrl);
static STATUS  ApEndStop(AP_END_DEVICE* pDrvCtrl);
static STATUS  ApEndUnload (AP_END_DEVICE* pDrvCtrl);
static int     ApEndIoctl(AP_END_DEVICE* pDrvCtrl, int cmd, caddr_t data);
static STATUS  ApEndMemInit();
static void    ApEndConfig(AP_END_DEVICE *pDrvCtrl);
static STATUS  ApEndMCastAdd (AP_END_DEVICE* pDrvCtrl, char* pAddress);
static STATUS  ApEndMCastDel (AP_END_DEVICE* pDrvCtrl, char* pAddress);
static STATUS  ApEndMCastGet (AP_END_DEVICE* pDrvCtrl,
							  MULTI_TABLE* pTable);
static STATUS  ApEndPollSend (AP_END_DEVICE* pDrvCtrl, M_BLK_ID pBuf);
static STATUS  ApEndPollRcv (AP_END_DEVICE* pDrvCtrl, M_BLK_ID pBuf);


/* END Specific interfaces. */

/* This is the only externally visible interface. */
END_OBJ*    wend_ApEndLoad(char* initString);
STATUS      wend_ApEndRecv(char *, unsigned long, unsigned long);
STATUS      wend_ApEndSend(AP_END_DEVICE* pDrvCtrl, M_BLK_ID pBuf);

/*
 * Declare our function table.  This is static across all driver
 * instances.
 */

static NET_FUNCS ApEndFuncTable =
{
	(FUNCPTR) ApEndStart,	  /* Function to start the device. */
	(FUNCPTR) ApEndStop,	  /* Function to stop the device. */
	(FUNCPTR) ApEndUnload,	  /* Unloading function for the driver. */
	(FUNCPTR) ApEndIoctl,	  /* Ioctl function for the driver. */
	(FUNCPTR) wend_ApEndSend,	 /* Send function for the driver. */
	(FUNCPTR) ApEndMCastAdd,	 /* Multicast add function for the */
	/* driver. */
	(FUNCPTR) ApEndMCastDel,	 /* Multicast delete function for */
	/* the driver. */
	(FUNCPTR) ApEndMCastGet,	 /* Multicast retrieve function for */
	/* the driver. */
	(FUNCPTR) ApEndPollSend,	 /* Polling send function */
	(FUNCPTR) ApEndPollRcv,		 /* Polling receive function */
	endEtherAddressForm,	  /* put address info into a NET_BUFFER */
	endEtherPacketDataGet,	  /* get pointer to data in NET_BUFFER */
	endEtherPacketAddrGet		 /* Get packet addresses. */
};

AP_END_DEVICE *ApEndDevPtr;

#ifdef WLAN_END_DBG
int wlan_debug_end = 0;
#endif

/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Loads Wlan END driver
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS  WEND_DrvLoad( void )
{
	int             count;
	END_TBL_ENTRY*  pDevTbl;
	END_OBJ*        pCookie = NULL;
	
	apEndEnable = FALSE;
	
	if (loadInitialized) return OK;
	loadInitialized = TRUE;
	
	DDA_LOG(DDA_LOG_WEND, "WEND_DrvLoad loading ...\n" );
	/* Add WEND END to the MUX */

	for (count = 0, pDevTbl = wendDevTable; pDevTbl->endLoadFunc != END_TBL_END;
		pDevTbl++, count++)
	{
		/* Make sure that device is not already installed */
		if (!pDevTbl->processed)
		{
			pCookie = muxDevLoad(pDevTbl->unit,
								 pDevTbl->endLoadFunc,
								 pDevTbl->endLoadString,
								 pDevTbl->endLoan,
								 pDevTbl->pBSP);
			if (pCookie == NULL)
			{
				DDA_LOG_ARG( DDA_LOG_WEND, 
							 "muxDevLoad failed for WEND device entry %d!\n", 
							 count, 0,0,0,0,0);
				return ERROR;
			}
			else
			{
				pDevTbl->processed = TRUE;
				if (muxDevStart(pCookie) == ERROR)
				{
					DDA_LOG_ARG( DDA_LOG_WEND,
								 "muxDevStart failed for WEND device entry %d!\n", 
								 count, 0,0,0,0,0);
					return ERROR;
				}
				else
				{
					ApEndDevPtr->pCookie = pCookie;
				}
			}
			
			
			AttachToBridgePort();  
		
		}
	
	}	 /* for */

	DDA_LOG_ARG( DDA_LOG_WEND, "%s: WEND loaded OK, pEnd=%x\n", __FUNCTION__,
				 (int) pCookie, 0,0,0,0 );

	return OK;
}

/******************************************************************************/

/*! \brief 	END driver module init function
	\return status of operation OK
*/
/******************************************************************************/

STATUS wdrv_endInit(void)
{
	wlanResetMibData();
	return OK;
}

extern void sysBrgIfaceModeChange(NSP_BRIDGE_IFACE_HND h_iface, 
								  NSP_BRIDGE_IFACE_MODE mode, BOOL turn_on);

/******************************************************************************/

/*! \brief 	END driver start
	\return status of operation OK
*/
/******************************************************************************/

STATUS WEND_Start(void)
{
	apEndEnable = TRUE;
	
	if (ApEndDevPtr)
	{
		sysBrgIfaceModeChange ( (NSP_BRIDGE_IFACE_HND) ApEndDevPtr->bridgeIfHandle,
								NSP_BRIDGE_IFACE_MODE_UP, TRUE );

	}
	return OK;
}

/******************************************************************************/

/*! \brief 	END driver stop function
	\return status of operation OK
*/
/******************************************************************************/

STATUS WEND_Stop(void)
{
	apEndEnable = FALSE;
	if (ApEndDevPtr)
	{
		sysBrgIfaceModeChange ( (NSP_BRIDGE_IFACE_HND) ApEndDevPtr->bridgeIfHandle,
								NSP_BRIDGE_IFACE_MODE_UP, FALSE );

	}	
	return OK;
}

/******************************************************************************/

/*! \brief 	Checks whether END driver has been initialized
	\return status TRUE if driver initiated or FALSE if not
*/
/******************************************************************************/

BOOL WEND_IsInitialized(void)
{
	return loadInitialized;
}

/******************************************************************************/

/*! \brief 	Resets END MIB counters
*/
/******************************************************************************/

void ApEndEndReset()
{
	wlanResetMibData();
}

/******************************************************************************/

/*! \brief 	Gets pointer to END driver control sturcture
	\return pointer to END driver control sturcture
*/
/******************************************************************************/

AP_END_DEVICE* wend_GetEndDevPtr()
{
	return(ApEndDevPtr);
}

/******************************************************************************/

/*! \brief 	Gets END driver cookie
	\return  cookie
*/
/******************************************************************************/

void * wend_GetEndDevCookie()
{
	return( ApEndDevPtr->pCookie );
}

/******************************************************************************/

/*! \brief 	WLAN END driver network pool status show

*/
/******************************************************************************/

void WEND_poolShow( void)
{
	if (ApEndDevPtr)
	{
		netPoolShow(ApEndDevPtr->end.pNetPool);
	}
}


/******************************************************************************/

/*! \brief 	Unloads WLAN END driver
	\return status of operation OK or ERROR
*/
/******************************************************************************/


STATUS WEND_Unload( void )
{
	STATUS status;

	status = ipDetach( WEND_UNIT_NUM, WEND_NAME );
	if (status != OK)
	{
		DDA_LOG_ARG( DDA_LOG_WEND, "WEND_Unload: detach error=%x \n", status, 0,0,0,0,0 );
	}

	status = muxDevUnload(WEND_NAME, WEND_UNIT_NUM);
	if (status != OK)
	{
		DDA_LOG_ARG( DDA_LOG_WEND, "WEND_Unload: unload error=%x \n", status, 0,0,0,0,0 );
	}

	return( status );
}


/******************************************************************************/

/*! \brief 	Initialize the driver and device
	\param 	pointer to initializing string
	\param 	optional parameter
	
	\return An END object pointer or NULL on error

*/
/******************************************************************************/



LOCAL END_OBJ * ApEndLoad(char* initString, void * par )
{

	DDA_LOG_ARG(DDA_LOG_WEND, "ApEndLoad: ApEnd initialization started initString %s\n",(int)initString,0,0,0,0,0);

	if (!initString[0])
	{
		DDA_LOG(DDA_LOG_WEND, "ApEndLoad: initString\n");
		bcopy((char *)WEND_NAME, initString, WEND_NAME_LEN + 1);
		return(END_OBJ*)0;
	}

	DDA_LOG(DDA_LOG_WEND, "Loading AP END ...\n");

	ApEndDevPtr = (AP_END_DEVICE *)calloc (sizeof (AP_END_DEVICE), 1);
	if (ApEndDevPtr == NULL)
	{
		DDA_LOG(DDA_LOG_ERR, "AP END load ApEndDevPtr Memory Allocation Fail\n");
		return NULL;
	}

	ApEndDevPtr->unit = WEND_UNIT_NUM;

	if (END_OBJ_INIT (&ApEndDevPtr->end, (DEV_OBJ *)ApEndDevPtr, WEND_NAME,
					  ApEndDevPtr->unit, &ApEndFuncTable,
					  "AP END Driver.") == ERROR  || 
		END_MIB_INIT (&ApEndDevPtr->end, M2_ifType_ethernet_csmacd,
					  (UCHAR*)NicApMacAddress,6, ETHERMTU, END_SPEED) == ERROR)
	{
		DDA_LOG(DDA_LOG_WEND, "AP END load END_OBJ_INIT || END_MIB_INIT fail\n");
		free ((char *)ApEndDevPtr);
		return NULL;
	}

	/* Perform memory allocation/distribution */
	if (ApEndMemInit (ApEndDevPtr) == ERROR)
	{
		DDA_LOG(DDA_LOG_WEND,"AP END load ApEndMemInit fail\n");
		free ((char *)ApEndDevPtr);
		return NULL;
	}

	/* reset and reconfigure the device */
	ApEndReset (ApEndDevPtr);
	ApEndConfig (ApEndDevPtr);

	/* set the flags to indicate readiness */
	END_OBJ_READY ( &ApEndDevPtr->end, IFF_UP | IFF_RUNNING | IFF_NOTRAILERS | 
					IFF_BROADCAST | IFF_MULTICAST );

	DDA_LOG (DDA_LOG_WEND,"Done loading AP END Driver...");
	DDA_LOG_ARG(DDA_LOG_WEND,"ApEndLoad initialized: Sucsess %x \n",(int)&ApEndDevPtr->end,0,0,0,0,0);
	return(&ApEndDevPtr->end);
}

/******************************************************************************/

/*! \brief 	Initialize memory for the chip
	\param 	pointer to END device control structure
	
	\return An END object pointer or NULL on error

*/
/******************************************************************************/


STATUS ApEndMemInit(AP_END_DEVICE * pDrvCtrl)
{
	if ((pDrvCtrl->end.pNetPool = malloc (sizeof(NET_POOL))) == NULL)
		return(ERROR);

	ApMclBlkConfig.mBlkNum 	= 256;
	ApClDescTbl[0].clNum 	= 1;
	ApMclBlkConfig.clBlkNum = ApMclBlkConfig.mBlkNum;

	/* Calculate the total memory for all the M-Blks and CL-Blks. */

	ApMclBlkConfig.memSize = (ApMclBlkConfig.mBlkNum *
							  (MSIZE + sizeof (long))) +
							 (ApMclBlkConfig.clBlkNum *
							  (CL_BLK_SZ + sizeof(long)));

	if ((ApMclBlkConfig.memArea = (char *) memalign(sizeof(long),
													ApMclBlkConfig.memSize)) == NULL)
	{
		return(ERROR);
	}

	/* Calculate the memory size of all the clusters. */
	ApClDescTbl[0].memSize = (ApClDescTbl[0].clNum *
							  (END_BUFSIZ + 8)) + sizeof(int);

	/* Allocate the memory for the clusters from cache safe memory. */

	ApClDescTbl[0].memArea =
	(char *) cacheDmaMalloc (ApClDescTbl[0].memSize);

	if (ApClDescTbl[0].memArea == NULL)
	{
		DDA_LOG(DDA_LOG_ERR,"system memory unavailable\n");
		return(ERROR);
	}

	/* Initialize the memory pool. */

	if (netPoolInit(pDrvCtrl->end.pNetPool, &ApMclBlkConfig,
					&ApClDescTbl[0], ApClDescTblNumEnt, NULL) == ERROR)
	{
		DDA_LOG(DDA_LOG_ERR,"Could not init buffering\n");
		return(ERROR);
	}

	/*
	 * If you need clusters to store received packets into then get them
	 * here ahead of time.
	 */

	if ((pDrvCtrl->pClPoolId = netClPoolIdGet (pDrvCtrl->end.pNetPool,
											   sizeof (RFD), FALSE)) == NULL)
	{
		return(ERROR);
	}

	DDA_LOG(DDA_LOG_WEND,"Memory setup complete\n");

	return OK;
}

/******************************************************************************/

/*! \brief 	start the device
	\param 	pointer to END device control structure
	
	\return OK

*/
/******************************************************************************/

static STATUS ApEndStart(AP_END_DEVICE *pDrvCtrl)
{
//   END_FLAGS_SET (&pDrvCtrl->end, IFF_UP | IFF_RUNNING);
	DDA_LOG(DDA_LOG_WEND,"ApEndStart interface is UP and Running\n");
	return(OK);
}

/******************************************************************************/

/*! \brief 	END receive function
	\param 	pointer to sent MSDU
	
	\return status of operation OK or ERROR

*/
/******************************************************************************/


STATUS WEND_EndRecv(mem_MSDU_T* pMsdu)
{
	M_BLK_ID       result;
	AP_END_DEVICE *pDrvCtrl;
	M_BLK_ID       pMblk = NULL;
	M_BLK_ID       pMblkEnd = NULL;
	CL_BLK_ID      pClBlk;
	UINT16         pktLength, length;
	unsigned long  total_packet_length;
	int            clusterIndex;
	mem_BD_T*      pBD;

	DDA_LOG_ARG( DDA_LOG_WEND,"%s is called Msdu=%x\n", __FUNCTION__,
				 pMsdu, 0,0,0,0);
	
	if (!apEndEnable)
	{
		if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
		{
			DDA_LOG(DDA_LOG_ERR, "ApEndRecv: wlan_memMngrFreeMSDU failed!!");
		}
		wlanMibData.indiscards++;
		return NOK;
	}

	pDrvCtrl = (AP_END_DEVICE *)(ApEndDevPtr);

	/* Receive the data, and initialize total_packet_length. */
	pBD = pMsdu->firstBDPtr;
	total_packet_length = memMgr_MsduDataSize(pMsdu);
	/* W/A if we get somehow Msdu with length=0, we get exception */
	if ( total_packet_length == 0)
	{
		if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
		{
			DDA_LOG(DDA_LOG_ERR, "ApEndRecv: wlan_memMngrFreeMSDU failed!!");
		}
		return NOK;
	}

	clusterIndex = 0;
	pktLength = 0;
	result = NULL;
	while ( pktLength < total_packet_length)
	{
		char*  ethernetPtr;

		clusterIndex++;
		length = memMgr_BufLength(pBD);	/* length of data in current BD */
		ethernetPtr = (char*)(memMgr_BufData(pBD) + memMgr_BufOffset(pBD));

		/* The data packet is already in the cluster now. Get a cluster block to make an MBUF */
		if ((pClBlk = netClBlkGet (pDrvCtrl->end.pNetPool, M_DONTWAIT)) == NULL)
		{
			/* Free the result in case we are half-way through the packet construction */
			if ( result != NULL )
			{
				/* Free the packet */
				netMblkClFree(result);
				wlanMibData.indiscards++;
			}
			if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
			{
				DDA_LOG(DDA_LOG_ERR,"ApEndRecv: wlan_memMngrFreeMSDU failed!!");
			}
			return(ERROR);
		}

		/* Get an M_BLK_ID and link it to the cluster */
		if ((pMblk = netMblkGet (pDrvCtrl->end.pNetPool, M_DONTWAIT, MT_DATA)) == NULL)
		{
			/* Free the cluster control block and the data buffer */
			logMsg( "ApEndRecv pMblk == NULL",0,0,0,0,0,0);
			netClBlkFree (ApEndDevPtr->end.pNetPool, pClBlk);
			netClBlkFree (pDrvCtrl->end.pNetPool, pClBlk);
			wlanMibData.indiscards++;
			if (wlan_memMngrFreeMSDU(pMsdu->handle) != OK)
			{
				DDA_LOG(DDA_LOG_ERR, "ApEndRecv: wlan_memMngrFreeMSDU failed!!");
			}
			return NOK;
		}

		pMblk->mBlkHdr.mFlags = 0;
		/* Remember the packet handle */
		if ( pktLength == 0 )
		{
			/* result is the first MBUF block */
			result = pMblk;
			pMblkEnd = pMblk;
			result->mBlkHdr.mFlags |= M_PKTHDR;	/* start of record */
		}

		/* Join the cluster to the MBlock */
		netClBlkJoin( pClBlk, ethernetPtr, length, wlan_memMngrFreeMSDU,
					  (clusterIndex == 1)? pMsdu->handle : WLAN_DRV_NULL_MEM_HANDLE, 0, 0);
		netMblkClJoin (pMblk, pClBlk);
		pMblk->mBlkHdr.mLen     = length;
		pMblk->mBlkHdr.mNextPkt = NULL;

		/* Link this MBUF block to the previous one */
		if (pktLength != 0)
		{
			pMblkEnd->mBlkHdr.mNext = pMblk;
			pMblkEnd = pMblk;
		}

		pktLength += length; /* Add length of current BD */
		pBD = memMgr_BufNext(pBD);
	}

	/* Total packet length */
	result->mBlkPktHdr.len   = pktLength;

	/* Set the end of record flag */
	pMblk->mBlkHdr.mFlags |= M_EOR;	/* end of record */

	/* Mark the end of the received packet */
	pMblk->mBlkHdr.mNext = NULL;

	if ( MACCMPF(result->mBlkHdr.mData))
	{
		wlanMibData.broadcastFramesIn++;
		wlan_UInt64_Add(&wlanMibData.broadcastBytesIn, total_packet_length);
	}
	else
	{
		if (result->mBlkHdr.mData[0] & 1)
		{
			/* Multicast */
			wlanMibData.multicastFramesIn++;
			wlan_UInt64_Add(&wlanMibData.multicastBytesIn, total_packet_length);
		}
		else
		{
			wlanMibData.unicastFramesIn++;
			wlan_UInt64_Add(&wlanMibData.unicastBytesIn, total_packet_length);
		}
	}

	/* Call the upper layer's receive routine. */
	END_RCV_RTN_CALL(&pDrvCtrl->end, result);

	return OK;
}

/******************************************************************************/

/*! \brief 	END send routine
	\param 	pointer to END device structure
	\param 	pointer to MBLK to send
	
	\return status of operation OK or ERROR

*/
/******************************************************************************/

#define HEADER_EXTRA_LENGTH   32

STATUS wend_ApEndSend(AP_END_DEVICE * pDrvCtrl,	/* device ptr */
					  M_BLK_ID     pMblk	 /* data to send */)
{
	mem_MSDU_T*    pMSDU = NULL;
	mem_BD_T*      pBD = NULL;
	mem_BD_T*      pCurrentMsduBd;
	int            dataLen, totalLen;

	DDA_LOG_ARG( DDA_LOG_WEND,"%s is called Mblk=%x len=%u\n", __FUNCTION__,
				 (int) pMblk, pMblk->mBlkPktHdr.len, 0,0,0);

	if (!apEndEnable || recoveryIsInProcess)
	{
		wlanMibData.outdiscards++;
		netMblkClChainFree (pMblk);
		return NOK;
	}

	/* check if there are non 32 bit size aligned  mblks      */
	/* if there are we have to copy all mblks to one buffer  */
	/* see GetMblksParams description */ 
	dataLen = GetMblksLen( pMblk );

	totalLen = dataLen + HEADER_EXTRA_LENGTH; /* extra length for the 802.11 Header and LLC */
	/* DO NOT SUPPORT SDRAM BUFFERS SMALLER THAN 2K */
	if (totalLen > DATA_BUF_LEN)
	{
		DDA_LOG_ARG(DDA_LOG_ERR,"wend_ApEndSend BUFFER IS TO SMALL totalLen=%d\n",totalLen,0,0,0,0,0);
		wlanMibData.outdiscards++;
		netMblkClChainFree (pMblk);
		return NOK;
	}

	wlan_memMngrAllocMSDU (&pMSDU, HEADER_EXTRA_LENGTH);
	if ((pMSDU == NULL) ||
		(pMSDU->firstBDPtr == NULL) ||
		(pMSDU->firstBDPtr->dataBuf == NULL))
	{
		netMblkClChainFree (pMblk);
		wlanMibData.outdiscards++;
		return NOK;
	}
	pMSDU->dataLen = dataLen;
	pMSDU->freeFunc = (ap_FreeMemFunc)netMblkClChainFree;
	pMSDU->freeArgs[0] = (UINT32)pMblk;
	pMSDU->firstBDPtr->bufToken = 0;

	/* we don't have copy buffer but only assign pointers to cluster */
	wlan_memMngrAllocBDs(1, &pBD);
	if (pBD == NULL)
	{
		if (wlan_memMngrFreeMSDU(pMSDU->handle) != OK)
		{
			DDA_LOG(DDA_LOG_ERR, "wend_ApEndSend: wlan_memMngrFreeMSDU failed!!");
		}
		netMblkClChainFree (pMblk);
		wlanMibData.outdiscards++;
		return NOK;
	}

	pBD->data = pMblk->mBlkHdr.mData;
	pBD->length = pMblk->mBlkHdr.mLen;

	pMSDU->firstBDPtr->nextBDPtr = pBD;

	if ( MACCMPF(pMblk->mBlkHdr.mData))
	{
		wlanMibData.broadcastFramesOut++;
		wlan_UInt64_Add(&wlanMibData.broadcastBytesOut, totalLen);
	}
	else
	{
		if (pMblk->mBlkHdr.mData[0] & 1)
		{
			wlanMibData.multicastFramesOut++;
			wlan_UInt64_Add(&wlanMibData.multicastBytesOut, totalLen);
		}
		else
		{
			wlanMibData.unicastFramesOut++;
			wlan_UInt64_Add(&wlanMibData.unicastBytesOut, totalLen);
		}
	}

	pCurrentMsduBd = pBD;
	
	while ((pMblk = pMblk->mBlkHdr.mNext) != NULL)
	{
		if (pMblk->mBlkHdr.mLen == 0)
			break;

		/* Allocate new DB for the new block */
		wlan_memMngrAllocBDs(1, &pBD);
		if (pBD == NULL)
		{
			if (wlan_memMngrFreeMSDU(pMSDU->handle) != OK)
			{
				DDA_LOG(DDA_LOG_ERR,"wend_ApEndSend: wlan_memMngrFreeMSDU failed!!");
			}
			netMblkClChainFree (pMblk);
			wlanMibData.outdiscards++;
			return NOK;
		}

		/* Set pointer to the data and the data length */
		pBD->data = pMblk->mBlkHdr.mData;
		pBD->length = pMblk->mBlkHdr.mLen;

		/* Concatinate the new BD */
		pCurrentMsduBd->nextBDPtr = pBD;
		pCurrentMsduBd = pBD;

		/* Modify MSDU last BD pointer */
		pMSDU->lastBDPtr = pBD;   
	}

	DDA_LOG_ARG(DDA_LOG_WEND,"tiwlan_ddaSend\n",(pMSDU),0,0,0,0,0)

	tiwlan_ddaSend(pMSDU);

	return OK;
}/* END wend_ApEndSend() */

/******************************************************************************/

/*! \brief 	END IOCTL routine
	\param 	pointer to END device structure
	\param 	IOCTL command
	\param 	IOCTL data
	
	\return status of operation OK or EINVAL
*/
/******************************************************************************/


static int ApEndIoctl(
					 AP_END_DEVICE * pDrvCtrl, /* device receiving command */
					 int cmd,		  /* ioctl command code */
					 caddr_t data	  /* command argument */
					 )
{
 STATUS error = 0;
 long   value;

 switch (cmd)
 {
	case EIOCSADDR:
		if (data == NULL)
			return(EINVAL);
		bcopy ((char *)data, (char *)END_HADDR(&pDrvCtrl->end),
			   END_HADDR_LEN(&pDrvCtrl->end));
		break;

	case EIOCGADDR:
		if (data == NULL)
			return(EINVAL);
		bcopy ((char *)END_HADDR(&pDrvCtrl->end), (char *)data,
			   END_HADDR_LEN(&pDrvCtrl->end));
		break;

	case EIOCSFLAGS:
		value = (long)data;
		if (value < 0)
		{
			value = -(--value);
			END_FLAGS_CLR (&pDrvCtrl->end, value);
		}
		else
		{
			END_FLAGS_SET (&pDrvCtrl->end, value);
		}
		break;

	case EIOCGFLAGS:
		*(int *)data = END_FLAGS_GET(&pDrvCtrl->end);
		break;

	case EIOCPOLLSTART:	 /* Begin polled operation */
		break;

	case EIOCPOLLSTOP:	 /* End polled operation */
		break;

	case EIOCGMIB2:	   /* return MIB information */
		if (data == NULL)
			return(EINVAL);
		bcopy((char *)&pDrvCtrl->end.mib2Tbl, (char *)data,
			  sizeof(pDrvCtrl->end.mib2Tbl));
		break;
	case EIOCGFBUF:	   /* return minimum First Buffer for chaining */
		if (data == NULL)
			return(EINVAL);
		*(int *)data = TEMPLATE_MIN_FBUF;
		break;
	case EIOCGMWIDTH:
		if (data == NULL)
			return(EINVAL);
		*(int *)data = 4;
		break;

	case EIOCGHDRLEN:
		if (data == NULL)
			return(EINVAL);
		*(int *)data = 14;
		break;

	case EIOCTI:		 /* TI WLAN Configuration */
		DDA_LOG_ARG(DDA_LOG_IO, "WEND_IOCTL EIOCTI cmd=%x data =0x%x\n", 
					cmd, data, 0,0,0,0);
		error = DDA_doIoctl((DDA_IOCTL_DATA_S *)data );

		break;

	default:
		error = EINVAL;
 }

 return(error);
}

static void ApEndConfig(AP_END_DEVICE *pDrvCtrl)
{
	/* stub, only return */
}

static STATUS ApEndPollRcv (
						   AP_END_DEVICE * pDrvCtrl, /* device to be polled */
						   M_BLK_ID      pMblk	  /* ptr to buffer */
						   )
{
	return(OK);
}

static STATUS ApEndPollSend(
						   AP_END_DEVICE*   pDrvCtrl,	/* device to be polled */
						   M_BLK_ID    pMblk   /* packet to send */
						   )
{
	return(OK);
}

static STATUS ApEndMCastAdd(
						   AP_END_DEVICE *pDrvCtrl,		/* device pointer */
						   char* pAddress	/* new address to add */
						   )
{
	return(OK);
}

static STATUS ApEndMCastDel (
							AP_END_DEVICE *pDrvCtrl,	 /* device pointer */
							char* pAddress		/* address to be deleted */
							)
{
	return(OK);
}

static STATUS ApEndMCastGet (
							AP_END_DEVICE *pDrvCtrl,		/* device pointer */
							MULTI_TABLE* pTable	   /* address table to be filled in */
							)
{
	return(OK);
}

static STATUS ApEndStop (
						AP_END_DEVICE *pDrvCtrl	  /* device to be stopped */
						)
{
	STATUS result = OK;
	return(result);
}

static STATUS ApEndUnload (
						  AP_END_DEVICE* pDrvCtrl	/* device to be unloaded */
						  )
{
	return(OK);
}

static void ApEndReset(
					  AP_END_DEVICE *pDrvCtrl
					  )
{
	/* Stub - reset the controller */
}

/******************************************************************************/

/*! \brief 	Sets receive function call back
	\param 	pointer to receive function
*/
/******************************************************************************/


void WEND_SetRecvRtn(recvRtnApEnd_t _brgEnabledRecvRtn)
{
	int oldLevel;

	oldLevel = intLock();
	/* replace receive  & save routines to bridge packets to Cbl as needed */
	ApEndDevPtr->end.receiveRtn = (void*)_brgEnabledRecvRtn;

	intUnlock(oldLevel);
}

/******************************************************************************/

/*! \brief 	Sets END driver administrative state
	\param 	state value
*/
/******************************************************************************/

void WEND_SetAdminStatus(int state)
{
	wlanAdminStatus = state;
}

/******************************************************************************/

/*! \brief 	Gets END driver administrative state
	\return END driver administrative state
*/
/******************************************************************************/

int WEND_GetAdminStatus(void)
{
	return wlanAdminStatus;
}

/******************************************************************************/

/*! \brief 	Gets MBLK length
	\param 	pointer to MBLK
	\return MBLK data length
*/
/******************************************************************************/


static int GetMblksLen(M_BLK_ID  pMblk)
{
	int totalLen=0;

	while (pMblk)
	{
		totalLen += pMblk->mBlkHdr.mLen;
		pMblk = pMblk->mBlkHdr.mNext;
	}

	return totalLen;
}


extern STATUS sysBrgRcvRtn(END_OBJ* pEndObj, M_BLK_ID pMblk);
extern STATUS sysBrgEndConnect(END_OBJ* pEndObj, void* rcvRtn, void* hnd);
extern NSP_BRIDGE_IFACE_HND sysBrgCreateIface( 
								PORT_CONFIG_T * ptr_port_config,
								END_OBJ       * pEndObj,
								NSP_BRIDGE_IFACE_SEND_RTN  end_drv_send );


/*! \brief 	Attaches Wlan END driver to Local Bridge
	\return status of operation OK or ERROR
*/
/******************************************************************************/


LOCAL STATUS AttachToBridgePort( void )
{
 PORT_CONFIG_T 	 portConfig[2];
 PORT_CONFIG_T * pConf;
 END_OBJ 	   * pEndObj;
 NSP_BRIDGE_IFACE_HND  h_iface;

 pEndObj = endFindByName(WEND_NAME, WEND_UNIT_NUM);

 if (!pEndObj)
 {
	 DDA_LOG_ARG( DDA_LOG_ERR, "%s: END %s unit %u is not found\n", __FUNCTION__, 
				  WEND_NAME, WEND_UNIT_NUM, 0,0,0 );
	 return ERROR;
 }
				/* Initialize the PORT configuration block. */
 memset ((char *)portConfig, 0, sizeof (portConfig));

 pConf = &portConfig[0];
 strcpy (pConf->port_name, WEND_NAME);
 pConf->unit_number = WEND_UNIT_NUM;
 strcpy ((char *)pConf->ip_address, WEND_IP_ADR);
 strcpy ((char *)pConf->subnet_mask,WEND_SUBNET_MASK);       
 memcpy ((char *)pConf->macAddress, NicApMacAddress, 6);
 pConf->portMedia = WIRELESS; /* VIRTUAL; */
 pConf->protocol = CSMACD;
 pConf->macSpecific.pAtm = NULL;
 pConf->macSpecific.pUsb = NULL;
 pConf->flags = NSP_MANAGED_PORT | NSP_BRIDGED_PORT;
 pConf->sys_information = sysInitializePortInfo (pConf);
 ((SYS_END_SPECIFIC_INFO_T*) pConf->sys_information)->
									end_information.endLoadFunc = ApEndLoad;
 pConf->pNextPort = NULL;

	 
 h_iface = sysBrgCreateIface(pConf, pEndObj, (NSP_BRIDGE_IFACE_SEND_RTN)pEndObj->pFuncTable->send);
 if (!h_iface)
 {
	 DDA_LOG_ARG( DDA_LOG_ERR, "%s: failed to create Bridge interface\n", __FUNCTION__, 
				  0,0,0,0,0 );
	 return ERROR;
 }
       /* Change the receive routine in the END to point to the 
	      BRIDGE receive instead of the IP stack. */
 sysBrgEndConnect (pEndObj, (void*)sysBrgRcvRtn, h_iface);

 ((AP_END_DEVICE *) pEndObj)->bridgeIfHandle = h_iface;

 return OK;
}


#ifdef WEND_TEST
#	include "WlanEndTest.c"
#endif

