/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlanMIB2.c                                               
    \brief This module implements SNMP counters
*/
/******************************************************************************/

/***********************************************************
	 includes.
*******************************************************/

#include "wlan_pal.h"
#include "pform_types.h"
#include "dda.h"
#include "memMngr.h"
#include "wlanEnd.h"
#include "wlanMIB2.h"

WLAN_MIB_DATA wlanMibData;

/***********************************************************
	static variables
************************************************************/

static int wlanOperStatus = 0;


/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Resets MIB counters
*/
/******************************************************************************/

void wlanResetMibData(void)
{
   wlanMibData.multicastFramesIn = 0;
   wlanMibData.broadcastFramesIn = 0;
   wlanMibData.unicastFramesIn = 0;

   wlanMibData.multicastFramesOut = 0;
   wlanMibData.broadcastFramesOut = 0;   
   wlanMibData.unicastFramesOut = 0;

   wlanMibData.inerrors = 0;
   wlanMibData.outerrors = 0;

   wlanMibData.indiscards = 0;
   wlanMibData.outdiscards = 0;

   wlanMibData.InUnknownProtos = 0;
   
   UI64_Zero(&(wlanMibData.unicastBytesOut));
   UI64_Zero(&(wlanMibData.multicastBytesOut));
   UI64_Zero(&(wlanMibData.broadcastBytesOut));

   UI64_Zero(&(wlanMibData.unicastBytesIn));
   UI64_Zero(&(wlanMibData.multicastBytesIn));
   UI64_Zero(&(wlanMibData.broadcastBytesIn));
}

/******************************************************************************/

/*! \brief 	Gets MIB counters by tupe
	\param 	device number
	\param 	counter type
	
	\return counter value
*/
/******************************************************************************/

unsigned long wlanMib2GetCounter(int dev_port, MIB2_CounterType type)
{
   
   UINT_64_t     	wk;
   unsigned long  counter = 0;

   
   switch (type)
   {
   case InBytes: 
		   wk = wlanMibData.unicastBytesIn;
	      wlan_UInt64_Add64(&wk, &(wlanMibData.multicastBytesIn));
		   wlan_UInt64_Add64(&wk, &(wlanMibData.broadcastBytesIn));
			counter = wk.low;
        break;

   case InBytesHigh: 
			wk = wlanMibData.unicastBytesIn;
	      wlan_UInt64_Add64(&wk, &(wlanMibData.multicastBytesIn));
		   wlan_UInt64_Add64(&wk, &(wlanMibData.broadcastBytesIn));
	      counter = wk.high;
         break;

   case InUnicastPkts: 
	    counter = wlanMibData.unicastFramesIn;
      break;

   case InUnicastPktsHigh:
      counter = 0;
      break;

   case InMulticastPkts:
	   counter = wlanMibData.multicastFramesIn;
      break;

   case InMulticastPktsHigh:
      counter = 0;
      break;

   case InBroadcastPkts:
	   counter = wlanMibData.broadcastFramesIn;
      break;

   case InBroadcastPktsHigh:
      counter = 0;
      break;

   case InDiscardPkts:
	   counter = wlanMibData.indiscards;
      break;

   case InErrorPkts:
	   counter = wlanMibData.inerrors;
      break;

   case InUnknownProtPkts:
      counter = wlanMibData.InUnknownProtos;
      break;

   case OutBytes:
		   wk = wlanMibData.unicastBytesOut;
	      wlan_UInt64_Add64(&wk, &(wlanMibData.multicastBytesOut));
		   wlan_UInt64_Add64(&wk, &(wlanMibData.broadcastBytesOut));
			counter = wk.low;
         break;

   case OutBytesHigh:
			wk = wlanMibData.unicastBytesOut;
	      wlan_UInt64_Add64(&wk, &(wlanMibData.multicastBytesOut));
		   wlan_UInt64_Add64(&wk, &(wlanMibData.broadcastBytesOut));
	      counter = wk.high;
          break;

   case OutUnicastPkts:
	    counter = wlanMibData.unicastFramesOut;
      break;

   case OutUnicastPktsHigh:
      counter = 0;
      break;

   case OutMulticastPkts:
	   counter = wlanMibData.multicastFramesOut;
      break;

   case OutMulticastPktsHigh:
      counter = 0;
      break;

   case OutBroadcastPkts:
	   counter = wlanMibData.broadcastFramesOut;
      break;

   case OutBroadcastPktsHigh:
      counter = 0;
      break;

   case OutDiscardPkts:
	  counter = wlanMibData.outdiscards;
      break;

   case OutErrorPkts:
	  counter = wlanMibData.outerrors;
      break;

   default:
	   counter = 0;
	   break; /* Do nothing */
   }

   return counter;
}


/******************************************************************************/

/*! \brief 	Sets MIB administrative state
	\param 	device number
	\param 	state value
	
*/
/******************************************************************************/


void wlanMib2SetAdminStatus(int dev_port, int state)
{
   WEND_SetAdminStatus(state);
   return;
}
 

/**************************************************************************
 *              int wlanMib2GetOperStatus(int dev_port)                   *
 **************************************************************************
 * DESCRIPTION: Get Device physical connection state                      *
 *                                                                        *
 * INPUT:       dev_port - port number for multiport devices              *
 * OUTPUT:      no.                                                       *
 * RETURN:      MIB2_STATUS_UP if connected, otherwise MIB2_STATUS_DOWN.  *
 *                                                                        *
 **************************************************************************/


/******************************************************************************/

/*! \brief 	Gets MIB operational status
	\param 	device number
	
	\return operational status
*/
/******************************************************************************/

int wlanMib2GetOperStatus(int dev_port)
{
   if(wlanOperStatus ==  0)
   {
      return MIB2_STATUS_DOWN;
   }
   return MIB2_STATUS_UP;
}


/******************************************************************************/

/*! \brief 	Gets Promiscuous Mode stub function
	\param 	device number
	
	\return 1
*/
/******************************************************************************/

int wlanMib2GetPromiscuousMode(int dev_port)
{
       return 1;
}

/******************************************************************************/

/*! \brief 	Gets port speed
	\param 	device number
	
	\return 54 Mb
*/
/******************************************************************************/


unsigned long wlan_Mib2GetSpeed(int dev_port)
{
	return 54000000; /* 54 MB */
}

/******************************************************************************/

/*! \brief 	Sets operation status
	\param 	operational status
	
*/
/******************************************************************************/

void wlanMib2SetOperStatus(int status)
{
   wlanOperStatus = status;
}

#define	WLAN_STAT_SPRINTF_ALIGN		28	/* alignment length in bytes use 80 columns terminal */
#define	WLAN_STAT_SPRINTF( buffer, str ) \
	{ \
		int len, i; \
		len = sprintf((buffer), "%s", (str)); \
		for( i=len; i < WLAN_STAT_SPRINTF_ALIGN; i++) buffer[i] = ' '; \
		buffer[i] = (char)0; \
	}

/******************************************************************************/

/*! \brief 	Show WLAN device MIB2 statistics on stdout
	\param 	device port number
	
*/
/******************************************************************************/

void wlanMib2StatShow( int dev_port)
{
	char			      pbuffer[ WLAN_STAT_SPRINTF_ALIGN + 2];

	printf("***************************** WLAN %d MIB stats ******************************\n",
      dev_port);

  	WLAN_STAT_SPRINTF(pbuffer,"InBytes");
	printf("%s: %10ld [Input bytes low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InBytes ));
  	WLAN_STAT_SPRINTF(pbuffer,"InBytesHigh");
	printf("%s: %10ld [Input bytes high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InBytesHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"InUnicastPkts");
	printf("%s: %10ld [Input unicast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InUnicastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"InUnicastPktsHigh");
	printf("%s: %10ld [Input unicast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InUnicastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"InMulticastPkts");
	printf("%s: %10ld [Input multicast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InMulticastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"InMulticastPktsHigh");
	printf("%s: %10ld [Input multicast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InMulticastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"InBroadcastPkts");
	printf("%s: %10ld [Input broadcast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InBroadcastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"InBroadcastPktsHigh");
	printf("%s: %10ld [Input broadcast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InBroadcastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"InDiscardPkts");
	printf("%s: %10ld [Input discard packets]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InDiscardPkts ));

  	WLAN_STAT_SPRINTF(pbuffer,"InErrorPkts");
	printf("%s: %10ld [Input error packets]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InErrorPkts ));

  	WLAN_STAT_SPRINTF(pbuffer,"InUnknownProtPkts");
	printf("%s: %10ld [Input unknown protocol packets]\n",
      pbuffer, wlanMib2GetCounter( dev_port, InUnknownProtPkts ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutBytes");
	printf("%s: %10ld [Output bytes low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutBytes ));
  	WLAN_STAT_SPRINTF(pbuffer,"OutBytesHigh");
	printf("%s: %10ld [Output bytes high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutBytesHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutUnicastPkts");
	printf("%s: %10ld [Output unicast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutUnicastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"OutUnicastPktsHigh");
	printf("%s: %10ld [Output unicast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutUnicastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutMulticastPkts");
	printf("%s: %10ld [Output multicast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutMulticastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"OutUnicastPktsHigh");
	printf("%s: %10ld [Output multicast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutUnicastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutBroadcastPkts");
	printf("%s: %10ld [Output broadcast packets low]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutBroadcastPkts ));
  	WLAN_STAT_SPRINTF(pbuffer,"OutBroadcastPktsHigh");
	printf("%s: %10ld [Output broadcast packets high]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutBroadcastPktsHigh ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutDiscardPkts");
	printf("%s: %10ld [Output discard packets]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutDiscardPkts ));

  	WLAN_STAT_SPRINTF(pbuffer,"OutErrorPkts");
	printf("%s: %10ld [Output error packets]\n",
      pbuffer, wlanMib2GetCounter( dev_port, OutErrorPkts ));


}/* END wlanMib2StatShow() */
