/***************************************************************
*             Copyright (C) 2003 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/

/*
 * file: make_PID.c
 * author: max_chung@sercomm.com
 * version: 0.1
 *
 * make_PID:
 *   will create a PID file (70 bytes) which contains H/W id and F/W version
 *
 * show_PID:
 *   reads in a PID file (70 bytes) and show the H/W id and F/W version info
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "pid.h"

char *encode_version(char * ver) 
{
	int  comb;
	int  major;
	int  minor; 
	char	*version;
	version=malloc(2);
	sscanf(ver,"%d.%d.%d",&comb,&major,&minor);
	version[0]=((comb & 0x0F) << 4) + (major &0x0F);
	version[1]=minor;
	return version;
}

char *encode_pid(char annex,int func, char *company)
{
	char *p_id;
	p_id=malloc(4);
	p_id[0]=annex;
	p_id[1]=func;
	p_id[2]=company[0];
	p_id[3]=company[1];
	return p_id;
}

void decode_version(char * version) 
{
	int  comb=0;
	int  major=0;
	int  minor=0; 
	comb = (version[0] & 0xF0) >> 4;
	major = version[0] & 0x0F;
	minor = version[1] & 0xFF;
	printf("Versoin %d.%02d.%02d\n",comb,major,minor);
}

void decode_pid(char *string) 
{
	printf("ANNEX %c\n",string[0]);	
	printf("Function %02d\n",string[1]);	
	printf("Company ID == %c%c\n",string[2],string[3]);
}




int show_PID(int argc, char **argv) 
{
	sercomm_pid_t pid;
 	FILE *fp;
	
	if (argc != 2) {
		fprintf(stderr, "Usage: %s PID_FILE\n", argv[0]);
		return -1;
	}
	
	fp=fopen(argv[1],"r");
	if(fp){
		fread(&pid,sizeof(sercomm_pid_t),1,fp);
		printf("HW_ID == %s\n",pid.hw_id);
		decode_version(pid.fw_ver);
		decode_pid(pid.p_id);
	}
	
	return 0;
}


int make_PID(int argc, char **argv) 
{
	sercomm_pid_t pid;
	char *hw_id=argv[1];
	char *version=argv[2]+1;
	char *company=argv[3];
	char annex=argv[2][0];
	int func=atoi(argv[4]);
	FILE *fp;	
	char filename[12]="";
	
	if (argc!=5) {
		fprintf(stderr, "\nUsage: %s HW_ID FW_version Company Function\n", argv[0]);
		fprintf(stderr, "HW_ID: a string less than 32 bytes\n");
		fprintf(stderr, "FW_version format: A0.00.00\n");
		fprintf(stderr, "Company: 00 - ZZ\n");
		fprintf(stderr, "Function: 0:router 1:bridge 2:vpn 3:mpvc\n");
		return -1;
	}

	sprintf(filename,"PID_%c%s",annex,version);
	fp=fopen(filename,"w");
	
	if(fp==NULL){
		fprintf(stderr,"Can not open %s\n",filename);
		return -1;
	}
	
	bzero(&pid,sizeof(sercomm_pid_t));
	strcpy(pid.magic_s,"sErCoMm");
	strcpy(pid.magic_e,"sErCoMm");
	strcpy(pid.hw_id,hw_id);
	memcpy(pid.fw_ver,encode_version(version),sizeof(pid.fw_ver));
	memcpy(pid.p_id,encode_pid(annex,func,company),sizeof(pid.p_id));
	
	fwrite(&pid,sizeof(sercomm_pid_t),1,fp);
	fclose(fp);
	return 0;
}


int main(int argc, char * argv[]) 
{


	// check the program name
	if (strstr(argv[0], "make_PID")) {
		make_PID(argc, argv);
	} else if (strstr(argv[0], "show_PID")) {
		show_PID(argc, argv);
	} else {
		fprintf(stderr, "You have to use the Makefile supplied with \
this package to build the programs!\n");
		fprintf(stderr, "Exiting...\n");
	}

	return 0;
}
