/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/**************************************************************************/
/*                                                                        */
/*   MODULE:  httpfunc.c                                                  */
/*   PURPOSE: HTTP functions										      */
/*                                                                        */
/**************************************************************************/

#define _WCFGLIB_C

#ifdef VXWORKS
#	include "wlan_pal.h"
#	include "dda.h"
#	include "ctype.h"
#	include "dda_trace.h"
#	include "dda_ioctl.h"
#	include "dda_wpaauth_ioctl.h"
#	include "dda_proc.h"
#	include "wcfglib.h"
#	include "wcfg_ioctl.h"
#	include "wcfg_net.h"
#	include "cli.h"
#	include "dda_proc_fs_defs.h"
#	include "drvDbRegDomainDefs.h"
#   include "env.h"
#   include "rebootLib.h"
#   include "sysLib.h"
#   include "defrag.h"
#   include "ffs.h"




#else
#	include <stdio.h>
#	include <unistd.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include "wcfglib.h"
#	include "wcfg_ioctl.h"
#	include "wcfg_net.h"
#	include "cli.h"
#	include "dda_proc_fs_defs.h"
#	include "cfg_proc_if.h"
#	include "drvDbRegDomainDefs.h"
#   include <libgen.h>
#   include <signal.h>
#   include <sys/wait.h>
#endif  /* VXWORKS */


#define	MAX_PRINT_STRING		128	
#define FFS_PRIMARY_PART			0
	

/* this definition is very important, it fixes the maximum number of bytes that can be returned to the html page.
		If you return a string that is bigger than this size, you will have a memory overflow. */
/*#define	MAX_RETURNED_STRING_LEN	800*/
#define	MAX_RETURNED_STRING_LEN	4096*8


extern wcfgd *wd;
struct storage storage_backup;

static unsigned char htol(char str[]);

unsigned char hexChar2int(unsigned char c);

BOOL TestPrivacy(const char *,
		char *, char *, char *, char *, char *, char *, char *,
		char *, char *, char *, char *, char *, char *, char *,
		char *, char *, char *, char *);
char *SavePrivacy(const char *,
		char *, char *, char *, char *, char *, char *, char *,
		char *, char *, char *, char *, char *, char *, char *,
		char *, char *, char *, char *);

static char tempbuf[MAX_RETURNED_STRING_LEN];

static UINT16	stationCurrentIndex;
static UINT16   tableBW_CurrentIndex;
static UINT32   globalWebSavingIndex = 0;
static UINT32   mSSID_errorNumber = 0;

#define MULTIPLE_SSID_MSG_ERROR_VLAN_ZERO 		1
#define MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS 	2
#define MULTIPLE_SSID_MSG_ERROR_SSID_EMPTY 		3
#define MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS 	4
#define MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR 	5
#define MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM	6
#define MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR   7
#define MULTIPLE_SSID_MSG_ERROR_WRONG_KEYS_LENGTH   8
#define MULTIPLE_SSID_MSG_ERROR_VLAN_OVERFLOW   9

static void create_js_msg_box(char *buf, char *message);
static BOOL wlan_htmlTestWepKey(char *key, int expectedlength);


#define validate_profile(n) \
	if( (atoi(n) < 0 || atoi(n) >= TIAP_PARAM_NUM_MULTIPLE_SSID) ) { \
		PRINT_CONFIG_ERR_MSG("Invalid profile index."); \
		return (char*)NULL; \
	}

#define validate_ip_octet(str, n, err) \
	do { \
		err = 0; \
		if( !sscanf(str, "%d", &n) ) { \
			err = 1; \
			break; \
		} \
		if( n < 0 || n > 255 ) { \
			err = 1; \
			break; \
		} \
	} while(0)

static unsigned char hex(unsigned char c)
{
	if (c >= 'A' && c <= 'F')
		return (c - 'A' + 10);
	else if (c >= 'a' && c <= 'f')
		return (c - 'a' + 10);
	return c - '0';
}

static unsigned char htol(char str[])
{
	unsigned char n;

	n = hex(str[0]) << 4;
	n |= hex(str[1]);
	return n;
}


typedef struct {    /* This is a storage for a list of Channel Range elements */
	tiap_param_cur_reg_domain_t cur_regdomain;
	UINT8                       ranges_nr;
	tiap_param_channel_rang_elem_t *rang_elem;
} cli_channel_range_tbl_elem_t;

/* 802.11d Allowed Channel Tables - hardcoded values */
static cli_channel_range_tbl_elem_t ch_rang_a[NUM_OF_REGULATORY_DOMAINS_A];
static cli_channel_range_tbl_elem_t ch_rang_b[NUM_OF_REGULATORY_DOMAINS_B];


int create_listening_socket(__u32);
int close_listening_socket(void);


static int profile_map_tbl[TIAP_PARAM_NUM_MULTIPLE_SSID];

/*
 * profile_map_tbl[i], means map web profile i to internal profile
 * used by most multi ssid API.
 */
static int mssid_act_to_inter(const char *ssid)
{
	int i, internal;
	tiap_param_multi_ssid_t multi_ssid;

	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
		exit(1);
	}

	internal = -1;
	for(i = 0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY ||
			multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY) 
		{
			internal++;
			if( !strncmp(multi_ssid.tbl[i].ssid, ssid, sizeof(multi_ssid.tbl[i].ssid)) )
				return internal;
		}
	return -1;
}

	
static void restart(void)
{
	char driver_load_failed;
	wcfg_storage_write(&wd->storage, (const char *)wd->storage_file);

#ifndef VXWORKS	
	close_listening_socket();
	
	/*restart library*/
	if( !(wd = tiap_wcfg_restart(wd, &driver_load_failed)) ) {
		fprintf(stderr, "tiweb: Failed to restart wcfg library.\n");
		exit(1);
	}

	if( create_listening_socket(INADDR_ANY) ) {
		fprintf(stderr, "tiweb: Sorry, couldn't create listening socket.\n");
		exit(1);
	}
#else
	
	/*restart library*/
	tiap_wcfg_system_reboot(wd);

#endif	/* VXWORKS */

}


void wlan_htmlInit(void)
{
	int i;
	tiap_param_multi_ssid_t multi_ssid;

	memset(profile_map_tbl, -1, sizeof(profile_map_tbl));

	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
		return;
	}
	
	for(i = 0; i < TIAP_PARAM_NUM_MULTIPLE_SSID-1; i++)
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY )
			profile_map_tbl[0] = mssid_act_to_inter(multi_ssid.tbl[i].ssid);
		else if( multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY)
			profile_map_tbl[i] = mssid_act_to_inter(multi_ssid.tbl[i].ssid);
	mSSID_errorNumber = 0;
}
	

void wlan_htmlRestart(void)
{
	restart();
}

void wlan_htmldbInitRangeTables(void)
{
	ch_rang_a[0].cur_regdomain.type = TIAP_PARAM_RD_US;
	ch_rang_a[0].ranges_nr = NUM_OF_RANGE_ELEMENTS_US;
	if( !(ch_rang_a[0].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}

	ch_rang_a[0].rang_elem[0].start = REG_DOMAIN_US_START_CH_0;
	ch_rang_a[0].rang_elem[0].stop = REG_DOMAIN_US_STOP_CH_0;
	ch_rang_a[0].rang_elem[0].step = REG_DOMAIN_US_STEP_OPT_0;
	ch_rang_a[0].rang_elem[0].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_0;
	ch_rang_a[0].rang_elem[1].start = REG_DOMAIN_US_START_CH_1;
	ch_rang_a[0].rang_elem[1].stop = REG_DOMAIN_US_STOP_CH_1;
	ch_rang_a[0].rang_elem[1].step = REG_DOMAIN_US_STEP_OPT_1;
	ch_rang_a[0].rang_elem[1].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_1;

	ch_rang_a[1].cur_regdomain.type = TIAP_PARAM_RD_WORLD;
	ch_rang_a[1].ranges_nr = NUM_OF_RANGE_ELEMENTS_WORLD;
	if( !(ch_rang_a[1].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*3)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_a[1].rang_elem[0].start = REG_DOMAIN_WORLD_START_CH_0;
	ch_rang_a[1].rang_elem[0].stop  = REG_DOMAIN_WORLD_STOP_CH_0;
	ch_rang_a[1].rang_elem[0].step = REG_DOMAIN_WORLD_STEP_OPT_0;
	ch_rang_a[1].rang_elem[0].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_0;
	ch_rang_a[1].rang_elem[1].start = REG_DOMAIN_WORLD_START_CH_1;
	ch_rang_a[1].rang_elem[1].stop  = REG_DOMAIN_WORLD_STOP_CH_1;
	ch_rang_a[1].rang_elem[1].step = REG_DOMAIN_WORLD_STEP_OPT_1;
	ch_rang_a[1].rang_elem[1].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_1;
	ch_rang_a[1].rang_elem[2].start = REG_DOMAIN_WORLD_START_CH_2;
	ch_rang_a[1].rang_elem[2].stop = REG_DOMAIN_WORLD_STOP_CH_2;
	ch_rang_a[1].rang_elem[2].step = REG_DOMAIN_WORLD_STEP_OPT_2;
	ch_rang_a[1].rang_elem[2].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_2;

	/*              - G - band              */
	ch_rang_b[0].cur_regdomain.type = TIAP_PARAM_RD_FCC;
	ch_rang_b[0].ranges_nr = 1;
	if( !(ch_rang_b[0].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[0].rang_elem[0].start = REG_DOMAIN_FCC_START_CH_0;
	ch_rang_b[0].rang_elem[0].stop  = REG_DOMAIN_FCC_STOP_CH_0;
	ch_rang_b[0].rang_elem[0].step = REG_DOMAIN_FCC_STEP_OPT_0;
	ch_rang_b[0].rang_elem[0].power_limit = REG_DOMAIN_FCC_POWER_LIMIT_CH_0;

	ch_rang_b[1].cur_regdomain.type = TIAP_PARAM_RD_IC;
	ch_rang_b[1].ranges_nr = 1;
	if( !(ch_rang_b[1].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[1].rang_elem[0].start = REG_DOMAIN_IC_START_CH_0;
	ch_rang_b[1].rang_elem[0].stop  = REG_DOMAIN_IC_STOP_CH_0;
	ch_rang_b[1].rang_elem[0].step = REG_DOMAIN_IC_STEP_OPT_0;
	ch_rang_b[1].rang_elem[0].power_limit = REG_DOMAIN_IC_POWER_LIMIT_CH_0;

	ch_rang_b[2].cur_regdomain.type = TIAP_PARAM_RD_ETSI;
	ch_rang_b[2].ranges_nr = 1;
	if( !(ch_rang_b[2].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[2].rang_elem[0].start = REG_DOMAIN_ETSI_START_CH_0;
	ch_rang_b[2].rang_elem[0].stop  = REG_DOMAIN_ETSI_STOP_CH_0;
	ch_rang_b[2].rang_elem[0].step = REG_DOMAIN_ETSI_STEP_OPT_0;
	ch_rang_b[2].rang_elem[0].power_limit = REG_DOMAIN_ETSI_POWER_LIMIT_CH_0;

	ch_rang_b[3].cur_regdomain.type = TIAP_PARAM_RD_SPAIN;
	ch_rang_b[3].ranges_nr = 1;
	if( !(ch_rang_b[3].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[3].rang_elem[0].start = REG_DOMAIN_SPAIN_START_CH_0;
	ch_rang_b[3].rang_elem[0].stop  = REG_DOMAIN_SPAIN_STOP_CH_0;
	ch_rang_b[3].rang_elem[0].step = REG_DOMAIN_SPAIN_STEP_OPT_0;
	ch_rang_b[3].rang_elem[0].power_limit = REG_DOMAIN_SPAIN_POWER_LIMIT_CH_0;

	ch_rang_b[4].cur_regdomain.type = TIAP_PARAM_RD_FRANCE;
	ch_rang_b[4].ranges_nr = 1;
	if( !(ch_rang_b[4].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[4].rang_elem[0].start = REG_DOMAIN_FRANCE_START_CH_0;
	ch_rang_b[4].rang_elem[0].stop  = REG_DOMAIN_FRANCE_STOP_CH_0;
	ch_rang_b[4].rang_elem[0].step = REG_DOMAIN_FRANCE_STEP_OPT_0;
	ch_rang_b[4].rang_elem[0].power_limit = REG_DOMAIN_FRANCE_POWER_LIMIT_CH_0;

	ch_rang_b[5].cur_regdomain.type = TIAP_PARAM_RD_MKK;
	ch_rang_b[5].ranges_nr = 1;
	if( !(ch_rang_b[5].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[5].rang_elem[0].start = REG_DOMAIN_MKK_START_CH_0;
	ch_rang_b[5].rang_elem[0].stop = REG_DOMAIN_MKK_STOP_CH_0;
	ch_rang_b[5].rang_elem[0].step = REG_DOMAIN_MKK_STEP_OPT_0 | NO_OFDM_OPT;
	ch_rang_b[5].rang_elem[0].power_limit = REG_DOMAIN_MKK_POWER_LIMIT_CH_0;

	ch_rang_b[6].cur_regdomain.type = TIAP_PARAM_RD_MKK1;
	ch_rang_b[6].ranges_nr = 2;
	if( !(ch_rang_b[6].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[6].rang_elem[0].start = REG_DOMAIN_MKK1_START_CH_0;
	ch_rang_b[6].rang_elem[0].stop  = REG_DOMAIN_MKK1_STOP_CH_0;
	ch_rang_b[6].rang_elem[0].step = REG_DOMAIN_MKK1_STEP_OPT_0;
	ch_rang_b[6].rang_elem[0].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_0;
	ch_rang_b[6].rang_elem[1].start = REG_DOMAIN_MKK1_START_CH_1;
	ch_rang_b[6].rang_elem[1].stop  = REG_DOMAIN_MKK1_STOP_CH_1;
	ch_rang_b[6].rang_elem[1].step = REG_DOMAIN_MKK1_STEP_OPT_1 | NO_OFDM_OPT;
	ch_rang_b[6].rang_elem[1].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_1;

    ch_rang_b[7].cur_regdomain.type = TIAP_PARAM_RD_ISRAEL;
	ch_rang_b[7].ranges_nr = 1;
	if( !(ch_rang_b[7].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[7].rang_elem[0].start = REG_DOMAIN_ISRAEL_START_CH_0;
	ch_rang_b[7].rang_elem[0].stop  = REG_DOMAIN_ISRAEL_STOP_CH_0;
	ch_rang_b[7].rang_elem[0].step = REG_DOMAIN_ISRAEL_STEP_OPT_0;
	ch_rang_b[7].rang_elem[0].power_limit = REG_DOMAIN_ISRAEL_POWER_LIMIT_CH_0;
}


static int get_domain_elems_num(int band, __u8 domain)
{
	switch (band)
	{
		case BAND_A:
			switch (domain)
			{
				case TIAP_PARAM_RD_US:
					return NUM_OF_RANGE_ELEMENTS_US;

				case TIAP_PARAM_RD_WORLD:
					return NUM_OF_RANGE_ELEMENTS_WORLD;

				default:
					break;
			}
			break;

		case BAND_B:
			switch (domain)
			{
				case TIAP_PARAM_RD_FCC:
					return 1;

				case TIAP_PARAM_RD_IC:
					return 1;

				case TIAP_PARAM_RD_ETSI:
					return 1;

				case TIAP_PARAM_RD_SPAIN:
					return 1;

				case TIAP_PARAM_RD_FRANCE:
					return 1;

				case TIAP_PARAM_RD_MKK:
					return 1;

				case TIAP_PARAM_RD_MKK1:
					return 2;

                case TIAP_PARAM_RD_ISRAEL:
                    return 1;

				default:
					break;
			}
			break;

		default:
			break;
	}
	return 0;
}


static int get_rangeElement(int band,
		tiap_param_cur_reg_domain_t cur_reg_domain,
		UINT8         RangeIndex,
		tiap_param_channel_rang_elem_t *rang_elem)
{
	if (RangeIndex == 0)    /* index is a positive number */
		return 1;

	switch (band)
	{
		case BAND_A:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_US:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_US)
						return 1;
					memcpy(rang_elem, &ch_rang_a[0].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_WORLD:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_WORLD)
						return 1;
					memcpy(rang_elem, &ch_rang_a[1].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				default: return 1;
			}
			break;
		case BAND_B:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_FCC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[0].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_IC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[1].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_ETSI:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[2].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_SPAIN:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[3].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_FRANCE:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[4].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[5].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK1:
					if (RangeIndex > 2)
						return 1;
					memcpy(rang_elem, &ch_rang_b[6].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

                case TIAP_PARAM_RD_ISRAEL:
                    if (RangeIndex > 1)
                        return 1;
                    memcpy(rang_elem, &ch_rang_b[7].rang_elem[RangeIndex-1],
                        sizeof(*rang_elem));
                    break;

				default:
					return 1;
			}

			break;

		default:
			return 1;
	}
	return OK;
}



/********************************
*		SETUP PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetApMacAddress(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns AP mac address. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP mac address												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetApMacAddress(void)
{
	__u8 p[ETH_ALEN];

	tiap_wcfg_get_hwaddr(wd, p);
		
	sprintf(tempbuf,"%02X-%02X-%02X-%02X-%02X-%02X",
            p[0], p[1], p[2], p[3], p[4], p[5]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetChannel(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns AP channel. 										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP channel													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetChannel(void)
{
#if 0
	unsigned char chanId;

	tempbuf[0] = 0;
	
	wdrv_dbGetChannel(0,&chanId);
	sprintf(tempbuf,"%d",chanId);

	return tempbuf;
#endif
	return (char*)NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetSsid(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns the SSID. 											*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     SSID														*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetSsid(const char *profile)
{
	char ssid[IW_ESSID_MAX_SIZE+1];
	int i, j;
	int nprofile;

	nprofile = atoi(profile);

	validate_profile(profile);

	if( !nprofile ) 
	{
		if( tiap_wcfg_get_primary_ssid(wd, ssid, !FROM_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get ssid.");
			return (char*)NULL;
		}
	}
	else 
	{
		int ssid_index;
		tiap_param_multi_ssid_t multi_ssid;

		if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid");
			return (char*)"";
		}

		if ((!multi_ssid.secondary_enabled) || (profile_map_tbl[nprofile] == -1))
			return "\"\"";

		if( tiap_wcfg_get_actual_ssid_ind(&multi_ssid, 
					profile_map_tbl[nprofile], &ssid_index) )
			return (char*)"";
		
		memcpy(ssid, multi_ssid.tbl[ssid_index].ssid, sizeof(ssid));
		ssid[IW_ESSID_MAX_SIZE] = '\0';
	}
	
	strcpy(tempbuf, "\"");

	for(i=0, j=strlen(tempbuf); i<=strlen(ssid); i++, j++) {
		if( ssid[i] == '"' ) {
			tempbuf[j] = '\0';
			strcat(tempbuf, "&#148");
			j = strlen(tempbuf) - 1;
		}
		else tempbuf[j] = ssid[i];
	}
	
	strcat(tempbuf, "\"");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetHiddenSsid(void)						    */
/****************************************************************************/
/* DESCRIPTION: Returns the SSID. 											*/ 
/*																		    */
/*																		    */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Hidden SSID Checked button                                  */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetHiddenSsid(void)
{
    __u16 hidden_ssid;

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_HIDDEN_SSID, &hidden_ssid, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get hidden ssid.");
	    return (char*)NULL;
    }

    return (hidden_ssid ? "checked" : "");

}


/****************************************************************************/
/*                      wlan_htmlVhcfCheck(void)								*/
/*                      wlan_htmlVhcfGetIpAddr1(void)							*/
/*                      wlan_htmlVhcfGetIpAddr2(void)							*/
/*                      wlan_htmlVhcfGetIpAddr3(void) 							*/
/*                      wlan_htmlVhcfGetIpAddr4(void) 							*/
/*                      wlan_htmlVhcfGetProtocol1(void)							*/
/*                      wlan_htmlVhcfGetPort1(void)								*/
/*                      wlan_htmlVhcfGetProtocol2(void)							*/
/*                      wlan_htmlVhcfGetPort2(void)								*/
/****************************************************************************/
/*																		    */
/* DESCRIPTION:  Return the vHCF parameters current status					*/ 
/*																		    */
/****************************************************************************/
char* wlan_htmlVhcfCheck(char* value)
{
#if 0
	int vhcfMode;
	UINT8 TotalWones;

	sscanf(value,"%d",&vhcfMode);

	wdrv_dbGetNumSupWone(&TotalWones);

	if (TotalWones == WONE_DUAL_BAND)
	{
		if (vhcfMode == 2) 
            return "checked disabled";
		else
            return " disabled";
	}

	if (wdrv_dbIsVhcfEnabled(void))
	{
        if (vhcfMode == 1)      
            return "checked";
		else
		    return " ";
	}
	else
	{
		if (vhcfMode == 2) 
            return "checked";
		else
			return " ";
    }
#endif

	int vhcfMode;
	__u8 wone_nr;
	tiap_param_vhcf_t vhcf;
	tiap_param_4x_t _4x;

	return " ";


	sscanf(value,"%d",&vhcfMode);

	/* If Dual AP is enabled, don't allow to set vHCF */
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	/* If 4x is enabled, don't allow to set vHCF */
	_4x.cmd = TIAP_PARAM_4X_4X;
	if (tiap_wcfg_get_4x(wd, &_4x, FROM_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
		return (char*)NULL;
	}

	if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND || _4x.value == TRUE) {
		if (vhcfMode == 2) 
			return "checked disabled";
		else
			return " disabled";
	}

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	if (vhcf.enable) {
		if (vhcfMode == 1)      
			return "checked";
		else
			return " ";
	}
	else {
		if (vhcfMode == 2) 
			return "checked";
		else
			return " ";
	}

}


char* wlan_htmlVhcfGetIpAddr1(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0xFF000000)>>24);
	return tempbuf;
#endif

	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy,dig;
	struct in_addr in;
	
	return " ";

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}
	
	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d",&dig, &dummy,&dummy,&dummy);
	
	sprintf(tempbuf,"%d",dig);
	
	
	return tempbuf;

}

char* wlan_htmlVhcfGetIpAddr2(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x00FF0000)>>16);
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;

	return " ";

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dig,&dummy,&dummy);
	sprintf(tempbuf,"%d",dig);
	return tempbuf;

}

char* wlan_htmlVhcfGetIpAddr3(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x0000FF00)>>8);
	return tempbuf;
#endif

	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;
	return " ";

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dummy,&dig,&dummy);
	sprintf(tempbuf,"%d",dig);

	return tempbuf;
}

char* wlan_htmlVhcfGetIpAddr4(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x000000FF));
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;
	return " ";

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dummy,&dummy,&dig);
	sprintf(tempbuf,"%d",dig);

	return tempbuf;

}

char* wlan_htmlVhcfGetProtocol1(char *option)
{
#if 0
    tVHCF_PARAMS  vhcfParams;
	int inputOption;

    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */

    sscanf(option,"%d",&inputOption);

	if (inputOption == vhcfParams.vhcfClassifier[0].Protocol + 1)
		return "selected";
	else
		return " ";
#endif

	tiap_param_vhcf_t vhcf;
	int inputOption;
	return " ";

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}
	
	sscanf(option,"%d",&inputOption);
	
	if (inputOption == vhcf.classifier[0].proto + 1)
		return "selected";
	else return " ";
}

char* wlan_htmlVhcfGetPort1(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",vhcfParams.vhcfClassifier[0].DestPort);
	return tempbuf;
#endif

	tiap_param_vhcf_t vhcf;
	return " ";

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	sprintf(tempbuf,"%d", vhcf.classifier[0].dest_port);
	return tempbuf;
}

char* wlan_htmlVhcfGetProtocol2(char *option)
{
#if 0
    tVHCF_PARAMS  vhcfParams;
	int inputOption;

    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */

    sscanf(option,"%d",&inputOption);

	if (inputOption == vhcfParams.vhcfClassifier[1].Protocol + 1)
		return "selected";
	else
		return " ";
#endif

	tiap_param_vhcf_t vhcf;
	int inputOption;
	
	return " ";

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	sscanf(option,"%d",&inputOption);

	if (inputOption == vhcf.classifier[1].proto + 1)
		return "selected";
	else
		return " ";
}

char* wlan_htmlVhcfGetPort2(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",vhcfParams.vhcfClassifier[1].DestPort);
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	return " ";

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}

	sprintf(tempbuf,"%d", vhcf.classifier[1].dest_port);
	return tempbuf;

}




char* wlan_htmlNotYet(void)
{
    return " ";
}

char* wlan_htmlNotYetValue(char *value)
{
    return "X";
}

/****************************************************************************/
/*                   wlan_htmlGetPucketBurstSupport(void)                       */
/****************************************************************************/
/* DESCRIPTION: Returns the Video Blust status.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetPucketBurstSupport(char *value)
{
#if 0   /* unsupported yet */
    UINT32 VideoBlustSupport = 0;
	int mode;

	sscanf(value,"%d",&mode);
    
    switch (VideoBlustSupport)
    {
        case 0:
            if (mode == 2) 
                return "checked";
            break;
        case 1:
            if (mode == 1) 
                return "checked";
            break;
    }
    
    return " ";
#else
    return " disabled=true ";
#endif
	return (char*)NULL;
}


/****************************************************************************/
/*                    wlan_htmlGetOutputPowerLevel(void)                        */
/****************************************************************************/
/* DESCRIPTION: Returns the Output Power Level.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetOutputPowerLevel(char *value)
{
#if 0
    UINT32 VideoBlustSupport = 0;
    UINT8 PowerLevel;
	int mode;


    wdrv_dbGetPowerLevel(whal_ParamsGetRadioChannel(0),&PowerLevel);

	sscanf(value,"%d",&mode);
    
    switch (PowerLevel)
    {
        case 1:
            if (mode == 0) 
                return "checked";
            break;
        case 2:
            if (mode == 1) 
                return "checked";
            break;
        case 3:
            if (mode == 2) 
                return "checked";
            break;
        case 4:
            if (mode == 3) 
                return "checked";
            break;
        case 5:
            if (mode == 4) 
                return "checked";
            break;
    }
    
    return " ";
#endif

    UINT32 PowerLevel;
    char buf[4096];
    int mode, len;

    if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_POWER_LEVEL,
				    buf, sizeof(buf))) == -1 ) {
	    PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_POWER_LEVEL);
	    return (char*)NULL;
    }
	sscanf(buf, "%d", &PowerLevel);
    sscanf(value,"%d",&mode);

    switch (PowerLevel)
    {
	    case 1:
		    if (mode == 0) 
			    return "checked";
		    break;
	    case 2:
		    if (mode == 1) 
			    return "checked";
		    break;
	    case 3:
		    if (mode == 2) 
			    return "checked";
		    break;
	    case 4:
		    if (mode == 3) 
			    return "checked";
		    break;
	    case 5:
		    if (mode == 4) 
			    return "checked";
		    break;
    }

    return " ";
}


/****************************************************************************/
/*                       wlan_htmlGet_A_G_PlusMode(void)                        */
/****************************************************************************/
/* DESCRIPTION: Returns the Video Blust status.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGet_A_G_PlusMode(char *value)
{
	int mode;
	tiap_param_4x_t _4x;
	tiap_param_vhcf_t vhcf;

	sscanf(value,"%d",&mode);


	_4x.cmd = TIAP_PARAM_4X_4X;
	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to get 4x.");
		return (char*)NULL;
	}

	if( tiap_wcfg_get_vhcf(wd, &vhcf, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return (char*)NULL;
	}


	switch (_4x.value)
	{
		case FALSE:
			if (mode == 2) 
			{
				if (vhcf.enable) 
					return "checked disabled";
				else
					return "checked";
			}
			else
			{
				if (vhcf.enable) 
					return " disabled";
				else
					return " ";
			}
			break;
		case TRUE:
			if (mode == 1) 
			{
				if (vhcf.enable) 
					return "checked disabled";
				else
					return "checked";
			}
			else
			{
				if (vhcf.enable) 
					return " disabled";
				else
					return " ";

			}
			break;
	}

	return " ";
}


/****************************************************************************/
/*                     wlan_htmlConfigEnergyDetection(void)                     */
/****************************************************************************/
/* DESCRIPTION: Returns the Interference Avoidance status.                  */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlConfigEnergyDetection(char *value)
{
#if 0
	int mode;
	UINT8 energyDetect;

	sscanf(value,"%d",&mode);

	wdrv_dbGetEnergyDetect(&energyDetect);
	switch (energyDetect)
	{
		case FALSE:
			if (mode == 2)
				return "checked";
			break;
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
	}
    
    return " ";
#endif

    int mode;
    __u8 energy_detect;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_ENERGY_DETECT, &energy_detect, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get energy detect configuration.");
	    return (char*)NULL;
    }

    switch (energy_detect)
    {
	    case FALSE:
		    if (mode == 2)
			    return "checked";
		    break;
	    case TRUE:
		    if (mode == 1) 
			    return "checked";
		    break;
    }

    return " ";

}

/****************************************************************************/
/*                     wlan_htmlConfigRadarDetectMode(void)                 */
/****************************************************************************/
/* DESCRIPTION: Returns the Radar detection mode							*/ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char *wlan_htmlConfigRadarDetectMode(char *value)
{
    int mode;
    __u8 radar_detect;
    tiap_param_wone_t wone;
    __u8 wone_nr;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get radar detect mode configuration.");
	    return (char*)NULL;
    }

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return (char*)NULL;
    }

    if (wone_nr != TIAP_PARAM_WONE_DUAL_BAND)
    {
	    wone.index = 0;
	    wone.type = wone_band;
	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get band.");
		    return (char*)NULL;
	    }

	    if( wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ )
		    return "disabled";
    }

    switch (radar_detect)
    {
	    case FALSE:
		    if (mode == 2)
			    return "checked";
		    break;
	    case TRUE:
		    if (mode == 1) 
			    return "checked";
		    break;
    }

    return " ";
}

/****************************************************************************/
/*                     wlan_htmlConfigBssIsolationMode(void)                */
/****************************************************************************/
/* DESCRIPTION: Returns the Bss Isolation mode							*/ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char *wlan_htmlConfigBssIsolationMode(char *value)
{
    int mode;
    __u32 bss_bridge_state;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get radar detect mode configuration.");
	    return (char*)NULL;
    }

    switch (bss_bridge_state)
    {
	    case TIAP_PARAM_BSS_BR_BLOCK:
		    if (mode == 1)
			    return "checked";
		    break;
	    default:
		    if (mode == 2) 
			    return "checked";
		    break;
    }

    return " ";
}



/****************************************************************************/
/*                      wlan_htmlSavePageSetup(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the setup page.						*/ 
/*																		    */
/*																		    */
/* INPUT:       Setup page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlSavePageSetup(const char *save_button, const char *enable_ap, char *ssid, char *hidden_ssid,
		char *vlan, char *band_select, char *channel_b_g, char *channel_a,
		char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa2_pre_auth, char *wpa_auth_radio, char *psk_value,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	int channel, len, bandType;
	BOOL BandTypeWasSelected = FALSE;
	__u8 enable_ap_val, wone_nr;
	tiap_param_wone_t wone;
	__u16 hidden_ssid_val, CurrentVlan;
	char *privacyResult = NULL;
    int savingIndex = 0, tablePhyIndex, vlan_id;
	BOOL DoSave = FALSE, mSSID_isEnabled = FALSE;
	tiap_param_multi_ssid_t multi_ssid;
    __u32 security_select_val;
	
	if (strncmp(save_button, "Saving",6))
        return "";
	if (strcmp(save_button, "Saving") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+6);
    
	if (DoSave || (globalWebSavingIndex <= savingIndex))
	{
		/* test vlan - should be unique */


		if( tiap_wcfg_get_primary_vlan(wd, &CurrentVlan, FROM_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get primary vlan.");
			return "";
		}
		if( CurrentVlan ) 
			mSSID_isEnabled = TRUE;;

		if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
			mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
			return (char*)"";
		}
		if (tiap_wcfg_lookUpMultiSsidBySsid(&multi_ssid, ssid, NULL, &tablePhyIndex) == OK) 
		{
			if (tablePhyIndex != 0) 
			{
				PRINT_CONFIG_ERR_MSG("The given SSID already exists in the Multiple SSID table. Setting aborted!\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS;
				return (char*)"";
			}
		}
		vlan_id = atoi(vlan);
		if (vlan_id <0) 
		{
			PRINT_CONFIG_ERR_MSG("Bad Vlan value\n");
			mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR;
			return (char*)"";
		}
		else if (vlan_id > 0) 
		{
			if (vlan_id >= 4096) 	/* MAX VLAN is 4095 => 12 bits */
			{
				PRINT_CONFIG_ERR_MSG("Vlan exeeds its maximal value - 4095\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_OVERFLOW;
				return (char*)"";
			}
			if (tiap_wcfg_lookUpMultiSsidByVlan(&multi_ssid, vlan_id, NULL, &tablePhyIndex) == OK) 
			{
				if (tablePhyIndex != 0) 
				{
					PRINT_CONFIG_ERR_MSG("Vlan already in use for ssid\n");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS;
					return (char*)"";
				}
			}

			/* enable multiple ssid feature */
			multi_ssid.secondary_enabled = 1;
			if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to set multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}

            /* If Multiple SSIDs is Enabled:                             *
             * If there is a profile configured to static WEP -->        *
             * Authentication Algorithm is Both Open System & Shared Key */
            security_select_val = atoi(security_select);
            if(security_select_val == 4 /* WEP */)
            {
                *wep_auth_select = '2';
            }
			
		}
		/* save security settings first - a lot to check there.. */
		if (TestPrivacy("0", security_select, wep_auth_select,
									wep_cipher_select, wep_key_radio, wep_key_val0,
									wep_key_val1, wep_key_val2, wep_key_val3,
									wpa2_pre_auth, wpa_auth_radio, psk_value,
									radius_ipaddr_octet0, radius_ipaddr_octet1,
									radius_ipaddr_octet2, radius_ipaddr_octet3,
									radius_port, radius_secret, gtk) == FALSE)
		{
			printf("*********** Security NOT OK ************\n");
			return "";
		}
		
		if ((mSSID_isEnabled == TRUE) && (vlan_id == 0))
		{
			int i;
			for (i=0; i< TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
			{
				if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY)
				{
					tiap_wcfg_delMultiSsid(wd, i, !UPDATE_STORAGE);
				}
				
			}
		}
		/* enable AP */
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &enable_ap_val, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
			return " ";
		}

		enable_ap_val = 2;
		if( !enable_ap_val && atoi(enable_ap) )
			enable_ap_val = 1;
		else if( enable_ap_val && !atoi(enable_ap) )
			enable_ap_val = 0;

		if(enable_ap_val != 2 &&
		   tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &enable_ap_val, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
			return " ";
		}


		/* ssid */
		len = strlen(ssid);
		if (len <= IW_ESSID_MAX_SIZE)
			if( tiap_wcfg_set_primary_ssid(wd, ssid, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set ssid.");
				return " ";
			}

		/*  hidden ssid */
		if( !strcmp(hidden_ssid, "NO_PARAM" ) )
			hidden_ssid_val = 0;
		else 
			hidden_ssid_val = atoi(hidden_ssid);

		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_HIDDEN_SSID, &hidden_ssid_val, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set hidden ssid.");
			return " ";
		}

		/* Vlan */
		if(tiap_wcfg_set_primary_vlan(wd, vlan_id, !UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set primary vlan.\n");
			return " ";
		}
		if (vlan_id == 0) 
		{
            tiap_param_qos_t QoSParams;

			if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}
            if ( tiap_wcfg_get_qos_params(wd, &QoSParams, FROM_STORAGE) ) {
                PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
                return (char*)"";
            }
            if( QoSParams.enabled && QoSParams.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_VLAN )
            {
                printf("Warning: Setting WME to disable before disabling Multiple SSID.\n");
                QoSParams.enabled = 0;
                if ( tiap_wcfg_set_qos_params(wd, (const tiap_param_qos_t *)&QoSParams, UPDATE_STORAGE) ) {
                    PRINT_CONFIG_ERR_MSG("Failed to set QoS params.");
                    return (char*)"";
                }
            }
			multi_ssid.secondary_enabled = 0;
			if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to set multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}
		}
		/* Band Type */
		sscanf(band_select, "%d", &bandType);
		if (bandType == 1)  /* band is B/G */
		{
			BandTypeWasSelected = TRUE;

			/* Check is working in dual band mode */
			if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
				return " ";
			}

			/* if yes - move to single band mode */
			if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND) {
				wone_nr = 1;
				if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
					return " ";
				}
			}

			/* check if already configred to B/G */
			/* if not - switch to the B/G band. */
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return " ";
			}
			if (wone.elem.band != TIAP_PARAM_RADIO_BAND_2_4GHZ)
				if( tiap_wcfg_set_dot11mode(wd, 0, 0, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
					return " ";
				}

			/* set channel */
			sscanf(channel_b_g, "%d", &channel);
			wone.index = 0;
			wone.type = wone_ch;
			wone.elem.ch = channel;
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set channel.");
				return " ";
			}
		}
		if (BandTypeWasSelected == FALSE)
		{
			sscanf(channel_a, "%d", &bandType);
			if (bandType == 2)  /* band is A */
			{
				BandTypeWasSelected = TRUE;

				/* Check is working in dual band mode */
				if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
					return " ";
				}

				/* if yes - move to single band mode */
				if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND) {
					wone_nr = 1;
					if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
						return " ";
					}
				}

				/* check if already configred to A */
				/* if not - switch to the A band. */
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return " ";
				}
				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
					if( tiap_wcfg_set_dot11mode(wd, 4, 0, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return " ";
					}

				/* set channel */
				sscanf(channel_a, "%d", &channel);
				wone.index = 0;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return " ";
				}
			}
		}

		if (BandTypeWasSelected == FALSE)
		{
			sscanf(band_select, "%d", &bandType);
			if (bandType == 3)  /* band is both B/G  & A */
			{
				BandTypeWasSelected = TRUE;
				/* Check if working in single mode */
				if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
					return " ";
				}
				/* if yes - switch to dual band mode.. */
				if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) {
					wone_nr = 2;
					if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
						return " ";
					}
				}

				/* ..and then set Wone 1 to B/G and Wone 2 to A */
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return " ";
				}
				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_2_4GHZ)
				{
					if( tiap_wcfg_set_dot11mode(wd, 0, 0, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return " ";
					}
				}
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return " ";
				}

				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
					if( tiap_wcfg_set_dot11mode(wd, 4, 1, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return " ";
					}

				/* set channels */
				sscanf(channel_b_g, "%d", &channel);
				wone.index = 0;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return " ";
				}
				sscanf(channel_a, "%d", &channel);
				wone.index = 1;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return " ";
				}
			}
		}
		privacyResult = SavePrivacy("0", security_select, wep_auth_select,
									wep_cipher_select, wep_key_radio, wep_key_val0,
									wep_key_val1, wep_key_val2, wep_key_val3,
									wpa2_pre_auth, wpa_auth_radio, psk_value,
									radius_ipaddr_octet0, radius_ipaddr_octet1,
									radius_ipaddr_octet2, radius_ipaddr_octet3,
									radius_port, radius_secret, gtk);
		if (strcmp(privacyResult,"OK") != 0)
		{
			return privacyResult;
		}
		wlan_htmlInit();
		restart();
	}

	return "";
}

/****************************************************************************/
/*                wlan_htmlGetChannelSelectBandBG(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the channel drop down			*/ 
/*				text box - BAND B.											*/
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's channel	*/
/*				"" if the value is different than the AP's channel			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetChannelSelectBandBG(void)
{
    UINT8 ch;
    UINT32  numOfRanges, i;
    char webBuffer[40];

    tiap_param_cur_reg_domain_t cur_regdomain, actual_regdomain;
    tiap_param_wone_t wone;
    tiap_param_channel_rang_elem_t rang_elem;

    cur_regdomain.band = TIAP_PARAM_RADIO_BAND_2_4GHZ;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return (char*)NULL;
    }
    
    wone.index = 0;
    wone.type = wone_ch;
    wone.elem.ch = 0;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get channel.");
	    return (char*)NULL;
    }
    tempbuf[0] = '\0';

    switch (cur_regdomain.type) 
    {
	    case TIAP_PARAM_RD_FCC:
	    case TIAP_PARAM_RD_US:
		    actual_regdomain.type = TIAP_PARAM_RD_FCC;
		    break;
	    case TIAP_PARAM_RD_ETSI:
	    case TIAP_PARAM_RD_WORLD:
		    actual_regdomain.type = TIAP_PARAM_RD_ETSI;
		    break;
	    case TIAP_PARAM_RD_IC:
	    case TIAP_PARAM_RD_SPAIN:
	    case TIAP_PARAM_RD_FRANCE:
	    case TIAP_PARAM_RD_MKK:
		case TIAP_PARAM_RD_MKK1:
        case TIAP_PARAM_RD_ISRAEL:
			actual_regdomain.type = cur_regdomain.type;
		    break;

	    default:
		    printf(tempbuf,"<OPTION selected value=0>0 ");
		    break;
    }
    numOfRanges = get_domain_elems_num(BAND_B, actual_regdomain.type);
    for (i = 1; i <= numOfRanges; i++) 
    {
	    get_rangeElement(BAND_B, actual_regdomain, i, &rang_elem);
	    for (ch=rang_elem.start;
			    ch<=rang_elem.stop;
			    ch+=(rang_elem.step & 0xF)) 
	    {
		    if (ch == wone.elem.ch) 
			    sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
		    else
			    sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
		    strcat(tempbuf,webBuffer);
	    }
    }
    return tempbuf;

}


/****************************************************************************/
/*                wlan_htmlGetChannelSelectBandA(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the channel drop down			*/ 
/*				text box - BAND A.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's channel	*/
/*				"" if the value is different than the AP's channel			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetChannelSelectBandA(void)
{
    UINT8 ch;
    UINT32  numOfRanges, i;
    char webBuffer[40];
   
    __u8 wone_nr;
    tiap_param_cur_reg_domain_t cur_regdomain, actual_regdomain;
    tiap_param_wone_t wone;
    tiap_param_channel_rang_elem_t rang_elem;
	
    cur_regdomain.band = TIAP_PARAM_RADIO_BAND_5GHZ;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return (char*)NULL;
    }

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return (char*)NULL;
    }

    wone.index = (wone_nr == TIAP_PARAM_WONE_DUAL_BAND ? 1 : 0);

    wone.type = wone_ch;
    wone.elem.ch = 0;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get channel.");
	    return (char*)NULL;
    }

    tempbuf[0] = '\0';

    switch (cur_regdomain.type)
    {
	    case TIAP_PARAM_RD_FCC:
		case TIAP_PARAM_RD_US:
			actual_regdomain.type = TIAP_PARAM_RD_US;
		    break;
	    case TIAP_PARAM_RD_IC:
	    case TIAP_PARAM_RD_ETSI:
	    case TIAP_PARAM_RD_SPAIN:
	    case TIAP_PARAM_RD_FRANCE:
	    case TIAP_PARAM_RD_MKK:
	    case TIAP_PARAM_RD_MKK1:
        case TIAP_PARAM_RD_ISRAEL:
		case TIAP_PARAM_RD_WORLD:
			actual_regdomain.type = TIAP_PARAM_RD_WORLD;
		    break;

	    default:
		    printf(tempbuf,"<OPTION selected value=0>0 ");
		    break;
    }
    numOfRanges = get_domain_elems_num(BAND_A, actual_regdomain.type);
    for (i = 1; i<= numOfRanges; i++) 
    {
	    get_rangeElement(BAND_A, actual_regdomain, i, &rang_elem);
	    for (ch=rang_elem.start;
			    ch<=rang_elem.stop;
			    ch+=(rang_elem.step & 0xF)) 
	    {
			if (ch > 64) { /* currently we assume that such channels are not supported */
				continue;
			}
			/*Alexa: TODO: check the type of the board and if it supports
			channels higher than 64 to allow them */

		    if (ch == wone.elem.ch) 
			    sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
		    else
			    sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
		    strcat(tempbuf,webBuffer);
	    }
    }
    return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetBandMode(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetBandMode(char *option)
{
    UINT32 boardType;
    __u8 wone_nr;
    int bandType, len;
    char buf[4096];
    tiap_param_wone_t wone;

    sscanf(option,"%d",&bandType);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return (char*)NULL;
    }

    if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ, buf, sizeof(buf))) == -1 ) {
	    PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ);
	    return (char*)NULL;
    }
    sscanf(buf, "%u", &boardType);
    switch(bandType)
    {
	    case 1: /* B */
		    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		    {
			    wone.index = 0;
			    wone.type = wone_band;
			    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				    PRINT_CONFIG_ERR_MSG("Failed to get band.");
				    return (char*)NULL;
			    }
			    if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ) 
				    return "<option value=1 selected>802.11b/g (2.4GHz)</option> ";
				else
					return "<option value=1>802.11b/g (2.4GHz)</option> ";
		    }
			else
				return "<option value=1>802.11b/g (2.4GHz)</option> ";
		    break;

	    case 2: /* A */
		    if (boardType != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ) 
			    return " ";
		    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		    {
			    wone.index = 0;
			    wone.type = wone_band;
			    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				    PRINT_CONFIG_ERR_MSG("Failed to get band.");
				    return (char*)NULL;
			    }
			    if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ) 
				    return "<option value=2 selected>802.11a   (5GHz)</option>";
				else
				    return "<option value=2>802.11a   (5GHz)</option>";
		    }
			else
				return "<option value=2>802.11a   (5GHz)</option>";
		    break;

	    case 3: /* both */
		    if (boardType != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ)
			    return " ";
		    if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND)
			    return "<option value=3 selected>wONE</option>";
			else
			    return "<option value=3>wONE</option>";
		    break;
    }
    return " ";
}


/****************************************************************************/
/*                      wlan_htmlGetFwVersion(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetFwVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = '\0';

	if( (len = dda_read_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FW_VER_ONLY, buf, sizeof(buf))) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FW_VER_ONLY);
	if( len > sizeof(buf) ) {
#ifndef VXWORKS
		fprintf(stderr, "\nweb: The returned length is bigger then the input one,"
				"in calling %s/%s proc entry\n",
				WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FW_VER_ONLY);
#else		
		fprintf(stderr, "\nweb: The returned length is bigger then the input one,"
				"in calling %d/%d proc entry\n",
				WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FW_VER_ONLY);
#endif  /* VXWORKS */
		return (char*)NULL;
	}
	memcpy(tempbuf, buf, sizeof(buf));
	tempbuf[len] = '\0';

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetSwVersion(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the SW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Sw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetSwVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = 0;
	
	if( (len = dda_read_proc_entry(WLAN_PROC_PROD_DIR_NAME,
                                        WLAN_PROC_PROD_VERSION, buf, sizeof(buf))) == -1 )
                PRINT_PROC_READ_ERR(WLAN_PROC_PROD_DIR_NAME, WLAN_PROC_PROD_VERSION);

	memcpy(tempbuf, buf, len);
	tempbuf[len]='\0';

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetRadioModuleVersion(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the Radio Module version.  							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Sw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetRadioModuleVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = 0;
	
	if( (len = dda_read_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RADIO_VER_ONLY, buf, sizeof(buf))) == -1 )
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RADIO_VER_ONLY);

	memcpy(tempbuf, buf, sizeof(buf));
	tempbuf[len]='\0';

	return tempbuf;
}
	

/****************************************************************************/
/*                      wlan_htmlGetEepromVersion(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the EEPROM version.									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Sw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetEepromVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = 0;
	
	if( (len = dda_read_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EEPROM_VER_ONLY, buf, sizeof(buf))) == -1 )
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EEPROM_VER_ONLY);

	memcpy(tempbuf, buf, sizeof(buf));
	tempbuf[len]='\0';

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetIpAddr1(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[0] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr1(void)
{
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return (char*)NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0xFF000000)>>24);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr2(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[1] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[1] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr2(void)
{
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return (char*)NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x00FF0000)>>16);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr3(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[2] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[2] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr3(void)
{
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return (char*)NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x0000FF00)>>8);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr4(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[3] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[3] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr4(void)
{
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return (char*)NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x000000FF));

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask1(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[0] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask1(void)
{
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return (char*)NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0xFF000000)>>24);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask2(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[1] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[1] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask2(void)
{
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return (char*)NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x00FF0000)>>16);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask3(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[2] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[2] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask3(void)
{
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return (char*)NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x0000FF00)>>8);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask4(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[3] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[3] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask4(void)
{
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return (char*)NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x000000FF));

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetDomain(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the AP's domain										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP's domain													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDomain(void)
{
	tiap_param_cur_reg_domain_t cur_regdomain;
	__u8 wone_nr;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	cur_regdomain.band = 0;
	if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
		return (char*)NULL;
	}

	switch (cur_regdomain.type)
	{
		case TIAP_PARAM_RD_FCC:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("FCC");
			else
				return("FCC(2.4Ghz)   US(5Ghz)");

		case TIAP_PARAM_RD_IC:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("IC");
			else
				return("IC(2.4Ghz)  WORLD(5Ghz)");

		case TIAP_PARAM_RD_ETSI:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("ETSI");
			else
				return("ETSI(2.4Ghz)    WORLD(5Ghz)");

		case TIAP_PARAM_RD_SPAIN:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("SPAIN");
			else
				return("SPAIN(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_FRANCE: 
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("FRANCE");
			else
				return("FRANCE(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_MKK:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("MKK");
			else
				return("MKK(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_MKK1:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("MKK1");
			else
				return("MKK1(2.4Ghz)   WORLD(5Ghz)");

        case TIAP_PARAM_RD_ISRAEL:
            if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
                return("ISRAEL");
            else
                return("ISRAEL(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_US:
			return("US");

		case TIAP_PARAM_RD_WORLD:
			return("WORLD");

		default :
			sprintf(tempbuf,"0x%X", cur_regdomain.type);
			break;
	}

	return tempbuf;
}

 
/********************************
*		CONFIG PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetBeaconPeriod(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the beacon period									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     beacon period												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetBeaconPeriod(void)
{
#if 0
	UINT16 beacon;
    UINT8 TotWone;

	tempbuf[0] = 0;
	
    wdrv_dbGetNumSupWone(&TotWone);
    if (TotWone == WONE_SINGLE_BAND) 
    {
        wdrv_dbGetStationConfigBeaconPeriod(&beacon);
        sprintf(tempbuf,"%d",beacon);
    }
    else
        strcpy(tempbuf,"-- disabled=true");

	return tempbuf;
#endif

	__u16 beacon;
	__u8 wone_nr;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return "";
	}
	if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_BEACON_PERIOD, &beacon, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get beacon period.\n");
			return (char*)NULL;
		}
		sprintf(tempbuf,"%d",beacon);
	}
	else
		strcpy(tempbuf,"-- disabled=true");

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetDtimPeriod(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the dtim period										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     dtim period													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDtimPeriod(void)
{
#if 0
	UINT8 dtim;

	tempbuf[0] = 0;
	
	wdrv_dbGetStationConfigDtimPeriod(&dtim);
	sprintf(tempbuf,"%d",dtim);

	return tempbuf;
#endif

	__u8 dtim;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_DTIM_PERIOD, &dtim, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get DTIM period.\n");
		return (char*)NULL;
	}
	sprintf(tempbuf,"%u", (__u32) dtim);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetRtsThreshold(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the rts threshold									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     rts threshold												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetRtsThreshold(void)
{
	__s32 rts = 0;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_giwrts(wd, &rts, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get rts period.\n");
		return (char*)NULL;
	}
	sprintf(tempbuf,"%hu", (__u16)rts);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetFragThreshold(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the fragmentation threshold							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     fragmentation threshold										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetFragThreshold(void)
{
#if 0
	UINT16 frag;

	tempbuf[0] = 0;
	
	wdrv_dbGetOperationFragmentationThreshold(&frag);
	sprintf(tempbuf,"%d",frag);

	return tempbuf;
#endif
	__s32 frag;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_giwfrag(wd, &frag, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get fragmentation threshold.");
		return (char*)NULL;
	}
	sprintf(tempbuf,"%hu", (__u16)frag);

	return tempbuf;

}

#if 0
static void CheckRateInList(UINT8 *ratesList, UINT8 ratesLen, UINT8 rate, char *str)
{
int i;
BOOL found=FALSE;

	for (i=0; i<ratesLen; i++)
	{
		if (rate == (ratesList[i]&0x7F))
		{
			if (ratesList[i]&0x80)
			{
				strcpy(str, "B");
			}
			else
			{
				strcpy(str, "Y");
			}
			found= TRUE;
			break;
		}
	}

	if (!found)
	{
		strcpy(str, "N");
	}
}
#endif


/****************************************************************************/
/*                      wlan_htmlSavePageConfig(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the config page.						*/ 
/*																		    */
/*																		    */
/* INPUT:       Config page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlSavePageConfig(char* save_button, char *beaconPeriod, char *dtimPeriod, char *rtsThreshold,
		char *fragThreshold, char *inputDot11Mode, char *hiddenSSID,
		char *packetBursting, char *outputPowerLevel, char *plusMode,
		char *qos,
		char *energyDetection, char *radarDetection, char *bssIsolation)
{


	__u8 dtim;
	__u16 beacon, rts, frag,input_dtim;
	int  maxTxPowerLevel;
	UINT8 woneIndex=111;
	__u8 wone_nr;
	tiap_param_wone_t wone;
	tiap_param_4x_t _4x;
	__u8 eneregy_detect;
	__u8 radar_detect;
	tiap_param_bss_br_state_t bss_bridge_state;
	tiap_param_qos_t qos_enable, classifierParams;
    int savingIndex = 0;
	BOOL DoSave = FALSE;
	tiap_param_multi_ssid_t multi_ssid;

	
	if (strncmp(save_button, "Saving",6))
        return "";
	if (strcmp(save_button, "Saving") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+6);
    
	if (DoSave || (globalWebSavingIndex <= (savingIndex + 1))) /* PATCH !!!!*/
	{
		storage_backup = wd->storage;

		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return (char*)NULL;
		}


		/* beacon */
		if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) 
		{ 
			if ( sscanf(beaconPeriod,"%hu",&beacon) <= 0){
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Beacon period must be a valid number");
				return tempbuf;
			}
			if (beacon < MIN_BEACON_PERIOD || beacon > MAX_BEACON_PERIOD) {
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Beacon period is out of range");
				return tempbuf;
			}

			if( tiap_wcfg_set_basic(wd, TIAP_PARAM_BEACON_PERIOD, &beacon, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set beacon period.\n");
				return (char*)NULL;
			}

		}


		/* dtim */
		if ( sscanf(dtimPeriod,"%hu",&input_dtim) <= 0) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"DTIM period must be a valid number");
			return tempbuf;
		}

		if (input_dtim < MIN_DTIM_PERIOD || input_dtim > MAX_DTIM_PERIOD){
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"DTIM period is out of range");
			return tempbuf;
		}

		dtim = input_dtim;
		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_DTIM_PERIOD, &dtim, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set DTIM period.\n");
			return (char*)NULL;
		}
		
		/* rts threshold */
		if (sscanf(rtsThreshold,"%hu",&rts) <= 0 ) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Rts threshold must be a valid number");
			return tempbuf;
		}
		if (rts > MAX_RTS_THRESHOLD) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Rts threshold is out of range");
			return tempbuf;
		}

		if( tiap_wcfg_siwrts(wd, rts, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set rts period.\n");
			return (char*)NULL;
		}

		/* frag threshold */
		if (sscanf(fragThreshold,"%hu",&frag) <= 0){
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Frag threshold must be a valid number");
			return tempbuf;
		}

		if ((frag % 2) !=0 ){
			/* printf("Invalid fragmentation threshold, must be an even number.\n");*/
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Frag threshold must be an even number");
			return tempbuf;
		}
		else {
			if (frag < MIN_FRAG_THRESHOLD || frag > MAX_FRAG_THRESHOLD){
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Frag threshold is out of range");
				return tempbuf;
			}
			if( tiap_wcfg_siwfrag(wd, frag, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set fragmentation threshold.");
				return (char*)NULL;
			}

		}


		/* dot11mode */
		switch (wone_nr) 
		{
			case 1:
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					woneIndex = 99;             /* 99 is a value denotes 'not b/g' */
				else
					woneIndex = 0;
				break;

			case 2:
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return (char*)NULL;
					}

					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						woneIndex = 99;   /* both Wone 0,1 are 'A' */
				}
				break;
		}

		if (woneIndex < 99) 
		{
#ifndef VXWORKS
			sscanf(inputDot11Mode,"%hhu",&wone.elem.dot11_mode);
#else
			wone.elem.dot11_mode = atoi(inputDot11Mode);
#endif
			/* check if already configred to that dot11 mode. */
			/* if not switch to that mode */ 
			if (wone.elem.dot11_mode != wd->storage.ldb.dot11_mode[0].elem.dot11_mode)
			{
				if( tiap_wcfg_set_dot11mode(wd, wone.elem.dot11_mode, 0, !UPDATE_STORAGE) ) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
					return (char*)NULL;
				}
			}
		}


		/* Power Level */
		sscanf(outputPowerLevel,"%d",&maxTxPowerLevel);
		if ((maxTxPowerLevel >= 0) && (maxTxPowerLevel <= 5))
		{
			__u8 i = (__u8)(maxTxPowerLevel+1);
			if( tiap_wcfg_set_basic(wd, TIAP_PARAM_MAX_TX_POWER_LEVEL, &i, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set tx power level.\n");
				return (char*)NULL;
			}
		}


		/* a/g+ ==> 4x */
		if (strcmp(plusMode,"1") == 0)
		{
			_4x.cmd = TIAP_PARAM_4X_4X;
			_4x.value = TRUE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x 4x mode.");
				return (char*)NULL;
			}

			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = TRUE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x concatenation mode.");
				return (char*)NULL;
			}
		}
		else if (strcmp(plusMode,"2") == 0) 
		{
			_4x.cmd = TIAP_PARAM_4X_4X;
			_4x.value = FALSE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x 4x mode.");
				return (char*)NULL;
			}

			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = FALSE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x concatenation mode.");
				return (char*)NULL;
			}
		}


		/* Interference Avoidance */
		if (strcmp(energyDetection,"1") == 0)
		{
			eneregy_detect = TRUE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_ENERGY_DETECT, &eneregy_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set energy detect.");
				return (char*)NULL;
			}
		}
		else if (strcmp(energyDetection,"2") == 0) 
		{
			eneregy_detect = FALSE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_ENERGY_DETECT, &eneregy_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set energy detect.");
				return (char*)NULL;
			}
		}


		/* Radar Detection */		
		if (strcmp(radarDetection,"1") == 0)
		{
			radar_detect = TRUE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set radar detect mode");
				return (char*)NULL;
			}
		}
		else if (strcmp(radarDetection,"2") == 0) 
		{
			radar_detect = FALSE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set radar detect mode");
				return (char*)NULL;
			}
		}


		/* BSS isolation */		
		if (strcmp(bssIsolation,"1") == 0)
		{
			bss_bridge_state = TIAP_PARAM_BSS_BR_BLOCK;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set bss isolation mode");
				return (char*)NULL;
			}
		}
		else if (strcmp(bssIsolation,"2") == 0) 
		{
			bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set bss isolation mode");
				return (char*)NULL;
			}
		}

		/* wme enable state */
		if ( tiap_wcfg_get_qos_params(wd, &qos_enable, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
			return (char*)NULL;
		}
		qos_enable.enabled = atoi(qos);	

		 /* get classification params */
		if ( tiap_wcfg_get_qos_params(wd, &classifierParams, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
			return (char*)NULL;
		}

		classifierParams.enabled = qos_enable.enabled;

		/*
		 * In case of VLAN classification, validate that mSSSID is enabled
		 */

		if (tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE))
		{
			printf("Failed to get multiple ssid structure\n");
			return (char*)NULL;
		}

		if( (classifierParams.enabled) && 
		   (classifierParams.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_VLAN) &&
			(!multi_ssid.secondary_enabled) )
		{
			PRINT_CONFIG_ERR_MSG("\n ERROR: Can not set WMM with 802.1Q classification when Multiple SSID is diabled. \n Please enable Multiple SSID support !");
		}
		else
		{
            if( tiap_wcfg_set_qos_params(wd, &classifierParams, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set wme parameters.");
				return "";

			}
		}
		restart();
	}

	return "";
}


/****************************************************************************/
/*                      wlan_htmlRestoreDefaults(void)							*/
/****************************************************************************/
/* DESCRIPTION: Rstores the AP defaults from nvram							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlRestoreDefaults(char* save_button)
{
    int savingIndex = 0;
	BOOL DoSave = FALSE;
	
	if (strncmp(save_button, "Defaults",8))
        return "";
	if (strcmp(save_button, "Defaults") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+8);
    
	if (DoSave || (globalWebSavingIndex <= savingIndex))
	{
		if (tiap_wcfg_set_defaults(wd)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set defaults.");
			return (char*)"";
		}
		restart();
	}

	return "";
}


/****************************************************************************/
/*                     wlan_html GetUsageTimeMode(void)					*/
/****************************************************************************/
/* DESCRIPTION: Rstores the AP defaults from nvram							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlAdvancedGetUsageTimeMode(char *value)
{
#if 0
    UINT8 TotWone, usageTimeMode;
	int mode;
	
    wdrv_dbGetNumSupWone(&TotWone);
    if (TotWone == WONE_SINGLE_BAND) 
    {
		return " disabled=true";
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetUsageTimeMode(&usageTimeMode);
	switch (usageTimeMode)
	{
		case FALSE:
			if (mode == 2)
				return "checked";
			break;
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
	}
    return " ";
#endif

    int mode;
    __u8 wone_nr, time_mode;

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return (char*)NULL;
    }

    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) 
	    return " disabled=true";

    sscanf(value,"%d",&mode);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_TIME_MODE, &time_mode, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set time mode.");
	    return (char*)NULL;
    }

    switch (time_mode)
    {
	    case 0:/* short */
		    if (mode == 2)
			    return "checked";
		    break;
	    case 1: /* long */
		    if (mode == 1) 
			    return "checked";
		    break;
    }
    return " ";

}


/****************************************************************************/
/*                      wlan_htmlGetPreambleCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the preamble radio buttons			*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's preamble mode	*/
/*				"" if the value is different than the AP's preamble mode	*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetPreambleCheck(char *value)
{
#if 0
	int preambleMode;
	int mode;
	UINT8 temp;

	sscanf(value,"%d",&mode);

	wdrv_dbGetPreambleMode(0,&temp);

	preambleMode = (int)temp;

	if (preambleMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int mode;
	tiap_param_wone_t wone;

	sscanf(value,"%d",&mode);

	wone.index = 0;
	wone.type = wone_pream_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get preamble.");
		return (char*)NULL;
	}

	if (wone.elem.pream_mode == mode)
		return "checked";
	else 
		return " ";

}


/****************************************************************************/
/*                      wlan_htmlGetSlotTimeCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's Slot        */
/*				 time mode													*/
/*				"" if the value is different than the AP's  mode			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetSlotTimeCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   SlotTimeAlgorithmMode;

	sscanf(value,"%d",&mode);

	wdrv_dbGetSlotTimeMode(0,&temp);
	SlotTimeAlgorithmMode = (int) temp;

	if (SlotTimeAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif
	int   mode;
	tiap_param_wone_t wone;

	sscanf(value,"%d",&mode);

	wone.index = 0;
	wone.type = wone_slot_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get slot time.");
		return (char*)NULL;
	}

	if (wone.elem.slot_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlGetPbccCheck(void)								*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's pbcc mode	*/
/*				"" if the value is different than the AP's pbcc mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetPbccCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   PBCCAlgorithmMode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetPBCCAlgorithmMode(woneIndex,&temp);
	PBCCAlgorithmMode = (int) temp;

	if (PBCCAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default:
			return (char*)NULL;
	}

	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_pbcc;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return (char*)NULL;
	}

	if (wone.elem.pbcc_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlErpCheck(void)									*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's ERP mode    */
/*				"" if the value is different than the AP's ERP mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlErpCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   ERPAlgorithmMode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetERPProtectionMode(woneIndex,&temp);
	ERPAlgorithmMode = (int) temp;

	if (ERPAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}
	
	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:
			
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;
		default:
			return (char*)NULL;
	}

	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_erp_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get erp mode.");
		return (char*)NULL;
	}


	if (wone.elem.erp_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlDot11ModeCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the dot11mode buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's dot11 mode  */
/*				"" if the value is different than the AP's dot11  mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlDot11ModeCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   dot11Mode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
	sscanf(value,"%d",&mode);

	wdrv_dbGetDot11Mode(woneIndex,&temp);
	dot11Mode = (int) temp;

    if (dot11Mode == 4)
        return " disabled=true ";
	else if (dot11Mode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;
		default: return (char*)NULL;
	}
	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return (char*)NULL;
	}

	if (wone.elem.dot11_mode == 4)
		return " disabled=true ";
	else if (wone.elem.dot11_mode == mode)
		return "checked";
	else return " ";
}


char *wlan_html4xCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGet4xEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return (char*)NULL;
}

char *wlan_html4xConcatCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGetConcatEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return (char*)NULL;
}

char *wlan_html4xAckEmulCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGetAckEmulEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return (char*)NULL;
}


char* wlan_htmlGetAdditionalOperRateSet_G_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if ((dot11Mode != DOT11_MODE_MIXED) && (dot11Mode != DOT11_MODE_OFDM_ONLY))
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
	whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	wone.index = 0;
	wone.type = wone_ch;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get channel.");
		return (char*)NULL;
	}

    if (wone.elem.ch == 14)
        return "N disabled=true ";

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}
    
    switch (wone_nr) 
    {
        case TIAP_PARAM_WONE_SINGLE_BAND:

		wone.index = 0;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return (char*)NULL;
		}

            if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case TIAP_PARAM_WONE_DUAL_BAND:

		wone.index = 0;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return (char*)NULL;
		}

		if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
		wone.index = 1;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return (char*)NULL;
		}
                if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;

	default: 
	    return (char*)NULL;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wone.index = woneIndex;
    wone.type = wone_dot11_mode;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
	    return (char*)NULL;
    }
    
    if ((wone.elem.dot11_mode != TIAP_PARAM_D11MODE_MIXED) && 
	(wone.elem.dot11_mode != TIAP_PARAM_D11MODE_OFDM))
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_add;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return (char*)NULL;
	}

	if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;

}


char* wlan_htmlGetOFDMOperRateSet_G_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if ((dot11Mode != DOT11_MODE_MIXED) && (dot11Mode != DOT11_MODE_OFDM_ONLY))
        return "N disabled=true ";   /* not G only mode */
    
    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalOFDMRateSetLength(0,&curRatesLen);
	whal_nvram_getOperationalOFDMRateSet(0,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	wone.index = 0;
	wone.type = wone_ch;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get channel.");
		return (char*)NULL;
	}

    if (wone.elem.ch == 14)
        return "N disabled=true ";

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return "N disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return "N disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default: 
			return (char*)NULL;
	}
	/* We are in B/G band - skipping the B modes, remain only with G */
	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return (char*)NULL;
	}

	if ((wone.elem.dot11_mode != TIAP_PARAM_D11MODE_MIXED) && 
		(wone.elem.dot11_mode != TIAP_PARAM_D11MODE_OFDM))
		return "N disabled=true ";   /* not G only mode */

	tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_ofdm;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return (char*)NULL;
	}

	if( wone.elem.ofdm_extend.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_extend.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;
}


char* wlan_htmlGetAdditionalOperRateSet_A_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band != RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'B/G' */
            }
            break;
    }
    /* We are in A band - Verifying A mode */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if (dot11Mode != DOT11_MODE_A_ONLY)
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
	whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return (char*)NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}

			if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
				return "N disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return (char*)NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					woneIndex = 1;
				else
					return "N disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default: 
			return (char*)NULL;
	}
	/* We are in B/G band - skipping the B modes, remain only with G */
	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return (char*)NULL;
	}

	if (wone.elem.dot11_mode != TIAP_PARAM_D11MODE_A)
		return "N disabled=true ";   /* not G only mode */

	tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_add;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return (char*)NULL;
	}

	if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMaxBasicRate(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max basic rate drop down		*/ 
/*				text box.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's max basic rate	*/
/*				"" if the value is different								*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMaxBasicRate(char* option)
{
#if 0
	UINT8 maxBasic;
	int rate;
    UINT8 TotalWones, band, woneIndex;

	sscanf(option,"%d",&rate);

    if (rate == 0) 
    {
        wdrv_dbGetNumSupWone(&TotalWones);
        switch (TotalWones) 
        {
            case WONE_SINGLE_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    return " disabled=true ";
                woneIndex = 0;
                break;
        
            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band);
                    if (band == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        return " disabled=true ";   /* both Wone 0,1 are 'A' */
                }
                break;
        }
    }
	wdrv_dbGetMaxCCKbasicRate(woneIndex,&maxBasic);
	
	switch(maxBasic)
	{
		case 0x82:
			maxBasic = 1;
			break;

		case 0x84:
			maxBasic = 2;
			break;

		case 0x8B:
			maxBasic = 3;
			break;

		case 0x96:
			maxBasic = 4;
			break;
	}

	if (maxBasic == rate)
		return "selected ";
	else 
		return " ";
#endif

	UINT8 maxBasic;
	int rate, i;
	UINT8 woneIndex=0;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return (char*)NULL;
		}
		
		switch (wone_nr) 
		{
			case TIAP_PARAM_WONE_SINGLE_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					return " disabled=true ";
				woneIndex = 0;
				break;

			case TIAP_PARAM_WONE_DUAL_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return (char*)NULL;
					}
					
					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						return " disabled=true ";   /* both Wone 0,1 are 'A' */
				}
				break;
			default:
				return (char*)NULL;
		}
	}

	maxBasic = 0;

	wone.index = woneIndex;
	wone.type = wone_oper_rate;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return (char*)NULL;
	}

	for(i=4; i>=0; i--)
	{
		if(wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
		{
			maxBasic = i;
			break;
		}
	}

	maxBasic++;

	if (maxBasic == rate)
		return "selected ";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlGetMaxActiveRate(void)						*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max active rate drop down	*/ 
/*				text box.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's max active rate	*/
/*				"" if the value is different								*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMaxActiveRate(char* option)
{
#if 0
	UINT8 maxActive;
	int rate;
	UINT8 TotalWones, band, woneIndex;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		wdrv_dbGetNumSupWone(&TotalWones);
		switch (TotalWones) 
		{
			case WONE_SINGLE_BAND:
				wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    return " disabled=true ";
                woneIndex = 0;
                break;
        
            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band);
                    if (band == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        return " disabled=true ";   /* both Wone 0,1 are 'A' */
                }
                break;
        }
    }
	wdrv_dbGetMaxCCKactiveRate(woneIndex,&maxActive);
	
	switch(maxActive)
	{
		case 0x02:
		case 0x82:
			maxActive = 1;
			break;

		case 0x04:
		case 0x84:
			maxActive = 2;
			break;

		case 0x0B:
		case 0x8B:
			maxActive = 3;
			break;

		case 0x16:
		case 0x96:
			maxActive = 4;
			break;

		case 0x2C:
			maxActive = 5;
			break;
	}

	if (maxActive == rate)
		return "selected ";
	else 
		return " ";
#endif

	UINT8 maxActive;
	int rate, i;
	UINT8 woneIndex=0;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return (char*)NULL;
		}
		
		switch (wone_nr) 
		{
			case TIAP_PARAM_WONE_SINGLE_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					return " disabled=true ";
				woneIndex = 0;
				break;

			case TIAP_PARAM_WONE_DUAL_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return (char*)NULL;
					}
					
					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						return " disabled=true ";   /* both Wone 0,1 are 'A' */
				}
				break;
			default:
				return (char*)NULL;
		}
	}

	maxActive = 0;

	wone.index = woneIndex;
	wone.type = wone_oper_rate;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return (char*)NULL;
	}

	for(i = 4; i>=0; i--)
	{
		if((wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED) ||
		   (wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED))
		{
			maxActive = i;
			break;
		}
	}

	maxActive++;

	if (maxActive == rate)
		return "selected ";
	else 
		return " ";
}



/****************************************************************************/
/*                      wlan_htmlSoftwareDownload(void)                         */
/****************************************************************************/
/* DESCRIPTION: Performs software download and reboot                       */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE                                                        */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:                                                                 */
/*                                                                          */
/****************************************************************************/

#define MTD0 "/dev/mtdblock/0"
#define MTD1 "/dev/mtdblock/1"

char* wlan_htmlSoftwareDownload(char* value, char *fs_img, char *kernel_img, 
                                char *tftpip1, char *tftpip2, char *tftpip3, char *tftpip4)
{
    char tftp_server_ipaddr[20];
    int savingIndex = 0;
    char  localFile[200],oldFile[200];



    if (strncmp(value, "Downloading", strlen("Downloading")))
        return "";

    savingIndex = atoi(value+strlen("Downloading"));


    #ifndef VXWORKS

    if (globalWebSavingIndex <= savingIndex)
        {
        struct in_addr in;
        pid_t pid;
        int status;
        char error_str[100];

        if( strlen(tftpip1) > 3 || strlen(tftpip2) > 3 || strlen(tftpip3) > 3 || strlen(tftpip4) > 3 ) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Invalid tftp server IP address: %s.%s.%s.%s"
                    "</FONT></br><br></br>",
                    tftpip1, tftpip2, tftpip3, tftpip4);
            return tempbuf;
        }

        sprintf(tftp_server_ipaddr, "%s.%s.%s.%s", tftpip1, tftpip2, tftpip3, tftpip4);

        printf("Entering download section\n");




        if (!inet_aton(tftp_server_ipaddr, &in)) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Invalid tftp server IP address: %s"
                    "</FONT></br><br></br>", tftp_server_ipaddr);
            return tempbuf;
        }


        if( (pid = fork()) == -1 ) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Fork : %s"
                    "</FONT></br><br></br>", strerror(errno));
            return tempbuf;
        }
        else if( !pid ) 
            { /*child*/
            execlp("tftp_updated_images.sh", "tftp_updated_images.sh", 
                   fs_img, MTD0, kernel_img, MTD1, tftp_server_ipaddr, NULL);
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Running tftp_updated_images.sh : %s"
                    "</FONT></br><br></br>", strerror(errno));
            return tempbuf;
        }

        if( waitpid(pid, &status, 0) == -1 ) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "waitpid : %s"
                    "</FONT></br><br></br>", strerror(errno));
            return tempbuf;
        }
        else if( WIFEXITED(status) )
            {
            switch(WEXITSTATUS(status))
                {
            case 1:
                strcpy(error_str, "internal error.");
                break;
            case 2:
                strcpy(error_str, "file system image file not found."); 
                break;
            case 3:
                strcpy(error_str, "kernel image file not found.");
                break;
            case 4:
                strcpy(error_str, "failed to updated file system partition,"
                       "warning system is unstable.");
                break;
            case 5:
                strcpy(error_str, "failed to updated kernel partition,"
                       "warning system is unstable.");
                break;
            default:
                strcpy(error_str, "unknown error occured.");

            }
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Failed to download images, %s"
                    "</FONT></br><br></br>",
                    error_str);
            return tempbuf;
        }
        else if( WIFSIGNALED(status) ) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Failed to download images, signal number: %d"
                    "</FONT></br><br></br>",
                    WTERMSIG(status));
            return tempbuf;
        }

    }
    #else
    if (globalWebSavingIndex <= savingIndex)
        {
        int tftpRet;

        if( strlen(tftpip1) > 3 || strlen(tftpip2) > 3 || strlen(tftpip3) > 3 || strlen(tftpip4) > 3 ) 
            {
            sprintf(tempbuf, "<br><FONT color=#800080>"
                    "Invalid tftp server IP address: %s.%s.%s.%s"
                    "</FONT></br><br></br>",
                    tftpip1, tftpip2, tftpip3, tftpip4);
            return tempbuf;
        }

        sprintf(tftp_server_ipaddr, "%s.%s.%s.%s", tftpip1, tftpip2, tftpip3, tftpip4);

        printf("Entering download section\n");

        sprintf(localFile,"/bin/%s",fs_img);
        if ( (tftpRet = tiap_tftp_read_file_to_file(fs_img, tftp_server_ipaddr,localFile)) == OK){
            sprintf(localFile,"m:f:\"%s\"",fs_img);
            if( sys_setenv( "BOOTCFG", localFile))
                {
                printf("%s: Error setting env var %s value %s\n", __FUNCTION__, 
                       "BOOTCFG", localFile);
                sys_defragenv();
            }else{
                reboot(BOOT_NORMAL);
            }
        }else if (tftpRet == NO_PLACE_ERROR){
            char oldDelete[200];
            strcpy (oldFile,sys_getenv("BOOTCFG"));
            strcpy (oldDelete,"/bin/");
            strcpy ((&(oldDelete[5])),(char *) (strchr(oldFile,'"') + 1));
            oldDelete[((int) strlen(oldDelete)) -1] = NULL;//delete the final "
            printf("%s: old file string is %s ",__FUNCTION__,oldDelete);
            if (strcmp(oldDelete,localFile) == 0) {// The new file is the same as the old file
                                                 // TFTP failed
                printf("%s: Error in TFTP new driver file %s ",__FUNCTION__,fs_img);
            }else{// old driver file is different, delete it and try TFTP again
                ffs_remove(oldDelete);
                printf("%s: Removing old driver file %s\n", __FUNCTION__, 
                               oldDelete);
                DefragFFS(FFS_PRIMARY_PART);

                if ( tiap_tftp_read_file_to_file(fs_img, tftp_server_ipaddr,localFile) == OK){
                    sprintf(localFile,"m:f:\"%s\"",fs_img);
                    if( sys_setenv( "BOOTCFG", localFile))
                        {
                        printf("%s: Error setting env var %s value %s\n", __FUNCTION__, 
                               "BOOTCFG", localFile);
                        sys_defragenv();
                    }else
                        reboot(BOOT_NORMAL);
                }else{// No more tries, TFTP failed
                    printf("%s: Error TFTP driver file %s \n", __FUNCTION__, 
                               fs_img);


                }


            }

        }


    }

    #endif
 	return (char*)"";
}
 
/****************************************************************************/
/*                      wlan_htmlGetDownloadSector(void)                        */
/****************************************************************************/
/* DESCRIPTION: returns the current sector the software running from        */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE.                                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     current sector the software running from                    */
/*                                                                          */
/****************************************************************************/
#ifndef VXWORKS
#define PSP_KERNEL_IMG "ram_zimage.bin"
#else
#define PSP_KERNEL_IMG ""
#endif

char *wlan_htmlGetKernelImage(void)
{
#if 0
    UINT32 sector;

    tempbuf[0] = 0;
    
    cm_serviceGetRunningSector(&sector);
    sprintf(tempbuf,"%d",sector);

    return tempbuf;
#endif

	return PSP_KERNEL_IMG;
}


/****************************************************************************/
/*                      wlan_htmlGetRunningFilename(void)                       */
/****************************************************************************/
/* DESCRIPTION: returns the current running image                           */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE.                                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     current running image                                       */
/*                                                                          */
/****************************************************************************/
#ifndef VXWORKS

#define PSP_FS_IMG "psp_fs.img"
#else
#define PSP_FS_IMG "vxWorks.gz"
#endif
char *wlan_htmlGetFSImage(void)
{
#if 0
    tempbuf[0] = 0;
    
    cm_serviceGetRunningImage(tempbuf);

    return tempbuf;
#endif

	return PSP_FS_IMG;
}



/****************************************************************************/
/*                      wlan_htmlGetTftpIpAddr(void)                            */
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's TFTP SERVER ip address          */ 
/*                                                                          */
/*																		    */
/* INPUT:       Component ID (1-4).                                         */
/* OUTPUT:      NONE.                                                       */
/*																			*/
/* RETURNS:     Byte[0] of the AP's TFTOP Server ip address                 */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetTftpIpAddr(char *ip_component)
{
#if 0
	unsigned long IPAddr, component;

    cm_hwGetHostIP(&IPAddr);
    sscanf(ipComponent,"%d",&component);

    switch (component) 
    {
        case 1:
            sprintf(tempbuf,"%d",IPAddr>>24);
            break;
        case 2:
            sprintf(tempbuf,"%d",(IPAddr>>16)&0xFF);
            break;
        case 3:
            sprintf(tempbuf,"%d",(IPAddr>>8)&0xFF);
            break;
        case 4:
            sprintf(tempbuf,"%d",(IPAddr & 0xFF));
            break;
    }
	return tempbuf;
#endif

	__u32 ipaddr, component;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name[0] != '\0' ? wd->br_name : wd->dev_name,
				SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get IP address.");
		return (char*)NULL;
	}
	ipaddr = ntohl(ipaddr);

	sscanf(ip_component,"%u", &component);
	switch (component) 
	{
		case 1:
			sprintf(tempbuf,"%u", ipaddr>>24);
			break;
		case 2:
			sprintf(tempbuf,"%u", (ipaddr>>16)&0xFF);
			break;
		case 3:
			sprintf(tempbuf,"%u", (ipaddr>>8)&0xFF);
			break;
		case 4:
			sprintf(tempbuf,"%u", (ipaddr & 0xFF));
			break;
		default:
			return (char*)NULL;
	}
	return tempbuf;
}




int printRatesInMbpsUnit(char *tmp, UINT8 rate)
{
#if 0
    UINT32 rate1, rate2, rate3;
    BOOL basic;
    int  len=0;

    if (rate)
    {
        rate1 = rate;
        basic = rate1 & 0x80;
        rate1 &= 0x7f;
        rate1 *= 500;
        rate2 = rate1/1000;
        rate3 = rate1%1000;

        if (rate3)
        {
            len = sprintf(tmp,"%d.%dMbps", rate2, 5);
            tmp += len;
        }
        else
        {
            len = sprintf(tmp,"%dMbps", rate2);
            tmp += len;
        }

        if (basic)
            len += sprintf(tmp,"B");

        return len;
    }
#endif
	return 0;
}


/****************************************************************************/
/*                      wlan_htmlGetAllRates(void)                              */
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max basic rate drop down     */ 
/*              text box.                                                   */
/*                                                                          */
/*                                                                          */
/* INPUT:       Drop down value                                             */
/* OUTPUT:                                                                  */
/*                                                                          */
/* RETURNS: "selected" if the value is identical to the AP's max basic rate */
/*          "" if the value is different                                    */
/*                                                                          */
/****************************************************************************/
char *wlan_htmlGetAllRates(void)
{
/*
	UINT8 maxBasic;
	int rate;

	sscanf(option,"%d",&rate);

	wdrv_dbGetDefaultActiveRate(&maxBasic);
	switch(maxBasic)
	{
		case 0x82:
			maxBasic = 1;
			break;

		case 0x84:
			maxBasic = 2;
			break;

		case 0x8B:
			maxBasic = 3;
			break;

		case 0x96:
			maxBasic = 4;
			break;
	}

	if (maxBasic == rate)
		return "selected";
	else 
		return " ";
*/

#if 0
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;
	char *buf = tempbuf;
	int i;
	int len;

	tempbuf[0] = 0;

	wdrv_dbGetStationConfigOperationalAllRateSet(0,allRates, &allRatesLen);

	len = sprintf(buf," ");
	buf += len;
	for (i = 0; i < allRatesLen; i++)
	{		 
		len = printRatesInMbpsUnit(buf, allRates[i]);
		if(len)
		{
			buf += len;
			if(i<allRatesLen-1)
			{			
				len = sprintf(buf,"__");
				buf += len;
			}
		}
	}

	return tempbuf;
#endif
	return (char*)NULL;
}


/********************************
*		STATISTICS PAGE				*	
*********************************/

/* LAN SEND COUNTERS */


static tiap_param_net_stats stats;
/****************************************************************************/
/*                      wlan_htmlGetLanSendUnicast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of unicast frames sent to the lan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of unicast frames sent to the lan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanSendUnicast(void)
{
		
	if( tiap_wcfg_get_priv_net_stats(wd, &stats) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get net statistics.");
		return (char*)NULL;
	}

	sprintf(tempbuf, "%d", stats.bss.tx_frames);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetLanSendMulticast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast frames sent to the lan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast frames sent to the lan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanSendMulticast(void)
{

	sprintf(tempbuf, "%d", stats.bss.tx_multicast_frames);

	return tempbuf;
}

/* LAN RECEIVE COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetLanRecvUnicast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of unicast frames recieved from the lan	*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of unicast frames recieved from the lan				*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanRecvUnicast(void)
{
	sprintf(tempbuf, "%u", stats.bss.rx_frames);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetLanRecvMulticast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast frames recieved from the lan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast frames recieved from the lan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanRecvMulticast(void)
{
	sprintf(tempbuf, "%u", stats.bss.rx_multicast_frames);

	return tempbuf;

}


/* WIRELESS SEND COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendFragments(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus sent to the wlan							*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendFragments(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_mpdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendMsdus(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus sent to the wlan							*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendMsdus(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendMulticasts(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast msdus sent to the wlan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast msdus sent to the wlan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendMulticasts(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_multicast_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendFailed(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus failed to sent to the wlan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus failed to sent to the wlan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendFailed(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_failed_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendRetry(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus that were retried to sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus that were retried to sent to the wlan		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendRetry(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_retry_msdus);

	return tempbuf;
}


/* WIRELESS RECEIVE COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvFragments(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus received from the wlan			*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus received from the wlan						*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvFragments(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_mpdus);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvMsdus(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus received from the wlan			*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus received from the wlan						*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvMsdus(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvMulticasts(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast msdus received from the wlan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast msdus received from the wlan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvMulticasts(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_multicast_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvFcsError(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus received from the wlan with FCS error				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus received from the wlan with FCS error		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvFcsError(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_fcs_error_mpdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMicFailureMSDUs(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of MIC failure MSDUs received from the wlan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of MIC failure MSDUs received from the wlan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMicFailureMSDUs(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_mic_failure_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetDecryptErrorMPDUs(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of decrypt error MPDUs					*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of decrypt error MPDUs								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDecryptErrorMPDUs(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_decrypt_error_mpdus);

	return tempbuf;
}

/********************************
*       PRIVACY PAGE            *	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetWepCheck(void)								*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the wep mode radio buttons			*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's WEP & authentication modes	*/
/*				"" if the value is different than the AP's WEP & authentication modes mode			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetWepCheck(char* value)
{
#if 0
	int inputWep, i, mode;
	BOOL privacyOption, openSystemEnable = FALSE, sharedKeyEnable = FALSE;
	UINT16 authAlgo;
	BOOL enable;

	sscanf(value,"%d",&inputWep);

	wdrv_dbGetStationConfigPrivacyOptionImplemented(&privacyOption);

	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		wdrv_dbGetAuthAlgo(i, &authAlgo);
		wdrv_dbGetAuthAlgoEnable(i, &enable);

		if ((authAlgo == MIB_OPEN_SYSTEM) && (enable == TRUE))
			openSystemEnable = TRUE;
		else if ((authAlgo == MIB_SHARED_KEY) && (enable == TRUE))
			sharedKeyEnable = TRUE;
	}
	
	if (!privacyOption)
		mode = 1;
	else
	{
		if ((openSystemEnable == TRUE) && (sharedKeyEnable == FALSE))
			mode = 2;

		else if ((openSystemEnable == FALSE) && (sharedKeyEnable == TRUE))
			mode = 3;

		else if ((openSystemEnable == TRUE) && (sharedKeyEnable == TRUE))
			mode = 4;
	}

	if (inputWep == mode)
		return "checked";
	else 
		return " ";
#endif
	return (char*)NULL;
}



BOOL wlan_htmlTestWepKey(char *key, int expectedlength)
{
 /*   int i;

	if (strlen(key) != expectedlength)
		return FALSE;

	for (i=0; i<strlen(key); i++) 
	{
		if (!isxdigit(key[i])) 
			return FALSE;
	}*/
    	return TRUE;
}


/****************************************************************************/
/*                      wlan_htmlTestPagePrivacy(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the privacy page.					*/ 
/*																		    */
/*																		    */
/* INPUT:       Test Privacy page parameters				                */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:	TRUE / FALSE													*/
/*																			*/
/****************************************************************************/

BOOL TestPrivacy(const char *profile, char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa2_pre_auth, char *wpa_auth_radio, char *psk_value,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	unsigned long ipAddr, len=0;
    __u8 wpa_auth_radio_val=0;
	__u32 security_select_val, auth_type, val;
	tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_param_security_type_t security_type;
	__u32 cipher, wep_cipher;

	/*              Working Mode  */
	security_select_val = atoi(security_select);
	wep_cipher = TIAP_WPA_CIPHER_WEP_40;
	security_type = 0;

	/* Before setting anything, checking that everything is valid (no way to undo!) */
	switch( security_select_val )
	{
		case 0: /* NONE */
			return TRUE;

		case 1: /* WPA */
		case 2: /* WPA2 */
		case 3: /* WPA any */

			switch(security_select_val)
			{
				case 1:
					security_type |= TIAP_PARAM_SECURITY_WPA;
					break;
				case 2:
					security_type |= TIAP_PARAM_SECURITY_WPA2;
					break;
				case 3:
					security_type |= TIAP_PARAM_SECURITY_WPA;
					security_type |= TIAP_PARAM_SECURITY_WPA2;
					break;
                default:
                    PRINT_CONFIG_ERR_MSG("Invalid security type.\n");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
					return FALSE;
			}

			/* PSK */
			/* Mode */
			wpa_auth_radio_val = atoi(wpa_auth_radio);
			switch( wpa_auth_radio_val )
			{
				case 1: /* TLS */
					break;

				case 2: /* HEX */
					PRINT_NOT_SUPPORTED_MSG("Psk hex mode");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
					return FALSE;

				case 3: /* PSK */
					len = strlen(psk_value);
					if ((len > TIAP_WPA_MAX_PSK_PWD_LEN) ||
						(len < TIAP_WPA_MIN_PSK_PWD_LEN)) 
					{
						PRINT_NOT_SUPPORTED_MSG("Valid PSK password length is from 8 to 63 symbols inclusive.");
						create_js_msg_box(tempbuf,                  
										  "Valid PSK password length is from 8 to 63 symbols inclusive.");
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
						return FALSE;
					}
					break;

				default:
					PRINT_CONFIG_ERR_MSG("Invalid authentication type 1.\n");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
					return FALSE;
			}
			break;

		case 4: /* WEP */
			security_type |= TIAP_PARAM_SECURITY_WEP;
	
			/* Auth Type */
			sscanf(wep_auth_select, "%u", &auth_type);
			tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
			tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
			switch (auth_type)
			{
				case 0:
				case 1:
				case 2:
					break;
				default:
					fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
					return FALSE;
			}
	
			/* WEP Cipher */
			sscanf(wep_cipher_select, "%u", &val);
			switch( val )
			{
				case 0:
					if ((wlan_htmlTestWepKey(wep_key_val0,10) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val1,10) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val2,10) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val3,10) == FALSE))
					{
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_WRONG_KEYS_LENGTH;
						return FALSE;
					}
					break;
				case 1:
					if ((wlan_htmlTestWepKey(wep_key_val0,26) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val1,26) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val2,26) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val3,26) == FALSE))
					{
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_WRONG_KEYS_LENGTH;
						return FALSE;
					}
					break;
				case 2:
					if ((wlan_htmlTestWepKey(wep_key_val0,58) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val1,58) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val2,58) == FALSE)
						|| (wlan_htmlTestWepKey(wep_key_val3,58) == FALSE))
					{
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_WRONG_KEYS_LENGTH;
						return FALSE;
					}
					break;
	
				default:
					fprintf(stderr, "tiweb: Invalid web cipher = %u", val);
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
					return FALSE;
			}
			break;
		
		case 5:	/* Dynamic WEP */
			security_type |= TIAP_PARAM_SECURITY_802_1X;
			break;
	
		default:
			mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
			return FALSE; /* changed from "" */
	}

	/* default cipher */
	if(security_type & TIAP_PARAM_SECURITY_WPA)
		cipher = TIAP_WPA_CIPHER_TKIP;
	else if(security_type & TIAP_PARAM_SECURITY_WPA2)
		cipher = TIAP_WPA_CIPHER_CCMP;
	else if( (security_type & TIAP_PARAM_SECURITY_802_1X) ||
		(security_type & TIAP_PARAM_SECURITY_WEP) )
		cipher = wep_cipher;
	else 
	{
		fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
		mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
		return FALSE;
	}

	/*              RADIUS          */
	if( !(security_type & TIAP_PARAM_SECURITY_WEP) && (wpa_auth_radio_val == 1) )
	{
		int fmt_err = 0;

		/* test ip address format*/
		if (sscanf(radius_ipaddr_octet0,"%ld",&ipAddr)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet1,"%ld",&ipAddr)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet2,"%ld",&ipAddr)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet3,"%ld",&ipAddr)<= 0) fmt_err = 1;

		if (fmt_err)
		{
			fprintf(stderr, "tiweb: Wrong radius server IP address format.\n");
			mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
			return FALSE;
		}

		/* test secret length*/
		len = strlen(radius_secret);
		if (len > TIAP_WPA_MAX_RADIUS_SECRET_LEN)
		{
			fprintf(stderr, "tiweb: Wrong radius secret length.\n");
			mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
			return FALSE;
		}
	}
    return TRUE;
}


/****************************************************************************/
/*                      wlan_htmlSavePagePrivacy(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the privacy page.					*/ 
/*																		    */
/*																		    */
/* INPUT:       Privacy page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char *SavePrivacy(const char *profile, char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa2_pre_auth, char *wpa_auth_radio, char *psk_value,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	unsigned long ipAddr1, ipAddr2, ipAddr3, ipAddr4, len=0,
	keyStrLen, tempKeyByte;

	char keyByte[(TIAP_WPA_WEP_KEY_LEN_MAX * 2 * 2) + 2];
	char inputStr[TIAP_WPA_WEP_KEY_LEN_MAX*2];

	int j, wpa2_pre_auth_val;

	__u8 wpa_auth_radio_val=0;
	__u32 security_select_val, auth_type, wep_key_selected, val;
	__u16 radius_port_val=0;
	__u32 gtk_val, radius_ipaddress;
	tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_wpa_wep_key_t wep_key, wep_key1, wep_key2, wep_key3;
	tiap_param_security_type_t security_type;
	__u32 cipher, wep_cipher;
	int nprofile = profile_map_tbl[atoi(profile)];

	validate_profile(profile);

	/*              Working Mode  */
	security_select_val = atoi(security_select);
	wep_cipher = TIAP_WPA_CIPHER_WEP_40;
	security_type = 0;

	/* Before setting anything, checking that everything is valid (no way to undo!) */
	switch( security_select_val )
	{
		case 0: /* NONE */
			security_type = TIAP_PARAM_SECURITY_WEP;
			if(tiap_wcfg_set_security_type(wd, security_type, nprofile, !UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to set security type.");
				return (char*)"";
			}
			
			if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_NON, nprofile, !UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to set default cipher.");
				return (char*)"";
			}
			tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
			tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
			tbl[0].enable = TRUE;
			tbl[1].enable = FALSE;
			/* Auth Type - Open */
			if (tiap_wcfg_set_auth_type(wd, tbl, !UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to set authentication algorithm table");
				return (char*)"";
			}
			return "OK";

		case 1: /* WPA */
		case 2: /* WPA2 */
		case 3: /* WPA any */

			switch(security_select_val)
			{
				case 1:
					security_type |= TIAP_PARAM_SECURITY_WPA;
					break;
				case 2:
					security_type |= TIAP_PARAM_SECURITY_WPA2;
					break;
				case 3:
					security_type |= TIAP_PARAM_SECURITY_WPA;
					security_type |= TIAP_PARAM_SECURITY_WPA2;
					break;
			}

			/* PSK */
			/* Mode */
			/* sscanf(wpa_auth_radio, "%u", &wpa_auth_radio_val);*/
			wpa_auth_radio_val = atoi(wpa_auth_radio);
			switch( wpa_auth_radio_val )
			{
				case 1: /* TLS */
					if (tiap_wcfg_set_psk_passphrase_mode(wd,
							TIAP_WPA_PSK_PARAM_NONE, nprofile, !UPDATE_STORAGE))
					{
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
						return (char*)"";
					}
					break;

				case 2: /* HEX */
					PRINT_NOT_SUPPORTED_MSG("Psk hex mode"); /* was validated in TestPrivacy */
					return (char*)"";

				case 3: /* PSK */
					len = strlen(psk_value);
					break;

				default:
					PRINT_CONFIG_ERR_MSG("Invalid authentication type 1.\n");
					return (char*)"";
			}
			break;

		case 4: /* WEP */
			security_type |= TIAP_PARAM_SECURITY_WEP;
	
			/* Auth Type */
			sscanf(wep_auth_select, "%u", &auth_type);
			tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
			tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
			switch (auth_type)
			{
				case 0:
					tbl[0].enable = TRUE;
					tbl[1].enable = FALSE;
					break;
	
				case 1:
					tbl[0].enable = FALSE;
					tbl[1].enable = TRUE;
					break;
	
				case 2:
					tbl[0].enable = TRUE;
					tbl[1].enable = TRUE;
					break;
	
				default:
					fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
					return (char*)"";
			}
	
			/* WEP Cipher */
			sscanf(wep_cipher_select, "%u", &val);
	
			switch ( val )
			{
				case 0:
					keyStrLen = 5;
					wep_cipher = TIAP_WPA_CIPHER_WEP_40;
					break;
	
				case 1:
					keyStrLen = 13;
					wep_cipher = TIAP_WPA_CIPHER_WEP_104;
					break;
	
				case 2:
					keyStrLen = 29;
					wep_cipher = TIAP_WPA_CIPHER_WEP_232;
					break;
	
				default:
					fprintf(stderr, "tiweb: Invalid web cipher = %u", val);
					return (char*)"";
			}
			/* key 0 */
			memset(keyByte, 0, sizeof(keyByte));
			memset(inputStr,'0',sizeof(inputStr));
			memcpy(inputStr, wep_key_val0, min(sizeof(inputStr), strlen(wep_key_val0)));
	
			for (j = 0; j < keyStrLen; j++)
			{
				//memcpy(&keyByte[j * 2], keyStr1Input + (j * 2), 2);
				memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
				sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
				wep_key.key[j] = (char)tempKeyByte;
			}
	
			wep_key.len = keyStrLen;
	
			/* key 1 */
			memset(keyByte, 0, sizeof(keyByte));
			memset(inputStr,'0',sizeof(inputStr));
			memcpy(inputStr, wep_key_val1,  min(sizeof(inputStr), strlen(wep_key_val1)));
	
			for (j = 0; j < keyStrLen; j++)
			{
				//memcpy(&keyByte[j * 2], keyStr2Input + (j * 2), 2);
				memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
				sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
				wep_key1.key[j] = (char)tempKeyByte;
			}
			wep_key1.len = keyStrLen;
	
			/* key 2 */
			memset(keyByte, 0, sizeof(keyByte));
			memset(inputStr,'0',sizeof(inputStr));
			memcpy(inputStr, wep_key_val2,  min(sizeof(inputStr), strlen(wep_key_val2)));
	
			for (j = 0; j < keyStrLen; j++)
			{
				//memcpy(&keyByte[j * 2], keyStr3Input + (j * 2), 2);
				memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
				sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
				wep_key2.key[j] = (char)tempKeyByte;
			}
			wep_key2.len = keyStrLen;
	
			/* key 3 */
			memset(keyByte, 0, sizeof(keyByte));
			memset(inputStr,'0',sizeof(inputStr));
			memcpy(inputStr, wep_key_val3,  min(sizeof(inputStr), strlen(wep_key_val3)));
	
			for (j = 0; j < keyStrLen; j++)
			{
				//memcpy(&keyByte[j * 2], keyStr4Input + (j * 2), 2);
				memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
				sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
				wep_key3.key[j] = (char)tempKeyByte;
			}
			wep_key3.len = keyStrLen;
			break;

		case 5:	/* Dynamic WEP */
			security_type = TIAP_PARAM_SECURITY_802_1X;

            /* WEP Cipher */
			sscanf(wep_cipher_select, "%u", &val);
	
			switch ( val )
			{
				case 0:
					wep_cipher = TIAP_WPA_CIPHER_WEP_40;
					break;
	
				case 1:
					wep_cipher = TIAP_WPA_CIPHER_WEP_104;
					break;
	
				case 2:
					wep_cipher = TIAP_WPA_CIPHER_WEP_232;
					break;
	
				default:
					fprintf(stderr, "tiweb: Invalid web cipher = %u", val);
					return (char*)"";
			}
            
            /* Setting PSK Mode to NONE - we want to work with TLS */
            if (tiap_wcfg_set_psk_passphrase_mode(wd,
                TIAP_WPA_PSK_PARAM_NONE, nprofile, !UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
                return (char*)"";
            }
			break;
	
		default:
			return (char*)NULL; /* changed from "" */
	}


	/* default cipher */
	if(security_type & TIAP_PARAM_SECURITY_WPA)
		cipher = TIAP_WPA_CIPHER_TKIP;
	else if(security_type & TIAP_PARAM_SECURITY_WPA2)
		cipher = TIAP_WPA_CIPHER_CCMP;
	else if( (security_type & TIAP_PARAM_SECURITY_802_1X) ||
		(security_type & TIAP_PARAM_SECURITY_WEP) )
		cipher = wep_cipher;
	else 
	{ 	/* already validated in TestPrivacy */
		fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
		return (char*)"";
	}

	/*              RADIUS          */
	if(((security_type & (TIAP_PARAM_SECURITY_WPA | TIAP_PARAM_SECURITY_WPA2)) && (wpa_auth_radio_val == 1))
		|| (security_type & TIAP_PARAM_SECURITY_802_1X))
	{
		unsigned char *ptr2ip = (unsigned char *)&radius_ipaddress;
		int fmt_err = 0;

		/* ip address */
		if (sscanf(radius_ipaddr_octet0,"%ld",&ipAddr1)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet1,"%ld",&ipAddr2)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet2,"%ld",&ipAddr3)<= 0) fmt_err = 1;
		if (sscanf(radius_ipaddr_octet3,"%ld",&ipAddr4)<= 0) fmt_err = 1;

		if (fmt_err)
		{
			fprintf(stderr, "tiweb: Wrong radius server IP address format.\n");
			return (char*)"";
		}
		else
		{
			ptr2ip[0] = ipAddr1;
			ptr2ip[1] = ipAddr2;
			ptr2ip[2] = ipAddr3;
			ptr2ip[3] = ipAddr4;
		}

	}

	/* All the values seem to be OK - let's set them */
	switch( security_select_val )
	{
		case 0: /* NONE */
			/* we really shouldn't be here! */
			return (char*)NULL;

		case 1: /* WPA */
		case 2: /* WPA2 */
		case 3: /* WPA any */

			switch( wpa_auth_radio_val )
			{
				case 1: /* TLS */
					if (tiap_wcfg_set_psk_passphrase_mode(wd,							TIAP_WPA_PSK_PARAM_NONE, nprofile, !UPDATE_STORAGE))
					{
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
						return (char*)"";
					}
					break;

				case 2: /* HEX */
					PRINT_NOT_SUPPORTED_MSG("Psk hex mode");
					break;

				case 3: /* PSK */
					if (tiap_wcfg_set_psk_passwd(wd, psk_value, len, nprofile, !UPDATE_STORAGE)) 
					{
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase");
						return (char*)"";
					}
					if (tiap_wcfg_set_psk_passphrase_mode(wd, TIAP_WPA_PSK_PARAM_ASCII, 
                                                          nprofile, !UPDATE_STORAGE))
					{
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
						return (char*)"";
					}

					break;

				default:
					PRINT_CONFIG_ERR_MSG("Invalid authentication type 2.\n");
					return (char*)"";
					
			}

			if (security_type & TIAP_PARAM_SECURITY_WPA2)
			{
				/* checkbox is only submitted being checked,
				   meaning we won't see its value if it's not checked! */
				if( !strcmp(wpa2_pre_auth, "NO_PARAM" ) )
					wpa2_pre_auth_val = 0;
				else 
					wpa2_pre_auth_val = atoi(wpa2_pre_auth);

				if (tiap_wcfg_set_pre_auth_flag(wd, wpa2_pre_auth_val, nprofile
                                                , !UPDATE_STORAGE)) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to set pre authentication flag");
					return (char*)"";
				}
			}
			break;

		case 4: /* WEP */
			security_type |= TIAP_PARAM_SECURITY_WEP;
	
			/* key ID */
			sscanf(wep_key_radio, "%u", &wep_key_selected);
			if (tiap_wcfg_set_wep_default_key_id(wd, (__u8) wep_key_selected, nprofile, !UPDATE_STORAGE))
				PRINT_CONFIG_ERR_MSG("Failed to set wep default key id.");
	
			/* key 0 */
			if(tiap_wcfg_set_def_wep_key(wd, 0, &wep_key, nprofile, !UPDATE_STORAGE))
				PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
	
			/* key 1 */
			if(tiap_wcfg_set_def_wep_key(wd, 1, &wep_key1, nprofile, !UPDATE_STORAGE))
				PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
	
			/* key 2 */
			if(tiap_wcfg_set_def_wep_key(wd, 2, &wep_key2, nprofile, !UPDATE_STORAGE))
				PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
	
			/* key 3 */
			if(tiap_wcfg_set_def_wep_key(wd, 3, &wep_key3, nprofile, !UPDATE_STORAGE))
				PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
	
			break;

		case 5:		/* Dynamic WEP */
			security_type |= TIAP_PARAM_SECURITY_802_1X;

            /* Setting PSK Mode to NONE - we want to work with TLS */
            if (tiap_wcfg_set_psk_passphrase_mode(wd,
                TIAP_WPA_PSK_PARAM_NONE, nprofile, !UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
                return (char*)"";
            }
            break;

		default:
			/* We shouldn't be here! */
			return (char*)NULL;
	}

	/* gtk */
	if (((0 < security_select_val) && (security_select_val < 4)) /*wpa*/
		|| (security_type == TIAP_PARAM_SECURITY_802_1X))
	{
        sscanf(gtk, "%u", &gtk_val);

        if ((gtk_val > 0) && (gtk_val < TIAP_WPA_MIN_GTK_PERIOD)) 
        {
            gtk_val = TIAP_WPA_MIN_GTK_PERIOD;
        }
		if (tiap_wcfg_set_gtk_interval(wd, gtk_val * 1000, nprofile, !UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set gtk interval.");
			return (char*)"";
		}
	}

	if(tiap_wcfg_set_security_type(wd, security_type, nprofile, !UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to set security type.");
		return (char*)"";
	}
    if(security_select_val == 4 /* WEP */)
    {
            /* Auth Type - Must be configured after setting security Type */
			if (tiap_wcfg_set_auth_type(wd, tbl, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set authentication algorithm table");
				return (char*)"";
			}
    }

	/* default cipher */
	if (tiap_wcfg_set_default_cipher(wd, cipher, nprofile, !UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to set default cipher.");
		return (char*)"";
	}

	/*              RADIUS          */
	if (((security_type & (TIAP_PARAM_SECURITY_WPA | TIAP_PARAM_SECURITY_WPA2)) && (wpa_auth_radio_val == 1))
		|| (security_type & TIAP_PARAM_SECURITY_802_1X))
	{ 
		if( tiap_wcfg_set_radius_ipaddr(wd, radius_ipaddress, nprofile, !UPDATE_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set radius IP address.");
			return (char*)"";
		}

		/* port */
		sscanf(radius_port,"%hu",&radius_port_val);
		if( tiap_wcfg_set_radius_port(wd, radius_port_val, nprofile, !UPDATE_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set radius port.");
			return (char*)"";
		}

		/* secret */
		len = strlen(radius_secret);
		if( tiap_wcfg_set_radius_secret(wd, radius_secret, len, nprofile, !UPDATE_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set radius secret.");
			return (char*)"";
		}
	}

	return "OK";
}


/****************************************************************************/
/*                      wlan_htmlGetKeyIdCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the default key id radio buttons	*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's default key id value	*/
/*				"" if the value is different than the AP's default key id value			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyIdCheck(char *value, const char *profile)
{
	int inputKeyId;
	__u8 key_id;

	validate_profile(profile);
	
	sscanf(value,"%d",&inputKeyId);

	if( profile_map_tbl[atoi(profile)] == -1 ) {
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		key_id = profile.sec.wep_def_key_id;
	}
	else tiap_wcfg_get_wep_default_key_id(wd, &key_id, profile_map_tbl[atoi(profile)]);

	if (inputKeyId == key_id)
		return "checked";
	else 
		return " ";
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect0(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the first default wep key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the first default wep key len	*/
/*				"" if the value is different than the first default wep key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect0(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(0, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return (char*)NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect1(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the second default wep key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the second wep default key len	*/
/*				"" if the value is different than the second wep default key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect1(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(1, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return (char*)NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect2(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the third wep default key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the third wep default key len	*/
/*				"" if the value is different than the third wep default key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect2(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(2, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return (char*)NULL;
}


char *wlan_htmlGetWpaPreAuthValue(const char *profile)
{
	__u8    pre_auth_val;
    BOOL    isPreAuthOn;
    int     dummy_profile_index;

	validate_profile(profile);

	if( profile_map_tbl[atoi(profile)] == -1 ) {
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		pre_auth_val = profile.pmk.pre_auth_flag;
	}
	else tiap_wcfg_get_pre_auth_flag(wd, &pre_auth_val, profile_map_tbl[atoi(profile)]);

    if(pre_auth_val)
    {
        return "checked";
    }

    tiap_wcfg_isPreAuthOnInMultiSsid(wd, &isPreAuthOn, &dummy_profile_index);
    if(isPreAuthOn)
    {
        return "disabled";
    }
	return "";
}


char *wlan_htmlGetVLAN(const char *profile)
{
	__u16 vlan;

	validate_profile(profile);

	if( !atoi(profile) ) 
	{
		if( tiap_wcfg_get_primary_vlan(wd, &vlan, FROM_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get primary vlan.\n");
			return (char*)NULL;
		}
	}
	else 
	{
		int ssid_index;
		tiap_param_multi_ssid_t multi_ssid;

		if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid");
			return (char*)NULL;
		}

		if( !multi_ssid.secondary_enabled ||
			 profile_map_tbl[atoi(profile)] == -1)
			return "\"\"";

		if( tiap_wcfg_get_actual_ssid_ind(&multi_ssid,
					profile_map_tbl[atoi(profile)], &ssid_index) )
			return (char*)NULL;

		vlan = multi_ssid.tbl[ssid_index].vlan;
	}

	//printf("%s:%d %d, vlan=%hu\n", __func__, __LINE__, vlan, atoi(profile));

	sprintf(tempbuf, "\"%hu\"", vlan);

	return tempbuf;
}

char* wlan_htmlIsPreAuthExists(const char *profile)
{
    BOOL    isPreAuthOn;
    int     dummy_profile_index;

    validate_profile(profile);
    
    tiap_wcfg_isPreAuthOnInMultiSsid(wd, &isPreAuthOn, &dummy_profile_index);
    if( (isPreAuthOn) && (atoi(profile) != dummy_profile_index) )
    {
        return "1";
    }
	return "0";
}

char *wlan_htmlGetWpaVersion(char *inputOption)
{
	int option, wpaVer1 = FALSE, wpaVer2 = FALSE;
	tiap_param_security_type_t security_type;

	if(tiap_wcfg_get_security_type(wd, &security_type, 0)) {
		PRINT_CONFIG_ERR_MSG("Failed to get wpa mode.\n");
		return (char*)NULL;
	}

	if (security_type & TIAP_PARAM_SECURITY_WPA) 
		wpaVer1 = TRUE;
	if (security_type & TIAP_PARAM_SECURITY_WPA2) 
		wpaVer2 = TRUE;

	option = atoi (inputOption);

	if ( (option == 1) && (wpaVer1 == TRUE) && (wpaVer2 == FALSE)) 
		return "selected";
	else if ( (option == 2) && (wpaVer2 == TRUE) && (wpaVer1 == FALSE)) 
		return "selected";
	else if ( (option == 3) && (wpaVer1 == TRUE) && (wpaVer2 == TRUE)) 
		return "selected";
	else
		return " ";
}
/********************************
*		MGMT PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetMgmtTableFirstRow(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table first row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableFirstRow(void)
{
	unsigned long len = 0;
	stationCurrentIndex = 0;

	tempbuf[0] = 0;
	
	len = sprintf(tempbuf,"<TABLE height=65 width=\"76%%\" border=1>\n");

	/* first row */
	len += sprintf(tempbuf + len,"<TBODY>\n");
	len += sprintf(tempbuf + len,"<TR>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"100%%\" bgColor=#006699 colSpan=10 height=12><b><font face=\"Arial\" color=\"#FFFFFF\">Associated STA Table</font></b></TD>\n");
	len += sprintf(tempbuf + len,"</TR>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMgmtTableSecondRow(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table second row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table secind row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableSecondRow(void)
{
	unsigned long len = 0;

	tempbuf[0] = 0;
	
	/* second row */
	len += sprintf(tempbuf + len,"<tr>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"25%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Mac Address</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>State</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>SSID #</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Active Rate</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Encryption</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>QoS</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Ban STA</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Security Status</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RX Packets</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>TX Packets</b></font></TD>\n");
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMgmtTableDynamicRow(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns a management table row								*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns a management table row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableDynamicRow(void)
{
	__u8 hwaddr[ETH_ALEN], formatAddr[40];	
	unsigned long len = 0;
	__u32 state;
	char ssid[33];

	tiap_param_statbl_t sta_tbl[TIAP_PARAM_MAX_STATIONS];

	tempbuf[0] = 0;

	if (stationCurrentIndex == TIAP_PARAM_MAX_STATIONS)
		return tempbuf;

	memset(sta_tbl,0,sizeof(sta_tbl));

	if( tiap_wcfg_get_sta_table(wd, sta_tbl) ) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to get sta table.");
		return (char*)NULL;
	}
	state = sta_tbl[stationCurrentIndex].state;

	if (state != TIAP_PARAM_SM_STATE_IDLE)
	{
		/* mac address */
		memcpy(hwaddr, sta_tbl[stationCurrentIndex].hwaddr, ETH_ALEN);
		sprintf(formatAddr,"%02X-%02X-%02X-%02X-%02X-%02X",hwaddr[0],hwaddr[1],hwaddr[2],
				hwaddr[3],hwaddr[4],hwaddr[5]);
		if (strcmp(formatAddr,"00-00-00-00-00-00") == 0)
			return tempbuf;
		/* now pour it to the buffer */
		len += sprintf(tempbuf + len,"<tr>\n");
		len += sprintf(tempbuf + len,
				"<td align=middle width=\"25%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
				hwaddr[0],hwaddr[1],hwaddr[2],hwaddr[3],hwaddr[4],hwaddr[5]);
		/* formatAddr is used below - Banned Station */

		/* state */
		/* now pour it to the buffer */
		switch (state)	
		{
			case TIAP_PARAM_SM_STATE_AUTH_KEY:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"middle\"><font color=\"#00ffff\"><b>Auth key</b></font></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_AUTHENTICATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00ccff\"><b>Authenticated</b></font></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_WAIT_FOR_ASS_RES:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Wait for assoc response</b></font></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_ASSOCIATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00cc00\"><b>Associated</b></font></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_AUTHORIZED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Authorized</b></font></td>\n");
				break;	

			default:
				break;
		}

		/* ssid */
		strcpy(ssid, sta_tbl[stationCurrentIndex].ssid);
		if (strlen(ssid) == 0) 
			strcpy(ssid,"-");
		len += sprintf(tempbuf + len,
				"<td width=\"15%%\" height=\"11\" align=\"center\">%s</td>\n",
				ssid);

		/* active rate */	
		if ((state == TIAP_PARAM_SM_STATE_ASSOCIATED) || (state == TIAP_PARAM_SM_STATE_AUTHORIZED))
		{
			switch (sta_tbl[stationCurrentIndex].active_rate)	
			{
				case DOT11_RATE_1:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">1 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_2:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">2 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_5_5:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">5.5 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_6:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">6 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_9:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">9 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_11:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">11 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_12:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">12 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_18:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">18 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_22:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">22 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_24:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">24 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_36:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">36 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_48:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">48 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_54:
					len += sprintf(tempbuf + len,
							"<td width=\"15%%\" height=\"11\""
							" align=\"center\">54 Mbit/sec</td>\n");
					break;

				default:
					break;
			}
		} /* STATE_ASSOCIATED */
		else
			len += sprintf(tempbuf + len,
					"<td width=\"15%%\" height=\"11\" align=\"center\">-----</td>\n");

		/* Encryption */
		switch (sta_tbl[stationCurrentIndex].cipher)
		{
		case TIAP_WPA_CIPHER_NON:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">-</td>\n");
			break;
		case TIAP_WPA_CIPHER_WEP_40:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"> WEP 40 </td>\n");
			break;
		case TIAP_WPA_CIPHER_WEP_104:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"> WEP 128 </td>\n");
			break;
		case TIAP_WPA_CIPHER_WEP_232:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"> WEP 256 </td>\n");
			break;
		case TIAP_WPA_CIPHER_CCMP:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"> AES </td>\n");
			break;
		case TIAP_WPA_CIPHER_TKIP:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"> TKIP </td>\n");
			break;
		default:
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">-</td>\n");
			break;
		}
		
		
		/* QoS */
		if (sta_tbl[stationCurrentIndex].wme) 
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Active</b></font></td>\n");
		else
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">Inactive</td>\n");

		/* Banned Station */

		len += sprintf(tempbuf + len,
				"<td width=\"15%%\" height=\"11\" align=\"center\">"
				"<INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" "
				"onclick='this.value=\"%s\"' type=submit value=\"    \""
				" name=BanSTA></TD>\n", formatAddr);

		//#warning "TODO: don't forget to implement wdrv_wpaGetStationSecAttack"
		/*undefine it when wdrv_wpaGetStationSecAttack will be implemented*/
#if 0

		if (wdrv_wpaGetStationSecAttack(stationCurrentIndex))
			len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#FF0000\"><b>Attacker</b></td>\n");
		else
			len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00CC99\"><b>Trusty</b></td>\n");
#else 
		len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00CC9
				9\"><b>Trusty</b></td>\n");
#endif

		/* RX Counter */
		len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">%d</td>\n",sta_tbl[stationCurrentIndex].rxCounter);

		/* TX Counter */
		len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">%d</td>\n",sta_tbl[stationCurrentIndex].txCounter);

		len += sprintf(tempbuf + len,"</tr>\n");
	}	/* !STATE_IDLE */

	stationCurrentIndex++;
	if (stationCurrentIndex == TIAP_PARAM_MAX_STATIONS)
		len += sprintf(tempbuf + len,"</table>\n");

	return tempbuf;


}


/****************************************************************************/
/*                    wlan_htmlGetMgmt_BW_TableFirstRow(void)                   */
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)       */ 
/*                                                                          */
/*                                                                          */
/* INPUT:                                                                   */
/* OUTPUT:                                                                  */
/*                                                                          */
/* RETURNS:	Returns the management Black & White table first row.           */
/*          Formats it to html format                                       */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetMgmt_BW_TableFirstRow(void)
{
	unsigned long len = 0;
	tableBW_CurrentIndex = 0;

	tempbuf[0] = 0;
	
	/* first row */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font face=\"Arial\" color=\"#FFFFFF\"><b>Delete</b></TD>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"80%%\" bgColor=#008080 height=11><font face=\"Arial\" color=\"#FFFFFF\"><b>Mac Address</b></TD>\n");
    len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
	return (char*)NULL;
}



/****************************************************************************/
/*                    wlan_htmlGetMgmt_BW_TableDynamicRow(void)				    */
/****************************************************************************/
/* DESCRIPTION: Returns a B&W management table row						    */ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:	Returns a management B&W table row.                             */
/*          Formats it to html format		                                */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMgmt_BW_TableDynamicRow(void)
{
#if 0
	char address[MAC_ADDR_LEN], zeroBuf[MAC_ADDR_LEN];	
	char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN], *ptr;
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    int index;
	UINT16 capabilities;
	UINT8 activeRate;
	unsigned long len = 0;
	UINT32 state;
    STATUS status;

	tempbuf[0] = 0;
	
	if (tableBW_CurrentIndex == MGMT_ACCESS_LIST_SIZE)
		return tempbuf;

    /* look for an entry in the B&W table */
    memset(zeroBuf,0,MAC_ADDR_LEN);

	status = mgmtAccessGetList(tmpTable, &length);
    if (status != OK) 
        return tempbuf;

    for (index=1, ptr=tmpTable; 
         index<=MGMT_ACCESS_LIST_SIZE; 
         index++, ptr+=MAC_ADDR_LEN) 
    {
        if (memcmp(ptr,zeroBuf,MAC_ADDR_LEN) == 0)
            continue;
        if (index > tableBW_CurrentIndex)
            break;
    }
    if (index == MGMT_ACCESS_LIST_SIZE+1) /* end of list */
        return tempbuf;
    tableBW_CurrentIndex = index;
    /* ptr is pointing to the current MacAddr */
    memcpy(address,ptr,MAC_ADDR_LEN);

/************************

    <TR>
    <TD align=middle width="20%" height=11>
    <P align=center><INPUT style="FONT-SIZE: 8pt; POSITION: relative" type=button value="    " name=DelSTA0></P></TD>
    <TD align=middle width="80%" height=11>00-08-21-31-18-40</TD>
    </TR>

*************************/

    /* now pour it to the buffer */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
    len += sprintf(tempbuf + len,"<P align=center><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSTA></P></TD>\n",tableBW_CurrentIndex);
    len += sprintf(tempbuf + len,"<td align=middle width=\"80%%\" height=11>%02X-%02X-%02X-%02X-%02X-%02X</td>\n",
                   address[0],address[1],address[2],address[3],address[4],address[5]);
    len += sprintf(tempbuf + len,"</tr>\n");

    /*len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%s\"' type=submit value=\"    \" name=BanSTA></TD>\n",formatAddr);*/
	return tempbuf;
#endif

	char address[ETH_ALEN], zeroBuf[ETH_ALEN];	
	__u8 (*ptr)[ETH_ALEN] = NULL;
	int i;
	unsigned long len = 0;
	tiap_param_access_list_t access_list;


	tempbuf[0] = 0;

	if (tableBW_CurrentIndex == TIAP_PARAM_ACCESS_LIST_SIZE)
		return tempbuf;

	/* look for an entry in the B&W table */
	memset(zeroBuf, 0, ETH_ALEN);

	access_list.cmd = TIAP_PARAM_ACCESS_LST;
	if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to get sta table.");
		return (char*)NULL;
	}


	for (i=1, ptr=(__u8 (*)[ETH_ALEN])access_list.list; i<=TIAP_PARAM_ACCESS_LIST_SIZE; i++) 
	{
		if (memcmp(ptr[i-1],zeroBuf,ETH_ALEN) == 0)
			continue;
		if (i > tableBW_CurrentIndex)
			break;
	}
	if (i == TIAP_PARAM_ACCESS_LIST_SIZE+1) /* end of list */
		return tempbuf;
	tableBW_CurrentIndex = i;
	/* ptr is pointing to the current MacAddr */
	memcpy(address, ptr[i-1], ETH_ALEN);

	/* now pour it to the buffer */
	len += sprintf(tempbuf + len,"<tr>\n");
	len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
	len += sprintf(tempbuf + len,"<P align=center><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSTA></P></TD>\n",tableBW_CurrentIndex);
	len += sprintf(tempbuf + len,"<td align=middle width=\"80%%\" height=11>%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX</td>\n",
			address[0],address[1],address[2],address[3],address[4],address[5]);
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;

}


/****************************************************************************/
/*                    wlan_htmlSavePageIpSettings(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the IP Settings page.    			*/ 
/*																		    */
/*																		    */
/* INPUT:       Setup page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/

char* wlan_htmlSavePageIpSettings(char* save_button, 
		char *ip0, char *ip1, char *ip2, char *ip3, 
		char *mask0, char *mask1, char *mask2, char *mask3,
		char *gw0, char *gw1, char *gw2, char *gw3)
{
	__u32 ipaddr, netmask, gateway;
	__u32 ipaddr0, ipaddr1, ipaddr2, ipaddr3,
	      netmask0, netmask1, netmask2, netmask3,
	      gateway0, gateway1, gateway2, gateway3;
	__u8 *p;
	int ip_err = 0, subnet_err = 0, gtw_err = 0;
    int savingIndex = 0;
	BOOL DoSave = FALSE;
	
	if (strncmp(save_button, "Saving",6))
        return "";
	if (strcmp(save_button, "Saving") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+6);
    
	if (DoSave || (globalWebSavingIndex <= savingIndex))
	{
		/* validate IP address, netmask and default gateway */
		 validate_ip_octet(ip0, ipaddr0, ip_err);
		 validate_ip_octet(ip1, ipaddr1, ip_err);
		 validate_ip_octet(ip2, ipaddr2, ip_err);
		 validate_ip_octet(ip3, ipaddr3, ip_err);
		if (ip_err)
			fprintf(stderr, "\ntiweb: Wrong IP address format.\n");

		validate_ip_octet(mask0, netmask0, subnet_err);
		validate_ip_octet(mask1, netmask1, subnet_err);
		validate_ip_octet(mask2, netmask2, subnet_err);
		validate_ip_octet(mask3, netmask3, subnet_err);
		if (subnet_err)
			fprintf(stderr, "\ntiweb: Wrong subnet mask format.\n");

		validate_ip_octet(gw0, gateway0, gtw_err);
		validate_ip_octet(gw1, gateway1, gtw_err);
		validate_ip_octet(gw2, gateway2, gtw_err);
		validate_ip_octet(gw3, gateway3, gtw_err);
		if (gtw_err)
			fprintf(stderr, "\ntiweb: Wrong default gateway format\n");

		if (ip_err || gtw_err || subnet_err) {
			fprintf(stderr, "\ntiweb: Failed to configure one or more of IP parameters.\n"
					"Previous settings will be restored after reboot.\n");
			return (char*)NULL;
		}


		/* delete previous default route */
		gtw_err = tiap_wcfg_get_default_gateway(&gateway);
		if( gtw_err ) {
			fprintf(stderr, "tiweb: Warning, failed to get default route, "
					"probably IP address of the default route device "
					"was changed recently.\n");
			fflush(stderr);
		}
		else if( tiap_wcfg_set_route(wd->br_name, 0, 0, gateway, SIOCDELRT) ) {
			fprintf(stderr, "tiweb: Warning, failed to delete default route.\n");
			fflush(stderr);
		}

		/* netmask mask */
		p = (__u8 *)&netmask;
		p[0] = (__u8)netmask0; p[1] = (__u8)netmask1;
		p[2] = (__u8)netmask2; p[3] = (__u8)netmask3;
#ifndef MWCODE
		subnet_err = tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFNETMASK, netmask);
		if(subnet_err) {
			fprintf(stderr, "\ntiweb: Failed to set netmask for %s.\n", wd->br_name);
			return (char*)NULL;
		}
#endif /* MWCODE */

		/* ip address */
		p = (__u8 *)&ipaddr;
		p[0] = (__u8)ipaddr0; p[1] = (__u8)ipaddr1;
		p[2] = (__u8)ipaddr2; p[3] = (__u8)ipaddr3;

#ifndef MWCODE
		ip_err = tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFADDR, ipaddr);
		if(ip_err) {
			fprintf(stderr, "\ntiweb: Failed to set IP address for %s.\n", wd->br_name);
			return (char*)NULL;
		}
#endif /* MWCODE */

		/* default gateway */
		p = (__u8 *)&gateway;
		p[0] = (__u8)gateway0; p[1] = (__u8)gateway1;
		p[2] = (__u8)gateway2; p[3] = (__u8)gateway3;

		gtw_err = tiap_wcfg_set_route(wd->br_name, 0, 0, gateway, SIOCADDRT);
		if( gtw_err ) {
			fprintf(stderr, "\ntiweb: Failed to set default route.\n");
			return (char*)NULL;
		}

		/* save them all */
		tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFADDR, ipaddr); /*store it in storage*/
		tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFNETMASK, netmask); /*store it in storage*/
		tiap_wcfg_set_storage_ipaddr(wd, SIOCADDRT, gateway); /*store it in storage*/
		restart(); 	/* restart the AP */
	}
	return NULL;

}



/********************************************************************************/
/*                      wlan_htmlGetApEnable(void)                                  */
/********************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the AP Enable buttons                  */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the AP's service state   */
/*				"" if the value is different than the AP's preamble mode        */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlGetApEnable(char *value)
{
	__u8 state;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
		return (char*)NULL;
	}

	return (state ? "checked" : "");
}


/****************************************************************************/
/*                 wlan_htmlGetEnableDisableButtonName(void)				    */
/****************************************************************************/
/* DESCRIPTION: Returns the name for the Enable/Disable button              */ 
/*              If the AP is disabled then returns Enable and vice versa.   */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetEnableDisableButtonName(char *value)
{
	__u8 state;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
		return (char*)NULL;
	}

	return (state == TRUE ? "Disable" : "Enable");
}


/****************************************************************************/
/*                      wlan_htmlUpdateEnableDisableAP(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns name for the button when pressing on it             */ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlUpdateEnableDisableAP(char *value)
{
     __u8 state;
    
    if( !value ) {
	    fprintf(stderr, "%s: invalid value: %s\n", __func__, "NULL");
	    return (char*)NULL;
    }

    if (!strcmp(value,"Disable"))
	    state = 0;
    else if (!strcmp(value,"Enable"))
	    state = 1;
    else 
		return "";

    if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, UPDATE_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
	    return (char*)NULL;
    }

    return "";
}


/****************************************************************************/
/*                  wlan_htmlGetUpdateButtonValueApEnable(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetUpdateButtonValueApEnable(void)
{
#if 0
    BOOL state;

	wdrv_dbGetWlanServiceState(&state);

    if (state == TRUE) 
        return "Disabling AP";
    else
        return "Enabling AP";
#endif

    __u8 state;

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
	    return (char*)NULL;
    }

    return (state == TRUE ? "Disabling AP" : "Enabling AP");
}


char* wlan_htmlUpdateStationPageMgmt(char *stationMAC)
{
#if 0
    int uintval, pDigitPos, pPos, hyphenCount, length;
    UINT8 enable, type, macval[6];
    char tmp_buf[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];

    
    if (stationMAC[0] != 'N')
    {
        whal_nvram_getAccessListEnable(&enable);
        if (enable == 0) 
        {
            return " "; /* BW table is disbled */
        }
        memset(macval,0,6);
        hyphenCount=0;

        uintval = 0;
        pDigitPos = 0;

        length = strlen(stationMAC);
        if (length > 17) 
            return FALSE;

        for (pPos = 0; pPos<length;pPos++) 
        {
            if (stationMAC[pPos] == '-') 
            {
                if (hyphenCount > 5) 
                    return " "; /* its the sixth '-' */
                if ((uintval == 0) && (pDigitPos == 0))
                    return " "; /* string begins with hyphen */
                macval[hyphenCount] = uintval;
                hyphenCount++;
                uintval = 0;
                pDigitPos = 0;
            } 
            else if (isxdigit(stationMAC[pPos])) 
            {
                pDigitPos++;
                if (pDigitPos > 2) 
                    return FALSE;
                uintval = uintval*16;
                if (isdigit(stationMAC[pPos]))
                    uintval += (stationMAC[pPos]-'0');
                else
                    uintval += hexChar2int(stationMAC[pPos]);
            } 
            else 
            {
                return " ";         /* bad value */
            }
        }
        
        if (hyphenCount < 5) 
            return " ";             /* have to have exactly 5 '-' */

        /* its the last octet */
        if ((uintval == 0) && (pDigitPos == 0))
           return " ";
        macval[hyphenCount] = uintval;

        whal_nvram_getAccessListType(&type);

        if (type == 1) /* table is black*/
            mgmtAccessSetMAC(macval);
        else /* remove the mac */
            mgmtAccessRemMACbyAddr(macval);
        
        /* Update table in NVRAM */
        length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
        if (mgmtAccessGetList(tmp_buf, &length) != OK) 
            return " ";
        else
    	    whal_nvram_setAccessList(tmp_buf, IMMEDIATELY);
    }
    return " ";
#endif











    int uintval, pDigitPos, pPos, hyphenCount, length;
    __u8 macval[6];
    tiap_param_access_list_t access_list;

    if (stationMAC[0] == 'N')
			return " ";
			
		access_list.cmd = TIAP_PARAM_ACCESS_LST;
		if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Reading configuration failed");
			return (char*)NULL;
		}
				
        memset(macval,0,6);
        hyphenCount=0;

        uintval = 0;
        pDigitPos = 0;

        length = strlen(stationMAC);
        if (length > 17) 
            return FALSE;

        for (pPos = 0; pPos<length;pPos++) 
        {
            if (stationMAC[pPos] == '-') 
            {
                if (hyphenCount > 5) 
                    return " "; /* its the sixth '-' */
                if ((uintval == 0) && (pDigitPos == 0))
                    return " "; /* string begins with hyphen */
                macval[hyphenCount] = uintval;
                hyphenCount++;
                uintval = 0;
                pDigitPos = 0;
            } 
            else if (isxdigit((int)stationMAC[pPos])) 
            {
                pDigitPos++;
                if (pDigitPos > 2) 
                    return FALSE;
                uintval = uintval*16;
                if (isdigit((int)stationMAC[pPos]))
                    uintval += (stationMAC[pPos]-'0');
                else
                    uintval += hexChar2int(stationMAC[pPos]);
            } 
            else 
            {
                return " ";         /* bad value */
            }
        }
        
        if (hyphenCount < 5) 
            return " ";             /* have to have exactly 5 '-' */

        /* its the last octet */
        if ((uintval == 0) && (pDigitPos == 0))
           return " ";
        macval[hyphenCount] = uintval;


        if (access_list.type == 1) { /* table is black*/
		access_list.cmd = TIAP_PARAM_ACCESS_ADD;
		memcpy(access_list.list[0], macval, ETH_ALEN);
	}
        else 
		{ /* remove the mac */
	    access_list.cmd = TIAP_PARAM_ACCESS_DEL_BY_ADDR;
	    memcpy(access_list.list[0], macval, ETH_ALEN);
	}

        /* Update table in NVRAM */
		if (tiap_wcfg_set_access_list(wd, &access_list, !UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to set access list.");
			return (char*)NULL;
	}
    
    return " ";

}


char* wlan_htmlUpdate_BW_DeletePageMgmt(char *id)
{
#if 0
    UINT32 staID;
    STATUS status;
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    char tmp_buf[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];

    sscanf(id,"%d",&staID);
    if ((staID > 0) && (staID < MGMT_ACCESS_LIST_SIZE))
    {
        mgmtAccessRemMAC(staID);

        /* Update table in NVRAM */
    	status = mgmtAccessGetList(tmp_buf, &length);
        if (status != OK) 
        {
            printf("remAccessMAC: Fatal Error from mgmtAccessGetList (file %s)\n",__FILE__);
            return " ";
        }
    	whal_nvram_setAccessList(tmp_buf, IMMEDIATELY);
    }
    return " ";
#endif

    UINT32 staID;
    tiap_param_access_list_t access_lst;

		if (strncmp(id, "NO_PARAM", 8) == 0)
			return " ";

    staID = atoi(id);
	/*sscanf(id,"%d",&staID);*/
    if ((staID > 0) && (staID < TIAP_PARAM_ACCESS_LIST_SIZE))
    {
	    access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	    if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
		    return (char*)NULL;
	    }
	    
	    access_lst.cmd = TIAP_PARAM_ACCESS_DEL_BY_INDEX;
	    access_lst.index = staID;

	    /* Update table in NVRAM */
	    if (tiap_wcfg_set_access_list(wd, &access_lst, !UPDATE_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to set access list.");
		    return (char*)NULL;
	    }
    }
    return " ";
}


char* wlan_htmlUpdate_BW_AddMacPageMgmt(char *value, char *mac1, char *mac2, 
                                        char *mac3, char *mac4, char *mac5, char *mac6)
{
#if 0
    UINT32 staID;
    UINT8 macAddr[6];
    char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;

    if (strcmp(value,"Add") == 0) 
    {
        /* check that each MAC component has a length of 2 */
        if ((strlen(mac1) != 2) ||
            (strlen(mac2) != 2) ||
            (strlen(mac3) != 2) ||
            (strlen(mac4) != 2) ||
            (strlen(mac5) != 2) ||
            (strlen(mac6) != 2))
        {
            return " ";
        }
        /* check that each component character is a hex digit */
        if ((isxdigit(mac1[0]) == FALSE) ||
            (isxdigit(mac1[1]) == FALSE) ||
            (isxdigit(mac2[0]) == FALSE) ||
            (isxdigit(mac2[1]) == FALSE) ||
            (isxdigit(mac3[0]) == FALSE) ||
            (isxdigit(mac3[1]) == FALSE) ||
            (isxdigit(mac4[0]) == FALSE) ||
            (isxdigit(mac4[1]) == FALSE) ||
            (isxdigit(mac5[0]) == FALSE) ||
            (isxdigit(mac5[1]) == FALSE) ||
            (isxdigit(mac6[0]) == FALSE) ||
            (isxdigit(mac6[1]) == FALSE))
        {
            return " ";
        }

        macAddr[0] = htol(mac1);
        macAddr[1] = htol(mac2);
        macAddr[2] = htol(mac3);
        macAddr[3] = htol(mac4);
        macAddr[4] = htol(mac5);
        macAddr[5] = htol(mac6);

        if (mgmtAccessSetMAC(macAddr) != OK)
        {
            return " ";
        }
        /* Update table in NVRAM */
        if (mgmtAccessGetList(tmpTable, &length) == OK)
        {
            whal_nvram_setAccessList(tmpTable, IMMEDIATELY);
        }
    }

    return " ";
#endif

    __u8 macAddr[ETH_ALEN];
    tiap_param_access_list_t access_lst;

		if (strncmp(value, "NO_PARAM", 8) == 0)
			return " ";

    if (strcmp(value,"Add") == 0) 
    {
	    /* check that each MAC component has a length of 2 */
	    if ((strlen(mac1) != 2) ||
			    (strlen(mac2) != 2) ||
			    (strlen(mac3) != 2) ||
			    (strlen(mac4) != 2) ||
			    (strlen(mac5) != 2) ||
			    (strlen(mac6) != 2))
	    {
		    return " ";
	    }
	    /* check that each component character is a hex digit */
	    if ((isxdigit((int)mac1[0]) == FALSE) ||
			    (isxdigit((int)mac1[1]) == FALSE) ||
			    (isxdigit((int)mac2[0]) == FALSE) ||
			    (isxdigit((int)mac2[1]) == FALSE) ||
			    (isxdigit((int)mac3[0]) == FALSE) ||
			    (isxdigit((int)mac3[1]) == FALSE) ||
			    (isxdigit((int)mac4[0]) == FALSE) ||
			    (isxdigit((int)mac4[1]) == FALSE) ||
			    (isxdigit((int)mac5[0]) == FALSE) ||
			    (isxdigit((int)mac5[1]) == FALSE) ||
			    (isxdigit((int)mac6[0]) == FALSE) ||
			    (isxdigit((int)mac6[1]) == FALSE))
	    {
		    return " ";
	    }

	    macAddr[0] = htol(mac1);
	    macAddr[1] = htol(mac2);
	    macAddr[2] = htol(mac3);
	    macAddr[3] = htol(mac4);
	    macAddr[4] = htol(mac5);
	    macAddr[5] = htol(mac6);


	    access_lst.cmd = TIAP_PARAM_ACCESS_LST;

	    /*get access list type*/
	    if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
		    return (char*)NULL;
	    }

	    access_lst.cmd = TIAP_PARAM_ACCESS_ADD;
	    memcpy(access_lst.list[0], macAddr, ETH_ALEN);

	    /* Update table in NVRAM and driver */
	    if (tiap_wcfg_set_access_list(wd, &access_lst, !UPDATE_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to set access list.");
		    return (char*)NULL;
	    }
    }

    return " ";
}


/********************************************************************************/
/*                    wlan_htmlGetBWTableStatusPageMgmt(void)                       */
/********************************************************************************/
/* DESCRIPTION: Select a value for the BW table radio buttons                   */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the BW's table state     */
/*				"" if the value is different than the BW table states           */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlGetBWTableStatusPageMgmt(char *value)
{
#if 0
	int preambleMode;
	int mode;
	UINT8 temp, enable, type;
    BOOL state;

	sscanf(value,"%d",&mode);

    whal_nvram_getAccessListEnable(&enable);
    if (enable == 1)
        whal_nvram_getAccessListType(&type);

    switch (mode) 
    {
        case 1:     /* White */
            if ((enable == 1) && (type == 0))
                return "checked";
            break;
        case 2:     /* Black */
            if ((enable == 1) && (type == 1))
                return "checked";
            break;
        case 3:     /* Disable */
            if (enable == 0) 
                return "checked";
            break;
    }
    return " ";
#endif

    int mode;

    tiap_param_access_list_t access_list;

    access_list.cmd = TIAP_PARAM_ACCESS_LST;
    if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
	    return (char*)NULL;
    }

    sscanf(value,"%d",&mode);

    switch (mode) 
    {
	    case 1:     /* White */
		    if ((access_list.enabled == 1) && (access_list.type == 0))
			    return "checked";
		    break;
	    case 2:     /* Black */
		    if ((access_list.enabled == 1) && (access_list.type == 1))
			    return "checked";
		    break;
	    case 3:     /* Disable */
		    if (access_list.enabled == 0) 
			    return "checked";
		    break;
    }
    return " ";

}


/********************************************************************************/
/*                    wlan_htmlGetBWTableStatusPageMgmt(void)                       */
/********************************************************************************/
/* DESCRIPTION: Select a value for the BW table radio buttons                   */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the BW's table state     */
/*				"" if the value is different than the BW table states           */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlSavePageMgmt(char *save_button, char *BWstatus, char *MultipleSSID, char *stam) 
{
	int bwTableStatus/*, multi_ssid_enabled*/;
	tiap_param_access_list_t access_lst;
	//tiap_param_multi_ssid_t multi_ssid;
    int savingIndex = 0;
	BOOL DoSave = FALSE;
	
	if (strncmp(save_button, "Saving",6))
        return "";
	if (strcmp(save_button, "Saving") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+6);
    
	if (DoSave || (globalWebSavingIndex <= savingIndex))
	{	
		sscanf(BWstatus,"%d",&bwTableStatus);

		access_lst.cmd = TIAP_PARAM_ACCESS_LST;
		if (tiap_wcfg_get_access_list(wd, &access_lst, !FROM_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Reading configuration failed");
			return (char*)NULL;
		}

		switch (bwTableStatus) 
		{
			case 1:	/* White */
				access_lst.enabled = 1;
				access_lst.type = 0;

				break;
			case 2:     /* Black */
				access_lst.enabled = 1;
				access_lst.type = 1;
				break;
			case 3:     /* Disable */
				access_lst.enabled = 0;
				break;
			default:
				return (char*)NULL;
		} 

		access_lst.cmd = TIAP_PARAM_ACCESS_LST;
		if (tiap_wcfg_set_access_list(wd, &access_lst, UPDATE_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Configuration failed");
			return (char*)NULL;
		}
		restart();                                /* Reboot the AP */
	}
	return " ";
}


/********************************************************************************/
/*                  wlan_htmlGetMultipleSsidStatusPageMgmt(void)                    */
/********************************************************************************/
/* DESCRIPTION: Select a value for the Mutiple SSID table radio buttons         */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:                                                                      */
/*                                                                              */
/* RETURNS:     "checked" if the value is identical to the SSID table state     */
/*              "" if the value is different than the SSID table states         */
/*                                                                              */
/********************************************************************************/
char *wlan_htmlGetMultipleSsidStatusPageMgmt(const char *profile)
{
	__u16 vlan;

	validate_profile(profile);

	if( tiap_wcfg_get_primary_vlan(wd, &vlan, FROM_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to get primary vlan.");
		return NULL;
	}

	if( !vlan ) 
		return "disabled";
	
	if( profile_map_tbl[atoi(profile)] != -1 )
		return "checked";

	return "";
}


char *wlan_htmlGetDescriptionSetupPage(char *option)
{
	int value = atoi(option);
	BOOL isError = TRUE;
	__u16 CurrentVlan;

	if (value == 0) 
	{
		/* end of page - reset error counter. It is shown only once */
		 mSSID_errorNumber = 0;
		 return " ";
	}
    if (value == 2) 
	{
		if ( tiap_wcfg_get_primary_vlan(wd, &CurrentVlan, FROM_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get primary vlan.");
			return "";
		}  
		if ( CurrentVlan )
		{
			strcpy(tempbuf,"<i>(Setting to 0 will reset the Multiple SSID configuration)</i>");
			return tempbuf;
		}
		else
			return " ";
	}
	if (mSSID_errorNumber == 0) 
	{
		strcpy(tempbuf,"<font face=\"Arial\">This page contains basic WLAN setup information.</font>");
	}
	else
	{
		/* Present current error to the user as part of the page, RED and HIGHLIGHTED (H3) */
		strcpy(tempbuf,"<h3><font color=\"#FF0000\">");
		switch (mSSID_errorNumber) 
		{
			case MULTIPLE_SSID_MSG_ERROR_VLAN_ZERO:
				strcat(tempbuf,"Set failed! Vlan should have a value greater than 0");
				break;
			case MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS:
				strcat(tempbuf,"Set failed! Vlan is already exists");
				break;
			case MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS:
				strcat(tempbuf,"Set failed! The given SSID already exists");
				break;
			case MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR:
				strcat(tempbuf,"Set failed! Internal ERROR!");
				break;
			case MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM:
				strcat(tempbuf,"Set failed! Bad security parameters");
				break;
			case MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR:
				strcat(tempbuf,"Set failed! Profile Error");
				break;
			case MULTIPLE_SSID_MSG_ERROR_WRONG_KEYS_LENGTH:
				strcat(tempbuf,"Set failed! WEP Keys have a wrong lenghts (use same for all)");
				break;
			case MULTIPLE_SSID_MSG_ERROR_VLAN_OVERFLOW:
				strcat(tempbuf,"Set failed! Vlan should not exceed 4095");
				break;
			default :
				isError = FALSE;
				break;
		}
		if (isError) 
			strcat(tempbuf,"</font></h3>");
		else
			strcpy(tempbuf," ");
	}
	return tempbuf;
}


char* wlan_htmlGetDescriptionMultipleSsidPage(char *option)
{
	int value = atoi(option);
	__u16 vlan_id;
	BOOL isError = TRUE;

	if (value == 0) 
	{
		/* end of page - reset error counter. It is shown only once */
		 mSSID_errorNumber = 0;
		 return "";
	}

	if (mSSID_errorNumber == 0) 
	{
		if( tiap_wcfg_get_primary_vlan(wd, &vlan_id, FROM_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get primary vlan.");
			return "";
		}
		/* no error - show normal message */
		if( !vlan_id ) 
			strcpy(tempbuf,"The Multiple SSID feature is disabled. A default Primary VLAN should be set first (see Setup page)");
		else
			strcpy(tempbuf,"This page contains configuration paramters for Mulitple SSID <i>(Note: Please configure one column at a time.)</i>");
	}
	else
	{
		/* Present current error to the user as part of the page, RED and HIGHLIGHTED (H3) */
		strcpy(tempbuf,"<h3><font color=\"#FF0000\">");
		switch (mSSID_errorNumber) 
		{
			case MULTIPLE_SSID_MSG_ERROR_VLAN_ZERO:
				strcat(tempbuf,"Set failed! Vlan should have a value greater than 0");
				break;
			case MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS:
				strcat(tempbuf,"Set failed! Vlan is already exists");
				break;
			case MULTIPLE_SSID_MSG_ERROR_SSID_EMPTY:
				strcat(tempbuf,"Set failed! SSID should not be empty!");
				break;
			case MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS:
				strcat(tempbuf,"Set failed! The given SSID already exists");
				break;
			case MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR:
				strcat(tempbuf,"Set failed! Internal ERROR!");
				break;
			case MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM:
				strcat(tempbuf,"Set failed! Bad security parameters");
				break;
			case MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR:
				strcat(tempbuf,"Set failed! Profile Error");
				break;
			case MULTIPLE_SSID_MSG_ERROR_VLAN_OVERFLOW:
				strcat(tempbuf,"Set failed! Vlan should not exceed 4095");
                break;
			default :
				isError = FALSE;
				break;
		}
		if (isError) 
			strcat(tempbuf,"</font></h3>");
		else
			strcpy(tempbuf,"");
	}
	return tempbuf;
}


char* wlan_htmlSavePageAdvanced(char* save_button, char *preambleMode, char *inputSlotTimeMode,
                                char *inputPbccMode, char *inputErpMode, char *InputMaxBasic, char *inputMaxActive,
                                char *addrate1, char *addrate2, char *addrate3, char *addrate4,
                                char *addrate5, char *addrate6, char *addrate7, char *addrate8,
                                char *grate1, char *grate2, char *grate3, char *grate4,
                                char *grate5, char *grate6, char *grate7, char *grate8,
                                char *arate1, char *arate2, char *arate3, char *arate4,
                                char *arate5, char *arate6, char *arate7, char *arate8, char* usageTimeMode)
{
#if 0
    unsigned long ratePbcc, maxBasic, maxActive;
    char ratesSet[8];
    unsigned long pbccMode, slotTimeMode, preamble, erpMode, usageTimeModeValue;
    UINT8 ratesLen, currentMaxActive;
    BOOL quit;
    int  i;
    char str[4], support[3];
    char rateString[OFDM_RATES_MAX][3] =
        { " 6", " 9", "12", "18", "24", "36", "48", "54" };
    UINT8 rateValue[OFDM_RATES_MAX] = { DOT11_RATE_6,  DOT11_RATE_9,
                                        DOT11_RATE_12, DOT11_RATE_18,
                                        DOT11_RATE_24, DOT11_RATE_36,
                                        DOT11_RATE_48, DOT11_RATE_54 };
    UINT8 curRates[OFDM_RATES_MAX];
    UINT8 curRatesLen=0;
    UINT8 newRates[OFDM_RATES_MAX];
    UINT8 newRatesLen=0;
    UINT8 dot11Mode;

    char  rates[CCK_PBCC_RATES_MAX];
    UINT8 grates[OFDM_RATES_MAX], arate[OFDM_RATES_MAX];
    UINT8 ratesLength = 0;

    char addrate[OFDM_RATES_MAX], grate[OFDM_RATES_MAX]; 
    UINT8 TotalWones, band1, band2, woneIndex;
    BOOL  doIt = TRUE;

    if (!strcmp(save_button,"Saving"))
    {
        wdrv_dbGetNumSupWone(&TotalWones);
        switch (TotalWones) 
        {
            case WONE_SINGLE_BAND:
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_5GHZ)
                    doIt = FALSE;
                woneIndex = 0;
                break;

            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band2);
                    if (band2 == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        doIt = FALSE;   /* both Wone 0,1 are 'A' */
                }
				sscanf(usageTimeMode,"%d",&usageTimeModeValue);	/* only in WONE */
				if (usageTimeModeValue == 2)
					usageTimeModeValue = 0;	/* Set to FALSE */
				wdrv_dbSetUsageTimeMode((UINT8)usageTimeModeValue, TRUE);
                break;
        }

        /* preamble */
        sscanf(preambleMode,"%d",&preamble);
        wdrv_dbSetPreambleMode(0,(UINT8)preamble);

        /* slot time */
        sscanf(inputSlotTimeMode,"%d",&slotTimeMode);
        wdrv_dbSetSlotTimeMode(0,(UINT8)slotTimeMode);

        if (doIt) 
        {
            /* ERP */
            sscanf(inputErpMode,"%d",&erpMode);
            wdrv_dbSetERPProtectionMode(0,(UINT8)erpMode);
        
            /* PBCC */
            sscanf(inputPbccMode,"%d",&pbccMode);
            wdrv_dbSetPBCCAlgorithmMode(0,(UINT8)pbccMode);

            /* rates */     
            memset(ratesSet, 0, 8);     
            ratesLen = 0;
            sscanf(InputMaxBasic,"%d",&maxBasic);
            sscanf(inputMaxActive,"%d",&maxActive);
        
            if (maxBasic <= maxActive)  
            {
                switch (maxActive)
                {
                    case 5: 
                        ratesSet[4] = 0x2C;
                        ratesLen++;
                        
                    case 4:
                        ratesLen++;
                        ratesSet[3] = 0x16;
                        
                    case 3:
                        ratesLen++;
                        ratesSet[2] = 0x0B;
                        
                    case 2:
                        ratesLen++;
                        ratesSet[1] = 0x04;
                        
                    case 1:
                        ratesSet[0] = 0x02;
                        ratesLen++;
                        break;
                        
                    default:
                        break;
                }
                switch (maxBasic)
                {
                    case 4:
                        ratesSet[3] = 0x96;
        
                    case 3:
                        ratesSet[2] = 0x8B;
    
                    case 2:
                        ratesSet[1] = 0x84;
    
                    case 1:
                        ratesSet[0] = 0x82;
                        break;
    
                    default:
                        break;
                }
                wdrv_dbSetStationConfigOperationalRateSet(0,ratesSet, ratesLen);
            }
        }

        if (doIt)   /* We are in B/G band - skipping the B modes, remain only with G */
        {
            wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
            if ((dot11Mode == DOT11_MODE_MIXED) || (dot11Mode == DOT11_MODE_OFDM_ONLY))
                doIt = TRUE;   /* this is G only mode */
            else
                doIt = FALSE;
        }
        
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(addrate1,"%c",&addrate[0]);
            sscanf(addrate2,"%c",&addrate[1]);
            sscanf(addrate3,"%c",&addrate[2]);
            sscanf(addrate4,"%c",&addrate[3]);
            sscanf(addrate5,"%c",&addrate[4]);
            sscanf(addrate6,"%c",&addrate[5]);
            sscanf(addrate7,"%c",&addrate[6]);
            sscanf(addrate8,"%c",&addrate[7]);
            sscanf(grate1,"%c",&grate[0]);
            sscanf(grate2,"%c",&grate[1]);
            sscanf(grate3,"%c",&grate[2]);
            sscanf(grate4,"%c",&grate[3]);
            sscanf(grate5,"%c",&grate[4]);
            sscanf(grate6,"%c",&grate[5]);
            sscanf(grate7,"%c",&grate[6]);
            sscanf(grate8,"%c",&grate[7]);

            whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);
                if ((addrate[i] != 'Y') && (addrate[i] != 'y') && (addrate[i] != 'N') && (addrate[i] != 'n') && (addrate[i] != 'B') && (addrate[i] != 'b'))
                    memcpy(&addrate[i], str, 1);
                if ((addrate[i] == 'Y') || (addrate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((addrate[i] == 'B') || (addrate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalAdditionalRateSet(woneIndex,newRates, newRatesLen);
            newRatesLen = 0;
            memset(newRates, 0, OFDM_RATES_MAX);
            whal_nvram_getOperationalOFDMRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalOFDMRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);

                if ((grate[i] != 'Y') && (grate[i] != 'y') && (grate[i] != 'N') && (grate[i] != 'n') && (grate[i] != 'B') && (grate[i] != 'b'))
                    memcpy(&grate[i], str, 1);
                if ((grate[i] == 'Y') || (grate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((grate[i] == 'B') || (grate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalOFDMRateSet(woneIndex,newRates, newRatesLen);
        }
        
        doIt = FALSE;
        switch (TotalWones) 
        {
            case 1:
                if (band1 == RADIO_BAND_5GHZ)
                    doIt = TRUE;
                woneIndex = 0;
                break;

            case 2:
                doIt = TRUE;
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_5GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band2);
                    if (band2 == RADIO_BAND_5GHZ)
                        woneIndex = 1;
                    else
                        doIt = FALSE;   /* both Wone 0,1 are 'A' */
                }
                break;
        }
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(arate1,"%c",&arate[0]);
            sscanf(arate2,"%c",&arate[1]);
            sscanf(arate3,"%c",&arate[2]);
            sscanf(arate4,"%c",&arate[3]);
            sscanf(arate5,"%c",&arate[4]);
            sscanf(arate6,"%c",&arate[5]);
            sscanf(arate7,"%c",&arate[6]);
            sscanf(arate8,"%c",&arate[7]);

            newRatesLen = 0;
            memset(newRates, 0, OFDM_RATES_MAX);
            whal_nvram_getOperationalOFDMRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalOFDMRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);

                if ((arate[i] != 'Y') && (arate[i] != 'y') && (arate[i] != 'N') && (arate[i] != 'n') && (arate[i] != 'B') && (arate[i] != 'b'))
                    memcpy(&arate[i], str, 1);
                if ((arate[i] == 'Y') || (arate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((arate[i] == 'B') || (arate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalOFDMRateSet(woneIndex,newRates, newRatesLen);
        }

        consoleReboot();
    }

    return (char*)NULL;
#endif

    unsigned long maxBasic, maxActive;
    char ratesSet[8];
    UINT8 ratesLen;
    int  i,k;
    /*char str[4];*/
    UINT8 newRates[TIAP_PARAM_OFDM_RATES_MAX];
    UINT8 newRatesLen=0;

    /*UINT8 rateValue[TIAP_PARAM_OFDM_RATES_MAX] = { DOT11_RATE_6,  DOT11_RATE_9,
	    DOT11_RATE_12, DOT11_RATE_18,
	    DOT11_RATE_24, DOT11_RATE_36,
	    DOT11_RATE_48, DOT11_RATE_54 };*/
    UINT8 arate[TIAP_PARAM_OFDM_RATES_MAX];

    char addrate[TIAP_PARAM_OFDM_RATES_MAX], grate[TIAP_PARAM_OFDM_RATES_MAX]; 
    UINT8 band1, band2, woneIndex=0;
    BOOL  doIt = TRUE;

    __u8 num_sup_wone;
    tiap_param_wone_t wone;
    __u32 time_mode, pream_mode, slot_mode, pbcc_mode, erp_mode;
    char rates[TIAP_PARAM_CCK_PBCC_RATES_MAX];
    int savingIndex = 0;
	BOOL DoSave = FALSE;
   	__u8 tmp;

	if (strncmp(save_button, "Saving",6))
        return "";
	if (strcmp(save_button, "Saving") == 0)
		DoSave = TRUE;
    savingIndex = atoi(save_button+6);
    
	if (DoSave || (globalWebSavingIndex <= (savingIndex+1)))
    {
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &num_sup_wone, FROM_STORAGE) ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return (char*)NULL;
	}
        switch (num_sup_wone) 
        {
            case TIAP_PARAM_WONE_SINGLE_BAND:

						wone.index = 0;
						wone.type = wone_band;
						if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to get band.");
							return (char*)NULL;
						}
				
						band1 = wone.elem.band;
					
						if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
							doIt = FALSE;
						
						woneIndex = 0;
						break;

			case TIAP_PARAM_WONE_DUAL_BAND:
		
						wone.index = 0;
						wone.type = wone_band;
						if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to get band.");
							return (char*)NULL;
						}
				
						band1 = wone.elem.band;
						
						if (band1 == TIAP_PARAM_RADIO_BAND_2_4GHZ)
								woneIndex = 0;
						else
						{
							wone.index = 1;
							wone.type = wone_band;
							if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
								PRINT_CONFIG_ERR_MSG("Failed to get band.");
								return (char*)NULL;
							}
				
							band2 = wone.elem.band;
				
							if (band2 == TIAP_PARAM_RADIO_BAND_2_4GHZ)
										woneIndex = 1;
									else
										doIt = FALSE;   /* both Wone 0,1 are 'A' */
						}
						
						
						sscanf(usageTimeMode, "%u", &time_mode);	/* only in WONE */
						
						if (time_mode == 2)
							time_mode = 0;	/* Set to FALSE */
   						tmp = (UINT8) time_mode;
						if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_TIME_MODE, &tmp, !UPDATE_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to set time mode.");
							return (char*)NULL;
						}
				
								break;

				default:
						return (char*)NULL;
        }

        /* preamble */
        sscanf(preambleMode, "%u", &pream_mode);
		wone.index = 0;
		wone.type = wone_pream_mode;
		wone.elem.pream_mode = (UINT8) pream_mode;

		if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return (char*)NULL;
		}
	
		/* slot time */
		sscanf(inputSlotTimeMode, "%u", &slot_mode);
		wone.index = 0;
		wone.type = wone_slot_mode;
		wone.elem.pream_mode = (UINT8) slot_mode;

		if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set slot mode.");
			return (char*)NULL;
		}

        if (doIt) 
        {
			/* ERP */
			sscanf(inputErpMode, "%u", &erp_mode);
			wone.index = 0;
			wone.type = wone_erp_mode;
			wone.elem.pream_mode = (UINT8) erp_mode;
	
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set erp mode.");
				return (char*)NULL;
			}
        
            /* PBCC */
            sscanf(inputPbccMode, "%u", &pbcc_mode);
			wone.index = 0;
			wone.type = wone_pbcc;
			wone.elem.pream_mode = (UINT8) pbcc_mode;
	
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set pbcc mode.");
				return (char*)NULL;
			}

			/* rates */     
			memset(ratesSet, TIAP_PARAM_RATE_NOT_SUPPORTED, 5);     
			ratesLen = 5;
			sscanf(InputMaxBasic,"%ld",&maxBasic);
			sscanf(inputMaxActive,"%ld",&maxActive);

			for(i=1, k=0; i<=ratesLen; i++)
			{
				if(i > maxActive)
					rates[k++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
				else if(i <= maxBasic)
					rates[k++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
				else
					rates[k++] = TIAP_PARAM_RATE_SUPPORTED;
			}
		
			wone.index = 0;
			wone.type = wone_oper_rate;
			memcpy((void *)wone.elem.b_legacy.rates, (void *)rates, ratesLen);
			wone.elem.b_legacy.len = ratesLen;
		
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set operational rates.");
				return (char*)NULL;
			}
	}

        if (doIt)   /* We are in B/G band - skipping the B modes, remain only with G */
        {
	    wone.index = woneIndex;
	    wone.type = wone_dot11_mode;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		    return (char*)NULL;
	    }

            if ((wone.elem.dot11_mode == TIAP_PARAM_D11MODE_MIXED) || (wone.elem.dot11_mode == TIAP_PARAM_D11MODE_OFDM))
                doIt = TRUE;   /* this is G only mode */
            else
                doIt = FALSE;
        }
        
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(addrate1,"%c",&addrate[0]);
            sscanf(addrate2,"%c",&addrate[1]);
            sscanf(addrate3,"%c",&addrate[2]);
            sscanf(addrate4,"%c",&addrate[3]);
            sscanf(addrate5,"%c",&addrate[4]);
            sscanf(addrate6,"%c",&addrate[5]);
            sscanf(addrate7,"%c",&addrate[6]);
            sscanf(addrate8,"%c",&addrate[7]);
            sscanf(grate1,"%c",&grate[0]);
            sscanf(grate2,"%c",&grate[1]);
            sscanf(grate3,"%c",&grate[2]);
            sscanf(grate4,"%c",&grate[3]);
            sscanf(grate5,"%c",&grate[4]);
            sscanf(grate6,"%c",&grate[5]);
            sscanf(grate7,"%c",&grate[6]);
            sscanf(grate8,"%c",&grate[7]);

	    wone.index = woneIndex;
	    wone.type = wone_oper_add;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		    return (char*)NULL;
	    }

	    
		for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
		{
			if(i > wone.elem.ofdm_legacy.len)
				break;

			if((addrate[i] != 'y')&&(addrate[i] != 'Y')&&(addrate[i] != 'n')&&(addrate[i] != 'N')&&(addrate[i] != 'b')&&(addrate[i] != 'B'))
			{
				if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
					addrate[i] = 'Y';
				else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
					addrate[i] = 'B';
				else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
					addrate[i] = 'N';
			}

			if ((addrate[i] == 'Y') || (addrate[i] == 'y'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
			else if ((addrate[i] == 'B') || (addrate[i] == 'b'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
			else
				newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
		}

	    wone.index = woneIndex;
	    wone.type = wone_oper_add;
	    memcpy((void *)&wone.elem.ofdm_legacy.rates, (void *)&newRates,
			    sizeof(wone.elem.ofdm_legacy.rates));
	    wone.elem.ofdm_legacy.len = newRatesLen;
	    if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to set operational additional rates.");
		    return (char*)NULL;
	    }

	    
        newRatesLen = 0;
        memset(newRates, 0, TIAP_PARAM_OFDM_RATES_MAX);

	    wone.index = woneIndex;
	    wone.type = wone_oper_ofdm;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get operational ofdm rates.");
		    return (char*)NULL;
	    }



		for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
		{
			if(i > wone.elem.ofdm_extend.len)
				break;

			if((grate[i] != 'y')&&(grate[i] != 'Y')&&(grate[i] != 'n')&&(grate[i] != 'N')&&(grate[i] != 'b')&&(grate[i] != 'B'))
			{
				if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
					grate[i] = 'Y';
				else if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
					grate[i] = 'B';
				else if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
					grate[i] = 'N';
			}

			if ((grate[i] == 'Y') || (grate[i] == 'y'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
			else if ((grate[i] == 'B') || (grate[i] == 'b'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
			else
				newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
		}


	    wone.index = woneIndex;
	    wone.type = wone_oper_ofdm;
	    memcpy((void *)&wone.elem.ofdm_extend.rates, (void *)&newRates,
			    sizeof(wone.elem.ofdm_extend.rates));
	    wone.elem.ofdm_extend.len = newRatesLen;
	    if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to set operational ofdm rates.");
		    return (char*)NULL;
	    }
        }
        
        doIt = FALSE;
        switch (num_sup_wone)
        {
            case 1:
                if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
                    doIt = TRUE;
                woneIndex = 0;
                break;

            case 2:
                doIt = TRUE;

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return (char*)NULL;
				}

				band1 = wone.elem.band;
		
                if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
                    woneIndex = 0;
                else
                {
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return (char*)NULL;
					}

					band2 = wone.elem.band;
					
					if (band2 == TIAP_PARAM_RADIO_BAND_5GHZ)
						woneIndex = 1;
					else
						doIt = FALSE;   /* both Wone 0,1 are 'A' */
				}
                break;
        }
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(arate1,"%c",&arate[0]);
            sscanf(arate2,"%c",&arate[1]);
            sscanf(arate3,"%c",&arate[2]);
            sscanf(arate4,"%c",&arate[3]);
            sscanf(arate5,"%c",&arate[4]);
            sscanf(arate6,"%c",&arate[5]);
            sscanf(arate7,"%c",&arate[6]);
            sscanf(arate8,"%c",&arate[7]);

            newRatesLen = 0;
            memset(newRates, 0, TIAP_PARAM_OFDM_RATES_MAX);
	    
			wone.index = woneIndex;
			wone.type = wone_oper_add;

			if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get operational ofdm rates.");
				return (char*)NULL;
			}
			
			for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
			{
				if(i > wone.elem.ofdm_legacy.len)
					break;

				if((arate[i] != 'y')&&(arate[i] != 'Y')&&(arate[i] != 'n')&&(arate[i] != 'N')&&(arate[i] != 'b')&&(arate[i] != 'B'))
				{
					if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
						arate[i] = 'Y';
					else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
						arate[i] = 'B';
					else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
						arate[i] = 'N';
				}

				if ((arate[i] == 'Y') || (arate[i] == 'y'))
					newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
				else if ((arate[i] == 'B') || (arate[i] == 'b'))
					newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
				else
					newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
			}

			wone.index = woneIndex;
			wone.type = wone_oper_add;
			memcpy((void *)&wone.elem.ofdm_legacy.rates, (void *)&newRates,
					sizeof(wone.elem.ofdm_legacy.rates));
			wone.elem.ofdm_legacy.len = newRatesLen;
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set operational ofdm rates.");
				return (char*)NULL;
			}
        }

	restart();
    }

    return (char*)"";

	
}


/******************************************************************************
*                               hexChar2int(void)                                 *
*******************************************************************************
DESCRIPTION:	Return the decimal value of an hexadecimal character.  						
																										
INPUT:			c - hexadecimal character.
OUTPUT:		   None.
 
RETURN:			decimal value.
																										
******************************************************************************/
unsigned char hexChar2int(unsigned char c)
{
   if (c >= 'A' && c <= 'F')
      return (c - 'A' + 10);
   else if (c >= 'a' && c <= 'f')
      return (c - 'a' + 10);
   return c - '0';
}


/*******************************************************************************

                                    Privacy                                     

********************************************************************************/

char* wlan_htmlGetPrivacyGroupKeyInterval(const char *profile)
{
    __u32 value;

    validate_profile(profile);

    if( profile_map_tbl[atoi(profile)] == -1 ) {
	    tiap_sec_profile_t profile;
	    wcfg_storage_setdefaults_sec_profile(&profile);
	    value = profile.sec.interval;
    }
    else tiap_wcfg_get_gtk_interval(wd, &value, profile_map_tbl[atoi(profile)]);

    value = value / 1000;
    sprintf(tempbuf, "%d", value);
    return tempbuf;
}

char* wlan_htmlGetPrivacyPskStringValue(const char *profile)
{
	__s8 passwd[TIAP_WPA_MAX_PSK_PWD_LEN+1];
	__u8 len;

	validate_profile(profile);

	if( profile_map_tbl[atoi(profile)] == -1 ) {
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		strncpy(passwd, profile.psk.psk_passwd, profile.psk.psk_passwd_len);
		passwd[TIAP_WPA_MAX_PSK_PWD_LEN] = '\0';
		len = profile.psk.psk_passwd_len;
	}
	else tiap_wcfg_get_psk_passwd(wd, passwd, &len, profile_map_tbl[atoi(profile)]);
	passwd[len] = '\0';

	sprintf(tempbuf, "\"%s\"", passwd);
	return tempbuf;
}

char* wlan_htmlGetPrivacyRadiusSecret(const char *profile)
{
	__s8 secret[TIAP_WPA_MAX_RADIUS_SECRET_LEN+1];
	__u8 len;

	validate_profile(profile);

	if( profile_map_tbl[atoi(profile)] == -1 ) {
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		strncpy(secret, profile.radius.secret, profile.radius.secret_len);
		len = profile.radius.secret_len;
	}
	else tiap_wcfg_get_radius_secret(wd, secret, &len, profile_map_tbl[atoi(profile)]);
	secret[len] = '\0';
	sprintf(tempbuf, "\"%s\"", secret);

	return tempbuf;
}


char* wlan_htmlGetPrivacyRadiusPort(const char *profile)
{
    __u16 port;

    validate_profile(profile);
    
    if( profile_map_tbl[atoi(profile)] == -1 ) {
	    tiap_sec_profile_t profile;
	    wcfg_storage_setdefaults_sec_profile(&profile);
	    port = profile.radius.port;
    }
    else tiap_wcfg_get_radius_port(wd, &port, profile_map_tbl[atoi(profile)]);
    sprintf(tempbuf,"%hu", port);

    return tempbuf;

}

char* wlan_htmlGetPrivacyRadiusAddr(char *ipComponent, const char *profile)
{
	__u32 ipaddr;
	__u8 *p = (__u8 *)&ipaddr;
	unsigned long component;

	validate_profile(profile);

	if( profile_map_tbl[atoi(profile)] == -1 ) {
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		ipaddr = profile.radius.serv_ip_addr;
	}
	else tiap_wcfg_get_radius_ipaddr(wd, &ipaddr, profile_map_tbl[atoi(profile)]);
	sscanf(ipComponent,"%ld",&component);

	switch (component) 
	{
		case 1:
			sprintf(tempbuf,"%u", p[0]);
			break;
		case 2:
			sprintf(tempbuf,"%u", p[1]);
			break;
		case 3:
			sprintf(tempbuf,"%u", p[2]);
			break;
		case 4:
			sprintf(tempbuf,"%u", p[3]);
			break;
	}
	return tempbuf;
}


char *wlan_htmlGetPrivacyPsk(char *option, const char *profile)
{
	int inputKeyId;
	__u32 value = 0;

	validate_profile(profile);

	sscanf(option, "%d", &inputKeyId);

	if (inputKeyId == 1) 
		return "disabled = true";	/* PSK-HEX not supported */

	if( profile_map_tbl[atoi(profile)] == -1 ) 
	{
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		value = profile.psk.pass_phrase_mode;
	}
	else 
		tiap_wcfg_get_psk_passphrase_mode(wd, &value, profile_map_tbl[atoi(profile)]);
	if (inputKeyId == value)
		return "checked";
	else 
		return "";
}

char* wlan_htmlGetPrivacyDefaultGateway(char *ipComponent)
{
#if 0
    UINT8 addr1, addr2, addr3, addr4;
	unsigned long component;

    wdrv_dbGetDefaultGateway(&addr1,&addr2,&addr3,&addr4);
    sscanf(ipComponent,"%d",&component);

    switch (component) 
    {
        case 1:
            sprintf(tempbuf,"%d",addr1);
            break;
        case 2:
            sprintf(tempbuf,"%d",addr2);
            break;
        case 3:
            sprintf(tempbuf,"%d",addr3);
            break;
        case 4:
            sprintf(tempbuf,"%d",addr4);
            break;
    }
	return tempbuf;
#endif

	unsigned long component;
	__u32 gateway;
	__u8 *p = (__u8 *)&gateway;

	tiap_wcfg_get_default_gateway(&gateway);
	sscanf(ipComponent,"%ld",&component);

	switch (component) 
	{
		case 1:
			sprintf(tempbuf,"%u", p[0]);
			break;
		case 2:
			sprintf(tempbuf,"%u", p[1]);
			break;
		case 3:
			sprintf(tempbuf,"%u", p[2]);
			break;
		case 4:
			sprintf(tempbuf,"%u", p[3]);
			break;
	}
	return tempbuf;
}

int is_wep_enabled(void)
{
	tiap_param_security_type_t security_type;
	__u32 i, cipher;

	for(i = 0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
		if( profile_map_tbl[i] != -1 )
	       	{
			if( tiap_wcfg_get_security_type(wd, &security_type, profile_map_tbl[i]) == 2 ) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to get security type.");
				exit(1);
			}
			if (security_type & (TIAP_PARAM_SECURITY_WEP | TIAP_PARAM_SECURITY_802_1X))
			{
				tiap_wcfg_get_default_cipher(wd, &cipher, profile_map_tbl[i]);
				if (cipher != TIAP_WPA_CIPHER_NON)
					return profile_map_tbl[i];
			}
		}

	return i;
}



char *wlan_htmlGetSecurityType(const char *web_security_type, const char *profile)
{
	int ret, security_type_val = atoi(web_security_type), nprofile = atoi(profile);
	tiap_param_security_type_t security_type;
	__u32 wep_profile, cipher;

	validate_profile(profile);

	/* In thoses cases where WEP or Dynamic-WEP are already enabled in another profile,
	   disable that option in all the other */
	if ((wep_profile = is_wep_enabled()) < TIAP_PARAM_NUM_MULTIPLE_SSID)
	{
		if ((wep_profile != profile_map_tbl[nprofile]) && 
			((security_type_val == 4) || (security_type_val == 5)))
			return " ";
	}

	if( profile_map_tbl[nprofile] == -1 ) 
	{
		tiap_sec_profile_t SecProfile;
		wcfg_storage_setdefaults_sec_profile(&SecProfile);
		security_type = SecProfile.sec.type;
		cipher = SecProfile.sec.default_cipher;
	}
	else 
	{
		if( (ret = tiap_wcfg_get_security_type(wd, &security_type,
					profile_map_tbl[nprofile])) == 1 ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get wpa mode.\n");
			return (char*)NULL;
		}
		tiap_wcfg_get_default_cipher(wd, &cipher, profile_map_tbl[nprofile]);
	}
	strcpy(tempbuf,"<option ");
	//printf("%s:%d %d %d\n", __func__, __LINE__, ret, security_type);
	//if( ret == 2 && !security_type_val)
		//return "selected";
	
	//printf("%s:%d %d %u\n", __func__, __LINE__, nprofile, cipher);
	if (security_type & TIAP_PARAM_SECURITY_WEP) 
	{
		if (cipher == TIAP_WPA_CIPHER_NON && (security_type_val==0)) /* no security */
			strcat(tempbuf,"selected");
		else if (cipher != TIAP_WPA_CIPHER_NON && (security_type_val == 4))
				strcat(tempbuf,"selected");
	}
	else if (security_type & TIAP_PARAM_SECURITY_802_1X) 
	{
		if ((cipher != TIAP_WPA_CIPHER_NON) && (security_type_val == 5))
				strcat(tempbuf,"selected");
	}
	else
	{
#define is_val_type(val, type) \
		(((val) & (type)) == (type))
	
		/* operator precedence in the following code is correct, see K&R */
		if (is_val_type(security_type, TIAP_PARAM_SECURITY_WPA | TIAP_PARAM_SECURITY_WPA2)
				&& security_type_val == 3)
			strcat(tempbuf,"selected");
		if (security_type & TIAP_PARAM_SECURITY_WPA && security_type_val == 1)
			strcat(tempbuf,"selected");
		if (security_type & TIAP_PARAM_SECURITY_WPA2 && security_type_val == 2)
			strcat(tempbuf,"selected");
	}

	switch (security_type_val) 
	{
		case 0:
			strcat(tempbuf," value=0> None ");
			break;
		case 1:
			strcat(tempbuf," value=1> WPA v.1 ");
			break;
		case 2:
			strcat(tempbuf," value=2> WPA v.2 ");
			break;
		case 3:
			strcat(tempbuf," value=3> WPA any ");
			break;
		case 4:
			strcat(tempbuf," value=4> WEP ");
			break;
		case 5:
			strcat(tempbuf," value=5> Dynamic WEP ");
			break;
	}
	strcat(tempbuf,"</option>");
	return tempbuf;
}


char* wlan_htmlGetPrivacyWepCipher(char *selected, const char *profile)
{
    __u32 wep_cipher;

    if( profile_map_tbl[atoi(profile)] == -1 ) {
	    tiap_sec_profile_t profile;
	    wcfg_storage_setdefaults_sec_profile(&profile);
	    wep_cipher = profile.sec.default_cipher;
    }
    else tiap_wcfg_get_default_cipher(wd, &wep_cipher, profile_map_tbl[atoi(profile)]);

    switch (wep_cipher) 
    {
	    case TIAP_WPA_CIPHER_WEP_40:
		    if (atoi(selected) == 0) 
			    return "selected";
		    break;
	    case TIAP_WPA_CIPHER_WEP_104:
		    if (atoi(selected) == 1) 
			    return "selected";
		    break;
	    case TIAP_WPA_CIPHER_WEP_232:
		    if (atoi(selected) == 2) 
			    return "selected";
		    break;
	    default:
		    break;
    }

    return " ";

}

char* wlan_htmlGetPrivacyAuthMode(char *selected, const char *profile)
{   
	int i;
	BOOL    openSystemEnable, sharedKeyEnable;
	tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];

	openSystemEnable = FALSE;
	sharedKeyEnable = FALSE;

	/*TODO: auth tbl is ioctl, how to get it from profile?*/

	if (profile_map_tbl[atoi(profile)] == -1)
	{
		if (!atoi(selected)) 
			return "selected";
		return " ";
	}
	if (tiap_wcfg_get_auth_type(wd, tbl, !FROM_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to get authentication algorithm table");
		return (char*)"";
	}

	for (i = 0; i < TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED; i++)
		if (tbl[i].enable == TRUE &&
			tbl[i].algo == TIAP_PARAM_MIB_OPEN_SYSTEM)
			openSystemEnable = TRUE;
		else if (tbl[i].enable == TRUE &&
			tbl[i].algo == TIAP_PARAM_MIB_SHARED_KEY)
			sharedKeyEnable = TRUE;

	if( !atoi(selected) && openSystemEnable && !sharedKeyEnable )
		return "selected";
	else if( atoi(selected) == 1 && !openSystemEnable && sharedKeyEnable)
		return "selected";
	else if( atoi(selected) == 2 && openSystemEnable && sharedKeyEnable)
		return "selected";

	return " ";
}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal0(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the first wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     first wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal0(const char *key_number, const char *profile)
{
	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, ret, stringLen = 0;
	
	validate_profile(profile);

	if( (atoi(key_number) < 0 || atoi(key_number) >= 4) ) {
		PRINT_CONFIG_ERR_MSG("Invalid wep key number.");
		return (char*)NULL;
	}

	tempbuf[0] = 0;


	if( profile_map_tbl[atoi(profile)] == -1 ) 
	{
		tiap_sec_profile_t profile;
		wcfg_storage_setdefaults_sec_profile(&profile);
		memcpy(&wep_key, &profile.sec.keys[atoi(key_number)], sizeof(wep_key));
		wep_cipher = profile.sec.default_cipher;
	}	
	else 
	{
		if( (ret = tiap_wcfg_get_def_wep_key(wd, atoi(key_number), &wep_key,
					profile_map_tbl[atoi(profile)])) == 1 ) 
		{
			PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
			return (char*)NULL;
		}
		tiap_wcfg_get_default_cipher(wd, &wep_cipher, atoi(profile));
	}
	
	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal1(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the second wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     second wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal1(void)
{
	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_def_wep_key(wd, 1, &wep_key,0) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return (char*)NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher,0);

	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal2(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the third wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     third wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal2(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(2, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif

	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_def_wep_key(wd, 2, &wep_key,0) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return (char*)NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher,0);
	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal3(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the fourth wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     fourth wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal3(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(3, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif
	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_get_def_wep_key(wd, 3, &wep_key,0) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return (char*)NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher,0);

	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;
}

char *SaveMultiSsid(const char *profile, const char *hidden, char *activate, char *save, char *ssid,
		char *vlan, char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa2_pre_auth, char *wpa_auth_radio, char *psk_value,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	int PhysicalIndex, LogicalIndex, nprofile, tablePhyIndex;
	tiap_param_multi_ssid_t multi_ssid;
	tiap_param_multi_ssid_elem_t elem;
    int savingIndex = 0;
	BOOL DoSave = FALSE;
	char *privacyResult=NULL;
	char actualProfile[5];
    __u32 security_select_val;
	
	if (strcmp(save, "Saving") == 0)
		DoSave = TRUE;
	else if (strlen(save) <= 8)
		return "";
    savingIndex = atoi(save+8);
    
	if (DoSave || (globalWebSavingIndex <= savingIndex))
	{

		nprofile = atoi(profile);
	
		/* checkbox is selected if equals to "on" */
		if( !strcmp(activate, "on") )
		{
			if (atoi(vlan) == 0) 
            {
				PRINT_CONFIG_ERR_MSG("Vlan is set to 0 or not exists\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_ZERO;
				return (char*)"";
			}
            if (atoi(vlan) <0) 
		    {
                PRINT_CONFIG_ERR_MSG("Bad Vlan value\n");
                mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR;
                return (char*)"";
		    }

			if (atoi(vlan) >= 4096) 	/* MAX VLAN is 4095 => 12 bits */
			{
				PRINT_CONFIG_ERR_MSG("Vlan exeeds its maximal value - 4095\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_OVERFLOW;
				return (char*)"";
			}
			if ((strcmp(ssid,"NO_PARAM")==0) || (strlen(ssid) == 0))
			{
				PRINT_CONFIG_ERR_MSG("SSID is empty or not exists\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_SSID_EMPTY;
				return (char*)"";
			}

            /* If Multiple SSIDs is Enabled:                             *
             * If there is a profile configured to static WEP -->        *
             * Authentication Algorithm is Both Open System & Shared Key */
            security_select_val = atoi(security_select);
            if(security_select_val == 4 /* WEP */)
            {
                *wep_auth_select = '2';
            }

			if(TestPrivacy(profile, security_select, wep_auth_select,
						   wep_cipher_select, wep_key_radio, wep_key_val0,
						   wep_key_val1, wep_key_val2, wep_key_val3,
						   wpa2_pre_auth, wpa_auth_radio, psk_value,
						   radius_ipaddr_octet0, radius_ipaddr_octet1,
						   radius_ipaddr_octet2, radius_ipaddr_octet3,                   
						   radius_port, radius_secret, gtk) == FALSE)
			{
				printf("*********** Security NOT OK ************\n");
				return (char*)"";
			}
			
			if( profile_map_tbl[nprofile] != -1 )
			{
				/* this mSSID entry already exists */
				LogicalIndex = mssid_act_to_inter(hidden);
				if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
					return (char*)"";
				}
				if (tiap_wcfg_get_actual_ssid_ind(&multi_ssid, LogicalIndex, &PhysicalIndex))
				{
					PRINT_CONFIG_ERR_MSG("Failed to get PhysicalIndex mSSID index.");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
					return (char*)"";
				}
				/* verify ssid or vlan doesn't already exist in other entries*/
				if (tiap_wcfg_lookUpMultiSsidByVlan(&multi_ssid, atoi(vlan), NULL, &tablePhyIndex) == OK) 
				{
					if (tablePhyIndex != PhysicalIndex) 
					{
						PRINT_CONFIG_ERR_MSG("Vlan already in use for ssid\n");
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS;
						return (char*)"";
					}
				}
				if (tiap_wcfg_lookUpMultiSsidBySsid(&multi_ssid, ssid, NULL, &tablePhyIndex) == OK) 
				{
					if (tablePhyIndex != PhysicalIndex) 
					{
						PRINT_CONFIG_ERR_MSG(
							"The given SSID already exists in the Multiple SSID table. Setting aborted!\n");
						mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS;
						return (char*)"";
					}
				}
				tiap_wcfg_delMultiSsid(wd, PhysicalIndex, !UPDATE_STORAGE);
				profile_map_tbl[nprofile] = -1;
			}
			else	/* new column is about to exist */
			{
				if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) 
				{
					PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
					return (char*)"";
				}
				/* verify ssid or vlan doesn't already exist */
				if (tiap_wcfg_lookUpMultiSsidByVlan(&multi_ssid, atoi(vlan), NULL, NULL) == OK) 
				{
					PRINT_CONFIG_ERR_MSG("Vlan already in use for ssid\n");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_VLAN_EXISTS;
					return (char*)"";
				}
				if (tiap_wcfg_lookUpMultiSsidBySsid(&multi_ssid, ssid, NULL, NULL) == OK) 
				{
					PRINT_CONFIG_ERR_MSG(
						"The given SSID already exists in the Multiple SSID table. Setting aborted!\n");
					mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_SSID_EXISTS;
					return (char*)"";
				}
			}
				
			/* add ssid and vlan to mssid tbl */
			elem.ssid_type = TIAP_PARAM_SSID_SECONDARY;
			elem.vlan = atoi(vlan);
			memcpy(elem.ssid, ssid, IW_ESSID_MAX_SIZE);
			elem.ssid[IW_ESSID_MAX_SIZE] = '\0';
			if (tiap_wcfg_addMultiSsid(wd, &elem, UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to add ssid to multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}

			/* save internal profile index for subsequent operations */
			if( (LogicalIndex = mssid_act_to_inter(ssid)) == -1 ) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to find internal profile index.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_PROFILE_ERROR;
				return (char*)"";
			}
			if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}
			if (tiap_wcfg_get_actual_ssid_ind(&multi_ssid, LogicalIndex, &PhysicalIndex))
			{
				PRINT_CONFIG_ERR_MSG("Failed to get PhysicalIndex mSSID index.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}
			wlan_htmlInit();
			/*profile_map_tbl[PhysicalIndex] = LogicalIndex;*/
			sprintf(actualProfile,"%d",LogicalIndex);
			privacyResult = SavePrivacy(actualProfile, security_select, wep_auth_select,
										wep_cipher_select, wep_key_radio, wep_key_val0,
										wep_key_val1, wep_key_val2, wep_key_val3,
                                        wpa2_pre_auth, wpa_auth_radio, psk_value,
										radius_ipaddr_octet0, radius_ipaddr_octet1,
										radius_ipaddr_octet2, radius_ipaddr_octet3,                                 
										radius_port, radius_secret, gtk);                                
			if (strcmp(privacyResult,"OK") != 0)
			{
				printf("*********** Save Security NOT OK ************\n");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_BAD_SECURITY_PARAM;
				return (char*)"";
			}

		}
		/* checkbox is not selected */
		else if( !strcmp(activate, "NO_PARAM") && profile_map_tbl[nprofile] != -1 )
		{ /* delete the profile as it is unchecked */
			if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
			{
				PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid table.");
				mSSID_errorNumber = MULTIPLE_SSID_MSG_ERROR_INTERNAL_ERROR;
				return (char*)"";
			}
			tiap_wcfg_get_actual_ssid_ind(&multi_ssid, profile_map_tbl[nprofile], &PhysicalIndex);
			tiap_wcfg_delMultiSsid(wd, PhysicalIndex, !UPDATE_STORAGE);
			profile_map_tbl[nprofile] = -1;
			wlan_htmlInit();
		}
			
		/* so far so good.. finish by restart */
		mSSID_errorNumber = 0;
		restart();
	}

	return (char*)"";
}

char *wlan_htmlSaveMultiSsid2(char *hidden, char *activate, char *save, char *ssid,
		char *vlan, char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa_auth_radio, char *psk_value, char *wpa2_pre_auth,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{	
	return SaveMultiSsid("1", hidden, activate, save, ssid, vlan, security_select,
			wep_auth_select, wep_cipher_select, wep_key_radio,
			wep_key_val0, wep_key_val1, wep_key_val2, wep_key_val3,
			wpa2_pre_auth, wpa_auth_radio, psk_value, radius_ipaddr_octet0,
			radius_ipaddr_octet1, radius_ipaddr_octet2, radius_ipaddr_octet3,
			radius_port, radius_secret, gtk);
}




char *wlan_htmlSaveMultiSsid3(char *hidden, char *activate, char *save, char *ssid, char *vlan,
		char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa_auth_radio, char *psk_value, char *wpa2_pre_auth,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	return SaveMultiSsid("2", hidden, activate, save, ssid, vlan, security_select,
			wep_auth_select, wep_cipher_select, wep_key_radio,
			wep_key_val0, wep_key_val1, wep_key_val2, wep_key_val3,
			wpa2_pre_auth, wpa_auth_radio, psk_value, radius_ipaddr_octet0,
			radius_ipaddr_octet1, radius_ipaddr_octet2, radius_ipaddr_octet3,
			radius_port, radius_secret, gtk);
}

char *wlan_htmlSaveMultiSsid4(char *hidden, char *activate, char *save, char *ssid, char *vlan,
		char *security_select, char *wep_auth_select,
		char *wep_cipher_select, char *wep_key_radio, char *wep_key_val0,
		char *wep_key_val1, char *wep_key_val2, char *wep_key_val3,
		char *wpa_auth_radio, char *psk_value, char *wpa2_pre_auth,
		char *radius_ipaddr_octet0, char *radius_ipaddr_octet1,
		char *radius_ipaddr_octet2, char *radius_ipaddr_octet3,
		char *radius_port, char *radius_secret, char *gtk)
{
	return SaveMultiSsid("3", hidden, activate, save, ssid, vlan, security_select,
			wep_auth_select, wep_cipher_select, wep_key_radio,
			wep_key_val0, wep_key_val1, wep_key_val2, wep_key_val3,
			wpa2_pre_auth, wpa_auth_radio, psk_value, radius_ipaddr_octet0,
			radius_ipaddr_octet1, radius_ipaddr_octet2, radius_ipaddr_octet3,
			radius_port, radius_secret, gtk);

}


/* 
 *   WDS page
 */

char *get_wds_mode(const char *select_value)
{
	__u8 enabled, wif_mode, crude_mode;
	int val = atoi(select_value);
	
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WDS_ENABLE, &enabled, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get WDS enable state.");
		return (char*)NULL;
	}
	
	if( enabled )
	{
		tiap_wcfg_wdsCrudeModeGet(wd, &crude_mode);
		tiap_wcfg_wdsWirelessIfModeGet(wd, &wif_mode);

		if( !val ) { /* bridge mode */
			if( !crude_mode && wif_mode )
				return "selected";
		}
		else if( val == 1 ) { /* repeater mode */
			if( !crude_mode && !wif_mode )
				return "selected";
		}
		else if( val == 2 ) { /* crude mode */
			if( crude_mode && wif_mode )
				return "selected";
		}
	}
	else if( !enabled && val == 3 )
		return "selected";

	return "";
}

char *get_activate_as_root(void)
{
	__u8 val;

	tiap_wcfg_wdsHopGet(wd, &val);
	return (!val ? "checked" : "");
}

char *get_wds_monitor_connection_status(void)
{
	/*TODO: ask Artiom*/
	return (char*)NULL;
}

char *get_wds_privacy(void)
{
	__u8 val;

	tiap_wcfg_wdsPrivacyGet(wd, &val);
	return (val ? "checked" : "");
}

char *get_wds_secret(void)
{
	char secret[WDS_SHARED_SECRET_STR_LEN+1];
	
	tiap_wcfg_wdsSharedSecretGet(wd, secret);
	secret[WDS_SHARED_SECRET_STR_LEN] = '\0'; /*just in case*/
	sprintf(tempbuf, "\"%s\"", secret);

	return tempbuf;
}

char *get_wds_name(void)
{
	char name[WDS_NAME_MAX_LEN+1];
	
	tiap_wcfg_wdsNameGet(wd, name);
	name[WDS_NAME_MAX_LEN] = '\0'; /*just in case*/
	sprintf(tempbuf, "\"%s\"", name);

	return tempbuf;
}

char *get_wds_auto_channel_select(void)
{
	__u8 val;

	tiap_wcfg_wdsChSelModeGet(wd, &val);
	return (val ? "checked" : "");
}
		
char *get_wds_auto_cfg(void)
{
	__u8 val;

	tiap_wcfg_wdsAutoCfgGet(wd, &val);
	return (val ? "checked" : "");
}



char *get_wds_uplink_chbox(void)
{
	char p[ETH_ALEN], empty[ETH_ALEN];

	tiap_wcfg_wdsUlMacGet(wd, p);
	memset(empty, 0, ETH_ALEN);
	
	return (memcmp(p, empty, ETH_ALEN) ? "checked" : "");
}

char *get_wds_uplink(void)
{
	char p[ETH_ALEN], empty[ETH_ALEN];

	tiap_wcfg_wdsUlMacGet(wd, p);
	memset(empty, 0, ETH_ALEN);
	
	if( !memcmp(p, empty, ETH_ALEN) )
	{
		return "";
	}
	
	sprintf(tempbuf,"%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
            p[0], p[1], p[2], p[3], p[4], p[5]);

	return tempbuf;
}


char *get_wds_downlinks_chbox(int nlink)
{
	char p[ETH_ALEN], empty[ETH_ALEN];

	if( !nlink ) tiap_wcfg_wdsDl0MacGet(wd, p);
	else if( nlink == 1 ) tiap_wcfg_wdsDl1MacGet(wd, p);
	else if( nlink == 2 ) tiap_wcfg_wdsDl2MacGet(wd, p);
	else if( nlink == 3 ) tiap_wcfg_wdsDl3MacGet(wd, p);
	else fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
	
	memset(empty, 0, ETH_ALEN);
	
	return (memcmp(p, empty, ETH_ALEN) ? "checked" : "");
}

char *get_wds_downlinks(int nlink)
{
	char p[ETH_ALEN], empty[ETH_ALEN];

	if( !nlink ) tiap_wcfg_wdsDl0MacGet(wd, p);
	else if( nlink == 1 ) tiap_wcfg_wdsDl1MacGet(wd, p);
	else if( nlink == 2 ) tiap_wcfg_wdsDl2MacGet(wd, p);
	else if( nlink == 3 ) tiap_wcfg_wdsDl3MacGet(wd, p);
	else fprintf(stderr, "tiweb: Internal error %s:%d\n", __func__, __LINE__);
	memset(empty, 0, ETH_ALEN);
	
	if( !memcmp(p, empty, ETH_ALEN) )
	{
		return "";
	}
	
	sprintf(tempbuf,"\"%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\"",
            p[0], p[1], p[2], p[3], p[4], p[5]);

	return tempbuf;
}

char *get_wds_states(int nlink)
{
    if(wd->storage.ldb.wdsEnable == 0) {
        return "NONE";
    }
    return (char *)tiap_wcfg_wdsLinkStateGet(wd, nlink);
}

char *get_wds_mngrState(int nlink)
{
    if(wd->storage.ldb.wdsEnable == 0) {
        return "NONE";
    }
    return (char *)tiap_wcfg_wdsMngrStateGet(wd, nlink);
}

enum wds_cfg_params {
	wds_mode, /* 0 */
	wds_active_as_root,
	wds_privacy,
	wds_secret,
	wds_name,
	wds_auto_channel_select, /* 5 */
	wds_auto_cfg,
	wds_uplink_chbox,
	wds_uplink,
	wds_downlinks_chbox,
	wds_downlinks,
	wds_states,
    wds_mngr_state
};

char *wlan_htmlWdsGetCfg(const char *cfg_param, const char *arg1)
{
	char *p;

#if 0
	printf("%s:%d cfg_param=%s argv1=%s\n", __func__, __LINE__, cfg_param, arg1);
	fflush(stdout);
#endif
	
	switch( atoi(cfg_param) )
	{
		case wds_mode:
			p = get_wds_mode(arg1);
			break;

		case wds_active_as_root:
			p = get_activate_as_root();
			break;

		case wds_privacy:
			p = get_wds_privacy();
			break;

		case wds_secret:
			p = get_wds_secret();
			break;

		case wds_name:
			p = get_wds_name();
			break;

		case wds_auto_channel_select:
			p = get_wds_auto_channel_select();
			break;

		case wds_auto_cfg:
			p = get_wds_auto_cfg();
			break;

		case wds_uplink_chbox:
			p = get_wds_uplink_chbox();
			break;

		case wds_uplink:
			p = get_wds_uplink();
			break;
			
		case wds_downlinks_chbox:
			p = get_wds_downlinks_chbox(atoi(arg1));
			break;

		case wds_downlinks:
			p = get_wds_downlinks(atoi(arg1));
			break;

		case wds_states:
			p = get_wds_states(atoi(arg1));
			break;
        case wds_mngr_state:
            p = get_wds_mngrState(atoi(arg1));
            break;
		
		default:
			fprintf(stderr, "tiweb: Invalid configuration parameter: %s\n", cfg_param);
			return "";
	}

	return p;
}

int convertMacStrToArray(const char * pHuman, unsigned char * pArray)
{
    char *ptr;
    int length, digitsNumber;
    int retCode = OK;
    unsigned int digit;
    length = strlen(pHuman);
    if(length > 17) {
        printf("%s : string is too long - %d.\n", __FUNCTION__, length);
        retCode = NOK;
    }
    else {
        ptr = (char *)pHuman;
        digitsNumber = 0;
        do {
            digit = strtoul(ptr, &ptr,16);
            if((digit == 0) || (digit == ULONG_MAX)) {
                if(errno == ERANGE) {
                    retCode = NOK;
                    printf("%s : Digit = %d, errno %d.\n", __FUNCTION__, digit, errno);
                    break;
                }
            }
            if(digit > 0xff) {
                retCode = NOK;
                printf("%s : Digit = %d.\n", __FUNCTION__, digit);
                break;
            }
            *(pArray + digitsNumber) = (unsigned char) digit;
            digitsNumber++;
            if(*ptr == 0) {
                break;
            }
            ptr++;
        } while ( digitsNumber < 6 );
        if(digitsNumber != 6) {
            retCode = NOK;
            printf("%s : DigitsNumber = %d.\n", __FUNCTION__, digitsNumber);
        }
    }
    return(retCode);

}

char *wlan_htmlWdsSaveCfg(const char *apply_button,
		const char *mode, const char *as_root,
		const char *privacy, const char *secret,
		const char *name, const char *channel_select,
		const char *auto_cfg, const char *uplink_chbox,
		const char *uplink, const char *downlink0_chbox,
		const char *downlink0, const char *downlink1_chbox,
		const char *downlink1, const char *downlink2_chbox,
		const char *downlink2, const char *downlink3_chbox,
		const char *downlink3)
{
	int val, autoCfgSet, chSelModeSet, privacy_val, wds_mode;
	__u8 uplink_mac[ETH_ALEN], 
         downlink0_mac[ETH_ALEN], 
         downlink1_mac[ETH_ALEN], 
         downlink2_mac[ETH_ALEN], 
         downlink3_mac[ETH_ALEN];
         
    static int isFirst = 1;
    static __u8 empty_mac[ETH_ALEN];
    __u8 enable;
    __u8 rootDevice;
    char * pErrStr = "";
	
#if 0
	printf("%s:%d aplly=%s mode=%s as_root=%s privacy=%s secret=%s name=%s channel_select=%s
			auto_cfg=%s uplink_chbox=%s uplink=%s downlink0_chbox=%s downlink0=%s
			downlink1_chbox=%s downlink1=%s downlink2_chbox=%s downlink2=%s
			downlink3_chbox=%s downlink3=%s\n", __func__, __LINE__,
			apply_button,
			mode, as_root, privacy, secret, name, channel_select, auto_cfg, uplink_chbox,
			uplink, downlink0_chbox, downlink0, downlink1_chbox, downlink1,
			downlink2_chbox, downlink2, downlink3_chbox, downlink3);
#endif
	
	if( strstr(apply_button, "Applying") != NULL)
	{

        if (isFirst)
        {
            memset(empty_mac,0,ETH_ALEN);
            isFirst=0;
        }
		
        /* mode */
        wds_mode = atoi(mode);
        
        /* activate as root */
        rootDevice = atoi(as_root);

		/* privacy */
        if(privacy_val) 
        {
            /* Save secret if privacy enabled */
            if( strlen(secret) > WDS_SHARED_SECRET_STR_LEN)
            {
                pErrStr = "Secret string is too long.\n";
                goto err_exit;
            }
        }
        privacy_val = atoi(privacy);
        if (((privacy_val) != 0) && ((privacy_val) != 1))
        {
            pErrStr = "Privacy set error.\n";
            goto err_exit;
        }
        
		/* automatic channel selection */
        chSelModeSet = atoi(channel_select);

		/* automatic configuration */
		autoCfgSet = atoi(auto_cfg);

		/* uplink connection */
		if( !strcmp(uplink_chbox, "NO_PARAM" ) )
		    val = 0;
		else 
		    val = atoi(uplink_chbox);
        if((rootDevice ) || (val == 0) || ((convertMacStrToArray(uplink, uplink_mac) == NOK)))
            memset(uplink_mac, 0, ETH_ALEN);

        if ((wds_mode != 3) && (rootDevice == 0) && (val == 0))
        {
            pErrStr = "Uplink address is not configured.\n";
            goto err_exit;
        }

		/* downlink1 connection */
		if((strcmp(downlink0_chbox, "NO_PARAM") == 0) || 
           (convertMacStrToArray(downlink0, downlink0_mac) == NOK))
		    memset(downlink0_mac, 0, ETH_ALEN);

		/* downlink2 connection */
        if((strcmp(downlink1_chbox, "NO_PARAM") == 0) || 
           (convertMacStrToArray(downlink1, downlink1_mac) == NOK))
		    memset(downlink1_mac, 0, ETH_ALEN);

		/* downlink3 connection */
        if((strcmp(downlink2_chbox, "NO_PARAM") == 0) || 
           (convertMacStrToArray(downlink2, downlink2_mac) == NOK))
		    memset(downlink2_mac, 0, ETH_ALEN);

		/* downlink4 connection */
        if((strcmp(downlink3_chbox, "NO_PARAM") == 0) || 
           (convertMacStrToArray(downlink3, downlink3_mac) == NOK))
		    memset(downlink3_mac, 0, ETH_ALEN);

        if (wds_mode != 3)  /* WDS mode is enabled */
        {
            /* Test the addresses values */

            if (memcmp(empty_mac,uplink_mac,ETH_ALEN))
            {
                /* Verify Uplink and DonlinkX are not identical */
                if (memcmp(empty_mac,downlink0_mac,ETH_ALEN))
                {
                    if (memcmp(uplink_mac,downlink0_mac,ETH_ALEN) == 0)     /* U,D0 */
                    {
                        pErrStr = "ERROR! The Uplink address is identical to a downlink address UL,DL0.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink1_mac,ETH_ALEN))
                {
                    if (memcmp(uplink_mac,downlink1_mac,ETH_ALEN) == 0)     /* U,D1 */
                    {
                        pErrStr = "ERROR! The Uplink address is identical to a downlink address UL,DL1.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink2_mac,ETH_ALEN))
                {
                    if (memcmp(uplink_mac,downlink2_mac,ETH_ALEN) == 0)     /* U,D2 */
                    {
                        pErrStr = "ERROR! The Uplink address is identical to a downlink address UL,DL2.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink3_mac,ETH_ALEN))
                {
                    if (memcmp(uplink_mac,downlink3_mac,ETH_ALEN) == 0)     /* U,D3 */
                    {
                        pErrStr = "ERROR! The Uplink address is identical to a downlink address UL,DL3.\n";
                        goto err_exit;
                    }
                }
            }
            /* Verify DownlinkX and DownlinkY are not identical */
            if (memcmp(empty_mac,downlink0_mac,ETH_ALEN))
            {
                if (memcmp(empty_mac,downlink1_mac,ETH_ALEN))
                {
                    if (memcmp(downlink0_mac,downlink1_mac,ETH_ALEN) == 0)  /* D0,D1 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL0,DL1.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink2_mac,ETH_ALEN))
                {
                    if (memcmp(downlink0_mac,downlink2_mac,ETH_ALEN) == 0)  /* D0,D2 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL0,DL2.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink3_mac,ETH_ALEN))
                {
                    if (memcmp(downlink0_mac,downlink3_mac,ETH_ALEN) == 0)  /* D0,D3 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL0,DL3.\n";
                        goto err_exit;
                    }
                }
            }
            if (memcmp(empty_mac,downlink1_mac,ETH_ALEN))
            {
                if (memcmp(empty_mac,downlink2_mac,ETH_ALEN))
                {
                    if (memcmp(downlink1_mac,downlink2_mac,ETH_ALEN) == 0)  /* D1,D2 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL1,DL2.\n";
                        goto err_exit;
                    }
                }
                if (memcmp(empty_mac,downlink3_mac,ETH_ALEN))
                {
                    if (memcmp(downlink1_mac,downlink3_mac,ETH_ALEN) == 0)  /* D1,D3 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL1,DL3.\n";
                        goto err_exit;
                    }
                }
            }
            if (memcmp(empty_mac,downlink2_mac,ETH_ALEN))
            {
                if (memcmp(empty_mac,downlink3_mac,ETH_ALEN))
                {
                    if (memcmp(downlink2_mac,downlink3_mac,ETH_ALEN) == 0)  /* D2,D3 */
                    {
                        pErrStr = "ERROR! Two Downlink addresses are identical DL2,DL3.\n";
                        goto err_exit;
                    }
                }
            }

			/* Verify that none of the downlinks and uplink is identical to AP mac */
			if (!memcmp(wd->storage.ldb.hwaddr, downlink0_mac, ETH_ALEN) ) {
				pErrStr = "Error: downlink0 MAC address is the same as AP MAC address\n";
				goto err_exit;

			}
			if (!memcmp(wd->storage.ldb.hwaddr, downlink1_mac, ETH_ALEN) ) {
				pErrStr = "Error: downlink1 MAC address is the same as AP MAC address\n";
				goto err_exit;

			}
			if (!memcmp(wd->storage.ldb.hwaddr, downlink2_mac, ETH_ALEN) ) {
				pErrStr = "Error: downlink2 MAC address is the same as AP MAC address\n";
				goto err_exit;

			}
			if (!memcmp(wd->storage.ldb.hwaddr, downlink3_mac, ETH_ALEN) ) {
				pErrStr = "Error: downlink3 MAC address is the same as AP MAC address\n";
				goto err_exit;

			}
			if (!memcmp(wd->storage.ldb.hwaddr, uplink_mac, ETH_ALEN) ) {
				pErrStr = "Error: uplink MAC address is the same as AP MAC address\n";
				goto err_exit;

			} 

            if (rootDevice)
            {
                /* Verify at least one Downlink was configured */
                if ((memcmp(empty_mac,downlink0_mac,ETH_ALEN)==0) &&
                    (memcmp(empty_mac,downlink1_mac,ETH_ALEN)==0) &&
                    (memcmp(empty_mac,downlink2_mac,ETH_ALEN)==0) &&
                    (memcmp(empty_mac,downlink3_mac,ETH_ALEN)==0))
                {
                    pErrStr = "ERROR! no Downlink address is configured.\n";
                    goto err_exit;
                }
            }
		    /* name */
            if(strcmp(name, "NO_PARAM") != 0)
            {
               if(tiap_wcfg_wdsNameSet(wd, name, !UPDATE_STORAGE)) 
               {
                   pErrStr = "Name is too long.\n";
                   goto err_exit;
               }
            }
        }   /* mode != 3 */

        /* Save parameters */
		if(tiap_wcfg_wdsHopSet(wd, !rootDevice, !UPDATE_STORAGE)) 
        {
            pErrStr = "Hop set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsPrivacySet(wd, privacy_val, !UPDATE_STORAGE)) 
        {
            pErrStr = "Privacy set error.\n";
            goto err_exit;
        }
        if(privacy_val) 
        {
            /* Save secret if privacy enabled */
            if(tiap_wcfg_wdsSharedSecretSet(wd, secret, !UPDATE_STORAGE)) 
            {
                pErrStr = "Error saving the secret string.\n";
                goto err_exit;
            }
        }
		if(tiap_wcfg_wdsAutoCfgSet(wd, autoCfgSet, !UPDATE_STORAGE)) {
            pErrStr = "Auto config set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsChSelModeSet(wd, chSelModeSet, !UPDATE_STORAGE)) {
            pErrStr = "Channel select set error.\n";
            goto err_exit;
        }
        if(tiap_wcfg_wdsUlMacSet(wd, uplink_mac, !UPDATE_STORAGE)) {
            pErrStr = "UL MAC set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsDl0MacSet(wd, downlink0_mac, !UPDATE_STORAGE)) {
            pErrStr = "DL#0 MAC set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsDl1MacSet(wd, downlink1_mac, !UPDATE_STORAGE)) {
            pErrStr = "DL#1 MAC set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsDl2MacSet(wd, downlink2_mac, !UPDATE_STORAGE)) {
            pErrStr = "DL#2 MAC set error.\n";
            goto err_exit;
        }
		if(tiap_wcfg_wdsDl3MacSet(wd, downlink3_mac, !UPDATE_STORAGE)) {
            pErrStr = "DL#3 MAC set error.\n";
            goto err_exit;
        }


        /* Save and restart WDS manager */
        switch(wds_mode) {
            case 0: /* Bridge */
                enable = 1;
                tiap_wcfg_wdsWirelessIfModeSet(wd, 1, !UPDATE_STORAGE);
                tiap_wcfg_wdsCrudeModeSet(wd, 0, !UPDATE_STORAGE);
                break;
            case 1: /* Repeater */
                enable = 1;
                tiap_wcfg_wdsWirelessIfModeSet(wd, 0, !UPDATE_STORAGE);
                tiap_wcfg_wdsCrudeModeSet(wd, 0, !UPDATE_STORAGE);
                break;
            case 2: /* Crude */
                enable = 1;
                tiap_wcfg_wdsWirelessIfModeSet(wd, 1, !UPDATE_STORAGE);
                tiap_wcfg_wdsCrudeModeSet(wd, 1, !UPDATE_STORAGE);
                break;
            case 3: /* Disable */
                enable = 0;
                break;
            default:
                enable = 0;
                break;
        }
		if( wds_mode == 3 ) 
			wds_mode = 0;
		else 
			wds_mode = 1;
        if((wd->storage.ldb.wdsEnable) && enable) {
            if(tiap_wcfg_configWds(wd, 1, UPDATE_STORAGE)) {
                pErrStr = "WDS configuration failed.\n";
                goto err_exit;

            }
        }
        if((wd->storage.ldb.wdsEnable && !enable) ||
           (!wd->storage.ldb.wdsEnable && enable)) {
            if(tiap_wcfg_enableWds(wd, enable, UPDATE_STORAGE)) {
                pErrStr = "WDS configuration failed.\n";
                goto err_exit;

            }
        }
	
	}
    return "";
    err_exit:
    strcpy(tempbuf,"<h3><font color=\"#FF0000\">");
    strcat(tempbuf, pErrStr);
    strcat(tempbuf,"</font></h3>");

    return tempbuf;
}

char *wlan_htmlWmeGetCfg(char *enable)
{
	tiap_param_qos_t qos_enable;

	if( tiap_wcfg_get_qos_params(wd, &qos_enable, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wme parameters.");
		return (char*)NULL;
	}
	
	if( atoi(enable) == 1 && qos_enable.enabled )
		return "checked";
	if( !atoi(enable) && !qos_enable.enabled )
		return "checked";

	return (char*)"";
}



/****************************************************************************/
/*                      wlan_htmlWdsGetStatHdr(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table first row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetStatHdr(void)
{
	unsigned long len = 0;

    if(wd->storage.ldb.wdsEnable == 0)
        return (char*)"";

    tempbuf[0] = 0;
	
	len = sprintf(tempbuf,"<TABLE height=65 width=\"76%%\" border=1>\n");

	/* first row */
	len += sprintf(tempbuf + len,"<TBODY>\n");
	len += sprintf(tempbuf + len,"<TR>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"100%%\" bgColor=#006699 colSpan=10 height=12><b><font face=\"Arial\" color=\"#FFFFFF\">WDS links statistics</font></b></TD>\n");
	len += sprintf(tempbuf + len,"</TR>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlWdsGetStatColNames(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table second row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table secind row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetStatColNames(void)
{
	unsigned long len = 0;

    if(wd->storage.ldb.wdsEnable == 0)
        return (char*)"";

    tempbuf[0] = 0;
	
	/* second row */
	len += sprintf(tempbuf + len,"<tr>\n");

    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Mac Address</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"4%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Dir</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>TX Packets</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>TX Errors</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>TX Drops</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RX Packets</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RX Errors</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"13%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RX Drops</b></font></TD>\n");
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlWdsGetStatNumbers(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns a management table row								*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns a management table row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetStatNumbers(void)
{
    wdsMngrReportRequest_s  request;
    wdsMngrStat_s           stat;
    int sockRep;
    int retCode;
    int i;
    UINT8 hwaddr[ETH_ALEN];
    char *pDir;
	unsigned long len = 0;

	tempbuf[0] = 0;

    if(wd->storage.ldb.wdsEnable == 0)
        return (char*)"";
    
    sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
    if(sockRep < 0) {
        printf("Error creating WDS report socket: %s", strerror(errno));
        return (char*)"";
    }

    dda_wdsGetStat(&request);
    retCode = tiap_wcfg_wdsReadResponse((void *)&stat, sizeof(wdsMngrStat_s), sockRep);
#ifndef VXWORKS
    unlink(request.addr.sun_path);
#endif /* ifndef VXWORKS */
    if (retCode != OK) {
        printf("Error reading WDS stat message: %s", strerror(errno));
        return (char*)"";
    }
    for(i = 0; i < WDS_LINKS_NUMBER; i++) {
        switch(i) {
            case 0: /* uplink */
                tiap_wcfg_wdsUlMacGet(wd, hwaddr);
                pDir = "UL";
                break;
            case 1: /* downlink0 */
                tiap_wcfg_wdsDl0MacGet(wd, hwaddr);
                pDir = "DL";
                break;
            case 2: /* downlink0 */
                tiap_wcfg_wdsDl1MacGet(wd, hwaddr);
                pDir = "DL";
                break;
            case 3: /* downlink0 */
                tiap_wcfg_wdsDl2MacGet(wd, hwaddr);
                pDir = "DL";
                break;
            case 4: /* downlink0 */
                tiap_wcfg_wdsDl3MacGet(wd, hwaddr);
                pDir = "DL";
                break;
            default: 
                return (char*)"";
                break;
        }
        if(WDS_IS_MAC_NOT_ZERO_m(hwaddr)) {
            len += sprintf(tempbuf + len,
                    "<td align=middle width=\"20%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
                    hwaddr[0],hwaddr[1],hwaddr[2],hwaddr[3],hwaddr[4],hwaddr[5]);
            len += sprintf(tempbuf + len, "<td width=\"4%%\" height=\"11\" align=\"center\">%s</td>\n",
                    pDir);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.tx_packets);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.tx_errors);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.tx_dropped);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.rx_packets);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.rx_errors);    
            len += sprintf(tempbuf + len, "<td width=\"13%%\" height=\"11\" align=\"center\">%u</td>\n",
                    stat.linkStat[i].ifStat.rx_dropped);    
            len += sprintf(tempbuf + len,"</tr>\n");
        }
    }

    len += sprintf(tempbuf + len,"</table>\n");  
    return tempbuf;
}



/****************************************************************************/
/*                      wlan_htmlWdsGetStatHdr(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table first row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetInfoDbHdr(void)
{
	unsigned long len = 0;

    if(wd->storage.ldb.wdsEnable == 0)
        return (char*)"";

	tempbuf[0] = 0;
	
	len = sprintf(tempbuf,"<TABLE height=65 width=\"76%%\" border=1>\n");

	/* first row */
	len += sprintf(tempbuf + len,"<TBODY>\n");
	len += sprintf(tempbuf + len,"<TR>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"100%%\" bgColor=#006699 colSpan=10 height=12><b><font face=\"Arial\" color=\"#FFFFFF\">WDS topology database</font></b></TD>\n");
	len += sprintf(tempbuf + len,"</TR>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlWdsGetStatColNames(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table second row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table secind row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetInfoDbColNames(void)
{
	unsigned long len = 0;

    if(wd->storage.ldb.wdsEnable == 0)
        return (char*)"";

    tempbuf[0] = 0;
	
	/* second row */
	len += sprintf(tempbuf + len,"<tr>\n");

    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>From</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"16%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>IP</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Uplink</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"8%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RSSI</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Downlink0</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"8%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RSSI</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Downlink1</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"8%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RSSI</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Downlink2</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"8%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RSSI</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>Downlink3</b></font></TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"8%%\" bgColor=#008080 height=11><font size=\"2\" face=\"Arial\" color=\"#FFFFFF\"><b>RSSI</b></font></TD>\n");
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlWdsGetStatNumbers(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns a management table row								*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns a management table row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlWdsGetInfoDbRows(void)
{
    wdsMngrReportRequest_s  request;
    wdsMngrInfoRecord_s     maintainDb[WDS_MNGR_INFO_TABLE_SIZE];
    int sockRep;
    int retCode;
    int i, devNum;
	unsigned long len = 0;
    struct in_addr addr;
    char ulRssiLevel[7];
    char dlRssiLevel[7];

	tempbuf[0] = 0;

    if(wd->storage.ldb.wdsEnable == 0) 
    {
        printf("WDS disabled");
        return (char*)"";
    }
    sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
    if(sockRep < 0) {
        printf("Error creating WDS report socket: %s", strerror(errno));
        return (char*)"";
    }

    dda_wdsGetInfoDb(&request);
    retCode = tiap_wcfg_wdsReadResponse((void *)&maintainDb, sizeof(maintainDb), sockRep);
#ifndef VXWORKS
    unlink(request.addr.sun_path);
#endif /* ifndef VXWORKS */
    if (retCode != OK) {
        printf("Error reading WDS stat message: %s", strerror(errno));
        return (char*)"";
    }
    for(i = 0; i < WDS_MNGR_INFO_TABLE_SIZE; i++) {
        if(WDS_IS_MAC_NOT_ZERO_m(maintainDb[i].sender)) { 
            len += sprintf(tempbuf + len,
                    "<td align=middle width=\"20%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
                       maintainDb[i].sender[0], maintainDb[i].sender[1], 
                       maintainDb[i].sender[2], maintainDb[i].sender[3], 
                       maintainDb[i].sender[4], maintainDb[i].sender[5]);
            addr.s_addr = maintainDb[i].ip;
            len += sprintf(tempbuf + len, "<td width=\"16%%\" height=\"11\" align=\"center\">%s</td>\n",
                           inet_ntoa(addr));
            len += sprintf(tempbuf + len,
                    "<td align=middle width=\"20%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
                           maintainDb[i].ul.mac[0], maintainDb[i].ul.mac[1],
                           maintainDb[i].ul.mac[2], maintainDb[i].ul.mac[3], 
                           maintainDb[i].ul.mac[4], maintainDb[i].ul.mac[5]);

            memset(ulRssiLevel, 0, 7);
            if(maintainDb[i].ul.rssi == 0)
            {
                strcpy(ulRssiLevel, " ");
            }
            else if(maintainDb[i].ul.rssi > -38)
            {
                strcpy(ulRssiLevel, "High");
            }
            else if(maintainDb[i].ul.rssi > -60)
            {
                strcpy(ulRssiLevel, "Normal");
            }
            else
            {
                strcpy(ulRssiLevel, "Low");
            }

            len += sprintf(tempbuf + len, "<td width=\"8%%\" height=\"11\" align=\"center\">%d %s</td>\n",
                           maintainDb[i].ul.rssi, ulRssiLevel);

            for(devNum = 0; devNum < WDS_MAX_DL_NUMBER; devNum++) {
                if(!WDS_IS_MAC_NOT_ZERO_m(maintainDb[i].dl[devNum].mac))
                    continue;
                len += sprintf(tempbuf + len,
                        "<td align=middle width=\"20%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
                               maintainDb[i].dl[devNum].mac[0], maintainDb[i].dl[devNum].mac[1],
                               maintainDb[i].dl[devNum].mac[2], maintainDb[i].dl[devNum].mac[3], 
                               maintainDb[i].dl[devNum].mac[4], maintainDb[i].dl[devNum].mac[5]);

                memset(dlRssiLevel, 0, 7);
                if(maintainDb[i].dl[devNum].rssi == 0)
                {
                    strcpy(dlRssiLevel, " ");
                }
                else if(maintainDb[i].dl[devNum].rssi > -38)
                {
                    strcpy(dlRssiLevel, "High");
                }
                else if(maintainDb[i].dl[devNum].rssi > -60)
                {
                    strcpy(dlRssiLevel, "Normal");
                }
                else
                {
                    strcpy(dlRssiLevel, "Low");
                }
                
                len += sprintf(tempbuf + len, "<td width=\"8%%\" height=\"11\" align=\"center\">%d %s</td>\n",
                    maintainDb[i].dl[devNum].rssi, dlRssiLevel);
            }
            len += sprintf(tempbuf + len,"</tr>\n");
        }
    }
    len += sprintf(tempbuf + len,"</table>\n");  
    return tempbuf;
}


static void create_js_msg_box(char *buf, char *message)
{
	char *msg_box = "<SCRIPT LANGUAGE=\"JavaScript\"> window.alert(\"%s\") </SCRIPT>";

	if (buf)
		sprintf(buf, msg_box, message);
}


char *wlan_htmlGetBoardType_string(void)
{
#if defined(AR7DB)
	if (1)
		return "AR7DB";
#endif
#if defined(AR7RD)
	if (1)
		return "AR7RD";
#endif
#if defined(AR7WRD)
	if (1)
		return "AR7WRD";
#endif
#if defined(AR7Wi)
	if (1)
		return "AR7Wi";
#endif
#if defined(AR7VWi)
	if (1)
		return "AR7VWi";
#endif
#if defined(PUMAS)
	if (1)
		return "PUMAS";
#endif
	return "Running..";

}


char *wlan_htmlSavingIndex(char *option)
{
	int value;

	value = atoi(option);
	if (value == 0)
		globalWebSavingIndex++;
    sprintf(tempbuf,"%d",globalWebSavingIndex);
    return tempbuf;
}
